#!/usr/bin/env python
import sys
import json
import re
import argparse

def main():
    args = parse_args()
    processed_name = process_game_name(args.game)
    json_path = "../../subjects/%s/games.json" % args.subject
    with open(json_path) as json_file:
        games_json = json.load(json_file)
        print(find_game_name(processed_name, games_json))


def parse_args():
    parser = argparse.ArgumentParser(description='Run content tests')
    parser.add_argument('-g', '--game', help='user-facing game name')
    parser.add_argument('-s', '--subject', default='sat', help='subject name (default "sat")')

    return parser.parse_args()

def process_game_name(name):
    processed_name = re.sub(" ", "", name)
    return processed_name.lower()

def find_game_name(processed_name, games_json):
    games_array = games_json['games']
    for game in games_array:
        for game_config in game['config']:
            if 'skill' not in game_config.keys():
                continue
            skill_name = game_config['skill']
            if skill_name == processed_name:
                return game_config['strategy']['content_type']
    return None

if __name__ == '__main__':
    main()
