--- Asserts
-- @author Created by karl on September 19, 2014.

-- ===========================================================================
class("Asserts")
-- ===========================================================================

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

--- Assert that an argument has an allowed type.
-- @param value (any) - The actual value of the argument.
-- @param name (string) - The name of the argument.
-- @param ...  (list of strings) - A list of allowed types.
function Asserts.assertType(value, name, ...)
  local valueType = type(value)
  for _, testType in pairs(arg) do
    if valueType == testType then
      return
    end
  end

  local allowedTypes = table.concat(arg, ", ")
  error(name .. " has invalid value [" .. tostring(value) .. "] of type " .. valueType .. ". Allowed types are: " .. allowedTypes)
end

--- Assert that an optional argument has an allowed type.
-- Only triggers if the argument's value is not nil.
-- @param value (any) - The actual value of the argument.
-- @param name (string) - The name of the argument.
-- @param ...  (list of strings) - A list of allowed types.
function Asserts.assertOptionalType(value, name, ...)
  if value ~= nil then
    Asserts.assertType(value, name, ...)
  end
end
