-- ===========================================================================
class("Colors")
-- ===========================================================================

require("utils.ColorUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

Colors.WHITE = ColorUtils.colorHex24ToStd(0xFFFFFF)
Colors.BLACK = ColorUtils.colorHex24ToStd(0x000000)

Colors.GOLD = ColorUtils.colorHex24StringToStd("#FDB041")
Colors.GREEN = ColorUtils.colorHex24StringToStd("#44D3C5")
Colors.ORANGE = ColorUtils.colorHex24StringToStd("#FC6644")
Colors.RED = ColorUtils.colorHex24StringToStd("#F64A81")
Colors.PURPLE = ColorUtils.colorHex24StringToStd("#7970FF")
Colors.YELLOW = ColorUtils.colorHex24StringToStd("#FFB22B")

Colors.LIGHT_THEME = "LIGHT"
Colors.DARK_THEME = "DARK"

-- -- Color numbers to usual roles
-- 100 - Subtle Background
-- 200 - UI Element Background
-- 300 - Stroke
-- 400 - Main Color / Low contrast text
-- 500 - Medium contrast text
-- 600 - High contrast text

Colors.COLOR_TABLES = {
  [Colors.LIGHT_THEME] = {
    -- Standard Themed UI colors
    PRIMARY100 = ColorUtils.colorHex24StringToStd("#F2FCFE"),
    PRIMARY200 = ColorUtils.colorHex24StringToStd("#DAF6FE"),
    PRIMARY300 = ColorUtils.colorHex24StringToStd("#82E1FC"),
    PRIMARY400 = ColorUtils.colorHex24StringToStd("#05C3F9"),
    PRIMARY500 = ColorUtils.colorHex24StringToStd("#0085BA"),
    GREEN100 = ColorUtils.colorHex24StringToStd("#F2FCF4"),
    GREEN200 = ColorUtils.colorHex24StringToStd("#D9F6DE"),
    GREEN300 = ColorUtils.colorHex24StringToStd("#80E18F"),
    GREEN400 = ColorUtils.colorHex24StringToStd("#02C31F"),
    GREEN500 = ColorUtils.colorHex24StringToStd("#0C9B21"),
    RED100 = ColorUtils.colorHex24StringToStd("#FFF5F6"),
    RED200 = ColorUtils.colorHex24StringToStd("#FFE3E6"),
    RED300 = ColorUtils.colorHex24StringToStd("#FEA1AB"),
    RED400 = ColorUtils.colorHex24StringToStd("#FE4358"),
    RED500 = ColorUtils.colorHex24StringToStd("#D13042"),
    LISTENING100 = ColorUtils.colorHex24StringToStd("#F2FCFE"),
    LISTENING200 = ColorUtils.colorHex24StringToStd("#FFECCA"),
    LISTENING300 = ColorUtils.colorHex24StringToStd("#FFD489"),
    LISTENING400 = ColorUtils.colorHex24StringToStd("#FFA200"),
    LISTENING500 = ColorUtils.colorHex24StringToStd("#D98C05"),
    PRIMARY_EXTRA100 = ColorUtils.colorHex24StringToStd("#F2FAFE"),
    PRIMARY_EXTRA200 = ColorUtils.colorHex24StringToStd("#CCEBFD"),
    PRIMARY_EXTRA300 = ColorUtils.colorHex24StringToStd("#7FCDF9"),
    PRIMARY_EXTRA400 = ColorUtils.colorHex24StringToStd("#009CF4"),
    PRIMARY_EXTRA500 = ColorUtils.colorHex24StringToStd("#0077BA"),
    SPEAKING100 = ColorUtils.colorHex24StringToStd("#FFF7F5"),
    SPEAKING200 = ColorUtils.colorHex24StringToStd("#FFE8E2"),
    SPEAKING300 = ColorUtils.colorHex24StringToStd("#FFB29D"),
    SPEAKING400 = ColorUtils.colorHex24StringToStd("#FF653C"),
    SPEAKING500 = ColorUtils.colorHex24StringToStd("#D85330"),
    READING100 = ColorUtils.colorHex24StringToStd("#FEF6F8"),
    READING200 = ColorUtils.colorHex24StringToStd("#FEE4EC"),
    READING300 = ColorUtils.colorHex24StringToStd("#FBA3BF"),
    READING400 = ColorUtils.colorHex24StringToStd("#F8477F"),
    READING500 = ColorUtils.colorHex24StringToStd("#CD3465"),
    WRITING100 = ColorUtils.colorHex24StringToStd("#F5FDFC"),
    WRITING200 = ColorUtils.colorHex24StringToStd("#E2F9F6"),
    WRITING300 = ColorUtils.colorHex24StringToStd("#9CE9E1"),
    WRITING400 = ColorUtils.colorHex24StringToStd("#3AD3C4"),
    WRITING500 = ColorUtils.colorHex24StringToStd("#2CAEA1"),
    TEAL100 = ColorUtils.colorHex24StringToStd("#F2F8FA"),
    TEAL200 = ColorUtils.colorHex24StringToStd("#D9ECF2"),
    TEAL300 = ColorUtils.colorHex24StringToStd("#7FC0D2"),
    TEAL400 = ColorUtils.colorHex24StringToStd("#0081A6"),
    TEAL500 = ColorUtils.colorHex24StringToStd("#0A5A71"),
    CROSSWORD_BUTTON100 = ColorUtils.colorHex24StringToStd("#F2F7F8"),
    CROSSWORD_BUTTON300 = ColorUtils.colorHex24StringToStd("#D9E6EA"),
    CROSSWORD400 = ColorUtils.colorHex24StringToStd("#00B4D8"),
    CROSSWORD500 = ColorUtils.colorHex24StringToStd("#00819A"),
    MATH100 = ColorUtils.colorHex24StringToStd("#F8F7FF"),
    MATH200 = ColorUtils.colorHex24StringToStd("#EBE9FF"),
    MATH300 = ColorUtils.colorHex24StringToStd("#BBB5FF"),
    MATH400 = ColorUtils.colorHex24StringToStd("#786BFF"),
    MATH500 = ColorUtils.colorHex24StringToStd("#5D52CE"),
    GRAY100 = ColorUtils.colorHex24StringToStd("#F6F6F6"),
    GRAY200 = ColorUtils.colorHex24StringToStd("#ECECEC"),
    GRAY300 = ColorUtils.colorHex24StringToStd("#DCDCDC"),
    GRAY400 = ColorUtils.colorHex24StringToStd("#999999"),
    GRAY500 = ColorUtils.colorHex24StringToStd("#606060"),
    GRAY600 = ColorUtils.colorHex24StringToStd("#1A1A1A"),
    SURFACE100 = ColorUtils.colorHex24StringToStd("#FFFFFF"),
    SURFACE200 = ColorUtils.colorHex24StringToStd("#F6F6F6"),
    -- Onboarding Text Colors
    ONBOARDING_TEXT_DARK = ColorUtils.colorHex24StringToStd("#333333"),
    ONBOARDING_TEXT_LIGHT = ColorUtils.colorHex24StringToStd("#B3B3B3"),
    ONBOARDING_TEXT_OUTRO = ColorUtils.colorHex24StringToStd("#BFBECA"),
    ONBOARDING_TEXT = ColorUtils.colorHex24StringToStd("#4C4C4C"),
    ONBOARDING_TEXT_ADDITIONAL = ColorUtils.colorHex24StringToStd("#949494")
  },
  [Colors.DARK_THEME] = {
    -- Standard Themed UI colors
    PRIMARY100 = ColorUtils.colorHex24StringToStd("#002F3D"),
    PRIMARY200 = ColorUtils.colorHex24StringToStd("#004C63"),
    PRIMARY300 = ColorUtils.colorHex24StringToStd("#0098C8"),
    PRIMARY400 = ColorUtils.colorHex24StringToStd("#00C2FF"),
    PRIMARY500 = ColorUtils.colorHex24StringToStd("#CDF7FF"),
    GREEN100 = ColorUtils.colorHex24StringToStd("#01390A"),
    GREEN200 = ColorUtils.colorHex24StringToStd("#055711"),
    GREEN300 = ColorUtils.colorHex24StringToStd("#178327"),
    GREEN400 = ColorUtils.colorHex24StringToStd("#00BE1D"),
    GREEN500 = ColorUtils.colorHex24StringToStd("#D9F6DE"),
    RED100 = ColorUtils.colorHex24StringToStd("#40040B"),
    RED200 = ColorUtils.colorHex24StringToStd("#700D17"),
    RED300 = ColorUtils.colorHex24StringToStd("#C53242"),
    RED400 = ColorUtils.colorHex24StringToStd("#FE4358"),
    RED500 = ColorUtils.colorHex24StringToStd("#FFE3E6"),
    LISTENING100 = ColorUtils.colorHex24StringToStd("#3C2803"),
    LISTENING200 = ColorUtils.colorHex24StringToStd("#593A03"),
    LISTENING300 = ColorUtils.colorHex24StringToStd("#B67509"),
    LISTENING400 = ColorUtils.colorHex24StringToStd("#FFA200"),
    LISTENING500 = ColorUtils.colorHex24StringToStd("#FFECCA"),
    PRIMARY_EXTRA100 = ColorUtils.colorHex24StringToStd("#061D54"),
    PRIMARY_EXTRA200 = ColorUtils.colorHex24StringToStd("#053370"),
    PRIMARY_EXTRA300 = ColorUtils.colorHex24StringToStd("#026FBC"),
    PRIMARY_EXTRA400 = ColorUtils.colorHex24StringToStd("#009CF4"),
    PRIMARY_EXTRA500 = ColorUtils.colorHex24StringToStd("#D6EFFF"),
    SPEAKING100 = ColorUtils.colorHex24StringToStd("#412A24"),
    SPEAKING200 = ColorUtils.colorHex24StringToStd("#642E20"),
    SPEAKING300 = ColorUtils.colorHex24StringToStd("#B5493B"),
    SPEAKING400 = ColorUtils.colorHex24StringToStd("#FF653C"),
    SPEAKING500 = ColorUtils.colorHex24StringToStd("#FFE2DB"),
    READING100 = ColorUtils.colorHex24StringToStd("#3E242C"),
    READING200 = ColorUtils.colorHex24StringToStd("#622437"),
    READING300 = ColorUtils.colorHex24StringToStd("#B73B63"),
    READING400 = ColorUtils.colorHex24StringToStd("#F8477F"),
    READING500 = ColorUtils.colorHex24StringToStd("#FEE4EC"),
    WRITING100 = ColorUtils.colorHex24StringToStd("#1B3230"),
    WRITING200 = ColorUtils.colorHex24StringToStd("#295F59"),
    WRITING300 = ColorUtils.colorHex24StringToStd("#329D93"),
    WRITING400 = ColorUtils.colorHex24StringToStd("#3AD3C4"),
    WRITING500 = ColorUtils.colorHex24StringToStd("#E2F9F6"),
    TEAL100 = ColorUtils.colorHex24StringToStd("#172B30"),
    TEAL200 = ColorUtils.colorHex24StringToStd("#15424F"),
    TEAL300 = ColorUtils.colorHex24StringToStd("#0A647E"),
    TEAL400 = ColorUtils.colorHex24StringToStd("#0081A6"),
    TEAL500 = ColorUtils.colorHex24StringToStd("#E2F9F6"),
    CROSSWORD_BUTTON100 = ColorUtils.colorHex24StringToStd("#141515"),
    CROSSWORD_BUTTON300 = ColorUtils.colorHex24StringToStd("#020303"),
    CROSSWORD400 = ColorUtils.colorHex24StringToStd("#00B4D8"),
    CROSSWORD500 = ColorUtils.colorHex24StringToStd("#00819A"),
    MATH100 = ColorUtils.colorHex24StringToStd("#151163"),
    MATH200 = ColorUtils.colorHex24StringToStd("#292474"),
    MATH300 = ColorUtils.colorHex24StringToStd("#564DC3"),
    MATH400 = ColorUtils.colorHex24StringToStd("#786BFF"),
    MATH500 = ColorUtils.colorHex24StringToStd("#EBE9FF"),
    GRAY100 = ColorUtils.colorHex24StringToStd("#2F2F2F"),
    GRAY200 = ColorUtils.colorHex24StringToStd("#484848"),
    GRAY300 = ColorUtils.colorHex24StringToStd("#6F6F6F"),
    GRAY400 = ColorUtils.colorHex24StringToStd("#9F9F9F"),
    GRAY500 = ColorUtils.colorHex24StringToStd("#D6D6D6"),
    GRAY600 = ColorUtils.colorHex24StringToStd("#F9F9F9"),
    SURFACE100 = ColorUtils.colorHex24StringToStd("#202020"),
    SURFACE200 = ColorUtils.colorHex24StringToStd("#171717"),
    -- Onboarding Text Colors
    ONBOARDING_TEXT_DARK = ColorUtils.colorHex24StringToStd("#F9F9F9"),
    ONBOARDING_TEXT_LIGHT = ColorUtils.colorHex24StringToStd("#B3B3B3"),
    ONBOARDING_TEXT_OUTRO = ColorUtils.colorHex24StringToStd("#BFBECA"),
    ONBOARDING_TEXT = ColorUtils.colorHex24StringToStd("#4C4C4C"),
    ONBOARDING_TEXT_ADDITIONAL = ColorUtils.colorHex24StringToStd("#F9F9F9")
  }
}

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function Colors.getColorTable(theme)
  local chosenColorTable = Colors.COLOR_TABLES[theme]

  if (chosenColorTable ~= nil) then
    return chosenColorTable
  else
    Log.error("Color theme not found, defaulting to light: " .. theme)
    return Colors.COLOR_TABLES[Colors.LIGHT_THEME]
  end
end

function Colors.getColorThemeForMode(isDarkMode)
  if (isDarkMode) then
    return Colors.DARK_THEME
  end

  return Colors.LIGHT_THEME
end

function Colors.getColorTableForMode(isDarkMode)
  local colorTheme = Colors.getColorThemeForMode(isDarkMode)
  return Colors.getColorTable(colorTheme)
end
