--- DesktopKeyboard
-- @author Created by Aaron Barrett on September 22, 2015.

class("DesktopKeyboard")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local KEY_BACKSPACE = "KEY_BACKSPACE"
local KEY_ENTER = "KEY_ENTER"

local TEXT_BY_KEY_CODE = {
  [12] = "Q",
  [13] = "W",
  [14] = "E",
  [15] = "R",
  [17] = "T",
  [16] = "Y",
  [32] = "U",
  [34] = "I",
  [31] = "O",
  [35] = "P",

  [0] = "A",
  [1] = "S",
  [2] = "D",
  [3] = "F",
  [5] = "G",
  [4] = "H",
  [38] = "J",
  [40] = "K",
  [37] = "L",

  [6] = "Z",
  [7] = "X",
  [8] = "C",
  [9] = "V",
  [11] = "B",
  [45] = "N",
  [46] = "M",

  [51] = KEY_BACKSPACE,
  [36] = KEY_ENTER,
  [49] = " ",

  -- Helpful for testing unsupported characters
  -- [47] = ".",
  -- [43] = ",",
  -- [18] = "1",
  -- [19] = "2",
  -- [20] = "3",
  -- [21] = "4",
  -- [23] = "5",
  -- [22] = "6",
  -- [26] = "7",
  -- [28] = "8",
  -- [25] = "9",
  -- [29] = "0"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getDeviceKeyboardTextLength()
  local deviceKeyboardText = DeviceKeyboard.getText()
  return #deviceKeyboardText
end

local function onKeyPressed(keyCode, keyIsDown)
  local keyText = TEXT_BY_KEY_CODE[keyCode]
  if (keyText == nil) then
    return
  end

  if (keyIsDown) then
    if (keyText == KEY_ENTER) then
      DeviceKeyboard.onKeyboardReturn()
    elseif (keyText == KEY_BACKSPACE) then
      local replacementStartIndex = getDeviceKeyboardTextLength() - 1
      local replacementLength = 1
      local replacementText = ""
      DeviceKeyboard.onKeyboardInput(replacementStartIndex, replacementLength, replacementText)
    else
      local replacementStartIndex = getDeviceKeyboardTextLength() + 1
      local replacementLength = 1
      DeviceKeyboard.onKeyboardInput(replacementStartIndex, replacementLength, keyText)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DesktopKeyboard.setEnabled(enabled)
  local moaiKeyboardSensor = MOAIInputMgr.device.keyboard

  if (enabled) then
    moaiKeyboardSensor:setCallback(onKeyPressed)
  else
    moaiKeyboardSensor:setCallback(nil)
  end
end
