--- FPSCounter
-- @author Created by Aaron Barrett on June 22, 2022.

class("FPSCounter")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_INDEX = 1000000

local NUMERIC_CHARACTERS = "0123456789."
local INITIAL_LABEL_TEXT = "000"

local BACKING_COLOR = {0.0, 0.0, 0.0}

local DISPLAY_COUNT_PER_SECOND = 3
local DISPLAY_INTERVAL = 1.0 / DISPLAY_COUNT_PER_SECOND

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function updateDisplay(self, time, timeSinceLastDisplay)
  local elapsedFrameCount = Screen.elapsedFrameCount
  local fpsCount = (elapsedFrameCount - self.lastDisplayFrameCount) / timeSinceLastDisplay
  self.label:setText(MathUtils.round(fpsCount))

  self.lastDisplayTime = time
  self.lastDisplayFrameCount = elapsedFrameCount
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function startUpdateCoroutine(self)
  runInCoroutine(function()
    while (true) do
      local time = MOAISim.getDeviceTime()
      local timeSinceLastDisplay = time - self.lastDisplayTime

      if (timeSinceLastDisplay >= DISPLAY_INTERVAL) then
        updateDisplay(self, time, timeSinceLastDisplay)
      end

      waitForFrames(1)
    end
  end)
end

-- Constructor ------------------------

function FPSCounter.new()
  local font = PrerenderedFont.newWithActualSize(FontFaces.DIN_OT_MEDIUM, 12.0, NUMERIC_CHARACTERS)
  local label = Label.new(INITIAL_LABEL_TEXT, font)

  local self = FilledRectangle.new(label:getWidth(), label:getHeight())
  Mixin.mix(self, FPSCounter)

  self:insertAtCenter(label)
  self.label = label

  self:setColor(unpack(BACKING_COLOR))
  self:setAnchorPointBottomLeft()

  self.lastDisplayFrameCount = Screen.elapsedFrameCount
  self.lastDisplayTime = MOAISim.getDeviceTime()

  startUpdateCoroutine(self)

  return self
end

-- Convenience ------------------------

function FPSCounter.show()
  local counter = FPSCounter.new()
  counter:setX(Screen.safeLeft + counter:getWidth())
  counter:setY(Screen.safeBottom + counter:getHeight())
  Screen.rootGroup:insert(counter, Z_INDEX)
end
