--- Log
-- Allows logging text at different priorities.
-- @author Created by andrewc on July 27, 2021.

class("Log")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LOG_LEVEL = {
  ERROR = 1,
  WARN = 2,
  INFO = 3,
  DEBUG = 4,
  VERBOSE = 5
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createStringToBeLogged(...)
  local args = table.pack(...)

  local result = ""
  for i = 1, args.n do
    local v = args[i]
    result = result .. tostring(v) .. " \t "
  end

  return result
end

local function logAtLevel(logLevel, ...)
  local stringToBeLogged = createStringToBeLogged(...)
  mslog(logLevel, stringToBeLogged)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function Log.error(...)
  local logLevel = LOG_LEVEL.ERROR
  logAtLevel(logLevel, ...)
end

function Log.warn(...)
  local logLevel = LOG_LEVEL.WARN
  logAtLevel(logLevel, ...)
end

function Log.info(...)
  local logLevel = LOG_LEVEL.INFO
  logAtLevel(logLevel, ...)
end

function Log.debug(...)
  local logLevel = LOG_LEVEL.DEBUG
  logAtLevel(logLevel, ...)
end

function Log.verbose(...)
  local logLevel = LOG_LEVEL.VERBOSE
  logAtLevel(logLevel, ...)
end
