--- Mixin funcionality
-- Examples:
--
-- -- No inheritance:
-- A = {}
-- function A.new()
--   self = {}
--   Mixin.mix(self, A)
--   return self
-- end
-- function A:foo()
--   print("A")
-- end

-- -- Simple inheritance:
-- B = {}
-- function B.new()
--   self = A.new()
--   Mixin.mix(self, B)
--   return self
-- end
-- function B:foo()
--   self.super.foo(self)
--   print("B")
-- end

-- -- Multi-level inheritance:
-- C = {}
-- function C.new()
--   self = B.new()
--   Mixin.mix(self, C, "B")
--   return self
-- end
-- function C:foo()
--   -- No infinite loop makes you happy.
--   self.B.foo(self)
--   print("C")
-- end

-- C.new():foo()



-- ===========================================================================
class("Mixin")
-- ===========================================================================

--- Mixes table into object, useful for extending MOAI userdata objects
-- @param object An object to receive the new values
-- @param table A table to mix in to object, may contain functions or values
-- @param superName When doing inheritance, name of the superclass (optional, defaults to "super").
-- @return Returns the mixed object, can make it more readable. local x = Mixin.mix(obj,A)
function Mixin.mix(object, table, superName)
  superName = superName or 'super'
  local super = object[superName] or {}
  super.instance = object

  for funcName, func in pairs(table) do
    if object[funcName] then
      super[funcName] = object[funcName]
      if (object[superName]) then super[superName] = object[superName] end
      object[superName] = super

    end
    object[funcName] = func
  end
  return object
end
