--- Particles
-- @author Created by Aaron Barrett on December 10, 2014.

class("Particles")

require("TextureLoader")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function Particles:createSourcePositionXTo(duration, ease, amount)
  return AHNonAttributeTo.new(self, duration, self.getSourcePositionX, self.setSourcePositionX, ease, amount)
end

function Particles:createSourcePositionXBy(duration, ease, amount)
  return AHNonAttributeBy.new(self, duration, self.getSourcePositionX, self.setSourcePositionX, ease, amount)
end

function Particles:createSourcePositionYTo(duration, ease, amount)
  return AHNonAttributeTo.new(self, duration, self.getSourcePositionY, self.setSourcePositionY, ease, amount)
end

function Particles:createSourcePositionYBy(duration, ease, amount)
  return AHNonAttributeBy.new(self, duration, self.getSourcePositionY, self.setSourcePositionY, ease, amount)
end

function Particles:createSourcePositionTo(duration, ease, xAmount, yAmount)
  return AHSpawn.new(
                     self:createSourcePositionXTo(duration, ease, xAmount),
                     self:createSourcePositionYTo(duration, ease, yAmount)
                     )
end

function Particles:createSourcePositionBy(duration, ease, xAmount, yAmount)
  return AHSpawn.new(
                     self:createSourcePositionXBy(duration, ease, xAmount),
                     self:createSourcePositionYBy(duration, ease, yAmount)
                     )
end

--- Creates an action to emit particles for `duration` seconds
-- NOTE: The particle system itself must be start()ed first.
-- @param duration (number) - how long the system should emit particles.
function Particles:createEmitAction(duration)
  local emitAction = AHSequence.new(
                                    function()
                                      self:startSystem()
                                    end,
                                    AHDelay.new(duration),
                                    function()
                                      self:stopSystem()
                                    end
                                    )
  return emitAction
end

--- Emits particles for `duration` seconds
-- NOTE: The particle system itself must be start()ed first.
-- @param duration (number) - how long the system should emit particles.
function Particles:emitFor(duration)
  local emitAction = self:createEmitAction(duration)
  return emitAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createDeckForTexture(texture)
  local deck = MOAIGfxQuad2D.new()
  deck:setTexture(texture)

  return deck
end

local function autoDisableAlphaPremultiplilcation(self)
  local sourceBlendFunction, destinationBlendFunction = self:getBlendMode()
  local isStandardBlendMode = ((sourceBlendFunction == MOAIProp.GL_SRC_ALPHA)
                               and (destinationBlendFunction == MOAIProp.GL_ONE_MINUS_SRC_ALPHA))
  local needsPremultiplication = (isStandardBlendMode)
  self:setPremultiply(needsPremultiplication)
end

-- Constructor ------------------------

--- Creates a MOAICCParticleSystem, wrapped with some additional Lua methods, from the given .pex file.
-- @param pexFilename (string) - the filename of the pex file to load.
-- @return (MOAICCParticleSystem) - the new system
function Particles.new(pexFilename)
  local self = MOAICCParticleSystem.new()
  Group.makeGroup(self)
  Mixin.mix(self, Particles)

  local pexPath = FileUtils.findFile(pexFilename)
  assert(pexPath ~= nil, "No particle configuration file found at '" .. tostring(pexFilename) .. "'")
  self:load(pexPath)

  local texturePath = self:getTextureName()
  local texture = TextureLoader.loadTexture(texturePath)

  local deck = createDeckForTexture(texture)
  self:setDeck(deck)

  autoDisableAlphaPremultiplilcation(self)

  return self
end
