--- ReferenceDevice
-- @author Created by karl on July 17, 2015.

require("Screen")


-- ============================================================================
local Device = privateClass()
-- ============================================================================

function Device.new(screenWidthInPixels, screenHeightInPixels, scalingFactor)
  return {
    screenWidthInPixels = screenWidthInPixels,
    screenHeightInPixels = screenHeightInPixels,
    scalingFactor = scalingFactor,
    screenWidthInPoints = screenWidthInPixels / scalingFactor,
    screenHeightInPoints = screenHeightInPixels / scalingFactor
  }
end


-- ============================================================================
class("ReferenceDevice")
-- ============================================================================

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local devices = {
  ["ipad"] = Device.new(1536, 2048, 4),
  ["ipad pro"] = Device.new(2048, 2732, 4),
  ["iphone 4"] = Device.new(640, 960, 2),
  ["iphone 5"] = Device.new(640, 1136, 2),
  ["iphone 6"] = Device.new(750, 1334, 2),
  ["iphone 6+"] = Device.new(1242, 2208, 3),
  ["iphone 11 pro"] = Device.new(1125, 2436, 3),
  ["720p"] = Device.new(720, 1280, 2),
  ["1080p"] = Device.new(1080, 1920, 3),
  ["apple tv"] = Device.new(1920, 1080, 3)
}

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

--- Scale the specified width from one relative to the reference device size
--  to a value relative to the current screen size.
function ReferenceDevice.getScaledWidth(width)
  return (Screen.width / ReferenceDevice.screenWidthInPoints) * width
end

--- Scale the specified height from one relative to the reference device size
--  to a value relative to the current screen size.
function ReferenceDevice.getScaledHeight(height)
  return (Screen.height / ReferenceDevice.screenHeightInPoints) * height
end

--- Scale the specified width from one relative to the reference device size
--  to a value relative to the current screen size.
function ReferenceDevice.getInverseScaledWidth(width)
  return (1 / (Screen.width / ReferenceDevice.screenWidthInPoints)) * width
end

--- Scale the specified height from one relative to the reference device size
--  to a value relative to the current screen size.
function ReferenceDevice.getInverseScaledHeight(height)
  return (1 / (Screen.height / ReferenceDevice.screenHeightInPoints)) * height
end

--- Set the current reference device.
function ReferenceDevice.setDevice(deviceName)
  local device = devices[deviceName]
  assert(device, deviceName .. ": Unknown device")
  for key, value in pairs(device) do
    ReferenceDevice[key] = value
  end
end
