--- ScreenshotHelper
-- @author Created by Aaron Barrett on September 13, 2023.

class("ScreenshotHelper")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SCREENSHOT_FORMAT_EXTENSION = ".png"
local SCREENSHOT_HOME_SUBFOLDER = "Desktop"
local SCREENSHOT_INDEX_FORMAT_STRING = "%03d"
local SCREENSHOT_FILENAME_PREFIX = "screenshot_"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getScreenshotDirectoryPath(homePath)
  if (homePath ~= nil) then
    return tostring(homePath) .. "/" .. SCREENSHOT_HOME_SUBFOLDER .. "/"
  else
    return ""
  end
end

local function getScreenshotPathPrefix(homePath, gameName)
  local prefix = getScreenshotDirectoryPath(homePath)
  prefix = prefix .. SCREENSHOT_FILENAME_PREFIX
  if (gameName ~= nil) then
    prefix = prefix .. tostring(gameName) .. "_"
  end
  return prefix
end

local function getScreenshotPath(pathPrefix, screenshotIndex)
  return pathPrefix .. string.format(SCREENSHOT_INDEX_FORMAT_STRING, screenshotIndex) .. SCREENSHOT_FORMAT_EXTENSION
end

local function getExistingScreenshotFilesCount(pathPrefix)
  local screenshotIndex = -1
  repeat
    screenshotIndex = screenshotIndex + 1
    local path = getScreenshotPath(pathPrefix, screenshotIndex)
  until (not MOAIFileSystem.checkFileExists(path))
  return screenshotIndex
end

local function getNextScreenshotPath(homePath, gameName)
  local pathPrefix = getScreenshotPathPrefix(homePath, gameName)
  local screenshotCount = getExistingScreenshotFilesCount(pathPrefix)
  local nextScreenshotIndex = screenshotCount
  return getScreenshotPath(pathPrefix, nextScreenshotIndex)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ScreenshotHelper.takeScreenshot(homePath, gameName)
  local screenshotPath = getNextScreenshotPath(homePath, gameName)
  Log.debug("Saving screenshot at path: '" .. screenshotPath .. "'")
  
  local image = MOAIImage.new()

  local function afterGrab()
    image:writePNG(screenshotPath)
  end

  local frameBuffer = MOAIGfxDevice.getFrameBuffer()
  frameBuffer:grabNextFrame(image, afterGrab)
end

