--- Representation of a playing sound

-- ===========================================================================
class("SoundSource")
-- ===========================================================================

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SoundSource:getTime()
  if not self.finished then
    return Sound.getTime(self.soundID)
  end
  return 0
end

function SoundSource:stop()
  if not self.finished then
    self.finished = true
    Sound.stopSound(self.soundID)
  end
end

function SoundSource:pause()
  if not self.finished then
    self.paused = true
    Sound.pauseSound(self.soundID)
  end
end

function SoundSource:resume()
  if not self.finished then
    self.paused = false
    Sound.resumeSound(self.soundID)
  end
end

function SoundSource:setPaused(isPaused)
  if isPaused ~= self.paused then
    if isPaused then
      self:resume()
    else
      self:pause()
    end
  end
end

function SoundSource:seek(time)
  if not self.finished then
    self.paused = false
    time = math.max(time, 0)
    Sound.seek(self.soundID, time)
  end
end

function SoundSource:isFinished()
  return self.finished
end

function SoundSource:getAveragePowerForAllChannels()
  if not self.finished then
    return Sound.getAveragePowerForAllChannels(self.soundID)
  end
  return 0
end

function SoundSource:isPaused()
  return self.paused
end

function SoundSource:addFinishedHandler(handler)
  if not self.finished then
    Sound.addSoundFinishedHandler(self.soundID, function(soundID) handler(self) end)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function SoundSource.new(soundID, sourcePath)
  local self = {}
  Mixin.mix(self, SoundSource)
  self.soundID = soundID
  self.sourcePath = sourcePath
  self.finished = false
  self.paused = false

  return self
end
