--- TouchEvent
-- @author Created by karl on June 18, 2014.

-- ===========================================================================
class("TouchEvent")
-- ===========================================================================

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Values returned by touch handlers when handling a specific event.
TouchEvent.HANDLING = {
  -- Continue processing as normal (the default, assumed when no value is returned)
  CONTINUE = "TouchEvent.HANDLING.CONTINUE",
  -- Hand off the event, potentially to another focused handler.
  HAND_OFF = "TouchEvent.HANDLING.HAND_OFF",
}

-- Maximum amount of movement before a touch's movement can no longer be
-- considered accidental.
local MAX_ACCIDENTAL_MOVEMENT = 9

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TouchEvent:isPhase(phase)
  return self.phase == phase
end

function TouchEvent:isTouchStarting()
  return (self.phase == TouchManager.TOUCH_BEGAN)
end

function TouchEvent:isTouchMoving()
  return (self.phase == TouchManager.TOUCH_MOVED)
end

function TouchEvent:isTouchEnding()
  return (self.phase == TouchManager.TOUCH_ENDED)
end

function TouchEvent:isExclusiveTo(handler)
  return self.exclusiveHandler == handler
end

function TouchEvent:isExclusive()
  return self.exclusiveHandler ~= nil
end

function TouchEvent:isWithinAccidentalMovementRange(startX, startY)
  local distance = MathUtils.distance(startX, startY, self.x, self.y)
  return distance <= MAX_ACCIDENTAL_MOVEMENT
end

function TouchEvent:blacklistHandler(handler)
  self.handlerBlacklist[handler] = true
end

function TouchEvent:isHandlerBlacklisted(handler)
  return self.handlerBlacklist[handler]
end

function TouchEvent:isHandoff()
  return self._isHandoff
end

function TouchEvent:beginHandoff()
  self.phase = TouchManager.TOUCH_BEGAN
  self.exclusiveHandler = nil
  self._isHandoff = true
end

function TouchEvent:clearHandoff()
  self._isHandoff = false
end

function TouchEvent:notifyHandledBy(handler)
  if handler:isExclusive() then
    self.exclusiveHandler = handler
  end
end

function TouchEvent:clone()
  return TableUtils.shallowCopy(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TouchEvent.new(phase, id, x, y, tapCount)
  local self = {
    phase = phase,
    id = id,
    x = x,
    y = y,
    tapCount = tapCount,
    handlerBlacklist = {}
  }
  Mixin.mix(self, TouchEvent)

  return self
end
