--- AHAnchorPoint
-- @author Created by karl on October 24, 2014.

-- ----------------------------------------------------------------------------
-- AHAnchorPointXTo
-- ----------------------------------------------------------------------------

class("AHAnchorPointXTo")
--- Creates a new absolute attribute action to modify a prop's X anchor point value over time.
-- @param target (MOAIProp2D) - the MOAIProp2D to move.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param amount (number) - the destination X anchor point.
-- @return (MOAIEaseDriver) - A new AHAnchorPointXTo action.
function AHAnchorPointXTo.new(target, duration, ease, amount)
  local dimension = target:getWidth() or 0
  local attributeID = MOAITransform2D.ATTR_X_PIV
  local deltaFunction = AHAttributeTo.absoluteDeltaFunction
  return AHAttributeAction.new(target, duration, attributeID, deltaFunction, ease, amount * dimension)
end

-- ----------------------------------------------------------------------------
-- AHAnchorPointXBy
-- ----------------------------------------------------------------------------

class("AHAnchorPointXBy")
--- Creates a new relative attribute action to modify a prop's X anchor point value over time.
-- @param target (MOAIProp2D) - the MOAIProp2D to move.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param amount (number) - the amount to offset the X anchor point.
-- @return (MOAIEaseDriver) - A new AHAnchorPointXBy action.
function AHAnchorPointXBy.new(target, duration, ease, amount)
  local dimension = target:getWidth() or 0
  local attributeID = MOAITransform2D.ATTR_X_PIV
  local deltaFunction = AHAttributeBy.relativeDeltaFunction
  return AHAttributeAction.new(target, duration, attributeID, deltaFunction, ease, amount * dimension)
end

-- ----------------------------------------------------------------------------
-- AHAnchorPointYTo
-- ----------------------------------------------------------------------------

class("AHAnchorPointYTo")
--- Creates a new absolute attribute action to modify a prop's Y anchor point value over time.
-- @param target (MOAIProp2D) - the MOAIProp2D to move.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param amount (number) - the destination Y anchor point.
-- @return (MOAIEaseDriver) - A new AHAnchorPointYTo action.
function AHAnchorPointYTo.new(target, duration, ease, amount)
  local dimension = target:getHeight() or 0
  local attributeID = MOAITransform2D.ATTR_Y_PIV
  local deltaFunction = AHAttributeTo.absoluteDeltaFunction
  return AHAttributeAction.new(target, duration, attributeID, deltaFunction, ease, amount * dimension)
end

-- ----------------------------------------------------------------------------
-- AHAnchorPointYBy
-- ----------------------------------------------------------------------------

class("AHAnchorPointYBy")
--- Creates a new relative attribute action to modify a prop's Y anchor point value over time.
-- @param target (MOAIProp2D) - the MOAIProp2D to move.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param amount (number) - the amount to offset the Y anchor point.
-- @return (MOAIEaseDriver) - A new AHAnchorPointYBy action.
function AHAnchorPointYBy.new(target, duration, ease, amount)
  local dimension = target:getHeight() or 0
  local attributeID = MOAITransform2D.ATTR_Y_PIV
  local deltaFunction = AHAttributeBy.relativeDeltaFunction
  return AHAttributeAction.new(target, duration, attributeID, deltaFunction, ease, amount * dimension)
end

-- ----------------------------------------------------------------------------
-- AHAnchorPointTo
-- ----------------------------------------------------------------------------

class("AHAnchorPointTo")
--- Creates a new absolute attribute action to modify a prop's anchor point over time.
-- @param target (MOAIProp2D) - the MOAIProp2D to move.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param xAmount (number) - the destination X coordinate.
-- @param yAmount (number) - the destination Y coordinate.
-- @return (MOAICoroutine) - A new AHAnchorPointTo action.
function AHAnchorPointTo.new(target, duration, ease, xAmount, yAmount)
  local xAction = AHAnchorPointXTo.new(target, duration, ease, xAmount)
  local yAction = AHAnchorPointYTo.new(target, duration, ease, yAmount)
  return AHSpawn.new(xAction, yAction)
end

-- ----------------------------------------------------------------------------
-- AHAnchorPointBy
-- ----------------------------------------------------------------------------

class("AHAnchorPointBy")
--- Creates a new relative attribute action to modify a prop's anchor point over time.
-- @param target (MOAIProp2D) - the MOAIProp2D to move.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param xAmount (number) - the amount to offset the X coordinate.
-- @param yAmount (number) - the amount to offset the Y coordinate.
-- @return (MOAICoroutine) - A new AHAnchorPointBy action.
function AHAnchorPointBy.new(target, duration, ease, xAmount, yAmount)
  local xAction = AHAnchorPointXBy.new(target, duration, ease, xAmount)
  local yAction = AHAnchorPointYBy.new(target, duration, ease, yAmount)
  return AHSpawn.new(xAction, yAction)
end
