--- AHCallLogic
-- @author Created by Aaron Barrett on December 27, 2013.

--- Use AHCallLogic to run logic as part of an AHSequence.
-- AHCallLogic is intended to have a duration of 0.0 seconds, so calling
-- wait() or coroutine.yield() in `functionToCall` may have undefined
-- results.

class("AHCallLogic")
-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function AHCallLogic:start()
  local arguments = self.arguments
  return self.super.start(self, unpack(arguments))
end

function AHCallLogic.wrapIfIsFunction(action)
  if type(action) == "function" then
    action = AHCallLogic.new(action)
  end
  return action
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

--- Creates a new AHCallLogic action.
-- @param functionToCall (function) - The function to call when the action is started.
-- @param ... - list of arguments to pass to `functionToCall`.
-- @return (MOAICoroutine) - A new AHCallLogic action.
function AHCallLogic.new(functionToCall, ...)
  assert(type(functionToCall) == "function", "`functionToCall` must be a function.")

  local self = MOAICoroutine.new()
  AHAction.makeAHAction(self)
  Mixin.mix(self, AHCallLogic)

  self.arguments = arg

  self:setFunction(functionToCall)

  return self
end
