--- AHCustomAction
--
-- Calls a user-supplied update function for custom control of the action.


class("AHCustomAction")

-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

-- Attributes -------------------------

local ATTR_ID = 0


-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

local function makeScriptNode(updateFunction)
    local scriptNode = MOAIScriptNode.new()
    scriptNode:reserveAttrs(1)
    scriptNode:setCallback(function(node)
        local normalizedTime = node:getAttr(ATTR_ID)
        updateFunction(normalizedTime)
    end)
    return scriptNode
end

function AHCustomAction:_AHAttributeAction_getInitialValue(target)
    return 0
end


-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

--- Creates a new AHCustomAction.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param updateFunction (function) - Gets called whenever the action updates. Params: (normalizedTime) a value from 0 to 1.
-- @return (AHCustomAction) - A new AHCustomAction action.
function AHCustomAction.new(duration, ease, updateFunction)
    local scriptNode = makeScriptNode(updateFunction)
    local self = AHAttributeTo.new(scriptNode, duration, ATTR_ID, ease, 1)
    Mixin.mix(self, AHCustomAction)
    return self
end
