--- AHDelay
-- @author Created by Aaron Barrett on December 28, 2013.

-- Use AHDelay to wait for a certain duration without doing anything.
-- Note that unlike attribute actions, AHDelay may have infinite duration.
-- To create an AHDelay that will continue running until it is manually stopped,
-- pass math.huge as the duration argument.
-- Also see wait.lua for waitFor(), which relies on AHDelay.

class("AHDelay")
-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

--- Creates a new AHDelay action.
-- @param duration (number) - the length of the action in seconds.
-- @return (MOAITimer) - A new AHDelay action.
function AHDelay.new(duration)
  assert(type(duration) == "number", "`duration` of an AHDelay must be a number.")
  assert(duration >= 0.0, "`duration` of an AHDelay cannot be negative.")

  local self = MOAITimer.new()
  AHAction.makeAHAction(self)
  Mixin.mix(self, AHDelay)

  self.actionType = AHActionTypes.DELAY

  self.duration = duration
  self:setSpan(duration)

  return self
end



-- ===========================================================================
-- AHDelayed
-- ===========================================================================

-- Convenience action: Runs an action after a delay.
class("AHDelayed")

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

--- Acts like a delayed spawn.
function AHDelayed.new(duration, ...)
  return AHSequence.new(AHDelay.new(duration), AHSpawn.new(...))
end
