--- AHLoop
-- @author Created by Aaron Barrett on December 26, 2013.

-- Use AHLoop to play the same AHAction multiple times.

class("AHLoop")
-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

local function playLoop(self)
  for i = 1, self.timesToLoop do
    self.actionToLoop:start()
    MOAICoroutine.blockOnAction(self.actionToLoop)
  end
end

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function AHLoop:stop()
  self.actionToLoop:stop()
  return self.super.stop(self)
end

function AHLoop:pause(isPaused)
  if(isPaused == nil) then isPaused = true end
  self.actionToLoop:pause(isPaused)
  self.super.pause(self, isPaused)
end

-- ---------------------------------------------------------------------------
-- Object creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

--- Creates a new AHLoop action.
-- @param actionToLoop (AHAction or function) - the action or function that this AHLoop will play.
-- @param timesToLoop (number) - the number of times to repeat the action. (default = math.huge)
-- @return (MOAICoroutine) - A new AHLoop action.
function AHLoop.new(actionToLoop, timesToLoop)
  timesToLoop = timesToLoop or math.huge

  assert(type(timesToLoop) == "number", "`timesToLoop` must be a number.")
  assert(timesToLoop > 0, "`timesToLoop` must be positive.")
  assert(timesToLoop % 1 == 0 or timesToLoop == math.huge, "`timesToLoop` must be an integer.")

  actionToLoop = AHCallLogic.wrapIfIsFunction(actionToLoop)

  assert(type(actionToLoop) == "userdata", "`actionToLoop` must be a MOAIAction.")
  assert(not actionToLoop:isActive(), "Cannot add an action to an AHLoop after it has started.")

  local self = MOAICoroutine.new()
  AHAction.makeAHAction(self)
  Mixin.mix(self, AHLoop)

  self.actionToLoop = actionToLoop
  self.timesToLoop = timesToLoop

  self:setFunction(function()
      playLoop(self)
    end)

  return self
end
