--- AHOpacity actions
-- @author Created by Aaron Barrett on December 26, 2013.

-- Use AHOpacity to modify a MOAIColor's opacity value.

-- ----------------------------------------------------------------------------
-- AHOpacityTo
-- ----------------------------------------------------------------------------

class("AHOpacityTo")
-- Constructor ------------------------

--- Creates a new absolute attribute action to modify a MOAIColor's opacity value over time.
-- @param target (MOAIColor) - the MOAIColor (often a MOAIProp2D or Sprite) to modify.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param amount (number) - the destination opacity value.
-- @return (MOAIEaseDriver) - A new AHOpacityTo action.
function AHOpacityTo.new(target, duration, ease, amount)
  local attributeID = MOAIColor.ATTR_OPACITY
  local deltaFunction = AHAttributeTo.absoluteDeltaFunction
  local self = AHAttributeAction.new(target, duration, attributeID, deltaFunction, ease, amount)
  return self
end

-- ----------------------------------------------------------------------------
-- AHOpacityBy
-- ----------------------------------------------------------------------------

class("AHOpacityBy")
-- Constructor ------------------------

--- Creates a new relative attribute action to modify a MOAIColor's opacity value over time.
-- @param target (MOAIColor) - the MOAIColor (often a MOAIProp2D or Sprite) to modify.
-- @param duration (number) - the length of the action in seconds.
-- @param ease (MOAIEase, function, curve, or `nil`) - The AHEase object to use (or `nil`, for a linear ease).
-- @param amount (number) - the amount to offset the opacity value.
-- @return (MOAIEaseDriver) - A new AHOpacityBy action.
function AHOpacityBy.new(target, duration, ease, amount)
  local attributeID = MOAIColor.ATTR_OPACITY
  local deltaFunction = AHAttributeBy.relativeDeltaFunction
  local self = AHAttributeAction.new(target, duration, attributeID, deltaFunction, ease, amount)
  return self
end
