--- AHRecursive
-- Applies an action recursively to a target and all of its children.
-- @author Created by karl on February 23, 2015.

class("AHRecursive")


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function addRecursive(self, actionConstructor, target, ...)
  self:addAction(actionConstructor(target, ...))
  if target.children then
    for _, child in ipairs(target.children) do
      addRecursive(self, actionConstructor, child, ...)
    end
  end
end

-- Constructor ------------------------

--- Create a new recursive action.
-- @param actionConstructor (function) - Function that will construct the actions.
-- @param target (MOAI prop) - The action's target.
-- @param ... (any) - Any parameters that must be passed to the constructor besides the target.
function AHRecursive.new(actionConstructor, target, ...)
  local self = AHSpawn.new()
  Mixin.mix(self, AHRecursive)

  addRecursive(self, actionConstructor, target, ...)

  return self
end
