--- AHRunForDuration
--
-- Runs an action or coroutine for a specific amount of time.
-- When the time is up, the action or coroutine is stopped immediately, even
-- if it was in the middle of something.


class("AHRunForDuration")

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function AHRunForDuration:stop()
  self.AHRunForDuration_childAction:stop()
  self.AHSequence.stop(self)
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

--- Construct a new AHRunForDuration
-- @param duration (number) - Duration of this action.
-- @param action (AHAction or MOAICoroutine) - action to run
-- @return (AHRunForDuration) - A new action.
function AHRunForDuration.new(duration, action)
  local self = AHSequence.new(
      function() action:start() end,
      AHDelay.new(duration),
      function() action:stop() end
  )
  Mixin.mix(self, AHRunForDuration, "AHSequence")

  self.AHRunForDuration_childAction = action

  return self
end
