class("AHSmoothShake")
--- Shakes the object by moving in all directions. All parameters have default values.
-- WARNING: If a shake is called while a shake is already playing, object might not return to initial position.
-- @param object - this will be shaken
-- @param xShakeAmp, yShakeAmp - how far the object offsets at the peak of the animation
-- @param shakesPerSecond, durationSeconds - yes.
-- @param completeHandler - function that's called after shake is fully decayed
-- @return waitable/stoppable action
function AHSmoothShake.new(object, xShakeAmp, yShakeAmp, shakesPerSecond, durationSeconds, completeHandler)
  local x0, y0 = object:getLoc()
  xShakeAmp = xShakeAmp or 10
  yShakeAmp = yShakeAmp or 0
  shakesPerSecond = shakesPerSecond or 20
  durationSeconds = durationSeconds or 1
  local shakeNumber = 0
  local shakeFrequency = durationSeconds/shakesPerSecond
  local shakeCount = durationSeconds * shakesPerSecond

  local prevXShake, prevYShake = 0, 0
  local xShake, yShake = xShakeAmp * MathUtils.randomSign(), yShakeAmp * MathUtils.randomSign()
  local dxShake, dyShake = 0, 0
  local shakeTime = shakeNumber * shakeFrequency
  local shakeStartTime = 0

  local shake = AHCustomAction.new(durationSeconds, nil, function(t)
    t = math.pow(t, 0.5)
    if (t > shakeTime) then
      prevXShake, prevYShake = xShake, yShake
      local shakeShrink = 1 - t
      xShake, yShake = -xShake * shakeShrink, -yShake * shakeShrink
      dxShake, dyShake = xShake - prevXShake, yShake - prevYShake
    end
    while(t > shakeTime) do
      shakeNumber = shakeNumber + 1
      shakeStartTime = shakeTime
      shakeTime = shakeNumber * shakeFrequency
    end

    local normalizedShakeTime = (t - shakeStartTime)/(shakeTime - shakeStartTime)
    object:setLoc(x0 + prevXShake + dxShake * normalizedShakeTime, y0 + dyShake * normalizedShakeTime)
  end)
  local afterShake = AHCallLogic.new(function()
    object:setLoc(x0, y0)
    if (completeHandler) then completeHandler() end
  end)

  local animation = AHSequence.new(shake, afterShake)
  return animation
end
