--- The different types of bonus in the games
-- @module BonusTypes

class("BonusTypes")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

BonusTypes.TYPES = {
  TIME_BONUS = "time",
  LIVES_BONUS = "lives",
  MAX_STREAK_BONUS = "streak",
  SPEED_BONUS = "speed",
  EXTRA_QUESTION_BONUS = "extra_question",
  ACCURACY_BONUS = "accuracy",
  PERFECT_SET_BONUS = "perfect_set",
  CONSISTENCY_BONUS = "consistency",
  EFFICIENCY_BONUS = "efficiency",
  EXTRA_COLLECTED_BONUS = "extra_collected",
  BONUS_ROUND_SPEED_BONUS = "bonus_round_speed",
  ADVANCED_RESPONSE_BONUS = "advanced_response",
  EXTRA_MATCH_BONUS = "extra_match"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Note, not all bonuses have a notification string. Only bonuses that are sent
-- to the notification tray in-game are mapped here
local NOTIFICATION_STRING_FOR_TYPES = {
  [BonusTypes.TYPES.TIME_BONUS] = "BonusTray.TIME_BONUS",
  [BonusTypes.TYPES.MAX_STREAK_BONUS] = "BonusTray.MAX_STREAK_BONUS",
  [BonusTypes.TYPES.LIVES_BONUS] = "BonusTray.LIVES_BONUS",
  [BonusTypes.TYPES.SPEED_BONUS] = "BonusTray.SPEED_BONUS",
  [BonusTypes.TYPES.EXTRA_QUESTION_BONUS] = "BonusTray.EXTRA_QUESTION_BONUS",
  [BonusTypes.TYPES.ACCURACY_BONUS] = "BonusTray.ACCURACY_BONUS",
  [BonusTypes.TYPES.PERFECT_SET_BONUS] = "BonusTray.PERFECT_SET_BONUS",
  [BonusTypes.TYPES.CONSISTENCY_BONUS] = "BonusTray.CONSISTENCY_BONUS",
  [BonusTypes.TYPES.EFFICIENCY_BONUS] = "BonusTray.EFFICIENCY_BONUS",
  [BonusTypes.TYPES.EXTRA_COLLECTED_BONUS] = "BonusTray.EXTRA_WORD_BONUS",
  [BonusTypes.TYPES.BONUS_ROUND_SPEED_BONUS] = "BonusTray.BONUS_ROUND_SPEED_BONUS",
  [BonusTypes.TYPES.ADVANCED_RESPONSE_BONUS] = "BonusTray.ADVANCED_RESPONSE_BONUS",
  [BonusTypes.TYPES.EXTRA_MATCH_BONUS] = "BonusTray.EXTRA_MATCH_BONUS"
}

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BonusTypes.getNotificationString(bonusType)
  local notificationString = NOTIFICATION_STRING_FOR_TYPES[bonusType]
  assert((notificationString ~= nil), string.format("No matching notification string for requested bonus type %s", bonusType))

  return notificationString
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------
