--- CompositeBranchNode
-- @author Created by andrewc on February 20, 2019
-- This class creates branch nodes which hold two other nodes.
-- Evaluating these composite nodes will evaluate the child nodes,
-- and compare those results with some logic.

class("CompositeBranchNode")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function addAndOperand(self)
  self.branchType = "and"
  self.evaluate = function()
    return self.childA:evaluate() and self.childB:evaluate()
  end
end

local function addOrOperand(self)
  self.branchType = "or"
  self.evaluate = function()
    return self.childA:evaluate() or self.childB:evaluate()
  end
end

local function newNode(branchA, branchB)
  local self = BranchNode.new()
  Mixin.mix(self, CompositeBranchNode)

  assert(branchA and branchB, "Composites require two branches to evaluate.")
  assert(branchA.evaluate ~= nil, "Invalid argument for composite node.")
  assert(branchB.evaluate ~= nil, "Invalid argument for composite node.")
  self.childA = branchA
  self.childB = branchB
  return self
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CompositeBranchNode:evaluate()
  return self.evaluate()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Node Constructors ------------------------

function CompositeBranchNode.newAnd(branchA, branchB)
  local self = newNode(branchA, branchB)
  addAndOperand(self)
  
  return self
end

function CompositeBranchNode.newOr(branchA, branchB)
  local self = newNode(branchA, branchB)
  addOrOperand(self)
  
  return self
end
