--- NumericBranchNode
-- @author Created by andrewc on February 25, 2019
-- Parent class for BranchNodes with primarily numbers as arguments. 

class("NumericBranchNode")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COMPARATORS = {
  GREATER_THAN = "greaterThan",
  LESS_THAN = "lessThan",
  EQUALS = "equals"
}

local DEFAULT_COMPARATOR = COMPARATORS.EQUALS

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getComparator(comparator)
  assert(TableUtils.contains(COMPARATORS, comparator), "Invalid comparator: " .. comparator)
  if (COMPARATORS.EQUALS == comparator) then
    local func = function(a, b)
      return a == b
    end
    return func
  end
  if (COMPARATORS.GREATER_THAN == comparator) then
    local func = function(a, b)
      return a > b
    end
    return func
  end
  if (COMPARATORS.LESS_THAN == comparator) then
    local func = function(a, b)
      return a < b
    end
    return func
  end
end

local function isComparator(comparator)
  return TableUtils.contains(COMPARATORS, comparator)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function NumericBranchNode.checkComparator(parameters)
  local answers, comparator = nil
  if isComparator(#parameters > 0 and parameters[1]) then
    assert(#parameters > 1, "Comparators require a value to compare against.")
    answers = TableUtils.subarray(parameters, 2, #parameters)
    comparator = getComparator(parameters[1])
  else
    answers = parameters
    comparator = getComparator(DEFAULT_COMPARATOR)
  end
  return answers, comparator
end

function NumericBranchNode.checkAnswers(correctAnswer, answers, comparator)
  if (correctAnswer ~= nil) then
    comparator = comparator or getComparator(DEFAULT_COMPARATOR)
    for _, answer in ipairs(answers) do
      if comparator(correctAnswer, answer) then
        return true
      end
    end
  end
  return false
end

function NumericBranchNode:convertAnswersToNumbers()
  local convertedAnswers = {}
  for _, element in ipairs(self.answers) do
    local number = tonumber(element)
    assert(type(number) == "number", "Number expected: " .. element)
    table.insert(convertedAnswers, number)
  end
  self.answers = convertedAnswers
end

function NumericBranchNode:evaluate()
  error("Evaluate should not be called on NumericBranchNode")
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Node Constructors ------------------------

function NumericBranchNode.new()
  local self = BranchNode.new()
  Mixin.mix(self, NumericBranchNode)

  return self
end
