--- ThresholdBranchNode
-- @author Created by andrewc on March 2, 2019
-- This class creates branch nodes which holds other branch nodes.
-- If a number of branches held in this node evaluate to true so they equal
-- or exceed the 'threshold', then this branch evaluates to true.

class("ThresholdBranchNode")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ThresholdBranchNode:evaluate()
  local numTrue = 0
  for _, branch in ipairs(self.branches) do
    if (branch:evaluate()) then
      numTrue = numTrue + 1
    end
  end

  return (numTrue >= self.threshold)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function ThresholdBranchNode.new(threshold, branches)
  local self = BranchNode.new()
  Mixin.mix(self, ThresholdBranchNode)
  self.branchType = "threshold"

  assert(threshold, "Threshold requires a threshold to check against")
  self.threshold = tonumber(threshold)
  assert(type(self.threshold) == "number", "Threshold requires a number.")

  assert(branches and #branches > 0, "Threshold require at least one branch to check.")
  self.branches = branches

  return self
end