--- CardCarouselCardBuffer
-- @author Created by Aaron Barrett on April 20, 2015.

class("CardCarouselCardBuffer")

require("containers.card_carousel.CardCarouselCardCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function stopRenderingAfterNextRender(self)
  runInCoroutine(function()
    local frameBufferTexture = self.frameBufferTexture
    local initialRenderCount = frameBufferTexture:getRenderCount()
    waitUntil(function() return frameBufferTexture:getRenderCount() > initialRenderCount end)
    MOAIRenderMgr.removeFromBufferTable(frameBufferTexture)
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CardCarouselCardBuffer:setCard(card)
  local frameBufferTexture = self.frameBufferTexture
  local oldCard = self.card

  if (card == oldCard) then
    return
  end

  if (card ~= nil) then
    self:setVisible(true)
    local layer = card:getLayer()
    local renderTable = {layer}
    frameBufferTexture:setRenderTable(renderTable)

    MOAIRenderMgr.addToBufferTable(frameBufferTexture)

    if (not card:isDynamic()) then
      stopRenderingAfterNextRender(self)
    end
  else
    self:setVisible(false)
    frameBufferTexture:setRenderTable({})
    MOAIRenderMgr.removeFromBufferTable(frameBufferTexture)
  end

  self.card = card
end

function CardCarouselCardBuffer:setScrollProgress(progress, isHorizontal)
  if (self.card ~= nil) then
    self.card:setScrollProgress(progress, isHorizontal)
  end
end

function CardCarouselCardBuffer:startAnimation()
  if (self.card ~= nil) then
    self.card:startAnimation()
  end
end

function CardCarouselCardBuffer:stopAnimation()
  if (self.card ~= nil) then
    self.card:stopAnimation()
  end
end

function CardCarouselCardBuffer:reset()
  if (self.card ~= nil) then
    self.card:reset()
  end
end

function CardCarouselCardBuffer:onEnter()
  if (self.card ~= nil) then
    self.card:onEnter()
  end
end

function CardCarouselCardBuffer:onLeaveBegin()
  if (self.card ~= nil) then
    self.card:onLeaveBegin()
  end
end

function CardCarouselCardBuffer:onLeaveComplete()
  if (self.card ~= nil) then
    self.card:onLeaveComplete()
  end
end

function CardCarouselCardBuffer:getFrameBufferTexture()
  return self.frameBufferTexture
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CardCarouselCardBuffer.new(clearColor)
  local self = Group.newFullScreen()
  Mixin.mix(self, CardCarouselCardBuffer)

  local frameBufferTexture = MOAIFrameBufferTexture.new()
  frameBufferTexture:init(MOAIEnvironment.horizontalResolution, MOAIEnvironment.verticalResolution)
  frameBufferTexture:setClearColor(unpack(clearColor))
  self.frameBufferTexture = frameBufferTexture

  local deck = MOAIGfxQuad2D.new()
  deck:setTexture(frameBufferTexture)
  deck:setRect(0.0, 0.0, Screen.width, Screen.height)
  deck:setUVRect(0.0, 0.0, 1.0, 1.0)

  self:setDeck(deck)
  MOAIRenderMgr.addToBufferTable(frameBufferTexture)

  self.card = nil

  return self
end
