--- CardCarouselCardCommon
-- @author Created by Aaron Barrett on April 21, 2015.

class("CardCarouselCardCommon")

require("containers.card_carousel.CardCarouselTextView")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

CardCarouselCardCommon.COLORS = {
  WHITE = {1, 1, 1},
  BLACK = {0, 0, 0},
  BLUE = ColorUtils.colorHex24ToStd(0x05c3f9),
  ORANGE = ColorUtils.colorHex24ToStd(0xffb237),
  LIGHT_GREY = ColorUtils.colorHex24ToStd(0xa1a1a1),
  LIGHT_GREY2 = ColorUtils.colorHex24ToStd(0xeeeeee),
  GREY = ColorUtils.colorHex24ToStd(0x737373),
  DARK_GREY = ColorUtils.colorHex24ToStd(0x444444)
}

CardCarouselCardCommon.MARGINS = {
  HORIZONTAL = Screen.getWidthPart(0.1),
  VERTICAL = Screen.getHeightPart(0.05),
}

CardCarouselCardCommon.CACHED_FONTS = {
  ITALIC_SMALL = Font.new(FontFaces.DIN_OT_MEDIUM_ITALIC, 15),
  MEDIUM_SMALL = Font.new(FontFaces.DIN_OT_MEDIUM, 15),
  MEDIUM_MEDIUM = Font.new(FontFaces.DIN_OT_MEDIUM, 18),
  MEDIUM_LARGE = Font.new(FontFaces.DIN_OT_MEDIUM, 35),
  LIGHT_SMALL = Font.new(FontFaces.DIN_OT_LIGHT, 26),
  LIGHT_MEDIUM = Font.new(FontFaces.DIN_OT_LIGHT, 35),
  LIGHT_LARGE = Font.new(FontFaces.DIN_OT_LIGHT, 53),
  TISA_MEDIUM = Font.new(FontFaces.TISA_OFFC, 18),
  TISA_ITALIC = Font.new(FontFaces.TISA_OFFC_ITALIC, 18)
}

CardCarouselCardCommon.CONTENT_WIDTH = Screen.width - CardCarouselCardCommon.MARGINS.HORIZONTAL * 2
CardCarouselCardCommon.CONTENT_HEIGHT = Screen.height - CardCarouselCardCommon.MARGINS.VERTICAL * 2

CardCarouselCardCommon.DEFAULT_STAGGER_AMOUNT = Screen.getWidthPart(0.05)

CardCarouselCardCommon.DEFAULT_TOUCH_SCROLLER_DECAY_FACTOR = 0.96

local KNOWN_TARGET_HEIGHT = 667 -- Polished on iPhone 6


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CardCarouselCardCommon.newStandardBackground(color)
  return FilledRectangle.newFullScreen(color or CardCarouselCardCommon.COLORS.WHITE)
end

function CardCarouselCardCommon.newStandardWorkArea()
  return Group.new(CardCarouselCardCommon.CONTENT_WIDTH, CardCarouselCardCommon.CONTENT_HEIGHT)
end

function CardCarouselCardCommon:newSeparatorLine()
  local rect = FilledRectangle.new(Screen.width - CardCarouselCardCommon.MARGINS.HORIZONTAL, 1, CardCarouselCardCommon.COLORS.LIGHT_GREY)
  local container = Group.new(CardCarouselCardCommon.CONTENT_WIDTH, rect:getHeight())
  rect:setAnchorPointBottomLeft()
  container:insert(rect)

  return container
end

function CardCarouselCardCommon.newTextAttribute(fontData, color, otherAttributes)
  local baseAttributes = {
    font = fontData,
    color = color
  }

  return TableUtils.unionSet(otherAttributes or {}, baseAttributes)
end

function CardCarouselCardCommon.newLabel(text, font, size, color)
  local label = Label.new(text, Font.new(font, size))
  if color then
    label:setColor(unpack(color))
  end
  return label
end

function CardCarouselCardCommon.newMultilineLabel(text, font, size, width, horizontalAlignment, color)
  local label = DynamicTextArea.newSimple(text, width, Font.new(font, size), horizontalAlignment)
  if color then
    label:setColor(unpack(color))
  end
  return label
end

function CardCarouselCardCommon.newCachedFontLabel(text, font, color)
  local label = Label.new(text, font)
  if color then
    label:setColor(unpack(color))
  end
  return label
end

function CardCarouselCardCommon.createBouncyTextAction(textViews)
  local bouncyAction = AHSpawn.new()
  local bouncyEase = AHEaseSimpleInOut.new(2.0)

  for _, textView in ipairs(textViews) do
    local bouncyViews = textView:getViewsWithAttribute("bouncy", true)

    for bouncyViewIndex, bouncyView in pairs(bouncyViews) do
      local staggerDuration = 0.0875 + bouncyViewIndex * 0.0125
      local bouncyViewAction = AHSequence.new(
                                              AHDelay.new(staggerDuration),
                                              AHLocationYBy.new(bouncyView, 0.2, bouncyEase, 4.0),
                                              AHLocationYBy.new(bouncyView, 0.2, bouncyEase, -7.0),
                                              AHLocationYBy.new(bouncyView, 0.2, bouncyEase, 4.5),
                                              AHLocationYBy.new(bouncyView, 0.2, bouncyEase, -1.5)
                                              )
      bouncyAction:addAction(bouncyViewAction)
    end
  end

  return bouncyAction
end
