--- CardCarouselNavigator
-- @author Created by Aaron Barrett on April 20, 2015.

class("CardCarouselNavigator")

require("containers.card_carousel.CardCarouselCardBuffer")
require("containers.card_carousel.CardCarouselCard")
require("containers.card_carousel.CardCarouselCardDefinitions")
require("containers.card_carousel.CardCarouselTouchTracker")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HOME_ROW = 2
local TITLE_COLUMN = 2

local SCROLL_CENTER_X = Screen.getWidthPart(0.5)
local SCROLL_CENTER_Y = Screen.getHeightPart(0.5)

local SCROLL_START_THRESHOLD = 4
local SCROLL_SPEED_THRESHOLD = 160
local SCROLL_DISTANCE_THRESHOLD = Screen.getWidthPart(0.2)
local TAP_MAX_TIME_BETWEEN_START_AND_END_EVENTS = 0.2

CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY = Screen.getWidthPart(0.22)

CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL = 0.6
CardCarouselNavigator.SCROLL_ANIMATION_DURATION_VERTICAL = 0.8

CardCarouselNavigator.DIRECTION_UP = "DIRECTION_UP"
CardCarouselNavigator.DIRECTION_DOWN = "DIRECTION_DOWN"
CardCarouselNavigator.DIRECTION_LEFT = "DIRECTION_LEFT"
CardCarouselNavigator.DIRECTION_RIGHT = "DIRECTION_RIGHT"

CardCarouselNavigator.TAP_DIRECTION_FORWARD = CardCarouselNavigator.DIRECTION_LEFT
CardCarouselNavigator.TAP_DIRECTION_BACK = CardCarouselNavigator.DIRECTION_RIGHT
CardCarouselNavigator.TAP_DIRECTION_DOWN = CardCarouselNavigator.DIRECTION_UP

local function newDirectionData(isHorizontal, scrollStart, scrollRange, futurePositionX, futurePositionY, futureIndexXOffset, futureIndexYOffset)
  return {
    isHorizontal = isHorizontal,
    scrollStart = scrollStart,
    scrollRange = scrollRange,
    futurePositionX = futurePositionX,
    futurePositionY = futurePositionY,
    futureIndexXOffset = futureIndexXOffset,
    futureIndexYOffset = futureIndexYOffset,
  }
end

local DIRECTION_DATA = {
  [CardCarouselNavigator.DIRECTION_UP] = newDirectionData(false, SCROLL_CENTER_Y,  Screen.height, Screen.getWidthPart(1.5), Screen.getHeightPart(0.5),  0,  1),
  [CardCarouselNavigator.DIRECTION_DOWN] = newDirectionData(false, SCROLL_CENTER_Y, -Screen.height, Screen.getWidthPart(1.5), Screen.getHeightPart(2.5),  0, -1),
  [CardCarouselNavigator.DIRECTION_LEFT] = newDirectionData(true,  SCROLL_CENTER_X, -Screen.width,  Screen.getWidthPart(2.5), Screen.getHeightPart(1.5),  1,  0),
  [CardCarouselNavigator.DIRECTION_RIGHT] = newDirectionData(true,  SCROLL_CENTER_X,  Screen.width,  Screen.getWidthPart(0.5), Screen.getHeightPart(1.5), -1,  0),
}

local Z_SCROLLER = 10
local Z_BUFFER = 20
local Z_SEPARATOR = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getDirectionData(direction)
  return DIRECTION_DATA[direction]
end

local function getDragDistance(touchTracker)
  local touchDistanceX = touchTracker:getDifferenceX()
  local touchDistanceY = touchTracker:getDifferenceY()

  return MathUtils.vectorLength(touchDistanceX, touchDistanceY)
end

local function didExceedTouchThreshold(touchTracker)
  return getDragDistance(touchTracker) > SCROLL_START_THRESHOLD
end

local function getScrollDirection(touchTracker)
  local touchDistanceX = touchTracker:getDifferenceX()
  local touchDistanceY = touchTracker:getDifferenceY()
  if (math.abs(touchDistanceX) > math.abs(touchDistanceY)) then
    if (touchDistanceX > 0) then
      return CardCarouselNavigator.DIRECTION_RIGHT
    else
      return CardCarouselNavigator.DIRECTION_LEFT
    end
  else
    if (touchDistanceY > 0) then
      return CardCarouselNavigator.DIRECTION_UP
    else
      return CardCarouselNavigator.DIRECTION_DOWN
    end
  end
end

local function getSpeedInCurrentDirection(self)
  local eventResolution = 4

  local events = self.touchTracker:getEvents()
  if #events < 2 then
    return 0
  end

  local endIndex = #events
  local startIndex = endIndex - eventResolution
  if startIndex < 1 then
    startIndex = 1
  end
  local startEvent = events[startIndex]
  local endEvent = events[endIndex]

  local duration = endEvent.time - startEvent.time
  local speed = 0
  if self.scrollDirectionData.isHorizontal then
    speed = (endEvent.x - startEvent.x) / duration
    if self.scrollDirectionData == getDirectionData(CardCarouselNavigator.DIRECTION_LEFT) then
      speed = -speed
    end
  else
    speed = (endEvent.y - startEvent.y) / duration
    if self.scrollDirectionData == getDirectionData(CardCarouselNavigator.DIRECTION_DOWN) then
      speed = -speed
    end
  end

  return speed
end

local function getCard(self, cardIndexX, cardIndexY)
  local cardColumn = self.cardColumns[cardIndexX]
  if (cardColumn ~= nil) then
    return cardColumn[cardIndexY]
  end
  return nil
end

local function resetScroller(self, cardIndexX, cardIndexY)
  self.isScrolling = false
  self.initialScrollingDirection = nil
  self.scrollProgress = 0.0
  self.currentCardIndexX = cardIndexX
  self.currentCardIndexY = cardIndexY
  self.futureCardIndexX = nil
  self.futureCardIndexY = nil
  self.scrollDirectionData = nil

  self.futureBuffer:setCard(nil)
  local card = getCard(self, cardIndexX, cardIndexY)
  self.currentBuffer:setCard(card)
  self.scroller:setLoc(SCROLL_CENTER_X, SCROLL_CENTER_Y)
end

local function terminateScroll(self, cardIndexX, cardIndexY)
  self.isTouchActive = false
  resetScroller(self, cardIndexX, cardIndexY)
end

local function completeScroll(self)
  self.futureBuffer:setScrollProgress(1.0, self.scrollDirectionData.isHorizontal)
  self.futureBuffer:onEnter()
  self.currentBuffer:onLeaveComplete()
  terminateScroll(self, self.futureCardIndexX, self.futureCardIndexY)
  self.screenSeparator:setLoc(self.scroller:getWidthPart(0.5), self.scroller:getHeightPart(0.5))
end

local function cancelScroll(self)
  terminateScroll(self, self.currentCardIndexX, self.currentCardIndexY)
end

local function getScrollProgress(self)
  return self.scrollProgress
end

local function setScrollProgress(self, scrollProgress)
  local scrollCoordinate = self.scrollDirectionData.scrollStart + self.scrollDirectionData.scrollRange * scrollProgress
  local scroller = self.scroller

  if self.scrollDirectionData.isHorizontal then
    scroller:setX(scrollCoordinate)
  else
    scroller:setY(scrollCoordinate)
  end

  self.futureBuffer:setScrollProgress(scrollProgress, self.scrollDirectionData.isHorizontal)
  self.scrollProgress = scrollProgress
end

local function newScrollToAction(self, duration, ease, scrollAmount)
  return AHNonAttributeTo.new(self, duration, getScrollProgress, setScrollProgress, ease, scrollAmount)
end

local function handleDrag(self, touchX, touchY)
  local scrollAmount = nil

  if self.scrollDirectionData.isHorizontal then
    scrollAmount = self.touchTracker:getDifferenceX()
  else
    scrollAmount = self.touchTracker:getDifferenceY()
  end

  local scrollProgress = scrollAmount / self.scrollDirectionData.scrollRange
  setScrollProgress(self, scrollProgress)
end

local function didScrollQuicklyEnoughToComplete(self)
  local scrollSpeed = getSpeedInCurrentDirection(self)
  return scrollSpeed > SCROLL_SPEED_THRESHOLD
end

local function didScrollFarEnoughToComplete(self)
  local distance = getDragDistance(self.touchTracker)

  return distance > SCROLL_DISTANCE_THRESHOLD
end

local function didBeginAndFinishScrollInSameDirection(self)
  return self.initialScrollingDirection == getScrollDirection(self.touchTracker)
end

local function handleDrop(self)
  local shouldCompleteScroll = didScrollQuicklyEnoughToComplete(self) or
                               (didScrollFarEnoughToComplete(self) and didBeginAndFinishScrollInSameDirection(self))

  local scrollValue = 0
  local scrollCompletion = nil
  local combinedSequence = AHSequence.new()

  if shouldCompleteScroll then
    combinedSequence:addAction(function() self.currentBuffer:onLeaveBegin() end)
    scrollValue = 1
    scrollCompletion = function() completeScroll(self) end
  else
    scrollValue = 0
    scrollCompletion = function() cancelScroll(self) end
  end

  local scrollDuration = nil
  if (self.scrollDirectionData.isHorizontal) then
    scrollDuration = 0.2
  else
    scrollDuration = 0.5
  end

  local scrollAction = newScrollToAction(self, scrollDuration, AHEaseSineOut.new(), scrollValue)

  combinedSequence:addAction(scrollAction)
  combinedSequence:addAction(scrollCompletion)

  self.scrollActionMutex:insert(combinedSequence)
end

local function getFutureCardIndices(self, directionData)
  local futureCardIndexX = self.currentCardIndexX + directionData.futureIndexXOffset
  local futureCardIndexY = self.currentCardIndexY + directionData.futureIndexYOffset
  if directionData.isHorizontal then
    futureCardIndexY = HOME_ROW
  end

  return futureCardIndexX, futureCardIndexY
end

local function getFutureCard(self, scrollDirection)
  local directionData = getDirectionData(scrollDirection)
  local futureCardIndexX, futureCardIndexY = getFutureCardIndices(self, directionData)
  return getCard(self, futureCardIndexX, futureCardIndexY)
end

--- Returns true if the navigator is physically capable of scrolling in the specified direction.
local function canScroll(self, scrollDirection)
  return getFutureCard(self, scrollDirection) ~= nil
end

--- Returns true if the navigator is allowed to scroll in the specified direction.
local function isScrollAllowed(self, scrollDirection)
  if not canScroll(self, scrollDirection) then
    return false
  end

  local currentCard = getCard(self, self.currentCardIndexX, self.currentCardIndexY)
  local futureCard = getFutureCard(self, scrollDirection)

  if not (futureCard and currentCard) then
    return false
  end

  return currentCard:canScrollFromThisCard(scrollDirection) and futureCard:canScrollToThisCard(scrollDirection)
end

--- Returns true if the navigator is allowed to tap navigate in the specified direction.
local function isTapNavigateAllowed(self, scrollDirection)
  if not canScroll(self, scrollDirection) then
    return false
  end

  local currentCard = getCard(self, self.currentCardIndexX, self.currentCardIndexY)
  local futureCard = getFutureCard(self, scrollDirection)

  if not (futureCard and currentCard) then
    return false
  end

  return currentCard:canTapNavigateFromThisCard(scrollDirection) and futureCard:canTapNavigateToThisCard(scrollDirection)
end

local function startScrolling(self, scrollDirection)
  if canScroll(self, scrollDirection) then
    local futureDirectionData = getDirectionData(scrollDirection)
    local futureCardIndexX, futureCardIndexY = getFutureCardIndices(self, futureDirectionData)

    local futureBuffer = self.futureBuffer
    local futureBufferX = futureDirectionData.futurePositionX
    local futureBufferY = futureDirectionData.futurePositionY
    futureBuffer:setLoc(futureBufferX, futureBufferY)

    local futureCard = getCard(self, futureCardIndexX, futureCardIndexY)
    futureBuffer:setCard(futureCard)
    futureBuffer:reset()

    self.scrollDirectionData = futureDirectionData
    self.futureCardIndexX = futureCardIndexX
    self.futureCardIndexY = futureCardIndexY
    self.isScrolling = true
    self.initialScrollingDirection = scrollDirection
    return true
  end
  return false
end

local function tryDragStart(self, touchX, touchY)
  if didExceedTouchThreshold(self.touchTracker) then
    local scrollDirection = getScrollDirection(self.touchTracker)
    if isScrollAllowed(self, scrollDirection) then
      startScrolling(self, scrollDirection)
      handleDrag(self, touchX, touchY)
    end
  end
end

local function handleTap(self, x, y)
  if (x < CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY) then
    if isTapNavigateAllowed(self, CardCarouselNavigator.TAP_DIRECTION_BACK) then
      self:forceTapNavigate(CardCarouselNavigator.DIRECTION_RIGHT, CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL)
    end
  else
    if isTapNavigateAllowed(self, CardCarouselNavigator.TAP_DIRECTION_FORWARD) then
      self:forceTapNavigate(CardCarouselNavigator.DIRECTION_LEFT, CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL)
    end
  end
end

local function handlePossibleTap(self)
  local events = self.touchTracker:getEvents()
  if #events < 2 then
    return
  end

  local startEvent = events[1]
  local endEvent = events[#events]
  local duration = endEvent.time - startEvent.time

  local shouldInterpretAsTap = (duration <= TAP_MAX_TIME_BETWEEN_START_AND_END_EVENTS)
  if shouldInterpretAsTap then
    local x = startEvent.x
    local y = startEvent.y
    handleTap(self, x, y)
  end
end

local function handleTouchDown(self, touchX, touchY)
  self.isTouchActive = true
end

local function handleTouchMoved(self, touchX, touchY)
  if self.isTouchActive then
    if self.isScrolling then
      handleDrag(self, touchX, touchY)
    else
      tryDragStart(self, touchX, touchY)
    end
  end
end

local function handleTouchUp(self, touchX, touchY)
  self.isTouchActive = false

  if (self.isScrolling) then
    handleDrop(self)
  else
    local currentCard = getCard(self, self.currentCardIndexX, self.currentCardIndexY)
    local willHandleTap = not currentCard:handlesOwnTapNavigation()
    if (not didExceedTouchThreshold(self.touchTracker) and willHandleTap) then
      handlePossibleTap(self)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CardCarouselNavigator:setTouchEnabled(enabled)
  self.scroller:setTouchEnabled(enabled)
end

function CardCarouselNavigator:forceScroll(scrollDirection, scrollDuration)
  scrollDuration = scrollDuration or 1.0

  return runInCoroutine(function()
    if canScroll(self, scrollDirection) then
      self.currentBuffer:onLeaveBegin()
      self:setTouchEnabled(false)
      startScrolling(self, scrollDirection)

      local scrollEase = AHEaseBezier.newFlat({0.4, 0.0, 0.2 , 1.0})
      local scrollAction = newScrollToAction(self, scrollDuration, scrollEase, 1.0)
      self.scrollActionMutex:insert(AHSequence.new(scrollAction, function() completeScroll(self) end))

      self:setTouchEnabled(true)
    end
  end)
end

function CardCarouselNavigator:forceTapNavigate(scrollDirection, scrollDuration, willCheckNavigationConstraints)
  scrollDuration = scrollDuration or 1.0

  if (willCheckNavigationConstraints == nil) then
    willCheckNavigationConstraints = false
  end

  return runInCoroutine(function()
    if canScroll(self, scrollDirection) then
      if (willCheckNavigationConstraints and not isTapNavigateAllowed(self, scrollDirection)) then
        return
      end

      self:forceScroll(scrollDirection, scrollDuration)
    end
  end)
end

function CardCarouselNavigator:onAppear()
  self.currentBuffer:onEnter()
  self:setTouchEnabled(true)
end

function CardCarouselNavigator:overloadOnEnterForCardsOfType(cardType, aFunction)
  for _, column in ipairs(self.cardColumns) do
    for _, card in ipairs(column) do
      if card:getCardType() == cardType then
        card:overloadOnEnter(aFunction)
      end
    end
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------


local function setForceNavigateCallbackOnCards(self)
  local function forceScrollCallback(scrollDirection, duration)
    self:forceScroll(scrollDirection, duration)
  end

  local function forceTapNavigateCallback(scrollDirection, duration, willCheckNavigationConstraints)
    self:forceTapNavigate(scrollDirection, duration, willCheckNavigationConstraints)
  end

  for _, column in ipairs(self.cardColumns) do
    for _, card in ipairs(column) do
      card:setForceScrollCallback(forceScrollCallback)
      card:setForceTapNavigateCallback(forceTapNavigateCallback)
    end
  end
end

local function addExitCallbacks(self)
  local exitFunction = function()
    self.completionCallback()
  end
  self:overloadOnEnterForCardsOfType(CardCarouselCardDefinitions.CARD_TYPES.EXIT, exitFunction)
  self:overloadOnEnterForCardsOfType(CardCarouselCardDefinitions.CARD_TYPES.PULL_TO_EXIT, exitFunction)

  for _, column in ipairs(self.cardColumns) do
    for _, card in ipairs(column) do
      card:setNavigatorExitCallback(exitFunction)
    end
  end
end

local function addCardCallbacks(self)
  setForceNavigateCallbackOnCards(self)
  addExitCallbacks(self)
end

local function initializeCards(cardColumns)
  for _, column in ipairs(cardColumns) do
    for _, card in ipairs(column) do
      card:initializeLayer()
    end
  end
end

local function newScreenSeparator()
  local lineWidth = 2
  local blurMargin = 0
  local color = CardCarouselCardCommon.COLORS.DARK_GREY
  return HollowRectangle.new(Screen.width + lineWidth * 2, Screen.height + lineWidth * 2, lineWidth, blurMargin, color)
end

-- Constructor ------------------------

function CardCarouselNavigator.new(cardColumns, completionCallback, clearColor)
  local self = Group.newFullScreen()
  Mixin.mix(self, CardCarouselNavigator)

  assert(completionCallback ~= nil, "A completion callback was not provided to CardCarouselNavigator.")

  initializeCards(cardColumns)

  self.cardColumns = cardColumns
  self.completionCallback = completionCallback

  local scroller = Group.new(Screen.getWidthPart(3.0), Screen.getHeightPart(3.0))
  self:insertAtCenter(scroller, Z_SCROLLER)
  self.scroller = scroller

  local currentBuffer = CardCarouselCardBuffer.new(clearColor)
  scroller:insertAtCenter(currentBuffer, Z_BUFFER)
  self.currentBuffer = currentBuffer
  local futureBuffer = CardCarouselCardBuffer.new(clearColor)
  futureBuffer:setLoc(Screen.getWidthPart(2.5), Screen.getHeightPart(1.5))
  scroller:insert(futureBuffer, Z_BUFFER)
  self.futureBuffer = futureBuffer

  local initialCardIndexX = TITLE_COLUMN
  local initialCardIndexY = HOME_ROW
  resetScroller(self, initialCardIndexX, initialCardIndexY)

  local touchPriorityBehindEverything = 0
  TouchManager.addTouchEnabledMethods(scroller, function(event)
      self.touchTracker:notifyTouchEvent(event)
      local x = event.x
      local y = event.y
      local phase = event.phase
      if (phase == TouchManager.TOUCH_BEGAN) then
        handleTouchDown(self, x, y)
      elseif (phase == TouchManager.TOUCH_MOVED) then
        handleTouchMoved(self, x, y)
      else
        handleTouchUp(self, x, y)
      end
    end,
    TouchHandler.BEHAVIOR_HANDOFF,
    touchPriorityBehindEverything)

  addCardCallbacks(self)

  self.screenSeparator = newScreenSeparator()
  scroller:insertAtCenter(self.screenSeparator, Z_SEPARATOR)

  currentBuffer:reset()

  self.touchTracker = CardCarouselTouchTracker.new()

  self.scrollActionMutex = ActionMutex.new()

  return self
end
