--- CrossfadeGenerator
-- Allows fading between two numeric generators.
-- @author Created by karl on July 21, 2014.

class("CrossfadeGenerator")
require("data.generators.Generator")


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossfadeGenerator:generateNextValue()
  local leftValue = self.leftGenerator:next()
  local rightValue = self.rightGenerator:next()
  if leftValue == nil or rightValue == nil then
    return nil
  end

  leftValue = leftValue * (1 - self.crossfade)
  rightValue = rightValue * self.crossfade

  return leftValue + rightValue
end

function CrossfadeGenerator:reset()
  self.leftGenerator:reset()
  self.rightGenerator:reset()
end

function CrossfadeGenerator:getCrossfade()
  return self.crossfade
end

--- Set crossfade.
-- @param crossfade (numeric) - The fade amount (0.0 = full left, 1.0 = full right)
function CrossfadeGenerator:setCrossfade(crossfade)
  self.crossfade = crossfade
end

function CrossfadeGenerator:newCrossfadeToAction(duration, ease, amount)
  return AHNonAttributeTo.new(self, duration, self.getCrossfade, self.setCrossfade, ease, amount)
end

function CrossfadeGenerator:newCrossfadeByAction(duration, ease, amount)
  return AHNonAttributeBy.new(self, duration, self.getCrossfade, self.setCrossfade, ease, amount)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CrossfadeGenerator.new(leftGenerator, rightGenerator)
  local self = Generator.new()
  Mixin.mix(self, CrossfadeGenerator)

  self.leftGenerator = leftGenerator
  self.rightGenerator = rightGenerator

  self.crossfade = 0.5

  return self
end
