--- AudioVisualizer
-- @author Created by karl on July 02, 2015.

class("AudioVisualizer")

require("data.visualization.SineWaveVisualizer")
require("data.generators.SoundAveragePowerGenerator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local VISUALIZER_DELAY = 10
local WAVEFORM_COUNT = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getOpacityForVisualizerIndex(index)
  local opacity = math.pow(0.5, (index - 1))
  return opacity
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function AudioVisualizer:visualizeSound(sound)
  local generator = SoundAveragePowerGenerator.new(sound)
  local cachingGenerator = CachingGenerator.new(generator, #self.visualizers * VISUALIZER_DELAY)

  for index, visualizer in ipairs(self.visualizers) do
    local delay = (index - 1) * VISUALIZER_DELAY
    local visualGenerator = cachingGenerator
    if delay > 0 then
      visualGenerator = DelayedGenerator.new(cachingGenerator, delay, 0)
    end
    visualizer:setGenerator(visualGenerator)
    visualizer:start()
  end
end

function AudioVisualizer:stop()
  for _, visualizer in ipairs(self.visualizers) do
    visualizer:stop()
  end
end

function AudioVisualizer:settleAndStop()
  for _, visualizer in ipairs(self.visualizers) do
    visualizer:settleAndStop()
  end
end

function AudioVisualizer:setColor(r, g, b, a)
  a = a or 1.0

  local modifiedOpacity = nil
  for visualizerIndex, visualizer in ipairs(self.visualizers) do
    modifiedOpacity = a * getOpacityForVisualizerIndex(visualizerIndex)
    visualizer:setColor(r, g, b, modifiedOpacity)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function AudioVisualizer.new(width, height, color)
  local self = Group.new(width, height)
  Mixin.mix(self, AudioVisualizer)

  local opacity = nil
  self.visualizers = {}

  for visualizerIndex = 1, WAVEFORM_COUNT, 1 do
    opacity = getOpacityForVisualizerIndex(visualizerIndex)

    local visualizer = SineWaveVisualizer.new(width, height, nil, nil, nil, opacity)
    visualizer:inheritColor(self)

    table.insert(self.visualizers, visualizer)
  end

  for _, visualizer in rpairs(self.visualizers) do
    self:insertAtCenter(visualizer)
  end

  if color then
    self:setColor(unpack(color))
  end

  return self
end
