--- DataVisualizer
-- A data visualizer attaches to a generator and monitors it, calling an update
-- function on itself for each value fetched. Fetching occurs once per coroutine
-- yield.
-- @author Created by karl on July 21, 2014.

class("DataVisualizer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DataVisualizer:start()
  assert(self._DataVisualizer_generator, "Call setGenerator before calling start.")
  self:stop()
  self._DataVisualizer_timer:start()
  self._DataVisualizer_previousTime = self._DataVisualizer_timer:getTime()
  self._DataVisualizer_coroutine = runInCoroutine(function()
    while true do
      local time = self._DataVisualizer_timer:getTime()
      self._DataVisualizer_previousTimeStep = time - self._DataVisualizer_previousTime
      self:_DataVisualizer_receiveNextSample(self._DataVisualizer_generator:next())
      self._DataVisualizer_previousTime = time
      coroutine.yield()
    end
  end)
end

function DataVisualizer:stop()
  if self._DataVisualizer_coroutine then
    self._DataVisualizer_coroutine:stop()
    self._DataVisualizer_timer:stop()
    self._DataVisualizer_coroutine = nil
  end
end

function DataVisualizer:_DataVisualizer_receiveNextSample(sample)
  error("Implement me")
end

function DataVisualizer:setGenerator(generator)
  assert(type(generator) == "table", "Invalid generator. Got " .. type(generator))
  self._DataVisualizer_generator = generator
end

function DataVisualizer:getTimeStep()
  return self._DataVisualizer_previousTimeStep or 0.0
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function DataVisualizer.makeDataVisualizer(self)
  Mixin.mix(self, DataVisualizer)
  self._DataVisualizer_timer = MOAITimer.new()
  self._DataVisualizer_timer:setSpan(math.huge)
  return self
end
