--- DifficultyCurves
-- @author Created by karl on September 10, 2014.

class("DifficultyCurves")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

DifficultyCurves.STANDARD_CURVE = "STANDARD_CURVE"
DifficultyCurves.LESS_MIXING = "LESS_MIXING"

local STOCK_KEY_FRAMES = {}
STOCK_KEY_FRAMES[DifficultyCurves.STANDARD_CURVE] = {}
STOCK_KEY_FRAMES[DifficultyCurves.LESS_MIXING] = {}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------
local function getKeyFramesForOne(contentDifficulties)
  assert(#contentDifficulties == 1, "Incorrect number of content difficulties defined for key frame getter")

  local keyFrames = {
    {key = 0.0, valueProbabilities = {[contentDifficulties[1]] = 1.0}},
    {key = 1.0, valueProbabilities = {[contentDifficulties[1]] = 1.0}}
  }

  return keyFrames
end

local function getKeyFramesForTwo(contentDifficulties)
  assert(#contentDifficulties == 2, "Incorrect number of content difficulties defined for key frame getter")

  local keyFrames = {
    {key = 0.0, valueProbabilities = {[contentDifficulties[1]] = 1.0,
                                      [contentDifficulties[2]] = 0.0}},

    {key = 0.75, valueProbabilities = {[contentDifficulties[1]] = 0.0,
                                       [contentDifficulties[2]] = 1.0}},

    {key = 1.0, valueProbabilities = {[contentDifficulties[1]] = 0.0,
                                      [contentDifficulties[2]] = 1.0}}
  }

  return keyFrames
end

-- Derived from an old standard curve equation. Unrolled for readability
local function getKeyFramesForThreeStandard(contentDifficulties)
  assert(#contentDifficulties == 3, "Incorrect number of content difficulties defined for key frame getter")

  local keyFrames = {
    {key = 0.0, valueProbabilities = {[contentDifficulties[1]] = 0.95,
                                      [contentDifficulties[2]] = 0.05}},

    {key = 0.075, valueProbabilities = {[contentDifficulties[1]] = 0.78,
                                        [contentDifficulties[2]] = 0.22}},

    {key = 0.15, valueProbabilities = {[contentDifficulties[1]] = 0.65,
                                       [contentDifficulties[2]] = 0.30,
                                       [contentDifficulties[3]] = 0.05}},

    {key = 0.225, valueProbabilities = {[contentDifficulties[1]] = 0.50,
                                        [contentDifficulties[2]] = 0.40,
                                        [contentDifficulties[3]] = 0.10}},

    {key = 0.3, valueProbabilities = {[contentDifficulties[1]] = 0.35,
                                      [contentDifficulties[2]] = 0.50,
                                      [contentDifficulties[3]] = 0.15}},

    {key = 0.375, valueProbabilities = {[contentDifficulties[1]] = 0.30,
                                        [contentDifficulties[2]] = 0.40,
                                        [contentDifficulties[3]] = 0.30}},

    {key = 0.45, valueProbabilities = {[contentDifficulties[1]] = 0.15,
                                       [contentDifficulties[2]] = 0.50,
                                       [contentDifficulties[3]] = 0.35}},

    {key = 0.525, valueProbabilities = {[contentDifficulties[1]] = 0.10,
                                        [contentDifficulties[2]] = 0.40,
                                        [contentDifficulties[3]] = 0.50}},

    {key = 0.6, valueProbabilities = {[contentDifficulties[1]] = 0.05,
                                      [contentDifficulties[2]] = 0.30,
                                      [contentDifficulties[3]] = 0.65}},

    {key = 0.675, valueProbabilities = {[contentDifficulties[2]] = 0.22,
                                        [contentDifficulties[3]] = 0.78}},

    {key = 0.75, valueProbabilities = {[contentDifficulties[2]] = 0.05,
                                       [contentDifficulties[3]] = 0.95}},

    {key = 1.0, valueProbabilities = {[contentDifficulties[3]] = 1.0}}
  }

  return keyFrames
end

local function getKeyFramesForFourStandard(contentDifficulties)
  assert(#contentDifficulties == 4, "Incorrect number of content difficulties defined for key frame getter")

  local keyFrames = {
    {
      key = 0.0,
      valueProbabilities = {
        [contentDifficulties[1]] = 1.0,
        [contentDifficulties[2]] = 0.0,
        [contentDifficulties[3]] = 0.0,
        [contentDifficulties[4]] = 0.0
      }
    },
    {
      key = 0.075,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.95,
        [contentDifficulties[2]] = 0.05,
        [contentDifficulties[3]] = 0.0,
        [contentDifficulties[4]] = 0.0
      }
    },
    {
      key = 0.15,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.85,
        [contentDifficulties[2]] = 0.15,
        [contentDifficulties[3]] = 0.0,
        [contentDifficulties[4]] = 0.0
      }
    },
    {
      key = 0.225,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.63,
        [contentDifficulties[2]] = 0.37,
        [contentDifficulties[3]] = 0.0,
        [contentDifficulties[4]] = 0.0
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.5,
        [contentDifficulties[2]] = 0.5,
        [contentDifficulties[3]] = 0.0,
        [contentDifficulties[4]] = 0.0
      }
    },
    {
      key = 0.375,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.45,
        [contentDifficulties[2]] = 0.55,
        [contentDifficulties[3]] = 0.0,
        [contentDifficulties[4]] = 0.0
      }
    },
    {
      key = 0.45,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.4,
        [contentDifficulties[2]] = 0.45,
        [contentDifficulties[3]] = 0.15,
        [contentDifficulties[4]] = 0.0
      }
    },
    {
      key = 0.525,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.25,
        [contentDifficulties[2]] = 0.45,
        [contentDifficulties[3]] = 0.3,
        [contentDifficulties[4]] = 0.0
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.15,
        [contentDifficulties[2]] = 0.3,
        [contentDifficulties[3]] = 0.5,
        [contentDifficulties[4]] = 0.05
      }
    },
    {
      key = 0.675,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.0,
        [contentDifficulties[2]] = 0.1,
        [contentDifficulties[3]] = 0.7,
        [contentDifficulties[4]] = 0.2
      }
    },
    {
      key = 0.75,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.0,
        [contentDifficulties[2]] = 0.0,
        [contentDifficulties[3]] = 0.55,
        [contentDifficulties[4]] = 0.45
      }
    },
    {
      key = 0.8,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.0,
        [contentDifficulties[2]] = 0.0,
        [contentDifficulties[3]] = 0.5,
        [contentDifficulties[4]] = 0.5
      }
    },
    {
      key = 0.9,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.0,
        [contentDifficulties[2]] = 0.0,
        [contentDifficulties[3]] = 0.4,
        [contentDifficulties[4]] = 0.6
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        [contentDifficulties[1]] = 0.0,
        [contentDifficulties[2]] = 0.0,
        [contentDifficulties[3]] = 0.3,
        [contentDifficulties[4]] = 0.7
      }
    }
  }

  return keyFrames
end

-- Derived from an old less mixing equation. Unrolled for readability
local function getKeyFramesForThreeLessMixing(contentDifficulties)
  assert(#contentDifficulties == 3, "Incorrect number of content difficulties defined for key frame getter")

  local keyFrames = {
    {key = 0.0, valueProbabilities = {[contentDifficulties[1]] = 1.0}},

    {key = 0.075, valueProbabilities = {[contentDifficulties[1]] = 0.95,
                                        [contentDifficulties[2]] = 0.05}},

    {key = 0.15, valueProbabilities = {[contentDifficulties[1]] = 0.85,
                                       [contentDifficulties[2]] = 0.15}},

    {key = 0.225, valueProbabilities = {[contentDifficulties[1]] = 0.65,
                                        [contentDifficulties[2]] = 0.35}},

    {key = 0.3, valueProbabilities = {[contentDifficulties[1]] = 0.35,
                                      [contentDifficulties[2]] = 0.65}},

    {key = 0.375, valueProbabilities = {[contentDifficulties[2]] = 1.0}},

    {key = 0.45, valueProbabilities = {[contentDifficulties[2]] = 0.65,
                                       [contentDifficulties[3]] = 0.35}},

    {key = 0.525, valueProbabilities = {[contentDifficulties[2]] = 0.35,
                                        [contentDifficulties[3]] = 0.65}},

    {key = 0.6, valueProbabilities = {[contentDifficulties[2]] = 0.15,
                                      [contentDifficulties[3]] = 0.85}},

    {key = 0.675, valueProbabilities = {[contentDifficulties[2]] = 0.05,
                                        [contentDifficulties[3]] = 0.95}},

    {key = 0.75, valueProbabilities = {[contentDifficulties[3]] = 1.00}},

    {key = 1.0, valueProbabilities = {[contentDifficulties[3]] = 1.00}}
  }

  return keyFrames
end

STOCK_KEY_FRAMES[DifficultyCurves.STANDARD_CURVE][1] = function(contentDifficulties) return getKeyFramesForOne(contentDifficulties) end
STOCK_KEY_FRAMES[DifficultyCurves.STANDARD_CURVE][2] = function(contentDifficulties) return getKeyFramesForTwo(contentDifficulties) end
STOCK_KEY_FRAMES[DifficultyCurves.STANDARD_CURVE][3] = function(contentDifficulties) return getKeyFramesForThreeStandard(contentDifficulties) end
STOCK_KEY_FRAMES[DifficultyCurves.STANDARD_CURVE][4] = function(contentDifficulties) return getKeyFramesForFourStandard(contentDifficulties) end


STOCK_KEY_FRAMES[DifficultyCurves.LESS_MIXING][1] = function(contentDifficulties) return getKeyFramesForOne(contentDifficulties) end
STOCK_KEY_FRAMES[DifficultyCurves.LESS_MIXING][2] = function(contentDifficulties) return getKeyFramesForTwo(contentDifficulties) end
STOCK_KEY_FRAMES[DifficultyCurves.LESS_MIXING][3] = function(contentDifficulties) return getKeyFramesForThreeLessMixing(contentDifficulties) end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DifficultyCurves.getKeyFrames(curveType, contentDifficulties)
  local contentDifficultyCount = #contentDifficulties

  local keyFrames = STOCK_KEY_FRAMES[curveType][contentDifficultyCount](contentDifficulties)

  assert(keyFrames, "No valueProbabilities function defined for " .. contentDifficultyCount .. " difficulties using curve type " .. tostring(curveType))

  return keyFrames
end

--- Returns a difficulty curve that is flat - i.e. it produces concepts only of a certain difficulty,
-- then leaps to the next content difficulty after a certain game difficulty is reached.
-- Format:
-- @param cutoffInfo =
-- {
--    {endDifficulty = 0.3, contentDifficulty = "Easy"},
--    {endDifficulty = 0.5, contentDifficulty = "Medium"},
--    {endDifficulty = 1, contentDifficulty = "Hard"}
-- }
function DifficultyCurves.getHardCutoffKeyframes(cutoffInfo)
  local previousEndDifficulty = 0
  local epsilon = 0.01
  local curve = {}
  for _, difficultyInfo in ipairs(cutoffInfo) do
    local endDifficulty = difficultyInfo.endDifficulty
    local contentDifficulty = difficultyInfo.contentDifficulty
    assert(endDifficulty >= previousEndDifficulty, "Must provide ordered set of game difficulty intervals.")

    local valueProbabilities = {[contentDifficulty] = 1.0}
    local intervalEnd = endDifficulty
    if(endDifficulty < 1) then intervalEnd = intervalEnd - epsilon end

    local segmentStart = {key = previousEndDifficulty, valueProbabilities = valueProbabilities}
    local segmentEnd = {key = intervalEnd, valueProbabilities = valueProbabilities}
    curve[#curve + 1] = segmentStart
    curve[#curve + 1] = segmentEnd

    previousEndDifficulty = endDifficulty
  end

  return curve
end
