--- LuaExtensions
-- @author Created by karl on July 28, 2014.

--- Iterate pairs by reverse index.
function rpairs(t)
  local function rpairsIterator(t, i)
    i = i - 1
    local v = t[i]
    if v == nil then return v end
    return i, v
  end
  return rpairsIterator, t, #t + 1
end

--- Packs varargs into a table, along with a the total number of arguments passed in.
-- Useful for iterating varargs with nils.
-- This is a LUA 5.2 feature, backported.
--
-- http://lua-users.org/lists/lua-l/2010-01/msg01427.html
--
-- http://stackoverflow.com/questions/7183998/in-lua-what-is-the-right-way-to-handle-varargs-which-contains-nil
--
-- http://lua-users.org/wiki/VarargTheSecondClassCitizen
function table.pack(...)
  return { n = select("#", ...), ...}
end

--- Override of the system print to send messages out to the host
if mslog then
  function print(...)

    local args = table.pack(...)

    local result = ""
    for i = 1, args.n do
      local v = args[i]
      result = result .. tostring(v) .. " \t "
    end

    local logPriority = 3

    if not mslog(logPriority, result) then
      MOAILogMgr.log(result.."\n")
    end
  end
end

-- Set up character access operator for strings.
-- e.g.
--   local char = someString[index]
-- is equivalent to:
--   local char = someString:sub(index, index)
local function setupStringAccessOperator()
  local defaultIndex = rawget(getmetatable(""), "__index")
  assert(type(defaultIndex) == "table",
        "This code expected the __index of the string metatable to have its default value, which is a table."
        .. " Found " .. tostring(defaultIndex) .." instead. It may now be unsafe to apply this extension.")

  local newIndex = function(object, key)
    if type(key) == "number" then
      return string.sub(object, key, key)
    else
      return defaultIndex[key]
    end
  end

  rawset(getmetatable(""), "__index", newIndex)
end
setupStringAccessOperator()
