--- Extensions of the MOAI modules
-- @module MOAIExtensions

MOAICoroutine.extend(
  "MOAICoroutine",
  function(interface, class, superInterface, superClass)
    function interface.stop(self)
      local waitedActions = self.waitedActions or {}
      for _, action in ipairs(waitedActions) do
        action:stop()
      end
      self.waitedActions = nil

      superInterface.stop(self)
    end

    function interface.setFunction(self, threadFunc)
      self.threadFunc = threadFunc
    end

    function interface.start(self, ...)
      local threadFunc = self.threadFunc
      assert(threadFunc, "MOAICoroutine started without assigning a function first with setFunction().")

      self:run(threadFunc, unpack(arg))
      return self
    end
  end
)


MOAIProp2D.extend(
  "MOAIProp2D",
  function(interface, class, superInterface, superClass)
    local basicSetVisible = interface.setVisible
    function interface.setVisible(self, isVisible)
      basicSetVisible(self, isVisible)
      self.__isVisible = isVisible
    end
    function interface.getVisible(self)
      if(self.__isVisible == nil) then
        -- Visibility is on by default
        return true
      end
      return self.__isVisible
    end
  end
)
