--- CircleGradient.lua
-- Created by Steven Mah on December 12, 2016

class("CircleGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function toMOAIColor(color)
  if (type(color) == "table") then
    return ColorUtils.tableToMOAIColor(color)
  end
  return color
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CircleGradient:setColor(centerColor, edgeColor)
  if (centerColor) then
    self.centerColor:setColor(unpack(centerColor))
  end

  if (edgeColor) then
    self.edgeColor:setColor(unpack(edgeColor))
  end
end

function CircleGradient:getColors()
  return self.centerColor, self.edgeColor
end

function CircleGradient:newGradientToAction(duration, ease, centerColor, edgeColor)
  return AHSpawn.new(
    AHColorTo.new(self.centerColor, duration, ease, centerColor),
    AHColorTo.new(self.edgeColor, duration, ease, edgeColor)
  )
end

function CircleGradient:newOpacityToAction(duration, ease, centerOpacity, edgeOpacity)
  return AHSpawn.new(
    AHOpacityTo.new(self.centerColor, duration, ease, centerOpacity),
    AHOpacityTo.new(self.edgeColor, duration, ease, edgeOpacity)
  )
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CircleGradient.new(radius, centerColor, edgeColor, blurMargin)
  local diameter = radius * 2
  local self = Group.new(diameter, diameter)
  Mixin.mix(self, CircleGradient)

  self.centerColor = toMOAIColor(centerColor)
  self.edgeColor = toMOAIColor(edgeColor)

  self.deck = MOAIScriptDeck.new()
  self.deck:setRect(0.0, 0.0, diameter, diameter)
  self:setDeck(self.deck)
  self.deck:setDrawCallback(function()
    -- NOTE: Used fillCircularSliceGradient instead of fillCircularGradient becasuse
    -- the latter doesn't have a blurMargin parameter
    MOAIDraw.fillCircularSliceGradient(radius, radius, radius, 360, 0, blurMargin, nil, self.centerColor, self.edgeColor)
  end)

  return self
end
