--- FlintstonesImage
-- @author Created by Aaron Barrett on January 21, 2016.

class("FlintstonesImage")

require("graphics.TiledImage")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

FlintstonesImage.DIRECTION = {
  LEFT = "LEFT",
  RIGHT = "RIGHT",
  UP = "UP",
  DOWN = "DOWN"
}

local DEFAULT_SCROLL_DIRECTION = FlintstonesImage.DIRECTION.LEFT

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function scrollLeft(self, uCoordinateOffset)
  self.backgroundDeck:setUVRect(uCoordinateOffset, self.numberOfRepeatsY, uCoordinateOffset + self.numberOfRepeatsX, 0.0)
end

local function scrollRight(self, uCoordinateOffset)
  self.backgroundDeck:setUVRect(uCoordinateOffset, self.numberOfRepeatsY, uCoordinateOffset - self.numberOfRepeatsX, 0.0)
end

local function scrollUp(self, uCoordinateOffset)
  self.backgroundDeck:setUVRect(0.0, vCoordinateOffset + self.numberOfRepeatsY, self.numberOfRepeatsX, vCoordinateOffset)
end

local function scrollDown(self, uCoordinateOffset)
  self.backgroundDeck:setUVRect(0.0, self.numberOfRepeatsY - vCoordinateOffset, self.numberOfRepeatsX, -vCoordinateOffset)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FlintstonesImage:setCycleOffset(cycleOffset)
  self.cycleOffset = cycleOffset

  local uCoordinateOffset = cycleOffset - math.floor(cycleOffset)
  self.scrollFunction(self, uCoordinateOffset)
end

function FlintstonesImage:getCycleOffset()
  return self.cycleOffset
end

function FlintstonesImage:playCycle(periodDuration, direction)
  if (direction) then
    self:setScrollDirection(direction)
  end
  local singleCycleUCoordinateOffset = 1.0
  local singleCycleAction = AHNonAttributeBy.new(self, periodDuration, self.getCycleOffset, self.setCycleOffset, nil, singleCycleUCoordinateOffset)

  self.cycleAction = AHLoop.new(
    singleCycleAction
  )
  return self.cycleAction:start()
end

function FlintstonesImage:stopCycle()
  self.cycleAction:stop()
  self.cycleAction = nil
end

function FlintstonesImage:setScrollDirection(direction)
  if (direction == FlintstonesImage.DIRECTION.LEFT) then
    self.scrollFunction = scrollLeft
  elseif (direction == FlintstonesImage.DIRECTION.RIGHT) then
    self.scrollFunction = scrollRight
  elseif (direction == FlintstonesImage.DIRECTION.UP) then
    self.scrollFunction = scrollUp
  elseif (direction == FlintstonesImage.DIRECTION.DOWN) then
    self.scrollFunction = scrollDown
  else
    error("Invalid scroll direction selected")
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FlintstonesImage.newWithTexture(texture, width, height, textureScale)
  textureScale = textureScale or 1.0

  local textureWidth, textureHeight = texture:getSize()
  if (height == nil) then
    height = textureHeight / Host.contentScale * textureScale
  end

  local self = TiledImage.newWithTexture(texture, width, height)
  Mixin.mix(self, FlintstonesImage)

  self.scrollFunction = nil
  self:setScrollDirection(DEFAULT_SCROLL_DIRECTION)

  self.numberOfRepeatsX = width / (textureWidth / Host.contentScale)  * textureScale
  self.numberOfRepeatsY = height / (textureHeight / Host.contentScale)  * textureScale

  self:setCycleOffset(0.0)
  self.cycleAction = nil
  self.textureScale = textureScale

  return self
end

function FlintstonesImage.new(filename, width, height, textureScale)
  local texture = TextureLoader.loadTexture(filename)
  return FlintstonesImage.newWithTexture(texture, width, height, textureScale)
end
