--- A triangle prop that has a gradient
-- @author Created by steven on August 18, 2016

class("TriangleGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function toMOAIColor(color)
  if type(color) == "table" then
    return ColorUtils.tableToMOAIColor(color)
  end

  return color
end

local function setDrawCallback(self)
  local drawFunction = MOAIDraw.fillTriangularGradient

  self.deck:setDrawCallback(function()
    drawFunction(
      self.points[1].x, self.points[1].y,
      self.points[2].x, self.points[2].y,
      self.points[3].x, self.points[3].y,
      self.colors[1], self.colors[2], self.colors[3]
    )
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TriangleGradient:newGradientToAction(duration, ease, color1, color2, color3)
  return AHSpawn.new(
    AHColorTo.new(self.colors[1], duration, ease, color1),
    AHColorTo.new(self.colors[2], duration, ease, color2),
    AHColorTo.new(self.colors[3], duration, ease, color3)
  )
end

function TriangleGradient:newOpacityToAction(duration, ease, opacity1, opacity2, opacity3)
  return AHSpawn.new(
    AHOpacityTo.new(self.colors[1], duration, ease, opacity1),
    AHOpacityTo.new(self.colors[2], duration, ease, opacity2),
    AHOpacityTo.new(self.colors[3], duration, ease, opacity3)
  )
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function TriangleGradient.new(points, colors)
  local width, height = ShapeUtils.getMaxEdges(points)
  local self = Group.new(width, height)
  Mixin.mix(self, TriangleGradient)

  local moaiColors = {}
  for _, color in ipairs(colors) do
    local moaiColor = toMOAIColor(color)
    table.insert(moaiColors, moaiColor)
  end
  self.colors = moaiColors
  self.points = points

  self.deck = MOAIScriptDeck.new()
  self.deck:setRect(0.0, 0.0, width, height)
  self:setDeck(self.deck)
  setDrawCallback(self)

  return self
end
