--- Arrow
-- @author created by Jay Austria on August 13, 2014

require("graphics.primitives.GraphicsPrimitive")


-- ===========================================================================
class("StickArrow")
-- ===========================================================================

-- ----------------------------------------------------------------------------
-- CONFIGURATION
-- ----------------------------------------------------------------------------

local DEFAULT_HEAD_ANGLE = math.pi / 4
local DEFAULT_THICKNESS = 1


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createArrowHead(length, thickness, headLength, headAngle)
  local startX = 0
  local startY = 0
  local endX = -headLength * math.cos(headAngle)
  local endY = headLength * math.sin(headAngle)
  local head = Line.new(startX, startY, endX, endY)
  head:setLineWidth(thickness)

  return head
end

local function createLayout(self, length, thickness, headLength, headAngle)
  local width, height = self:getDims()

  local arrowStartX = 0
  local arrowStartY = 0
  local arrowEndX = length
  local arrowEndY = 0

  local arrowBody = Line.new(arrowStartX, arrowStartY, arrowEndX, arrowEndY)
  arrowBody:setLineWidth(thickness)
  self:insertAtCenter(arrowBody)

  local arrowLeftHead = createArrowHead(length, thickness, headLength, headAngle)
  arrowLeftHead:setAnchorPointBottomRight()
  arrowLeftHead:setLoc(arrowEndX, thickness * 0.5)
  self:insert(arrowLeftHead)

  local arrowRightHead = createArrowHead(length, thickness, headLength, math.pi - headAngle)
  arrowRightHead:setAnchorPointTopRight()
  arrowRightHead:setLoc(arrowEndX, thickness * 0.5)
  self:insert(arrowRightHead)

  self:linkColorAttrToChildren()
end


-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------
function StickArrow.new(length, headLength, headAngle, thickness)
  assert(length, "length cannot be nil")
  assert(headLength, "headLength cannot be nil")

  thickness = thickness or DEFAULT_THICKNESS
  headAngle = headAngle or DEFAULT_HEAD_ANGLE

  local selfW, selfH = length, headLength * math.sin(headAngle)
  local self = Group.new(length, thickness)

  createLayout(self, length, thickness, headLength, headAngle)

  return self
end
