--- Checkmark
-- @author Created by karl on October 15, 2014.

require("graphics.ShapeUtils")

-- ===========================================================================
local CheckmarkCommon = privateClass()
-- ===========================================================================

-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local CHECKMARK_DEFAULT_SHORT_PROPORTION = 0.39
local CHECKMARK_DEFAULT_THICKNESS_PROPORTION = 0.22

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function CheckmarkCommon.makeCheckmark(self, width, shortSegmentProportion, thicknessProportion)
  Mixin.mix(self, CheckmarkCommon)

  shortSegmentProportion = shortSegmentProportion or CHECKMARK_DEFAULT_SHORT_PROPORTION
  thicknessProportion = thicknessProportion or CHECKMARK_DEFAULT_THICKNESS_PROPORTION

  local points = ShapeUtils.makeCheckmarkPoints(1, shortSegmentProportion, thicknessProportion)
  local _, proportionH = ShapeUtils.getMaxEdges(points)
  local height = proportionH * width
  ShapeUtils.scalePoints(points, 1, 2-proportionH)
  self._normalizedPointLists = {points}

  self:setDims(width, height)

  return self
end



-- ===========================================================================
class("FilledCheckmark")
-- ===========================================================================

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------


--- Create a new filled checkmark.
-- @param width (number) - The width of the checkmark.
-- @param shortSegmentProportion (number) - The proportion of the width to use for the short checkmark segment.
-- @param thicknessProportion (number) - The thickness of the checkmark, measured as a proportion of width.
-- @param color (table) - The color to apply to this object (optional)
function FilledCheckmark.new(width, shortSegmentProportion, thicknessProportion, color)
  local self = FilledGraphicsPrimitive.new(width, 1, color)
  CheckmarkCommon.makeCheckmark(self, width, shortSegmentProportion, thicknessProportion)
  Mixin.mix(self, FilledCheckmark)

  return self
end



-- ===========================================================================
class("HollowCheckmark")
-- ===========================================================================

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

--- Create a new hollow checkmark.
-- @param width (number) - The width of the checkmark.
-- @param shortSegmentProportion (number) - The proportion of the width to use for the short checkmark segment.
-- @param thicknessProportion (number) - The thickness of the checkmark, measured as a proportion of width.
-- @param color (table) - The color to apply to this object (optional)
-- @param lineWidth (number) - The width of the line to draw the checkmark with,
--                             Regardless of line width, the checkmark will never draw
--                             outside of its bounds (default 1.0)
-- @param blurMargin (number) - The amount of blur to apply to the edges (default 0.0)
function HollowCheckmark.new(width, shortSegmentProportion, thicknessProportion, color, lineWidth, blurMargin)
  local self = HollowGraphicsPrimitive.new(width, 1, color, lineWidth, blurMargin, true)
  CheckmarkCommon.makeCheckmark(self, width, shortSegmentProportion, thicknessProportion)
  Mixin.mix(self, HollowCheckmark)

  return self
end
