-- Circles and dots and whatnot
-- @author Created by Aaron Barrett on April 8, 2013.

-- ===========================================================================
local CircleCommon = privateClass()
-- ===========================================================================

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

local function rebuildCirclePoints(self)
  local points = ShapeUtils.makeArcPoints(0, math.pi * 2, 0.5, 0.5, 0.5, self.pointCount)

  -- The final point has the same location as the first point, so remove it
  table.remove(points)

  self._normalizedPointLists = {points}
end


-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

--- The number of points to use when rendering the circle.
-- nil = let ShapeUtils.makeArcPoints decide.
function CircleCommon:getPointCount()
  return self.pointCount
end

function CircleCommon:setPointCount(pointCount)
  self.pointCount = pointCount
  rebuildCirclePoints(self)
  self:updateGraphicsPrimitive()
end


-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function CircleCommon.makeCircle(self)
  CircularPrimitive.makeCircularPrimitive(self)
  Mixin.mix(self, CircleCommon)
  rebuildCirclePoints(self)

  return self
end



-- ===========================================================================
class("FilledCircle")
-- ===========================================================================

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

--- Create a new filled circle.
-- @param radius (number) - The circle's radius
-- @param color (table) - The color to apply to this object (optional)
function FilledCircle.new(radius, color)
  local self = FilledGraphicsPrimitive.new(radius*2, radius*2, color)
  CircleCommon.makeCircle(self)
  Mixin.mix(self, FilledCircle)
  self:updateGraphicsPrimitive()

  return self
end



-- ===========================================================================
class("HollowCircle")
-- ===========================================================================

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

--- Create a new hollow circle.
-- @param radius (number) - The circle's radius
-- @param color (table) - The color to apply to this object (optional)
-- @param lineWidth (number) - The width of the line to draw the circle with,
--                             Regardless of line width, the circle will never draw
--                             outside of its bounds (default 1.0)
-- @param blurMargin (number) - The amount of blur to apply to the edges (default 0.0)
function HollowCircle.new(radius, color, lineWidth, blurMargin)
  local self = HollowGraphicsPrimitive.new(radius*2, radius*2, color, lineWidth, blurMargin, true)
  CircleCommon.makeCircle(self)
  Mixin.mix(self, HollowCircle)
  self:updateGraphicsPrimitive()

  return self
end
