--- A polyline object
-- @author Created by Alexey Chernikov on April 10, 2013
-- @module Line

class("Line")
require("graphics.ShapeUtils")


-- ---------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function Line:append(x, y)
  -- Max size is around ~3280 elements. If more are needed, use several arrays
  table.insert(self.vertices, x or 0)
  table.insert(self.vertices, y or 0)
end

function Line:setLineWidth(lineWidth)
  self.lineWidth = lineWidth
end

function Line:getLineWidth()
  return self.lineWidth
end

function Line:setVertexPosition(vertexIndex, x, y)
  local xIndex = vertexIndex * 2 - 1
  local yIndex = xIndex + 1
  self.vertices[xIndex] = x
  self.vertices[yIndex] = y
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

--- Creates a line primitive from a set of ordered points
-- @param x0, y0, x1, y1, x2, y2, ... - an extendable list of points (alternatively, specify a table array: [[x0, y0], [x1, y1], ...])
function Line.new(...)
  local self = Group.newDeprecated()
  Mixin.mix(self, Line)

  local vertices = arg
  if (type(vertices[1]) == "table") then
      vertices = ShapeUtils.flattenCoordPairs(vertices[1])
  end
  self.vertices = vertices

  self.lineWidth = 1

  local line = MOAIScriptDeck.new()
  line:setDrawCallback(function()
    MOAIGfxDevice.setPenWidth(self.lineWidth)
    MOAIDraw.drawLine(self.vertices)
  end)

  local xMin, yMin, xMax, yMax = math.huge, math.huge, -math.huge, -math.huge

  for i=1,#vertices,2 do
    local coordX, coordY = vertices[i], vertices[i+1]
    xMin = math.min(xMin, coordX)
    xMax = math.max(xMax, coordX)
    yMin = math.min(yMin, coordY)
    yMax = math.max(yMax, coordY)
  end

  line:setRect(xMin, yMin, xMax, yMax)
  self:setDeck(line)
  self:setDims(xMax, yMax)
  self:setAnchorPointCenter()

  return self
end
