--- Regular Polygon (cyclic and equilateral)
-- @author Created by Karl Stenerud on November 20, 2014.

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createNGonPoints(sideCount, startAngle)
  local angle = startAngle or 0
  local dAngle = 2 * math.pi/sideCount
  local points = {}
  for i = 1, sideCount do
    table.insert(points, {x=math.cos(angle) * 0.5 + 0.5, y=math.sin(angle) * 0.5 + 0.5})
    angle = angle + dAngle
  end
  return points
end



-- ===========================================================================
class("FilledRegularPolygon")
-- ===========================================================================

-- Constructor ------------------------

--- Create a new filled polygon.
-- @param radius (number) - the radius for the polygon (center to corner distance).
-- @param color - the color for the polygon's filling.
-- @return the new Polygon object.
function FilledRegularPolygon.new(radius, sideCount, startAngle, color)
  local self = FilledGraphicsPrimitive.new(radius*2, radius*2, color)
  CircularPrimitive.makeCircularPrimitive(self)
  Mixin.mix(self, FilledRegularPolygon)

  self._normalizedPointLists = {createNGonPoints(sideCount, startAngle)}
  self:updateGraphicsPrimitive()

  return self
end



-- ===========================================================================
class("HollowRegularPolygon")
-- ===========================================================================

-- Constructor ------------------------

--- Create a new hollow polygon.
-- @param radius (number) - the radius for the polygon (center to corner distance).
-- @param color - the color for the polygon's filling.
-- @param lineWidth (number) - The width of the line to draw the polygon with,
--                             Regardless of line width, the polygon will never draw
--                             outside of its bounds (default 1.0)
-- @param blurMargin (number) - The amount of blur to apply to the edges (default 0.0)
-- @return the new Polygon object.
function HollowRegularPolygon.new(radius, sideCount, startAngle, color, lineWidth, blurMargin)
  local self = HollowGraphicsPrimitive.new(radius*2, radius*2, color, lineWidth, blurMargin, true)
  CircularPrimitive.makeCircularPrimitive(self)
  Mixin.mix(self, HollowRegularPolygon)

  self._normalizedPointLists = {createNGonPoints(sideCount, startAngle)}
  self:updateGraphicsPrimitive()

  return self
end
