--- ComposedGlyphLabel
-- @author Created by karl on September 11, 2014.

class("ComposedGlyphLabel")
require("text.GlyphLabel")


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ComposedGlyphLabel:sublabel(startIndex, endIndex)
  assert(type(startIndex) == "number", "startIndex must be a number (got " .. type(startIndex) .. ")")
  assert(type(endIndex) == "number", "endIndex must be a number (got " .. type(endIndex) .. ")")
  local newLabel = Row.new()
  Mixin.mix(newLabel, ComposedGlyphLabel)

  for i=startIndex,endIndex do
    local glyphLabel = self.children[i]
    newLabel:insert(glyphLabel:clone())
  end

  newLabel:alignChildren()
  newLabel:linkColorAttrToChildren(true)

  return newLabel
end

function ComposedGlyphLabel:setLabel(sourceLabel, glyphs)
  if sourceLabel ~= nil then
    glyphs = glyphs or sourceLabel:getGlyphTable()
  end
  self:removeAll()

  if glyphs == nil then
    self:insert(Group.new(0, sourceLabel:getHeight()))
  else
    local lastGlyph = nil
    local lastX = 0
    for _, glyph in ipairs(glyphs) do
      local adjustedGlyph = TableUtils.shallowCopy(glyph)
      adjustedGlyph.xMin = lastX
      local glyphLabel = GlyphLabel.new(sourceLabel, adjustedGlyph)
      self:insert(glyphLabel)
      lastX = glyph.xMax + 1
    end
  end

  self:alignChildren()
  self:linkColorAttrToChildren(true)
  self.text = sourceLabel.text
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ComposedGlyphLabel.new(sourceLabel, glyphs)
  local self = Row.new()
  Mixin.mix(self, ComposedGlyphLabel)

  self:setLabel(sourceLabel, glyphs)

  return self
end
