--- DynamicTextArea
-- @author Created by amorant on November 21, 2014.

class("DynamicTextArea")

require("text.dynamic.TextInterpreter")
require("text.dynamic.TextModel")
require("text.dynamic.TextView")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DynamicTextArea:setText(text)
  if text ~= self:getText() then
    local textInterpreter = self.textInterpreter
    local textModel = self.textModel

    local items = textInterpreter:interpretXmlText(text)
    textModel:resetItems(items)

    self.text = text
  end
end

function DynamicTextArea:getText()
  return self.text
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

-- @param text (string) - What to display in the text area.
-- @param width (number) - The width of the text area.
-- @param baseAttributes (table) - Attributes to apply to every item; common ones include 'font' and 'color'.
-- @param attributesForLabels (table, optional) - Attributes to set on an item when a tag is found.
-- @param horizontalAlignment (alignment, optional) - What horizontal text alignment to use. Default Alignment.LEFT
-- @param verticalAlignment (alignment, optional) - What vertical text alignment to use. Default Alignment.TOP
-- @param lineSpacing (number, optional) - Spacing between lines. Default 1
-- @param isPointSpaced (boolean, optional) - True if lineSpacing is in font points, scaled to font size
function DynamicTextArea.new(text, width, baseAttributes, attributesForLabels, horizontalAlignment, verticalAlignment, lineSpacing, isPointSpaced)
  assert(baseAttributes, "Must provide baseAttributes")
  attributesForLabels = attributesForLabels or {}

  local self = TextView.new(width, horizontalAlignment, verticalAlignment, lineSpacing, isPointSpaced)
  Mixin.mix(self, DynamicTextArea)

  local textInterpreter = TextInterpreter.new(baseAttributes, attributesForLabels)
  local textModel = TextModel.new(textInterpreter:interpretXmlText(""))
  self:setModel(textModel)

  self.textInterpreter = textInterpreter
  self.textModel = textModel
  self:setText(text)

  return self
end

function DynamicTextArea.newSimple(text, width, font, horizontalAlignment, verticalAlignment, lineSpacing, isPointSpaced)
  local baseAttributes = { font = font }
  return DynamicTextArea.new(text, width, baseAttributes, nil, horizontalAlignment, verticalAlignment, lineSpacing, isPointSpaced)
end
