--- GlyphLabel
--
--- A glyph label uses an existing label as a kind of sprite sheet, re-using the
-- label's texture to create lightweight single-glyph props.
--
-- Note: Be VERY CAREFUL about releasing textures when using this system, as
--       many labels will be using the same texture.
--
-- @author Created by karl on September 04, 2014.

-- ===========================================================================
class("GlyphLabel")
-- ===========================================================================


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GlyphLabel:clone()
  local selfW, selfH = self:getDims()
  local labelClone = Group.new(selfW, selfH)
  Mixin.mix(labelClone, GlyphLabel)

  labelClone.deck = MOAIGfxQuad2D.new()
  labelClone.glyph = TableUtils.shallowCopy(self.glyph)
  labelClone.texture = self.texture
  labelClone.uvRect = TableUtils.shallowCopy(self.uvRect)

  labelClone.deck:setTexture(labelClone.texture)
  labelClone.deck:setRect(0.0, 0.0, selfW, selfH)
  labelClone.deck:setUVRect(unpack(labelClone.uvRect))
  labelClone:setDeck(labelClone.deck)

  return labelClone
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

--- Create a new GlyphLabel.
-- @param sourceLabel (Label) - Label whose texture to use.
-- @param glyph (table) - A glyph from the source label's glyph table.
-- @return (GlyphLabel) - A new GlyphLabel.
function GlyphLabel.new(sourceLabel, glyph)
  local texture = sourceLabel.texture
  local scale = Host.contentScale
  local textureW, textureH = texture:getSize()
  local renderW, renderH = texture.renderedWidth, texture.renderedHeight
  local minY, maxY = 0, renderH / scale
  local minX, maxX = glyph.xMin / scale, glyph.xMax / scale
  local width, height = maxX-minX, maxY-minY
  local minUX, maxUX, minUY, maxUY = minX*scale/textureW, maxX*scale/textureW, minY*scale/textureH, maxY*scale/textureH

  local self = Group.new(width, height)
  Mixin.mix(self, GlyphLabel)
  self.glyph = glyph
  self.texture = texture
  self.uvRect = {minUX, maxUY, maxUX, minUY}

  local deck = MOAIGfxQuad2D.new()
  deck:setTexture(texture)
  deck:setRect(0.0, 0.0, width, height)
  deck:setUVRect(unpack(self.uvRect))
  self:setDeck(deck)
  self.deck = deck

  return self
end
