--- TutorialUtils
-- @author Created by aaroncantor on August 07, 2020.

class("TutorialUtils")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

--- Links an attribute to a parent via an operation
--- Returns a group that can be used to manipulate the attribute, e.g. via AHActions
function TutorialUtils.combineAttr(group, attribute, controller, operation, sourceAttribute)
  sourceAttribute = sourceAttribute or attribute
  local attrContainer = Group.newMinimumSized()
  attrContainer:setAttr(attribute, group:getAttr(attribute))

  group.combineCoroutine = group.combineCoroutine or {}
  group.combineCoroutine[attribute] = runInCoroutine(function()
    while group and controller do
      group:setAttr(attribute, operation(attrContainer:getAttr(attribute), controller:getAttr(sourceAttribute)))
      coroutine.yield()
    end
  end)

  return attrContainer
end

function TutorialUtils.stopCombiningAttr(group, attribute)
  if group.combineCoroutine then
    local combineCoroutine = group.combineCoroutine[attribute]
    if combineCoroutine and combineCoroutine:isBusy() then
      combineCoroutine:stop()
    end
  end
end

function TutorialUtils.multiplyAttr(group, attribute, controller, sourceAttribute)
  return TutorialUtils.combineAttr(group, attribute, controller, function(a, b) return a*b end, sourceAttribute)
end

function TutorialUtils.multiplyOpacity(group, controller)
  return TutorialUtils.multiplyAttr(group, MOAIColor.ATTR_OPACITY, controller)
end

function TutorialUtils.stopMultipyingOpacity(group)
  TutorialUtils.stopCombiningAttr(group, MOAIColor.ATTR_OPACITY)
end

function TutorialUtils.createTexture(group)
  local groupWidth = group:getWidth()
  local groupHeight = group:getHeight()
  local textureWidth = MathUtils.incrementUntilPowerOf2(groupWidth * Host.contentScale)
  local textureHeight = MathUtils.incrementUntilPowerOf2(groupHeight * Host.contentScale)
  local texture = OffscreenRenderedTexture.new(group, textureWidth, textureHeight)
  return texture
end

function TutorialUtils.createSprite(group, rotation)
  local doRotationHack = (rotation == 45)
  local texture = TutorialUtils.createTexture(group)

  local quad = MOAIGfxQuad2D.new()
  quad:setTexture(texture)

  local textureWidth, textureHeight = texture:getSize()
  textureWidth = textureWidth / Host.contentScale
  textureHeight = textureHeight / Host.contentScale

  local rectX = 0
  local rectY = 0

  -- This is a hack to account for a rotated group getting cut off
  -- on the Left/Bottom of the sprite. A small shift in where the
  -- group is centered in the texture fixes the visual issue.
  if doRotationHack then
    group:offsetX(5)
    group:offsetY(5)
    rectX = -5
    rectY = -5
  end

  local rectWidth = textureWidth
  local rectHeight = textureHeight

  quad:setRect(
    rectX,
    rectY,
    rectX + rectWidth,
    rectY + rectHeight
  )

  quad:setUVRect(0, 1, 1, 0)

  local sprite = Sprite.newWithDeck(quad)
  local groupWidth, groupHeight = group:getDims()
  local centerXFraction = groupWidth / (2 * rectWidth)
  local centerYFraction = groupHeight / (2 * rectHeight)
  sprite:setAnchorPoint(centerXFraction, centerYFraction)

  return sprite
end
