--- A set of scoring system related utilities
-- @author Created by Jay Austria on February 19, 2016

-- ===========================================================================
class("ScoringUtils")
-- ===========================================================================

require("bonuses.BonusTypes")


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ScoringUtils.applyAccuracyBonus(pointsPerAccuracyBonus, bonusTray)
  GameSession.increaseScore(pointsPerAccuracyBonus)

  local notificationString = BonusTypes.getNotificationString(BonusTypes.TYPES.ACCURACY_BONUS)
  bonusTray:indicateSimpleTextBonus(notificationString, pointsPerAccuracyBonus)

  GameSession.bonusReporter:incrementBonusCount(BonusTypes.TYPES.ACCURACY_BONUS)
end

function ScoringUtils.applySpeedBonus(earnedBonusProportion, bonusProportionsSum, pointsPerFullSpeedBonus, bonusTray)
  local speedBonusPoints = pointsPerFullSpeedBonus * earnedBonusProportion
  GameSession.increaseScore(speedBonusPoints)

  if (speedBonusPoints >= 1.0) then
    local notificationString = BonusTypes.getNotificationString(BonusTypes.TYPES.SPEED_BONUS)
    bonusTray:indicateSimpleTextBonus(notificationString, speedBonusPoints)
  end

  GameSession.bonusReporter:incrementBonusCount(BonusTypes.TYPES.SPEED_BONUS)
end

function ScoringUtils.applyLivesBonus(livesRemaining, pointsPerLivesBonus, bonusTray)
  local extraLives = livesRemaining - 1
  local extraLivesPoints = extraLives * pointsPerLivesBonus
  if (extraLivesPoints > 0) then
    GameSession.increaseScore(extraLivesPoints)

    --TODO any sort of animation that needs to be done

    local notificationString = BonusTypes.getNotificationString(BonusTypes.TYPES.LIVES_BONUS)
    bonusTray:indicateSimpleTextBonus(notificationString, extraLivesPoints)
  end

  GameSession.bonusReporter:setBonusCount(BonusTypes.TYPES.LIVES_BONUS, livesRemaining)
end

function ScoringUtils.applyTimeBonus(timeRemainingSeconds, pointsPerSecondRemaining, bonusTray)
  local timeBonusPoints = timeRemainingSeconds * pointsPerSecondRemaining
  if (timeBonusPoints > 0) then
    GameSession.increaseScore(timeBonusPoints)

    local notificationString = BonusTypes.getNotificationString(BonusTypes.TYPES.TIME_BONUS)
    bonusTray:indicateSimpleTextBonus(notificationString, timeBonusPoints)
  end

  --TODO any sort of animation that needs to be done
end

function ScoringUtils.applyMaxStreakBonus(maxStreakCount, pointsPerStreak, bonusTray)
  bonusTray:notifyStreakEnd()

  local maxStreakBonusPoints = maxStreakCount * pointsPerStreak
  if (maxStreakBonusPoints > 0) then
    GameSession.increaseScore(maxStreakBonusPoints)

    local notificationString = BonusTypes.getNotificationString(BonusTypes.TYPES.MAX_STREAK_BONUS)
    bonusTray:indicateSimpleTextBonus(notificationString, maxStreakBonusPoints)
  end

  --TODO any sort of animation that needs to be done
  GameSession.bonusReporter:setBonusCount(BonusTypes.TYPES.MAX_STREAK_BONUS, maxStreakCount)
end

function ScoringUtils.applyPerfectSetBonus(pointsPerPerfectSetBonus, bonusTray)
  GameSession.increaseScore(pointsPerPerfectSetBonus)

  local notificationString = BonusTypes.getNotificationString(BonusTypes.TYPES.PERFECT_SET_BONUS)
  bonusTray:indicateSimpleTextBonus(notificationString, pointsPerPerfectSetBonus)

  GameSession.bonusReporter:incrementBonusCount(BonusTypes.TYPES.PERFECT_SET_BONUS)
end

function ScoringUtils.applyCustomBonus(bonusPoints, bonusType, bonusTray)
  GameSession.increaseScore(bonusPoints)
  local notificationString = BonusTypes.getNotificationString(bonusType)
  bonusTray:indicateSimpleTextBonus(notificationString, bonusPoints)
end

function ScoringUtils.getDifficultyScaledPoints(premultipliedPoints)
  local postmultipliedPoints = premultipliedPoints * GameSession.getDifficultyScoreMultiplier()
  return postmultipliedPoints
end
