--- Vector operations

-- ===========================================================================
class("Vector")
-- ===========================================================================

-- Add a point to an array of points
function Vector.addPoint ( points, x, y, z )
  local point = {}
  point.x = x
  point.y = y
  point.z = z

  table.insert ( points, point )
end

-- Get the cross product of two 3D vectors.
function Vector.cross(p1,p2)
  local ret = {}
  ret.x = p1.y*p2.z - p1.z*p2.y
  ret.y = p1.z*p2.x - p1.x*p2.z
  ret.z = p1.x*p2.y - p1.y*p2.x
  return ret
end

-- Add two vectors
function Vector.add(p1,p2)
  local ret = {}
  ret.x = p1.x + p2.x
  ret.y = p1.y + p2.y
  if p1.z and p2.z then
    ret.z = p1.z + p2.z
  end
  return ret
end

-- Scale a vector
function Vector.scale(const,p1)
  local ret = {}
  ret.x = p1.x * const
  ret.y = p1.y * const
  if p1.z then
    ret.z = p1.z * const
  end
  return ret
end

-- Get the length of a vector
function Vector.length(p1)
  if p1.z then
    return math.sqrt(p1.x*p1.x + p1.y*p1.y + p1.z*p1.z)
  end
  return math.sqrt(p1.x*p1.x + p1.y*p1.y)
end

-- Normalize a vector
function Vector.normalize(p1)
  return Vector.scale(1/Vector.length(p1),p1)
end
