--- WordMatcher
-- @author Created by karl on February 11, 2015.

class("WordMatcher")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Fairly safe assumption. Until it's not.
local MAX_WORD_LENGTH = 30

local MATCH_DATA_POINTS = {
  [0] = 0,
  [4] = 1,
  [7] = 2,
  [11] = 3
}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getClosestMatch(self, word)
  local shortestDistance = math.huge
  local closestMatch = nil

  for potentialMatch in pairs(self.wordSet) do
    local distance = StringUtils.getLevenshteinDistance(word, potentialMatch)
    if distance <= shortestDistance then
      shortestDistance = distance
      closestMatch = potentialMatch
    end
  end

  return closestMatch, shortestDistance
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WordMatcher:getClosestMatch(word)
  local match, distance = getClosestMatch(self, word)
  local maxDistance = self.matchProfile[#match] or 0
  if distance <= maxDistance then
    return match
  end
  return nil
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newMatchProfile(dataPoints)
  local matchLength = dataPoints[0]
  local profile = {}
  for wordLength=1, MAX_WORD_LENGTH do
    matchLength = dataPoints[wordLength] or matchLength
    table.insert(profile, matchLength)
  end
  return profile
end

-- Constructor ------------------------

--- Create a new word matcher.
-- @param wordSet (set) - Set of words to match against.
function WordMatcher.new(wordSet)
  local self = {}
  Mixin.mix(self, WordMatcher)

  self.wordSet = wordSet
  self.matchProfile = newMatchProfile(MATCH_DATA_POINTS)

  return self
end
