--- XmlElement
-- @author Created by amorant on October 10, 2014.
-- XmlElements in an parsed XML document, generated by XmlParser.

class("XmlElement")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Element types ----------------------

--- Find out if this is a text element.
-- @return (boolean) - True if this element represents plain text.
function XmlElement:isTextElement()
  return self.text ~= nil
end

--- Find out if this is a tag element.
-- @return (boolean) - True if this element represents a tag.
function XmlElement:isTagElement()
  return self.label ~= nil
end

--- Find out if this is a root element.
-- @return (boolean) - True if this element represents the root element in an XML document.
function XmlElement:isRootElement()
  return self.text == nil and self.label == nil
end

-- Data accessors ---------------------

function XmlElement:getText()
  return self.text
end

function XmlElement:getLabel()
  return self.label
end

function XmlElement:getAttributes()
  return self.attributes
end

function XmlElement:getChildren()
  return self.children
end

function XmlElement:getPosition()
  return self.position
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

--- Constructor. Creates a new XmlElement containing text.
-- @param text (string) - Text for the element.
-- @param position (number) - Start character index of the element in the original text.
-- @return (XmlElement) - The new instance.
function XmlElement.newTextElement(text, position)
  local self = {}
  Mixin.mix(self, XmlElement)

  self.text = text
  self.label = nil
  self.attributes = {}
  self.children = {}
  self.position = position

  return self
end

--- Constructor. Creates a new XmlElement containing a tag.
-- @param label (string) - Tag label for the element.
-- @param attributes (table) - Map of attributes of the tag, (string) to (string).
-- @param children (array) - Child XmlElements of this element.
-- @param position (number) - Start character index of the element in the original text.
-- @return (XmlElement) - The new instance.
function XmlElement.newTagElement(label, attributes, children, position)
  local self = {}
  Mixin.mix(self, XmlElement)

  self.text = nil
  self.label = label
  self.attributes = attributes or {}
  self.children = children or {}
  self.position = position

  return self
end
