--- AlertPulse

class("AlertPulse")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_PULSE_DEFAULT    = { 0.8, 1, 1, 0.6 }

local RADIUS_DEFAULT         = 6
local PULSE_RADIUS_MAX_SCALE = 4
local PULSE_DURATION_DEFAULT = 0.4
local PULSE_DELAY_DEFAULT    = 0.1


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function animatePulse(self, color)
  return runInCoroutine(function()
    local circle = FilledCircle.new(self.radius, color)
    self:insertAtCenter(circle)
    circle:inheritColor(self)
    table.insert(self.animatingPulses, circle)

    local endScale = PULSE_RADIUS_MAX_SCALE
    local endOpacity = 0

    local scaleChange = AHScaleTo.new(circle, self.duration, AHEaseSineIn.new(), endScale)
    local opacityChange = AHOpacityTo.new(circle, self.duration, AHEaseSineIn.new(), endOpacity)
    local function removeCircleWrapper()
      circle:removeSelf()
      TableUtils.removeValueFromArray(self.animatingPulses, circle)
    end

    local pulseAnimation = AHSequence.new(AHSpawn.new(scaleChange, opacityChange), removeCircleWrapper)
    wait(pulseAnimation:start())
  end)
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function AlertPulse:pulse(count, color)
  count = count or 3
  color = color or self.color

  return runInCoroutine(function()
    for i = 1, count do
      local action = animatePulse(self, color)
      if i < count then
        waitFor(self.delay)
      else
        wait(action)
      end
    end
  end)
end

function AlertPulse:reset()
  for _, pulse in pairs(self.animatingPulses) do
    pulse:removeSelf()
  end
  self.animatingPulses = {}
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function AlertPulse.new(radius, color, duration, delay)
  radius = radius or RADIUS_DEFAULT
  color = color or COLOR_PULSE_DEFAULT
  duration = duration or PULSE_DURATION_DEFAULT
  delay = delay or PULSE_DELAY_DEFAULT

  local self = Group.new(radius*2, radius*2)
  self.radius = radius
  self.color = color
  self.duration = duration
  self.delay = delay
  Mixin.mix(self, AlertPulse)

  self.animatingPulses = {}

  return self
end
