--- AudioRequiredOverlayButton
-- @author Created by Aaron Barrett on October 30, 2020.

class("AudioRequiredOverlayButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HORIZONTAL_MARGIN = 35.0
local WIDTH = Screen.fullWidth - 2.0 * HORIZONTAL_MARGIN
local HEIGHT = 55.0
local CORNER_RADIUS = 3.0

local BLUE_BACKGROUND_COLOR = {0.200, 0.765, 0.976}
local BLUE_BACKGROUND_COLOR_DOWN_STATE = {0.0, 0.52, 0.73}
local WHITE_BACKGROUND_COLOR = {1.0, 1.0, 1.0, 0.3}
local WHITE_BACKGROUND_COLOR_DOWN_STATE = {1.0, 1.0, 1.0, 0.15}

local Z_BACKGROUND = 10
local Z_LABEL = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setPressed(self, pressed)
  local color = nil
  if (pressed) then
    color = self.backgroundColorDownState
  else
    color = self.backgroundColor
  end
  self.background:setColor(unpack(color))
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function configureColors(self, useWhiteColor)
  if (useWhiteColor) then
    self.backgroundColor = WHITE_BACKGROUND_COLOR
    self.backgroundColorDownState = WHITE_BACKGROUND_COLOR_DOWN_STATE
  else
    self.backgroundColor = BLUE_BACKGROUND_COLOR
    self.backgroundColorDownState = BLUE_BACKGROUND_COLOR_DOWN_STATE
  end
end

local function buildLabel(self, text)
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 18.0)
  local label = Label.new(text, font)
  self:insertAtCenter(label, Z_LABEL)
  self.label = label
end

local function buildBackground(self)
  local background = FilledRoundedRectangle.new(WIDTH, HEIGHT, CORNER_RADIUS)
  background:setColor(unpack(self.backgroundColor))
  self:insertAtCenter(background, Z_BACKGROUND)
  self.background = background
end

local function configureTouchManagement(self, onPressedCallback)
  local function onTouchDown()
    setPressed(self, true)
  end

  local function onTouchUp()
    setPressed(self, false)
    onPressedCallback()
  end

  local function onTouchCancel()
    setPressed(self, false)
  end

  Button.makeButton(self, onTouchDown, onTouchUp, onTouchCancel)
end

-- Constructor ------------------------

function AudioRequiredOverlayButton.new(text, onPressedCallback, useWhiteColor)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, AudioRequiredOverlayButton)

  configureColors(self, useWhiteColor)

  buildBackground(self)
  buildLabel(self, text)
  self:linkColorAttrToChildren()

  configureTouchManagement(self, onPressedCallback)

  return self
end
