--- KeyboardWarning
-- @author Created by karl on November 03, 2015.

class("KeyboardWarning")

require("widgets.TapToContinue")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function KeyboardWarning:show(onCompletion)
  onCompletion = onCompletion or function() end
  local fadeDuration = 0.4
  local fadeInAction = AHOpacityTo.new(self, fadeDuration, nil, 1):start()
  local tapToContinue = nil

  tapToContinue = TapToContinue.new(self, nil, function()
    runInCoroutine(function()
      fadeInAction:stop()
      AHSequence.new(
        AHOpacityTo.new(self, fadeDuration, nil, 0),
        function() tapToContinue:removeSelf() end
      ):start()
      onCompletion()
    end)
  end)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLabel(text, font, size, color)
  local label = Label.new(text, Font.new(font, size))
  label:setColor(unpack(color))
  return label
end

-- Constructor ------------------------

function KeyboardWarning.new(topMessage, bottomMessage)
  local self = Group.newFullScreen()
  Mixin.mix(self, KeyboardWarning)

  self:insertAtCenter(FilledRectangle.newFullScreen({0, 0, 0}))

  local col = Column.new(0)
  col:insert(Sprite.newWithName("warning_triangle.png"))
  col:insertSpacer(Screen.getHeightPart(0.02))
  col:insert(createLabel(topMessage, FontFaces.DIN_OT_MEDIUM, 17, {1, 1, 1}))
  col:insertSpacer(Screen.getHeightPart(0.01))
  col:insert(createLabel(bottomMessage, FontFaces.DIN_OT_MEDIUM, 16, {1, 1, 1, 0.5}))
  col:alignChildren()
  self:insertAtCenter(col)

  local continueText = StringUtils.capitalizeFirstLetter(string.lower(Localizer.get("Misc.CONTINUE")))
  local continueLabel = createLabel(continueText, FontFaces.DIN_OT_MEDIUM, 16, {1, 1, 1, 0.3})
  continueLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.07))
  self:insert(continueLabel)

  self:linkColorAttrToChildren(true)
  self:setOpacity(0)

  return self
end

function KeyboardWarning.newSingleWordWarning()
  return KeyboardWarning.new(Localizer.get("KeyboardWarning.SINGLE_TOP"), Localizer.get("KeyboardWarning.SINGLE_BOTTOM"))
end

function KeyboardWarning.newUnsupportedCharacterWarning()
  return KeyboardWarning.new(Localizer.get("KeyboardWarning.UNSUPPORTED_TOP"), Localizer.get("KeyboardWarning.UNSUPPORTED_BOTTOM"))
end
