--- A pause button widget
-- @author Created by Alexey Chernikov on May 1, 2013

-- ===========================================================================
class("PauseButton")
-- ===========================================================================

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DEFAULT_COLOR = {1.0, 1.0, 1.0, 1.0}
local DEFAULT_OPACITY = 1.0
local TAP_AREA_WIDTH = 40.0
local TAP_AREA_HEIGHT = 40.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function onPauseButtonTapped(self)
  if self.pauseCallback then
    self.pauseCallback()
  end

  GameSession.notifyGamePaused()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- UIOverlay Widget -------------------

function PauseButton:getKeyColor()
  return self.keyColor
end

function PauseButton:setTouchEnabled(isEnabled)
  self.virtualButton:setTouchEnabled(isEnabled)
end

function PauseButton:isTouchEnabled()
  return self.virtualButton:isTouchEnabled()
end

function PauseButton:getVirtualButton()
  return self.virtualButton
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function PauseButton.new(color, pauseCallback)
  color = color or DEFAULT_COLOR
  color[4] = color[4] or DEFAULT_OPACITY

  local self = Sprite.newWithName("shared_pause_button.png")
  Mixin.mix(self, PauseButton)

  self.keyColor = color
  self:setColor(unpack(color))

  self.pauseCallback = pauseCallback

  local onTouchDown = nil
  local onTouchUp = function() onPauseButtonTapped(self) end
  local onTouchCancel = nil
  self.virtualButton = Button.addVirtualButton(self,
                                               TAP_AREA_WIDTH,
                                               TAP_AREA_HEIGHT,
                                               onTouchDown,
                                               onTouchUp,
                                               onTouchCancel)
  self.virtualButton:setTouchEnabled(false)

  return self
end
