--- Label where show the current score of the game
-- @module ScoreLabel

-- ===========================================================================
class("ScoreLabel")
-- ===========================================================================

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MIN_STEP = 13
local MAX_STEP = 73
local POINTS_FOR_MIN_STEP = 50
local POINTS_FOR_MAX_STEP = 2000

local DEFAULT_COLOR = {1.0, 1.0, 1.0, 1.0}
local DEFAULT_OPACITY = 1.0

local SCORE_CHARACTERS = "0123456789"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getStep(oldScore, newScore)
  local points = newScore - oldScore
  points = math.abs(points)
  points = MathUtils.clamp(points, POINTS_FOR_MIN_STEP, POINTS_FOR_MAX_STEP)

  local step = MathUtils.interpolateLinear(points, POINTS_FOR_MIN_STEP, POINTS_FOR_MAX_STEP, MIN_STEP, MAX_STEP)
  step = MathUtils.round(step)
  step = step * MathUtils.sign(newScore - oldScore)

  return step
end

local function updateText(self, score)
  -- Choose floor because the host will display scores as floored values, too
  -- (truncates via type conversion from floating point -> integer)
  local flooredScore = math.floor(score)

  self:setText(flooredScore)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ScoreLabel:setScore(newScore)
  local oldScore = self.currentScore
  self.currentScore = newScore

  local function animatedSetScore()
    local step = getStep(oldScore, newScore)

    for score = oldScore, newScore - step, step do
      updateText(self, score)
      coroutine.yield()
    end

    updateText(self, newScore)
  end

  if (oldScore ~= newScore) then
    runInCoroutine(animatedSetScore)
  end
end

function ScoreLabel:setScoreInstant(newScore)
  updateText(self, newScore)
end

-- UIOverlay Widget -------------------

function ScoreLabel:getKeyColor()
  return self.keyColor
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ScoreLabel.new(font, fontSize, color, initialScore)
  initialScore = initialScore or 0

  color = color or DEFAULT_COLOR
  color[4] = color[4] or DEFAULT_OPACITY

  local prerenderedFont = PrerenderedFont.newWithActualSize(font, fontSize, SCORE_CHARACTERS)

  local self = Label.new(initialScore, prerenderedFont)
  Mixin.mix(self, ScoreLabel)

  self.currentScore = initialScore

  self:setColor(unpack(color))
  self.keyColor = color

  return self
end
