--- KeyPreview

class("KeyPreview")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_BACKGROUND = {1, 1, 1}
local COLOR_TEXT = {0, 0, 0, 0.8}
local COLOR_SHADOW = {0, 0, 0, 0.5}


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function KeyPreview.new(keyString, keyW, keyH, cornerRadius, bgColor, textColor)
  bgColor = bgColor or COLOR_BACKGROUND
  textColor = textColor or COLOR_TEXT
  local stemW, stemH = keyW, keyH * 1.3
  local previewW, previewH = keyW * 1.5, keyH * 1.5

  local self = Group.new(previewW, stemH + previewH)

  local keyBG = FilledRoundedRectangle.new(previewW, previewH, cornerRadius, bgColor)
  keyBG:setLoc(self:getWidthPart(0.5), self:getHeight() - keyBG:getHeightPart(0.5))

  local stem = FilledRectangle.new(stemW, stemH, bgColor)
  stem:setLoc(self:getWidthPart(0.5), keyBG:getY() - keyBG:getHeightPart(0.5) - stem:getHeightPart(0.5))

  local fontSize = keyH * 0.85
  local label = DeprecatedLabel.newSingleLine(keyString, FontFaces.DIN_OT_LIGHT, fontSize)
  label:setColor(unpack(textColor))
  keyBG:insertAtCenter(label)

  local shadow = Sprite.newWithName("shared_key_shadow.png")
  shadow:setColor(unpack(COLOR_SHADOW))
  shadow:setEffectiveDims(keyBG:getWidthPart(1.7), keyBG:getHeightPart(1.5))
  shadow:setLoc(keyBG:getX(), keyBG:getY() - keyBG:getHeightPart(0.05))

  self:insert(shadow)
  self:insert(keyBG)
  self:insert(stem)

  return self
end
