--- CircleStepProgressIndicator
-- @author Created by karl on October 10, 2014.

-- ===========================================================================
class("CircleStepProgressIndicator")
-- ===========================================================================

require("widgets.progress.ProgressIndicator")

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function updateStepCount(self, stepCount)
  while #self.children > stepCount do
    self:remove(self.children[#self.children])
  end
  while #self.children < stepCount do
    local circle = FilledCircle.new(self.circleRadius, self.neutralColor)
    self:insert(circle)
    circle:inheritColor(self)
  end
  self:alignChildren()
  self:setProgress(self:getProgress())
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CircleStepProgressIndicator:setProgress(progress)
  self.super.setProgress(self, progress)

  local step = self:getProgressStep()
  for i=1, self:getStepCount() do
    local color = self.neutralColor
    if (i - 1) == step then
      color = self.highlightColor
    end
    if (i > #self.children) then
      Log.error(string.format("CircleStepProgressIndicator tried to update the color for step %s but there are only %s steps total.", tostring(i), tostring(#self.children)))
    else
      self.children[i]:setColor(unpack(color))
    end
  end
end

function CircleStepProgressIndicator:setStepCount(stepCount)
  self.super.setStepCount(self, stepCount)
  updateStepCount(self, stepCount)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CircleStepProgressIndicator.new(circleRadius, spacing, highlightColor, neutralColor, stepCount)
  stepCount = stepCount or 1
  local self = Row.new(spacing)
  ProgressIndicator.makeProgressIndicator(self, stepCount)
  Mixin.mix(self, CircleStepProgressIndicator)

  self.circleRadius = circleRadius
  self.neutralColor = neutralColor
  self.highlightColor = highlightColor

  updateStepCount(self, stepCount)

  return self
end
