--- NumericProgressIndicator.
-- Displays progress as a discrete integral ratio.
-- Best manipulated using increaseProgress() and decreaseProgress().
-- @author Created by karl on September 10, 2014.

-- ===========================================================================
class("NumericProgressIndicator")
-- ===========================================================================

require("widgets.progress.ProgressIndicator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DEFAULT_SEPARATOR_TEXT = "/"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getFormatText(shouldShowMaxValue, separatorText)
  if shouldShowMaxValue then
    return "%d" .. separatorText .. "%d"
  end
  return "%d"
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function NumericProgressIndicator:setProgress(progress)
  self.super.setProgress(self, progress)
  local value = MathUtils.interpolateLinearFromNormalized(self:getProgress(), self.minValue, self.maxValue)
  local discreteValue = MathUtils.round(value)
  self.label:setText(string.format(self.format, discreteValue, self.maxValue))
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function getMaxDimensions(font, fontSize, format, maxValue)
  local label = DeprecatedLabel.newSingleLine(string.format(format, maxValue, maxValue), font, fontSize)
  return label:getDims()
end

-- Constructor ------------------------

--- Create a new progress indicator.
-- @param font (Font) - The font to use
-- @param fontSize (number) - The size of font.
-- @param minValue (integer) - The minimum value to display.
-- @param maxValue (integer) - The maximum value to display.
-- @param shouldShowMaxValue (boolean, default true) - If true, show with the max value (e.g. 1/10).
-- @param separatorText (string, optional) - The text to be displayed between the min and max values if shouldShowMaxValue is true
function NumericProgressIndicator.new(font, fontSize, minValue, maxValue, shouldShowMaxValue, separatorText)
  if shouldShowMaxValue == nil then
    shouldShowMaxValue = true
  end
  separatorText = separatorText or DEFAULT_SEPARATOR_TEXT

  local formatText = getFormatText(shouldShowMaxValue, separatorText)
  local self = Group.new(getMaxDimensions(font, fontSize, formatText, maxValue))
  local stepCount = maxValue - minValue
  ProgressIndicator.makeProgressIndicator(self, stepCount)
  Mixin.mix(self, NumericProgressIndicator)

  self.label = DeprecatedLabel.newSingleLine(string.format(formatText, minValue, maxValue), font, fontSize)
  self:insertAtCenter(self.label)

  self.minValue = minValue
  self.maxValue = maxValue
  self.format = formatText

  self:linkColorAttrToChildren(true)

  return self
end
