--- A progress bar that fills up a circular shape
-- @author Created by Alexey Chernikov on April 29, 2013

-- ===========================================================================
class("PieProgressIndicator")
-- ===========================================================================

require("widgets.progress.ProgressIndicator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_LOW  = 0
local Z_HIGH = 100

local DEFAULT_FILL_COLOR = {1.0, 1.0, 1.0, 1.0}
local DEFAULT_FILL_OPACITY = 1.0

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PieProgressIndicator:getAngleForProgress(progress)
  return self.baseAngle + progress * math.pi * 2
end

function PieProgressIndicator:setProgress(progress)
  self.super.setProgress(self, progress)
  progress = self:getProgress()
  self.pie:setEndAngle(self:getAngleForProgress(progress))
end

-- UIOverlay Widget -------------------

function PieProgressIndicator:getKeyColor()
  return self.keyColor
end

function PieProgressIndicator:createFadeInAction(duration)
  local keyColor = self:getKeyColor()
  local keyOpacity = keyColor[4]

  local fadeCircleAction = AHOpacityTo.new(self.underCircle, duration, nil, self.backOpacity)
  local fadePieAction = AHOpacityTo.new(self.pie, duration, nil, keyOpacity)
  local fadeAction = AHSpawn.new(fadeCircleAction, fadePieAction)
  return fadeAction
end

function PieProgressIndicator:prepareForGameIntro()
  self.pie:setOpacity(0.0)
  self.underCircle:setOpacity(0.0)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

--- Creates a pie-shaped progress bar.
-- When the value is at maximum, it takes the shape of a circle. As the value decreases, proportionate slices get "cut out"
-- @param radius - (float) the radius of the circle representing the progress pie
-- @param fillColor - (table) fill color in [0, 1] space
-- @param backOpacity - (number) the opacity to set the backing circle to
-- @param stepCount The number of steps. Can leave nil if you aren't going to use step functions.
function PieProgressIndicator.new(radius, fillColor, backOpacity, stepCount)
  local underColor = ColorUtils.colorWithDefaultOpacity(fillColor, backOpacity)
  fillColor = ColorUtils.colorWithDefaultOpacity(fillColor, fillColor[4] or DEFAULT_FILL_OPACITY)

  local self = Group.new(radius*2, radius*2)
  ProgressIndicator.makeProgressIndicator(self, stepCount)
  Mixin.mix(self, PieProgressIndicator)

  local selfW, selfH = self:getDims()

  self.radius = radius
  self.baseAngle = math.pi * 0.5
  self.fillColor = fillColor

  self.underCircle = FilledCircle.new(radius, underColor)
  self.underCircle:setLoc(selfW/2, selfH/2)

  local pieColor = fillColor
  self.pie = FilledArc.new(radius, self.baseAngle, self.baseAngle, pieColor)
  self.pie:setLoc(selfW/2, selfH/2)

  self:insert(self.underCircle, Z_LOW)
  self:insert(self.pie, Z_HIGH)

  self:linkAttrToChildren(MOAIColor.INHERIT_COLOR, MOAIColor.COLOR_TRAIT)

  self.keyColor = fillColor
  self.backOpacity = backOpacity

  return self
end
