--- Progress Timer
-- Turns any ProgressIndicator into a timer.
-- @module ProgressTimer

-- ===========================================================================
class("ProgressTimer")
-- ===========================================================================

require("Timer")
require("AudioTimer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local STANDARD_WARNING_SOUND = "shared_countdown_warning"

-- ----------------------------------------------------------------------------
-- Internal API
-- ----------------------------------------------------------------------------

function ProgressTimer:addStandardAudibleWarningSequence(volume)
  self:addWarningHandler(function()
    Sound.playSoundEffect(STANDARD_WARNING_SOUND, volume)
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ProgressTimer:setPausedOpacity(opacity)
  self._ProgressTimer_opacityPaused = opacity
end

function ProgressTimer:setActiveOpacity(opacity)
  self._ProgressTimer_opacityActive = opacity
end

function ProgressTimer:getPausedOpacity()
  return self._ProgressTimer_opacityPaused
end

function ProgressTimer:getActiveOpacity()
  return self._ProgressTimer_opacityActive
end

function ProgressTimer:pause()
  self.super.pause(self)
  if self._ProgressTimer_opacityPaused then
    self:setOpacity(self._ProgressTimer_opacityPaused)
  end
end

function ProgressTimer:resume()
  self.super.resume(self)
  if self._ProgressTimer_opacityActive then
    self:setOpacity(self._ProgressTimer_opacityActive)
  end
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function makeTimerCommon(self, onTimeout)
  self:setTimeoutHandler(onTimeout)

  local lastTimeRemaining = nil
  self:setUpdateHandler(function()
    local nowTimeRemaining = self:getNormalizedTimeRemaining()
    if nowTimeRemaining ~= lastTimeRemaining then
      self:setProgress(nowTimeRemaining)
      lastTimeRemaining = nowTimeRemaining
    end
  end)
end

-- Constructor ------------------------

function ProgressTimer.makeTimer(progressIndicator, duration, onTimeout)
  local self = progressIndicator
  Timer.makeTimer(self, duration)
  Mixin.mix(self, ProgressTimer)

  makeTimerCommon(self, onTimeout)

  return self
end

function ProgressTimer.makeAudioTimer(progressIndicator, soundFilename, duration, onTimeout)
  local self = progressIndicator
  AudioTimer.makeTimer(self, soundFilename, duration)
  Mixin.mix(self, ProgressTimer)

  makeTimerCommon(self, onTimeout)

  return self
end
