--- ChatConversationUtils

class("ChatConversationUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ChatConversationUtils.MESSAGE_SPLIT_DELIMITER = "\n\n"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ChatConversationUtils.validateConversationDescriptor(conversationDescriptor)
  Asserts.assertType(conversationDescriptor, "conversationDescriptor", "table")

  if (conversationDescriptor ~= nil) then
    assert(conversationDescriptor[ChatRationale.CONVERSATION_INTRO_KEY] ~= nil, "Invalid conversationDescriptor. No \"intro\" key found.")

    local intro = conversationDescriptor[ChatRationale.CONVERSATION_INTRO_KEY]
    if (intro ~= nil) then
      local conversationLength = string.len(intro)
      assert(conversationLength > 0, "Invalid conversationDescriptor. Intro had length of 0 characters.")
    end

    assert(conversationDescriptor[ChatRationale.CONVERSATION_OPTIONS_KEY] ~= nil, "Invalid conversationDescriptor. No \"options\" key found.")

    local options = conversationDescriptor[ChatRationale.CONVERSATION_OPTIONS_KEY]
    if (options ~= nil) then
      local optionsCount = TableUtils.getSize(options)
      assert(optionsCount > 0, "Invalid conversationDescriptor. Options list contained no options.")

      if (optionsCount > 0) then
        for _, option in ipairs(options) do
          Asserts.assertType(option, "option", "table")

          local id = option[ChatRationale.CONVERSATION_OPTIONS_ID_KEY]
          if (id ~= nil) then
            Asserts.assertType(option[ChatRationale.CONVERSATION_OPTIONS_ID_KEY], "option[\"id\"]", "string")
            local idLength = string.len(id)
            assert(idLength > 0, "Invalid conversationDescriptor. An \"id\" of length 0 was encountered.")
          end

          local userMessage = option[ChatRationale.CONVERSATION_OPTIONS_USER_MESSAGE_KEY]
          if (userMessage ~= nil) then
            Asserts.assertType(option[ChatRationale.CONVERSATION_OPTIONS_USER_MESSAGE_KEY], "option[\"userMessage\"]", "string")
            local userMessageLength = string.len(userMessage)
            assert(userMessageLength > 0, "Invalid conversationDescriptor. A \"userMessage\" of length 0 was encountered.")
          end

          local systemMessage = option[ChatRationale.CONVERSATION_OPTIONS_SYSTEM_MESSAGE_KEY]
          if (systemMessage ~= nil) then
            Asserts.assertType(option[ChatRationale.CONVERSATION_OPTIONS_SYSTEM_MESSAGE_KEY], "option[\"systemMessage\"]", "string")
            local systemMessageLength = string.len(systemMessage)
            assert(systemMessageLength > 0, "Invalid conversationDescriptor. A \"systemMessage\" of length 0 was encountered.")
          end
        end
      end
    end
  end
end

function ChatConversationUtils.createFormattedConversationDescriptor(conversationDescriptor)
  local formattedConversationDescriptor = {}

  -- Format intro
  local intro = conversationDescriptor[ChatRationale.CONVERSATION_INTRO_KEY]
  local introMessages = StringUtils.split(intro, ChatConversationUtils.MESSAGE_SPLIT_DELIMITER)
  formattedConversationDescriptor[ChatRationale.CONVERSATION_INTRO_KEY] = introMessages

  -- Format options
  local options = conversationDescriptor[ChatRationale.CONVERSATION_OPTIONS_KEY]
  local newOptions = {}
  for _, option in pairs(options) do
    local newOption = {}

    newOption[ChatRationale.CONVERSATION_OPTIONS_ID_KEY] = option[ChatRationale.CONVERSATION_OPTIONS_ID_KEY]

    newOption[ChatRationale.CONVERSATION_OPTIONS_USER_MESSAGE_KEY] = option[ChatRationale.CONVERSATION_OPTIONS_USER_MESSAGE_KEY]

    local newSystemMessage = option[ChatRationale.CONVERSATION_OPTIONS_SYSTEM_MESSAGE_KEY]
    newOption[ChatRationale.CONVERSATION_OPTIONS_SYSTEM_MESSAGE_KEY] = StringUtils.split(newSystemMessage, ChatConversationUtils.MESSAGE_SPLIT_DELIMITER)

    table.insert(newOptions, newOption)
  end
  formattedConversationDescriptor[ChatRationale.CONVERSATION_OPTIONS_KEY] = newOptions

  return formattedConversationDescriptor
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
