--- ChatRationaleAnalytics

class("ChatRationaleAnalytics")

require("Analytics")
require("ExperimentsConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local EVENT_KEY_MAIN_RATIONALE_SEEN = "ExpandedRationalesMainRationaleSeen"
local EVENT_KEY_LEARN_MORE_TAPPED = "ExpandedRationalesTellMeMoreTapped"
local EVENT_KEY_DISMISSED = "ExpandedRationalesDismissed"
local EVENT_KEY_OPTION_TAPPED = "ExpandedRationalesOptionTapped"

local PROPERTY_KEY_SKILL = "skill"
local PROPERTY_KEY_HAS_LEARN_MORE_AVAILABLE = "has_tell_me_more_available"
local PROPERTY_KEY_CONCEPT_ID = "concept_id"
local PROPERTY_KEY_EXPLANTION_ID = "expanded_rationale_explanation_id"

local PROPERTY_VALUE_NOT_SET = "unknown"

local SKILL = PROPERTY_VALUE_NOT_SET
local CONCEPT_ID = PROPERTY_VALUE_NOT_SET

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function reportEvent(eventKey, propertiesMapping)
  if (SKILL == PROPERTY_VALUE_NOT_SET) then
    Log.warn("ChatRationaleAnalytics.setSkill(skill) must be called before reporting")
  end

  if (CONCEPT_ID == PROPERTY_VALUE_NOT_SET) then
    Log.warn("ChatRationaleAnalytics.setConceptId(conceptId) must be called before reporting")
  end

  local commonProperties = {
    [PROPERTY_KEY_SKILL] = SKILL,
    [PROPERTY_KEY_CONCEPT_ID] = CONCEPT_ID
  }

  Analytics.reportEvent(eventKey, TableUtils.unionSet(commonProperties, propertiesMapping))
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ChatRationaleAnalytics.reportMainRationaleSeen(hasLearnMoreAvailable)
  local properties = {
    [PROPERTY_KEY_HAS_LEARN_MORE_AVAILABLE] = hasLearnMoreAvailable
  }
  reportEvent(EVENT_KEY_MAIN_RATIONALE_SEEN, properties)
end

function ChatRationaleAnalytics.reportLearnMoreTapped()
  reportEvent(EVENT_KEY_LEARN_MORE_TAPPED)
end

function ChatRationaleAnalytics.reportDismissed()
  reportEvent(EVENT_KEY_DISMISSED)
end

function ChatRationaleAnalytics.reportOptionTapped(explanationId)
  local properties = {
    [PROPERTY_KEY_EXPLANTION_ID] = explanationId
  }
  reportEvent(EVENT_KEY_OPTION_TAPPED, properties)
end

function ChatRationaleAnalytics.setSkill(skill)
  SKILL = skill
end

function ChatRationaleAnalytics.setConceptId(conceptId)
  CONCEPT_ID = conceptId
end

function ChatRationaleAnalytics.reportExperimentExposed()
  ExperimentsConfiguration.EXPANDED_RATIONALES_EXPERIMENT:reportExperimentExposed()
end
