--- ChatRationaleLearnMoreButton

class("ChatRationaleLearnMoreButton")

require("widgets.rationales.ChatRationaleAnalytics")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local OUTLINE_THICKNESS = 2
local LABEL_TEXT = "TELL ME MORE"
local LABEL_COLOR = ColorUtils.colorHex24ToStd(0x606060)
local OUTLINE_COLOR = ColorUtils.colorHex24ToStd(0xcccccc)
local ROW_PADDING_X = 12
local PADDING_X = 16
local PADDING_Y = 8
local VIRTUAL_BUTTON_PADDING = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function handleTouchDown(self)
  if (self.pressedCallback ~= nil) then
    self.pressedCallback()
  end

  ChatRationaleAnalytics.reportLearnMoreTapped(skill)
  Sound.playSoundEffect("shared_rationales_start")
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ChatRationaleLearnMoreButton:setTouchEnabled(isEnabled)
  self.virtualButton:setTouchEnabled(isEnabled)
end

function ChatRationaleLearnMoreButton:isTouchEnabled()
  return self.virtualButton:isTouchEnabled()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(color)
  local brainIcon = Sprite.newWithName("shared_brain_icon.png")
  brainIcon:setScl(0.8)

  local brainIconHeight = brainIcon:getEffectiveHeight()

  local forwardIcon = Sprite.newWithName("forward_icon.png")

  local label = Label.new(LABEL_TEXT, Font.new(FontFaces.DIN_OT_MEDIUM, 14))
  label:setColor(unpack(LABEL_COLOR))

  local contentsRow = Row.new(ROW_PADDING_X)
  contentsRow:insert(brainIcon)
  contentsRow:insert(label)
  contentsRow:insert(forwardIcon)
  contentsRow:alignChildren()

  local width = contentsRow:getWidth() + (2 * PADDING_X)
  local height = contentsRow:getHeight() + (2 * PADDING_Y)
  local radius = height / 2

  local self = FilledRoundedRectangle.new(width, height, radius, OUTLINE_COLOR)
  local insideFill = FilledRoundedRectangle.new(width - OUTLINE_THICKNESS, height - OUTLINE_THICKNESS, radius - OUTLINE_THICKNESS / 2, color)
  self.insideFill = insideFill

  self:insertAtCenter(insideFill)
  self:insertAtCenter(contentsRow)

  return self
end

function ChatRationaleLearnMoreButton.new(color, pressedCallback)
  Asserts.assertType(pressedCallback, "pressedCallback", "function")

  local self = createLayout(color)
  Mixin.mix(self, ChatRationaleLearnMoreButton)

  local onTouchDown = function() handleTouchDown(self) end
  local onTouchUp = nil
  local onTouchCancel = nil

  self.virtualButton = Button.addVirtualButton(self,
                                               self:getWidth() + VIRTUAL_BUTTON_PADDING,
                                               self:getHeight() + VIRTUAL_BUTTON_PADDING,
                                               onTouchDown,
                                               onTouchUp,
                                               onTouchCancel)
  self.virtualButton:setTouchEnabled(false)

  self.pressedCallback = pressedCallback

  return self
end
