--- ArcticAnimatedLine
-- @author Created by Joe Poka on November 28, 2016.

class("ArcticAnimatedLine")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_INDEX = {
  endCaps = 10,
  rectangle = 20
}

local INITIAL_SCALE = 0.01

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function toggleVisibility(self, visible)
  for _, item in ipairs(self.items) do
    item:setVisible(visible)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticAnimatedLine:reset()
  self.rectangle:setScl(INITIAL_SCALE, 1.0)
  local isVisible = false
  toggleVisibility(self, isVisible)
end

function ArcticAnimatedLine:adjustDeltaX(deltaXAdjust)
  self.deltaScaleX = self.defaultDeltaScaleX * deltaXAdjust
end

function ArcticAnimatedLine:show()
  local isVisible = true
  toggleVisibility(self, isVisible)
end

function ArcticAnimatedLine:getAppearAnimation()
  local maximumScaleX = 1.0
  local angleRadians = math.rad(self.angleDegrees)
  local appearAnimation = nil

  local scaleUp = function()
    local scaleX, _ = self.rectangle:getScl()
    local updatedScaleX = math.min(scaleX + self.deltaScaleX, maximumScaleX)
    self.rectangle:setScl(updatedScaleX, 1.0)

    local effectiveWidth = self.rectangle:getEffectiveWidth()
    local x = self.endCapInitialX + effectiveWidth * math.cos(angleRadians)
    local y = self.endCapInitialY + effectiveWidth * math.sin(angleRadians)
    self.endCap:setLoc(x, y)

    local done = (math.abs(updatedScaleX) >= maximumScaleX)
    if (done) then
      appearAnimation:stop()
    end
  end

  appearAnimation = AHLoop.new(scaleUp)
  return appearAnimation
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, parameters)
  local width = parameters.length or Screen.getWidthPart(0.5)
  local height = parameters.lineWidth or Screen.getHeightPart(0.005)
  self.items = {}

  local color = {1.0, 1.0, 1.0, 1.0}
  self.rectangle = FilledRectangle.new(width, height, color)
  self.rectangle:transformAnchorPoint(0.0, 0.5)
  self:insertAtCenter(self.rectangle, Z_INDEX.rectangle)
  table.insert(self.items, self.rectangle)

  self.angleDegrees = parameters.angleDegrees or 0.0
  self.rectangle:setRot(self.angleDegrees)

  local radius = height * 0.5
  local startCap = FilledCircle.new(radius, color)
  self:insertAtCenter(startCap, Z_INDEX.endCaps)
  table.insert(self.items, startCap)

  self.endCap = FilledCircle.new(radius, color)
  self:insertAtCenter(self.endCap, Z_INDEX.endCaps)
  table.insert(self.items, self.endCap)

  self.endCapInitialX, self.endCapInitialY = self.endCap:getLoc()
  self.defaultDeltaScaleX = parameters.deltaX or 0.03
  self.deltaScaleX = self.defaultDeltaScaleX

  self:reset()
end

-- Constructor ------------------------

function ArcticAnimatedLine.new(parameters)
  local self = Group.newFullScreen()
  Mixin.mix(self, ArcticAnimatedLine)

  createLayout(self, parameters)

  return self
end
