--- ArcticConfiguration
-- @author Created by joepoka on October 19, 2016.

class("ArcticConfiguration")

require("difficulty.DifficultyUtils")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ArcticConfiguration.GAME_DURATION = 70
ArcticConfiguration.POINTS_PER_FULL_SPEED_BONUS = 50
ArcticConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 4
ArcticConfiguration.SPEED_BONUS_DECAY_START = 0
ArcticConfiguration.INCORRECT_RESPONSE_TIME_PENALTY = 5

local POINTS_PER_CORRECT_ANSWER_BY_CONTENT_DIFFICULTY = {
  Easy = 1200,
  Medium = 1205,
  Hard = 1215,
  Advanced = 1225
}

local CORRECT_NEEDED_COUNT_RANK_TWO = 5
local CORRECT_NEEDED_COUNT_RANK_THREE = 7

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getPointsForCorrectAnswerForRankThreshold()
  return ArcticConfiguration.getPointsForCorrectAnswer("Easy")
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticConfiguration.getPointsForCorrectAnswer(contentDifficulty)
  local points = POINTS_PER_CORRECT_ANSWER_BY_CONTENT_DIFFICULTY[contentDifficulty]
  assert(points ~= nil, "ContentDifficulty is not associated with a point value: " .. tostring(contentDifficulty))
  return points
end

function ArcticConfiguration.getRankTwoScore()
  local rankTwoScore = getPointsForCorrectAnswerForRankThreshold() * CORRECT_NEEDED_COUNT_RANK_TWO
  return rankTwoScore
end

function ArcticConfiguration.getRankThreeScore()
  local rankThreeScore = getPointsForCorrectAnswerForRankThreshold() * CORRECT_NEEDED_COUNT_RANK_THREE
  return rankThreeScore
end

function ArcticConfiguration.getDifficultyKeyFrames()
  local keyFrames = {
    {
      key = 0.0,
      valueProbabilities = {
        Easy = 1,
      }
    },
    {
      key = 0.075,
      valueProbabilities = {
        Easy = 0.95,
        Medium = 0.05,
      }
    },
    {
      key = 0.15,
      valueProbabilities = {
        Easy = 0.85,
        Medium = 0.15,
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.3,
        Medium = 0.7,
      }
    },
    {
      key = 0.35,
      valueProbabilities = {
        Easy = 0.1,
        Medium = 0.9,
      }
    },
    {
      key = 0.4,
      valueProbabilities = {
        Medium = 0.8,
        Hard = 0.2
      }
    },
    {
      key = 0.5,
      valueProbabilities = {
        Medium = 0.6,
        Hard = 0.4
      }
    },
    {
      key = 0.7,
      valueProbabilities = {
        Medium = 0.3,
        Hard = 0.7
      }
    },
    {
      key = 0.75,
      valueProbabilities = {
        Medium = 0.2,
        Hard = 0.7,
        Advanced = 0.1
      }
    },
    {
      key = 0.9,
      valueProbabilities = {
        Hard = 0.4,
        Advanced = 0.6
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 0.2,
        Advanced = 0.8
      }
    }
  }

  return keyFrames
end
