--- ArcticIncorrectAnswerScreen
-- @author Created by Joe Poka on November 18, 2016.

class("ArcticIncorrectAnswerScreen")

require("ArcticCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticIncorrectAnswerScreen:getEnterAnimation(duration, animateLeft)
  local deltaX = self:getWidthPart(1.0)
  local offsetLeft = (not animateLeft)
  if (offsetLeft) then
    deltaX = deltaX * -1.0
  end

  local destinationX = self.equationLabel:getX()
  local offsetX = destinationX + deltaX
  self.equationLabel:setX(offsetX)

  local slideDuration = duration
  local delay = 0.65 * duration

  local ease = AHEaseExponentialOut.new()
  local slideIn = AHLocationXTo.new(self.equationLabel, slideDuration, ease, destinationX)
  local delayedSlideIn = AHDelayed.new(delay, slideIn)

  local fadeDuration = 0.15 * duration
  local fadeIn = AHOpacityTo.new(self, fadeDuration, nil, 1.0)
  local appear = AHSpawn.new(delayedSlideIn, fadeIn)
  return appear
end

function ArcticIncorrectAnswerScreen:getExitAnimation(duration, animateLeft)
  local fadeDuration = 0.3 * duration
  local fadeOut = AHOpacityTo.new(self, fadeDuration, nil, 0.0)
  return fadeOut
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, topText, equationText, tapLeftArrow, tapRightArrow, showRightArrow)
  local margin = self:getWidthPart(0.13)
  local width = self:getWidth() - 2.0 * margin
  local lineSpacing = self:getHeightPart(0.01)

  local baseAttributes = {
    font = Font.new(FontFaces.DIN_OT_LIGHT, 27.0),
    color = ArcticCommon.COLORS.TEXT_WHITE
  }

  local textArea = DynamicTextArea.new(topText, width, baseAttributes, nil, nil, nil, lineSpacing)
  self:insert(textArea, ArcticMessageOverlay.Z_INDEX.ui)

  local ruleHeight = self:getHeightPart(0.001)
  local ruleColor = {0.3, 0.3, 0.3, 1.0}
  local horizontalRule = FilledRectangle.new(width, ruleHeight, ruleColor)
  self:insert(horizontalRule)

  local equationFontSize = 27.0
  local equationFont = Font.new(FontFaces.DIN_OT_MEDIUM, equationFontSize)
  self.equationLabel = Label.new(equationText, equationFont)
  local equationScale = math.min(1.0, width / self.equationLabel:getWidth())
  self.equationLabel:setScl(equationScale)
  self.equationLabel:setColor(unpack(ArcticCommon.COLORS.TEXT_WHITE))
  self:insert(self.equationLabel)

  local tapWidth = margin
  local tapHeight = self:getHeightPart(0.5)

  local tapAreaRight = nil
  if (showRightArrow) then
    tapAreaRight = Group.new(tapWidth, tapHeight)
    self:insert(tapAreaRight)

    local arrowSpriteRight = Sprite.newWithName("tiparrow.png")
    Button.makeButton(tapAreaRight, tapRightArrow)
    tapAreaRight:insertAtCenter(arrowSpriteRight)
  end

  local tapAreaLeft = Group.new(tapWidth, tapHeight)
  Button.makeButton(tapAreaLeft, tapLeftArrow)
  self:insert(tapAreaLeft)

  local arrowSpriteLeft = Sprite.newWithName("tiparrow.png")
  arrowSpriteLeft:setScl(-1.0, 1.0)
  tapAreaLeft:insertAtCenter(arrowSpriteLeft)

  local totalHeight = textArea:getHeight() + self.equationLabel:getHeight()

  local verticalSpacing = self:getHeightPart(0.04)
  local textAreaAnchorY = self:getHeightPart(0.53)
  totalHeight = totalHeight + 2.0 * verticalSpacing

  local x = margin + textArea:getWidthPart(0.5)
  local y = textAreaAnchorY + totalHeight * 0.5 - textArea:getHeightPart(0.5)
  textArea:setLoc(x, y)

  x = margin + textArea:getWidthPart(0.5)
  y = y - verticalSpacing - textArea:getHeightPart(0.5)
  horizontalRule:setLoc(x, y)

  x = margin + self.equationLabel:getEffectiveWidthPart(0.5)
  y = y - verticalSpacing - self.equationLabel:getHeightPart(0.5)
  self.equationLabel:setLoc(x, y)

  if (showRightArrow) then
    x = self:getWidth() - margin * 0.5
    y = textAreaAnchorY
    tapAreaRight:setLoc(x, y)
  end

  x = margin * 0.5
  y = textAreaAnchorY
  tapAreaLeft:setLoc(x, y)
end

-- Constructor ------------------------

function ArcticIncorrectAnswerScreen.new(topText, equationText, tapLeftArrow, tapRightArrow, showRightArrow)
  local self = Group.newFullScreen()
  Mixin.mix(self, ArcticIncorrectAnswerScreen)

  createLayout(self, topText, equationText, tapLeftArrow, tapRightArrow, showRightArrow)

  return self
end
