--- ArcticIncorrectAnswerScreenIntro
-- @author Created by Joe Poka on November 21, 2016.

class("ArcticIncorrectAnswerScreenIntro")

require("ArcticCommon")
require("ArcticShake")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CROSS_LOCATION_Y_FRACTIONAL = 0.67

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getShakeAnimation(triangle, minimumDelta, maximumDelta, shakeCount)
  minimumDelta = minimumDelta or Screen.getWidthPart(0.0008)
  maximumDelta = maximumDelta or Screen.getWidthPart(0.08)
  shakeCount = shakeCount or 8

  local parameters = {
    minimumDelta = minimumDelta,
    maximumDelta = maximumDelta,
    shakeCount = shakeCount,
    useRandomSigns = true
  }

  local shake = ArcticShake.getShakeAnimation(triangle, parameters)
  return shake
end

local function createCross(self)
  local width = self:getWidthPart(0.725)
  local height = 0.3 * width
  local color = {0.99, 0.925, 0.84, 1.0}

  local cross = FilledRectangle.new(width, height, color)

  local shortWidth = (width - height) * 0.5
  local topPiece = FilledRectangle.new(height, shortWidth, color)
  local x = width * 0.5
  local y = height + shortWidth * 0.5
  topPiece:setLoc(x, y)
  cross:insert(topPiece)

  local bottomPiece = FilledRectangle.new(height, shortWidth, color)
  y = shortWidth * -0.5
  bottomPiece:setLoc(x, y)
  cross:insert(bottomPiece)

  cross:setRot(45.0)
  cross:linkOpacityAttrToChildren(true)

  return cross
end

local function getShakyTriangleAnimation(self)
  local sideLength = Screen.getWidthPart(0.6)
  local height = sideLength * 0.866
  local color = {1.0, 0.97, 0.84, 1.0}
  local triangle = FilledTriangle.new(0.0, 0.0, sideLength, 0.0, sideLength * 0.5, height, color)
  local duration = 0.7

  local x = self:getWidthPart(0.5)
  local y = self:getHeightPart(0.65)
  triangle:setLoc(x, y)

  local showTriangle = function()
    self:insert(triangle)
  end

  local destinationY = self:getHeightPart(0.7)
  local moveTriangle = AHLocationYTo.new(triangle, duration, AHEaseSimpleOut.new(), destinationY)
  local shakeTriangle = getShakeAnimation(triangle)
  local fadeTriangle = AHOpacityTo.new(triangle, duration * 0.25, nil, 0.0)
  local delayedFadeTriangle = AHDelayed.new(0.1, fadeTriangle)

  self.cross = createCross(self)
  self.cross:setOpacity(0.0)

  local showCross = function()
    self:insertAtLocationFractional(self.cross, 0.5, CROSS_LOCATION_Y_FRACTIONAL)
    local fade = AHOpacityTo.new(self.cross, 0.1, nil, 1.0)
    local scale = AHScaleTo.new(self.cross, 0.4, AHEaseSimpleOut.new(5), 0.375)

    local minimumDelta = Screen.getWidthPart(0.0004)
    local maximumDelta = minimumDelta * 100.0
    local shakeCount = 4
    local shake = getShakeAnimation(self.cross, minimumDelta, maximumDelta, shakeCount)

    fade:start()
    scale:start()
    shake:start()
  end
  local delayedShowCross = AHDelayed.new(duration * 0.2, showCross)

  local triangleHeight = height * 0.72
  local rectangleCount = 7
  local leftSide = false
  local width = sideLength * 0.45
  local particleHeight = height * 0.1
  local minimumAlpha = 0.85
  local maximumAlpha = 1.0
  local defaultColorRGB = {0.894, 0.463, 0.447}
  self.particles = ArcticShinyRectangles.new(triangleHeight, rectangleCount, leftSide, width, particleHeight, minimumAlpha, maximumAlpha, defaultColorRGB)
  self.particles:setRot(90.0)
  self.particles:setScl(-1.0, 1.0)
  self.particles:setBlendMode(MOAIProp2D.BLEND_ADD)

  y = self:getHeightPart(0.72)
  self.particles:setLoc(x, y)

  local showParticles = function()
    self:insert(self.particles)
    self.particlesPulseAnimation = self.particles:getPulsingAnimation()
    self.particlesPulseAnimation:start()
  end

  local delayedShowParticles = AHDelayed.new(duration * 0.3, showParticles)

  local animateItems = AHSpawn.new(moveTriangle, shakeTriangle, delayedFadeTriangle, delayedShowCross, delayedShowParticles)
  local animation = AHSequence.new(showTriangle, animateItems)
  return animation
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticIncorrectAnswerScreenIntro:getEnterAnimation(duration, animateLeft)
  local fadeDuration = 0.15 * duration
  local fadeIn = AHOpacityTo.new(self, fadeDuration, nil, 1.0)
  return fadeIn
end

function ArcticIncorrectAnswerScreenIntro:getExitAnimation(duration, animateLeft)
  local fadeDuration = 0.3 * duration
  local fadeOut = AHOpacityTo.new(self, fadeDuration, nil, 0.0)
  return fadeOut
end

function ArcticIncorrectAnswerScreenIntro:getHideAnimation(duration)
  local hideParticles = self.particles:getFadeOutAnimation(duration * 2.0)
  local fadeCross = AHOpacityTo.new(self.cross, duration, nil, 0.0)
  local stopParticlesPulsing = function()
    self.particlesPulseAnimation:stop()
  end
  local animation = AHSpawn.new(hideParticles, fadeCross, stopParticlesPulsing)
  return animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createAnswerLabel(self, valueString, isIncorrect)
  local incorrectColor = {0.99, 0.05, 0.05, 1.0}
  local labelColor = ArcticCommon.COLORS.TEXT_WHITE
  if (isIncorrect) then
    labelColor = incorrectColor
  end

  local font = Font.new(FontFaces.DIN_OT_LIGHT, 50.0)

  local label = Label.new(valueString, font)
  label:setColor(unpack(labelColor))

  if (isIncorrect) then
    local strikeHangingWidth = self:getWidthPart(0.03)
    local strikeWidth = label:getWidth() + 2.0 * strikeHangingWidth
    local strikeHeight = self:getHeightPart(0.003)
    local strike = FilledRectangle.new(strikeWidth, strikeHeight, incorrectColor)
    label:insertAtCenter(strike)
  end

  return label
end

local function createLayout(self, round, submittedAnswer, tapLearnMore)
  local values = round.values
  local valuesCount = #values
  local submittedAnswerString = tostring(submittedAnswer)
  local correctAnswerString = tostring(round.answer)
  local sum = TableUtils.sum(values)

  local shakyTriangleAnimation = getShakyTriangleAnimation(self)
  shakyTriangleAnimation:start()

  local spacingY = self:getHeightPart(0.06)
  if (round.tipType == "extended") then
    spacingY = self:getHeightPart(0.04)
  end
  local textColumn = Column.new(spacingY)

  local spacingX = self:getWidthPart(0.22)
  local answersRow = Row.new(spacingX)

  local isIncorrect = true
  local incorrectAnswerLabel = createAnswerLabel(self, submittedAnswerString, isIncorrect)
  answersRow:insert(incorrectAnswerLabel)

  isIncorrect = false
  local correctAnswerLabel = createAnswerLabel(self, correctAnswerString, isIncorrect)
  answersRow:insert(correctAnswerLabel)

  answersRow:alignChildren()
  textColumn:insert(answersRow)

  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 17.0)
  local margin = self:getWidthPart(0.1)
  local width = self:getWidth() - 2.0 * margin
  local lineSpacing = 4

  local textAttributes = {
    font = font,
    color = ArcticCommon.COLORS.TEXT_WHITE,
  }

  local rationaleStringTemplate = Localizer.get("RATIONALE_TEMPLATE")
  local text = string.format(rationaleStringTemplate, sum, valuesCount, correctAnswerString)
  local textAreaTop = DynamicTextArea.new(text, width, textAttributes, nil, nil, nil, lineSpacing, true)
  textColumn:insert(textAreaTop)

  local includeMiddleNumberComment = (round.tipType == "extended")
  if (includeMiddleNumberComment) then
    text = Localizer.get("MIDDLE_NUMBER_COMMENT")
    local textAreaBottom = DynamicTextArea.new(text, width, textAttributes, nil, nil, nil, lineSpacing, true)
    textColumn:insert(textAreaBottom)
  end

  local learnMoreColor = {0.5, 0.5, 0.5, 1.0}
  local learnMoreFont = Font.new(FontFaces.DIN_OT_MEDIUM, 14.0)
  local learnMoreLabel = Label.new(Localizer.get("LEARN_MORE").." →", learnMoreFont)
  learnMoreLabel:setColor(unpack(learnMoreColor))
  textColumn:insert(learnMoreLabel)

  local tapWidth = self:getWidth()
  local tapHeight = self:getHeightPart(0.12)
  Button.addVirtualButton(learnMoreLabel, tapWidth, tapHeight, tapLearnMore)

  textColumn:alignChildren()
  textColumn:setAnchorPointTop()
  self:insertAtLocationFractional(textColumn, 0.5, 0.56, ArcticMessageOverlay.Z_INDEX.ui)
end

-- Constructor ------------------------

function ArcticIncorrectAnswerScreenIntro.new(round, submittedAnswer, tapLearnMore)
  local self = Group.newFullScreen()
  Mixin.mix(self, ArcticIncorrectAnswerScreenIntro)

  createLayout(self, round, submittedAnswer, tapLearnMore)

  return self
end
