--- ArcticLargeStar
-- @author Created by Joe Poka on December 1, 2016.

class("ArcticLargeStar")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticLargeStar:getAppearAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = 0.375
  local scale = AHScaleTo.new(self.triangleGroup, duration, nil, 1.0)
  table.insert(actions, scale)

  local moveDisk1 = AHLocationTo.new(self.sideDisk1, duration, nil, self.sideDisk1DestinationX, self.sideDisk1DestinationY)
  local moveDisk2 = AHLocationTo.new(self.sideDisk2, duration, nil, self.sideDisk2DestinationX, self.sideDisk2DestinationY)
  table.insert(actions, moveDisk1)
  table.insert(actions, moveDisk2)

  local scaleDisk1 = AHScaleTo.new(self.sideDisk1, duration, nil, 1.0)
  local scaleDisk2 = AHScaleTo.new(self.sideDisk2, duration, nil, 1.0)
  table.insert(actions, scaleDisk1)
  table.insert(actions, scaleDisk2)

  local largeStarAnimation = AHSpawn.new(actions)

  return self.super.getAppearAnimation(self, largeStarAnimation)
end

function ArcticLargeStar:getPulseAnimation()
  local randomPulseDuration = math.random(3, 5)
  local ease = AHEaseSineInOut.new()

  local additionalAction = AHSpawn.new(
    AHSequence.new(
      AHOpacityTo.new(self.triangleGroup, randomPulseDuration, ease, 0.25),
      AHOpacityTo.new(self.triangleGroup, randomPulseDuration, ease, 1.0)
    ),
    AHSequence.new(
      AHOpacityTo.new(self.centerDisk, randomPulseDuration, ease, 0.0),
      AHOpacityTo.new(self.centerDisk, randomPulseDuration, ease, 0.1)
    )
  )

  local pulseAnimation = self.super.getPulseAnimation(self, randomPulseDuration, additionalAction)
  return pulseAnimation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local sideLength = self:getWidthPart(0.03)
  local triangleHeight = self:getHeightPart(0.3)
  local triangleColor = {1.0, 1.0, 1.0, 0.8}
  local lineWidth = self:getWidthPart(0.01)
  local blurMargin = self:getWidthPart(0.025)
  local triangleParameters = {0.0, 0.0, sideLength, 0.0, sideLength * 0.5, triangleHeight, triangleColor, lineWidth, blurMargin}
  local triangleCount = 4

  local triangleGroup = Group.new(self:getDims())
  for i = 1, triangleCount do
    local triangle = HollowTriangle.new(unpack(triangleParameters))
    triangle:setAnchorPointBottom()
    triangle:setRot((i - 1) * 90.0)
    triangleGroup:insertAtCenter(triangle)
  end
  triangleGroup:setScl(0.01)
  triangleGroup:linkOpacityAttrToChildren()
  self:insertAtCenter(triangleGroup)
  self.triangleGroup = triangleGroup

  local flipFactor = 1
  if (self:getX() > Screen.getWidthPart(0.5)) then
    flipFactor = -1
  end

  local defaultAngleDegrees = 5.0 * flipFactor
  local deltaAngleDegrees = 5.0 * flipFactor
  local duration = 1.5
  local rotationActions = {
    AHRotationTo.new(self.triangleGroup, duration, nil, defaultAngleDegrees + deltaAngleDegrees),
    AHRotationTo.new(self.triangleGroup, duration, nil, defaultAngleDegrees)
  }
  local rotateOnce = AHSequence.new(rotationActions)
  local rotate = AHLoop.new(rotateOnce)
  rotate:start()

  local radius = triangleHeight * 0.9
  local color = {1.0, 1.0, 1.0, 0.2}
  self.sideDisk1 = FilledCircle.new(radius, color)
  local offset = self:getWidthPart(0.4)
  local x = self:getWidthPart(0.5)
  local y = self:getHeightPart(0.5)
  self.sideDisk1DestinationX = x + offset * flipFactor
  self.sideDisk1DestinationY = y - offset
  self.sideDisk1:setLoc(x, y)
  self.sideDisk1:setScl(0.01)
  self:insert(self.sideDisk1)

  radius = triangleHeight * 0.6
  color = {1.0, 1.0, 1.0, 0.15}
  self.sideDisk2 = FilledCircle.new(radius, color)
  offset = self:getWidthPart(0.55)
  self.sideDisk2DestinationX = x - offset * flipFactor
  self.sideDisk2DestinationY = y + offset
  self.sideDisk2:setLoc(x, y)
  self.sideDisk2:setScl(0.01)
  self:insert(self.sideDisk2)

  local randomScale = MathUtils.round(MathUtils.getRandomFloat(0.25, 0.5), 2)
  self:setScl(randomScale)
end

-- Constructor ------------------------

function ArcticLargeStar.new(x, y)
  local self = ArcticStar.new(x, y)
  Mixin.mix(self, ArcticLargeStar)

  self.shouldFadeOutDisk = false

  createLayout(self)

  return self
end
