--- ArcticMessageOverlay
-- @author Created by Joe Poka on November 17, 2016.

class("ArcticMessageOverlay")

require("ArcticCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ArcticMessageOverlay.Z_INDEX = {
  darkBackground = 10,
  ui = 20,
  rectangleGradient = 30,
  tapLabel = 40
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticMessageOverlay:setSubmitLabeltext(text)
  self.tapLabel:setText(text)
end

function ArcticMessageOverlay:getTapToDismissGroup()
  return self
end

function ArcticMessageOverlay:getAnimationDuration()
  local duration = 0.3
  return duration
end

function ArcticMessageOverlay:getShowAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = self:getAnimationDuration()
  local darkenBackground = AHOpacityTo.new(self.darkBackground, duration, nil, 0.8)
  table.insert(actions, darkenBackground)

  local fadeInTapLabel = AHOpacityTo.new(self.tapLabelArea, duration, nil, 1.0)
  table.insert(actions, fadeInTapLabel)

  local destinationAlpha = 0.45
  local fadeInGradient = self.rectangleGradient:newOpacityToAction(duration, nil, destinationAlpha, destinationAlpha)
  table.insert(actions, fadeInGradient)

  local showAnimation = AHSpawn.new(actions)
  return showAnimation
end

function ArcticMessageOverlay:getHideAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = self:getAnimationDuration()
  local lightenBackground = AHOpacityTo.new(self.darkBackground, duration, nil, 0.0)
  table.insert(actions, lightenBackground)

  local fadeOutTapLabel = AHOpacityTo.new(self.tapLabelArea, duration, nil, 0.0)
  table.insert(actions, fadeOutTapLabel)

  local destinationAlpha = 0.0
  local fadeOutGradient = self.rectangleGradient:newOpacityToAction(duration, nil, destinationAlpha, destinationAlpha)
  table.insert(actions, fadeOutGradient)

  local hideAnimation = AHSpawn.new(actions)

  local remove = AHRemove.new(self)
  local hideAndRemove = AHSequence.new(hideAnimation, remove)
  return hideAndRemove
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local backgroundColor = {0.0, 0.0, 0.0, 1.0}
  self.darkBackground = FilledRectangle.new(Screen.width, Screen.height, backgroundColor)
  self.darkBackground:setOpacity(0.0)
  self:insertAtCenter(self.darkBackground, ArcticMessageOverlay.Z_INDEX.darkBackground)

  local tapWidth = self:getWidth()
  local tapHeight = self:getHeightPart(0.10)
  self.tapLabelArea = Group.new(tapWidth, tapHeight)
  self.tapLabelArea:setOpacity(0.0)
  self:insert(self.tapLabelArea, ArcticMessageOverlay.Z_INDEX.tapLabel)

  local continueString = Localizer.get("Misc.TAP_TO_CONTINUE")
  local continueFont = Font.new(FontFaces.DIN_OT_MEDIUM, 14.0)
  self.tapLabel = Label.new(continueString, continueFont)
  self.tapLabelArea:insertAtCenter(self.tapLabel)

  local width = self:getWidth()
  local height = self:getHeight()
  local beginColor = {1.0, 0.0, 0.0, 0.0}
  local endColor = {0.0, 0.0, 0.0, 0.0}
  local isHorizontal = true
  self.rectangleGradient = RectangleGradient.new(width, height, beginColor, endColor, isHorizontal)
  self:insertAtCenter(self.rectangleGradient, ArcticMessageOverlay.Z_INDEX.rectangleGradient)
  self.rectangleGradient:setBlendMode(MOAIProp2D.BLEND_ADD)

  local recursive = true
  self.tapLabelArea:linkOpacityAttrToChildren(recursive)

  local tapLabelAreaX = self:getWidthPart(0.5)
  local tapLabelAreaY = self.tapLabelArea:getHeightPart(0.5) + Screen.safeBottom
  self.tapLabelArea:setLoc(tapLabelAreaX, tapLabelAreaY)
end

-- Constructor ------------------------

function ArcticMessageOverlay.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ArcticMessageOverlay)

  createLayout(self)

  return self
end
