--- ArcticMessageOverlayAccidentalSum
-- @author Created by Joe Poka on November 17, 2016.

class("ArcticMessageOverlayAccidentalSum")

require("ArcticMessageOverlay")
require("ArcticCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticMessageOverlayAccidentalSum:getAnimationDuration()
  local duration = 0.3
  return duration
end

function ArcticMessageOverlayAccidentalSum:getShowAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = self:getAnimationDuration()
  for _, item in ipairs(self.items) do
    local fadeIn = AHOpacityTo.new(item, duration, nil, 1.0)
    table.insert(actions, fadeIn)
  end

  local showAnimation = AHSpawn.new(actions)
  return self.super.getShowAnimation(self, showAnimation)
end

function ArcticMessageOverlayAccidentalSum:getHideAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = self:getAnimationDuration()
  for _, item in ipairs(self.items) do
    local fadeOut = AHOpacityTo.new(item, duration, nil, 0.0)
    table.insert(actions, fadeOut)
  end

  local hideAnimation = AHSpawn.new(actions)
  return self.super.getHideAnimation(self, hideAnimation)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  self.items = {}

  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 16.0)
  local margin = self:getWidthPart(0.15)
  local width = self:getWidth() - 2.0 * margin
  local lineSpacing = 4

  local textAttributes = {
    font = font,
    color = ArcticCommon.COLORS.TEXT_WHITE
  }

  local text = Localizer.get("ENTERED_SUM")
  local textAreaTop = DynamicTextArea.new(text, width, textAttributes, nil, nil, nil, lineSpacing, true)
  self:insert(textAreaTop, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.items, textAreaTop)

  text = Localizer.get("TO_FIND_AVERAGE")
  local textAreaBottom = DynamicTextArea.new(text, width, textAttributes, nil, nil, nil, lineSpacing, true)
  self:insert(textAreaBottom, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.items, textAreaBottom)

  local sideLength = Screen.getWidthPart(0.525)
  height = 0.866 * sideLength
  local color = {1.0, 0.97, 0.84, 1.0}
  local dummyTriangle = FilledTriangle.new(0.0, 0.0, sideLength, 0.0, sideLength * 0.5, height, color)
  self:insert(dummyTriangle, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.items, dummyTriangle)

  for _, item in ipairs(self.items) do
    item:setOpacity(0.0)
  end

  local x = margin + textAreaTop:getWidthPart(0.5)
  local y = self:getHeightPart(0.35)
  textAreaTop:setLoc(x, y)

  y = y - textAreaBottom:getHeightPart(1.2)
  textAreaBottom:setLoc(x, y)

  x = self:getWidthPart(0.5)
  y = self:getHeightPart(0.65)
  dummyTriangle:setLoc(x, y)
end

-- Constructor ------------------------

function ArcticMessageOverlayAccidentalSum.new()
  local self = ArcticMessageOverlay.new()
  Mixin.mix(self, ArcticMessageOverlayAccidentalSum)

  createLayout(self)

  return self
end
