--- ArcticMessageOverlayBasicInformation
-- @author Created by Joe Poka on November 17, 2016.

class("ArcticMessageOverlayBasicInformation")

require("ArcticMessageOverlay")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DIMMED_ITEM_FULL_ALPHA = 0.7

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticMessageOverlayBasicInformation:getAnimationDuration()
  local duration = 0.3
  return duration
end

function ArcticMessageOverlayBasicInformation:getShowAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = self:getAnimationDuration()
  for _, item in ipairs(self.items) do
    local fadeIn = AHOpacityTo.new(item, duration, nil, 1.0)
    table.insert(actions, fadeIn)
  end

  for _, item in ipairs(self.dimItems) do
    local fadeIn = AHOpacityTo.new(item, duration, nil, DIMMED_ITEM_FULL_ALPHA)
    table.insert(actions, fadeIn)
  end

  local fadeInRule = AHOpacityTo.new(self.horizontalRule, duration, nil, 1.0)
  table.insert(actions, fadeInRule)

  local showAnimation = AHSpawn.new(actions)
  return self.super.getShowAnimation(self, showAnimation)
end

function ArcticMessageOverlayBasicInformation:getHideAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = self:getAnimationDuration()
  for _, item in ipairs(self.items) do
    local fadeOut = AHOpacityTo.new(item, duration, nil, 0.0)
    table.insert(actions, fadeOut)
  end

  for _, item in ipairs(self.dimItems) do
    local fadeOut = AHOpacityTo.new(item, duration, nil, 0.0)
    table.insert(actions, fadeOut)
  end

  local fadeOutRule = AHOpacityTo.new(self.horizontalRule, duration, nil, 0.0)
  table.insert(actions, fadeOutRule)

  local hideAnimation = AHSpawn.new(actions)
  return self.super.getHideAnimation(self, hideAnimation)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  self.items = {}
  self.dimItems = {}

  local font = Font.new(FontFaces.DIN_OT_LIGHT, 40.0)
  local aboutLabel = Label.new(Localizer.get("ABOUT"), font)
  aboutLabel:setColor(unpack(ArcticCommon.COLORS.TEXT_WHITE))
  self:insert(aboutLabel, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.items, aboutLabel)

  local averagesLabel = Label.new(Localizer.get("AVERAGES"), font)
  averagesLabel:setColor(unpack(ArcticCommon.COLORS.TEXT_WHITE))
  self:insert(averagesLabel, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.items, averagesLabel)

  local sumText = Localizer.get("FIRST_ADD")
  local sumFont = Font.new(FontFaces.DIN_OT_MEDIUM, 16.0)
  local margin = self:getWidthPart(0.1)
  local sumWidth = self:getWidth() - 2.0 * margin
  local sumLineSpacing = 4

  local sumTextAttributes = {
    font = sumFont,
    color = ArcticCommon.COLORS.TEXT_WHITE
  }

  local sumTextArea = DynamicTextArea.new(sumText, sumWidth, sumTextAttributes, nil, nil, nil, sumLineSpacing, true)
  self:insert(sumTextArea, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.dimItems, sumTextArea)

  local spacing = "    "
  local addStrings = {"1", "+", "2", "+", "6", "=", "9"}
  local addString = ""
  for _, s in ipairs(addStrings) do
    addString = addString .. s .. spacing
  end
  addString = StringUtils.trim(addString)

  local addFont = Font.new(FontFaces.DIN_OT_MEDIUM, 27.0)
  local addLabel = Label.new(addString, addFont)
  self:insert(addLabel, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.items, addLabel)

  local ruleWidth = self:getWidth() - 2.0 * margin
  local ruleHeight = self:getHeightPart(0.001)
  local ruleColor = {0.4, 0.4, 0.4, 1.0}
  self.horizontalRule = FilledRectangle.new(ruleWidth, ruleHeight, ruleColor)
  self:insert(self.horizontalRule, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.items, self.horizontalRule)

  local quotientText = Localizer.get("THEN_DIVIDE")
  local quotientWidth = sumWidth
  local quotientLineSpacing = sumLineSpacing
  local quotientTextAttributes = sumTextAttributes

  local quotientTextArea = DynamicTextArea.new(quotientText, quotientWidth, quotientTextAttributes, nil, nil, nil, quotientLineSpacing, true)
  quotientTextArea:setColor(unpack(ArcticCommon.COLORS.TEXT_WHITE))
  self:insert(quotientTextArea, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.dimItems, quotientTextArea)

  local divideStrings = {"9", "÷", "3", "=", "3"}
  local divideString = ""
  for _, s in ipairs(divideStrings) do
    divideString = divideString .. s .. spacing
  end
  divideString = StringUtils.trim(divideString)

  local divideFont = Font.new(FontFaces.DIN_OT_MEDIUM, 27.0)
  local divideLabel = Label.new(divideString, divideFont)
  self:insert(divideLabel, ArcticMessageOverlay.Z_INDEX.ui)
  table.insert(self.items, divideLabel)

  for _, item in ipairs(self.items) do
    item:setOpacity(0.0)
  end

  for _, item in ipairs(self.dimItems) do
    item:setOpacity(0.0)
  end

  local x = margin + aboutLabel:getWidthPart(0.5)
  local y = self:getHeightPart(0.825)
  aboutLabel:setLoc(x, y)

  x = margin + averagesLabel:getWidthPart(0.5)
  y = y - aboutLabel:getHeightPart(0.5) - averagesLabel:getHeightPart(0.45)
  averagesLabel:setLoc(x, y)

  x = margin + sumTextArea:getWidthPart(0.5)
  y = y - averagesLabel:getHeightPart(0.5) - sumTextArea:getHeightPart(0.9)
  sumTextArea:setLoc(x, y)

  x = margin + addLabel:getWidthPart(0.5)
  y = y - sumTextArea:getHeightPart(0.5) - addLabel:getHeightPart(0.95)
  addLabel:setLoc(x, y)

  x = self:getWidthPart(0.5)
  y = y - addLabel:getHeightPart(1.3)
  self.horizontalRule:setLoc(x, y)

  x = margin + quotientTextArea:getWidthPart(0.5)
  y = y - addLabel:getHeightPart(0.5) - quotientTextArea:getHeightPart(0.65)
  quotientTextArea:setLoc(x, y)

  x = margin + divideLabel:getWidthPart(0.5)
  y = y - quotientTextArea:getHeightPart(0.5) - divideLabel:getHeightPart(0.95)
  divideLabel:setLoc(x, y)
end

-- Constructor ------------------------

function ArcticMessageOverlayBasicInformation.new()
  local self = ArcticMessageOverlay.new()
  Mixin.mix(self, ArcticMessageOverlayBasicInformation)

  createLayout(self)

  return self
end
