--- ArcticMessageOverlayIncorrectAnswer
-- @author Created by Joe Poka on November 17, 2016.

class("ArcticMessageOverlayIncorrectAnswer")

require("ArcticIncorrectAnswerScreen")
require("ArcticIncorrectAnswerScreenIntro")
require("ArcticMessageOverlay")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getScreenOne(self, round, submittedAnswer, tapLearnMore)
  local screenOne = ArcticIncorrectAnswerScreenIntro.new(round, submittedAnswer, tapLearnMore)
  return screenOne
end

local function getScreenTwo(self, round, tapLeftArrow, tapRightArrow)
  local topText = Localizer.get("FIRST_FIND_SUM")

  local addStrings = {}
  local values = round.values
  local sum = TableUtils.sum(values)

  for _, value in ipairs(values) do
    table.insert(addStrings, tostring(value))
    table.insert(addStrings, "+")
  end
  table.remove(addStrings, #addStrings)
  table.insert(addStrings, "=")
  table.insert(addStrings, tostring(sum))

  local spacing = "  "
  local addString = ""
  for _, s in ipairs(addStrings) do
    addString = addString .. s .. spacing
  end
  addString = StringUtils.trim(addString)

  local showRightArrow = true
  local screen = ArcticIncorrectAnswerScreen.new(topText, addString, tapLeftArrow, tapRightArrow, showRightArrow)
  return screen
end

local function getScreenThree(self, round, tapLeftArrow, tapRightArrow)
  local topText = Localizer.get("NEXT_FIND_AVERAGE")

  local values = round.values
  local valuesCount = #values
  local sum = TableUtils.sum(values)

  local divideStrings = {
    tostring(sum),
    "÷",
    tostring(valuesCount),
    "=",
    tostring(round.answer)
  }

  local spacing = "   "
  local divideString = ""
  for _, s in ipairs(divideStrings) do
    divideString = divideString .. s .. spacing
  end
  divideString = StringUtils.trim(divideString)

  local showRightArrow = false
  local screen = ArcticIncorrectAnswerScreen.new(topText, divideString, tapLeftArrow, tapRightArrow, showRightArrow)
  return screen
end

local function transitionToScreen(self, destinationScreenIndex)
  if (self.ignoringNavigationTaps) then
    return
  end

  self.ignoringNavigationTaps = true

  local currentScreen = self.screens[self.currentScreenIndex]
  local destinationScreen = self.screens[destinationScreenIndex]

  local screenWidth = currentScreen:getWidth()
  local deltaX = screenWidth
  local animateLeft = (destinationScreenIndex > self.currentScreenIndex)
  if (animateLeft) then
    deltaX = deltaX * -1.0
  end

  local actions = {}
  local duration = 1.0
  for _, screen in ipairs(self.screens) do
    local ease = AHEaseExponentialOut.new()
    local animate = AHLocationXBy.new(screen, duration, ease, deltaX)
    table.insert(actions, animate)
  end

  local exitAnimation = currentScreen:getExitAnimation(duration, animateLeft)
  local enterAnimation = destinationScreen:getEnterAnimation(duration, animateLeft)
  table.insert(actions, exitAnimation)
  table.insert(actions, enterAnimation)

  local wantToShowInstructions = (destinationScreenIndex == 1)
  local showNumbersGroup = wantToShowInstructions
  local toggleInstructions = function()
    self.toggleInstructionsCallback(wantToShowInstructions, showNumbersGroup)
  end
  table.insert(actions, toggleInstructions)

  local animate = AHSpawn.new(actions)
  local onAnimationComplete = function()
    self.currentScreenIndex = destinationScreenIndex
    self.ignoringNavigationTaps = false
  end

  local transition = AHSequence.new(animate, onAnimationComplete)
  transition:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticMessageOverlayIncorrectAnswer:getTapToDismissGroup()
  return self.tapLabelArea
end

function ArcticMessageOverlayIncorrectAnswer:getAnimationDuration()
  local duration = 0.3
  return duration
end

function ArcticMessageOverlayIncorrectAnswer:getShowAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = self:getAnimationDuration()
  for _, item in ipairs(self.items) do
    local fadeIn = AHOpacityTo.new(item, duration, nil, 1.0)
    table.insert(actions, fadeIn)
  end

  local showAnimation = AHSpawn.new(actions)
  return self.super.getShowAnimation(self, showAnimation)
end

function ArcticMessageOverlayIncorrectAnswer:getHideAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = self:getAnimationDuration()
  for _, item in ipairs(self.items) do
    local fadeOut = AHOpacityTo.new(item, duration, nil, 0.0)
    table.insert(actions, fadeOut)
  end

  for _, screen in ipairs(self.screens) do
    if (screen.getHideAnimation ~= nil) then
      local hide = screen:getHideAnimation(duration)
      table.insert(actions, hide)
    end
  end

  local showInstructions = function()
    local showInstructions = true
    local showNumbersGroup = false
    self.toggleInstructionsCallback(showInstructions, showNumbersGroup)
  end
  table.insert(actions, showInstructions)

  local hideAnimation = AHSpawn.new(actions)
  return self.super.getHideAnimation(self, hideAnimation)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, round, submittedAnswer)
  self.items = {}

  local tapLearnMore = function()
    local destinationScreenIndex = 2
    transitionToScreen(self, destinationScreenIndex)
  end

  local tapLeftArrow = function()
    local destinationScreenIndex = self.currentScreenIndex - 1
    transitionToScreen(self, destinationScreenIndex)
  end

  local tapRightArrow = function()
    local destinationScreenIndex = self.currentScreenIndex + 1
    transitionToScreen(self, destinationScreenIndex)
  end

  local screenOne = getScreenOne(self, round, submittedAnswer, tapLearnMore)
  local screenTwo = getScreenTwo(self, round, tapLeftArrow, tapRightArrow)
  local screenThree = getScreenThree(self, round, tapLeftArrow, tapRightArrow)

  self.currentScreenIndex = 1
  self.screens = {
    screenOne,
    screenTwo,
    screenThree
  }

  local screenWidth = screenOne:getWidth()
  local x = screenWidth * 0.5
  local y = screenOne:getHeightPart(0.5)

  for _, screen in ipairs(self.screens) do
    screen:setLoc(x, y)
    self:insert(screen, ArcticMessageOverlay.Z_INDEX.ui)
    table.insert(self.items, screen)
    local recursive = true
    screen:linkOpacityAttrToChildren(recursive)
    screen:setOpacity(0.0)
    x = x + screenWidth
  end

  local alternativeSubmitText = Localizer.get("Misc.RETURN_TO_GAME")
  self:setSubmitLabeltext(alternativeSubmitText)
end

-- Constructor ------------------------

function ArcticMessageOverlayIncorrectAnswer.new(round, submittedAnswer, toggleInstructionsCallback)
  local self = ArcticMessageOverlay.new()
  Mixin.mix(self, ArcticMessageOverlayIncorrectAnswer)

  self.toggleInstructionsCallback = toggleInstructionsCallback

  createLayout(self, round, submittedAnswer)

  return self
end
