--- ArcticOutro
-- @author Created by Joe Poka on December 1, 2016.

class("ArcticOutro")

require("ArcticShinyRectangles")
require("graphics.primitives.Checkmark")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_INDEX = {
  shinyRectangleGroups = 10,
  checkmark = 20,
  triangle = 30
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticOutro.getOutroAnimationDuration()
  return 4.0
end

function ArcticOutro:getOutroAnimation()
  local rectangleGroupCount = 6
  local defaultWidth = self:getWidthPart(0.75)
  local defaultHeight = self:getHeightPart(0.085)
  local leftStartX = -2.0 * defaultWidth
  local rightStartX = self:getWidth() + 3.0 * defaultWidth
  local duration = ArcticOutro.getOutroAnimationDuration()
  local delay = 0.0
  local deltaDelay = 0.125
  local moveRectangleGroups = {}

  local defaultRectangleCount = 16
  local rectangleCount = defaultRectangleCount

  local defaultTriangleHeight = self:getHeight()
  local triangleHeight = defaultTriangleHeight

  local defaultY = self:getHeightPart(0.5)
  local maximumY = self:getHeightPart(0.625)
  local y = defaultY

  local minimumAlpha = 0.0
  local maximumAlpha = 1.0

  for i = 1, rectangleGroupCount do
    local x = leftStartX
    local destinationX = rightStartX
    local leftSide = true

    if (i % 2 == 0) then
      x = rightStartX
      destinationX = leftStartX
      leftSide = false
    end

    local width = defaultWidth
    local height = defaultHeight / i

    local rectangleGroup = ArcticShinyRectangles.new(triangleHeight, rectangleCount, leftSide, width, height, minimumAlpha, maximumAlpha)
    rectangleGroup:setLoc(x, y)
    self:insert(rectangleGroup, Z_INDEX.shinyRectangleGroups)

    local ease = AHEaseExponentialOut.new()
    local moveRectangleGroup = AHLocationXTo.new(rectangleGroup, duration, ease, destinationX)
    local delayedMoveRectangleGroup = AHDelayed.new(delay, moveRectangleGroup)
    table.insert(moveRectangleGroups, delayedMoveRectangleGroup)

    delay = delay + deltaDelay
    triangleHeight = triangleHeight * 0.85
    rectangleCount = math.max(rectangleCount - 3, 7)
    y = math.min(y * 1.2, maximumY)
  end

  local checkmarkWidth = Screen.getWidthPart(0.4)
  local shortSegmentProportion = 0.425
  local thicknessProportion = 0.25
  local checkmarkColor = {1.0, 1.0, 1.0, 1.0}
  local checkmark = FilledCheckmark.new(checkmarkWidth, shortSegmentProportion, thicknessProportion, checkmarkColor)
  checkmark:setOpacity(0.0)
  local x = self:getWidthPart(0.5)
  checkmark:setLoc(x, maximumY)
  self:insert(checkmark, Z_INDEX.checkmark)

  local fadeInCheckmark = AHOpacityTo.new(checkmark, duration * 0.2, nil, 1.0)
  local delayedFadeInCheckmark = AHDelayed.new(duration * 0.185, fadeInCheckmark)

  local fadeInGradient = self.rectangleGradient:newOpacityToAction(duration * 0.2, nil, 0.6, 0.6)

  local animation = AHSpawn.new(moveRectangleGroups, delayedFadeInCheckmark, fadeInGradient)
  return animation
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()
  local beginColor = {0.54, 1.0, 0.0, 0.0}
  local endColor = {1.0, 0.38, 0.0, 0.0}
  local isHorizontal = true
  self.rectangleGradient = RectangleGradient.new(width, height, beginColor, endColor, isHorizontal)
  self:insertAtCenter(self.rectangleGradient)
  self.rectangleGradient:setBlendMode(MOAIProp2D.BLEND_ADD)
end

-- Constructor ------------------------

function ArcticOutro.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ArcticOutro)

  createLayout(self)

  return self
end
