--- ArcticRadialGradientTexture
-- @author Created by Joe Poka on August 1, 2017.

class("ArcticRadialGradientTexture")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function populateImage(imageDataSource)
  local width, height = imageDataSource:getSize()
  local edgeColor = {0.02, 0.0, 0.16, 0.65}
  local centerColor = {0.0, 0.0, 0.0, 0.0}
  local centerX = width * 0.5
  local centerY = height
  local dx = centerX
  local dy = height
  local maximumRadialDistance = math.sqrt(dx * dx + dy * dy)

  -- row/column is (0, 0) at upper-left
  for row = 0, height do
    local minimumRadialDistance = 1.0 + (row / height) * maximumRadialDistance
    local deltaRadialDistance = maximumRadialDistance - minimumRadialDistance
    for column = 0, width do
      local radialDistance = MathUtils.distance(centerX, centerY, column, row)
      local radialDistance01 = (radialDistance - minimumRadialDistance) / deltaRadialDistance
      radialDistance01 = MathUtils.clamp(radialDistance01, 0.0, 1.0)
      local radialValue = math.pow(radialDistance01, 3.0)
      local resultColor = {0.0, 0.0, 0.0, 0.0}
      for i = 1, 4 do
        resultColor[i] = (1.0 - radialValue) * centerColor[i] + radialValue * edgeColor[i]
      end
      imageDataSource:setRGBA(column, row, unpack(resultColor))
    end
  end
end

local function getDataSource()
  local rowCount = math.floor(Screen.height / 8)
  local columnCount = math.floor(Screen.width / 2)

  local dataSource = MOAIImage.new()
  dataSource:init(columnCount, rowCount)
  populateImage(dataSource)
  return dataSource
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ArcticRadialGradientTexture.new()
  local self = MOAITexture.new()
  Mixin.mix(self, ArcticRadialGradientTexture)

  local dataSource = getDataSource()
  self:load(dataSource)

  return self
end
