--- ArcticScrollingParticles
-- @author Created by Joe Poka on December 14, 2016.

class("ArcticScrollingParticles")

require("ArcticMultiTextureBlend")
require("ArcticShinyRectangles")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createBackgroundGroup(self, sideLength, triangleHeight)
  self.particleGroups = {}
  local groupCount = 6
  local rectangleCount = 10
  local backgroundGroup = Group.newFullScreen()
  local x = 0.0
  local y = backgroundGroup:getHeightPart(0.5)
  local leftSide = true
  local width = self:getWidthPart(0.4)
  local height = self:getHeightPart(0.0125)
  local minimumAlpha = 0.75
  local maximumAlpha = 0.925
  local color = {1.0, 1.0, 1.0}

  for i = 1, groupCount do
    local particles = ArcticShinyRectangles.new(triangleHeight, rectangleCount, leftSide, width, height, minimumAlpha, maximumAlpha, color)
    particles:setLoc(x, y)
    backgroundGroup:insert(particles)
    table.insert(self.particleGroups, particles)
  end

  return backgroundGroup
end

local function animateParticles(self, sideLength)
  local destinationX = self:getWidth() + sideLength
  local resetX = 0.0
  local fastMoveDuration = 1.5
  local slowMoveDuration = 4.0
  local delay = 0.0
  local deltaDelay = 1.0
  local actions = {}

  for i = 1, #self.particleGroups do
    local particleGroup = self.particleGroups[i]
    local duration = slowMoveDuration
    local movingFast = (i % 2 == 0)
    if (movingFast) then
      duration = fastMoveDuration
    end
    local move = AHLocationXTo.new(particleGroup, duration, nil, destinationX)
    local reset = function()
      local _, y = particleGroup:getLoc()
      particleGroup:setLoc(resetX, y)
    end

    local moveOnce = AHSequence.new(move, reset)
    local moveLoop = AHLoop.new(moveOnce)
    local delayedMoveLoop = AHDelayed.new(delay, moveLoop)
    table.insert(actions, delayedMoveLoop)
    delay = delay + deltaDelay
  end

  local animation = AHSpawn.new(actions)

  local introDelay = 1.0
  local delayedAnimation = AHDelayed.new(introDelay, animation)
  delayedAnimation:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticScrollingParticles:show()
  for _, particleGroup in ipairs(self.particleGroups) do
    particleGroup:setDefaultAlpha()
  end
end

function ArcticScrollingParticles:hide()
  for _, particleGroup in ipairs(self.particleGroups) do
    particleGroup:setAlpha(0.0)
  end
end

function ArcticScrollingParticles:reset()
  for _, particleGroup in ipairs(self.particleGroups) do
    local _, y = particleGroup:getLoc()
    particleGroup:setLoc(0.0, y)
  end
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local sideLength = self:getHeightPart(0.3)
  local triangleHeight = sideLength * 0.866

  local backgroundGroup = createBackgroundGroup(self, sideLength, triangleHeight)
  local maskGroup = Group.newFullScreen()

  local maskColor = {1.0, 1.0, 1.0, 1.0}
  local maskTriangle = FilledTriangle.new(0.0, 0.0, sideLength, 0.0, sideLength * 0.5, triangleHeight, maskColor)
  maskGroup:insertAtCenter(maskTriangle)

  local maskedObject = ArcticMultiTextureBlend.new(backgroundGroup, maskGroup)
  local x, y = maskedObject:getDims()
  maskedObject:setLoc(x, y)
  self:insert(maskedObject)

  animateParticles(self, sideLength)
end

-- Constructor ------------------------

function ArcticScrollingParticles.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ArcticScrollingParticles)

  createLayout(self)

  return self
end
