--- ArcticShake
-- @author Created by Joe Poka on December 16, 2016.

class("ArcticShake")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticShake.getShakeAnimation(groupToShake, shakingAdjustments)
  shakingAdjustments = shakingAdjustments or {}

  local minimumDelta = shakingAdjustments.minimumDelta or Screen.getWidthPart(0.0001)
  local maximumDelta = shakingAdjustments.maximumDelta or Screen.getWidthPart(0.01)
  local deltaDifference = maximumDelta - minimumDelta

  local minimumDuration = 0.01
  local maximumDuration = 0.02
  local deltaDuration = maximumDuration - minimumDuration

  local shakeCount = shakingAdjustments.shakeCount or 12
  local shakeParameters = {}

  local upperBound = shakeCount / 2
  for i = 1, upperBound do
    local fraction = i / upperBound
    local parameters = {
      delta = minimumDelta + deltaDifference * fraction,
      duration = minimumDuration + deltaDuration * fraction
    }
    table.insert(shakeParameters, parameters)
  end

  for i = upperBound, 1, -1 do
    local fraction = i / upperBound
    local parameters = {
      delta = minimumDelta + deltaDifference * fraction,
      duration = minimumDuration + deltaDuration * fraction
    }
    table.insert(shakeParameters, parameters)
  end

  local actions = {}
  local useRandomSigns = shakingAdjustments.useRandomSigns

  for _, parameters in ipairs(shakeParameters) do
    local delta = parameters.delta
    local duration = parameters.duration
    local dx = delta
    local dy = delta

    if (useRandomSigns) then
      dx = delta * MathUtils.randomSign()
      dy = delta * MathUtils.randomSign()
    end

    local moveOut = AHLocationBy.new(groupToShake, duration * 0.5, nil, dx, dy)
    local moveBack = AHLocationBy.new(groupToShake, duration * 0.5, nil, -dx, -dy)
    table.insert(actions, moveOut)
    table.insert(actions, moveBack)
  end

  local shake = AHSequence.new(actions)
  return shake
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------
