--- ArcticStar
-- @author Created by Joe Poka on November 22, 2016.

class("ArcticStar")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticStar:getAppearAnimation(animation)
  local actions = {}

  if (animation ~= nil) then
    table.insert(actions, animation)
  end

  local duration = 0.25
  local scaleDisk = AHScaleTo.new(self.centerDisk, duration, nil, 1.0)
  local scaleDiamond = AHScaleTo.new(self.diamond, duration, nil, 1.0)
  local playAppearSound = function() Sound.playSoundEffect("star_lands") end
  table.insert(actions, scaleDisk)
  table.insert(actions, scaleDiamond)
  table.insert(actions, playAppearSound)
  local scale = AHSpawn.new(actions)

  local diskOpacity = 0.1
  if (self.shouldFadeOutDisk) then
    diskOpacity = 0.0
  end

  local fadeOutDisk = AHOpacityTo.new(self.centerDisk, duration * 3.0, AHEaseSimpleOut.new(), diskOpacity)

  local appearAnimation = AHSequence.new(scale, fadeOutDisk)
  return appearAnimation
end

function ArcticStar:getPulseAnimation(randomPulseDuration, additionalAction)
  randomPulseDuration = randomPulseDuration or math.random(3, 5)
  additionalAction = additionalAction or AHDoNothing.new()
  local ease = AHEaseSineInOut.new()

  local pulseAnimation = AHLoop.new(
    AHSpawn.new(
      AHSequence.new(
        AHOpacityTo.new(self.diamond, randomPulseDuration, ease, 0.25),
        AHOpacityTo.new(self.diamond, randomPulseDuration, ease, 1.0)
      ),
      AHSequence.new(
        self.glow:newOpacityToAction(randomPulseDuration, ease, 0.0, 0.0),
        self.glow:newOpacityToAction(randomPulseDuration, ease, self.originalCenterOpacity, self.originalEdgeOpacity)
      ),
      additionalAction
    )
  )

  return pulseAnimation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local radius = self:getHeightPart(0.4)
  local color = {1.0, 1.0, 1.0, 0.25}
  self.centerDisk = FilledCircle.new(radius, color)
  self.centerDisk:setScl(0.01)
  self:insertAtCenter(self.centerDisk)

  local length = self:getWidthPart(0.07)
  color = {1.0, 1.0, 1.0, 1.0}
  self.diamond = FilledRectangle.new(length, length, color)
  self.diamond:setRot(45.0)
  self.diamond:setScl(0.01)
  self:insertAtCenter(self.diamond, 1111)

  local glowRadius = self:getHeightPart(0.2)
  local shouldGetHighRange = math.random() >= 0.9
  local randomCenterOpacity = MathUtils.round(MathUtils.getRandomFloat(0.15, 0.3), 2)

  if (shouldGetHighRange) then
    randomCenterOpacity = MathUtils.round(MathUtils.getRandomFloat(0.4, 0.6), 2)
  end

  local edgeOpacity = math.max(0, randomCenterOpacity - 0.25)
  local centerColor = {1.0, 1.0, 1.0, randomCenterOpacity}
  local edgeColor = {1.0, 1.0, 1.0, edgeOpacity}
  local blurMargin = glowRadius / 2
  self.glow = CircleGradient.new(glowRadius, centerColor, edgeColor, blurMargin)
  self:insertAtCenter(self.glow, 1112)
  self.originalCenterOpacity = randomCenterOpacity
  self.originalEdgeOpacity = edgeOpacity
  local randomScale = MathUtils.round(MathUtils.getRandomFloat(0.5, 1.0), 2)
  self:setScl(randomScale)
end

-- Constructor ------------------------

function ArcticStar.new(x, y)
  local width = Screen.getWidthPart(0.2)
  local self = Group.new(width, width)
  Mixin.mix(self, ArcticStar)

  self.shouldFadeOutDisk = true
  self:setLoc(x, y)

  createLayout(self)

  return self
end
