--- ArcticStarsGroup
-- @author Created by Joe Poka on November 21, 2016.

class("ArcticStarsGroup")

require("ArcticStar")
require("ArcticLargeStar")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local STAR_LOCATIONS = {
  -- LEFT = 1
  {
    { x = Screen.getWidthPart(0.01), y = Screen.getHeightPart(0.82)},
    { x = Screen.getWidthPart(0.03), y = Screen.getHeightPart(0.86)},
    { x = Screen.getWidthPart(0.03), y = Screen.getHeightPart(0.91)},
    { x = Screen.getWidthPart(0.15), y = Screen.getHeightPart(0.91)},
    { x = Screen.getWidthPart(0.23), y = Screen.getHeightPart(0.77)},
    { x = Screen.getWidthPart(0.24), y = Screen.getHeightPart(0.90)}
  },
  -- MIDDLE = 2
  {
    { x = Screen.getWidthPart(0.28), y = Screen.getHeightPart(0.97)},
    { x = Screen.getWidthPart(0.38), y = Screen.getHeightPart(0.95)},
    { x = Screen.getWidthPart(0.48), y = Screen.getHeightPart(0.98)},
    { x = Screen.getWidthPart(0.57), y = Screen.getHeightPart(0.96)},
    { x = Screen.getWidthPart(0.63), y = Screen.getHeightPart(0.97)},
    { x = Screen.getWidthPart(0.77), y = Screen.getHeightPart(0.97)}
  },
  -- RIGHT = 3
  {
    { x = Screen.getWidthPart(0.79), y = Screen.getHeightPart(0.91)},
    { x = Screen.getWidthPart(0.90), y = Screen.getHeightPart(0.91)},
    { x = Screen.getWidthPart(0.92), y = Screen.getHeightPart(0.85)},
    { x = Screen.getWidthPart(0.93), y = Screen.getHeightPart(0.79)},
    { x = Screen.getWidthPart(0.96), y = Screen.getHeightPart(0.75)},
    { x = Screen.getWidthPart(0.98), y = Screen.getHeightPart(0.88)}
  }
}

local SMALL_STAR_LOCATIONS = {
  { x = Screen.getWidthPart(0.07), y = Screen.getHeightPart(0.92)},
  { x = Screen.getWidthPart(0.14), y = Screen.getHeightPart(0.87)},
  { x = Screen.getWidthPart(0.23), y = Screen.getHeightPart(0.93)},
  { x = Screen.getWidthPart(0.23), y = Screen.getHeightPart(0.83)},
  { x = Screen.getWidthPart(0.31), y = Screen.getHeightPart(0.85)},
  { x = Screen.getWidthPart(0.39), y = Screen.getHeightPart(0.95)},
  { x = Screen.getWidthPart(0.50), y = Screen.getHeightPart(0.83)},
  { x = Screen.getWidthPart(0.62), y = Screen.getHeightPart(0.82)},
  { x = Screen.getWidthPart(0.72), y = Screen.getHeightPart(0.94)},
  { x = Screen.getWidthPart(0.87), y = Screen.getHeightPart(0.92)},
  { x = Screen.getWidthPart(0.90), y = Screen.getHeightPart(0.86)},
  { x = Screen.getWidthPart(0.44), y = Screen.getHeightPart(0.89)},
  { x = Screen.getWidthPart(0.54), y = Screen.getHeightPart(0.89)},
  { x = Screen.getWidthPart(0.44), y = Screen.getHeightPart(0.77)},
  { x = Screen.getWidthPart(0.53), y = Screen.getHeightPart(0.78)}
}

local LAST_RESORT_STAR_LOCATION = TableUtils.deepCopy(STAR_LOCATIONS[3][#STAR_LOCATIONS[3]])

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createSmallStarLayer(self)
  local durationPerStar = ArcticConfiguration.GAME_DURATION / #SMALL_STAR_LOCATIONS
  local starsFadeInAction = AHSequence.new()
  local smallStarLayer = Group.newFullScreen()
  local white = {1.0, 1.0, 1.0}
  for _, point in ipairs(SMALL_STAR_LOCATIONS) do
    local width = math.random(1, 2) / Host.contentScale
    local star = FilledRectangle.new(width, width, white)
    star:setOpacity(0.0)
    star:setLoc(point.x, point.y)
    smallStarLayer:insert(star)

    starsFadeInAction:addAction(
      AHOpacityTo.new(star, durationPerStar, nil, 1.0)
    )
  end

  starsFadeInAction:start()
  self:insertAtCenter(smallStarLayer)
  self.smallStarLayer = smallStarLayer
  self.smallStarsFadeInAction = starsFadeInAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticStarsGroup:getNextStarLocation()
  local currentStarArea = STAR_LOCATIONS[self.currentStarAreaIndex]

  if (#currentStarArea == 0) then
    return LAST_RESORT_STAR_LOCATION.x, LAST_RESORT_STAR_LOCATION.y
  end

  local doRemove = true
  local nextStarLocation = TableUtils.getRandomElement(currentStarArea, doRemove)

  self.currentStarAreaIndex = self.currentStarAreaIndex + 1
  if (self.currentStarAreaIndex > #STAR_LOCATIONS) then
      self.currentStarAreaIndex = 1
  end

  return nextStarLocation.x, nextStarLocation.y
end

function ArcticStarsGroup:getAddStarAnimation(x, y)
  local starType = self.newStarCycle[self.newStarIndex]
  self.newStarIndex = self.newStarIndex + 1
  if (self.newStarIndex > #self.newStarCycle) then
      self.newStarIndex = 1
  end

  local star = starType.new(x, y)
  local actions = {}

  local addStar = function()
    self:insert(star)
  end
  table.insert(actions, addStar)

  local appearAnimation = star:getAppearAnimation()
  table.insert(actions, appearAnimation)

  local pulseAction = star:getPulseAnimation()
  pulseAction:start()

  local animation = AHSpawn.new(actions)
  return animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  createSmallStarLayer(self)
end

-- Constructor ------------------------

function ArcticStarsGroup.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ArcticStarsGroup)

  self.newStarCycle = {
    ArcticStar,
    ArcticStar,
    ArcticLargeStar,
    ArcticStar
  }

  self.newStarIndex = 1
  self.currentStarAreaIndex = 1

  createLayout(self)

  return self
end
