--- ArcticTrees
-- @author Created by Joe Poka on November 23, 2016.

class("ArcticTrees")

require("ArcticShake")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INTRO_OUTER_TREE_COLOR = {0.35, 0.26, 0.3, 1.0}
local INTRO_INNER_TREE_COLOR = {0.47, 0.34, 0.4, 1.0}
local DEFAULT_OUTER_TREE_COLOR = {0.07, 0.02, 0.03, 1.0}
local DEFAULT_INNER_TREE_COLOR = {0.22, 0.1, 0.13, 1.0}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticTrees:getShakingAnimation()
  local shake = ArcticShake.getShakeAnimation(self)

  local moveDuration = 0.8
  local moveUp = AHLocationYBy.new(self, moveDuration, AHEaseSimpleOut.new(), self.yOffset)

  local colorChanges = AHSpawn.new()
  for i = 1, #self.trees do
    local tree = self.trees[i]
    local destinationColor = DEFAULT_INNER_TREE_COLOR
    if (i > 2) then
      destinationColor = DEFAULT_OUTER_TREE_COLOR
    end
    local colorChange = AHColorTo.new(tree, moveDuration, nil, destinationColor)
    colorChanges:addAction(colorChange)
  end

  local shakeAndMove = AHSpawn.new(shake, moveUp, colorChanges)
  return shakeAndMove
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local defaultTreeWidth = self:getWidthPart(0.7)
  local defaultTreeHeight = self:getHeightPart(0.8)

  local trees = {
    {
      width = defaultTreeWidth * 0.5,
      height = defaultTreeHeight * 0.4,
      color = INTRO_INNER_TREE_COLOR,
      rotationDegrees = 0.0,
      position = {
        x = self:getWidthPart(0.31),
        y = self:getHeightPart(0.125)
      }
    },
    {
      width = defaultTreeWidth * 0.6,
      height = defaultTreeHeight * 0.6,
      color = INTRO_INNER_TREE_COLOR,
      rotationDegrees = 0.0,
      position = {
        x = self:getWidthPart(0.78),
        y = self:getHeightPart(0.125)
      }
    },
    {
      width = defaultTreeWidth,
      height = defaultTreeHeight,
      color = INTRO_OUTER_TREE_COLOR,
      rotationDegrees = -8.0,
      position = {
        x = self:getWidthPart(0.0175),
        y = self:getHeightPart(0.425)
      }
    },
    {
      width = defaultTreeWidth,
      height = defaultTreeHeight * 0.75,
      color = INTRO_OUTER_TREE_COLOR,
      rotationDegrees = 4.0,
      position = {
        x = self:getWidthPart(1.03),
        y = self:getHeightPart(0.2)
      }
    }
  }

  self.yOffset = self:getHeightPart(0.075)
  self.trees = {}

  for _, parameters in ipairs(trees) do
    local tree = FilledTriangle.new(
      0.0, 0.0,
      parameters.width, 0.0,
      parameters.width * 0.5, parameters.height,
      parameters.color
    )

    local x = parameters.position.x
    local y = parameters.position.y - self.yOffset
    tree:setLoc(x, y)
    tree:setRot(parameters.rotationDegrees)
    self:insert(tree)
    table.insert(self.trees, tree)
  end
end

-- Constructor ------------------------

function ArcticTrees.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ArcticTrees)

  createLayout(self)

  return self
end
