--- ArcticVerticalGradientTexture
-- @author Created by Joe Poka on July 31, 2017.

class("ArcticVerticalGradientTexture")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function populateImage(imageDataSource)
  local _, height = imageDataSource:getSize()
  local cutBottom = 0.55 * height
  local cutMiddle = 0.25 * height
  local bottomColor = {0.83, 0.0, 0.13, 1.0}
  local middleColor = {0.33, 0.12, 0.23, 1.0}
  local topColor = {0.0, 0.0, 0.0, 1.0}
  local periodFactor = 2.5

  -- row/column is (0, 0) at upper-left
  for row = 0, height do
    local resultColor = {1.0, 1.0, 1.0, 1.0}
    if (row >= cutBottom) then
      local f = MathUtils.clamp((row - cutBottom) / (height - cutBottom), 0.0, 1.0)
      resultColor = ColorUtils.makeColorBetweenTwoColors(middleColor, topColor, f)
    elseif (row >= cutMiddle) then
      local f = MathUtils.clamp((row - cutMiddle) / (cutBottom - cutMiddle), 0.0, 1.0)
      resultColor = ColorUtils.makeColorBetweenTwoColors(bottomColor, middleColor, f)
    end

    if (row >= cutMiddle) then
      local f = (row - cutMiddle) / (height - cutMiddle)
      local fNormalized = MathUtils.clamp(1.0 - f, 0.0, 1.0)
      local value = 1.0 - math.pow(fNormalized, 6.0)
      if (f < 0.3175) then
        value = 0.5 * (1.0 - math.sin(math.pi * periodFactor * f + math.pi * 0.5))
      end

      for i = 1, 4 do
        resultColor[i] = MathUtils.clamp(resultColor[i] + (1.0 - value), 0.0, 1.0)
      end
    end

    imageDataSource:setRGBA(0, row, unpack(resultColor))
  end
end

local function getDataSource()
  local rowCount = math.floor(Screen.height / 2)
  local columnCount = 1

  local dataSource = MOAIImage.new()
  dataSource:init(columnCount, rowCount)
  populateImage(dataSource)
  return dataSource
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ArcticVerticalGradientTexture.new()
  local self = MOAITexture.new()
  Mixin.mix(self, ArcticVerticalGradientTexture)

  local dataSource = getDataSource()
  self:load(dataSource)

  return self
end
