varying LOWP vec2 uvVaryingBackground;
varying LOWP vec2 uvVaryingSource;

uniform sampler2D backgroundSampler;
uniform sampler2D sourceSampler;

uniform LOWP float normalizationValueHorizontal;
uniform LOWP float normalizationValueVertical;

void main() {
	// normalizationValues help gradients start/end at screen bounds (instead of larger texture bounds)
	LOWP float horizontalDistance = clamp(uvVaryingSource.x / normalizationValueHorizontal, 0.0, 1.0);
	LOWP float verticalDistance = clamp((uvVaryingSource.y - normalizationValueVertical) / (1.0 - normalizationValueVertical), 0.0, 1.0);

	// brings shape of radial gradient closer to a circle (instead of a tall oval)
	verticalDistance = clamp(verticalDistance * 1.5 - 0.425, 0.0, 1.0);

	LOWP vec4 backgroundColor = texture2D(backgroundSampler, uvVaryingBackground);
	LOWP vec4 sourceColor = texture2D(sourceSampler, uvVaryingSource);

	LOWP float radialDistance = distance(vec2(horizontalDistance, verticalDistance), vec2(0.5, 0.5));

	// helps shrink the radial gradient so it fits inside the triangle
	const LOWP float maxRadialDistance = 0.2;
	radialDistance = clamp(radialDistance / maxRadialDistance, 0.0, 1.0);

	const LOWP vec4 innerColor = vec4(1.000, 0.953, 0.580, 0.1);
	const LOWP vec4 outerColor = vec4(1.000, 0.333, 0.333, 0.2);
	LOWP vec4 gradientColor = mix(innerColor, outerColor, radialDistance);

	LOWP float cutOffValue = backgroundColor.a * sourceColor.a;
	LOWP vec4 resultColor = mix(backgroundColor, gradientColor, backgroundColor.a) * cutOffValue;

	gl_FragColor = resultColor;
}
