--- ArcticValidator
-- @author Created by joepoka on November 15, 2016.
-- ConceptValidator for Arctic.

class("ArcticValidator")

require("validators.ConceptValidator")
require("widgets.number_pad.NumberPadInputField")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'answer',
  'value1',
  'value2',
  'tipType',
  'ruleNumber'
}

local OPTIONAL_KEYS = {
  'value3',
  'value4'
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function isEmpty(value)
  return ((value == nil) or (value == ""))
end

local function insertIfNotEmpty(destinationTable, value)
  if (not isEmpty(value)) then
    table.insert(destinationTable, value)
  end
end

local function validateAnswer(self, concept)
  local values = {
    concept.value1,
    concept.value2
  }
  insertIfNotEmpty(values, concept.value3)
  insertIfNotEmpty(values, concept.value4)

  local sum = TableUtils.sum(values)
  local count = #values

  local expectedAnswer = sum / count
  self:expectFloatEqual(concept.answer, expectedAnswer, "Calculated answer didn't match the value found in the concept's answer field")
end

-- Answers must end in .25, .5, .75, or have no decimal point
-- Check this by seeing if answer neatly can be divided into quarters
local function validateFractionalAnswer(self, concept)
  local answer = concept.answer
  local hasNoRemainder = ((answer % 0.25) == 0)

  self:expectTrue(hasNoRemainder, "Answer didn't contain either a whole value or a .25, .5, or .75 fractional value")
end


local function validateAnswerWidth(self, concept)
  local answerString = tostring(concept.answer)
  local answerLength = #answerString

  self:expectLessOrEqual(answerLength, NumberPadInputField.DEFAULT_MAX_DIGITS, "Answer length exceeds allowable string length")
end

local function validateNumberType(self, concept)
  self:expectType(concept.answer, "number", "answer", "Incorrect type. Expected a number but got a string, probably from a comma in the content.")
  self:expectType(concept.value1, "number", "value1", "Incorrect type. Expected a number but got a string, probably from a comma in the content.")
  self:expectType(concept.value2, "number", "value2", "Incorrect type. Expected a number but got a string, probably from a comma in the content.")

  if (not isEmpty(concept.value3)) then
    self:expectType(concept.value3, "number", "value3", "Incorrect type. Expected a number but got a string, probably from a comma in the content.")
  end

  if (not isEmpty(concept.value4)) then
    self:expectType(concept.value4, "number", "value4", "Incorrect type. Expected a number but got a string, probably from a comma in the content.")
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ArcticValidator:validateConcept(concept)
  local isOptional = false
  self:validateRequiredKeys(concept, REQUIRED_KEYS, isOptional)

  isOptional = true
  self:validateRequiredKeys(concept, OPTIONAL_KEYS, isOptional)

  validateAnswer(self, concept)
  validateFractionalAnswer(self, concept)
  validateAnswerWidth(self, concept)
  validateNumberType(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ArcticValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, ArcticValidator)

  return self
end
