class("BrevityBackground")
require("BrevityBackgroundLayout")
require("BrevityColors")

local MAX_X_ROAM = 10
local MAX_Y_ROAM = 10
local MIN_ROAM_DURATION = 3
local MAX_ROAM_DURATION = 8
local FRAME_WIDTH = 20

local function createLayout(self)
  local colorRectangleCollection = {}

  local layoutRectangleCollection = BrevityBackgroundLayout.getBackgroundLayoutRectangles()
  for _, layoutRectangle in pairs(layoutRectangleCollection) do
    local color = BrevityColors.getPaletteColor(layoutRectangle.colorIndex)
    local colorRectangle = FilledRectangle.new(layoutRectangle.width, layoutRectangle.height, color)
    colorRectangle:setLoc(layoutRectangle.x, layoutRectangle.y)
    colorRectangle:setRot(layoutRectangle.rotation)
    colorRectangle:setOpacity(layoutRectangle.opacity)
    self:insert(colorRectangle)
    table.insert(colorRectangleCollection, colorRectangle)

    colorRectangle.colorIndices = layoutRectangle.colorIndices

    local roamXDuration = math.random(MIN_ROAM_DURATION, MAX_ROAM_DURATION)
    local roamYDuration = math.random(MIN_ROAM_DURATION, MAX_ROAM_DURATION)

    local roamXAmount = math.random() * MAX_X_ROAM
    local roamXAction = AHLocationXBy.new(colorRectangle, roamXDuration, AHEaseSimpleInOut.new(), roamXAmount)
    local returnXAction = AHLocationXBy.new(colorRectangle, roamXDuration, AHEaseSimpleInOut.new(), -roamXAmount)

    local roamYAmount = math.random() * MAX_Y_ROAM
    local roamYAction = AHLocationYBy.new(colorRectangle, roamYDuration, AHEaseSimpleInOut.new(), roamYAmount)
    local returnYAction = AHLocationYBy.new(colorRectangle, roamYDuration, AHEaseSimpleInOut.new(), -roamYAmount)

    local roamXSequence = AHSequence.new(roamXAction, returnXAction)
    local roamYSequence = AHSequence.new(roamYAction, returnYAction)

    local roamAction = AHSpawn.new(roamXSequence, roamYSequence)
    AHLoop.new(roamAction):start()
  end


  self.colorRectangleCollection = colorRectangleCollection

  -- Needed for framing the bg outside of the bounds of the screen. This way when the screen
  -- shake effect is used, no colors show on the edges
  local bottomFrame = FilledRectangle.new(Screen.fullWidth, FRAME_WIDTH, WHITE)
  bottomFrame:setLoc(Screen.getWidthPart(0.5), FRAME_WIDTH * -0.5)
  self:insert(bottomFrame)

  local leftFrame = FilledRectangle.new(FRAME_WIDTH, Screen.fullHeight, WHITE)
  leftFrame:setLoc(FRAME_WIDTH * -0.5, Screen.getHeightPart(0.5))
  self:insert(leftFrame)

  local rightFrame = FilledRectangle.new(FRAME_WIDTH, Screen.fullHeight, WHITE)
  rightFrame:setLoc(Screen.fullWidth + FRAME_WIDTH * 0.5, Screen.getHeightPart(0.5))
  self:insert(rightFrame)
end

function BrevityBackground.new()
  local self = FilledRectangle.newFullScreen(WHITE)
  Mixin.mix(self, BrevityBackground)

  createLayout(self)

  return self
end
