--- BrevityConfiguration
-- @author created by Joe Poka on May 2, 2016

class("BrevityConfiguration")

require("data.keyframes.KeyframeSet")
require("data.generators.LearningFocusedRoundGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

BrevityConfiguration.GAME_DURATION = 50

BrevityConfiguration.TIME_REMAINING_TO_START_RESURFACING_MISTAKES =
  BrevityConfiguration.GAME_DURATION *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING

BrevityConfiguration.SPEED_BONUS_WINDOW = 2.0
BrevityConfiguration.SPEED_BONUS_DECAY_START = 0.0
BrevityConfiguration.INCORRECT_RESPONSE_TIME_PENALTY = 2
BrevityConfiguration.POINTS_PER_CORRECT_RESPONSE = 1000
BrevityConfiguration.POINTS_PER_FULL_SPEED_BONUS = 100
BrevityConfiguration.TUTORIAL_DELAY = 8

local CONCEPTS_CORRECT_TO_PASS = 1
local CONCEPTS_CORRECT_FOR_RANK_AND_DIFFICULTY_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {rankTwo = 5, rankThree = 7}),
  Keyframe.new(0.3, {rankTwo = 6, rankThree = 8})
)


local INCREASE_DIFFICULTY_AFTER_CORRECT_CONCEPTS_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {increaseDifficultyIndex = 7}),
  Keyframe.new(0.3, {increaseDifficultyIndex = 6})
)

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BrevityConfiguration.getCorrectConceptIndexToIncreaseDifficulty()
  local difficulty = GameSession.getDifficulty()
  local correctConceptIndexKeyframe = INCREASE_DIFFICULTY_AFTER_CORRECT_CONCEPTS_THRESHOLDS:getFrameAt(difficulty)
  return correctConceptIndexKeyframe.increaseDifficultyIndex
end


function BrevityConfiguration.getPassScore()
  local passScore = CONCEPTS_CORRECT_TO_PASS * BrevityConfiguration.POINTS_PER_CORRECT_RESPONSE

  return passScore
end

function BrevityConfiguration.getRankTwoScore()
  local difficulty = GameSession.getDifficulty()
  local rankRequirementKeyframe = CONCEPTS_CORRECT_FOR_RANK_AND_DIFFICULTY_THRESHOLDS:getFrameAt(difficulty)
  local rankTwoScore = rankRequirementKeyframe.rankTwo * BrevityConfiguration.POINTS_PER_CORRECT_RESPONSE

  return rankTwoScore
end

function BrevityConfiguration.getRankThreeScore()
  local difficulty = GameSession.getDifficulty()
  local rankRequirementKeyframe = CONCEPTS_CORRECT_FOR_RANK_AND_DIFFICULTY_THRESHOLDS:getFrameAt(difficulty)
  local rankThreeScore = rankRequirementKeyframe.rankThree * BrevityConfiguration.POINTS_PER_CORRECT_RESPONSE

  return rankThreeScore
end

function BrevityConfiguration.getDifficultyKeyFrames()
    return {
      {key = 0.0, valueProbabilities = {Basic = 1.0}},

      {key = 0.075, valueProbabilities = {Basic = 0.95,
                                          Easy = 0.05}},

      {key = 0.15, valueProbabilities = {Basic = 0.60,
                                         Easy = 0.40}},

      {key = 0.2, valueProbabilities = {Basic = 0.20,
                                        Easy = 0.70,
                                        Hard = 0.10}},

      {key = 0.25, valueProbabilities = {Easy = 0.70,
                                         Medium = 0.3}},

      {key = 0.3, valueProbabilities = {Easy = 0.35,
                                        Medium = 0.50,
                                        Hard = 0.15}},

      {key = 0.375, valueProbabilities = {Easy = 0.30,
                                          Medium = 0.40,
                                          Hard = 0.30}},

      {key = 0.45, valueProbabilities = {Easy = 0.15,
                                         Medium = 0.50,
                                         Hard = 0.35}},

      {key = 0.525, valueProbabilities = {Easy = 0.10,
                                          Medium = 0.40,
                                          Hard = 0.50}},

      {key = 0.6, valueProbabilities = {Easy = 0.07,
                                        Medium = 0.33,
                                        Hard = 0.60}},

      {key = 0.675, valueProbabilities = {Easy = 0.05,
                                          Medium = 0.25,
                                          Hard = 0.7}},

      {key = 0.75, valueProbabilities = {Easy = 0.05,
                                         Medium = 0.10,
                                         Hard = 0.85}},

      {key = 0.8, valueProbabilities = {Medium = 0.05,
                                        Hard = 0.85,
                                        Advanced = 0.1}},

      {key = 0.9, valueProbabilities = {Hard = 0.8,
                                        Advanced = 0.2}},

      {key = 1.0, valueProbabilities = {Hard = 0.7,
                                        Advanced = 0.3}},
    }
end
