--- BrevityRationaleOverlay
-- @author Created by andrewcousins on May 19, 2023.

class("BrevityRationaleOverlay")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HIGHLIGHT_TAG = "i"

local LABEL_ATTRIBUTES = {[HIGHLIGHT_TAG] = {font = Font.new(FontFaces.DIN_OT_MEDIUM, 20)}}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getTappedToContinueCallback(self)
  return function()
    self:hide()
    if self.rationaleDismissedCallback then
      self.rationaleDismissedCallback()
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BrevityRationaleOverlay:hide()
  self.tapOverlay:setTouchEnabled(false)
  local duration = 0.333
  local ease = AHEaseBezier.newFlat({0.5, 0.4, 0.0, 1.0})
  local overlayDestinationY = -self.overlay:getHeightPart(0.5)
  AHSequence.new(
    AHLocationYTo.new(self.overlay, duration, ease, overlayDestinationY),
    function() self.overlay:setOpacity(0) end
  ):start()
end

function BrevityRationaleOverlay:show(rationaleText, tangramColor, rationaleDismissedCallback)
  self.rationaleDismissedCallback = rationaleDismissedCallback
  self.overlay:setOpacity(1)

  local labelAttributes = TableUtils.deepCopy(LABEL_ATTRIBUTES)
  labelAttributes[HIGHLIGHT_TAG]['color'] = tangramColor
  self.rationaleSizeConstrainedTextArea:setLabelAttributes(labelAttributes)
  self.rationaleSizeConstrainedTextArea:setText(rationaleText)
  self.rationaleSizeConstrainedTextArea:rerender()

  local duration = 0.5
  local ease = AHEaseBezier.newFlat({0.5, 0.4, 0.0, 1.0})
  local overlayDestinationY = self.overlay:getHeightPart(0.5)
  AHSequence.new(
    AHLocationYTo.new(self.overlay, duration, ease, overlayDestinationY),
    function() self.tapOverlay:setTouchEnabled(true) end
  ):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, overlayHeight)
  local overlay = FilledRectangle.new(self:getWidth(), overlayHeight, BrevityColors.getRationaleColor())
  self:insertAtBottom(overlay)
  overlay:offsetY(overlay:getHeightPart(-0.5))
  overlay:setOpacity(0)
  self.overlay = overlay

  local tapToContinueFont = Font.new(FontFaces.DIN_OT_MEDIUM, 12)
  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), tapToContinueFont)
  tapToContinueLabel:setColor(0.8, 0.8, 0.8)
  overlay:insertAtBottom(tapToContinueLabel)
  tapToContinueLabel:offsetY(Screen.safeBottom + self:getHeightPart(0.07))

  local rationaleBuffer = overlayHeight * 0.1
  local rationaleWidth = self:getWidth() - (rationaleBuffer * 2)
  local rationaleMaxHeight = overlayHeight - tapToContinueLabel:getYTop() - (rationaleBuffer * 2)
  local baseAttributes = {font = Font.new(FontFaces.DIN_OT_MEDIUM, 20)}
  local labelAttributes = LABEL_ATTRIBUTES
  local minmumFontSize = 10
  local rationaleSizeConstrainedTextArea = SizeConstrainedTextArea.new(
    "",
    rationaleWidth,
    rationaleMaxHeight,
    baseAttributes,
    labelAttributes,
    minimumFontSize)
  rationaleSizeConstrainedTextArea:setAnchorPointTop()
  self.overlay:insertAtTop(rationaleSizeConstrainedTextArea)
  rationaleSizeConstrainedTextArea:offsetY(-rationaleBuffer)
  self.rationaleSizeConstrainedTextArea = rationaleSizeConstrainedTextArea
end

-- Constructor ------------------------

function BrevityRationaleOverlay.new(overlayHeight)
  local self = Group.newFullScreen()
  Mixin.mix(self, BrevityRationaleOverlay)

  createLayout(self, overlayHeight)

  self.tapOverlay = Button.addVirtualButton(self, nil, nil, nil, getTappedToContinueCallback(self))
  self.tapOverlay:setTouchEnabled(false)

  return self
end
