--- BrevityRoundGenerator
-- @author created by Joe Poka on May 2, 2016

module("BrevityRoundGenerator", package.seeall)

require("data.generators.Generator")
require("BrevityData")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_UNIQUE_CONCEPT_TYPE_GENERATION_ATTEMPTS = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getFormattedRoundDataFromConcept(concept)

  local formattedRoundData = {}
  formattedRoundData.identifier = concept.identifier
  formattedRoundData.rawSentence = concept.sentence
  formattedRoundData.rationale = concept.rationale

  return formattedRoundData
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BrevityRoundGenerator:reset()

end

function BrevityRoundGenerator:generateNextValue()
  local concept = self.conceptGenerator:next()
  local attemptIndex = 0
  local conceptType = concept.conceptType

  while(self.seenConceptTypes[conceptType] and attemptIndex < MAX_UNIQUE_CONCEPT_TYPE_GENERATION_ATTEMPTS) do
    attemptIndex = attemptIndex + 1

    concept = self.conceptGenerator:next()
    conceptType = concept.conceptType
  end
  self.seenConceptTypes[conceptType] = true

  local roundData = getFormattedRoundDataFromConcept(concept)
  return roundData
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function BrevityRoundGenerator.new()
  local self = Generator.new()
  Mixin.mix(self, BrevityRoundGenerator)

  self.conceptGenerator = GameSession.conceptGenerator
  self.seenConceptTypes = {}

  return self
end
