--- BrevityTextAnimationBuilder
-- @author created by Joe Poka on May 2, 2016

class("BrevityTextAnimationBuilder")

require("text.dynamic.TextAnimationBuilder")

--- TODO: In a subsequent pass, make the private functions in TextAnimationBuilder
-- protected instead to remove the duplicate code here. That way, only the specific functions
-- that need to be overrided will need to be contained here.

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MOVE_DURATION = 1.0
local SHOW_DURATION = 1.0
local HIDE_DURATION = 0.5
local ADJUST_HEIGHT_DURATION = 1.0
local LOCATION_COMPARISON_DECIMAL_PLACES = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function didItemsLocationChange(self)
  return self.didItemsXChange or self.didItemsYChange
end

local function setDurations(self)
  self:setMoveDuration(MOVE_DURATION)
  self:setShowDuration(SHOW_DURATION)
  self:setHideDuration(HIDE_DURATION)
  self:setAdjustHeightDuration(ADJUST_HEIGHT_DURATION)
end

local function setHeightShift(self, textView, newHeight)
  if newHeight ~= textView:getHeight() then
    self.viewHeightShift = MathUtils.round((math.abs(newHeight - textView:getHeight())), LOCATION_COMPARISON_DECIMAL_PLACES)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Protected API ----------------------

function BrevityTextAnimationBuilder:_createMoveItemAnimation(itemView, itemLayout)
  return AHLocationTo.new(itemView, self.moveDuration, AHEaseExponentialOut.new(), itemLayout:getX(), itemLayout:getY())
end

function BrevityTextAnimationBuilder:_createAllMoveItemAnimations(oldItemLayouts, newItemLayouts)
  local moveAnimations = AHSpawn.new()

  for itemView, itemLayout in pairs(newItemLayouts) do
    local oldItemLayout = oldItemLayouts[itemView]
    local shouldMoveItem = (oldItemLayout ~= nil) and (not itemLayout:equals(oldItemLayout))

    if oldItemLayout ~= nil then
      if not self.didItemsXChange then
        local oldX = MathUtils.round(oldItemLayout:getX(), LOCATION_COMPARISON_DECIMAL_PLACES)
        local newX = MathUtils.round(itemLayout:getX(), LOCATION_COMPARISON_DECIMAL_PLACES)
        self.didItemsXChange = newX ~= oldX
      end
      if not self.didItemsYChange then
        local itemYShift = MathUtils.round(math.abs(itemLayout:getY() - oldItemLayout:getY()), LOCATION_COMPARISON_DECIMAL_PLACES)
        self.didItemsYChange = self.viewHeightShift ~= itemYShift
      end
    end

    if shouldMoveItem then
      local itemAnimation = self:_createMoveItemAnimation(itemView, itemLayout)
      moveAnimations:addAction(itemAnimation)
    end
  end

  local playSoundEffect = function()
    Sound.playSoundEffect("round_change")
  end

  if didItemsLocationChange(self) then
    moveAnimations:addAction(playSoundEffect)
  end
  return moveAnimations
end

function BrevityTextAnimationBuilder:_createAdjustHeightAnimation(textView, newHeight)
  return AHDimYTo.new(textView, ADJUST_HEIGHT_DURATION, AHEaseExponentialOut.new(), newHeight)
end

-- Public API -------------------------

function BrevityTextAnimationBuilder:createItemsRemovedAnimation(textView, oldItemLayouts, newItemLayouts, newHeight)
  setHeightShift(self, textView, newHeight)
  local animation = self.super.createItemsRemovedAnimation(self, textView, oldItemLayouts, newItemLayouts, newHeight)
  return animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function BrevityTextAnimationBuilder.new()
  local self = TextAnimationBuilder.new()
  Mixin.mix(self, BrevityTextAnimationBuilder)
  self.didItemsXChange = false
  self.didItemsYChange = false
  self.viewHeightShift = 0
  setDurations(self)

  return self
end
