--- BrevityTextArea
-- @author created by Joe Poka on May 2, 2016

class("BrevityTextArea")
require("BrevityTextModel")
require("BrevityTextAnimationBuilder")
require("BrevityTextInterpreter")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LINE_SPACING = Screen.getSafeHeightPart(0.02)
local FONT = Font.new(FontFaces.DEFAULT_FONT, DEFAULT_FONT_SIZE)

local BASE_ATTRIBUTES = {
  font = FONT,
  itemType = "TYPE_TEXT",
  isReducible = false,
  isDroppable = false
}

local LABEL_ATTRIBUTES = {
  reduce = {isReducible = true},
  drop = {isDroppable = true}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BrevityTextArea:applyItemViewOpacityAttributeLinks()
  local itemViews = self:getItemViews()
  for _, itemView in pairs(itemViews) do
    itemView:inheritOpacity(self)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function BrevityTextArea.new(rawText, width, alignment)
  alignment = alignment or Alignment.LEFT
  local self = TextView.new(width, alignment, nil, LINE_SPACING)
  Mixin.mix(self, BrevityTextArea)

  self.animationBuilder = BrevityTextAnimationBuilder.new()

  local textInterpreter = BrevityTextInterpreter.new(BASE_ATTRIBUTES, LABEL_ATTRIBUTES)
  local textModel = BrevityTextModel.new(textInterpreter:interpretXmlText(rawText))

  self:setModel(textModel)

  self.textInterpreter = textInterpreter
  self.textModel = textModel

  return self
end
