--- Castles Background
-- @author Created by Adam Villalobos on Oct 28, 2013.

class("CastlesBackground")
require("CastlesLimitLine")
require("CastlesCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_CASTLE_COLOR = ColorUtils.colorRGBToStd(162, 123, 95, 76)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Create actions ---------------------

local function createScaleAction(self, height, duration)
  local scaleEase = AHEaseExponentialIn.new()
  local scaleAction = AHScaleTo.new(self.castleHeightBackground, duration, scaleEase, 1.0, height / Screen.height)
  local moveAction = AHLocationYTo.new(self.castleHeightBackground, duration, scaleEase, height * 0.5)
  local combinedSpawn = AHSpawn.new(scaleAction, moveAction)

  return combinedSpawn
end

local function createLineScaleAction(line)
  local TIMER_DURATION = 0.4
  local scaleAction = AHScaleTo.new(line, TIMER_DURATION, nil, 1.0, 1.0)
  local moveAction = AHLocationXTo.new(line, TIMER_DURATION, Screen.width * 0.5)
  local combinedSpawn = AHSpawn.new(scaleAction, moveAction)

  return combinedSpawn
end

 -- Play actions -----------------------

 local function startScaleTo(self,height, duration)
  local scaleAction = createScaleAction(self, height, duration)
  scaleAction:start()
 end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesBackground:setNewCastleHeight(height, fallDuration)
  startScaleTo(self, height, fallDuration)
  self.currentHeight = height
end

function CastlesBackground:playLose()
  self:insert(self.loseBar)
  self.loseBar:setY(self.currentHeight)

  local scaleAction = createLineScaleAction(self.loseBar)
  scaleAction:start()
end

function CastlesBackground:playIntro()
  local expandCoroutine = self.limitLine:doExpandEffect()
  return expandCoroutine
end

function CastlesBackground:removeLimitLine()
  local limitLine = self.limitLine
  limitLine:removeSelf()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function CastlesBackground.new(lineY)
  local self = Group.new(Screen.width, Screen.height)
  Mixin.mix(self, CastlesBackground)

  local background = Sprite.newWithName("bg_texture.png")
  local backgroundWidth, backgroundHeight = background:getDims()

  -- Scale bg up if bg texture is too small. Never scale down.
  local scaleX = math.max(Screen.width / backgroundWidth, 1.0)
  local scaleY = math.max(Screen.height / backgroundHeight, 1.0)
  background:setScl(scaleX, scaleY)

  self:insertAtCenter(background)

  local limitLine = CastlesLimitLine.newNotExpanded()
  limitLine:setLoc(Screen.width * 0.5, lineY)
  self.limitLine = limitLine
  self:insert(limitLine)

  --up moving background
  local castleHeightBackground = FilledRectangle.newFullScreen(BACKGROUND_CASTLE_COLOR)
  castleHeightBackground:setX(Screen.width * 0.5)
  castleHeightBackground:setScl(1.0, 0.0)
  self.castleHeightBackground = castleHeightBackground
  self:insert(castleHeightBackground)

  --bars
  local width = Screen.width
  local height = Screen.height/150

  local loseBar = FilledRectangle.new(width, height, CastlesCommon.LOSE_COLOR)
  loseBar:setScl(0, 1)
  self.loseBar = loseBar

  return self
end
