--- Castles Block Basic square shape
-- @author Created by Adam Villalobos on Oct 28, 2013.

class("CastlesBlockBasic")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLORS =
{
  ColorUtils.colorRGBToStd(76, 102 ,91), --GREEN
  ColorUtils.colorRGBToStd(98, 62, 60), --LIGHT PURPLE
  ColorUtils.colorRGBToStd(190, 140, 70), --YELLOW
  ColorUtils.colorRGBToStd(56, 39 ,48), --DARK PURPLE
  ColorUtils.colorRGBToStd(155, 78, 67), --RED
}

 -- ----------------------------------------------------------------------------
 -- Utility
 -- ----------------------------------------------------------------------------

local function createFallAction(self)
  local blockMoveDuration = self:getFallDuration()
  local blockMoveY = self.finalY
  local blockAction = AHLocationYTo.new(self, blockMoveDuration, AHEaseExponentialIn.new(), blockMoveY)

  return blockAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesBlockBasic:getXIndex()
  return self.xIndex
end

function CastlesBlockBasic:getYIndex()
  return self.yIndex
end

function CastlesBlockBasic:getFallDuration()
  return 0.55 * ((Screen.height + self.offsetY - self.finalY) / Screen.height)
end

function CastlesBlockBasic:startFalling()
  local fallAction = createFallAction(self)
  fallAction:start()
end

function CastlesBlockBasic:randomizeColor()
  self:setColor(unpack(COLORS[math.random(#COLORS)]))
end

function CastlesBlockBasic:resetPosition()
  self:setLoc(self.initialX, self.initialY)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function CastlesBlockBasic.new(width, height, data, settings)
  local self = FilledRectangle.new(width, height, COLORS[math.random(#COLORS)])
  Mixin.mix(self, CastlesBlockBasic)
  self.flag = data.flag

  -- add one to compensate for zero-based y-index in castle data
  self.yIndex = data.y + 1 + settings.yOffset
  self.xIndex = data.x + settings.xOffset
  self.finalY = self.yIndex * height - height * 0.5
  self.finalX = self.xIndex * width + width * 0.5
  self.offsetY = height

  self.initialX = self.xIndex * width + width * 0.5
  self.initialY = Screen.height + self.offsetY
  self:resetPosition()

  return self
end
