--- CastlesChatRationaleHelper.lua

class("CastlesChatRationaleHelper")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local OPTION_IDS = {
  EXAMPLE_SENTENCE = "id_example_sentence",
  PRONUNCIATION = "id_pronunciation"
}

local INTRO_MESSAGE_ENDER = "What else would you like to know?"

local EXAMPLE_SENTENCE_OPENER = "Here's an example:"

local EXAMPLE_SENTENCE_USER_MESSAGE_FORMATTED = "How do I use <b>%s</b> in a sentence?"
local PRONUNCIATION_USER_MESSAGE_FORMATTED = "How is <b>%s</b> pronounced?"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createIntro(wordData)
  local message = wordData.etymology .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
                  INTRO_MESSAGE_ENDER
  return message
end

local function createExampleSentenceOption(wordData)
  local userMessage = string.format(EXAMPLE_SENTENCE_USER_MESSAGE_FORMATTED, wordData.word)

  local systemMessage = EXAMPLE_SENTENCE_OPENER .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
                        wordData.exampleSentence

  return {
    id = OPTION_IDS.EXAMPLE_SENTENCE,
    userMessage = userMessage,
    systemMessage = systemMessage
  }
end

local function createPronunciationOption(wordData)
  local userMessage = string.format(PRONUNCIATION_USER_MESSAGE_FORMATTED, wordData.word)

  local systemMessage = wordData.pronunciation

  return {
    id = OPTION_IDS.PRONUNCIATION,
    userMessage = userMessage,
    systemMessage = systemMessage
  }
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesChatRationaleHelper.createConversation(wordData)
  local conversation = {}

  conversation.intro = createIntro(wordData)

  conversation.options = {}
  table.insert(conversation.options, createExampleSentenceOption(wordData))
  table.insert(conversation.options, createPronunciationOption(wordData))

  return conversation
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
