--- CastlesCoachUI
-- @author created by Jay Austria on January 26, 2015

class("CastlesCoachUI")

require("CastlesCommon")
require("FlickInteractionCoachUI")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local POSITIVE_SPRITE_NAME = "positive.png"
local NEGATIVE_SPRITE_NAME = "negative.png"
local POSITIVE_LABEL_STRING = Localizer.get("POSITIVE")
local NEGATIVE_LABEL_STRING = Localizer.get("NEGATIVE")

local SPRITE_NAME_FOR_FLAG_TYPE = {
  [CastlesCommon.FlagType.POSITIVE] = POSITIVE_SPRITE_NAME,
  [CastlesCommon.FlagType.NEGATIVE] = NEGATIVE_SPRITE_NAME
}

local LABEL_STRING_FOR_FLAG_TYPE = {
  [CastlesCommon.FlagType.POSITIVE] = POSITIVE_LABEL_STRING,
  [CastlesCommon.FlagType.NEGATIVE] = NEGATIVE_LABEL_STRING
}

local FONT_NAME = CastlesCommon.BOLD_FONT
local INTRO_FADE_DURATION = 1.0

local RESPONSE_ANIMATION_HOLD_DURATION = 0.5
local RESPONSE_ANIMATION_DURATION = 0.15
local RESPONSE_SCALE = 1.1

local COACH_UI_COLOR = ColorUtils.colorRGBToStd(0, 0, 0)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getSpriteNameForFlag(flagType)
  local spriteName = SPRITE_NAME_FOR_FLAG_TYPE[flagType]
  assert(spriteName, "Sprite name not found for flag type")

  return spriteName
end

local function getLabelStringForFlag(flagType)
  local labelString = LABEL_STRING_FOR_FLAG_TYPE[flagType]
  assert(labelString, "Label string not found for flag type")

  return labelString
end

local function getResponseAction(self, color)
  local colorUpAction = AHColorTo.new(self, RESPONSE_ANIMATION_DURATION, nil, color)
  local scaleUpAction = AHScaleTo.new(self, RESPONSE_ANIMATION_DURATION, nil, RESPONSE_SCALE)

  local colorDownAction = AHColorTo.new(self, RESPONSE_ANIMATION_DURATION, nil, self:getInitialColor())
  local scaleDownAction = AHScaleTo.new(self, RESPONSE_ANIMATION_DURATION, nil, self:getNeutralScale())
  local opacityDownAction = AHOpacityTo.new(self, RESPONSE_ANIMATION_DURATION, nil, self:getFadedVisibleOpacity())

  local combinedResponseUp = AHSpawn.new(colorUpAction, scaleUpAction)
  local holdDelay = AHDelay.new(RESPONSE_ANIMATION_HOLD_DURATION)
  local combinedResponseDown = AHSpawn.new(colorDownAction, scaleDownAction, opacityDownAction)

  local responseAction = AHSequence.new(combinedResponseUp, holdDelay, combinedResponseDown)

  return responseAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesCoachUI:introFadeIn()
  local startingVisibleOpacity = self:getStartingVisibleOpacity()
  local fadeAction = self:getFadeOpacityToAction(startingVisibleOpacity, INTRO_FADE_DURATION)

  self.currentRunningAction:insert(fadeAction)
end

function CastlesCoachUI:getCorrectAnimation()
  local correctAnimation = getResponseAction(self, CastlesCommon.WIN_COLOR)

  self.currentRunningAction:stop()

  return correctAnimation
end

function CastlesCoachUI:getShowCorrectAnimation()
  local colorUpAction = AHColorTo.new(self, RESPONSE_ANIMATION_DURATION, nil, CastlesCommon.WIN_COLOR)
  local scaleUpAction = AHScaleTo.new(self, RESPONSE_ANIMATION_DURATION, nil, RESPONSE_SCALE)

  local combinedResponseUp = AHSpawn.new(colorUpAction, scaleUpAction)

  return combinedResponseUp
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function CastlesCoachUI.new(flagType)
  local spriteName = getSpriteNameForFlag(flagType)
  local labelString = getLabelStringForFlag(flagType)

  local self = FlickInteractionCoachUI.new(spriteName, labelString, FONT_NAME, COACH_UI_COLOR)
  Mixin.mix(self, CastlesCoachUI)

  return self
end
