class("CastlesConfiguration")

require("CastlesCommon")
require("data.keyframes.KeyframeSet")
require("data.generators.LearningFocusedRoundGenerator")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

CastlesConfiguration.POINTS_PER_EXTRA_SECOND_REMAINING = 20
CastlesConfiguration.GAME_DURATION_IN_SECONDS = 30

CastlesConfiguration.TIME_REMAINING_TO_START_RESURFACING_MISTAKES =
  CastlesConfiguration.GAME_DURATION_IN_SECONDS *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING

local BONUS_FOR_INCORRECT_RESPONSES_COUNT = {
  [0] = 2100,
  [1] = 1300,
  [2] = 700,
  [3] = 300
}
local INCORECT_RESPONSES_ALLOWED_RANK_3 = 0
local INCORECT_RESPONSES_ALLOWED_RANK_2 = 1

local POINT_TOTAL_FROM_CORRECT_ANSWERS = 6400
local CORRECT_ANSWERS_TO_WIN_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {correctAnswersToWin = 7}),
  Keyframe.new(0.2, {correctAnswersToWin = 8})
)
local EXTRA_CONCEPTS_TO_LOAD_COUNT = 4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesConfiguration.getCorrectAnswersToWinCount()
  local difficulty = GameSession.getDifficulty()
  local correctAnswersThreshold = CORRECT_ANSWERS_TO_WIN_THRESHOLDS:getFrameAt(difficulty)
  return correctAnswersThreshold.correctAnswersToWin
end

function CastlesConfiguration.getPointsForCorrectAnswer()
  local correctAnswersToWinCount = CastlesConfiguration.getCorrectAnswersToWinCount()
  return POINT_TOTAL_FROM_CORRECT_ANSWERS / correctAnswersToWinCount
end

function CastlesConfiguration.getPointsForConsistencyBonus(incorrectResponseCount)
  if (incorrectResponseCount <= #BONUS_FOR_INCORRECT_RESPONSES_COUNT) then
    return BONUS_FOR_INCORRECT_RESPONSES_COUNT[incorrectResponseCount]
  else
    return 0
  end
end

function CastlesConfiguration.getConceptsToLoadCount()
  return CastlesConfiguration.getCorrectAnswersToWinCount() + EXTRA_CONCEPTS_TO_LOAD_COUNT
end

function CastlesConfiguration.getPassScore()
  return CastlesConfiguration.getPointsForCorrectAnswer() * CastlesConfiguration.getCorrectAnswersToWinCount()
end

function CastlesConfiguration.getRankTwoScore()
  local passScore = CastlesConfiguration.getPassScore()
  local consistencyBonusRankTwo = CastlesConfiguration.getPointsForConsistencyBonus(INCORECT_RESPONSES_ALLOWED_RANK_2)
  local rankTwoScore = passScore + consistencyBonusRankTwo

  return rankTwoScore
end

function CastlesConfiguration.getRankThreeScore()
  local passScore = CastlesConfiguration.getPassScore()
  local consistencyBonusRankThree = CastlesConfiguration.getPointsForConsistencyBonus(INCORECT_RESPONSES_ALLOWED_RANK_3)
  local rankThreeScore = passScore + consistencyBonusRankThree

  return rankThreeScore
end

function CastlesConfiguration.getDifficultyKeyFrames()
  ExperimentsConfiguration.DIFFICULTY_KEYFRAMES_EXPERIMENT:reportExperimentExposed()
  if ExperimentsConfiguration.DIFFICULTY_KEYFRAMES_EXPERIMENT:isActive() then
    return {
      {key = 0.0, valueProbabilities = {Basic = 0.75,
                                        Easy = 0.25}},

      {key = 0.25, valueProbabilities = {Basic = 0.10,
                                         Easy = 0.50,
                                         Medium = 0.4}},

      {key = 0.5, valueProbabilities = {Easy = 0.25,
                                        Medium = 0.5,
                                        Hard = 0.25}},

      {key = 0.75, valueProbabilities = {Medium = 0.4,
                                         Hard = 0.5,
                                         Advanced = 0.1}},

      {key = 1.0, valueProbabilities = {Hard = 0.5,
                                        Advanced = 0.5}}
    }
  end

  return {
    {key = 0.0, valueProbabilities = {Basic = 0.80,
                                      Easy = 0.20}},

    {key = 0.075, valueProbabilities = {Basic = 0.30,
                                        Easy = 0.70}},

    {key = 0.15, valueProbabilities = {Easy = 0.70,
                                       Medium = 0.30}},

    {key = 0.225, valueProbabilities = {Easy = 0.55,
                                        Medium = 0.45}},

    {key = 0.375, valueProbabilities = {Easy = 0.35,
                                        Medium = 0.65}},

    {key = 0.45, valueProbabilities = {Easy = 0.25,
                                       Medium = 0.70,
                                       Hard = 0.05}},

    {key = 0.525, valueProbabilities = {Easy = 0.15,
                                        Medium = 0.70,
                                        Hard = 0.15}},

    {key = 0.6, valueProbabilities = {Easy = 0.10,
                                      Medium = 0.65,
                                      Hard = 0.25}},

    {key = 0.675, valueProbabilities = {Easy = 0.05,
                                        Medium = 0.55,
                                        Hard = 0.40}},

    {key = 0.75, valueProbabilities = {Medium = 0.45,
                                       Hard = 0.55}},

    {key = 0.8, valueProbabilities = {Medium = 0.4,
                                      Hard = 0.55,
                                      Advanced = 0.05}},

    {key = 0.9, valueProbabilities = {Medium = 0.3,
                                      Hard = 0.6,
                                      Advanced = 0.1}},

    {key = 1.0, valueProbabilities = {Medium = 0.2,
                                      Hard = 0.6,
                                      Advanced = 0.2}}
  }
end

