--- CastlesFlickInteractionLayer
-- @author created by Jay Austria on January 21, 2015

class("CastlesFlickInteractionLayer")

require("CastlesCommon")
require("CastlesCoachUI")
require("FlickInteractionHandler")
require("FlickInteractionLayer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------
local INVISIBLE_OPACITY = 0.0
local CASTLES_FLAG_TYPES = {
  CastlesCommon.FlagType.NEGATIVE,
  CastlesCommon.FlagType.POSITIVE
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createCoachUIHidden(descriptor)
  local coachUI = CastlesCoachUI.new(descriptor.flagType)
  coachUI:setLoc(descriptor.x, descriptor.y)

  coachUI:setOpacity(INVISIBLE_OPACITY)

  return coachUI
end

local function createCoachUIs()
  local coachUIDescriptors =
  {
    {
      flagType = CastlesCommon.FlagType.NEGATIVE,
      x = Screen.getWidthPart(0.125),
      y = CastlesCommon.COACH_MARKS_Y
    },
    {
      flagType = CastlesCommon.FlagType.POSITIVE,
      x = Screen.getWidthPart(0.875),
      y = CastlesCommon.COACH_MARKS_Y
    }
  }

  local coachUIs = {}
  for _, coachUIDescriptor in ipairs(coachUIDescriptors) do
    local coachUI = createCoachUIHidden(coachUIDescriptor)
    table.insert(coachUIs, coachUI)
  end

  return coachUIs
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesFlickInteractionLayer:playIntroFadeIn()
  local coachUIs = self.coachUIs
  for _, coachUI in ipairs(coachUIs) do
    coachUI:introFadeIn()
  end
end

function CastlesFlickInteractionLayer:getShowCorrectAction(flagType)
  local coachUIToAnimate = self:getCoachUIForFlagType(flagType)
  local coachUIShowCorrectAnimation = coachUIToAnimate:getShowCorrectAnimation()

  return coachUIShowCorrectAnimation
end

function CastlesFlickInteractionLayer:returnToNeutral()
  local coachUIs = self.coachUIs
  for _, coachUI in ipairs(coachUIs) do
    coachUI:returnToNeutral()
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function CastlesFlickInteractionLayer.new()
  local coachUIs = createCoachUIs()

  local self = FlickInteractionLayer.new(CASTLES_FLAG_TYPES, coachUIs)
  Mixin.mix(self, CastlesFlickInteractionLayer)

  return self
end
