-- @author Created by Jay Austria on Nov. 19, 2013
class("CastlesLimitLine")
require("CastlesCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GROUP_HEIGHT = 10
local DASH_SPACING_X = 5
local LABEL_OFFSET_Y = 10
local FINISH_LABEL_FONT_SIZE = 20
local LABEL_SHOW_DELAY = 0.3
local LABEL_HIDE_DELAY = 0.4
local FADE_SPEED = 0.1
local FADE_OUT_SPEED = 1.0
local LABEL_LOCATION_X = Screen.width * 0.8
local LABEL_TEXT = Localizer.get("FINISH_LINE")

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesLimitLine:doEraseEffect()
  local function effectFunction()
    for _, dash in pairs(self.dashes) do
      dash:removeSelf()
      coroutine.yield()
    end
    self.dashes = {}
  end

  local effectCoroutine = MOAICoroutine.new()
  effectCoroutine:run(effectFunction)
  return effectCoroutine
end

function CastlesLimitLine:doExpandEffect()
  local function finishLineLabelFunction()
    local finishLabel = Label.new(LABEL_TEXT, Font.new(CastlesCommon.BOLD_FONT, FINISH_LABEL_FONT_SIZE))
    finishLabel:setColor(unpack(CastlesCommon.DARK_UI_COLOR))
    finishLabel:setLoc(LABEL_LOCATION_X, GROUP_HEIGHT + LABEL_OFFSET_Y)
    finishLabel:setOpacity(0)
    self:insert(finishLabel)

    local showDelay = AHDelay.new(LABEL_SHOW_DELAY)
    local fadeIn = AHOpacityTo.new(finishLabel, FADE_SPEED, nil, CastlesCommon.DARK_UI_COLOR[4])
    local hideDelay = AHDelay.new(LABEL_HIDE_DELAY)
    local fadeOut = AHOpacityTo.new(finishLabel, FADE_OUT_SPEED, nil, 0)
    local labelSequence = AHSequence.new(
                                         showDelay,
                                         fadeIn,
                                         hideDelay,
                                         fadeOut
                                         )
    wait(labelSequence:start())
    finishLabel:removeSelf()
  end
  MOAICoroutine.new():run(finishLineLabelFunction)

  local function effectFunction()
    self.dashes = {}
    local penX = 0
    while(penX < Screen.width) do
      local newDash = Sprite.newWithName("dash.png")
      local dashW, dashH = newDash:getDims()
      newDash:setColor(unpack(CastlesCommon.UI_COLOR))
      newDash:setLoc(penX, dashH * 1.5)
      penX = penX + dashW + DASH_SPACING_X
      self:insert(newDash)
      table.insert(self.dashes, newDash)
      coroutine.yield()
    end
  end

  local effectCoroutine = MOAICoroutine.new()
  effectCoroutine:run(effectFunction)
  return effectCoroutine
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function CastlesLimitLine.newNotExpanded()
  local self = Group.new(Screen.width, GROUP_HEIGHT)
  Mixin.mix(self, CastlesLimitLine)
  return self
end
