--- CastlesPrompt
-- @author Created by Adam Villalobos on Oct 28, 2013.

class("CastlesPrompt")
require("CastlesCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FONT_NAME = CastlesCommon.NORMAL_FONT
local TEXT_COLOR = {1.0,1.0,1.0}
local BACKGROUND_COLOR = ColorUtils.colorHex24ToStd(0x5D554B)
local START_SCALE = 0.95
local DOWN_SCALE = 1.00
local WORD_LABEL_FONT_SIZE = 24

 -- ----------------------------------------------------------------------------
 -- Utility
 -- ----------------------------------------------------------------------------

-- Create actions ---------------------

local function createScaleAction(self)
 local duration = 0.15
 local scaleAction = AHScaleTo.new(self, duration, nil, 1.0)
 return scaleAction
end

local function createOpacityAction(object, opacity, duration)
  duration = duration or 0.2
  local opacityAction = AHOpacityTo.new(object, duration, nil, opacity)
  return opacityAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesPrompt:notifyFlickInteractionBegin()
  self.box:setScl(DOWN_SCALE)
  if (self.label) then
    self.label:setScl(DOWN_SCALE)
  end
end

function CastlesPrompt:notifyFlickInteractionEnd()
  self.box:setScl(START_SCALE)
  if (self.label) then
    self.label:setScl(START_SCALE)
  end
end

function CastlesPrompt:getPlayInAnimation()
  self:setScl(0.0)
  self:forceUpdate()
  self.label:setOpacity(1.0)
  self.box:setOpacity(1.0)
  self:setLoc(self.originalX, self.originalY)

  local scaleUp = createScaleAction(self)
  return scaleUp
end

function CastlesPrompt:getFadeOutAnimation(duration)
  local endOpacity = 0.0
  local labelFadeOut = createOpacityAction(self.label, endOpacity, duration)
  local boxFadeOut = createOpacityAction(self.box, endOpacity, duration)

  local fadeSpawn = AHSpawn.new(labelFadeOut, boxFadeOut)
  return fadeSpawn
end

function CastlesPrompt:resetOpacity()
  self.label:setOpacity(1.0)
  self.box:setOpacity(1.0)
end

function CastlesPrompt:setText(text)
  if (self.label) then
    self.label:removeSelf()
  end

  local selfWidth = self:getWidth()
  self.label = Label.new(text, Font.new(FONT_NAME, WORD_LABEL_FONT_SIZE), selfWidth, Alignment.CENTER)
  self.label:setColor(unpack(TEXT_COLOR))
  self.label:setScl(START_SCALE)
  self:insertAtCenter(self.label)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function CastlesPrompt.new(width, height, x, y)
  local self = Group.new(width, height)
  Mixin.mix(self, CastlesPrompt)

  self.box = FilledRoundedRectangle.new(width, height, 5, BACKGROUND_COLOR)
  self.box:setScl(START_SCALE)
  self:insertAtCenter(self.box)

  self:setAnchorPointCenter()

  self:setScl(0, 0)

  self.originalX = x
  self.originalY = y

  return self
end
