--- CastlesStateMachine
-- @author Created by Joe Poka on March 30, 2018.

class("CastlesStateMachine")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesStateMachine:addState(state, onEnter, onLeave)
  assert(onEnter ~= nil and onLeave ~= nil, "must have both onEnter and onLeave")
  local callbacks = {
    onEnter = onEnter,
    onLeave = onLeave
  }
  self.states[state] = callbacks
end

function CastlesStateMachine:transition(destinationState)
  assert(self.currentState ~= nil, "must set current state before transitioning")
  local currentCallbacks = self.states[self.currentState]
  local destinationCallbacks = self.states[destinationState]
  currentCallbacks.onLeave()
  self.currentState = destinationState
  destinationCallbacks.onEnter()
end

function CastlesStateMachine:getCurrentState()
  assert(self.currentState ~= nil, "must set initial state")
  return self.currentState
end

function CastlesStateMachine:setCurrentState(state)
  self.currentState = state
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CastlesStateMachine.new()
  local self = {}
  Mixin.mix(self, CastlesStateMachine)

  self.states = {}

  return self
end
