--- FlickInteractionHelper
-- @author Created by andrewc on February 18, 2021.

-- Instructional layer used by FlickInteractionLayer

class("FlickInteractionHelper")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HELPER_TEXT_X = Screen.getWidthPart(0.5)
local HELPER_TEXT_Y = Screen.getHeightPart(0.28)
local HELPER_TEXT = "swipe"
local HELPER_FONT = FontFaces.DIN_OT_MEDIUM
local HELPER_FONT_SIZE = 18
local ARROW_SPRITE_NAME = "shared_arrow.png"
local RIGHT_ARROW_OFFSET = 50
local LEFT_ARROW_OFFSET = -RIGHT_ARROW_OFFSET
local ARROW_SCALE = 0.6
local HELPER_FADE_DURATION = 0.2
local ANIMATION_DISTANCE = 20
local OUT_ANIMATION_DURATION = 0.25
local IN_ANIMATION_DURATION = 0.7

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FlickInteractionHelper:show()
  self.activeFadeAnimation:stop()
  self.activeFadeAnimation = AHOpacityTo.new(self, HELPER_FADE_DURATION, nil, 1.0)
  self.activeFadeAnimation:start()

  self.activelyLoopingArrowAnimation = AHLoop.new(self.arrowAnimationToLoop)
  self.activelyLoopingArrowAnimation:start()
end

function FlickInteractionHelper:hide()
  self.activeFadeAnimation:stop()
  self.activeFadeAnimation = AHSequence.new(
    AHOpacityTo.new(self, HELPER_FADE_DURATION, nil, 0.0),
    function() self.activelyLoopingArrowAnimation:stop() end)
  self.activeFadeAnimation:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createAnimatingArrow(self, locationX, locationY, rotation)
  local arrow = Sprite.newWithName(ARROW_SPRITE_NAME)

  local arrowWidth = arrow:getWidth()
  local arrowYOffset = -arrowWidth * 0.1

  arrow:setLoc(locationX, locationY + arrowYOffset)
  arrow:setRot(rotation)
  arrow:setScl(ARROW_SCALE)

  local vectorDirection = -rotation/math.abs(rotation) --Will be either positive or negative 1

  local outDuration = OUT_ANIMATION_DURATION
  local outDistance = ANIMATION_DISTANCE * vectorDirection
  local outEase = AHEaseSineOut.new()
  local outAnimation = AHLocationXBy.new(arrow, outDuration, outEase, outDistance)

  local inDuration = IN_ANIMATION_DURATION
  local inDistance = -outDistance
  local inEase = AHEaseSimpleInOut.new()
  local inAnimation = AHLocationXBy.new(arrow, inDuration, inEase, inDistance)

  local animationSequence = AHSequence.new(outAnimation, inAnimation)
  self.arrowAnimationToLoop:addAction(animationSequence)

  return arrow
end

local function createLayout(self)
  local helpFont = Font.new(HELPER_FONT, HELPER_FONT_SIZE)
  local helpText = Label.new(HELPER_TEXT, helpFont)
  helpText:setLoc(HELPER_TEXT_X, HELPER_TEXT_Y)
  self:insert(helpText)

  local leftArrow = createAnimatingArrow(self, HELPER_TEXT_X + LEFT_ARROW_OFFSET, HELPER_TEXT_Y, 90)
  self:insert(leftArrow)

  local rightArrow = createAnimatingArrow(self, HELPER_TEXT_X + RIGHT_ARROW_OFFSET, HELPER_TEXT_Y, -90)
  self:insert(rightArrow)

  self:linkOpacityAttrToChildren(true)
  self:setOpacity(0)
end

-- Constructor ------------------------

function FlickInteractionHelper.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, FlickInteractionHelper)

  self.arrowAnimationToLoop = AHSpawn.new()
  self.activelyLoopingArrowAnimation = AHDoNothing.new()
  self.activeFadeAnimation = AHDoNothing.new()

  createLayout(self)

  return self
end
