varying LOWP vec2 uvVarying;

uniform LOWP sampler2D sampler;
uniform LOWP float normalizationValueHorizontal;
uniform LOWP float normalizationValueVertical;
uniform LOWP float tintColorWeight;

void main()
{
  // normalizationValues help gradients start/end at screen bounds (instead of larger texture bounds)
  LOWP float horizontalDistance = clamp(uvVarying.x / normalizationValueHorizontal, 0.0, 1.0);
  LOWP float verticalDistance = clamp((uvVarying.y - normalizationValueVertical) / (1.0 - normalizationValueVertical), 0.0, 1.0);

  LOWP vec4 color = texture2D(sampler, uvVarying);
  const LOWP vec3 topColor = vec3(0.1, 0.1, 0.1);
  LOWP vec3 tintColor = mix(color.rgb, topColor, tintColorWeight);

  // The following code implements the color blending equation on page 328 of
  // https://www.adobe.com/content/dam/Adobe/en/devnet/acrobat/pdfs/PDF32000_2008.pdf

  // from Lum(C) equation on page 326
  const LOWP vec3 lumVec = vec3(0.3, 0.59, 0.11);

  // SetLum(C, l) up to call to ClipColor(C)
  LOWP vec3 a1 = tintColor + vec3(dot(color.rgb, lumVec) - dot(tintColor, lumVec));

  // start of ClipColor(C) (called from SetLum(C, l))
  LOWP float minComponent = min(a1.r, min(a1.g, a1.b));
  LOWP float maxComponent = max(a1.r, max(a1.g, a1.b));

  // at this point in ClipColor(C), three things can happen:
  // 1 - return color passed to ClipColor(C)
  // 2 - return color when minComponent < 0.0
  // 3 - return color when maxComponent > 1.0

  LOWP vec3 L = vec3(dot(a1, lumVec));
  LOWP vec3 A = (L + (((a1 - L) * (vec3(1.0) - L)) / (vec3(maxComponent) - L)));
  LOWP vec3 B = (L + (((a1 - L) * L) / (L - vec3(minComponent))));

  gl_FragColor = vec4((maxComponent > 1.0 ? A : (minComponent < 0.0 ? B : a1)), 1.0) * color.a;
}
