--- CastlesValidator
-- @author Created by Anton Morant on April 17, 2014.
-- ConceptValidator for Castles.

class("CastlesValidator")

require("validators.ConceptValidator")
require("validators.TagValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'word',
  'connotation',
  'rationale'
  -- , 'difficulty' -- TODO: Needs adding to games.json mapping.
}

local OPTIONAL_KEYS = {
  'exampleSentence',
  'etymology',
  'pronunciation'
}

local CONNOTATION_VALUES = {
  "positive",
  "negative"
}

local OPTIONAL_PAIRED_TAGS = {
  "b",
  "i"
}

local MAX_WORD_LENGTH = 14
local MAX_POST_GAME_RATIONALE_LENGTH = 150

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Note: Only really protects against double-quotes. Single quotes are more difficult
-- to detect without risking false positives with apostrophes!
local function validateNoQuotes(self, text)
  self:expectNil(string.find(text, "\""), "Encountered double quotes in the text.")
end

local function validateExpandedContent(self, concept)
  if ((concept.exampleSentence ~= nil) and (string.len(concept.exampleSentence) > 0)) then
    self:validateTags(concept.exampleSentence, nil, nil, nil, OPTIONAL_PAIRED_TAGS)
    validateNoQuotes(self, concept.exampleSentence)
  end

  if ((concept.etymology ~= nil) and (string.len(concept.etymology) > 0)) then
    self:validateTags(concept.etymology, nil, nil, nil, OPTIONAL_PAIRED_TAGS)
    validateNoQuotes(self, concept.etymology)
  end

  if ((concept.pronunciation ~= nil) and (string.len(concept.pronunciation) > 0)) then
    self:validateTags(concept.pronunciation, nil, nil, nil, OPTIONAL_PAIRED_TAGS)
    validateNoQuotes(self, concept.pronunciation)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CastlesValidator:validateConcept(concept)
  local allowEmpty = false
  self:validateRequiredKeys(concept, REQUIRED_KEYS, allowEmpty)

  self:expectStringIsLettersOnly(concept.word, "word", "-")
  self:expectLessOrEqual(#concept.word, MAX_WORD_LENGTH, "word is too long")
  self:expectLessOrEqual(#concept.rationale, MAX_POST_GAME_RATIONALE_LENGTH, "post game rationale is too long")
  self:expectIsContained(concept.connotation, CONNOTATION_VALUES, "connotation")

  allowEmpty = true
  self:validateRequiredKeys(concept, OPTIONAL_KEYS, allowEmpty)
  validateExpandedContent(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CastlesValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, CastlesValidator)
  TagValidator.make(self)

  return self
end
