class("ClarityConfiguration")

require("ExperimentsConfiguration")
require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ClarityConfiguration.POINTS_PER_FULL_SPEED_BONUS = 100
ClarityConfiguration.SPEED_BONUS_DECAY_START = 0.0
ClarityConfiguration.NUMBER_OF_LIVES = 4
ClarityConfiguration.POINTS_PER_EXTRA_LIFE = 1000
ClarityConfiguration.TUTORIAL_DELAY = 8

local TOTAL_POINTS_FROM_CORRECT_RESPONSES = 4800
local CORRECT_ANSWERS_REQUIRED_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {correctAnswersNeeded = 5}),
  Keyframe.new(0.2, {correctAnswersNeeded = 6}),
  Keyframe.new(0.3, {correctAnswersNeeded = 7})
)

local EXTRA_LIVES_RANK_TWO = 2
local EXTRA_LIVES_RANK_THREE = 3

local SPEED_BONUS_WINDOW_IN_SECONDS = 3.0
local DIFFICULTY_BUCKETS = {"Easy", "Medium", "Hard", "Advanced"}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getBonusPointsForExtraLives(numberOfLives)
  local extraLivesBonus = numberOfLives * ClarityConfiguration.POINTS_PER_EXTRA_LIFE
  return extraLivesBonus
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ClarityConfiguration.getCorrectAnswersToWinCount()
  local difficulty = GameSession.getDifficulty()
  local correctAnswersToWinThreshold = CORRECT_ANSWERS_REQUIRED_THRESHOLDS:getFrameAt(difficulty)
  return correctAnswersToWinThreshold.correctAnswersNeeded
end

function ClarityConfiguration.getPointsForCorrectAnswer()
  local correctAnswersToWin = ClarityConfiguration.getCorrectAnswersToWinCount()
  return math.ceil(TOTAL_POINTS_FROM_CORRECT_RESPONSES / correctAnswersToWin)
end

function ClarityConfiguration.getSpeedBonusWindowInSeconds()
  return SPEED_BONUS_WINDOW_IN_SECONDS
end

function ClarityConfiguration.getPassScore()
  local correctNeeded = ClarityConfiguration.getCorrectAnswersToWinCount()
  local passScore = correctNeeded * ClarityConfiguration.getPointsForCorrectAnswer()

  return passScore
end

function ClarityConfiguration.getRankTwoScore()
  local passScore = ClarityConfiguration.getPassScore()
  local rankTwoBonusPoints = getBonusPointsForExtraLives(EXTRA_LIVES_RANK_TWO)
  local rankTwoScore = passScore + rankTwoBonusPoints

  return rankTwoScore
end

function ClarityConfiguration.getRankThreeScore()
  local passScore = ClarityConfiguration.getPassScore()
  local rankThreeBonusPoints = getBonusPointsForExtraLives(EXTRA_LIVES_RANK_THREE)
  local rankThreeScore = passScore + rankThreeBonusPoints

  return rankThreeScore
end

function ClarityConfiguration.getDifficultyKeyFrames()
  return DifficultyCurves.getKeyFrames(DifficultyCurves.STANDARD_CURVE, DIFFICULTY_BUCKETS)
end
