--- ClarityInterstitialLabel
-- @author created by Jay Austria sometime in March, 2014

class("ClarityInterstitialLabel")

require("ClarityColors")
require("text.SizeConstrainedTextArea")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INTERSTITIAL_LABEL_COLOR = ClarityColors.WHITE
local INTERSTITIAL_FONT = Font.new(FontFaces.DEFAULT_FONT, 18)
local FADE_DURATION = 0.5
local SHOW_DURATION = .6
local BORDER_PADDING_X = 16
local BORDER_PADDING_Y = 8
local SHOW_OPACITY = 0.5

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ClarityInterstitialLabel:getHideAction()
  local hideBackingAction = AHOpacityTo.new(self, FADE_DURATION, nil, INVISIBLE_OPACITY)
  local hideLabelAction = AHOpacityTo.new(self.label, FADE_DURATION, nil, INVISIBLE_OPACITY)

  local combinedAction = AHSpawn.new(hideBackingAction, hideLabelAction)

  return combinedAction
end

function ClarityInterstitialLabel:getShowAction()
  local showBackingAction = AHOpacityTo.new(self, FADE_DURATION, nil, SHOW_OPACITY)
  local showLabelAction = AHOpacityTo.new(self.label, FADE_DURATION, nil, VISIBLE_OPACITY)

  local combinedAction = AHSpawn.new(showBackingAction, showLabelAction)

  return combinedAction
end

function ClarityInterstitialLabel:getFullAnimation()
  local combinedShow = self:getShowAction()
  local delayAction = AHDelay.new(SHOW_DURATION)
  local combinedHide = self:getHideAction()

  local combinedAction = AHSequence.new(combinedShow, delayAction, combinedHide)
  return combinedAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(message, maxWidth, maxHeight)
  local fullBorderPaddingX = 2 * BORDER_PADDING_X
  local fullBorderPaddingY = 2 * BORDER_PADDING_Y

  local baseAttributes = { font = INTERSTITIAL_FONT, color = INTERSTITIAL_LABEL_COLOR }
  local maxTextWidth = maxWidth - fullBorderPaddingX
  local maxTextHeight = maxHeight - fullBorderPaddingY
  local label = SizeConstrainedTextArea.new(message, maxTextWidth, maxTextHeight, baseAttributes, nil, Alignment.CENTER, Alignment.CENTER)

  local backingWidth = label:getTextWidth() + fullBorderPaddingX
  local backingHeight = label:getTextHeight() + fullBorderPaddingY
  local backingColor = ClarityColors.getInterstitialBackingColor()
  local group = FilledRectangle.new(backingWidth, backingHeight, backingColor)

  group:insertAtCenter(label)
  group.label = label

  group:setOpacity(INVISIBLE_OPACITY)
  group.label:setOpacity(INVISIBLE_OPACITY)

  return group
end

function ClarityInterstitialLabel.newWithMessage(message, maxWidth, maxHeight)
  local self = createLayout(message, maxWidth, maxHeight)
  Mixin.mix(self, ClarityInterstitialLabel)

  return self
end
