--- ClarityOutroEffect
-- @author created by Jay Austria sometime in March, 2014

class("ClarityOutroEffect")
require("ClarityCommon")
require("ClarityColors")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local NUMBER_OF_DOTS = 8
local NUMBER_OF_PERSISTENT_TRAILS = 3
local MOVE_DOT_START_DELAY = 0.5

local DOT_RADIUS = 4
local DOT_APPEAR_DURATION = 0.1
local DOT_MOVE_DURATION = 0.4
local DOT_SCALE_DURATION = 0.5

local FULL_CIRCLE_ANGLE = 360

local DOT_INSTANCE_STAGGER_DELAY = 0.05
local DOT_INSTANCE_STAGGER_DISAPPEAR = 2.0

local GROUP_ROTATION_DURATION = 4.5
local GROUP_ROTATION_AMOUNT = -1250

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createOneMovingDot(self, color, disappearDelay, opacity, createDelay)
  local w, h = self:getDims()
  local dotGroup = Group.new(w, h)
  self:insertAtCenter(dotGroup)

  local dot = FilledCircle.new(DOT_RADIUS, color)
  dot:setOpacity(opacity)
  dot:setScl(SCALE_NONE)

  local startDelay = AHDelay.new(createDelay + MOVE_DOT_START_DELAY)
  local appear = AHScaleTo.new(dot, DOT_APPEAR_DURATION, nil, SCALE_FULL)
  local moveOut = AHLocationXBy.new(dot, DOT_MOVE_DURATION, AHEaseSimpleOut.new(), self.diameter * 0.5)
  local waitAWhile = AHDelay.new(disappearDelay)
  local disappear = AHScaleTo.new(dot, DOT_SCALE_DURATION, nil, SCALE_NONE)

  local fullSequence = AHSequence.new(startDelay, appear, moveOut, waitAWhile, disappear)
  dotGroup:insertAtCenter(dot)

  return dotGroup, fullSequence
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------
function ClarityOutroEffect:getAllDotsAction()
  return self.allDotsAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local dotActionCollection = {}
  for angularIndex = 0, NUMBER_OF_DOTS - 1 do
    for instanceIndex = 1, NUMBER_OF_PERSISTENT_TRAILS do
        local angle = angularIndex * (FULL_CIRCLE_ANGLE / NUMBER_OF_DOTS)

        local palette = ClarityColors.getPalette()
        local colorIndex = angularIndex % #palette + 1
        local color = ClarityColors.getPaletteColor(colorIndex)

        local createDelay = DOT_INSTANCE_STAGGER_DELAY * instanceIndex

        local disappearStaggerAmount = DOT_INSTANCE_STAGGER_DISAPPEAR * (1 - (angularIndex - 1) / NUMBER_OF_DOTS)
        local opacity = 1.0 - (instanceIndex - 1) / NUMBER_OF_PERSISTENT_TRAILS

        local dotGroup, dotActions = createOneMovingDot(self, color, 1 + disappearStaggerAmount, opacity, createDelay)
        dotGroup:setRot(angle)
        local rotationDelay = AHDelay.new(createDelay)
        local groupRotationAction = AHRotationBy.new(dotGroup, GROUP_ROTATION_DURATION, nil, GROUP_ROTATION_AMOUNT)
        local combinedRotationAction = AHSequence.new(rotationDelay, groupRotationAction)

        local dotCompleteAction = AHSpawn.new(dotActions, combinedRotationAction)
        table.insert(dotActionCollection, dotCompleteAction)
    end
  end

  local allDotsAction = AHSpawn.new(unpack(dotActionCollection))

  local playOutroSoundAction = AHDelayed.new(0.5, function() Sound.playSoundEffect("goal_reached") end)
  allDotsAction:addAction(playOutroSoundAction)

  self.allDotsAction = allDotsAction
end

function ClarityOutroEffect.newWithDiameter(diameter)
  local self = Group.new(diameter, diameter)
  self.diameter = diameter
  Mixin.mix(self, ClarityOutroEffect)

  createLayout(self)

  return self
end
