--- ClarityRoundGenerator
-- @author Created by karl on August 13, 2014.

module("ClarityRoundGenerator", package.seeall)

require("data.generators.Generator")
require("ClarityData")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_UNIQUE_CONCEPT_TYPE_GENERATION_ATTEMPTS = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getFormattedRoundDataFromConcept(concept)

  local formattedRoundData = {}
  formattedRoundData.replacement = concept.replacement
  formattedRoundData.distractor = concept.distractor
  formattedRoundData.identifier = concept.identifier
  formattedRoundData.rawSentence = concept.sentence
  formattedRoundData.rationale = concept.rationale

  return formattedRoundData
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ClarityRoundGenerator:reset()

end

function ClarityRoundGenerator:generateNextValue()
  local concept = self.conceptGenerator:next()
  local attemptIndex = 0
  local phrase = concept.phrase

  while (self.seenPhrases[phrase] and attemptIndex < MAX_UNIQUE_CONCEPT_TYPE_GENERATION_ATTEMPTS) do
    attemptIndex = attemptIndex + 1

    concept = self.conceptGenerator:next()
    phrase = concept.phrase
  end
  self.seenPhrases[phrase] = true

  local roundData = getFormattedRoundDataFromConcept(concept)
  return roundData
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ClarityRoundGenerator.new()
  local self = Generator.new()
  Mixin.mix(self, ClarityRoundGenerator)

  self.conceptGenerator = GameSession.conceptGenerator
  self.seenPhrases = {}

  return self
end
