--- ClarityTextArea
-- @author Created by Steven Mah on January 6, 2016.

class("ClarityTextArea")
require("ClarityTextModel")
require("ClarityTextAnimationBuilder")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LINE_SPACING = Screen.getSafeHeightPart(0.02)
local FONT = Font.new(FontFaces.DEFAULT_FONT, DEFAULT_FONT_SIZE)

local BASE_ATTRIBUTES = {
  font = FONT,
  itemType = "TYPE_TEXT",
  isReducible = false
}

local LABEL_ATTRIBUTES = {
  reduce = {isReducible = true}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ClarityTextArea:applyItemViewOpacityAttributeLinks()
  local itemViews = self:getItemViews()
  for _, itemView in pairs(itemViews) do
    itemView:inheritOpacity(self)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ClarityTextArea.new(rawText, width)
  local self = TextView.new(width, Alignment.LEFT, nil, LINE_SPACING)
  Mixin.mix(self, ClarityTextArea)

  self.animationBuilder = ClarityTextAnimationBuilder.new()

  local textInterpreter = TextInterpreter.new(BASE_ATTRIBUTES, LABEL_ATTRIBUTES)
  local textModel = ClarityTextModel.new(textInterpreter:interpretXmlText(rawText))

  self:setModel(textModel)

  self.textInterpreter = textInterpreter
  self.textModel = textModel

  return self
end
