--- ClarityWordView
-- @author Created by Jay Austria, Febtober 18, 2014

class("ClarityWordView")
require("ClarityCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WRONG_HIGHLIGHT_COLOR = ClarityColors.BLACK
local INVISIBLE_COLOR = {0.0, 0.0, 0.0, 0.0}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function onTouchDown(self, event)
  if self.onTappedCallback then
    local tapLocation =
    {
      x = event.x,
      y = event.y
    }
    self.onTappedCallback(self, tapLocation)
  end
end

local function createHighlight(self, backingColor)
  local highlightWidth = self:getWidth() + self:getLeftMargin() + self:getRightMargin() + WORD_RECT_PADDING_X
  local highlightHeight = self:getHeight() + WORD_RECT_PADDING_Y

  local highlightBacking = FilledRectangle.new(highlightWidth, highlightHeight, backingColor)
  return highlightBacking
end

local function addHighlight(self, backingColor)
  local highlightBacking = createHighlight(self, backingColor)
  self:insertAtCenter(highlightBacking)
  self.highlightBacking = highlightBacking


  local labelColor = ColorUtils.getTextColorForBackingColorYIQ(backingColor)
  local highlightFont = Font.new(FontFaces.DEFAULT_FONT, DEFAULT_FONT_SIZE)
  local highlightLabel = Label.new(self.text, highlightFont)
  highlightLabel:setColor(unpack(labelColor))
  highlightBacking:insertAtCenter(highlightLabel)
  self.highlightLabel = highlightLabel

  highlightBacking:setAttrLink(MOAIProp2D.ATTR_VISIBLE, self)
  highlightLabel:setAttrLink(MOAIProp2D.ATTR_VISIBLE, self)

  highlightBacking:setAttrLink(MOAIColor.ATTR_OPACITY, self)
  highlightLabel:setAttrLink(MOAIColor.ATTR_OPACITY, self)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ClarityWordView:doFallOff()
  local MAX_ROTATION = 30
  local ROTATION_DURATION = 0.2
  local SCALE_DURATION = 0.3
  local randomRotationAmount = math.random() * MAX_ROTATION - MAX_ROTATION * 0.5
  local FALL_DURATION = 0.45
  local FALL_DELAY = 0.07
  local END_SCALE_AMOUNT = 0.6

  local rotateAction = AHRotationBy.new(self, ROTATION_DURATION, AHEaseSimpleOut.new(3.0), randomRotationAmount)
  local fallOffAction = AHLocationYTo.new(self, FALL_DURATION, AHEaseSimpleIn.new(4.0), Screen.fullHeight * - 0.9)
  local scaleAction = AHScaleTo.new(self, SCALE_DURATION, nil, END_SCALE_AMOUNT)

  local function fallEffect()
    scaleAction:start()
    rotateAction:start()
    waitFor(FALL_DELAY)
    fallOffAction:start()
  end
  runInCoroutine(fallEffect)
end

function ClarityWordView:showAsCorrect(highlightColor)
  addHighlight(self, highlightColor)
end

function ClarityWordView:showAsIncorrect()
  addHighlight(self, WRONG_HIGHLIGHT_COLOR)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function ClarityWordView.makeTappable(view, index, onTappedCallback)
  Mixin.mix(view, ClarityWordView)

  local selfW, selfH = view:getDims()

  local rectW = selfW + WORD_RECT_PADDING_X
  local rectH = selfH + WORD_RECT_PADDING_Y

  local tapRect = FilledRectangle.new(rectW, rectH, INVISIBLE_COLOR)
  view:insertAtCenter(tapRect)

  local function onTouchDownWrapper(self, event)
    onTouchDown(view, event)
    return true
  end

  Button.makeButton(tapRect, onTouchDownWrapper)

  view.wordIndex = index

  view.onTappedCallback = onTappedCallback

  return view
end
