--- CollateColors
-- @author Created by Jay Austria on May 15, 2014.

class("CollateColors")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local function COLOR_TABLE(hex24bit)
  return ColorUtils.colorHex24ToStd(hex24bit)
end

local NEW_COLOR = COLOR_TABLE(0xb7bdd6)
local FROZEN_ITEM_COLOR = COLOR_TABLE(0xb7bdd6)
local CORRECT_COLOR = COLOR_TABLE(0x1bff98)
local INCORRECT_COLOR = COLOR_TABLE(0xe91d2e)
local SLIDE_HELPER_COLOR = COLOR_TABLE(0xbbbbbb)
local HINT_LABEL_COLOR = COLOR_TABLE(0x555555)

local BACKGROUND_COLOR_INITIAL_TOP = COLOR_TABLE(0x222222)
local BACKGROUND_COLOR_INITIAL_BOTTOM = COLOR_TABLE(0x333333)
local BACKGROUND_COLOR_MAGENTAS_TOP = COLOR_TABLE(0x210b28)
local BACKGROUND_COLOR_MAGENTAS_BOTTOM = COLOR_TABLE(0x623270)
local BACKGROUND_COLOR_BLUES_TOP = COLOR_TABLE(0x1e203e)
local BACKGROUND_COLOR_BLUES_BOTTOM = COLOR_TABLE(0x1a507a)
local BACKGROUND_COLOR_PURPLES_TOP = COLOR_TABLE(0x221e28)
local BACKGROUND_COLOR_PURPLES_BOTTOM = COLOR_TABLE(0x5f4ba8)

local COLOR_SET_MAGENTAS =
{
  COLOR_TABLE(0xff83f8),
  COLOR_TABLE(0xff72fc),
  COLOR_TABLE(0xfe61ff),
  COLOR_TABLE(0xf84fff),
  COLOR_TABLE(0xf43dff),
  COLOR_TABLE(0xf525ff)
}

local COLOR_SET_PURPLES =
{
  COLOR_TABLE(0xd17cf9),
  COLOR_TABLE(0xc86df9),
  COLOR_TABLE(0xbd5cf8),
  COLOR_TABLE(0xa44ef8),
  COLOR_TABLE(0x943df7),
  COLOR_TABLE(0x920af2)
}

local COLOR_SET_BLUES =
{
  COLOR_TABLE(0xb9fcf9),
  COLOR_TABLE(0xa4ecf9),
  COLOR_TABLE(0x86d4fb),
  COLOR_TABLE(0x6abdfb),
  COLOR_TABLE(0x5cb1fc),
  COLOR_TABLE(0x3997ff)
}

local COLOR_SET_GREENS =
{
  COLOR_TABLE(0x56ffce),
  COLOR_TABLE(0x43ffc2),
  COLOR_TABLE(0x2effb7),
  COLOR_TABLE(0x0affaa),
  COLOR_TABLE(0x00ff9e),
  COLOR_TABLE(0x00ff96)
}

local COLOR_SET_REDS =
{
  COLOR_TABLE(0xff0000),
  COLOR_TABLE(0xff2222),
  COLOR_TABLE(0xff4444)
}

local PALETTES =
{
  {
    itemColors = COLOR_SET_MAGENTAS,
    backgroundColors = {BACKGROUND_COLOR_MAGENTAS_TOP, BACKGROUND_COLOR_MAGENTAS_BOTTOM}
  },
  {
    itemColors = COLOR_SET_BLUES,
    backgroundColors = {BACKGROUND_COLOR_BLUES_TOP, BACKGROUND_COLOR_BLUES_BOTTOM}
  },
  {
    itemColors = COLOR_SET_PURPLES,
    backgroundColors = {BACKGROUND_COLOR_PURPLES_TOP, BACKGROUND_COLOR_PURPLES_BOTTOM}
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CollateColors.getColorForStrobe(index)
  assert(index > 0 and index <= #PALETTES, "Invalid index for palette")

  return PALETTES[index].itemColors[1]
end

function CollateColors.getInitialBackgroundColors()
  return BACKGROUND_COLOR_INITIAL_TOP, BACKGROUND_COLOR_INITIAL_BOTTOM
end

function CollateColors.getNewItemColor()
  return NEW_COLOR
end

function CollateColors.getFrozenItemColor()
  return FROZEN_ITEM_COLOR
end

function CollateColors.getCorrectColor()
  local randomIndex = math.random(#COLOR_SET_GREENS)
  return COLOR_SET_GREENS[randomIndex]
end

function CollateColors.getIncorrectColor()
  local randomIndex = math.random(#COLOR_SET_REDS)
  return COLOR_SET_REDS[randomIndex]
end

function CollateColors.getSingleCorrectColor()
  return CORRECT_COLOR
end

function CollateColors.getSingleIncorrectColor()
  return INCORRECT_COLOR
end

function CollateColors.getPaletteForStyleIndex(index)
  assert(index > 0 and index <= #PALETTES, "No palette for style index")

  return PALETTES[index]
end

function CollateColors.getSlideOutHelperColor()
  return SLIDE_HELPER_COLOR
end

function CollateColors.getHintColor()
  return HINT_LABEL_COLOR
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------
