--- CollateConceptGenerator
-- @author Created by karl on December 17, 2014.

class("CollateConceptGenerator")

require("CollateData")
require("data.generators.UtilityGenerators")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ITEMS_PER_SET = 6

local USED_GAME_CATEGORIES =
{
  "weight",
  "length"
}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function insertFormattedConceptToTable(concept, index, outTable)
  local value = tonumber(concept["value"..index])
  local unit = concept["unit"..index]
  table.insert(outTable, CollateData.getUnitString(value, unit))
end

local function getPaletteStyleIndex(styleName)
  for i, conversionTable in pairs(CollateData.BASE_UNIT_CONVERSIONS) do
    if conversionTable.category == styleName then
      return i
    end
  end

  return nil
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CollateConceptGenerator:reset()
  self.conceptGenerator:reset()
end

function CollateConceptGenerator:generateNextValue()
  local paletteStyleIndex = getPaletteStyleIndex(self.category)
  self.conceptGenerator:setFilters({category = self.category})
  local nextConcept = self.conceptGenerator:next()

  local formattedItems = {}
  local rawUnits = {}
  local rawValues = {}
  for i = 1, ITEMS_PER_SET do
    insertFormattedConceptToTable(nextConcept, i, formattedItems)
    table.insert(rawUnits, nextConcept["unit"..i])
    table.insert(rawValues, nextConcept["value"..i])
  end

  return {
    items = formattedItems,
    units = rawUnits,
    values = rawValues,
    category = self.category,
    palette = CollateColors.getPaletteForStyleIndex(paletteStyleIndex),
    identifier = nextConcept.identifier
  }
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CollateConceptGenerator.new()
  local self = Generator.new()
  Mixin.mix(self, CollateConceptGenerator)
  self.conceptGenerator = GameSession.conceptGenerator

  self.category = TableUtils.getRandomElement(USED_GAME_CATEGORIES)

  return self
end
