--- CollateConfiguration
-- @author Created by Jay Austria on May 15, 2014.

class("CollateConfiguration")

require("GameSession")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

CollateConfiguration.POINTS_PER_CORRECT_RESPONSE = 400
CollateConfiguration.RESPONSES_NEEDED_TO_WIN = 15

CollateConfiguration.POINTS_PER_COMPLETED_ROUND = 2000
CollateConfiguration.NUMBER_OF_ROUNDS = 3

CollateConfiguration.POINTS_PER_PERFECT_SET_BONUS = 200

local CORRECT_ANSWERS_NEEDED_RANK_ONE = 1
local CORRECT_ANSWERS_NEEDED_RANK_TWO = 10
local CORRECT_ANSWERS_NEEDED_RANK_THREE = 14

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CollateConfiguration.getPassScore()
  local minimumScoreFromCompletedRounds = CollateConfiguration.POINTS_PER_COMPLETED_ROUND * CollateConfiguration.NUMBER_OF_ROUNDS
  local correctAnswerPoints = CORRECT_ANSWERS_NEEDED_RANK_ONE * CollateConfiguration.POINTS_PER_CORRECT_RESPONSE
  return correctAnswerPoints + minimumScoreFromCompletedRounds
end

function CollateConfiguration.getRankTwoScore()
  local minimumScoreFromCompletedRounds = CollateConfiguration.POINTS_PER_COMPLETED_ROUND * CollateConfiguration.NUMBER_OF_ROUNDS
  local correctAnswerPoints = CORRECT_ANSWERS_NEEDED_RANK_TWO * CollateConfiguration.POINTS_PER_CORRECT_RESPONSE
  return correctAnswerPoints + minimumScoreFromCompletedRounds
end

function CollateConfiguration.getRankThreeScore()
  local minimumScoreFromCompletedRounds = CollateConfiguration.POINTS_PER_COMPLETED_ROUND * CollateConfiguration.NUMBER_OF_ROUNDS
  local correctAnswerPoints = CORRECT_ANSWERS_NEEDED_RANK_THREE * CollateConfiguration.POINTS_PER_CORRECT_RESPONSE
  return correctAnswerPoints + minimumScoreFromCompletedRounds
end

function CollateConfiguration.getDifficultyKeyFrames()
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Basic = 1.0
      }
    },
    {
      key = 0.075,
      valueProbabilities = {
        Basic = 0.5,
        Easy = 0.45,
        Medium = 0.05
      }
    },

    {
      key = 0.15,
      valueProbabilities = {
        Basic = 0.1,
        Easy = 0.8,
        Medium = 0.1
      }
    },
    {
      key = 0.225,
      valueProbabilities = {
        Easy = 0.6,
        Medium = 0.4
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.4,
        Medium = 0.6
      }
    },
    {
      key = 0.375,
      valueProbabilities =
      {
        Medium = 1.0
      }
    },
    {
      key = 0.45,
      valueProbabilities = {
        Medium = 0.6,
        Hard = 0.4
      }
    },
    {
      key = 0.525,
      valueProbabilities = {
        Medium = 0.35,
        Hard = 0.65
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        Medium = 0.15,
        Hard = 0.80,
        Advanced = 0.05
      }
    },
    {
      key = 0.675,
      valueProbabilities = {
        Medium = 0.15,
        Hard = 0.50,
        Advanced = 0.35
      }
    },
    {
      key = 0.75,
      valueProbabilities = {
        Medium = 0.10,
        Hard = 0.40,
        Advanced = 0.50
      }
    },
    {
      key = 0.8,
      valueProbabilities = {
        Medium = 0.05,
        Hard = 0.40,
        Advanced = 0.55
      }
    },
    {
      key = 0.9,
      valueProbabilities = {
        Hard = 0.40,
        Advanced = 0.6
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 0.3,
        Advanced = 0.7
      }
    }
  }
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------
