--- CollateGame
-- @author Created by Jay Austria on May 15, 2014.

class("CollateGame")
require("CollateScene")
require("CollateColors")
require("CollateCommon")
require("CollateConfiguration")
require("CollateProgressIndicator")
require("CollateConceptGenerator")
require("CollateData")
require("CollateItem")
require("CollateSortableList")
require("CollateSortOrderReminder")
require("CollateTapToContinue")
require("CollateInteractionHelperLayer")

require("actions.advanced.Shake")
require("StandardUIBuilder")
require("graphics.TiledImage")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local UI_COMPONENTS_COLOR = {0.8, 0.8, 0.8}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getOnItemAddedCallback(self)
  local onItemAddedCallback = function(item, isCorrect)
    if isCorrect then
      GameSession.contentTracker:notifyCorrectAnswer(self.concept.identifier, item.string)
      GameSession.incrementCorrectAnswerCount()

      GameSession.increaseScore(CollateConfiguration.POINTS_PER_CORRECT_RESPONSE)
      self.scoreLabel:setScore(GameSession.getScore())
    else
      GameSession.contentTracker:notifyIncorrectAnswer(self.concept.identifier, item.string, adjacentValuesString)
      GameSession.incrementIncorrectAnswerCount()
      self.isPerfectRound = false
    end
  end
  return onItemAddedCallback
end

local function startNewRound(self)
  self.roundNumber = self.roundNumber + 1
  self.isPerfectRound = true
  local concept = self.conceptGenerator:next()
  self.concept = concept
  GameSession.reportSeenConcept(concept.identifier)
  GameSession.contentTracker:notifyPresentedConcept(concept.identifier)

  self.scene:startRoundWithConcept(concept)
end

local function getOnRoundCompletedCallback(self)
  local roundCompletedCallback = function()
    GameSession.increaseScore(CollateConfiguration.POINTS_PER_COMPLETED_ROUND)
    self.scoreLabel:setScore(GameSession.getScore())

    if self.isPerfectRound then
      ScoringUtils.applyPerfectSetBonus(CollateConfiguration.POINTS_PER_PERFECT_SET_BONUS, self.bonusTray)
      wait(self.scene:playPerfectSetAnimation())
      self.scoreLabel:setScore(GameSession.getScore())
    end

    self.scene:clearItems()

    if (self.roundNumber >= CollateConfiguration.NUMBER_OF_ROUNDS) then
      self.uiOverlay:prepareForGameOutro()
      wait(self.scene:playWinAnimation())
      GameSession.notifyGameWin()
    else
      startNewRound(self)
    end
  end
  return roundCompletedCallback
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CollateGame:gameFlow()
  startNewRound(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function configureGameDifficulty(self)
  GameSession.setPassScore(CollateConfiguration.getPassScore())
  GameSession.setScoreThresholds(CollateConfiguration.getRankTwoScore(), CollateConfiguration.getRankThreeScore())

  local difficultyKeyFrames = CollateConfiguration.getDifficultyKeyFrames()
  local difficultyNameChooser = KeyFrameInterpolator.new(difficultyKeyFrames, GameSession.getDifficulty())
  GameSession.conceptGenerator:setDifficultyNameChooser(difficultyNameChooser)

  GameSession.setPerfectSetsMaximum(CollateConfiguration.NUMBER_OF_ROUNDS)
  GameSession.bonusReporter:initializeBonusCount(BonusTypes.TYPES.PERFECT_SET_BONUS)
end

local function createGameElements(self)
  local onItemAddedCallback = getOnItemAddedCallback(self)
  local onRoundCompleteCallback = getOnRoundCompletedCallback(self)
  local scene = CollateScene.new(onItemAddedCallback, onRoundCompleteCallback)
  self:insertAtCenter(scene)
  self.scene = scene
end

local function createUI(self)
  local builder = StandardUIBuilder.new(self.uiOverlay, UI_COMPONENTS_COLOR)

  builder:addPauseButton()
  self.scoreLabel = builder:addScoreLabel()
  self.bonusTray = builder:addBonusTray()
end

-- Constructor ------------------------

function CollateGame.new()
  local self = BaseGame.new()
  Mixin.mix(self, CollateGame)

  configureGameDifficulty(self)

  self.conceptGenerator = CollateConceptGenerator.new()
  self.perfectSetCount = 0
  self.roundNumber = 0

  createGameElements(self)
  createUI(self)

  return self
end
