--- CollateInteractionHelperLayer
-- @author Created by Jay Austria on October 1, 2014.

class("CollateInteractionHelperLayer")
require("CollateCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------
local ARROW_SCALE = 0.6
local HELPER_HIDE_DURATION = 0.2
local ANIMATION_DISTANCE = 5
local OUT_ANIMATION_DURATION = 0.25
local IN_ANIMATION_DURATION = 0.7
local INTERACTION_HELPER_MESSAGE = "Drag and place the block"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------
local function createAnimatingArrow(locationX, locationY)
  local arrow = Sprite.newWithName("shared_arrow.png")

  local arrowWidth = arrow:getWidth()
  local arrowYOffset = -arrowWidth * 0.1

  arrow:setScl(ARROW_SCALE)
  arrow:setLoc(locationX, locationY + arrowYOffset)

  local outDuration = OUT_ANIMATION_DURATION
  local outDistance = ANIMATION_DISTANCE
  local outEase = AHEaseSineOut.new()
  local outAnimation = AHLocationYBy.new(arrow, outDuration, outEase, outDistance)

  local inDuration = IN_ANIMATION_DURATION
  local inDistance = -outDistance
  local inEase = AHEaseSimpleInOut.new()
  local inAnimation = AHLocationYBy.new(arrow, inDuration, inEase, inDistance)

  local animationSequence = AHSequence.new(outAnimation, inAnimation)
  local loopedAnimation = AHLoop.new(animationSequence)

  loopedAnimation:start()

  return arrow
end

local function createLayout(self)
  local interactionText = Label.new(INTERACTION_HELPER_MESSAGE, Font.new(FontFaces.DEFAULT_FONT, 15))
  interactionText:setLoc(Screen.width * 0.5, START_PLATFORM_HEIGHT)
  self:insert(interactionText)

  local leftArrowX = (Screen.width * 0.5 - ITEM_WIDTH * 0.5) * 0.5
  local leftArrow = createAnimatingArrow(leftArrowX, START_PLATFORM_HEIGHT)
  self:insert(leftArrow)

  local rightArrowX = Screen.width - (Screen.width * 0.5 - ITEM_WIDTH * 0.5) * 0.5
  local rightArrow = createAnimatingArrow(rightArrowX, START_PLATFORM_HEIGHT)
  self:insert(rightArrow)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function CollateInteractionHelperLayer.new()
  local self = FilledRectangle.newFullScreen({0,0,0,0.4})
  Mixin.mix(self, CollateInteractionHelperLayer)

  createLayout(self)

  return self
end
