--- CollateItem
-- @author Created by Jay Austria on May 15, 2014.

class("CollateItem")
require("actions.advanced.Shake")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

CollateItem.SLIDE_OUT_SCALE = 0.85

local SCALE_DURATION = 0.1
local SCALE_AMOUNT = 1.06
local ORIGINAL_SCALE = 1.0
local CORRECT_ANSWER_QUICK_DURATION = 0.05
local CORRECT_ANSWER_DURATION = 0.3
local CORRECTION_DURATION = 0.6
local RETURN_HOME_DURATION = 0.3
local BAND_WIDTH_START = 1
local BAND_WIDTH_END = 3
local BAND_STEP_SCALE = 0.7
local BAND_EFFECT_DURATION = 0.3
local RADIATE_DISTANCE = 8
local INVISIBLE_OPACITY = 0
local LABEL_FONT_SIZE = 20
local HINT_LABEL_FONT_SIZE = 14
local COLOR_CHANGE_DELAY_DURATION = 0.2
local NUMBER_OF_BANDS = 3
local SLIDE_OUT_DURATION = 0.5
local TAP_ICON_SCALE = 0.5
local TAP_ICON_OFFSET_X = 3
local TAP_ICON_OFFSET_Y = -3
local TAP_ICON_OPACITY = 0.7
local LABEL_PADDING_X = 10

local Z_INDEX = {
  SLIDE_OUT = 0,
  BACKGROUND = 10,
  TAP_ICON = 20,
  HINT = 30,
  LABEL = 40,
  BAND = 50
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function runBandGroupEffect(self, originX, directionX)
  local height = self:getHeight()
  local bandHeight = height * BAND_STEP_SCALE
  for i = 1, NUMBER_OF_BANDS do
    local band = FilledRectangle.new(BAND_WIDTH_START, bandHeight , CollateColors.getCorrectColor())
    band:setLoc(originX, height * 0.5)
    self:insert(band, Z_INDEX.BAND)

    bandHeight = bandHeight * BAND_STEP_SCALE

    local scaleUpAction = AHScaleXTo.new(band, BAND_EFFECT_DURATION * 0.5, nil, BAND_WIDTH_END)
    local scaleDownAction = AHScaleXTo.new(band, BAND_EFFECT_DURATION * 0.5, nil, BAND_WIDTH_START)

    local scaleAction = AHSequence.new(scaleUpAction, scaleDownAction)

    local radiateAction = AHLocationXBy.new(band, BAND_EFFECT_DURATION, AHEaseSimpleOut.new(), directionX * i)
    local fadeOutAction = AHOpacityTo.new(band, BAND_EFFECT_DURATION, nil, INVISIBLE_OPACITY)

    local bandAction = AHSpawn.new(scaleAction, radiateAction, fadeOutAction)

    local function removeSelfWrapper()
      band:removeSelf()
    end

    local bandActionWithRemove = AHSequence.new(bandAction, removeSelfWrapper)
    bandActionWithRemove:start()
  end
end

local function runBandEffect(self)
  local width = self:getWidth()

  runBandGroupEffect(self, 0, -RADIATE_DISTANCE)
  runBandGroupEffect(self, width, RADIATE_DISTANCE)
end

local function getScaleAction(self, targetScale, scaleDuration)
  local scaleDuration = scaleDuration or SCALE_DURATION
  local scaleAction = AHScaleTo.new(self, scaleDuration, nil, targetScale)
  return scaleAction
end

local function stopScaling(self)
  if self.scaleAction then
    self.scaleAction:stop()
    self.scaleAction = nil
  end
end

local function scaleDown(self)
  stopScaling(self)
  local scaleAction = getScaleAction(self, ORIGINAL_SCALE)
  scaleAction:start()
  self.scaleAction = scaleAction
end

local function scaleUp(self)
  stopScaling(self)
  local scaleAction = getScaleAction(self, SCALE_AMOUNT)
  scaleAction:start()
  self.scaleAction = scaleAction
end

local function returnHome(self)
  if self.returnHomeAction then
    self.returnHomeAction:stop()
  end

  scaleDown(self)

  local returnHomeAction = AHLocationTo.new(self, RETURN_HOME_DURATION, AHEaseSimpleInOut.new(), self.homeX, self.homeY)
  returnHomeAction:start()

  self.returnHomeAction = returnHomeAction
end

local function onDrop(self, doReturnHome)
  if doReturnHome then
    returnHome(self)

    if self.onReturnHomeCallback then
      self.onReturnHomeCallback()
    end
  elseif not self.showingAsIncorrect then
    scaleDown(self)

    if self.onSortCallback then
      self.onSortCallback()
    end
  end
end

local function onPickup(self)
  self:bringToFront()
  scaleUp(self)
end

local function flashBackground(self)
  local flashSyncDelay = AHDelay.new(0.01)
  local flashActionIn = AHColorTo.new(self.background, 0.01, nil, {0.9, 0.9, 0.9})
  local flashActionOut = AHColorTo.new(self.background, 0.2, nil, {1.0, 1.0, 1.0})

  local flashSequence = AHSequence.new(flashSyncDelay, flashActionIn, flashActionOut)
  flashSequence:start()
end

local function onTap(self)
  local oldText = self.label.text

  flashBackground(self)

  self.label:setText(self.alternateText)
  self.alternateText = oldText

  Sound.playSoundEffect("info_button_tap")
end

local function addTapIcon(self)
  local tapIcon = Sprite.newWithName("convert_icon.png")
  tapIcon:setScl(TAP_ICON_SCALE)
  local groupW, groupH = self:getDims()
  local w, h = tapIcon:getEffectiveDims()
  tapIcon:setLoc(groupW - w + TAP_ICON_OFFSET_X, h + TAP_ICON_OFFSET_Y)
  tapIcon:setOpacity(0)
  self:insert(tapIcon, Z_INDEX.TAP_ICON)

  local showAction = AHOpacityTo.new(tapIcon, 0.2, nil, TAP_ICON_OPACITY)
  showAction:start()

  return tapIcon
end

local function enableTapToShowConversion(self)
  if self.string ~= self.alternateText then
    local function onTouchDownWrapper()
      onTap(self)
    end

    local tapIcon = addTapIcon(self)
    Button.makeButton(self, onTouchDownWrapper)

    self.tapIcon = tapIcon
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CollateItem:onClear()
  if self.tapIcon then
    local hideAction = AHOpacityTo.new(self.tapIcon, 0.2, nil, 0.0)
    hideAction:start()
  end
end

function CollateItem:setHome(homeX, homeY)
  self.homeX = homeX
  self.homeY = homeY
end

function CollateItem:getIncorrectAnimation()
  local width, _ = self:getDims()
  local shakeDisplacementX = width * 1.2
  local shakeSpeedMultiplier = 0.5
  local shakeAction = AHAxisOffsetShake.new(self, shakeDisplacementX, AHAxisOffsetShake.SHAKE_AXIS.X, shakeSpeedMultiplier)
  return shakeAction
end

function CollateItem:getCorrectionAnimation()
  local scaleAction = getScaleAction(self, ORIGINAL_SCALE, CORRECTION_DURATION)
  local colorAction = AHColorTo.new(self.background, CORRECTION_DURATION, nil, {1,1,1})
  local correctionAnimation = AHSpawn.new(scaleAction, colorAction)
  return correctionAnimation
end

function CollateItem:setBackgroundColor(color)
  self.background:setColor(unpack(color))
end

function CollateItem:getAnimateBackgroundColorToAction(color)
  local colorChangeAction = AHColorTo.new(self.background, COLOR_CHANGE_DURATION, nil, color)
  return colorChangeAction
end

function CollateItem:animateBackgroundColorTo(color)
  local colorChangeAction = self:getAnimateBackgroundColorToAction(color)
  colorChangeAction:start()
end

function CollateItem:showAsCorrect(doBandEffect, slotColor)
  slotColor = slotColor or CollateColors.getCorrectColor()

  self.background:setColor(unpack(CollateColors.getSingleCorrectColor()))

  local scaleUpAction = getScaleAction(self, SCALE_AMOUNT, CORRECT_ANSWER_QUICK_DURATION)
  local scaleBackAction = getScaleAction(self, ORIGINAL_SCALE, CORRECT_ANSWER_DURATION)

  local colorChangeAction = AHColorTo.new(self.background, COLOR_CHANGE_DURATION, nil, slotColor)
  local colorChangeDelay = AHDelay.new(COLOR_CHANGE_DELAY_DURATION)
  local colorChangeWithDelay = AHSequence.new(colorChangeDelay, colorChangeAction)

  local function bandEffectWrapper()
    if doBandEffect then
      runBandEffect(self)
    end
  end

  local settleInAction = AHSpawn.new(scaleBackAction, colorChangeWithDelay)

  local scaleSequence = AHSequence.new(scaleUpAction, bandEffectWrapper, settleInAction)

  scaleSequence:start()
end

function CollateItem:showAsIncorrect()
  self.background:setColor(unpack(CollateColors.getSingleIncorrectColor()))
  self.showingAsIncorrect = true
end

function CollateItem:setAlternateTextFromUnit(unitTo)
  self.alternateText = CollateData.getUnitConversionString(self.value, self.unit, unitTo)
  enableTapToShowConversion(self)
end

function CollateItem:getSlideOutHintAnimation(hintString)
  local slideAmountY = -self:getHeightPart(CollateItem.SLIDE_OUT_SCALE)
  local slideOutAction = AHLocationYBy.new(self.slideOut, SLIDE_OUT_DURATION, AHEaseSimpleOut.new(), slideAmountY)
  local fadeInAction = AHOpacityTo.new(self.hintLabel, SLIDE_OUT_DURATION, AHEaseSimpleOut.new(), 1)

  self.hintLabel:setText(hintString)

  return AHSpawn.new(slideOutAction, fadeInAction)
end

function CollateItem:getSlideInHintAnimation()
  local slideAmountY = self:getHeightPart(CollateItem.SLIDE_OUT_SCALE)
  local slideInAction = AHLocationYBy.new(self.slideOut, SLIDE_OUT_DURATION, AHEaseSimpleOut.new(), slideAmountY)
  local fadeOutAction = AHOpacityTo.new(self.hintLabel, SLIDE_OUT_DURATION, AHEaseSimpleOut.new(), 0)

  return AHSpawn.new(slideInAction, fadeOutAction)
end

function CollateItem:notifyDNDDrop(container)
  local doReturnHome = (container == nil)
  onDrop(self, doReturnHome)
end

function CollateItem:notifyDNDPickUp()
  onPickup(self)
end

function CollateItem:setOnSortCallback(callback)
  self.onSortCallback = callback
end

function CollateItem:setReturnHomeCallback(callback)
  self.onReturnHomeCallback = callback
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function CollateItem.new(width, height, color, data)
  local self = Group.new(width, height)
  Mixin.mix(self, CollateItem)

  local background = FilledRectangle.new(width, height, color)
  self:insertAtCenter(background, Z_INDEX.BACKGROUND)
  self.background = background

  local labelWidth = width - LABEL_PADDING_X * 2

  local label = Label.new(data.string, Font.new(FontFaces.DEFAULT_FONT, LABEL_FONT_SIZE), labelWidth, Alignment.CENTER)
  self:insertAtCenter(label, Z_INDEX.LABEL)
  local textColor = ColorUtils.getTextColorForBackingColorYIQ(color)
  label:setColor(unpack(textColor))
  self.label = label

  local LINE_SPACING = 5
  local hintLabel = DynamicTextArea.newSimple("", labelWidth, Font.new(FontFaces.DEFAULT_FONT, HINT_LABEL_FONT_SIZE), Alignment.CENTER, Alignment.CENTER, LINE_SPACING)
  hintLabel:setColor(unpack(CollateColors.getHintColor()))
  hintLabel:setOpacity(0)
  self:insertAtLocationFractional(hintLabel, 0.5, -0.25, Z_INDEX.HINT)
  self.hintLabel = hintLabel

  local slideOut = FilledRectangle.new(width, height * CollateItem.SLIDE_OUT_SCALE, color)
  self:insertAtCenter(slideOut, Z_INDEX.SLIDE_OUT)
  self.slideOut = slideOut

  slideOut:overrideColor(background)
  slideOut:inheritOpacity(background)

  self.string = data.string
  self.gameIndex = data.gameIndex
  self.unit = data.unit
  self.alternateText = ""
  self.value = data.value
  self.unit = data.unit

  return self
end
