--- CollateOutroPattern
-- @author Created by Jay Austria on May 15, 2014.

class("CollateOutroPattern")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INITIAL_SIZE = 50
local PIXEL_TRAILS = 5
local INITIAL_LEFT_CENTER = 8

local checkPattern =
{
  "            xxx",
  "           xxx ",
  "          xxx  ",
  "         xxx   ",
  "xxx     xxx    ",
  " xxx   xxx     ",
  "  xxx xxx      ",
  "   xxxxx       ",
  "    xxx        ",
  "     x         "
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function addDroppingPixelAt(script, boxIndex, scriptStep)
  local opacity = 1.0
  for i = 1, PIXEL_TRAILS do
    local newPixel = {}
    newPixel.index =  boxIndex
    newPixel.color = {unpack(CollateColors.getCorrectColor())}

    local pixelsAtStep = script[scriptStep + (i - 1)]
    table.insert(pixelsAtStep, newPixel)

    opacity = opacity - 0.16
    newPixel.color[4] = opacity
  end
end

local function createEmptyScript(numSteps)
  local script = {}
  for i = 1, INITIAL_SIZE do
    script[i] = {}
  end

  return script
end

local function createWaveAtStep(numSteps, script, scriptStep)
  local leftIndex = INITIAL_LEFT_CENTER
  local rightIndex = INITIAL_LEFT_CENTER + 1
  while (leftIndex > 0 or rightIndex <= numSteps) do
    if leftIndex > 0 then
      addDroppingPixelAt(script, leftIndex, scriptStep)
      leftIndex = leftIndex - 1
    end

    if rightIndex <= numSteps then
      addDroppingPixelAt(script, rightIndex, scriptStep)
      rightIndex = rightIndex + 1
    end

    scriptStep = scriptStep + 1
  end
end

local function createOutroScript(numSteps)
  local script = createEmptyScript(INITIAL_SIZE)

  createWaveAtStep(numSteps, script, 1)
  createWaveAtStep(numSteps, script, 10)
  createWaveAtStep(numSteps, script, 20)

  return script
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CollateOutroPattern.getOutroScript(numSteps)
  return createOutroScript(numSteps)
end

function CollateOutroPattern.getCheckPattern()
  return checkPattern
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------
