--- CollateSortOrderReminder
-- @author Created by Jay Austria on May 15, 2014.

class("CollateSortOrderReminder")
require("CollateCommon")
require("widgets.progress.ProgressIndicator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REMINDER_UI_OPACITY = 0.4
local STEP_OFFSET = 1
local ICON_PADDING_Y = 5
local DASH_COUNT = 50
local ANIMATE_DURATION = 0.3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function animateToProgressStep(self, step)
  local repositionSpeed = ANIMATE_DURATION
  local positionY = self.cellHeight * (step + STEP_OFFSET)
  local moveAction = AHLocationYTo.new(self.plusIcon, repositionSpeed, AHEaseSimpleInOut.new(), positionY)
  moveAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CollateSortOrderReminder:hide()
  local hideAction = AHOpacityTo.new(self, 0.1, nil, 0.0)
  hideAction:start()
end

function CollateSortOrderReminder:setProgressStep(step)
  self.super.setProgressStep(self, step)
  animateToProgressStep(self, step)
end

function CollateSortOrderReminder:runUpdateCoroutine()
  if self.updateCoroutine then
    self.updateCoroutine:stop()
  end

  local function updateDashes()
    while(true) do
      local iconH = self.plusIcon:getHeight()
      local _, endY = self.plusIcon:getLoc()
      for _, dash in pairs(self.dashes) do
        local isDashVisible = false

        local _, dashY = dash:getLoc()
        if (dashY + iconH * 0.5 <= endY) then
          isDashVisible = true
        end

        dash:setVisible(isDashVisible)
      end
      coroutine.yield()
    end
  end

  self.updateCoroutine = runInCoroutine(updateDashes)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, plusIcon, minusIcon)
  local w, h = self:getDims()
  local iconH = plusIcon:getHeight()

  plusIcon:setLoc(w * 0.5, iconH * 1.5 + ICON_PADDING_Y)
  minusIcon:setLoc(w * 0.5, iconH * 0.5)

  plusIcon:setAttrLink(MOAIColor.ATTR_OPACITY, self)
  minusIcon:setAttrLink(MOAIColor.ATTR_OPACITY, self)

  self:insert(plusIcon)
  self:insert(minusIcon)

  local startY = iconH
  local endY = startY + self.cellHeight * self:getStepCount()
  local dashes = {}
  for i = 1, DASH_COUNT do
    local y = startY + i * (endY - startY) / DASH_COUNT
    local newDash = Sprite.newWithName("line.png")
    newDash:setLoc(w * 0.5, y)
    self:insert(newDash)
    table.insert(dashes, newDash)
    newDash:setAttrLink(MOAIColor.ATTR_OPACITY, self)
  end
  self.dashes = dashes
end

function CollateSortOrderReminder.new(cellHeight, stepCount)
  local plusIcon = Sprite.newWithName("plus.png")
  local minusIcon = Sprite.newWithName("minus.png")

  local groupWidth = plusIcon:getWidth()
  local groupHeight = cellHeight

  local self = Group.new(groupWidth, groupHeight)
  self.cellHeight = cellHeight
  self.plusIcon = plusIcon
  self.minusIcon = minusIcon

  ProgressIndicator.makeProgressIndicator(self, stepCount)
  Mixin.mix(self, CollateSortOrderReminder)

  createLayout(self, plusIcon, minusIcon)
  self:setAnchorPointBottom()

  self:setOpacity(REMINDER_UI_OPACITY)

  return self
end
