--- CollateTapToContinue
-- @author Created by Jay Austria on July 30, 2014.

class("CollateTapToContinue")
require("widgets.TapToContinue")
require("widgets.AlertPulse")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------
local PULSE_DURATION = 0.8
local PULSE_DELAY = 0.2
local PULSE_LOCATION_X = Screen.width * 0.5
local PULSE_LOCATION_Y = (Screen.height * 0.15) + Screen.safeBottom
local TAP_MESSAGE_FONT_SIZE = 16
local VISIBLE_OPACITY = 1.0
local INVISIBLE_OPACITY = 0.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------
local function fadeLabelToOpacity(self, targetOpacity)
  if self.labelAction then
    self.labelAction:stop()
  end

  local labelAction = AHOpacityTo.new(self.tapToContinueLabel, 0.1, nil, targetOpacity)
  labelAction:start()

  self.labelAction = labelAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------
function CollateTapToContinue:activate()
  self:rearm()

  fadeLabelToOpacity(self, VISIBLE_OPACITY)

  local function pulseActionWrapper()
    for i = 1, 10 do
      wait(self.pulse:pulse())
      waitFor(0.1)
    end
  end
  self.pulseAction = runInCoroutine(pulseActionWrapper)
end

function CollateTapToContinue:deactivate()
  fadeLabelToOpacity(self, INVISIBLE_OPACITY)

  if self.pulseAction then
    self.pulseAction:stop()
    self.pulseAction = nil
  end

  if self.pulse then
    self.pulse:reset()
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------
local function createLayout(self)
  local visualLayer = Group.newFullScreen()

  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DEFAULT_FONT, TAP_MESSAGE_FONT_SIZE))
  tapToContinueLabel:setLoc(Screen.width * 0.5, (START_PLATFORM_HEIGHT * 0.5) + Screen.safeBottom)
  tapToContinueLabel:setOpacity(0.0)
  visualLayer:insert(tapToContinueLabel)
  self.tapToContinueLabel = tapToContinueLabel

  local overrideRadius = nil
  local overrideColor = nil
  local pulse = AlertPulse.new(overrideRadius, overrideColor, PULSE_DURATION, PULSE_DELAY)
  pulse:setLoc(PULSE_LOCATION_X, PULSE_LOCATION_Y)
  visualLayer:insert(pulse)
  self.pulse = pulse

  local timeoutInSeconds = nil
  local autoInsertAtFront = false

  self:insertAtCenter(visualLayer)
end

function CollateTapToContinue.new(callback)
  local self = Group.newFullScreen()
  Mixin.mix(self, TapToContinue)
  Mixin.mix(self, CollateTapToContinue)

  createLayout(self)

  self.callback = callback

  return self
end
