--- CommasAnswerAnimations
-- @author Created by andrewc on June 18, 2019.

class("CommasAnswerAnimations")

require("CommasColors")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_PULSE_RADIUS = 18
local TAP_PULSE_RADIUS = 36

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function pulseColor(parent, color, startScale, endScale)
  local radius = COLOR_PULSE_RADIUS
  local opacity = 0.3
  local duration = 0.333
  local ease = AHEaseExponentialOut.new()
  startScale = 0


  local circle = FilledCircle.new(radius, color)
  parent:insertAtCenter(circle)
  circle:setScl(startScale)
  circle:setOpacity(opacity)

  local scaleUp = AHScaleTo.new(circle, duration, ease, endScale)
  local scaleDown = AHScaleTo.new(circle, 0.333, AHEaseBezier.newFlat({0.25,0,0,1}), 0)
  local removeCircle = function()
    AHSequence.new(scaleDown, AHRemove.new(circle)):start()
  end

  return scaleUp, removeCircle
end

local function getColorPulses(color)
  local pulseGroup = Group.new(COLOR_PULSE_RADIUS * 2, COLOR_PULSE_RADIUS * 2)

  local show1, hide1 = pulseColor(pulseGroup, color, 0.2, 0.45)
  local show2, hide2 = pulseColor(pulseGroup, color, 0.3, 0.75)
  local show3, hide3 = pulseColor(pulseGroup, color, 0.2, 1)

  pulseGroup.show = AHSpawn.new(show1, show2, show3)
  pulseGroup.hide = AHSpawn.new(hide1, hide2, hide3)

  return pulseGroup
end

local function getTapPulse(group, widthFactor, heightFactor)
  local duration = 0.417
  local ease = AHEaseBezier.newFlat({0,0,0,1})
  local scaleStart = 0.25
  local opacityStart = 0.75
  local radius = TAP_PULSE_RADIUS
  local color = CommasColors.WHITE

  local circle = FilledCircle.new(radius, color)
  circle:setScl(scaleStart)
  circle:setOpacity(opacityStart)
  group:insertAtLocationFractional(circle, widthFactor, heightFactor)
  CommasCommon.offsetItemToComma(circle)

  local animation = AHSpawn.new()
  animation:addAction(AHScaleTo.new(circle, duration, ease, 1))
  animation:addAction(AHOpacityTo.new(circle, duration, ease, 0))

  return animation
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasAnswerAnimations.getTapAnimation(group, widthFactor, heightFactor)
  return getTapPulse(group, widthFactor, heightFactor)
end

function CommasAnswerAnimations.getSuccessObject()
  local color = CommasColors.CORRECT_GREEN
  return getColorPulses(color)
end

function CommasAnswerAnimations.getFailureObject()
  local color = CommasColors.INCORRECT_RED
  return getColorPulses(color)
end
