--- CommasBackground
-- @author Created by andrewc on June 14, 2019.

class("CommasBackground")

require("widgets.TapToContinue")
require("CommasFilledComma")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ERROR_BACKGROUND = "wrong-answer-overlay.png"

local BASE_COMMA_SIZE = ReferenceDevice.getScaledWidth(285)

local Z_INDEX = {
  GRADIENT = 100,
  ERROR = 110
}

local EASE = AHEaseBezier.newFlat({0.25,0.1,0.08,1})
local DURATION = 0.333

local COMMAS = {
  {
    COLOR = {18,161,229},
    Z_INDEX = 10,
    DELAY = 0,
    SCALE = {
      START = 1.65,
      FINAL = 4.30
    },
    POSITION = {
      START = {x=374,y=1334-1018},
      FINAL = {x=57,y=1334-860}
    },
    ROTATION = {
      START = 0,
      FINAL = 51
    },
    OPACITY = {
      START = 1,
      FINAL = 1
    }
  },
  {
    COLOR = {18,8,36},
    Z_INDEX = 20,
    DELAY = 0,
    SCALE = {
      START = 0.5,
      FINAL = 1.88
    },
    POSITION = {
      START = {x=374,y=1334-800},
      FINAL = {x=319,y=1334-925}
    },
    OPACITY = {
      START = 0,
      FINAL = 1
    },
    ROTATION = {
      START = 0,
      FINAL = 69
    }
  },
  {
    COLOR = {230,159,182},
    Z_INDEX = 30,
    DELAY = 0.1,
    SCALE = {
      START = 0.5,
      FINAL = 1.62
    },
    POSITION = {
      START = {x=374,y=1334-800},
      FINAL = {x=396,y=1334-950}
    },
    OPACITY = {
      START = 0,
      FINAL = 1
    },
    ROTATION = {
      START = 0,
      FINAL = 55
    }
  },
  {
    COLOR = {0,67,150},
    Z_INDEX = 40,
    DELAY = 0.2,
    SCALE = {
      START = 0.5,
      FINAL = 1.28
    },
    POSITION = {
      START = {x=374,y=1334-800},
      FINAL =  {x=440,y=1334-965}
    },
    OPACITY = {
      START = 0,
      FINAL = 0.9
    },
    ROTATION = {
      START = 0,
      FINAL = 36
    }
  },
  {
    COLOR = {17,191,255},
    Z_INDEX = 50,
    DELAY = 0.3,
    SCALE = {
      START = 0.5,
      FINAL = 1.05
    },
    POSITION = {
      START = {x=374,y=1334-800},
      FINAL = {x=450,y=1334-987}
    },
    OPACITY = {
      START = 0,
      FINAL = 0.9
    },
    ROTATION = {
      START = 0,
      FINAL = 11
    }
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function addNewComma(color)
  local color = ColorUtils.colorRGBToStd(unpack(color))
  local comma = CommasFilledComma.new(BASE_COMMA_SIZE, BASE_COMMA_SIZE, color)
  comma:setAnchorPointCenter()
  return comma
end

local function getCommaScaleAnim(comma, startScale, endScale)
  comma:setScl(startScale)

  local scaleAnim = AHScaleTo.new(comma, DURATION, EASE, endScale)
  return scaleAnim
end

local function getCommaPositionAnim(comma, startPosition, endPosition)
  local startX = ReferenceDevice.getScaledWidth(startPosition.x)/2
  local startY = ReferenceDevice.getScaledHeight(startPosition.y)/2
  comma:setLoc(startX, startY)

  local endX = ReferenceDevice.getScaledWidth(endPosition.x)/2
  local endY = ReferenceDevice.getScaledHeight(endPosition.y)/2

  local posAnim = AHLocationTo.new(comma, DURATION, EASE, endX, endY)
  return posAnim
end

local function getCommaRotationAnim(comma, startRotation, endRotation)
  comma:setRot(-startRotation)

  local rotAnim = AHRotationTo.new(comma, DURATION, EASE, -endRotation)
  return rotAnim
end

local function getCommaOpacityAnim(comma, startOpacity, endOpacity)
  comma:setOpacity(startOpacity)

  local opacityAnim = AHOpacityTo.new(comma, DURATION, EASE, endOpacity)
  return opacityAnim
end

local function buildStartAnimation(self)
  local fullAnim = AHSpawn.new()

  local fadeInAtStartAnim = AHSpawn.new()
  local fadeInAtStartDuration = 0.083
  local fadeInAtStartEase = AHEaseBezier.newFlat({0.62,0,0.19,0.97})
  fadeInAtStartAnim:addAction(AHOpacityTo.new(self, fadeInAtStartDuration, fadeInAtStartEase, 1))

  local playSound = function()
    Sound.playSoundEffect("intro")
  end

  fadeInAtStartAnim:addAction(playSound)

  for _, data in ipairs(COMMAS) do
    local color = data.COLOR
    local comma = addNewComma(color)
    local anim = AHSpawn.new()
    if data.SCALE then
      anim:addAction(getCommaScaleAnim(comma, data.SCALE.START, data.SCALE.FINAL))
    end
    if data.POSITION then
      anim:addAction(getCommaPositionAnim(comma, data.POSITION.START, data.POSITION.FINAL))
    end
    if data.ROTATION then
      anim:addAction(getCommaRotationAnim(comma, data.ROTATION.START, data.ROTATION.FINAL))
    end
    if data.OPACITY then
      anim:addAction(getCommaOpacityAnim(comma, data.OPACITY.START, data.OPACITY.FINAL))
    end
    self.commasGroup:insert(comma, data.Z_INDEX)
    fullAnim:addAction(AHDelayed.new(data.DELAY, anim))
    if data.DELAY == 0 and data.OPACITY.START > 0 then
      comma:setOpacity(0)
      fadeInAtStartAnim:addAction(AHOpacityTo.new(comma, fadeInAtStartDuration, fadeInAtStartEase, data.OPACITY.START))
    end
  end

  return AHSequence.new(fadeInAtStartAnim, fullAnim)
end

local function buildCloseAnimation(self)
  local duration = CommasCommon.ROUND_TRANSITION_IN
  local ease = AHEaseBezier.newFlat({0.75,0,0,1})

  local closeAnimation = AHSpawn.new()
  closeAnimation:addAction(self.background:newGradientToAction(duration, ease, {0,0,0,0.3}, {0.012,0.047,0.118,0.8}))
  closeAnimation:addAction(self:getBackgroundScaleAnimation(duration))

  return closeAnimation
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasBackground:getBackgroundScaleAnimation(duration)
  local duration = duration or 0.25 + CommasCommon.ROUND_TRANSITION_IN
  local ease = AHEaseBezier.newFlat({0.49,0,0.31,1})
  local soundEffect = function() Sound.playSoundEffect("transition") end
  local scale = AHScaleTo.new(self.commasGroup, duration, ease, self.commasGroup:getScl() * 1.02)
  return AHSpawn.new(scale, soundEffect)
end

function CommasBackground:getShowErrorAnimation()
  local duration = 0.333
  local easeIn = AHEaseBezier.newFlat({0.62,0,0.19,0.97})
  return AHOpacityTo.new(self.errorBackground, duration, easeIn, 0.9)
end

function CommasBackground:getHideErrorAnimation()
  local duration = 0.333
  local easeOut = AHEaseBezier.newFlat({0.25,0,0,1})
  return AHOpacityTo.new(self.errorBackground, duration, easeOut, 0)
end

function CommasBackground:startGame()
  local fadeInAnimation = AHOpacityTo.new(self, 0.333, AHEaseBezier.newFlat({0.62,0,0.19,0.97}), 1)
  local commasEnterAnimation = buildStartAnimation(self)

  local backdropAnimation = AHSequence.new(fadeInAnimation, commasEnterAnimation)

  local background = RectangleGradient.new(Screen.width, Screen.height, {0,0,0,0}, {0.0,0.0,0.0,0.0})
  self:insertAtCenter(background, Z_INDEX.GRADIENT)
  self.background = background

  local delay = 0.8
  local closeAnimation = buildCloseAnimation(self)

  fullAnimation = AHSpawn.new(
    backdropAnimation,
    AHDelayed.new(delay, closeAnimation))
  wait(fullAnimation:start())
end

function CommasBackground:getCommasGroup()
  return self.commasGroup
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CommasBackground.new()
  local color = CommasColors.BACKGROUND_COLOR
  local width, height = Screen.width, Screen.height

  local self = FilledRectangle.new(width, height, color)
  Mixin.mix(self, CommasBackground)

  self:setOpacity(0)

  local errorBackground = Sprite.newWithName(ERROR_BACKGROUND)
  errorBackground:setEffectiveDims(width, height)
  errorBackground:setOpacity(0)
  self:insertAtCenter(errorBackground, Z_INDEX.ERROR)
  self.errorBackground = errorBackground

  local commasGroup = Group.newFullScreen()
  self:insertAtCenter(commasGroup)

  self.commasGroup = commasGroup

  return self
end
