--- CommasCommaHolder
-- @author Created by andrewc on June 24, 2019.

class("CommasCommaHolder")

require("CommasAnswerAnimations")
require("CommasConfetti")
require("CommasColors")
require("graphics.LinePathTracer")
require("graphics.primitives.Checkmark")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LINE_WIDTH = 1
local BLUR_MARGIN = 5
local RADIUS = 18
local COLOR = CommasColors.WHITE

local OPACITY = {
  BORDER = 0.21,
  SELECTED_BORDER = 0.66,
  FILL = 0.1,
  SHADOW = 0.3
}

local Z_INDEX = {
  SHADOW = 10,
  FILL = 20,
  BORDER = 30,
  COMMA = 40
}

local LINE_PARAMS = {
  lineWidth = 3,
  maxVisibleViews = 4,
  maxTailLength = 40
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getTargetOffset(self, obj)
  local offsetX, offsetY = self:getWorldDistance(obj, 0.5, 0.5, nil, 1, 0.5, nil)
  offsetX = offsetX + self:getWidthPart(0.6)
  offsetY = offsetY + (obj:getHeightPart(0.1))

  return offsetX, offsetY
end

local function getLineTraceAnimation(self, obj, toText)
  local startX = self:getWidthPart(0.5)
  local startY = self:getHeightPart(0.5)

  local offsetX, offsetY = getTargetOffset(self, obj)

  local linePath = LinePathTracer.new(LINE_PARAMS)

  self:insert(linePath)

  local lineAnim
  local duration = CommasCommon.LASER_TIME
  if toText then
    lineAnim = linePath:createTracePathAnimation(startX, startY, offsetX, offsetY, duration)
  else
    lineAnim = linePath:createTracePathAnimation(offsetX, offsetY, startX, startY, duration)
  end

  return AHSequence.new(lineAnim, AHRemove.new(linePath))
end

local function getCheckmarkSwap(self)
  return function()
    self.commaLabel:setOpacity(0)
    self.checkmark:setOpacity(1)
  end
end

local function getPopAnimation(self)
  local popAnim = AHSpawn.new()

  local fillFade = AHOpacityTo.new(self.fill, 0.05, AHEaseLinear.new(), 1)
  local swapInCheckmark = getCheckmarkSwap(self)
  popAnim:addAction(AHSequence.new(fillFade, swapInCheckmark))

  local scaleDown = AHScaleTo.new(self, 0.05, AHEaseLinear.new(), 0.7)
  popAnim:addAction(scaleDown)

  local scaleCommaDown = AHScaleTo.new(self.commaLabel, 0.05, AHEaseLinear.new(), 0.7)
  popAnim:addAction(scaleCommaDown)

  local upEase = AHEaseBezier.newFlat({0,0,0,1})
  local scaleUp = AHScaleTo.new(self, 0.5, upEase, 1)
  popAnim:addAction(AHDelayed.new(0.05, scaleUp))

  local fadeEase = AHEaseBezier.newFlat({0,0,0.67,1})
  local glowFade = AHOpacityTo.new(self.shadow, 0.167, fadeEase, 0)
  popAnim:addAction(AHDelayed.new(0.05, glowFade))

  return popAnim
end

local function getFlashAnimation(self)
  local simpleBezierEase = AHEaseBezier.newFlat({0,0,0,1})

  local flashAnim = AHSpawn.new()

  local fillFadeIn = AHOpacityTo.new(self.fill, 0.05, AHEaseLinear.new(), 1)
  local fillFadeOut = AHOpacityTo.new(self.fill, 0.5, simpleBezierEase, 0.1)
  flashAnim:addAction(AHSequence.new(fillFadeIn, fillFadeOut))

  local scaleDown = AHScaleTo.new(self, 0.05, AHEaseLinear.new(), 0.7)
  local scaleUp = AHScaleTo.new(self, 0.5, simpleBezierEase, 1)
  flashAnim:addAction(AHSequence.new(scaleDown, scaleUp))

  local scaleCommaDown = AHScaleTo.new(self.commaLabel, 0.05, AHEaseLinear.new(), 0.7)
  local scaleCommaUp = AHScaleTo.new(self.commaLabel, 0.5, simpleBezierEase, 1)
  flashAnim:addAction(AHSequence.new(scaleCommaDown, scaleCommaUp))

  return flashAnim
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasCommaHolder:getCloseAnimation()
  local animation = AHSpawn.new()
  local duration = CommasCommon.ROUND_TRANSITION_OUT
  local ease1 = AHEaseBezier.newFlat({0.75,0,0,1})
  local ease2 = AHEaseBezier.newFlat({0.75,0,0.98,1.02})
  animation:addAction(AHOpacityTo.new(self.fill, duration, ease1, 0.0))
  animation:addAction(AHOpacityTo.new(self.checkmark, duration, ease1, 0))
  animation:addAction(AHScaleTo.new(self, duration, ease2, 0.55))
  animation:addAction(AHScaleTo.new(self.commaLabel, duration, ease2, 0.55))

  local fadeAnim = AHSpawn.new()

  fadeAnim:addAction(AHOpacityTo.new(self, duration/2, ease2, 0))
  fadeAnim:addAction(AHOpacityTo.new(self.commaLabel, duration/2, ease2, 0))
  fadeAnim:addAction(AHOpacityTo.new(self.border, duration/2, ease2, 0))

  animation:addAction(AHDelayed.new(duration/2, fadeAnim))
  return animation
end

function CommasCommaHolder:getOpenAnimation()
  local duration = CommasCommon.ROUND_TRANSITION_IN
  local ease = AHEaseBezier.newFlat({0,0,0,1})
  local startScl = 0.55
  local animation = AHSpawn.new()
  animation:addAction(AHScaleTo.new(self, duration, ease, 1))
  animation:addAction(AHScaleTo.new(self.commaLabel, duration, ease, 1))
  animation:addAction(AHOpacityTo.new(self, duration, ease, 1))
  animation:addAction(AHOpacityTo.new(self.commaLabel, duration, ease, 1))
  local setup = function()
    self:setScl(startScl)
    self.commaLabel:setScl(startScl)
    self:setOpacity(0)
    self.commaLabel:setOpacity(0)
  end
  return AHSequence.new(setup, animation)
end

function CommasCommaHolder:animateCommaToItem(obj, isCorrect, isEndOfGame)
  local fullAnim = AHSpawn.new()
  if (not isEndOfGame) then
    fullAnim:addAction(function()
      waitFor(0.167)
      Sound.playSoundEffect("laserbeam")
    end)
  end

  local lineAnim = getLineTraceAnimation(self, obj, true)
  fullAnim:addAction(AHDelayed.new(0.217, lineAnim))

  if isCorrect then
    local holderPopAnimation = getPopAnimation(self)
    fullAnim:addAction(AHDelayed.new(0.167, holderPopAnimation))


    local confetti = CommasConfetti.new(RADIUS * 4)
    self.delegate:insert(confetti)
    confetti:alignToProp(obj)
    confetti:offsetX(obj:getWidth())
    CommasCommon.offsetItemToComma(confetti)
    fullAnim:addAction(AHDelayed.new(0.5, confetti:getAnimation()))

    if (not isEndOfGame) then
      local impact = self.delegate:getScene():getImpactAnimation()
      fullAnim:addAction(AHDelayed.new(0.45, impact))
      fullAnim:addAction(AHDelayed.new(0.45, function() Sound.playSoundEffect("correct_hit") end))
    end
  else
    local holderFlashAnimation = getFlashAnimation(self)
    fullAnim:addAction(AHDelayed.new(0.167, holderFlashAnimation))

    local textShake = self.delegate:getScene():getShakeAnimation()
    fullAnim:addAction(AHDelayed.new(CommasCommon.LASER_TIME + 0.1085, textShake))
    if (not disableAudio) then
      fullAnim:addAction(AHDelayed.new(0.45, function() Sound.playSoundEffect("incorrect_hit") end))
    end
  end

  wait(fullAnim:start())
end

function CommasCommaHolder:insertAndAnimateComma(initialItem, isCorrect)
  local fullAnim = AHSpawn.new()

  fullAnim:addAction(AHDelayed.new(0.167, function()
    Sound.playSoundEffect("laserbeam")
  end))

  local lineAnim = getLineTraceAnimation(self, initialItem, false)
  fullAnim:addAction(AHDelayed.new(0.167, lineAnim))

  if isCorrect then
    local confetti = CommasConfetti.new(RADIUS * 4)
    self.delegate:insert(confetti)
    confetti:alignToProp(initialItem)
    confetti:offsetX(initialItem:getWidth())
    CommasCommon.offsetItemToComma(confetti)
    fullAnim:addAction(AHDelayed.new(0.167, confetti:getAnimation()))

    local impact = self.delegate:getScene():getImpactAnimation()
    fullAnim:addAction(AHDelayed.new(0.167, impact))

    local popAnim = getPopAnimation(self)
    fullAnim:addAction(AHDelayed.new(0.4, popAnim))

    fullAnim:addAction(AHDelayed.new(0.4, function() Sound.playSoundEffect("correct_hit") end))
  else
    local holderFlashAnimation = getFlashAnimation(self)
    fullAnim:addAction(AHDelayed.new(0.4, holderFlashAnimation))

    local textShake = self.delegate:getScene():getShakeAnimation()
    fullAnim:addAction(AHDelayed.new(0.65, textShake))
    fullAnim:addAction(AHDelayed.new(0.4, function() Sound.playSoundEffect("incorrect_hit") end))
  end

  wait(fullAnim:start())
end

function CommasCommaHolder:setSelected(selected)
  if selected then
    local animation = AHSpawn.new()
    local duration = 0.167
    local ease = AHEaseLinear.new()
    animation:addAction(AHOpacityTo.new(self.border, duration, ease, OPACITY.SELECTED_BORDER))
    animation:addAction(AHOpacityTo.new(self.shadow, duration, ease, OPACITY.SHADOW))
    animation:start()
  else
    local animation = AHSpawn.new()
    local duration = 0.167
    local ease = AHEaseLinear.new()
    animation:addAction(AHOpacityTo.new(self.border, duration, ease, OPACITY.BORDER))
    animation:addAction(AHOpacityTo.new(self.shadow, duration, ease, 0))
    animation:start()
  end
end

function CommasCommaHolder:markAsFound()
  self.fill:setOpacity(1)
  local swap = getCheckmarkSwap(self)
  swap()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local border = HollowCircle.new(RADIUS, COLOR, LINE_WIDTH, 0)
  border:setOpacity(OPACITY.BORDER)
  local shadow = HollowCircle.new(RADIUS+LINE_WIDTH, COLOR, 0, BLUR_MARGIN)
  shadow:setOpacity(0)
  local fill = FilledCircle.new(RADIUS, COLOR)
  fill:setOpacity(OPACITY.FILL)

  local commaSize = RADIUS * 0.3
  local commaLabel = CommasFilledComma.new(commaSize, commaSize, {1,1,1})

  self:insertAtLocationFractional(commaLabel, 0.51, 0.46, Z_INDEX.COMMA)

  local checkmark = FilledCheckmark.new(self:getWidthPart(0.3), nil, nil, ColorUtils.colorRGBToStd(42,97,132))
  checkmark:setOpacity(0)
  self:insertAtCenter(checkmark, Z_INDEX.COMMA)

  self:insertAtCenter(border, Z_INDEX.BORDER)
  self:insertAtCenter(shadow, Z_INDEX.SHADOW)
  self:insertAtCenter(fill, Z_INDEX.FILL)

  self.border = border
  self.shadow = shadow
  self.fill = fill
  self.checkmark = checkmark
  self.commaLabel = commaLabel
end

-- Constructor ------------------------

function CommasCommaHolder.new(delegate)
  local self = Group.new(RADIUS * 2, RADIUS * 2)
  Mixin.mix(self, CommasCommaHolder)

  self.delegate = delegate

  createLayout(self)

  return self
end
