--- CommasConfetti
-- @author Created by andrewc on July 02, 2019.

class("CommasConfetti")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PARTICLE_COLORS = {
  {0.432,0.435,1},
  {0.849,0.561,0.651},
  {0.29,0.941,0.973},
  {0.11,0.373,0.753},
  {0.114,0.835,1}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getNoise(min, max)
  return min + ((max - min) * math.random())
end

local function makePartical(size, color)
  return FilledCircle.new(size, color)
end

local function makeParticalRing(ringSize, particleSize, particleColor, maxParticles)
  local ring = Group.new(ringSize, ringSize)

  local duration = 0.7
  local speedEase = AHEaseBezier.newFlat({0,0,0,1})

  local anim = AHSpawn.new()

  local numParticles = math.random(maxParticles/2, maxParticles)
  for i=1, numParticles do
    local particle = makePartical(particleSize * getNoise(0.8, 1.2), particleColor)
    ring:insertAtCenter(particle)

    local rotation = ((i / numParticles) * getNoise(0.8, 1.2)) * 2 * math.pi
    local destinationNoise = getNoise(0.7, 1)
    local x0 = (ringSize / 2) + math.cos(rotation) * ringSize * destinationNoise
    local y0 = (ringSize / 2) + math.sin(rotation) * ringSize * destinationNoise
    local particleMove = AHLocationTo.new(particle, duration , speedEase, x0, y0)
    local particleFade = AHScaleTo.new(particle, duration, speedEase, 0)

    anim:addAction(particleMove)
    anim:addAction(particleFade)
  end

  ring:linkOpacityAttrToChildren(true)

  ring.anim = AHSequence.new(function() ring:setOpacity(1) end, anim)

  ring:setOpacity(0)
  return ring
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasConfetti:getAnimation()
  return self.animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CommasConfetti.new(maxSize)
  local self = Group.new(maxSize, maxSize)
  Mixin.mix(self, CommasConfetti)

  local animation = AHSpawn.new()

  for _, color in ipairs(PARTICLE_COLORS) do
    local ring = makeParticalRing(maxSize, 8, color, 6)
    self:insertAtCenter(ring)

    animation:addAction(ring.anim)
  end

  self.animation = animation

  return self
end
