--- CommasConfiguration
-- @author Created by andrewc on May 29, 2019.

class("CommasConfiguration")

require("CommasCommon")
require("data.keyframes.KeyframeSet")
require("data.generators.LearningFocusedRoundGenerator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

CommasConfiguration.GAME_DURATION = 60
CommasConfiguration.INCORRECT_RESPONSE_TIME_PENALTY = 3

CommasConfiguration.TIME_REMAINING_TO_START_RESURFACING_MISTAKES =
  CommasConfiguration.GAME_DURATION *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING

-- Scoring -------------------------

CommasConfiguration.POINTS_FOR_FULL_SPEED_BONUS = 50
CommasConfiguration.SPEED_BONUS_WINDOW = 5
CommasConfiguration.SPEED_BONUS_DECAY_START = 1

local POINTS_PER_COMMA = 1000
local COMMA_POINTS_DECAY_RATE = 0.92

local POINTS_PER_COMPLETED_ROUND = 1000
local ROUND_COMPLETED_POINTS_DECAY_RATE = 0.92

-- Difficulty -------------------------

local ESTIMATED_COMMAS_PER_ROUND_FOR_SCORE = 2
local ROUNDS_COMPLETED_RANK_THREE = 6
local ROUNDS_COMPLETED_RANK_TWO = 4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getEstimatedPointsForCompletedRounds(roundsCompleted)
  local estimatedScore = 0
  for i = 1, roundsCompleted do
    estimatedScore = estimatedScore + CommasConfiguration.getPointsForRoundCompleted(i)
  end

  local commasFound = math.floor(roundsCompleted * ESTIMATED_COMMAS_PER_ROUND_FOR_SCORE)
  for i = 1, commasFound do
    estimatedScore = estimatedScore + CommasConfiguration.getPointsForCorrectComma(i)
  end

  return estimatedScore
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasConfiguration.getPointsForCorrectComma(correctCommasCount)
  return POINTS_PER_COMMA * (COMMA_POINTS_DECAY_RATE ^ correctCommasCount)
end

function CommasConfiguration.getPointsForRoundCompleted(roundsCompletedCount)
  return POINTS_PER_COMPLETED_ROUND * (ROUND_COMPLETED_POINTS_DECAY_RATE ^ roundsCompletedCount)
end

function CommasConfiguration.getPassScore()
  return POINTS_PER_COMMA + POINTS_PER_COMPLETED_ROUND
end

function CommasConfiguration.getRankTwoScore()
  return getEstimatedPointsForCompletedRounds(ROUNDS_COMPLETED_RANK_TWO)
end

function CommasConfiguration.getRankThreeScore()
  return getEstimatedPointsForCompletedRounds(ROUNDS_COMPLETED_RANK_THREE)
end

function CommasConfiguration.getDifficultyKeyFrames()
  local keyFrames = {
    {
      key = 0.0,
      valueProbabilities = {
        Basic = 1,
      }
    },
    {
      key = 0.1,
      valueProbabilities = {
        Basic = 0.6,
        Easy = 0.4
      }
    },
    {
      key = 0.2,
      valueProbabilities = {
        Basic = 0.1,
        Easy = 0.6,
        Medium = 0.3
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.6,
        Medium = 0.4
      }
    },
    {
      key = 0.4,
      valueProbabilities = {
        Easy = 0.4,
        Medium = 0.5,
        Hard = 0.1
      }
    },
    {
      key = 0.5,
      valueProbabilities = {
        Easy = 0.3,
        Medium = 0.5,
        Hard = 0.2
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        Easy = 0.2,
        Medium = 0.5,
        Hard = 0.3
      }
    },
    {
      key = 0.7,
      valueProbabilities = {
        Medium = 0.5,
        Hard = 0.5
      }
    },
    {
      key = 0.8,
      valueProbabilities = {
        Medium = 0.3,
        Hard = 0.6,
        Advanced = 0.1
      }
    },
    {
      key = 0.9,
      valueProbabilities = {
        Medium = 0.1,
        Hard = 0.5,
        Advanced = 0.4
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 0.3,
        Advanced = 0.7
      }
    }
  }

  return keyFrames
end
