--- CommasEncourageText
-- @author Created by andrewc on July 18, 2019.

class("CommasEncourageText")

require("CommasCommon")
require("CommasColors")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKING_HEIGHT_MOD = 0.4

local encouragement = {
  "one comma left",
  "two commas left",
  "three commas left"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function waitForSelfToFadeOut(self)
  if self.currAnim then
    self.currAnim:stop()
  end

  if self:getOpacity() > 0 then
    local duration = 0.167
    local ease = AHEaseLinear.new()
    local fadeOut = AHSpawn.new()
    fadeOut:addAction(AHOpacityTo.new(self.label, duration, ease, 0))
    fadeOut:addAction(AHOpacityTo.new(self.backing, duration, ease, 0))
    wait(fadeOut:start())
  end
end


local function getShowAnim(self, numLeft)
  local fullAnim = AHSequence.new()
  -- inital fadeIn
  local duration = 0.167
  local ease = AHEaseLinear.new()
  local fadeInAnim = AHSpawn.new()
  fadeInAnim:addAction(AHOpacityTo.new(self.label, duration, ease, 1))
  fadeInAnim:addAction(AHOpacityTo.new(self.backing, duration, ease, 0.4))

  fullAnim:addAction(fadeInAnim)

  --pause
  duration = (numLeft > 0) and 1 or 0.5
  fullAnim:addAction(AHDelay.new(duration))

  --fade out
  duration = 0.167
  ease = AHEaseLinear.new()
  local fadeOutAnim = AHSpawn.new()
  fadeOutAnim:addAction(AHOpacityTo.new(self.label, duration, ease, 0))
  fadeOutAnim:addAction(AHOpacityTo.new(self.backing, duration, ease, 0))

  fullAnim:addAction(fadeOutAnim)

  return fullAnim
end

local function setText(self, text)
  self.label:setText(text)
  local backingMargin = self.label:getHeightPart(BACKING_HEIGHT_MOD)
  local targetWidth = self.label:getWidth() + backingMargin*4
  local targetHeight = self.label:getHeight() + backingMargin

  self.backing:setAttr(MOAITransform2D.ATTR_X_SCL, targetWidth/self.backing:getWidth())
  self.backing:setAttr(MOAITransform2D.ATTR_Y_SCL, targetHeight/self.backing:getHeight())
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasEncourageText:encourage(numLeft, delay)
  waitForSelfToFadeOut(self)
  delay = delay or 0

  if ((numLeft == 0) or (numLeft > #encouragement)) then
    return
  end

  setText(self, encouragement[numLeft])

  local showAnim = AHDelayed.new(delay, getShowAnim(self, numLeft))

  self.currAnim = showAnim
  showAnim:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CommasEncourageText.new()
  local label = Label.new("one comma left", CommasCommon.FONT.ENCOURAGEMENT)
  local self = Group.new(label:getWidth(), label:getHeight())
  Mixin.mix(self, CommasEncourageText)

  local backing = FilledRoundedRectangle.new(label:getWidth(), label:getHeight(), 3, CommasColors.BLACK)
  backing:setOpacity(0)
  self:insertAtCenter(backing)

  label:setOpacity(0)
  self:insertAtCenter(label)

  self.label = label
  self.backing = backing

  return self
end
