--- CommasFilledComma
-- @author Created by andrewc on June 17, 2019.

class("CommasFilledComma")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TAIL_HEIGHT_FRACTION = 0.588

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getBezierPoints(a,b,c,d,t)
  t = t or 10
  local points = {}
  for i=0,t do
    table.insert(points, MathUtils.bezier(a,b,c,d,i/t))
  end
  return points
end

local function getCurvePoints(xOrigin, xControl1, xControl2, xFinal, yOrigin, yControl1, yControl2, yFinal)
  local xPoints = getBezierPoints(xOrigin, xControl1, xControl2, xFinal)
  local yPoints = getBezierPoints(yOrigin, yControl1, yControl2, yFinal)

  local points = {}
  for i=1,#xPoints do
    table.insert(points, {x=xPoints[i],y=yPoints[i]})
  end

  return points
end

local function getEntireCurve(bezierPoints)
  local points = {}
  for i=4, #bezierPoints, 3  do
    local origin = bezierPoints[i - 3]
    local control1 = bezierPoints[i - 2]
    local control2 = bezierPoints[i - 1]
    local final = bezierPoints[i]
    local curvePoints = getCurvePoints(origin.x, control1.x, control2.x, final.x, origin.y, control1.y, control2.y, final.y)
    TableUtils.concatenateArrays(points, curvePoints)
  end
  return points
end

local function getInnerCurvePoints()
  local controlPoints = {
    {x=0.009,y=0.194},
    {x=0.112,y=0.21},
    {x=0.196,y=0.233},
    {x=0.265,y=0.263},
    {x=0.334,y=0.293},
    {x=0.39,y=0.325},
    {x=0.433,y=0.36},
    {x=0.457,y=0.381},
    {x=0.478,y=0.403},
    {x=0.494,y=0.424},
    {x=0.504,y=0.438},
    {x=0.511,y=0.452},
    {x=0.519,y=0.467},
    {x=0.534,y=0.503},
    {x=0.537,y=0.537},
    {x=0.532,y=0.568}
  }
  return getEntireCurve(controlPoints)
end

local function getOuterCurvePoints()
  local controlPoints = {
    {x=1,y=0.607},
    {x=1,y=0.459},
    {x=0.923,y=0.332},
    {x=0.765,y=0.226},
    {x=0.61,y=0.119},
    {x=0.358,y=0.044},
    {x=0.009,y=0}
  }
  return getEntireCurve(controlPoints)
end


local function getTailPoints()
  local centerPoint = {x=0.55,y=0.19}
  local initalPoint = {x=0.532,y=0.568}
  local points = {centerPoint, initalPoint}

  TableUtils.concatenateArrays(points, getOuterCurvePoints())
  TableUtils.concatenateArrays(points, getInnerCurvePoints())

  return points
end

local function getBoxPoints()
  local points = {}

  table.insert(points, {x=0,y=0.568})
  table.insert(points, {x=0.532,y=0.568})
  table.insert(points, {x=1,y=0.607})
  table.insert(points, {x=1,y=1})
  table.insert(points, {x=0,y=1})

  return points
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasFilledComma:getWeightedHeightFraction()
  return (TAIL_HEIGHT_FRACTION + 1) / 2
end

function CommasFilledComma:setAnchorPointWeightedCenter()
  self:setAnchorPoint(0.5, self:getWeightedHeightFraction())
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CommasFilledComma.new(width, height, color)
  height = height / (1 - TAIL_HEIGHT_FRACTION)
  local self = FilledGraphicsPrimitive.new(width, height, color)
  Mixin.mix(self, CommasFilledComma)

  self._normalizedPointLists =  {getBoxPoints(), getTailPoints()}

  self:updateGraphicsPrimitive()

  return self
end
