--- CommasOutro
-- @author Created by andrewc on May 30, 2019.

class("CommasOutro")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {59,165,246}

local BASE_COMMA_SIZE = ReferenceDevice.getScaledWidth(88)

local COMMAS = {
  {
    COLOR = {237,19,87},
    Z_INDEX = 90,
    MIN = 0.25,
    MAX = 1,
    SCALE_UP = {
      DELAY = 0.8,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 1.633,
      DURATION = 0.333
    },
    Y = 782
  },
  {
    COLOR = {86,9,87},
    Z_INDEX = 80,
    MIN = 0.25,
    MAX = 1.3,
    SCALE_UP = {
      DELAY = 0.7,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 1.533,
      DURATION = 0.433
    },
    Y = 819
  },
  {
    COLOR = {155,217,238},
    Z_INDEX = 70,
    MIN = 0.25,
    MAX = 1.65,
    SCALE_UP = {
      DELAY = 0.6,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 1.433,
      DURATION = 0.533
    },
    Y = 861
  },
  {
    COLOR = {61,2,117},
    Z_INDEX = 60,
    MIN = 0.25,
    MAX = 2.1,
    SCALE_UP = {
      DELAY = 0.5,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 1.333,
      DURATION = 0.633
    },
    Y = 911
  },
  {
    COLOR = {17,191,255},
    Z_INDEX = 50,
    MIN = 0.25,
    MAX = 2.65,
    SCALE_UP = {
      DELAY = 0.4,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 1.233,
      DURATION = 0.733
    },
    Y = 984
  },
  {
    COLOR = {28,97,194},
    Z_INDEX = 40,
    MIN = 0.25,
    MAX = 3.3,
    SCALE_UP = {
      DELAY = 0.3,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 1.133,
      DURATION = 0.833
    },
    Y = 1059
  },
  {
    COLOR = {239,225,244},
    Z_INDEX = 30,
    MIN = 0.25,
    MAX = 3.95,
    SCALE_UP = {
      DELAY = 0.2,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 1.033,
      DURATION = 0.933
    },
    Y = 1139
  },
  {
    COLOR = {2,14,80},
    Z_INDEX = 20,
    MIN = 0.25,
    MAX = 4.65,
    SCALE_UP = {
      DELAY = 0.1,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 0.933,
      DURATION = 1.033
    },
    Y = 1221
  },
  {
    COLOR = {78,203,248},
    Z_INDEX = 10,
    MIN = 0.25,
    MAX = 5.3,
    SCALE_UP = {
      DELAY = 0,
      DURATION = 0.833
    },
    SCALE_DOWN = {
      DELAY = 0.833,
      DURATION = 1.133
    },
    Y = 1304
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function addNewComma(color)
  local color = ColorUtils.colorRGBToStd(unpack(color))
  local comma = CommasFilledComma.new(BASE_COMMA_SIZE, BASE_COMMA_SIZE, color)
  comma:setOpacity(0.9)
  return comma
end

local function getCommaScaleAnim(comma, startScale, endScale, duration, ease)
  local setStartScale = function() comma:setScl(startScale) end
  local scaleAnim = AHScaleTo.new(comma, duration, ease, endScale)
  return AHSequence.new(setStartScale, scaleAnim)
end

local function buildAnimation(self)
  local animation = AHSpawn.new()

  local ease0 = AHEaseBezier.newFlat({0,0,0,1})
  local ease1 = AHEaseBezier.newFlat({1,0,1,1})

  local firstComma = nil

  for _, data in ipairs(COMMAS) do
    local comma = addNewComma(data.COLOR)
    if firstComma == nil then
      firstComma = comma
    end
    comma:setOpacity(0)
    self:insert(comma, data.Z_INDEX)
    comma:setAnchorPointTop()
    comma:setLoc(self:getWidthPart(0.5), self:getHeightPart(((1334-data.Y)/1334) + 0.1))
    comma:setAnchorPointCenter()
    --comma:setAnchorPointWeightedCenter()
    local addComma = function() comma:setOpacity(1) end
    animation:addAction(AHDelayed.new(data.SCALE_UP.DELAY, addComma))
    local scaleUpAnim = getCommaScaleAnim(comma, data.MIN, data.MAX, data.SCALE_UP.DURATION, ease0)
    animation:addAction(AHDelayed.new(data.SCALE_UP.DELAY, scaleUpAnim))
    local scaleDownAnim = getCommaScaleAnim(comma, data.MAX, data.MIN, data.SCALE_DOWN.DURATION, ease1)
    animation:addAction(AHDelayed.new(data.SCALE_DOWN.DELAY, scaleDownAnim))
  end

  local checkmarkEase = AHEaseBezier.newFlat({0.96,0,0.48,1})
  local checkmark = FilledCheckmark.new(0.75 * BASE_COMMA_SIZE, nil, nil, {1,1,1})
  firstComma:insertAtLocationFractional(checkmark, 0.5, firstComma:getWeightedHeightFraction())
  firstComma:linkOpacityAttrToChildren()
  local addCheckmark = function() checkmark:setOpacity(1) end
  animation:addAction(AHDelayed.new(0.8, addCheckmark))
  local detachCheckmark = function()
    self:takeChild(checkmark, 100)
  end
  animation:addAction(AHDelayed.new(1.633, detachCheckmark))
  local scaleUpAnim = AHScaleTo.new(checkmark, 0.666, checkmarkEase, 100)
  animation:addAction(AHDelayed.new(1.633, scaleUpAnim))
  local moveAnim = AHLocationYBy.new(checkmark, 0.666, checkmarkEase, Screen.getHeightPart(0.8))
  animation:addAction(AHDelayed.new(1.633, moveAnim))

  return animation
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasOutro:play()
  Sound.playSoundEffect("outro")
  wait(AHOpacityTo.new(self.background, 0.4, AHEaseBezier.newFlat({0.42,0,0.58,1}), 1):start())
  local animation = buildAnimation(self)
  wait(animation:start())
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CommasOutro.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, CommasOutro)

  local background = FilledRectangle.newFullScreen(ColorUtils.colorRGBToStd(unpack(BACKGROUND_COLOR)))
  self:insertAtCenter(background, 0)
  background:setOpacity(0)

  self.background = background

  return self
end
