--- CommasRationale
-- @author Created by andrewc on May 31, 2019.

class("CommasRationale")

require("CommasCommon")
require("CommasConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PANEL_HEIGHT = 0.45

local Z_INDEX = {
  BACKGROUND = 0,
  TAP_TO_CONTINUE = 10,
  BUTTON = 20
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function updateCloseButton(self, onRelease)
  local continueButton = Group.newFullScreen()
  local tapToHide = function()
    wait(self:hide())
    if onRelease ~= nil then
      onRelease()
    end
    continueButton:removeSelf()
  end

  Button.makeButton(continueButton, nil, tapToHide)
  continueButton:setTouchEnabled(false)
  self:insertAtCenter(continueButton, Z_INDEX.TAP_TO_CONTINUE)

  local setButtonEnabled = function()
    continueButton:setTouchEnabled(true)
  end

  self.continueButton = continueButton

  AHDelayed.new(0.75, setButtonEnabled):start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasRationale:show(onRelease)
  updateCloseButton(self, onRelease)

  local showAnimation = AHSpawn.new()
  local duration = CommasCommon.RATIONALE_IN
  local ease = CommasCommon.RATIONALE_IN_CURVE
  showAnimation:addAction(AHLocationYTo.new(self, duration, ease, self:getHeight()))
  showAnimation:addAction(AHDelayed.new(0.25, AHOpacityTo.new(self.tapLabel, 0.25, 1, 0.5)))

  showAnimation:start()
end

function CommasRationale:setRationale(rationaleType)
  self.rationaleText = Localizer.get(rationaleType)
  self.rationaleLabel:setText(self.rationaleText)
end

function CommasRationale:hide()
  local duration = CommasCommon.RATIONALE_OUT
  local ease = AHEaseBezier.newFlat({0.25,0,0,1})
  AHLocationYBy.new(self, duration, ease, -Screen.getHeightPart(PANEL_HEIGHT)):start()
  self.tapLabel:setOpacity(0)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local background = FilledRectangle.new(self:getWidth(), self:getHeight(), {0,0,0,0.5})
  background:setAnchorPointBottom()
  self:insertAtBottom(background, Z_INDEX.BACKGROUND)
  self.background = background

  local baseAttributesForRationale = { font = CommasCommon.FONT.RATIONALE }
  local labelAttributesForRationale = { i = { font = CommasCommon.FONT.RATIONALE_ITALIC } }

  local tipLabel = Label.new("TIP", CommasCommon.FONT.TINY)
  local marginX = Screen.getWidthPart(0.075)
  local marginY = tipLabel:getHeight()

  tipLabel:setAnchorPointTopLeft()
  tipLabel:setOpacity(0.4)
  local tipLabelX = marginX
  local tipLabelY = self.background:getYTop() - marginY
  self:insertAtLocation(tipLabel, tipLabelX, tipLabelY)
  self.tipLabel = tipLabel

  local rationaleLabel = DynamicTextArea.new("", Screen.getWidthPart(0.85), baseAttributesForRationale, labelAttributesForRationale, Alignment.LEFT, Alignment.CENTER)
  rationaleLabel:setAnchorPointTopLeft()
  local rationaleLabelX = marginX
  local rationaleLabelY = self.tipLabel:getYBottom() - marginY
  self:insertAtLocation(rationaleLabel, rationaleLabelX, rationaleLabelY)
  self.rationaleLabel = rationaleLabel

  local tapLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), CommasCommon.FONT.TAP_TO_CONTINUE, nil, Alignment.CENTER)
  local tapLabelX = Screen.width * 0.5
  local tapLabelMinimumY = Screen.safeBottom + tapLabel:getHeightPart(0.5)
  local tapLabelY = math.max(tapLabel:getHeightPart(1.5), tapLabelMinimumY)
  tapLabel:setOpacity(0)
  tapLabel:setAnchorPointBottom()
  self:insertAtLocation(tapLabel, tapLabelX, tapLabelY)
  self.tapLabel = tapLabel
end

-- Constructor ------------------------

function CommasRationale.new(delegate)
  local width = Screen.width
  local height = Screen.height * PANEL_HEIGHT
  local self = Group.new(width, height)
  Mixin.mix(self, CommasRationale)

  self.delegate = delegate

  self:setAnchorPointTop()

  createLayout(self)

  return self
end
