--- CommasRound
-- @author Created by andrewc on May 30, 2019.

class("CommasRound")

require("CommasCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CORRECT_WORD_PATTERN = '<comma>(.-)</comma>'
local DISTRACTOR_WORD_PATTERN = '<distractor>(.-)</distractor>'
local PREPOPULATED_WORD_PATTERN = '<prepopulated>,</prepopulated>'

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- only call before prepopulating commas, as any that come pre-populated are
-- oxford commas.
local function addOxfordTags(self)
  self.text = self.text:gsub("%s(%S-)"..PREPOPULATED_WORD_PATTERN, " <oxford>%1</oxford>"..PREPOPULATED_WORD_PATTERN)
  -- if a close tag got caught in the oxford word, swap the tags to avoid an xml crash.
  self.text = self.text:gsub("(</%S->)</oxford>", "</oxford>%1")
end

local function parseObj(self, roundObj)
  self.identifier = roundObj.identifier
  self.text = roundObj.text
  self.numberOfDistractorCommas = roundObj.numDistractors
  self.numberOfCommas = roundObj.numCommas

  addOxfordTags(self)

  self.correctWords = {}
  for word in string.gmatch(self.text, CORRECT_WORD_PATTERN) do
    table.insert(self.correctWords, word)
  end

  self.distractorWords = {}
  for word in string.gmatch(self.text, DISTRACTOR_WORD_PATTERN) do
    table.insert(self.distractorWords, word)
  end

  self.rationale = {}
  if roundObj.distractor_1 ~= nil then
    table.insert(self.rationale, roundObj.distractor_1)
  end
  if roundObj.distractor_2 ~= nil then
    table.insert(self.rationale, roundObj.distractor_2)
  end
  if roundObj.distractor_3 ~= nil then
    table.insert(self.rationale, roundObj.distractor_3)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasRound:getCorrectWordIndex(word)
  return TableUtils.indexOf(self.correctWords, word)
end

function CommasRound:prepopulateComma(indexToPrepopulate)
  local commaIndex = indexToPrepopulate

  if (commaIndex == nil) then
    commaIndex = math.random(1, self.numberOfCommas)
  end

  local wordToPopulate = self.correctWords[commaIndex]

  -- Remove this comma/word from class tables tables
  table.remove(self.correctWords, commaIndex)
  self.numberOfCommas = self.numberOfCommas - 1

  -- Replace the word with the <comma> tag with
  -- a word with the <ignore> label and a comma with the <prepoulated> label
  local text = self.text
  local tagStart, tagEnd = string.find(text, "<comma>"..wordToPopulate.."</comma>", 1, true)

  local comma = ' <'..CommasCommon.PREPOPULATED_COMMA_TAG_LABEL ..'>,</'..CommasCommon.PREPOPULATED_COMMA_TAG_LABEL..'>'
  local wordToPopulate = ' <'..CommasCommon.IGNORE_WORD ..'>'..wordToPopulate..'</'..CommasCommon.IGNORE_WORD..'>'
  local wordWithComma = wordToPopulate .. comma

  text = text:sub(1,tagStart - 1) .. wordWithComma .. text:sub(tagEnd + 1, #text)
  self.text = text
end

function CommasRound:getIdentifier()
  return self.identifier
end

function CommasRound:getText()
  return self.text
end

function CommasRound:getNumberOfInteractions()
  return self.numberOfCommas
end

function CommasRound:getRationale(index)
  return self.rationale[index]
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CommasRound.new(roundObj, maximumNumerOfCommas)
  local self = {}
  Mixin.mix(self, CommasRound)

  parseObj(self, roundObj)

  while (self.numberOfCommas > maximumNumerOfCommas) do
    self:prepopulateComma()
  end

  return self
end
