--- CommasScene
-- @author Created by andrewc on May 30, 2019.

class("CommasScene")

require("CommasTextView")
require("CommasStatusbar")
require("CommasOutro")
require("CommasRationale")
require("CommasCommon")
require("CommasBackground")
require("CommasAnswerAnimations")
require("CommasHelperText")
require("graphics.CircleGradient")
require("CommasEncourageText")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TUTORIAL_MASK = {1, 1, 1, 1}
local COLOR_TUTORIAL_OVERLAY = {0.039, 0.055, 0.082, 0.6}

local Z_INDEX = {
  TRANSITION = 10,
  BACKGROUND = 20,
  TUTORIAL = 21,
  MARKERS = 25,
  TEXT = 30,
  EFFECTS = 35,
  RATIONALE = 40,
  OVERLAY = 50
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function moveTextForRationale(self, moveOut)
  local duration = CommasCommon.RATIONALE_IN
  local ease = CommasCommon.RATIONALE_IN_CURVE
  local offsetY = self.roundTextOffsetY or 0
  local offsetX = self.roundTextOffsetX or 0

  local centerScreen = Screen.height * 0.5

  local scale = self.roundTextScale or 1
  if moveOut then
    self.roundTextScale = self.roundText:getScl()
    local textHeight = self.roundText:getHeight() * self.roundTextScale
    local maxHeight = centerScreen - 45
    if (maxHeight < textHeight) then
      scale = maxHeight / textHeight
    end

    self.roundTextOffsetX = self.roundText:getWidthPart(1-scale)/2
    offsetX = -self.roundTextOffsetX
  end

  if moveOut then
    local yBot = self.roundText:getYBottom()
    offsetY = (centerScreen - yBot) * scale
    self.roundTextOffsetY = -offsetY
  else
    duration = CommasCommon.RATIONALE_OUT
    ease = AHEaseBezier.newFlat({0.25,0,0,1})
  end

  local moveAnim = AHLocationYBy.new(self.roundText, duration, ease, offsetY)
  local anchorLeftAnim = AHLocationXBy.new(self.roundText, duration, ease, offsetX)
  local scaleAnim = AHScaleTo.new(self.roundText, duration, ease, scale)

  return AHSpawn.new(moveAnim, anchorLeftAnim, scaleAnim)
end

local function showEncouragement(self, numLeft, delay)
  self.encourageText:setY((self.statusbar:getYBottom() + self.roundText:getYTop()) / 2)
  local maxHeight = self.statusbar:getYBottom() - self.roundText:getYTop() - 20
  if (maxHeight < self.encourageText:getHeight()) then
    self.encourageText:setScl(maxHeight / self.encourageText:getHeight())
  else
    self.encourageText:setScl(1)
  end
  self.encourageText:encourage(numLeft, delay)
end

local function makeRoundText(self)
  local roundText = CommasTextView.new(self.answerHandler, self:getWidth() * 0.85)
  self:insertAtCenter(roundText, Z_INDEX.TEXT)
  return roundText
end

local function showRationale(self, item, itemView, rationaleClosedCallback)
  self.numberOfAnimations = self.numberOfAnimations + 1
  showEncouragement(self, 0)
  self.roundText:setTouchEnabled(false)
  self.background:getShowErrorAnimation():start()
  self.helperText:hide()
  local rationaleIndex = self.roundText:getDistractorIndexForItem(item)
  local rationaleType = self.round:getRationale(rationaleIndex)

  local failureMarker = CommasAnswerAnimations.getFailureObject()
  itemView:insertAtRight(failureMarker)
  CommasCommon.offsetItemToComma(failureMarker)
  failureMarker.show:start()

  local updatedRationaleClosedCallback = function()
    failureMarker.hide:start()
    self:onHideRationale()
    rationaleClosedCallback()
  end

  self.rationaleOverlay:setRationale(rationaleType)
  self.rationaleOverlay:show(updatedRationaleClosedCallback)
  moveTextForRationale(self, true):start()
  self.roundText:getHideMarkerAnimation(item):start()
  self.statusbar:temporarilyHide()
end

local function getEndOfRoundTextAnimation(self)
  local duration = CommasCommon.ROUND_TRANSITION_OUT
  local ease = AHEaseBezier.newFlat({0.75,0,0.98,0.98})
  local animation = AHSpawn.new()
  animation:addAction(AHScaleTo.new(self.roundText, duration, ease, self.roundText:getScl() * 1.1))
  animation:addAction(AHOpacityTo.new(self.roundText, duration, ease, 0))
  animation:addAction(self.roundText:getHideAllMarkersAnimation())
  return animation
end

local function getStartOfRoundTextAnimation(self)
  local duration = CommasCommon.ROUND_TRANSITION_IN
  local ease = AHEaseBezier.newFlat({0,0,0,1})
  local textScale = self.roundText:getScl()
  local animation = AHSpawn.new()
  animation:addAction(function() self.roundText:setScl(textScale * 0.9) end)
  animation:addAction(AHScaleTo.new(self.roundText, duration, ease, textScale))
  animation:addAction(AHOpacityTo.new(self.roundText, duration, ease, 1))
  animation:addAction(AHDelayed.new(0.3, self.roundText:getShowAllMarkersAnimation()))
  return animation
end

local function setupNewRound(self)
  local roundText = makeRoundText(self)
  roundText:startRound(self.round)
  roundText:setOpacity(0)
  local maxHeight = self.statusbar:getYBottom() - 36 - self.helperText:getYTop() - self.encourageText:getHeight()
  local scale = math.min(maxHeight / roundText:getHeight(), 1)
  if scale < 1 then
    roundText:offsetY(-(roundText:getYBottom() - self.helperText:getYTop()) / 2)
  else
    roundText:setLocUsingAnchorPoint(Screen.width / 2, Screen.height / 2, Z_INDEX.TEXT, 0, 0)
  end
  roundText:setScl(scale)
  self.statusbar:startRound(self.round)
  return roundText
end

local function getNewRoundAnimation(self)
  local animation = AHSpawn.new()
  animation:addAction(getStartOfRoundTextAnimation(self))
  animation:addAction(self.statusbar:getStartRoundAnimation())
  return animation
end

local function enableNewRound(self)
  self.roundText:setTouchEnabled(true)
  self.helperText:setupText()

  if self.shouldShowHelperText then
    self.helperText:show()
  end
end

local function endRound(self)
  self.roundText:setTouchEnabled(false)
end

local function getEndRoundAnimation(self)
  local animation = AHSpawn.new()
  animation:addAction(self.statusbar:getEndRoundAnimation())
  animation:addAction(getEndOfRoundTextAnimation(self))
  return animation
end

local function getBackgroundScaleAnimation(self)
  local duration = 0.25 + CommasCommon.ROUND_TRANSITION_IN
  local ease = AHEaseBezier.newFlat({0.49,0,0.31,1})
  local commas = self.background:getCommasGroup()
  local soundEffect = function() Sound.playSoundEffect("transition") end
  local scale = AHScaleTo.new(commas, duration, ease, commas:getScl() * 1.02)
  return AHSpawn.new(scale, soundEffect)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CommasScene:revealCorrectCommas()
  self:waitForAnimationsToComplete()
  local correctItemViewPairs = self.roundText:getRemainingCorrectItemViewPairs()
  local isEndOfGame = true -- We play the same sounds & impact animation in this function to avoid the playing more than once
  for _, itemViewPair in ipairs(correctItemViewPairs) do
    self.roundText:commaFound(itemViewPair[1])
    self.statusbar:getCommaFoundAnimation(itemViewPair[2], isEndOfGame):start()
  end
  if (#correctItemViewPairs > 0) then
    AHSpawn.new(
      AHDelayed.new(0.167, function() Sound.playSoundEffect("laserbeam") end),
      AHDelayed.new(0.45, function() Sound.playSoundEffect("correct_hit") end),
      AHDelayed.new(0.45, self:getImpactAnimation())
    ):start()
    self.roundText:getHideAllMarkersAnimation():start()
    waitFor(1.25)
  end
end

function CommasScene:correctAnswer(item, itemView, commaRemovedCallback, interactionsRemaining)
  self.shouldShowHelperText = false
  self.helperText:hide()
  self.numberOfAnimations = self.numberOfAnimations + 1
  local animation = AHSequence.new(
    self.statusbar:getCommaFoundAnimation(itemView),
    function()
      self.numberOfAnimations = self.numberOfAnimations - 1
      commaRemovedCallback()
    end)
  local correctAnswerActions = AHSpawn.new(animation,
    function()
      self.roundText:commaFound(item)
      showEncouragement(self, interactionsRemaining, 1)
    end)
  correctAnswerActions:start()
end

function CommasScene:incorrectAnswer(item, itemView, rationaleClosedCallback, interactionsRemaining)
  self.numberOfAnimations = self.numberOfAnimations + 1
  local callback = function()
    self.numberOfAnimations = self.numberOfAnimations - 1
    rationaleClosedCallback()
    showEncouragement(self, interactionsRemaining)
  end

  local incorrectCommaAnimation = self.statusbar:animateIncorrectComma(itemView, false)
  incorrectCommaAnimation:start()
  waitFor(CommasCommon.LASER_TIME)
  showRationale(self, item, itemView, callback)
  self.roundText:getHideMarkerAnimation(item):start()
end

function CommasScene:endFinalRound()
  self.roundText:setTouchEnabled(false)
  self.helperText:hide()
  local animation = AHSequence.new()

  animation:addAction(AHDelay.new(CommasCommon.ROUND_TRANSITION_OUT))
  animation:addAction(function() endRound(self) end)
  animation:addAction(AHSpawn.new(
    getBackgroundScaleAnimation(self),
    getEndRoundAnimation(self)))

  wait(animation:start())
end

function CommasScene:startNewRound(round, isFirstRound)
  if not isFirstRound then
    endRound(self)
    getBackgroundScaleAnimation(self):start()
    wait(getEndRoundAnimation(self):start())
  end
  self.round = round
  self.roundText:removeSelf()
  self.roundText = setupNewRound(self)
  wait(getNewRoundAnimation(self):start())
  enableNewRound(self)
end

function CommasScene:clearScene()
  self.roundText:clear()
  self.statusbar:clear()
end

function CommasScene:startGame()
  local background = CommasBackground.new()
  self:insertAtCenter(background, Z_INDEX.BACKGROUND)
  wait(background:startGame())

  self.background = background
  self.shouldShowHelperText = true
end

function CommasScene:playOutro()
  local outro = CommasOutro.new()
  self:insertAtCenter(outro, Z_INDEX.OVERLAY)
  wait(outro:play())
end

function CommasScene:onHideRationale()
  local animation = AHSpawn.new()
  animation:addAction(self.background:getHideErrorAnimation())
  animation:addAction(moveTextForRationale(self, false))
  animation:addAction(self.statusbar:getReshowAnimation())
  wait(animation:start())
  self.numberOfAnimations = self.numberOfAnimations - 1
  self.roundText:setTouchEnabled(true)
  if self.shouldShowHelperText then
    self.helperText:show()
  end
end

function CommasScene:getImpactAnimation()
  local ease = AHEaseBezier.newFlat({0.42,0,0.58,1})
  local textAnimation = AHSequence.new()
  local roundTextScale = self.roundText:getScl()
  textAnimation:addAction(AHScaleTo.new(self.roundText, 0.167, ease, roundTextScale * 0.98))
  textAnimation:addAction(AHScaleTo.new(self.roundText, 0.333, ease, roundTextScale))

  local overlayAnimation = AHSequence.new()
  overlayAnimation:addAction(AHSpawn.new(
    self.impactOverlay[1]:newOpacityToAction(0.167, ease, 0, 0.61),
    self.impactOverlay[2]:newOpacityToAction(0.167, ease, 0, 0.31)))
  overlayAnimation:addAction(AHSpawn.new(
    self.impactOverlay[1]:newOpacityToAction(0.333, ease, 0, 0),
    self.impactOverlay[2]:newOpacityToAction(0.333, ease, 0, 0)))

  return AHSpawn.new(textAnimation, overlayAnimation)
end

function CommasScene:getShakeAnimation()
  local shakeAnimation = AHSpawn.new()

  local ease = AHEaseBezier.newFlat({0,0,0.58,1})

  local textShakeAnimation = AHSequence.new()
  textShakeAnimation:addAction(AHLocationXBy.new(self.roundText, 0.083, ease, 0.75 * ReferenceDevice.getScaledWidth(8)))
  textShakeAnimation:addAction(AHLocationXBy.new(self.roundText, 0.083, ease, 0.75 * ReferenceDevice.getScaledWidth(-13)))
  textShakeAnimation:addAction(AHLocationXBy.new(self.roundText, 0.167, ease, 0.75 * ReferenceDevice.getScaledWidth(10)))
  textShakeAnimation:addAction(AHLocationXBy.new(self.roundText, 0.333, ease, 0.75 * ReferenceDevice.getScaledWidth(-5)))
  shakeAnimation:addAction(textShakeAnimation)

  local backgroundShakeAnimation = AHSequence.new()
  local backgroundCommas = self.background:getCommasGroup()
  backgroundShakeAnimation:addAction(AHLocationXBy.new(backgroundCommas, 0.083, ease, ReferenceDevice.getScaledWidth(-12)))
  backgroundShakeAnimation:addAction(AHLocationXBy.new(backgroundCommas, 0.083, ease, ReferenceDevice.getScaledWidth(17)))
  backgroundShakeAnimation:addAction(AHLocationXBy.new(backgroundCommas, 0.167, ease, ReferenceDevice.getScaledWidth(-10)))
  backgroundShakeAnimation:addAction(AHLocationXBy.new(backgroundCommas, 0.333, ease, ReferenceDevice.getScaledWidth(5)))
  shakeAnimation:addAction(backgroundShakeAnimation)

  return shakeAnimation
end

function CommasScene:waitForAnimationsToComplete()
  self.roundText:setTouchEnabled(false)
  waitUntil(function() return self.numberOfAnimations <= 0 end)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, answerHandler)
  local width, height = self:getDims()
  local centerX = width * 0.5

  self.answerHandler = answerHandler
  local roundText = makeRoundText(self)

  local statusbarHeight = 36
  local statusbar = CommasStatusbar.new(self, width * 0.5, statusbarHeight)
  statusbar:setAnchorPointTop()
  self:insert(statusbar, Z_INDEX.EFFECTS)
  statusbar:setX(centerX)

  local topMarginForLargeSafeArea = Host.safeAreaTopInset + 8
  local topMarginForSmallSafeArea = statusbarHeight
  local topMargin = math.max(topMarginForSmallSafeArea, topMarginForLargeSafeArea)
  statusbar:setY(height - topMargin)

  local rationaleOverlay = CommasRationale.new(self)
  self:insertAtBottom(rationaleOverlay, Z_INDEX.RATIONALE)

  local helperText = CommasHelperText.new()
  helperText:setAnchorPointBottom()
  self:insertAtLocationFractional(helperText, 0.5, 0.05, Z_INDEX.TEXT)

  local overlay1 = CircleGradient.new(height * 2, {0,0,0,0}, {0.45,0.91,0.97,0.61})
  overlay1:newOpacityToAction(0,0,0,0):start()
  self:insertAtCenter(overlay1, Z_INDEX.OVERLAY)
  local overlay2 = CircleGradient.new(height, {0,0,0,0}, {0.31,0.8,0.97,0.11})
  overlay2:newOpacityToAction(0,0,0,0):start()
  self:insertAtCenter(overlay2, Z_INDEX.OVERLAY)

  local encourageText = CommasEncourageText.new()
  self:insertAtCenter(encourageText, Z_INDEX.TEXT)
  encourageText:setY(statusbar:getYBottom() - encourageText:getHeight())

  self.roundText = roundText
  self.statusbar = statusbar
  self.rationaleOverlay = rationaleOverlay
  self.helperText = helperText
  self.impactOverlay = {overlay1, overlay2}
  self.encourageText = encourageText
end

-- Constructor ------------------------

function CommasScene.new(answerHandler)
  local self = Group.newFullScreen()
  Mixin.mix(self, CommasScene)

  self.shouldShowHelperText = true
  self.numberOfAnimations = 0

  createLayout(self, answerHandler)

  return self
end
