--- ContentReviewAnimatedWord
-- @author Created by Joe Poka on September 27, 2016.

class("ContentReviewAnimatedWord")

require("ContentReviewTextViewMemory")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local OFFSET_Y = -Screen.getHeightPart(0.075)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewAnimatedWord:getAnimation()
  local duration = 0.3
  local ease = AHEaseExponentialOut.new()
  local move = AHLocationYTo.new(self.animatedWordPart, duration, ease, 0.0)
  local show = AHOpacityTo.new(self.animatedWordPart, duration * 0.1, nil, 1.0)
  local moveAndShow = AHSpawn.new(move, show)
  return moveAndShow
end

function ContentReviewAnimatedWord:resetAnimation()
  self.animatedWordPart:setY(OFFSET_Y)
  self.animatedWordPart:setOpacity(0.0)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ContentReviewAnimatedWord.new(word, font, width)
  local height = Screen.getHeightPart(0.1)
  local self = Group.new(width, height)
  Mixin.mix(self, ContentReviewAnimatedWord)

  local baseAttributesList = {
    {
      font = font,
      color = ContentReviewCommon.Colors.darkGray
    },
    {
      font = font,
      color = ContentReviewCommon.Colors.lightGray
    }
  }

  local words = {
    string.sub(word, 1, 2),
    string.sub(word, 3, #word)
  }

  local textView = ContentReviewTextViewMemory.new(words, width, baseAttributesList)
  self:insertAtCenter(textView)

  self.animatedWordPart = textView:getAnimatedWordView()
  self.animatedWordPart:setOpacity(0.0)

  local underlineWidth = self.animatedWordPart:getWidth()
  local underlineHeight = Screen.getHeightPart(0.0015)
  local underline = FilledRectangle.new(underlineWidth, underlineHeight, ContentReviewCommon.Colors.lightGray)
  textView:insertAtCenter(underline)

  local x = self.animatedWordPart:getX() + self.animatedWordPart:getWidthPart(0.5)
  underline:setX(x)

  local dy = self.animatedWordPart:getHeightPart(0.425)
  underline:setY(underline:getY() - dy)

  local coverHeight = self.animatedWordPart:getHeightPart(1.0)
  local cover = FilledRectangle.new(underlineWidth, coverHeight, {1.0, 1.0, 1.0, 1.0})
  local y = underline:getY() - coverHeight * 0.5 - underlineHeight * 0.5
  cover:setLoc(x, y)
  textView:insert(cover)

  self:resetAnimation()

  return self
end
