--- ContentReviewCard
-- @author Created by Joe Poka on June 27, 2016.

class("ContentReviewCard")

require("containers.card_carousel.CardCarouselCard")
require("ContentReviewCommon")
require("ContentReviewTextViewPrompt")
require("ContentReviewTextViewRationale")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewCard:onEnter()
  self.exitButton:setTouchEnabled(true)
  
  self.super.onEnter(self)
end

function ContentReviewCard:onLeaveBegin()
  self.exitButton:setTouchEnabled(false)

  self.super.onLeaveBegin(self)
end

function ContentReviewCard:setIndex(cardIndex, totalCount)
  local indexString = cardIndex .. "/" .. totalCount
  self.countLabel:setText(indexString)

  local rightMargin = Screen.getWidthPart(0.0385)
  local x = Screen.width - rightMargin - self.countLabel:getWidthPart(0.5)
  self.countLabel:setX(x)
end

function ContentReviewCard:getExitAction()
  local duration = 0.35

  local exitReviewAction = AHSequence.new(
    AHSpawn.new(
      AHLocationYBy.new(self, 0.5, AHEaseExponentialIn.new(), -Screen.height),
      AHOpacityTo.new(self, duration, nil, 0.0)
    ),
    function()
      GameSession.notifyGameWin()
    end)

  return exitReviewAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ContentReviewCard.new(cardType, concept)
  local self = CardCarouselCard.new(cardType, concept)
  Mixin.mix(self, ContentReviewCard)

  local background = FilledRectangle.new(Screen.width, Screen.height, CardCarouselCardCommon.COLORS.WHITE)
  self:insertAtCenter(background)

  local countFont = Font.new(FontFaces.DIN_OT_MEDIUM, 14.0)
  self.countLabel = Label.new("", countFont)
  self.countLabel:setColor(unpack({0.6, 0.6, 0.6, 1.0}))
  self.countLabel:setLoc(UIOverlay.NORTHEAST.x, UIOverlay.NORTHEAST.y)
  self:insert(self.countLabel)

  local exitButton = Sprite.newWithName("close.png")
  exitButton:setLoc(UIOverlay.NORTHWEST.x, UIOverlay.NORTHWEST.y)

  local onTouchUp = function()
    local isRecursive = true
    self:linkOpacityAttrToChildren(isRecursive)
    self:getExitAction():start()
  end

  local buttonWidth = exitButton:getWidthPart(5.0)
  Button.addVirtualButton(exitButton, buttonWidth, buttonWidth, nil, onTouchUp)
  exitButton:setTouchEnabled(false)
  self:insert(exitButton)
  self.exitButton = exitButton

  return self
end
