--- ContentReviewCardAssociation
-- @author Created by Steven Mah on November 3, 2016

class("ContentReviewCardAssociation")

require("ContentReviewCard")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WORD_FONT = Font.new(FontFaces.DIN_OT_BOLD, 35.0)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getCardData(concept)
  local synonyms = concept.synonyms
  local userInputJson = GameSession.answersDatasByConceptIdentifier[concept.identifier]
  local userInputTable = MOAIJsonParser.decode(userInputJson)[1]
  local selectedSynonym = userInputTable.question_identifier

  for _, synonym in ipairs(synonyms) do
    if (synonym.word == selectedSynonym) then
      return {
        word = synonym.word,
        definition = synonym.rationale
      }
    end
  end

  -- NOTE: In the event that the synonym wasn't found, that means that user tracking
  -- data is out of sync/no longer valid with the current content.db. Grab the
  -- first synonym in the concept to present so content review doesn't crash.
  local firstSynonym = synonyms[1]
  return {
    word = firstSynonym.word,
    definition = firstSynonym.rationale
  }
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, concept)
  local cardData = getCardData(concept)
  local word = string.upper(cardData.word)
  local definition = cardData.definition
  local contentGroup = Group.newFullScreen()


  local wordLabel = Label.new(word, WORD_FONT)
  wordLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  local x = Screen.getWidthPart(0.5)
  local y = Screen.getHeightPart(0.55) + wordLabel:getHeight()
  wordLabel:setLoc(x, y)

  local definitionWidth = Screen.getWidthPart(0.8)
  local definitionFontSize = 18.0
  local definitionLabel = ContentReviewTextViewRationale.new(definition, definitionWidth, definitionFontSize, Alignment.CENTER)
  definitionLabel:setAnchorPointTop()
  y = y - wordLabel:getHeightPart(0.75)
  definitionLabel:setLoc(x, y)

  local promptString = "Synonym - " .. concept.prompt
  local promptLabel = Label.new(promptString, Font.new(FontFaces.DIN_OT_MEDIUM, definitionFontSize))
  promptLabel:setColor(unpack(ContentReviewCommon.Colors.lightGray))
  y = y - definitionLabel:getHeight() - promptLabel:getHeightPart(1.5)
  promptLabel:setLoc(x, y)

  contentGroup:insert(wordLabel)
  contentGroup:insert(definitionLabel)
  contentGroup:insert(promptLabel)

  self:insertAtCenter(contentGroup)

  return contentGroup
end

-- Constructor ------------------------

function ContentReviewCardAssociation.new(concept, parameters)
  local self = ContentReviewCard.new()
  Mixin.mix(self, ContentReviewCardAssociation, "ContentReviewCard")

  createLayout(self, concept)

  return self
end
