--- ContentReviewCardEloquence
-- @author Created by Joe Poka on October 5, 2016.

class("ContentReviewCardEloquence")

require("ContentReviewCard")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_SUGGEST_COUNT = 4
local PROMPT_HEADER_TEXT = "REPLACE THE WORD %s TO MAKE THE SENTENCE MORE %s"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, concept)
  local replaceWord = concept.replaceWord
  local task = concept.task
  local prompt = concept.prompt
  local useSmallFont = concept.useSmallFont
  local suggestList = TableUtils.shuffleArray(StringUtils.split(concept.suggestList, ","))

  local contentWidth = ContentReviewCommon.defaultContentWidth
  local mainFontSize = 35
  if (useSmallFont) then
    mainFontSize = 28.0
  end
  local mainFont = Font.new(FontFaces.DIN_OT_LIGHT, mainFontSize)
  local headerFont = Font.new(FontFaces.DIN_OT_MEDIUM, 15.0)

  local baseAttributes = {
    font = mainFont,
    color = ContentReviewCommon.Colors.darkGray
  }

  local attributesForLabels = {
    replace = {
      color = ContentReviewCommon.Colors.lightGray,
      strike = true
    }
  }

  local defaultValue = nil
  local promptHeaderText = string.format(PROMPT_HEADER_TEXT, string.upper(replaceWord), string.upper(task))
  local promptHeader = DynamicTextArea.newSimple(promptHeaderText, contentWidth, headerFont, defaultValue, defaultValue, Screen.getHeightPart(0.01))
  promptHeader:setColor(unpack(ContentReviewCommon.Colors.lightGray))

  local promptLabel = ContentReviewTextView.new(prompt, contentWidth, baseAttributes, attributesForLabels, defaultValue, defaultValue, defaultValue, ContentReviewCommon.GAMES.ELOQUENCE)

  local suggestionHeader = Label.new("SUGGESTED REPLACEMENTS", headerFont)
  suggestionHeader:setColor(unpack(ContentReviewCommon.Colors.lightGray))

  local suggestCount = math.min(MAX_SUGGEST_COUNT, #suggestList)
  local suggestionColumn = Column.new(0, Alignment.LEFT)
  local suggestionColor = ContentReviewCommon.Colors.correctGreen
  for i = 1, suggestCount do
    local suggestionText = StringUtils.trim(suggestList[i])
    local suggestionLabel = Label.new(suggestionText, mainFont)
    suggestionLabel:setColor(unpack(suggestionColor))
    suggestionColumn:insert(suggestionLabel)
  end
  suggestionColumn:alignChildren()

  local content = Column.new(Screen.getHeightPart(0.02), Alignment.LEFT)
  content:insert(promptHeader)
  content:insert(promptLabel)
  content:insertSpacer(Screen.getHeightPart(0.02))
  content:insert(suggestionHeader)
  content:insert(suggestionColumn)
  content:alignChildren()

  content:setLoc(Screen.getWidthPart(0.5), Screen.getHeightPart(0.8))
  content.isLocationSet = true

  self:insertAtCenter(content)

  return content
end

-- Constructor ------------------------

function ContentReviewCardEloquence.new(concept, parameters)
  local self = ContentReviewCard.new()
  Mixin.mix(self, ContentReviewCardEloquence, "ContentReviewCard")

  createLayout(self, concept)

  return self
end
