--- ContentReviewCardIntro
-- @author Created by Joe Poka on June 29, 2016.

class("ContentReviewCardIntro")

require("ContentReviewCard")
require("ContentReviewTextViewIntro")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TAP_TIP_COLOR = ContentReviewCommon.Colors.lightGray

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getIntroData()
  local introData = {
    [ContentReviewCommon.GAMES.MEMORY] = {
      name = Localizer.get("RECALL.NAME"),
      phrase = "had some trouble recalling words."
    },
    [ContentReviewCommon.GAMES.SPELLING] = {
      name = Localizer.get("SPELLING.NAME"),
      phrase = "made some errors identifying the correct spelling of words."
    },
    [ContentReviewCommon.GAMES.ERROR_AVOIDANCE] = {
      name = Localizer.get("ERROR_AVOIDANCE.NAME"),
      phrase = "mixed up some commonly confused words."
    },
    [ContentReviewCommon.GAMES.SYNTAX] = {
      name = Localizer.get("SYNTAX.NAME"),
      phrase = "had some trouble with grammar."
    },
    [ContentReviewCommon.GAMES.AGILITY] = {
      name = Localizer.get("AGILITY.NAME"),
      phrase = "had some trouble with vocabulary."
    },
    [ContentReviewCommon.GAMES.DICTION] = {
      name = Localizer.get("DICTION.NAME"),
      phrase = "had some trouble with word choice."
    },
    [ContentReviewCommon.GAMES.BREVITY] = {
      name = Localizer.get("BREVITY.NAME"),
      phrase = "had some trouble with redundancy."
    },
    [ContentReviewCommon.GAMES.CLARITY] = {
      name = Localizer.get("CLARITY.NAME"),
      phrase = "made some errors while creating more concise sentences."
    },
    [ContentReviewCommon.GAMES.ELOQUENCE] = {
      name = Localizer.get("ELOQUENCE.NAME"),
      phrase = "had some trouble improving sentences."
    },
    [ContentReviewCommon.GAMES.INVERSION] = {
      name = Localizer.get("INVERSION.NAME"),
      phrase = "had some trouble identifying antonyms."
    },
    [ContentReviewCommon.GAMES.EXPRESSION] = {
      name = Localizer.get("EXPRESSION.NAME"),
      phrase = "had some trouble with commonly used expressions."
    },
    [ContentReviewCommon.GAMES.WORD_PARTS] = {
      name = Localizer.get("WORD_PARTS.NAME"),
      phrase = "had some trouble putting word parts together."
    },
    [ContentReviewCommon.GAMES.VISUALIZATION] = {
      name = Localizer.get("VISUALIZATION.NAME"),
      phrase = "had some trouble associating words with images."
    },
    [ContentReviewCommon.GAMES.ADJECTIVE_RECALL] = {
      name = Localizer.get("ADJECTIVE_RECALL.NAME"),
      phrase = "had some trouble finding descriptive adjectives."
    },
    [ContentReviewCommon.GAMES.ASSOCIATION] = {
      name = Localizer.get("ASSOCIATION.NAME"),
      phrase = "had some trouble matching verbs."
    },
    [ContentReviewCommon.GAMES.CONTEXT] = {
      name = Localizer.get("CONTEXT.NAME"),
      phrase = "had some trouble with context."
    },
    [ContentReviewCommon.GAMES.SUBTRACTION] = {
      name = Localizer.get("SUBTRACTION.NAME"),
      phrase = "had some trouble subtracting."
    },
    [ContentReviewCommon.GAMES.PUNCTUATION] = {
      name = Localizer.get("PUNCTUATION.NAME"),
      phrase = "had some trouble with punctuation."
    }
  }

  local data = introData[ContentReviewCommon.SKILL]

  local sentencePrefix = "Recently,<br/>you've "
  data.phrase = sentencePrefix .. data.phrase

  return data
end

-- TODO - a similar tap/swipe tip exists in ExerciseCardTitle and could be moved
-- into a shared location after we fix an issue in the shared assets sheet where
-- colors/opacity on the assets is not working as intended. Issue related to
-- webp processing of the shared_sheet.
local function createTapTip()
  local columnSpacing = 10.0
  local tip = Column.new(columnSpacing)

  local hand = Sprite.newWithName("tap_to_continue.png")
  tip:insert(hand)

  local label = CardCarouselCardCommon.newLabel("TAP TO CONTINUE", FontFaces.DIN_OT_MEDIUM, 20)
  tip:insert(label)

  tip:alignChildren()

  tip:linkColorAttrToChildren()
  tip:setColor(unpack(TAP_TIP_COLOR))

  return tip
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewCardIntro:createAnimationAction()
  local titleAnimation = self.titleLabel:getTextAnimation()
  local tapTip = self.tapTip

  local reviewFadeDuration = 0.6
  local fadeInReviewLabel = AHOpacityTo.new(self.reviewLabel, reviewFadeDuration, nil, 1.0)
  local fadeInExitButton = AHOpacityTo.new(self.exitButton, reviewFadeDuration, nil, 1.0)
  local swipTipFadeIn = AHDelayed.new(0.25, AHOpacityTo.new(tapTip, 1.2, nil, 1.0))

  local startIntroCardActions = AHSpawn.new(titleAnimation,
                                            fadeInReviewLabel,
                                            fadeInExitButton,
                                            swipTipFadeIn)

  delayDuration = 0.2
  local delayedAnimation = AHDelayed.new(delayDuration, startIntroCardActions)

  return delayedAnimation
end

function ContentReviewCardIntro:reset()
  self.reviewLabel:setOpacity(0.0)
  self.exitButton:setOpacity(0.0)
  self.titleLabel:resetTextAnimation()
  self.tapTip:setOpacity(0.0)
end

function ContentReviewCardIntro:onEnter()
  self.super.onEnter(self)
  self.forwardTapArea:setTouchEnabled(true)
  self.backTapArea:setTouchEnabled(true)
end

function ContentReviewCardIntro:onLeaveBegin()
  self.super.onLeaveBegin(self)
  self.forwardTapArea:setTouchEnabled(false)
  self.backTapArea:setTouchEnabled(false)
end

function ContentReviewCardIntro:handlesOwnTapNavigation()
  return true
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createTapAreas(self)
  local width = self:getWidth() - CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY
  local height = self:getHeight()
  local onTouchDown = nil
  local onTouchUp = function()
    local duration = CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL
    local willCheckNavigationConstraints = true
    self:forceTapNavigation(CardCarouselNavigator.TAP_DIRECTION_FORWARD, duration, willCheckNavigationConstraints)
  end
  local onTouchCancel = nil
  local forwardTapArea = Button.new(width, height, onTouchDown, onTouchUp, onTouchCancel)
  forwardTapArea:setAnchorPointRight()
  self:insertAtRight(forwardTapArea)
  forwardTapArea:setTouchEnabled(false)
  self.forwardTapArea = forwardTapArea

  width = CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY
  local height = self:getHeight()
  onTouchUp = function()
    self.exitButton:setTouchEnabled(false)
    self.forwardTapArea:setTouchEnabled(false)
    self.backTapArea:setTouchEnabled(false)
    local exitAction = self:getExitAction()
    exitAction:start()
  end
  local backTapArea = Button.new(width, height, onTouchDown, onTouchUp, onTouchCancel)
  backTapArea:setAnchorPointLeft()
  self:insertAtLeft(backTapArea)
  backTapArea:setTouchEnabled(false)
  self.backTapArea = backTapArea
end

local function createLayout(self)
  local introData = getIntroData()

  local width = ContentReviewCommon.defaultContentWidth
  local height = Screen.getHeightPart(0.75)
  local color = {1.0, 1.0, 1.0, 1.0}
  local content = FilledRectangle.new(width, height, color)

  local titleLineSpacing = height * -0.006
  local title = introData.phrase
  self.titleLabel = ContentReviewTextViewIntro.new(title, width, titleLineSpacing)
  content:insert(self.titleLabel)

  local review = "Review these concepts from " .. ContentReviewCommon.hexagonPlaceholder .. " " .. introData.name .. " to improve your performance."
  self.reviewLabel = ContentReviewTextViewRationale.newWithHexagon(review, width)
  self.reviewLabel:linkOpacityAttrToChildren()
  self.reviewLabel:setColor(unpack(ContentReviewCommon.Colors.lightGray))
  self.reviewLabel:setOpacity(0.0)
  content:insert(self.reviewLabel)

  local titleWidth = self.titleLabel:getWidth()
  local titleHeight = self.titleLabel:getHeight()
  local x = titleWidth * 0.5
  local y = height * 0.65
  self.titleLabel:setLoc(x, y)

  local verticalSpacing = height * 0.07
  local reviewWidth = self.reviewLabel:getWidth()
  local reviewHeight = self.reviewLabel:getHeight()
  x = reviewWidth * 0.5
  y = y - titleHeight * 0.5 - verticalSpacing - reviewHeight * 0.5
  self.reviewLabel:setLoc(x, y)

  self:insertAtCenter(content)

  local tapTip = createTapTip()
  local y = self:getHeightPart(0.025) + tapTip:getHeightPart(0.5)
  if (Screen.hasNotch()) then
    y = 42.0 + tapTip:getHeightPart(0.5)
  end
  tapTip:setLoc(self:getWidthPart(0.5), y)
  self:insert(tapTip)
  self.tapTip = tapTip
end

-- Constructor ------------------------

function ContentReviewCardIntro.new()
  local self = ContentReviewCard.new()
  Mixin.mix(self, ContentReviewCardIntro, "ContentReviewCard")

  createLayout(self)
  createTapAreas(self)

  return self
end
