--- ContentReviewCardInversion
-- @author Created by Joe Poka on October 4, 2016.

class("ContentReviewCardInversion")

require("ContentReviewCard")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWordFontSize(useSmallFont)
  local smallFontSize = 35.0
  local largeFontSize = 58.0

  local fontSize = largeFontSize
  if (useSmallFont) then
    fontSize = smallFontSize
  end

  return fontSize
end

local function createLayout(self, concept)
  local word = string.upper(concept.kind_a)
  local definition = concept.definition_a
  local antonym = concept.kind_b

  local defaultContentWidth = ContentReviewCommon.defaultContentWidth
  local defaultContentHeight = defaultContentWidth

  local contentHeightFraction = 0.185
  local contentHeight = Screen.getHeightPart(contentHeightFraction)

  local fontSize = getWordFontSize(concept.useSmallFont)
  local wordFont = Font.new(FontFaces.DIN_OT_BOLD, fontSize)

  local wordLabel = Label.new(word, wordFont)
  wordLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))

  local verticalSpacingMultiplier = 0.14
  if (concept.useSmallFont) then
    verticalSpacingMultiplier = 0.09
  end

  local verticalSpacing = defaultContentHeight * verticalSpacingMultiplier

  local definitionFontSize = 18.0
  local definitionWidth = defaultContentWidth * 0.95
  local definitionLabel = ContentReviewTextViewRationale.new(definition, definitionWidth, definitionFontSize, Alignment.CENTER)

  local antonymFontSize = 18.0
  local antonymFont = Font.new(FontFaces.DIN_OT_MEDIUM, antonymFontSize)
  local antonymString = "Antonym - " .. StringUtils.capitalizeFirstLetter(antonym)
  local antonymLabel = Label.new(antonymString, antonymFont)
  antonymLabel:setColor(unpack(ContentReviewCommon.Colors.lightGray))

  -- layout items
  local wordHeight = wordLabel:getHeight()
  local x = defaultContentWidth * 0.5
  local y = contentHeight - wordHeight * 0.5
  wordLabel:setLoc(x, y)

  local definitionHeight = definitionLabel:getHeight()
  y = y - verticalSpacing - definitionHeight * 0.5
  definitionLabel:setLoc(x, y)

  local antonymHeight = antonymLabel:getHeight()
  y = y - definitionHeight * 0.5 - verticalSpacing - antonymHeight * 0.5
  antonymLabel:setLoc(x, y)

  local content = Group.new(defaultContentWidth, contentHeight)
  content:insert(wordLabel)
  content:insert(definitionLabel)
  content:insert(antonymLabel)

  self:insertAtCenter(content)

  return content
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ContentReviewCardInversion.new(concept, parameters)
  local self = ContentReviewCard.new()
  Mixin.mix(self, ContentReviewCardInversion, "ContentReviewCard")

  createLayout(self, concept)

  return self
end
