--- ContentReviewCardVisualization
-- @author Created by Steven Mah on October 20, 2016.

class("ContentReviewCardVisualization")

require("ContentReviewCard")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local IMAGE_SIZE = 150

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, concept)
  local word = string.upper(concept.prompt)
  local definition = concept.definition

  local defaultContentWidth = ContentReviewCommon.defaultContentWidth

  local contentHeightFraction = 0.1
  local contentHeight = Screen.getHeightPart(contentHeightFraction)

  local wordFont = Font.new(FontFaces.DIN_OT_BOLD, 35.0)

  local image = AssetHelper.createPhotoSpriteForConcept(concept, concept.image)
  image:setEffectiveDims(IMAGE_SIZE, IMAGE_SIZE)

  local imageWidth, imageHeight = image:getEffectiveDims()
  local backing = FilledRoundedRectangle.new(imageWidth * 1.05, imageHeight * 1.05, 3, {0.0, 0.0, 0.0})
  backing:insertAtCenter(image)

  self.wordLabel = Label.new(word, wordFont)
  self.wordLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))

  local definitionFontSize = 18.0
  local definitionWidth = defaultContentWidth * 0.95
  local definitionLabel = ContentReviewTextViewRationale.new(definition, definitionWidth, definitionFontSize, Alignment.CENTER)

  -- layout items
  local topWordPadding = self.wordLabel:getHeightPart(0.5)
  local x = defaultContentWidth * 0.5
  local y = contentHeight * 1.5
  backing:setLoc(x, y)

  y = backing:getYBottom() - topWordPadding
  self.wordLabel:setAnchorPointTop()
  self.wordLabel:setLoc(x, y)

  local bottomWordPadding = self.wordLabel:getHeightPart(0.2)
  y = self.wordLabel:getYBottom() - bottomWordPadding
  definitionLabel:setAnchorPointTop()
  definitionLabel:setLoc(x, y)

  local content = Group.new(defaultContentWidth, contentHeight)
  content:insert(backing)
  content:insert(self.wordLabel)
  content:insert(definitionLabel)

  self:insertAtCenter(content)

  return content
end

-- Constructor ------------------------

function ContentReviewCardVisualization.new(concept, parameters)
  local self = ContentReviewCard.new()
  Mixin.mix(self, ContentReviewCardVisualization, "ContentReviewCard")

  createLayout(self, concept)

  return self
end
