--- ContentReviewCardWordDefinition
-- @author Created by Joe Poka on July 13, 2016.

class("ContentReviewCardWordDefinition")

require("ContentReviewCard")
require("ContentReviewAnimatedWord")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getCardData(concept)
  local skillName = ContentReviewCommon.SKILL
  local data = {
    [ContentReviewCommon.GAMES.MEMORY] = {
      word = concept.answer,
      definition = concept.prompt
    },
    [ContentReviewCommon.GAMES.WORD_PARTS] = {
      word = concept.assembledParts,
      definition = concept.wordDefinition
    }
  }

  if (ContentReviewCommon.SKILL == ContentReviewCommon.GAMES.WORD_PARTS) then
    if (concept.partCount == 1) then
      data[skillName].definition = "word root that means " .. concept.wordDefinition
    end
  end

  return data[skillName]
end

local function getWordFontSize(self, useSmallFont)
  local smallWordPartsFontSize = 26.0
  local middleFontSize = 35.0
  local largeFontSize = 50.0

  if (useSmallFont) then
    if (self.playingWordParts) then
      return smallWordPartsFontSize
    end
    return middleFontSize
  else
    if (self.playingWordParts) then
      return middleFontSize
    end
    return largeFontSize
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewCardWordDefinition:createAnimationAction()
  if (not self.playingMemory) then
    return AHDoNothing.new()
  end

  local showLatterWord = self.wordLabel:getAnimation()
  return showLatterWord
end

function ContentReviewCardWordDefinition:reset()
  if (not self.playingMemory) then
    return
  end

  self.wordLabel:resetAnimation()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, concept)
  local cardData = getCardData(concept)
  local word = string.upper(cardData.word)
  local definition = cardData.definition

  local defaultContentWidth = ContentReviewCommon.defaultContentWidth
  local defaultContentHeight = defaultContentWidth

  local contentHeightFraction = 0.165
  if (self.playingMemory) then
    contentHeightFraction = 0.25
  end
  local contentHeight = Screen.getHeightPart(contentHeightFraction)

  local fontSize = getWordFontSize(self, concept.useSmallFont)
  local wordFont = Font.new(FontFaces.DIN_OT_BOLD, fontSize)

  if (self.playingMemory) then
    self.wordLabel = ContentReviewAnimatedWord.new(word, wordFont, defaultContentWidth)
  else
    self.wordLabel = Label.new(word, wordFont)
  end

  self.wordLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))

  local verticalSpacingMultiplier = 0.15
  if (concept.useSmallFont) then
    verticalSpacingMultiplier = 0.1
  end
  local verticalSpacing = defaultContentHeight * verticalSpacingMultiplier

  local definitionFontSize = 18.0
  if (self.playingWordParts and concept.useSmallFont) then
    definitionFontSize = 16.0
  end

  local definitionWidth = defaultContentWidth * 0.95
  local definitionLabel = ContentReviewTextViewRationale.new(definition, definitionWidth, definitionFontSize, Alignment.CENTER)

  -- layout items
  local wordHeight = self.wordLabel:getHeight()
  local x = defaultContentWidth * 0.5
  local y = contentHeight - wordHeight * 0.5
  self.wordLabel:setLoc(x, y)

  local definitionHeight = definitionLabel:getHeight()
  x = defaultContentWidth * 0.5
  y = y - verticalSpacing - definitionHeight * 0.5
  definitionLabel:setLoc(x, y)

  local content = Group.new(defaultContentWidth, contentHeight)
  content:insert(self.wordLabel)
  content:insert(definitionLabel)

  self:insertAtCenter(content)

  return content
end

-- Constructor ------------------------

function ContentReviewCardWordDefinition.new(concept, parameters)
  local self = ContentReviewCard.new()
  Mixin.mix(self, ContentReviewCardWordDefinition, "ContentReviewCard")

  -- only memory has an animation
  self.playingMemory = (ContentReviewCommon.SKILL == ContentReviewCommon.GAMES.MEMORY)
  self.playingWordParts = (ContentReviewCommon.SKILL == ContentReviewCommon.GAMES.WORD_PARTS)

  createLayout(self, concept)

  return self
end
