--- ContentReviewCarousel
-- @author Created by Joe Poka on June 27, 2016.

class("ContentReviewCarousel")

require("containers.Carousel")
require("ContentReviewCardEloquence")
require("ContentReviewCardIntro")
require("ContentReviewCardInversion")
require("ContentReviewCardOutro")
require("ContentReviewCardPromptRationale")
require("ContentReviewCardWordDefinition")
require("ContentReviewCardWordParts")
require("ContentReviewCardVisualization")
require("ContentReviewCardAssociation")
require("ContentReviewCardAnimatedMath")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselCardExit")
require("containers.card_carousel.CardCarouselNavigator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CLEAR_COLOR = CardCarouselCardCommon.COLORS.WHITE

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- naive string to int hash but it works for our purposes of guaranteeing
-- the same randomness if concept ids are the same
local function getConceptIdsHash(concepts)
  local conceptsString = ""
  for _, concept in ipairs(concepts) do
    conceptsString = conceptsString .. concept.identifier
  end

  local sum = 0
  local length = #conceptsString
  for characterIndex = 1, length do
    local character = conceptsString:sub(characterIndex, characterIndex)
    sum = sum + string.byte(character)
  end

  return sum
end

local function getCardType()
  local cardTypes = {
    [ContentReviewCommon.GAMES.AGILITY] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.MEMORY] = ContentReviewCardWordDefinition,
    [ContentReviewCommon.GAMES.SPELLING] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.ERROR_AVOIDANCE] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.SYNTAX] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.DICTION] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.BREVITY] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.CLARITY] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.ELOQUENCE] = ContentReviewCardEloquence,
    [ContentReviewCommon.GAMES.INVERSION] = ContentReviewCardInversion,
    [ContentReviewCommon.GAMES.EXPRESSION] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.WORD_PARTS] = ContentReviewCardWordDefinition,
    [ContentReviewCommon.GAMES.VISUALIZATION] = ContentReviewCardVisualization,
    [ContentReviewCommon.GAMES.ADJECTIVE_RECALL] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.ASSOCIATION] = ContentReviewCardAssociation,
    [ContentReviewCommon.GAMES.CONTEXT] = ContentReviewCardPromptRationale,
    [ContentReviewCommon.GAMES.SUBTRACTION] = ContentReviewCardAnimatedMath,
    [ContentReviewCommon.GAMES.PUNCTUATION] = ContentReviewCardPromptRationale
  }

  local cardType = cardTypes[ContentReviewCommon.SKILL]
  return cardType
end

local function getUseSmallFont(concepts)
  local useSmallFont = false
  local skillName = ContentReviewCommon.SKILL

  local variableFontSizeGames = {
    [ContentReviewCommon.GAMES.AGILITY] = true,
    [ContentReviewCommon.GAMES.MEMORY] = true,
    [ContentReviewCommon.GAMES.INVERSION] = true,
    [ContentReviewCommon.GAMES.WORD_PARTS] = true,
    [ContentReviewCommon.GAMES.ELOQUENCE] = true
  }

  local reviewingVariableFontSizeGame = variableFontSizeGames[skillName]

  if (reviewingVariableFontSizeGame) then
    local skillToKey = {
      [ContentReviewCommon.GAMES.AGILITY] = "question",
      [ContentReviewCommon.GAMES.MEMORY] = "answer",
      [ContentReviewCommon.GAMES.INVERSION] = "kind_a",
      [ContentReviewCommon.GAMES.WORD_PARTS] = "assembledParts",
      [ContentReviewCommon.GAMES.ELOQUENCE] = "prompt"
    }

    local key = skillToKey[skillName]

    for _, concept in ipairs(concepts) do
      local word = concept[key]
      local characterCount = #word
      local isWordParts = (skillName == ContentReviewCommon.GAMES.WORD_PARTS)
      local isEloquence = (skillName == ContentReviewCommon.GAMES.ELOQUENCE)
      local isSpecialCase = (isWordParts or isEloquence)
      if (isWordParts) and (characterCount >= 13) then
        useSmallFont = true
        break
      elseif (isEloquence) and (characterCount >= 76) then
        -- NOTE: 76 derived from cutoff of 60 characters but adding 16 to account
        -- for the replace tags
        useSmallFont = true
        break
      elseif (characterCount >= ContentReviewCommon.smallFontCharacterCountThreshold) and (not isSpecialCase) then
        useSmallFont = true
        break
      end
    end
  end

  return useSmallFont
end

local function createCardColumn(self, cardType, concept, parameters)
  local card = cardType.new(concept, parameters)

  table.insert(self.cards, {card})
  card:setIndex(self.cardIndex, self.totalCardCount)
  self.cardIndex = self.cardIndex + 1
end

-- NOTE: Special case needed since a single concept can be spread across multiple cards
local function createWordPartsCards(self, cardType, concept)
  if (concept.partCount > 1) then
    cardType = ContentReviewCardWordParts
    for wordPartIndex = 1, concept.partCount do
      local parameters = {
        wordPartIndex = wordPartIndex
      }
      createCardColumn(self, cardType, concept, parameters)
    end
    cardType = ContentReviewCardWordDefinition
  else
    concept.wordDefinition = concept.definition1
  end
  createCardColumn(self, cardType, concept)
end

local function addExitCards(arrangedCardData)
  local cardGlobals = {closeColor = CLEAR_COLOR}

  -- Front exit card
  table.insert(arrangedCardData, 1, {CardCarouselCardExit.new(CardCarouselCardDefinitions.CARD_TYPES.EXIT, cardGlobals)})

  -- Back exit card
  table.insert(arrangedCardData, {CardCarouselCardExit.new(CardCarouselCardDefinitions.CARD_TYPES.EXIT, cardGlobals)})

  -- Top row exit cards
  for _, column in ipairs(arrangedCardData) do
    table.insert(column, 1, CardCarouselCardExit.newPullToExit(CardCarouselCardDefinitions.CARD_TYPES.PULL_TO_EXIT, cardGlobals))
  end
end

local function createCarousel(self, concepts)
  local skillName = ContentReviewCommon.SKILL
  local isWordParts = (skillName == ContentReviewCommon.GAMES.WORD_PARTS)
  local isAdjectiveRecall = (skillName == ContentReviewCommon.GAMES.ADJECTIVE_RECALL)
  local cardType = getCardType()

  self.cards = {}
  local cardParameters = {}
  self.cardIndex = 1
  self.totalCardCount = #concepts

  table.insert(self.cards, {ContentReviewCardIntro.new()})

  local useSmallFont = getUseSmallFont(concepts)
  local cardCreationFunction = createCardColumn

  -- NOTE: Special case needed since a single concept can be spread across multiple cards
  if (isWordParts) then
    for _, wordPartConcept in ipairs(concepts) do
      local partCount = wordPartConcept.partCount
      if (partCount > 1) then
        self.totalCardCount = self.totalCardCount + partCount
      end
    end
    cardCreationFunction = createWordPartsCards
  -- NOTE: Special case needed because Adjective Recall requires concept id hash
  elseif (isAdjectiveRecall) then
    cardParameters = {
      conceptIdsHash = getConceptIdsHash(concepts)
    }
  end

  for i, concept in ipairs(concepts) do
    concept.useSmallFont = useSmallFont
    cardCreationFunction(self, cardType, concept, cardParameters)
  end

  local outroConcept = {}
  local outroCard = ContentReviewCardOutro.new(outroConcept)
  table.insert(self.cards, {outroCard})

  addExitCards(self.cards)

  local exitCallback = function()
    if (self.onExit) then
      self:onExit()
    end
  end

  self.carousel = CardCarouselNavigator.new(self.cards, exitCallback, CLEAR_COLOR)
  self:insertAtCenter(self.carousel)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewCarousel:onAppear()
  self.carousel:onAppear()
end

function ContentReviewCarousel:setTouchEnabled(enabled)
  self.carousel:setTouchEnabled(enabled)
end

function ContentReviewCarousel:hasVisitedDoneCard()
  return self.carousel:hasVisitedDoneCard()
end

function ContentReviewCarousel:onAppear()
  self.carousel:onAppear()
end
-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ContentReviewCarousel.new(concepts, onExit)
  local self = Group.newFullScreen()
  Mixin.mix(self, ContentReviewCarousel)

  self.onExit = onExit

  createCarousel(self, concepts)

  return self
end
