--- ContentReviewCountup
-- @author Created by Steven Mah on January 4, 2017.

class("ContentReviewCountup")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SPACING_X = Screen.getWidthPart(0.02)
local FADED_VISIBLE_OPACITY = 0.2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createTipLabel(text, fontSize, opacity)
  opacity = opacity or 1.0

  local label = Label.new(text, Font.new(FontFaces.DIN_OT_MEDIUM, fontSize))
  label:setColor(unpack(ContentReviewCommon.Colors.darkGray))

  label:setOpacity(opacity)

  return label
end

local function createShowActionOnElements(elements, opacity)
  opacity = opacity or 1.0
  local showAction = AHSpawn.new()
  for _, element in ipairs(elements) do
    showAction:addAction(AHOpacityTo.new(element, 0.2, nil, opacity))
  end

  return showAction
end

local function createEquationRow(self, leftValue, rightValue, result)
  local equationRow = Row.new(SPACING_X)
  local equationFontSize = 26

  equationRow.leftLabel = createTipLabel(tostring(leftValue), equationFontSize, 0.0)
  equationRow.plusLabel = createTipLabel("+", equationFontSize, 0.0)
  equationRow.rightLabel = createTipLabel(tostring(rightValue), equationFontSize, 0.0)
  equationRow.equalLabel = createTipLabel("=", equationFontSize, 0.0)
  equationRow.resultLabel = createTipLabel(tostring(result), equationFontSize, 0.0)

  equationRow:insert(equationRow.leftLabel)
  equationRow:insert(equationRow.plusLabel)
  equationRow:insert(equationRow.rightLabel)
  equationRow:insert(equationRow.equalLabel)
  equationRow:insert(equationRow.resultLabel)

  table.insert(self.createdElements, equationRow.leftLabel)
  table.insert(self.createdElements, equationRow.plusLabel)
  table.insert(self.createdElements, equationRow.rightLabel)
  table.insert(self.createdElements, equationRow.equalLabel)
  table.insert(self.createdElements, equationRow.resultLabel)

  equationRow:alignChildren()

  local originX, originY = equationRow.rightLabel:getLoc()
  equationRow.rightLabel.origin = { x = originX, y = originY }

  equationRow.resultCopy = createTipLabel(tostring(result), equationFontSize, 0.0)
  equationRow.resultCopy:setLoc(equationRow.resultLabel:getLoc())
  equationRow:insert(equationRow.resultCopy)
  table.insert(self.createdElements, equationRow.resultCopy)

  return equationRow
end

local function createEquationRows(self)
  local minuend = self.minuend
  local minuendDigitsCount = #tostring(minuend)

  local currentCounter = self.subtrahend

  local equationRows = {}

  local rightIndex = 1
  while (rightIndex < minuendDigitsCount) do
    local currentCounterString = tostring(currentCounter)
    local counterDigitIndex = #currentCounterString - rightIndex + 1
    local placeMultiplier = math.pow(10, rightIndex)
    local digit = tonumber(currentCounterString:sub(counterDigitIndex))

    local valueToAdd = (placeMultiplier - digit)

    if (digit ~= 0) then
      local result = currentCounter + valueToAdd
      local willAddingExceedMinuend = (result > minuend)
      if willAddingExceedMinuend then
        break
      end

      local equationRow = createEquationRow(self, currentCounter, valueToAdd, result)
      table.insert(self.createdElements, equationRow)
      table.insert(equationRows, equationRow)

      currentCounter = result
    end

    rightIndex = rightIndex + 1
  end

  if (currentCounter < minuend) then
    local remainder = minuend - currentCounter

    local equationRow = createEquationRow(self, currentCounter, remainder, minuend)
    table.insert(self.createdElements, equationRow)
    table.insert(equationRows, equationRow)
  end

  return equationRows
end

local function createElements(self)
  local problemFontSize = 40
  local problemRow = Row.new(SPACING_X)
  problemRow.leftLabel = createTipLabel(tostring(self.minuend), problemFontSize)
  problemRow.subtractLabel = createTipLabel("-", problemFontSize)
  problemRow.rightLabel = createTipLabel(tostring(self.subtrahend), problemFontSize)

  problemRow:insert(problemRow.leftLabel)
  problemRow:insert(problemRow.subtractLabel)
  problemRow:insert(problemRow.rightLabel)
  problemRow:alignChildren()

  problemRow.subtrahendCopy = createTipLabel(tostring(self.subtrahend), problemFontSize)
  problemRow.subtrahendCopy:setLoc(problemRow.rightLabel:getLoc())
  problemRow:insert(problemRow.subtrahendCopy)

  table.insert(self.createdElements, problemRow.leftLabel)
  table.insert(self.createdElements, problemRow.subtractLabel)
  table.insert(self.createdElements, problemRow.rightLabel)
  table.insert(self.createdElements, problemRow.subtrahendCopy)
  self:insertAtCenter(problemRow)
  self.problemRow = problemRow

  local equationsColumn = Column.new()
  local equations = {}
  local additionElements = {}
  local additionalRows = createEquationRows(self)
  for _, additionalRow in ipairs(additionalRows) do
    equationsColumn:insert(additionalRow)
    table.insert(equations, additionalRow)
  end
  equationsColumn:alignChildren()
  equationsColumn:setAnchorPointTop()
  local totalHeight = problemRow:getHeight() + equationsColumn:getHeight()
  local slideOffset = totalHeight / 2 - problemRow:getHeightPart(0.5)
  equationsColumn:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5) + slideOffset - problemRow:getHeightPart(0.5))
  table.insert(self.createdElements, equationsColumn)
  self:insert(equationsColumn)
  self.equationsColumn = equationsColumn
  self.equations = equations

  local elementsToFadeInAtEnd = {}

  local sumLabel = createTipLabel(tostring(self.answer), 26, 0.0)
  sumLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.4))
  table.insert(self.createdElements, sumLabel)
  table.insert(additionElements, sumLabel)
  self:insert(sumLabel)
  self.sumLabel = sumLabel

  local equalLine = FilledRectangle.new(sumLabel:getWidthPart(2.0), 1, ContentReviewCommon.Colors.darkGray)
  equalLine:setOpacity(0.0)
  equalLine:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.45))
  table.insert(self.createdElements, equalLine)
  table.insert(additionElements, equalLine)
  self:insert(equalLine)
  self.equalLine = equalLine

  local plusLabel = createTipLabel("+", 26, 0.0)
  plusLabel:setLoc(equalLine:getXLeft(), self:getHeightPart(0.52))
  table.insert(self.createdElements, plusLabel)
  table.insert(additionElements, plusLabel)
  self:insert(plusLabel)
  self.plusLabel = plusLabel

  local thereforeLabel = Label.new("THEREFORE", Font.new(FontFaces.DIN_OT_MEDIUM, 18))
  thereforeLabel:setAnchorPointTop()
  thereforeLabel:setLoc(self:getWidthPart(0.5), sumLabel:getYBottom() - self:getHeightPart(0.05))
  thereforeLabel:setColor(unpack(ContentReviewCommon.Colors.lightGray))
  thereforeLabel:setOpacity(0.0)
  self:insert(thereforeLabel)
  table.insert(self.createdElements, thereforeLabel)
  table.insert(elementsToFadeInAtEnd, thereforeLabel)
  self.thereforeLabel = thereforeLabel

  local equationString = self.minuend .. " - " .. self.subtrahend .. " = <answer>" .. self.answer .. "</answer>"
  local basicAttributes = {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, 26),
    color = ContentReviewCommon.Colors.darkGray
  }
  local attributesForLabels = {
    answer = {
      color = ContentReviewCommon.Colors.correctGreen
    }
  }
  local finalAnswerLabel = ContentReviewTextView.new(equationString, self:getWidth(), basicAttributes, attributesForLabels, nil, Alignment.CENTER)
  finalAnswerLabel:setAnchorPointTop()
  finalAnswerLabel:setLoc(self:getWidthPart(0.5), thereforeLabel:getYBottom() - self:getHeightPart(0.025))
  finalAnswerLabel:setOpacity(0.0)
  finalAnswerLabel:linkOpacityAttrToChildren(true)
  self:insert(finalAnswerLabel)
  table.insert(self.createdElements, finalAnswerLabel)
  table.insert(elementsToFadeInAtEnd, finalAnswerLabel)
  self.finalAnswerLabel = finalAnswerLabel

  -- topY Derived from original animation for final y of the top-most addition term
  -- Sloppy at the moment; clean up if possible
  local topY = self:getHeightPart(0.52) + (#equations - 1) * self:getHeightPart(0.09) + equations[1]:getHeightPart(0.5)
  local bottomY = self.finalAnswerLabel:getYBottom()
  local elementsCenterY = (topY + bottomY) / 2
  local offsetY = self:getHeightPart(0.5) - elementsCenterY
  for _, element in ipairs(elementsToFadeInAtEnd) do
    element:setY(element:getY() + offsetY)
  end
  for _, element in ipairs(additionElements) do
    element:setY(element:getY() + offsetY)
  end

  self.additionsYOffset = offsetY
  self.additionElements = additionElements
  self.elementsToFadeInAtEnd = elementsToFadeInAtEnd
end

local function createWaterfallAction(elementsToFadeOut, elementsToShow, fallingElement, targetElement, x, y)
  local fadeAction = AHSpawn.new()
  local fadeDuration = 0.2
  for _, element in ipairs(elementsToFadeOut) do
    fadeAction:addAction(AHOpacityTo.new(element, fadeDuration, nil, FADED_VISIBLE_OPACITY))
  end

  local showAction = AHSpawn.new()
  for _, element in ipairs(elementsToShow) do
    showAction:addAction(AHOpacityTo.new(element, 0.2, nil, 1.0))
  end

  local elementScaleDown = targetElement:getWidth() / fallingElement:getWidth()

  local waterfallAction = AHSpawn.new(
    fadeAction,
    AHScaleTo.new(fallingElement, 0.4, AHEaseSimpleInOut.new(), elementScaleDown),
    AHSequence.new(
      AHRotationBy.new(fallingElement, 0.4, AHEaseSimpleInOut.new(), -30),
      AHRotationBy.new(fallingElement, 0.6, AHEaseSimpleInOut.new(), 30)
    ),
    AHLocationTo.new(fallingElement, 0.75, AHEaseSimpleInOut.new(), x, y),
    AHSequence.new(
      AHDelay.new(1.0),
      showAction,
      AHOpacityTo.new(fallingElement, 0.0, nil, 0.0)
    )
  )

  return waterfallAction
end

local function createCompleteWaterfallAction(self)
  local totalHeight = self.problemRow:getHeight() + self.equationsColumn:getHeight()
  local slideOffset = totalHeight / 2 - self.problemRow:getHeightPart(0.5)
  local firstEquation = self.equations[1]

  local pageAction = AHSequence.new(
    AHLocationYBy.new(self.problemRow, 0.5, AHEaseSimpleInOut.new(), slideOffset),
    AHDelay.new(0.5)
  )

  local elementsToFadeOut = {}
  local elementsToShow = {
    firstEquation.leftLabel,
    firstEquation.plusLabel,
    firstEquation.rightLabel,
    firstEquation.equalLabel,
    firstEquation.resultLabel,
    firstEquation.resultCopy
  }
  local fallingElement = self.problemRow.subtrahendCopy
  local targetElement = firstEquation.leftLabel
  local columnX, columnY = self.equationsColumn:transformLoc(firstEquation, firstEquation.leftLabel:getLoc())
  columnY = columnY - slideOffset
  local worldX, worldY = self:transformLoc(self.equationsColumn, columnX, columnY)
  local x, y = self.problemRow:transformLoc(self, worldX, worldY)

  local initialWaterfallAction = createWaterfallAction(elementsToFadeOut, elementsToShow, fallingElement, targetElement, x, y)
  pageAction:addAction(initialWaterfallAction)

  for equationIndex = 2, #self.equations do
    local topEquation = self.equations[equationIndex - 1]
    local bottomEquation = self.equations[equationIndex]

    elementsToFadeOut = {
      topEquation.leftLabel,
      topEquation.plusLabel,
      topEquation.rightLabel,
      topEquation.equalLabel,
      topEquation.resultLabel
    }
    elementsToShow = {
      bottomEquation.leftLabel,
      bottomEquation.plusLabel,
      bottomEquation.rightLabel,
      bottomEquation.equalLabel,
      bottomEquation.resultLabel,
      bottomEquation.resultCopy
    }

    fallingElement = topEquation.resultCopy
    targetElement = bottomEquation.leftLabel

    x, y = topEquation:transformLoc(bottomEquation, bottomEquation.leftLabel:getLoc())
    initialWaterfallAction = createWaterfallAction(elementsToFadeOut, elementsToShow, fallingElement, targetElement, x, y)
    pageAction:addAction(AHDelayed.new(0.5, initialWaterfallAction))
  end

  return pageAction
end

local function createFadeAllExceptAdditionsAction(self)
  local duration = 0.2
  local fadeAction = AHSpawn.new(
    AHOpacityTo.new(self.problemRow.leftLabel, duration, nil, 0.0),
    AHOpacityTo.new(self.problemRow.subtractLabel, duration, nil, 0.0),
    AHOpacityTo.new(self.problemRow.rightLabel, duration, nil, 0.0)
  )

  for _, equation in ipairs(self.equations) do
      fadeAction:addAction(AHOpacityTo.new(equation.leftLabel, duration, nil, 0.0))
      fadeAction:addAction(AHOpacityTo.new(equation.plusLabel, duration, nil, 0.0))
      fadeAction:addAction(AHOpacityTo.new(equation.resultLabel, duration, nil, 0.0))
      fadeAction:addAction(AHOpacityTo.new(equation.equalLabel, duration, nil, 0.0))
      fadeAction:addAction(AHOpacityTo.new(equation.resultCopy, duration, nil, 0.0))
      fadeAction:addAction(AHOpacityTo.new(equation.rightLabel, duration, nil, 1.0))
  end

  return fadeAction
end

local function createOrganizeAdditionsAction(self)
  local organizeAction = AHSpawn.new()
  local baseX = self.sumLabel:getXRight()
  local baseY = self.plusLabel:getY()
  local additionTerms = {}
  for equationIndex, equation in ipairs(self.equations) do
    local addition = equation.rightLabel
    self:takeChild(addition)
    table.insert(additionTerms, addition)

    local desiredX = baseX
    local desiredY = baseY + (#self.equations - equationIndex) * self:getHeightPart(0.09)

    organizeAction:addAction(AHSpawn.new(
      AHAnchorPointTo.new(addition, 0.5, AHEaseSimpleInOut.new(), 1.0, 0.5),
      AHLocationTo.new(addition, 0.5, AHEaseSimpleInOut.new(), desiredX, desiredY)
    ))
  end

  self.additionTerms = additionTerms

  return organizeAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewCountup:createAnimations()
  local delayDuration = 1.0
  return AHSequence.new(
    createCompleteWaterfallAction(self),
    AHDelayed.new(
      delayDuration * 0.75,
      createFadeAllExceptAdditionsAction(self),
      createOrganizeAdditionsAction(self)
    ),
    AHDelayed.new(
      delayDuration * 0.75,
      createShowActionOnElements(self.additionElements, 1.0),
      createShowActionOnElements(self.additionTerms, 1.0)
    ),
    AHDelayed.new(
      delayDuration,
      createShowActionOnElements(self.additionElements, FADED_VISIBLE_OPACITY),
      createShowActionOnElements(self.additionTerms, FADED_VISIBLE_OPACITY),
      createShowActionOnElements(self.elementsToFadeInAtEnd)
    )
  )
end

function ContentReviewCountup:reset()
  local problemRow = self.problemRow
  problemRow.leftLabel:setOpacity(1.0)
  problemRow.subtractLabel:setOpacity(1.0)
  problemRow.rightLabel:setOpacity(1.0)
  problemRow.subtrahendCopy:setOpacity(1.0)
  problemRow.subtrahendCopy:setScl(1.0)
  problemRow.subtrahendCopy:setRot(0.0)
  problemRow.subtrahendCopy:setLoc(problemRow.rightLabel:getLoc())
  problemRow:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))

  for equationIndex, equation in ipairs(self.equations) do
    equation.leftLabel:setOpacity(0.0)
    equation.plusLabel:setOpacity(0.0)
    equation.equalLabel:setOpacity(0.0)
    equation.resultLabel:setOpacity(0.0)

    local rightLabel = equation.rightLabel
    local resultCopy = equation.resultCopy
    rightLabel:setOpacity(0.0)
    rightLabel:setScl(1.0)
    resultCopy:setOpacity(0.0)
    resultCopy:setRot(0.0)
    resultCopy:setScl(0.0)
    resultCopy:setLoc(equation.resultLabel:getLoc())
    rightLabel:setAnchorPointCenter()
    equation:takeChild(rightLabel)
    rightLabel:setLoc(rightLabel.origin.x, rightLabel.origin.y)
  end

  self.plusLabel:setOpacity(0.0)
  self.equalLine:setOpacity(0.0)
  self.sumLabel:setOpacity(0.0)

  for _, element in ipairs(self.elementsToFadeInAtEnd) do
    element:setOpacity(0.0)
  end
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function ContentReviewCountup.new(width, minuend, subtrahend, answer)
  local backingColor = {0.98, 0.98, 0.98}
  local self = FilledRectangle.new(width, width, backingColor)
  Mixin.mix(self, ContentReviewCountup)

  self.createdElements = {}
  self.minuend = minuend
  self.subtrahend = subtrahend
  self.answer = answer

  createElements(self)

  return self
end
