--- ContentReviewDouble
-- @author Created by Steven Mah on January 4, 2017.

class("ContentReviewDouble")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local NUMBER_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 33)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createElements(self)
   -- makes equation look better. Without it, there right side of the minuend string seems cramped
  local hackedSpacing = 10
  local equationRow = Row.new(hackedSpacing)

  local itemsToShift = {}

  local minuendString = tostring(self.minuend) .. " - "
  local minuendLabel = Label.new(minuendString, NUMBER_FONT)
  minuendLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  table.insert(self.createdElements, minuendLabel)
  equationRow:insert(minuendLabel)
  self.minuendLabel = minuendLabel

  local subtrahendString = tostring(self.subtrahend)
  local subtrahendLabel = Label.new(subtrahendString, NUMBER_FONT)
  subtrahendLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  table.insert(self.createdElements, subtrahendLabel)
  equationRow:insert(subtrahendLabel)

  equationRow:alignChildren()

  equationRow:setLoc(self:getWidthPart(0.5), self.equationFinalY)
  self:insert(equationRow)
  table.insert(self.createdElements, equationRow)
  table.insert(itemsToShift, equationRow)
  self.equationRow = equationRow

  local subtrahendWorldX, _ = self:transformLoc(equationRow, subtrahendLabel:getLoc())

  local paddingY = -self:getHeightPart(0.03)

  local multiplyLabel = Label.new("x 2", NUMBER_FONT)
  multiplyLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  multiplyLabel:setOpacity(0.0)
  multiplyLabel:setAnchorPointRight()
  multiplyLabel:setLoc(subtrahendWorldX + subtrahendLabel:getWidthPart(0.5), equationRow:getYBottom() + 1.5 * paddingY)
  self:insert(multiplyLabel)
  table.insert(self.createdElements, multiplyLabel)
  table.insert(itemsToShift, multiplyLabel)
  self.multiplyLabel = multiplyLabel

  local equalLine = FilledRectangle.new(multiplyLabel:getWidthPart(2.0), 1)
  equalLine:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  equalLine:setOpacity(0.0)
  equalLine:setLoc(subtrahendWorldX, multiplyLabel:getYBottom() + 0.75 * paddingY)
  self:insert(equalLine)
  table.insert(self.createdElements, equalLine)
  table.insert(itemsToShift, equalLine)
  self.equalLine = equalLine

  local answerString = tostring(self.minuend)
  local answerLabel = Label.new(answerString, NUMBER_FONT)
  answerLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  answerLabel:setOpacity(0.0)
  answerLabel:setAnchorPointTop()
  answerLabel:setLoc(subtrahendWorldX, equalLine:getYBottom() + 0.75 * paddingY)
  self:insert(answerLabel)
  table.insert(self.createdElements, answerLabel)
  table.insert(itemsToShift, answerLabel)
  self.answerLabel = answerLabel

  local thereforeLabel = Label.new("THEREFORE", Font.new(FontFaces.DIN_OT_MEDIUM, 18))
  thereforeLabel:setAnchorPointTop()
  thereforeLabel:setLoc(self:getWidthPart(0.5), answerLabel:getYBottom() + paddingY)
  thereforeLabel:setColor(unpack(ContentReviewCommon.Colors.lightGray))
  thereforeLabel:setOpacity(0.0)
  self:insert(thereforeLabel)
  table.insert(self.createdElements, thereforeLabel)
  self.thereforeLabel = thereforeLabel

  local equationString = self.minuend .. " - " .. self.subtrahend .. " = <answer>" .. self.answer .. "</answer>"
  local basicAttributes = {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, 28),
    color = ContentReviewCommon.Colors.darkGray
  }
  local attributesForLabels = {
    answer = {
      color = ContentReviewCommon.Colors.correctGreen
    }
  }
  local finalAnswerLabel = ContentReviewTextView.new(equationString, self:getWidth(), basicAttributes, attributesForLabels, nil, Alignment.CENTER)
  finalAnswerLabel:setAnchorPointTop()
  finalAnswerLabel:setLoc(self:getWidthPart(0.5), thereforeLabel:getYBottom() - self:getHeightPart(0.025))
  finalAnswerLabel:setOpacity(0.0)
  finalAnswerLabel:linkOpacityAttrToChildren(true)
  self:insert(finalAnswerLabel)
  table.insert(self.createdElements, finalAnswerLabel)
  self.finalAnswerLabel = finalAnswerLabel

  -- layout needed to be based on final positioning; calculate and set starting
  -- positions here
  local topY = itemsToShift[1]:getYTop()
  local bottomY = itemsToShift[#itemsToShift]:getYBottom()
  local groupCenterY = (topY + bottomY) / 2
  local yOffset = (self:getHeightPart(0.5) - groupCenterY)
  for _, item in ipairs(itemsToShift) do
    local effectiveY = item:getY() + yOffset
    item:setY(effectiveY)
    item.originY = effectiveY
  end

  self.itemsToShift = itemsToShift
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewDouble:createAnimations()
  local delayDuration = 2.0

  local shiftAction = AHSpawn.new()
  local deltaY = self.equationFinalY - self.equationRow:getY()
  for _, item in ipairs(self.itemsToShift) do
    shiftAction:addAction(
      AHLocationYBy.new(item, 1.0, AHEaseSimpleInOut.new(5), deltaY)
    )
  end

  local pageAction = AHSequence.new(
    AHDelayed.new(
      delayDuration / 2,
      AHOpacityTo.new(self.minuendLabel, 0.5, nil, 0.2),
      AHOpacityTo.new(self.multiplyLabel, 0.5, nil, 1.0),
      AHOpacityTo.new(self.equalLine, 0.5, nil, 1.0)
    ),
    AHDelayed.new(
      delayDuration,
      AHOpacityTo.new(self.answerLabel, 0.4, nil, 1.0)
    ),
    AHDelay.new(delayDuration),
    AHStaggeredSpawn.new(
      0.3,
      shiftAction,
      AHSpawn.new(
        AHOpacityTo.new(self.thereforeLabel, 0.3, nil, 1.0),
        AHOpacityTo.new(self.finalAnswerLabel, 0.3, nil, 1.0)
      )
    )
  )

  return pageAction
end

function ContentReviewDouble:reset()
  for _, item in ipairs(self.itemsToShift) do
    item:setY(item.originY)
  end
  self.minuendLabel:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  self.multiplyLabel:setOpacity(0.0)
  self.equalLine:setOpacity(0.0)
  self.answerLabel:setOpacity(0.0)
  self.thereforeLabel:setOpacity(0.0)
  self.finalAnswerLabel:setOpacity(0.0)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function ContentReviewDouble.new(width, minuend, subtrahend, answer)
  local backingColor = {0.98, 0.98, 0.98}
  local self = FilledRectangle.new(width, width, backingColor)
  Mixin.mix(self, ContentReviewDouble)

  self.createdElements = {}
  self.minuend = minuend
  self.subtrahend = subtrahend
  self.answer = answer
  self.equationStartingY = self:getHeightPart(0.5)
  self.equationFinalY = self:getHeightPart(0.8)

  createElements(self)

  return self
end
