--- ContentReviewGame
-- @author Created by Joe Poka on June 27, 2016.

class("ContentReviewGame")

require("ContentReviewCarousel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {1, 1, 1, 1}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function onExit(self)
  self.carousel:setTouchEnabled(false)
  runInCoroutine(function()

    -- Wait three extra frames to fill the screen and the framebuffer with the same image
    -- and prevent the app from flickering.
    waitForFrames(3)

    GameSession.notifyGameWin()
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewGame:gameFlow()
  self.carousel:onAppear()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createGameElements(self)
  local background = FilledRectangle.newFullScreen(BACKGROUND_COLOR)
  self:insertAtCenter(background)
end

local function createUI(self, concepts)
  self.carousel = ContentReviewCarousel.new(concepts, function() onExit(self) end)
  self:insertAtCenter(self.carousel)
end

-- Constructor ------------------------

function ContentReviewGame.new()
  local self = BaseGame.new()
  Mixin.mix(self, ContentReviewGame)

  GameSession.setScoreThresholds(0, 0)

  createGameElements(self)

  -- back button exits the game w/ a loss
  MOAIInputMgr.device.back:setCallback(function(buttonIsDown)
    runInCoroutine(function()
      GameSession.notifyGameLoss()
    end)
  end)

  local concepts = {}

  -- grab all concepts
  repeat
    local concept = GameSession.conceptGenerator:generateNextValue()
    local haveConcept = (concept ~= nil)
    if (haveConcept) then
      table.insert(concepts, concept)
    end
  until (not haveConcept)

  FontScaling.setScalingMethod(FontScaling.SCALING_METHOD_HEIGHT)

  createUI(self, concepts)

  return self
end
