--- ContentReviewTakeaway
-- @author Created by Steven Mah on January 4, 2017.

class("ContentReviewTakeaway")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local NUMBER_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 33)
local NUMBER_KERNING_X = Screen.getWidthPart(0.015)
local EQUAL_LINE_SPACER_Y_HEIGHT = 10
local FADED_VISIBLE_OPACITY = 0.2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createHiddenDigitView(valueString)
  local view = Label.new(valueString, NUMBER_FONT)
  view:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  view:setOpacity(0.0)

  return view
end

local function createEquationRow(self)
  local equationRow = Row.new(NUMBER_KERNING_X, Alignment.TOP, Row.GrowLeft)
  local minuendString = tostring(self.minuend)
  local subtrahendString = tostring(self.subtrahend)
  local answerString = tostring(self.answer)

  local minuendDigitCount = #minuendString
  local subtrahendDigitCount = #subtrahendString
  local answerDigitCount = #answerString

  local minuendDigits = {}
  local subtrahendDigits = {}
  local answerDigits = {}

  for digitIndex = 1, minuendDigitCount do
    local digitIndexFromRight = minuendDigitCount - digitIndex + 1
    local minuendDigit = minuendString:sub(digitIndexFromRight, digitIndexFromRight)
    local minuendDigitView = Label.new(minuendDigit, NUMBER_FONT)
    minuendDigitView:setColor(unpack(ContentReviewCommon.Colors.darkGray))
    table.insert(self.createdElements, minuendDigitView)
    table.insert(minuendDigits, minuendDigitView)

    local subtrahendDigitIndexForMinuendDigit = digitIndexFromRight - (minuendDigitCount - subtrahendDigitCount)
    local subtrahendDigitView = nil
    if (subtrahendDigitIndexForMinuendDigit > 0) then
      local subtrahendDigit = subtrahendString:sub(subtrahendDigitIndexForMinuendDigit, subtrahendDigitIndexForMinuendDigit)
      subtrahendDigitView = Label.new(subtrahendDigit, NUMBER_FONT)
      subtrahendDigitView:setColor(unpack(ContentReviewCommon.Colors.darkGray))
      table.insert(self.createdElements, subtrahendDigitView)
      table.insert(subtrahendDigits, subtrahendDigitView)
    end

    local answerDigitIndexForMinuendDigit = digitIndexFromRight - (minuendDigitCount - answerDigitCount)
    local answerDigitView = nil
    if (answerDigitIndexForMinuendDigit > 0) then
      local answerDigit = answerString:sub(answerDigitIndexForMinuendDigit, answerDigitIndexForMinuendDigit)
      answerDigitView = createHiddenDigitView(answerDigit)
      table.insert(self.createdElements, answerDigitView)
      table.insert(answerDigits, answerDigitView)
    end

    local digitColumn = Column.new()

    digitColumn:insert(minuendDigitView)

    if subtrahendDigitView then
      digitColumn:insert(subtrahendDigitView)
    else
      digitColumn:insertSpacer(minuendDigitView:getHeight())
    end

    digitColumn:insertSpacer(EQUAL_LINE_SPACER_Y_HEIGHT)

    if answerDigitView then
      digitColumn:insert(answerDigitView)
    else
      digitColumn:insertSpacer(minuendDigitView:getHeight())
    end
    digitColumn:alignChildren()

    equationRow:insert(digitColumn)
  end

  equationRow:alignChildren()
  self.minuendDigits = minuendDigits
  self.subtrahendDigits = subtrahendDigits
  self.answerDigits = answerDigits

  return equationRow
end

local function createElements(self)
  local itemsToShift = {}

  local equationRow = createEquationRow(self)
  equationRow:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.65))
  self:insert(equationRow)
  table.insert(itemsToShift, equationRow)
  self.equationRow = equationRow

  local equalLine = FilledRectangle.new(equationRow:getWidthPart(2.0), 1)
  local rightmostEquationColumn = equationRow.children[1]
  local answerDigitView = rightmostEquationColumn.children[#rightmostEquationColumn.children]
  local _, answerDigitRowY = self:transformLoc(equationRow, answerDigitView:getLoc())
  local answerDigitWorldYTop = answerDigitRowY + answerDigitView:getHeightPart(0.5)
  equalLine:setLoc(self:getWidthPart(0.5), answerDigitWorldYTop)
  equalLine:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  self:insert(equalLine)
  table.insert(itemsToShift, equalLine)
  table.insert(self.createdElements, equalLine)
  self.equalLine = equalLine

  local subtractView = Label.new("-", NUMBER_FONT)
  local leftmostEquationColumn = equationRow.children[#equationRow.children]
  local minuendDigitView = leftmostEquationColumn.children[1]
  local minuendDigitRowX, _ = self:transformLoc(equationRow, minuendDigitView:getLoc())
  subtractView:setLoc(minuendDigitRowX - minuendDigitView:getWidthPart(1.5), answerDigitRowY + minuendDigitView:getHeight() + EQUAL_LINE_SPACER_Y_HEIGHT)
  subtractView:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  self:insert(subtractView)
  table.insert(itemsToShift, subtractView)
  table.insert(self.createdElements, subtractView)
  self.subtractView = subtractView

  local thereforeLabel = Label.new("THEREFORE", Font.new(FontFaces.DIN_OT_MEDIUM, 18))
  thereforeLabel:setAnchorPointTop()
  thereforeLabel:setLoc(self:getWidthPart(0.5), equationRow:getYBottom() - self:getHeightPart(0.05))
  thereforeLabel:setColor(unpack(ContentReviewCommon.Colors.lightGray))
  thereforeLabel:setOpacity(0.0)
  self:insert(thereforeLabel)
  table.insert(self.createdElements, thereforeLabel)
  self.thereforeLabel = thereforeLabel

  local equationString = self.minuend .. " - " .. self.subtrahend .. " = <answer>" .. self.answer .. "</answer>"
  local basicAttributes = {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, 28),
    color = ContentReviewCommon.Colors.darkGray
  }
  local attributesForLabels = {
    answer = {
      color = ContentReviewCommon.Colors.correctGreen
    }
  }
  local finalAnswerLabel = ContentReviewTextView.new(equationString, self:getWidth(), basicAttributes, attributesForLabels, nil, Alignment.CENTER)
  finalAnswerLabel:setAnchorPointTop()
  finalAnswerLabel:setLoc(self:getWidthPart(0.5), thereforeLabel:getYBottom() - self:getHeightPart(0.025))
  finalAnswerLabel:setOpacity(0.0)
  finalAnswerLabel:linkOpacityAttrToChildren(true)
  self:insert(finalAnswerLabel)
  table.insert(self.createdElements, finalAnswerLabel)
  self.finalAnswerLabel = finalAnswerLabel

  local topY = self.equationRow:getYTop()
  local bottomY = self.equationRow:getYBottom()
  local groupCenterY = (topY + bottomY) / 2
  local yOffset = (self:getHeightPart(0.5) - groupCenterY)
  for _, item in ipairs(itemsToShift) do
    local effectiveY = item:getY() + yOffset
    item:setY(effectiveY)
    item.originY = effectiveY
  end

  self.itemsToShift = itemsToShift
  self.shiftDelta = yOffset
end

local function createMoreOpacityAction(self, columnIndex, opacity)
  local equationRow = self.equationRow
  local column = equationRow.children[columnIndex]

  local morphAction = AHSpawn.new()

  for rowIndex = 1, 3 do
    local view = column.children[rowIndex]
    if view then
      morphAction:addAction(AHOpacityTo.new(view, 0.6, nil, opacity))
    end
  end

  return morphAction
end

local function createDimAction(self, excludedIndex)
  local dimAction = AHSpawn.new()
  for columnIndex = 1, #self.equationRow.children do
    local destinationOpacity = nil

    if (columnIndex == excludedIndex) then
      destinationOpacity = 1.0
    else
      destinationOpacity = FADED_VISIBLE_OPACITY
    end

    dimAction:addAction(createMoreOpacityAction(self, columnIndex, destinationOpacity))
  end

  return dimAction
end

local function createBrightenAllAction(self)
  local brightenAction = AHSpawn.new()
  for columnIndex = 1, #self.equationRow.children do
    brightenAction:addAction(createMoreOpacityAction(self, columnIndex, 1.0))
  end

  return brightenAction
end

local function createShowDifferenceAction(self, columnIndex)
  local column = self.equationRow.children[columnIndex]
  local answerView = column.children[#column.children]
  local showAction = AHSpawn.new(
    AHOpacityTo.new(answerView, 0.3, nil, 1.0)
  )

  return showAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewTakeaway:createAnimations()
  local pageAction = AHSequence.new(
    AHDelay.new(0.5)
  )

  local columnsCount = #self.equationRow.children
  for columnIndex = 1, columnsCount do
    if columnsCount > 1 then
      local dimAction = AHSequence.new(createDimAction(self, columnIndex), AHDelay.new(0.5))
      pageAction:addAction(dimAction)
    end

    local showDifferenceAction = AHSequence.new(createShowDifferenceAction(self, columnIndex), AHDelay.new(1.0))
    pageAction:addAction(showDifferenceAction)
  end

  local shiftAction = AHSpawn.new()
  local deltaY = -self.shiftDelta
  for _, item in ipairs(self.itemsToShift) do
    shiftAction:addAction(
      AHLocationYBy.new(item, 1.0, AHEaseSimpleInOut.new(5), deltaY)
    )
  end

  pageAction:addAction(createBrightenAllAction(self))
  pageAction:addAction(
    AHStaggeredSpawn.new(
      0.3,
      shiftAction,
      AHSpawn.new(
        AHOpacityTo.new(self.thereforeLabel, 0.3, nil, 1.0),
        AHOpacityTo.new(self.finalAnswerLabel, 0.3, nil, 1.0)
      )
    )
  )

  return pageAction
end

function ContentReviewTakeaway:reset()
  for _, item in ipairs(self.itemsToShift) do
    item:setY(item.originY)
  end

  for _, digit in ipairs(self.minuendDigits) do
    digit:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  end

  for _, digit in ipairs(self.subtrahendDigits) do
    digit:setColor(unpack(ContentReviewCommon.Colors.darkGray))
  end

  for _, digit in ipairs(self.answerDigits) do
    digit:setOpacity(0.0)
  end

  self.thereforeLabel:setOpacity(0.0)
  self.finalAnswerLabel:setOpacity(0.0)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function ContentReviewTakeaway.new(width, minuend, subtrahend, answer)
  local backingColor = {0.98, 0.98, 0.98}
  local self = FilledRectangle.new(width, width, backingColor)
  Mixin.mix(self, ContentReviewTakeaway)

  self.createdElements = {}
  self.minuend = minuend
  self.subtrahend = subtrahend
  self.answer = answer

  createElements(self)

  return self
end
