--- ContentReviewTextViewIntro
-- @author Created by Joe Poka on September 26, 2016.

class("ContentReviewTextViewIntro")

require("ContentReviewTextView")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getGlyphs(self)
  local itemViews = self:getItemViews()
  local glyphs = {}

  for _, view in ipairs(itemViews) do
    if (view.layout) then
      local rows = view.layout:getRows()
      for _, row in ipairs(rows) do
        for _, glyph in ipairs(row) do
          table.insert(glyphs, glyph)
        end
      end
    end
  end

  return glyphs
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ContentReviewTextViewIntro:getTextAnimation()
  local moveGlyphs = AHSpawn.new()
  local moveDuration = 0.8
  local ease = AHEaseExponentialOut.new()
  local minimumDelay = 0.0
  local maximumDelay = 0.25

  for _, glyph in ipairs(self.glyphs) do
    local move = AHLocationYTo.new(glyph, moveDuration, ease, 0.0)
    local delayDuration = MathUtils.getRandomFloat(minimumDelay, maximumDelay)
    local delayedMove = AHDelayed.new(delayDuration, move)
    moveGlyphs:addAction(delayedMove)
  end

  return moveGlyphs
end

function ContentReviewTextViewIntro:resetTextAnimation()
  local glyphOffsetY = Screen.getHeightPart(0.6) + self:getHeightPart(0.25)
  for _, glyph in ipairs(self.glyphs) do
    local x, _ = glyph:getLoc()
    glyph:setLoc(x, glyphOffsetY)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ContentReviewTextViewIntro.new(xmlText, textWidth, lineSpacing)
  local baseAttributes = {
    font = PrerenderedFont.new(FontFaces.DIN_OT_LIGHT, 42.0),
    color = ContentReviewCommon.Colors.darkGray
  }

  local attributesForLabels = {
    highlight = {
      color = ContentReviewCommon.Colors.highlightBlue
    }
  }

  local self = ContentReviewTextView.new(xmlText, textWidth, baseAttributes, attributesForLabels, lineSpacing)
  Mixin.mix(self, ContentReviewTextViewIntro)

  self.glyphs = getGlyphs(self)
  self:resetTextAnimation()

  return self
end
