--- CrossCellState
-- @author Created by andrewcousins on March 17, 2022.

class("CrossCellState")


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossCellState:getIsWall()
  return self.isWall
end

function CrossCellState:hasClueNumber()
  return (self.clueNumber ~= "")
end

function CrossCellState:getClueNumber()
  return self.clueNumber
end

function CrossCellState:hasContent()
  return (self.contentText ~= "")
end

function CrossCellState:getContent()
  return self.contentText
end

function CrossCellState:getRevealed()
  return self.isRevealed
end

function CrossCellState:isEmpty()
  return not (self:getIsWall() or self:hasContent())
end

function CrossCellState:setContent(content, isRevealed)
  if (content == nil) then
    content = ""
  end
  if (isRevealed == nil) then
    isRevealed = false
  end
  self.isRevealed = isRevealed
  self.contentText = content
end

function CrossCellState:reset()
  self.contentText = ""
  self.isRevealed = false
end

function CrossCellState:setRevealDelay(delay)
  self.revealDelay = delay
end

function CrossCellState:getRevealDelay()
  return self.revealDelay
end

-- Note: isIncorrect tracks if the cell is marked as incorrect
function CrossCellState:getIsIncorrect()
  return self.isIncorrect
end

function CrossCellState:setIsIncorrect(isIncorrect)
  self.isIncorrect = isIncorrect
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CrossCellState.new(isWall, clueNumber)
  local self = {}
  Mixin.mix(self, CrossCellState)

  self.isWall = isWall or false
  self.clueNumber = clueNumber or ""

  self.contentText = ""
  self.isRevealed = false
  self.revealDelay = 0

  self.isIncorrect = false

  return self
end
