--- CrossClue
-- @author Created by andrewcousins on March 10, 2022.

class("CrossClue")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

CrossClue.DIRECTION = {
  ACROSS = "CLUE_DIRECTION.ACROSS",
  DOWN = "CLUE_DIRECTION.DOWN"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function buildClueDescription(self)
  local description = tostring(self:getClueNumber()) .. "-"
  if self:isAcross() then
    description = description .. "across"
  else
    description = description .. "down"
  end
  return description
end



-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossClue:getClueNumber()
  return self.clueNumber
end

function CrossClue:getClueText(crosswordDifficulty)
  assert(self.clueTextByCrosswordDifficulty[crosswordDifficulty], "No clue text provided for " .. buildClueDescription(self) .. " for difficulty: " .. tostring(crosswordDifficulty))
  return self.clueTextByCrosswordDifficulty[crosswordDifficulty]
end

function CrossClue:isDown()
  return (self.direction == CrossClue.DIRECTION.DOWN)
end

function CrossClue:isAcross()
  return (self.direction == CrossClue.DIRECTION.ACROSS)
end

function CrossClue:getDirection()
  return self.direction
end

function CrossClue:getFirstGridCoordinates()
  return self.clueCellCoordinates[1]
end

function CrossClue:getClueCellCoordinates()
  return self.clueCellCoordinates
end

function CrossClue:addCellCoordinatesToClue(column, row)
  table.insert(self.clueCellCoordinates, {column, row})
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CrossClue.new(clueNumber, clueTextByCrosswordDifficulty, direction, supportedDifficulties)
  local self = {}
  Mixin.mix(self, CrossClue)

  self.clueNumber = clueNumber
  self.clueTextByCrosswordDifficulty = clueTextByCrosswordDifficulty
  self.direction = direction

  self.clueCellCoordinates = {}

  assert(clueTextByCrosswordDifficulty, "No clue text provided for " .. buildClueDescription(self))
  if supportedDifficulties then
    for _, crosswordDifficulty in ipairs(supportedDifficulties) do
      assert(clueTextByCrosswordDifficulty[crosswordDifficulty], "No clue text provided for " .. buildClueDescription(self) .. " for difficulty: " .. tostring(crosswordDifficulty))
    end
  end

  return self
end
