--- CrossClueBarArrowButton
-- @author Created by andrewcousins on May 17, 2023.

class("CrossClueBarArrowButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INTRO_ANIMATION_MOVE_OFFSET = 14

local PRESS_ANIMATION_MOVE_OFFSET = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getOnTouchDown(self)
  return function()
    self.backgroundActionInterrupt:insert(AHColorTo.new(self.buttonBackground, 0, nil, CrossColors.CLUE_DOWNSTATE))
  end
end

local function getOnTouchUp(self)
  local arrowSpriteHomeX = self.arrowSprite:getX()
  local duration = 0.333
  local ease = AHEaseBezier.newFlat({0.0, 0.0, 0.3, 1.0})
  local onPressAnimation = AHSequence.new(
    function() self.arrowSprite:setX(arrowSpriteHomeX + PRESS_ANIMATION_MOVE_OFFSET) end,
    AHLocationXTo.new(self.arrowSprite, duration, ease, arrowSpriteHomeX)
  )

  return function()
    Sound.playSoundEffect("clue_bar_tap")
    self.backgroundActionInterrupt:insert(AHColorTo.new(self.buttonBackground, 0.1, nil, CrossColors.CLUE_BACKGROUND))
    self.arrowActionInterrupt:insert(onPressAnimation)
    self.callback()
  end
end

local function getOnTouchCancel(self)
  return function()
    self.backgroundActionInterrupt:insert(AHColorTo.new(self.buttonBackground, 0.1, nil, CrossColors.CLUE_BACKGROUND))
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossClueBarArrowButton:getIntroAnimation()
  self.line:setScaleY(0)
  self.arrowSprite:setOpacity(0)
  self.arrowSprite:offsetX(-INTRO_ANIMATION_MOVE_OFFSET)

  local lineDrawDuration = 0.333
  local lineDrawEase = AHEaseBezier.newFlat({0.0, 0.0, 0.0, 1})
  local lineDrawAnimation = AHScaleYTo.new(self.line, lineDrawDuration, lineDrawEase, 1)

  local arrowMoveDelay = 0.05
  local arrowMoveDuration = 0.333
  local arrowMoveEase = AHEaseBezier.newFlat({0.0, 0.0, 0.0, 1})
  local arrowMoveAnimation = AHDelayed.new(arrowMoveDelay, AHLocationXBy.new(self.arrowSprite, arrowMoveDuration, arrowMoveEase, INTRO_ANIMATION_MOVE_OFFSET))

  local arrowFadeDelay = 0.05
  local arrowFadeDuration = 0.05
  local arrowFadeEase = AHEaseLinear.new()
  local arrowFadeAnimation = AHDelayed.new(arrowFadeDelay, AHOpacityTo.new(self.arrowSprite, arrowFadeDuration, arrowFadeEase, 1))

  return AHSpawn.new(lineDrawAnimation, arrowMoveAnimation, arrowFadeAnimation)
end

function CrossClueBarArrowButton:getFadeOutAnimation(duration, ease)
  return AHSpawn.new(
    AHOpacityTo.new(self.arrowSprite, duration, ease, 0),
    AHOpacityTo.new(self.line, duration, ease, 0)
  )
end

function CrossClueBarArrowButton:prepareForReview()
  self.arrowSprite:setOpacity(0)
  self.line:setOpacity(0)
end

function CrossClueBarArrowButton:showForReview()
  self.arrowSprite:setOpacity(1)
  self.line:setOpacity(1)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function makeButton(self)
  Button.makeButton(self, getOnTouchDown(self), getOnTouchUp(self), getOnTouchCancel(self))
end

local function createLayout(self)
  local buttonBackgroundWidth = self:getWidthPart(0.75)
  local buttonBackgroundHeight = self:getHeightPart(0.778)
  local buttonBackgroundCornerRadius = buttonBackgroundWidth * 0.125
  local buttonBackground = FilledRoundedRectangle.new(buttonBackgroundWidth, buttonBackgroundHeight, buttonBackgroundCornerRadius, CrossColors.CLUE_BACKGROUND)
  self:insertAtCenter(buttonBackground)
  self.buttonBackground = buttonBackground

  local spriteHeight = self:getHeightPart(0.28)
  local lineHeight = self:getHeightPart(0.56)

  local line = FilledRectangle.new(1, lineHeight, CrossColors.CLUE_LINE)
  self:insertAtLeft(line)
  self.line = line

  local arrowSprite = Sprite.newWithName("arrow_button.png")
  arrowSprite:setEffectiveDims(nil, spriteHeight)
  self:insertAtCenter(arrowSprite)
  self.arrowSprite = arrowSprite
end

-- Constructor ------------------------

function CrossClueBarArrowButton.new(height, callback)
  local width = height * 0.889

  local self = Group.new(width, height)
  Mixin.mix(self, CrossClueBarArrowButton)

  self.callback = callback
  self.arrowActionInterrupt = ActionInterrupt.new()
  self.backgroundActionInterrupt = ActionInterrupt.new()

  createLayout(self)
  makeButton(self)
  
  return self
end
