--- CrossDesktopKeyboard
-- @author Created by Aaron Barrett on September 21, 2023.

class("CrossDesktopKeyboard")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

CrossDesktopKeyboard.LEFT_KEY = "LEFT"
CrossDesktopKeyboard.RIGHT_KEY = "RIGHT"
CrossDesktopKeyboard.DELETE_KEY = "DELETE"
CrossDesktopKeyboard.SLASH_KEY = "SLASH"

CrossDesktopKeyboard.ENTER = "ENTER"
CrossDesktopKeyboard.TAB = "TAB"
CrossDesktopKeyboard.SPACE = "SPACE"

CrossDesktopKeyboard.LEFT_ARROW = "LEFT_ARROW"
CrossDesktopKeyboard.RIGHT_ARROW = "RIGHT_ARROW"
CrossDesktopKeyboard.DOWN_ARROW = "DOWN_ARROW"
CrossDesktopKeyboard.UP_ARROW = "UP_ARROW"

local TEXT_BY_KEY_CODE = {
  [12] = "Q",
  [13] = "W",
  [14] = "E",
  [15] = "R",
  [17] = "T",
  [16] = "Y",
  [32] = "U",
  [34] = "I",
  [31] = "O",
  [35] = "P",

  [0] = "A",
  [1] = "S",
  [2] = "D",
  [3] = "F",
  [5] = "G",
  [4] = "H",
  [38] = "J",
  [40] = "K",
  [37] = "L",

  [6] = "Z",
  [7] = "X",
  [8] = "C",
  [9] = "V",
  [11] = "B",
  [45] = "N",
  [46] = "M",

  [33] = CrossDesktopKeyboard.LEFT_KEY, -- "'[' key triggers move to previous clue"
  [30] = CrossDesktopKeyboard.RIGHT_KEY, -- "']' key triggers move to next clue"
  [51] = CrossDesktopKeyboard.DELETE_KEY, -- "Delete/backspace key triggers clear content at cursor"
  [42] = CrossDesktopKeyboard.SLASH_KEY, -- "Backslash key triggers move to alternate clue"

  [36] = CrossDesktopKeyboard.ENTER, -- "Enter" moves to the next clue
  [48] = CrossDesktopKeyboard.TAB, -- "Tab" moves to the next clue
  [49] = CrossDesktopKeyboard.SPACE, -- "Space" clears any content and moves to the next empty space in the current word

  [123] = CrossDesktopKeyboard.LEFT_ARROW, -- "Left" moves one square left or switches to alternate clue if clue is vertical
  [124] = CrossDesktopKeyboard.RIGHT_ARROW, -- "Right" moves one square right or switches to alternate clue if clue is vertical
  [125] = CrossDesktopKeyboard.DOWN_ARROW, -- "Down" moves one square down or switches to alternate clue if clue is horizontal
  [126] = CrossDesktopKeyboard.UP_ARROW, -- "Up" moves one square up or switches to alternate clue if clue is horizontal
}

CrossDesktopKeyboard.pressDownCallbacksByKeyContent = {}
CrossDesktopKeyboard.pressUpCallbacksByKeyContent = {}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function onKeyPressed(keyCode, keyIsDown)
  local keyText = TEXT_BY_KEY_CODE[keyCode]
  if (keyText == nil) then
    return
  end

  if (keyIsDown) then
    local pressDownCallback = CrossDesktopKeyboard.pressDownCallbacksByKeyContent[keyText]
    if (pressDownCallback ~= nil) then
      runInCoroutine(pressDownCallback)
    end
  else
    local pressUpCallback = CrossDesktopKeyboard.pressUpCallbacksByKeyContent[keyText]
    if (pressUpCallback ~= nil) then
      runInCoroutine(pressUpCallback)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossDesktopKeyboard.setEnabled(enabled)
  local moaiKeyboardSensor = MOAIInputMgr.device.keyboard
  if (enabled) then
      moaiKeyboardSensor:setCallback(onKeyPressed)
  else
      moaiKeyboardSensor:setCallback(nil)
  end
end

function CrossDesktopKeyboard.setPressDownCallbackForKeyContent(pressDownCallback, keyContent)
  CrossDesktopKeyboard.pressDownCallbacksByKeyContent[keyContent] = pressDownCallback
end

function CrossDesktopKeyboard.setPressUpCallbackForKeyContent(pressUpCallback, keyContent)
  CrossDesktopKeyboard.pressUpCallbacksByKeyContent[keyContent] = pressUpCallback
end
