--- CrossDifficultyButton
-- @author Created by andrewcousins on November 27, 2023.

class("CrossDifficultyButton")

require("CrossUIButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_MARGIN_SIZE = 12

local DIFFICULTY_TEXT_KEYS = {
  [CrossRound.CROSSWORD_DIFFICULTY.EASY] = "EASY_CAPS",
  [CrossRound.CROSSWORD_DIFFICULTY.MEDIUM] = "MEDIUM_CAPS",
  [CrossRound.CROSSWORD_DIFFICULTY.HARD] = "HARD_CAPS"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossDifficultyButton:updateLabelForDifficulty(crosswordDifficulty)
  local initialRightEdge = self:getXRight()
  local difficultyText = Localizer.get(DIFFICULTY_TEXT_KEYS[crosswordDifficulty])
  self.buttonLabel:setText(difficultyText)

  local buttonWidth = self.buttonLabel:getWidth() + (BUTTON_MARGIN_SIZE * 2)
  local virtualButtonDirectionCapIsLeft = false
  self:setButtonWidth(buttonWidth, virtualButtonDirectionCapIsLeft, self.virtualButtonExtensionCap)
  self.buttonLabel:setX(buttonWidth / 2)

  self:offsetX(initialRightEdge - self:getXRight())
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, crosswordDifficulty)
  local buttonFont = Font.new(FontFaces.DIN_OT_MEDIUM, 12)
  local buttonLabel = Label.new("", buttonFont)
  buttonLabel:setColor(unpack(CrossColors.UI_BUTTON_TEXT))

  self:insertAtCenter(buttonLabel)

  self.buttonLabel = buttonLabel

  self:updateLabelForDifficulty(crosswordDifficulty)
end

-- Constructor ------------------------

function CrossDifficultyButton.new(crosswordDifficulty, buttonPressedCallback, virtualButtonExtensionCap)
  local self = CrossUIButton.new(buttonPressedCallback)
  Mixin.mix(self, CrossDifficultyButton)

  self.virtualButtonExtensionCap = virtualButtonExtensionCap

  createLayout(self, crosswordDifficulty)
  self:linkOpacityToChildren()

  return self
end
