--- CrossFeedbackScreenFeedbackButtons
-- @author Created by andrewcousins on June 14, 2023.

class("CrossFeedbackScreenFeedbackButtons")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_DATA = {
  {{text = "Too hard", id = "feedback_too_hard"}, {text = "Too easy", id = "feedback_too_easy"}},
  {{text = "Just right", id = "feedback_just_right"}}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function selectButton(self, selectedButton)
  if selectedButton.isSelected then
    selectedButton.setSelected(false)
  else
    for _, button in ipairs(self.buttons) do
      button.setSelected(false)
    end
    selectedButton.setSelected(true)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossFeedbackScreenFeedbackButtons:getFeedback()
  local feedbackMap = {}
  for _, button in ipairs(self.buttons) do
    feedbackMap[button.id] = button.isSelected
  end
  return feedbackMap
end

function CrossFeedbackScreenFeedbackButtons:setTouchEnabled(enabled)
  for _, button in ipairs(self.buttons) do
    button:setTouchEnabled(enabled)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createButton(self, buttonData)
  local text = buttonData.text
  local id = buttonData.id
  local spacing = FontScaling.getScaledSize(16)
  local label = Label.new(text, Font.new(FontFaces.DIN_OT_MEDIUM, 16))
  label:setColor(unpack(CrossColors.FEEDBACK_SCREEN_BUTTON_TEXT))
  local height = label:getHeight() + spacing
  local width = label:getWidth() + (spacing * 2)
  local cornerRadius = (height / 2) - 1
  local lineWidth = 1
  local button = HollowRoundedRectangle.new(width, height, cornerRadius, CrossColors.FEEDBACK_SCREEN_BUTTON_OUTLINE, lineWidth)

  local buttonFill = FilledRoundedRectangle.new(width, height, cornerRadius, CrossColors.FEEDBACK_SCREEN_BUTTON_OUTLINE)
  buttonFill:setOpacity(0)
  button:insertAtCenter(buttonFill)

  button:insertAtCenter(label)
  label:offsetY(1)

  button.isSelected = false
  button.id = id

  button.setSelected = function(selected)
    button.isSelected = selected
    if button.isSelected then
      buttonFill:setOpacity(1)
      label:setColor(unpack(CrossColors.FEEDBACK_SCREEN_BUTTON_TEXT_SELECTED))
    else
      buttonFill:setOpacity(0)
      label:setColor(unpack(CrossColors.FEEDBACK_SCREEN_BUTTON_TEXT))
    end
  end


  local onTouchUp = function()
    selectButton(self, button)
  end

  Button.makeButton(button, nil, onTouchUp)
  return button
end

local function createLayout(self)
  local spacing = FontScaling.getScaledSize(16)
  local buttonColumn = Column.new(spacing)
  for i=1, 2 do
    local buttonRow = Row.new(spacing)
    for _, buttonData in ipairs(BUTTON_DATA[i]) do
      local button = createButton(self, buttonData)
      buttonRow:insert(button)
      table.insert(self.buttons, button)
    end
    buttonRow:alignChildren()
    buttonColumn:insert(buttonRow)
  end
  buttonColumn:alignChildren()
  self:insertAtCenter(buttonColumn)
end

-- Constructor ------------------------

function CrossFeedbackScreenFeedbackButtons.new()
  local self = Group.new(Screen.width, Screen.getSafeHeightPart(0.2))
  Mixin.mix(self, CrossFeedbackScreenFeedbackButtons)

  self.buttons = {}

  createLayout(self)

  return self
end
