--- CrossFeedbackScreenStarRater
-- @author Created by andrewcousins on June 14, 2023.

class("CrossFeedbackScreenStarRater")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local EMPTY_STAR_SPRITE_NAME = "star_empty.png"
local FILLED_STAR_SPRITE_NAME = "star_filled.png"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getOnTouchUpCallback(self, starIndex)
  return function()
    self.starRating = starIndex
    for i, star in ipairs(self.stars) do
      local isFilled = (i <= self.starRating)
      star.setFill(isFilled)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossFeedbackScreenStarRater:getRating()
  return self.starRating
end

function CrossFeedbackScreenStarRater:setTouchEnabled(enabled)
  for _, star in ipairs(self.stars) do
    star:setTouchEnabled(enabled)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createStar(size, onTouchUp)
  local star = Group.new(size, size)
  local emptyStar = Sprite.newWithName(EMPTY_STAR_SPRITE_NAME)
  emptyStar:setEffectiveDims(size)
  star:insertAtCenter(emptyStar)

  local filledStar = Sprite.newWithName(FILLED_STAR_SPRITE_NAME)
  filledStar:setEffectiveDims(size)
  star:insertAtCenter(filledStar)
  filledStar:setOpacity(0)

  star.setFill = function(isFilled)
    if isFilled then
      filledStar:setOpacity(1)
      emptyStar:setOpacity(0)
    else
      filledStar:setOpacity(0)
      emptyStar:setOpacity(1)
    end
  end

  Button.makeButton(star, nil, onTouchUp)

  return star
end

local function createLayout(self)
  local starSize = math.min(self:getHeightPart(0.4), Screen.getSafeWidthPart(1/6))
  local stars = {}
  local starRow = Row.new()
  for i=1, 5 do
    local onTouchUp = getOnTouchUpCallback(self, i)
    local star = createStar(starSize, onTouchUp)
    table.insert(stars, star)
    starRow:insert(star)
  end

  starRow:alignChildren()
  self:insertAtCenter(starRow)
  self.stars = stars
end

-- Constructor ------------------------

function CrossFeedbackScreenStarRater.new()
  local self = Group.new(Screen.width, Screen.getSafeHeightPart(0.15))
  Mixin.mix(self, CrossFeedbackScreenStarRater)

  self.starRating = 0

  createLayout(self)

  return self
end
