--- CrossPauseButton
-- @author Created by andrewcousins on May 03, 2023.

class("CrossPauseButton")

require("CrossUIButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local VERTICAL_BUFFER_PERCENT = (23 / 40)
local HORIZONTAL_BUFFER_PERCENT = (20 / 34)

local PAUSE_BAR_PERCENT = (1 / 3)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWidth(self, timer)
  return (self:getHeightPart(1 + (HORIZONTAL_BUFFER_PERCENT / 2)) + timer:getWidth())
end

local function positionTimer(self, timer)
  local horizontalBuffer = self:getHeightPart(HORIZONTAL_BUFFER_PERCENT)
  timer:setX(self:getWidth() - (timer:getWidth() + horizontalBuffer) * 0.5)
end

local function addTimerLabelChangedListener(self, timer)
  timer:addLabelAddedCallback(function()
    runInCoroutine(function()
      coroutine.yield(2) -- Wait for the timer to actually update before adjusting size
      local initialLeftEdge = self:getXLeft()
      local width = getWidth(self, timer)

      self:setButtonWidth(width)
      self:offsetX(initialLeftEdge - self:getXLeft())

      positionTimer(self, timer)

      self:linkOpacityToChildren()
    end)
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createPauseLabel(size)
  local pauseIconHeight = size * (1 - VERTICAL_BUFFER_PERCENT)
  local pauseGroupWidth = size * (1 - HORIZONTAL_BUFFER_PERCENT)
  local pauseIconWidth = pauseGroupWidth * PAUSE_BAR_PERCENT

  local pauseGroup = Group.new(pauseIconHeight, pauseGroupWidth)
  for i = 1, 2 do
    local pauseBar = FilledRectangle.new(pauseIconWidth, pauseIconHeight, CrossColors.UI_BUTTON_TEXT)
    pauseGroup:insertAtCenter(pauseBar)
    local offset = pauseIconWidth
    if (i == 2) then
      offset = -offset
    end
    pauseBar:offsetX(offset)
  end
  return pauseGroup
end

local function createLayout(self, timer)
  local width, height = self:getEffectiveDims()

  local pauseIcon = createPauseLabel(height)

  self:insertAtLeft(pauseIcon)
  pauseIcon:offsetX(height / 2)

  self:insertAtCenter(timer)
  positionTimer(self, timer)
end

-- Constructor ------------------------

function CrossPauseButton.new(timer)
  local buttonPressedCallback = function()
    GameSession.notifyGamePaused()
  end
  local self = CrossUIButton.new(buttonPressedCallback)
  Mixin.mix(self, CrossPauseButton)

  local width = getWidth(self, timer)
  self:setButtonWidth(width)

  createLayout(self, timer)
  self:linkOpacityToChildren()

  addTimerLabelChangedListener(self, timer)

  return self
end
