--- CrossPostGameButtonGroup
-- @author Created by andrewcousins on October 20, 2023.

class("CrossPostGameButtonGroup")

require("CrossPostGameButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CONTINUE_TEXT = Localizer.get("CONTINUE")
local RESET_TEXT = Localizer.get("RESET")
local SHARE_TEXT = Localizer.get("SHARE")

local RESET_ICON = "icon_reset.png"
local SHARE_ICON = "icon_share.png"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossPostGameButtonGroup:setOpacity(opacity)
  for _, button in ipairs(self.buttons) do
    button:setOpacity(opacity)
  end
end

function CrossPostGameButtonGroup:getFadeAnimation(duration, ease, opacity)
  local animation = AHSpawn.new()
  for _, button in ipairs(self.buttons) do
    animation:addAction(AHOpacityTo.new(button, duration, ease, opacity))
  end
  return animation
end

function CrossPostGameButtonGroup:setButtonGroupYPosition()
  self:setY(Screen.safeBottom + self:getHeightPart(0.5) + (CrossPostGameButton.getButtonHeight() * 0.25))
end

function CrossPostGameButtonGroup:setTouchEnabled(enabled)
  for _, button in ipairs(self.buttons) do
    button:setTouchEnabled(enabled)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, shareCallback, resetCallback, withContinueButton, continueCallback)
  if withContinueButton then
    local continueButton = CrossPostGameButton.new(CONTINUE_TEXT, continueCallback)
    self:insert(continueButton)
    table.insert(self.buttons, continueButton)
  end

  local buttonRow = Row.new(CrossPostGameButton.getHalfButtonSpacing())

  local shareButton = CrossPostGameButton.newInverted(SHARE_TEXT, shareCallback, SHARE_ICON)
  table.insert(self.buttons, shareButton)
  buttonRow:insert(shareButton)

  local resetButton = CrossPostGameButton.newInverted(RESET_TEXT, resetCallback, RESET_ICON)
  table.insert(self.buttons, resetButton)
  buttonRow:insert(resetButton)

  buttonRow:alignChildren()
  self:insert(buttonRow)
  self:alignChildren()
end

-- Constructor ------------------------

function CrossPostGameButtonGroup.new(shareCallback, resetCallback, withContinueButton, continueCallback)
  local buttonColumnSpacing = 9
  local self = Column.new(buttonColumnSpacing)
  Mixin.mix(self, CrossPostGameButtonGroup)

  self.buttons = {}

  createLayout(self, shareCallback, resetCallback, withContinueButton, continueCallback)

  return self
end
