--- CrossTimer
-- @author Created by andrewcousins on April 05, 2022.

class("CrossTimer")

require("CrossTimeDisplay")
require("Timer")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FONT_SIZE = 18
local FONT = FontFaces.DIN_OT_MEDIUM
local TIME_CHARACTERS = "0123456789:"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Only auto save once every second
local function saveStateIfNeeded(self)
  if (self.saveManager ~= nil) then
    local currentTimeInSeconds = math.floor(self:getTime())
    if (currentTimeInSeconds > self.lastAutoSaveTime) then
      self.saveManager:save()
      self.lastAutoSaveTime = currentTimeInSeconds
    end
  else
    Log.warn("No save manager has been set - not saving progress.")
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossTimer:start()
  self.timeTracker:start()
end

function CrossTimer:pause()
  self.timeTracker:pause()
end

function CrossTimer:getTime()
  return self.timeTracker:getTime()
end

function CrossTimer:setInitialTime(time)
  -- Update the last auto save time so the timer will not
  -- trigger a save when setTime() is called
  self.lastAutoSaveTime = time
  self.timeTracker:setTime(time)
end

function CrossTimer:reset()
  self.timeTracker:setTime(0)
end

function CrossTimer:setSaveManager(saveManager)
  self.saveManager = saveManager
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function CrossTimer.new()
  local prerenderedFont = PrerenderedFont.newWithActualSize(FONT, FONT_SIZE, TIME_CHARACTERS)
  local self = CrossTimeDisplay.new(prerenderedFont)
  Mixin.mix(self, CrossTimer)

  self.timeTracker = {}
  Timer.makeTimer(self.timeTracker, math.huge)
  self.timeTracker:initialize()

  self.lastAutoSaveTime = 0
  self.timeTracker:setUpdateHandler(function()
    saveStateIfNeeded(self)
    self:setDisplayedTime(self:getTime())
  end)

  return self
end
