--- CrossTrayToggleButton
-- @author Created by andrewcousins on July 31, 2023.

class("CrossTrayToggleButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local AUTOCHECK_TEXT = Localizer.get("AUTOCHECK")

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playToggleAnimation(self)
  local backingColor = CrossColors.TRAY_BUTTON_TOGGLE_BACKING
  local switchX = self.switch.disabledLocationX
  if self.isEnabled then
    backingColor = CrossColors.TRAY_BUTTON_TOGGLE_BACKING_ACTIVE
    switchX = self.switch.enabledLocationX
  end
  local duration = 0.167
  local ease = AHEaseLinear.new()
  local animation = AHSpawn.new(
    AHColorTo.new(self.backing, duration, ease, backingColor),
    AHLocationXTo.new(self.switch, duration, ease, switchX),
    AHLocationXTo.new(self.dropShadow, duration, ease, switchX)
  )
  self.actionInterrupt:insert(animation)
end

local function createToggle(self)
  local height = self:getHeightPart(0.5)
  local width = height * 1.66
  local cornerRadius = math.floor(height / 2)
  local backing = FilledRoundedRectangle.new(width, height, cornerRadius, CrossColors.TRAY_BUTTON_TOGGLE_BACKING)

  local spacing = height * 0.04
  local radius = (height / 2) - spacing

  local dropShadow = FilledCircle.new(radius, CrossColors.BLACK)
  dropShadow:setOpacity(0.05)
  backing:insertAtLeft(dropShadow)
  dropShadow:offsetX(radius + spacing)
  dropShadow:offsetY(-spacing * 2)

  local switch = FilledCircle.new(radius, CrossColors.TRAY_BUTTON_TOGGLE_SWITCH)
  backing:insertAtCenter(switch)

  switch.disabledLocationX = radius + spacing
  switch.enabledLocationX = width - radius - spacing

  switch:setX(switch.disabledLocationX)

  self.backing = backing
  self.switch = switch
  self.dropShadow = dropShadow

  return backing
end

local function getOnToggle(self, toggle)
  return function()
    self.isEnabled = not self.isEnabled
    playToggleAnimation(self)
    self.callback(self.isEnabled)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CrossTrayToggleButton:setState(state)
  self.isEnabled = state
  playToggleAnimation(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, text)
  local width, height = self:getDims()
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 16)
  local label = Label.new(text, font)
  label:setColor(unpack(CrossColors.TRAY_BUTTON_TEXT))

  local horizontalPadding = width * 0.05
  self:insertAtLeft(label)
  label:offsetX(label:getWidthPart(0.5) + horizontalPadding)

  local toggleLocationFractionalX = 0.85
  local toggle = createToggle(self)
  self:insertAtLocationFractional(toggle, toggleLocationFractionalX, 0.5)

  Button.makeButton(self, nil, getOnToggle(self, toggle))

  self.toggle = toggle
end

-- Constructor ------------------------

function CrossTrayToggleButton.newAutocheck(width, height, callback)
  self = Group.new(width, height)
  Mixin.mix(self, CrossTrayToggleButton)

  self.isEnabled = false
  self.callback = callback
  self.actionInterrupt = ActionInterrupt.new()

  local text = AUTOCHECK_TEXT

  createLayout(self, text)

  return self
end
