--- DepthBubbleInsidesView

class("DepthBubbleInsidesView")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createMoneyLabel(amount, font)
  local labelText = string.format("%.02f", amount)
  local label = Label.new(labelText, Font.new(font, 24))

  return label
end

local function createFractionView(self, fraction, total, bubbleRadius)
  local result = Group.newDeprecated()

  local fractionView = createMoneyLabel(fraction, FontFaces.DIN_OT_MEDIUM)
  local totalView = createMoneyLabel(total, FontFaces.DIN_OT_LIGHT)

  local division = Sprite.newWithName("vertical_separator.png")
  division:setColor(1, 1, 1, 0.5)
  division:setRot(90)
  division:setEffectiveDims(division:getWidth(), 1.4 * totalView:getWidth())

  local yOffset = 0.3 * bubbleRadius
  fractionView:setLoc(0, yOffset)
  totalView:setLoc(0, -yOffset)

  result:insert(fractionView)
  result:insert(totalView)
  result:insert(division)

  result:linkOpacityAttrToChildren(true)

  self.numerator = fraction
  self.denominator = total

  return result
end

local function createLabelView(label, bubbleRadius)
  local result = Label.new(label, Font.new(FontFaces.DIN_OT_MEDIUM, 24))
  return result
end

local function createLayout(self, concept, bubbleRadius)
  local view = nil
  if(concept.fraction and concept.total) then
    view = createFractionView(self, concept.fraction, concept.total, bubbleRadius)
  else
    view = createLabelView(concept.problem, bubbleRadius)
  end
  view:setAttrLink(MOAIColor.ATTR_OPACITY, self)
  self:insert(view)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DepthBubbleInsidesView:getNumerator()
  return self.numerator
end

function DepthBubbleInsidesView:getDenominator()
  return self.denominator
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function DepthBubbleInsidesView.new(concept, bubbleRadius)
  local self = Group.newDeprecated()
  Mixin.mix(self, DepthBubbleInsidesView)
  createLayout(self, concept, bubbleRadius)
  return self
end
