--- DepthCircleBurst
-- @author Created by Aaron Barrett on January 30, 2014.

class("DepthCircleBurst")
-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local Z_CIRCLE = 10

local DEFAULT_CIRCLE_COUNT = 8

local DEFAULT_MIN_DESTINATION_RADIUS = 90.0
local DEFAULT_MAX_DESTINATION_RADIUS = 160.0

local DIAMETER = DEFAULT_MAX_DESTINATION_RADIUS * 2.0
local WIDTH = DIAMETER
local HEIGHT = DIAMETER

local DEFAULT_MIN_CIRCLE_RADIUS = 3.0
local DEFAULT_MAX_CIRCLE_RADIUS = 8.0

local CIRCLE_COLOR = {0.56, 0.94, 0.90}

local CIRCLE_STEPS = 16.0

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function DepthCircleBurst:createBurstAction()
  local seconds = 0.6
  local circles = self.circles

  local burstEaseIn = AHEaseSimpleIn.new(2.0)
  local burstEaseOut = AHEaseSimpleOut.new(2)
  local fadeEase = AHEaseSimpleOut.new(0.5)

  local circleActions = {}
  for i, circle in ipairs(circles) do
    local circleDestinationX = circle.destinationX
    local circleDestinationY = circle.destinationY

    local circleLocationXAction = AHLocationXTo.new(circle, seconds, burstEaseOut, circleDestinationX)
    table.insert(circleActions, circleLocationXAction)
    local circleLocationYAction = AHLocationYTo.new(circle, seconds, burstEaseOut, circleDestinationY)
    table.insert(circleActions, circleLocationYAction)
    local circleScaleAction = AHScaleTo.new(circle, seconds, burstEaseIn, 0.0)
    table.insert(circleActions, circleScaleAction)
    local circleFadeAction = AHOpacityTo.new(circle, seconds, fadeEase, 0.0)
    table.insert(circleActions, circleFadeAction)
  end

  local burstAction = AHSpawn.new(unpack(circleActions))

  return burstAction
end

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function DepthCircleBurst.new(circleCount, minRingRadius, maxRingRadius, minBubbleRadius, maxBubbleRadius)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, DepthCircleBurst)

  local circles = {}

  circleCount = circleCount or DEFAULT_CIRCLE_COUNT
  minRingRadius = minRingRadius or DEFAULT_MIN_DESTINATION_RADIUS
  maxRingRadius = maxRingRadius or DEFAULT_MAX_DESTINATION_RADIUS
  minBubbleRadius = minBubbleRadius or DEFAULT_MIN_CIRCLE_RADIUS
  maxBubbleRadius = maxBubbleRadius or DEFAULT_MAX_CIRCLE_RADIUS

  for i = 1, circleCount do
    local circleRadius = minBubbleRadius + math.random() * (maxBubbleRadius - minBubbleRadius)

    local circle = FilledCircle.new(circleRadius, CIRCLE_COLOR, CIRCLE_STEPS)

    local destinationRadius = minRingRadius + math.random() * (maxRingRadius - minRingRadius)
    local angle = math.random() * math.pi * 2.0

    local destinationX = maxRingRadius + math.cos(angle) * destinationRadius
    local destinationY = maxRingRadius + math.sin(angle) * destinationRadius

    circle.destinationX = destinationX
    circle.destinationY = destinationY

    circle:setLoc(maxRingRadius, maxRingRadius)
    circle:setScl(1)

    self:insert(circle, Z_CIRCLE)
    table.insert(circles, circle)
  end

  self.circles = circles

  return self
end
