--- DepthConfiguration
-- @author Created by Jay Austria on Augtober 15, 2014.

class("DepthConfiguration")

require("DepthConstants")
require("data.keyframes.KeyframeSet")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

DepthConfiguration.POINTS_PER_PASSING_SECOND = 100
DepthConfiguration.SPEED_BONUS_SCREEN_PROPORTION_THRESHOLD = 0.25
DepthConfiguration.POINTS_PER_FULL_SPEED_BONUS = 100

local NUMBER_OF_LIVES_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {lives = 5}),
  Keyframe.new(0.2, {lives = 4})
)

local MAXIMUM_EXTRA_LIVES_BONUS = 2100
local MISTAKES_ALLOWED_RANK_TWO = 1
local MISTAKES_ALLOWED_RANK_THREE = 0

local INITIAL_BUBBLE_SPEED_MODIFIER_KEYFRAMES = KeyframeSet.newInterpolated(
  Keyframe.new(0.0, {bubbleSpeedModifier = 0.75}),
  Keyframe.new(0.5, {bubbleSpeedModifier = 1.0}),
  Keyframe.new(1.0, {bubbleSpeedModifier = 1.0})
)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getMaximumBonusLives()
  return DepthConfiguration.getNumberOfLives() - 1
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DepthConfiguration.getInitialBubbleSpeedModifier()
  local difficulty = GameSession.getDifficulty()
  local frameForDifficulty = INITIAL_BUBBLE_SPEED_MODIFIER_KEYFRAMES:getFrameAt(difficulty)
  return frameForDifficulty.bubbleSpeedModifier
end

function DepthConfiguration.getNumberOfLives()
  local difficulty = GameSession.getDifficulty()
  local frameForDifficulty = NUMBER_OF_LIVES_THRESHOLDS:getFrameAt(difficulty)
  return frameForDifficulty.lives
end

function DepthConfiguration.getExtraLivesBonus()
  local maximumBonusLives = getMaximumBonusLives()
  return MAXIMUM_EXTRA_LIVES_BONUS / maximumBonusLives
end

function DepthConfiguration.getPassScore()
  local scoredSeconds = (DepthConstants.GAME_DURATION - 1)
  return DepthConfiguration.POINTS_PER_PASSING_SECOND * scoredSeconds
end

function DepthConfiguration.getRankTwoScore()
  local passScore = DepthConfiguration.getPassScore()
  local extraLivesRequired = getMaximumBonusLives() - MISTAKES_ALLOWED_RANK_TWO
  local extraLivesBonus = extraLivesRequired * DepthConfiguration.getExtraLivesBonus()
  local rankTwoScore = passScore + extraLivesBonus
  return rankTwoScore
end

function DepthConfiguration.getRankThreeScore()
  local passScore = DepthConfiguration.getPassScore()
  local extraLivesRequired = getMaximumBonusLives() - MISTAKES_ALLOWED_RANK_THREE
  local extraLivesBonus = extraLivesRequired * DepthConfiguration.getExtraLivesBonus()
  local rankThreeScore = passScore + extraLivesBonus
  return rankThreeScore
end


function DepthConfiguration.getDifficultyKeyFrames()
  ExperimentsConfiguration.DIFFICULTY_KEYFRAMES_EXPERIMENT:reportExperimentExposed()
  if ExperimentsConfiguration.DIFFICULTY_KEYFRAMES_EXPERIMENT:isActive() then
    return {
      {key = 0.0, valueProbabilities = {Basic = 1.0}},

      {key = 0.25, valueProbabilities = {Basic = 0.25,
                                         Easy = 0.5,
                                         Medium = 0.25}},

      {key = 0.5, valueProbabilities = {Easy = 0.25,
                                        Medium = 0.5,
                                        Hard = 0.25}},

      {key = 0.75, valueProbabilities = {Medium = 0.25,
                                         Hard = 0.5,
                                         Advanced = 0.25}},

      {key = 1.0, valueProbabilities = {Hard = 0.5,
                                        Advanced = 0.5}}
    }
  end
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Basic = 1.0
    }},
    {
      key = 0.1,
      valueProbabilities = {
        Basic = 0.5,
        Easy = 0.5
    }},
    {
      key = 0.2,
      valueProbabilities = {
        Easy = 1.0
    }},
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.3,
        Medium = 0.7
    }},
    {
      key = 0.4,
      valueProbabilities = {
        Medium = 1.0
    }},
    {
      key = 0.45,
      valueProbabilities = {
        Medium = 0.6,
        Hard = 0.4
    }},
    {
      key = 0.5,
      valueProbabilities = {
        Medium = 0.4,
        Hard = 0.6
    }},
    {
      key = 0.6,
      valueProbabilities = {
        Medium = 0.2,
        Hard = 0.8
    }},
    {
      key = 0.7,
      valueProbabilities = {
        Medium = 0.1,
        Hard = 0.80,
        Advanced = 0.1
    }},
    {
      key = 0.75,
      valueProbabilities = {
        Hard = 0.60,
        Advanced = 0.40
    }},
    {
      key = 0.8,
      valueProbabilities = {
        Hard = 0.45,
        Advanced = 0.55
    }},
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 0.2,
        Advanced = 0.8
    }}
  }
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------
