--- DepthConstants
-- @author Created by Aaron Barrett on January 28, 2014.

class("DepthConstants")
-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------
local HEIGHT = Screen.height

DepthConstants.GAME_DURATION = 45.0

DepthConstants.BUBBLE_BASE_RADIUS = 64.0
DepthConstants.BUBBLE_MAX_RANDOM_RADIUS = 5.0

DepthConstants.MAX_CORRECT_BUBBLES_IN_A_ROW = 2
DepthConstants.MAX_WRONG_BUBBLES_IN_A_ROW = 2


local SPAWN_PARAMETER_SETS =
{
  slow =
  {
    INITIAL_BUBBLE_INTERVAL = 3.75,
    BUBBLE_INTERVAL_DECREMENT = 0.25,
    MIN_BUBBLE_INTERVAL = 2.5,

    INITIAL_GAME_SPEED = HEIGHT / 6.8,
    BUBBLE_SPEED_INCREMENT = HEIGHT / 250.0,
    MAX_BUBBLE_SPEED = HEIGHT / 3.5,
  },
  quick =
  {
    INITIAL_BUBBLE_INTERVAL = 3.5,
    BUBBLE_INTERVAL_DECREMENT = 0.25,
    MIN_BUBBLE_INTERVAL = 2.5,

    INITIAL_GAME_SPEED =     HEIGHT / 5.25,
    BUBBLE_SPEED_INCREMENT = HEIGHT / 250.0,
    MAX_BUBBLE_SPEED =       HEIGHT / 3.5
  }
}

local function setSpawnType(type)
  local parameters = SPAWN_PARAMETER_SETS[type]
  for key, value in pairs(parameters) do
    DepthConstants[key] = value
  end
end


local function chooseRoundPercentage()
  local typeRoll = math.random()
  local roll20 = 0.33
  local roll25 = 0.66
  if(GameSession.getDifficulty() < 0.3) then
    -- Exclude 15% games when the game difficulty is below a threshold
    roll20 = 0.5
    roll25 = math.huge
  end
  if(typeRoll < roll20) then
    return "20"
  elseif(typeRoll < roll25) then
    return "25"
  else
    return "15"
  end
end
function DepthConstants.init()
  local percentage = chooseRoundPercentage()
  DepthConstants.CORRECT_ANSWER_PERCENTAGE = percentage

  if(percentage == "15") then
    setSpawnType("slow")
  else
    setSpawnType('quick')
  end
end
