require("DepthPrize")
require("DepthSquiggleBurst")

class("DepthFishOutroDance")
------------------------------------------------------
------------------------- UTILITY --------------------
------------------------------------------------------

local function selectLineAndIndexOnLine(pointIndex, takenLineIndices)
  -- Always put the very first first on the middle line
  local lineIndex = 2
  if(pointIndex == 1) then
    -- Make sure there's only one fish at the front of the swarm
    if(takenLineIndices[lineIndex][pointIndex]) then
      pointIndex = pointIndex + 1
    end
  else
    -- After the first fish, choose the line randomly
    lineIndex = math.random(3)
  end
  -- Maybe the line is already taken at the current vertical slice?
  while(takenLineIndices[lineIndex][pointIndex]) do
    local nextPointRoll = math.random()
    if(nextPointRoll < 0.2) then
      -- Don't have to fill the whole vertical slice.
      -- Also if it's full, move on to the next one (eventually)
      -- Automatically that means that it's free, because that vertical slice hasn't been used yet
      pointIndex = pointIndex + 1
      break
    end
    lineIndex = math.random(3)
  end
  takenLineIndices[lineIndex][pointIndex] = true
  return lineIndex, pointIndex
end

local function createLayout(self)
  local prizesEarned = DepthPrize.getSpawnedFishSequence()
  local fishies = {}

  --  +--------+     Start offscreen, forming a school that swims diagonally
  --  |        |                   tr   (l1)
  --  |        |(l1)   tl
  --  | Screen |                   mr   (l2)
  --  |        |(l2)   ml
  --  |        |                   br   (l3)
  --  |        |(l3)   bl
  --  |        |
  --  +--------+

  local flockWidth = 1500
  local flockHeight = 160
  local flockDiagOffset = 200
  local flockVOffset = 0
  local widthRandomization = 1000
  local tl = {Screen.width, Screen.height/2 + flockHeight/2 + flockVOffset}
  local ml = {Screen.width, Screen.height/2 + flockVOffset}
  local bl = {Screen.width, Screen.height/2 - flockHeight/2 + flockVOffset}
  local tr = {tl[1] + flockWidth + math.random() * widthRandomization, tl[2] + flockDiagOffset}
  local mr = {ml[1] + flockWidth + math.random() * widthRandomization, ml[2] + flockDiagOffset}
  local br = {bl[1] + flockWidth + math.random() * widthRandomization, bl[2] + flockDiagOffset}

  local numberOfInbetweenPoints = 8
  local l1 = MathUtils.getLinearPointsOnSegment(tl, tr, numberOfInbetweenPoints)
  local l2 = MathUtils.getLinearPointsOnSegment(ml, mr, numberOfInbetweenPoints)
  local l3 = MathUtils.getLinearPointsOnSegment(bl, br, numberOfInbetweenPoints)
  local takenLineIndices = {{}, {}, {}}
  local lines = {l1, l2, l3}
  local lineIndex, pointIndex = 1, 1

  local dx = {tl[1] - tr[1], ml[1] - mr[1], bl[1] - br[1]}
  local dy = {tl[2] - tr[2], ml[2] - mr[2], bl[2] - br[2]}

  local maxFishes = 13
  local spawnOrder = 1
  for i = #prizesEarned, 1, -1 do
    local prizeIndex = prizesEarned[i]
    if(spawnOrder > maxFishes) then
      break
    end
    spawnOrder = spawnOrder + 1

    local fish = DepthPrize.createFish(prizeIndex)
    fish:startFinMotion()
    self:insert(fish)
    fishies[#fishies + 1] = fish
    lineIndex, pointIndex = selectLineAndIndexOnLine(pointIndex, takenLineIndices)
    local point = lines[lineIndex][pointIndex]
    fish:setLoc(point[1] + 50, point[2])
    fish:setAnchorPointLeft()

    local scale = 0.8
    fish:setScl(scale)

    fish.dx, fish.dy = dx[lineIndex], dy[lineIndex]
  end

  self.fishies = fishies
end

local function createParticleEffect(self)
  local particles1 = DepthSquiggleBurst.new(nil, 10, Screen.width, Screen.height/2)
  local particles2 = DepthSquiggleBurst.new(nil, 10, Screen.width, Screen.height/2)
  particles1:setLoc(Screen.width/2, -100)
  particles2:setLoc(Screen.width/2, -100)
  self:insert(particles1, -100)
  self:insert(particles2, -100)
  local particleTime = 1.8
  -- These get created as the very last thing in the game, so there's no need to clean them up,
  -- The main app will destroy the whole game context.
  local particleEffect = particles1:createBurstAction(particleTime, 6 * Screen.height/4, 4)
  particleEffect:start()
  local particleEffect = particles2:createBurstAction(particleTime, 5 * Screen.height/4, 4)
  particleEffect:start()
end

local function moveFish(self, finishHandler)
  local totalAnimationTime = 2.5

  for _, fish in pairs(self.fishies) do
    local dx, dy = fish.dx, fish.dy
    local l = MathUtils.vectorLength(dx, dy)
    local cosA, sinA = dx/l, dy/l
    local tanA = dy/dx
    local x0, y0 = fish:getLoc()
    local offscreenDistane = math.abs(x0)
    local travelDx = offscreenDistane + Screen.width + 1000
    local travelDy = tanA * travelDx
    local time = totalAnimationTime
    AHLocationBy.new(fish, time, nil, -travelDx, -travelDy):start()
  end

  AHSequence.new(AHDelay.new(1.4), finishHandler):start()
end

------------------------------------------------------
----------------------------- API --------------------
------------------------------------------------------

function DepthFishOutroDance:start(finishHandler)
  createParticleEffect(self)
  AHSequence.new(AHDelay.new(0.2), function()
    Sound.playSoundEffect("depth_goalreach")
    moveFish(self, finishHandler)
  end):start()
end

------------------------------------------------------
--------------------- CONSTRUCTOR --------------------
------------------------------------------------------

function DepthFishOutroDance.new()
  -- Full screen effect
  local self = Group.new(Screen.width/2, Screen.height/2)
  self:setAnchorPointBottomLeft()
  Mixin.mix(self, DepthFishOutroDance)
  createLayout(self)
  return self
end
