--- DepthLevelJellyfish
-- @author Created by Aaron Barrett on January 28, 2014.

class("DepthLevelJellyfish")
-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local Z_HOLDER = 10
local Z_JELLYFISH = 20

local JELLYFISH_ANGLE = 34.0

local JELLYFISH_OFFSETS = {
                          {x = -20.0, y = -39.0},
                          {x = 29.0, y = 49.0},
                          {x = -39.0, y = 34.0},
                          {x = 34.0, y = -32.0}
                          }

local HOLDER_WIDTH = 100.0
local HOLDER_HEIGHT = 100.0

local MARGIN_X = 120.0
local MARGIN_Y = 220.0

local HOVER_DURATION = 0.9
local HOVER_AMOUNT = 8.8

local MIN_DISTANCE_FACTOR = 0.5

local JELLYFISH_KIND_COUNT = 3

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function DepthLevelJellyfish.new(opacity)
  local distanceFactor = math.random() * (1.0 - MIN_DISTANCE_FACTOR) + MIN_DISTANCE_FACTOR

  local jellyfishOpacity = opacity * distanceFactor
  local jellyfishHolder = Group.new(HOLDER_WIDTH, HOLDER_HEIGHT)

  local self = Group.new(DepthLevel.WIDTH, DepthLevel.HEIGHT)
  Mixin.mix(self, DepthLevelJellyfish)

  local jellyfishRotation = JELLYFISH_ANGLE
  local isFacingRight = MathUtils.getRandomBoolean()
  if (isFacingRight) then
    jellyfishRotation = -jellyfishRotation
  end

  local hoverEase = AHEaseSimpleInOut.new(2.0)

  local jellyfishKind = math.random(JELLYFISH_KIND_COUNT)
  local jellyfishFilename = "jellyfish_" .. jellyfishKind .. ".png"

  for i, offset in ipairs(JELLYFISH_OFFSETS) do
    local jellyfish = Sprite.newWithName(jellyfishFilename)
    jellyfish:setOpacity(jellyfishOpacity)

    local x = nil
    if (isFacingRight) then
      x = offset.x
    else
      x = -offset.x
    end
    local y = offset.y
    jellyfish:setLoc(x, y)

    jellyfish:setRot(jellyfishRotation)

    jellyfishHolder:insert(jellyfish, Z_JELLYFISH)

    local hoverAction = AHSequence.new(
                                       AHDelay.new(math.random() * 0.5),
                                       AHLoop.new(
                                                  AHSequence.new(
                                                                 AHLocationYBy.new(jellyfish, HOVER_DURATION, hoverEase, -HOVER_AMOUNT),
                                                                 AHLocationYBy.new(jellyfish, HOVER_DURATION, hoverEase, HOVER_AMOUNT)
                                                                 ),
                                                  10
                                                  )
                                       )
    hoverAction:start()
  end

  local holderX = math.random() * (DepthLevel.WIDTH - MARGIN_X) + MARGIN_X * 0.5
  local holderY = math.random() * (DepthLevel.HEIGHT - MARGIN_Y) + MARGIN_Y * 0.5
  jellyfishHolder:setLoc(holderX, holderY)
  jellyfishHolder:setScl(distanceFactor)

  self:insert(jellyfishHolder, Z_HOLDER)

  return self
end
