--- DepthMeter
-- @author Created by Aaron Barrett on January 27, 2014.

class("DepthMeter")require("actions.advanced.AHBlink")
-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local Z_BAR = 10

local WIDTH = 24.0
local HEIGHT = Screen.safeHeight * 0.86

local CENTER_X = WIDTH * 0.5
local CENTER_Y = HEIGHT * 0.5

local MARGIN = WIDTH * 0.2
local CORNER_RADIUS = 4.0

local BACKGROUND_OPACITY = 0.1
local FOREGROUND_OPACITY = 0.2

local BACKGROUND_COLOR = {1.0, 1.0, 1.0, BACKGROUND_OPACITY}
local FOREGROUND_COLOR = {1.0, 1.0, 1.0, FOREGROUND_OPACITY}
local CLOSE_TO_FINISH_HAPPY_COLOR = {0.5, 0.9, 0.45, FOREGROUND_OPACITY}

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function DepthMeter:getProgressProportion()
  local bar = self.bar
  local _, barScaleY = bar:getScl()
  local progressProportion = barScaleY
  return progressProportion
end

function DepthMeter:pause(isPaused)
  local progressAction = self.progressAction
  progressAction:pause(isPaused)
end

function DepthMeter:start()
  local bar = self.bar
  local duration = self.duration
  local onFinishCallback = self.onFinishCallback

  local secondsSpentOnBlinking = math.min(6, 0.5 * duration)
  local secondsBeforeBlink = duration - secondsSpentOnBlinking
  local completionBeforeBlink = secondsBeforeBlink/duration
  local blinkAfterEndGameSeconds = 0.5
  local neutralTimeSeconds = 0.1

  local blinksPerSecond = 3.5
  local blinkEase = AHEaseSimpleIn.new(2.0)
  local blinkBaseColor = FOREGROUND_COLOR
  local blinkSecondColor = CLOSE_TO_FINISH_HAPPY_COLOR

  local scaleUpToBlink = AHScaleYTo.new(bar, secondsBeforeBlink, nil, completionBeforeBlink)

  local blink = AHBlink.new(bar, secondsSpentOnBlinking + blinkAfterEndGameSeconds, blinkEase, blinkBaseColor, blinkSecondColor, blinksPerSecond)
  local scaleFinish = AHScaleYTo.new(bar, secondsSpentOnBlinking, nil, 1)
  local finish = AHSequence.new(AHDelay.new(secondsSpentOnBlinking + neutralTimeSeconds), onFinishCallback)
  local endSequence = AHSpawn.new(blink, scaleFinish, finish)

  local progressAction = AHSequence.new(scaleUpToBlink, endSequence)

  self.progressAction = progressAction
  self.duration = duration
  self.progressAction:start()
end

-- UIOverlay Widget -------------------

function DepthMeter:prepareForGameIntro()
  self:setOpacity(0.0)
  self.bar:setOpacity(0.0)
end

function DepthMeter:createFadeInAction(duration)
  local fadeInAction = AHSpawn.new(
                                   AHOpacityTo.new(self, duration, nil, BACKGROUND_OPACITY),
                                   AHOpacityTo.new(self.bar, duration, nil, FOREGROUND_OPACITY)
                                   )
  return fadeInAction
end

function DepthMeter:createFadeOutAction(duration)
  local fadeOutAction = AHSpawn.new(
                                    AHOpacityTo.new(self, duration, nil, 0.0),
                                    AHOpacityTo.new(self.bar, duration, nil, 0.0)
                                    )

  return fadeOutAction
end

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function DepthMeter.new(duration, onFinishCallback)
  local self = FilledRoundedRectangle.new(WIDTH, HEIGHT, CORNER_RADIUS, BACKGROUND_COLOR)
  Mixin.mix(self, DepthMeter)

  self.duration = duration
  local bar = FilledRectangle.new(WIDTH - MARGIN, HEIGHT - MARGIN, FOREGROUND_COLOR)
  bar:setAnchorPointBottom()
  bar:setLoc(CENTER_X, MARGIN * 0.5)
  bar:setScl(1.0, 0.0)
  self:insert(bar, Z_BAR)
  self.bar = bar

  self.onFinishCallback = onFinishCallback

  return self
end
