--- DepthSchool
-- @author Created by Aaron Barrett on February 3, 2014.

class("DepthSchool")
-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local WIDTH = Screen.width * 1.75
local HEIGHT = Screen.height * 0.5

local MAX_FISH_COUNT = 20
local MIN_FISH_COUNT = 5

local FISH_TO_ADD_PER_FRAME = 3

---------------------------------------

local INTERMEDIATE_POINT_COUNT = 3
local FIRST_AND_LAST_POINT_COUNT = 2
local TOTAL_POINT_COUNT = FIRST_AND_LAST_POINT_COUNT + INTERMEDIATE_POINT_COUNT

local DEVIANCE_RADIUS_X = Screen.width * 0.02
local DEVIANCE_RADIUS_Y = Screen.height * 0.15

local MIN_TOTAL_DURATION = 3.4
local MAX_TOTAL_DURATION = 8.1

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

local function addSchoolStaggered(self, points, isGoingRight, duration, ease)
  local fishOpacity = self.fishOpacity
  local fishCount = math.random(MIN_FISH_COUNT, MAX_FISH_COUNT)

  runInCoroutine(function()
                     local fishAddedCount = 0

                     repeat
                       local fish = DepthSchoolFish.new(points)
                       fish:setOpacity(fishOpacity)
                       local fishX, fishY = fish:getFirstPointLocation()
                       fish:setLoc(fishX, fishY)
                       self:insert(fish)

                       local fishSwimAction = fish:createSwimAction(duration, ease)
                       fishSwimAction:start()

                       fishAddedCount = fishAddedCount + 1
                       if (fishAddedCount % FISH_TO_ADD_PER_FRAME == 0) then
                         coroutine.yield()
                       end

                     until (fishAddedCount >= fishCount)
                   end)
end

local function generatePoints(firstPoint, lastPoint)
  local totalDX = lastPoint.x - firstPoint.x
  local totalDY = lastPoint.y - firstPoint.y

  local points = {firstPoint}

  local previousX = firstPoint.x
  local previousY = firstPoint.y

  for i = 2, TOTAL_POINT_COUNT - 1 do
    local completedProportion = (i - 1) / (TOTAL_POINT_COUNT - 1)

    local x = firstPoint.x + totalDX * completedProportion + math.random() * DEVIANCE_RADIUS_X * 2.0 - DEVIANCE_RADIUS_X
    local y = firstPoint.y + totalDY * completedProportion + math.random() * DEVIANCE_RADIUS_Y * 2.0 - DEVIANCE_RADIUS_Y

    local point = {x = x, y = y, progress = completedProportion}
    table.insert(points, point)
  end

  table.insert(points, lastPoint)

  return points
end

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function DepthSchool:startSchool()
  local firstPoint = {x = WIDTH, y = math.random() * HEIGHT}
  local lastPoint = {x = 0.0, y = math.random() * HEIGHT}
  local isGoingRight = MathUtils.getRandomBoolean()
  if (isGoingRight) then
    firstPoint, lastPoint = lastPoint, firstPoint
  end
  firstPoint.progress = 0.0
  lastPoint.progress = 1.0

  local duration = MIN_TOTAL_DURATION + math.random() * (MAX_TOTAL_DURATION - MIN_TOTAL_DURATION)

  local ease = nil
  local shouldUseEaseIn = MathUtils.getRandomBoolean()
  if (shouldUseEaseIn) then
    ease = AHEaseSimpleIn.new(1.0 + math.random())
  else
    ease = AHEaseSimpleOut.new(1.0 + math.random())
  end

  local points = generatePoints(firstPoint, lastPoint)
  addSchoolStaggered(self, points, isGoingRight, duration, ease)

  local removalAction = AHSequence.new(
                                       AHDelay.new(duration),
                                       AHCallLogic.new(function()
                                                           self:removeSelf()
                                                         end)
                                       )
  removalAction:start()
end

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function DepthSchool.new(fishOpacity)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, DepthSchool)

  self.fishOpacity = fishOpacity

  return self
end
