--- DepthSchoolFish
-- @author Created by aab29 on March 04, 2014.

class("DepthSchoolFish")

-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local DEVIANCE_RADIUS_X = Screen.width * 0.04
local DEVIANCE_RADIUS_Y = Screen.height * 0.04
local DEVIANCE_RADIUS_PROGRESS = 0.08

local OFFSET_RADIUS_X = Screen.height * 0.18
local OFFSET_RADIUS_Y = Screen.height * 0.09

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

local function catmullRom(a, b, c, d, t)
  local t2 = t * t
  local t3 = t2 * t

  return 0.5 * ((2.0 * b) + (-a + c) * t + (2.0 * a - 5.0 * b + 4.0 * c - d) * t2 + (-a + 3.0 * b - 3.0 * c + d) * t3)
end

local function generatePoints(schoolPoints)
  local pointsCount = #schoolPoints

  local points = {}

  local offsetX = math.random() * OFFSET_RADIUS_X * 2.0 - OFFSET_RADIUS_X
  local offsetY = math.random() * OFFSET_RADIUS_Y * 2.0 - OFFSET_RADIUS_Y

  for i, schoolPoint in ipairs(schoolPoints) do
    local devianceX = math.random() * DEVIANCE_RADIUS_X * 2.0 - DEVIANCE_RADIUS_X
    local devianceY = math.random() * DEVIANCE_RADIUS_Y * 2.0 - DEVIANCE_RADIUS_Y

    local x = schoolPoint.x + offsetX + devianceX
    local y = schoolPoint.y + offsetY + devianceY

    local progress = nil
    local isFirstOrLastPoint = (i == 1 or i == pointsCount)
    if (isFirstOrLastPoint) then
      progress = schoolPoints[i].progress
    else
      local devianceProgress = math.random() * DEVIANCE_RADIUS_PROGRESS * 2.0 - DEVIANCE_RADIUS_PROGRESS
      progress = schoolPoint.progress + devianceProgress
    end

    local point = {x = x, y = y, progress = progress}
    table.insert(points, point)
  end

  return points
end

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function DepthSchoolFish:getSwimProgress()
  return self.swimProgress
end

function DepthSchoolFish:setSwimProgress(progress)
  self.swimProgress = progress

  local points = self.points
  local pointsCount = #points

  while (progress > self.nextPointProgress) do
    self.pointIndex = self.pointIndex + 1
    self.nextPointProgress = points[self.pointIndex + 1].progress
  end

  local pointIndex = self.pointIndex

  local currentPointProgress = points[pointIndex].progress
  local segmentProgress = (progress - currentPointProgress) / (self.nextPointProgress - currentPointProgress)

  local x0 = points[pointIndex].x
  local y0 = points[pointIndex].y

  local x1 = points[pointIndex + 1].x
  local y1 = points[pointIndex + 1].y

  local cX0 = nil
  local cY0 = nil
  if (pointIndex >= 2) then
    cX0 = points[pointIndex - 1].x
    cY0 = points[pointIndex - 1].y
  else
    cX0 = x0
    cY0 = y0
  end

  local cX1 = nil
  local cY1 = nil
  if (pointIndex < pointsCount - 1) then
    cX1 = points[pointIndex + 2].x
    cY1 = points[pointIndex + 2].y
  else
    cX1 = x1
    cY1 = y1
  end

  local x = catmullRom(cX0, x0, x1, cX1, segmentProgress)
  local y = catmullRom(cY0, y0, y1, cY1, segmentProgress)

  local previousX, previousY = self:getLoc()
  local dX = x - previousX
  local dY = y - previousY
  local rotation = 180.0 + math.deg(math.atan2(dY, dX))
  self:setRot(rotation)

  self:setLoc(x, y)
end

function DepthSchoolFish:createSwimAction(schoolDuration, ease)
  local swimAction = AHNonAttributeTo.new(self, schoolDuration, self.getSwimProgress, self.setSwimProgress, ease, 1.0)
  return swimAction
end

function DepthSchoolFish:getFirstPointLocation()
  local firstPoint = self.points[1]
  local firstX = firstPoint.x
  local firstY = firstPoint.y
  return firstX, firstY
end

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function DepthSchoolFish.new(schoolPoints)
  local self = Sprite.newWithName("school_fish.png")
  Mixin.mix(self, DepthSchoolFish)

  self.points = generatePoints(schoolPoints)
  self.pointIndex = 1
  self.nextPointProgress = self.points[2].progress

  self.swimProgress = 0.0

  return self
end
