--- DepthSquiggleBurst
-- @author Created by Aaron Barrett on January 30, 2014.

class("DepthSquiggleBurst")
-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local WIDTH = 100.0
local HEIGHT = 100.0

local Z_CIRCLE = 10

local CIRCLE_COUNT = 5
local HOLLOW_CIRCLE_COUNT = 2

local MAX_CIRCLE_RADIUS = 12.0
local MIN_CIRCLE_RADIUS = 4.0

local CIRCLE_STEPS = 16

local CIRCLE_COLOR = {1.0, 1.0, 1.0}

local MAX_DESTINATION_OPACITY = 0.8
local MIN_DESTINATION_OPACITY = 0.2

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function DepthSquiggleBurst:createBurstAction(time, height, squiggleAmount)
  time = time or 0.25
  height = height or 180
  squiggleAmount = squiggleAmount or 2
  local circles = self.circles

  local circleActions = {}
  local squiggleEase = AHEaseSimpleInOut.new(2.0)

  for i, circle in ipairs(circles) do
    local fadeDelayDuration = math.random() * time/3
    local destinationOpacity = circle.destinationOpacity
    local circleFadeAction = AHSequence.new(
                                            AHOpacityTo.new(circle, time/3, nil, destinationOpacity),
                                            AHDelay.new(fadeDelayDuration),
                                            AHOpacityTo.new(circle, time/3, nil, 0.0)
                                            )
    table.insert(circleActions, circleFadeAction)

    local squiggleAmount = math.random() * squiggleAmount * 2 - squiggleAmount
    local circleSquiggleAction = AHLoop.new(
                                            AHSequence.new(
                                                           AHLocationXBy.new(circle, 0.133, squiggleEase, squiggleAmount),
                                                           AHLocationXBy.new(circle, 0.133, squiggleEase, -squiggleAmount)
                                                           ),
                                            12
                                            )
    table.insert(circleActions, circleSquiggleAction)
  end

  local riseAction = AHLocationYBy.new(self, time, nil, height)

  local burstAction = AHSpawn.new(
                                  riseAction,
                                  unpack(circleActions)
                                  )

  return burstAction
end

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function DepthSquiggleBurst.new(opacity, squiggleCount, width, height, spawnHeightProportion)
  squiggleCount = squiggleCount or CIRCLE_COUNT
  width = width or WIDTH
  height = height or HEIGHT

  local self = Group.new(width, height)
  Mixin.mix(self, DepthSquiggleBurst)

  local circles = {}

  for i = 1, squiggleCount do
    local isOutlineOnly = (i <= HOLLOW_CIRCLE_COUNT)

    local circleRadius = MIN_CIRCLE_RADIUS + math.random() * (MAX_CIRCLE_RADIUS - MIN_CIRCLE_RADIUS)

    local circle = FilledCircle.new(circleRadius, CIRCLE_COLOR, CIRCLE_STEPS, isOutlineOnly)

    local destinationOpacity = MIN_DESTINATION_OPACITY + math.random() * (MAX_DESTINATION_OPACITY - MIN_DESTINATION_OPACITY)
    circle.destinationOpacity = destinationOpacity
    circle:setOpacity(0.0)

    local x = math.random() * width
    local y = math.random() * height
    circle:setLoc(x, y)

    self:insert(circle, Z_CIRCLE)
    table.insert(circles, circle)
  end

  self.circles = circles

  return self
end
