--- DepthStreakScroller
-- @author Created by aab29 on March 10, 2014.

class("DepthStreakScroller")

-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.height

local CENTER_X = Screen.width * 0.5

local WAVE_CYCLE_DURATION = 9.3

local STREAK_COUNT = 5
local STREAK_SPACING = HEIGHT / (STREAK_COUNT - 1)

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function DepthStreakScroller:setUOffset(uOffset)
  self.uOffset = uOffset
  local waveDeck = self.waveDeck
  local wavesPerStreak = self.wavesPerStreak
  waveDeck:setUVRect(uOffset, 1.0, uOffset + wavesPerStreak, 0.0)
end

function DepthStreakScroller:getUOffset()
  return self.uOffset
end

function DepthStreakScroller:startScrolling()
  runInCoroutine(function()
                     while (true) do
                       local uAction = AHNonAttributeTo.new(self, WAVE_CYCLE_DURATION, self.getUOffset, self.setUOffset, nil, 1.0)
                       wait(uAction:start())
                       self:setUOffset(0.0)
                     end
                   end)
end

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

-- Constructor ------------------------

function DepthStreakScroller.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, DepthStreakScroller)

  local texture = TextureLoader.loadTexture("tiles/streak.png")

  local textureWidth, textureHeight = texture:getSize()
  texture:setWrap(true, false)

  local waveWidth = textureWidth / Host.contentScale
  local waveHeight = textureHeight / Host.contentScale

  local wavesPerStreak = WIDTH / waveWidth

  local waveDeck = MOAIGfxQuad2D.new()
  waveDeck:setTexture(texture)
  waveDeck:setRect(0.0, 0.0, WIDTH, waveHeight)
  waveDeck:setUVRect(0.0, 1.0, wavesPerStreak, 0.0)

  self.waveDeck = waveDeck
  self.wavesPerStreak = wavesPerStreak
  self.uOffset = 0.0

  for i = 1, STREAK_COUNT do
    local streak = DepthStreak.new(waveDeck, waveHeight)
    local y = (i - 1) * STREAK_SPACING

    streak:setLoc(CENTER_X, y)
    self:insert(streak)
  end

  return self
end
