class("FractionGenerator")
----------------------------------------------------------------------------
-------------------------------- CONFIGURATION -----------------------------
----------------------------------------------------------------------------

local MAX_NUMBER = 200
local FRACTION_MULTIPLIERS =
{
  {2, 3, 4, 5},
  {2, 3, 4, 5, 6, 7, 8, 9},
  {2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 13, 14, 15, 16, 17, 18, 19}
}
local MAX_GENERATION_ATTEMPTS = 200


----------------------------------------------------------------------------
----------------------------------- UTILITY  -------------------------------
----------------------------------------------------------------------------
local function getNumeratorForDenominatorAndPercentage(denominator, percentage)
  return percentage * denominator/100
end

local function generateRandomFraction()
  local maxDenominator = 999
  local percentage = math.random(0, 100)
  local denominator = math.random()
  denominator = math.floor(denominator * maxDenominator + 1)
  local numerator = getNumeratorForDenominatorAndPercentage(denominator, percentage)
  while(math.floor(numerator) ~= numerator) do
    denominator = math.floor(math.random() * maxDenominator + 1)
    numerator = getNumeratorForDenominatorAndPercentage(denominator, percentage)
  end
  return numerator, denominator, percentage
end

-------------- Generate error by modifying a fraction in steps -------------
local function percentageToFraction(startPercentage)
  -- get the percentage into a numerator form with the lowest possible denominator
  -- Divide by 100: 5 * 5 * 2 * 2

  local numerator = startPercentage
  local denominator = 100
  while(denominator % 5 == 0 and numerator % 5 == 0) do
    numerator = numerator/5
    denominator = denominator/5
  end
  while(denominator % 2 == 0 and numerator % 2 == 0) do
    numerator = numerator/2
    denominator = denominator/2
  end
  local wereChangedApplied = numerator ~= startPercentage
  return numerator, denominator, wereChangedApplied, "Simplify "..numerator.."/"..denominator
end

local function changePower(numerator, denominator, normalizedDifficulty)
  -- Change the fraction by a multiple of 10, e.g. "30/200 -> 300/2000"
  local abortRoll = math.random()
  if(abortRoll < 0.3) then
    return numerator, denominator, false
  end
  local numeratorDigits = #(""..numerator)
  local denominatorDigits = #(""..denominator)
  local maxNumberDigits = math.max(numeratorDigits, denominatorDigits)
  local maxAllowedDigits = math.floor(math.log10(MAX_NUMBER))

  local maxDigits = math.max(maxAllowedDigits - maxNumberDigits, 0)
  local minDigits = 0

  local changeAmount = math.random(minDigits, maxDigits)
  local wasChanged = changeAmount ~= 0
  local changeMultiplier = math.pow(10, changeAmount)

  while changeMultiplier > 1 and (changeMultiplier * denominator > MAX_NUMBER or changeMultiplier * numerator > MAX_NUMBER) do
    changeMultiplier = changeMultiplier - 1
  end

  return numerator * changeMultiplier, denominator * changeMultiplier, wasChanged, "Power 10^"..changeAmount
end
local function getMultiplier(normalizedDifficulty, maxMultiplier)
  local difficultyIndex = normalizedDifficulty * #FRACTION_MULTIPLIERS
  difficultyIndex = math.floor(difficultyIndex + math.random() * 0.8)
  if(difficultyIndex < 1) then
    return 1
  end

  local multiplierSet = FRACTION_MULTIPLIERS[difficultyIndex]
  multiplierSet = TableUtils.getFilteredArray(multiplierSet, function(multiplier) return multiplier <= maxMultiplier end)
  if #multiplierSet == 0 then
    return 1
  end

  local multiplierIndex = math.random(1, #multiplierSet)
  return multiplierSet[multiplierIndex]
end
local function multiplyNumerAndDenom(numerator, denominator, normalizedDifficulty)
  -- Change the fraction by an integer multiple, e.g. "30/200 -> 60/400"
  local abortRoll = math.random()
  local maxValue = math.max(numerator, denominator)
  local maxMultiplier = math.floor(MAX_NUMBER/maxValue)
  local multiplier = getMultiplier(normalizedDifficulty, maxMultiplier)

  if(abortRoll < 0.3 or multiplier == 1) then
    return numerator, denominator, false
  end

  local wasModified = true
  local multipliedNumerator = numerator * multiplier
  local multipliedDenominator = denominator * multiplier

  return multipliedNumerator, multipliedDenominator, wasModified, "Mult by "..multiplier
end

local function chooseErrorWithinMargin(errorAmp, minErrorPercent, maxErrorPercent, source)
  local errorPercentAmp = maxErrorPercent - minErrorPercent
  local errorPercent = math.random() * errorPercentAmp + minErrorPercent
  local errorAbsValue = math.max(errorPercent * errorAmp, 1)
  local errorValue = MathUtils.randomSign() * math.floor(errorAbsValue)

  return source + errorValue, errorValue
end
local function chooseSmallError(errorAmp, source)
  local minErrorPercent = 0.0
  local maxErrorPercent = 0.4
  return chooseErrorWithinMargin(errorAmp, minErrorPercent, maxErrorPercent, source)
end
local function chooseBigError(errorAmp, source)
  local minErrorPercent = 0.4
  local maxErrorPercent = 0.7
  return chooseErrorWithinMargin(errorAmp, minErrorPercent, maxErrorPercent, source)
end
local function chooseErrorPercentage(sourcePercentage, normalizedDifficulty)
  local errorAmp = sourcePercentage - 1 -- -1 to avoid being equal to the percentage
  local errorTypeRoll = math.random() * 0.8 + normalizedDifficulty
  if(errorTypeRoll < 1) then
    return chooseBigError(errorAmp, sourcePercentage)
  else
    return chooseSmallError(errorAmp, sourcePercentage)
  end
end
local function chooseErrorFractionForPercentage(excludedPercentage, normalizedDifficulty)
  local errorPercentage, difference = chooseErrorPercentage(excludedPercentage, normalizedDifficulty)
  local fraction, total, wasModified, fractionKind = percentageToFraction(errorPercentage, normalizedDifficulty)
  return fraction, total, true, "Error By "..difference..","..fractionKind
end
local function chooseFractionForPercentage(percentage, normalizedDifficulty)
  local fraction, total, wasModified, fractionKind = percentageToFraction(percentage, normalizedDifficulty)
  return fraction, total, wasModified, fractionKind
end

local function generateObfuscatedFraction(startPercentage, normalizedDifficulty, percentageConversionFunction, modifierFunctions)
  local obfuscationLevel = normalizedDifficulty * 4 -- maximum 4 steps
  local obfPointsLeft = obfuscationLevel

  local modifiers = {"Source: "..startPercentage}
  local result = {numerator = 0, denominator = 0, modifiers = modifiers, difficulty = 0}

  local function processResult(numerator, denominator, wasModified, modifierName)
    result.numerator, result.denominator = numerator, denominator
    if(wasModified) then
      obfPointsLeft = obfPointsLeft - 1
      result.difficulty = result.difficulty + 1
      modifiers[#modifiers + 1] = modifierName
    end
  end

  local numerator, denominator, wasModified, errorKind = percentageConversionFunction(startPercentage, normalizedDifficulty)
  processResult(numerator, denominator, wasModified, errorKind)

  for _, modifierInfo in ipairs(modifierFunctions) do
    if(obfPointsLeft <= 0) then return result end
    local cost = modifierInfo.cost
    if(obfPointsLeft >= cost) then
      local modifier = modifierInfo.modifier
      numerator, denominator, wasModified, errorKind = modifier(numerator, denominator, normalizedDifficulty)
      processResult(numerator, denominator, wasModified, errorKind)
    end
  end

  return result
end

local function isFractionEqual(f1, f2)
  return f1.numerator == f2.numerator and f1.denominator == f2.denominator
end

local function generateErrorFraction(startPercentage, normalizedDifficulty, excludedFractionsList)
  local percentageConversionFunction = chooseErrorFractionForPercentage
  local modifiers = {
                      {modifier = multiplyNumerAndDenom, cost = 1},
                      {modifier = changePower, cost = 0.5}
                    }
  local result = generateObfuscatedFraction(startPercentage, normalizedDifficulty, percentageConversionFunction, modifiers)

  local attempt = 0

  while(excludedFractionsList and attempt < MAX_GENERATION_ATTEMPTS) do
    attempt = attempt + 1
    local success = true
    for i, excludedFraction in pairs(excludedFractionsList) do
      if(isFractionEqual(excludedFraction, result)) then
        result = generateObfuscatedFraction(startPercentage, normalizedDifficulty, percentageConversionFunction, modifiers)
        success = false
        break
      end
    end
    if(success) then break end
  end
  return result
end
local function generateCorrectFraction(percentage, normalizedDifficulty)
  percentageConversionFunction = chooseFractionForPercentage
  local modifiers = {
                      {modifier = multiplyNumerAndDenom, cost = 1},
                      {modifier = changePower, cost = 0.5}
                    }
  return generateObfuscatedFraction(percentage, normalizedDifficulty, percentageConversionFunction, modifiers)
end

----------------------------------------------------------------------------
--------------------------------- API  -------------------------------------
----------------------------------------------------------------------------

function FractionGenerator.generateRandomPercentageFraction(correctPercentage, excludedFractionsList)
  local normalizedDifficulty = GameSession.getDifficulty()
  return generateErrorFraction(correctPercentage, normalizedDifficulty, excludedFractionsList)
end

function FractionGenerator.generatePercentageFraction(percentage)
  local normalizedDifficulty = GameSession.getDifficulty()
  return generateCorrectFraction(percentage, normalizedDifficulty)
end
