--- DepthValidator
-- @author Created by amorant on July 28, 2014.
-- ConceptValidator for Depth.

class("DepthValidator")

require("validators.ConceptValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  "difficulty",
  "percent",
  "fraction",
  "total"
}

local REQUIRED_EMPTY_ALLOWED_KEYS = {
  "fraction_errors",
  "total_errors"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validatePercentage(self, concept)
  local percentage = concept.percent
  local numerator = concept.fraction
  local denominator = concept.total

  self:expectNumberInRange(percentage, 0, 100, "percent")
  self:expectFloatEqual(numerator/denominator, percentage/100, "part_correct/total_correct should match percentage")
end

local function validateErrors(self, concept)
  local percentage = concept.percent
  local numerator = concept.fraction
  local denominator = concept.total
  local badNumeratorsString = concept.fraction_errors
  local badDenominatorsString = concept.total_errors

  local badNumerators = TableUtils.csvToArray(badNumeratorsString)
  for _, badNumerator in ipairs(badNumerators) do
    badNumerator = tonumber(badNumerator)
    self:expectNotEqual(badNumerator/denominator, percentage/100, "Combinations of part_incorrect/total_correct should not match percentage")
    self:expectNotEqual(badNumerator, denominator, "part_incorrect must not contain total_correct")
  end

  local badDenominators = TableUtils.csvToArray(badDenominatorsString)
  for _, badDenominator in ipairs(badDenominators) do
    badDenominator = tonumber(badDenominator)
    self:expectNotEqual(numerator/badDenominator, percentage/100, "Combinations of part_correct/total_incorrect should not match percentage")
    self:expectNotEqual(badDenominator, numerator, "total_incorrect must not contain part_correct")
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DepthValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)
  self:validateRequiredKeys(concept, REQUIRED_EMPTY_ALLOWED_KEYS, true)

  validatePercentage(self, concept)
  validateErrors(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function DepthValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, DepthValidator)

  return self
end
