--- EggcornsBackground
-- @author Created by Jay Austria on August 14, 2015.

class("EggcornsBackground")

require("EggcornsCommon")
require("graphics.TiledImage")
require("graphics.MultiGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INITIAL_BEGIN_COLOR = ColorUtils.colorHex24ToStd(0x000000)
local INITIAL_END_COLOR = ColorUtils.colorHex24ToStd(0x009f7c)
local INCORRECT_END_COLOR = ColorUtils.colorHex24ToStd(0x810000)

local BACKGROUND_FILLING_COLORS = {
  ColorUtils.colorHex24ToStd(0x000000),
  ColorUtils.colorHex24ToStd(0x00414F),
  ColorUtils.colorHex24ToStd(0x009f7c)
}

local BACKGROUND_FILLING_COLORS_INCORRECT = {
  ColorUtils.colorHex24ToStd(0x000000),
  ColorUtils.colorHex24ToStd(0x4F002D),
  ColorUtils.colorHex24ToStd(0x810000)
}

local BACKGROUND_FILLING_COLORS_OUTRO = {
  EggcornsCommon.COLORS.OVEREXPOSED_AQUA,
  EggcornsCommon.COLORS.OVEREXPOSED_AQUA,
  EggcornsCommon.COLORS.OVEREXPOSED_AQUA
}

local BACKGROUND_FILLING_THICKNESSES = {
  0.2,
  0.8
}

local BACKGROUND_FILLING_THICKNESSES_INCORRECT = BACKGROUND_FILLING_THICKNESSES
local SECONDARY_GRADIENT_START = ColorUtils.colorHex32ToStd(0x00000099)
local SECONDARY_GRADIENT_END = ColorUtils.colorHex32ToStd(0x00454900)
local EDGE_GLOW_CORRECT_COLOR = EggcornsCommon.COLORS.AQUA
local EDGE_GLOW_INCORRECT_COLOR = EggcornsCommon.COLORS.RED
local BLEED_OVER_X = 2 * EggcornsCommon.SHAKE_MAGNITUDE
local BLEED_OVER_Y = 2 * EggcornsCommon.SHAKE_MAGNITUDE
local EDGE_GLOW_OPACITY = 0.7
local SCREEN_GLOW_GRADIENT_START = ColorUtils.colorHex32ToStd(0xFFFFFF55)
local SCREEN_GLOW_GRADIENT_END = ColorUtils.colorHex32ToStd(0x98E2DB44)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------



-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------
function EggcornsBackground:createOutroAction()
  local backgroundGradient = self.backgroundGradient
  local tintAction = backgroundGradient:newGradientToAction(0.2, nil, BACKGROUND_FILLING_COLORS_OUTRO)

  return tintAction
end

function EggcornsBackground:createTintRedAction()
  local backgroundGradient = self.backgroundGradient
  local tintAction = backgroundGradient:newGradientToAction(0.2, nil, BACKGROUND_FILLING_COLORS_INCORRECT)

  return tintAction
end

function EggcornsBackground:createTintDefaultAction()
  local backgroundGradient = self.backgroundGradient
  local tintAction = backgroundGradient:newGradientToAction(0.5, nil, BACKGROUND_FILLING_COLORS)

  return tintAction
end

function EggcornsBackground:createFadeInEdgeGlowAction()
  local fadeDuration = 0.1
  local fadeAction = AHOpacityTo.new(self.edgeGlow, fadeDuration, nil, EDGE_GLOW_OPACITY)

  return fadeAction
end

function EggcornsBackground:createFadeOutEdgeGlowAction()
  local fadeDuration = 0.3
  local fadeAction = AHOpacityTo.new(self.edgeGlow, fadeDuration, nil, EggcornsCommon.INVISIBLE_OPACITY)

  return fadeAction
end

function EggcornsBackground:fadeInScreenGlow()
  local fadeDuration = 0.01
  local fadeAction = self.screenGlow:newGradientToAction(fadeDuration, nil, SCREEN_GLOW_GRADIENT_START, SCREEN_GLOW_GRADIENT_END)

  return fadeAction:start()
end

function EggcornsBackground:fadeOutScreenGlow()
  local fadeDuration = 0.3
  local fadeAction = self.screenGlow:newGradientToAction(fadeDuration, nil, EggcornsCommon.COLORS.INVISIBLE_WHITE, EggcornsCommon.COLORS.INVISIBLE_WHITE)

  return fadeAction:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local screenDiagonalLength = MathUtils.distance(0, 0, Screen.fullWidth, Screen.fullHeight) + BLEED_OVER_X
  local isHorizontal = false
  local backgroundGradient = MultiGradient.new(screenDiagonalLength, screenDiagonalLength, isHorizontal, BACKGROUND_FILLING_COLORS, BACKGROUND_FILLING_THICKNESSES)
  backgroundGradient:setRot(45)
  self:insertAtCenter(backgroundGradient)

  local backgroundTexture = TiledImage.newWithFilename("tiles/background_tile.png", self:getWidth() * 1.3, self:getHeight(), 6)
  backgroundTexture:setOpacity(0.35)
  self:insertAtCenter(backgroundTexture)

  local secondaryGradient = RectangleGradient.new(screenDiagonalLength, screenDiagonalLength, SECONDARY_GRADIENT_START, SECONDARY_GRADIENT_END)
  secondaryGradient:setRot(45)
  self:insertAtCenter(secondaryGradient)

  self.backgroundGradient = backgroundGradient

  local edgeGlow = Sprite.newWithName("edge_glow.png")
  local widthScale = self:getWidth() / edgeGlow:getWidth()
  local heightScale = self:getHeight() / edgeGlow:getHeight()

  edgeGlow:setColor(unpack(EDGE_GLOW_CORRECT_COLOR))
  edgeGlow:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)
  edgeGlow:setScl(widthScale, heightScale)
  self:insertAtCenter(edgeGlow)
  self.edgeGlow = edgeGlow

  local screenGlow = RectangleGradient.new(screenDiagonalLength, screenDiagonalLength, EggcornsCommon.COLORS.INVISIBLE_WHITE, EggcornsCommon.COLORS.INVISIBLE_WHITE)
  screenGlow:setRot(45)
  self:insertAtCenter(screenGlow)
  self.screenGlow = screenGlow
end

function EggcornsBackground.new()
  local width = Screen.fullWidth + BLEED_OVER_X
  local height = Screen.fullHeight + BLEED_OVER_Y
  local self = Group.new(width, height)
  Mixin.mix(self, EggcornsBackground)

  createLayout(self)

  return self
end
