--- EggcornsBlipTrail
-- @author Created by Jay Austria on August 28, 2015.

class("EggcornsBlipTrail")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DEFAULT_MAX_SCALE = 2
local BLIP_COUNT = 8

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createBlip(self, index, count, maxScale)
  assert(index <= count, "Invalid index used when creating blip")
  assert(count > 0, "Invalid count used when creating blip")

  local interpolationValue = ((index - 1) / (count - 1))

  local minScale = 1
  local scale = MathUtils.interpolateLinearFromNormalized(interpolationValue, minScale, maxScale)

  local minOpacity = 0.01
  local maxOpacity = 0.8
  local opacity = MathUtils.interpolateLinearFromNormalized(interpolationValue, maxOpacity, minOpacity)

  local blip = FilledCircle.new(self.radius, EggcornsCommon.COLORS.OVEREXPOSED_AQUA)
  blip:setScl(scale)
  blip:setOpacity(opacity)
  blip:setBlendMode(MOAIProp2D.BLEND_ADD)

  return blip
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

local function updateTrail(self)
  local x, y = self.target:getLoc()
  local firstBlip = self.children[1]
  local previousX, previousY = firstBlip:getLoc()

  local downScaleFactor = 0.98

  for _, blip in ipairs(self.children) do
    previousX, previousY = blip:getLoc()
    blip:setLoc(x, y)
    local lastScale = blip:getScl()
    blip:setScl(math.max(lastScale * downScaleFactor, 1.0))


    x, y = previousX, previousY
  end
end

function EggcornsBlipTrail:follow()
  local x, y = self.target:getLoc()
  for _, blip in ipairs(self.children) do
    blip:setLoc(x, y)
  end

  self.updateCoroutine = runInCoroutine(function()
    while true do
      updateTrail(self)
      coroutine.yield()
    end
  end)

  self:setVisibleRecursive(true)
end

function EggcornsBlipTrail:createDestroyAction()
  local destroyFunction = function()
    self.updateCoroutine:stop()
    self:removeSelf()
  end

  return destroyFunction
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, maxScale)
  for blipIndex = 1, BLIP_COUNT do
    local blip = createBlip(self, blipIndex, BLIP_COUNT, maxScale)
    blip:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.5))
    self:insert(blip)
  end

  self:setVisibleRecursive(false)
end

function EggcornsBlipTrail.newDisabled(target, radius, maxScale)
  local self = Group.newFullScreen()
  Mixin.mix(self, EggcornsBlipTrail)

  maxScale = maxScale or DEFAULT_MAX_SCALE

  self.target = target
  self.radius = radius

  createLayout(self, maxScale)

  return self
end
