--- EggcornsCircularContainer
-- @author Created by Jay Austria on August 19, 2015.

class("EggcornsCircularContainer")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getLocationCircleUnitSpace(xModelSpace, yModelSpace, width, height)
  local xUnitSpace = 2.0 * (xModelSpace / width) - 1.0
  local yUnitSpace = 2.0 * (yModelSpace / height) - 1.0

  local xCircleUnitSpace = xUnitSpace * math.sqrt(1 - (math.pow(yUnitSpace, 2) / 2))
  local yCircleUnitSpace = yUnitSpace * math.sqrt(1 - (math.pow(xUnitSpace, 2) / 2))

  return xCircleUnitSpace, yCircleUnitSpace
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function EggcornsCircularContainer:createMockSphereAnimation(textArea)
  local mockSphereAnimation = AHSpawn.new()
  local width, height = textArea:getDims()
  local warpDuration = 0.1
  local moveDuration = 0.3
  local minScale = 0.78
  local maxScale = 1.0

  for _, view in ipairs(textArea.itemViews) do
    if view.layout then
      local rows = view.layout:getRows()
      for _, row in ipairs(rows) do
        for _, glyph in ipairs(row) do
          local xModelSpace, _ = glyph:getLoc()

          local xTextAreaSpace, yTextAreaSpace = view:getLoc()
          xTextAreaSpace = xTextAreaSpace + xModelSpace

          local xCircleUnitSpace, yCircleUnitSpace = getLocationCircleUnitSpace(xTextAreaSpace, yTextAreaSpace, width, height)
          local randomYOffset = math.random() * math.random() * -yCircleUnitSpace

          local xCircleMapped = xCircleUnitSpace * self.radius + self:getWidthPart(0.5)
          local yCircleMapped = (yCircleUnitSpace + randomYOffset) * self.radius + self:getHeightPart(0.5)

          local moveAction = AHLocationTo.new(glyph, moveDuration, AHEaseSineInOut.new(), xCircleMapped, yCircleMapped)

          local fadeDuration = MathUtils.getRandomFloat(0.1, 0.3)
          local fadeAction = AHDelayed.new(warpDuration, AHOpacityTo.new(glyph, fadeDuration, nil, 0))

          local circleUnitSpaceDistanceFromCenter = MathUtils.distance(0, 0, xCircleUnitSpace, yCircleUnitSpace)
          local interpolationValue = MathUtils.clamp(math.abs(circleUnitSpaceDistanceFromCenter), 0, 1)
          local scaleAmount = MathUtils.interpolateLinearFromNormalized(interpolationValue, maxScale, minScale)
          local scaleDownAction = AHScaleTo.new(glyph, warpDuration, nil, scaleAmount)
          local scaleOutAction = AHScaleBy.new(glyph, 0.3, nil, 0.7)
          local scaleSequence = AHSequence.new(
            scaleDownAction,
            scaleOutAction
          )

          local rotationDuration = MathUtils.getRandomFloat(0.1, 0.2)
          local rotationAmount = MathUtils.getRandomFloat(-90, 90)
          local rotationAction = AHDelayed.new(warpDuration, AHRotationBy.new(glyph, rotationDuration, AHEaseSimpleIn.new(), rotationAmount))

          local groupRotationAction = AHRotationBy.new(self, 0.8, AHEaseExponentialIn.new(), 150)

          local glyphAction = AHSpawn.new(
            function()
              self:takeChild(glyph)
              glyph:clearAttrLink(MOAIColor.INHERIT_COLOR)
              glyph:setColor(unpack(EggcornsCommon.COLORS.WHITE))
            end,
            moveAction,
            fadeAction,
            scaleSequence,
            groupRotationAction
          )

          mockSphereAnimation:addAction(glyphAction)
        end
      end
    end
  end

  return mockSphereAnimation
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------


function EggcornsCircularContainer.new(radius)
  local self = Group.new(radius, radius)
  Mixin.mix(self, EggcornsCircularContainer)

  self.radius = radius

  return self
end
