--- EggcornsHighLevelInterstitial
-- @author Created by Jay Austria on December 11, 2015.

class("EggcornsHighLevelInterstitial")

require("EggcornsBackground")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BODY_TEXT = Localizer.get("HIGH_DIFFICULTY_INTERSTITIAL")
local CONTINUE_TEXT = Localizer.get("CONTINUE")
local BODY_COLOR = ColorUtils.colorHex24ToStd(0xFFFFFF)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local background = EggcornsBackground.new()
  self:insertAtCenter(background)

  local columnWidth = Screen.getSafeWidthPart(0.8)
  local interstitialFont = Font.new(FontFaces.DIN_OT_MEDIUM, 18)
  local baseAttributes = {font = interstitialFont}

  local bodyLabel = DynamicTextArea.new(BODY_TEXT, columnWidth, baseAttributes, nil, Alignment.LEFT, Alignment.CENTER)
  bodyLabel:setColor(unpack(BODY_COLOR))

  local continueLabel = Label.new(CONTINUE_TEXT, interstitialFont, columnWidth, Alignment.CENTER)

  local layoutColumn = Column.new()

  layoutColumn:insert(bodyLabel)
  layoutColumn:insertSpacer(Screen.getSafeHeightPart(0.35))
  layoutColumn:insert(continueLabel)

  layoutColumn:alignChildren()

  layoutColumn:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.35))
  self:insert(layoutColumn)
  self.layoutColumn = layoutColumn

  local isRecursive = true
  layoutColumn:linkOpacityAttrToChildren(isRecursive)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function EggcornsHighLevelInterstitial:prepareForShowAction()
  self.layoutColumn:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)
end

function EggcornsHighLevelInterstitial:startShowAction()
  local fadeDuration = 0.5
  local fadeIn = AHOpacityTo.new(self.layoutColumn, fadeDuration, nil, EggcornsCommon.VISIBLE_OPACITY)

  return fadeIn:start()
end

function EggcornsHighLevelInterstitial:startHideAction()
  local fadeDuration = 0.5
  local fadeOut = AHOpacityTo.new(self.layoutColumn, fadeDuration, nil, EggcornsCommon.INVISIBLE_OPACITY)
  local removeFromScene = AHRemove.new(self)

  local combinedFadeOut = AHSequence.new(fadeOut, removeFromScene)

  return combinedFadeOut:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function EggcornsHighLevelInterstitial.new()
  local self = Group.newFullScreen()

  Mixin.mix(self, EggcornsHighLevelInterstitial)

  createLayout(self)

  return self
end
