--- EggcornsProgressBar
-- @author Created by Jay Austria on August 14, 2015.

class("EggcornsProgressBar")

require("EggcornsProgressCircle")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LINE_THICKNESS = 1
local MIDLINE_COLOR = EggcornsCommon.COLORS.WHITE
local BAR_ORIGINAL_OPACITY = 0.10
EggcornsProgressBar.CIRCLE_DIAMETER = EggcornsCommon.GAME_UI_WIDTH / ((EggcornsConfiguration.CONCEPTS_TO_COMPLETE * 2) + 1)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playIncrementAnimation(self, progressStep)
  local progressCircle = self.leftIndexedDots[progressStep]
  progressCircle:playIncrementAnimation()
end

local function createMidlinePiece(self, circleDiameter)
  local midlinePiece = FilledRectangle.new(circleDiameter, LINE_THICKNESS, MIDLINE_COLOR)
  midlinePiece:inheritOpacity(self)

  return midlinePiece
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function EggcornsProgressBar:createFadeOutAction(ignoreDots, duration)
  local midLine = self.midLine
  duration = duration or EggcornsCommon.UI_FADE_DURATION
  local fadeOut = AHSpawn.new(AHOpacityTo.new(self, duration, nil, EggcornsCommon.INVISIBLE_OPACITY))

  if not ignoreDots then
    for _, child in ipairs(self.progressCircleRow.children) do
      if child.createFadeOutAnimation then
        fadeOut:addAction(child:createFadeOutAnimation(duration))
      end
    end
  end

  return fadeOut
end

function EggcornsProgressBar:createFadeInAction()
  local midLine = self.midLine
  local fadeIn = AHSpawn.new(AHOpacityTo.new(self, 0.3, nil, BAR_ORIGINAL_OPACITY))

  for _, child in ipairs(self.progressCircleRow.children) do
    if child.createFadeInAnimation then
      fadeIn:addAction(child:createFadeInAnimation())
    end
  end

  return fadeIn
end

function EggcornsProgressBar:incrementProgress()
  self.super.incrementProgress(self)

  local progressStep = self:getProgressStep()
  playIncrementAnimation(self, progressStep)
end

function EggcornsProgressBar:getNextDotLocation()
  local nextDotIndex = self:getProgressStep() + 1
  local nextProgressDot = self.leftIndexedDots[nextDotIndex]
  local worldX, worldY = nextProgressDot:getCenterWorldLoc()

  return worldX, worldY
end

function EggcornsProgressBar:getDots()
  return self.leftIndexedDots
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, steps, circleDiameter)
  -- Inserted right to left to keep leftmost animations z-sorted higher
  local progressCircleRow = Row.new(0, Alignment.CENTER, Row.GrowLeft)

  progressCircleRow:insert(createMidlinePiece(self, circleDiameter))

  local leftIndexedDots = {}
  for currentStepIndex = 1, steps do
    local progressCircle = EggcornsProgressCircle.newHidden(circleDiameter)
    progressCircleRow:insert(progressCircle)

    local indexFromLeft = steps - currentStepIndex + 1
    leftIndexedDots[indexFromLeft] = progressCircle

    progressCircleRow:insert(createMidlinePiece(self, circleDiameter))
  end

  self.leftIndexedDots = leftIndexedDots

  progressCircleRow:alignChildren()
  self:insertAtCenter(progressCircleRow)
  self.progressCircleRow = progressCircleRow

  self:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)
end

function EggcornsProgressBar.newHidden(steps)
  local circleDiameter = EggcornsProgressBar.CIRCLE_DIAMETER
  local self = Group.new(EggcornsCommon.GAME_UI_WIDTH, circleDiameter)

  ProgressIndicator.makeProgressIndicator(self, steps)
  Mixin.mix(self, EggcornsProgressBar)

  createLayout(self, steps, circleDiameter)

  return self
end
