--- EggcornsProgressCircle
-- @author Created by Jay Austria on August 31, 2015.

class("EggcornsProgressCircle")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------
local OUTLINE_COLOR = EggcornsCommon.COLORS.WHITE
local OUTLINE_OPACITY = 0.14
local BACKING_OPACITY = 0.06
local BACKING_COLOR = EggcornsCommon.COLORS.WHITE
local FILL_INITIAL_SCALE = 2
local HALO_THICKNESS = 1
local HALO_SCALE_AMOUNT = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createIncrementAnimation(self)
  local halo = self.halo
  local fill = self.fill

  local incrementAnimation = AHSpawn.new(
    AHOpacityTo.new(halo, 0.1, nil, EggcornsCommon.VISIBLE_OPACITY),
    AHSequence.new(
      AHScaleTo.new(halo, 0.2, AHEaseSimpleInOut.new(), HALO_SCALE_AMOUNT),
      AHDelay.new(0.1),
      AHOpacityTo.new(halo, 0.3, nil, EggcornsCommon.INVISIBLE_OPACITY)
    ),
    function() fill:setOpacity(EggcornsCommon.VISIBLE_OPACITY) end,
    AHScaleTo.new(fill, 0.4, AHEaseBackOut.new(), self.fillFitScale)
  )

  self.incrementAnimation = incrementAnimation
end

local function createFadeAction(self, endOpacity, fillEndOpacity, duration)
  local backing = self.backing
  local outline = self.outline
  local fill = self.fill

  duration = duration or EggcornsCommon.UI_FADE_DURATION

  local fade = AHSpawn.new(
    AHOpacityTo.new(backing, duration, nil, endOpacity),
    AHOpacityTo.new(outline, duration, nil, endOpacity),
    AHOpacityTo.new(fill, duration, nil, fillEndOpacity)
  )

  return fade
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function EggcornsProgressCircle:playIncrementAnimation()
  self.incrementAnimation:start()
end

function EggcornsProgressCircle:createFadeOutAnimation(duration)
  local fill = self.fill

  self.originalFillOpacity = self.fill:getOpacity()

  local fillEndOpacity = EggcornsCommon.INVISIBLE_OPACITY
  local fadeOut = createFadeAction(self, EggcornsCommon.INVISIBLE_OPACITY, fillEndOpacity, duration)

  return fadeOut
end

function EggcornsProgressCircle:createFadeInAnimation()
  local fillEndOpacity = self.originalFillOpacity
  local fadeIn = createFadeAction(self, BACKING_OPACITY, fillEndOpacity)

  return fadeIn
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, diameter)
  local radius = diameter / 2
  local haloRadius = diameter + 2
  local backing = FilledCircle.new(radius, BACKING_COLOR)
  local outline = HollowCircle.new(radius, EggcornsProgressCircle.OUTLINE_COLOR)
  local halo = HollowCircle.new(haloRadius, EggcornsCommon.COLORS.OVEREXPOSED_AQUA, HALO_THICKNESS)
  local fill = Sprite.newWithName("filled_circle.png")
  self.fillFitScale = diameter / fill:getWidth()

  outline:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)
  backing:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)

  halo:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)
  fill:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)
  fill:setScl(FILL_INITIAL_SCALE * self.fillFitScale)

  self.originalFillOpacity = EggcornsCommon.INVISIBLE_OPACITY

  self:insertAtCenter(backing)
  self:insertAtCenter(outline)
  self:insertAtCenter(halo)
  self:insertAtCenter(fill)

  self.halo = halo
  self.fill = fill
  self.outline = outline
  self.backing = backing
end

function EggcornsProgressCircle.newHidden(diameter)
  local self = Group.new(diameter, diameter)
  Mixin.mix(self, EggcornsProgressCircle)

  createLayout(self, diameter)
  createIncrementAnimation(self)

  return self
end
