--- EggcornsPromptMorpher
-- @author Created by Jay Austria on August 20, 2015.

class("EggcornsPromptMorpher")

require("EggcornsCircularContainer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function EggcornsPromptMorpher:attachTextArea(textArea)
    self.attachedTextArea = textArea
end

function EggcornsPromptMorpher:setEndLocation(x, y)
  self.endLocationX = x
  self.endLocationY = y
end

function EggcornsPromptMorpher:createMorphAction()
  assert(self.attachedTextArea ~= nil, "Text area must be attached to the morpher to create the animations")
  assert((self.endLocationX ~= nil) and (self.endLocationY ~= nil), "End location must be specified to the morpher to create the animations")

  local circularContainer = self.circularContainer
  local textArea = self.attachedTextArea
  local promptSprite = self.promptSprite

  local promptAnimation = Animation.newWithPrefix("sphere_")

  local circularContainerAnimation = circularContainer:createMockSphereAnimation(textArea)
  local spriteAnimateAction = AHAnimate.new(promptSprite, promptAnimation, 30)
  local spriteFadeInAction = AHOpacityTo.new(promptSprite, 0.3, nil, 1)

  local showProgressBlip = function()
    local blip = FilledCircle.new(8, EggcornsCommon.COLORS.OVEREXPOSED_AQUA)

    if self.onShowBlipCallback then
      self.onShowBlipCallback()
    end
  end

  local morphAction = AHSpawn.new(
    AHLocationTo.new(self, 0.7, AHEaseExponentialIn.new(), self.endLocationX, self.endLocationY),
    AHDelayed.new(0.6, showProgressBlip),
    circularContainerAnimation,
    AHDelayed.new(0.2,
      AHSpawn.new(
        spriteAnimateAction,
        spriteFadeInAction
      )
    )
  )

  return morphAction
end

function EggcornsPromptMorpher:setOnShowBlipCallback(callback)
  self.onShowBlipCallback = callback
end

function EggcornsPromptMorpher:createAndPlayMorphAction()
  local morphAction = self:createMorphAction()

  return morphAction:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, promptSprite)
  self:insertAtCenter(promptSprite)
  self.promptSprite = promptSprite

  local circularContainer = EggcornsCircularContainer.new(promptSprite:getWidthPart(0.4))
  self:insertAtCenter(circularContainer)
  self.circularContainer = circularContainer
end

function EggcornsPromptMorpher.new()
  local promptSprite = Sprite.newWithName("sphere_000.png")
  promptSprite:setOpacity(0)

  local self = Group.new(promptSprite:getDims())
  Mixin.mix(self, EggcornsPromptMorpher)

  createLayout(self, promptSprite)

  return self
end
