--- EggcornsRationaleOverlay
-- @author Created by Jay Austria on September 1, 2015.

class("EggcornsRationaleOverlay")
require("text.SizeConstrainedTextArea")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TAP_TO_CONTINUE_LABEL_Y_MINIMUM = Screen.getHeightPart(0.126)
local SLIDE_OFFSET_Y = Screen.getHeightPart(0.1)
local SPACING_Y = Screen.getHeightPart(0.05)
local TAP_TO_CONTINUE_LABEL_VISIBLE_OPACITY = 0.5
local STRONG_TEXT_RED_COLOR = EggcornsCommon.COLORS.LIGHT_RED

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function EggcornsRationaleOverlay:prepareToDisplay(rationaleText, screenY)
  local tapToContinueY = math.max(TAP_TO_CONTINUE_LABEL_Y_MINIMUM, Screen.safeBottom + SPACING_Y) + SLIDE_OFFSET_Y
  self.tapToContinueLabel:setLoc(Screen.getSafeX(0.5), tapToContinueY)

  local baseAttributes = { font = Font.new(FontFaces.DIN_OT_MEDIUM, 20) }
  local attributesForLabels = { strong = {color = STRONG_TEXT_RED_COLOR} }
  local labelInitialY = screenY - SPACING_Y + SLIDE_OFFSET_Y
  local labelHeight = labelInitialY - tapToContinueY - SPACING_Y

  local rationaleLabel = SizeConstrainedTextArea.new(rationaleText,
                                                     Screen.getSafeWidthPart(0.8),
                                                     labelHeight,
                                                     baseAttributes,
                                                     attributesForLabels)
  rationaleLabel:setAnchorPointTop()
  rationaleLabel:setLoc(Screen.getSafeX(0.5), labelInitialY)
  rationaleLabel:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)
  self:insert(rationaleLabel)
  self.rationaleLabel = rationaleLabel
end

function EggcornsRationaleOverlay:createDisplayAction()
  local rationaleLabel = self.rationaleLabel
  local tapToContinueLabel = self.tapToContinueLabel
  local displayDuration = 0.5

  local fadeIn = AHSpawn.new(
    AHOpacityTo.new(rationaleLabel, displayDuration, nil, EggcornsCommon.VISIBLE_OPACITY),
    AHOpacityTo.new(tapToContinueLabel, displayDuration, nil, TAP_TO_CONTINUE_LABEL_VISIBLE_OPACITY)
  )
  local lowerAction = AHSpawn.new(
    AHLocationYBy.new(rationaleLabel, displayDuration, AHEaseSimpleOut.new(), -SLIDE_OFFSET_Y),
    AHLocationYBy.new(tapToContinueLabel, displayDuration, AHEaseSimpleOut.new(), -SLIDE_OFFSET_Y)
  )

  return AHSpawn.new(
    fadeIn,
    lowerAction
  )
end

function EggcornsRationaleOverlay:createDismissAction()
  local rationaleLabel = self.rationaleLabel
  local tapToContinueLabel = self.tapToContinueLabel
  local displayDuration = 0.3

  local fadeOut = AHSpawn.new(
    AHOpacityTo.new(rationaleLabel, displayDuration, nil, EggcornsCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(tapToContinueLabel, displayDuration, nil, EggcornsCommon.INVISIBLE_OPACITY)
  )
  local dropAction = AHSpawn.new(
    AHLocationYBy.new(rationaleLabel, 0.6, AHEaseExponentialIn.new(), -self:getHeightPart(0.5)),
    AHLocationYBy.new(tapToContinueLabel, 0.6, AHEaseExponentialIn.new(), -self:getHeightPart(0.5))
  )

  local dismissAction = AHSequence.new(
    AHSpawn.new(
      fadeOut,
      dropAction),
    AHRemove.new(rationaleLabel)
  )

  return dismissAction
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  tapToContinueLabel:setOpacity(EggcornsCommon.INVISIBLE_OPACITY)
  self:insert(tapToContinueLabel)
  self.tapToContinueLabel = tapToContinueLabel
end

function EggcornsRationaleOverlay.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, EggcornsRationaleOverlay)

  createLayout(self)

  self.rationaleLabel = nil

  return self
end
