--- EggcornsRoundGenerator
-- @author Created by Jay Austria on August 20, 2015.

class("EggcornsRoundGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BLANK_TAG = "<blank/>"
local LEFT_MARGIN_SINGLE_TAG = "<leftMargin/>"
local RIGHT_MARGIN_SINGLE_TAG = "<rightMargin/>"

local MAX_UNIQUE_TASK_GENERATION_ATTEMPTS = 10

local HIGHLIGHT_START_TAG = "<replacement>"
local HIGHLIGHT_END_TAG = "</replacement>"
if EggcornsConfiguration.willHighlightPhrases() then
  HIGHLIGHT_START_TAG = "<replacementBlack>"
  HIGHLIGHT_END_TAG = "</replacementBlack>"
end

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getConceptWithUniqueExpression(self)
  local concept = nil
  local attemptIndex = 0
  local tooManyAttempts = false
  local hasSeenExpression = false

  repeat
    concept = self.conceptGenerator:next()
    attemptIndex = attemptIndex + 1
    tooManyAttempts = (attemptIndex >= MAX_UNIQUE_TASK_GENERATION_ATTEMPTS)
    hasSeenExpression = (self.seenExpressions[concept.reportKey])
  until not hasSeenExpression or tooManyAttempts

  self.seenExpressions[concept.reportKey] = true

  return concept
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function EggcornsRoundGenerator:getFormattedPromptForConcept(concept, isCorrect)
  local replacementText = nil

  if (isCorrect) then
    replacementText = concept.formattedReplacementTextCorrect
  else
    replacementText = concept.formattedReplacementTextIncorrect
  end

  return concept.prompt:gsub(BLANK_TAG, replacementText)
end

function EggcornsRoundGenerator:reset()
  self.conceptGenerator:reset()
end

function EggcornsRoundGenerator:generateNextValue()
  local concept = getConceptWithUniqueExpression(self)
  local correctedReplacementText = concept.correctReplace

  local formattedReplacementTextCorrect = HIGHLIGHT_START_TAG .. concept.correctReplace .. HIGHLIGHT_END_TAG
  local formattedReplacementTextIncorrect = HIGHLIGHT_START_TAG .. concept.incorrectReplace .. HIGHLIGHT_END_TAG

  if EggcornsConfiguration.willHighlightPhrases() then
    -- Include margins to compensate for the width of the highlights
    formattedReplacementTextCorrect = LEFT_MARGIN_SINGLE_TAG .. formattedReplacementTextCorrect .. RIGHT_MARGIN_SINGLE_TAG
    formattedReplacementTextIncorrect = LEFT_MARGIN_SINGLE_TAG .. formattedReplacementTextIncorrect .. RIGHT_MARGIN_SINGLE_TAG
  end

  local round =
  {
    prompt = concept.prompt,
    rationale = concept.rationale,
    identifier = concept.identifier,
    correctedReplacementText = correctedReplacementText,
    formattedReplacementTextCorrect = formattedReplacementTextCorrect,
    formattedReplacementTextIncorrect = formattedReplacementTextIncorrect
  }

  return round
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function EggcornsRoundGenerator.new()
  local self = Generator.new()
  Mixin.mix(self, EggcornsRoundGenerator)

  self.conceptGenerator = GameSession.conceptGenerator
  self.seenExpressions = {}

  return self
end
