--- ExerciseCardArrow
-- Adds a bouncing arrow overlay to a card.
-- @author Created by karl on May 15, 2015.

class("ExerciseCardArrow")

require("containers.card_carousel.CardCarouselCardCommon")

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getArrowStartY(arrow, useDefaultStartY)
  local startY = arrow:getHeightPart(2)

  if (Screen.hasNotch()) then
    -- high position (for keeping arrow above text in speed_reading_diagnostic)
    startY = Screen.getHeightPart(0.136)
    if (useDefaultStartY) then
      -- low position (for public_speaking_protips + speed_reading_protips)
      startY = Screen.getHeightPart(0.1)
    end
  end

  return startY
end

local function newArrowAnimation(arrow)
  local bounceHeight = arrow:getHeight()
  return AHLoop.new(AHSequence.new(
    AHDelay.new(2),
    AHLocationYBy.new(arrow, 1.0, AHEaseExponentialIn.new(), bounceHeight),
    AHLocationYBy.new(arrow, 0.5, AHEaseExponentialOut.new(), -bounceHeight)
  ))
end

local function wrapMethod(object, methodName, outerFunction)
  local existingFunction = object[methodName] or function() end
  object[methodName] = function(self, ...)
    return outerFunction(object, existingFunction, ...)
  end
end

local function addTouchNavigationToArrow(arrow, card)
  local onTouchDown = function()
    local willCheckNavigationConstraints = true
    card:forceTapNavigation(CardCarouselNavigator.TAP_DIRECTION_DOWN,
                            CardCarouselNavigator.SCROLL_ANIMATION_DURATION_VERTICAL,
                            willCheckNavigationConstraints)
  end
  local onTouchUp = nil
  local onTouchCancel = nil
  local tapAreaWidth = 4 * math.max(arrow:getWidth(), arrow:getHeight())
  local virtualButton = Button.addVirtualButton(arrow, tapAreaWidth, tapAreaWidth, onTouchDown, onTouchUp, onTouchCancel)

  wrapMethod(card, "onEnter", function(self, existingFunction, ...)
    virtualButton:setTouchEnabled(true)
    existingFunction(self, ...)
  end)

  wrapMethod(card, "onLeaveBegin", function(self, existingFunction, ...)
    virtualButton:setTouchEnabled(false)
    existingFunction(self, ...)
  end)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newArrow()
  local sprite = Sprite.newWithName("arrow.png")
  sprite:setColor(unpack(CardCarouselCardCommon.COLORS.GREY))
  return sprite
end

-- Constructor ------------------------

function ExerciseCardArrow.attachToCard(card)
  local arrow = newArrow()
  card:insert(arrow, math.huge)

  card.navigationArrow = arrow

  local cardType = card:getCardType()
  local useDefaultStartY = (cardType == ExerciseCardDefinitions.CARD_TYPES.QUOTE)

  wrapMethod(card, "reset", function(self, existingFunction, ...)
    arrow:setLoc(self:getWidthPart(0.5), getArrowStartY(arrow, useDefaultStartY))
    existingFunction(self, ...)
  end)

  wrapMethod(card, "createAnimationAction", function(self, existingFunction, ...)
    return AHSpawn.new(newArrowAnimation(arrow), existingFunction(self, ...))
  end)

  addTouchNavigationToArrow(arrow, card)
end
