--- ExerciseCardBuilder
-- @author Created by Aaron Barrett on May 06, 2015.

class("ExerciseCardBuilder")

require("ExerciseCardConstructors")
require("ExerciseCardArrow")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createCard(cardGlobals, cardData)
  local cardType = cardData.cardType

  local constructor = ExerciseCardConstructors.CONSTRUCTORS_BY_CARD_TYPE[cardType]
  assert(constructor ~= nil, "No constructor defined for card type: `" .. tostring(cardType) .. "`")
  local card = constructor(cardType, cardGlobals, cardData)
  return card
end

local function newCardColumn(cardGlobals, columnOfCardData)
  local columnOfCards = {}
  for index, cardData in ipairs(columnOfCardData) do
    local card = createCard(cardGlobals, cardData)
    if index < #columnOfCardData and index > 1 then
      ExerciseCardArrow.attachToCard(card)
    end
    table.insert(columnOfCards, card)
  end

  return columnOfCards
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardBuilder.createInitialArrangedCards(exerciseData, cardGlobals)
  local arrangedCards = {}

  for i = 1, 2 do
    local columnOfCardData = exerciseData[i]
    if columnOfCardData then
      table.insert(arrangedCards, newCardColumn(cardGlobals, columnOfCardData))
    end
  end

  return arrangedCards
end

function ExerciseCardBuilder.createRemainingArrangedCards(exerciseData, cardGlobals, arrangedCards)
  for i = 3, #exerciseData do
    local columnOfCardData = exerciseData[i]
    if columnOfCardData then
      table.insert(arrangedCards, newCardColumn(cardGlobals, columnOfCardData))
      coroutine.yield()
    end
  end
end

function ExerciseCardBuilder.createAllArrangedCards(exerciseData, cardGlobals)
  local arrangedCards = {}

  for _, columnOfCardData in ipairs(exerciseData) do
    table.insert(arrangedCards, newCardColumn(cardGlobals, columnOfCardData))
  end

  return arrangedCards
end
