--- ExerciseCardDefinitions
-- @author Created by Aaron Barrett on May 04, 2015.

class("ExerciseCardDefinitions")

require("containers.card_carousel.CardCarouselCardDefinitions")
require("containers.card_carousel.CardCarouselFieldMapping")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ExerciseCardDefinitions.CARD_TYPES = TableUtils.unionSet(CardCarouselCardDefinitions.CARD_TYPES, {
  DONE = "done",
  EXAMPLE_AND_TEXT = "example and text",
  EYE_MOVEMENT = "eye movement practice",
  FLASH_CARD = "flash card",
  IMAGE_AND_CAPTION = "image and caption",
  IMAGE_AND_TEXT = "image and text",
  IMAGE_EXPLANATION = "image explanation",
  INSTRUCTION_AND_DETAIL = "instruction and detail",
  INTRO_TEXT = "intro text",
  MULTIPLE_CHOICE = "multiple choice",
  PASSAGE = "passage",
  PASSAGE_TITLE = "passage title",
  QUOTE = "quote",
  SPEECH_ANALYSIS = "speech analysis",
  SPEECH_INTRO = "speech intro",
  SPEED_READING_SENTENCE_GROUPING = "sentence grouping",
  SPEED_READING_TEST_COMPLETE = "speed test complete",
  TEXT = "text",
  TEXT_AND_INSTRUCTIONS = "text and instructions",
  TIP_EXPLANATION = "tip explanation",
  TITLE = "title",
  TITLE_SUBTITLE = "title and subtitle",
  WORD_HIGHLIGHT = "word highlight",
  WORD_PART_EXAMPLE = "word part example",
  WORD_PART_EXPLANATION = "word part explanation",
})

ExerciseCardDefinitions.FIELD_MAPPING_BY_CARD_TYPE = TableUtils.unionSet(CardCarouselCardDefinitions.FIELD_MAPPING_BY_CARD_TYPE, {
  [ExerciseCardDefinitions.CARD_TYPES.DONE] = CardCarouselFieldMapping.new(),

  [ExerciseCardDefinitions.CARD_TYPES.EXAMPLE_AND_TEXT] = CardCarouselFieldMapping.new(
    {"title", "data_field_1", CardCarouselFieldMapping.OPTIONAL},
    {"text", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML},
    {"image", "image_link_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING, CardCarouselFieldMapping.IMAGE}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.EYE_MOVEMENT] = CardCarouselFieldMapping.new(
    {"text", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.FLASH_CARD] = CardCarouselFieldMapping.new(
    {"image", "image_link_1", CardCarouselFieldMapping.OPTIONAL, CardCarouselFieldMapping.IMAGE},
    {"word", "data_field_1", CardCarouselFieldMapping.REQUIRED},
    {"wordDefinition", "data_field_2", CardCarouselFieldMapping.OPTIONAL},
    {"example1", "data_field_3", CardCarouselFieldMapping.OPTIONAL},
    {"exampleDefinition1", "data_field_4", CardCarouselFieldMapping.OPTIONAL},
    {"example2", "data_field_5", CardCarouselFieldMapping.OPTIONAL},
    {"exampleDefinition2", "data_field_6", CardCarouselFieldMapping.OPTIONAL}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.IMAGE_AND_CAPTION] = CardCarouselFieldMapping.new(
    {"image", "image_link_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.IMAGE},
    {"title", "data_field_1", CardCarouselFieldMapping.REQUIRED},
    {"text", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.IMAGE_AND_TEXT] = CardCarouselFieldMapping.new(
    {"image1", "image_link_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.IMAGE},
    {"title", "data_field_1", CardCarouselFieldMapping.OPTIONAL},
    {"image2", "image_link_2", CardCarouselFieldMapping.OPTIONAL, CardCarouselFieldMapping.IMAGE},
    {"text", "data_field_2", CardCarouselFieldMapping.OPTIONAL, CardCarouselFieldMapping.XML}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.IMAGE_EXPLANATION] = CardCarouselFieldMapping.new(
    {"image", "image_link_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.IMAGE},
    {"overallDescription", "data_field_1", CardCarouselFieldMapping.REQUIRED},
    {"location1", "data_field_2", CardCarouselFieldMapping.REQUIRED},
    {"description1", "data_field_3", CardCarouselFieldMapping.REQUIRED},
    {"examples1", "data_field_4", CardCarouselFieldMapping.REQUIRED},
    {"location2", "data_field_5", CardCarouselFieldMapping.REQUIRED},
    {"description2", "data_field_6", CardCarouselFieldMapping.REQUIRED},
    {"examples2", "data_field_7", CardCarouselFieldMapping.REQUIRED},
    {"location3", "data_field_8", CardCarouselFieldMapping.OPTIONAL},
    {"description3", "data_field_9", CardCarouselFieldMapping.OPTIONAL},
    {"examples3", "data_field_10", CardCarouselFieldMapping.OPTIONAL}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.INSTRUCTION_AND_DETAIL] = CardCarouselFieldMapping.new(
    {"instruction", "data_field_1", CardCarouselFieldMapping.REQUIRED},
    {"text", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.INTRO_TEXT] = CardCarouselFieldMapping.new(
    {"title", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"text", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING, CardCarouselFieldMapping.XML}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.MULTIPLE_CHOICE] = CardCarouselFieldMapping.new(
    {"prompt", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"correctAnswer", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"distractor1", "data_field_3", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"distractor2", "data_field_4", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"distractor3", "data_field_5", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.PASSAGE] = CardCarouselFieldMapping.new(
    {"text", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.PASSAGE_TITLE] = CardCarouselFieldMapping.new(
    {"title", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"subtitle", "data_field_2", CardCarouselFieldMapping.OPTIONAL, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.QUOTE] = CardCarouselFieldMapping.new(
    {"image", "image_link_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.IMAGE},
    {"name", "data_field_1", CardCarouselFieldMapping.REQUIRED},
    {"famousFor", "data_field_2", CardCarouselFieldMapping.REQUIRED},
    {"quote", "data_field_3", CardCarouselFieldMapping.REQUIRED}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.SPEECH_ANALYSIS] = CardCarouselFieldMapping.new(
    {"audio", "audio_link_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.AUDIO, CardCarouselFieldMapping.REPEATING},
    {"duration", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"theme", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"analysis", "data_field_3", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.SPEECH_INTRO] = CardCarouselFieldMapping.new(
    {"speaker", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"context", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"image", "image_link_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.IMAGE, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.SPEED_READING_SENTENCE_GROUPING] = CardCarouselFieldMapping.new(
    {"text", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.SPEED_READING_TEST_COMPLETE] = CardCarouselFieldMapping.new(),

  [ExerciseCardDefinitions.CARD_TYPES.TEXT] = CardCarouselFieldMapping.new(
    {"text", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.TEXT_AND_INSTRUCTIONS] = CardCarouselFieldMapping.new(
    {"topInstructions", "data_field_1", CardCarouselFieldMapping.OPTIONAL},
    {"text", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML},
    {"bottomInstructions", "data_field_3", CardCarouselFieldMapping.OPTIONAL, CardCarouselFieldMapping.XML}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.TIP_EXPLANATION] = CardCarouselFieldMapping.new(
    {"image", "image_link_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING, CardCarouselFieldMapping.IMAGE},
    {"location", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"description", "data_field_2", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING},
    {"tip", "data_field_3", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.TITLE] = CardCarouselFieldMapping.new(),

  [ExerciseCardDefinitions.CARD_TYPES.TITLE_SUBTITLE] = CardCarouselFieldMapping.new(
    {"title", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML, CardCarouselFieldMapping.REPEATING},
    {"subtitle", "data_field_2", CardCarouselFieldMapping.OPTIONAL, CardCarouselFieldMapping.REPEATING}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.WORD_HIGHLIGHT] = CardCarouselFieldMapping.new(
    {"text", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML},
    {"hint", "data_field_2", CardCarouselFieldMapping.OPTIONAL}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.WORD_PART_EXAMPLE] = CardCarouselFieldMapping.new(
    {"partName", "data_field_1", CardCarouselFieldMapping.REQUIRED},
    {"part", "data_field_2", CardCarouselFieldMapping.REQUIRED},
    {"meaning", "data_field_3", CardCarouselFieldMapping.REQUIRED},
    {"examples", "data_field_4", CardCarouselFieldMapping.REQUIRED}
  ),

  [ExerciseCardDefinitions.CARD_TYPES.WORD_PART_EXPLANATION] = CardCarouselFieldMapping.new(
    {"word", "data_field_1", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML},
    {"partDescription", "data_field_2", CardCarouselFieldMapping.REQUIRED},
    {"explanation", "data_field_3", CardCarouselFieldMapping.REQUIRED, CardCarouselFieldMapping.XML}
  ),

})
