--- ExerciseCardLoaderOverlay
-- @author Created by karl on July 01, 2015.

-- ============================================================================
local DotAnimator = privateClass()
-- ============================================================================

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DotAnimator:newAnimation()
  local fadeDuration = 0.5
  local spawn = AHStaggeredSpawn.new(0.2)
  for _, dot in ipairs(self.children) do
    spawn:addAction(AHSequence.new(
      AHOpacityTo.new(dot, fadeDuration, nil, 1),
      AHOpacityTo.new(dot, fadeDuration, nil, 0),
      AHDelay.new(0.2)
    ))
  end

  return spawn
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newDot()
  local radius = Screen.getWidthPart(0.02)
  return FilledCircle.new(radius, ColorUtils.colorWithOpacity(CardCarouselCardCommon.COLORS.BLUE, 0))
end

-- Constructor ------------------------

function DotAnimator.new()
  local spacing = Screen.getWidthPart(0.04)
  local self = Row.new(spacing)
  Mixin.mix(self, DotAnimator)
  for i = 1, 3 do
    self:insert(newDot())
  end
  self:alignChildren()
  return self
end



-- ============================================================================
class("ExerciseCardLoaderOverlay")
-- ============================================================================

require("containers.card_carousel.CardCarouselCardCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardLoaderOverlay:start()
  if not self.action then
    self.action = AHSequence.new(
      AHLoop.new(self.dots:newAnimation(), self.cycles),
      self.onCompletion
    ):start()
  end
end

function ExerciseCardLoaderOverlay:stop()
  if self.action then
    self.action:stop()
    self.action = nil
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardLoaderOverlay.new(title, description, cycles, onCompletion)
  local self = FilledRectangle.newFullScreen(CardCarouselCardCommon.COLORS.WHITE)
  Mixin.mix(self, ExerciseCardLoaderOverlay)

  self.cycles = cycles
  self.onCompletion = onCompletion

  local titleFontSize = 21
  local descriptionFontSize = 17

  self.dots = DotAnimator.new()
  local loaderColumn = Column.new(self:getWidthPart(0.07))
  loaderColumn:insert(self.dots)
  loaderColumn:insert(CardCarouselCardCommon.newLabel(title, FontFaces.DIN_OT_BOLD, titleFontSize, CardCarouselCardCommon.COLORS.BLUE))
  loaderColumn:alignChildren()

  self:insertAtCenter(loaderColumn)

  if description then
    local descriptionLabel = CardCarouselCardCommon.newMultilineLabel(description,
                                FontFaces.DIN_OT_MEDIUM, descriptionFontSize, self:getWidthPart(0.8),
                                Alignment.CENTER, CardCarouselCardCommon.COLORS.LIGHT_GREY)
    descriptionLabel:setAnchorPointBottom()
    descriptionLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.075))
    self:insert(descriptionLabel)
  end

  self:linkColorAttrToChildren(true)

  return self
end
