--- ExerciseGame
-- @author Created by karl on April 02, 2015.

class("ExerciseGame")

require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselNavigator")
require("containers.card_carousel.CardCarouselPreloadedCards")
require("ExerciseCardBuilder")
require("ExerciseGenerator")
require("ExerciseNavigator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BASE_SCORE = 100

local Z_BACKGROUND = 10
local Z_NAVIGATOR = 20

local CLEAR_COLOR = CardCarouselCardCommon.COLORS.BLUE

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function onExit(self)
  self.navigator:setTouchEnabled(false)
  runInCoroutine(function()

    -- Wait three extra frames to fill the screen and the framebuffer with the same image
    -- and prevent the app from flickering.
    waitForFrames(3)

    if (self.navigator:hasVisitedDoneCard()) then
      GameSession.setScore(GameSession.getPassScore())
      GameSession.notifyGameWin()
    else
      GameSession.notifyGameLoss()
    end
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseGame:gameFlow()
  local arrangedCards = CardCarouselPreloadedCards.getCards()
  local navigator = ExerciseNavigator.new(arrangedCards, function() onExit(self) end, CLEAR_COLOR)
  self:insertAtCenter(navigator, Z_NAVIGATOR)
  self.navigator = navigator

  navigator:onAppear()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseGame.new()
  local self = BaseGame.new()
  Mixin.mix(self, ExerciseGame)

  local background = FilledRectangle.newFullScreen(CLEAR_COLOR)
  self:insertAtCenter(background, Z_BACKGROUND)

  GameSession.setPassScore(BASE_SCORE)
  GameSession.setScoreThresholds(BASE_SCORE * 2, BASE_SCORE * 3)

  local cardIndexProportion = math.random()
  local exerciseGenerator = ExerciseGenerator.new(cardIndexProportion)
  local arrangedCardData = exerciseGenerator:next()
  local cardGlobals = {closeColor = CLEAR_COLOR}
  CardCarouselPreloadedCards.setCards(ExerciseCardBuilder.createAllArrangedCards(arrangedCardData, cardGlobals))

  return self
end
