--- ExerciseGenerator
-- @author Created by karl on April 09, 2015.

class("ExerciseGenerator")

require("data.generators.UtilityGenerators")

require("ExerciseCardDefinitions")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SKIP_CARD_CONTENTS = "[DO NOT SHOW CARD]"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getIndexedCardValue(values, cardIndexProportion)
  local highValue = #values
  local lowValue = 1
  local range = highValue - lowValue + 1
  local index = lowValue + math.floor(range * cardIndexProportion)
  if index > highValue then
    index = highValue
  end
  return values[index]
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createCardData(cardFromConcept, concept, cardIndexProportion)
  local cardType = cardFromConcept["card_type"]

  local fieldMapping = ExerciseCardDefinitions.FIELD_MAPPING_BY_CARD_TYPE[cardType]
  assert(fieldMapping ~= nil, "No field mapping info defined for card type: `" .. tostring(cardType) .. "`")

  local cardData = {}
  cardData.skippedCards = {}
  for _, key in ipairs(fieldMapping:getAllKeys()) do
    local values = fieldMapping:getValues(cardFromConcept, key)
    local fieldIndexProportion = cardIndexProportion
    if not fieldMapping:isArray(key) then
      fieldIndexProportion = 0
    end
    local value = getIndexedCardValue(values, fieldIndexProportion)
    if value == SKIP_CARD_CONTENTS then
      return nil
    end

    cardData[key] = value
  end

  cardData.concept = concept

  return cardData
end

local function newCardDefinition(cardType, data)
  data = data or {}
  local finalCardData = TableUtils.shallowCopy(data)
  finalCardData.cardType = cardType
  return finalCardData
end

local function newCardDefinitionInColumn(cardType, data)
  return {newCardDefinition(cardType, data)}
end

local function addTitleCardColumn(arrangedCardData, concept)
  local cardData = {
    text = concept.title,
    description = concept.description,
    icon = concept.icon,
    concept = concept
  }
  table.insert(arrangedCardData, 1, newCardDefinitionInColumn(ExerciseCardDefinitions.CARD_TYPES.TITLE, cardData))
end

local function addDoneCardColumn(arrangedCardData, concept)
  local cardData = {
    icon = concept.icon,
    concept = concept
  }
  table.insert(arrangedCardData, newCardDefinitionInColumn(ExerciseCardDefinitions.CARD_TYPES.DONE, cardData))
end

local function addExitCards(arrangedCardData)
  -- Front exit card
  table.insert(arrangedCardData, 1, newCardDefinitionInColumn(ExerciseCardDefinitions.CARD_TYPES.EXIT, cardData))

  -- Back exit card
  table.insert(arrangedCardData, newCardDefinitionInColumn(ExerciseCardDefinitions.CARD_TYPES.EXIT))

  -- Top row exit cards
  for _, column in ipairs(arrangedCardData) do
    table.insert(column, 1, newCardDefinition(ExerciseCardDefinitions.CARD_TYPES.PULL_TO_EXIT))
  end
end

-- Constructor ------------------------

function ExerciseGenerator.new(cardIndexProportion)
  local self = TransformationGenerator.new(GameSession.conceptGenerator, function(concept)
    local arrangedCardData = {}

    addTitleCardColumn(arrangedCardData, concept)

    local cardsFromConcept = concept["cards"]
    for _, cardFromConcept in ipairs(cardsFromConcept) do
      local rowIndex = tonumber(cardFromConcept["file"])

      local columnIndexOffsetBecauseOfTitleCard = 1
      local columnIndex = tonumber(cardFromConcept["rank"]) + columnIndexOffsetBecauseOfTitleCard

      local column = arrangedCardData[columnIndex] or {}
      arrangedCardData[columnIndex] = column

      column[rowIndex] = createCardData(cardFromConcept, concept, cardIndexProportion)
    end

    addDoneCardColumn(arrangedCardData, concept)
    addExitCards(arrangedCardData)

    return arrangedCardData
  end)
  Mixin.mix(self, ExerciseGenerator)

  return self
end
