--- ExerciseMultipleChoiceButton
-- @author Created by Jay Austria on April 20, 2015.

class("ExerciseMultipleChoiceButton")

require("containers.card_carousel.CardCarouselCardCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_BUTTON_COUNT = 4
local BUTTON_CORNER_RADIUS = 3
local BUTTON_LAYER_COLORS =
{
  ColorUtils.colorRGBToStd(0, 168, 217),
  ColorUtils.colorRGBToStd(0, 191, 232),
  ColorUtils.colorRGBToStd(0, 204, 239),
  ColorUtils.colorRGBToStd(0, 211, 240)
}

local PRESSED_COLOR = {0.32, 0.64, 0.75}

ExerciseMultipleChoiceButton.BUTTON_AREA_HEIGHT = Screen.getHeightPart(0.5)
ExerciseMultipleChoiceButton.BUTTON_MARGIN_Y = Screen.getHeightPart(0.015)
ExerciseMultipleChoiceButton.BOTTOM_MARGIN_Y = Screen.getHeightPart(0.03)
ExerciseMultipleChoiceButton.BUTTON_HEIGHT = (ExerciseMultipleChoiceButton.BUTTON_AREA_HEIGHT - (ExerciseMultipleChoiceButton.BUTTON_MARGIN_Y * (MAX_BUTTON_COUNT - 1)) - ExerciseMultipleChoiceButton.BOTTOM_MARGIN_Y) / MAX_BUTTON_COUNT

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function fanOpenLayers(self, directionParity)
  local layers = self.layers
  local fanAmount = Screen.getWidthPart(0.1)
  for layerIndex, layer in ipairs(layers) do
    local offsetX = (layerIndex - 1) * fanAmount * directionParity
    layer:setX(self:getWidthPart(0.5) + offsetX)
  end
end

local function getFanCloseAnimation(self)
  local layers = self.layers
  local topLayer = self.topLayer
  local closeSpawn = AHSpawn.new()
  for layerIndex, layer in ipairs(layers) do
    local delayDuration = (layerIndex - 1) * 0.03 + 0.1
    local closeAction = AHDelayed.new(delayDuration, AHLocationXTo.new(layer, 0.4, AHEaseSimpleInOut.new(), self:getWidthPart(0.5)))
    closeSpawn:addAction(closeAction)
  end

  return closeSpawn
end

local function createLayout(self, text)
  local radius = BUTTON_CORNER_RADIUS
  local width, height = self:getDims()
  local layers = {}

  for _, layerColor in ipairs(BUTTON_LAYER_COLORS) do
    local buttonLayer = FilledRoundedRectangle.new(width, height, radius, layerColor)
    self:insertAtCenter(buttonLayer)

    table.insert(layers, buttonLayer)
  end

  local buttonLabel = Label.new(text, PrerenderedFont.new(FontFaces.DIN_OT_MEDIUM, 19))
  local topLayer = layers[#layers]
  topLayer:insertAtCenter(buttonLabel)

  Button.makeButton(self, function()
    if self.callback then
      self.callback(text)
    end
  end)

  self.layers = layers
  self.topLayer = topLayer
  self.text = text
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseMultipleChoiceButton:prepareForEnterAnimation(directionParity)
  fanOpenLayers(self, directionParity)
  self:setX(self.originX + Screen.width * directionParity)
end

function ExerciseMultipleChoiceButton:getEnterAnimation()
  local closeAnimation = getFanCloseAnimation(self)
  local moveAnimation = AHLocationXTo.new(self, 0.4, AHEaseSimpleInOut.new(), self.originX)

  local enterAnimation = AHSpawn.new(closeAnimation, moveAnimation)
  return enterAnimation
end

function ExerciseMultipleChoiceButton:playPressed()
  local topLayer = self.topLayer
  local pressedAction = AHSpawn.new(
                                    AHColorTo.new(topLayer, 0.1, nil, PRESSED_COLOR),
                                    AHSequence.new(
                                                   AHScaleTo.new(self, 0.1, AHEaseSimpleIn.new(2.0), 0.97),
                                                   AHScaleTo.new(self, 0.1, AHEaseBackOut.new(1.2), 1.0)
                                                   )
                                    )
  return pressedAction:start()
end

function ExerciseMultipleChoiceButton:setOriginX(x)
  self.originX = x
end

function ExerciseMultipleChoiceButton:setButtonCallback(callback)
  self.callback = callback
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function ExerciseMultipleChoiceButton.new(text)
  local width = CardCarouselCardCommon.CONTENT_WIDTH
  local height = ExerciseMultipleChoiceButton.BUTTON_HEIGHT

  local self = Group.new(width, height)
  Mixin.mix(self, ExerciseMultipleChoiceButton)

  createLayout(self, text)

  return self
end
