--- ExercisePlainButton
-- @author Created by Aaron Barrett on May 11, 2015.

class("ExercisePlainButton")

require("containers.card_carousel.CardCarouselCardCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.width - CardCarouselCardCommon.MARGINS.HORIZONTAL * 2.0
local HEIGHT = Screen.getHeightPart(0.11)
local CORNER_RADIUS = 3.0

local COLOR_NORMAL = {0.0, 0.83, 0.94}
local COLOR_PRESSED = {0.32, 0.64, 0.75}

local Z_BACKGROUND = 10
local Z_LABEL = 20

local TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.WHITE)
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExercisePlainButton:playPressed()
  local background = self.background
  local pressedAction = AHSpawn.new(
                                    AHColorTo.new(background, 0.1, nil, COLOR_PRESSED),
                                    AHSequence.new(
                                                   AHScaleTo.new(self, 0.1, AHEaseSimpleIn.new(2.0), 0.97),
                                                   AHScaleTo.new(self, 0.1, AHEaseBackOut.new(1.2), 1.0)
                                                   )
                                    )
  return pressedAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExercisePlainButton.new(text, onPressedCallback)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, ExercisePlainButton)

  local background = FilledRoundedRectangle.new(WIDTH, HEIGHT, CORNER_RADIUS, COLOR_NORMAL)
  self:insertAtCenter(background, Z_BACKGROUND)
  self.background = background

  local label = CardCarouselTextView.new(text, WIDTH, TEXT_ATTRIBUTES, Alignment.CENTER)
  self:insertAtCenter(label, Z_LABEL)

  Button.makeButton(self, function()
    self:setTouchEnabled(false)
    self:playPressed()
    onPressedCallback()
  end)

  self:setTouchEnabled(false)

  return self
end
