--- ExerciseTutorial
-- @author Created by Aaron Barrett on May 06, 2015.

class("ExerciseTutorial")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newSlider(labelText, imageFilename, imageScaleX, imageScaleY)
  local columnSpacing = Screen.getHeightPart(0.02)
  local slider = Column.new(columnSpacing)

  local image = Sprite.newWithName(imageFilename)
  image:inheritOpacity(slider)
  image:setScl(imageScaleX or 1, imageScaleY or 1)
  local imageContainer = Group.new(image:getDims())
  imageContainer:insertAtCenter(image)
  slider:insert(imageContainer)

  local label = CardCarouselCardCommon.newCachedFontLabel(labelText,
                                                          CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM,
                                                          CardCarouselCardCommon.COLORS.DARK_GREY)
  label:inheritOpacity(slider)
  slider:insert(label)

  slider:alignChildren()

  return slider
end

local function animateOut(self)
  return AHOpacityTo.new(self, 0.4, nil, 0):start()
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseTutorial:addLeftRightSlider(labelText, imageFilename, imageScaleX, imageScaleY)
  local slider = newSlider(labelText, imageFilename, imageScaleX, imageScaleY)
  slider:setColor(unpack(CardCarouselCardCommon.COLORS.BLUE))
  self:insert(slider)
  table.insert(self.leftRightSliders, slider)
  return slider
end

function ExerciseTutorial:addTopBottomSlider(labelText, imageFilename, imageScaleX, imageScaleY)
  local slider = newSlider(labelText, imageFilename, imageScaleX, imageScaleY)
  slider:setColor(unpack(CardCarouselCardCommon.COLORS.BLUE))
  self:insert(slider)
  table.insert(self.topBottomSliders, slider)
  return slider
end

function ExerciseTutorial:animateIn()
  local action = AHSequence.new()
  action:addAction(function()
    TouchManager.addTouchHandler(self, function(event)
      action:stop()
      TouchManager.removeTouchHandler(self)
      runInCoroutine(function()
        wait(animateOut(self))
        self:removeSelf()
      end)
    end, TouchHandler.BEHAVIOR_FOCUS)
  end)

  local nextAction = AHSpawn.new()
  local destinations = {}
  for _, group in ipairs(self.leftRightSliders) do
    local x, y = group:getLoc()
    destinations[group] = {x = x, y = y}
    nextAction:addAction(function() group:setX(x + Screen.width) end)
  end
  for _, group in ipairs(self.topBottomSliders) do
    local x, y = group:getLoc()
    destinations[group] = {x = x, y = y}
    nextAction:addAction(function() group:setY(y - Screen.height) end)
  end
  action:addAction(nextAction)

  local nextAction = AHSpawn.new(AHOpacityTo.new(self, 0.45, nil, 0.9))
  local moveDuration = 0.9
  local moveEase = AHEaseExponentialOut.new()

  for _, group in ipairs(self.leftRightSliders) do
    local location = destinations[group]
    nextAction:addAction(AHLocationXTo.new(group, moveDuration, moveEase, location.x))
  end
  for _, group in ipairs(self.topBottomSliders) do
    local location = destinations[group]
    nextAction:addAction(AHLocationYTo.new(group, moveDuration, moveEase, location.y))
  end
  action:addAction(nextAction)

  return action:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newTutorial()
  local self = FilledRectangle.new(Screen.width, Screen.height, CardCarouselCardCommon.COLORS.WHITE_COLOR)
  Mixin.mix(self, ExerciseTutorial)

  self:setOpacity(0.0)

  self.leftRightSliders = {}
  self.topBottomSliders = {}

  return self
end

-- Constructor ------------------------

function ExerciseTutorial.newIntro()
  local self = newTutorial()

  local iconScale = 1.3
  local topSlider = self:addLeftRightSlider("TAP TO CONTINUE", "tap_to_continue.png", iconScale, iconScale)
  topSlider:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.7))

  local bottomSlider = self:addTopBottomSlider("SWIPE UP TO READ MORE", "pull_up_to_read_more.png")
  bottomSlider:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.3))

  self:linkColorAttrToChildren(true)

  return self
end

function ExerciseTutorial.newOutro()
  local self = newTutorial()

  local bottomSlider = self:addTopBottomSlider("SWIPE DOWN TO EXIT", "pull_up_to_read_more.png", 1, -1)
  bottomSlider:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))

  self:linkColorAttrToChildren(true)

  return self
end
