--- ExerciseZoomCircle
-- @author Created by karl on May 04, 2015.

-- ============================================================================
class("ExerciseZoomCircle")
-- ============================================================================


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_CIRCLE_BLUE = CardCarouselCardCommon.COLORS.BLUE
local COLOR_CIRCLE_GREY = {0.98, 0.98, 0.98}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newWobbleXAction(target, duration, distance)
  return AHLoop.new(AHSequence.new(
    AHLocationXBy.new(target, duration, AHEaseSineInOut.new(), distance),
    AHLocationXBy.new(target, duration, AHEaseSineInOut.new(), -distance)
  ))
end

local function newWobbleYAction(target, duration, distance)
  return AHLoop.new(AHSequence.new(
    AHLocationYBy.new(target, duration, AHEaseSineInOut.new(), distance),
    AHLocationYBy.new(target, duration, AHEaseSineInOut.new(), -distance)
  ))
end

local function newCircleAnimation(self, fadeDelay, xWobbleDelay, yWobbleDelay)
  local wobbleDistance = Screen.getWidthPart(0.02)
  local wobbleDuration = 3.0
  local zoomInDuration = 0.8

  return AHSpawn.new(
    AHScaleTo.new(self, zoomInDuration, AHEaseExponentialInOut.new(), 1),
    AHDelayed.new(fadeDelay, AHOpacityTo.new(self, zoomInDuration, nil, 1)),
    AHDelayed.new(xWobbleDelay, newWobbleXAction(self, wobbleDuration, wobbleDistance)),
    AHDelayed.new(yWobbleDelay, newWobbleYAction(self, wobbleDuration, wobbleDistance))
  )
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseZoomCircle:getAnimation()
  return AHDoNothing.new()
end

function ExerciseZoomCircle:reset()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseZoomCircle.new(radius, color)
  local self = FilledCircle.new(radius, color)
  Mixin.mix(self, ExerciseZoomCircle)

  return self
end



-- ============================================================================
class("BlueGrowCircle")
-- ============================================================================


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BlueGrowCircle:getAnimation()
  local fadeDelay = 0.0
  local xWobbleDelay = 0.0
  local yWobbleDelay = 2.0
  return newCircleAnimation(self, fadeDelay, xWobbleDelay, yWobbleDelay)
end

function BlueGrowCircle:reset()
  self:setScl(0)
  self:setOpacity(0)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function BlueGrowCircle.new(radius)
  local self = ExerciseZoomCircle.new(radius, COLOR_CIRCLE_BLUE)
  Mixin.mix(self, BlueGrowCircle)
  self:reset()
  return self
end



-- ============================================================================
class("BlueShrinkCircle")
-- ============================================================================


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BlueShrinkCircle:getAnimation()
  local fadeDelay = 0.2
  local xWobbleDelay = 0.0
  local yWobbleDelay = 2.0
  return newCircleAnimation(self, fadeDelay, xWobbleDelay, yWobbleDelay)
end

function BlueShrinkCircle:reset()
  self:setEffectiveDims(Screen.getHeightPart(1.5))
  self:setOpacity(0)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function BlueShrinkCircle.new(radius)
  local self = ExerciseZoomCircle.new(radius, COLOR_CIRCLE_BLUE)
  Mixin.mix(self, BlueShrinkCircle)
  self:reset()
  return self
end



-- ============================================================================
class("GreyGrowCircle")
-- ============================================================================


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GreyGrowCircle:getAnimation()
  local fadeDelay = 0.0
  local xWobbleDelay = 2.0
  local yWobbleDelay = 0.0
  return newCircleAnimation(self, fadeDelay, xWobbleDelay, yWobbleDelay)
end

function GreyGrowCircle:reset()
  self:setScl(0.5)
  self:setOpacity(0)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GreyGrowCircle.new(radius)
  local self = ExerciseZoomCircle.new(radius, COLOR_CIRCLE_GREY)
  Mixin.mix(self, GreyGrowCircle)
  self:reset()
  return self
end
