--- ExerciseCardDone
-- @author Created by Aaron Barrett on April 21, 2015.

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("ExerciseIcon")
require("ExerciseSession")


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getTimeUntilReview()
  local days = 7
  if ExerciseSession then
    days = ExerciseSession.getDaysUntilNextExerciseReview()
  end
  if days == 1 then
    return "1 day"
  end
  return tostring(days) .. " days"
end

local function areNotificationsEnabled()
  local result = true
  if ExerciseSession then
    result = ExerciseSession.areNotificationsEnabled()
  end
  return result
end

local function setReminderEnabled(isEnabled)
  if ExerciseSession then
    ExerciseSession.setUserAcceptedToReviewExercise(isEnabled)
  end
end



-- ============================================================================
local DoneIcon = privateClass()
-- ============================================================================


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CIRCLE_COUNT = 3

local CIRCLE_COLORS = {
  ColorUtils.colorHex24ToStd(0x88e0ff),
  ColorUtils.colorHex24ToStd(0x66d6ff),
  ColorUtils.colorHex24ToStd(0x26baf3),
}

local CIRCLE_LOCATIONS = {
  {-Screen.getWidthPart(0.6), Screen.getHeightPart(0.1)},
  {Screen.getWidthPart(1.2), Screen.getHeightPart(0)},
  {-Screen.getWidthPart(0.6), -Screen.getHeightPart(0.1)},
}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DoneIcon:reset()
  self:setVisibleRecursive(false)
end

function DoneIcon:newAnimation()
  local moveDuration = 0.5
  local scaleUpDuration = 0.4
  local scaleDownDuration = 0.2
  local fadeDuration = 0.5
  local scaleUpDelay = 0
  local fadeDelay = moveDuration

  local centerX, centerY = self:getCenter()

  local initialize = function()
    self:setVisibleRecursive(true)
    for i, circle in ipairs(self.circles) do
      circle:setLoc(unpack(CIRCLE_LOCATIONS[i]))
    end
    self.image:setOpacity(0)
  end

  local action = AHSpawn.new()
  for _, circle in ipairs(self.circles) do
    action:addAction(AHLocationTo.new(circle, moveDuration, AHEaseExponentialOut.new(), centerX, centerY))
  end
  action:addAction(AHDelayed.new(scaleUpDelay,
    AHSequence.new(
      AHScaleTo.new(self, scaleUpDuration, AHEaseExponentialInOut.new(), 1.2),
      AHScaleTo.new(self, scaleDownDuration, AHEaseSineInOut.new(), 1)
    )
  ))
  action:addAction(AHDelayed.new(fadeDelay, AHOpacityTo.new(self.image, fadeDuration, nil, 1)))

  return AHSequence.new(initialize, action)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constuctor -------------------------

function DoneIcon.new(concept, iconName)
  local size = Screen.getWidthPart(0.35)
  local self = Group.new(size, size)
  Mixin.mix(self, DoneIcon)

  self.circles = {}

  for i=1, CIRCLE_COUNT do
    local circle = FilledCircle.new(size / 2, CIRCLE_COLORS[i])
    self:insertAtCenter(circle)
    table.insert(self.circles, circle)
    circle:setVisible(false)
  end

  self.image = ExerciseIcon.new(concept, iconName)
  self.image:setEffectiveDims(size)
  self:insertAtCenter(self.image)

  self:reset()

  return self
end



-- ============================================================================
local DaysRemainingLabel = privateClass()
-- ============================================================================

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DaysRemainingLabel:reset()
  for _, dot in ipairs(self.dotContainer.children) do
    dot:setScl(0)
  end
  self.label:setVisibleRecursive(false)
end

function DaysRemainingLabel:newAnimation()
  local scaleDuration = 0.35
  local fadeDuration = 0.15
  local stagger = 0.1

  local function newScaleAction(dot, duration, delay)
    return AHDelayed.new(delay, AHSequence.new(
      AHScaleTo.new(dot, duration / 2, AHEaseSineInOut.new(), 1),
      AHScaleTo.new(dot, duration / 2, AHEaseSineInOut.new(), 0)
    ))
  end

  local dots = self.dotContainer.children
  local preparationAction = AHLoop.new(AHSequence.new(
    AHSpawn.new(
      newScaleAction(dots[1], scaleDuration, 0),
      newScaleAction(dots[2], scaleDuration, stagger),
      newScaleAction(dots[3], scaleDuration, stagger * 2)
    )
  ), 2)

  -- For some reason, fading self.label in doesn't work (it always shows 100% opacity)
  -- so fake it with a white rectangle in front of the label.
  local labelCover = FilledRectangle.new(self.label:getWidth(), self.label:getHeight(), CardCarouselCardCommon.COLORS.WHITE)
  local fadeAction = AHSequence.new(
    function()
      labelCover:setOpacity(1)
      self.label:insertAtCenter(labelCover, math.huge)
    end,
    function()
      self.label:setVisibleRecursive(true)
    end,
    AHOpacityTo.new(labelCover, fadeDuration, nil, 0),
    AHRemove.new(labelCover)
  )

  return AHSequence.new(preparationAction, fadeAction)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newDot(font)
  local dotRadius = font:getActualSize() * 0.2
  return FilledCircle.new(dotRadius, CardCarouselCardCommon.COLORS.ORANGE)
end

-- Constuctor -------------------------

function DaysRemainingLabel.new()
  local font = CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM
  local label = Label.new(getTimeUntilReview(), font)
  label:setColor(unpack(CardCarouselCardCommon.COLORS.ORANGE))

  local self = Group.new(label:getDims())
  Mixin.mix(self, DaysRemainingLabel)
  self:insertAtCenter(label)

  self.dotContainer = Row.new(font:getActualSize() * 0.2)
  for i=1, 3 do
    self.dotContainer:insert(newDot(font))
  end
  self.dotContainer:alignChildren()
  self.dotContainer:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
  self:insert(self.dotContainer)

  self.label = label

  self:reset()

  return self
end



-- ============================================================================
class("ExerciseCardDone")
-- ============================================================================

require("ExerciseCheckbox")
require("ExerciseTutorial")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MESSAGE_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.LIGHT_GREY)
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardDone:onEnter()
  self.super.onEnter(self)
  self.timerAction = AHDelayed.new(9, function()
    local tutorial = ExerciseTutorial.newOutro()
    self:insertAtCenter(tutorial)
    tutorial:animateIn()
  end):start()
  self.checkmark:setTouchEnabled(true)
end

function ExerciseCardDone:onLeaveComplete()
  self.super.onLeaveComplete(self)
  if self.timerAction then
    self.timerAction:stop()
  end
  setReminderEnabled(self.isReminderChecked)
  self.checkmark:setTouchEnabled(false)
end

function ExerciseCardDone:reset()
  self.workArea:setY(self:getHeightPart(0.4))
  self.workArea:setOpacity(0)
  self.icon:reset()
  self.daysRemainingLabel:reset()
end

function ExerciseCardDone:createAnimationAction()
  return AHSpawn.new(
    AHOpacityTo.new(self.workArea, 0.5, nil, 1),
    AHLocationYTo.new(self.workArea, 0.5, AHEaseExponentialOut.new(), self:getHeightPart(0.5)),
    self.icon:newAnimation(),
    AHDelayed.new(0.5, self.daysRemainingLabel:newAnimation())
  )
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newReviewDateLine()
  local leftLabel = Label.new("this material in:", PrerenderedFont.new(FontFaces.DIN_OT_MEDIUM, 18))
  leftLabel:setColor(unpack(CardCarouselCardCommon.COLORS.LIGHT_GREY))
  local remainingLabel = DaysRemainingLabel.new()

  local row = Row.new(leftLabel:getHeightPart(0.2))
  row:insert(leftLabel)
  row:insert(remainingLabel)
  row:alignChildren()

  return row, remainingLabel
end

local function newRemindMeLine(self)
  local label = Label.new("Remind me:", PrerenderedFont.new(FontFaces.DIN_OT_MEDIUM, 18))
  label:setColor(unpack(CardCarouselCardCommon.COLORS.LIGHT_GREY))

  local function onCheck(isChecked)
    self.isReminderChecked = isChecked
  end

  local checkmark = ExerciseCheckbox.new(label:getHeightPart(0.8), CardCarouselCardCommon.COLORS.LIGHT_GREY, onCheck)
  checkmark:setChecked(self.isReminderChecked)

  local row = Row.new(label:getHeightPart(0.4))
  row:insert(label)
  row:insert(checkmark)
  row:alignChildren()

  return row, checkmark
end

-- Constructor ------------------------

function ExerciseCardDone.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardDone)

  self.isReminderChecked = areNotificationsEnabled()

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())
  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(workArea)

  local concept = data.concept
  local iconName = data.icon
  self.icon = DoneIcon.new(concept, iconName)
  self:insert(self.icon)
  local iconPlaceholder = Group.new(self.icon:getDims())

  local doneLabel = Label.new("YOU'RE DONE", Font.new(FontFaces.DIN_OT_BOLD, 42))
  doneLabel:setColor(unpack(CardCarouselCardCommon.COLORS.BLUE))

  local remindMeLine, checkmark = newRemindMeLine(self)
  checkmark:setTouchEnabled(false)
  self.checkmark = checkmark

  local col = Column.new(0)

  col:insert(iconPlaceholder)
  col:insertSpacer(Screen.getWidthPart(0.12))
  col:insert(doneLabel)
  col:insertSpacer(Screen.getWidthPart(0.05))
  col:insert(CardCarouselTextView.new("We recommend you review", CardCarouselCardCommon.CONTENT_WIDTH, MESSAGE_TEXT_ATTRIBUTES, Alignment.CENTER))
  local line, daysRemainingLabel = newReviewDateLine()
  col:insert(line)
  col:insertSpacer(Screen.getWidthPart(0.08))
  col:insert(remindMeLine)
  workArea:insertAtCenter(col)
  col:alignChildren()

  workArea:linkOpacityAttrToChildren(true)

  runInCoroutine(function()
    coroutine.yield()
    self.icon:setLoc(self:transformObjectLoc(iconPlaceholder))
  end)

  self.workArea = workArea
  self.daysRemainingLabel = daysRemainingLabel

  return self
end
