--- ExerciseCardExampleAndText
-- @author Created by Jay Austria on May 27, 2015.

class("ExerciseCardExampleAndText")

require("cards.ExerciseCard")
require("cards.ExerciseCardText")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_ATTRIBUTES_WITH_TITLE = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_SMALL, CardCarouselCardCommon.COLORS.GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_SMALL, CardCarouselCardCommon.COLORS.BLUE)
}

local TEXT_ATTRIBUTES_NO_TITLE = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.BLUE)
}

local TITLE_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.BLUE, {spacing = 14})
}

local DIVIDER_COLOR = ColorUtils.colorHex24ToStd(0xCDCDCD)
local SPACING_MULTIPLIER = 0.05
local DEFAULT_BODY_SPACING = SPACING_MULTIPLIER
local DEFAULT_BASELINE_Y_MULTIPLIER = 0.47

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getSplitAnimation(self)
  local image = self.image
  local workArea = self.workArea
  local textColumn = self.textColumn
  local divider = self.divider

  local splitDuration = 1.5
  local dividerFadeDelayDuration = 0.8
  local textFadeDelayDuraton = 0.4

  local spaceFromCenter = workArea:getHeightPart(SPACING_MULTIPLIER)

  local slideImageAction = AHLocationYTo.new(image, splitDuration, AHEaseExponentialInOut.new(), workArea:getHeightPart(self.baselineYMultiplier) + spaceFromCenter)
  local slideTextAction = AHLocationYTo.new(textColumn, splitDuration, AHEaseExponentialInOut.new(), workArea:getHeightPart(self.baselineYMultiplier) - spaceFromCenter)
  local fadeTextAction = AHDelayed.new(textFadeDelayDuraton, AHOpacityTo.new(textColumn, splitDuration - textFadeDelayDuraton, AHEaseExponentialInOut.new(), 1))
  local fadeDividerAction = AHDelayed.new(dividerFadeDelayDuration, AHOpacityTo.new(divider, splitDuration - dividerFadeDelayDuration, nil, 1))

  local splitAnimation = AHSpawn.new(slideImageAction, slideTextAction, fadeTextAction, fadeDividerAction)

  return splitAnimation
end

local function getSlowScaleAnimation(self)
  local image = self.image
  local animationDuration = 14

  local slowScaleSequence = AHSequence.new(
                              AHScaleBy.new(image, animationDuration/2, AHEaseSimpleInOut.new(), 0.05),
                              AHScaleBy.new(image, animationDuration/2, AHEaseSimpleInOut.new(), -0.05)
                            )
  local loopedSequence = AHLoop.new(slowScaleSequence)

  return loopedSequence
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardExampleAndText:isDynamic()
  return true
end

function ExerciseCardExampleAndText:reset()
  local image = self.image
  local textLabel = self.textLabel
  local workArea = self.workArea
  local divider = self.divider
  local textColumn = self.textColumn

  image:setOpacity(1)
  textColumn:setOpacity(0)
  divider:setOpacity(0)

  image:setLoc(workArea:getWidthPart(0.5), workArea:getHeightPart(self.baselineYMultiplier) - image:getEffectiveHeightPart(0.5))
  textColumn:setLoc(workArea:getWidthPart(0.5), workArea:getHeightPart(self.baselineYMultiplier) + textLabel:getHeightPart(0.5))
end

function ExerciseCardExampleAndText:createAnimationAction()
  local animationAction = AHSpawn.new(getSplitAnimation(self), getSlowScaleAnimation(self))
  return animationAction
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardExampleAndText.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardExampleAndText)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(workArea)
  self.workArea = workArea

  local concept = data.concept

  local imageName = data.image
  local image = AssetHelper.createPhotoSpriteForConcept(concept, imageName)
  local maxHeight = workArea:getHeightPart(0.5)
  local imageScale = math.min(1.0, maxHeight / image:getHeight())
  image:setScl(imageScale)
  image:setAnchorPointBottom()

  workArea:insert(image)
  self.image = image

  local columnSpacing = workArea:getHeightPart(SPACING_MULTIPLIER)
  local textColumn = Column.new(columnSpacing)
  textColumn:setAnchorPointTop()
  self.textColumn = textColumn

  local labelAttributes = TEXT_ATTRIBUTES_NO_TITLE
  local baselineYMultiplier = DEFAULT_BASELINE_Y_MULTIPLIER

  local titleText = data.title
  if titleText and #titleText > 0 then
    local title = CardCarouselTextView.new(titleText, workArea:getWidth(), TITLE_ATTRIBUTES)
    title:setColor(unpack(CardCarouselCardCommon.COLORS.BLUE))
    textColumn:insert(title)
    labelAttributes = TEXT_ATTRIBUTES_WITH_TITLE
    baselineYMultiplier = 0.52
    textColumn:setSpacing(workArea:getHeightPart(0.02))
  end

  local textLabel = CardCarouselTextView.new(data.text, workArea:getWidth(), labelAttributes)
  textColumn:insert(textLabel)
  self.textLabel = textLabel

  workArea:insert(textColumn)
  textColumn:alignChildren()
  textColumn:linkOpacityAttrToChildren()

  local thickness = 0.5
  local divider = FilledRectangle.new(self:getWidthPart(0.9), thickness, DIVIDER_COLOR)
  divider:setLoc(self:getWidth() - divider:getWidthPart(0.5), self:getHeightPart(baselineYMultiplier))
  self:insert(divider)
  self.divider = divider

  self.baselineYMultiplier = baselineYMultiplier

  return self
end
