--- ExerciseCardExplanationCommon
-- @author Created by karl on October 09, 2015.

-- ============================================================================
class("AnimatedDot")
-- ============================================================================

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

AnimatedDot.DIRECTIONS = {
  LEFT = "LEFT",
  RIGHT = "RIGHT"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newScaleLoop(ring)
  local duration = 0.8
  local ease = AHEaseSineInOut.new()
  return AHLoop.new(AHSequence.new(
    AHScaleTo.new(ring, duration, ease, ring.maxScale),
    AHScaleTo.new(ring, duration, ease, 1)
  ))
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function AnimatedDot:reset()
  for _, ring in ipairs(self.rings) do
    ring:setScl(1)
    ring:setOpacity(ring.baseOpacity)
  end
  self.fixedRing:setOpacity(0)
end

function AnimatedDot:newLoopAction()
  return AHSpawn.new(
    newScaleLoop(self.rings[1]),
    AHDelayed.new(0.2, newScaleLoop(self.rings[2])),
    AHDelayed.new(0.4, newScaleLoop(self.rings[3]))
  )
end

function AnimatedDot:setActive(isActive)
  local fixedRing = self.fixedRing
  local duration = 0.2
  local innerOpacity = 0.2
  local outerOpacity = 0.2
  local innerScale = 1.0
  local innerScaleEnd = 1.0
  local fixedRingOpacity = 0.0
  local fixedRingScale = 1.0

  fixedRing:setScl(fixedRingScale)

  if isActive then
    innerOpacity = 0.0
    outerOpacity = 0.0
    fixedRingOpacity = 1.0
    innerScale = 2.0
    innerScaleEnd = 0.0

    fixedRing:setOpacity(0)
    fixedRing:setScl(2.0)
  end

  self.actionInterrupt:insert(AHSpawn.new(
    AHDelayed.new(0.2, AHOpacityTo.new(self.rings[1], duration, nil, innerOpacity)),
    AHDelayed.new(0.2, AHOpacityTo.new(self.rings[2], duration, nil, innerOpacity)),
    AHDelayed.new(0.2, AHOpacityTo.new(self.rings[3], duration, nil, outerOpacity)),
    AHDelayed.new(0.1, AHOpacityTo.new(fixedRing, duration, AHEaseSimpleInOut.new(), fixedRingOpacity)),
    AHDelayed.new(0.2, AHScaleTo.new(fixedRing, duration * 1.3, AHEaseBackOut.new(), fixedRingScale))
  ))
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createRing(class, radius)
  local lineWidth = 2.5
  return class.new(radius, CardCarouselCardCommon.COLORS.BLUE, lineWidth)
end

local function insertRing(self, class, radius, maxScale, opacity)
  local ring = createRing(class, radius)

  opacity = opacity or 0.5

  ring.maxScale = maxScale
  ring:setOpacity(opacity)
  ring.baseOpacity = opacity
  self:insertAtCenter(ring)
  table.insert(self.rings, ring)
end

-- Constructor ------------------------

function AnimatedDot.new(startDirection, onTouched)
  local baseRadius = 7
  local diameter = baseRadius * 2

  local size = diameter * 2.5
  local self = Group.new(size, size)
  Mixin.mix(self, AnimatedDot)

  self.rings = {}

  insertRing(self, FilledCircle, baseRadius, 1.5)
  insertRing(self, FilledCircle, baseRadius * 1.8, 1.2)
  insertRing(self, HollowCircle, baseRadius * 2.75, 1.1, 0.3)

  local fixedRing = createRing(HollowCircle, baseRadius * 2.75)
  fixedRing:setOpacity(0)
  self:insertAtCenter(fixedRing)
  self.fixedRing = fixedRing

  local expandedWidth = self:getWidth() + 40
  local expandedHeight = self:getHeight() + 20
  local virtualButton = Button.addVirtualButton(self, expandedWidth, expandedHeight, onTouched)
  self.virtualButton = virtualButton

  FunctionUtils.forwardMethodCalls(self, virtualButton, "setTouchEnabled")

  self:setTouchEnabled(false)

  self.actionInterrupt = ActionInterrupt.new()
  return self
end



-- ============================================================================
class("SlidingGroup")
-- ============================================================================

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SlidingGroup:slideDown(distance, willBeActive)
  if self.slideDownAction then
    self.slideDownAction:stop()
  end
  local opacity = 0
  if willBeActive then
    opacity = 1
  end
  local duration = 0.3
  local opacityDelayDuration = 0.15
  self.slideDownAction = AHSpawn.new(
    AHLocationYBy.new(self, duration, AHEaseSineInOut.new(), -distance),
    AHDelayed.new(opacityDelayDuration, AHOpacityTo.new(self, duration - opacityDelayDuration, nil, opacity))
  ):start()
  return self.slideDownAction
end

function SlidingGroup:reset(isActive)
  local opacity = 0
  if isActive then
    opacity = 1
  end
  self:setOpacity(opacity)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function SlidingGroup.makeSlidingGroup(group)
  Mixin.mix(group, SlidingGroup)
end
