--- ExerciseCardEyeMovement
-- @author Created by karl on May 18, 2015.

class("ExerciseCardEyeMovement")

require("cards.ExerciseCard")
require("ExerciseWPMSlider")
require("widgets.progress.TimeTimer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TIMER_SECONDS = 60

local WORDS_PER_MINUTE_MIN = 100
local WORDS_PER_MINUTE_MAX = 500
local SECONDS_PER_MINUTE = 60

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getDelayAmount(self)
  local wordsPerMinute = self.slider:getValue()
  local wordsPerSecond = wordsPerMinute / SECONDS_PER_MINUTE
  local secondsPerWord = 1 / wordsPerSecond
  return secondsPerWord
end

local function showNextWord(self, wordIndex, labelIndex, labelIndexChooseFunction)
  local label = self.labels[labelIndex]
  label:setText(self.words[wordIndex])
  label:setOpacity(1)
  waitFor(getDelayAmount(self))
  label:setOpacity(0)
  wordIndex = (wordIndex % #self.words) + 1
  labelIndex = labelIndexChooseFunction()
  return wordIndex, labelIndex
end

local function startUpdateRoutine(self)
  self.updateRoutine = runInCoroutine(function()
    local wordIndex = 1
    local labelIndex = math.random(#self.labels)

    local labelIndexChooseFunction = function()
      return (labelIndex % #self.labels) + 1
    end

    for i=1, #self.labels * 2 do
      wordIndex, labelIndex = showNextWord(self, wordIndex, labelIndex, labelIndexChooseFunction)
    end

    labelIndexChooseFunction = function()
      local index = math.random(#self.labels - 1)
      if index >= labelIndex then
        index = index + 1
      end
      return index
    end

    while true do
      wordIndex, labelIndex = showNextWord(self, wordIndex, labelIndex, labelIndexChooseFunction)
    end
  end)
end

local function stopUpdateRoutine(self)
  if self.updateRoutine then
    self.updateRoutine:stop()
  end
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardEyeMovement:onEnter()
  self.super.onEnter(self)
  self.slider:setTouchEnabled(true)

  local delayedStart = function()
    waitFor(1.2)
    self.timer:start()
    startUpdateRoutine(self)
  end
  runInCoroutine(delayedStart)
end

function ExerciseCardEyeMovement:onLeaveComplete()
  self.super.onLeaveComplete(self)
  self.slider:setTouchEnabled(false)
  stopUpdateRoutine(self)
  self.timer:stop()
  self.timer:setTime(TIMER_SECONDS)
end


function ExerciseCardEyeMovement:reset()
  if (self.timer == nil) then
    local timer = TimeTimer.new(FontFaces.DIN_OT_MEDIUM, 18, TIMER_SECONDS, function()
      self:forceScrollNavigation(CardCarouselNavigator.DIRECTION_LEFT, CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL)
    end)
    timer:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.9))
    timer:setColor(unpack(CardCarouselCardCommon.COLORS.BLUE))
    local isRecursive = true
    timer:linkOpacityAttrToChildren(isRecursive)
    self:insert(timer)
    self.timer = timer
  end

  for _, label in ipairs(self.labels) do
    label:setOpacity(0)
  end

  self.slider:reset()
  self.slider:prepareForAnimateIn()
  self.timer:setOpacity(0)
end

function ExerciseCardEyeMovement:createAnimationAction()
  local timerAction = AHDelayed.new(1.4, AHOpacityTo.new(self.timer, 0.5, nil, 1.0))
  local animateAction = AHSpawn.new(timerAction, self.slider:getAnimateInAction())

  return animateAction
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function insertLabel(self, workArea, anchorX, anchorY, x, y)
  local font = FontFaces.TISA_OFFC
  local fontSize = 32
  local color = CardCarouselCardCommon.COLORS.GREY
  local label = CardCarouselCardCommon.newLabel("X", font, fontSize, color)
  label:setAnchorPoint(anchorX, anchorY)
  label:setLoc(x, y)
  workArea:insert(label)
  table.insert(self.labels, label)
end

-- Constructor ------------------------

function ExerciseCardEyeMovement.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardEyeMovement)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())
  local workArea = Group.new(CardCarouselCardCommon.CONTENT_WIDTH, self:getHeightPart(0.65))
  self:insertAtCenter(workArea)

  local slider = ExerciseWPMSlider.new(WORDS_PER_MINUTE_MIN, WORDS_PER_MINUTE_MAX)
  slider:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.1))
  slider:setTouchEnabled(false)
  slider:createTip()
  self:insert(slider)
  slider:setValue(WORDS_PER_MINUTE_MIN)
  self.slider = slider

  self.labels = {}
  insertLabel(self, workArea, 0, 1, 0, workArea:getHeight())
  insertLabel(self, workArea, 1, 1, workArea:getWidth(), workArea:getHeight())
  insertLabel(self, workArea, 0, 0, 0, 0)
  insertLabel(self, workArea, 1, 0, workArea:getWidth(), 0)
  insertLabel(self, workArea, 0.5, 0.5, workArea:getWidthPart(0.5), workArea:getHeightPart(0.5))

  self.words = {}
  local wordMatchRegex = "[^%s]+"
  for word in data.text:gmatch(wordMatchRegex) do
    table.insert(self.words, word)
  end

  return self
end
