--- ExerciseCardImageAndCaption
-- @author Created by Jay Austria on May 27, 2015.

class("ExerciseCardImageAndCaption")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("cards.ExerciseCardText")
require("containers.card_carousel.CardCarouselTextView")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.BLUE)
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getEnterAnimation(self)
  local title = self.title
  local text = self.text
  local column = self.column

  local enterAnimation = AHSpawn.new()
  local slideDuration = 1.0
  local slideStagger = 0

  if title then
    enterAnimation:addAction(AHLocationXTo.new(title, slideDuration, AHEaseExponentialOut.new(), column:getWidthPart(0.5)))
    slideStagger = 0.2
  end

  if text then
    enterAnimation:addAction(AHLocationXTo.new(text, slideDuration + slideStagger, AHEaseExponentialOut.new(), column:getWidthPart(0.5)))
  end

  return enterAnimation
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardImageAndCaption:isDynamic()
  return true
end

function ExerciseCardImageAndCaption:reset()
  local title = self.title
  local text = self.text
  local workArea = self.workArea

  if title then
    title:setX(self:getWidthPart(1.5))
  end

  if text then
    text:setX(self:getWidthPart(1.5))
  end
end

function ExerciseCardImageAndCaption:createAnimationAction()
  return getEnterAnimation(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardImageAndCaption.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardImageAndCaption)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(workArea)
  self.workArea = workArea

  local concept = data.concept

  local columnSpacing = Screen.getWidthPart(0.02)
  local column = Column.new(columnSpacing)
  self.column = column

  local imageName = data.image
  local image = AssetHelper.createPhotoSpriteForConcept(concept, imageName)

  local desiredImageWidth = Screen.getWidthPart(1.0)
  local imageScale = desiredImageWidth / image:getWidth()
  if Screen.isTabletAspectRatio() then
    local desiredImageHeight = Screen.getHeightPart(0.6)
    imageScale = desiredImageHeight / image:getHeight()
  end
  image:setScl(imageScale)

  column:insert(image)
  column:insertSpacer(workArea:getHeightPart(0.08))

  if data.title then
    local title = Label.new(data.title, PrerenderedFont.new(FontFaces.DIN_OT_MEDIUM, 28))
    title:setColor(unpack(CardCarouselCardCommon.COLORS.DARK_GREY))
    column:insert(title)
    self.title = title
  end
  if data.text then
    local text = CardCarouselTextView.new(data.text, workArea:getWidth(), TEXT_ATTRIBUTES, Alignment.CENTER)
    column:insert(text)
    self.text = text
  end
  column:alignChildren()
  column:setAnchorPointBottom()

  column:setLoc(workArea:getWidthPart(0.5), 0)
  workArea:insert(column)

  return self
end
