--- ExerciseCardImageAndText
-- @author Created by Aaron Barrett on April 21, 2015.

class("ExerciseCardImageAndText")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("cards.ExerciseCardText")
require("containers.card_carousel.CardCarouselTextView")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SCALE_DURATION = 4.0
local FADE_IN_DURATION = 2.5
local FADE_OUT_DURATION = 1.5

local START_SCALE = 1.2
local END_SCALE = 1.5

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardImageAndText:isDynamic()
  return true
end

function ExerciseCardImageAndText:reset()
  local secondImage = self.secondImage
  if (secondImage ~= nil) then
    secondImage:setScl(START_SCALE)
    secondImage:setOpacity(0.0)

    local firstImage = self.firstImage
    firstImage:setScl(START_SCALE)
    firstImage:setOpacity(0.0)
  end
end

function ExerciseCardImageAndText:createAnimationAction()
  local secondImage = self.secondImage
  if (secondImage ~= nil) then
    local firstImage = self.firstImage
    local animationAction = AHLoop.new(
                                       AHSequence.new(
                                                      AHSpawn.new(
                                                                  AHScaleTo.new(firstImage, SCALE_DURATION, nil, END_SCALE),
                                                                  AHSequence.new(
                                                                                 AHOpacityTo.new(firstImage, FADE_IN_DURATION, nil, 1.0),
                                                                                 AHOpacityTo.new(firstImage, FADE_OUT_DURATION, nil, 0.0)
                                                                                 ),
                                                                  AHScaleTo.new(secondImage, SCALE_DURATION, nil, START_SCALE)
                                                                  ),
                                                      AHSpawn.new(
                                                                  AHScaleTo.new(secondImage, SCALE_DURATION, nil, END_SCALE),
                                                                  AHSequence.new(
                                                                                 AHOpacityTo.new(secondImage, FADE_IN_DURATION, nil, 1.0),
                                                                                 AHOpacityTo.new(secondImage, FADE_OUT_DURATION, nil, 0.0)
                                                                                 ),
                                                                  AHScaleTo.new(firstImage, SCALE_DURATION, nil, START_SCALE)
                                                                  )
                                                      )
                                       )
    return animationAction
  else
    return nil
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createImageSwapper(firstImage, secondImage)
  local imageHeight = firstImage:getHeight()
  local scaledImageHeight = imageHeight * END_SCALE

  local imageSwapper = Group.new(Screen.getWidthPart(0.75), scaledImageHeight)

  imageSwapper:insertAtCenter(firstImage)
  imageSwapper:insertAtCenter(secondImage)

  return imageSwapper
end

-- Constructor ------------------------

function ExerciseCardImageAndText.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardImageAndText)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local workArea = Group.new(self:getWidth() - CardCarouselCardCommon.MARGINS.HORIZONTAL * 2, self:getHeight() - CardCarouselCardCommon.MARGINS.VERTICAL * 2)
  self:insertAtCenter(workArea)

  local concept = data.concept

  local columnSpacing = Screen.getWidthPart(0.05)
  local column = Column.new(columnSpacing)

  local firstImageName = data.image1
  local firstImage = AssetHelper.createPhotoSpriteForConcept(concept, firstImageName)

  local secondImageName = data.image2
  local hasSecondImage = ((secondImageName ~= nil) and (secondImageName ~= ""))
  if (hasSecondImage) then
    local secondImage = AssetHelper.createPhotoSpriteForConcept(concept, secondImageName)

    self.firstImage = firstImage
    self.secondImage = secondImage

    local imageSwapper = createImageSwapper(firstImage, secondImage)
    column:insert(imageSwapper)
  else
    column:insert(firstImage)
  end

  if data.title then
    column:insert(CardCarouselCardCommon.newCachedFontLabel(data.title, CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.GREY))
  end
  if data.subtitle then
    column:insert(CardCarouselCardCommon.newCachedFontLabel(data.subtitle, CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.GREY))
  end
  if data.text then
    column:insert(CardCarouselTextView.new(data.text, workArea:getWidth(), ExerciseCardText.TEXT_ATTRIBUTES))
  end
  column:alignChildren()

  workArea:insertAtCenter(column)

  return self
end
