--- ExerciseCardInstructionAndDetail
-- @author Created by Jay Austria on May 30, 2015

class("ExerciseCardInstructionAndDetail")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INSTRUCTION_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.BLUE)
}

local TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(Font.new(FontFaces.TISA_OFFC, 19), CardCarouselCardCommon.COLORS.LIGHT_GREY, {lineSpacing = -2})
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardInstructionAndDetail:onEnter()
  self.super.onEnter(self)
  self.scroller:setTouchEnabled(true)
  self.scroller:setInertialScrollingEnabled(true)
  self.forwardTapArea:setTouchEnabled(true)
  self.backTapArea:setTouchEnabled(true)
end

function ExerciseCardInstructionAndDetail:onLeaveBegin()
  self.super.onLeaveBegin(self)
  self.forwardTapArea:setTouchEnabled(false)
  self.backTapArea:setTouchEnabled(false)
end

function ExerciseCardInstructionAndDetail:onLeaveComplete()
  self.super.onLeaveComplete(self)
  self.scroller:setTouchEnabled(false)
  self.scroller:setInertialScrollingEnabled(false)
end

-- Disable tap navigation through the CardCarouselNavigator and let this card handle
-- its own tap navigation through its own tap areas. This is done to work around
-- a limitation where the TouchScroller on this card swallows TouchBegan and
-- TouchEnded events with no way to hand them off to the card's touch handler.
function ExerciseCardInstructionAndDetail:handlesOwnTapNavigation()
  return true
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createTapAreas(self)
  local width = self:getWidth() - CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY
  local height = self:getHeight()
  local onTouchDown = nil
  local duration = CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL
  local willCheckNavigationConstraints = true
  local onTouchUp = function()
    self:forceTapNavigation(CardCarouselNavigator.DIRECTION_LEFT, duration, willCheckNavigationConstraints)
  end
  local onTouchCancel = nil
  local forwardTapArea = Button.new(width, height, onTouchDown, onTouchUp, onTouchCancel)
  forwardTapArea:setAnchorPointRight()
  self:insertAtRight(forwardTapArea)
  forwardTapArea:setTouchEnabled(false)
  self.forwardTapArea = forwardTapArea

  width = CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY
  local height = self:getHeight()
  onTouchUp = function()
    self:forceTapNavigation(CardCarouselNavigator.DIRECTION_RIGHT, duration, willCheckNavigationConstraints)
  end
  local backTapArea = Button.new(width, height, onTouchDown, onTouchUp, onTouchCancel)
  backTapArea:setAnchorPointLeft()
  self:insertAtLeft(backTapArea)
  backTapArea:setTouchEnabled(false)
  self.backTapArea = backTapArea
end

-- Constructor ------------------------

function ExerciseCardInstructionAndDetail.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardInstructionAndDetail)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(workArea)

  local column, anchorOffsetY
  if (Screen.hasNotch()) then
    column = Column.new(32)
    anchorOffsetY = 8
  else
    column = Column.new(14)
    anchorOffsetY = 38
  end

  local formattedInstruction = string.upper(data.instruction)
  local instructionsLabel = CardCarouselTextView.new(formattedInstruction, CardCarouselCardCommon.CONTENT_WIDTH, INSTRUCTION_ATTRIBUTES, Alignment.CENTER)
  column:insert(instructionsLabel)

  local label = CardCarouselTextView.new(data.text, CardCarouselCardCommon.CONTENT_WIDTH, TEXT_ATTRIBUTES)
  column:insert(label)

  column:alignChildren()
  column:setAnchorPointTop()
  column:setLoc(workArea:getWidthPart(0.5), workArea:getHeight() + anchorOffsetY)

  local scroller = TouchScroller.new(CardCarouselCardCommon.CONTENT_WIDTH, CardCarouselCardCommon.CONTENT_HEIGHT, column)
  scroller:setHandoffOnScrollDownEnabled(false)
  scroller:setHandoffOnScrollUpEnabled(false)
  scroller:setHandoffOnHorizontalSwipeEnabled(true)
  workArea:insertAtCenter(scroller, 0)
  scroller:setTouchEnabled(false)
  scroller:setScrollDecayFactor(CardCarouselCardCommon.DEFAULT_TOUCH_SCROLLER_DECAY_FACTOR)
  self.scroller = scroller

  createTapAreas(self)

  return self
end
