--- ExerciseCardIntroText

class("ExerciseCardIntroText")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")
require("ExerciseCardLoaderOverlay")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ExerciseCardIntroText.TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_SMALL, CardCarouselCardCommon.COLORS.GREY),
  h1 = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_SMALL, CardCarouselCardCommon.COLORS.BLUE),
  h1highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.BLUE),
  divide = {itemType = CardCarouselTextView.ITEM_TYPE_DIVIDE}
}

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardIntroText:isScrollAwayAllowed(scrollDirection)
  return self.canScroll
end

function ExerciseCardIntroText:isTapNavigateAwayAllowed(tapDirection)
  return self.canScroll
end

function ExerciseCardIntroText:onEnter()
  self.super.onEnter(self)
  if self.overlay then
    self.overlay:start()
  end
end

function ExerciseCardIntroText:reset()
  self.super.reset(self)
  if self.overlay and self.navigationArrow then
    self.navigationArrow:setOpacity(0)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function addIntroOverlay(self, title)
  local cycles = 2
  local callback = function()
    self.canScroll = true
    local action = AHSpawn.new()
    action:addAction(AHOpacityTo.new(self.overlay, 0.5, AHEaseExponentialIn.new(), 0))
    if self.navigationArrow then
      action:addAction(AHDelayed.new(0.3, AHOpacityTo.new(self.navigationArrow, 0.5, AHEaseExponentialIn.new(), 1)))
    end
    wait(action:start())
    self.overlay:stop()
    self.overlay:removeSelf()
    self.overlay = nil
  end
  local description = nil
  self.overlay = ExerciseCardLoaderOverlay.new(title, description, cycles, callback)

  self:insertAtCenter(self.overlay)
end

-- Constructor ------------------------

function ExerciseCardIntroText.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardIntroText)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local label = CardCarouselTextView.new(data.text, CardCarouselCardCommon.CONTENT_WIDTH, ExerciseCardIntroText.TEXT_ATTRIBUTES)
  self:insertAtCenter(label)

  self.canScroll = false

  addIntroOverlay(self, string.upper(data.title))

  return self
end
