--- ExerciseCardPassage
-- @author Created by Jay Austria on April 30, 2015.

class("ExerciseCardPassage")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselNavigator")
require("containers.card_carousel.CardCarouselCardCommon")
require("widgets.TouchScroller")
require("ExercisePlainButton")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.TISA_MEDIUM, CardCarouselCardCommon.COLORS.DARK_GREY),
  i = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.TISA_ITALIC, CardCarouselCardCommon.COLORS.DARK_GREY)
}

local BOTTOM_BUTTON_SPACING = Screen.getHeightPart(0.04)

local Z_BACKGROUND = 10
local Z_SCROLLER = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardPassage:reset()
  local cardGlobals = self:getCardGlobals()
  cardGlobals.readingWordsCount = self.wordsCount
end

function ExerciseCardPassage:onEnter()
  self.super.onEnter(self)
  self.timer:start()
  self.scroller:setTouchEnabled(true)
  self.bottomButton:setTouchEnabled(true)
  self.scroller:setInertialScrollingEnabled(true)
end

function ExerciseCardPassage:onLeaveComplete()
  self.super.onLeaveComplete(self)
  self.scroller:setTouchEnabled(false)
  self.bottomButton:setTouchEnabled(false)
  self.scroller:setInertialScrollingEnabled(false)
end

function ExerciseCardPassage:isScrollAwayAllowed(scrollDirection)
  return false
end

function ExerciseCardPassage:isTapNavigateAwayAllowed(tapDirection)
  return false
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function getWordsCount(text)
  local wordsCount = 0
  local luaPatternToExtractWords = "%a+"
  for word in string.gmatch(text, luaPatternToExtractWords) do
    wordsCount = wordsCount + 1
  end
  return wordsCount
end

-- Constructor ------------------------

function ExerciseCardPassage.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardPassage)

  local background = CardCarouselCardCommon.newStandardBackground()
  self:insertAtCenter(background, Z_BACKGROUND)

  local scrollerContents = Column.new(BOTTOM_BUTTON_SPACING)

  local labelWidth = self:getWidth() - CardCarouselCardCommon.MARGINS.HORIZONTAL * 2.0
  local label = CardCarouselTextView.new(data.text, labelWidth, TEXT_ATTRIBUTES)
  local labelHeight = label:getHeight()
  scrollerContents:insert(label)

  self.wordsCount = getWordsCount(data.text)

  local bottomButton = ExercisePlainButton.new("I'm finished reading.", function()
    self:forceScrollNavigation(CardCarouselNavigator.DIRECTION_LEFT, CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL)

    self.timer:stop()
    local readingDuration = self.timer:getTime()
    cardGlobals.readingDuration = readingDuration
  end)
  scrollerContents:insert(bottomButton)
  self.bottomButton = bottomButton

  scrollerContents:alignChildren()

  local scroller = TouchScroller.new(labelWidth, self:getHeight() - CardCarouselCardCommon.MARGINS.VERTICAL * 2.0, scrollerContents)
  scroller:setHandoffOnScrollDownEnabled(true)
  scroller:setHandoffOnHorizontalSwipeEnabled(true)
  scroller:setScrollDecayFactor(CardCarouselCardCommon.DEFAULT_TOUCH_SCROLLER_DECAY_FACTOR)
  self:insertAtCenter(scroller, Z_SCROLLER)
  scroller:setTouchEnabled(false)
  self.scroller = scroller

  local timer = MOAITimer.new()
  timer:setSpan(math.huge)
  self.timer = timer

  return self
end
