--- ExerciseCardPassageTitle
-- @author Created by Jay Austria on May 30, 2015.

class("ExerciseCardPassageTitle")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TITLE_TEXT_ATTRIBUTES = {
  default = {font = Font.new(FontFaces.TISA_OFFC, 46), color = CardCarouselCardCommon.COLORS.DARK_GREY, spacing = 14}
}

local SUBTITLE_TEXT_ATTRIBUTES = {
  default = {font = Font.new(FontFaces.TISA_OFFC, 16), color = CardCarouselCardCommon.COLORS.LIGHT_GREY, spacing = 10}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardPassageTitle:onEnter()
  self.instructionLabelButton:setTouchEnabled(true)
  self.super.onEnter(self)
end

function ExerciseCardPassageTitle:onLeaveBegin()
  self.instructionLabelButton:setTouchEnabled(false)
  self.super.onLeaveBegin(self)
end

function ExerciseCardPassageTitle:setScrollProgress(progress, isHorizontal)
  self.untimedAction:applyAtTime(progress)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createTappableInstructionLabel(self, workArea)
  local instructionLabel = Label.new("SWIPE UP TO START", Font.new(FontFaces.DIN_OT_MEDIUM, 16))
  instructionLabel:setColor(unpack(CardCarouselCardCommon.COLORS.LIGHT_GREY))
  instructionLabel:setLoc(workArea:getWidthPart(0.5), 0)
  workArea:insert(instructionLabel)

  local scaleLargerAmount = 1.6
  local buttonWidth = instructionLabel:getWidth() * scaleLargerAmount
  local buttonHeight = instructionLabel:getHeight() * scaleLargerAmount
  local onTouchDown = function()
    self:forceTapNavigation(CardCarouselNavigator.TAP_DIRECTION_DOWN, CardCarouselNavigator.SCROLL_ANIMATION_DURATION_VERTICAL)
  end
  local onTouchUp = nil
  local onTouchCancel = nil
  local instructionLabelButton = Button.addVirtualButton(instructionLabel,
                                                         buttonWidth,
                                                         buttonHeight,
                                                         onTouchDown,
                                                         onTouchUp,
                                                         onTouchCancel)
  instructionLabelButton:setTouchEnabled(false)
  self.instructionLabelButton = instructionLabelButton
end

-- Constructor ------------------------

function ExerciseCardPassageTitle.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardPassageTitle)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(workArea)

  local title = data.title
  local subtitle = data.subtitle

  local staggerAmount = CardCarouselCardCommon.DEFAULT_STAGGER_AMOUNT
  local scrollAction = AHSpawn.new()
  local col = Column.new(Screen.getWidthPart(0.01), Alignment.LEFT)
  local titleLabel = CardCarouselTextView.new(title, workArea:getWidth(), TITLE_TEXT_ATTRIBUTES, Alignment.CENTER)
  col:insert(titleLabel)
  titleLabel:prepareForSlideIn(staggerAmount * 2, staggerAmount)
  scrollAction:addAction(titleLabel:newSlideInAction())

  if subtitle then
    local subtitleLabel = CardCarouselTextView.new(subtitle, workArea:getWidth(), SUBTITLE_TEXT_ATTRIBUTES, Alignment.CENTER)
    col:insert(subtitleLabel)
    subtitleLabel:prepareForSlideIn(staggerAmount * 3, staggerAmount)
    scrollAction:addAction(subtitleLabel:newSlideInAction())
  end

  col:alignChildren()
  col:setLoc(workArea:getWidthPart(0.5), workArea:getHeightPart(0.55))
  workArea:insert(col)

  createTappableInstructionLabel(self, workArea)

  self.untimedAction = AHUntimedAction.new(scrollAction)
  self.untimedAction:start()

  return self
end
