--- ExerciseCardQuote
-- @author Created by Aaron Barrett on April 21, 2015.

class("ExerciseCardQuote")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_BACKGROUND = 10
local Z_PORTRAIT = 20
local Z_FOG = 30
local Z_MARK = 40
local Z_QUOTE_LABEL = 50
local Z_TITLE_LABEL = 70
local Z_SEPARATORS = 80

local PORTRAIT_START_X = Screen.getWidthPart(0.3)
local PORTRAIT_END_X = Screen.getWidthPart(0.35)
local PORTRAIT_START_Y = Screen.getHeightPart(0.6)
local PORTRAIT_END_Y = Screen.getHeightPart(0.65)

local PORTRAIT_START_SCALE = 1.0
local PORTRAIT_END_SCALE = 1.05
local PORTRAIT_FLOAT_DURATION = 11


local RIGHT_MARK_START_X = Screen.getWidthPart(1.5)
local LEFT_MARK_START_X = -Screen.getWidthPart(0.4)
local LEFT_MARK_END_X = Screen.getWidthPart(0.14)

local MARK_START_SCALE = 2.5
local MARK_END_SCALE = 0.8

local QUOTE_LABEL_START_X = Screen.getWidthPart(0.55)
local QUOTE_LABEL_END_X = Screen.getWidthPart(0.5)

local TITLE_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.BLUE)
}

local SUBTITLE_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.GREY)
}

local RIGHT_MARGIN_X = Screen.width

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardQuote:reset()
  local portrait = self.portrait
  portrait:setScl(self.portraitStartScale)
  portrait:setLoc(PORTRAIT_START_X, PORTRAIT_START_Y)

  local leftMark = self.leftMark
  leftMark:setX(LEFT_MARK_START_X)
  leftMark:setScl(MARK_START_SCALE)

  local rightMark = self.rightMark
  rightMark:setX(RIGHT_MARK_START_X)
  rightMark:setScl(MARK_START_SCALE)

  self.titleLabel:setX(self.titleStartX)
  self.subtitleLabel:setX(self.subtitleStartX)
end

function getTitleLabelProgress(progress)
  return MathUtils.mapLinear(progress, 0.35, 0.9, 0.0, 1.0)
end

function getSubtitleLabelProgress(progress)
  return MathUtils.mapLinear(progress, 0.45, 1.0, 0.0, 1.0)
end

function ExerciseCardQuote:setScrollProgress(progress, isHorizontal)
  if (isHorizontal == nil) then
    isHorizontal = true
  end
  local isScrollingVertically = (not isHorizontal)
  local forcedFulllyComplete = 1.0

  if isScrollingVertically then
    progress = forcedFulllyComplete
  end

  self.untimedAction:applyAtTime(progress)
end

function ExerciseCardQuote:createAnimationAction()
  local portrait = self.portrait
  local leftMark = self.leftMark
  local rightMark = self.rightMark
  local rightMarkEndX = self.rightMarkEndX

  local portraitEase = nil

  local portraitAction = AHLoop.new(
                                    AHSequence.new(
                                                   AHSpawn.new(
                                                               AHScaleTo.new(portrait, PORTRAIT_FLOAT_DURATION, portraitEase, self.portraitEndScale),
                                                               AHLocationTo.new(portrait, PORTRAIT_FLOAT_DURATION, portraitEase, PORTRAIT_END_X, PORTRAIT_END_Y)
                                                               ),
                                                   AHSpawn.new(
                                                               AHScaleTo.new(portrait, PORTRAIT_FLOAT_DURATION, portraitEase, self.portraitStartScale),
                                                               AHLocationTo.new(portrait, PORTRAIT_FLOAT_DURATION, portraitEase, PORTRAIT_START_X, PORTRAIT_START_Y)
                                                               )
                                                   )
                                    )

  local markEase = AHEaseExponentialOut.new()

  local leftMarkAction = AHSpawn.new(
                                     AHLocationXTo.new(leftMark, 0.8, markEase, LEFT_MARK_END_X),
                                     AHScaleTo.new(leftMark, 0.8, markEase, MARK_END_SCALE)
                                     )

  local rightMarkAction = AHSpawn.new(
                                      AHLocationXTo.new(rightMark, 0.8, markEase, rightMarkEndX),
                                      AHScaleTo.new(rightMark, 0.8, markEase, MARK_END_SCALE)
                                      )

  local slideDuration = 1.0
  local slideStagger = 0.2
  local ease = AHEaseExponentialOut.new()
  local textSlideAction = AHStaggeredSpawn.new(slideStagger,
                                      AHLocationXTo.new(self.titleLabel, slideDuration, ease, self.titleEndX),
                                      AHLocationXTo.new(self.subtitleLabel, slideDuration, ease, self.subtitleEndX)
                                      )

  local animationAction = AHSpawn.new(
                                      portraitAction,
                                      leftMarkAction,
                                      rightMarkAction,
                                      textSlideAction
                                      )

  return animationAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createMark()
  local mark = Sprite.newWithName("big_quote_mark.png")
  mark:setColor(unpack(CardCarouselCardCommon.COLORS.GREY))
  mark:setOpacity(0.25)
  return mark
end

-- Constructor ------------------------

function ExerciseCardQuote.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardQuote)

  local background = CardCarouselCardCommon.newStandardBackground()
  self:insertAtCenter(background, Z_BACKGROUND)

  local portrait = AssetHelper.createPhotoSpriteForConcept(data.concept, data.image)
  self:insert(portrait, Z_PORTRAIT)
  self.portrait = portrait

  -- Improve look of portrait on smaller devices
  local idealImageHeightProportion = 0.77 --polished on iPhone6
  portraitScale = idealImageHeightProportion / (portrait:getHeight() / Screen.height)
  self.portraitStartScale = PORTRAIT_START_SCALE * portraitScale
  self.portraitEndScale = PORTRAIT_END_SCALE * portraitScale

  local fog = Group.new(self:getWidth(), self:getHeightPart(0.6))

  -- NOTE: On certain devices (but not on others), Moai seems to render gradients without premultiplied alpha.  This
  -- usually results in only-slightly-messed-up visuals that most people won't notice.  In this particular case, however,
  -- the effect is very noticeable and ends up cutting off the image behind it.  To get around this problem, use a scaled
  -- sprite image in place of a `RectangleGradient` for now.
  local fogTop = Sprite.newWithName("white_vertical_gradient.png")

  local horizontalFudgeToAvoidGaps = self:getWidthPart(0.3)
  local fogTopWidth = self:getWidth() + horizontalFudgeToAvoidGaps
  local fogTopHeight = self:getHeightPart(0.2)
  fogTop:setEffectiveDims(fogTopWidth, fogTopHeight)

  local verticalFudgeToAvoidGaps = 3.0
  fogTop:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5) - verticalFudgeToAvoidGaps)
  fog:insert(fogTop)

  local fogBottom = FilledRectangle.new(self:getWidth(), self:getHeightPart(0.4), fogBottomColor)
  fogBottom:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.2))
  fog:insert(fogBottom)
  fog:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.3))
  self:insert(fog, Z_FOG)

  local leftMark = createMark()
  self:insert(leftMark, Z_MARK)
  self.leftMark = leftMark

  local rightMark = createMark()
  self:insert(rightMark, Z_MARK)
  self.rightMark = rightMark

  local quoteText = data.quote
  local quoteWidth = self:getWidth() - 2.0 * CardCarouselCardCommon.MARGINS.HORIZONTAL

  local cachedFont = CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM
  if (Screen.hasNotch()) then
    cachedFont = CardCarouselCardCommon.CACHED_FONTS.LIGHT_SMALL
  end

  local quoteAttributes = {
    default = CardCarouselCardCommon.newTextAttribute(cachedFont, CardCarouselCardCommon.COLORS.BLACK),
  }

  local quoteLabel = CardCarouselTextView.new(quoteText, quoteWidth, quoteAttributes, Alignment.LEFT)
  local quoteLabelY = math.max(self:getHeightPart(0.25), self:getHeightPart(0.05) + quoteLabel:getHeightPart(0.5))
  quoteLabel:setLineSpacing(0)
  quoteLabel:setLoc(self:getWidthPart(0.5), quoteLabelY)
  self:insert(quoteLabel, Z_QUOTE_LABEL)
  self.quoteLabel = quoteLabel

  local titleLabelY = self:getHeightPart(0.68)

  local quoteYOffset = leftMark:getHeightPart(0.35)
  local markY = quoteLabelY + quoteLabel:getHeightPart(0.5) - quoteYOffset
  leftMark:setY(markY)

  self.rightMarkEndX = LEFT_MARK_END_X + leftMark:getWidthPart(0.95)
  rightMark:setY(markY)

  local titleWidth = self:getWidthPart(0.35)
  local titleSubtitleCol = Column.new(0, Alignment.LEFT)

  local titleText = data.name:upper()
  local titleLabel = CardCarouselTextView.new(titleText, titleWidth, TITLE_TEXT_ATTRIBUTES)
  titleLabel:setLineSpacing(0)
  titleSubtitleCol:insert(titleLabel)

  local subtitleText = data.famousFor:upper()
  local subtitleLabel = CardCarouselTextView.new(subtitleText, titleWidth, SUBTITLE_TEXT_ATTRIBUTES)
  titleSubtitleCol:insert(subtitleLabel)

  titleSubtitleCol:alignChildren()
  titleSubtitleCol:setAnchorPointRight()
  titleSubtitleCol:setLoc(RIGHT_MARGIN_X, titleLabelY)
  self:insert(titleSubtitleCol, Z_TITLE_LABEL)

  local titleLabelWidth = titleLabel:getWidth()
  local subtitleLabelWidth = subtitleLabel:getWidth()

  local titlePaddingX = 10
  local titleLabelEndX = self:getWidth() - titleLabel:getWidth() - titlePaddingX

  local staggerAmount = CardCarouselCardCommon.DEFAULT_STAGGER_AMOUNT
  quoteLabel:prepareForSlideIn(staggerAmount * 2, staggerAmount)

  local untimedAction = AHUntimedAction.new(quoteLabel:newSlideInAction())
  untimedAction:start()
  self.untimedAction = untimedAction

  self.titleLabel = titleLabel
  self.subtitleLabel = subtitleLabel
  self.titleEndX = titleLabel:getX()
  self.subtitleEndX = subtitleLabel:getX()

  local distance = Screen.getWidthPart(0.4)
  titleLabel:setX(titleLabel:getX() + distance)
  subtitleLabel:setX(subtitleLabel:getX() + distance)

  self.titleStartX = titleLabel:getX()
  self.subtitleStartX = subtitleLabel:getX()

  return self
end
