--- ExerciseCardSentenceGrouping
-- @author Created by Aaron Barrett on May 12, 2015.

class("ExerciseCardSentenceGrouping")


require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("ExerciseWPMSlider")
require("widgets.progress.TimeTimer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TIMER_SECONDS = 60

local Z_BACKGROUND = 10
local Z_SHOWING_LABEL = 20
local Z_SLIDER = 30
local Z_TIMER = 40
local Z_SEPARATORS = 50

local SHOWING_LABEL_WIDTH = 260.0
local WORDS_PER_SHOWING = 5

local WORDS_PER_MINUTE_MIN = 100
local WORDS_PER_MINUTE_MAX = 900

local TEXT_ATTRIBUTES = {default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.TISA_MEDIUM, CardCarouselCardCommon.COLORS.GREY)}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function convertToShowings(text)
  local showings = {}

  local currentShowing = ""
  local spacesCount = 0

  local luaPatternToExtractCharacters = "."
  for character in string.gmatch(text, luaPatternToExtractCharacters) do
    if (character == " ") then
      spacesCount = spacesCount + 1
      if (spacesCount >= WORDS_PER_SHOWING) then
        table.insert(showings, currentShowing)
        spacesCount = 0
        currentShowing = ""
      else
        currentShowing = currentShowing .. character
      end
    else
      currentShowing = currentShowing .. character
    end
  end

  table.insert(showings, currentShowing)

  return showings
end

local function startNextShowing(self)
  local showings = self.showings
  local showingIndex = self.showingIndex
  local slider = self.slider

  local showingsCount = #showings

  showingIndex = showingIndex + 1
  if (showingIndex > showingsCount) then
    showingIndex = 1
  end

  if (self.showingLabel ~= nil) then
    local labelToRemoveAfterDelay = self.showingLabel
    runInCoroutine(function()
      -- Wait for a frame to get around Moai's wait-one-frame-before-actually-showing-anything-added-to-the-screen bug.
      waitForFrames(1)
      labelToRemoveAfterDelay:removeSelf()
    end)
  end

  local showing = showings[showingIndex]
  local showingLabel = CardCarouselTextView.new(showing, SHOWING_LABEL_WIDTH, TEXT_ATTRIBUTES, Alignment.CENTER)
  self:insertAtCenter(showingLabel, Z_SHOWING_LABEL)
  self.showingLabel = showingLabel

  self.showingIndex = showingIndex

  local wordsPerMinute = slider:getValue()
  local secondsPerMinute = 60.0
  local delayDuration = WORDS_PER_SHOWING * secondsPerMinute / (wordsPerMinute)

  local showingAction = AHDelayed.new(delayDuration,
                                      function()
                                        startNextShowing(self)
                                      end
                                      )
  self.showingAction = showingAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------


function ExerciseCardSentenceGrouping:reset()
  self.showingIndex = 0

  if (self.timer == nil) then
    self.timer = TimeTimer.new(FontFaces.DIN_OT_BOLD, Screen.getWidthPart(0.06), TIMER_SECONDS, function()
      self:forceScrollNavigation(CardCarouselNavigator.DIRECTION_LEFT, CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL)
    end)
    self.timer:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.9))
    self.timer:setColor(unpack(CardCarouselCardCommon.COLORS.BLUE))
    self:insert(self.timer, Z_TIMER)
  end

  if self.showingLabel then
    self.showingLabel:removeSelf()
    self.showingLabel = nil
  end

  if self.showingAction then
    self.showingAction:stop()
    self.showingAction = nil
  end

  local showings = convertToShowings(self.text)
  self.showings = showings

  self.slider:reset()
  self.slider:prepareForAnimateIn()
  self.timer:setOpacity(0)
end

function ExerciseCardSentenceGrouping:onEnter()
  self.super.onEnter(self)
  self.slider:setTouchEnabled(true)

  local delayedStart = function()
    waitFor(1.2)
    self.timer:start()
    startNextShowing(self)
  end
  runInCoroutine(delayedStart)
end

function ExerciseCardSentenceGrouping:onLeaveComplete()
  self.super.onLeaveComplete(self)
  self.slider:setTouchEnabled(false)
  if (self.showingAction ~= nil) then
    self.showingAction:stop()
  end
  self.timer:stop()
  self.timer:setTime(TIMER_SECONDS)
end

function ExerciseCardSentenceGrouping:createAnimationAction()
  local timerAction = AHDelayed.new(1.4, AHOpacityTo.new(self.timer, 0.5, nil, 1.0))
  local animateAction = AHSpawn.new(timerAction, self.slider:getAnimateInAction())

  return animateAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardSentenceGrouping.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardSentenceGrouping)

  self.text = data.text

  local background = CardCarouselCardCommon.newStandardBackground()
  self:insertAtCenter(background, Z_BACKGROUND)

  local slider = ExerciseWPMSlider.new(WORDS_PER_MINUTE_MIN, WORDS_PER_MINUTE_MAX)
  slider:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.1))
  slider:createTip()
  slider:setTouchEnabled(false)
  self:insert(slider, Z_SLIDER)
  self.slider = slider

  return self
end
