--- ExerciseCardSpeechAnalysis
-- @author Created by karl on July 02, 2015.

class("ExerciseCardSpeechAnalysis")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("data.visualization.AudioVisualizer")
require("widgets.progress.TimeTimer")
require("AudioTimer")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ExerciseCardSpeechAnalysis.totalCards = 0

local VISUALIZER_WIDTH = Screen.getWidthPart(1)
local VISUALIZER_HEIGHT = Screen.getHeightPart(0.13)
local VISUALIZER_COLOR = CardCarouselCardCommon.COLORS.BLUE

local FOOTER_FONT_SIZE = 15
local FOOTER_HEIGHT = FOOTER_FONT_SIZE * 5
local FOOTER_MARGIN = Screen.getWidthPart(0.08)

local TEXT_VERTICAL_MOVEMENT = 10

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newFadeOverlayFunction(self, duration, ease, value)
  local action = AHSpawn.new(AHOpacityTo.new(self.audioOverlay, duration, ease, value))
  if self.navigationArrow then
    action:addAction(AHOpacityTo.new(self.navigationArrow, duration, ease, 1 - value))
  end
  return action
end

local function fadeToAudio(self)
  return runInCoroutine(function()
    self.analysis:setLoc(self:getWidthPart(0.5), self:getHeight())
    self.analysis:setOpacity(1)
    self.playAgainButton:setTouchEnabled(false)
    wait(AHStaggeredSpawn.new(0.5,
      AHDelayed.new(0.1, AHSpawn.new(
        AHOpacityTo.new(self.analysis, 0.4, nil, 0),
        AHLocationYTo.new(self.analysis, 0.4, AHEaseSineOut.new(), self:getHeight() - TEXT_VERTICAL_MOVEMENT)
      )),
      newFadeOverlayFunction(self, 0.4, nil, 1)
    ):start())
    self.isOnAudioPanel = true
  end)
end

local function fadeToAnalysis(self)
  return runInCoroutine(function()
    self.analysis:setLoc(self:getWidthPart(0.5), self:getHeight() - TEXT_VERTICAL_MOVEMENT)
    self.analysis:setOpacity(0)
    wait(AHSpawn.new(
      newFadeOverlayFunction(self, 0.2, AHEaseExponentialOut.new(), 0),
      AHDelayed.new(0.1, AHSpawn.new(
        AHOpacityTo.new(self.analysis, 0.4, nil, 1),
        AHLocationYTo.new(self.analysis, 0.4, AHEaseSineOut.new(), self:getHeight())
      ))
    ):start())
    self.playAgainButton:setTouchEnabled(true)
    self.isOnAudioPanel = false
  end)
end

local function playAudio(self)
  return runInCoroutine(function()
    wait(fadeToAudio(self))
    if self.isActiveCard and not self.audioTimer:isPlaying() then
      self.audioTimer:start()
      self.visualizer:visualizeSound(self.audioTimer:getSound())
    end
  end)
end

local function stopAudio(self)
  self.audioTimer:stop()
  self.visualizer:settleAndStop()
end

local function newAudioTimer(font, size, filename, duration)
  local audioTimer = TimeProgressIndicator.new(font, size, duration)
  ProgressTimer.makeAudioTimer(audioTimer, filename, duration)
  audioTimer:setColor(unpack(CardCarouselCardCommon.COLORS.BLUE))

  return audioTimer
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardSpeechAnalysis:reset()
  self.super.reset(self)
  self.cardIndexLabel:setText(tostring(self.cardIndex) .. " of " .. tostring(ExerciseCardSpeechAnalysis.totalCards))
  if self.isOnAudioPanel and self.navigationArrow then
    self.navigationArrow:setOpacity(0)
  end

  if (self.audioTimer == nil) then
    self.audioTimer = newAudioTimer(FontFaces.DIN_OT_LIGHT, Screen.getWidthPart(0.1), self.audioSoundPath, self.audioDuration)
    self.audioTimer:setLoc(self.visualizer:getX(), self.visualizer:getY() + self.visualizer:getHeight())
    self.audioOverlay:insert(self.audioTimer)
    self.audioOverlay:linkColorAttrToChildren(true)

    self.audioTimer:setTimeoutHandler(function()
      runInCoroutine(function()
        wait(fadeToAnalysis(self))
      end)
    end)
  end
end

function ExerciseCardSpeechAnalysis:onEnter()
  self.super.onEnter(self)
  if self.isOnAudioPanel then
    playAudio(self)
  end
end

function ExerciseCardSpeechAnalysis:onLeaveComplete()
  self.super.onLeaveComplete(self)
  stopAudio(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function addAudioOverlay(self, data)
  local audio = data.audio
  local duration = tonumber(data.duration)
  local soundPath = AssetHelper.getAbsolutePathForSound(data.concept, audio)
  local theme = data.theme

  self.audioDuration = duration
  self.audioSoundPath = soundPath

  self.audioOverlay = FilledRectangle.new(Screen.width, Screen.height, CardCarouselCardCommon.COLORS.WHITE)
  self.visualizer = AudioVisualizer.new(VISUALIZER_WIDTH, VISUALIZER_HEIGHT, VISUALIZER_COLOR)
  self.audioOverlay:insertAtCenter(self.visualizer)

  local themeLabel = CardCarouselCardCommon.newLabel(theme, FontFaces.DIN_OT_MEDIUM, 19, CardCarouselCardCommon.COLORS.GREY)
  themeLabel:setLoc(self:getWidthPart(0.5), self.audioOverlay:getHeightPart(0.9))
  self.audioOverlay:insert(themeLabel)
  themeLabel:inheritColor(self.audioOverlay)

  self.audioOverlay:setAnchorPointTop()
  self.audioOverlay:setLoc(self:getWidthPart(0.5), self:getHeight())
  self:insert(self.audioOverlay)
end

local function addFooter(self)
  local footer = Group.new(self:getWidth(), FOOTER_HEIGHT)

  self.cardIndexLabel = CardCarouselCardCommon.newLabel("", FontFaces.DIN_OT_MEDIUM, FOOTER_FONT_SIZE, CardCarouselCardCommon.COLORS.GREY)
  self.cardIndexLabel:setAnchorPointTop()
  self.cardIndexLabel:setLoc(footer:getWidthPart(0.5), footer:getHeight())
  footer:insert(self.cardIndexLabel)

  footer:setAnchorPointBottom()
  footer:setLoc(self:getWidthPart(0.5), 0)
  self:insert(footer)
end

local function newPlayAgainButton(callback)
  local color = CardCarouselCardCommon.COLORS.LIGHT_GREY
  local fontSize = 15
  local label = CardCarouselCardCommon.newLabel("PLAY AGAIN", FontFaces.DIN_OT_MEDIUM, fontSize, color)
  local sprite = Sprite.newWithName("replay.png")
  sprite:setEffectiveDims(fontSize * 0.7)
  sprite:setColor(unpack(color))
  local row = Row.new(fontSize / 2)
  row:insert(sprite)
  row:insert(label)
  row:alignChildren()
  Button.makeButton(row, callback)
  return row
end

local function addAnalysis(self, data)
  local theme = data.theme
  local analysis = data.analysis

  local font = FontFaces.DIN_OT_LIGHT
  local headerFontSize = Screen.getWidthPart(0.08)
  local analysisFontSize = headerFontSize * 0.8
  local col = Column.new(Screen.getWidthPart(0.07), Alignment.LEFT)
  col:insert(CardCarouselCardCommon.newMultilineLabel(theme, font, headerFontSize, CardCarouselCardCommon.CONTENT_WIDTH, nil, CardCarouselCardCommon.COLORS.BLUE))
  col:insert(CardCarouselCardCommon.newMultilineLabel(analysis, font, analysisFontSize, CardCarouselCardCommon.CONTENT_WIDTH, nil, CardCarouselCardCommon.COLORS.DARK_GREY))
  self.playAgainButton = newPlayAgainButton(function() playAudio(self) end)
  self.playAgainButton:setTouchEnabled(false)
  col:insert(self.playAgainButton)
  col:alignChildren()

  self.analysis = FilledRectangle.new(Screen.width, Screen.height, CardCarouselCardCommon.COLORS.WHITE)
  self.analysis:setAnchorPointTop()
  self.analysis:setLoc(self:getWidthPart(0.5), self:getHeight())
  self.analysis:insertAtCenter(col)
  self:insert(self.analysis)

  self.analysis:linkColorAttrToChildren(true)
end

-- Constructor ------------------------

function ExerciseCardSpeechAnalysis.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardSpeechAnalysis)

  ExerciseCardSpeechAnalysis.totalCards = ExerciseCardSpeechAnalysis.totalCards + 1
  self.cardIndex = ExerciseCardSpeechAnalysis.totalCards
  self.isOnAudioPanel = true

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  addAnalysis(self, data)
  addAudioOverlay(self, data)
  addFooter(self)

  return self
end
