--- ExerciseCardSpeechIntro
-- @author Created by karl on July 02, 2015.

class("ExerciseCardSpeechIntro")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("ExerciseCardLoaderOverlay")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardSpeechIntro:isScrollAwayAllowed(scrollDirection)
  return self.canScroll
end

function ExerciseCardSpeechIntro:isTapNavigateAwayAllowed(tapDirection)
  return self.canScroll
end

function ExerciseCardSpeechIntro:onEnter()
  self.super.onEnter(self)
  if self.overlay then
    self.overlay:start()
  end
end

function ExerciseCardSpeechIntro:reset()
  self.super.reset(self)
  if self.overlay then
    if self.navigationArrow then
      self.navigationArrow:setOpacity(0)
    end
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function addLoadingOverlay(self)
  local cycles = 2
  local callback = function()
    self.canScroll = true
    local action = AHSpawn.new()
    action:addAction(AHOpacityTo.new(self.overlay, 0.5, AHEaseExponentialIn.new(), 0))
    if self.navigationArrow then
      action:addAction(AHDelayed.new(0.3, AHOpacityTo.new(self.navigationArrow, 0.5, AHEaseExponentialIn.new(), 1)))
    end
    wait(action:start())
    self.overlay:stop()
    self.overlay:removeSelf()
    self.overlay = nil
  end
  local title = "CHOOSING A SPEECH"
  local description = "Speeches are abridged for educational purposes and narrated by an actor."
  self.overlay = ExerciseCardLoaderOverlay.new(title, description, cycles, callback)

  self:insertAtCenter(self.overlay)
end

-- Constructor ------------------------

function ExerciseCardSpeechIntro.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardSpeechIntro)
  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local speaker = data.speaker
  local context = data.context
  local image = data.image

  local titleFontSize = 19
  local contextFontSize = 30
  local image = AssetHelper.createPhotoSpriteForConcept(data.concept, image)
  image:setAnchorPointBottom()
  image:setLoc(self:getWidthPart(0.5), 0)
  self:insert(image)

  local col = Column.new(titleFontSize * 0.8)
  col:insert(CardCarouselCardCommon.newLabel(speaker:upper(), FontFaces.DIN_OT_BOLD, titleFontSize, CardCarouselCardCommon.COLORS.LIGHT_GREY))
  col:insert(CardCarouselCardCommon.newMultilineLabel(context, FontFaces.DIN_OT_LIGHT, contextFontSize, Screen.getWidthPart(0.8), Alignment.CENTER, CardCarouselCardCommon.COLORS.DARK_GREY))
  col:alignChildren()
  col:setAnchorPointTop()
  col:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.9))
  self:insert(col)

  self.canScroll = false

  addLoadingOverlay(self)

  return self
end
