--- ExerciseCardSpeedReadingTestComplete
-- @author Created by Aaron Barrett on May 08, 2015.

class("ExerciseCardSpeedReadingTestComplete")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.BLUE, {bouncy = true})
}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWordsPerMinute(cardGlobals)
  local secondsPerMinute = 60.0
  return MathUtils.round(cardGlobals.readingWordsCount * secondsPerMinute / cardGlobals.readingDuration)
end

local function newWorkAreaWithText(text)
  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  local label = CardCarouselTextView.new(text, workArea:getWidth(), TEXT_ATTRIBUTES)
  workArea:insertAtCenter(label)
  workArea.label = label

  return workArea
end

local function newSuccessWorkArea(cardGlobals)
  local questionsCount = cardGlobals.multipleChoiceQuestionsCount
  local correctCount = cardGlobals.multipleChoiceCorrectCount
  local wordsPerMinute = getWordsPerMinute(cardGlobals)
  local text = string.format("You answered <highlight>%d out of %d</highlight> correctly.<br/><br/>Your reading speed is <highlight>%d WPM</highlight>.", correctCount, questionsCount, wordsPerMinute)
  return newWorkAreaWithText(text)
end

local function newFailWorkArea(cardGlobals)
  local text = "You didn't answer accurately enough to be given a WPM reading.<br/><br/>Exit and return to this exercise to try again."
  return newWorkAreaWithText(text)
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardSpeedReadingTestComplete:createAnimationAction()
  local bouncyViews = {self.label}
  local animationAction = CardCarouselCardCommon.createBouncyTextAction(bouncyViews)
  return animationAction
end

function ExerciseCardSpeedReadingTestComplete:reset()
  if self.workArea then
    self.workArea:removeSelf()
  end

  local cardGlobals = self:getCardGlobals()

  local proportionCorrect = cardGlobals.multipleChoiceCorrectCount / cardGlobals.multipleChoiceQuestionsCount

  local workArea
  if proportionCorrect >= 0.75 then
    workArea = newSuccessWorkArea(cardGlobals)
  else
    workArea = newFailWorkArea(cardGlobals)
  end

  self.label = workArea.label

  self:insertAtCenter(workArea)
  self.workArea = workArea
end

function ExerciseCardSpeedReadingTestComplete:isScrollEnterAllowed(scrollDirection)
  return scrollDirection == CardCarouselNavigator.DIRECTION_RIGHT or
         scrollDirection == CardCarouselNavigator.DIRECTION_DOWN
end

function ExerciseCardSpeedReadingTestComplete:isTapNavigateEnterAllowed(tapDirection)
  return tapDirection == CardCarouselNavigator.TAP_DIRECTION_BACK
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardSpeedReadingTestComplete.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardSpeedReadingTestComplete)

  local background = CardCarouselCardCommon.newStandardBackground()
  self:insertAtCenter(background)

  self.label = nil

  return self
end
