--- ExerciseCardText
-- @author Created by Aaron Barrett on April 21, 2015.

class("ExerciseCardText")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")
require("widgets.TouchScroller")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_DARK_COLOR = ColorUtils.colorHex24ToStd(0x333333)
local HEADER_DARK_COLOR = ColorUtils.colorHex24ToStd(0x111111)
local TEXT_BLUE_COLOR = CardCarouselCardCommon.COLORS.BLUE

ExerciseCardText.TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.TISA_MEDIUM, TEXT_DARK_COLOR),
  h1 = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, HEADER_DARK_COLOR, {size = 23}),
  h2 = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, HEADER_DARK_COLOR),
  i = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.TISA_ITALIC, TEXT_DARK_COLOR),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.TISA_MEDIUM, TEXT_BLUE_COLOR),
  h1highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, TEXT_BLUE_COLOR, {size = 23}),
  divide = {itemType = CardCarouselTextView.ITEM_TYPE_DIVIDE}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardText:onEnter()
  self.super.onEnter(self)
  self.scroller:setTouchEnabled(true)
  self.scroller:setInertialScrollingEnabled(true)
  self.forwardTapArea:setTouchEnabled(true)
  self.backTapArea:setTouchEnabled(true)
end

function ExerciseCardText:onLeaveBegin()
  self.super.onLeaveBegin(self)
  self.forwardTapArea:setTouchEnabled(false)
  self.backTapArea:setTouchEnabled(false)
end

function ExerciseCardText:onLeaveComplete()
  self.super.onLeaveComplete(self)
  self.scroller:setTouchEnabled(false)
  self.scroller:setInertialScrollingEnabled(false)
end

-- Disable tap navigation through the CardCarouselNavigator and let this card handle
-- its own tap navigation through its own tap areas. This is done to work around
-- a limitation where the TouchScroller on this card swallows TouchBegan and
-- TouchEnded events with no way to hand them off to the card's touch handler.
function ExerciseCardText:handlesOwnTapNavigation()
  return true
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createTapAreas(self)
  local width = self:getWidth() - CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY
  local height = self:getHeight()
  local onTouchDown = nil
  local duration = CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL
  local willCheckNavigationConstraints = true
  local onTouchUp = function()
    self:forceTapNavigation(CardCarouselNavigator.DIRECTION_LEFT, duration, willCheckNavigationConstraints)
  end
  local onTouchCancel = nil
  local forwardTapArea = Button.new(width, height, onTouchDown, onTouchUp, onTouchCancel)
  forwardTapArea:setAnchorPointRight()
  self:insertAtRight(forwardTapArea)
  forwardTapArea:setTouchEnabled(false)
  self.forwardTapArea = forwardTapArea

  width = CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY
  local height = self:getHeight()
  onTouchUp = function()
    self:forceTapNavigation(CardCarouselNavigator.DIRECTION_RIGHT, duration, willCheckNavigationConstraints)
  end
  local backTapArea = Button.new(width, height, onTouchDown, onTouchUp, onTouchCancel)
  backTapArea:setAnchorPointLeft()
  self:insertAtLeft(backTapArea)
  backTapArea:setTouchEnabled(false)
  self.backTapArea = backTapArea
end

-- Constructor ------------------------

function ExerciseCardText.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardText)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local label = CardCarouselTextView.new(data.text, CardCarouselCardCommon.CONTENT_WIDTH, ExerciseCardText.TEXT_ATTRIBUTES)

  local scroller = TouchScroller.new(CardCarouselCardCommon.CONTENT_WIDTH, CardCarouselCardCommon.CONTENT_HEIGHT, label)
  scroller:setHandoffOnScrollDownEnabled(true)
  scroller:setHandoffOnScrollUpEnabled(true)
  scroller:setHandoffOnHorizontalSwipeEnabled(true)
  scroller:setScrollDecayFactor(CardCarouselCardCommon.DEFAULT_TOUCH_SCROLLER_DECAY_FACTOR)
  self:insertAtCenter(scroller, Z_SCROLLER)
  scroller:setTouchEnabled(false)
  self.scroller = scroller

  createTapAreas(self)

  return self
end
