--- ExerciseCardTextAndInstructions
-- @author Created by Jay Austria on May 30, 2015

class("ExerciseCardTextAndInstructions")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TOP_INSTRUCTIONS_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.BLUE),
}

local TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.BLUE),
}

local BOTTOM_INSTRUCTIONS_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.LIGHT_GREY)
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardTextAndInstructions:setScrollProgress(progress, isHorizontal)
  self.untimedAction:applyAtTime(progress)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardTextAndInstructions.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardTextAndInstructions)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(workArea)

  local topInstructions = data.topInstructions
  if topInstructions and #topInstructions > 0 then
    local topInstructionsLabel = CardCarouselTextView.new(topInstructions:upper(), workArea:getWidth(), TOP_INSTRUCTIONS_ATTRIBUTES, Alignment.CENTER)
    topInstructionsLabel:setLoc(workArea:getWidthPart(0.5), workArea:getHeight() - topInstructionsLabel:getHeightPart(0.5))
    workArea:insert(topInstructionsLabel)
  end

  local textLabel = CardCarouselTextView.new(data.text, workArea:getWidth(), TEXT_ATTRIBUTES)
  workArea:insertAtCenter(textLabel)

  local bottomInstructions = data.bottomInstructions
  if bottomInstructions and #bottomInstructions > 0 then
    local bottomInstructionsLabel = CardCarouselTextView.new(bottomInstructions:upper(), workArea:getWidth(), BOTTOM_INSTRUCTIONS_ATTRIBUTES, Alignment.CENTER)
    bottomInstructionsLabel:setLoc(workArea:getWidthPart(0.5), bottomInstructionsLabel:getHeightPart(0.5))
    workArea:insert(bottomInstructionsLabel)
  end

  local staggerAmount = CardCarouselCardCommon.DEFAULT_STAGGER_AMOUNT
  textLabel:prepareForSlideIn(staggerAmount * 2, staggerAmount)
  self.untimedAction = AHUntimedAction.new(textLabel:newSlideInAction())
  self.untimedAction:start()

  return self
end
