--- ExerciseCardTitle
-- @author Created by Aaron Barrett on April 21, 2015.

class("ExerciseCardTitle")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")
require("ExerciseIcon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DESCRIPTION_TEXT_COLOR = {0.76, 0.94, 0.99}
local SWIPE_TIP_COLOR = {0.94, 0.97, 0.98}

local Z_BACKGROUND = 10
local Z_MAIN_COLUMN = 20
local Z_SWIPE_TIP = 30
local Z_OVERLAY = 40

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createFadeOutAnimation(self)
  local fadeDuration = 0.2
  local fadeAction = AHOpacityTo.new(self.overlay, fadeDuration, nil, 1.0)

  return fadeAction
end

local function handleTapNavigateBack(self)
  self.forwardTapArea:setTouchEnabled(false)
  self.backTapArea:setTouchEnabled(false)

  local fadeOutAnimation = createFadeOutAnimation(self)

  wait(fadeOutAnimation:start())

  if (self.navigatorExitCallback ~= nil) then
    self.navigatorExitCallback()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardTitle:reset()
  local icon = self.icon
  local textColumn = self.textColumn
  local overlay = self.overlay
  local swipeTip = self.swipeTip

  icon:setScl(0.0)
  textColumn:setScl(0.0)
  overlay:setOpacity(1.0)
  swipeTip:setOpacity(0.0)
end

function ExerciseCardTitle:createAnimationAction()
  local icon = self.icon
  local textColumn = self.textColumn
  local overlay = self.overlay
  local swipeTip = self.swipeTip

  local scaleDuration = 0.4
  local fadeDuration = 0.2
  local iconScaleAction = AHScaleTo.new(icon, scaleDuration, AHEaseExponentialOut.new(), 1.0)
  local textColumnScaleAction = AHScaleTo.new(textColumn, scaleDuration, AHEaseExponentialOut.new(), 1.0)
  local fadeAction = AHOpacityTo.new(overlay, fadeDuration, nil, 0.0)

  local animationAction = AHSpawn.new(iconScaleAction,
                                      textColumnScaleAction,
                                      fadeAction,
                                      AHOpacityTo.new(swipeTip, 1.2, nil, 1.0))

  return animationAction
end

function ExerciseCardTitle:handlesOwnTapNavigation()
  return true
end

function ExerciseCardTitle:onEnter()
  self.super.onEnter(self)
  self.forwardTapArea:setTouchEnabled(true)
  self.backTapArea:setTouchEnabled(true)
end

function ExerciseCardTitle:onLeaveBegin()
  self.super.onLeaveBegin(self)
  self.forwardTapArea:setTouchEnabled(false)
  self.backTapArea:setTouchEnabled(false)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createTapAreas(self)
  local width = self:getWidth() - CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY
  local height = self:getHeight()
  local onTouchDown = nil
  local duration = CardCarouselNavigator.SCROLL_ANIMATION_DURATION_HORIZONTAL
  local willCheckNavigationConstraints = true
  local onTouchUp = function()
    self:forceTapNavigation(CardCarouselNavigator.TAP_DIRECTION_FORWARD, duration, willCheckNavigationConstraints)
  end
  local onTouchCancel = nil
  local forwardTapArea = Button.new(width, height, onTouchDown, onTouchUp, onTouchCancel)
  forwardTapArea:setAnchorPointRight()
  self:insertAtRight(forwardTapArea)
  forwardTapArea:setTouchEnabled(false)
  self.forwardTapArea = forwardTapArea

  width = CardCarouselNavigator.TAP_NAVIGATE_LEFT_BOUNDARY
  local height = self:getHeight()
  onTouchUp = function()
    handleTapNavigateBack(self)
  end
  local backTapArea = Button.new(width, height, onTouchDown, onTouchUp, onTouchCancel)
  backTapArea:setAnchorPointLeft()
  self:insertAtLeft(backTapArea)
  backTapArea:setTouchEnabled(false)
  self.backTapArea = backTapArea
end

local function createSwipeTip()
  local columnSpacing = 10.0
  local tip = Column.new(columnSpacing)

  local hand = Sprite.newWithName("tap_to_continue.png")
  tip:insert(hand)

  local label = CardCarouselCardCommon.newLabel("TAP TO CONTINUE", FontFaces.DIN_OT_MEDIUM, 20)
  tip:insert(label)

  tip:alignChildren()

  tip:linkColorAttrToChildren()
  tip:setColor(unpack(SWIPE_TIP_COLOR))

  return tip
end

-- Constructor ------------------------

function ExerciseCardTitle.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardTitle)

  local background = CardCarouselCardCommon.newStandardBackground(CardCarouselCardCommon.COLORS.BLUE)
  self:insertAtCenter(background, Z_BACKGROUND)

  local mainColumnSpacing = -10
  local mainColumn = Column.new(mainColumnSpacing)

  local concept = data.concept
  local iconName = data.icon
  local icon = ExerciseIcon.new(concept, iconName)
  mainColumn:insert(icon)
  self.icon = icon

  local textColumnSpacing = 6.0
  local textColumn = Column.new(textColumnSpacing)

  local title = string.upper(data.text)
  local titleLabel = CardCarouselCardCommon.newMultilineLabel(title,
                                                          FontFaces.DIN_OT_BOLD,
                                                          28,
                                                          Screen.getWidthPart(0.9),
                                                          Alignment.CENTER)
  textColumn:insert(titleLabel)

  local description = data.description
  local descriptionAttributes = {
    default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, DESCRIPTION_TEXT_COLOR)
  }

  local descriptionLabel = CardCarouselTextView.new(description,
                                                CardCarouselCardCommon.CONTENT_WIDTH,
                                                descriptionAttributes,
                                                Alignment.CENTER)
  textColumn:insert(descriptionLabel)

  textColumn:alignChildren()
  mainColumn:insert(textColumn)
  self.textColumn = textColumn

  mainColumn:alignChildren()
  mainColumn:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.55))
  self:insert(mainColumn, Z_MAIN_COLUMN)

  local swipeTip = createSwipeTip()
  local y = self:getHeightPart(0.025) + swipeTip:getHeightPart(0.5)
  if (Screen.hasNotch()) then
    y = 42.0 + swipeTip:getHeightPart(0.5)
  end
  swipeTip:setLoc(self:getWidthPart(0.5), y)
  self:insert(swipeTip, Z_SWIPE_TIP)
  self.swipeTip = swipeTip

  local overlay = CardCarouselCardCommon.newStandardBackground(CardCarouselCardCommon.COLORS.BLUE)
  self:insertAtCenter(overlay, Z_OVERLAY)
  self.overlay = overlay

  createTapAreas(self)

  return self
end
