--- ExerciseCardTitleSubtitle
-- @author Created by Aaron Barrett on April 21, 2015.

class("ExerciseCardTitleSubtitle")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TITLE_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.BLUE, {bouncy = true})
}

local SUBTITLE_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.LIGHT_GREY),
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardTitleSubtitle:reset()
  self.titleView:updateLayout()
end

function ExerciseCardTitleSubtitle:createAnimationAction()
  local bouncyViews = {self.titleView}
  local animationAction = CardCarouselCardCommon.createBouncyTextAction(bouncyViews)
  return animationAction
end

function ExerciseCardTitleSubtitle:setScrollProgress(progress, isHorizontal)
  if (isHorizontal == nil) then
    isHorizontal = true
  end
  local isScrollingVertically = (not isHorizontal)
  local forcedFulllyComplete = 1.0

  if isScrollingVertically then
    progress = forcedFulllyComplete
  end

  self.untimedAction:applyAtTime(progress)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardTitleSubtitle.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardTitleSubtitle)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(workArea)

  local title = data.title
  local subtitle = data.subtitle

  local titleView = CardCarouselTextView.new(title, workArea:getWidth(), TITLE_TEXT_ATTRIBUTES)
  self.titleView = titleView

  local subtitleView = nil
  if subtitle then
    subtitleView = CardCarouselTextView.new(subtitle, workArea:getWidth(), SUBTITLE_TEXT_ATTRIBUTES)
  end

  local staggerAmount = CardCarouselCardCommon.DEFAULT_STAGGER_AMOUNT
  local scrollAction = AHSpawn.new()
  local col = Column.new(Screen.getWidthPart(0.04), Alignment.LEFT)
  col:insert(titleView)
  titleView:prepareForSlideIn(staggerAmount * 2, staggerAmount)
  scrollAction:addAction(titleView:newSlideInAction())
  if subtitle then
    col:insert(subtitleView)
    subtitleView:prepareForSlideIn(staggerAmount * 3, staggerAmount)
    scrollAction:addAction(subtitleView:newSlideInAction())
  end
  col:alignChildren()

  workArea:insertAtCenter(col)

  self.untimedAction = AHUntimedAction.new(scrollAction)
  self.untimedAction:start()

  return self
end
