--- ExerciseCardWordHighlight
-- @author Created by Aaron Barrett on April 21, 2015.

class("ExerciseCardWordHighlight")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("containers.card_carousel.CardCarouselTextView")
require("ExerciseZoomCircle")
require("ExerciseSlideOrchestrator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_CIRCLE_BLUE = CardCarouselCardCommon.COLORS.BLUE
local COLOR_CIRCLE_GREY = {0.98, 0.98, 0.98}

local TITLE_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_LARGE, CardCarouselCardCommon.COLORS.DARK_GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_LARGE, CardCarouselCardCommon.COLORS.DARK_GREY, {highlighted = true}),
}

local SUBTITLE_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.BLACK)
}

local HINT_LABEL_OPACITY = 0.4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newWobbleXAction(target, duration, distance)
  return AHLoop.new(AHSequence.new(
    AHLocationXBy.new(target, duration, AHEaseSineInOut.new(), distance),
    AHLocationXBy.new(target, duration, AHEaseSineInOut.new(), -distance)
  ))
end

local function newWobbleYAction(target, duration, distance)
  return AHLoop.new(AHSequence.new(
    AHLocationYBy.new(target, duration, AHEaseSineInOut.new(), distance),
    AHLocationYBy.new(target, duration, AHEaseSineInOut.new(), -distance)
  ))
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardWordHighlight:createAnimationAction()
  local zoomInDuration = 0.8
  local letterColorChange = AHSpawn.new()
  for _, letter in ipairs(self.highlightLetters) do
    letterColorChange:addAction(AHColorTo.new(letter, zoomInDuration * 0.5, nil, CardCarouselCardCommon.COLORS.WHITE))
  end

  local greyCircleAnimation = AHSpawn.new(
                                self.greyCircle:getAnimation(),
                                AHLocationTo.new(self.greyCircle, 0.8, AHEaseExponentialInOut.new(), self.greyCircleEndX, self.greyCircleEndY)
                              )

  local action = AHSpawn.new(
    self.blueCircle:getAnimation(),
    AHDelayed.new(0.15, greyCircleAnimation),
    AHDelayed.new(0.2, letterColorChange),
    AHDelayed.new(0.45, self.slider:newAnimateAction())
  )

  return action
end

function ExerciseCardWordHighlight:reset()
  local textHeight = SUBTITLE_TEXT_ATTRIBUTES.default.size

  self.greyCircle:reset()
  self.greyCircle:setLoc(self.circleStartX, self.circleStartY)

  self.blueCircle:reset()
  self.blueCircle:setLoc(self.circleStartX, self.circleStartY)

  for _, letter in ipairs(self.highlightLetters) do
    letter:setColor(unpack(TITLE_TEXT_ATTRIBUTES.highlight.color))
  end

  self.slider:reset()
end

function ExerciseCardWordHighlight:setScrollProgress(progress, isHorizontal)
  self.untimedAction:applyAtTime(progress)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardWordHighlight.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardWordHighlight)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())

  local workArea = Group.new(self:getWidth() - CardCarouselCardCommon.MARGINS.HORIZONTAL * 2, self:getHeight() - CardCarouselCardCommon.MARGINS.VERTICAL * 2)
  self:insertAtCenter(workArea)

  local col = Column.new(Screen.getWidthPart(0.04), Alignment.LEFT)
  local textLabel = CardCarouselTextView.new(data.text, workArea:getWidth(), TITLE_TEXT_ATTRIBUTES)
  col:insert(textLabel)
  local hintLabel
  if data.hint then
    hintLabel = CardCarouselTextView.new(data.hint:upper(), workArea:getWidth(), SUBTITLE_TEXT_ATTRIBUTES)
    hintLabel:setOpacity(HINT_LABEL_OPACITY)
    col:insert(hintLabel)
  end
  col:alignChildren()

  col:setAnchorPointTop()
  col:setLoc(workArea:getWidthPart(0.5), workArea:getHeightPart(0.7))
  workArea:insert(col)

  local padding = 30
  self.highlightLetters = textLabel:getViewsWithAttribute("highlighted")
  local width, height, centerX, centerY = textLabel:getDimensionsAndCenterOfViewsWithAttribute("highlighted")
  local radius = math.max(width, height) * 0.5 + padding

  self.blueCircle = BlueGrowCircle.new(radius)

  self.blueCircle:setLoc(centerX, centerY)
  textLabel:insert(self.blueCircle)
  self.blueCircle:bringToBack()
  self.circleStartX, self.circleStartY = self.blueCircle:getLoc()

  local radius = textLabel:getWidthPart(0.7)
  self.greyCircle = GreyGrowCircle.new(radius)
  textLabel:insert(self.greyCircle)
  self.greyCircle:bringToBack()
  self.greyCircleEndX, self.greyCircleEndY = textLabel:getWidthPart(0.5), textLabel:getHeightPart(0.5)

  self.slider = ExerciseSlideOrchestrator.new()
  if data.hint then
    self.slider:addProps(hintLabel)
  end

  local staggerAmount = CardCarouselCardCommon.DEFAULT_STAGGER_AMOUNT
  textLabel:prepareForSlideIn(staggerAmount * 2, staggerAmount)
  self.untimedAction = AHUntimedAction.new(textLabel:newSlideInAction())
  self.untimedAction:start()

  return self
end
