--- ExerciseCardWordPartExample
-- @author Created by karl on May 04, 2015.

class("ExerciseCardWordPartExample")

require("cards.ExerciseCard")
require("containers.card_carousel.CardCarouselCardCommon")
require("ExerciseZoomCircle")
require("ExerciseSlideOrchestrator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newSectionLabel(name)
  return CardCarouselCardCommon.newCachedFontLabel(name, CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.DARK_GREY)
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardWordPartExample:createAnimationAction()
  local zoomInDuration = 0.8
  local letterColorChangeAction = AHColorTo.new(self.partLabel, zoomInDuration * 0.5, nil, CardCarouselCardCommon.COLORS.WHITE)

  local greyMoveAction = AHLocationTo.new(self.greyCircle, zoomInDuration, AHEaseExponentialOut.new(), self.greyCircleEndX, self.greyCircleEndY)

  local action = AHSpawn.new(
    self.blueCircle:getAnimation(),
    AHDelayed.new(0.15, self.greyCircle:getAnimation(), greyMoveAction),
    AHDelayed.new(0.15, letterColorChangeAction),
    AHDelayed.new(0.45, self.slider:newAnimateAction())
  )

  return action
end

function ExerciseCardWordPartExample:reset()
  self.greyCircle:reset()
  self.greyCircle:setLoc(self.circleStartX, self.circleStartY)

  self.blueCircle:reset()
  self.blueCircle:setLoc(self.circleStartX, self.circleStartY)

  self.partLabel:setColor(unpack(self.partLabelOriginalColor))

  self.slider:reset()
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardWordPartExample.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardWordPartExample)

  local partName = StringUtils.capitalizeFirstLetter(data.partName)
  local part = data.part:upper()

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())
  self.workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(self.workArea)

  self.partLabelOriginalColor = CardCarouselCardCommon.COLORS.GREY
  self.partLabel = CardCarouselCardCommon.newLabel(part, FontFaces.DIN_OT_MEDIUM, 63, self.partLabelOriginalColor)

  local meaningAttributes = {
    default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.LIGHT_GREY)
  }

  local examplesAttributes = {
    default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.LIGHT_GREY)
  }

  local col = Column.new(0, Alignment.LEFT)
  col:insert(newSectionLabel(partName))
  col:insert(self.partLabel)
  col:insertSpacer(Screen.getHeightPart(0.15))
  local meaningHeaderLabel = newSectionLabel("Meaning")
  col:insert(meaningHeaderLabel)
  local meaningLabel = CardCarouselTextView.new(data.meaning, self.workArea:getWidth(), meaningAttributes)
  col:insert(meaningLabel)
  col:insertSpacer(Screen.getHeightPart(0.05))
  local examplesHeaderLabel = newSectionLabel("Examples")
  col:insert(examplesHeaderLabel)
  local examplesLabel = CardCarouselTextView.new(data.examples, self.workArea:getWidth(), examplesAttributes)
  col:insert(examplesLabel)
  col:alignChildren()
  col:setAnchorPointTopLeft()
  col:setLoc(0, self.workArea:getHeightPart(0.94))

  local maxDimension = math.max(self.partLabel:getWidth(), self.partLabel:getHeight())
  local radius = maxDimension / 2 + Screen.getWidthPart(0.1)
  self.blueCircle = BlueShrinkCircle.new(radius)
  self.blueCircle:setLoc(self.partLabel:getLoc())
  col:insert(self.blueCircle)
  self.blueCircle:bringToBack()
  self.circleStartX, self.circleStartY = self.blueCircle:getLoc()

  local radius = col:getWidthPart(0.8)
  self.greyCircle = GreyGrowCircle.new(radius)
  self.greyCircle:setLoc(self.blueCircle:getLoc())
  self.workArea:insert(self.greyCircle)
  self.greyCircle:bringToBack()
  self.greyCircleEndX, self.greyCircleEndY = self.workArea:getWidthPart(0.25), self.workArea:getHeightPart(0.2)

  self.workArea:insert(col)

  self.slider = ExerciseSlideOrchestrator.new()
  self.slider:addProps(meaningHeaderLabel, meaningLabel, examplesHeaderLabel, examplesLabel)

  return self
end
