--- ExerciseCardWordPartExplanation
-- @author Created by karl on May 06, 2015.

class("ExerciseCardWordPartExplanation")

require("cards.ExerciseCard")
require("ExerciseSlideOrchestrator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WORD_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(Font.new(FontFaces.DIN_OT_MEDIUM, 43), CardCarouselCardCommon.COLORS.DARK_GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(Font.new(FontFaces.DIN_OT_MEDIUM, 43), CardCarouselCardCommon.COLORS.DARK_GREY, {highlighted = true})
}

local EXPLANATION_TEXT_ATTRIBUTES = {
  default = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.GREY),
  highlight = CardCarouselCardCommon.newTextAttribute(CardCarouselCardCommon.CACHED_FONTS.LIGHT_MEDIUM, CardCarouselCardCommon.COLORS.BLUE),
}

local SLIDE_DISTANCE = CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM:getActualSize() * 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExerciseCardWordPartExplanation:createAnimationAction()
  local zoomInDuration = 0.8
  local letterColorChange = AHSpawn.new()
  for _, letter in ipairs(self.highlightLetters) do
    letterColorChange:addAction(AHColorTo.new(letter, zoomInDuration * 0.5, nil, CardCarouselCardCommon.COLORS.WHITE))
  end

  local action = AHSpawn.new(
    self.blueCircle:getAnimation(),
    AHDelayed.new(0.2, letterColorChange),
    AHDelayed.new(0.45, self.slider:newAnimateAction())
  )

  return action
end

function ExerciseCardWordPartExplanation:reset()
  local textHeight = WORD_TEXT_ATTRIBUTES.default.size

  self.blueCircle:reset()
  self.blueCircle:setLoc(self.circleStartX, self.circleStartY)

  for _, letter in ipairs(self.highlightLetters) do
    letter:setColor(unpack(WORD_TEXT_ATTRIBUTES.highlight.color))
  end

  self.slider:reset()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExerciseCardWordPartExplanation.new(cardType, cardGlobals, data)
  local self = ExerciseCard.new(cardType, cardGlobals)
  Mixin.mix(self, ExerciseCardWordPartExplanation)

  self:insertAtCenter(CardCarouselCardCommon.newStandardBackground())
  local workArea = CardCarouselCardCommon.newStandardWorkArea()
  self:insertAtCenter(workArea)

  local outerCol = Column.new(0, Alignment.LEFT)

  local topCol = Column.new(0)
  local wordLabel = CardCarouselTextView.new(data.word:upper(), workArea:getWidth(), WORD_TEXT_ATTRIBUTES, Alignment.CENTER)
  topCol:insert(wordLabel)
  topCol:insertSpacer(self:getHeightPart(0.03))
  local descriptionLayer = CardCarouselCardCommon.newCachedFontLabel(data.partDescription:upper(), CardCarouselCardCommon.CACHED_FONTS.MEDIUM_MEDIUM, CardCarouselCardCommon.COLORS.GREY)
  topCol:insert(descriptionLayer)
  topCol:insertSpacer(self:getHeightPart(0.08))

  topCol:alignChildren()
  outerCol:insert(topCol)

  local bottomCol = Column.new(0, Alignment.LEFT)
  bottomCol:insert(FilledRectangle.new(Screen.width - CardCarouselCardCommon.MARGINS.HORIZONTAL, 1, CardCarouselCardCommon.COLORS.LIGHT_GREY))
  bottomCol:insertSpacer(self:getHeightPart(0.08))

  local explanationLabel = CardCarouselTextView.new(data.explanation, workArea:getWidth(), EXPLANATION_TEXT_ATTRIBUTES)
  bottomCol:insert(explanationLabel)
  self.explanationLabel = explanationLabel

  bottomCol:alignChildren()
  outerCol:insert(bottomCol)

  local outerColumnOffsetY = workArea:getHeightPart(-0.1)

  outerCol:alignChildren()
  outerCol:setAnchorPointTopLeft()
  outerCol:setLoc(0, workArea:getHeight() + outerColumnOffsetY)
  workArea:insert(outerCol)

  local padding = 15
  self.highlightLetters = wordLabel:getViewsWithAttribute("highlighted")
  local width, height, centerX, centerY = wordLabel:getDimensionsAndCenterOfViewsWithAttribute("highlighted")
  local radius = math.max(width, height) * 0.5 + padding

  self.blueCircle = BlueShrinkCircle.new(radius)

  self.blueCircle:setLoc(centerX, centerY)
  wordLabel:insert(self.blueCircle)
  self.blueCircle:bringToBack()
  self.circleStartX, self.circleStartY = self.blueCircle:getLoc()

  self.slider = ExerciseSlideOrchestrator.new()
  self.slider:addProps(descriptionLayer, unpack(bottomCol.children))

  return self
end
