--- ExpelliarmusAnswerButton
-- @author Created by karl on September 15, 2014.

class("ExpelliarmusAnswerButton")
require("ExpelliarmusCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TEXT = {1, 1, 1, 1}

local FONT_SIZE = 35
local FONT_TEXT = Font.new(FontFaces.DIN_OT_LIGHT, FONT_SIZE)
local FONT_NO_CHANGE = Font.new(FontFaces.DIN_OT_MEDIUM, 13)

local FINETUNE_LETTERS = TableUtils.createSetFromString("acegjmnopqrsuvwxyz")

local BUTTON_SIZE = Screen.getWidthPart(0.22)
local MARGIN = Screen.getWidthPart(0.05)
local FINETUNE_Y = FONT_SIZE * 0.13

local NO_CHANGE = ""

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

local function scaleBackingIfNeeded(self)
  local labelWidth = self.textLabel:getWidth()
  if (labelWidth > self.backing:getWidthPart(0.95)) then
    self.backing:setEffectiveWidth(labelWidth + MARGIN)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusAnswerButton:getBackingWidth()
  return self.backing:getEffectiveWidth()
end

function ExpelliarmusAnswerButton:setBackingWidth(width)
  return self.backing:setEffectiveWidth(width)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newNoChangeLabel(self)
  local labelProperties = {
    font = FONT_NO_CHANGE,
    fitInLine = true
  }
  local label = DynamicTextArea.new(Localizer.get("NO_CHANGE"), self:getWidthPart(0.8), labelProperties, nil, Alignment.CENTER, Alignment.CENTER)
  label:setColor(unpack(COLOR_TEXT))
  label:setLoc(self:getCenter())
  return label
end

local function newLetterLabel(self, prefix, letter)
  local label = ExpelliarmusCommon.newKernedLabel(prefix, letter, FONT_TEXT)
  label:setColor(unpack(COLOR_TEXT))
  local labelX, labelY = self:getCenter()
  local nonFineTuneFound = false
  for i=1,#letter do
    local char = string.sub(letter, i, i)
    if not FINETUNE_LETTERS[char] then
      nonFineTuneFound = true
    end
  end
  if not nonFineTuneFound then
    labelY = labelY + FINETUNE_Y
  end
  label:setLoc(labelX, labelY)
  return label
end

local function addText(self, prefix, text)
  if text == NO_CHANGE then
    return newNoChangeLabel(self), false
  else
    return newLetterLabel(self, prefix, text), true
  end
end

-- Constructor ------------------------

function ExpelliarmusAnswerButton.new(answer, color, onPressed)
  local self = Group.new(BUTTON_SIZE, BUTTON_SIZE)
  Mixin.mix(self, ExpelliarmusAnswerButton)

  local backing = FilledRectangle.new(BUTTON_SIZE, BUTTON_SIZE, color)
  self:insertAtCenter(backing)
  self.backing = backing

  self.textLabel, self.isChange = addText(self, nil, answer)
  self:insert(self.textLabel)

  scaleBackingIfNeeded(self)

  Button.makeButton(self, onPressed)

  self:linkOpacityAttrToChildren(true)

  return self
end

function ExpelliarmusAnswerButton.newCorrect(prefix, answer, color)
  local self = Group.new(BUTTON_SIZE, BUTTON_SIZE)
  Mixin.mix(self, ExpelliarmusAnswerButton)

  local selfW, selfH = self:getDims()
  local cornerRadius = selfW * 0.2
  local backing = FilledRoundedRectangle.new(selfW, selfH, cornerRadius, color)
  self:insertAtCenter(backing)
  self.backing = backing

  self.textLabel, self.isChange = addText(self, prefix, answer)
  self:insert(self.textLabel)

  scaleBackingIfNeeded(self)

  self:linkOpacityAttrToChildren(true)

  self.newExpandAction = function(self, duration)
    local size = math.max(Screen.fullWidth, Screen.fullHeight) * 2
    local radius = size * 0.45
    return AHSpawn.new(
      AHDimsTo.new(backing, duration, nil, size, size),
      backing:newCornerRadiusTo(duration, nil, radius)
    )
  end

  return self
end
