--- ExpelliarmusAnswerSelector
-- @author Created by karl on August 29, 2014.

class("ExpelliarmusAnswerSelector")
require("ExpelliarmusAnswerButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_SPACING = 10


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusAnswerSelector:getAnswerButton(answer)
  return self.answerButtons[answer]
end

function ExpelliarmusAnswerSelector:getOtherButtons(...)
  local ignoreButtons = TableUtils.createSetFromArray(arg)
  local buttons = {}
  for answer, button in pairs(self.answerButtons) do
    if not ignoreButtons[answer] then
      table.insert(buttons, button)
    end
  end
  return buttons
end

function ExpelliarmusAnswerSelector:setTouchEnabled(enabled)
  for _, button in pairs(self.answerButtons) do
    button:setTouchEnabled(enabled)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusAnswerSelector.new(answers, color, onAnswerSelected)
  local self = Column.new(BUTTON_SPACING)
  Mixin.mix(self, ExpelliarmusAnswerSelector)

  self.answerButtons = {}
  local maxOptionWidth = 0
  for _, answer in ipairs(answers) do
    local button = ExpelliarmusAnswerButton.new(answer, color, function() onAnswerSelected(answer) end)
    self:insert(button)
    self.answerButtons[answer] = button
    maxOptionWidth = math.max(button:getBackingWidth(), maxOptionWidth)
  end

  for _, button in pairs(self.answerButtons) do
    button:setBackingWidth(maxOptionWidth)
  end

  self:alignChildren()

  return self
end
