--- ExpelliarmusConceptGenerator
-- @author Created by karl on August 29, 2014.

class("ExpelliarmusConceptGenerator")

require("data.generators.UtilityGenerators")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ExpelliarmusConceptGenerator.CORRECT_MARKER = "no change"


local CONTENT_TYPE_TO_FILTER_TYPE = {
  ["Flip-flop"] = "typeA",
  ["Double or single letter?"] = "typeB",
  ["Endings"] = "typeC",
  ["Add a letter or OK?"] = "typeD",
  ["Plurals"] = "typeE",
  ["Vowel choice"] = "typeF",

  ["¿Juntas o separadas?"] = "typeA",
  ["¿Una o dos letras?"] = "typeB",
  ["Dudosas"] = "typeC",
  ["Elige la correcta"] = "typeD",
  ["Volteadas"] = "typeE"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function convertAnswer(answer)
  if answer == nil or answer == "" then
    return nil
  elseif answer == ExpelliarmusConceptGenerator.CORRECT_MARKER then
    return ""
  else
    return answer
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusConceptGenerator.getTextComponents(promptText)
  local startTag = "<replace>"
  local endTag = "</replace>"
  local tag1Start, tag1End = string.find(promptText, startTag)
  local tag2Start, tag2End = string.find(promptText, endTag)
  local text1Start, text1End = 1, tag1Start-1
  local text2Start, text2End = tag1End+1, tag2Start-1
  local text3Start, text3End = tag2End+1, #promptText
  return string.sub(promptText, text1Start, text1End), string.sub(promptText, text2Start, text2End), string.sub(promptText, text3Start, text3End)
end

function ExpelliarmusConceptGenerator:setTopic(topic)
  local filterType = CONTENT_TYPE_TO_FILTER_TYPE[topic]
  self.generator:setFilters({type = filterType})
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusConceptGenerator.new()
  local conceptGenerator = GameSession.conceptGenerator
  local difficultyKeyFrames = ExpelliarmusConfiguration.getDifficultyKeyFrames()
  local difficultyNameChooser = KeyFrameInterpolator.new(difficultyKeyFrames, GameSession.getDifficulty())
  conceptGenerator:setDifficultyNameChooser(difficultyNameChooser)
  local self = TransformationGenerator.new(conceptGenerator, function(concept)
    concept.leftText, concept.correct, concept.rightText = ExpelliarmusConceptGenerator.getTextComponents(concept.prompt)
    if concept.correct == ExpelliarmusConceptGenerator.CORRECT_MARKER then
      concept.correct = ""
    end
    concept.incorrect1 = convertAnswer(concept.incorrect1)
    concept.incorrect2 = convertAnswer(concept.incorrect2)
    concept.answers = {concept.correct, concept.incorrect1}
    if concept.incorrect2 then
      table.insert(concept.answers, concept.incorrect2)
    end
    TableUtils.shuffleArray(concept.answers)

    return concept
  end)
  Mixin.mix(self, ExpelliarmusConceptGenerator)

  return self
end
