class("ExpelliarmusConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ExpelliarmusConfiguration.QUESTIONS_PER_TOPIC_COUNT = 4
ExpelliarmusConfiguration.TOPIC_COUNT = 3
ExpelliarmusConfiguration.INITIAL_LIVES_COUNT = 4
ExpelliarmusConfiguration.POINTS_PER_CORRECT_RESPONSE = 500
ExpelliarmusConfiguration.POINTS_PER_LIVES_BONUS = 1000
ExpelliarmusConfiguration.POINTS_PER_FULL_SPEED_BONUS = 50
ExpelliarmusConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 1.5
ExpelliarmusConfiguration.SPEED_BONUS_DECAY_START = 0.5
ExpelliarmusConfiguration.QUESTION_TIMER_DURATIONS = {7, 5.75, 4.5}

local EXTRA_LIVES_NEEDED_RANK_TWO = 2
local EXTRA_LIVES_NEEDED_RANK_THREE = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function minPointsWithExtraLives(extraLives)
  local answerPoints = ExpelliarmusConfiguration.getPassScore()
  local livesPoints = extraLives * ExpelliarmusConfiguration.POINTS_PER_LIVES_BONUS

  return answerPoints + livesPoints
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusConfiguration.getMaxCorrectAnswers()
  return ExpelliarmusConfiguration.TOPIC_COUNT * ExpelliarmusConfiguration.QUESTIONS_PER_TOPIC_COUNT
end

function ExpelliarmusConfiguration.getMaxSpeedBonuses()
  local correctPossible = ExpelliarmusConfiguration.getMaxCorrectAnswers()
  local maxBonuses = correctPossible * GameSession.config.speed_bonus
  return maxBonuses
end

function ExpelliarmusConfiguration.getPassScore()
  local questionCount = ExpelliarmusConfiguration.TOPIC_COUNT * ExpelliarmusConfiguration.QUESTIONS_PER_TOPIC_COUNT
  local passScore = questionCount * ExpelliarmusConfiguration.POINTS_PER_CORRECT_RESPONSE
  return passScore
end

function ExpelliarmusConfiguration.getRankTwoScore()
  return minPointsWithExtraLives(EXTRA_LIVES_NEEDED_RANK_TWO)
end

function ExpelliarmusConfiguration.getRankThreeScore()
  return minPointsWithExtraLives(EXTRA_LIVES_NEEDED_RANK_THREE)
end

function ExpelliarmusConfiguration.getRoundKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {["Flip-flop"] = 0.17,
                                      ["Double or single letter?"] = 0.20,
                                      ["Endings"] = 0.15,
                                      ["Add a letter or OK?"] = 0.30,
                                      ["Plurals"] = 0.08,
                                      ["Vowel choice"] = 0.10}},

    {key = 0.399, valueProbabilities = {["Flip-flop"] = 0.17,
                                        ["Double or single letter?"] = 0.20,
                                        ["Endings"] = 0.15,
                                        ["Add a letter or OK?"] = 0.30,
                                        ["Plurals"] = 0.08,
                                        ["Vowel choice"] = 0.10}},

    {key = 0.4, valueProbabilities = {["Flip-flop"] = 0.18,
                                      ["Double or single letter?"] = 0.21,
                                      ["Endings"] = 0.15,
                                      ["Add a letter or OK?"] = 0.31,
                                      ["Plurals"] = 0.05,
                                      ["Vowel choice"] = 0.10}},

    {key = 1.0, valueProbabilities = {["Flip-flop"] = 0.18,
                                      ["Double or single letter?"] = 0.21,
                                      ["Endings"] = 0.15,
                                      ["Add a letter or OK?"] = 0.31,
                                      ["Plurals"] = 0.05,
                                      ["Vowel choice"] = 0.10}}
  }
end

function ExpelliarmusConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 1.0}},

    {key = 0.075, valueProbabilities = {Basic = 0.5,
                                        Easy = 0.5}},

    {key = 0.15, valueProbabilities = {Easy = 1.0}},

    {key = 0.225, valueProbabilities = {Easy = 0.63,
                                        Medium = 0.37}},

    {key = 0.3, valueProbabilities = {Easy = 0.50,
                                      Medium = 0.50}},

    {key = 0.375, valueProbabilities = {Easy = 0.40,
                                        Medium = 0.55,
                                        Hard = 0.05}},

    {key = 0.45, valueProbabilities = {Easy = 0.25,
                                       Medium = 0.60,
                                       Hard = 0.15}},

    {key = 0.525, valueProbabilities = {Easy = 0.10,
                                        Medium = 0.60,
                                        Hard = 0.30}},

    {key = 0.6, valueProbabilities = {Medium = 0.50,
                                      Hard = 0.45,
                                      Advanced = 0.05}},

    {key = 0.675, valueProbabilities = {Medium = 0.25,
                                        Hard = 0.60,
                                        Advanced = 0.15}},

    {key = 0.75, valueProbabilities = {Hard = 0.60,
                                       Advanced = 0.40}},

    {key = 0.8, valueProbabilities = {Hard = 0.50,
                                      Advanced = 0.50}},

    {key = 0.9, valueProbabilities = {Hard = 0.40,
                                      Advanced = 0.60}},

    {key = 1.0, valueProbabilities = {Hard = 0.30,
                                      Advanced = 0.70}}
  }
end
