--- ExpelliarmusEndScreen
-- @author Created by karl on September 08, 2014.

class("ExpelliarmusEndScreen")
require("ExpelliarmusCheckmark")
require("ExpelliarmusCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLORS = {
  {
    ColorUtils.colorHex24ToStd(0xDE27E7),
    ColorUtils.colorHex24ToStd(0xF62BFA),
    ColorUtils.colorHex24ToStd(0xF563FB),
    ColorUtils.colorHex24ToStd(0xF62BFA),
    ColorUtils.colorHex24ToStd(0xBA1EC0)
  },
  {
    ColorUtils.colorHex24ToStd(0x05E3EC),
    ColorUtils.colorHex24ToStd(0x06EFFD),
    ColorUtils.colorHex24ToStd(0x57F8FF),
    ColorUtils.colorHex24ToStd(0x06EFFD),
    ColorUtils.colorHex24ToStd(0x04CED9)
  },
  {
    ColorUtils.colorHex24ToStd(0x06B3EA),
    ColorUtils.colorHex24ToStd(0x08BAFD),
    ColorUtils.colorHex24ToStd(0x57D6FE),
    ColorUtils.colorHex24ToStd(0x08BAFD),
    ColorUtils.colorHex24ToStd(0x059ACD)
  },
  {
    ColorUtils.colorHex24ToStd(0xffffff),
    ColorUtils.colorHex24ToStd(0xffffff),
    ColorUtils.colorHex24ToStd(0xffffff),
    ColorUtils.colorHex24ToStd(0xffffff),
    ColorUtils.colorHex24ToStd(0xffffff)
  }
}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newAnimation(self, pieces, colorIndex)
  local selfW, selfH = self:getDims()
  local colors = COLORS[colorIndex]
  local endLocs = TableUtils.map(pieces, function(piece) return {piece:getLoc()} end)
  local startLocs = TableUtils.deepCopy(endLocs)
  startLocs[1][1] = selfW
  startLocs[2][1] = selfW
  startLocs[3][2] = selfH
  startLocs[4][1] = 0
  startLocs[5][2] = 0

  local function initialize()
    for index, piece in ipairs(pieces) do
      piece:setLoc(unpack(startLocs[index]))
      piece:setColor(unpack(colors[index]))
    end
  end

  local ease = AHEaseSineInOut.new()
  local stagger = 0.1
  local duration = 0.5

  return AHSequence.new(
    initialize,
    AHSpawn.new(
      AHStaggeredSpawn.new(stagger,
        AHLocationTo.new(pieces[1], duration, ease, unpack(endLocs[1])),
        AHLocationTo.new(pieces[2], duration, ease, unpack(endLocs[2])),
        AHLocationTo.new(pieces[3], duration, ease, unpack(endLocs[3]))
      ),
      AHSequence.new(
        AHDelay.new(stagger * 3),
        AHSpawn.new(
          AHLocationTo.new(pieces[4], duration, ease, unpack(endLocs[4])),
          AHLocationTo.new(pieces[5], duration, ease, unpack(endLocs[5]))
        )
      )
    )
  )
end

local function insertNewPieceSet(self)
  local selfW, selfH = self:getDims()
  local checkmarkW = selfW * 0.62
  local shortLength = ExpelliarmusCommon.CHECKMARK_SHORT_LENGTH
  local thickness = ExpelliarmusCommon.CHECKMARK_THICKNESS

  local pieces = ExpelliarmusCheckmark.newExternalPieces(checkmarkW, shortLength, thickness, selfW, selfH)

  for _, piece in ipairs(pieces) do
    piece:setOpacity(0)
    self:insert(piece)
  end

  return pieces
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusEndScreen:start()
  local selfW, selfH = self:getDims()
  local bg = FilledRectangle.new(selfW, selfH, {1, 1, 1, 0})
  self:insertAtCenter(bg)

  local action = AHStaggeredSpawn.new(0.5)
  for i=1,#COLORS do
    local pieces = insertNewPieceSet(self)
    action:addAction(newAnimation(self, pieces, i))
  end

  return AHSequence.new(
    AHOpacityTo.new(bg, 0.5, nil, 1),
    function()
      Sound.playSoundEffect("win")
    end,
    action
  ):start()
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusEndScreen.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ExpelliarmusEndScreen)

  return self
end
