--- ExpelliarmusExplanationCard
-- @author Created by karl on September 16, 2014.

class("ExpelliarmusExplanationCard")
require("ExpelliarmusCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_BACKGROUND = {1, 1, 1}
local COLOR_HEADER = ColorUtils.colorHex24ToStd(0x949494)
local COLOR_DESCRIPTION = ColorUtils.colorHex24ToStd(0x4c4c4c)
local COLOR_SEPARATOR = ColorUtils.colorHex24ToStd(0x949494)
local COLOR_TAP_TO_CONTINUE = ColorUtils.colorHex24ToStd(0x838383)

local FONT_HEIGHT_MULTIPLIER = 1.32

local FONT_SIZE_REGULAR = 20
local FONT_SIZE_REDUCED = 16
local FONT_SIZE_SMALL = 14

local HEADER_FONT_SIZE_REGULAR = 14
local HEADER_FONT_SIZE_REDUCED = 13
local HEADER_FONT_SIZE_SMALL = 12

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newSection(self, header, description, fontSize, headerFontSize)
  local headerFont = Font.new(FontFaces.DIN_OT_MEDIUM, headerFontSize)
  local descriptionFont = Font.new(FontFaces.DIN_OT_MEDIUM, fontSize)
  local italicFont = Font.new(FontFaces.DIN_OT_MEDIUM_ITALIC, fontSize)
  local sectionWidth = self:getWidthPart(0.8)

  local sectionColumn = Column.new(0, Alignment.LEFT)
  local headerLabel = Label.new(header, headerFont)
  headerLabel:setColor(unpack(COLOR_HEADER))
  sectionColumn:insert(headerLabel)
  sectionColumn:insertSpacer(self:getHeightPart(0.03))

  local baseAttributes = {
    font = descriptionFont
  }

  local attributesForLabels = {
    i = {
      font = italicFont
    }
  }

  local descriptionArea = DynamicTextArea.new(description, sectionWidth, baseAttributes, attributesForLabels, Alignment.LEFT, Alignment.CENTER, 0.9)
  descriptionArea:setColor(unpack(COLOR_DESCRIPTION))
  sectionColumn:insert(descriptionArea)
  sectionColumn:alignChildren()

  return sectionColumn
end

local function newSeparator(self)
  return FilledRectangle.new(self:getWidth(), 1, COLOR_SEPARATOR)
end

local function hasData(text)
  return text and text ~= ""
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusExplanationCard.new(concept)
  local self = FilledRectangle.new(Screen.fullWidth, Screen.getSafeY(0.6) + ExpelliarmusCommon.CARD_BUMP_AMOUNT, COLOR_BACKGROUND)
  Mixin.mix(self, ExpelliarmusExplanationCard)
  local selfW, selfH = self:getDims()

  local hasExamples = hasData(concept.examples)
  local hasExceptions = hasData(concept.exceptions)
  local sectionCount = 1
  for _, hasSection in ipairs({hasExamples, hasExceptions}) do
    if hasSection then
      sectionCount = sectionCount + 1
    end
  end

  local fontSize = FONT_SIZE_REGULAR
  local headerFontSize = HEADER_FONT_SIZE_REGULAR
  if (sectionCount == 2) then
    fontSize = FONT_SIZE_REDUCED
    headerFontSize = HEADER_FONT_SIZE_REDUCED
  elseif (sectionCount > 2) then
    fontSize = FONT_SIZE_SMALL
    headerFontSize = HEADER_FONT_SIZE_SMALL
  end

  local columnSpacing = self:getHeightPart(0.05)
  local col = Column.new(columnSpacing)
  col:insertSpacer(0)

  col:insert(newSection(self, Localizer.get("Misc.TIP"), concept.tip, fontSize, headerFontSize))

  if hasExamples then
    col:insert(newSeparator(self))
    col:insert(newSection(self, Localizer.get("EXAMPLES"), concept.examples, fontSize, headerFontSize))
  end
  if hasExceptions then
    col:insert(newSection(self, Localizer.get("EXCEPTIONS"), concept.exceptions, fontSize, headerFontSize))
  end

  col:alignChildren()
  col:setAnchorPointTop()
  col:setLoc(Screen.getSafeX(0.5), self:getHeight())
  self:insert(col)

  local tapLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, headerFontSize))
  tapLabel:setColor(unpack(COLOR_TAP_TO_CONTINUE))
  tapLabel:setAnchorPointBottom()

  local maxTapLabelHeight = (col:getYBottom() - Screen.safeBottom) / 2
  if (tapLabel:getHeight() > maxTapLabelHeight) then
    tapLabel:setEffectiveDims(nil, maxTapLabelHeight)
  end

  local minimumY = Screen.safeBottom + tapLabel:getHeightPart(0.5)
  local tapLabelY = math.max(minimumY, tapLabel:getHeightPart(1.7))
  self:insertAtLocation(tapLabel, Screen.getSafeX(0.5), tapLabelY)

  return self
end
