--- ExpelliarmusQuestionCard
-- @author Created by karl on August 21, 2014.

class("ExpelliarmusQuestionCard")
require("ExpelliarmusQuestionLabel")
require("ExpelliarmusExplanationCard")
require("ExpelliarmusCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FONT_DEFINITION = Font.new(FontFaces.DIN_OT_LIGHT, 19.8)

local COLOR_TEXT = ExpelliarmusCommon.COLOR_TEXT
local COLOR_DESCRIPTION = ColorUtils.colorHex24ToStd(0x7D7D7D)

local DEFINITION_MARGIN = Screen.getHeightPart(0.04)

local Z_BG = 0
local Z_ANSWERS = 100
local Z_OVERLAY = 110
local Z_GRADIENT = 120
local Z_DEFINITION = 130
local Z_EXPLANATION = 140
local Z_QUESTION = 200

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function insertNewAnswerOverlay(self, outlineColor, textColor)
  local button = self.answerSelector:getAnswerButton(self.concept.correct)
  local overlay = ExpelliarmusAnswerButton.newCorrect(self.concept.leftText, self.concept.correct, outlineColor)
  overlay:setLoc(self:transformObjectLoc(button))
  if textColor then
    overlay.textLabel:setColor(unpack(textColor))
  end
  overlay:setOpacity(0)
  self:insert(overlay, Z_OVERLAY)

  return overlay
end

local function newTakeTextAction(self, textLabel, isChange)
  local leftLabel = self.promptLabel:getLeftLabel()
  local rightLabel = self.promptLabel:getRightLabel()

  local overshootUp = 4
  local overshootDown = 4
  local separateAmount = 10
  local extraXSpacing = 8

  local openDuration = 0.3
  local closeDuration = 0.3

  local leftStartX = leftLabel:getX()
  local rightStartX = rightLabel:getX()
  local leftMidX = leftStartX - separateAmount
  local rightMidX = rightStartX + separateAmount
  local widthDiff = self.promptLabel:getBlankLabel():getWidth()
  if isChange then
    widthDiff = widthDiff - textLabel:getWidth()
  end
  local leftEndX = leftStartX + widthDiff / 2
  local rightEndX = rightStartX - widthDiff / 2


  return AHSequence.new(
    function()
      if (not isChange) then
        textLabel:setVisibleRecursive(false)
      end
      self.promptLabel:takeChild(textLabel)
    end,
    AHSpawn.new(
      AHLocationYTo.new(textLabel, openDuration, AHEaseSineInOut.new(), self.promptLabel:getHeightPart(0.5) + overshootUp),
      AHLocationXTo.new(leftLabel, openDuration, AHEaseSineInOut.new(), leftMidX),
      AHLocationXTo.new(rightLabel, openDuration, AHEaseSineInOut.new(), rightMidX)
    ),
    AHSpawn.new(
      AHLocationYBy.new(textLabel, closeDuration, AHEaseSineInOut.new(), -overshootDown),
      AHLocationXTo.new(leftLabel, closeDuration, AHEaseSineInOut.new(), leftEndX),
      AHLocationXTo.new(rightLabel, closeDuration, AHEaseSineInOut.new(), rightEndX)
    )
  )
end

local function insertFakeButton(self)
  local group = Group.new(0, 0)
  group.textLabel = Group.new(0, 0)
  group:insert(group.textLabel)
  self:insert(group)
  return group
end

local function createDefinitionLabel(self)
  local definitionText = self.concept.definition
  local definitionLabel = DynamicTextArea.newSimple(
    definitionText, self:getWidthPart(0.8), FONT_DEFINITION, Alignment.CENTER, Alignment.CENTER
  )
  definitionLabel:setColor(unpack(COLOR_DESCRIPTION))
  definitionLabel:setOpacity(0)

  local definitionLabelY = Screen.getSafeY(0.5) + DEFINITION_MARGIN + definitionLabel:getHeightPart(0.5) + Screen.getHeightPart(0.1)
  definitionLabel:setLoc(Screen.getSafeX(0.5), definitionLabelY)

  return definitionLabel
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusQuestionCard:markCorrect(answer)
  return runInCoroutine(function()
    local selfW, selfH = self:getDims()
    local button = self.answerSelector:getAnswerButton(answer)
    local overlay = insertNewAnswerOverlay(self, self.buttonColor)
    local overlayLabel = overlay.textLabel
    local opacityDuration = 0.1
    local endDuration = 0.5

    local gradientEndColors = {
      {0.0, 0.57, 0.63, 0.0},
      {0.0, 0.3, 0.65, 0.0},
      {0.52, 0.14, 0.74, 0.0}
    }

    local beginColor = TableUtils.deepCopy(self.buttonColor)
    beginColor[4] = 0.0
    local endColor = gradientEndColors[self.topicNumber]
    local gradient = RectangleGradient.new(selfW, selfH, beginColor, endColor)
    gradient:setAnchorPointBottom()
    gradient:setLoc(selfW/2, 0)
    self:insert(gradient, Z_GRADIENT)

    local expandAction = AHSequence.new(
      AHSpawn.new(
        AHOpacityTo.new(button, opacityDuration, nil, 0),
        AHOpacityTo.new(overlay, opacityDuration, nil, 1)
      ),
      AHSpawn.new(
        overlay:newExpandAction(0.5),
        gradient:newOpacityToAction(0.4, nil, 1.0, 1.0)
      )
    )

    local takeTextAction = AHSequence.new(
      AHLocationYBy.new(overlay, opacityDuration, AHEaseSineInOut.new(), -20),
      newTakeTextAction(self, overlayLabel, button.isChange)
    )

    local slideOutAction = AHSequence.new(
      function()
        self.promptLabel:takeChild(overlay.textLabel)
        self.answerSelector:removeSelf()
      end,
      AHLocationXBy.new(self.promptLabel, 0.15, AHEaseSineInOut.new(), 4),
      AHSpawn.new(
        AHLocationXBy.new(self.promptLabel, endDuration, AHEaseSineInOut.new(), -selfW),
        AHOpacityTo.new(overlay, endDuration, nil, 0),
        gradient:newOpacityToAction(endDuration, nil, 0.0, 0.0)
      )
    )

    local action = AHSequence.new(
      AHSpawn.new(
        expandAction,
        takeTextAction,
        AHSequence.new(
          AHDelay.new(0.3),
          function()
            self.promptLabel:removeBlankLabel()
          end
        )
      ),
      AHDelay.new(0.5),
      slideOutAction
    )
    wait(action:start())
  end)
end

function ExpelliarmusQuestionCard:markIncorrect(answer)
  return runInCoroutine(function()
    local selfW, selfH = self:getDims()
    local black = {0.2, 0.2, 0.2}
    local white = {1, 1, 1}
    local grey = ColorUtils.colorHex24ToStd(0x7D7D7D)

    local opacityDuration = 0.1
    local fadeDuration = 0.1
    local cardSlideInDuration = 0.3
    local cardSlideOutDuration = 0.3

    local bg = FilledRectangle.new(selfW, selfH, black)
    bg:setOpacity(0)
    self:insertAtCenter(bg, Z_BG)

    local correctButton = self.answerSelector:getAnswerButton(self.concept.correct)
    local incorrectButton = self.answerSelector:getAnswerButton(answer) or insertFakeButton(self)
    local otherButtons = self.answerSelector:getOtherButtons(answer, self.concept.correct)
    local overlay = insertNewAnswerOverlay(self, black, white)
    local overlayLabel = overlay.textLabel

    local explanationCard = ExpelliarmusExplanationCard.new(self.concept)
    explanationCard:setAnchorPointTop()
    explanationCard:setLoc(selfW/2, 0)
    self:insert(explanationCard, Z_EXPLANATION)

    local definitionLabel = createDefinitionLabel(self)
    self:insert(definitionLabel, Z_DEFINITION)

    local promptLabelOffsetAmount = self.promptLabel:getHeightPart(0.1)
    local promptLabelY = definitionLabel:getY() + definitionLabel:getHeightPart(0.5) + DEFINITION_MARGIN + promptLabelOffsetAmount
    local promptUpAction = AHSequence.new(
      AHDelay.new(0.1),
      AHLocationYTo.new(self.promptLabel, cardSlideInDuration, AHEaseSineInOut.new(), promptLabelY)
    )

    local cardInAction = AHSequence.new(
      AHDelay.new(0.1),
      AHLocationYTo.new(explanationCard, cardSlideInDuration, AHEaseSineInOut.new(), explanationCard:getHeight()),
      AHLocationYTo.new(explanationCard, cardSlideInDuration, AHEaseSineInOut.new(), explanationCard:getHeight()-ExpelliarmusCommon.CARD_BUMP_AMOUNT)
    )

    local fadeAction = AHSpawn.new(
      AHOpacityTo.new(bg, fadeDuration, nil, 1),
      AHOpacityTo.new(overlay, fadeDuration, nil, 1),
      AHColorTo.new(self.promptLabel, fadeDuration, nil, grey),
      AHColorTo.new(correctButton.backing, fadeDuration, nil, black),
      AHColorTo.new(incorrectButton.textLabel, fadeDuration, nil, black),
      AHColorTo.new(incorrectButton, fadeDuration, nil, white),
      AHColorTo.new(self.promptLabel:getBlankLabel(), fadeDuration, nil, black)
    )
    for _, widget in ipairs(self.uiOverlay:getWidgets()) do
      fadeAction:addAction(AHColorTo.new(widget, fadeDuration, nil, white))
    end

    for _, button in pairs(otherButtons) do
      fadeAction:addAction(AHOpacityTo.new(button, fadeDuration, nil, 0))
    end

    local takeTextAction = AHSequence.new(
      AHLocationYBy.new(overlayLabel, opacityDuration, AHEaseSineInOut.new(), -20),
      newTakeTextAction(self, overlayLabel, correctButton.isChange)
    )

    local fadeIncorrectAction = AHOpacityTo.new(incorrectButton, fadeDuration, nil, 0)
    local fadeDefinitionAction = AHOpacityTo.new(definitionLabel, fadeDuration, nil, COLOR_DESCRIPTION[4])

    local fadeOutAction = AHSpawn.new(
      AHSequence.new(
        AHLocationYBy.new(explanationCard, cardSlideOutDuration, AHEaseSineInOut.new(), ExpelliarmusCommon.CARD_BUMP_AMOUNT),
        AHLocationYTo.new(explanationCard, cardSlideOutDuration, AHEaseSineInOut.new(), 0)
      ),
      AHSequence.new(
        AHLocationXBy.new(self.promptLabel, cardSlideOutDuration, AHEaseSineInOut.new(), -ExpelliarmusCommon.CARD_BUMP_AMOUNT),
        AHLocationXBy.new(self.promptLabel, cardSlideOutDuration, AHEaseSineInOut.new(), self:getWidth())
      ),
      AHSequence.new(
        AHLocationXBy.new(definitionLabel, cardSlideOutDuration, AHEaseSineInOut.new(), -ExpelliarmusCommon.CARD_BUMP_AMOUNT),
        AHLocationXBy.new(definitionLabel, cardSlideOutDuration, AHEaseSineInOut.new(), self:getWidth())
      ),
      AHSequence.new(
        AHDelay.new(cardSlideOutDuration),
        AHColorTo.new(bg, cardSlideOutDuration, nil, white)
      )
    )

    local restoreUIAction = AHSpawn.new()
    for _, widget in ipairs(self.uiOverlay:getWidgets()) do
      restoreUIAction:addAction(AHColorTo.new(widget, fadeDuration, nil, ExpelliarmusCommon.COLOR_UI))
    end

    local action = AHSequence.new(
      fadeAction,
      AHSpawn.new(
        takeTextAction,
        fadeIncorrectAction,
        promptUpAction,
        cardInAction,
        AHSequence.new(
          AHDelay.new(0.2),
          function()
            self.promptLabel:removeBlankLabel()
          end,
          fadeDefinitionAction
        )
      ),
      function()
        self.answerSelector:removeSelf()
        overlay:removeSelf()
      end
    )

    wait(action:start())

    self.promptLabel:removeBlankLabel()

    local isDone = false
    TapToContinue.new(nil, nil, function()
      isDone = true
    end)

    waitUntil(function() return isDone end)
    local runningAction = fadeOutAction:start()
    waitFor(0.3)
    restoreUIAction:start()
    wait(runningAction)
  end)
end

function ExpelliarmusQuestionCard:setTouchEnabled(enabled)
  self.answerSelector:setTouchEnabled(enabled)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusQuestionCard.new(concept, topicNumber, uiOverlay, onQuestionAnswered)
  local self = Group.newFullScreen()
  Mixin.mix(self, ExpelliarmusQuestionCard)

  self.topicNumber = topicNumber
  local color = ExpelliarmusCommon.TOPIC_COLORS[self.topicNumber]

  local promptLabel = ExpelliarmusQuestionLabel.new(concept)
  promptLabel:setColor(unpack(COLOR_TEXT))

  local selector = ExpelliarmusAnswerSelector.new(concept.answers, color, onQuestionAnswered)
  selector:setTouchEnabled(false)
  selector:setAnchorPointTop()
  self:insert(selector, Z_ANSWERS)

  local topButton = selector.answerButtons[concept.answers[1]]
  local promptLabelY = Screen.getSafeY(0.5) + promptLabel:getHeightPart(0.5) + topButton:getHeight()
  promptLabel:setLoc(Screen.getSafeX(0.5), promptLabelY)
  self:insert(promptLabel, Z_QUESTION)

  local underline = promptLabel:getBlankLabel()
  underline:setColor(unpack(color))
  local answerSpacer = self:getHeightPart(0.0465)
  local underlineX, underlineY = self:transformObjectLoc(underline)
  selector:setLoc(underlineX, underlineY - answerSpacer)

  self.promptLabel = promptLabel
  self.buttonColor = color
  self.concept = concept
  self.answerSelector = selector
  self.uiOverlay = uiOverlay

  return self
end
