--- ExpelliarmusQuestionLabel
-- @author Created by karl on September 12, 2014.

class("ExpelliarmusQuestionLabel")
require("ExpelliarmusCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FONT_TEXT = Font.new(FontFaces.DIN_OT_LIGHT, 36)
local FONT_UNDERLINE = Font.new(FontFaces.DIN_OT_BOLD, 36)

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusQuestionLabel:getLeftLabel()
  return self.leftLabel
end

function ExpelliarmusQuestionLabel:getRightLabel()
  return self.rightLabel
end

function ExpelliarmusQuestionLabel:getBlankLabel()
  return self.underlineLabel
end

function ExpelliarmusQuestionLabel:removeBlankLabel()
  return self.underlineLabel:removeSelf()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newComposedGlyphLabel(text)
  local default = nil
  local buildGlyphs = true
  local label = Label.new(text, FONT_TEXT, default, default, buildGlyphs)
  return ComposedGlyphLabel.new(label, label:getGlyphTable())
end

--- Creata an underline label with kerning based on the last letter of the
-- left text in the concept.
local function newUnderlineLabel(concept)
  local underlineLabel = ExpelliarmusCommon.newKernedLabel(concept.leftText, "__", FONT_UNDERLINE)
  underlineLabel:setSpacing(-2) -- Make the underscores merge together.
  underlineLabel:alignChildren()

  return underlineLabel
end

-- Constructor ------------------------

function ExpelliarmusQuestionLabel.new(concept)
  local self = Row.new()
  Mixin.mix(self, ExpelliarmusQuestionLabel)

  local fullAnswer = concept.leftText .. concept.correct .. concept.rightText
  local fullAnswerLabel = newComposedGlyphLabel(fullAnswer)

  local label1Start, label1End = 1, #concept.leftText
  local label2Start, label2End = label1End+1, label1End+#concept.correct
  local label3Start, label3End = label2End+1, label2End+#concept.rightText

  self.leftLabel = fullAnswerLabel:sublabel(label1Start, label1End)
  self.rightLabel = fullAnswerLabel:sublabel(label3Start, label3End)
  self.underlineLabel = newUnderlineLabel(concept)

  self.leftLabel:inheritColor(self)
  self.rightLabel:inheritColor(self)

  self:insert(self.leftLabel)
  self:insert(self.underlineLabel)
  self:insert(self.rightLabel)
  self:alignChildren()

  return self
end
