--- ExpelliarmusRoundLabel
-- @author Created by karl on September 03, 2014.

class("ExpelliarmusRoundLabel")
require("ExpelliarmusCommon")
require("text.GlyphLabel")
require("text.ComposedGlyphLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FONT = Font.new(FontFaces.DIN_OT_BOLD, 34)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newAnimateInAction(glyph)
  local moveX = 10
  local rotate = 10
  local duration = 0.07
  local durationAdd = duration * 0.6
  local stagger = 0.005

  local sequence = AHSequence.new()
  sequence:addAction(AHStaggeredSpawn.new(stagger,
    AHSpawn.new(
      AHLocationXBy.new(glyph, duration, AHEaseSineInOut.new(), -moveX*0.5),
      AHOpacityTo.new(glyph, duration, nil, 1)
    ),
    AHRotationBy.new(glyph, duration, AHEaseSineInOut.new(), rotate)
  ))

  duration = duration  + durationAdd
  sequence:addAction(AHStaggeredSpawn.new(stagger,
    AHLocationXBy.new(glyph, duration, AHEaseSineInOut.new(), moveX*1.5),
    AHRotationBy.new(glyph, duration, AHEaseSineInOut.new(), -rotate*1.5)
  ))

  duration = duration  + durationAdd
  sequence:addAction(AHStaggeredSpawn.new(stagger,
    AHLocationXBy.new(glyph, duration, AHEaseSineInOut.new(), -moveX*1.0),
    AHRotationBy.new(glyph, duration, AHEaseSineInOut.new(), rotate*1.0)
  ))

  duration = duration  + durationAdd
  sequence:addAction(AHStaggeredSpawn.new(stagger,
    AHLocationXBy.new(glyph, duration, AHEaseSineInOut.new(), moveX*0.5),
    AHRotationBy.new(glyph, duration, AHEaseSineInOut.new(), -rotate*0.5)
  ))

  return sequence
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusRoundLabel:animateIn()
  local action = AHStaggeredSpawn.new(0.005)
  for _, glyph in ipairs(self.children) do
    action:addAction(newAnimateInAction(glyph))
  end

  return action:start()
end

function ExpelliarmusRoundLabel:animateOut()
  return AHOpacityTo.new(self, 0.3, nil, 0):start()
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusRoundLabel.new()
  local buildGlyphDescriptors = true
  local default = nil
  local label = Label.new(Localizer.get("ROUNDS"), FONT, default, default, buildGlyphDescriptors)

  local self = ComposedGlyphLabel.new(label)
  Mixin.mix(self, ExpelliarmusRoundLabel)

  for i, glyphLabel in ipairs(self.children) do
    local color = ExpelliarmusCommon.TOPIC_COLORS[math.ceil(i/2)]
    glyphLabel:setColor(unpack(color))
    glyphLabel:setOpacity(0)
  end

  return self
end
