--- ExpelliarmusTopicBar
-- @author Created by karl on September 18, 2014.

class("ExpelliarmusTopicBar")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 14)
local BAR_WIDTH = Screen.fullWidth
local BAR_HEIGHT = Screen.getHeightPart(0.05)

local BAR_LEFT_MARGIN = BAR_HEIGHT * 0.3

local SLIDE_AMOUNT = Screen.getWidthPart(0.8)
local SLIDE_IN_DURATION = 0.3
local SLIDE_OUT_DURATION = 0.2
local SMALL_BUMP_AMOUNT = Screen.getWidthPart(0.02)
local LARGE_BUMP_AMOUNT = Screen.getWidthPart(0.1)


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusTopicBar:newSlideInAction()
  local ease = AHEaseSineInOut.new()
  return AHSequence.new(
    AHLocationXBy.new(self, SLIDE_IN_DURATION, ease, -(SLIDE_AMOUNT + SMALL_BUMP_AMOUNT)),
    AHLocationXBy.new(self, SLIDE_IN_DURATION, ease, SMALL_BUMP_AMOUNT)
  )
end

function ExpelliarmusTopicBar:newBumpOutAction()
  return AHLocationXBy.new(self, SLIDE_IN_DURATION, AHEaseSineInOut.new(), -LARGE_BUMP_AMOUNT)
end

function ExpelliarmusTopicBar:newSlideOutAction()
  return AHLocationXBy.new(self, SLIDE_OUT_DURATION, nil, SLIDE_AMOUNT + LARGE_BUMP_AMOUNT)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusTopicBar.new(text, color)
  local self = FilledRectangle.new(BAR_WIDTH, BAR_HEIGHT, color)
  Mixin.mix(self, ExpelliarmusTopicBar)

  local label = Label.new(text, FONT)
  label:setAnchorPointLeft()
  label:setLoc(BAR_LEFT_MARGIN, self:getHeightPart(0.5))
  self:insert(label)

  return self
end
