--- ExpelliarmusTopicInterstitial
-- @author Created by karl on September 18, 2014.

class("ExpelliarmusTopicInterstitial")
require("ExpelliarmusTopicBar")
require("ExpelliarmusCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_BG = {1, 1, 1}
local FONT = Font.new(FontFaces.DIN_OT_BOLD, 34)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newSlideInAction(target)
  local ease = AHEaseSineInOut.new()
  local bumpAmount = Screen.getWidthPart(0.02)
  local duration = 0.3
  return AHSequence.new(
    AHLocationXBy.new(target, duration, ease, -Screen.fullWidth),
    AHLocationXBy.new(target, duration, ease, bumpAmount)
  )
end

local function newSlideOutAction(target, slideOutDuration)
  local ease = AHEaseSineInOut.new()
  local bumpAmount = Screen.getWidthPart(0.1)
  local bumpDuration = 0.2
  return AHSequence.new(
    AHLocationXBy.new(target, bumpDuration, ease, -bumpAmount),
    AHLocationXBy.new(target, slideOutDuration, ease, bumpAmount + Screen.fullWidth)
  )
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusTopicInterstitial:animateIn()
  return AHStaggeredSpawn.new(0.1,
    newSlideInAction(self.roundLabel),
    newSlideInAction(self.topicBar)
  ):start()
end

function ExpelliarmusTopicInterstitial:animateOut()
  return AHSpawn.new(
    newSlideOutAction(self.roundLabel, 0.3),
    newSlideOutAction(self.topicBar, 0.2)
  ):start()
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusTopicInterstitial.new(roundNumber, topic, customHeader)
  local self = Group.newFullScreen()
  Mixin.mix(self, ExpelliarmusTopicInterstitial)
  local selfW, selfH = self:getDims()

  local color = ExpelliarmusCommon.TOPIC_COLORS[roundNumber]
  local bg = FilledRectangle.new(selfW, selfH, COLOR_BG)
  self:insertAtCenter(bg)

  local centerY = Screen.getSafeY(0.55)
  local yOffset = self:getHeightPart(0.01)

  if customHeader then
    self.roundLabel = Label.new(customHeader, FONT)
  else
    local text = Localizer.get("ROUND") .. " "
    self.roundLabel = Label.new(text .. roundNumber, FONT)
  end
  self.roundLabel:setColor(unpack(color))
  self.roundLabel:setAnchorPointBottomLeft()
  self.roundLabel:setLoc(self:getWidthPart(1.2), centerY + yOffset)
  self:insert(self.roundLabel)

  self.topicBar = ExpelliarmusTopicBar.new(topic, color)
  self.topicBar:setAnchorPointTopLeft()
  self.topicBar:setLoc(self:getWidthPart(1.2), centerY - yOffset)
  self:insert(self.topicBar)

  return self
end
