--- ExpelliarmusTopicList
-- @author Created by karl on September 02, 2014.

class("ExpelliarmusTopicList")
require("ExpelliarmusCommon")
require("ExpelliarmusRoundLabel")
require("ExpelliarmusTopicBar")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TAP_TO_CONTINUE = ColorUtils.colorHex24ToStd(0x838383)
local TOPIC_SPACING = Screen.getWidthPart(0.02)
local COLOR_BG = {1, 1, 1}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusTopicList:animateIn()
  return runInCoroutine(function()
    local roundsAction = self.roundsLabel:animateIn()
    local barsAction = AHStaggeredSpawn.new(0.1)

    for _, bar in ipairs(self.bars) do
      barsAction:addAction(bar:newSlideInAction())
    end

    barsAction:start()
    wait(roundsAction, barsAction)
  end)
end

function ExpelliarmusTopicList:animateOut()
  return runInCoroutine(function()
    local roundsAction = self.roundsLabel:animateOut()
    local staggeredAction = AHStaggeredSpawn.new(0.07)

    for index, bar in ipairs(self.bars) do
      staggeredAction:addAction(bar:newSlideOutAction())
    end

    local barsAction = AHSequence.new(
      self.bars[1]:newBumpOutAction(),
      staggeredAction
    )

    barsAction:start()
    wait(roundsAction, barsAction)
    wait(AHOpacityTo.new(self, 0.2, nil, 0):start())
  end)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusTopicList.new(topics)
  local self = Group.newFullScreen()
  Mixin.mix(self, ExpelliarmusTopicList)

  local selfW, selfH = self:getDims()

  local bg = FilledRectangle.new(selfW, selfH, COLOR_BG)
  self:insertAtCenter(bg)

  self.roundsLabel = ExpelliarmusRoundLabel.new()
  self.roundsLabel:setAnchorPointLeft()
  self.roundsLabel:setLoc(Screen.getSafeX(0.185), Screen.getSafeY(0.67))
  self:insert(self.roundsLabel)

  local col = Column.new(TOPIC_SPACING)
  for index, topic in ipairs(topics) do
    local topicBar = ExpelliarmusTopicBar.new(topic, ExpelliarmusCommon.TOPIC_COLORS[index])
    col:insert(topicBar)
  end
  col:alignChildren()
  col:setAnchorPointTopLeft()
  col:setLoc(self:getWidth(), Screen.getSafeY(0.6))
  self:insert(col)

  self.bars = col.children

  local tapToContinueText = Localizer.get("Misc.TAP_TO_CONTINUE")
  self.tapToContinueLabel = Label.new(tapToContinueText, Font.new(FontFaces.DIN_OT_MEDIUM, 14))
  self.tapToContinueLabel:setColor(unpack(COLOR_TAP_TO_CONTINUE))

  self.tapToContinueLabel:setAnchorPointBottom()
  local minimumY = Screen.safeBottom + self.tapToContinueLabel:getHeight()
  local tapLabelY = math.max(minimumY, self.tapToContinueLabel:getHeightPart(1.5))
  self:insertAtLocation(self.tapToContinueLabel, Screen.getSafeX(0.5), tapLabelY)

  self:linkColorAttrToChildren()

  return self
end
