--- ExpelliarmusValidator
-- @author Created by karl on August 21, 2014.
-- ConceptValidator for Expelliarmus.

class("ExpelliarmusValidator")

require("validators.ConceptValidator")
require("validators.TagValidator")
require("ExpelliarmusConceptGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REPLACE_TAG = "replace"

local REQUIRED_KEYS = {
  'prompt',
  'incorrect1',
  'definition',
  'corr_nomark',
  'incorr_nomark1',
  'filter_type',
  'tip'
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function replaceNoChange(text)
  if text == ExpelliarmusConceptGenerator.CORRECT_MARKER then
    return ""
  end
  return text
end

local function expectStrippedValueContainsLettersOrSlashOnly(self, fieldName, fullValue, strippedValue)
  local match = string.find(strippedValue, "[^%a/]")
  if match then
    local char = strippedValue:sub(match, match)
    self:expectFailed("Field " .. fieldName .. " [" .. fullValue .. "] contains invalid character [" .. char .. "]")
  end
end

local function expectCommaSeparatedList(self, fieldName, value)
  local strippedValue = string.gsub(value, ", ", "")
  local foundComma = string.find(strippedValue, ",")
  local errorMessage = "Malformed list in " .. fieldName .. " [" .. value .. "]"
  self:expectNil(foundComma, errorMessage)
  expectStrippedValueContainsLettersOrSlashOnly(self, fieldName, value, strippedValue)
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpelliarmusValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)
  self:validateRequiredKeys(concept, REQUIRED_KEYS, true)
  self:assertHasTag(concept.prompt, REPLACE_TAG)

  expectCommaSeparatedList(self, "examples", concept.examples)
  expectCommaSeparatedList(self, "exceptions", concept.exceptions)

  self:validateTags(concept.tip, nil, nil, nil, {"i"})

  local left, mid, right = ExpelliarmusConceptGenerator.getTextComponents(concept.prompt)
  local correct = left .. replaceNoChange(mid) .. right
  local incorrect = left .. replaceNoChange(concept.incorrect1) .. right

  self:expectEqual(correct, concept.corr_nomark, "correct does not match corr_nomark")
  self:expectEqual(incorrect, concept.incorr_nomark1, "incorrect does not match incorr_nomark1")
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpelliarmusValidator.new()
  local self = ConceptValidator.new()
  TagValidator.make(self)
  Mixin.mix(self, ExpelliarmusValidator)

  return self
end
