--- ExpendituresConfiguration
-- @author Created by Jay Austria on October 20, 2014.

class("ExpendituresConfiguration")

require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ExpendituresConfiguration.CORRECT_ANSWER_POINTS = 1000
ExpendituresConfiguration.MAX_STREAK_BONUS_POINTS = 50
ExpendituresConfiguration.CONCEPT_COUNT_YEARLY = 3
ExpendituresConfiguration.CONCEPT_COUNT_MONTHLY = 3
ExpendituresConfiguration.CONCEPT_COUNT_WEEKLY = 3
ExpendituresConfiguration.NUMBER_INCORRECT_IN_FREQUENCY_TO_SHOW_HINT = 1

local CONSISTENCY_BONUS_RANK_THREE = 4000
local MISTAKES_ALLOWED_RANK_THREE = 0

local CONSISTENCY_BONUS_RANK_TWO = 2000
local MISTAKES_ALLOWED_RANK_TWO = 1

local GAME_DURATION = 85
local TIME_BONUS_PER_SECOND = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpendituresConfiguration.getConsistencyBonusForMistakes(mistakeCount)
  if (mistakeCount <= MISTAKES_ALLOWED_RANK_THREE) then
    return CONSISTENCY_BONUS_RANK_THREE
  elseif (mistakeCount <= MISTAKES_ALLOWED_RANK_TWO) then
    return CONSISTENCY_BONUS_RANK_TWO
  else
    return 0
  end
end

function ExpendituresConfiguration.getGameDuration()
  return GAME_DURATION
end

function ExpendituresConfiguration.getTimeBonusPerSecond()
  return TIME_BONUS_PER_SECOND
end

function ExpendituresConfiguration.getConceptsToComplete()
  local yearlyCount = ExpendituresConfiguration.CONCEPT_COUNT_YEARLY
  local monthlyCount = ExpendituresConfiguration.CONCEPT_COUNT_MONTHLY
  local weeklyCount = ExpendituresConfiguration.CONCEPT_COUNT_WEEKLY

  local completeCount = yearlyCount + monthlyCount + weeklyCount

  return completeCount
end

function ExpendituresConfiguration.getPassScore()
  local pointsPerCorrect = ExpendituresConfiguration.CORRECT_ANSWER_POINTS
  local passScore = ExpendituresConfiguration.getConceptsToComplete() * pointsPerCorrect
  return passScore
end

function ExpendituresConfiguration.getRankTwoScore()
  local passScore = ExpendituresConfiguration.getPassScore()
  local consistencyBonusForRankTwo = ExpendituresConfiguration.getConsistencyBonusForMistakes(MISTAKES_ALLOWED_RANK_TWO)
  local rankTwoScore = passScore + consistencyBonusForRankTwo
  return rankTwoScore
end

function ExpendituresConfiguration.getRankThreeScore()
  local passScore = ExpendituresConfiguration.getPassScore()
  local consistencyBonusForRankThree = ExpendituresConfiguration.getConsistencyBonusForMistakes(MISTAKES_ALLOWED_RANK_THREE)
  local rankThreeScore = passScore + consistencyBonusForRankThree
  return rankThreeScore
end

function ExpendituresConfiguration.getDifficultyKeyFrames(difficulties)
  return {
    {key = 0.0, valueProbabilities = {[difficulties[1]] = 1.0}},

    {key = 0.15, valueProbabilities = {[difficulties[1]] = 1.0}},

    {key = 0.225, valueProbabilities = {[difficulties[1]] = 0.9,
                                        [difficulties[2]] = 0.1}},

    {key = 0.3, valueProbabilities = {[difficulties[1]] = 0.75,
                                      [difficulties[2]] = 0.25}},

    {key = 0.375, valueProbabilities = {[difficulties[1]] = 0.50,
                                        [difficulties[2]] = 0.50}},

    {key = 0.45, valueProbabilities = {[difficulties[1]] = 0.10,
                                       [difficulties[2]] = 0.90}},

    {key = 0.4875, valueProbabilities = {[difficulties[2]] = 1.0}},

    {key = 0.5625, valueProbabilities = {[difficulties[2]] = 0.95,
                                        [difficulties[3]] = 0.05}},

    {key = 0.6, valueProbabilities = {[difficulties[2]] = 0.45,
                                      [difficulties[3]] = 0.55}},

    {key = 0.675, valueProbabilities = {[difficulties[2]] = 0.10,
                                        [difficulties[3]] = 0.90}},

    {key = 0.75, valueProbabilities = {[difficulties[3]] = 1.0}},

    {key = 1.0, valueProbabilities = {[difficulties[3]] = 1.0}}
  }
end
