--- ExpendituresGrid
-- @author Created by Jay Austria on Nov 4, 2014.

class("ExpendituresGrid")
require("graphics.TiledImage")

require("ExpendituresCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GRID_COLOR_INITIAL = ColorUtils.colorHex32ToStd(0x3d02c499)
local GRID_COLOR_INCORRECT = { ExpendituresCommon.Color.ORANGE[1], ExpendituresCommon.Color.ORANGE[2], ExpendituresCommon.Color.ORANGE[3], 0.7}
local COLOR_TO_DURATION = 0.05
local FADE_DELAY_DURATION = 0.05
local COLOR_BACK_DURATION = 0.7

local EXPECTED_COLUMN_COUNT = 8

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpendituresGrid:getIncorrectAnimation()
  local colorTo = AHColorTo.new(self, COLOR_TO_DURATION, nil, GRID_COLOR_INCORRECT)
  local fadeOutDelay = AHDelay.new(FADE_DELAY_DURATION)
  local colorBack = AHColorTo.new(self, COLOR_BACK_DURATION, nil, GRID_COLOR_INITIAL)

  local incorrectAnimation = AHSequence.new(colorTo, fadeOutDelay, colorBack)
  return incorrectAnimation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function ExpendituresGrid.new(textureCutOffY)
  local gridTileTexture = TextureLoader.loadTexture("tiles/grid_tile.png")
  local textureSize = gridTileTexture:getSize() / Host.contentScale

  local tileColumns = Screen.width / textureSize
  local textureScale = tileColumns / EXPECTED_COLUMN_COUNT

  local tiledWidth = Screen.width
  local tiledHeight = Screen.height - textureCutOffY

  local tileRows = math.floor(tiledHeight / (textureSize * textureScale))

  local self = Group.newFullScreen()
  Mixin.mix(self, ExpendituresGrid)

  local gridSprite = TiledImage.newWithTexture(gridTileTexture, tiledWidth, tiledHeight, textureScale)
  gridSprite:setLoc(Screen.width * 0.5, textureCutOffY + (tiledHeight / 2))
  self:insert(gridSprite)

  self:linkColorAttrToChildren()
  self:setColor(unpack(GRID_COLOR_INITIAL))

  return self
end
