--- ExpendituresHelper
-- @author Created by Jay Austria on November 9, 2014.

class("ExpendituresHelper")

require("ExpendituresHelperControls")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local EXAMPLE_COLOR = {1.0, 1.0, 1.0}
local BACKGROUND_COLOR = {0.059, 0.027, 0.298}

local GROUP_WIDTH = Screen.width
local GROUP_HEIGHT = ExpendituresCommon.HELPER_HEIGHT

local DEFAULT_EXPRESSION_Y = ((GROUP_HEIGHT - Screen.safeBottom) * 0.40) + Screen.safeBottom
local CENTER_X = GROUP_WIDTH * 0.5

local TIP_SPACING = 25
local LEFT_ALIGN_BUFFER = 30

local TIP_HEADER_FONT = FontFaces.DIN_OT_MEDIUM
local TIP_HEADER_FONT_SIZE = 18
local HEADER_COLOR = ExpendituresCommon.Color.CYAN

local TIP_FONT = FontFaces.DIN_OT_REGULAR
local TIP_FONT_SIZE = 20

local EXAMPLE_FONT = FontFaces.DIN_OT_REGULAR
local EXAMPLE_FONT_SIZE = 19

local YEARLY_TEXT_TEMPLATE = "TIP_YEARLY_%d"
local YEARLY_TEXT_COUNT = 2
local MONTHLY_TEXT_TEMPLATE = "TIP_MONTHLY_%d"
local MONTHLY_TEXT_COUNT = 4
local WEEKLY_TEXT_TEMPLATE = "TIP_WEEKLY_%d"
local WEEKLY_TEXT_COUNT = 5

local CONTINUE_TEXT = Localizer.get("Misc.RETURN_TO_GAME")
local CONTINUE_FONT = FontFaces.DIN_OT_MEDIUM
local CONTINUE_FONT_SIZE = 15
local CONTINUE_TEXT_BUFFER = GROUP_HEIGHT * 0.16

local PRICE_TEMPLATE = Localizer.get("CURRENCY_TEMPLATE")
local EQUALS_SIGN = "="
local MULTIPLICATION_SIGN = "×"
local PLUS_SIGN = "+"
local DIVIDE_SIGN = "/"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function updateStepText(self)
  local currentStep = self.pageIndex
  self.currentStepLabel:setText(currentStep)
end

local function createExampleLabel(text, font, fontSize)
  font = font or EXAMPLE_FONT
  fontSize = fontSize or EXAMPLE_FONT_SIZE

  local label = Label.new(text, Font.new(font, fontSize))
  label:setColor(unpack(EXAMPLE_COLOR))
  return label
end

local function getPriceString(price)
  return string.format(PRICE_TEMPLATE, price)
end

local function toggleButtonStates(self)
  local isFirstPage = (self.pageIndex == 1)
  local isLastPage = (self.pageIndex == (#self.equationLabels))

  self.arrowControls:updateButtonStates(isFirstPage, isLastPage)
end

local function getMoveAnimation(self, currentPage, targetPage)
  local isMovingForward = (currentPage < targetPage)
  local duration = 0.5
  local nextEquationDelay = 0.333
  local ease = nil
  local animation = AHSpawn.new()

  animation:addAction(AHSequence.new(
    AHOpacityTo.new(self.currentStepLabel, duration, ease, 0),
    function() updateStepText(self) end,
    AHOpacityTo.new(self.currentStepLabel, duration, ease, 1)
  ))

  local currentPageHasHelperLabel = ((currentPage > 0) and (currentPage <= #self.helperTextLabels))
  local targetPageHasHelperLabel = ((targetPage > 0) and (targetPage <= #self.helperTextLabels))

  if (currentPageHasHelperLabel and targetPageHasHelperLabel) then
    local helperTextDuration = duration
    local helperLabelForCurrentPage = self.helperTextLabels[currentPage]
    local helperLabelForTargetPage = self.helperTextLabels[targetPage]
    if (helperLabelForCurrentPage:getText() == helperLabelForTargetPage:getText()) then
      helperTextDuration = 0
    end
    animation:addAction(AHSequence.new(
      AHOpacityTo.new(helperLabelForCurrentPage, helperTextDuration, ease, 0),
      AHOpacityTo.new(helperLabelForTargetPage, helperTextDuration, ease, 1)
    ))
  end

  local currentPageHasEquationLabel = ((currentPage > 0) and (currentPage <= #self.equationLabels))
  local targetPageHasEquationLabel = ((targetPage > 0) and (targetPage <= #self.equationLabels))

  if (currentPageHasEquationLabel and targetPageHasEquationLabel) then
    local equationLabelDuration = duration
    local equationLabelDelay = nextEquationDelay
    local equationLabelForCurrentPage = self.equationLabels[currentPage]
    local equationLabelForTargetPage = self.equationLabels[targetPage]
    if (equationLabelForCurrentPage:getText() == equationLabelForTargetPage:getText()) then
      equationLabelDuration = 0
      equationLabelDelay = 0
    end
    if (not isMovingForward) then
      equationLabelDelay = 0
    end
    animation:addAction(AHSequence.new(
      AHOpacityTo.new(equationLabelForCurrentPage, equationLabelDuration, ease, 0),
      AHDelay.new(equationLabelDelay),
      AHOpacityTo.new(equationLabelForTargetPage, equationLabelDuration, ease, 1)
    ))
  end

  return animation
end

local function createHelperLabel(self, tipText)
  local font = Font.new(TIP_FONT, TIP_FONT_SIZE)
  local width = Screen.safeWidth - (LEFT_ALIGN_BUFFER * 2)
  local tipLabel = DynamicTextArea.newSimple(tipText, width, font)
  tipLabel:setAnchorPointTopLeft()
  local tipLabelY = self.tipHeaderLabel:getYBottom() - (TIP_SPACING / 2)
  tipLabel:setLoc(LEFT_ALIGN_BUFFER + Screen.safeLeft, tipLabelY)
  tipLabel:setOpacity(0)
  self:insert(tipLabel)

  return tipLabel
end

local function createStepLabel(self)
  local headerFont = Font.new(TIP_HEADER_FONT, TIP_HEADER_FONT_SIZE)

  local totalStepCount = #self.helperTextLabels
  local tipStepText = string.format(Localizer.get("TIP_STEP"), totalStepCount)
  local totalStepsLabel = Label.new(tipStepText, headerFont)
  totalStepsLabel:setColor(unpack(HEADER_COLOR))

  local adjustmentForRightAlignedText = LEFT_ALIGN_BUFFER + totalStepsLabel:getWidthPart(0.5)
  local tipStepLocationX = Screen.safeRight - adjustmentForRightAlignedText
  local tipStepLocationY = self.tipHeaderLabel:getYCenter()
  totalStepsLabel:setLoc(tipStepLocationX, tipStepLocationY)
  table.insert(self.fadeInElements, totalStepsLabel)
  self:insert(totalStepsLabel)

  local currentStepCount = 1
  local currentStepLabel = Label.new(currentStepCount, headerFont)
  currentStepLabel:setColor(unpack(HEADER_COLOR))

  local distanceBetweenWords = (headerFont:getActualSize() * 0.4)
  local distanceFromTotalStepsLabel = currentStepLabel:getWidth(0.5) + distanceBetweenWords
  local currentStepLocationX = totalStepsLabel:getXLeft() - distanceFromTotalStepsLabel
  currentStepLabel:setLoc(currentStepLocationX, tipStepLocationY)
  table.insert(self.fadeInElements, currentStepLabel)
  self:insert(currentStepLabel)

  self.currentStepLabel = currentStepLabel
end

local function createHelperLayout(self)
  local headerFont = Font.new(TIP_HEADER_FONT, TIP_HEADER_FONT_SIZE)

  local tipHeaderLabel = Label.new(Localizer.get("Misc.TIP"), headerFont)
  tipHeaderLabel:setColor(unpack(HEADER_COLOR))

  local tipHeaderLocationX = LEFT_ALIGN_BUFFER + tipHeaderLabel:getWidthPart(0.5) + Screen.safeLeft
  local tipHeaderLocationY = self:getHeight() - TIP_SPACING - tipHeaderLabel:getHeightPart(0.5)
  tipHeaderLabel:setLoc(tipHeaderLocationX, tipHeaderLocationY)
  table.insert(self.fadeInElements, tipHeaderLabel)
  self:insert(tipHeaderLabel)

  self.tipHeaderLabel = tipHeaderLabel

  local onTapReturn = function()
    self.canReturnToGame = true
  end

  local returnToGameLabel = Label.new(CONTINUE_TEXT, Font.new(CONTINUE_FONT, CONTINUE_FONT_SIZE))
  returnToGameLabel:setColor(unpack(EXAMPLE_COLOR))
  returnToGameLabel:setOpacity(0.72)
  Button.addVirtualButton(returnToGameLabel, returnToGameLabel:getWidthPart(2.0), returnToGameLabel:getHeightPart(3.0), onTapReturn)

  local returnToGameLabelY = math.max(ReferenceDevice.getScaledHeight(CONTINUE_TEXT_BUFFER), Screen.safeBottom)

  returnToGameLabel:setLoc(CENTER_X, returnToGameLabelY)
  self:insert(returnToGameLabel)

  local turnPageCallback = function(pageIndexModifier)
    local newPageIndex = self.pageIndex + pageIndexModifier
    local turnPageAnimation = getMoveAnimation(self, self.pageIndex, newPageIndex)
    self.pageIndex = newPageIndex
    AHDelayed.new(0.5, function() updateStepText(self) end):start()
    wait(turnPageAnimation:start())
    toggleButtonStates(self)
  end

  local arrowControls = ExpendituresHelperControls.new(self:getWidth(), self:getHeight(), DEFAULT_EXPRESSION_Y, LEFT_ALIGN_BUFFER, turnPageCallback)

  self:insertAtCenter(arrowControls)
  self.arrowControls = arrowControls

  toggleButtonStates(self)
  arrowControls:prepareToFadeIn()

  return self
end

local function createEquationLabel(self, labelText)
  local equationLabel = createExampleLabel(labelText)

  local maxWidth = self.arrowControls:getWidthBetweenButtons() * 0.9
  if (equationLabel:getWidth() > maxWidth) then
    equationLabel:setEffectiveWidth(maxWidth)
  end

  local exampleHeight = DEFAULT_EXPRESSION_Y
  equationLabel:setLoc(CENTER_X, exampleHeight)
  equationLabel:setOpacity(0)
  self:insert(equationLabel)

  return equationLabel
end

local function createStackedEquationLables(self, labelTextA, labelTextB)
  local stackedEquationLabels = Column.new()
  local equationlabelA = createEquationLabel(self, labelTextA)
  local equationlabelB = createEquationLabel(self, labelTextB)
  stackedEquationLabels:setLoc(equationlabelA:getLoc())
  stackedEquationLabels:takeChild(equationlabelA)
  stackedEquationLabels:takeChild(equationlabelB)
  stackedEquationLabels:alignChildren()
  stackedEquationLabels:linkOpacityAttrToChildren()
  stackedEquationLabels:setOpacity(0)
  stackedEquationLabels.getText = function() return labelTextA .. labelTextB end
  self:insert(stackedEquationLabels)
  return stackedEquationLabels
end

local function newWeekly(self, price, frequency)
  for i=1, WEEKLY_TEXT_COUNT do
    local localizerKey = string.format(WEEKLY_TEXT_TEMPLATE, i)
    local helperString = Localizer.get(localizerKey)
    local helperLabel = createHelperLabel(self, helperString)
    table.insert(self.helperTextLabels, helperLabel)
  end

  local unitPriceString = getPriceString(price)
  local weeklyPrice = price * frequency
  local weeklyPriceString = getPriceString(weeklyPrice)
  local frequencyString = string.format(Localizer.get("HELPER_WEEKLY"), frequency)
  if (frequency == 1) then
    frequencyString = string.format(Localizer.get("HELPER_WEEKLY_SINGULAR"), frequency)
  end

  local equationText = table.concat({unitPriceString, MULTIPLICATION_SIGN, frequencyString, EQUALS_SIGN, weeklyPriceString}, " ")
  local equationLabel = createEquationLabel(self, equationText)
  table.insert(self.equationLabels, equationLabel)


  local weeklyPriceTimesOneHundred = weeklyPrice * 100
  local weeklyPriceTimesFour = weeklyPrice * 4
  local weeklyPriceTimesOneHundredString = getPriceString(weeklyPriceTimesOneHundred)
  local weeklyPriceTimesFourString = getPriceString(weeklyPriceTimesFour)

  local timesOneHundredString = table.concat({weeklyPriceString, MULTIPLICATION_SIGN, "100", EQUALS_SIGN, weeklyPriceTimesOneHundredString}, " ")
  local timesFourString = table.concat({weeklyPriceString, MULTIPLICATION_SIGN, "4", EQUALS_SIGN, weeklyPriceTimesFourString}, " ")

  local weeklyEquationLabels = createStackedEquationLables(self, timesOneHundredString, timesFourString)
  table.insert(self.equationLabels, weeklyEquationLabels)


  local addedSum = weeklyPriceTimesOneHundred + weeklyPriceTimesFour
  local addedSumString = getPriceString(addedSum)
  local addTogetherString = table.concat({weeklyPriceTimesOneHundred, PLUS_SIGN, weeklyPriceTimesFour, EQUALS_SIGN, addedSumString}, " ")
  local addTogtherLabel = createEquationLabel(self, addTogetherString)
  table.insert(self.equationLabels, addTogtherLabel)


  local finalAnswer = addedSum / 2
  local finalAnswerString = getPriceString(finalAnswer)
  local divideByTwoString = table.concat({addedSumString, DIVIDE_SIGN, "2", EQUALS_SIGN, finalAnswerString}, " ")
  local finalAnswerLabel = createEquationLabel(self, divideByTwoString)
  table.insert(self.equationLabels, finalAnswerLabel)

  local finalSumLabel = createEquationLabel(self, finalAnswerString)
  table.insert(self.equationLabels, finalSumLabel)
end

local function newMonthly(self, price, frequency)
  for i=1, MONTHLY_TEXT_COUNT do
    local localizerKey = string.format(MONTHLY_TEXT_TEMPLATE, i)
    local helperString = Localizer.get(localizerKey)
    local helperLabel = createHelperLabel(self, helperString)
    table.insert(self.helperTextLabels, helperLabel)
  end

  local unitPriceString = getPriceString(price)
  local monthlyPrice = price * frequency
  local monthlyPriceString = getPriceString(monthlyPrice)
  local frequencyString = string.format(Localizer.get("HELPER_MONTHLY"), frequency)
  if (frequency == 1) then
    frequencyString = string.format(Localizer.get("HELPER_MONTHLY_SINGULAR"), frequency)
  end

  local equationText = table.concat({unitPriceString, MULTIPLICATION_SIGN, frequencyString, EQUALS_SIGN, monthlyPriceString}, " ")
  local equationLabel = createEquationLabel(self, equationText)
  table.insert(self.equationLabels, equationLabel)


  local monthlyPriceTimesTen = monthlyPrice * 10
  local monthlyPriceTimesTwo = monthlyPrice * 2
  local monthlyPriceTimesTenString = getPriceString(monthlyPriceTimesTen)
  local monthlyPriceTimesTwoString = getPriceString(monthlyPriceTimesTwo)

  local timesTenString = table.concat({monthlyPriceString, MULTIPLICATION_SIGN, "10", EQUALS_SIGN, monthlyPriceTimesTenString}, " ")
  local timesTwoString = table.concat({monthlyPriceString, MULTIPLICATION_SIGN, "2", EQUALS_SIGN, monthlyPriceTimesTwoString}, " ")

  local monthlyEquationLabels = createStackedEquationLables(self, timesTenString, timesTwoString)
  table.insert(self.equationLabels, monthlyEquationLabels)


  local finalSum = monthlyPriceTimesTen + monthlyPriceTimesTwo
  local finalSumString = getPriceString(finalSum)
  local finalSumEquationString = table.concat({monthlyPriceTimesTenString, PLUS_SIGN, monthlyPriceTimesTwoString, EQUALS_SIGN, finalSumString}, " ")
  local finalSumEquationLabel = createEquationLabel(self, finalSumEquationString)
  table.insert(self.equationLabels, finalSumEquationLabel)

  local finalSumLabel = createEquationLabel(self, finalSumString)
  table.insert(self.equationLabels, finalSumLabel)
end

local function newYearly(self, price, frequency)
  for i=1, YEARLY_TEXT_COUNT do
    local localizerKey = string.format(YEARLY_TEXT_TEMPLATE, i)
    local helperString = Localizer.get(localizerKey)
    local helperLabel = createHelperLabel(self, helperString)
    table.insert(self.helperTextLabels, helperLabel)
  end

  local unitPriceString = getPriceString(price)
  local totalPriceString = getPriceString(price * frequency)
  local frequencyString = string.format(Localizer.get("HELPER_ANNUALLY"), frequency)
  if (frequency == 1) then
    frequencyString = string.format(Localizer.get("HELPER_ANNUALLY_SINGULAR"), frequency)
  end

  local equationText = table.concat({unitPriceString, MULTIPLICATION_SIGN, frequencyString, EQUALS_SIGN, totalPriceString}, " ")
  local equationLabel = createEquationLabel(self, equationText)

  table.insert(self.equationLabels, equationLabel)

  local totalPriceLabel = createEquationLabel(self, totalPriceString)
  table.insert(self.equationLabels, totalPriceLabel)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpendituresHelper:returnToGame()
  return self.canReturnToGame
end

function ExpendituresHelper:getIntroAction()
  return self.introAction
end

function ExpendituresHelper:getHideAction()
  return self.hideAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function ExpendituresHelper.new(price, frequency, frequencyType)
  local self = FilledRectangle.new(GROUP_WIDTH, GROUP_HEIGHT, BACKGROUND_COLOR)
  Mixin.mix(self, ExpendituresHelper)

  self.fadeInElements = {self}

  self.animationElements = nil
  self.pageIndex = 1
  self.canReturnToGame = false
  self.helperTextLabels = {}
  self.equationLabels = {}

  createHelperLayout(self)

  if frequencyType == ExpendituresCommon.FrequencyType.YEARLY then
    newYearly(self, price, frequency)
  elseif frequencyType == ExpendituresCommon.FrequencyType.MONTHLY then
    newMonthly(self, price, frequency)
  elseif frequencyType == ExpendituresCommon.FrequencyType.WEEKLY then
    newWeekly(self, price, frequency)
  end

  createStepLabel(self)

  self.activeAnimation = AHDoNothing.new()

  local showDuration = 0.5
  local hideDuration = 0.25
  local ease = nil
  local opacity = 1.0
  self.introAction = AHSpawn.new(
    AHOpacityTo.new(self.equationLabels[self.pageIndex], showDuration, ease, opacity),
    AHOpacityTo.new(self.helperTextLabels[self.pageIndex], showDuration, ease, opacity))

  self.hideAction = AHSpawn.new(function() self.activeAnimation:stop() end)
  for _, equationLabel in ipairs(self.equationLabels) do
    self.hideAction:addAction(AHOpacityTo.new(equationLabel, hideDuration, ease, 0))
  end
  for _, helperTextLabel in ipairs(self.helperTextLabels) do
    self.hideAction:addAction(AHOpacityTo.new(helperTextLabel, hideDuration, ease, 0))
  end

  for _, element in ipairs(self.fadeInElements) do
    element:setOpacity(0)
    local targetOpacity = opacity
    if element.fadeInOpacity ~= nil then
      targetOpacity = element.fadeInOpacity
    end
    self.introAction:addAction(AHOpacityTo.new(element, showDuration, ease, targetOpacity))
    self.hideAction:addAction(AHOpacityTo.new(element, hideDuration, ease, 0))
  end

  self.introAction:addAction(self.arrowControls:getFadeInAnimation(showDuration, ease))
  self.hideAction:addAction(self.arrowControls:getFadeOutAnimation(hideDuration, ease))

  return self
end
