--- ExpendituresHelperArrow
-- @author Created by andrewc on March 22, 2021.

class("ExpendituresHelperArrow")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ExpendituresHelperArrow.LEFT = "left"
ExpendituresHelperArrow.RIGHT = "right"

local SPRITE_NAME = "tiparrow.png"

local SPRITE_MINIMUM_OPACITY = 0.25
local SPRITE_MAXIMUM_OPACITY = 1.0

local BACKGROUND_COLOR = ExpendituresCommon.Color.WHITE

local BACKGROUND_RADIUS_AS_PROPORTION_OF_ARROW_HEIGHT = 1.2
local BACKGROUND_MINIMUM_OPACITY = 0.1
local BACKGROUND_MAXIMUM_OPACITY = 0.2

local BUTTON_SIZE_PROPORTION_OF_ARROW = 5.0

local BUTTON_TAP_ANIMATION_SCALE = 1.5
local BUTTON_TAP_ANIMATION_DURATION = 0.1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getFadeAnimation(self, duration, ease, backgroundOpacity, spriteOpacity, touchEnabled)
  return AHSpawn.new(
    AHOpacityTo.new(self, duration, ease, backgroundOpacity),
    AHOpacityTo.new(self.arrowSprite, duration, ease, spriteOpacity),
    function() self:setTouchEnabled(touchEnabled) end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpendituresHelperArrow:createTappedAnimation()
  return AHScaleTo.new(self, BUTTON_TAP_ANIMATION_DURATION, AHEaseSimpleInOut.new(), BUTTON_TAP_ANIMATION_SCALE)
end

function ExpendituresHelperArrow:createScaleToAnimation(scale)
  return AHScaleTo.new(self, BUTTON_TAP_ANIMATION_DURATION, nil, scale)
end

function ExpendituresHelperArrow:setEnabled(enabled)
  self:setTouchEnabled(enabled)

  if (enabled) then
    self:setOpacity(BACKGROUND_MAXIMUM_OPACITY)
    self.arrowSprite:setOpacity(SPRITE_MAXIMUM_OPACITY)
  else
    self:setOpacity(BACKGROUND_MINIMUM_OPACITY)
    self.arrowSprite:setOpacity(SPRITE_MINIMUM_OPACITY)
  end

  self.isEnabled = enabled
end

function ExpendituresHelperArrow:prepareToFadeIn()
  self:setTouchEnabled(false)
  self:setOpacity(0)
  self.arrowSprite:setOpacity(0)
end

function ExpendituresHelperArrow:getFadeInAnimation(duration, ease)
  local backgroundOpacity = BACKGROUND_MAXIMUM_OPACITY
  local spriteOpacity = SPRITE_MAXIMUM_OPACITY
  if (not self.isEnabled) then
    backgroundOpacity = BACKGROUND_MINIMUM_OPACITY
    spriteOpacity = SPRITE_MINIMUM_OPACITY
  end

  return getFadeAnimation(self, duration, ease, backgroundOpacity, spriteOpacity, self.isEnabled)
end

function ExpendituresHelperArrow:getFadeOutAnimation(duration, ease)
  local isTouchEnabled = false
  return getFadeAnimation(self, duration, ease, 0, 0, isTouchEnabled)
end

function ExpendituresHelperArrow:getArrowSprite()
  return self.arrowSprite
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ExpendituresHelperArrow.new(arrowDirection, onTouchCallback)
  local arrowSprite = Sprite.newWithName(SPRITE_NAME)
  local backgroundRadius = arrowSprite:getHeight() * BACKGROUND_RADIUS_AS_PROPORTION_OF_ARROW_HEIGHT

  local self = FilledCircle.new(backgroundRadius, BACKGROUND_COLOR)
  Mixin.mix(self, ExpendituresHelperArrow)

  self:setOpacity(BACKGROUND_MAXIMUM_OPACITY)
  arrowSprite:setOpacity(SPRITE_MINIMUM_OPACITY)

  if (arrowDirection == ExpendituresHelperArrow.LEFT) then
    arrowSprite:setRot(180)
  elseif (arrowDirection ~= ExpendituresHelperArrow.RIGHT) then
    print("WARNING - Helper arrow direction must be either left or right.")
  end
  self:insertAtCenter(arrowSprite)

  local buttonWidth = arrowSprite:getWidth() * BUTTON_SIZE_PROPORTION_OF_ARROW
  local buttonHeight = arrowSprite:getHeight() * BUTTON_SIZE_PROPORTION_OF_ARROW
  Button.addVirtualButton(self, buttonWidth, buttonHeight, onTouchCallback)

  self.arrowSprite = arrowSprite

  self.isEnabled = false

  return self
end
