--- ExpendituresHelperControls
-- @author Created by andrewc on March 22, 2021.

class("ExpendituresHelperControls")

require("ExpendituresHelperArrow")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function disableButtons(self)
  self.leftArrow:setTouchEnabled(false)
  self.rightArrow:setTouchEnabled(false)
end

local function buttonPressedFunction(self, direction)
  local arrowPressed = self.rightArrow
  local indexModifier = 1
  if (direction == ExpendituresHelperArrow.LEFT) then
    arrowPressed = self.leftArrow
    indexModifier = -1
  end

  wait(arrowPressed:createTappedAnimation():start())

  AHSpawn.new(
    self.rightArrow:createScaleToAnimation(0.0),
    self.leftArrow:createScaleToAnimation(0.0)
  ):start()

  wait(self.arrowTappedCallback(indexModifier))

  AHSpawn.new(
    self.rightArrow:createScaleToAnimation(1.0),
    self.leftArrow:createScaleToAnimation(1.0)
  ):start()
end

local function getButtonPressedCallbackFunction(self, direction)
  return function()
    disableButtons(self)
    self.currentCoroutine = runInCoroutine(function()
      buttonPressedFunction(self, direction)
    end)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpendituresHelperControls:prepareToFadeIn()
  self.leftArrow:prepareToFadeIn()
  self.rightArrow:prepareToFadeIn()
end

function ExpendituresHelperControls:getFadeInAnimation(duration, ease)
  return AHSpawn.new(
    self.leftArrow:getFadeInAnimation(duration, ease),
    self.rightArrow:getFadeInAnimation(duration, ease))
end

function ExpendituresHelperControls:getFadeOutAnimation(duration, ease)
  return AHSpawn.new(
    function() self.currentCoroutine:stop() end,
    self.leftArrow:getFadeOutAnimation(duration, ease),
    self.rightArrow:getFadeOutAnimation(duration, ease))
end

function ExpendituresHelperControls:updateButtonStates(isFirstPage, isLastPage)
  self.leftArrow:setEnabled(not isFirstPage)
  self.rightArrow:setEnabled(not isLastPage)
end

function ExpendituresHelperControls:getWidthBetweenButtons()
  return (self.rightArrow:getXLeft() - self.leftArrow:getXRight())
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createArrowForDirection(self, direction)
  local callbackFunction = getButtonPressedCallbackFunction(self, direction)
  return ExpendituresHelperArrow.new(direction, callbackFunction)
end

local function createLayout(self, arrowHeight, sideBuffer)
  local leftArrow = createArrowForDirection(self, ExpendituresHelperArrow.LEFT)
  local rightArrow = createArrowForDirection(self, ExpendituresHelperArrow.RIGHT)

  local bufferAdjustment = leftArrow:getWidthPart(0.5) - (leftArrow:getWidth() - leftArrow:getArrowSprite():getWidth()) / 2
  local buffer = sideBuffer + bufferAdjustment

  leftArrow:setLoc(buffer + Screen.safeLeft, arrowHeight)
  self:insert(leftArrow)

  rightArrow:setLoc(Screen.safeRight - buffer, arrowHeight)
  self:insert(rightArrow)

  self.leftArrow = leftArrow
  self.rightArrow = rightArrow
end

-- Constructor ------------------------

function ExpendituresHelperControls.new(width, height, arrowHeight, sideBuffer, arrowTappedCallback)
  local self = Group.new(width, height)
  Mixin.mix(self, ExpendituresHelperControls)

  createLayout(self, arrowHeight, sideBuffer)

  self.arrowTappedCallback = arrowTappedCallback

  self.currentCoroutine = MOAICoroutine.new()

  return self
end
