--- ExpendituresIconStage
-- @author Created by Jay Austria on October 30, 2014.

class("ExpendituresIconStage")
require("ExpendituresBanner")
require("ExpendituresIcon")
require("ExpendituresItemsLeftLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SURFACE_GRADIENT_COLOR_BOTTOM = ColorUtils.colorHex32ToStd(0x26147600)
local SURFACE_GRADIENT_COLOR_TOP = ColorUtils.colorHex32ToStd(0x331B9BCC)
local SURFACE_GRADIENT_HEIGHT = 25

local ICON_END_LOCATION_X = Screen.width * 1.2

local SLIDE_ACROSS_DURATION = 0.22
local SLIDE_ACROSS_STAGGER_DURATION = 0.12

local CORRECT_SLIDE_OUT_DURATION = 0.3

local PRESENT_ICON_DURATION = 0.3
local BANNER_OFFSET_Y = -28

local ITEMS_LEFT_LABEL_LOCATION_X = Screen.getWidthPart(0.5)
local ITEMS_LEFT_LABEL_LOCATION_Y = -30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createNewBanner(self)
  local banner = ExpendituresBanner.new()
  banner:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5) + BANNER_OFFSET_Y)
  self:insert(banner)

  return banner
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpendituresIconStage:getCorrectAnimation(icon)
  local iconCorrectAnimation = icon:getCorrectAnimation()

  local rockIcon = icon:getExitRockAnimation()
  local slideOut = AHLocationXTo.new(icon, CORRECT_SLIDE_OUT_DURATION, AHEaseSimpleIn.new(), ExpendituresCommon.ICON_OFFSCREEN_LOCATION_X)
  local slideBanner = AHLocationXTo.new(self.banner, CORRECT_SLIDE_OUT_DURATION, AHEaseSimpleIn.new(), ExpendituresCommon.ICON_OFFSCREEN_LOCATION_X)
  local function removeGroups()
    icon:removeSelf()
    self.banner:removeSelf()
  end

  local leaveAction = AHSpawn.new(rockIcon, slideOut, slideBanner)

  local correctAnimation = AHSequence.new(iconCorrectAnimation, leaveAction, removeGroups)

  return correctAnimation
end

function ExpendituresIconStage:getIncorrectAnimation(icon)
  local iconIncorrectAnimation = icon:getIncorrectAnimation()
  local revealIncorrectBanner = AHOpacityTo.new(self.banner.incorrectBanner, ExpendituresCommon.INCORRECT_MORPH_DURATION, nil, 1.0)

  local incorrectAnimation = AHSpawn.new(iconIncorrectAnimation, revealIncorrectBanner)

  return incorrectAnimation
end

function ExpendituresIconStage:getRejectAnimation(icon)
  local rejectXLocation = ICON_END_LOCATION_X

  local moveIcon = AHLocationXTo.new(icon, PRESENT_ICON_DURATION, AHEaseSimpleIn.new(), rejectXLocation)
  local moveBanner = AHLocationXTo.new(self.banner, PRESENT_ICON_DURATION, AHEaseSimpleIn.new(), rejectXLocation)

  local slideObjects = AHSpawn.new(moveIcon, moveBanner)

  local function removeGroups()
    self.banner:removeSelf()
    -- Don't yet remove the icon from the scene. He's at the end of the line to be shown again.
  end

  local rejectAnimation = AHSequence.new(slideObjects, removeGroups)

  return rejectAnimation
end

function ExpendituresIconStage:getPresentIconAnimation(icon)
  local destinationX = self:getWidthPart(0.5)

  local function prepIcon()
    icon:prepForPresent()
  end
  local moveAction = AHLocationXTo.new(icon, PRESENT_ICON_DURATION, nil, destinationX)
  local priceFadeIn = icon:getPriceFadeInAction()
  local moveAndArriveSequence = AHSequence.new(prepIcon, moveAction, priceFadeIn)

  local rockAction = icon:getRockAnimation()

  local newBanner = createNewBanner(self)
  local unrollAction = newBanner:newUnrollAnimation()
  self.banner = newBanner

  local presentAction = AHSpawn.new(moveAndArriveSequence, rockAction, unrollAction)


  presentAction:start()
end

function ExpendituresIconStage:getIntroAnimation(icons)
  local introAction = AHSpawn.new()
  for i, icon in ipairs(icons) do
    local appearanceDelay = SLIDE_ACROSS_STAGGER_DURATION * (#icons - (i - 1))
    local slideAction = AHLocationXTo.new(icon, SLIDE_ACROSS_DURATION, nil, ICON_END_LOCATION_X)

    local iconIntroAction = AHDelayed.new(appearanceDelay, slideAction)
    introAction:addAction(iconIntroAction)
  end

  return introAction
end

function ExpendituresIconStage:showProgress(progressValue)
  self.itemsLeftLabel:setValue(progressValue)
  self.itemsLeftLabel:show()
end

function ExpendituresIconStage:hideProgress()
  self.itemsLeftLabel:hide()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function ExpendituresIconStage.new()
  local self = Group.new(Screen.width, SURFACE_GRADIENT_HEIGHT)
  Mixin.mix(self, ExpendituresIconStage)

  local surfaceGradient = RectangleGradient.new(Screen.width, SURFACE_GRADIENT_HEIGHT, SURFACE_GRADIENT_COLOR_BOTTOM, SURFACE_GRADIENT_COLOR_TOP)
  self:insertAtCenter(surfaceGradient)

  local initialBanner = createNewBanner(self)
  self.banner = initialBanner

  local itemsLeftLabel = ExpendituresItemsLeftLabel.new()
  itemsLeftLabel:setLoc(ITEMS_LEFT_LABEL_LOCATION_X, ITEMS_LEFT_LABEL_LOCATION_Y)
  self:insert(itemsLeftLabel)
  itemsLeftLabel:hide()
  self.itemsLeftLabel = itemsLeftLabel

  return self
end
