--- ExpendituresItemsLeftLabel
-- @author Created by Jay Austria on November 19, 2014.

class("ExpendituresItemsLeftLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INITIAL_VALUE = 0
local VALUE_LABEL_FONT = FontFaces.DIN_OT_BOLD
local VALUE_LABEL_FONT_SIZE = 40

local TOP_TEXT = "ITEMS"
local BOTTOM_TEXT = "LEFT"
local WORDS_LABEL_FONT = FontFaces.DIN_OT_BOLD
local WORDS_LABEL_FONT_SIZE = 14

local NUMBER_LABEL_SPACING = 8
local WORDS_LABEL_SPACING = -2

local VALUE_AND_WORDS_ALIGNMENT_NUDGE_OFFSET = 5

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------
function ExpendituresItemsLeftLabel:setValue(value)
  self.valueLabel:setText(value)

  self:alignChildren()
end

function ExpendituresItemsLeftLabel:show()
  self:setOpacity(1.0)
end

function ExpendituresItemsLeftLabel:hide()
  self:setOpacity(0.0)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout()
  local self = Row.new(NUMBER_LABEL_SPACING)

  local valueAsString = string.format("%d", INITIAL_VALUE)
  local valueLabel = Label.new(valueAsString, Font.new(VALUE_LABEL_FONT, VALUE_LABEL_FONT_SIZE))
  self:insert(valueLabel)
  self.valueLabel = valueLabel

  local wordsColumn = Column.new(WORDS_LABEL_SPACING, Alignment.LEFT, Column.GrowUp)
  local topLabel = Label.new(TOP_TEXT, Font.new(WORDS_LABEL_FONT, WORDS_LABEL_FONT_SIZE))
  local bottomLabel = Label.new(BOTTOM_TEXT, Font.new(WORDS_LABEL_FONT, WORDS_LABEL_FONT_SIZE))

  wordsColumn:insert(bottomLabel)
  wordsColumn:insert(topLabel)
  wordsColumn:insertSpacer(VALUE_AND_WORDS_ALIGNMENT_NUDGE_OFFSET)

  wordsColumn:alignChildren()

  self:insert(wordsColumn)
  self:alignChildren()

  local linkRecursively = true
  self:linkOpacityAttrToChildren(linkRecursively)

  return self
end

function ExpendituresItemsLeftLabel.new()
  local self = createLayout()
  Mixin.mix(self, ExpendituresItemsLeftLabel)

  return self
end
