--- ExpendituresOutroLayer
-- @author Created by Jay Austria on November 07, 2014.

class("ExpendituresOutroLayer")
require("graphics.LinePathTracer")
require("ExpendituresCommon")
require("ExpendituresTrajectoryBuilder")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BG_TOP = ColorUtils.colorHex24ToStd(0x00282B)
local BG_BOTTOM = ColorUtils.colorHex24ToStd(0x00A8A6)
local INVISIBLE_COLOR = {0,0,0,0}

local FADE_IN_DURATION = 0.3

local CHECK_TILE_WIDTH = 40
local CHECK_WIDTH = CHECK_TILE_WIDTH * 5
local CHECK_HEIGHT = CHECK_TILE_WIDTH * 3

local BOTTOM_LEFT = "bottom_left"
local BOTTOM_RIGHT = "bottom_right"
local TOP_LEFT = "top_left"
local TOP_RIGHT = "top_right"

local TRIANGLE_COORDS =
{
  [BOTTOM_LEFT] = {0, 0, CHECK_TILE_WIDTH, 0, 0, CHECK_TILE_WIDTH},
  [BOTTOM_RIGHT] = {0, 0, CHECK_TILE_WIDTH, 0, CHECK_TILE_WIDTH, CHECK_TILE_WIDTH},
  [TOP_LEFT] = {0, CHECK_TILE_WIDTH, 0, 0, CHECK_TILE_WIDTH, CHECK_TILE_WIDTH},
  [TOP_RIGHT] = {0, CHECK_TILE_WIDTH, CHECK_TILE_WIDTH, 0, CHECK_TILE_WIDTH, CHECK_TILE_WIDTH}
}

local TRIANGLE_COLOR =
{
  A = ColorUtils.colorRGBToStd(16, 246, 233),
  B = ColorUtils.colorRGBToStd(127, 255, 245),
  C = ColorUtils.colorRGBToStd(0, 216, 203),
  D = ColorUtils.colorRGBToStd(94, 255, 246),
  E = ColorUtils.colorRGBToStd(68, 234, 224)
}

local TRIANGLES =
{
  {
    orientation = TOP_RIGHT,
    gridPosition = {0, 2},
    color = TRIANGLE_COLOR.A
  },
  {
    orientation = BOTTOM_LEFT,
    gridPosition = {0, 2},
    color = TRIANGLE_COLOR.B
  },
  {
    orientation = TOP_LEFT,
    gridPosition = {0, 1},
    color = TRIANGLE_COLOR.C
  },
  {
    orientation = BOTTOM_RIGHT,
    gridPosition = {0, 1},
    color = TRIANGLE_COLOR.D
  },
  {
    orientation = TOP_RIGHT,
    gridPosition = {0, 0},
    color = TRIANGLE_COLOR.A
  },
  {
    orientation = BOTTOM_LEFT,
    gridPosition = {0, 0},
    color = TRIANGLE_COLOR.D
  },
  {
    orientation = TOP_LEFT,
    gridPosition = {1, 0},
    color = TRIANGLE_COLOR.A
  },
  {
    orientation = BOTTOM_RIGHT,
    gridPosition = {1, 0},
    color = TRIANGLE_COLOR.C
  },
  {
    orientation = TOP_RIGHT,
    gridPosition = {2, 0},
    color = TRIANGLE_COLOR.A
  },
  {
    orientation = BOTTOM_LEFT,
    gridPosition = {2, 0},
    color = TRIANGLE_COLOR.E
  },
  {
    orientation = TOP_LEFT,
    gridPosition = {3, 0},
    color = TRIANGLE_COLOR.A
  },
  {
    orientation = BOTTOM_RIGHT,
    gridPosition = {3, 0},
    color = TRIANGLE_COLOR.E
  },
  {
    orientation = TOP_RIGHT,
    gridPosition = {4, 0},
    color = TRIANGLE_COLOR.B
  },
  {
    orientation = BOTTOM_LEFT,
    gridPosition = {4, 0},
    color = TRIANGLE_COLOR.A
  },
}

-- Assumes a 40 frame animation
local LINES =
{
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 28, x = 10, y = 16},
    {frameNumber = 32, x = 12, y = 16},
    {frameNumber = 40, x = 12, y = 20},
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 15, x = 14, y = 14},
    {frameNumber = 20, x = 14, y = 17},
    {frameNumber = 25, x = 16, y = 17},
    {frameNumber = 30, x = 16, y = 18},
    {frameNumber = 40, x = 20, y = 22}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 10, x = 14, y = 14},
    {frameNumber = 15, x = 16, y = 14},
    {frameNumber = 40, x = 22, y = 20}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 25, x = 16, y = 10},
    {frameNumber = 30, x = 16, y = 12},
    {frameNumber = 40, x = 20, y = 12}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 15, x = 14, y = 8},
    {frameNumber = 30, x = 16, y = 9},
    {frameNumber = 40, x = 21, y = 6}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 15, x = 16, y = 6},
    {frameNumber = 20, x = 12, y = 4},
    {frameNumber = 25, x = 14, y = 3},
    {frameNumber = 30, x = 16, y = 4},
    {frameNumber = 40, x = 22, y = 0}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 40, x = 10, y = -2}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 7, x = 10, y = 8},
    {frameNumber = 13, x = 6, y = 6},
    {frameNumber = 19, x = 6, y = 4},
    {frameNumber = 26, x = 3, y = 2},
    {frameNumber = 34, x = 5, y = 1},
    {frameNumber = 40, x = 3, y = -1}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 15, x = 2, y = 6},
    {frameNumber = 25, x = 2, y = 4},
    {frameNumber = 40, x = -4, y = 0}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 10, x = 8, y = 10},
    {frameNumber = 20, x = 6, y = 9},
    {frameNumber = 40, x = -4, y = 9}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 10, x = 8, y = 10},
    {frameNumber = 20, x = 6, y = 11},
    {frameNumber = 30, x = 3, y = 11},
    {frameNumber = 40, x = 0, y = 13}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 20, x = 4, y = 13},
    {frameNumber = 30, x = 4, y = 15},
    {frameNumber = 40, x = 0, y = 18}
  },
  {
    {frameNumber = 0, x = 10, y = 10},
    {frameNumber = 16, x = 6, y = 12},
    {frameNumber = 27, x = 6, y = 16},
    {frameNumber = 40, x = 0, y = 21}
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpendituresOutroLayer:getAnimation()
  local showSelf = self:newGradientToAction(FADE_IN_DURATION, nil, BG_TOP, BG_BOTTOM)

  local showTriangles = AHSpawn.new()

  for i, triangle in ipairs(self.triangles) do
    local delayDuration = (i-1) * 0.02 + 0.5
    local showDelay = AHDelay.new(delayDuration)

    local showAction = AHOpacityTo.new(triangle, 0.05, nil, 1.0)
    local destinationColor = TRIANGLES[i].color
    local changeColor = AHColorTo.new(triangle, 0.1, nil, destinationColor)

    local showTriangle = AHSequence.new(showDelay, showAction, changeColor)
    showTriangles:addAction(showTriangle)
  end

  local lineDelay = AHDelay.new(0.7)
  local lineSequence = AHSequence.new(lineDelay, self.lineAnimations)

  local expandBlurCircle = AHScaleTo.new(self.blurCircle, 0.5, AHEaseSimpleOut.new(), 15)
  local fadeBlurCircle = AHOpacityTo.new(self.blurCircle, 0.7, nil, 0.0)
  local circleAnimation = AHSpawn.new(expandBlurCircle, fadeBlurCircle)
  local circleDelay = AHDelay.new(0.7)
  local circleSequence = AHSequence.new(circleDelay, circleAnimation)

  local outroLengthDelay = AHDelay.new(3.2)

  local outroAnimation = AHSpawn.new(showSelf, showTriangles, lineSequence, circleSequence, outroLengthDelay)
  return outroAnimation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createCheckMark(self)
  local checkGroup = Group.new(CHECK_WIDTH, CHECK_HEIGHT)
  checkGroup:setRot(45)
  checkGroup:setLoc(self:getWidthPart(0.52), self:getHeightPart(0.6))
  self:insert(checkGroup)
  self.checkGroup = checkGroup

  self.triangles = {}

  for _, triangle in ipairs(TRIANGLES) do
    local triangleOrientation = triangle.orientation
    local x1, y1, x2, y2, x3, y3 = unpack(TRIANGLE_COORDS[triangleOrientation])
    local newTriangle = FilledTriangle.new(x1, y1, x2, y2, x3, y3, ExpendituresCommon.Color.WHITE)
    local triangleX = triangle.gridPosition[1] * CHECK_TILE_WIDTH + CHECK_TILE_WIDTH * 0.5
    local triangleY = triangle.gridPosition[2] * CHECK_TILE_WIDTH + CHECK_TILE_WIDTH * 0.5

    newTriangle:setLoc(triangleX, triangleY)
    newTriangle:setOpacity(0)
    table.insert(self.triangles, newTriangle)

    self.checkGroup:insert(newTriangle)
  end
end


local function createLineEffect(self)
  local lineAnimations = AHSpawn.new()

  local LINE_VIEW_PARAMS =
  {
    lineWidth = 2,
    maxVisibleViews = 10,
    maxTailLength = 10
  }

  for _, line in ipairs(LINES) do
    local trajectoryBuilder = ExpendituresTrajectoryBuilder.new(line)
    local linePath = LinePathTracer.new(LINE_VIEW_PARAMS, trajectoryBuilder)

    local lineAnimation = linePath:createTracePathAnimation(0, 0, 0, 0, 0.7)
    lineAnimations:addAction(lineAnimation)

    self:insertAtCenter(linePath)
  end

  self.lineAnimations = lineAnimations
end

local function createLayout(self)
  createLineEffect(self)
  createCheckMark(self)

  local blurCircle = Sprite.newWithName("blur_circle.png")
  blurCircle:setScl(0)
  blurCircle:setColor(unpack(ExpendituresCommon.Color.CYAN))
  self:insertAtCenter(blurCircle)
  self.blurCircle = blurCircle
end

function ExpendituresOutroLayer.new()
  local self = RectangleGradient.new(Screen.width, Screen.height, INVISIBLE_COLOR, INVISIBLE_COLOR)
  Mixin.mix(self, ExpendituresOutroLayer)

  createLayout(self)

  return self
end
