--- ExpendituresWindowBorder
-- @author Created by Jay Austria on October 27, 2014.

class("ExpendituresWindowBorder")
require("ExpendituresCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INITIAL_WIDTH = 4
local INITIAL_HEIGHT = 4
local CORNER_RADIUS = 6
local LINE_WIDTH = 2
local BLUR_MARGIN = 0
local EXPAND_WIDTH_DURATION = 0.5
local EXPAND_WIDTH_EASE = AHEaseSimpleInOut.new()
local EXPAND_HEIGHT_DELAY_DURATION = 0.1
local EXPAND_HEIGHT_DURATION = 0.3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ExpendituresWindowBorder:getExpandAnimation()
  local appear = function() self:setOpacity(1) end

  local expandWidthAction = AHDimXTo.new(self, EXPAND_WIDTH_DURATION, EXPAND_WIDTH_EASE, self.expandedWidth)
  local expandHeightDelay = AHDelay.new(EXPAND_HEIGHT_DELAY_DURATION)

  local roundCornersAction = self:newCornerRadiusTo(0.1, nil, CORNER_RADIUS)
  local expandHeightAction = AHDimYTo.new(self, EXPAND_HEIGHT_DURATION, nil, self.expandedHeight)
  local expandDividerHeightAction = AHScaleYTo.new(self.divider, EXPAND_HEIGHT_DURATION, nil, 1.0)

  local expandHeightSpawn = AHSpawn.new(roundCornersAction, expandHeightAction, expandDividerHeightAction)

  local expandSequence = AHSequence.new(appear, expandWidthAction, expandHeightDelay, expandHeightSpawn)

  return expandSequence
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function ExpendituresWindowBorder.newCollapsed(width, height, color)
  local initialCornerRadius = INITIAL_HEIGHT / 2
  local self = HollowRoundedRectangle.new(INITIAL_WIDTH, INITIAL_HEIGHT, initialCornerRadius, color, LINE_WIDTH, BLUR_MARGIN)
  Mixin.mix(self, ExpendituresWindowBorder)

  self:setAnchorPointBottom()

  local dividerLocationX = width * ExpendituresCommon.DIVIDER_PLACEMENT_X_MULTIPLIER
  local divider = FilledRectangle.new(LINE_WIDTH, height, color)

  divider:setLoc(dividerLocationX, 0)
  divider:setAnchorPointBottom()
  divider:setScl(1.0, 0.0)
  self.divider = divider

  self:insert(divider)

  self.expandedWidth = width
  self.expandedHeight = height

  self:linkOpacityAttrToChildren()

  self:setOpacity(0)

  return self
end
