--- FoodAnswerOptionBar
-- @author Created by Aaron Barrett on January 06, 2015.

class("FoodAnswerOptionBar")
require("FoodAnswerOptionButton")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.fullWidth
local HEIGHT = Screen.getSafeY(0.14)

local BACKGROUND_COLOR = {0.0, 0.0, 0.0, 0.3}

local Z_BACKGROUND = 10
local Z_ANSWER_OPTION_BUTTON_ROW = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodAnswerOptionBar:playAppear()
  local destinationY = self:getHeightPart(0.5)
  local appearAction = AHLocationYTo.new(self, 0.9, AHEaseExponentialOut.new(), destinationY)
  return appearAction:start()
end

function FoodAnswerOptionBar:playDisappear()
  local destinationY = -self:getHeightPart(0.5)
  local disappearAction = AHLocationYTo.new(self, 0.9, AHEaseExponentialOut.new(), destinationY)
  return disappearAction:start()
end

function FoodAnswerOptionBar:setTouchEnabled(touchEnabled)
  local answerOptionButtons = self.answerOptionButtons
  for i, answerOptionButton in ipairs(answerOptionButtons) do
    answerOptionButton:setTouchEnabled(touchEnabled)
  end
end

function FoodAnswerOptionBar:displayForTutorial()
  local answerOptionButtons = self.answerOptionButtons
  local isEnabled = false
  for i, answerOptionButton in ipairs(answerOptionButtons) do
    answerOptionButton:setEnabledForTutorial(isEnabled)
    answerOptionButton:stayGrayOnTap()
  end
end

function FoodAnswerOptionBar:walkThroughAnswerOptionInteraction(step)
  local answerOptionButtons = self.answerOptionButtons
  local highlightedButton = nil
  for _, answerOptionButton in ipairs(answerOptionButtons) do
    if (answerOptionButton.amount:equals(step)) then
      answerOptionButton:setEnabledForTutorial(true)
      highlightedButton = answerOptionButton
    else
      answerOptionButton:setEnabledForTutorial(false)
    end
  end

  return highlightedButton
end

function FoodAnswerOptionBar:endWalkThrough()
  local answerOptionButtons = self.answerOptionButtons
  for _, answerOptionButton in ipairs(answerOptionButtons) do
    answerOptionButton:setEnabledForTutorial(false)
  end
end

-- Wrong Animation --------------------

function FoodAnswerOptionBar:playWrong()
  local answerOptionButtons = self.answerOptionButtons
  for _, answerOptionButton in ipairs(answerOptionButtons) do
    answerOptionButton:playWrong()
  end
end

function FoodAnswerOptionBar:getButton(amount)
  for _, button in ipairs(self.answerOptionButtons) do
    if button.amount:equals(amount) then
      return button
    end
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodAnswerOptionBar.new(round, roundColor, onPressedCallback)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, FoodAnswerOptionBar)

  local background = FilledRectangle.new(WIDTH, HEIGHT, BACKGROUND_COLOR)
  self:insertAtCenter(background, Z_BACKGROUND)

  local answerOptionAmounts = round:getAnswerOptionAmounts()

  local answerOptionButtonRow = Row.new()
  local answerOptionButtons = {}
  for _, answerOptionAmount in ipairs(answerOptionAmounts) do
    local answerOptionButton = FoodAnswerOptionButton.new(answerOptionAmount, roundColor, onPressedCallback)

    answerOptionButtonRow:insert(answerOptionButton)
    table.insert(answerOptionButtons, answerOptionButton)
  end
  answerOptionButtonRow:alignChildren()
  answerOptionButtonRow:setLoc(Screen.getSafeX(0.5), HEIGHT - answerOptionButtonRow:getHeightPart(0.5))
  self:insert(answerOptionButtonRow, Z_ANSWER_OPTION_BUTTON_ROW)

  self.answerOptionButtons = answerOptionButtons

  return self
end
