--- FoodAnswerOptionChooser
-- @author Created by Aaron Barrett on November 21, 2014.

class("FoodAnswerOptionChooser")

require("FoodFraction")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

FoodAnswerOptionChooser.MAXIMUM_ANSWER_OPTION_COUNT = 4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getMinAnswerOptionCount()
  local difficulty = GameSession.getDifficulty()

  if (difficulty < 0.25) then
    return 2
  elseif (difficulty < 0.7) then
    return 3
  end
  return 4
end

local function convertOptimalStepsToDenominatorsSet(optimalSteps)
  local answerOptionDenominatorsSet = {}

  for i, step in ipairs(optimalSteps) do
    local denominator = step:getDenominator()
    answerOptionDenominatorsSet[denominator] = true
  end

  return answerOptionDenominatorsSet
end

local function convertToAnswerOptionAmountsArray(answerOptionDenominatorsSet)
  local answerOptionDenominatorsArray = TableUtils.createArrayFromSet(answerOptionDenominatorsSet)
  table.sort(answerOptionDenominatorsArray)

  local answerOptionAmountsArray = {}

  for _, denominator in ipairs(answerOptionDenominatorsArray) do
    local answerOptionAmount = FoodFraction.new(1, denominator)
    table.insert(answerOptionAmountsArray, answerOptionAmount)
  end

  return answerOptionAmountsArray
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------


function FoodAnswerOptionChooser.getAnswerOptionAmounts(optimalSteps, givenAmount, factor, correctAnswer)

  -- Always include all AnswerOptions needed to form the optimal solution, regardless of the minimum
  -- AnswerOption count.
  local answerOptionDenominatorsSet = convertOptimalStepsToDenominatorsSet(optimalSteps)

  local minAnswerOptionCount = getMinAnswerOptionCount()

  -- If you still haven't reached the minimum AnswerOption count, include one whole.  If that doesn't
  -- get you there, attempt to include one over the correct answer's denominator.  Next, try one
  -- over the given amount's denominator.  Next, one over the factor's denominator.  If you're still
  -- not there, try 1/2, 1/3, 1/4, and so on.
  local potentialDenominators = {
    1,
    correctAnswer:getDenominator(),
    givenAmount:getDenominator(),
    factor:getDenominator(),
    2,
    3,
    4,
    6,
    8
  }

  for _, denominator in ipairs(potentialDenominators) do
    if (TableUtils.getSize(answerOptionDenominatorsSet) < minAnswerOptionCount) then
      answerOptionDenominatorsSet[denominator] = true
    else
      break
    end
  end

  local answerOptionAmountsArray = convertToAnswerOptionAmountsArray(answerOptionDenominatorsSet)
  return answerOptionAmountsArray
end
