--- FoodBackground
-- @author Created by Jay Austria on January 15, 2015

class("FoodBackground")

require("graphics.TiledImage")
require("FoodConstants")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TOP = {0.09, 0.07, 0.12}
local COLOR_BOTTOM = {0.19, 0.04, 0.20}
local TUTORIAL_COLOR_TOP = {0.10, 0.05, 0.07}
local TUTORIAL_COLOR_BOTTOM = {0.23, 0.23, 0.23}
local WIDTH = Screen.fullWidth
local HEIGHT = Screen.fullHeight
local TUTORIAL_FONT_SIZE = 23
local TUTORIAL_LABEL_X = Screen.getSafeX(0.5)

local WRONG_COLOR_TOP = {0.05, 0.0, 0.0}
local WRONG_COLOR_BOTTOM = {0.23, 0.0, 0.0}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function morphToColor(self, topColor, bottomColor)
  local fadeAction = self.gradient:newGradientToAction(FoodConstants.TUTORIAL_MORPH_DURATION, nil, topColor, bottomColor)
  fadeAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodBackground:displayForGame()
  morphToColor(self, COLOR_TOP, COLOR_BOTTOM)

  if self.tutorialLabel then
    self.tutorialLabel:removeSelf()
  end

end

function FoodBackground:displayForTutorial(tutorialLabelY)
  morphToColor(self, TUTORIAL_COLOR_TOP, TUTORIAL_COLOR_BOTTOM)

  local tutorialText = "TUTORIAL"
  local tutorialLabel = Label.new(tutorialText, Font.new(FontFaces.DIN_OT_BOLD, TUTORIAL_FONT_SIZE))
  self.tutorialLabel = tutorialLabel

  tutorialLabel:setLoc(TUTORIAL_LABEL_X, tutorialLabelY)
  tutorialLabel:setOpacity(0.0)
  self:insert(tutorialLabel)

  local fadeLabelIn = AHOpacityTo.new(tutorialLabel, FoodConstants.TUTORIAL_MORPH_DURATION, nil, 1.0)
  fadeLabelIn:start()
end

function FoodBackground:playWrong()
  local gradient = self.gradient

  local redRemainDuration = 1.55

  local wrongAction = AHSequence.new(
                                    gradient:newGradientToAction(0.1, nil, WRONG_COLOR_TOP, WRONG_COLOR_BOTTOM),
                                    gradient:newGradientToAction(redRemainDuration, AHEaseSimpleOut.new(1.4), COLOR_TOP, COLOR_BOTTOM)
                                    )
  return wrongAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local gradient = RectangleGradient.new(WIDTH, HEIGHT, COLOR_TOP, COLOR_BOTTOM)
  self:insertAtCenter(gradient)
  self.gradient = gradient

  local tiles = TiledImage.newWithFilename("tiles/background_tile.png", WIDTH, HEIGHT)
  tiles:setOpacity(0.4)
  self:insertAtCenter(tiles)
end

-- Constructor ------------------------

function FoodBackground.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, FoodBackground)

  createLayout(self)

  return self
end
