--- FoodCentrifuge
-- @author Created by Aaron Barrett on January 13, 2015.

class("FoodCentrifuge")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_DOT = 10

local RADIUS = Screen.getSafeWidthPart(0.29)
local RADIANS_IN_FULL_CIRCLE = math.pi * 2.0

local OUTRO_DEGREES_TO_SPIN = 600.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodCentrifuge:playOutro()
  local dots = self.dots

  local dotsCount = #dots
  local anglePerDot = RADIANS_IN_FULL_CIRCLE / dotsCount

  local centerX = self.spinner:getWidthPart(0.5)
  local centerY = self.spinner:getHeightPart(0.5)

  for dotIndex, dot in ipairs(dots) do

    local angle = anglePerDot * (dotIndex - 1.0)
    local destinationX = centerX + math.cos(angle) * RADIUS
    local destinationY = centerY + math.sin(angle) * RADIUS

    local duration = 1.5 + dotIndex * 0.03
    local ease = AHEaseBackOut.new(0.5 + dotIndex * 0.082)
    local dotLocationAction = AHLocationTo.new(dot, duration, ease, destinationX, destinationY)

    local delayDuration = 0.02 * dotIndex
    local dotAction = AHDelayed.new(delayDuration, dotLocationAction)
    dotAction:start()

    dot:playOutro()
  end

  local spinAction = AHRotationBy.new(self.spinner, 3.9, AHEaseSimpleInOut.new(3.0), OUTRO_DEGREES_TO_SPIN)
  return spinAction:start()
end

function FoodCentrifuge:takeDots()
  local dots = self.dots

  for _, dot in ipairs(dots) do
    self.spinner:takeChild(dot, Z_DOT)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodCentrifuge.new(dots)
  local self = Group.newFullScreen()
  Mixin.mix(self, FoodCentrifuge)

  local spinner = Group.new(Screen.safeWidth, Screen.safeHeight)
  spinner:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.5))
  self:insert(spinner)

  self.dots = dots
  self.spinner = spinner

  return self
end
