--- LinksConfiguration
-- @author Created by Aaron Barrett on September 17, 2014.

class("FoodConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

FoodConfiguration.POINTS_PER_CORRECT_RESPONSE = 1000
FoodConfiguration.SETS_TO_COMPLETE_COUNT = 6
FoodConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 4.5
FoodConfiguration.SPEED_BONUS_DECAY_START = 1.0
FoodConfiguration.POINTS_PER_FULL_SPEED_BONUS = 50
FoodConfiguration.TUTORIAL_DELAY = 8
-- Perfect Bonus replaces Efficiency Bonus if earned
-- but both are labeled as Efficient
FoodConfiguration.POINTS_PER_EFFICIENCY_BONUS = 50
FoodConfiguration.POINTS_PER_PERFECT_BONUS = 200

local MAXIMUM_CONSISTENCY_BONUS = 3000
local CONSISTENCY_BONUS_DEDUCTION_FOR_MISTAKE = 1000

local MISTAKES_ALLOWED_RANK_TWO = 1
local PERFECT_BONUS_NEEDED_RANK_TWO = 3
local MISTAKES_ALLOWED_RANK_THREE = 0
local PERFECT_BONUS_NEEDED_RANK_THREE = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getConsistencyBonusForMistakeCount(mistakeCount)
  local deductionAmount = mistakeCount * CONSISTENCY_BONUS_DEDUCTION_FOR_MISTAKE
  local consistencyBonus = MAXIMUM_CONSISTENCY_BONUS - deductionAmount
  return MathUtils.clamp(consistencyBonus, 0, MAXIMUM_CONSISTENCY_BONUS)
end

local function getBonusPointsRankTwo()
  local consistencyBonus = getConsistencyBonusForMistakeCount(MISTAKES_ALLOWED_RANK_TWO)
  local perfectBonusPoints = PERFECT_BONUS_NEEDED_RANK_TWO * FoodConfiguration.POINTS_PER_PERFECT_BONUS

  return consistencyBonus + perfectBonusPoints
end

local function getBonusPointsRankThree()
  local consistencyBonus = getConsistencyBonusForMistakeCount(MISTAKES_ALLOWED_RANK_THREE)
  local perfectBonusPoints = PERFECT_BONUS_NEEDED_RANK_THREE * FoodConfiguration.POINTS_PER_PERFECT_BONUS

  return consistencyBonus + perfectBonusPoints
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodConfiguration.getConsistencyBonus(mistakeCount)
  return getConsistencyBonusForMistakeCount(mistakeCount)
end

function FoodConfiguration.getPassScore()
  local roundsCount = FoodConfiguration.SETS_TO_COMPLETE_COUNT
  local pointsPerRound = FoodConfiguration.POINTS_PER_CORRECT_RESPONSE

  local passScore = roundsCount * pointsPerRound
  return passScore
end

function FoodConfiguration.getRankTwoScore()
  local passScore = FoodConfiguration.getPassScore()
  local bonusPoints = getBonusPointsRankTwo()

  local rankTwoScore = passScore + bonusPoints
  return rankTwoScore
end

function FoodConfiguration.getRankThreeScore()
  local passScore = FoodConfiguration.getPassScore()
  local bonusPoints = getBonusPointsRankThree()

  local rankThreeScore = passScore + bonusPoints
  return rankThreeScore
end

function FoodConfiguration.getRoundKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {double = 0.34,
                                      triple = 0.33,
                                      half = 0.33}},

    {key = 0.225, valueProbabilities = {double = 0.34,
                                        triple = 0.33,
                                        half = 0.33}},

    {key = 0.2625, valueProbabilities = {double = 0.25,
                                        triple = 0.25,
                                        half = 0.25,
                                        third = 0.125,
                                        fourth = 0.125}},

    {key = 0.3, valueProbabilities = {double = 0.225,
                                      triple = 0.225,
                                      half = 0.225,
                                      third = 0.1625,
                                      fourth = 0.1625}},

    {key = 0.375, valueProbabilities = {double = 0.2,
                                        triple = 0.2,
                                        half = 0.2,
                                        third = 0.2,
                                        fourth = 0.2}},

    {key = 0.45, valueProbabilities = {double = 0.15,
                                      triple = 0.15,
                                      half = 0.15,
                                      third = 0.275,
                                      fourth = 0.275}},

      {key = 0.525, valueProbabilities = {third = 0.5,
                                        fourth = 0.5}},

    {key = 1.0, valueProbabilities = {third = 0.5,
                                      fourth = 0.5}}
  }
end


function FoodConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Easy = 1.0}},

    {key = 0.075, valueProbabilities = {Easy = 0.9,
                                        Medium = 0.1}},

    {key = 0.15, valueProbabilities = {Easy = 0.7,
                                       Medium = 0.3}},

    {key = 0.225, valueProbabilities = {Easy = 0.5,
                                        Medium = 0.5}},

    {key = 0.2625, valueProbabilities = {Easy = 0.4,
                                         Medium = 0.6}},

    {key = 0.3, valueProbabilities = {Easy = 0.3,
                                      Medium = 0.7}},

    {key = 0.375, valueProbabilities = {Easy = 0.1,
                                        Medium = 0.8,
                                        Hard = 0.1}},

    {key = 0.45, valueProbabilities = {Medium = 0.7,
                                       Hard = 0.3}},

    {key = 0.525, valueProbabilities = {Medium = 0.5,
                                        Hard = 0.5}},

    {key = 0.6, valueProbabilities = {Medium = 0.4,
                                      Hard = 0.5,
                                      Advanced = 0.1}},

    {key = 0.675, valueProbabilities = {Medium = 0.25,
                                        Hard = 0.45,
                                        Advanced = 0.3}},

    {key = 0.75, valueProbabilities = {Medium = 0.1,
                                       Hard = 0.4,
                                       Advanced = 0.5}},

    {key = 0.8, valueProbabilities = {Hard = 0.4,
                                      Advanced = 0.6}},

    {key = 0.9, valueProbabilities = {Hard = 0.3,
                                      Advanced = 0.7}},

    {key = 1.0, valueProbabilities = {Hard = 0.2,
                                      Advanced = 0.8}}
  }
end
