--- FoodFractionLabel
-- @author Created by Aaron Barrett on December 01, 2014.

class("FoodFractionLabel")

require("FoodFractionPartLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setOnlyWholePart(self, wholePart)
  local wholePartLabel = self.wholePartLabel
  local fractionPartLabel = self.fractionPartLabel

  wholePartLabel:setText(tostring(wholePart))
  local wholePartLabelWidth, wholePartLabelHeight = wholePartLabel:getDims()
  wholePartLabel:setLoc(wholePartLabelWidth * 0.5, wholePartLabelHeight * 0.5)
  wholePartLabel:setVisible(true)

  fractionPartLabel:setVisibleRecursive(false)

  self:setDims(wholePartLabelWidth, wholePartLabelHeight)
end

local function setOnlyFractionPart(self, remainder, denominator)
  local wholePartLabel = self.wholePartLabel
  local fractionPartLabel = self.fractionPartLabel

  wholePartLabel:setVisible(false)

  fractionPartLabel:setNumeratorAndDenominator(remainder, denominator)
  local fractionPartLabelWidth, fractionPartLabelHeight = fractionPartLabel:getDims()
  fractionPartLabel:setLoc(fractionPartLabelWidth * 0.5, fractionPartLabelHeight * 0.5)
  fractionPartLabel:setVisibleRecursive(true)

  self:setDims(fractionPartLabelWidth, fractionPartLabelHeight)
end

local function setWholePartAndFractionPart(self, wholePart, remainder, denominator)
  local wholePartLabel = self.wholePartLabel
  local fractionPartLabel = self.fractionPartLabel
  local partSpacing = self.partSpacing

  wholePartLabel:setText(tostring(wholePart))
  local wholePartLabelWidth, wholePartLabelHeight = wholePartLabel:getDims()
  wholePartLabel:setLoc(wholePartLabelWidth * 0.5, wholePartLabelHeight * 0.5)
  wholePartLabel:setVisible(true)

  fractionPartLabel:setNumeratorAndDenominator(remainder, denominator)
  local fractionPartLabelWidth = fractionPartLabel:getWidth()
  fractionPartLabel:setLoc(wholePartLabelWidth + partSpacing + fractionPartLabelWidth * 0.5, wholePartLabelHeight * 0.5)
  fractionPartLabel:setVisibleRecursive(true)

  local width = wholePartLabelWidth + partSpacing + fractionPartLabelWidth
  local height = wholePartLabelHeight
  self:setDims(width, height)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodFractionLabel:setAmountFromComponents(wholePart, numerator, denominator)
  if wholePart > 0 and numerator > 0 then
    setWholePartAndFractionPart(self, wholePart, numerator, denominator)
  elseif numerator > 0 then
    setOnlyFractionPart(self, numerator, denominator)
  else
    setOnlyWholePart(self, wholePart)
  end
end

function FoodFractionLabel:setAmount(amount)
  local numerator = amount:getNumerator()
  local denominator = amount:getDenominator()

  local wholePart = math.floor(numerator / denominator)
  local remainder = numerator - wholePart * denominator

  self:setAmountFromComponents(wholePart, remainder, denominator)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodFractionLabel.new(fontSize, initialAmount)
  local self = Group.new(2.0, 2.0)
  Mixin.mix(self, FoodFractionLabel)

  local wholePartLabel = Label.new("", Font.new(FontFaces.DIN_OT_MEDIUM, fontSize))
  wholePartLabel:inheritColor(self)
  self:insert(wholePartLabel)
  self.wholePartLabel = wholePartLabel

  local fractionPartLabel = FoodFractionPartLabel.new(fontSize)
  fractionPartLabel:inheritColor(self)
  self:insert(fractionPartLabel)
  self.fractionPartLabel = fractionPartLabel

  self.partSpacing = fontSize * 0.24

  if initialAmount then
    self:setAmount(initialAmount)
  end

  return self
end
