--- FoodFractionPartLabel
-- @author Created by Aaron Barrett on January 12, 2015.

class("FoodFractionPartLabel")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function updateLayout(self)
  local numeratorLabel = self.numeratorLabel
  local denominatorLabel = self.denominatorLabel
  local slash = self.slash
  local height = self.height
  local fractionalSpacing = self.fractionalSpacing

  local numeratorWidth = numeratorLabel:getWidth()
  local denominatorWidth = denominatorLabel:getWidth()

  local width = (numeratorWidth + denominatorWidth + fractionalSpacing) * 0.5

  local numeratorX = numeratorWidth * 0.5
  local denominatorX = width - denominatorWidth * 0.5
  local slashX = (numeratorX + denominatorX) * 0.5

  local numeratorY = height * 0.65
  local denominatorY = height * 0.35
  local slashY = height * 0.5

  numeratorLabel:setLoc(numeratorX, numeratorY)
  denominatorLabel:setLoc(denominatorX, denominatorY)
  slash:setLoc(slashX, slashY)

  self:setDims(width, height)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodFractionPartLabel:setNumeratorAndDenominator(numerator, denominator)
  local numeratorLabel = self.numeratorLabel
  local denominatorLabel = self.denominatorLabel
  numeratorLabel:setText(tostring(numerator))
  denominatorLabel:setText(tostring(denominator))
  updateLayout(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodFractionPartLabel.new(fontSize)
  local self = Group.new(2.0, 2.0)
  Mixin.mix(self, FoodFractionPartLabel)

  local fractionalFontSize = fontSize * 0.55
  local numeratorLabel = Label.new("", Font.new(FontFaces.DIN_OT_MEDIUM, fractionalFontSize))
  numeratorLabel:inheritColor(self)
  self:insert(numeratorLabel)
  self.numeratorLabel = numeratorLabel

  local denominatorLabel = Label.new("", Font.new(FontFaces.DIN_OT_MEDIUM, fractionalFontSize))
  denominatorLabel:inheritColor(self)
  self:insert(denominatorLabel)
  self.denominatorLabel = denominatorLabel

  local slashFontSize = fontSize * 0.8
  local slash = Label.new("/", Font.new(FontFaces.DIN_OT_MEDIUM, slashFontSize))
  slash:inheritColor(self)
  self:insert(slash)
  self.slash = slash

  self.fractionalSpacing = fontSize * 1.3
  self.height = fontSize

  return self
end
