--- FoodOutro
-- @author Created by Aaron Barrett on January 13, 2015.

class("FoodOutro")

require("FoodCentrifuge")
require("FoodRoundColors")
require("graphics.primitives.Checkmark")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ROUND_COLOR_COUNT = 6

local CHECKMARK_LINE_WIDTH = 1
local FINAL_CHECKMARK_LINE_WIDTH = 2

local CHECKMARK_DURATION = 2.667
local CHECKMARK_EASE = AHEaseBezier.newFlat({0.8, 0.0, 0.4, 1.0})

local DELAY_BETWEEN_CHECKMARKS = 0.067

local Z_SHADOW_DOT = 10
local Z_CENTRIFUGE = 20
local Z_CHECKMARK_GLOW = 30
local Z_CHECKMARK = 40

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createCheckmarkAnimation(checkmark)
  local scaleAnimation = AHScaleTo.new(checkmark, CHECKMARK_DURATION, CHECKMARK_EASE, 1.0)
  local fadeAnimation = AHOpacityTo.new(checkmark, CHECKMARK_DURATION, CHECKMARK_EASE, 1.0)
  local rotateAnimation = AHRotationBy.new(checkmark, CHECKMARK_DURATION, CHECKMARK_EASE, 360)
  return AHSpawn.new(scaleAnimation, fadeAnimation, rotateAnimation)
end

local function playCheckmarkOutro(self)
  local outroAnimation = AHSpawn.new()
  local delay = 0
  for _, checkmark in ipairs(self.checkmarks) do
    outroAnimation:addAction(AHDelayed.new(delay, createCheckmarkAnimation(checkmark)))
    delay = delay + DELAY_BETWEEN_CHECKMARKS
  end
  return outroAnimation:start()
end

local function addShadowDots(self, centrifuge)
  local progressIndicator = self.progressIndicator
  local dots = progressIndicator:getDots()

  for _, dot in ipairs(dots) do
    dot:addShadowDots(self, centrifuge, Z_SHADOW_DOT)
  end

end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodOutro:playOutro()
  local centrifuge = self.centrifuge
  centrifuge:playOutro()

  addShadowDots(self, centrifuge)

  playCheckmarkOutro(self)

  local delayAction = AHDelay.new(3.9)
  return delayAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildCentrifuge(self, progressIndicator)
  local dots = progressIndicator:getDots()
  local centrifuge = FoodCentrifuge.new(dots)
  self:insertAtCenter(centrifuge, Z_CENTRIFUGE)
  centrifuge:takeDots()
  self.centrifuge = centrifuge
end

local function createCheckmark(self, color, lineWidth)
  local width = Screen.getSafeWidthPart(0.36)
  local checkmark = HollowCheckmark.new(width, nil, nil, color, lineWidth)
  checkmark:setOpacity(0.0)
  checkmark:setScl(0.3)
  checkmark:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.5))
  self:insert(checkmark, Z_CHECKMARK)
  return checkmark
end

local function buildCheckmarks(self)
  local checkmarks = {}
  for roundIndex = 1, ROUND_COLOR_COUNT do
    local color = FoodRoundColors.getColorForRoundIndex(roundIndex)
    local lineWidth = CHECKMARK_LINE_WIDTH
    if roundIndex == ROUND_COLOR_COUNT then
      lineWidth = FINAL_CHECKMARK_LINE_WIDTH
    end
    local checkmark = createCheckmark(self, color, lineWidth)
    table.insert(checkmarks, checkmark)
  end
  self.checkmarks = checkmarks
end

-- Constructor ------------------------

function FoodOutro.new(progressIndicator)
  local self = Group.newFullScreen()
  Mixin.mix(self, FoodOutro)

  buildCentrifuge(self, progressIndicator)
  buildCheckmarks(self)

  self.progressIndicator = progressIndicator

  return self
end
