--- FoodPanelRecipe
-- @author Created by Aaron Barrett on January 05, 2015.

class("FoodPanelRecipe")

require("FoodDot")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DOT_WIDTH = Screen.getSafeWidthPart(0.9)
local DOT_START_Y = Screen.getHeightPart(-0.6)

local Z_DOT = 10
local Z_INGREDIENTS_LABEL = 20
local Z_TITLE_LABEL = 30

local DOT_FLY_START_DELAY_DURATIONS = {
  [1] = 0.00,
  [2] = 0.16,
  [3] = 0.08,
  [4] = 0.22,
  [5] = 0.12,
  [6] = 0.28
}

local ALL_DOTS_FLY_IN_DURATION = 1.2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodPanelRecipe:playFlyToCenter()
  return runInCoroutine(function()
    local dots = self.dots
    for dotIndex, dot in ipairs(dots) do
      runInCoroutine(function()
        local dotFlyStartDelayDuration = DOT_FLY_START_DELAY_DURATIONS[dotIndex]
        assert(dotFlyStartDelayDuration ~= nil, "No dot fly start delay duration defined")
        waitFor(dotFlyStartDelayDuration)
        dot:playFlyToCenter()
      end)
    end

    local titleLabel = self.titleLabel
    local titleAction = AHOpacityTo.new(titleLabel, 0.5, nil, 1.0)
    titleAction:start()

    local ingredientsLabel = self.ingredientsLabel
    local ingredientsAction = AHSpawn.new(
                                          AHOpacityTo.new(ingredientsLabel, 0.5, nil, 0.5),
                                          AHLocationYTo.new(ingredientsLabel, 0.5, AHEaseSimpleOut.new(4.0), Screen.getSafeY(0.638))
                                          )
    ingredientsAction:start()

    waitFor(ALL_DOTS_FLY_IN_DURATION)
  end)
end

function FoodPanelRecipe:playFadeIngredientsLabel()
  local ingredientsLabel = self.ingredientsLabel
  local ingredientsLabelAction = AHOpacityTo.new(ingredientsLabel, 0.6, nil, 0.0)
  return ingredientsLabelAction:start()
end

function FoodPanelRecipe:getDots()
  return self.dots
end

function FoodPanelRecipe:getTitleLabel()
  return self.titleLabel
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildDots(self, rounds)
  local dotCount = #rounds
  local dotSpacing = DOT_WIDTH / dotCount
  local dotBaseX = Screen.getSafeX(0.5) - dotSpacing * dotCount * 0.5

  local dots = {}

  for roundIndex, round in ipairs(rounds) do
    local dot = FoodDot.new(round, roundIndex)

    local dotX = dotBaseX + (roundIndex - 0.5) * dotSpacing

    dot:setLoc(dotX, DOT_START_Y)
    self:insert(dot, Z_DOT)
    table.insert(dots, dot)
  end

  self.dots = dots
end

local function buildIngredientsLabel(self)
  local ingredientsLabel = Label.new("TODAY'S INGREDIENTS", Font.new(FontFaces.DIN_OT_MEDIUM, 18.0))
  ingredientsLabel:setOpacity(0.0)
  ingredientsLabel:setLoc(Screen.getSafeX(0.5), Screen.getHeightPart(1.25))
  self:insert(ingredientsLabel, Z_INGREDIENTS_LABEL)
  self.ingredientsLabel = ingredientsLabel
end

local function buildTitleLabel(self, rounds)
  local firstRound = rounds[1]
  local factor = firstRound:getFactor()
  local title = factor:getTitle()

  local titleLabel = Label.new(title, Font.new(FontFaces.DIN_OT_MEDIUM, 30.0))
  titleLabel:setOpacity(0.0)
  titleLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.59))
  self:insert(titleLabel, Z_TITLE_LABEL)
  self.titleLabel = titleLabel
end



-- Constructor ------------------------

function FoodPanelRecipe.new(rounds)
  local self = Group.newFullScreen()
  Mixin.mix(self, FoodPanelRecipe)

  buildDots(self, rounds)
  buildTitleLabel(self, rounds)
  buildIngredientsLabel(self)

  return self
end
