--- FoodPieFraction
-- @author Created by andrewc on September 24, 2020.

class("FoodPieFraction")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FILLING_NAMES_BY_MIXED_FORM = {
  ["1"] = "filling_whole.png",
  ["1/2"] = "filling_half.png",
  ["1/3"] = "filling_third.png",
  ["1/4"] = "filling_fourth.png",
  ["1/6"] = "filling_sixth.png",
  ["1/8"] = "filling_eighth.png",
  ["1/12"] = "filling_twelfth.png"
}

local OUTLINE_NAMES_BY_MIXED_FORM = {
  ["1"] = "outline_whole.png",
  ["1/2"] = "outline_half.png",
  ["1/3"] = "outline_third.png",
  ["1/4"] = "outline_fourth.png",
  ["1/6"] = "outline_sixth.png",
  ["1/8"] = "outline_eighth.png",
  ["1/12"] = "outline_twelfth.png"
}

local BACKING_OUTLINE = "outline_whole.png"

local Z_BACK_OUTLINE = 10
local Z_FRONT_OUTLINE = 20
local Z_FRONT_FILLING = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createFillingWithMixedForm(mixedForm)
  local fillingName = FILLING_NAMES_BY_MIXED_FORM[mixedForm]
  assert(fillingName ~= nil, "No filling sprite filename provided for fraction with mixed form: `" .. tostring(mixedForm) .. "`")
  local filling = Sprite.newWithName(fillingName)
  return filling
end

local function createOutlineWithMixedForm(mixedForm)
  local outlineName = OUTLINE_NAMES_BY_MIXED_FORM[mixedForm]
  assert(outlineName ~= nil, "No outline sprite filename provided for fraction with mixed form: `" .. tostring(mixedForm) .. "`")
  local outline = Sprite.newWithName(outlineName)
  return outline
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodPieFraction:createFadeAction(duration, ease, opacity)
  return AHSpawn.new(
    AHOpacityTo.new(self.backOutline, duration, ease, self.outlineOpacity * opacity),
    AHOpacityTo.new(self.frontFilling, duration, ease, self.fillOpacity * opacity),
    AHOpacityTo.new(self.frontOutline, duration, ease, opacity))
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodPieFraction.new(amount, color, fillOpacity, outlineOpacity)
  local backOutline = Sprite.newWithName(BACKING_OUTLINE)
  backOutline:setColor(unpack(color))
  backOutline:setOpacity(outlineOpacity)

  local mixedForm = amount:getMixedForm()

  local frontFilling = createFillingWithMixedForm(mixedForm)
  frontFilling:setColor(unpack(color))
  frontFilling:setOpacity(fillOpacity)

  local frontOutline = createOutlineWithMixedForm(mixedForm)
  frontOutline:setColor(unpack(color))
  -- Font outline always has opacity 1

  local self = Group.new(backOutline:getDims())
  Mixin.mix(self, FoodPieFraction)

  self:insertAtCenter(backOutline, Z_BACK_OUTLINE)
  self:insertAtCenter(frontFilling, Z_FRONT_FILLING)
  self:insertAtCenter(frontOutline, Z_FRONT_OUTLINE)
  self.backOutline = backOutline
  self.frontFilling = frontFilling
  self.frontOutline = frontOutline

  self.fillOpacity = fillOpacity
  self.outlineOpacity = outlineOpacity

  return self
end
