--- FoodRound
-- @author Created by Aaron Barrett on November 14, 2014.

class("FoodRound")

require("FoodFraction")
require("FoodFractionConstants")

require("FoodSolutionFinder")
require("FoodAnswerOptionChooser")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FACTORS_BY_FACTOR_NAME = {
  ["triple"] = FoodFractionConstants.THREE_WHOLES,
  ["double"] = FoodFractionConstants.TWO_WHOLES,
  ["half"] = FoodFractionConstants.ONE_HALF,
  ["third"] = FoodFractionConstants.ONE_THIRD,
  ["fourth"] = FoodFractionConstants.ONE_FOURTH
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getFactorForFactorName(factorName)
  local factor = FACTORS_BY_FACTOR_NAME[factorName]
  assert(factor ~= nil, "No factor constant defined for factor name: `" .. tostring(factorName) .. "`")
  return factor
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodRound:getIdentifier()
  return self.identifier
end

function FoodRound:getFactor()
  return self.factor
end

function FoodRound:getGivenAmount()
  return self.givenAmount
end

function FoodRound:getCorrectAnswer()
  return self.correctAnswer
end

function FoodRound:getOptimalSteps()
  return self.optimalSteps
end

function FoodRound:getAnswerOptionAmounts()
  return self.answerOptionAmounts
end

function FoodRound:getTipType()
  return self.tipType
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodRound.new(identifier, factor, givenAmount, tipType)
  local self = {}
  Mixin.mix(self, FoodRound)

  self.identifier = identifier
  self.factor = factor
  self.givenAmount = givenAmount
  self.tipType = tipType

  local correctAnswer = givenAmount:multiply(factor)
  self.correctAnswer = correctAnswer

  local optimalSteps = FoodSolutionFinder.getOptimalSteps(correctAnswer)
  self.optimalSteps = optimalSteps

  local answerOptionAmounts = FoodAnswerOptionChooser.getAnswerOptionAmounts(optimalSteps, givenAmount, factor, correctAnswer)
  self.answerOptionAmounts = answerOptionAmounts

  return self
end

function FoodRound.newWithConcept(concept)
  local identifier = concept["identifier"]

  local factorName = concept["round"]
  local factor = getFactorForFactorName(factorName)

  local numerator = concept["numerator"]
  local denominator = concept["denominator"]
  local givenAmount = FoodFraction.new(numerator, denominator)

  local tipType = concept["tipType"]

  return FoodRound.new(identifier, factor, givenAmount, tipType)
end
