--- FoodRoundedButtonShape
-- @author Created by Aaron Barrett on November 25, 2014.

class("FoodRoundedButtonShape")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LEFT_START_ANGLE = math.pi * 0.5
local LEFT_END_ANGLE = math.pi * 1.5

local RIGHT_START_ANGLE = math.pi * 1.5
local RIGHT_END_ANGLE = math.pi * 2.5

local SIDES_PER_ROUNDING = 14

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodRoundedButtonShape.new(width, height, color, isFilled)
  local self = Group.newCustomDraw(width, height, color)
  Mixin.mix(self, FoodRoundedButtonShape)

  local radius = height * 0.5

  local leftCenterX = radius
  local leftCenterY = radius
  local points = ShapeUtils.makeArcPoints(LEFT_START_ANGLE,
                                          LEFT_END_ANGLE,
                                          radius,
                                          leftCenterX,
                                          leftCenterY,
                                          SIDES_PER_ROUNDING)

  local rightCenterX = width - radius
  local rightCenterY = radius
  local points = ShapeUtils.makeArcPoints(RIGHT_START_ANGLE,
                                          RIGHT_END_ANGLE,
                                          radius,
                                          rightCenterX,
                                          rightCenterY,
                                          SIDES_PER_ROUNDING,
                                          points)

  local coordinates = ShapeUtils.flattenCoordPairsNew(points)

  if (isFilled) then
    self.deck:setDrawCallback(function() MOAIDraw.fillFan(coordinates) end)
  else
    local lineWidth = 0.6
    local blurMargin = 0.5
    self.deck:setDrawCallback(function() MOAIDraw.drawBeveledLineLoop(coordinates, lineWidth, blurMargin) end)
  end

  return self
end
