--- FoodSolutionFinder
-- @author Created by Aaron Barrett on November 21, 2014.

class("FoodSolutionFinder")

require("FoodFractionConstants")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ANSWER_OPTION_AMOUNTS = {
  FoodFractionConstants.ONE_WHOLE,
  FoodFractionConstants.ONE_HALF,
  FoodFractionConstants.ONE_THIRD,
  FoodFractionConstants.ONE_FOURTH,
  FoodFractionConstants.ONE_SIXTH,
  FoodFractionConstants.ONE_EIGHTH
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function doesAnswerOptionAmountsContainOneThird(answerOptionAmounts)
  for _, amount in ipairs(answerOptionAmounts) do
    if (amount:equals(FoodFractionConstants.ONE_THIRD)) then
      return true
    end
  end
  return false
end

local function getOptimalStepsInternal(totalAmount, answerOptionAmounts)
  if (totalAmount:getNumerator() == 0) then
    return {}
  end

  -- Prevent one half piece plus one sixth piece from showing up in the solution when the
  -- total amount ends in two-thirds.  Use two third pieces instead.
  if (totalAmount:equals(FoodFractionConstants.TWO_THIRDS) and
      doesAnswerOptionAmountsContainOneThird(answerOptionAmounts)) then
      return {FoodFractionConstants.ONE_THIRD, FoodFractionConstants.ONE_THIRD}
  end

  for _, answerOptionAmount in ipairs(answerOptionAmounts) do
    if (answerOptionAmount:toNumber() <= totalAmount:toNumber()) then
      local remainingAmount = totalAmount:subtract(answerOptionAmount)
      local optimalSteps = getOptimalStepsInternal(remainingAmount, answerOptionAmounts)
      if (optimalSteps ~= nil) then
        table.insert(optimalSteps, 1, answerOptionAmount)
        return optimalSteps
      end
    end
  end

  return nil
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodSolutionFinder.getOptimalSteps(totalAmount, answerOptionAmounts)
  answerOptionAmounts = answerOptionAmounts or ANSWER_OPTION_AMOUNTS

  local optimalSteps = getOptimalStepsInternal(totalAmount, answerOptionAmounts)
  assert(optimalSteps ~= nil, "Ran out of answer option amounts when calculating solution for `" .. totalAmount:getMixedForm() .. "`")

  return optimalSteps
end
