--- FoodSubmitButton
-- @author Created by Aaron Barrett on January 06, 2015.

class("FoodSubmitButton")

require("FoodRoundedButtonShape")
require("FoodConstants")
require("FoodFractionConstants")
require("FoodFractionLabel")
require("FoodStrobeAction")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.getSafeWidthPart(0.45)
local HEIGHT = Screen.getSafeHeightPart(0.07)

local TOTAL_LABEL_X = WIDTH * 0.29
local SUBMIT_LABEL_X = WIDTH * 0.61

local Z_FILLING = 10
local Z_OUTLINE = 20
local Z_SUBMIT_LABEL = 30
local Z_TOTAL_LABEL = 40

local TUTORIAL_COLOR = {0.6, 0.6, 0.6}
local HIGHLIGHT_COLOR = {0.01, 0.78, 0.98}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playPress(self)
  local filling = self.filling

  local pressAction = AHSequence.new(
                                     AHSpawn.new(
                                                 AHScaleTo.new(self, 0.11, AHEaseSimpleIn.new(1.6), 0.9),
                                                 AHOpacityTo.new(filling, 0.2, nil, 0.27)
                                                 ),
                                     AHSpawn.new(
                                                 AHScaleTo.new(self, 0.14, AHEaseSimpleOut.new(1.6), 1.0),
                                                 AHOpacityTo.new(filling, 0.8, nil, 0.0)
                                                 )
                                     )
  return pressAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Displayed Amount -------------------

function FoodSubmitButton:setDisplayedAmount(displayedAmout)
  local totalLabel = self.totalLabel

  local shouldSetAmount = (displayedAmout:getNumerator() > 0)
  if (shouldSetAmount) then
    totalLabel:setAmount(displayedAmout)
  end
end

-- Appear Animation -------------------

function FoodSubmitButton:hideInitially()
  local outline = self.outline
  local totalLabel = self.totalLabel
  local submitLabel = self.submitLabel
  outline:setOpacity(0.0)
  totalLabel:setOpacity(0.0)
  submitLabel:setOpacity(0.0)
end

function FoodSubmitButton:createAppearAction()
  local outline = self.outline
  local totalLabel = self.totalLabel
  local submitLabel = self.submitLabel
  local appearAction = AHSpawn.new(
                                   function()
                                     self.hasAppeared = true
                                   end,
                                   AHOpacityTo.new(outline, 0.8, AHEaseExponentialOut.new(), 1.0),
                                   AHOpacityTo.new(totalLabel, 0.3, nil, 1.0),
                                   AHOpacityTo.new(submitLabel, 0.3, nil, 1.0)
                                   )
  return appearAction
end

-- Disappear Animation ----------------

function FoodSubmitButton:createFadeOutAction()
  local outline = self.outline
  local totalLabel = self.totalLabel
  local submitLabel = self.submitLabel
  local shrinkOutAction = AHSpawn.new(
                                      function()
                                        self.hasAppeared = false
                                      end,
                                      AHOpacityTo.new(outline, 0.7, nil, 0.0),
                                      AHOpacityTo.new(totalLabel, 0.7, nil, 0.0),
                                      AHOpacityTo.new(submitLabel, 0.7, nil, 0.0)
                                      )
  return shrinkOutAction
end

function FoodSubmitButton:createShrinkOutAction()
  local outline = self.outline
  local totalLabel = self.totalLabel
  local submitLabel = self.submitLabel
  local shrinkOutAction = AHSpawn.new(
                                      function()
                                        self.hasAppeared = false
                                      end,
                                      AHOpacityTo.new(outline, 0.3, AHEaseSimpleIn.new(2.0), 0.0),
                                      AHOpacityTo.new(totalLabel, 0.15, nil, 0.0),
                                      AHOpacityTo.new(submitLabel, 0.15, nil, 0.0)
                                      )
  return shrinkOutAction
end

-- Strobe Animation -------------------

function FoodSubmitButton:playStrobe()
  local outline = self.outline
  local filling = self.filling
  local strobeAction = AHSpawn.new(
                                    FoodStrobeAction.new(outline),
                                    FoodStrobeAction.new(filling)
                                    )
  return strobeAction:start()
end

-- Wrong Animation --------------------

function FoodSubmitButton:playWrong()
  local outline = self.outline
  local filling = self.filling
  local roundColor = self.roundColor

  local redRemainDuration = 1.55

  local ease = AHEaseSimpleOut.new(1.4)

  local wrongAction = AHSequence.new(
                                     AHSpawn.new(
                                                 AHColorTo.new(outline, 0.1, nil, FoodConstants.WRONG_RED_COLOR),
                                                 AHColorTo.new(filling, 0.1, nil, FoodConstants.WRONG_RED_COLOR)
                                                 ),
                                     AHSpawn.new(
                                                 AHColorTo.new(outline, redRemainDuration, ease, roundColor),
                                                 AHColorTo.new(filling, redRemainDuration, ease, roundColor)
                                                 )
                                     )
  wrongAction:start()
end

-- Loss Animation ---------------------

function FoodSubmitButton:playLoss()
  local outline = self.outline
  local totalLabel = self.totalLabel
  local submitLabel = self.submitLabel

  local lossAction = AHSpawn.new(
                                 AHOpacityTo.new(outline, 0.4, nil, 0.0),
                                 AHOpacityTo.new(totalLabel, 0.4, nil, 0.0),
                                 AHOpacityTo.new(submitLabel, 0.4, nil, 0.0)
                                 )

  return lossAction:start()
end

-- Tutorial Animations ----------------------

function FoodSubmitButton:displayForTutorial()
  local fadeAction = AHColorTo.new(self.outline, FoodConstants.TUTORIAL_MORPH_DURATION, nil, TUTORIAL_COLOR)
  fadeAction:start()
end

function FoodSubmitButton:tutorialEnable()
  local fadeAction = AHColorTo.new(self.outline, FoodConstants.TUTORIAL_MORPH_DURATION, nil, self.roundColor)
  fadeAction:start()
end

function FoodSubmitButton:setTouchEnabled(touchEnabled)
  self.virtualButton:setTouchEnabled(touchEnabled)
end

function FoodSubmitButton:playTutorialShake(holderCenterX)
  local ease = AHEaseSimpleInOut.new(2.0)
  local shakeAmplitude = self:getWidthPart(0.015)

  local locationAction = AHSequence.new(
                                        AHDelay.new(0.5),
                                        AHLocationXTo.new(self, 0.15, ease, holderCenterX + shakeAmplitude),
                                        AHLocationXTo.new(self, 0.15, ease, holderCenterX - shakeAmplitude),
                                        AHLocationXTo.new(self, 0.10, ease, holderCenterX + shakeAmplitude * 0.7),
                                        AHLocationXTo.new(self, 0.10, ease, holderCenterX - shakeAmplitude * 0.7),
                                        AHLocationXTo.new(self, 0.05, ease, holderCenterX + shakeAmplitude * 0.4),
                                        AHLocationXTo.new(self, 0.05, ease, holderCenterX - shakeAmplitude * 0.4),
                                        AHLocationXTo.new(self, 0.05, ease, holderCenterX)
                                        )

  local tutorialShakeAction = AHLoop.new(locationAction)
  self.tutorialShakeAction = tutorialShakeAction

  return tutorialShakeAction:start()
end

function FoodSubmitButton:playStopTutorialShake(holderCenterX)
  self.tutorialShakeAction:stop()

  local stopTutorialShakeAction = AHLocationXTo.new(self, 0.05, AHEaseSimpleInOut.new(2.0), holderCenterX)

  return stopTutorialShakeAction:start()
end

function FoodSubmitButton:highlight()
  self.outline:setColor(unpack(HIGHLIGHT_COLOR))
end

function FoodSubmitButton:removeHighlight()
  self.outline:setColor(unpack(self.roundColor))
end

function FoodSubmitButton:getSprite()
  local shape = FoodRoundedButtonShape.new(WIDTH, HEIGHT, {1, 1, 1, 1}, true)
  shape:setLoc(WIDTH / 2, HEIGHT / 2)
  return shape
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodSubmitButton.new(roundColor, submitCallback)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, FoodSubmitButton)

  local isFillingFilled = true
  local filling = FoodRoundedButtonShape.new(WIDTH, HEIGHT, roundColor, isFillingFilled)
  filling:setOpacity(0.0)
  self:insertAtCenter(filling, Z_FILLING)
  self.filling = filling

  local isOutlineFilled = false
  local outline = FoodRoundedButtonShape.new(WIDTH, HEIGHT, roundColor, isOutlineFilled)
  self:insertAtCenter(outline, Z_OUTLINE)
  self.outline = outline

  local submitLabel = Label.new("SUBMIT", Font.new(FontFaces.DIN_OT_MEDIUM, 12.0))
  submitLabel:setColor(1.0, 1.0, 1.0)
  submitLabel:setLoc(SUBMIT_LABEL_X, self:getHeightPart(0.5))
  self:insert(submitLabel, Z_SUBMIT_LABEL)
  self.submitLabel = submitLabel

  local totalLabelFontSize = 20.0
  local initialSelectedAmount = FoodFractionConstants.ZERO
  local totalLabel = FoodFractionLabel.new(totalLabelFontSize, initialSelectedAmount)
  totalLabel:setLoc(TOTAL_LABEL_X, self:getHeightPart(0.5))
  self:insert(totalLabel, Z_TOTAL_LABEL)
  self.totalLabel = totalLabel

  self.hasAppeared = false

  local function onTouchDown()
    if (self.hasAppeared) then
      playPress(self)
      submitCallback()
    end
  end

  self.onTouchDown = onTouchDown

  local touchAreaPaddingX = Screen.getSafeWidthPart(0.15)
  local touchAreaPaddingY = Screen.getSafeHeightPart(0.06)
  local touchAreaWidth = self:getWidth() + touchAreaPaddingX
  local touchAreaHeight = self:getHeight() + touchAreaPaddingY
  local virtualButton = Button.addVirtualButton(self, touchAreaWidth, touchAreaHeight, onTouchDown)
  self.virtualButton = virtualButton

  self.roundColor = roundColor

  self.tutorialShakeAction = nil

  return self
end
