--- FoodTapToRemoveTipOverlay
-- @author Created by Jay Austria on January 15, 2015

class("FoodTapToRemoveTipOverlay")

require("widgets.TapToContinue")

local HELPER_FONT_SIZE = 16
local ARROW_MARGIN_X = 20
local ARROW_SCALE = 0.4
local FLIP_ROTATION = 180
local VISIBLE_OPACITY = 1.0

local function createAnimatingArrow(self, arrowX, arrowY)
  local arrow = Sprite.newWithName("shared_arrow.png")
  arrow:setLoc(arrowX, arrowY)
  arrow:setScl(ARROW_SCALE)
  arrow:setRot(FLIP_ROTATION)
  self:insert(arrow)

  local hoverEase = AHEaseSimpleInOut.new(2.0)
  local hoverAmount = Screen.getSafeHeightPart(0.008)
  local hoverDuration = 0.7

  local hoverAction = AHSequence.new(AHLocationYBy.new(arrow, hoverDuration, hoverEase, hoverAmount),
                                     AHLocationYBy.new(arrow, hoverDuration, hoverEase, -hoverAmount))
  local hoverLoop = AHLoop.new(hoverAction)

  hoverLoop:start()
end

local function createLayout()
  local helperText = "TAP PIECES TO REMOVE THEM"
  local helperLabel = Label.new(helperText, Font.new(FontFaces.DIN_OT_MEDIUM, HELPER_FONT_SIZE))

  local self = Group.new(helperLabel:getDims())
  self:insertAtCenter(helperLabel)

  local helperTextMinX = -helperLabel:getWidthPart(0.5)
  local helperTextMaxX = helperLabel:getWidthPart(0.5)

  local baseX = self:getWidthPart(0.5)
  local arrowY = self:getHeightPart(0.5)

  local leftArrowX = baseX + helperTextMinX - ARROW_MARGIN_X
  local rightArrowX = baseX + helperTextMaxX + ARROW_MARGIN_X

  createAnimatingArrow(self, leftArrowX, arrowY)
  createAnimatingArrow(self, rightArrowX, arrowY)

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)

  return self
end

function FoodTapToRemoveTipOverlay.new()
  local self = createLayout()
  Mixin.mix(self, FoodTapToRemoveTipOverlay)

  self:setOpacity(0.0)

  local fadeInDuration = 0.4
  local fadeIn = AHOpacityTo.new(self, fadeInDuration, nil, VISIBLE_OPACITY)
  fadeIn:start()

  return self
end
