--- FoodTipItem
-- @author Created by Jay Austria on January 13, 2015

class("FoodTipItem")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local OUTLINE_NAMES_BY_DENOMINATOR = {
  [1] = "outline_whole.png",
  [2] = "outline_half.png",
  [3] = "outline_third.png",
  [4] = "outline_fourth.png",
  [6] = "outline_sixth.png",
  [8] = "outline_eighth.png",
  [12] = "outline_twelfth.png"
}

local FILL_NAMES_BY_DENOMINATOR = {
  [1] = "filling_whole.png",
  [2] = "filling_half.png",
  [3] = "filling_third.png",
  [4] = "filling_fourth.png",
  [6] = "filling_sixth.png",
  [8] = "filling_eighth.png",
  [12] = "filling_twelfth.png"
}

local BACKING_NAMES_BY_DENOMINATOR = {
  [1] = "tips_whole.png",
  [2] = "tips_half.png",
  [3] = "tips_third.png",
  [4] = "tips_fourth.png",
  [6] = "tips_sixth.png",
  [8] = "tips_eighth.png",
  [12] = "tips_twelfth.png"
}

local RADIUS = Screen.getSafeHeightPart(0.03)
local DIAMETER = RADIUS * 2.0
local FILL_COLOR = {0.5, 0.5, 0.5}

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------
local function createBacking(denominator)
  local backingName = BACKING_NAMES_BY_DENOMINATOR[denominator]
  assert(backingName ~= nil, "No backing sprite filename provided for denominator: `" .. tostring(denominator) .. "`")

  local backing = Sprite.newWithName(backingName)
  backing:setEffectiveDims(DIAMETER, DIAMETER)
  return backing
end

local function createSliver(denominator)
  local sliverGroup = Group.new(DIAMETER, DIAMETER)

  local outlineName = OUTLINE_NAMES_BY_DENOMINATOR[denominator]
  assert(outlineName ~= nil, "No outline sprite filename provided for fraction with denominator: `" .. tostring(denominator) .. "`")
  local outlineSprite = Sprite.newWithName(outlineName)
  outlineSprite:setEffectiveDims(DIAMETER, DIAMETER)

  local fillName = FILL_NAMES_BY_DENOMINATOR[denominator]
  assert(fillName ~= nil, "No fill sprite filename provided for fraction with denominator: `" .. tostring(denominator) .. "`")
  local fillSprite = Sprite.newWithName(fillName)
  fillSprite:setColor(unpack(FILL_COLOR))
  fillSprite:setEffectiveDims(DIAMETER, DIAMETER)

  sliverGroup:insertAtCenter(fillSprite)
  sliverGroup:insertAtCenter(outlineSprite)

  return sliverGroup
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Internal Constructor ---------------

local function newInternal(wholePart, numerator, denominator)
  assert(wholePart + numerator / denominator <= 1, "Fractions to display as FoodTipItems cannot be greater than 1")

  local self = Group.new(DIAMETER, DIAMETER)

  local numerator = numerator + wholePart * denominator

  local backOutline = createBacking(denominator)
  self:insertAtCenter(backOutline)

  for i = 1, numerator do
    local sliver = createSliver(denominator)
    self:insertAtCenter(sliver)

    local rotationDelta = 360 / denominator
    local sliverRotation = (i - 1) * rotationDelta
    sliver:setRot(sliverRotation)
  end

  return self
end

-- Constructors -----------------------

function FoodTipItem.new(wholePart, numerator, denominator)
  local self = newInternal(wholePart, numerator, denominator)
  return self
end
