--- FoodValidator
-- @author Created by Jay Austria on January 9, 2015
-- ConceptValidator for Food.

class("FoodValidator")

require("validators.ConceptValidator")
require("FoodRound")
require("FoodSolutionFinder")
require("FoodAnswerOptionChooser")
require("Mocks") -- TODO: Figure out if this should be included by default in all Validators (default.lua template)

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  "numerator",
  "denominator",
  "round",
  "tipType"
}

local VALID_ROUNDS = {
  "double",
  "triple",
  "half",
  "third",
  "fourth"
}

local VALID_TIP_TYPES = {
  "double_tip",
  "triple_tip",
  "half_tip",
  "third_tip",
  "quarter_tip",
  "third_tip_special",
  "half_tip_special",
  "triple_tip_special",
  "double_tip_special",
  "third_tip_easy",
  "quarter_tip_easy",
  "double_tip_easy",
  "triple_tip_easy"
}

local VALID_DENOMINATORS = {
  1, 2, 3, 4, 6, 8
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validateSolution(self, concept)
  local round = FoodRound.newWithConcept(concept)
  local correctAnswer = round:getCorrectAnswer()

  -- NOTE: Calling FoodSolutionFinder.getOptimalSteps will result in a failed assert if there is no
  --       solution. That error will be caught and reported as a failed test.
  local optimalSteps = FoodSolutionFinder.getOptimalSteps(correctAnswer)
  local optimalStepAmounts = TableUtils.map(optimalSteps, function(fraction) return fraction:toNumber() end)
  local answerOptionTypes = TableUtils.createSetFromArray(optimalStepAmounts)
  local answerOptionTypesCount = TableUtils.getSize(answerOptionTypes)
  self:expectLessOrEqual(answerOptionTypesCount, FoodAnswerOptionChooser.MAXIMUM_ANSWER_OPTION_COUNT,
    string.format("The amount cannot be expressed in the required number of fractions types, optimal solution found: %s", StringUtils.tostring(optimalStepAmounts)))
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FoodValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)

  local numerator = concept.numerator
  self:expectType(numerator, "number", "numerator")

  local denominator = concept.denominator
  self:expectIsContained(denominator, VALID_DENOMINATORS, "denominator")

  local round = concept.round
  self:expectIsContained(round, VALID_ROUNDS, "round")

  local tipType = concept.tipType
  self:expectIsContained(tipType, VALID_TIP_TYPES, "tipType")

  validateSolution(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FoodValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, FoodValidator)

  return self
end
