--- FunkyConfiguration
-- @author Created by Aaron Barrett on February 01, 2016.

class("FunkyConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

FunkyConfiguration.GAME_DURATION = 60.0

FunkyConfiguration.CONCEPTS_COUNT_PER_GAME = 28

FunkyConfiguration.POINTS_PER_REGULAR_MATCH = 750
FunkyConfiguration.POINTS_PER_EXTRA_MATCH_INITIAL = 1000
FunkyConfiguration.POINTS_PER_EXTRA_MATCH_MINIMUM = 100

FunkyConfiguration.WRONG_ANSWER_PENALTY_IN_SECONDS = 7.0

FunkyConfiguration.TUTORIAL_DELAY = 8

local MINIMUM_MATCHES_NEEDED_BY_DIFFICULTY_THRESHOLD = {
  {threshold = 0.0, matches = 4},
  {threshold = 0.2, matches = 5},
  {threshold = 0.3, matches = 6},
  {threshold = 0.4, matches = 7},
  {threshold = 0.7, matches = 8}
}

local EXTRA_MATCHES_FOR_RANK_TWO = 5
local EXTRA_MATCHES_FOR_RANK_THREE = 9

local BONUS_DECAY_RATE = 0.8

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyConfiguration.getMinimumMatchesNeeded()
  local difficulty = GameSession.getDifficulty()
  local minimumMatchesNeeded = -1
  for _, matchesAndThreshold in ipairs(MINIMUM_MATCHES_NEEDED_BY_DIFFICULTY_THRESHOLD) do
    local threshold = matchesAndThreshold.threshold
    local matches = matchesAndThreshold.matches
    if ((difficulty >= threshold) and (minimumMatchesNeeded < matches)) then
      minimumMatchesNeeded = matches
    end
  end
  assert(minimumMatchesNeeded ~= -1, "The minimum matches needed was not set properly.")
  return minimumMatchesNeeded
end

function FunkyConfiguration.getPassScore()
  local matchesCountToPass = FunkyConfiguration.getMinimumMatchesNeeded()
  return matchesCountToPass * FunkyConfiguration.POINTS_PER_REGULAR_MATCH
end

function FunkyConfiguration.getRankTwoScore()
  local passScore = FunkyConfiguration.getPassScore()

  local extraMatchPointsForRankTwo = 0
  for i = 1, EXTRA_MATCHES_FOR_RANK_TWO do
    extraMatchPointsForRankTwo = extraMatchPointsForRankTwo + FunkyConfiguration.getDecayingBonusPoints(i)
  end

  return passScore + extraMatchPointsForRankTwo
end

function FunkyConfiguration.getRankThreeScore()
  local passScore = FunkyConfiguration.getPassScore()

  local extraMatchPointsForRankThree = 0
  for i = 1, EXTRA_MATCHES_FOR_RANK_THREE do
    extraMatchPointsForRankThree = extraMatchPointsForRankThree + FunkyConfiguration.getDecayingBonusPoints(i)
  end

  return passScore + extraMatchPointsForRankThree
end

function FunkyConfiguration.getDifficultyKeyFrames()
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Easy = 1.0
      }
    },
    {
      key = 0.1,
      valueProbabilities = {
        Easy = 0.9,
        Medium = 0.1
      }
    },
    {
      key = 0.125,
      valueProbabilities = {
        Easy = 0.85,
        Medium = 0.15
      }
    },
    {
      key = 0.2,
      valueProbabilities = {
        Easy = 0.45,
        Medium = 0.55
      }
    },
    {
      key = 0.375,
      valueProbabilities = {
        Medium = 1.0
      }
    },
    {
      key = 0.7,
      valueProbabilities = {
        Medium = 0.8,
        Hard = 0.2
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Medium = 0.1,
        Hard = 0.9
      }
    }
  }
end

function FunkyConfiguration.getDecayingBonusPoints(numberOverMinimum)
  local numberOfDecayInstances = numberOverMinimum - 1
  local totalDecayFactor = BONUS_DECAY_RATE ^ numberOfDecayInstances
  local decayedBonusPoints = FunkyConfiguration.POINTS_PER_EXTRA_MATCH_INITIAL * totalDecayFactor
  decayedBonusPoints = math.max(decayedBonusPoints, FunkyConfiguration.POINTS_PER_EXTRA_MATCH_MINIMUM)

  return decayedBonusPoints
end
