--- FunkyDiamond
-- @author Created by Aaron Barrett on February 25, 2016.

class("FunkyDiamond")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_DARK_FILLING = {0.06, 0.0, 0.19, 1.0}

local COLOR_PURPLE = {0.44, 0.04, 1.0, 0.15}
local COLOR_OTHER_PURPLE = {0.59, 0.0, 1.0, 0.15}

local COLOR_GREEN = {0.0, 0.88, 1.0, 0.25}
local COLOR_OTHER_GREEN = {1.0, 1.0, 1.0, 0.25}

local COLOR_PINK = {0.98, 0.0, 0.52, 0.25}
local COLOR_OTHER_PINK = {0.98, 0.0, 0.96, 0.25}

local COLOR_OUTRO_NON_CHECKMARK = {0.01, 0.42, 0.37, 0.25}
local COLOR_OTHER_OUTRO_NON_CHECKMARK = {0.01, 0.06, 0.22, 0.25}

local COLOR_OUTRO_CHECKMARK = {0.0, 1.0, 0.91, 0.95}
local COLOR_OTHER_OUTRO_CHECKMARK = {0.56, 1.0, 0.97, 0.95}

local COLOR_INTRO_PURPLE = {0.73, 0.69, 0.99, 1.0}
local COLOR_OTHER_INTRO_PURPLE = {0.98, 0.93, 1.0, 1.0}

local Z_FILLING = 10
local Z_GLOW = 20
local Z_OUTLINE = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createColorWithRandomizedOpacity(originalColor, minimumOpacity, maximumOpacity)
  local randomizedColor = TableUtils.shallowCopy(originalColor)
  randomizedColor[4] = MathUtils.getRandomFloat(minimumOpacity, maximumOpacity)
  return randomizedColor
end

local function createRandomColorBetweenColors(firstColor, secondColor)
  local colorBalance = math.random()
  return ColorUtils.makeColorBetweenTwoColors(firstColor, secondColor, colorBalance)
end

local function createRandomPurpleColor()
  return createRandomColorBetweenColors(COLOR_PURPLE, COLOR_OTHER_PURPLE)
end

local function createRandomGreenColor()
  return createRandomColorBetweenColors(COLOR_GREEN, COLOR_OTHER_GREEN)
end

local function createRandomPinkColor()
  return createRandomColorBetweenColors(COLOR_PINK, COLOR_OTHER_PINK)
end

local function createRandomOutroNonCheckmarkColor()
  return createRandomColorBetweenColors(COLOR_OUTRO_NON_CHECKMARK, COLOR_OTHER_OUTRO_NON_CHECKMARK)
end

local function createRandomOutroCheckmarkColor()
  return createRandomColorBetweenColors(COLOR_OUTRO_CHECKMARK, COLOR_OTHER_OUTRO_CHECKMARK)
end

local function createRandomWhitishPinkColor()
  return createRandomColorBetweenColors(COLOR_INTRO_PURPLE, COLOR_OTHER_INTRO_PURPLE)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyDiamond:playIntro()
  local fillingColor = createColorWithRandomizedOpacity(COLOR_DARK_FILLING, 0.0, 1.0)
  return AHColorTo.new(self.filling, 0.9, nil, fillingColor):start()
end

function FunkyDiamond:createCorrectAction()
  local greenColor = createRandomGreenColor()
  local purpleColor = createRandomPurpleColor()
  return AHSpawn.new(
    AHSequence.new(
      AHColorTo.new(self.glow, 0.1, nil, greenColor),
      AHColorTo.new(self.glow, 0.4, nil, purpleColor)
    ),
    AHSequence.new(
      AHColorTo.new(self.outline, 0.1, nil, greenColor),
      AHDelay.new(0.25),
      AHColorTo.new(self.outline, 0.8, nil, purpleColor)
    )
  )
end

function FunkyDiamond:createWrongAction()
  local pinkColor = createRandomPinkColor()
  local purpleColor = createRandomPurpleColor()
  local purpleOutlineColor = createColorWithRandomizedOpacity(purpleColor, 0.15, 0.75)
  return AHSequence.new(
    AHSpawn.new(
      AHColorTo.new(self.glow, 0.4, nil, pinkColor),
      AHColorTo.new(self.outline, 0.4, nil, pinkColor)
    ),
    AHSpawn.new(
      AHColorTo.new(self.glow, 0.4, nil, purpleColor),
      AHColorTo.new(self.outline, 0.4, nil, purpleOutlineColor)
    )
  )
end

function FunkyDiamond:createNonCheckmarkOutroAction()
  local greenColor = createRandomOutroNonCheckmarkColor()
  return AHSpawn.new(
    AHColorTo.new(self.glow, 0.1, nil, greenColor),
    AHColorTo.new(self.outline, 0.1, nil, greenColor)
  )
end

function FunkyDiamond:createCheckmarkOutroAction()
  local greenColor = createRandomOutroCheckmarkColor()
  return AHSpawn.new(
    AHColorTo.new(self.glow, 0.1, nil, greenColor),
    AHColorTo.new(self.outline, 0.1, nil, greenColor)
  )
end

function FunkyDiamond:overrideColor(color)
  self.filling:setColor(color)
  self.glow:setColor(color)
  self.outline:setColor(color)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FunkyDiamond.new()
  local filling = Sprite.newWithName("diamond_filling.png")
  local glow = Sprite.newWithName("diamond_glow.png")
  local outline = Sprite.newWithName("diamond_outline.png")

  local size = outline:getWidth() * math.sqrt(2.0)

  local self = Group.new(size, size)
  Mixin.mix(self, FunkyDiamond)

  local randomPurpleColor = createRandomPurpleColor()

  local initialFillingColor = createRandomWhitishPinkColor()
  filling:setColor(unpack(initialFillingColor))
  filling:setRot(45.0)
  self:insertAtCenter(filling, Z_FILLING)
  glow:setColor(unpack(randomPurpleColor))
  glow:setRot(45.0)
  self:insertAtCenter(glow, Z_GLOW)
  local randomPurpleColorWithRandomOpacity = createColorWithRandomizedOpacity(randomPurpleColor, 0.15, 0.75)
  outline:setColor(unpack(randomPurpleColorWithRandomOpacity))
  outline:setRot(45.0)
  self:insertAtCenter(outline, Z_OUTLINE)

  self.glow = glow
  self.filling = filling
  self.outline = outline

  return self
end
