--- FunkyExpressionImage
-- @author Created by Aaron Barrett on February 23, 2016.

class("FunkyExpressionImage")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FILLED_SECTION_NAMES_BY_DENOMINATOR = {
  [2] = "half.png",
  [3] = "third.png",
  [4] = "fourth.png",
  [5] = "fifth.png",
  [6] = "sixth.png",
  [8] = "eighth.png",
  [10] = "tenth.png"
}

local OUTLINED_SECTION_NAMES_BY_DENOMINATOR = {
  [2] = "half_outline.png",
  [3] = "third_outline.png",
  [4] = "fourth_outline.png",
  [5] = "fifth_outline.png",
  [6] = "sixth_outline.png",
  [8] = "eighth_outline.png",
  [10] = "tenth_outline.png"
}

local UNFILLED_COLOR = {0.25, 0.25, 0.25}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function getPieceSpriteName(denominator, filled)
  local name = nil
  if (filled) then
    name = FILLED_SECTION_NAMES_BY_DENOMINATOR[denominator]
  else
    name = OUTLINED_SECTION_NAMES_BY_DENOMINATOR[denominator]
  end
  assert(name ~= nil, "No section sprite defined for denominator `" .. tostring(denominator) .. "`")
  return name
end

local function createPiece(self, pieceSpriteName, filled)
  local piece = Sprite.newWithName(pieceSpriteName)

  piece:inheritColor(self)
  if (not filled) then
    piece:setOpacity(0.5)
  end

  return piece
end

-- Constructor ------------------------

function FunkyExpressionImage.new(fraction, size)
  local self = Group.new(size, size)
  Mixin.mix(self, FunkyExpressionImage)

  local numerator = fraction:getNumerator()
  local denominator = fraction:getDenominator()

  local pieces = {}
  for pieceIndex = 1, denominator do
    local filled = (pieceIndex <= numerator)
    local pieceSpriteName = getPieceSpriteName(denominator, filled)
    local piece = createPiece(self, pieceSpriteName, filled)
    table.insert(pieces, piece)
  end

  local firstPiece = pieces[1]
  local pieceHolder = Group.new(firstPiece:getWidth(), firstPiece:getHeight())

  local rotationPerPiece = 360.0 / denominator

  for pieceIndex, piece in ipairs(pieces) do
    local rotation = (pieceIndex - 1) * rotationPerPiece
    piece:setRot(rotation)
    pieceHolder:insertAtCenter(piece)
  end

  pieceHolder:setEffectiveDims(self:getWidth(), self:getHeight())
  self:insertAtCenter(pieceHolder)

  return self
end
