--- FunkyFractionPermutation
-- @author Created by Aaron Barrett on February 01, 2016.

class("FunkyFractionPermutation")

require("FunkyFraction")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local POSSIBLE_FRACTIONS = {
  FunkyFraction.new(1, 2),
  FunkyFraction.new(1, 3),
  FunkyFraction.new(2, 3),
  FunkyFraction.new(1, 4),
  FunkyFraction.new(3, 4),
  FunkyFraction.new(1, 6),
  FunkyFraction.new(5, 6),
  FunkyFraction.new(1, 8),
  FunkyFraction.new(3, 8),
  FunkyFraction.new(5, 8),
  FunkyFraction.new(7, 8)
}

local BOGUS_CONCEPT_ID = "N/A"
local BOGUS_CONCEPT_DIFFICULTY = "N/A"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function isFractionContainedInArray(fractionToFind, fractions)
  for _, fraction in ipairs(fractions) do
    if (fractionToFind:equals(fraction)) then
      return true
    end
  end

  return false
end

local function getFirstFractionNotIncluded(self, disallowedFractions)
  for _, fraction in ipairs(POSSIBLE_FRACTIONS) do
    if (not isFractionContainedInArray(fraction, disallowedFractions)) then
      return fraction
    end
  end
  error("Out of possible fractions.")
end

local function getRandomFormat(possibleFormats)
  local formatsCount = #possibleFormats
  local formatIndex = math.random(formatsCount)
  local format = possibleFormats[formatIndex]
  return format
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyFractionPermutation:createPermutationWithChangedFraction(usedFractions)
  local disallowedFractions = TableUtils.shallowCopy(usedFractions)
  local fraction = getFirstFractionNotIncluded(self, disallowedFractions)

  local format = self.format
  local multiplier = self.multiplier
  local permutation = FunkyFractionPermutation.newGenerated(fraction, format, multiplier)

  return permutation
end

function FunkyFractionPermutation:createForcedMatch(otherPermutation)
  local fraction = otherPermutation:getFraction()

  local format = nil
  local multiplier = 0

  local otherFormat = otherPermutation:getFormat()

  if (otherFormat == FunkyFractionFormat.FRACTION) then
    format = getRandomFormat({FunkyFractionFormat.IMAGE, FunkyFractionFormat.PERCENTAGE})
  elseif (otherFormat == FunkyFractionFormat.IMAGE) then
    format = getRandomFormat({FunkyFractionFormat.PERCENTAGE, FunkyFractionFormat.FRACTION})
  elseif (otherFormat == FunkyFractionFormat.PERCENTAGE) then
    format = getRandomFormat({FunkyFractionFormat.IMAGE, FunkyFractionFormat.FRACTION})
  elseif (otherFormat == FunkyFractionFormat.MONEY) then
    format = getRandomFormat({FunkyFractionFormat.IMAGE, FunkyFractionFormat.FRACTION, FunkyFractionFormat.PERCENTAGE})
  else
    error("No action defined for fraction format: `" .. tostring(otherFormat) .. "`")
  end

  if (format == FunkyFractionFormat.FRACTION) then
    multiplier = 1
  end

  local forcedMatch = FunkyFractionPermutation.newGenerated(fraction, format, multiplier)

  return forcedMatch
end

function FunkyFractionPermutation:canMatch(otherPermutation)
  local otherFraction = otherPermutation:getFraction()
  if (self.fraction:equals(otherFraction)) then

    local otherFormat = otherPermutation:getFormat()
    if (self.format ~= otherFormat) then
      return true
    end
  end

  return false
end

function FunkyFractionPermutation:hasFractionFrom(fractions)
  for _, fraction in ipairs(fractions) do
    if (self.fraction:equals(fraction)) then
      return true
    end
  end
  return false
end

function FunkyFractionPermutation:getFraction()
  return self.fraction
end

function FunkyFractionPermutation:getFormat()
  return self.format
end

function FunkyFractionPermutation:getMultiplier()
  return self.multiplier
end

function FunkyFractionPermutation:getDifficulty()
  return self.difficulty
end

function FunkyFractionPermutation:getConceptId()
  return self.conceptId
end

function FunkyFractionPermutation:getName()
  local reducedForm = self.fraction:getReducedForm()
  local format = tostring(self.format)
  local name = reducedForm .. " , " .. format
  if (self.multiplier ~= nil) then
    name = name .. " , " .. tostring(self.multiplier)
  end
  return name
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FunkyFractionPermutation.newFromConcept(fraction, format, multiplier, difficulty, conceptId)
  local self = {}
  Mixin.mix(self, FunkyFractionPermutation)

  self.fraction = fraction
  self.format = format
  self.multiplier = multiplier
  self.difficulty = difficulty
  self.conceptId = conceptId

  return self
end

function FunkyFractionPermutation.newGenerated(fraction, format, multiplier)
  return FunkyFractionPermutation.newFromConcept(fraction, format, multiplier, BOGUS_CONCEPT_DIFFICULTY, BOGUS_CONCEPT_ID)
end
