--- FunkyIntroOverlay
-- @author Created by Aaron Barrett on March 01, 2016.

class("FunkyIntroOverlay")

require("FunkyShadowedBox")
require("FunkyProgressMeter")
require("FunkySkewedGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_DARK_LABEL = {0.23, 0.16, 0.51, 1.0}

local Z_LOWER_GRADIENT = 10
local Z_NEEDED_MATCHES_BOX = 20
local Z_PROGRESS_METER = 30
local Z_TAP_TO_CONTINUE_LABEL = 40

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyIntroOverlay:hasBeenTapped()
  return self.tapped
end

function FunkyIntroOverlay:playAppear()
  local progressMeter = self.progressMeter
  local neededMatchesBox = self.neededMatchesBox

  local ease = AHEaseExponentialOut.new()

  local appearAction = AHSpawn.new(
    AHLocationYTo.new(progressMeter, 1.0, ease, Screen.getSafeY(0.5) + progressMeter:getHeightPart(0.65)),
    AHDelayed.new(0.05,
      AHLocationYTo.new(neededMatchesBox, 1.0, ease, Screen.getSafeY(0.5) - neededMatchesBox:getHeightPart(0.5))
    ),
    AHDelayed.new(0.65,
      AHSpawn.new(
        function()
          self:setTouchEnabled(true)
        end,
        AHOpacityTo.new(self.tapToContinueLabel, 1.0, nil, 1.0)
      )
    )
  )

  self.actionInterrupt:insert(appearAction)
end

function FunkyIntroOverlay:playDisappear()
  local progressMeter = self.progressMeter
  local neededMatchesBox = self.neededMatchesBox
  progressMeter:playTurnClear()
  neededMatchesBox:playTurnPurple()

  local ease = AHEaseExponentialOut.new()

  local disappearAction = AHSequence.new(
    AHSpawn.new(
      AHOpacityTo.new(self.lowerGradient, 0.2, nil, 0.0),
      AHOpacityTo.new(self.tapToContinueLabel, 0.8, nil, 0.0),
      AHLocationYTo.new(progressMeter, 1.2, ease, Screen.getSafeY(0.9)),
      AHLocationYTo.new(neededMatchesBox, 1.5, ease, Screen.getSafeY(0.87) - neededMatchesBox:getHeightPart(0.5))
    ),
    AHRemove.new(self)
  )

  self.actionInterrupt:insert(disappearAction)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildLowerGradient(self)
  local lowerGradient = FunkySkewedGradient.new()
  lowerGradient:setScl(1.0, -2.0)
  lowerGradient:setLoc(self:getWidthPart(0.5), lowerGradient:getHeight())
  self:insert(lowerGradient, Z_LOWER_GRADIENT)
  self.lowerGradient = lowerGradient
end

local function buildNeededMatchesBox(self, neededMatchesCount)
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 14.0)
  local neededString = tostring(neededMatchesCount)
  local toPassString = Localizer.get("TO_PASS")
  local text = string.format("%s %s", neededString, toPassString)
  local label = Label.new(text, font)
  label:setColor(unpack(COLOR_DARK_LABEL))
  local shadowFilename = "shadow_short.png"
  local box = FunkyShadowedBox.new(label, shadowFilename)
  box:setLoc(Screen.getSafeX(0.5), -self:getHeightPart(0.3))
  self:insert(box, Z_NEEDED_MATCHES_BOX)
  self.neededMatchesBox = box
end

local function buildProgressMeter(self)
  local progressMeter = FunkyProgressMeter.new()
  progressMeter:setLoc(Screen.getSafeX(0.5), -self:getHeightPart(0.3))
  self:insert(progressMeter, Z_PROGRESS_METER)
  self.progressMeter = progressMeter
end

local function buildTapToContinueLabel(self)
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 12.0)
  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), font)
  tapToContinueLabel:setColor(unpack(COLOR_DARK_LABEL))
  tapToContinueLabel:setOpacity(0.0)
  tapToContinueLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.10))
  self:insert(tapToContinueLabel, Z_TAP_TO_CONTINUE_LABEL)
  self.tapToContinueLabel = tapToContinueLabel
end

-- Constructor ------------------------

function FunkyIntroOverlay.new(neededMatchesCount, handoffProgressMeterCallback)
  local self = Group.newFullScreen()
  Mixin.mix(self, FunkyIntroOverlay)

  buildLowerGradient(self)
  buildNeededMatchesBox(self, neededMatchesCount)
  buildProgressMeter(self)
  buildTapToContinueLabel(self)

  self.actionInterrupt = ActionInterrupt.new()
  self.tapped = false

  local function onTapped()
    self:setTouchEnabled(false)
    Sound.playSoundEffect("start_game")
    handoffProgressMeterCallback(self.progressMeter, self.neededMatchesBox)
    self:playDisappear()
    self.tapped = true
  end
  Button.makeButton(self, onTapped)
  self:setTouchEnabled(false)

  return self
end
