--- FunkyItem
-- @author Created by Aaron Barrett on February 26, 2016.

class("FunkyItem")

require("FunkyBackground")
require("FunkyFractionFormat")
require("FunkyItemFaceFraction")
require("FunkyItemFaceImage")
require("FunkyItemFaceMoney")
require("FunkyItemFacePercentage")
require("FunkyRibbon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_BLACK = {0.0, 0.0, 0.0}
local COLOR_WHITE = {1.0, 1.0, 1.0}

local COLOR_OUTLINE_PINK = {0.98, 0.0, 0.52}
local COLOR_FACE_PINK = {1.0, 0.42, 0.55}

local COLOR_OUTLINE_PURPLE = {0.40, 0.10, 0.71}
local COLOR_FACE_PURPLE = {0.84, 0.47, 0.99}

local COLOR_GREEN = {0.17, 1.0, 0.81}
local COLOR_GRAY = {0.37, 0.37, 0.37}

local SCALE_SELECTED = 1.05
local SCALE_CORRECT = 1.05
local SCALE_WRONG = 1.05
local SCALE_OVERSCALE = 1.115

local Z_ROLL_GROUP = 10

local Z_FILLING = 10
local Z_GLOW = 20
local Z_RIBBON = 30

local Z_FACE = 20

local FACE_CLASSES_BY_FORMAT = {
  [FunkyFractionFormat.FRACTION] = FunkyItemFaceFraction,
  [FunkyFractionFormat.IMAGE] = FunkyItemFaceImage,
  [FunkyFractionFormat.MONEY] = FunkyItemFaceMoney,
  [FunkyFractionFormat.PERCENTAGE] = FunkyItemFacePercentage
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playSelect(self)
  self:setZIndex(FunkyBunch.Z_ITEM_SELECTED)

  local selectAction = AHSpawn.new(
    AHSequence.new(
      AHScaleTo.new(self, 0.05, AHEaseSineIn.new(), SCALE_OVERSCALE),
      AHScaleTo.new(self, 0.07, AHEaseSineOut.new(), SCALE_SELECTED)
    ),
    AHColorTo.new(self.face, 0.05, nil, COLOR_WHITE),
    AHColorTo.new(self.ribbon, 0.05, nil, COLOR_WHITE),
    AHColorTo.new(self.glow, 0.05, nil, COLOR_WHITE)
  )

  self.selectionInterrupt:insert(selectAction)
end

local function playDeselect(self)
  self:setZIndex(FunkyBunch.Z_ITEM_UNSELECTED)

  local deselectAction = AHSpawn.new(
    AHScaleTo.new(self, 0.2, AHEaseSineIn.new(), 1.0),
    AHColorTo.new(self.face, 0.2, nil, COLOR_FACE_PURPLE),
    AHColorTo.new(self.ribbon, 0.2, nil, COLOR_OUTLINE_PURPLE),
    AHColorTo.new(self.glow, 0.2, nil, COLOR_OUTLINE_PURPLE)
  )

  self.selectionInterrupt:insert(deselectAction)
end

local function buildAndReturnFace(self, permutation, size)
  local fraction = permutation:getFraction()
  local format = permutation:getFormat()
  local multiplier = permutation:getMultiplier()

  local faceClass = FACE_CLASSES_BY_FORMAT[format]
  local face = faceClass.new(size, size, fraction, multiplier)
  face:setColor(unpack(COLOR_FACE_PURPLE))
  return face
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyItem:setSelected(selected)
  if (self.selected ~= selected) then
    if (selected) then
      playSelect(self)
    else
      playDeselect(self)
    end
  end
  self.selected = selected
end

function FunkyItem:isSelected()
  return self.selected
end

function FunkyItem:getPermutation()
  return self.permutation
end

function FunkyItem:getConceptId()
  local permutation = self.permutation
  local conceptId = permutation:getConceptId()

  return conceptId
end

function FunkyItem:getDifficulty()
  local permutation = self.permutation
  local difficulty = permutation:getDifficulty()

  return difficulty
end

function FunkyItem:getFraction()
  return self.permutation:getFraction()
end

function FunkyItem:getFractionString()
  local fraction = self:getFraction()
  local fractionString = fraction:getNumerator() .. "/" .. fraction:getDenominator()
  return fractionString
end

function FunkyItem:matches(otherItem)
  local fraction = self:getFraction()
  local otherFraction = otherItem:getFraction()
  return (fraction:equals(otherFraction))
end

function FunkyItem:playAppear()
  local ribbon = self.ribbon
  local progressAfterFirstSegment = self.progressAfterFirstSegment
  local ease = AHEaseExponentialOut.new()

  local appearAction = AHSpawn.new(
    AHScaleTo.new(self, 0.5, AHEaseSineOut.new(), 1.0),
    AHOpacityTo.new(self.filling, 0.5, nil, 1.0),
    AHColorTo.new(ribbon, 0.5, nil, COLOR_OUTLINE_PURPLE),
    ribbon:createBandEndProgressToAction(1.15, ease, 1.0),
    ribbon:createBandStartProgressToAction(1.4, ease, progressAfterFirstSegment),
    AHDelayed.new(0.5,
      AHSpawn.new(
        AHOpacityTo.new(self.face, 0.5, nil, 1.0),
        AHOpacityTo.new(self.glow, 0.5, nil, 1.0)
      )
    )
  )

  self.appearInterrupt:insert(appearAction)
end

function FunkyItem:playDisappear(duration)
  local disappearAction = AHSpawn.new(
    AHOpacityTo.new(self.face, duration, nil, 0.0),
    AHOpacityTo.new(self.glow, duration, nil, 0.0),
    AHOpacityTo.new(self.filling, duration, nil, 0.0),
    AHOpacityTo.new(self.ribbon, duration, nil, 0.0)
  )

  self.appearInterrupt:insert(disappearAction)
end

function FunkyItem:hide()
  self:setScl(1.1)

  self.face:setOpacity(0.0)
  self.filling:setOpacity(0.0)
  self.glow:setOpacity(0.0)

  self.ribbon:setColor(unpack(COLOR_OUTLINE_PINK))
  self.ribbon:setBandStartProgress(0.0)
  self.ribbon:setBandEndProgress(0.0)
end

function FunkyItem:jumpToWrongState()
  self:setScl(SCALE_WRONG * FunkyBackground.SCALE_AFTER_INTRO)

  self.face:setColor(unpack(COLOR_FACE_PINK))
  self.glow:setColor(unpack(COLOR_OUTLINE_PINK))

  local ribbon = self.ribbon
  ribbon:setColor(unpack(COLOR_OUTLINE_PINK))
  ribbon:setBandStartProgress(self.progressAfterFirstSegment)
  ribbon:setBandEndProgress(1.0)
end

function FunkyItem:playCorrect()
  local correctAction = AHSpawn.new(
    AHSequence.new(
      AHScaleTo.new(self, 0.1, AHEaseSineIn.new(), SCALE_OVERSCALE),
      AHScaleTo.new(self, 0.15, AHEaseSineOut.new(), SCALE_CORRECT)
    ),
    AHColorTo.new(self.face, 0.25, nil, COLOR_GREEN),
    AHColorTo.new(self.glow, 0.25, nil, COLOR_GREEN),
    AHColorTo.new(self.ribbon, 0.25, nil, COLOR_GREEN),
    AHLoop.new(
      AHSequence.new(
        AHColorTo.new(self.filling, 0.01, nil, COLOR_WHITE),
        AHColorTo.new(self.filling, 0.01, nil, COLOR_GREEN)
      ),
      2
    ),
    AHDelayed.new(0.25,
      AHSpawn.new(
        AHOpacityTo.new(self.face, 0.25, nil, 0.0),
        AHOpacityTo.new(self.glow, 0.25, nil, 0.0),
        AHOpacityTo.new(self.filling, 0.25, nil, 0.0),
        self.ribbon:createBandEndProgressToAction(1.05, AHEaseSineInOut.new(), 0.0),
        self.ribbon:createBandStartProgressToAction(1.05, AHEaseExponentialOut.new(), 0.0)
      )
    )
  )

  self.selectionInterrupt:insert(correctAction)
end

function FunkyItem:playWrong()
  local wrongAction = AHSpawn.new(
    AHSequence.new(
      AHScaleTo.new(self, 0.1, AHEaseSineIn.new(), SCALE_OVERSCALE),
      AHScaleTo.new(self, 0.15, AHEaseSineOut.new(), SCALE_WRONG)
    ),
    AHColorTo.new(self.face, 0.25, nil, COLOR_FACE_PINK),
    AHColorTo.new(self.glow, 0.25, nil, COLOR_OUTLINE_PINK),
    AHColorTo.new(self.ribbon, 0.25, nil, COLOR_OUTLINE_PINK)
  )

  self.selectionInterrupt:insert(wrongAction)
end

function FunkyItem:playRevert()
  local revertAction = AHSpawn.new(
    AHScaleTo.new(self, 0.25, AHEaseSineIn.new(), 1.0),
    AHColorTo.new(self.face, 0.25, nil, COLOR_FACE_PURPLE),
    AHColorTo.new(self.glow, 0.25, nil, COLOR_OUTLINE_PURPLE),
    AHColorTo.new(self.ribbon, 0.25, nil, COLOR_OUTLINE_PURPLE)
  )

  self.selectionInterrupt:insert(revertAction)
end

function FunkyItem:playGray()
  local grayAction = AHSequence.new(
    AHSpawn.new(
      AHColorTo.new(self.face, 0.25, nil, COLOR_GRAY),
      AHColorTo.new(self.glow, 0.25, nil, COLOR_GRAY),
      AHColorTo.new(self.ribbon, 0.25, nil, COLOR_GRAY)
    ),
    AHSpawn.new(
      AHColorTo.new(self.face, 0.25, nil, COLOR_FACE_PURPLE),
      AHColorTo.new(self.glow, 0.25, nil, COLOR_OUTLINE_PURPLE),
      AHColorTo.new(self.ribbon, 0.25, nil, COLOR_OUTLINE_PURPLE)
    )
  )

  self.selectionInterrupt:insert(grayAction)
end

function FunkyItem:playRollTo(destinationX, destinationY, destinationRotation)
  local ease = AHEaseExponentialInOut.new()
  local rollAction = AHSpawn.new(
    AHRotationTo.new(self.rollGroup, 0.75, ease, destinationRotation),
    AHLocationTo.new(self, 0.75, ease, destinationX, destinationY),
    AHScaleTo.new(self, 0.75, ease, 1.11)
  )
  return rollAction:start()
end

function FunkyItem:playMoveToTop(destinationX, destinationY, destinationFaceOpacity)
  local ease = AHEaseSineInOut.new()
  local moveAction = AHSpawn.new(
    AHLocationTo.new(self, 0.5, ease, destinationX, destinationY),
    AHScaleTo.new(self, 0.5, ease, 0.75),
    AHColorTo.new(self.face, 0.5, nil, COLOR_WHITE),
    AHOpacityTo.new(self.face, 0.5, nil, destinationFaceOpacity),
    AHOpacityTo.new(self.filling, 0.25, nil, 0.0),
    AHOpacityTo.new(self.glow, 0.25, nil, 0.0),
    AHOpacityTo.new(self.ribbon, 0.25, nil, 0.0)
  )
  return moveAction:start()
end

function FunkyItem:createHighlightAtTopAction()
  local highlightAction = AHSpawn.new(
    AHOpacityTo.new(self.face, 0.5, nil, 1.0),
    AHSequence.new(
      AHScaleTo.new(self, 0.2, AHEaseSineIn.new(), 1.0),
      AHScaleTo.new(self, 0.3, AHEaseSineOut.new(), 0.75)
    )
  )
  return highlightAction
end

function FunkyItem:createDimAtTopAction()
  local dimAction = AHOpacityTo.new(self.face, 0.5, nil, 0.25)
  return dimAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildRollGroup(self)
  local rollGroup = Group.new(self:getWidth(), self:getHeight())
  self:insertAtCenter(rollGroup, Z_ROLL_GROUP)
  self.rollGroup = rollGroup
end

local function buildFilling(self, filling, orientationAngle)
  filling:setColor(unpack(COLOR_BLACK))
  filling:setRot(orientationAngle)
  self.rollGroup:insertAtCenter(filling, Z_FILLING)
  self.filling = filling
end

local function buildGlow(self, orientationAngle)
  local glow = Sprite.newWithName("diamond_item_glow.png")
  glow:setColor(unpack(COLOR_OUTLINE_PURPLE))
  glow:setRot(orientationAngle)
  self.rollGroup:insertAtCenter(glow, Z_GLOW)
  self.glow = glow
end

local function buildRibbon(self, size, orientationAngle)
  local bandThickness = 3.0
  local itemSize = size * 0.6
  local cornerRadius = itemSize * 0.05

  local startPoint = {
    x = itemSize + cornerRadius + Screen.getWidthPart(1.5),
    y = itemSize + cornerRadius * 2.0
  }

  local startAngle = math.rad(180.0)

  local segmentDescriptors = {
    {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = itemSize + Screen.getWidthPart(1.5)},
    {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = math.rad(90.0), radius = cornerRadius},
    {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = itemSize},
    {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = math.rad(90.0), radius = cornerRadius},
    {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = itemSize},
    {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = math.rad(90.0), radius = cornerRadius},
    {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = itemSize},
    {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = math.rad(90.0), radius = cornerRadius},
    {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = itemSize}
  }

  local ribbonSize = itemSize + cornerRadius * 2.0
  local ribbon = FunkyRibbon.new(ribbonSize, ribbonSize, startPoint, startAngle, segmentDescriptors, bandThickness)

  ribbon:setColor(unpack(COLOR_OUTLINE_PURPLE))
  ribbon:setRot(orientationAngle)

  self.rollGroup:insertAtCenter(ribbon, Z_RIBBON)
  self.ribbon = ribbon

  self.progressAfterFirstSegment = ribbon:getRelativeIntervalOfSegmentAtIndex(1)
end

local function buildFace(self, permutation, size)
  local face = buildAndReturnFace(self, permutation, size)
  face:setOpacity(0.0)
  self:insertAtCenter(face, Z_FACE)
  self.face = face
  self.permutation = permutation
end

local function createTouchHandlerCallback(self, onItemTappedCallback)
  return function(event)
    if (event:isPhase(TouchManager.TOUCH_BEGAN)) then
      local modelX, modelY = self:worldToModel(event.x, event.y)
      local halfSize = self.size * 0.5
      local x = halfSize - modelX
      local y = halfSize - modelY

      local touchFallsWithinDiamond = (math.abs(x) + math.abs(y) < halfSize)
      if (touchFallsWithinDiamond) then
        onItemTappedCallback(self)
      end
    end

    return TouchEvent.HANDLING.CONTINUE
  end
end

-- Constructor ------------------------

function FunkyItem.new(permutation, orientationAngle, onItemTappedCallback)
  orientationAngle = orientationAngle or 45.0

  local filling = Sprite.newWithName("diamond_opaque.png")

  local squareWidth = filling:getWidth()
  local size = squareWidth * math.sqrt(2.0)

  local self = Group.new(size, size)
  Mixin.mix(self, FunkyItem)

  buildRollGroup(self)
  buildFilling(self, filling, orientationAngle)
  buildGlow(self, orientationAngle)
  buildRibbon(self, size, orientationAngle)
  buildFace(self, permutation, size)

  self.size = size
  self.selectionInterrupt = ActionInterrupt.new()
  self.appearInterrupt = ActionInterrupt.new()

  if (onItemTappedCallback ~= nil) then
    local touchHandlerCallback = createTouchHandlerCallback(self, onItemTappedCallback)
    TouchManager.addTouchEnabledMethods(self, touchHandlerCallback, TouchHandler.BEHAVIOR_ALWAYS)
    self:setTouchEnabled(false)
  end

  return self
end
