--- FunkyItemLayoutManager
-- @author Created by Aaron Barrett on February 26, 2016.

class("FunkyItemLayoutManager")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CENTRAL_DIAMONDS_INDICES_BY_ROUND_NUMBER = {
  {FunkyBackground.INDEX_TOP_LEFT_DIAMOND, FunkyBackground.INDEX_TOP_DIAMOND, FunkyBackground.INDEX_TOP_RIGHT_DIAMOND},
  {FunkyBackground.INDEX_TOP_LEFT_DIAMOND, FunkyBackground.INDEX_BOTTOM_DIAMOND, FunkyBackground.INDEX_TOP_RIGHT_DIAMOND},
  {FunkyBackground.INDEX_TOP_LEFT_DIAMOND, FunkyBackground.INDEX_TOP_DIAMOND , FunkyBackground.INDEX_TOP_RIGHT_DIAMOND, FunkyBackground.INDEX_BOTTOM_DIAMOND},
  {FunkyBackground.INDEX_TOP_LEFT_DIAMOND, FunkyBackground.INDEX_TOP_DIAMOND , FunkyBackground.INDEX_TOP_RIGHT_DIAMOND, FunkyBackground.INDEX_BOTTOM_DIAMOND},
  {FunkyBackground.INDEX_TOP_LEFT_DIAMOND, FunkyBackground.INDEX_BOTTOM_DIAMOND, FunkyBackground.INDEX_TOP_RIGHT_DIAMOND, FunkyBackground.INDEX_BOTTOM_LEFT_DIAMOND, FunkyBackground.INDEX_BOTTOM_RIGHT_DIAMOND}
}

local LAYOUTS_COUNT = #(CENTRAL_DIAMONDS_INDICES_BY_ROUND_NUMBER)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyItemLayoutManager:getNextLocationsArray()
  self.roundNumber = self.roundNumber + 1

  local locationsArrayIndex = math.min(self.roundNumber, LAYOUTS_COUNT)
  local locations = self.locationsArraysByRoundNumber[locationsArrayIndex]
  return locations
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildLocationsArrays(self, background)
  local centralDiamonds = background:getCentralDiamonds()
  local locationsArraysByRoundNumber = {}

  for _, centralDiamondsIndices in ipairs(CENTRAL_DIAMONDS_INDICES_BY_ROUND_NUMBER) do
    local locations = {}
    for _, diamondIndex in ipairs(centralDiamondsIndices) do
      local diamond = centralDiamonds[diamondIndex]
      local x, y = diamond:getLoc()
      local location = {x = x, y = y}
      table.insert(locations, location)
    end
    table.insert(locationsArraysByRoundNumber, locations)
  end

  self.locationsArraysByRoundNumber = locationsArraysByRoundNumber
end

-- Constructor ------------------------

function FunkyItemLayoutManager.new(background)
  local self = {}
  Mixin.mix(self, FunkyItemLayoutManager)

  buildLocationsArrays(self, background)

  self.roundNumber = 0

  return self
end
