--- FunkyMoneyFormatter
-- @author Created by Aaron Barrett on February 24, 2016.

class("FunkyMoneyFormatter")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getLeadingZeroesCount(text)
  local leadingZeroesCount = 0

  local luaPatternToExtractCharacters = "."
  for character in string.gmatch(text, luaPatternToExtractCharacters) do
    if (character == "0") then
      leadingZeroesCount = leadingZeroesCount + 1
    else
      break
    end
  end

  return leadingZeroesCount
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyMoneyFormatter.getTextFormattedAsDollarsAndCents(baseNumber, multiplierInCents)
  local multipliedNumber = baseNumber * multiplierInCents * 0.01
  local text = string.format("%.2f", multipliedNumber)
  return text
end

function FunkyMoneyFormatter.getTextOfDollarsPart(baseNumber, multiplierInCents)
  local multipliedNumber = math.floor(baseNumber * multiplierInCents * 0.01)
  local text = tostring(multipliedNumber)
  return text
end

function FunkyMoneyFormatter.getTextOfCentsPart(baseNumber, multiplierInCents)
  local multipliedNumber = (baseNumber * multiplierInCents) % 100
  local text = string.format("%02d", multipliedNumber)
  return text
end

function FunkyMoneyFormatter.getTextWithLeadingZeroesRemoved(text)
  local leadingZeroesCount = getLeadingZeroesCount(text)
  local textLength = #text
  return text:sub(leadingZeroesCount + 1, textLength)
end

function FunkyMoneyFormatter.getLeadingZeroesText(text)
  local leadingZeroesCount = getLeadingZeroesCount(text)
  return text:sub(1, leadingZeroesCount)
end
