--- FunkyOutroRibbonOverlay
-- @author Created by Aaron Barrett on March 08, 2016.

class("FunkyOutroRibbonOverlay")

require("FunkyRibbon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BAND_THICKNESS = 3.0
local RIBBON_COLOR = {0.17, 1.0, 0.81}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyOutroRibbonOverlay:playOutro()
  for _, ribbon in ipairs(self.ribbons) do
    local ribbonAction = AHStaggeredSpawn.new(0.2,
      ribbon:createBandStartProgressToAction(1.7, AHEaseSineInOut.new(), 1.0),
      ribbon:createBandEndProgressToAction(1.7, AHEaseExponentialOut.new(), 1.0)
    )
    ribbonAction:start()
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function addRibbon(self, diamond, xOffset, yOffset, startAngle, segmentDescriptors)
  local ribbon = FunkyRibbon.new(self:getWidth(),
                                 self:getHeight(),
                                 {x = diamond:getX() + xOffset, y = diamond:getY() + yOffset},
                                 startAngle,
                                 segmentDescriptors,
                                 BAND_THICKNESS,
                                 RIBBON_COLOR)
  ribbon:setBandStartProgress(0.0)
  ribbon:setBandEndProgress(0.0)

  self:insertAtCenter(ribbon)
  table.insert(self.ribbons, ribbon)
end

local function buildFirstDiamondRibbons(self, diamond)
  addRibbon(self,
            diamond,
            68.0,
            20.0,
            math.rad(45.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )

  addRibbon(self,
            diamond,
            35.0,
            53.0,
            math.rad(45.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 150.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = math.rad(90.0), radius = 5.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 190.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = -math.rad(90.0), radius = 5.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 500.0}
            }
            )

  addRibbon(self,
            diamond,
            -10.0,
            78.0,
            math.rad(135.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )

  addRibbon(self,
            diamond,
            -68.0,
            20.0,
            math.rad(135.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )

  addRibbon(self,
            diamond,
            -23.0,
            -65.0,
            -math.rad(135.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )
end

local function buildSecondDiamondRibbons(self, diamond)
  addRibbon(self,
            diamond,
            -73.0,
            -15.0,
            -math.rad(135.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )

  addRibbon(self,
            diamond,
            -13.0,
            -75.0,
            -math.rad(135.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )

  addRibbon(self,
            diamond,
            43.0,
            -45.0,
            -math.rad(45.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 170.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = -math.rad(90.0), radius = 5.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 200.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = math.rad(90.0), radius = 5.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 500.0}
            }
            )

  addRibbon(self,
            diamond,
            68.0,
            -20.0,
            -math.rad(45.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )
end

local function buildThirdDiamond(self, diamond)
  addRibbon(self,
            diamond,
            8.0,
            -80.0,
            -math.rad(45.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )

  addRibbon(self,
            diamond,
            38.0,
            -50.0,
            -math.rad(45.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )

    addRibbon(self,
              diamond,
              74.0,
              -14.0,
              -math.rad(45.0),
              {
                {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
              }
              )
end

local function buildFourthDiamond(self, diamond)
  addRibbon(self,
            diamond,
            8.0,
            80.0,
            math.rad(45.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 680.0}
            }
            )

  addRibbon(self,
            diamond,
            40.0,
            48.0,
            math.rad(45.0),
            {
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 70.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_ARC, arcAngle = -math.rad(90.0), radius = 5.0},
              {segmentType = FunkyRibbon.SEGMENT_TYPE_LINE, length = 500.0}
            }
            )
end

-- Constructor ------------------------

function FunkyOutroRibbonOverlay.new(width, height, checkmarkDiamonds)
  local self = Group.new(width, height)
  Mixin.mix(self, FunkyOutroRibbonOverlay)

  self.ribbons = {}

  local firstDiamond = checkmarkDiamonds[1]
  buildFirstDiamondRibbons(self, firstDiamond)

  local secondDiamond = checkmarkDiamonds[2]
  buildSecondDiamondRibbons(self, secondDiamond)

  local thirdDiamond = checkmarkDiamonds[3]
  buildThirdDiamond(self, thirdDiamond)

  local fourthDiamond = checkmarkDiamonds[4]
  buildFourthDiamond(self, fourthDiamond)

  return self
end
