--- FunkyPresentationFraction
-- @author Created by Aaron Barrett on February 23, 2016.

class("FunkyPresentationFraction")

require("FunkyPresentationBase")
require("FunkyExpressionFraction")
require("FunkyExpressionMoney")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PAGE_TYPE_REDUCED_FRACTION = "PAGE_TYPE_REDUCED_FRACTION"

local PAGE_TYPE_MONEY = "PAGE_TYPE_MONEY"
local PAGE_TYPE_UNREDUCED_FRACTION = "PAGE_TYPE_UNREDUCED_FRACTION"
local PAGE_TYPE_FACTORED_OUT = "PAGE_TYPE_FACTORED_OUT"
local PAGE_TYPE_CROSSED_OUT = "PAGE_TYPE_CROSSED_OUT"
local PAGE_TYPE_EQUALITY = "PAGE_TYPE_EQUALITY"
local PAGE_TYPE_MONEY_ALREADY_REDUCED_EQUALITY = "PAGE_TYPE_MONEY_ALREADY_REDUCED_EQUALITY"

local FONT_SIZE = 32.0
local FONT = Font.new(FontFaces.DIN_OT_MEDIUM, FONT_SIZE)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createCombineMoneyAction(self, destinationFractionExpression)
  local numeratorDestinationY = self:getHeightPart(0.5) - destinationFractionExpression:getHeightPart(0.5) + destinationFractionExpression:getNumeratorY()
  local denominatorDestinationY = self:getHeightPart(0.5) - destinationFractionExpression:getHeightPart(0.5) + destinationFractionExpression:getDenominatorY()

  local combineMoneyAction = AHSpawn.new(
    self.numeratorMoneyExpression:createHideDecimalAction(),
    AHLocationYTo.new(self.numeratorMoneyExpression, 0.5, AHEaseSineInOut.new(), numeratorDestinationY),
    AHOpacityTo.new(self.outOfLabel, 0.5, nil, 0.0),
    self.denominatorMoneyExpression:createHideDecimalAction(),
    AHLocationYTo.new(self.denominatorMoneyExpression, 0.5, AHEaseSineInOut.new(), denominatorDestinationY),
    AHOpacityTo.new(self.duplicateBar, 0.5, nil, 1.0)
  )

  return combineMoneyAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- FunkyPresentation Protocol ---------

function FunkyPresentationFraction:getTitleText()
  return self.titleText
end

function FunkyPresentationFraction:prepareToShowPage(pageIndex)
  self:hide()

  local pageTypes = self.pageTypes
  local previousPageType = pageTypes[pageIndex - 1]
  local pageType = pageTypes[pageIndex]

  if (previousPageType == nil) then

    if (pageType == PAGE_TYPE_REDUCED_FRACTION) then
      self.reducedFractionExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
    elseif (pageType == PAGE_TYPE_UNREDUCED_FRACTION) then
      self.unreducedFractionExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
    elseif (pageType == PAGE_TYPE_MONEY) then
      self.numeratorMoneyExpression:jumpToDecimalShownState()
      self.numeratorMoneyExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.69))
      self.denominatorMoneyExpression:jumpToDecimalShownState()
      self.outOfLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
      self.denominatorMoneyExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.31))
    end

  elseif (previousPageType == PAGE_TYPE_MONEY) then

    self.numeratorMoneyExpression:setOpacity(1.0)
    self.numeratorMoneyExpression:jumpToDecimalShownState()
    self.numeratorMoneyExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.69))
    self.outOfLabel:setOpacity(1.0)
    self.denominatorMoneyExpression:setOpacity(1.0)
    self.denominatorMoneyExpression:jumpToDecimalShownState()
    self.denominatorMoneyExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.31))

    self.duplicateBar:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5) - FunkyExpressionFraction.VERTICAL_BAR_OFFSET)
    if (pageType == PAGE_TYPE_MONEY_ALREADY_REDUCED_EQUALITY) then
      self.reducedFractionExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
    elseif (pageType == PAGE_TYPE_UNREDUCED_FRACTION) then
      self.unreducedFractionExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
    end

  elseif (previousPageType == PAGE_TYPE_UNREDUCED_FRACTION) then

    self.equalsSign:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))

    self.unreducedFractionExpression:setOpacity(1.0)
    self.unreducedFractionExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))

    self.factoredOutFractionExpression:jumpToUnstruckState()
    self.factoredOutFractionExpression:setLoc(self:getWidthPart(0.75), self:getHeightPart(0.5))

  elseif (previousPageType == PAGE_TYPE_FACTORED_OUT) then

    self.unreducedFractionExpression:setOpacity(1.0)
    self.unreducedFractionExpression:setLoc(self:getWidthPart(0.25), self:getHeightPart(0.5))

    self.equalsSign:setOpacity(1.0)
    self.equalsSign:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))

    self.factoredOutFractionExpression:jumpToUnstruckState()
    self.factoredOutFractionExpression:setOpacity(1.0)
    self.factoredOutFractionExpression:setLoc(self:getWidthPart(0.75), self:getHeightPart(0.5))

  elseif (previousPageType == PAGE_TYPE_CROSSED_OUT) then

    self.unreducedFractionExpression:setOpacity(1.0)
    self.unreducedFractionExpression:setLoc(self:getWidthPart(0.25), self:getHeightPart(0.5))

    self.equalsSign:setOpacity(1.0)
    self.equalsSign:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))

    self.factoredOutFractionExpression:jumpToStruckState()
    self.factoredOutFractionExpression:setOpacity(1.0)
    self.factoredOutFractionExpression:setLoc(self:getWidthPart(0.75), self:getHeightPart(0.5))

    self.reducedFractionExpression:setX(self:getWidthPart(0.75) - self.factoredOutFractionExpression:getWidthPart(0.5) + self.reducedFractionExpression:getWidthPart(0.5))
    self.reducedFractionExpression:setY(self:getHeightPart(0.5))
    self.reducedFractionExpression:setOpacity(1.0)

  end
end

function FunkyPresentationFraction:playShowPage(pageIndex)
  local showAction = AHSpawn.new()

  local pageTypes = self.pageTypes
  local previousPageType = pageTypes[pageIndex - 1]
  local pageType = pageTypes[pageIndex]

  if (pageType == PAGE_TYPE_REDUCED_FRACTION) then

    showAction:addAction(AHOpacityTo.new(self.reducedFractionExpression, 0.5, nil, 1.0))

  elseif (pageType == PAGE_TYPE_MONEY) then

    showAction:addAction(AHOpacityTo.new(self.numeratorMoneyExpression, 0.5, nil, 1.0))
    showAction:addAction(AHOpacityTo.new(self.outOfLabel, 0.5, nil, 1.0))
    showAction:addAction(AHOpacityTo.new(self.denominatorMoneyExpression, 0.5, nil, 1.0))

  elseif (pageType == PAGE_TYPE_UNREDUCED_FRACTION) then

    if (previousPageType == nil) then
      showAction:addAction(AHOpacityTo.new(self.unreducedFractionExpression, 0.5, nil, 1.0))
    elseif (previousPageType == PAGE_TYPE_MONEY) then
      local destinationFractionExpression = self.unreducedFractionExpression
      showAction:addAction(createCombineMoneyAction(self, destinationFractionExpression))
    end

  elseif (pageType == PAGE_TYPE_FACTORED_OUT) then

    showAction:addAction(AHOpacityTo.new(self.factoredOutFractionExpression, 0.5, nil, 1.0))
    showAction:addAction(AHOpacityTo.new(self.equalsSign, 0.5, nil, 1.0))

    showAction:addAction(AHLocationXTo.new(self.unreducedFractionExpression, 0.5, AHEaseSineInOut.new(), self:getWidthPart(0.25)))
    showAction:addAction(AHLocationXTo.new(self.factoredOutFractionExpression, 0.5, AHEaseSineInOut.new(), self:getWidthPart(0.75)))

  elseif (pageType == PAGE_TYPE_CROSSED_OUT) then

    showAction:addAction(self.factoredOutFractionExpression:createStrikeAction())

  elseif (pageType == PAGE_TYPE_EQUALITY) then

    showAction:addAction(AHOpacityTo.new(self.factoredOutFractionExpression, 0.25, nil, 0.0))
    showAction:addAction(AHDelayed.new(0.25, AHLocationXTo.new(self.reducedFractionExpression, 0.25, AHEaseSineInOut.new(), self:getWidthPart(0.75))))

  elseif (pageType == PAGE_TYPE_MONEY_ALREADY_REDUCED_EQUALITY) then

    local destinationFractionExpression = self.reducedFractionExpression
    showAction:addAction(createCombineMoneyAction(self, destinationFractionExpression))

  else
    error("No actions specified for page type `" .. tostring(pageType) .. "`")
  end

  return showAction:start()
end

function FunkyPresentationFraction:getBlurbTextForPageIndex(pageIndex)
  return self.blurbTextsByPageIndex[pageIndex]
end

function FunkyPresentationFraction:playDisappear()
  local disappearAction = AHSpawn.new()

  local elements = self.elements
  for _, element in ipairs(elements) do
    local elementAction = AHOpacityTo.new(element, 0.5, nil, 0.0)
    disappearAction:addAction(elementAction)
  end

  return disappearAction:start()
end

function FunkyPresentationFraction:hide()
  local elements = self.elements
  for _, element in ipairs(elements) do
    element:setOpacity(0.0)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------


local function createPageTypes(permutation)
  local format = permutation:getFormat()
  local multiplier = permutation:getMultiplier()

  local fractionIsAlreadyReduced = ((multiplier == nil) or (multiplier <= 1))

  if (format == FunkyFractionFormat.FRACTION) then

    if (fractionIsAlreadyReduced) then
      return {PAGE_TYPE_REDUCED_FRACTION}
    else
      return {PAGE_TYPE_UNREDUCED_FRACTION, PAGE_TYPE_FACTORED_OUT, PAGE_TYPE_CROSSED_OUT, PAGE_TYPE_EQUALITY}
    end

  elseif (format == FunkyFractionFormat.MONEY) then

    if (fractionIsAlreadyReduced) then
      return {PAGE_TYPE_MONEY, PAGE_TYPE_MONEY_ALREADY_REDUCED_EQUALITY}
    else
      return {PAGE_TYPE_MONEY, PAGE_TYPE_UNREDUCED_FRACTION, PAGE_TYPE_FACTORED_OUT, PAGE_TYPE_CROSSED_OUT, PAGE_TYPE_EQUALITY}
    end

  else
    error("No action defined for fraction format `" .. tostring(format) .. "`")
  end
end

local function createReducedFractionExpression(self, fraction, shouldAddDuplicateBar)
  local reducedFractionExpression = FunkyExpressionFraction.newWithReducedFraction(fraction, FONT_SIZE)
  self:insert(reducedFractionExpression)
  self.reducedFractionExpression = reducedFractionExpression

  local equalsSign = Label.new("=", FONT)
  self:insert(equalsSign)
  self.equalsSign = equalsSign

  local duplicateBar = nil
  if (shouldAddDuplicateBar) then
    duplicateBar = reducedFractionExpression:createDuplicateBar()
    self:insert(duplicateBar)
    self.duplicateBar = duplicateBar
  end

  return {reducedFractionExpression, equalsSign, duplicateBar}
end

local function createMoneyExpressions(self, numerator, denominator, multiplier)
  local numeratorMoneyExpression = FunkyExpressionMoney.newWithVanishingDecimal(numerator, multiplier, FONT_SIZE)
  self:insert(numeratorMoneyExpression)
  self.numeratorMoneyExpression = numeratorMoneyExpression

  local outOfLabel = Label.new(Localizer.get("OUT_OF"), FONT)
  self:insert(outOfLabel)
  self.outOfLabel = outOfLabel

  local denominatorMoneyExpression = FunkyExpressionMoney.newWithVanishingDecimal(denominator, multiplier, FONT_SIZE)
  self:insert(denominatorMoneyExpression)
  self.denominatorMoneyExpression = denominatorMoneyExpression

  return {numeratorMoneyExpression, outOfLabel, denominatorMoneyExpression}
end

local function createUnreducedFractionExpression(self, numerator, denominator, multiplier)
  local numeratorText = tostring(numerator * multiplier)
  local denominatorText = tostring(denominator * multiplier)

  local unreducedFractionExpression = FunkyExpressionFraction.new(numeratorText, denominatorText, FONT_SIZE)
  self:insert(unreducedFractionExpression)
  self.unreducedFractionExpression = unreducedFractionExpression

  local duplicateBar = unreducedFractionExpression:createDuplicateBar()
  self:insert(duplicateBar)
  self.duplicateBar = duplicateBar

  return {unreducedFractionExpression, duplicateBar}
end

local function createFactoredOutFractionExpression(self, numerator, denominator, multiplier)
  local factoredOutFractionExpression = FunkyExpressionFraction.newMultiplied(numerator, denominator, multiplier, FONT_SIZE)
  self:insert(factoredOutFractionExpression)
  self.factoredOutFractionExpression = factoredOutFractionExpression

  return {factoredOutFractionExpression}
end

local function buildElements(self, permutation, pageTypes)
  local fraction = permutation:getFraction()
  local numerator = fraction:getNumerator()
  local denominator = fraction:getDenominator()

  local multiplier = permutation:getMultiplier()

  local elements = {}

  for _, pageType in ipairs(pageTypes) do
    local pageElements = nil

    if (pageType == PAGE_TYPE_REDUCED_FRACTION) then
      pageElements = createReducedFractionExpression(self, fraction)
    elseif (pageType == PAGE_TYPE_MONEY) then
      pageElements = createMoneyExpressions(self, numerator, denominator, multiplier)
    elseif (pageType == PAGE_TYPE_UNREDUCED_FRACTION) then
      pageElements = createUnreducedFractionExpression(self, numerator, denominator, multiplier)
    elseif (pageType == PAGE_TYPE_FACTORED_OUT) then
      pageElements = createFactoredOutFractionExpression(self, numerator, denominator, multiplier)
    elseif (pageType == PAGE_TYPE_CROSSED_OUT) then
      pageElements = {}
    elseif (pageType == PAGE_TYPE_EQUALITY) then
      pageElements = createReducedFractionExpression(self, fraction)
    elseif (pageType == PAGE_TYPE_MONEY_ALREADY_REDUCED_EQUALITY) then
      local shouldAddDuplicateBar = true
      pageElements = createReducedFractionExpression(self, fraction, shouldAddDuplicateBar)
    else
      error("No elements specified for page type `" .. tostring(pageType) .. "`")
    end

    TableUtils.concatenateArrays(elements, pageElements)
  end

  self.elements = elements
end

local function buildBlurbText(self, pageTypes)
  local blurbTextsByPageIndex = {}
  for _, pageType in ipairs(pageTypes) do

    local blurbText = nil

    if (pageType == PAGE_TYPE_REDUCED_FRACTION) then
      blurbText = Localizer.get("ALREADY_REDUCED")
    elseif (pageType == PAGE_TYPE_MONEY) then
      blurbText = Localizer.get("CONVERT_TO")
    elseif ((pageType == PAGE_TYPE_UNREDUCED_FRACTION) or
            (pageType == PAGE_TYPE_FACTORED_OUT) or
            (pageType == PAGE_TYPE_CROSSED_OUT) or
            (pageType == PAGE_TYPE_EQUALITY)) then
      blurbText = Localizer.get("REDUCE_FRACTION")
    elseif (pageType == PAGE_TYPE_MONEY_ALREADY_REDUCED_EQUALITY) then
      blurbText = Localizer.get("AFTER_DECIMAL")
    else
      error("No blurb text defined for page type `" .. tostring(pageType) .. "`")
    end

    table.insert(blurbTextsByPageIndex, blurbText)

  end
  self.blurbTextsByPageIndex = blurbTextsByPageIndex
end

local function buildTitleText(self, pageTypes)
  local firstPageType = pageTypes[1]
  if (firstPageType == PAGE_TYPE_MONEY) then
    self.titleText = Localizer.get("CONVERTING_MONEY")
  else
    self.titleText = Localizer.get("REDUCING_A_FRACTION")
  end
end

-- Constructor ------------------------

function FunkyPresentationFraction.new(permutation)
  local pageTypes = createPageTypes(permutation)
  local pagesCount = #pageTypes

  local self = FunkyPresentationBase.new(permutation, pagesCount)
  Mixin.mix(self, FunkyPresentationFraction)

  buildElements(self, permutation, pageTypes)
  buildBlurbText(self, pageTypes)
  buildTitleText(self, pageTypes)

  self.pageTypes = pageTypes

  return self
end
