--- FunkyPresentationImage
-- @author Created by Aaron Barrett on February 23, 2016.

class("FunkyPresentationImage")

require("FunkyPresentationBase")
require("FunkyExpressionImage")
require("FunkyExpressionFraction")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PAGES_COUNT = 2

local BLURB_TEXT = Localizer.get("CONVERT_PIE")

local PAGE_INDEX_BIG_PIE = 1
local PAGE_INDEX_CONVERTED = 2

local Z_FRACTION_EXPRESSION = 10
local Z_IMAGE_EXPRESSION = 20

local FONT_SIZE = 32.0
local FONT = Font.new(FontFaces.DIN_OT_MEDIUM, FONT_SIZE)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function prepareToShowBigPie(self)
  local imageExpression = self.imageExpression
  local fractionExpression = self.fractionExpression
  local equalsSign = self.equalsSign

  imageExpression:setOpacity(0.0)
  fractionExpression:setOpacity(0.0)
  equalsSign:setOpacity(0.0)

  imageExpression:setScl(1.0)
  imageExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
end

local function createShowBigPiePageAction(self)
  local pieAction = AHOpacityTo.new(self.imageExpression, 0.5, nil, 1.0)
  return pieAction
end

local function prepareToShowConverted(self)
  local imageExpression = self.imageExpression
  local fractionExpression = self.fractionExpression
  local equalsSign = self.equalsSign

  imageExpression:setOpacity(1.0)
  fractionExpression:setOpacity(0.0)
  equalsSign:setOpacity(0.0)

  imageExpression:setScl(1.0)

  imageExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
  fractionExpression:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
  equalsSign:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
end

local function createShowConvertedPageAction(self)
  local imageExpression = self.imageExpression
  local fractionExpression = self.fractionExpression
  local duration = 0.5
  local ease = AHEaseSineInOut.new()

  local convertedAction = AHSpawn.new(
    AHLocationXTo.new(imageExpression, duration, ease, self:getWidthPart(0.3)),
    AHLocationXTo.new(fractionExpression, duration, ease, self:getWidthPart(0.7)),
    AHScaleTo.new(imageExpression, duration, ease, 0.75),
    AHOpacityTo.new(self.equalsSign, duration, nil, 1.0),
    AHOpacityTo.new(fractionExpression, duration, nil, 1.0)
  )
  return convertedAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- FunkyPresentation Protocol ---------

function FunkyPresentationImage:getTitleText()
  return Localizer.get("REDUCING_PIE")
end

function FunkyPresentationImage:prepareToShowPage(pageIndex)
  if (pageIndex == PAGE_INDEX_BIG_PIE) then
    prepareToShowBigPie(self)
  elseif (pageIndex == PAGE_INDEX_CONVERTED) then
    prepareToShowConverted(self)
  else
    error("No page preparation provided for pageIndex `" .. tostring(pageIndex) .. "`")
  end
end

function FunkyPresentationImage:playShowPage(pageIndex)
  local showAction = nil

  if (pageIndex == PAGE_INDEX_BIG_PIE) then
    showAction = createShowBigPiePageAction(self)
  elseif (pageIndex == PAGE_INDEX_CONVERTED) then
    showAction = createShowConvertedPageAction(self)
  else
    error("No page action provided for pageIndex `" .. tostring(pageIndex) .. "`")
  end

  return showAction:start()
end

function FunkyPresentationImage:getBlurbTextForPageIndex(pageIndex)
  return BLURB_TEXT
end

function FunkyPresentationImage:playDisappear()
  local disappearAction = AHSpawn.new(
    AHOpacityTo.new(self.imageExpression, 0.5, nil, 0.0),
    AHOpacityTo.new(self.equalsSign, 0.5, nil, 0.0),
    AHOpacityTo.new(self.fractionExpression, 0.5, nil, 0.0)
  )

  return disappearAction:start()
end

function FunkyPresentationImage:hide()
  self.imageExpression:setOpacity(0.0)
  self.equalsSign:setOpacity(0.0)
  self.fractionExpression:setOpacity(0.0)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildImageExpression(self, fraction)
  local expressionSize = 90.0
  local imageExpression = FunkyExpressionImage.new(fraction, expressionSize)
  self:insert(imageExpression, Z_IMAGE_EXPRESSION)
  self.imageExpression = imageExpression
end

local function buildEqualsSign(self)
  local equalsSign = Label.new("=", FONT)
  self:insert(equalsSign)
  self.equalsSign = equalsSign
end

local function buildFractionExpression(self, fraction)
  local fractionExpression = FunkyExpressionFraction.newWithReducedFraction(fraction, FONT_SIZE)
  self:insert(fractionExpression, Z_FRACTION_EXPRESSION)
  self.fractionExpression = fractionExpression
end

-- Constructor ------------------------

function FunkyPresentationImage.new(permutation)
  local self = FunkyPresentationBase.new(permutation, PAGES_COUNT)
  Mixin.mix(self, FunkyPresentationImage)

  local fraction = permutation:getFraction()

  buildFractionExpression(self, fraction)
  buildEqualsSign(self)
  buildImageExpression(self, fraction)

  return self
end
