--- FunkyProgressMeter
-- @author Created by Aaron Barrett on March 01, 2016.

class("FunkyProgressMeter")

require("FunkyShadowedBox")
require("ExperimentsConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_DARK_LABEL = {0.23, 0.16, 0.51, 1.0}

local FONT_SIZE = 18.0
local TABLET_FONT_SIZE = 12.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function updateText(self)
  local leftLabel = self.leftLabel
  local numberLabel = self.numberLabel

  local extraSpaceToAvoidTextBug = " "
  local leftLabelText = Localizer.get("MATCHES") .. ": " .. extraSpaceToAvoidTextBug
  leftLabel:setText(leftLabelText)
  local numberLabelText = tostring(self.matchesCount)
  numberLabel:setText(numberLabelText)

  local face = self.face
  face:setWidth(leftLabel:getWidth() + numberLabel:getWidth())
  face:alignChildren()
  face:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyProgressMeter:playShrink()
  self.shrunken = true

  local padding = 16.0
  local updatedWidth = self.face:getWidth() + padding

  return self:playResize(updatedWidth)
end

function FunkyProgressMeter:incrementMatchesCount()
  self.matchesCount = self.matchesCount + 1

  updateText(self)

  if (not (self.shrunken)) then
    self:playShrink()
  end
end

function FunkyProgressMeter:getMatchesCount()
  return self.matchesCount
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLightFont(fontSize)
  return Font.new(FontFaces.DIN_OT_LIGHT, fontSize)
end

local function createBoldFont(fontSize)
  return PrerenderedFont.new(FontFaces.DIN_OT_MEDIUM, fontSize)
end

local function createLeftLabel(font)
  local text = Localizer.get("MATCH_EQUIVALENT")
  return Label.new(text, font)
end

local function createNumberLabel(font)
  local text = ""
  return Label.new(text, font)
end

local function createFace(leftLabel, numberLabel)
  local face = Row.new()

  face:insert(leftLabel)
  face:insert(numberLabel)
  face:alignChildren()

  leftLabel:inheritColor(face)
  numberLabel:inheritColor(face)

  face:setColor(unpack(COLOR_DARK_LABEL))

  return face
end

-- Constructor ------------------------

function FunkyProgressMeter.new()
  local fontSize = FONT_SIZE
  if Screen.isTabletAspectRatio() then
    fontSize = TABLET_FONT_SIZE
  end
  local lightFont = createLightFont(fontSize)
  local boldFont = createBoldFont(fontSize)

  local leftLabel = createLeftLabel(lightFont)
  local numberLabel = createNumberLabel(boldFont)
  local face = createFace(leftLabel, numberLabel)

  local shadowFilename = "shadow.png"
  local self = FunkyShadowedBox.new(face, shadowFilename)
  Mixin.mix(self, FunkyProgressMeter)

  self.matchesCount = 0

  self.leftLabel = leftLabel
  self.numberLabel = numberLabel
  self.face = face

  self.shrunken = false

  local maxWidth = Screen.safeWidth * 0.95
  if (self:getWidth() > maxWidth) then
    self:setScl(maxWidth / self:getWidth())
  end

  return self
end
