--- FunkyRibbon
-- @author Created by Aaron Barrett on February 18, 2016.

class("FunkyRibbon")

require("FunkyRibbonSegmentBase")
require("FunkyRibbonSegmentLine")
require("FunkyRibbonSegmentArc")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

FunkyRibbon.SEGMENT_TYPE_LINE = "FunkyRibbon.SEGMENT_TYPE_LINE"
FunkyRibbon.SEGMENT_TYPE_ARC = "FunkyRibbon.SEGMENT_TYPE_ARC"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyRibbon:getBandStartProgress()
  return self.bandStartProgress
end

function FunkyRibbon:setBandStartProgress(bandStartProgress)
  self.bandStartProgress = bandStartProgress
end

function FunkyRibbon:getBandEndProgress()
  return self.bandEndProgress
end

function FunkyRibbon:setBandEndProgress(bandEndProgress)
  self.bandEndProgress = bandEndProgress
end

function FunkyRibbon:createBandStartProgressToAction(duration, ease, amount)
  return AHNonAttributeTo.new(self, duration, self.getBandStartProgress, self.setBandStartProgress, ease, amount)
end

function FunkyRibbon:createBandEndProgressToAction(duration, ease, amount)
  return AHNonAttributeTo.new(self, duration, self.getBandEndProgress, self.setBandEndProgress, ease, amount)
end

function FunkyRibbon:getRelativeIntervalOfSegmentAtIndex(segmentIndex)
  local segment = self.segments[segmentIndex]
  return segment:getRelativeInterval()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createSegment(segmentDescriptor, startPoint, startAngle, bandThickness)
  local segmentType = segmentDescriptor.segmentType
  if (segmentType == FunkyRibbon.SEGMENT_TYPE_LINE) then
    local length = segmentDescriptor.length
    return FunkyRibbonSegmentLine.new(startPoint, startAngle, length, bandThickness)
  elseif (segmentType == FunkyRibbon.SEGMENT_TYPE_ARC) then
    local arcAngle = segmentDescriptor.arcAngle
    local radius = segmentDescriptor.radius
    return FunkyRibbonSegmentArc.new(startPoint, startAngle, arcAngle, radius, bandThickness)
  else
    error("No method defined to create segment for segmentType: `" .. tostring(segmentType) .. "`")
  end
end

local function buildSegments(self, startPoint, startAngle, segmentDescriptors, bandThickness)
  local segmentStartPoint = startPoint
  local segmentStartAngle = startAngle

  local segments = {}
  for _, segmentDescriptor in ipairs(segmentDescriptors) do
    local segment = createSegment(segmentDescriptor, segmentStartPoint, segmentStartAngle, bandThickness)
    segmentStartPoint = segment:getEndPoint()
    segmentStartAngle = segment:getEndAngle()
    table.insert(segments, segment)
  end

  self.segments = segments
end

local function buildIntervals(self)
  local totalLength = 0.0
  for _, segment in ipairs(self.segments) do
    totalLength = totalLength + segment:getLength()
  end

  local progress = 0.0
  for _, segment in ipairs(self.segments) do
    local startProgress = progress
    progress = progress + segment:getLength() / totalLength
    segment:configureRibbonInterval(startProgress, progress)
  end
end

-- Constructor ------------------------

function FunkyRibbon.new(width, height, startPoint, startAngle, segmentDescriptors, bandThickness, color)
  local self = Group.newCustomDraw(width, height, color)
  Mixin.mix(self, FunkyRibbon)

  buildSegments(self, startPoint, startAngle, segmentDescriptors, bandThickness)
  buildIntervals(self)

  self.bandStartProgress = 0.0
  self.bandEndProgress = 1.0

  local drawCallback = function()
    local coordinates = {}
    for _, segment in ipairs(self.segments) do
      segment:appendCoordinatesAlongRibbonInterval(coordinates, self.bandStartProgress, self.bandEndProgress)
    end
    MOAIDraw.fillStrip(coordinates)
  end
  self.deck:setDrawCallback(drawCallback)

  return self
end
