--- FunkyRibbonSegmentArc
-- @author Created by Aaron Barrett on February 19, 2016.

class("FunkyRibbonSegmentArc")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DIVISIONS_COUNT_IN_FULL_CIRCLE = 48

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- FunkyRibbonSegment protocol --------

function FunkyRibbonSegmentArc:appendCoordinatesAlongInterval(coordinates, startProgress, endProgress)
  local interval = endProgress - startProgress
  local divisionsCount = math.max(math.ceil(math.abs(interval * self.divisionsCountInFullArc)), 2)

  for pointIndex = 1, divisionsCount + 1 do
    local interpolatedProgress = startProgress + interval * (pointIndex - 1) / divisionsCount
    local angle = self.angleFromCenter + self.arcAngle * interpolatedProgress

    table.insert(coordinates, self.centerPoint.x + math.cos(angle) * self.outerRadius)
    table.insert(coordinates, self.centerPoint.y + math.sin(angle) * self.outerRadius)

    table.insert(coordinates, self.centerPoint.x + math.cos(angle) * self.innerRadius)
    table.insert(coordinates, self.centerPoint.y + math.sin(angle) * self.innerRadius)
  end
end

function FunkyRibbonSegmentArc:getEndPoint()
  return self.endPoint
end

function FunkyRibbonSegmentArc:getEndAngle()
  return self.endAngle
end

function FunkyRibbonSegmentArc:getLength()
  return self.length
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FunkyRibbonSegmentArc.new(startPoint, startAngle, arcAngle, radius, bandThickness)
  local self = FunkyRibbonSegmentBase.new()
  Mixin.mix(self, FunkyRibbonSegmentArc)

  self.startAngle = startAngle
  self.arcAngle = arcAngle
  self.length = math.abs(arcAngle * radius)

  local angleToCenter = nil
  if (arcAngle > 0.0) then
    angleToCenter = startAngle + math.rad(90.0)
  else
    angleToCenter = startAngle - math.rad(90.0)
  end
  self.centerPoint = {
    x = startPoint.x + math.cos(angleToCenter) * radius,
    y = startPoint.y + math.sin(angleToCenter) * radius
  }

  local halfBandThickness = bandThickness * 0.5
  self.innerRadius = radius - halfBandThickness
  self.outerRadius = radius + halfBandThickness

  local endAngle = startAngle + arcAngle
  local angleFromCenter = angleToCenter + math.rad(180.0)
  local angleToEndPoint = angleFromCenter + arcAngle
  self.endPoint = {
    x = self.centerPoint.x + math.cos(angleToEndPoint) * radius,
    y = self.centerPoint.y + math.sin(angleToEndPoint) * radius
  }
  self.endAngle = endAngle
  self.angleFromCenter = angleFromCenter

  self.divisionsCountInFullArc = math.ceil(DIVISIONS_COUNT_IN_FULL_CIRCLE * arcAngle / (math.pi * 2.0))

  return self
end
