--- FunkyRibbonSegmentLine
-- @author Created by Aaron Barrett on February 19, 2016.

class("FunkyRibbonSegmentLine")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function appendInnerAndOuterCoordinatesAtProgress(self, coordinates, progress)
  local multipliedLength = self.length * progress

  local interpolatedX = self.startPoint.x + self.xComponent * multipliedLength
  local interpolatedY = self.startPoint.y + self.yComponent * multipliedLength

  table.insert(coordinates, interpolatedX + self.offsetX)
  table.insert(coordinates, interpolatedY + self.offsetY)

  table.insert(coordinates, interpolatedX - self.offsetX)
  table.insert(coordinates, interpolatedY - self.offsetY)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- FunkyRibbonSegment protocol --------

function FunkyRibbonSegmentLine:appendCoordinatesAlongInterval(coordinates, startProgress, endProgress)
  appendInnerAndOuterCoordinatesAtProgress(self, coordinates, startProgress)
  appendInnerAndOuterCoordinatesAtProgress(self, coordinates, endProgress)
end

function FunkyRibbonSegmentLine:getEndPoint()
  return self.endPoint
end

function FunkyRibbonSegmentLine:getEndAngle()
  return self.endAngle
end

function FunkyRibbonSegmentLine:getLength()
  return self.length
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FunkyRibbonSegmentLine.new(startPoint, startAngle, length, bandThickness)
  local self = FunkyRibbonSegmentBase.new()
  Mixin.mix(self, FunkyRibbonSegmentLine)

  self.startPoint = startPoint
  self.endAngle = startAngle
  self.length = length

  local xComponent = math.cos(startAngle)
  local yComponent = math.sin(startAngle)

  self.endPoint = {
    x = startPoint.x + xComponent * length,
    y = startPoint.y + yComponent * length
  }

  self.xComponent = xComponent
  self.yComponent = yComponent

  local halfBandThickness = bandThickness * 0.5

  local outerAngle = startAngle + math.rad(90.0)
  self.offsetX = math.cos(outerAngle) * halfBandThickness
  self.offsetY = math.sin(outerAngle) * halfBandThickness

  return self
end
