--- FunkyShadowedBox
-- @author Created by Aaron Barrett on February 29, 2016.

class("FunkyShadowedBox")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local VERTICAL_PADDING = 16.0
local HORIZONTAL_PADDING = 24.0
local CORNER_RADIUS = 2.5

local SHADOW_PADDING = 10.0

local COLOR_WHITE = {1.0, 1.0, 1.0, 1.0}
local COLOR_SHADOW = {0.0, 0.0, 0.0, 0.25}
local COLOR_PURPLE = {0.79, 0.37, 0.96, 1.0}

local Z_SHADOW = 10
local Z_BACKGROUND = 20
local Z_FACE_GROUP = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyShadowedBox:playTurnClear()
  local ease = AHEaseExponentialOut.new()
  local turnClearAction = AHSpawn.new(
    AHOpacityTo.new(self.background, 0.9, ease, 0.1),
    AHOpacityTo.new(self.shadow, 0.9, ease, 0.0),
    AHColorTo.new(self.faceGroup, 0.9, ease, COLOR_WHITE)
  )

  return turnClearAction:start()
end

function FunkyShadowedBox:playTurnPurple()
  local ease = AHEaseExponentialOut.new()
  local turnPurpleAction = AHSpawn.new(
    AHOpacityTo.new(self.background, 0.9, ease, 0.0),
    AHOpacityTo.new(self.shadow, 0.9, ease, 0.0),
    AHColorTo.new(self.faceGroup, 0.9, ease, COLOR_PURPLE)
  )

  return turnPurpleAction:start()
end

function FunkyShadowedBox:playDisappear(duration)
  local disappearAction = AHSequence.new(
    AHOpacityTo.new(self.background, duration, nil, 0.0),
    AHOpacityTo.new(self.shadow, duration, nil, 0.0),
    AHOpacityTo.new(self.faceGroup, duration, nil, 0.0),
    AHRemove.new(self)
  )
  return disappearAction:start()
end

function FunkyShadowedBox:playResize(updatedWidth)
  local updatedScaleX =  updatedWidth / self:getWidth()

  local shadow = self.shadow
  local updatedShadowWidth = updatedWidth + SHADOW_PADDING
  local updatedShadowScaleX = updatedShadowWidth / shadow:getWidth()

  local ease = AHEaseExponentialOut.new()
  local resizeAction = AHSpawn.new(
    AHScaleXTo.new(self.background, 0.75, ease, updatedScaleX),
    AHScaleXTo.new(shadow, 0.75, ease, updatedShadowScaleX)
  )

  return resizeAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FunkyShadowedBox.new(faceGroup, shadowFilename)
  local width = faceGroup:getWidth() + HORIZONTAL_PADDING
  local height = faceGroup:getHeight() + VERTICAL_PADDING

  local self = Group.new(width, height)
  Mixin.mix(self, FunkyShadowedBox)

  local shadow = Sprite.newWithName(shadowFilename)
  shadow:setColor(unpack(COLOR_SHADOW))
  shadow:setEffectiveDims(width + SHADOW_PADDING, height + SHADOW_PADDING)
  self:insertAtCenter(shadow, Z_SHADOW)
  self.shadow = shadow

  local background = FilledRoundedRectangle.new(width, height, CORNER_RADIUS, COLOR_WHITE)
  self:insertAtCenter(background, Z_BACKGROUND)
  self.background = background

  self:insertAtCenter(faceGroup, Z_FACE_GROUP)
  self.faceGroup = faceGroup

  return self
end
