--- FunkyTipItemsOverlay
-- @author Created by Aaron Barrett on March 02, 2016.

class("FunkyTipItemsOverlay")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MOVE_TO_TOP_Y = Screen.getSafeY(0.79)

local Z_INEQUALITY_SIGN = 10
local Z_DUPLICATE_ITEM = 20
local OPACITY_FADED = 0.36

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playHighlightItem(self, itemToHighlight, itemToDim)
  local inequalitySign = self.inequalitySign
  local highlightAction = AHSequence.new(
    AHSpawn.new(
      itemToDim:createDimAtTopAction(),
      AHOpacityTo.new(inequalitySign, 0.5, nil, 0.0)
    ),
    AHSpawn.new(
      itemToHighlight:createHighlightAtTopAction(),
      AHOpacityTo.new(inequalitySign, 0.5, nil, OPACITY_FADED)
    )
  )

  return highlightAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyTipItemsOverlay:getCaptionOffset()
  return self.leftDuplicateItem:getWidthPart(0.39)
end

function FunkyTipItemsOverlay:getPermutations()
  return self.permutations
end

function FunkyTipItemsOverlay:playAppear()
  local xOffset = self:getCaptionOffset()
  local centerX = Screen.getSafeX(0.5)
  local leftDestinationX = centerX - xOffset
  local rightDestinationX = centerX + xOffset

  local destinationY = Screen.getSafeY(0.55)

  self.leftDuplicateItem:playRollTo(leftDestinationX, destinationY, -45.0)
  self.rightDuplicateItem:playRollTo(rightDestinationX, destinationY, 45.0)
end

function FunkyTipItemsOverlay:playDisappear()
  local duration = 0.5
  self.leftDuplicateItem:playDisappear(duration)
  self.rightDuplicateItem:playDisappear(duration)

  local fadeOutAction = AHOpacityTo.new(self.inequalitySign, 0.5, nil, 0.0)
  return fadeOutAction:start()
end

function FunkyTipItemsOverlay:playMoveToTop()
  local leftDestinationFaceOpacity = 1.0
  local rightDestinationFaceOpacity = OPACITY_FADED

  self.leftDuplicateItem:playMoveToTop(Screen.getSafeX(0.32), MOVE_TO_TOP_Y, leftDestinationFaceOpacity)
  self.rightDuplicateItem:playMoveToTop(Screen.getSafeX(0.68), MOVE_TO_TOP_Y, rightDestinationFaceOpacity)

  local fadeInAction = AHDelayed.new(0.25,
    AHOpacityTo.new(self.inequalitySign, 0.2, nil, OPACITY_FADED)
  )
  return fadeInAction:start()
end

function FunkyTipItemsOverlay:playHighlightLeftItem()
  return playHighlightItem(self, self.leftDuplicateItem, self.rightDuplicateItem)
end

function FunkyTipItemsOverlay:playHighlightRightItem()
  return playHighlightItem(self, self.rightDuplicateItem, self.leftDuplicateItem)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildInequalitySign(self)
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 48.0)
  local equalSignLabel = Label.new("=", font)

  local slashWidth = ReferenceDevice.getScaledWidth(2.0)
  local slashHeight = 0.6 * MathUtils.distance(0.0, 0.0, equalSignLabel:getWidth(), equalSignLabel:getHeight())
  local slash = FilledRectangle.new(slashWidth, slashHeight)
  slash:setRot(-45.0)

  local inequalitySign = Group.new(equalSignLabel:getWidth(), equalSignLabel:getHeight())
  local equalSignLabelY = inequalitySign:getHeightPart(0.6)
  equalSignLabel:setLoc(inequalitySign:getWidthPart(0.5), equalSignLabelY)
  inequalitySign:insert(equalSignLabel)
  inequalitySign:insertAtCenter(slash)

  equalSignLabel:inheritOpacity(inequalitySign)
  slash:inheritOpacity(inequalitySign)
  inequalitySign:setOpacity(0.0)

  inequalitySign:setLoc(Screen.getSafeX(0.5), MOVE_TO_TOP_Y)
  self:insert(inequalitySign, Z_INEQUALITY_SIGN)
  self.inequalitySign = inequalitySign
end

local function getLeftmostItem(firstItem, secondItem)
  if (firstItem:getX() < secondItem:getX()) then
    return firstItem
  else
    return secondItem
  end
end

local function getConvertedLocationOfItem(item)
  local worldX, worldY = item:modelToWorld(item:getWidthPart(0.5), item:getHeightPart(0.5))
  return {x = worldX, y = worldY}
end

local function buildDuplicateItems(self, firstItem, secondItem)
  local leftItem = getLeftmostItem(firstItem, secondItem)
  local rightItem = nil
  if (leftItem == firstItem) then
    rightItem = secondItem
  else
    rightItem = firstItem
  end

  local leftPermutation = leftItem:getPermutation()
  local rightPermutation = rightItem:getPermutation()

  local leftDuplicateItem = FunkyItem.new(leftPermutation)
  leftDuplicateItem:jumpToWrongState()
  local leftConvertedLocation = getConvertedLocationOfItem(leftItem)
  leftDuplicateItem:setLoc(leftConvertedLocation.x, leftConvertedLocation.y)
  self:insert(leftDuplicateItem, Z_DUPLICATE_ITEM)

  local rightDuplicateItem = FunkyItem.new(rightPermutation)
  rightDuplicateItem:jumpToWrongState()
  local rightConvertedLocation = getConvertedLocationOfItem(rightItem)
  rightDuplicateItem:setLoc(rightConvertedLocation.x, rightConvertedLocation.y)
  self:insert(rightDuplicateItem, Z_DUPLICATE_ITEM)

  self.leftDuplicateItem = leftDuplicateItem
  self.rightDuplicateItem = rightDuplicateItem
  self.permutations = {leftPermutation, rightPermutation}
end

-- Constructor ------------------------

function FunkyTipItemsOverlay.new(firstItem, secondItem)
  local self = Group.newFullScreen()
  Mixin.mix(self, FunkyTipItemsOverlay)

  buildInequalitySign(self)
  buildDuplicateItems(self, firstItem, secondItem)

  return self
end
