--- FunkyTipPanel
-- @author Created by Aaron Barrett on February 23, 2016.

class("FunkyTipPanel")

require("FunkyFractionFormat")
require("FunkyPresentationImage")
require("FunkyPresentationFraction")
require("FunkyPresentationPercentage")
require("FunkyTipItemsOverlay")
require("FunkyTipBasicOverlay")
require("FunkyTipAdvancedOverlay")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {0.0, 0.0, 0.0}

local Z_BACKGROUND = 10
local Z_ITEMS_OVERLAY = 20
local Z_BASIC_OVERLAY = 30
local Z_ADVANCED_OVERLAY = 40

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyTipPanel:playAppear()
  self.basicOverlay:playAppear()
  self.itemsOverlay:playAppear()

  local appearAction = AHOpacityTo.new(self.background, 0.5, nil, 1.0)
  self.actionInterrupt:insert(appearAction)
end

function FunkyTipPanel:playDisappear()
  self.itemsOverlay:playDisappear()

  local disappearAction = AHSequence.new(
    AHOpacityTo.new(self.background, 0.5, nil, 0.0),
    function()
      self:removeSelf()
      self.dismissed = true
    end
  )
  self.actionInterrupt:insert(disappearAction)
end

function FunkyTipPanel:isDismissed()
  return self.dismissed
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildBackground(self)
  local background = FilledRectangle.newFullScreen(BACKGROUND_COLOR)
  background:setOpacity(0.0)
  self:insertAtCenter(background, Z_BACKGROUND)
  self.background = background
end

local function buildItemsOverlay(self, firstItem, secondItem)
  local itemsOverlay = FunkyTipItemsOverlay.new(firstItem, secondItem)
  self:insertAtCenter(itemsOverlay, Z_ITEMS_OVERLAY)
  self.itemsOverlay = itemsOverlay
  local permutations = itemsOverlay:getPermutations()
  self.leftPermutation = permutations[1]
  self.rightPermutation = permutations[2]
end

local function buildBasicOverlay(self, onPanelDismissedCallback, onMovedToAdvancedOverlayCallback)
  local captionOffset = self.itemsOverlay:getCaptionOffset()
  local basicOverlay = FunkyTipBasicOverlay.new(self.leftPermutation, self.rightPermutation, captionOffset, onPanelDismissedCallback, onMovedToAdvancedOverlayCallback)
  self:insertAtCenter(basicOverlay, Z_BASIC_OVERLAY)
  self.basicOverlay = basicOverlay
end

local function buildAdvancedOverlay(self, onPanelDismissedCallback)
  local function highlightLeftItemCallback()
    self.itemsOverlay:playHighlightLeftItem()
  end

  local function highlightRightItemCallback()
    self.itemsOverlay:playHighlightRightItem()
  end

  local advancedOverlay = FunkyTipAdvancedOverlay.new(self.leftPermutation, self.rightPermutation, onPanelDismissedCallback, highlightLeftItemCallback, highlightRightItemCallback)
  self:insertAtCenter(advancedOverlay, Z_ADVANCED_OVERLAY)
  self.advancedOverlay = advancedOverlay
end

-- Constructor ------------------------

function FunkyTipPanel.new(firstItem, secondItem)
  local self = Group.newFullScreen()
  Mixin.mix(self, FunkyTipPanel)

  local function onPanelDismissedCallback()
    self:playDisappear()
  end

  local function onMovedToAdvancedOverlayCallback()
    buildAdvancedOverlay(self, onPanelDismissedCallback)
    self.advancedOverlay:playAppear()
    self.itemsOverlay:playMoveToTop()
  end

  buildBackground(self)
  buildItemsOverlay(self, firstItem, secondItem)
  buildBasicOverlay(self, onPanelDismissedCallback, onMovedToAdvancedOverlayCallback)

  self.actionInterrupt = ActionInterrupt.new()
  self.dismissed = false

  return self
end
