--- FunkyValidator
-- @author Created by Aaron Barrett on February 01, 2016.
-- ConceptValidator for Funky.

class("FunkyValidator")

require("validators.ConceptValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'numerator',
  'denominator',
  'format',
  'multiplier'
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validateNonzeroDenominator(self, concept)
  local denominator = concept.denominator
  local denominatorIsZero = (denominator == 0)

  self:expectFalse(denominatorIsZero, "Concept has a denominator that is zero.")
end

local function validateLowestTerms(self, concept)
  local numerator = concept.numerator
  local denominator = concept.denominator
  local greatestCommonDivisor = MathUtils.greatestCommonDivisor(numerator, denominator)
  local greatestCommonDivisorIsOne = (greatestCommonDivisor == 1)

  self:expectTrue(greatestCommonDivisorIsOne, "Concept is not in lowest terms.")
end

local function validateMoneyMultiplierIsMultipleOfFive(self, concept)
  local format = concept.format
  local multiplier = concept.multiplier
  local isMoney = format == "money"

  if (isMoney and (multiplier ~= 1)) then
    local isMultipleOfFive = (multiplier % 5 == 0)
    self:expectTrue(isMultipleOfFive, "Concept with format `money` is not a multiple of five.")
  end
end

local function validateFractionIsNumber(self, concept)
  local numerator = concept.numerator
  local denominator = concept.denominator

  local fractionIsNumber = ((type(numerator) == "number") and (type(denominator) == "number"))
  local numeratorIsWholeNumber = (MathUtils.isInteger(numerator) and (denominator > 0))
  local denominatorIsWholeNumber = (MathUtils.isInteger(denominator) and (numerator > 0))

  local fractionIsNumberPositiveNumber = (fractionIsNumber and numeratorIsWholeNumber and denominatorIsWholeNumber)
  self:expectTrue(fractionIsNumberPositiveNumber, "Concept has numerator or denominator that is not a whole number.")
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FunkyValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)
  validateNonzeroDenominator(self, concept)
  validateLowestTerms(self, concept)
  validateMoneyMultiplierIsMultipleOfFive(self, concept)
  validateFractionIsNumber(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function FunkyValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, FunkyValidator)

  return self
end
