--- GoodsAdvanceButton
-- @author Created by Aaron Barrett on July 27, 2021.

class("GoodsAdvanceButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ADVANCE_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 15.0)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getOpacityForPressedState(self, pressed)
  if (pressed) then
    return 0.5
  else
    return 0.75
  end
end

local function setPressed(self, pressed)
  local opacity = getOpacityForPressedState(self, pressed)
  self.label:setOpacity(opacity)
end

local function playPush(self)
  setPressed(self, true)
  return AHDelayed.new(0.1, function()
    setPressed(self, false)
  end):start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsAdvanceButton:getLabelHeight()
  return self.label:getHeight()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GoodsAdvanceButton.new(text, onTouchCallback)
  local label = Label.new(text, ADVANCE_FONT)

  local touchTargetWidth = label:getWidth() * 1.5
  local touchTargetHeight = label:getHeight() * 3.0

  local self = Group.new(touchTargetWidth, touchTargetHeight)
  Mixin.mix(self, GoodsAdvanceButton)

  self.onTouchCallback = onTouchCallback

  self:insertAtCenter(label)
  label:inheritColor(self)
  self.label = label

  Button.makeButton(self, function()
    self.onTouchCallback()
    playPush(self)
  end)

  setPressed(self, false)

  return self
end
