--- GoodsArrowButton
-- @author Created by Aaron Barrett on July 12, 2021.

class("GoodsArrowButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_BASE = 10
local Z_CHEVRON = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setPressed(self, pressed)
  if ((self.onTouchCallback == nil) or pressed) then
    self.base:setOpacity(0.1)
    self.chevron:setOpacity(0.2)
  else
    self.base:setOpacity(0.2)
    self.chevron:setOpacity(1.0)
  end
end

local function playPush(self)
  setPressed(self, true)
  return AHDelayed.new(0.1, function()
    setPressed(self, false)
  end):start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GoodsArrowButton.new(onTouchCallback)
  local base = Sprite.newWithName("arrow_button_base.png")
  local chevron = Sprite.newWithName("arrow_button_chevron.png")

  local touchTargetSizeFactor = 2.0
  local width = base:getWidth() * touchTargetSizeFactor
  local height = base:getHeight() * touchTargetSizeFactor

  local self = Group.new(width, height)
  Mixin.mix(self, GoodsArrowButton)

  self:insertAtCenter(base, Z_BASE)
  self:insertAtCenter(chevron, Z_CHEVRON)
  self:linkColorAttrToChildren()

  self.base = base
  self.chevron = chevron
  self.onTouchCallback = onTouchCallback

  if (onTouchCallback ~= nil) then
    Button.makeButton(self, function()
      self.onTouchCallback()
      playPush(self)
    end)
  end

  setPressed(self, false)

  return self
end
