--- GoodsConfiguration
-- @author Created by Aaron Barrett on July 10, 2014.

class("GoodsConfiguration")
require("GameSession")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

GoodsConfiguration.POINTS_PER_CORRECT_RESPONSE = 400
GoodsConfiguration.POINTS_PER_LIVES_BONUS = 1000
GoodsConfiguration.INITIAL_LIVES_COUNT = 4
GoodsConfiguration.POINTS_PER_FULL_SPEED_BONUS = 200
GoodsConfiguration.INITIAL_COUNTDOWN_DURATION = 11.0
GoodsConfiguration.ADDITIONAL_COUNTDOWN_DURATION_PER_ROUND = 4.0

local ITEMS_COUNT_FOR_RANK_TWO = 16
local MAX_ITEMS_PER_GAME = 20
local EXTRA_LIVES_NEEDED_RANK_TWO = 2
local EXTRA_LIVES_NEEDED_RANK_THREE = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsConfiguration.getRankTwoScore()
  local pointsForItems = ITEMS_COUNT_FOR_RANK_TWO * GoodsConfiguration.POINTS_PER_CORRECT_RESPONSE
  local pointsForExtraLives = EXTRA_LIVES_NEEDED_RANK_TWO * GoodsConfiguration.POINTS_PER_LIVES_BONUS
  local rankTwoScore = pointsForItems + pointsForExtraLives

  return rankTwoScore
end

function GoodsConfiguration.getRankThreeScore()
  local pointsForItems = MAX_ITEMS_PER_GAME * GoodsConfiguration.POINTS_PER_CORRECT_RESPONSE
  local pointsForExtraLives = EXTRA_LIVES_NEEDED_RANK_THREE * GoodsConfiguration.POINTS_PER_LIVES_BONUS
  local rankThreeScore = pointsForItems + pointsForExtraLives

  return rankThreeScore
end

function GoodsConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Easy = 1.0}},

    {key = 0.075, valueProbabilities = {Easy = 0.95,
                                        Medium = 0.05}},

    {key = 0.15, valueProbabilities = {Easy = 0.85,
                                       Medium = 0.15}},

    {key = 0.225, valueProbabilities = {Easy = 0.75,
                                        Medium = 0.25}},

    {key = 0.3, valueProbabilities = {Easy = 0.55,
                                      Medium = 0.45}},

    {key = 0.375, valueProbabilities = {Easy = 0.4,
                                        Medium = 0.6}},

    {key = 0.45, valueProbabilities = {Easy = 0.25,
                                       Medium = 0.75}},

    {key = 0.525, valueProbabilities = {Easy = 0.1,
                                        Medium = 0.9}},

    {key = 0.6, valueProbabilities = {Medium = 1.0}},

    {key = 0.675, valueProbabilities = {Medium = 0.75,
                                        Hard = 0.25}},

    {key = 0.75, valueProbabilities = {Medium = 0.5,
                                       Hard = 0.5}},

    {key = 0.8, valueProbabilities = {Medium = 0.2,
                                      Hard = 0.8}},

    {key = 0.9, valueProbabilities = {Medium = 0.1,
                                      Hard = 0.9}},

    {key = 1.0, valueProbabilities = {Hard = 1.0}},
  }
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------
