--- GoodsCountdown
-- @author Created by Aaron Barrett on June 12, 2014.

class("GoodsCountdown")
require("graphics/MultiGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.height

local CENTER_X = WIDTH * 0.5

-- Band Colors ------------------------

local FILLING_COLORS = {
  {0.00, 0.72, 0.93, 0.00},
  {0.00, 0.72, 0.93, 0.30},
  {0.27, 0.63, 0.87, 0.70},
  {0.36, 0.50, 0.92, 0.90},
  {0.53, 0.24, 1.00, 1.00}
}

local FILLING_THICKNESSES = {
  0.2,
  0.4,
  1.0,
  1.8
}

-- Z-indices --------------------------

local Z_FILLING = 10

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Timer ------------------------------

function GoodsCountdown:restartWithDuration(duration)
  local onExpirationCallback = self.onExpirationCallback

  local countdownAction = AHSequence.new(
                                         AHScaleYTo.new(self.filling, duration, nil, 0.0),
                                         onExpirationCallback
                                         )
  self.currentAction = countdownAction

  countdownAction:start()
  self.currentTotalDuration = duration
end

function GoodsCountdown:stop()
  if (self.currentAction) then
    self.currentAction:stop()
    self.currentAction = nil
  end
end

function GoodsCountdown:playRefill()
  local refillAction = AHScaleYTo.new(self.filling, 1.0, AHEaseSimpleInOut.new(2.0), 1.0)

  return refillAction:start()
end

function GoodsCountdown:fluster()
  self.currentAction:stop()

  local filling = self.filling
  local _, yScale = filling:getScl()

  local flusterAction = AHScaleYTo.new(filling, 1.0, AHEaseElasticOut.new(0.12), yScale + 0.05)
  self.currentAction = flusterAction
  flusterAction:start()
end

function GoodsCountdown:resumeAfterIncorrect()
  local onExpirationCallback = self.onExpirationCallback
  local currentTotalDuration = self.currentTotalDuration

  local filling = self.filling
  local _, yScale = filling:getScl()

  local remainingDuration = currentTotalDuration * yScale

  local countdownAction = AHSequence.new(
                                         AHScaleYTo.new(filling, remainingDuration, nil, 0.0),
                                         onExpirationCallback
                                         )
  self.currentAction = countdownAction

  countdownAction:start()
end

function GoodsCountdown:isTimedOut()
  local filling = self.filling
  local _, yScale = filling:getScl()
  return MathUtils.round(yScale, 4) == 0
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GoodsCountdown.new(onExpirationCallback)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, GoodsCountdown)

  local isHorizontal = false
  local fillingWidth = WIDTH
  local fillingHeight = HEIGHT * 1.1
  local filling = MultiGradient.new(fillingWidth, fillingHeight, isHorizontal, FILLING_COLORS, FILLING_THICKNESSES)

  filling:setAnchorPointBottom()
  filling:setLoc(CENTER_X, 0.0)
  self:insert(filling, Z_FILLING)
  self.filling = filling

  self.currentAction = nil
  self.currentTotalDuration = -1.0
  self.onExpirationCallback = onExpirationCallback

  return self
end
