--- GoodsExampleLinesBuilder
-- @author Created by Aaron Barrett on July 27, 2021.

class("GoodsExampleLinesBuilder")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FORMAT_STRING_TEN_PERCENT = "10%% of %s = %s"
local FORMAT_STRING_DIVIDE_BY_TWO = "%s ÷ 2 = %s"
local FORMAT_STRING_SUM_VALUES = "%s + %s = %s"
local FORMAT_STRING_MULTIPLY_BY_TWO = "%s × 2 = %s"
local FORMAT_STRING_MULTIPLY_BY_THREE = "%s × 3 = %s"
local FORMAT_STRING_SUBTRACT_VALUE = "%s - %s = %s"
local FORMAT_STRING_DIVIDE_BY_FOUR = "%s ÷ 4 = %s"
local FORMAT_STRING_DIVIDE_BY_THREE = "%s ÷ 3 = %s"
local FORMAT_STRING_MULTIPLY_BY_FOUR = "%s × 4 = %s"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getLineText(self, formatString, decimalSubstitutions)
  local currencySubstitutions = TableUtils.map(decimalSubstitutions, GoodsCurrencyFormatter.convertDecimalToCurrency)
  return string.format(formatString, unpack(currencySubstitutions))
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsExampleLinesBuilder.getLines(basePrice, percentageAsInteger)
  assert(MathUtils.isInteger(percentageAsInteger), "Percentages values must be integers between 1 and 99.")
  if (percentageAsInteger == 5) then
    local tenPercentOfBasePrice = basePrice * 0.10
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_DIVIDE_BY_TWO, {tenPercentOfBasePrice, tenPercentOfBasePrice / 2})
    }
  elseif (percentageAsInteger == 10) then
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, basePrice * 0.10})
    }
  elseif (percentageAsInteger == 15) then
    local tenPercentOfBasePrice = basePrice * 0.10
    local fivePercentOfBasePrice = tenPercentOfBasePrice / 2
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_DIVIDE_BY_TWO, {tenPercentOfBasePrice, fivePercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_SUM_VALUES, {tenPercentOfBasePrice, fivePercentOfBasePrice, tenPercentOfBasePrice + fivePercentOfBasePrice})
    }
  elseif (percentageAsInteger == 20) then
    local tenPercentOfBasePrice = basePrice * 0.10
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_MULTIPLY_BY_TWO, {tenPercentOfBasePrice, basePrice * 0.10 * 2})
    }
  elseif (percentageAsInteger == 25) then
    return {
      getLineText(self, FORMAT_STRING_DIVIDE_BY_FOUR, {basePrice, basePrice / 4})
    }
  elseif (percentageAsInteger == 30) then
    local tenPercentOfBasePrice = basePrice * 0.10
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_MULTIPLY_BY_THREE, {tenPercentOfBasePrice, tenPercentOfBasePrice * 3})
    }
  elseif (percentageAsInteger == 33) then
    return {
      getLineText(self, FORMAT_STRING_DIVIDE_BY_THREE, {basePrice, basePrice / 3})
    }
  elseif (percentageAsInteger == 40) then
    local tenPercentOfBasePrice = basePrice * 0.10
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_MULTIPLY_BY_FOUR, {tenPercentOfBasePrice, tenPercentOfBasePrice * 4})
    }
  elseif (percentageAsInteger == 50) then
    return {
      getLineText(self, FORMAT_STRING_DIVIDE_BY_TWO, {basePrice, basePrice / 2})
    }
  elseif (percentageAsInteger == 60) then
    local tenPercentOfBasePrice = basePrice * 0.10
    local fiftyPercentOfBasePrice = basePrice / 2
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_DIVIDE_BY_TWO, {basePrice, fiftyPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_SUM_VALUES, {tenPercentOfBasePrice, fiftyPercentOfBasePrice, tenPercentOfBasePrice + fiftyPercentOfBasePrice})
    }
  elseif (percentageAsInteger == 70) then
    local tenPercentOfBasePrice = basePrice * 0.10
    local thirtyPercentOfBasePrice = tenPercentOfBasePrice * 3
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_MULTIPLY_BY_THREE, {tenPercentOfBasePrice, thirtyPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_SUBTRACT_VALUE, {basePrice, thirtyPercentOfBasePrice, basePrice - thirtyPercentOfBasePrice})
    }
  elseif (percentageAsInteger == 75) then
    local twentyFivePercentOfBasePrice = basePrice / 4
    return {
      getLineText(self, FORMAT_STRING_DIVIDE_BY_FOUR, {basePrice, twentyFivePercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_SUBTRACT_VALUE, {basePrice, twentyFivePercentOfBasePrice, basePrice - twentyFivePercentOfBasePrice})
    }
  elseif (percentageAsInteger == 80) then
    local tenPercentOfBasePrice = basePrice * 0.10
    local twentyPercentOfBasePrice = tenPercentOfBasePrice * 2
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_MULTIPLY_BY_TWO, {tenPercentOfBasePrice, twentyPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_SUBTRACT_VALUE, {basePrice, twentyPercentOfBasePrice, basePrice - twentyPercentOfBasePrice})
    }
  elseif (percentageAsInteger == 90) then
    local tenPercentOfBasePrice = basePrice * 0.10
    return {
      getLineText(self, FORMAT_STRING_TEN_PERCENT, {basePrice, tenPercentOfBasePrice}),
      getLineText(self, FORMAT_STRING_SUBTRACT_VALUE, {basePrice, tenPercentOfBasePrice, basePrice - tenPercentOfBasePrice})
    }
  else
    error("Unhandled percentage '" .. tostring(percentageAsInteger) .. "' in example text generation")
  end
end
