--- GoodsGreenParticles
-- @author Created by Aaron Barrett on June 11, 2014.

class("GoodsGreenParticles")
require("Particles")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.height

-- Particle systems -------------------

local SYSTEMS_COUNT = 6

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsGreenParticles:clearItems()
  local systems = self.systems

  for _, system in ipairs(systems) do
    system:reset()
    system:stopSystem()
    system:clearAttrLink(MOAIProp2D.INHERIT_TRANSFORM)
  end

  self.currentSystemIndex = 1
end

function GoodsGreenParticles:addItem(item)
  local systems = self.systems

  local x, y = item:getLoc()

  local system = systems[self.currentSystemIndex]
  assert(system ~= nil, "Out of systems")
  
  system:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, item, MOAIProp2D.TRANSFORM_TRAIT)

  item:setParticleSystem(system)

  self.currentSystemIndex = self.currentSystemIndex + 1
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GoodsGreenParticles.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, GoodsGreenParticles)

  local systems = {}
  for i = 1, SYSTEMS_COUNT do
    local system = Particles.new("green_particles.pex")
    system:setX(GoodsItem.WIDTH * 0.5)
    system:setY(GoodsItem.HEIGHT * 0.5)
    system:start()

    table.insert(systems, system)
    self:insert(system)
  end

  self.systems = systems
  self.currentSystemIndex = 1

  return self
end
