--- GoodsInstructions
-- @author Created by Aaron Barrett on June 12, 2014.

class("GoodsInstructions")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

local WIDTH = Screen.width

local BAR_SPACING = Screen.getSafeHeightPart(0.025)
local TOP_MARGIN = Screen.safeTopInset + 20 + BAR_SPACING

local LABEL_WIDTH = WIDTH * 0.83

local BAR_HEIGHT = 1.0

-- Font -------------------------------

local FONT_SIZE = 14
local FONT = Font.new(FontFaces.DIN_OT_BOLD, FONT_SIZE)

-- Text -------------------------------

local INSTRUCTIONS_TEXT = Localizer.get("INSTRUCTIONS")

-- Colors -----------------------------

local COLOR = {1.0, 1.0, 1.0}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsInstructions:playAppear()
  return AHOpacityTo.new(self, 0.2, nil, 1.0):start()
end

function GoodsInstructions:playDisappear()
  return AHSequence.new(
    AHOpacityTo.new(self, 0.2, nil, 0.0),
    function()
      self.label:removeSelf()
      self.label = nil
    end
  ):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildLabel(self)
  local horizontalAlignment = Alignment.CENTER
  local label = DynamicTextArea.newSimple(INSTRUCTIONS_TEXT, LABEL_WIDTH, FONT, horizontalAlignment)
  label:setColor(unpack(COLOR))
  self:insert(label)
  self.label = label
end

local function buildBar(self)
  local bar = FilledRectangle.new(WIDTH, BAR_HEIGHT, COLOR)
  self:insert(bar)
  self.bar = bar
end

-- Constructor ------------------------

function GoodsInstructions.new()
  local self = Column.new(spacing, Alignment.CENTER)
  Mixin.mix(self, GoodsInstructions)

  self:insertSpacer(TOP_MARGIN)

  buildLabel(self)

  self:insertSpacer(BAR_SPACING)

  buildBar(self)

  self:alignChildren()

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)

  self:setOpacity(0.0)

  return self
end
