--- GoodsItemArranger
-- @author Created by Aaron Barrett on July 01, 2021.

class("GoodsItemArranger")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local THREE_ROWS_HEIGHT = GoodsItem.HEIGHT * 2.1

local IDEAL_BASE_Y = Screen.getSafeY(0.5)
local FALLBACK_BASE_Y = Screen.getSafeY(1.0) - THREE_ROWS_HEIGHT

local RATIONALE_ROWS = {
  TOP = "RATIONALE_ROWS.TOP",
  MIDDLE = "RATIONALE_ROWS.MIDDLE",
  BOTTOM = "RATIONALE_ROWS.BOTTOM",
  BOTTOM_MIDDLE = "RATIONALE_ROWS.BOTTOM_MIDDLE"
}

local ITEM_LAYOUTS_BY_ITEMS_COUNT = {
  [2] = {
    {homeXProportion = 0.32, homeYProportion = 0.50, rationaleRow = RATIONALE_ROWS.BOTTOM_MIDDLE},
    {homeXProportion = 0.68, homeYProportion = 0.50, rationaleRow = RATIONALE_ROWS.BOTTOM_MIDDLE},
  },
  [3] = {
    {homeXProportion = 0.18, homeYProportion = 0.50, rationaleRow = RATIONALE_ROWS.BOTTOM_MIDDLE},
    {homeXProportion = 0.50, homeYProportion = 0.50, rationaleRow = RATIONALE_ROWS.BOTTOM_MIDDLE},
    {homeXProportion = 0.82, homeYProportion = 0.50, rationaleRow = RATIONALE_ROWS.BOTTOM_MIDDLE}
  },
  [4] = {
    {homeXProportion = 0.18, homeYProportion = 0.5, rationaleRow = RATIONALE_ROWS.MIDDLE},
    {homeXProportion = 0.50, homeYProportion = 0.66, rationaleRow = RATIONALE_ROWS.TOP},
    {homeXProportion = 0.50, homeYProportion = 0.33, rationaleRow = RATIONALE_ROWS.BOTTOM},
    {homeXProportion = 0.82, homeYProportion = 0.5, rationaleRow = RATIONALE_ROWS.MIDDLE}
  },
  [5] = {
    {homeXProportion = 0.18, homeYProportion = 0.66, rationaleRow = RATIONALE_ROWS.TOP},
    {homeXProportion = 0.50, homeYProportion = 0.66, rationaleRow = RATIONALE_ROWS.TOP},
    {homeXProportion = 0.82, homeYProportion = 0.66, rationaleRow = RATIONALE_ROWS.TOP},
    {homeXProportion = 0.32, homeYProportion = 0.33, rationaleRow = RATIONALE_ROWS.BOTTOM},
    {homeXProportion = 0.68, homeYProportion = 0.33, rationaleRow = RATIONALE_ROWS.BOTTOM}
  },
  [6] = {
    {homeXProportion = 0.18, homeYProportion = 0.66, rationaleRow = RATIONALE_ROWS.TOP},
    {homeXProportion = 0.50, homeYProportion = 0.66, rationaleRow = RATIONALE_ROWS.TOP},
    {homeXProportion = 0.82, homeYProportion = 0.66, rationaleRow = RATIONALE_ROWS.TOP},
    {homeXProportion = 0.18, homeYProportion = 0.33, rationaleRow = RATIONALE_ROWS.BOTTOM},
    {homeXProportion = 0.50, homeYProportion = 0.33, rationaleRow = RATIONALE_ROWS.BOTTOM},
    {homeXProportion = 0.82, homeYProportion = 0.33, rationaleRow = RATIONALE_ROWS.BOTTOM}
  }
}

local ITEM_HEIGHT_MULTIPLIER_BY_RATIONALE_ROW = {
  [RATIONALE_ROWS.BOTTOM] = 0.5,
  [RATIONALE_ROWS.BOTTOM_MIDDLE] = 0.85,
  [RATIONALE_ROWS.MIDDLE] = 1.05,
  [RATIONALE_ROWS.TOP] = 1.6
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getLayoutForItems(self, items)
  local itemsCount = #items
  local layout = ITEM_LAYOUTS_BY_ITEMS_COUNT[itemsCount]
  assert(layout ~= nil, "No layout found for itemsCount: " .. tostring(itemsCount))
  return layout
end

local function getYForRationaleRow(self, rationaleRow)
  local itemHeightMultiplier = ITEM_HEIGHT_MULTIPLIER_BY_RATIONALE_ROW[rationaleRow]
  assert(itemHeightMultiplier ~= nil, "No item height multiplier provided for rationale row value: '" .. tostring(rationaleRow) .. "'")
  return self.baseY + itemHeightMultiplier * GoodsItem.HEIGHT
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsItemArranger:arrangeItems(items)
  local layout = getLayoutForItems(self, items)

  local shuffledItems = TableUtils.createShuffledArray(items)
  for itemIndex, item in ipairs(shuffledItems) do
    local descriptor = layout[itemIndex]

    local homeX = Screen.getSafeX(descriptor.homeXProportion)
    local homeY = Screen.getSafeY(descriptor.homeYProportion)
    item:setHomeLocation(homeX, homeY)

    local rationaleRow = descriptor.rationaleRow
    local rationaleY = getYForRationaleRow(self, rationaleRow)
    item:setRationaleY(rationaleY)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GoodsItemArranger.new()
  local self = {}
  Mixin.mix(self, GoodsItemArranger)

  local topRowItemTopYFromIdealBaseY = IDEAL_BASE_Y + THREE_ROWS_HEIGHT
  if (topRowItemTopYFromIdealBaseY < Screen.getSafeY(1.0)) then
    self.baseY = IDEAL_BASE_Y
  else
    self.baseY = FALLBACK_BASE_Y
  end

  return self
end
