--- GoodsOutro
-- @author Created by aab29 on June 16, 2014.

class("GoodsOutro")
require("graphics/MultiGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.height

local CENTER_X = WIDTH * 0.5

local RAINBOW_HEIGHT = HEIGHT * 24.0

-- Stripes ----------------------------

local STRIPE_COLORS = {
  {0.70, 0.22, 0.95, 0.00}, -- Invisible Purple
  {0.70, 0.22, 0.95, 1.00}, -- Purple
  {0.13, 0.46, 0.87, 1.00}, -- Blue
  {0.24, 1.00, 0.95, 1.00}, -- Teal
  {0.08, 0.03, 0.13, 1.00}, -- Dark Purple
  {0.08, 0.03, 0.13, 1.00}  -- Dark Purple
}

local STRIPE_THICKNESSES = {
  1.0,
  1.0,
  1.0,
  1.0,
  1.0
}

-- Z-indices --------------------------

local Z_RAINBOW = 10
local Z_CHECKMARK = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsOutro:playOutro()
  local rainbow = self.rainbow
  local rainbowAction = AHLocationYTo.new(rainbow, 3.0, nil, RAINBOW_HEIGHT)

  local checkmark = self.checkmark
  local checkmarkAction = AHSequence.new(
                                         AHSpawn.new(
                                                     AHOpacityTo.new(checkmark, 0.6, AHEaseSimpleIn.new(2.0), 0.6),
                                                     AHRotationTo.new(checkmark, 0.6, AHEaseSimpleIn.new(4.0), 0.0),
                                                     AHScaleTo.new(checkmark, 0.6, AHEaseSimpleIn.new(3.0), 0.8)
                                                     ),
                                         AHScaleTo.new(checkmark, 0.1, AHEaseSimpleOut.new(3.0), 1.0),
                                         AHDelay.new(1.5),
                                         AHOpacityTo.new(checkmark, 0.3, nil, 0.0)
                                         )

  local outroAction = AHSpawn.new(
                                  rainbowAction,
                                  checkmarkAction
                                  )

  return outroAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GoodsOutro.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, GoodsOutro)

  local isHorizontal = false
  local rainbow = MultiGradient.new(WIDTH, RAINBOW_HEIGHT, isHorizontal, STRIPE_COLORS, STRIPE_THICKNESSES)

  rainbow:setAnchorPointTop()
  rainbow:setLoc(CENTER_X, 0.0)
  self:insert(rainbow, Z_RAINBOW)

  local checkmark = Sprite.newWithName("checkmark.png")
  self:insertAtCenter(checkmark, Z_CHECKMARK)
  checkmark:setOpacity(0.0)
  checkmark:setScl(2.0)
  checkmark:setRot(-45.0)

  self.rainbow = rainbow
  self.checkmark = checkmark

  return self
end
