--- GoodsRationaleOverlay
-- @author Created by Aaron Barrett on July 01, 2021.

class("GoodsRationaleOverlay")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {0.0, 0.0, 0.0}
local FULL_OPACITY = 0.7

local Z_RATIONALE_CARD = 10
local Z_ARROW_BUTTONS = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function presentCardAtIndex(self, destinationIndex)
  local presentedIndex = self.presentedCard:getCardIndex()
  local destinationCard = self.cards[destinationIndex]
  assert(destinationCard ~= nil, "No card exists at destination index: " .. tostring(destinationIndex))

  local presentedCardIsAppearing = false
  local destinationCardIsAppearing = true

  if (destinationIndex < presentedIndex) then
    self.presentedCard:playSlideRight(presentedCardIsAppearing)
    destinationCard:playSlideRight(destinationCardIsAppearing)
  elseif (destinationIndex > presentedIndex) then
    self.presentedCard:playSlideLeft(presentedCardIsAppearing)
    destinationCard:playSlideLeft(destinationCardIsAppearing)
  else
    error("Attempted to present card at index that is already presented.")
  end

  self.presentedCard = destinationCard
end

local function buildDeferrableTextElements(self)
  for _, card in ipairs(self.cards) do
    card:buildDeferrableTextElements()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsRationaleOverlay:playFadeOut()
  self.presentedCard:prepareToDismiss()
  return AHSequence.new(
            AHDelay.new(0.4),
            AHSpawn.new(
              self.presentedCard:createSlideDownAction(),
              AHOpacityTo.new(self, 1.0, AHEaseExponentialOut.new(), 0.0)
            ),
            AHRemove.new(self)
          ):start()
end

function GoodsRationaleOverlay:playFadeIn()
  return AHSequence.new(
    AHDelay.new(GoodsGame.PRE_RATIONALE_OPTIMIZATION_DURATION),
    function()
      if (self.cards ~= nil) then
        local firstCard = self.cards[1]
        firstCard:playSlideUp()
        self.presentedCard = firstCard
      end
    end,
    AHOpacityTo.new(self, 1.0, AHEaseExponentialOut.new(), FULL_OPACITY),
    function()
      buildDeferrableTextElements(self)
    end
  ):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function getRationaleCardsCount(self, chosenEffect, cheapestEffect)
  if (chosenEffect == nil) then
    if (cheapestEffect:hasPercentageOperation()) then
      return 2
    else
      return 1
    end
  elseif (chosenEffect:hasPercentageOperation() and cheapestEffect:hasPercentageOperation()) then
    return 3
  elseif (chosenEffect:hasPercentageOperation()) then
    return 2
  elseif (cheapestEffect:hasPercentageOperation()) then
    return 2
  else
    return 1
  end
end

local function buildRationaleCards(self, chosenItem, cheapestItem, autoTapAndDismissCallback)
  local chosenEffect = nil
  if (chosenItem ~= nil) then
    chosenEffect = chosenItem:getEffect()
  end
  local cheapestEffect = cheapestItem:getEffect()
  local cardsCount = getRationaleCardsCount(self, chosenEffect, cheapestEffect)

  local function presentCardAtIndexCallback(cardIndex)
    presentCardAtIndex(self, cardIndex)
  end

  self.cards = {}
  for cardIndex = 1, cardsCount do
    local useDeferredTextOptimization = (cardIndex > 1)
    local card = GoodsRationaleCard.new(chosenItem, cheapestItem, cardIndex, presentCardAtIndexCallback, autoTapAndDismissCallback, useDeferredTextOptimization)
    card:hideImmediately()
    card:setX(self:getWidthPart(0.5))
    card:setY(card:getHeightPart(0.5))
    self:insert(card, Z_RATIONALE_CARD)
    table.insert(self.cards, card)
  end
end

-- Constructor ------------------------

function GoodsRationaleOverlay.new(chosenItem, cheapestItem, autoTapAndDismissCallback)
  local self = FilledRectangle.newFullScreen(BACKGROUND_COLOR)
  Mixin.mix(self, GoodsRationaleOverlay)

  self.cards = nil
  self.presentedCard = nil

  buildRationaleCards(self, chosenItem, cheapestItem, autoTapAndDismissCallback)

  self:setOpacity(0.0)

  return self
end
