--- GoodsRationaleTextArea
-- @author Created by Aaron Barrett on July 15, 2021.

class("GoodsRationaleTextArea")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LARGEST_FONT_SIZE = 16.0
local SMALLEST_FONT_SIZE = 14.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getFontSizeScaleProportion(self, characterCount)
  if (characterCount < 140) then
    return 1.0
  elseif (characterCount < 145) then
    return 0.5
  else
    return 0.0
  end
end

local function getFontSizeForText(self, text)
  local characterCount = text:len()
  local scaleProportion = getFontSizeScaleProportion(self, characterCount)
  return MathUtils.interpolateLinearFromNormalized(scaleProportion, SMALLEST_FONT_SIZE, LARGEST_FONT_SIZE)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsRationaleTextArea:getBodyFontSize()
  return self.bodyFontSize
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GoodsRationaleTextArea.new(width, text, useLargestFont)
  local fontSize = nil
  if (useLargestFont) then
    fontSize = LARGEST_FONT_SIZE
  else
    fontSize = getFontSizeForText(self, text)
  end

  local bodyFont = Font.new(FontFaces.DIN_OT_LIGHT, fontSize)
  local boldFont = Font.new(FontFaces.DIN_OT_MEDIUM, fontSize)

  local baseAttributes = {font = bodyFont}
  local attributesForLabels = {
    b = {font = boldFont}
  }

  local self = GoodsRationaleTextView.new(width)
  Mixin.mix(self, GoodsRationaleTextArea)

  local textInterpreter = TextInterpreter.new(baseAttributes, attributesForLabels)
  local textModel = TextModel.new(textInterpreter:interpretXmlText(text))
  self:setModel(textModel)

  self.textInterpreter = textInterpreter
  self.textModel = textModel

  self.bodyFontSize = fontSize

  return self
end
