--- GoodsRoundGenerator
-- @author Created by Aaron Barrett on June 19, 2014.

class("GoodsRoundGenerator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createFiltersForItemCount(itemCount)
  local filters = {item_count = tostring(itemCount)}
  return filters
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GoodsRoundGenerator:getNextRound(roundNumber, itemCount, countdownDuration)
  self.conceptGenerator:setFilters(createFiltersForItemCount(itemCount))
  local concept = self.conceptGenerator:next()
  local identifier = concept.identifier
  local difficulty = concept.difficulty

  local items = {}
  for i = 1, itemCount do
    local basePriceKey = "base_price_" .. i
    local basePrice = concept[basePriceKey]
    assert(type(basePrice) == "number", "Concept '" .. tostring(identifier) .. "' is missing '" .. tostring(basePriceKey) .. "'.  Got '" .. tostring(basePrice) .. "' of type '" .. type(basePrice) .. "'.")

    local operationKey = "operation_" .. i
    local operation = concept[operationKey]
    assert(type(operation) == "string", "Concept '" .. tostring(identifier) .. "' is missing '" .. tostring(operationKey) .. "'.  Got '" .. tostring(operation) .. "' of type '" .. type(operation) .. "'.")

    local operationAmountKey = "op_amount_" .. i
    local operationAmount = concept[operationAmountKey]
    assert(type(operationAmount) == "number", "Concept '" .. tostring(identifier) .. "' is missing '" .. tostring(operationAmountKey) .. "'.  Got '" .. tostring(operationAmount) .. "' of type '" .. type(operationAmount) .. "'.")

    local effect = GoodsEffect.new(operation, operationAmount)
    local item = GoodsItem.new(basePrice, effect)
    table.insert(items, item)
  end

  local centralPrice = concept.central_price
  assert(type(centralPrice) == "number", "Concept '" .. tostring(identifier) .. "' is missing 'central_price'.  Got '" .. tostring(centralPrice) .. "' of type '" .. type(centralPrice) .. "'.")

  local round = GoodsRound.new(identifier, difficulty, roundNumber, items, centralPrice, countdownDuration)
  return round
end

function GoodsRoundGenerator.new()
  local self = {}
  Mixin.mix(self, GoodsRoundGenerator)

  self.conceptGenerator = GameSession.conceptGenerator

  return self
end
