--- LinksAudioTimer
-- @author Created by amorant on June 06, 2014.

class("LinksAudioTimer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -----------------------------

local WIDTH = Screen.getWidthPart(0.16)
local HEIGHT = Screen.getHeightPart(0.025)
local LABEL_X = 0.65 * WIDTH
local LABEL_Y = 0.50 * HEIGHT

-- Text -------------------------------

local FONT = Font.new(FontFaces.DIN_OT_BOLD, 16)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setCurrentTime(self, time)
  if time ~= self.currentTime then
    local timeString = tostring(time)
    if time < 10 then
      timeString = ":0" .. timeString
    else
      timeString = ":" .. timeString
    end
    self.label:setText(timeString)
    self.currentTime = time
  end
end

local function createFadeAction(self, duration, targetOpacity)
  return AHOpacityTo.new(self.label, duration, nil, targetOpacity)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinksAudioTimer:playAudioAndWait(concept, soundFile, duration)
  setCurrentTime(self, duration)

  local fadeIn = createFadeAction(self, 0.5, self.visibleOpacity)
  wait(fadeIn:start())

  local sound = AssetHelper.playSoundForConcept(concept, soundFile)

  while not sound:isFinished() do
    local audioTime = sound:getTime()
    local remainingTime = math.floor(duration - audioTime)
    remainingTime = math.max(remainingTime, 0)

    -- This check is to prevent a bug that sometimes happens where the sound has finished but it
    -- still returns false for isFinished. Then, getTime() returns 0 as if the playback where at
    -- the beginning of the file, and remainingTime is back to the full duration. This situation
    -- can last for a couple of frames.
    if remainingTime < self.currentTime then
      setCurrentTime(self, remainingTime)
    end

    coroutine.yield()
  end

  setCurrentTime(self, 0)
  local fadeOut = createFadeAction(self, 0.5, 0)
  wait(fadeOut:start())
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self, color)
  local label = Label.new("00", FONT)
  label:setColor(unpack(color))
  label:setLoc(LABEL_X, LABEL_Y)
  self:insert(label)
  self.label = label

  label:setOpacity(0)
end

-- Constructor ------------------------

function LinksAudioTimer.new(color)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, LinksAudioTimer)

  createElements(self, color)

  self.currentTime = 0
  self.visibleOpacity = color[4] or 1

  return self
end
