--- LinksCollapsibleLine
-- @author Created by amorant on June 04, 2014.

class("LinksCollapsibleLine")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinksCollapsibleLine:setLengthScale(lengthScale)
  lengthScale = MathUtils.clamp(lengthScale, 0, 1)

  local x = MathUtils.interpolateLinearFromNormalized(lengthScale, self.x1, self.x2)
  local y = MathUtils.interpolateLinearFromNormalized(lengthScale, self.y1, self.y2)
  self.super.setVertexPosition(self, 2, x, y)

  self.lengthScale = lengthScale
end

function LinksCollapsibleLine:getLengthScale(lengthScale)
  return self.lengthScale
end

-- Line overrides ---------------------

function LinksCollapsibleLine:setVertexPosition(vertexIndex, x, y)
  error("This operation is not allowed on a LinksCollapsibleLine")
end

function LinksCollapsibleLine:append(x, y)
  error("This operation is not allowed on a LinksCollapsibleLine")
end

-- Animations -------------------------

function LinksCollapsibleLine:createGrowAnimation(duration, ease)
  return AHNonAttributeTo.new(self, duration, self.getLengthScale, self.setLengthScale, ease, 1)
end

function LinksCollapsibleLine:createShrinkAnimation(duration, ease)
  return AHNonAttributeTo.new(self, duration, self.getLengthScale, self.setLengthScale, ease, 0)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function LinksCollapsibleLine.make(line)
  assert(#line.vertices == 4, "LinksCollapsibleLine works only on 2-point lines.")

  Mixin.mix(line, LinksCollapsibleLine)

  line.x1 = line.vertices[1]
  line.y1 = line.vertices[2]
  line.x2 = line.vertices[3]
  line.y2 = line.vertices[4]
  line.lengthScale = 1

  return line
end

function LinksCollapsibleLine.new(x1, y1, x2, y2)
  local self = Line.new(x1, y1, x2, y2)
  LinksCollapsibleLine.make(self)

  return self
end
