--- LinksConceptGenerator
-- @author Created by amorant on May 07, 2014.

class("LinksConceptGenerator")

require("data.generators.UtilityGenerators")


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function processConnection(link, questionIdentifier, node, setData)
  if link then
    local linkData = {
      text = link,
      questionIdentifier = questionIdentifier,
      node = node
    }
    table.insert(setData.links, linkData)
  end

  if node then
    local objectData = {
      text = node,
      link = link
    }
    table.insert(setData.nodes, objectData)
  end

  if link and node then
    setData.numConnections = setData.numConnections + 1
  end
end

local function processSet(set)
  local setData = {
    links = {},
    nodes = {},
    numConnections = 0
  }

  processConnection(set.prompt_1, set.question_1_id, set.answer_1, setData)
  processConnection(set.prompt_2, set.question_2_id, set.answer_2, setData)
  processConnection(nil, nil, set.distractor, setData)

  TableUtils.shuffleArray(setData.links)
  TableUtils.shuffleArray(setData.nodes)

  return setData
end

local function processRound(round)
  local roundData = {
    audioClip = round.audio_link,
    audioClipDuration = tonumber(round.run_time),
    sets = {}
  }

  -- The game supports multiple sets per round, but current content is not using that.
  local setData = processSet(round)
  table.insert(roundData.sets, setData)

  return roundData
end

local function processConcept(concept)
  local conceptData = {
    identifier = concept.identifier,
    assets_path = concept.assets_path,
    rootNode = concept.subjectNode,
    rounds = {}
  }

  for _, round in ipairs(concept.rounds) do
    local roundData = processRound(round)

    table.insert(conceptData.rounds, roundData)
  end

  return conceptData
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function LinksConceptGenerator.new()
  return TransformationGenerator.new(GameSession.conceptGenerator, processConcept)
end
