--- LinksConfiguration
-- @author Created by amorant on June 03, 2014.

class("LinksConfiguration")

require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

LinksConfiguration.POINTS_PER_CORRECT_RESPONSE = 1200
LinksConfiguration.POINTS_PER_EXTRA_SECOND_REMAINING = 25

local INITIAL_LIVES_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {lives = 5}),
  Keyframe.new(0.125, {lives = 4})
)

local TOTAL_POINTS_FOR_BONUS_LIVES = 3000

local MAX_LIVES_LOST_RANK_TWO = 1
local MAX_LIVES_LOST_RANK_THREE = 0

local ROUNDS_PER_CONCEPT = 3
local ANSWERS_PER_ROUND = 2
local MINIMUM_QUESTION_TIMER_DURATION = 7
local MAXIMUM_QUESTION_TIMER_DURATION = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getExtraLivesBonusForLivesLost(numberOfLivesLost)
  local initialLivesCount = LinksConfiguration.getInitialLivesCount()
  local bonusPointsPerLife = LinksConfiguration.getPointsPerLifeBonus()
  local extraLivesRemaining = initialLivesCount - numberOfLivesLost - 1
  return bonusPointsPerLife * extraLivesRemaining
end

function getPassScore()
  local numCorrectAnswers = ROUNDS_PER_CONCEPT * ANSWERS_PER_ROUND
  local passScore = numCorrectAnswers * LinksConfiguration.POINTS_PER_CORRECT_RESPONSE
  return passScore
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinksConfiguration.getInitialLivesCount()
  local difficulty = GameSession.getDifficulty()
  local frameForDifficulty = INITIAL_LIVES_THRESHOLDS:getFrameAt(difficulty)
  return frameForDifficulty.lives
end

function LinksConfiguration.getPointsPerLifeBonus()
  local maximumBonusLives = (LinksConfiguration.getInitialLivesCount() - 1)
  return TOTAL_POINTS_FOR_BONUS_LIVES / maximumBonusLives
end

function LinksConfiguration.getRankTwoScore()
  local passScore = getPassScore()
  local livesBonus = getExtraLivesBonusForLivesLost(MAX_LIVES_LOST_RANK_TWO)
  local rankTwoScore = passScore + livesBonus
  return rankTwoScore
end

function LinksConfiguration.getRankThreeScore()
  local passScore = getPassScore()
  local livesBonus = getExtraLivesBonusForLivesLost(MAX_LIVES_LOST_RANK_THREE)
  local rankThreeScore = passScore + livesBonus
  return rankThreeScore
end

function LinksConfiguration.getQuestionTimerDuration(difficulty)
  local questionTimerDuration = MathUtils.interpolateLinearWithClamp(difficulty,
                                                                     0.0,
                                                                     0.75,
                                                                     MAXIMUM_QUESTION_TIMER_DURATION,
                                                                     MINIMUM_QUESTION_TIMER_DURATION)
  Log.debug("Links question timer duration set to " .. tostring(questionTimerDuration))
  return questionTimerDuration
end

function LinksConfiguration.getSecondsForTimeBonus(remainingSeconds)
  -- Since time bonus is based on remaining time per question, and question timer duration
  -- decreases with difficulty, we need to normalize the remaining seconds at the end of the game
  -- to make the maximum time bonus reachable at all difficulties.
  local difficulty = GameSession.getDifficulty()
  local questionTimerDuration = LinksConfiguration.getQuestionTimerDuration(difficulty)
  local secondsForTimeBonus = MathUtils.mapLinear(remainingSeconds, 0, questionTimerDuration, 0, MAXIMUM_QUESTION_TIMER_DURATION)
  return secondsForTimeBonus
end
