--- LinksDustParticles
-- @author Created by amorant on June 16, 2014.

class("LinksDustParticles")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local NUM_PARTICLES = 30
local RADIUS = 1
local COLOR = { 0, 1, 1, 0.5 }
local MAX_DISTANCE = Screen.width/3
local LIFE_MIN_DURATION = 5
local LIFE_MAX_DURATION = 8
local TIME_BETWEEN_SPAWNS = 1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createParticle(self)
  local particle = FilledCircle.new(RADIUS, COLOR)
  local x = math.random(0, self:getWidth())
  local y = math.random(0, self:getHeight())
  particle:setLoc(x, y)
  return particle
end

local function moveParticle(particle)
  local distance = math.random(MAX_DISTANCE/2, MAX_DISTANCE)
  local angle = math.random(0, 360)
  local distanceX = distance * math.cos(angle)
  local distanceY = distance * math.sin(angle)
  local duration = math.random() + math.random(LIFE_MIN_DURATION, LIFE_MAX_DURATION - 1)
  local animation = AHLocationBy.new(particle, duration, nil, distanceX, distanceY)
  wait(animation:start())
end

local function showParticle(self, particle)
  particle:setScl(0)
  self:insert(particle)
  local animation = AHScaleTo.new(particle, 0.2, AHEaseSineOut.new(), 1)
  wait(animation:start())
end

local function killParticle(self, particle)
  local animation = AHScaleTo.new(particle, 0.2, AHEaseSineIn.new(), 0)
  wait(animation:start())
  particle:removeSelf()
end

local function doParticleLifeCycle(self)
  self.numParticles = self.numParticles + 1
  runInCoroutine(function()
    local particle = createParticle(self)
    showParticle(self, particle)
    moveParticle(particle)
    killParticle(self, particle)
    self.numParticles = self.numParticles - 1
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinksDustParticles:start()
  runInCoroutine(function()
    while true do
      while self.numParticles < NUM_PARTICLES do
        doParticleLifeCycle(self)
      end

      waitFor(TIME_BETWEEN_SPAWNS)
    end
  end)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function LinksDustParticles.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, LinksDustParticles)

  self.numParticles = 0

  return self
end
