--- LinksQuestionTimer
-- @author Created by amorant on May 20, 2014.

class("LinksQuestionTimer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local THICKNESS = 1

local COLOR_BAR = { 0.8, 0.9, 1, 1 }
local COLOR_BACKGROUND = { 0.8, 0.9, 1, 0.2 }
local COLOR_TIMEOUT = ColorUtils.colorRGBToStd(236, 71, 109)

local FLASH_DURATION = 0.5
local WARNING_COUNT = 3
local WARNING_INTERVAL = 1

local Z_BACKGROUND = 10
local Z_BAR = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWarningHandler(self)
  return function()
    local flashInAction = AHColorTo.new(self.bar, FLASH_DURATION/2, nil, COLOR_TIMEOUT)
    local flashOutAction = AHColorTo.new(self.bar, FLASH_DURATION/2, nil, COLOR_BAR)
    local flashSequence = AHSequence.new(flashInAction, flashOutAction)

    flashSequence:start()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinksQuestionTimer:setProgress(progress)
  self.super.setProgress(self, progress)
  progress = self:getProgress()
  self.bar:setScl(progress, 1)
end

function LinksQuestionTimer:show()
  local fadeIn = AHSpawn.new(
    AHOpacityTo.new(self.bar, 0.5, nil, COLOR_BAR[4]),
    AHOpacityTo.new(self.background, 0.5, nil, COLOR_BACKGROUND[4])
  )
  fadeIn:start()
end

function LinksQuestionTimer:hide()
  local fadeOut = AHSpawn.new(
    AHOpacityTo.new(self.bar, 0.5, nil, 0),
    AHOpacityTo.new(self.background, 0.2, nil, 0)
  )
  fadeOut:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()

  local bar = FilledRectangle.new(width, height, COLOR_BAR)
  self:insertAtCenter(bar, Z_BAR)
  self.bar = bar

  local background = FilledRectangle.new(width, height, COLOR_BACKGROUND)
  self:insertAtCenter(background, Z_BACKGROUND)
  self.background = background

  bar:setOpacity(0)
  background:setOpacity(0)
end

-- Constructor ------------------------

function LinksQuestionTimer.new(width)
  local self = Group.new(width, THICKNESS)
  ProgressIndicator.makeProgressIndicator(self)
  ProgressTimer.makeTimer(self)
  Mixin.mix(self, LinksQuestionTimer)

  self:setWarningCount(WARNING_COUNT)
  self:setWarningInterval(WARNING_INTERVAL)
  createLayout(self)
  self:addWarningHandler(getWarningHandler(self))
  self:addStandardAudibleWarningSequence()

  return self
end
