--- LinksRectangleIntersector
-- @author Created by amorant on June 02, 2014.

class("LinksRectangleIntersector")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getSegmentsIntersection(ax1, ay1, ax2, ay2, bx1, by1, bx2, by2)
  local q = - (bx2 - bx1) / (by2 - by1)
  local k = - (ax2 - ax1) / (ay2 - ay1)
  local alpha = - (ax1 - bx1 + q * (ay1 - by1)) / (ax2 - ax1 + q * (ay2 - ay1))
  local beta = (ax1 - bx1 + k * (ay1 - by1)) / (bx2 - bx1 + k * (by2 - by1))

  if MathUtils.isRealNumber(alpha) and not MathUtils.isRealNumber(beta) then
    if bx1 ~= bx2 then
      beta = (ax1 - bx1 + alpha * (ax2 - ax1)) / (bx2 - bx1)
    else
      beta = (ay1 - by1 + alpha * (ay2 - ay1)) / (by2 - by1)
    end
  elseif MathUtils.isRealNumber(beta) and not MathUtils.isRealNumber(alpha) then
    if ax1 ~= ax2 then
      alpha = (bx1 - ax1 + beta * (bx2 - bx1)) / (ax2 - ax1)
    else
      alpha = (by1 - ay1 + beta * (by2 - by1)) / (ay2 - ay1)
    end
  end

  if alpha < 0 or alpha > 1 or beta < 0 or beta > 1 then
    return nil
  else
    local mx = ax1 + alpha * (ax2 - ax1)
    local my = ay1 + alpha * (ay2 - ay1)
    return { mx, my }
  end
end

local function getPointsWithoutDuplicates(points)
  -- Note: This is not optimal, but we have at most 4 elements in the array.
  local uniquePoints = {}

  for _, point in ipairs(points) do
    local duplicateFound = false
    for _, otherPoint in ipairs(uniquePoints) do
      if point[1] ==  otherPoint[1] and point[2] == otherPoint[2] then
        duplicateFound = true
        break
      end
    end

    if not duplicateFound then
      table.insert(uniquePoints, point)
    end
  end

  return uniquePoints
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinksRectangleIntersector.getRectangleAndSegmentIntersections(rx, ry, width, height, sx1, sy1, sx2, sy2)
  local intersections = {}
  local bottomSegment = { rx,         ry,          rx + width, ry          }
  local rightSegment  = { rx + width, ry,          rx + width, ry + height }
  local topSegment    = { rx + width, ry + height, rx,         ry + height }
  local leftSegment   = { rx,         ry + height, rx,         ry          }

  for _, segment in ipairs({topSegment, rightSegment, bottomSegment, leftSegment}) do
    local ax1, ay1, ax2, ay2 = unpack(segment)
    local intersection = getSegmentsIntersection(ax1, ay1, ax2, ay2, sx1, sy1, sx2, sy2)
    if intersection ~= nil then
      table.insert(intersections, intersection)
    end
  end

  local uniqueIntersections = getPointsWithoutDuplicates(intersections)

  return uniqueIntersections
end

function LinksRectangleIntersector.doesRectangleContainPoint(rx, ry, width, height, px, py)
  assert(width > 0, "Expected positive width")
  assert(height > 0, "Expected positive height")
  return (px >= rx) and (px <= rx + width) and (py >= ry) and (py <= ry + height)
end
