--- LinksRootNode
-- @author Created by amorant on June 03, 2014.

class("LinksRootNode")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Text -------------------------------

local HORIZONTAL_MARGIN = 30
local FONT = Font.new(FontFaces.DIN_OT_BOLD, 70)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getRelativeLocation(self, sourceObject)
  local sourceX, sourceY =  nil, nil

  if sourceObject.parent then
    sourceX, sourceY = self:getWorldDistance(sourceObject)
  else
    local selfX, selfY = self:getLoc()
    sourceX, sourceY = sourceObject:getLoc()
    sourceX = sourceX - selfX
    sourceY = sourceY - selfY
  end

  return sourceX, sourceY
end

local function scaleDownLabelIfNeeded(self)
  local maxLabelWidth = self:getWidth() - HORIZONTAL_MARGIN
  local label = self.label
  if (label:getWidth() > maxLabelWidth) then
    label:setEffectiveDims(maxLabelWidth)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinksRootNode:createShowAnimation()
  local duration = 0.5
  return AHSequence.new(
    AHScaleTo.new(self, duration/2, AHEaseSineInOut.new(), 1.05),
    AHScaleTo.new(self, duration/2, AHEaseSineInOut.new(), 1)
  )
end

function LinksRootNode:createHideAnimation()
  local duration = 0.5
  return AHSequence.new(
    AHScaleTo.new(self, duration/2, AHEaseSineInOut.new(), 1.05),
    AHScaleTo.new(self, duration/2, AHEaseSineInOut.new(), 0)
  )
end

function LinksRootNode:createShakeAnimation(sourceObject)
  local duration = 0.55
  local distance = -3

  local maxRotation = 4
  if sourceObject then
    local sourceX, sourceY =  getRelativeLocation(self, sourceObject)

    if (sourceX < 0 and sourceY < 0) or (sourceX > 0 and sourceY > 0) then
      maxRotation = -maxRotation
    end

    if sourceY < 0 then
      distance = -distance
    end
  end

  local rotation = AHSequence.new(
    AHRotationTo.new(self.label, 3/22 * duration, AHEaseSineInOut.new(), maxRotation),
    AHRotationTo.new(self.label, 6/22 * duration, AHEaseSineInOut.new(), - 0.7 * maxRotation),
    AHRotationTo.new(self.label, 6/22 * duration, AHEaseSineInOut.new(), 0.4 * maxRotation),
    AHRotationTo.new(self.label, 4/22 * duration, AHEaseSineInOut.new(), - 0.2 * maxRotation),
    AHRotationTo.new(self.label, 3/22 * duration, AHEaseSineInOut.new(), 0)
  )

  local movement = AHSequence.new(
    AHLocationYBy.new(self.label, duration/3, AHEaseSineInOut.new(), distance),
    AHLocationYBy.new(self.label, duration/3, AHEaseSineInOut.new(), -distance)
  )

  return AHSpawn.new(rotation, movement)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self, text)
  local label = Label.new(text:upper(), FONT)
  self:insertAtCenter(label)
  self.label = label
end

-- Constructor ------------------------

function LinksRootNode.new(width, height, text)
  local self = Group.new(width, height)
  Mixin.mix(self, LinksRootNode)

  createElements(self, text)
  scaleDownLabelIfNeeded(self)
  self:setScl(0)

  return self
end
