--- LinksValidator
-- @author Created by amorant on May 07, 2014.
-- ConceptValidator for Links.

class("LinksValidator")

require("validators.ConceptValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'subjectNode',
  'rounds'
}

local ROUND_FIELDS = {
  "prompt_1",
  "prompt_2",
  "answer_1",
  "answer_2",
  "question_1_id",
  "question_2_id",
  "distractor",
  "audio_link",
  "run_time"
}

local MIN_CLIP_DURATION = 0
local MAX_CLIP_DURATION = 45 -- We really aim for around 30s, here we are just checking that the value is reasonable

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validateRounds(self, rounds)
  for roundNumber, round in ipairs(rounds) do
    self:expectHasKeys(round, ROUND_FIELDS)

    local audioClipDuration = tonumber(round["run_time"])
    self:expectNumberInRange(audioClipDuration, MIN_CLIP_DURATION, MAX_CLIP_DURATION, "run_time for round " .. roundNumber)
  end
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinksValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)

  validateRounds(self, concept.rounds)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function LinksValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, LinksValidator)

  return self
end
