--- LinusConfiguration
-- @author Created by Jay Austria on October 20, 2015.

class("LinusConfiguration")

require("difficulty.DifficultyUtils")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

LinusConfiguration.CORRECT_ANSWER_POINTS = 1200
LinusConfiguration.ROUNDS_TO_COMPLETE = 7
LinusConfiguration.INITIAL_LIVES_COUNT = 3
LinusConfiguration.POINTS_PER_LIVES_BONUS = 900
LinusConfiguration.DIFFICULTY_ESCALATION_AFTER_CORRECT_ANSWER_COUNT = 4
LinusConfiguration.DIFFICULTY_ESCALATION_AMOUNT = 0.2
LinusConfiguration.MAXIMUM_DIGIT_COUNT = 5

LinusConfiguration.SPEED_BONUS_SECONDS = 5.0
LinusConfiguration.SPEED_BONUS_DECAY_START = 1.0
LinusConfiguration.POINTS_PER_FULL_SPEED_BONUS = 80
LinusConfiguration.HIDDEN_SPEED_BONUS_SECONDS = 30
LinusConfiguration.HIDDEN_SPEED_BONUS_DECAY_START = LinusConfiguration.SPEED_BONUS_SECONDS
LinusConfiguration.HIDDEN_POINTS_PER_FULL_SPEED_BONUS = 15

-- Value used in CoreMS's HighDifficultyReachedHighlightProducer, update if modified
local HIGH_LEVEL_THRESHOLD = 0.6
local MAX_EXTRA_LIVES = LinusConfiguration.INITIAL_LIVES_COUNT - 1
local EXTRA_LIVES_NEEDED_RANK_TWO = MAX_EXTRA_LIVES - 1
local EXTRA_LIVES_NEEDED_RANK_THREE = MAX_EXTRA_LIVES


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusConfiguration.getIsHighLevel()
  return GameSession.getDifficulty() > HIGH_LEVEL_THRESHOLD
end

function LinusConfiguration.shouldShowHighDifficultyInterstitial()
  return DifficultyUtils.shouldShowHighDifficultyInterstitial(HIGH_LEVEL_THRESHOLD, GameSession.getDifficulty())
end

function LinusConfiguration.getLowestPassScore()
  local maxLivesLost = LinusConfiguration.INITIAL_LIVES_COUNT - 1
  local roundsCorrect = LinusConfiguration.ROUNDS_TO_COMPLETE - maxLivesLost
  local passScore = LinusConfiguration.CORRECT_ANSWER_POINTS * roundsCorrect

  return passScore
end

function LinusConfiguration.getRankTwoScore()
  local passScore = LinusConfiguration.getLowestPassScore()

  local extraPointsRankTwo = LinusConfiguration.POINTS_PER_LIVES_BONUS * EXTRA_LIVES_NEEDED_RANK_TWO
  local extraConceptPoints = LinusConfiguration.CORRECT_ANSWER_POINTS * EXTRA_LIVES_NEEDED_RANK_TWO
  local rankTwoScore = passScore + extraPointsRankTwo + extraConceptPoints

  return rankTwoScore
end

function LinusConfiguration.getRankThreeScore()
  local passScore = LinusConfiguration.getLowestPassScore()

  local extraPointsRankThree = LinusConfiguration.POINTS_PER_LIVES_BONUS * EXTRA_LIVES_NEEDED_RANK_THREE
  local extraConceptPoints = LinusConfiguration.CORRECT_ANSWER_POINTS * EXTRA_LIVES_NEEDED_RANK_THREE
  local rankThreeScore = passScore + extraPointsRankThree + extraConceptPoints

  return rankThreeScore
end

function LinusConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 0.50,
                                      Easy = 0.50}},

    {key = 0.075, valueProbabilities = {Basic = 0.15,
                                        Easy = 0.80,
                                        Medium = 0.05}},

    {key = 0.15, valueProbabilities = {Easy = 0.85,
                                       Medium = 0.15}},

    {key = 0.225, valueProbabilities = {Easy = 0.65,
                                      Medium = 0.35}},

    {key = 0.3, valueProbabilities = {Easy = 0.35,
                                      Medium = 0.65}},

    {key = 0.375, valueProbabilities = {Easy = 0.15,
                                      Medium = 0.85}},

    {key = 0.45, valueProbabilities = {Medium = 0.75,
                                       Hard = 0.25}},

    {key = 0.525, valueProbabilities = {Medium = 0.45,
                                        Hard = 0.55}},

    {key = 0.6, valueProbabilities = {Medium = 0.25,
                                      Hard = 0.75}},

    {key = 0.675, valueProbabilities = {Medium = 0.20,
                                        Hard = 0.80}},

    {key = 0.75, valueProbabilities = {Medium = 0.05,
                                       Hard = 0.95}},

    {key = 0.8, valueProbabilities = {Hard = 1.0}},

    {key = 1.0, valueProbabilities = {Hard = 1.0}}
  }
end
