--- LinusDiamond
-- @author Created by Jay Austria on October 27, 2015.

class("LinusDiamond")

require("LinusCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

LinusDiamond.HEIGHT_TO_WIDTH_RATIO = 1.85
LinusDiamond.DEFAULT_WIDTH = math.min(Screen.getSafeWidthPart(0.125), Screen.getSafeHeightPart(0.06))
LinusDiamond.DEFAULT_HEIGHT = LinusDiamond.DEFAULT_WIDTH * LinusDiamond.HEIGHT_TO_WIDTH_RATIO

local DRIFT_MAX_X = 12
local DRIFT_MAX_Y = 12
local DRIFT_DURATION_MIN = 1
local DRIFT_DURATION_MAX = 3
local DIAMOND_VERTICES = {
  {x = 0, y = LinusDiamond.DEFAULT_HEIGHT / 2},
  {x = LinusDiamond.DEFAULT_WIDTH / 2, y = LinusDiamond.DEFAULT_HEIGHT},
  {x = LinusDiamond.DEFAULT_WIDTH, y = LinusDiamond.DEFAULT_HEIGHT / 2},
  {x = LinusDiamond.DEFAULT_WIDTH / 2, y = 0}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusDiamond:getRandomHoverAnimation(scale, driftParameter, additionalScaling)
  local maxDriftX = DRIFT_MAX_X / (math.pow(scale, 2))
  maxDriftX = maxDriftX * driftParameter * additionalScaling
  local driftXDuration = MathUtils.getRandomFloat(DRIFT_DURATION_MIN, DRIFT_DURATION_MAX)

  local loopedHoverAnimationX = AHSequence.new(
    AHLocationXBy.new(self, driftXDuration/2.0, AHEaseSimpleInOut.new(), -maxDriftX * 0.5),
    AHLoop.new(
      AHSequence.new(
        AHLocationXBy.new(self, driftXDuration/2.0, AHEaseSimpleInOut.new(), maxDriftX),
        AHLocationXBy.new(self, driftXDuration/2.0, AHEaseSimpleInOut.new(), -maxDriftX)
      )
    )
  )

  local maxDriftY = DRIFT_MAX_Y / (math.pow(scale, 2))
  maxDriftY = maxDriftY * driftParameter * additionalScaling
  local driftYDuration = MathUtils.getRandomFloat(DRIFT_DURATION_MIN, DRIFT_DURATION_MAX)

  local loopedHoverAnimationY = AHSequence.new(
    AHLocationYBy.new(self, driftYDuration, AHEaseSimpleInOut.new(), -maxDriftY * 0.5),
    AHLoop.new(
      AHSequence.new(
        AHLocationYBy.new(self, driftYDuration, AHEaseSimpleInOut.new(), maxDriftY),
        AHLocationYBy.new(self, driftYDuration, AHEaseSimpleInOut.new(), -maxDriftY)
      )
    )
  )

  return AHSpawn.new(loopedHoverAnimationX, loopedHoverAnimationY)
end

function LinusDiamond:hide()
  self:setOpacity(LinusCommon.INVISIBLE_OPACITY)
end

function LinusDiamond:createShowAction()
  local showAction = AHOpacityTo.new(self, 0.3, nil, self.initialOpacity)

  return showAction
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function LinusDiamond.newWithDescriptor(descriptor)
  local self = FilledPolygon.newAutomatic(DIAMOND_VERTICES, descriptor.color)
  Mixin.mix(self, LinusDiamond)

  self.initialOpacity = self:getOpacity()

  return self
end
