--- LinusDigitsLabel
-- @author Created by Joe Poka on April 2, 2018.

class("LinusDigitsLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusDigitsLabel:setText(text)
  for _, digit in ipairs(self.digits) do
    digit:setVisible(false)
  end

  local digitCount = #text
  local index = #self.digits - digitCount + 1
  local width = 0.0

  for i = 1, digitCount do
    local character = text:sub(i, i)
    local digit = self.digits[index]
    digit:setText(character)
    digit:setVisible(true)
    width = width + digit:getWidth()
    index = index + 1
  end

  return width
end

function LinusDigitsLabel:updateDigitsLayout(width)
  self:setWidth(width)

  local x = width
  local y = self:getHeightPart(0.5)
  local horizontalSpacing = Screen.getWidthPart(0.01)

  for _, digit in rpairs(self.digits) do
    local halfWidth = digit:getWidthPart(0.5)
    x = x - halfWidth
    digit:setLoc(x, y)
    x = x - halfWidth - horizontalSpacing
  end
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, font)
  local defaultCharacter = "0"
  local dummy = Label.new(defaultCharacter, font)
  local width = dummy:getWidth()
  local height = dummy:getHeight()
  local digitCount = LinusConfiguration.MAXIMUM_DIGIT_COUNT

  self.digits = {}
  self:setHeight(height)

  for i = 1, digitCount do
    local digit = Label.new(defaultCharacter, font, width, Alignment.CENTER)
    digit:setVisible(false)
    self:insert(digit)
    table.insert(self.digits, digit)
  end
end

-- Constructor ------------------------

function LinusDigitsLabel.new(font)
  local self = Group.new(1, 1)
  Mixin.mix(self, LinusDigitsLabel)

  createLayout(self, font)

  return self
end
