--- LinusLightSource
-- @author Created by Jay Austria on November 2, 2015.

class("LinusLightSource")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createFadeIntensityAction(self, targetIntensity)
  local fadeDuration = 0.2
  local fadeAction = AHNonAttributeTo.new(self, fadeDuration, self.getIntensity, self.setIntensity, nil, targetIntensity)

  return fadeAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusLightSource:initializeIntensity(value)
  self:setIntensity(value)

  self.initialIntensity = value
end

function LinusLightSource:setIntensity(value)
  self.intensity = value
end

function LinusLightSource:getIntensity()
  return self.intensity
end

function LinusLightSource:setZ(value)
  self.z = value
end

function LinusLightSource:getZ()
  return self.z
end

function LinusLightSource:startRandomWobbleAction()
  if (self.wobbleAction ~= nil) then
    self.wobbleAction:stop()
    self.wobbleAction = nil
  end

  local driftXDuration = MathUtils.getRandomFloat(1.0, 1.5)
  local driftYDuration = MathUtils.getRandomFloat(1.0, 1.5)

  local maxDriftX = MathUtils.getRandomFloat(20, 30)
  local maxDriftY = MathUtils.getRandomFloat(20, 30)

  local loopedHoverAnimationX = AHLoop.new(
    AHSequence.new(
      AHLocationXBy.new(self, driftXDuration/2.0, AHEaseSimpleInOut.new(), maxDriftX),
      AHLocationXBy.new(self, driftXDuration/2.0, AHEaseSimpleInOut.new(), -maxDriftX)
    )
  )

  local loopedHoverAnimationY = AHLoop.new(
    AHSequence.new(
      AHLocationYBy.new(self, driftYDuration, AHEaseSimpleInOut.new(), maxDriftY),
      AHLocationYBy.new(self, driftYDuration, AHEaseSimpleInOut.new(), -maxDriftY)
    )
  )

  self.wobbleAction = AHSpawn.new(loopedHoverAnimationX, loopedHoverAnimationY)

  return self.wobbleAction:start()
end

function LinusLightSource:createFadeOnAction()
  local fadeAction = createFadeIntensityAction(self, self.initialIntensity)

  return fadeAction
end

function LinusLightSource:createFadeOffAction()
  local fadeAction = createFadeIntensityAction(self, 0.0)

  return fadeAction
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function LinusLightSource.new()
  local self = Group.newMinimumSized()
  Mixin.mix(self, LinusLightSource)

  self.z = 0
  self.initialIntensity = 0

  return self
end
