--- LinusNumberPad
-- @author Created by Jay Austria on October 20, 2015.

class("LinusNumberPad")

require("widgets.number_pad.NumberPad")
require("LinusCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local KEY_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 28)
local FILL_VISIBLE_OPACITY = 0.4
local TAP_EFFECT_CIRCLE_DESCRIPTORS = {
  {
    radius = Screen.getWidthPart(0.1),
    color = ColorUtils.colorHex24ToStd(0xFF00FF),
    xOffset = -20,
    yOffset = 10,
    thickness = 2
  },
  {
    radius = Screen.getWidthPart(0.1),
    color = ColorUtils.colorHex24ToStd(0x00FFFF),
    xOffset = 20,
    yOffset = -10,
    thickness = 2
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createTapEffectCircle(descriptor)
  local circle = FilledCircle.new(descriptor.radius, descriptor.color)
  local outline = HollowCircle.new(descriptor.radius, descriptor.color, descriptor.thickness)

  circle:setOpacity(FILL_VISIBLE_OPACITY)
  circle:insertAtCenter(outline)
  circle.outline = outline
  circle:setBlendMode(MOAIProp2D.BLEND_ADD)

  return circle
end

local function startTapEffectOverKey(key)
  local tapAction = AHSpawn.new()
  for _, tapEffectCircleDescriptor in ipairs(TAP_EFFECT_CIRCLE_DESCRIPTORS) do
    local tapEffectCircle = createTapEffectCircle(tapEffectCircleDescriptor)

    tapEffectCircle:setLoc(key:getWidthPart(0.5) + tapEffectCircleDescriptor.xOffset,
                           key:getHeightPart(0.5) + tapEffectCircleDescriptor.yOffset)
    tapEffectCircle:setScl(0.0)
    key:insert(tapEffectCircle)

    tapAction:addAction(AHSequence.new(
      AHSpawn.new(
        AHOpacityTo.new(tapEffectCircle, 0.6, nil, LinusCommon.INVISIBLE_OPACITY),
        AHOpacityTo.new(tapEffectCircle.outline, 0.6, nil, LinusCommon.INVISIBLE_OPACITY),
        AHScaleTo.new(tapEffectCircle, 0.2, AHEaseBackOut.new(3.0), 1.0)
      ),
      AHRemove.new(tapEffectCircle)
    ))
  end

  tapAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusNumberPad:_registerSubmit(key)
  startTapEffectOverKey(key)

  self.super._registerSubmit(self, key)
end

function LinusNumberPad:hide()
  self:setOpacity(LinusCommon.INVISIBLE_OPACITY)
end

function LinusNumberPad:createShowAction()
  local showAction = AHOpacityTo.new(self, 0.3, nil, LinusCommon.VISIBLE_OPACITY)

  return showAction
end

function LinusNumberPad:createHideAction()
  local hideAction = AHOpacityTo.new(self, 0.3, nil, LinusCommon.INVISIBLE_OPACITY)

  return hideAction
end

function LinusNumberPad:createCorrectResponseAction()
  local correctAction = AHSequence.new(
    AHColorTo.new(self, 0.1, nil, LinusCommon.COLORS.CYAN),
    AHDelay.new(0.5),
    AHColorTo.new(self, 0.1, nil, LinusCommon.COLORS.MAGENTA)
  )

  return correctAction
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function LinusNumberPad.new()
  local self = NumberPad.new(LinusCommon.COLORS.MAGENTA, KEY_FONT)
  Mixin.mix(self, LinusNumberPad)

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)
  self:linkRawColorAttrToChildren(isRecursive)

  self:setColor(unpack(LinusCommon.COLORS.MAGENTA))

  return self
end
