--- LinusOverExposure
-- @author Created by Jay Austria on November 1, 2015.

class("LinusOverExposure")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local UPSCALE_AMOUNT = 4
local OPACITY_LOW_EXPOSURE = 0.0
local OPACITY_FULL_EXPOSURE = 0.3
local OPACITY_DEFAULT = OPACITY_LOW_EXPOSURE
local GAME_OVEREXPOSURE_COLOR = ColorUtils.colorHex24ToStd(0xB266CC)
local CORRECT_OVEREXPOSURE_COLOR = LinusCommon.COLORS.CYAN

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------


local function createOverExposureSprite(color)
  local overExposureSprite = Sprite.newWithName("exposure.png")
  overExposureSprite:setColor(unpack(color))
  overExposureSprite:setScl(UPSCALE_AMOUNT)
  overExposureSprite:setBlendMode(MOAIProp2D.BLEND_ADD)

  overExposureSprite:setOpacity(OPACITY_DEFAULT)

  return overExposureSprite
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusOverExposure:createCorrectResponseOverExposeAction()
  local correctOverExposeAction = AHSequence.new(
    AHOpacityTo.new(self.correctOverExposureSprite, 0.10, nil, 0.3),
    AHDelay.new(0.28),
    AHOpacityTo.new(self.correctOverExposureSprite, 0.01, nil, 0.15),
    AHOpacityTo.new(self.correctOverExposureSprite, 0.01, nil, 0.3),
    AHOpacityTo.new(self.correctOverExposureSprite, 0.01, nil, 0.15),
    AHOpacityTo.new(self.correctOverExposureSprite, 0.01, nil, 0.3),
    AHOpacityTo.new(self.correctOverExposureSprite, 1.5, AHEaseExponentialOut.new(), 0.0)
  )

  return correctOverExposeAction
end

function LinusOverExposure:startQuickGlowAction()
  local quickGlowAction = AHSequence.new(
    AHOpacityTo.new(self.gameOverExposureSprite, 0.10, nil, OPACITY_FULL_EXPOSURE),
    AHOpacityTo.new(self.gameOverExposureSprite, 1.5, AHEaseExponentialOut.new(), OPACITY_DEFAULT)
  )

  return quickGlowAction:start()
end

function LinusOverExposure:createOutroOverExposeAction()
  local outroOverExposeAction = AHOpacityTo.new(self, 0.3, nil, OPACITY_FULL_EXPOSURE)

  return outroOverExposeAction
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------


function LinusOverExposure.new()
  local gameOverExposureSprite = createOverExposureSprite(GAME_OVEREXPOSURE_COLOR)
  local width, height = gameOverExposureSprite:getDims()

  local self = Group.new(width, height)
  Mixin.mix(self, LinusOverExposure)

  self:insertAtCenter(gameOverExposureSprite)
  self.gameOverExposureSprite = gameOverExposureSprite

  local correctOverExposureSprite = createOverExposureSprite(CORRECT_OVEREXPOSURE_COLOR)
  self:insertAtCenter(correctOverExposureSprite)
  self.correctOverExposureSprite = correctOverExposureSprite

  return self
end
