--- LinusRationale
-- @author Created by Jay Austria on November 3, 2015.

class("LinusRationale")

require("LinusCommon")
require("LinusTipPagesTakeaway")
require("LinusTipPagesDouble")
require("LinusTipPagesCountup")
require("text.SizeConstrainedTextArea")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TIP_DESCRIPTORS = {
  TAKEAWAY = {
    text = "TIP_TAKEAWAY",
    pagesObject = LinusTipPagesTakeaway
  },

  DOUBLE = {
    text = "TIP_DOUBLE",
    pagesObject = LinusTipPagesDouble
  },

  COUNTUP = {
    text = "TIP_COUNTUP",
    pagesObject = LinusTipPagesCountup
  }
}
local HEADER_PADDING_Y = Screen.getSafeHeightPart(0.03)
local TIP_TEXT_FONT_SIZE = LinusCommon.getAspectRatioAdjustedFontSize(20)
local TIP_INTERACTABLES_FONT_SIZE = LinusCommon.getAspectRatioAdjustedFontSize(15)
local TIP_TEXT_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, TIP_TEXT_FONT_SIZE)
local TIP_INTERACTABLES_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, TIP_INTERACTABLES_FONT_SIZE)
local TIP_MAX_HEIGHT_PROPORTION = 0.30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createTipPages(self, tipPagesHeight)
  local pagesObject = TIP_DESCRIPTORS[self.type].pagesObject

  local pages = pagesObject.new(tipPagesHeight,
                                self.minuend,
                                self.subtrahend,
                                self.answer)
  pages:initialize()

  return pages
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusRationale:startShowAction()
  local showAction = AHSequence.new(
    AHOpacityTo.new(self, 0.4, nil, LinusCommon.VISIBLE_OPACITY),
    function() self.tipPages:startFirstPage() end
  )

  return showAction:start()
end

function LinusRationale:startDismissAction()
  local dismissAction = self:createDismissAction()

  return dismissAction:start()
end

function LinusRationale:createDismissAction()
  local dismissAction = AHSequence.new(
    AHSpawn.new(
      self.tipPages:createDismissAction(),
      AHOpacityTo.new(self, 0.3, nil, LinusCommon.INVISIBLE_OPACITY)
    ),
    AHRemove.new(self)
  )

  return dismissAction
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local textAreaWidth = math.min(Screen.getWidthPart(0.85), Screen.safeWidth)
  local tipHeader = DynamicTextArea.newSimple(Localizer.get("Misc.TIP"), textAreaWidth, TIP_TEXT_FONT, Alignment.LEFT)
  tipHeader:setAnchorPointTop()
  tipHeader:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  local tipOffsetY = Screen.getSafeHeightPart(0.05)
  local tipLocationY = Screen.safeTop - tipOffsetY
  tipHeader:setLoc(self:getWidthPart(0.5), tipLocationY)
  self:insert(tipHeader)

  local textForTip = Localizer.get(TIP_DESCRIPTORS[self.type].text)
  local tipMaxHeight = Screen.getSafeHeightPart(TIP_MAX_HEIGHT_PROPORTION)
  local baseAttributes = { font = TIP_TEXT_FONT, color = LinusCommon.COLORS.WHITE }
  local tipBody = SizeConstrainedTextArea.new(textForTip, textAreaWidth, tipMaxHeight, baseAttributes, nil, nil, Alignment.LEFT, nil, nil)
  tipBody:setAnchorPointTop()
  tipBody:setLoc(self:getWidthPart(0.5), tipHeader:getYBottom() - HEADER_PADDING_Y)
  self:insert(tipBody)

  local continueLabel = Label.new(Localizer.get("Misc.RETURN_TO_GAME"), TIP_INTERACTABLES_FONT)
  continueLabel:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  continueLabel:setAnchorPointBottom()
  local continueLabelOffsetY = Screen.getSafeHeightPart(0.03)
  local continueLabelLocationY = Screen.safeBottom + continueLabelOffsetY
  continueLabel:setLoc(self:getWidthPart(0.5), continueLabelLocationY)
  self:insert(continueLabel)

  local underlineThickness = LinusCommon.SINGLE_PIXEL_DIMENSION
  local continueUnderline = FilledRectangle.new(continueLabel:getWidth(), underlineThickness)
  local underlineYOffset = continueLabel:getHeightPart(-0.03)
  continueUnderline:setLoc(continueLabel:getWidthPart(0.5), underlineYOffset)
  continueLabel:insert(continueUnderline)
  continueLabel:linkColorAttrToChildren()

  local tipPagesHeight = tipBody:getYBottom() - continueLabel:getYTop()
  local tipPages = createTipPages(self, tipPagesHeight)
  tipPages:setAnchorPointBottom()
  tipPages:setLoc(self:getWidthPart(0.5), continueLabel:getYTop())
  self:insert(tipPages)
  self.tipPages = tipPages

  local expandedWidth = continueLabel:getWidthPart(1.5)
  local expandedHeight = continueLabel:getHeightPart(2.0)
  local continueVirtualButton = Button.addVirtualButton(continueLabel, expandedWidth, expandedHeight, self.dismissalCallback)
  continueVirtualButton:setTouchEnabled(false)

  local delayedButtonEnable = function()
    local enableDelayDuration = 1.0
    waitFor(enableDelayDuration)
    continueVirtualButton:setTouchEnabled(true)
  end
  runInCoroutine(delayedButtonEnable)

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)
end

function LinusRationale.new(type, minuend, subtrahend, answer, dismissalCallback)
  local self = Group.newFullScreen()
  Mixin.mix(self, LinusRationale)

  self.type = type
  self.minuend = minuend
  self.subtrahend = subtrahend
  self.answer = answer
  self.dismissalCallback = dismissalCallback

  self:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  createLayout(self)

  return self
end
