--- LinusRoundIndicator
-- @author Created by Jay Austria on October 20, 2015.

class("LinusRoundIndicator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INITIAL_ROUND_INDEX = 1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getRoundString(round, max)
  return string.format("%d/%d", round, max)
end

local function setRoundNumber(self)
  local indicatorString = getRoundString(self.currentRound, self.roundCount)
  self.indicatorLabel:setText(indicatorString)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusRoundIndicator:incrementRound()
  self.currentRound = self.currentRound + 1

  setRoundNumber(self)
end

function LinusRoundIndicator:getRound()
  return self.currentRound
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function LinusRoundIndicator.new(roundCount)
  local initialLabelString = getRoundString(INITIAL_ROUND_INDEX, roundCount)
  local indicatorLabel = Label.new(initialLabelString, Font.new(FontFaces.DIN_OT_MEDIUM, 16))
  indicatorLabel:setColor(unpack(LinusCommon.UI_COMPONENTS_COLOR_DEFAULT))

  -- Create group from dimensions of an initialized label so StandardUIBuilder can
  -- align properly for intro play in
  local self = Group.new(indicatorLabel:getDims())
  Mixin.mix(self, LinusRoundIndicator)

  self:insertAtCenter(indicatorLabel)
  self.indicatorLabel = indicatorLabel
  self.currentRound = INITIAL_ROUND_INDEX
  self.roundCount = roundCount

  self:linkColorAttrToChildren()
  self:linkOpacityAttrToChildren()

  return self
end
