--- LinusTipPagesCountup
-- @author Created by Jay Austria on November 4, 2015.

class("LinusTipPagesCountup")

require("LinusTipPagesBase")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PROBLEM_SLIDE_Y_OFFSET = Screen.getSafeHeightPart(0.1)
local SPACING_X = Screen.getWidthPart(0.02)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createTipLabel(text, fontSize, opacity)
  opacity = opacity or LinusCommon.VISIBLE_OPACITY

  local label = Label.new(text, Font.new(FontFaces.DIN_OT_MEDIUM, fontSize))
  label:setColor(unpack(LinusCommon.COLORS.MAGENTA))

  label:setOpacity(opacity)

  return label
end

local function createShowActionOnElements(elements)
  local showAction = AHSpawn.new()
  for _, element in ipairs(elements) do
    showAction:addAction(AHOpacityTo.new(element, 0.2, nil, LinusCommon.VISIBLE_OPACITY))
  end

  return AHDelayed.new(0.5, showAction)
end

local function createEquationRow(self, leftValue, rightValue, result)
  local equationRow = Row.new(SPACING_X)
  local equationFontSize = LinusCommon.getAspectRatioAdjustedFontSize(26)

  equationRow.leftLabel = createTipLabel(tostring(leftValue), equationFontSize, LinusCommon.INVISIBLE_OPACITY)
  equationRow.plusLabel = createTipLabel("+", equationFontSize, LinusCommon.INVISIBLE_OPACITY)
  equationRow.rightLabel = createTipLabel(tostring(rightValue), equationFontSize, LinusCommon.INVISIBLE_OPACITY)
  equationRow.equalLabel = createTipLabel("=", equationFontSize, LinusCommon.INVISIBLE_OPACITY)
  equationRow.resultLabel = createTipLabel(tostring(result), equationFontSize, LinusCommon.INVISIBLE_OPACITY)

  equationRow:insert(equationRow.leftLabel)
  equationRow:insert(equationRow.plusLabel)
  equationRow:insert(equationRow.rightLabel)
  equationRow:insert(equationRow.equalLabel)
  equationRow:insert(equationRow.resultLabel)

  table.insert(self.createdElements, equationRow.leftLabel)
  table.insert(self.createdElements, equationRow.plusLabel)
  table.insert(self.createdElements, equationRow.rightLabel)
  table.insert(self.createdElements, equationRow.equalLabel)
  table.insert(self.createdElements, equationRow.resultLabel)

  equationRow:alignChildren()

  equationRow.resultCopy = createTipLabel(tostring(result), equationFontSize, LinusCommon.INVISIBLE_OPACITY)
  equationRow.resultCopy:setLoc(equationRow.resultLabel:getLoc())
  equationRow:insert(equationRow.resultCopy)
  table.insert(self.createdElements, equationRow.resultCopy)

  return equationRow
end

local function createEquationRows(self)
  local minuend = self.minuend
  local minuendString = tostring(minuend)
  local minuendDigitsCount = #minuendString

  local currentCounter = self.subtrahend

  local equationRows = {}

  local rightIndex = 1
  while (rightIndex < minuendDigitsCount) do
    local currentCounterString = tostring(currentCounter)
    local counterDigitIndex = #currentCounterString - rightIndex + 1
    local placeMultiplier = math.pow(10, rightIndex)
    local digit = tonumber(currentCounterString:sub(counterDigitIndex))

    local valueToAdd = (placeMultiplier - digit)

    if (digit ~= 0) then
      local result = currentCounter + valueToAdd
      local willAddingExceedMinuend = (result > minuend)
      if willAddingExceedMinuend then
        break
      end

      local equationRow = createEquationRow(self, currentCounter, valueToAdd, result)
      table.insert(self.createdElements, equationRow)
      table.insert(equationRows, equationRow)

      currentCounter = result
    end

    rightIndex = rightIndex + 1
  end

  if (currentCounter < minuend) then
    local remainder = minuend - currentCounter

    local equationRow = createEquationRow(self, currentCounter, remainder, minuend)
    table.insert(self.createdElements, equationRow)
    table.insert(equationRows, equationRow)
  end

  return equationRows
end

local function createFirstPageElements(self)
  local elementsToShow = {}

  local problemFontSize = LinusCommon.getAspectRatioAdjustedFontSize(27)
  local problemRow = Row.new(SPACING_X)
  problemRow.leftLabel = createTipLabel(tostring(self.minuend), problemFontSize, LinusCommon.INVISIBLE_OPACITY)
  problemRow.subtractLabel = createTipLabel("-", problemFontSize, LinusCommon.INVISIBLE_OPACITY)
  problemRow.rightLabel = createTipLabel(tostring(self.subtrahend), problemFontSize, LinusCommon.INVISIBLE_OPACITY)

  problemRow:insert(problemRow.leftLabel)
  problemRow:insert(problemRow.subtractLabel)
  problemRow:insert(problemRow.rightLabel)
  problemRow:alignChildren()

  problemRow.subtrahendCopy = createTipLabel(tostring(self.subtrahend), problemFontSize, LinusCommon.INVISIBLE_OPACITY)
  problemRow.subtrahendCopy:setLoc(problemRow.rightLabel:getLoc())
  problemRow:insert(problemRow.subtrahendCopy)

  problemRow:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
  table.insert(elementsToShow, problemRow.leftLabel)
  table.insert(elementsToShow, problemRow.subtractLabel)
  table.insert(elementsToShow, problemRow.rightLabel)
  table.insert(elementsToShow, problemRow.subtrahendCopy)
  table.insert(self.createdElements, problemRow.leftLabel)
  table.insert(self.createdElements, problemRow.subtractLabel)
  table.insert(self.createdElements, problemRow.rightLabel)
  table.insert(self.createdElements, problemRow.subtrahendCopy)
  self:insert(problemRow)
  self.problemRow = problemRow

  local equationsColumn = Column.new()
  local equations = {}
  local additionalRows = createEquationRows(self)
  for _, additionalRow in ipairs(additionalRows) do
    equationsColumn:insert(additionalRow)
    table.insert(equations, additionalRow)
  end
  equationsColumn:alignChildren()
  equationsColumn:setAnchorPointTop()
  equationsColumn:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5) + PROBLEM_SLIDE_Y_OFFSET - self:getHeightPart(0.05))
  table.insert(self.createdElements, equationsColumn)
  self:insert(equationsColumn)
  self.equationsColumn = equationsColumn
  self.equations = equations

  local elementsToFadeInAtEnd = {}

  local answerFontSize = LinusCommon.getAspectRatioAdjustedFontSize(34)
  local answerLabel = createTipLabel(tostring(self.answer), answerFontSize, LinusCommon.INVISIBLE_OPACITY)
  answerLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.44))
  table.insert(self.createdElements, answerLabel)
  table.insert(elementsToFadeInAtEnd, answerLabel)
  self:insert(answerLabel)
  self.answerLabel = answerLabel

  local equalLine = FilledRectangle.new(answerLabel:getWidthPart(2.0), LinusCommon.SINGLE_PIXEL_DIMENSION, LinusCommon.COLORS.MAGENTA)
  equalLine:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  equalLine:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.49))
  table.insert(self.createdElements, equalLine)
  table.insert(elementsToFadeInAtEnd, equalLine)
  self:insert(equalLine)
  self.equalLine = equalLine

  local plusFontSize = LinusCommon.getAspectRatioAdjustedFontSize(26)
  local plusLabel = createTipLabel("+", plusFontSize, LinusCommon.INVISIBLE_OPACITY)
  plusLabel:setLoc(equalLine:getXLeft(), self:getHeightPart(0.56))
  table.insert(self.createdElements, plusLabel)
  table.insert(elementsToFadeInAtEnd, plusLabel)
  self:insert(plusLabel)
  self.plusLabel = plusLabel

  local thereforeFontSize = LinusCommon.getAspectRatioAdjustedFontSize(18)
  local thereforeLabel = Label.new(Localizer.get("THEREFORE"), Font.new(FontFaces.DIN_OT_MEDIUM, thereforeFontSize))
  thereforeLabel:setAnchorPointTop()
  thereforeLabel:setLoc(self:getWidthPart(0.5), answerLabel:getYBottom() - self:getHeightPart(0.05))
  thereforeLabel:setColor(unpack(LinusCommon.COLORS.CYAN))
  thereforeLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  self:insert(thereforeLabel)
  table.insert(self.createdElements, thereforeLabel)
  table.insert(elementsToFadeInAtEnd, thereforeLabel)
  self.thereforeLabel = thereforeLabel

  local equationString = self.minuend .. " - " .. self.subtrahend .. " = " .. self.answer
  local finalAnswerFontSize = LinusCommon.getAspectRatioAdjustedFontSize(28)
  local finalAnswerLabel = Label.new(equationString, Font.new(FontFaces.DIN_OT_MEDIUM, finalAnswerFontSize))
  finalAnswerLabel:setAnchorPointTop()
  finalAnswerLabel:setLoc(self:getWidthPart(0.5), thereforeLabel:getYBottom() - self:getHeightPart(0.025))
  finalAnswerLabel:setColor(unpack(LinusCommon.COLORS.WHITE))
  finalAnswerLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  self:insert(finalAnswerLabel)
  table.insert(self.createdElements, finalAnswerLabel)
  table.insert(elementsToFadeInAtEnd, finalAnswerLabel)
  self.finalAnswerLabel = finalAnswerLabel

  self.elementsToShow = elementsToShow
  self.elementsToFadeInAtEnd = elementsToFadeInAtEnd
end

local function createFirstPageAction(self)
  local pageAction = createShowActionOnElements(self.elementsToShow)

  return pageAction
end

local function createWaterfallAction(elementsToFadeOut, elementsToShow, fallingElement, targetElement, x, y)
  local fadeAction = AHSpawn.new()
  local fadeOpacity = 0.4
  local fadeDuration = 0.2
  for _, element in ipairs(elementsToFadeOut) do
    fadeAction:addAction(AHOpacityTo.new(element, fadeDuration, nil, fadeOpacity))
  end

  local showAction = AHSpawn.new()
  for _, element in ipairs(elementsToShow) do
    showAction:addAction(AHOpacityTo.new(element, 0.2, nil, LinusCommon.VISIBLE_OPACITY))
  end

  local elementScaleDown = targetElement:getWidth() / fallingElement:getWidth()

  local waterfallAction = AHSpawn.new(
    fadeAction,
    AHScaleTo.new(fallingElement, 0.2, AHEaseSimpleInOut.new(), elementScaleDown),
    AHDelayed.new(0.5, AHOpacityTo.new(fallingElement, 0.1, nil, LinusCommon.INVISIBLE_OPACITY)),
    AHSequence.new(
      AHRotationBy.new(fallingElement, 0.2, AHEaseSimpleInOut.new(), -30),
      AHRotationBy.new(fallingElement, 0.4, AHEaseSimpleInOut.new(), 30)
    ),
    AHLocationTo.new(fallingElement, 0.5, AHEaseSimpleInOut.new(), x, y),
    AHDelayed.new(0.4, showAction)
  )

  return waterfallAction
end

local function createSecondPageAction(self)
  local firstEquation = self.equations[1]

  local pageAction = AHSequence.new(
    AHLocationYBy.new(self.problemRow, 0.5, AHEaseSimpleInOut.new(), PROBLEM_SLIDE_Y_OFFSET)
  )

  local elementsToFadeOut = {
    self.problemRow.leftLabel,
    self.problemRow.subtractLabel,
    self.problemRow.rightLabel
  }
  local elementsToShow = {
    firstEquation.leftLabel,
    firstEquation.plusLabel,
    firstEquation.rightLabel,
    firstEquation.equalLabel,
    firstEquation.resultLabel,
    firstEquation.resultCopy
  }
  local fallingElement = self.problemRow.subtrahendCopy
  local targetElement = firstEquation.leftLabel

  local columnX, columnY = self.equationsColumn:transformLoc(firstEquation, firstEquation.leftLabel:getLoc())
  columnY = columnY - PROBLEM_SLIDE_Y_OFFSET

  local worldX, worldY = self:transformLoc(self.equationsColumn, columnX, columnY)
  local x, y = self.problemRow:transformLoc(self, worldX, worldY)
  local initialWaterfallAction = createWaterfallAction(elementsToFadeOut, elementsToShow, fallingElement, targetElement, x, y)
  pageAction:addAction(initialWaterfallAction)

  for equationIndex = 2, #self.equations do
    local topEquation = self.equations[equationIndex - 1]
    local bottomEquation = self.equations[equationIndex]

    elementsToFadeOut = {
      topEquation.leftLabel,
      topEquation.plusLabel,
      topEquation.rightLabel,
      topEquation.equalLabel,
      topEquation.resultLabel
    }
    elementsToShow = {
      bottomEquation.leftLabel,
      bottomEquation.plusLabel,
      bottomEquation.rightLabel,
      bottomEquation.equalLabel,
      bottomEquation.resultLabel,
      bottomEquation.resultCopy
    }
    fallingElement = topEquation.resultCopy
    targetElement = bottomEquation.leftLabel

    x, y = topEquation:transformLoc(bottomEquation, bottomEquation.leftLabel:getLoc())
    initialWaterfallAction = createWaterfallAction(elementsToFadeOut, elementsToShow, fallingElement, targetElement, x, y)
    pageAction:addAction(AHDelayed.new(0.2, initialWaterfallAction))
  end

  return pageAction
end

local function transitionToFirstPage(self)
  self:removeCreatedElements()
  createFirstPageElements(self)

  self.mostRecentPageIndex = 1

  return createFirstPageAction(self):start()
end

local function transitionToSecondPage(self)
  local pageAction = AHSequence.new()

  if self.mostRecentPageIndex == 3 then
    self:removeCreatedElements()
    createFirstPageElements(self)

     -- IMPORTANT. First page actions are super buggy for some reason without this
    waitForFrames(1)

    pageAction:addAction(createFirstPageAction(self))
  end

  pageAction:addAction(createSecondPageAction(self))

  self.mostRecentPageIndex = 2

  return pageAction:start()
end

local function createFadeAllExceptAdditionsAction(self)
  local duration = 0.2
  local fadeAction = AHSpawn.new(
    AHOpacityTo.new(self.problemRow.leftLabel, duration, nil, LinusCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(self.problemRow.subtractLabel, duration, nil, LinusCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(self.problemRow.rightLabel, duration, nil, LinusCommon.INVISIBLE_OPACITY)
  )

  for _, equation in ipairs(self.equations) do
      fadeAction:addAction(AHOpacityTo.new(equation.leftLabel, duration, nil, LinusCommon.INVISIBLE_OPACITY))
      fadeAction:addAction(AHOpacityTo.new(equation.plusLabel, duration, nil, LinusCommon.INVISIBLE_OPACITY))
      fadeAction:addAction(AHOpacityTo.new(equation.resultLabel, duration, nil, LinusCommon.INVISIBLE_OPACITY))
      fadeAction:addAction(AHOpacityTo.new(equation.equalLabel, duration, nil, LinusCommon.INVISIBLE_OPACITY))
      fadeAction:addAction(AHOpacityTo.new(equation.resultCopy, duration, nil, LinusCommon.INVISIBLE_OPACITY))

      -- right labels fade to visible
      fadeAction:addAction(AHOpacityTo.new(equation.rightLabel, duration, nil, LinusCommon.VISIBLE_OPACITY))
  end

  return fadeAction
end

local function createOrganizeAdditionsAction(self)
  local additions = {}
  for _, equation in ipairs(self.equations) do
    local addition = equation.rightLabel
    table.insert(additions, addition)
  end

  local organizeAction = AHSpawn.new()
  for additionIndex, addition in ipairs(additions) do
    self:takeChild(addition)
    table.insert(self.createdElements, addition)

    local desiredX = self.answerLabel:getXRight()
    local desiredY = self:getHeightPart(0.56) + (#additions - additionIndex) * self:getHeightPart(0.10)
    addition.desiredX = desiredX
    addition.desiredY = desiredY

    organizeAction:addAction(AHSpawn.new(
      AHAnchorPointTo.new(addition, 0.5, AHEaseSimpleInOut.new(), 1.0, 0.5),
      AHLocationTo.new(addition, 0.5, AHEaseSimpleInOut.new(), addition.desiredX, addition.desiredY),
      AHScaleTo.new(addition, 0.5, AHEaseSimpleInOut.new(), 1.3)
    ))
  end

  return organizeAction
end

local function transitionToThirdPage(self)
  local pageAction = AHSequence.new(
    createFadeAllExceptAdditionsAction(self),
    createOrganizeAdditionsAction(self),
    createShowActionOnElements(self.elementsToFadeInAtEnd)
  )

  self.mostRecentPageIndex = 3

  return pageAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusTipPagesCountup:_getPageFunctions()
  return {
    function() return transitionToFirstPage(self) end,
    function() return transitionToSecondPage(self) end,
    function() return transitionToThirdPage(self) end
  }
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function LinusTipPagesCountup.new(tipPagesHeight, minuend, subtrahend, answer)
  local self = LinusTipPagesBase.new(tipPagesHeight, minuend, subtrahend, answer)
  Mixin.mix(self, LinusTipPagesCountup)

  self.mostRecentPageIndex = nil

  return self
end
