--- LinusTipPagesDouble
-- @author Created by Jay Austria on November 4, 2015.

class("LinusTipPagesDouble")

require("LinusTipPagesBase")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local NUMBER_FONT_SIZE = LinusCommon.getAspectRatioAdjustedFontSize(33)
local NUMBER_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, NUMBER_FONT_SIZE)
local EQUATION_SLIDE_Y_AMOUNT = Screen.getSafeHeightPart(0.15)


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function transitionToFirstPage(self)
  self:removeCreatedElements()

  local elementsToShow = {}

   -- makes equation look better. Without it, there right side of the minuend string seems cramped
  local hackedSpacing = 10
  local equationRow = Row.new(hackedSpacing)

  local minuendString = tostring(self.minuend) .. " - "
  local minuendLabel = Label.new(minuendString, NUMBER_FONT)
  minuendLabel:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  minuendLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  table.insert(elementsToShow, minuendLabel)
  table.insert(self.createdElements, minuendLabel)
  equationRow:insert(minuendLabel)
  self.minuendLabel = minuendLabel

  local subtrahendString = tostring(self.subtrahend)
  local subtrahendLabel = Label.new(subtrahendString, NUMBER_FONT)
  subtrahendLabel:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  subtrahendLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  table.insert(elementsToShow, subtrahendLabel)
  table.insert(self.createdElements, subtrahendLabel)
  equationRow:insert(subtrahendLabel)

  equationRow:alignChildren()

  equationRow:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.8))
  self:insert(equationRow)
  table.insert(self.createdElements, equationRow)
  self.equationRow = equationRow

  local subtrahendWorldX, _ = self:transformLoc(equationRow, subtrahendLabel:getLoc())

  local paddingY = -self:getHeightPart(0.03)

  local multiplyLabel = Label.new("x 2", NUMBER_FONT)
  multiplyLabel:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  multiplyLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  multiplyLabel:setAnchorPointRight()
  multiplyLabel:setLoc(subtrahendWorldX + subtrahendLabel:getWidthPart(0.5), equationRow:getYBottom() + paddingY)
  self:insert(multiplyLabel)
  table.insert(self.createdElements, multiplyLabel)
  self.multiplyLabel = multiplyLabel

  local equalLine = FilledRectangle.new(multiplyLabel:getWidthPart(2.0), LinusCommon.SINGLE_PIXEL_DIMENSION)
  equalLine:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  equalLine:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  equalLine:setLoc(subtrahendWorldX, multiplyLabel:getYBottom() + paddingY)
  self:insert(equalLine)
  table.insert(self.createdElements, equalLine)
  self.equalLine = equalLine

  local answerString = tostring(self.minuend)
  local answerLabel = Label.new(answerString, NUMBER_FONT)
  answerLabel:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  answerLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  answerLabel:setAnchorPointTop()
  answerLabel:setLoc(subtrahendWorldX, equalLine:getYBottom() + paddingY)
  self:insert(answerLabel)
  table.insert(self.createdElements, answerLabel)
  self.answerLabel = answerLabel

  local thereforeFontSize = LinusCommon.getAspectRatioAdjustedFontSize(18)
  local thereforeLabel = Label.new(Localizer.get("THEREFORE"), Font.new(FontFaces.DIN_OT_MEDIUM, thereforeFontSize))
  thereforeLabel:setAnchorPointTop()
  thereforeLabel:setLoc(self:getWidthPart(0.5), answerLabel:getYBottom() + paddingY)
  thereforeLabel:setColor(unpack(LinusCommon.COLORS.CYAN))
  thereforeLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  self:insert(thereforeLabel)
  table.insert(self.createdElements, thereforeLabel)
  self.thereforeLabel = thereforeLabel

  local equationString = self.minuend .. " - " .. self.subtrahend .. " = " .. self.answer
  local finalAnswerFontSize = LinusCommon.getAspectRatioAdjustedFontSize(28)
  local finalAnswerLabel = Label.new(equationString, Font.new(FontFaces.DIN_OT_MEDIUM, finalAnswerFontSize))
  finalAnswerLabel:setAnchorPointTop()
  finalAnswerLabel:setLoc(self:getWidthPart(0.5), thereforeLabel:getYBottom() - self:getHeightPart(0.025))
  finalAnswerLabel:setColor(unpack(LinusCommon.COLORS.WHITE))
  finalAnswerLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  self:insert(finalAnswerLabel)
  table.insert(self.createdElements, finalAnswerLabel)
  self.finalAnswerLabel = finalAnswerLabel

  local originX, originY = equationRow:getLoc()
  equationRow:setLoc(originX, originY - EQUATION_SLIDE_Y_AMOUNT)

  local showAction = AHSpawn.new()
  for _, element in ipairs(elementsToShow) do
    showAction:addAction(AHDelayed.new(0.5, AHOpacityTo.new(element, 0.2, nil, LinusCommon.VISIBLE_OPACITY)))
  end

  return showAction:start()
end

local function transitionToSecondPage(self)
  local pageAction = AHSequence.new(
    AHSpawn.new(
      AHLocationYBy.new(self.equationRow, 0.4, AHEaseSimpleInOut.new(), EQUATION_SLIDE_Y_AMOUNT),
      AHColorTo.new(self.minuendLabel, 0.4, nil, LinusCommon.COLORS.DIMMED_MAGENTA),
      AHDelayed.new(0.5, AHSpawn.new(
        AHOpacityTo.new(self.multiplyLabel, 0.2, nil, LinusCommon.VISIBLE_OPACITY),
        AHOpacityTo.new(self.equalLine, 0.2, nil, LinusCommon.VISIBLE_OPACITY)
      ))
    ),
    AHDelayed.new(0.5, AHSpawn.new(
      AHOpacityTo.new(self.answerLabel, 0.1, nil, LinusCommon.VISIBLE_OPACITY)
    )),
    AHDelayed.new(0.5, AHSpawn.new(
      AHOpacityTo.new(self.thereforeLabel, 0.2, nil, LinusCommon.VISIBLE_OPACITY),
      AHOpacityTo.new(self.finalAnswerLabel, 0.2, nil, LinusCommon.VISIBLE_OPACITY)
    ))
  )

  return pageAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusTipPagesDouble:_getPageFunctions()
  return {
    function() return transitionToFirstPage(self) end,
    function() return transitionToSecondPage(self) end
  }
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function LinusTipPagesDouble.new(tipPagesHeight, minuend, subtrahend, answer)
  local self = LinusTipPagesBase.new(tipPagesHeight, minuend, subtrahend, answer)
  Mixin.mix(self, LinusTipPagesDouble)

  return self
end
