--- LinusTipPagesTakeaway
-- @author Created by Jay Austria on November 4, 2015.

class("LinusTipPagesTakeaway")

require("LinusTipPagesBase")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local NUMBER_FONT_SIZE = LinusCommon.getAspectRatioAdjustedFontSize(33)
local NUMBER_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, NUMBER_FONT_SIZE)
local NUMBER_KERNING_X = Screen.getWidthPart(0.015)
local EQUAL_LINE_SPACER_Y_HEIGHT = 10

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createHiddenDigitView(valueString)
  local view = Label.new(valueString, NUMBER_FONT)
  view:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  view:setOpacity(LinusCommon.INVISIBLE_OPACITY)

  return view
end

local function startShowActionOnElements(elements)
  local showAction = AHSpawn.new()
  for _, element in ipairs(elements) do
    showAction:addAction(AHOpacityTo.new(element, 0.2, nil, LinusCommon.VISIBLE_OPACITY))
  end

  return AHDelayed.new(0.5, showAction):start()
end

local function createEquationRow(self)
  local equationRow = Row.new(NUMBER_KERNING_X, Alignment.TOP, Row.GrowLeft)
  local minuendString = tostring(self.minuend)
  local subtrahendString = tostring(self.subtrahend)
  local answerString = tostring(self.answer)

  local minuendDigitCount = #minuendString
  local subtrahendDigitCount = #subtrahendString
  local answerDigitCount = #answerString
  local viewsToShowNow = {}

  for digitIndex = 1, minuendDigitCount do
    local digitIndexFromRight = minuendDigitCount - digitIndex + 1
    local minuendDigit = minuendString:sub(digitIndexFromRight, digitIndexFromRight)
    local minuendDigitView = createHiddenDigitView(minuendDigit)
    table.insert(self.createdElements, minuendDigitView)
    table.insert(viewsToShowNow, minuendDigitView)

    local subtrahendDigitIndexForMinuendDigit = digitIndexFromRight - (minuendDigitCount - subtrahendDigitCount)
    local subtrahendDigitView = nil
    if (subtrahendDigitIndexForMinuendDigit > 0) then
      local subtrahendDigit = subtrahendString:sub(subtrahendDigitIndexForMinuendDigit, subtrahendDigitIndexForMinuendDigit)
      subtrahendDigitView = createHiddenDigitView(subtrahendDigit)
      table.insert(viewsToShowNow, subtrahendDigitView)
      table.insert(self.createdElements, subtrahendDigitView)
    end

    local answerDigitIndexForMinuendDigit = digitIndexFromRight - (minuendDigitCount - answerDigitCount)
    local answerDigitView = nil
    if (answerDigitIndexForMinuendDigit > 0) then
      local answerDigit = answerString:sub(answerDigitIndexForMinuendDigit, answerDigitIndexForMinuendDigit)
      answerDigitView = createHiddenDigitView(answerDigit)
      table.insert(self.createdElements, answerDigitView)
    end

    local digitColumn = Column.new()

    digitColumn:insert(minuendDigitView)

    if subtrahendDigitView then
      digitColumn:insert(subtrahendDigitView)
    else
      digitColumn:insertSpacer(minuendDigitView:getHeight())
    end

    digitColumn:insertSpacer(EQUAL_LINE_SPACER_Y_HEIGHT)

    if answerDigitView then
      digitColumn:insert(answerDigitView)
    else
      digitColumn:insertSpacer(minuendDigitView:getHeight())
    end
    digitColumn:alignChildren()

    equationRow:insert(digitColumn)
  end

  equationRow:alignChildren()

  return equationRow, viewsToShowNow
end

local function transitionToFirstPage(self)
  self:removeCreatedElements()

  local elementsToShow = {}

  local equationRow, viewsVisibleNow = createEquationRow(self)
  equationRow:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.65))
  self:insert(equationRow)

  TableUtils.concatenateArrays(elementsToShow, viewsVisibleNow)

  local equalLine = FilledRectangle.new(equationRow:getWidthPart(2.0), LinusCommon.SINGLE_PIXEL_DIMENSION)
  local rightmostEquationColumn = equationRow.children[1]
  local answerDigitView = rightmostEquationColumn.children[#rightmostEquationColumn.children]
  local _, answerDigitRowY = self:transformLoc(equationRow, answerDigitView:getLoc())
  local answerDigitWorldYTop = answerDigitRowY + answerDigitView:getHeightPart(0.5)
  equalLine:setLoc(self:getWidthPart(0.5), answerDigitWorldYTop)
  equalLine:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  equalLine:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  self:insert(equalLine)
  table.insert(self.createdElements, equalLine)
  table.insert(elementsToShow, equalLine)

  local subtractView = Label.new("-", NUMBER_FONT)
  local leftmostEquationColumn = equationRow.children[#equationRow.children]
  local minuendDigitView = leftmostEquationColumn.children[1]
  local minuendDigitRowX, _ = self:transformLoc(equationRow, minuendDigitView:getLoc())
  subtractView:setLoc(minuendDigitRowX - minuendDigitView:getWidthPart(1.5), answerDigitRowY + minuendDigitView:getHeight() + EQUAL_LINE_SPACER_Y_HEIGHT)
  subtractView:setColor(unpack(LinusCommon.COLORS.MAGENTA))
  subtractView:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  self:insert(subtractView)
  table.insert(self.createdElements, subtractView)
  table.insert(elementsToShow, subtractView)

  self.equationRow = equationRow

  local thereforeFontSize = LinusCommon.getAspectRatioAdjustedFontSize(18)
  local thereforeLabel = Label.new(Localizer.get("THEREFORE"), Font.new(FontFaces.DIN_OT_MEDIUM, thereforeFontSize))
  thereforeLabel:setAnchorPointTop()
  thereforeLabel:setLoc(self:getWidthPart(0.5), equationRow:getYBottom() - self:getHeightPart(0.05))
  thereforeLabel:setColor(unpack(LinusCommon.COLORS.CYAN))
  thereforeLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  self:insert(thereforeLabel)
  table.insert(self.createdElements, thereforeLabel)
  self.thereforeLabel = thereforeLabel

  local equationString = self.minuend .. " - " .. self.subtrahend .. " = " .. self.answer
  local finalAnswerFontSize = LinusCommon.getAspectRatioAdjustedFontSize(28)
  local finalAnswerLabel = Label.new(equationString, Font.new(FontFaces.DIN_OT_MEDIUM, finalAnswerFontSize))
  finalAnswerLabel:setAnchorPointTop()
  finalAnswerLabel:setLoc(self:getWidthPart(0.5), thereforeLabel:getYBottom() - self:getHeightPart(0.025))
  finalAnswerLabel:setColor(unpack(LinusCommon.COLORS.WHITE))
  finalAnswerLabel:setOpacity(LinusCommon.INVISIBLE_OPACITY)
  self:insert(finalAnswerLabel)
  table.insert(self.createdElements, finalAnswerLabel)
  self.finalAnswerLabel = finalAnswerLabel

  return startShowActionOnElements(elementsToShow)
end

local function createMorphColumnColorAction(self, columnIndex, color)
  local equationRow = self.equationRow
  local column = equationRow.children[columnIndex]

  local morphAction = AHSpawn.new()

  for rowIndex = 1, 3 do
    local view = column.children[rowIndex]
    if view then
      morphAction:addAction(AHColorTo.new(view, 0.6, nil, color))
    end
  end

  return morphAction
end

local function createDimAction(self, excludedIndex)
  local dimAction = AHSpawn.new()
  for columnIndex = 1, #self.equationRow.children do
    local destinationColor = nil

    if (columnIndex == excludedIndex) then
      destinationColor = LinusCommon.COLORS.MAGENTA
    else
      destinationColor = LinusCommon.COLORS.DIMMED_MAGENTA
    end

    dimAction:addAction(createMorphColumnColorAction(self, columnIndex, destinationColor))
  end

  return dimAction
end

local function createBrightenAllAction(self)
  local brightenAction = AHSpawn.new()
  for columnIndex = 1, #self.equationRow.children do
    brightenAction:addAction(createMorphColumnColorAction(self, columnIndex, LinusCommon.COLORS.MAGENTA))
  end

  return brightenAction
end

local function createShowDifferenceAction(self, columnIndex)
  local column = self.equationRow.children[columnIndex]
  local answerView = column.children[#column.children]
  local showAction = AHSpawn.new(
    AHOpacityTo.new(answerView, 0.3, nil, LinusCommon.VISIBLE_OPACITY)
  )

  return showAction
end

local function transitionToSecondPage(self)
  local pageAction = AHSequence.new()

  local columnsCount = #self.equationRow.children
  for columnIndex = 1, columnsCount do
    if columnsCount > 1 then
      local dimAction = AHSequence.new(createDimAction(self, columnIndex), AHDelay.new(0.4))
      pageAction:addAction(dimAction)
    end

    local showDifferenceAction = AHSequence.new(createShowDifferenceAction(self, columnIndex), AHDelay.new(0.5))
    pageAction:addAction(showDifferenceAction)
  end

  pageAction:addAction(createBrightenAllAction(self))

  pageAction:addAction(AHDelayed.new(0.5, AHSpawn.new(
    AHOpacityTo.new(self.thereforeLabel, 0.3, nil, LinusCommon.VISIBLE_OPACITY),
    AHOpacityTo.new(self.finalAnswerLabel, 0.3, nil, LinusCommon.VISIBLE_OPACITY)
  )))

  return pageAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LinusTipPagesTakeaway:_getPageFunctions()
  return {
    function() return transitionToFirstPage(self) end,
    function() return transitionToSecondPage(self) end
  }
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function LinusTipPagesTakeaway.new(tipPagesHeight, minuend, subtrahend, answer)
  local self = LinusTipPagesBase.new(tipPagesHeight, minuend, subtrahend, answer)
  Mixin.mix(self, LinusTipPagesTakeaway)

  return self
end
