--- ManslapGame
-- @author Created by alexeychernikov on March 31, 2014.

class("ManslapCorrectionInformationDisplay")
require("ManslapLabelWithContext")
require("ManslapRoundedRectangleSpeechBubble")
require("ManslapColors")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ERROR_LABEL_OFFSET_Y = Screen.getHeightPart(0.04)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getBubbleTextAndBackgroundColor(wasAnsweredCorrectly)
  local textColor = {1, 1, 1}
  local backgroundColor = {0, 0, 0}
  if (wasAnsweredCorrectly) then
    textColor = {0, 0, 0}
    backgroundColor = {1, 1, 1}
  end
  return textColor, backgroundColor
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapCorrectionInformationDisplay:setNeutralState()
  self.wrongLabel:removeStrikethroughForLabel()
  self.wrongLabel:setLabelColor(ManslapColors.NEUTRAL_TEXT_COLOR)
  self.correctLabel:setVisibleRecursive(false)
end

function ManslapCorrectionInformationDisplay:setCorrectionState()
  self.wrongLabel:setLabelColor(ManslapColors.CORRECT_ANSWER_TEXT_COLOR)
  self.wrongLabel:addStrikethroughForLabel(ManslapColors.CORRECT_ANSWER_TEXT_COLOR)
  self.correctLabel:setVisibleRecursive(true)
  local bubbleTextColor, bubbleBackgroundColor = getBubbleTextAndBackgroundColor(true)
  self.correctLabel:setBackgroundAndTextColor(bubbleBackgroundColor, bubbleTextColor)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createCorrectAndWrongLabel(self, information, wasAnsweredCorrectly, correctionY)
  local preContext = information.preContext
  local postContext = information.postContext
  local strikethroughColor = ManslapColors.INCORRECT_ANSWER_COLOR

  if (wasAnsweredCorrectly) then
    preContext = ""
    postContext = ""
    strikethroughColor = ManslapColors.CORRECT_ANSWER_TEXT_COLOR
  end

  local text = string.format("%s <error>%s</error> %s", preContext, information.incorrect, postContext)
  local wrongLabel = ManslapLabelWithContext.new(text, wasAnsweredCorrectly)
  wrongLabel:setLoc(Screen.getWidthPart(0.5), correctionY)
  local strikethrough = wrongLabel:addStrikethroughForLabel(strikethroughColor)
  self:insert(wrongLabel)
  self.wrongLabel = wrongLabel

  local bubbleTextColor, bubbleBackgroundColor = getBubbleTextAndBackgroundColor(wasAnsweredCorrectly)
  local correctLabel = ManslapRoundedRectangleSpeechBubble.new(information.correct, bubbleTextColor, bubbleBackgroundColor)
  self.correctLabel = correctLabel

  wrongLabel:forceNodeUpdateForLocations()
  local x, y = strikethrough:getCenterWorldLoc()
  local yOffset = math.max(strikethrough.offsetForBubble + correctLabel.pointer:getHeight(), ERROR_LABEL_OFFSET_Y)
  local raisedY = y + yOffset
  self:insertAtLocation(correctLabel, x, raisedY)
end

function ManslapCorrectionInformationDisplay.new(information, wasAnsweredCorrectly, correctionY)
  local self = Group.newFullScreen()
  Mixin.mix(self, ManslapCorrectionInformationDisplay)

  createCorrectAndWrongLabel(self, information, wasAnsweredCorrectly, correctionY)

  return self
end
