--- ManslapDynamicTextArea
-- @author Created by Jay Austria on February 22, 2017.

class("ManslapDynamicTextArea")

require("ManslapTextInterpreter")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BASE_ATTRIBUTES = {
  font = Font.new(FontFaces.DIN_OT_LIGHT, 26.0)
}
local LINE_SPACING = Screen.getHeightPart(0.03)

local FULL_OPACITY = 0.8

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function centerAlignIfSingleLine(self)
  local numberOfLines = #self:getViewsByRow()
  if (numberOfLines <= 1) then
    self:setHorizontalAlignment(Alignment.CENTER)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function ManslapDynamicTextArea.new(text, width, attributesForLabels)
  local self = TextView.new(width, Alignment.LEFT, nil, LINE_SPACING)
  Mixin.mix(self, DynamicTextArea)

  local textInterpreter = ManslapTextInterpreter.new(BASE_ATTRIBUTES, attributesForLabels)
  local textModel = TextModel.new(textInterpreter:interpretXmlText(""))
  self:setModel(textModel)

  self.textInterpreter = textInterpreter
  self.textModel = textModel

  self:setText(text)

  centerAlignIfSingleLine(self)

  self:setOpacity(FULL_OPACITY)

  return self
end
