--- ManslapErrorButton
-- @author Created by Aaron Barrett on October 26, 2021.

class("ManslapErrorButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ERROR_TEXT = "ERROR"

local TOUCH_TARGET_MARGIN = 70.0

local RED_COLOR = {0.807, 0.101, 0.110}
local GREEN_COLOR = {0.388, 1.000, 0.737}
local BLACK_COLOR = {0.0, 0.0, 0.0}
local WHITE_COLOR = {1.0, 1.0, 1.0}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setOpacityOnComponents(self, opacity)
  for _, component in ipairs(self.components) do
    component:setOpacity(opacity)
  end
end

local function getFadeAction(self, destinationOpacity)
  local action = AHSpawn.new()
  for _, component in ipairs(self.components) do
    action:addAction(AHOpacityTo.new(component, 0.333, nil, destinationOpacity))
  end
  return action
end

local function setCenterElementsColor(self, color)
  self.label:setColor(unpack(color))
  self.label:setVisible(true)
  self.cross:setColor(unpack(color))
  self.cross:setVisible(true)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapErrorButton:showBlackOutline()
  self.filling:setVisible(false)
  self.outline:setColor(unpack(BLACK_COLOR))
  self.outline:setVisible(true)
  setCenterElementsColor(self, BLACK_COLOR)
end

function ManslapErrorButton:showWhiteOutline()
  self.filling:setVisible(false)
  self.outline:setColor(unpack(WHITE_COLOR))
  self.outline:setVisible(true)
  setCenterElementsColor(self, WHITE_COLOR)
end

function ManslapErrorButton:showGreen()
  self.filling:setColor(unpack(GREEN_COLOR))
  self.filling:setVisible(true)
  self.outline:setVisible(false)
  setCenterElementsColor(self, BLACK_COLOR)
end

function ManslapErrorButton:showRed()
  self.filling:setColor(unpack(RED_COLOR))
  self.filling:setVisible(true)
  self.outline:setVisible(false)
  setCenterElementsColor(self, WHITE_COLOR)
end

function ManslapErrorButton:hide()
  for _, component in ipairs(self.components) do
    component:setVisible(false)
  end
end

function ManslapErrorButton:showAppearAnimation()
  return getFadeAction(self, 1.0):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ManslapErrorButton.new()
  local outline = Sprite.newWithName("error_button_outline.png")

  local self = Group.new(outline:getWidth() + TOUCH_TARGET_MARGIN, outline:getHeight() + TOUCH_TARGET_MARGIN)
  Mixin.mix(self, ManslapErrorButton)

  local filling = Sprite.newWithName("error_button_filling.png")

  local row = Row.new(12.0)
  local label = Label.new(ERROR_TEXT, Font.new(FontFaces.DIN_OT_BOLD, 18.0))
  local cross = Sprite.newWithName("validity_cross.png")
  cross:setScl(0.9)
  row:insert(cross)
  row:insert(label)
  row:alignChildren()

  self:insertAtCenter(filling)
  self:insertAtCenter(outline)
  self:insertAtCenter(row)

  self.filling = filling
  self.outline = outline
  self.label = label
  self.cross = cross

  self.components = {filling, outline, cross, label}

  self:showWhiteOutline()
  setOpacityOnComponents(self, 0.0)

  return self
end
