--- ManslapGame
-- @author Created by alexeychernikov on March 31, 2014.

class("ManslapLabelWithContext")

require("ManslapColors")
require("ManslapDynamicTextArea")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BLACK_COLOR = ManslapColors.FULL_BLACK

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getErrorLabels(self)
  local errorLabels = {}
  for _, view in ipairs(self.itemViews) do
    local itemForView = self.itemsForViews[view]
    if (itemForView:getAttribute("isError") == true) then
      table.insert(errorLabels, view)
    end
  end

  assert(#errorLabels > 0, "No error label found in ManslapLabelWithContext")

  return errorLabels
end

local function removeStrikethrough(label)
  local strikeThrough = label.strikeThrough
  if (strikeThrough) then
    if (strikeThrough.labels) then
      for _, label in pairs(strikeThrough.labels) do
        label.strikeThrough = nil
      end
    end
    strikeThrough:removeSelf()
    label.strikeThrough = nil
  end
end

local function addStrikethroughForLabelList(labels, color, width)
  local x0 = labels[1]:getX()
  local x1 = labels[#labels]:getX()
  local w1, h1 = labels[#labels]:getDims()
  local x = x1 + w1

  local w = x - x0
  local margin = 6
  local strikeThrough = FilledRectangle.new(w + margin, width, color)
  strikeThrough:setAnchorPointLeft()

  strikeThrough:setLoc(-margin/2, h1/2 - 3)

  labels[1]:insert(strikeThrough)
  strikeThrough.labels = labels
  for _, label in pairs(labels) do
    label.strikeThrough = strikeThrough
  end

  strikeThrough.offsetForBubble = h1/2 + 3

  return strikeThrough
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapLabelWithContext:forceNodeUpdateForLocations()
  self:forceUpdate()

  for _, errorLabel in ipairs(self.errorLabels) do
    errorLabel:forceUpdate()

    if (errorLabel.strikeThrough) then
      errorLabel.strikeThrough:forceUpdate()
    end
  end
end

function ManslapLabelWithContext:addStrikethroughForLabel(color)
  return addStrikethroughForLabelList(self.errorLabels, color, 1.5)
end

function ManslapLabelWithContext:removeStrikethroughForLabel()
  local errorLabel = self.errorLabels[1]
  removeStrikethrough(errorLabel)
end

function ManslapLabelWithContext:setLabelColor(color)
  for _, errorLabel in ipairs(self.errorLabels) do
    errorLabel:clearAttrLink(MOAIColor.INHERIT_COLOR)
    errorLabel:setColor(unpack(color))
  end
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------


function ManslapLabelWithContext.new(text, wasAnsweredCorrectly)
  local errorFontFace = nil
  local errorFontColor = nil
  if (wasAnsweredCorrectly) then
    errorFontFace = FontFaces.DEFAULT_FONT
    errorFontColor = ManslapColors.CORRECT_ANSWER_TEXT_COLOR
  else
    errorFontFace = FontFaces.DIN_OT_MEDIUM
    errorFontColor = ManslapColors.INCORRECT_ANSWER_COLOR
  end

  local width = Screen.getWidthPart(0.82)
  local attributesForLabels = {
    error = {font = Font.new(errorFontFace, 26), isError = true}
  }

  local self = ManslapDynamicTextArea.new(text, width, attributesForLabels)
  Mixin.mix(self, ManslapLabelWithContext)

  self:setColor(unpack(BLACK_COLOR))

  self.errorLabels = getErrorLabels(self)

  self:setLabelColor(errorFontColor)

  return self
end
