--- ManslapRationaleTextArea
-- @author Created by Aaron Barrett on October 27, 2021.

class("ManslapRationaleTextArea")

require("ManslapColors")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BLACK_COLOR = ManslapColors.FULL_BLACK
local CORRECT_COLOR = ManslapColors.RATIONALE_CORRECT_COLOR
local INCORRECT_COLOR = ManslapColors.RATIONALE_INCORRECT_COLOR

local FULL_OPACITY = 0.8

local STANDARD_FONT_SIZE = 23.0
local TABLET_FONT_SIZE = 18.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getFontSize()
  if (Screen.isTabletAspectRatio()) then
    return TABLET_FONT_SIZE
  else
    return STANDARD_FONT_SIZE
  end
end

local function getBaseAttributes()
  local fontSize = getFontSize()
  return {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, fontSize),
    color = BLACK_COLOR
  }
end

local function getAttributesForLabels()
  local fontSize = getFontSize()
  return {
    correct = {
      font = Font.new(FontFaces.DIN_OT_BOLD, fontSize),
      color = CORRECT_COLOR
    },
    incorrect = {
      font = Font.new(FontFaces.DIN_OT_BOLD, fontSize),
      color = INCORRECT_COLOR
    },
    i = {
      font = Font.new(FontFaces.DIN_OT_MEDIUM_ITALIC, fontSize),
      color = BLACK_COLOR
    }
  }
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapRationaleTextArea:showAppearAnimation()
  local riseDistance = Screen.getSafeHeightPart(0.2)
  self:offsetY(-riseDistance)

  local ease = AHEaseBezier.newFlat({0.5, 0.4, 0.0, 1.0})

  return AHSpawn.new(
    AHLocationYBy.new(self, 0.5, ease, riseDistance),
    AHOpacityTo.new(self, 0.5, ease, FULL_OPACITY)
  ):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ManslapRationaleTextArea.new(text, width)
  local baseAttributes = getBaseAttributes()
  local attributesForLabels = getAttributesForLabels()

  local self = DynamicTextArea.new(text, width, baseAttributes, attributesForLabels)
  Mixin.mix(self, ManslapRationaleTextArea)

  self:setOpacity(0.0)

  return self
end
