--- ManslapRewindEffectGroup
-- @author Created by alexeychernikov on March 31, 2014.

class("ManslapRewindEffectGroup")
require("ManslapDistort")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local POSTFIX_REGEXP = "%d*\.png"
local FRAMERATE = 25
local DISTORTION_PARAMS =
{
  width = Screen.width,
  height = Screen.height/8 + 20,
  bigDistortion = 0.0,
    distortionMagnitude = 0.05,
  smallDistortion = 0.05,
  speed = 10.0,
  rollSpeed = 0.0
}

local STRIP_Y_POSITIONS =
{
  Screen.height/4,
  Screen.height/2 + Screen.height/8,
  Screen.height * 3/4
}

local VISIBLE_COLOR = {1, 1, 1, 1}
local INVISIBLE_COLOR = {1, 1, 1, 0}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createAnimationLoop(firstFrameName)
  local namePrefix = string.gsub(firstFrameName, POSTFIX_REGEXP, "")
  local view = Sprite.newWithName(firstFrameName)
  local anim = Animation.newWithPrefix(namePrefix)
  local loop = AHLoop.new(AHAnimate.new(view, anim, FRAMERATE))
  view:setColor(0, 0, 0)

  return view, loop
end

local function createFatStrip()
  return createAnimationLoop("fat/vhsFat_000.png")
end

local function createSkinnyStrip()
  return createAnimationLoop("skinny/vhsSkinny_000.png")
end

local function createStrips()
  local stripsGroup = Group.newFullScreen()
  local stripCount = 3
  local strips = {}

  for i = 1, stripCount do
    if (i % 2 == 0) then
      strips[#strips + 1] = {createSkinnyStrip()}
    else
      strips[#strips + 1] = {createFatStrip()}
    end
    local strip = strips[#strips][1]
    stripsGroup:insert(strip)
    strip:setLoc(Screen.width/2, STRIP_Y_POSITIONS[i])
  end
  return stripsGroup, strips
end

local function initDistortion(self)
  local distortion = ManslapDistort.new(DISTORTION_PARAMS)
  self:insertAtCenter(distortion)

  self.distortion = distortion
  distortion:setUndistorted()
end

local function seekStripColor(strip, color)
  if (strip.animation) then
    strip.animation:stop()
  end

  strip.animation = AHColorTo.new(strip, 0.3, nil, color)
  strip.animation:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapRewindEffectGroup:setGroupToDistort(groupToDistort)
  self.distortion:setRenderedGroup(groupToDistort)
  local w, h = groupToDistort:getDims()
  self.distortion:setSize(w, h)

  self.groupToDistort = groupToDistort
end

function ManslapRewindEffectGroup:start()
  if (self.distortionAction) then
    self.distortionAction:stop()
  end

  self.stripsGroup:setVisibleRecursive(true)
  for _, stripInfo in ipairs(self.strips) do
    seekStripColor(stripInfo[1], VISIBLE_COLOR)
    stripInfo[2]:start()
  end

  local time = 0
  local getT = function()
    return time
  end

  local setT = function(distortion, newT)
    time = newT
    distortion:setTime(time)
  end

  self.distortion:windUp()
  if (self.distortion) then
    self.distortionAction = AHNonAttributeTo.new(self.distortion, 1.4, getT, setT, nil, 0.085):start()
  end
end

function ManslapRewindEffectGroup:stop()
  for _, stripInfo in ipairs(self.strips) do
    seekStripColor(stripInfo[1], INVISIBLE_COLOR)
  end
  self.distortion:windDown()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local stripsGroup, strips = createStrips()
  stripsGroup:setVisibleRecursive(false)
  self:insertAtCenter(stripsGroup)

  self.stripsGroup = stripsGroup
  self.strips = strips
end

function ManslapRewindEffectGroup.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ManslapRewindEffectGroup)
  createLayout(self)

  initDistortion(self)

  return self
end
