--- ManslapGame
-- @author Created by alexeychernikov on March 31, 2014.

class("ManslapRewindSign")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GROUP_WIDTH = 100
local GROUP_HEIGHT = 50

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createLeftFacingTriangle(width, height, color)
  local x0, y0 = 0, height/2
  local x1, y1 = width, height
  local x2, y2 = width, 0
  local triangle = FilledTriangle.new(x0, y0, x1, y1, x2, y2, color)
  return triangle
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapRewindSign:blink()
  local show = AHCallLogic.new(function() self:show()  end)
  local hide = AHCallLogic.new(function() self:hide()  end)
  local delay = AHDelay.new(0.08)
  local hideShow = AHSequence.new(delay, hide, delay, show)
  local blinker = AHLoop.new(hideShow)
  blinker:start()

  self.blinker = blinker
end

function ManslapRewindSign:stopBlink()
  if (self.blinker) then
    self.blinker:stop()
    self.blinker = nil
  end
  self:show()
end

function ManslapRewindSign:show()
  self:setVisibleRecursive(true)
end

function ManslapRewindSign:hide()
  self:setVisibleRecursive(false)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, color)
  local width, height = self:getDims()
  local t1, t2 = createLeftFacingTriangle(width/2, height, color), createLeftFacingTriangle(width/2, height, color)

  t1:setLoc(width/4, 0)
  t2:setLoc(3 * width/4, 0)

  self:insert(t1)
  self:insert(t2)
end

function ManslapRewindSign.new(color)
  local self = Group.new(GROUP_WIDTH, GROUP_HEIGHT)
  Mixin.mix(self, ManslapRewindSign)
  createLayout(self, color)

  return self
end
