--- ManslapGame
-- @author Created by alexeychernikov on March 31, 2014.


class("ManslapRoundGenerator")

require("data.generators.UtilityGenerators")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CORRECT_ANSWER_MARGIN_SECONDS = 3
local REWIND_BEFORE_PRONOUNCE_POINT = 4

local CUTOFF_INFO = {
  {endDifficulty = 0.225, contentDifficulty = "Easy"},
  {endDifficulty = 0.45, contentDifficulty = "Medium"},
  {endDifficulty = 1.0, contentDifficulty = "Hard"}
}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function assignEndAndRewindTimesWhereApplicable(round)
  for _, info in pairs(round.correctionTimes) do
    info.beginTime = info.beginTime or info.time
    info.endTime = info.endTime or (info.beginTime + CORRECT_ANSWER_MARGIN_SECONDS)
    info.rewindToTime = info.rewindToTime or (info.beginTime - REWIND_BEFORE_PRONOUNCE_POINT)
  end
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ManslapRoundGenerator.new()
  local difficultyKeyFrames = DifficultyCurves.getHardCutoffKeyframes(CUTOFF_INFO)
  local difficultyNameChooser = KeyFrameInterpolator.new(difficultyKeyFrames, GameSession.getDifficulty())
  local conceptGenerator = GameSession.conceptGenerator

  conceptGenerator:setDifficultyNameChooser(difficultyNameChooser)
  return TransformationGenerator.new(conceptGenerator, function(round)
    assignEndAndRewindTimesWhereApplicable(round)
    return round
  end)
end
