--- ManslapGame
-- @author Created by alexeychernikov on March 31, 2014.

class("ManslapRoundedRectangleSpeechBubble")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DEFAULT_BACKGROUND_COLOR = {1, 1, 1, 1}
local DEFAULT_TEXT_COLOR = {0, 0, 0}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapRoundedRectangleSpeechBubble:setBackgroundAndTextColor(backgroundColor, textColor)
  self:setColor(unpack(backgroundColor))
  self.pointer:setColor(unpack(backgroundColor))
  self.textView:setColor(unpack(textColor))
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createrPointer(width, height, color)
  local p1x = -width / 2
  local p1y = height
  local p2x = 0
  local p2y = 0
  local p3x = width / 2
  local p3y = height
  local pointer = FilledTriangle.new(p1x, p1y, p2x, p2y, p3x, p3y, color)

  return pointer
end

local function createLayout(text, textColor, backgroundColor)
  local textView = Label.new(text, Font.new(FontFaces.DIN_OT_BOLD, 20))
  textView:setColor(unpack(textColor))

  local width, height = textView:getDims()
  local xPadding = 24
  local yPadding = 4
  local pointerW, pointerH = 9, 6

  local self = FilledRoundedRectangle.new(width + xPadding, height + yPadding, 3, backgroundColor)
  local pointer = createrPointer(pointerW, pointerH, backgroundColor)
  pointer:setAnchorPointTop()

  self:insertAtCenter(textView)
  self:insertAtLocationFractional(pointer, 0.5, 0)

  self.textView = textView
  self.pointer = pointer

  return self
end

function ManslapRoundedRectangleSpeechBubble.new(text, textColor, backgroundColor)
  textColor = textColor or DEFAULT_TEXT_COLOR
  backgroundColor = backgroundColor or DEFAULT_BACKGROUND_COLOR

  local self = createLayout(text, textColor, backgroundColor)
  Mixin.mix(self, ManslapRoundedRectangleSpeechBubble)

  return self
end
