--- ManslapSoundVisualizer
-- @author Created by karl on July 22, 2014.

class("ManslapSoundVisualizer")
require("data.visualization.AudioVisualizer")
require("data.generators.CrossfadeGenerator")
require("data.generators.SoundAveragePowerGenerator")
require("data.generators.UtilityGenerators")
require("data.generators.ValueGenerators")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local VISUALIZER_WIDTH = Screen.width
local VISUALIZER_HEIGHT = 80
local MIN_PEAKS = 3
local MAX_PEAKS = 5


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapSoundVisualizer:setPenaltyColor(color)
  for _, visualizer in ipairs(self.visualizers) do
    visualizer:setPenaltyColor(color)
  end
end

function ManslapSoundVisualizer:showPenaltyAnimation()
  for _, visualizer in ipairs(self.visualizers) do
    visualizer:showPenaltyAnimation(MIN_PEAKS, MAX_PEAKS)
  end
end

function ManslapSoundVisualizer:showRewardAnimation()
  for _, visualizer in ipairs(self.visualizers) do
    visualizer:showRewardAnimation()
  end
end

function ManslapSoundVisualizer:showRestoreAnimation()
  for _, visualizer in ipairs(self.visualizers) do
    visualizer:showRestoreAnimation()
  end
end

function ManslapSoundVisualizer:getTotalPointCount()
  local pointCount = self.visualizers[1]:getTotalPointCount()
  return pointCount
end

function ManslapSoundVisualizer:morph(targetShape)
  for _, visualizer in ipairs(self.visualizers) do
    visualizer:morph(targetShape)
  end
end

function ManslapSoundVisualizer:hide()
  self:setVisibleRecursive(false)
end

function ManslapSoundVisualizer:reappear()
  self:setVisibleRecursive(true)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ManslapSoundVisualizer.new(penaltyColor)
  local self = AudioVisualizer.new(VISUALIZER_WIDTH, VISUALIZER_HEIGHT, penaltyColor)
  Mixin.mix(self, ManslapSoundVisualizer)

  return self
end
