--- ManslapGame
-- @author Created by alexeychernikov on March 31, 2014.

class("ManslapStateIndicator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GROUP_WIDTH = 8
local NEUTRAL_COLOR = {0.5, 0.5, 0.5}
local CORRECT_COLOR = {0.4, 0.8, 0.4}
local WRONG_COLOR = {0.8, 0.4, 0.4}

local DEFAULT_COLORS =
{
  neutral = NEUTRAL_COLOR,
  correct = CORRECT_COLOR,
  wrong = WRONG_COLOR
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function activateState(self, stateToActivate)
  for stateName, stateView in pairs(self.states) do
    if (stateName == stateToActivate) then
      stateView:setVisible(true)
    else
      stateView:setVisible(false)
    end
  end
  self.state = stateToActivate
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapStateIndicator:setColorPalette(colors)
  self.states.neutral:setColor(unpack(colors.neutral))
  self.states.wrong:setColor(unpack(colors.wrong))
  self.states.correct:setColor(unpack(colors.correct))
end

function ManslapStateIndicator:setCorrect()
  activateState(self, "correct")
end

function ManslapStateIndicator:setWrong()
  activateState(self, "wrong")
end

function ManslapStateIndicator:setNeutral()
  activateState(self, "neutral")
end

function ManslapStateIndicator:focus()
  self.states.focused:setVisible(true)
  local w, h = Screen.width, Screen.height
  -- Grow the circle to fill the whole screen
  local exactBoundingR = math.sqrt(w * w + h * h)
  local guaranteedBoundingR = exactBoundingR * 1.2
  self.states.focused:setScl(guaranteedBoundingR/self.r)
  self.states["neutral"]:setVisible(true)
end

function ManslapStateIndicator:removeFocus()
  local animTime = 0.3
  local applyFocusRemoval = AHCallLogic.new(function()
    self.focused = false
    if (self.state ~= "neutral") then
      self.states["neutral"]:setVisible(false)
    end
  end)
  local scaleDown = AHScaleTo.new(self.states.focused, animTime, nil, 0, 0)
  local animation = AHSequence.new(scaleDown, applyFocusRemoval)
  animation:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, colors)
  local neutral = Sprite.newWithName("circleStroke.png")
  local wrong = Sprite.newWithName("circleFill.png")
  local correct = Sprite.newWithName("circleFill.png")

  local r = correct:getWidthPart(0.5)

  local focused = FilledCircle.new(r, colors.background)

  wrong:setColor(unpack(colors.wrong))
  correct:setColor(unpack(colors.correct))

  self:insertAtCenter(wrong)
  self:insertAtCenter(correct)
  self:insertAtCenter(focused)
  self:insertAtCenter(neutral)

  self.r = r

  self.states = {neutral = neutral, wrong = wrong, correct = correct, focused = focused}

  wrong:setAttrLink(MOAIColor.ATTR_OPACITY, self)
  correct:setAttrLink(MOAIColor.ATTR_OPACITY, self)
  neutral:setAttrLink(MOAIColor.ATTR_OPACITY, self)
  focused:setAttrLink(MOAIColor.ATTR_OPACITY, self)
end

function ManslapStateIndicator.new(colors)
  colors = colors or DEFAULT_COLORS

  local self = Group.new(GROUP_WIDTH, GROUP_WIDTH)
  Mixin.mix(self, ManslapStateIndicator)
  createLayout(self, colors)

  self:setNeutral()

  return self
end
