--- ManslapGame
-- @author Created by alexeychernikov on March 31, 2014.

class("ManslapStateProgressBar")
require("ManslapStateIndicator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GROUP_WIDTH = Screen.width
local GROUP_HEIGHT = 16

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function animateIn(item)
  if (item.animation) then
    item.animation:stop()
  end
  item:bringToFront()
end

local function getFadeAction(self, destinationOpacity)
  local fadeAction = AHSpawn.new()
  for _, item in ipairs(self.items) do
    fadeAction:addAction(AHOpacityTo.new(item, 0.4, nil, destinationOpacity))
  end
  return fadeAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapStateProgressBar:setIndicatorColors(colors)
  for i, item in ipairs(self.items) do
    item:setColorPalette(colors)
  end
end

function ManslapStateProgressBar:complete(index)
  local item = self.items[index]
  item:setCorrect()

  animateIn(item)
end

function ManslapStateProgressBar:fail(index)
  local item = self.items[index]
  item:setWrong()

  animateIn(item)
end

function ManslapStateProgressBar:focus(index)
  self.items[index]:bringToBack()
  self.items[index]:focus()
end

function ManslapStateProgressBar:removeFocus(index)
  self.items[index]:removeFocus()
end

function ManslapStateProgressBar:neutralize(index)
  self.items[index]:setNeutral()
end

function ManslapStateProgressBar:getStaggeredSuckInAction(dx, dy, dt, durationPerItem)
  local action = AHSpawn.new()
  local delayByIndex =
  {
    5 * dt,
    0 * dt,
    4 * dt,
    2 * dt,
    3 * dt,
    7 * dt,
    6 * dt,
    8 * dt
  }
  for i, item in ipairs(self.items) do
    local delaySeconds = delayByIndex[i] or ((i - 1) * dt)
    local delay = AHDelay.new(delaySeconds)
    local duration = durationPerItem
    local hideDuration = duration/3

    local translation = AHLocationBy.new(item, duration, MOAIEaseType.SMOOTH_EASE_IN, dx, dy)

    local hide = AHSequence.new(AHDelay.new(duration - hideDuration), AHOpacityTo.new(item, hideDuration, nil, 0))
    local suckIn = AHSpawn.new(translation, hide)

    local animation = AHSequence.new(delay, suckIn)
    action:addAction(animation)
  end
  return action
end

function ManslapStateProgressBar:hide()
  for _, item in ipairs(self.items) do
    item:setOpacity(0.0)
  end
end

function ManslapStateProgressBar:reappear()
  for _, item in ipairs(self.items) do
    item:setOpacity(1.0)
  end
end

function ManslapStateProgressBar:showAppearAnimation()
  return getFadeAction(self, 1.0):start()
end

function ManslapStateProgressBar:showDisappearAnimation()
  return getFadeAction(self, 0.0):start()
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, itemCount, indicatorColors)
  local itemWidth = 8
  local dx = 16
  local itemsRowWidth = itemCount * (itemWidth + dx) - dx
  local startX = GROUP_WIDTH * 0.5 - itemsRowWidth * 0.5 + itemWidth * 0.5
  local items = {}
  for itemIndex = 1, itemCount do
    local item = ManslapStateIndicator.new(indicatorColors)
    local x = startX + (itemWidth + dx) * (itemIndex - 1)
    item:setLoc(x, self:getHeightPart(0.5))
    self:insert(item)

    items[#items + 1] = item
  end

  self.items = items
end

function ManslapStateProgressBar.new(itemCount, indicatorColors)
  local self = Group.new(GROUP_WIDTH, GROUP_HEIGHT)
  Mixin.mix(self, ManslapStateProgressBar)
  createLayout(self, itemCount, indicatorColors)
  return self
end
