--- ManslapGame
-- @author Created by alexeychernikov on March 31, 2014.

class("ManslapWaveUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ManslapWaveUtils.getCheckmarkShape(numberOfPoints, w, h)
  numberOfPoints = numberOfPoints or 128
  w, h = w or 100, h or 80

  ---------
  --               TR
  --   +-------------...TR1
  --TL ...TL1     ...  |
  --   |...  M ...     |
  --   |  ......       |
  --   +---...---------+
  --      BL
  local keyPoints = {}
  keyPoints[1] = {-w * 0.45, 0} -- TL
  keyPoints[2] = {-w * 0.3, h * 0.2} -- TL1
  keyPoints[3] = {-w * 0.1, 0} -- M
  keyPoints[4] = {w * 0.3, h * 0.5} -- TR
  keyPoints[5] = {w * 0.45, h * 0.35} -- TR1
  keyPoints[6] = {-w * 0.1, -h * 0.35} -- BL

  -- distribute the points other than the keypoints between the segments formed by the key points
  -- |==|==| 3 key points form 2 segments
  local inbetweenPointsPerSegment = math.floor((numberOfPoints - #keyPoints)/(#keyPoints - 1))
  local extraPoints = numberOfPoints - inbetweenPointsPerSegment * #keyPoints - #keyPoints - 1

  -- A loop over N points forms N segments (A -> B -> C -> A : 3 points, 3 segments)
  local segment1 = MathUtils.getLinearPointsOnSegment(keyPoints[1], keyPoints[2], inbetweenPointsPerSegment)
  local segment2 = MathUtils.getLinearPointsOnSegment(keyPoints[2], keyPoints[3], inbetweenPointsPerSegment)
  local segment3 = MathUtils.getLinearPointsOnSegment(keyPoints[3], keyPoints[4], inbetweenPointsPerSegment)
  local segment4 = MathUtils.getLinearPointsOnSegment(keyPoints[4], keyPoints[5], inbetweenPointsPerSegment + extraPoints)
  local segment5 = MathUtils.getLinearPointsOnSegment(keyPoints[5], keyPoints[6], inbetweenPointsPerSegment)
  local segment6 = MathUtils.getLinearPointsOnSegment(keyPoints[6], keyPoints[1], inbetweenPointsPerSegment)

  -- The segments all include the edge points, clear up one edge point per segment
  segment1[#segment1] = nil
  segment2[#segment2] = nil
  segment3[#segment3] = nil
  segment4[#segment4] = nil
  segment5[#segment5] = nil

  local result = TableUtils.createConcatenatedArray(segment1, segment2, segment3, segment4, segment5, segment6)

  for _, point in pairs(result) do
    point[1] = point[1] + Screen.width/2
  end

  return result
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
