--- MatryoshkaConceptGenerator
-- @author Created by amorant on October 01, 2014.
-- Implements adaptive difficulty by limiting the number of concepts seen during a game. To achieve
-- this, it initializes a pool of concepts and only returns concepts from that pool (implementation
-- by inheriting from SequentialValueGenerator).

class("MatryoshkaConceptGenerator")
require("data.generators.ValueGenerators")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_CONCEPTS_PER_GAME = 16

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getConceptPoolSize()
  local conceptsPerRound = MatryoshkaConfiguration.IMAGE_COUNTS
  local roundsCount = MatryoshkaConfiguration.SET_COUNT

  assert(#conceptsPerRound >= roundsCount,
         "An insufficient number of conceptsPerRound counts were defined for the number of defined rounds.")

  local minPoolSize = conceptsPerRound[roundsCount]

  local difficulty = GameSession.getDifficulty()
  local poolSize = MathUtils.interpolateLinearWithClamp(difficulty, 0, 0.75, minPoolSize, MAX_CONCEPTS_PER_GAME)

  return MathUtils.round(poolSize)
end

local function getConceptPool()
  local conceptGenerator = GameSession.conceptGenerator

  local conceptPoolSize = getConceptPoolSize()
  local concepts = conceptGenerator:nextAsArray(conceptPoolSize)
  TableUtils.shuffleArray(concepts)

  return concepts
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MatryoshkaConceptGenerator.new()
  local conceptPool = getConceptPool()
  local self = SequentialValueGenerator.new(conceptPool)

  Mixin.mix(self, MatryoshkaConceptGenerator)

  return self
end
