--- MatryoshkaConceptImage
-- @author Created by Karl around September 27, 2013.

class("MatryoshkaConceptImage")

require("MatryoshkaCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_BORDER = {54/255, 18/255, 17/255}

local Z_INDEX = {
  BORDER = 0,
  PHOTO = 10,
  SHADOW = 20,
  OVERLAY = 30
}

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MatryoshkaConceptImage:setVisible(isVisible)
  for _, child in pairs(self.children) do
    child:setVisible(isVisible)
  end
end

function MatryoshkaConceptImage:scaleAndRemove()
  return runInCoroutine(function()
    if not self.isRemoving then
      self.isRemoving = true
      wait(AHScaleTo.new(self, 0.1, AHEaseSineInOut.new(), 1.1):start())
      wait(AHScaleTo.new(self, 0.25, AHEaseSineIn.new(), 0):start())
      self:removeSelf()
      self:stopWiggle()
    end
  end)
end

function MatryoshkaConceptImage:fadeScaleAndRemove()
  if not self.isRemoving then
    local selfWidth, selfHeight = self:getDims()
    local fadeOverlay = FilledRectangle.new(selfWidth, selfHeight, {0, 0, 0, 0})
    self:insertAtCenter(fadeOverlay, Z_INDEX.OVERLAY)
    AHOpacityTo.new(fadeOverlay, 0.1, nil, 0.5):start()
  end
  return self:scaleAndRemove()
end

function MatryoshkaConceptImage:wiggle()
  local wiggleFactor = 0.005;
  local wiggleDurationMin = 1.0;
  local wiggleDurationMax = 1.5;

  local selfWidth, selfHeight = self:getDims()
  local durationX = MathUtils.getRandomFloat(wiggleDurationMin, wiggleDurationMax)
  local durationY = MathUtils.getRandomFloat(wiggleDurationMin, wiggleDurationMax)
  local deltaX = selfWidth * wiggleFactor * MathUtils.randomSign()
  local deltaY = selfHeight * wiggleFactor * MathUtils.randomSign()

  self.wiggleAction = AHLoop.new(AHSpawn.new(
    AHSequence.new(
      AHLocationXBy.new(self, durationX / 2, AHEaseSineOut.new(), deltaX / 2),
      AHLocationXBy.new(self, durationX, AHEaseSineInOut.new(), -deltaX),
      AHLocationXBy.new(self, durationX / 2, AHEaseSineIn.new(), deltaX / 2)
      ),
    AHSequence.new(
      AHLocationYBy.new(self, durationY / 2, AHEaseSineOut.new(), deltaY / 2),
      AHLocationYBy.new(self, durationY, AHEaseSineInOut.new(), -deltaY),
      AHLocationYBy.new(self, durationY / 2, AHEaseSineIn.new(), deltaY / 2)
      )
    ))
  self.wiggleAction:start()
end

function MatryoshkaConceptImage:stopWiggle()
  if self.wiggleAction then
    self.wiggleAction:stop()
    self.wiggleAction = nil
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MatryoshkaConceptImage.new(size, concept, onPressed)
  local self = Group.new(size, size)
  Mixin.mix(self, MatryoshkaConceptImage)

  self.concept = concept

  local photoSpriteSize = math.floor(size * 0.975)

  local border = FilledRectangle.new(size, size, COLOR_BORDER)
  self:insertAtCenter(border, Z_INDEX.BORDER)

  local photoSprite = AssetHelper.createPhotoSpriteForConcept(concept, concept.image)
  photoSprite:setEffectiveDims(photoSpriteSize, photoSpriteSize)
  self:insertAtCenter(photoSprite, Z_INDEX.PHOTO)

  local frameShadow = Sprite.newWithName("frame_shadow.png")
  frameShadow:setEffectiveDims(photoSpriteSize, photoSpriteSize)
  self:insertAtCenter(frameShadow, Z_INDEX.SHADOW)

  Button.makeButton(self, onPressed)

  return self
end
