class("MatryoshkaConfiguration")
require("GameSession")
require("MatryoshkaCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

MatryoshkaConfiguration.SET_COUNT = 4
MatryoshkaConfiguration.IMAGE_COUNTS = {3, 4, 5, 6}
MatryoshkaConfiguration.POINTS_PER_CORRECT_RESPONSE = 375
MatryoshkaConfiguration.POINTS_PER_LIVES_BONUS = 1000
MatryoshkaConfiguration.POINTS_PER_FULL_SPEED_BONUS = 100
MatryoshkaConfiguration.INITIAL_LIVES_COUNT = 4
MatryoshkaConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 1.6
MatryoshkaConfiguration.SPEED_BONUS_DECAY_START = 0.0

local EXTRA_LIVES_NEEDED_RANK_TWO = 2
local EXTRA_LIVES_NEEDED_RANK_THREE = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MatryoshkaConfiguration.getNumberCorrectPossibleForSets(setCount)
  local totalPoints = 0
  local imageCounts = MatryoshkaConfiguration.IMAGE_COUNTS
  local maxPossibleThisRound = imageCounts[1]
  for round = 1, setCount do
    totalPoints = totalPoints + maxPossibleThisRound
    maxPossibleThisRound = maxPossibleThisRound + 1
  end

  return totalPoints
end

function MatryoshkaConfiguration.getPassScore()
  local setsToComplete = MatryoshkaConfiguration.SET_COUNT
  local possibleWrongAnswers = MatryoshkaConfiguration.INITIAL_LIVES_COUNT - 1
  local correctPossible = MatryoshkaConfiguration.getNumberCorrectPossibleForSets(setsToComplete) - possibleWrongAnswers
  local pointsPerAnswer = MatryoshkaConfiguration.POINTS_PER_CORRECT_RESPONSE

  local passScore = correctPossible * pointsPerAnswer
  return passScore
end

function MatryoshkaConfiguration.getRankTwoScore()
  local passScore = MatryoshkaConfiguration.getPassScore()
  local extraLivesBonusScore = (EXTRA_LIVES_NEEDED_RANK_TWO * MatryoshkaConfiguration.POINTS_PER_LIVES_BONUS)
  local extraAnswersScore = (EXTRA_LIVES_NEEDED_RANK_TWO * MatryoshkaConfiguration.POINTS_PER_CORRECT_RESPONSE)
  local rankTwoScore = passScore + extraLivesBonusScore + extraAnswersScore

  return rankTwoScore
end

function MatryoshkaConfiguration.getRankThreeScore()
  local passScore = MatryoshkaConfiguration.getPassScore()
  local extraLivesBonusScore = (EXTRA_LIVES_NEEDED_RANK_THREE * MatryoshkaConfiguration.POINTS_PER_LIVES_BONUS)
  local extraAnswersScore = (EXTRA_LIVES_NEEDED_RANK_THREE * MatryoshkaConfiguration.POINTS_PER_CORRECT_RESPONSE)
  local rankThreeScore = passScore + extraLivesBonusScore + extraAnswersScore

  return rankThreeScore
end

-- If this cutoff changes, make sure to update how many Easy concepts are loaded.
-- The number of Easy concepts to load is determined by MatryoshkaConceptGenerator.
function MatryoshkaConfiguration.getDifficultyKeyFrames()
  return DifficultyCurves.getHardCutoffKeyframes({
    {endDifficulty = 0.4, contentDifficulty = "Easy"},
    {endDifficulty = 1.0, contentDifficulty = "Hard"}
  })
end
