--- MatryoshkaDoll
-- @author Created by Karl around September 27, 2013.

class("MatryoshkaDoll")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_PARTICLES = 10
local Z_IMAGE = 20
local Z_DOLL = 30

local DOLL_TYPE_COUNT = 6
local COLOR_SNAP_IMAGE = {255/255, 255/255, 37/255}
local COLOR_SMAP_DOLL =  {233/255, 134/255, 37/255}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function durationFrames(frames)
  return frames / 25
end

local function showSnapParticles(self, color, isOnRightSide)
  return runInCoroutine(function()
    local particles = Sprite.newWithName("snap_triangles/snap_triangles_005.png")
    particles:setColor(unpack(color))
    local selfW, selfH = self:getDims()
    if isOnRightSide then
      particles:setAnchorPointLeft()
      particles:setLoc(selfW, self.dollTopBaseY)
      particles:setScl(2)
    else
      particles:setAnchorPointLeft()
      particles:setLoc(0, self.dollTopBaseY)
      particles:setScl(-2, 2)
    end
    self:insert(particles, Z_PARTICLES)
    local anim = Animation.newWithPrefix("snap_triangles/snap_triangles_")
    wait(AHAnimate.new(particles, anim, 30):start())
    particles:removeSelf()
  end)
end

local function showRightSnapParticles(self, color)
  return showSnapParticles(self, color, true)
end

local function showLeftSnapParticles(self, color)
  return showSnapParticles(self, color, false)
end

local function squetchAction(self, duration, scaleMultiplierX, scaleMultiplierY)
  scaleMultiplierX = scaleMultiplierX or 1.1
  scaleMultiplierY = scaleMultiplierY or 0.92
  local scaleX, scaleY = self:getScl()
  return AHSequence.new(
    AHScaleTo.new(self, duration/2, AHEaseSineOut.new(), scaleX * scaleMultiplierX, scaleY * scaleMultiplierY),
    AHScaleTo.new(self, duration/2, AHEaseSineIn.new(), scaleX, scaleY)
    )
end

local function open(self)
  local selfW, selfH = self:getDims()
  local duration = durationFrames(3)
  return AHSpawn.new(
      AHLocationYBy.new(self.top, duration, AHEaseSineOut.new(), math.floor(selfH * 0.2)),
      AHRotationTo.new(self.top, duration, AHEaseSineOut.new(), 20)
      ):start()
end

local function close(self, color)
  return runInCoroutine(function()
    local duration = durationFrames(1)
    wait(AHSpawn.new(
      AHLocationYTo.new(self.top, duration, AHEaseSineOut.new(), self.dollTopBaseY),
      AHRotationTo.new(self.top, duration, AHEaseSineOut.new(), 0)
      ):start())
    if color then
      showRightSnapParticles(self, color)
    end
    wait(squetchAction(self, durationFrames(2), 1.02, 0.97):start())
  end)
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MatryoshkaDoll:eatImage(image)
  return runInCoroutine(function()
    local duration = 0.3
    local selfW, selfH = self:getDims()
    local selfEffW, selfEffH = self:getEffectiveDims()
    local scaleX, scaleY = image:scaleForEffectiveDims(selfEffW/3, selfEffW/3)
    local destX, destY = self.parent:transformLoc(self, selfW*0.5, selfH*0.3)
    self.parent:takeChild(image, self.zIndex-1)
    AHSpawn.new(
      AHLocationTo.new(image, duration, AHEaseSineInOut.new(), destX, destY),
      AHScaleTo.new(image, duration, AHEaseSineInOut.new(), scaleX, scaleY)
      ):start()
    open(self)
    waitFor(duration * 1.15)
    wait(close(self, COLOR_SNAP_IMAGE))
    image:removeSelf()
  end)
end

function MatryoshkaDoll:prepareToEatDoll(doll, isStraight)
  local selfW, selfH = self:getDims()
  local distance = selfH
  local topX, topY = self.top:getLoc()
  local bottomX, bottomY = self.bottom:getLoc()
  local dollScale = doll:getScl()

  bottomY = -selfH
  topY = selfH * 1.3
  local rotation = 30
  local scale = dollScale

  if isStraight then
    bottomY = -selfH * 3
    topY = selfH * 3
    rotation = 0
    scale = dollScale * 1.4
  end
  self.shouldShowLeftSnapParticles = isStraight

  self.top:setRot(rotation)
  self.top:setLoc(topX, topY)
  self.bottom:setLoc(bottomX, bottomY)
  self:setScl(scale)
end

function MatryoshkaDoll:eatDoll(doll)
  return runInCoroutine(function()
    Sound.playSoundEffect("matryoshka_snap")
    local duration = 0.2
    local selfW, selfH = self:getDims()
    wait(AHSpawn.new(
      AHLocationTo.new(self.top, duration, nil, 0, self.dollTopBaseY),
      AHRotationTo.new(self.top, duration, nil, 0),
      AHLocationTo.new(self.bottom, duration, nil, 0, self.dollBottomBaseY)
      ):start())
    showRightSnapParticles(self, COLOR_SMAP_DOLL)
    if self.shouldShowLeftSnapParticles then
      showLeftSnapParticles(self, COLOR_SMAP_DOLL)
    end
    doll:removeSelf()
    wait(squetchAction(self, durationFrames(0.8), 1.02, 0.97):start())
  end)
end

function MatryoshkaDoll:spitOutDoll(doll, spacing, dollIndex)
  return runInCoroutine(function()
    local selfW, selfH = self:getEffectiveDims()
    doll:setEffectiveDims(nil, math.floor(selfH * MatryoshkaCommon.END_SEQUENCE_DOLL_SIZE_PROPORTION))
    local dollW, dollH = doll:getEffectiveDims()
    doll:setLoc(self:getLoc())
    self.parent:takeChild(doll, self.zIndex - 1)

    local distanceX = math.floor(selfW/2 + dollW/2 + spacing)
    local distanceY = math.floor((selfH - dollH) * 3)
    local hopDuration = 0.35

    local hopAction = AHSpawn.new(
      AHLocationXBy.new(doll, hopDuration, AHEaseSineInOut.new(), distanceX),
      AHSequence.new(
        AHLocationYBy.new(doll, hopDuration/2, AHEaseQuadraticIn.new(), distanceY),
        AHLocationYBy.new(doll, hopDuration/2, AHEaseQuadraticOut.new(), -distanceY)
      )
    ):start()

    local soundEffectName = string.format("matryoshka_goal_reach_doll_%d", dollIndex)
    Sound.playSoundEffect(soundEffectName)

    wait(open(self))
    wait(close(self))
    wait(hopAction)

  end)

end

function MatryoshkaDoll:rock()
  local rotation = 5
  return AHSequence.new(
    AHRotationTo.new(self, 0.1, AHEaseSineInOut.new(), rotation),
    AHRotationTo.new(self, 0.1, AHEaseSineInOut.new(), -rotation),
    AHRotationTo.new(self, 0.1, AHEaseSineInOut.new(), rotation/2),
    AHRotationTo.new(self, 0.1, AHEaseSineInOut.new(), -rotation/2),
    AHRotationTo.new(self, 0.1, AHEaseSineInOut.new(), 0)
    ):start()
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MatryoshkaDoll.new(dollType)
  dollType = (dollType-1) % DOLL_TYPE_COUNT + 1
  local top = Sprite.newWithName("doll0" .. dollType .. "_top.png")
  local topW, topH = top:getDims()

  local bottom = Sprite.newWithName("doll0" .. dollType .. "_bottom.png")
  local bottomW, bottomH = bottom:getDims()

  local self = Group.new(bottomW, topH + bottomH)
  Mixin.mix(self, MatryoshkaDoll)

  self.dollTopBaseY = bottomH - 1
  self.dollBottomBaseY = 0

  self.top = top
  self.top:setAnchorPointBottomLeft()
  self.top:setLoc(0, self.dollTopBaseY)
  self:insert(self.top, Z_DOLL)

  self.bottom = bottom
  self.bottom:setAnchorPointBottomLeft()
  self.bottom:setLoc(0, self.dollBottomBaseY)
  self:insert(self.bottom, Z_DOLL)

  self:setAnchorPointBottom()

  return self
end
