--- MatryoshkaImageGrid
-- @author Created by Karl around September 27, 2013.

class("MatryoshkaImageGrid")

require("MatryoshkaConceptImage")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MatryoshkaImageGrid:showNextConcepts()
  for _, image in pairs(self.currentImages) do
    image:removeSelf()
  end
  self.currentImages = self.nextImages
  self.nextImages = {}
  local images = self.currentImages

  return runInCoroutine(function()
    Sound.playSoundEffect("matryoshka_appear")

    for _, image in pairs(images) do
      self:insert(image)
      image:setScl(0)
    end

    -- Workaround for timing bug
    coroutine.yield()
    for _, image in pairs(images) do
      image:setVisible(true)
    end

    for _, image in pairs(images) do
      AHSequence.new(
        AHScaleTo.new(image, 0.08, nil, 1.05),
        AHScaleTo.new(image, 0.05, AHEaseSineInOut.new(), 1)
        ):start()
      waitFor(0.02)
    end

    for _, image in pairs(images) do
      image:wiggle()
    end
  end)
end

function MatryoshkaImageGrid:setNextConcepts(concepts)
  self.nextImages = {}
  local positions = self.positions[#concepts]

  for index, concept in ipairs(concepts) do
    local image = MatryoshkaConceptImage.new(self.imageSize, concept, self.onImagePressed)
    image:setLoc(unpack(positions[index]))
    image:setVisible(false)
    table.insert(self.nextImages, image)
  end
end

function MatryoshkaImageGrid:getCurrentImages()
  return self.currentImages
end

function MatryoshkaImageGrid:removeImages()
  local lastAction = nil
  for _, image in pairs(self.children) do
    lastAction = image:fadeScaleAndRemove()
  end
  return lastAction
end

function MatryoshkaImageGrid:setTouchEnabled(isEnabled)
  for _, image in pairs(self.children) do
    image:setTouchEnabled(isEnabled)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function calculatePositions(self, imageSize, bufferSize)
  local width, height = self:getDims()

  local imageSizeHeightLimitation = (height / 3)
  local imageSizeWidthLimitation = (width / 2)
  local limitingDimension = math.min(imageSizeHeightLimitation, imageSizeWidthLimitation)
  local bufferSize = limitingDimension * 0.03
  local imageSize = math.floor(limitingDimension - (bufferSize * 2))

  local centerImage = imageSize / 2
  local centerWidth = width / 2
  local centerHeight = height / 2
  local imageOffset = centerImage + bufferSize

  local yPositions = {
    math.floor(centerHeight + (2 * imageOffset)),
    math.floor(centerHeight + imageOffset),
    math.floor(centerHeight),
    math.floor(centerHeight - imageOffset),
    math.floor(centerHeight - (2 * imageOffset)),
  }

  local xPositions = {
    math.floor(centerWidth - (2 * imageOffset)),
    math.floor(centerWidth - imageOffset),
    math.floor(centerWidth),
    math.floor(centerWidth + imageOffset),
    math.floor(centerWidth + (2 * imageOffset))
  }

  local positions = {
    {
      {xPositions[3], yPositions[3]}
    },
    {
      {xPositions[3], yPositions[2]},
      {xPositions[3], yPositions[4]}
    },
    {
      {xPositions[3], yPositions[2]},
      {xPositions[2], yPositions[4]},
      {xPositions[4], yPositions[4]}
    },
    {
      {xPositions[2], yPositions[2]},
      {xPositions[4], yPositions[2]},
      {xPositions[2], yPositions[4]},
      {xPositions[4], yPositions[4]}
    },
    {
      {xPositions[2], yPositions[1]},
      {xPositions[4], yPositions[1]},
      {xPositions[2], yPositions[3]},
      {xPositions[4], yPositions[3]},
      {xPositions[3], yPositions[5]}
    },
    {
      {xPositions[2], yPositions[1]},
      {xPositions[4], yPositions[1]},
      {xPositions[2], yPositions[3]},
      {xPositions[4], yPositions[3]},
      {xPositions[2], yPositions[5]},
      {xPositions[4], yPositions[5]}
    }
  }

  self.imageSize = imageSize

  return positions
end

-- Constructor ------------------------

function MatryoshkaImageGrid.new(width, height, onImagePressed)
  local self = Group.new(width, height)
  Mixin.mix(self, MatryoshkaImageGrid)

  self.positions = calculatePositions(self)
  self.onImagePressed = onImagePressed
  self.currentImages = {}

  return self
end
