--- MatryoshkaInGameUi
-- @author Created by Karl around September 27, 2013.

class("MatryoshkaInGameUi")

require("MatryoshkaCommon")
require("MatryoshkaQuestionLabel")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_CORRECT = {62/255, 183/255, 30/255}

local Z_ANSWER = 10
local Z_QUESTION = 20

local QUESTION_Y_START = UIOverlay.NORTH.y - 20
local FREE_AREA_PADDING = -10
local ANSWER_Y_OFFSET = -2
local IPAD_SCREEN_RATIO = 0.75

local TICK_CROSS_SIZE = Screen.fullWidth * 0.3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function moveAnswerDown(self)
  return runInCoroutine(function()
    self.answerLabel:setVisible(true)
    wait(AHLocationYTo.new(self.answerLabel, 0.2, nil, self.answerLowY + ANSWER_Y_OFFSET):start())
  end)
end

local function moveAnswerUp(self)
  return runInCoroutine(function()
    wait(AHLocationYTo.new(self.answerLabel, 0.2, nil, self.answerHighY):start())
    self.answerLabel:setVisible(false)
  end)
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MatryoshkaInGameUi:runCorrectSequence(image)
  return runInCoroutine(function()
    local tickMark = Sprite.newWithName("tick/shine_000.png")
    local shineAnim = Animation.newWithPrefix("tick/shine_")
    local spinoutAnim = Animation.newWithPrefix("tick/spinout_")
    local maxScale = image:getWidth() / tickMark:getWidth()
    tickMark:setLoc(self:transformObjectLoc(image))
    self:insert(tickMark)
    tickMark:setScl(0)
    self.answerLabel:setVisible(true)
    self.answerLabel:setColor(unpack(COLOR_CORRECT))
    moveAnswerDown(self)
    AHScaleBy.new(image, 0.05, AHEaseSineInOut.new(), 0.1):start()
    wait(AHScaleTo.new(tickMark, 0.15, AHEaseSineOut.new(), maxScale):start())
    wait(AHAnimate.new(tickMark, shineAnim):start())
    runInCoroutine(function()
      wait(AHAnimate.new(tickMark, spinoutAnim):start())
      moveAnswerUp(self)
      tickMark:removeSelf()
    end)
  end)
end

function MatryoshkaInGameUi:runIncorrectSequence(imageTapped, imageForRationale, otherImages)
  local cross = Sprite.newWithName("cross/cross_000.png")
  local crossAnim = Animation.newWithPrefix("cross/cross_")
  local scale = imageTapped:getWidth() / cross:getWidth()
  cross:setLoc(self:transformObjectLoc(imageTapped))
  cross:setScl(scale)
  self:insert(cross)

  local duration = 0.5
  local ease = AHEaseBezier.newFlat({0.7, 0.0, 0.3, 1.0})
  local scaleOutAction = AHSpawn.new()
  for _, image in ipairs(otherImages) do
    scaleOutAction:addAction(AHScaleTo.new(image, duration, ease, 0))
  end
  scaleOutAction:addAction(AHScaleTo.new(cross, duration, ease, 0))

  local incorrectSequence = AHSequence.new(
    AHAnimate.new(cross, crossAnim),
    AHDelay.new(0.1),
    scaleOutAction,
    AHRemove.new(cross)
  )

  return incorrectSequence:start()
end

function MatryoshkaInGameUi:runReturnFromIncorrectSequence(imageForRationale, otherImages)
  local delay = 0.050
  local duration = 0.5
  local ease = AHEaseBezier.newFlat({0.7, 0.0, 0.3, 1.0})
  local scaleInAction = AHSpawn.new()


  scaleInAction:addAction(AHDelay.new(delay))
  for _, image in ipairs(otherImages) do
    scaleInAction:addAction(AHScaleTo.new(image, duration, ease, 1))
  end

  return scaleInAction:start()
end

function MatryoshkaInGameUi:clearQuestion()
  return self.questionLabel:fadeOut()
end

function MatryoshkaInGameUi:setQuestionAndAnswer(question, answer)
  return runInCoroutine(function()
    self.answerLabel:setVisible(false)
    self.answerLabel:setText(answer)
    local answerW, answerH = self.answerLabel:getDims()
    wait(self.questionLabel:fadeOut())
    wait(self.questionLabel:fadeIn(question, answerW))
  end)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MatryoshkaInGameUi.new()
  local questionLabel = MatryoshkaQuestionLabel.new()
  local labelHeight = questionLabel:getHeight()

  local fullHeight = labelHeight * 2.2

  local self = Group.new(Screen.width, fullHeight)
  Mixin.mix(self, MatryoshkaInGameUi)

  self:setAnchorPointTop()

  self:insertAtTop(questionLabel, Z_QUESTION)
  self.questionLabel = questionLabel

  local questionY = self.questionLabel:getY()
  self.answerHighY = questionY
  self.answerLowY = (questionY - fullHeight) + labelHeight

  self.answerLabel = Label.new("testing", Font.new(MatryoshkaCommon.FONT_NORMAL, MatryoshkaCommon.FONT_SIZE))
  self.answerLabel:setLoc(self:getWidthPart(0.5), self.answerHighY)
  self.answerLabel:setVisible(false)
  self:insert(self.answerLabel, Z_ANSWER)

  return self
end
