--- MatryoshkaQuestionLabel
-- @author Created by Karl around September 27, 2013.

class("MatryoshkaQuestionLabel")

require("MatryoshkaCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TEXT = {1, 1, 1, 1}
local COLOR_BG = {0.49, 0.44, 0.33, 1}

local PADDING_X = 8
local PADDING_Y = 2

local FADE_DURATION = 0.1
local LABEL_OFFSET_Y = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MatryoshkaQuestionLabel:fadeOut()
  return AHScaleTo.new(self, FADE_DURATION, AHEaseSineInOut.new(), 1, 0):start()
end

function MatryoshkaQuestionLabel:fadeIn(text, minWidth)
  return runInCoroutine(function()
    self.label:setText(text)
    local labelWidth, labelHeight = self.label:getDims()
    if labelWidth < minWidth then
      labelWidth = minWidth
    end

    self:setDims(labelWidth + PADDING_X * 2, labelHeight + PADDING_Y * 2)
    local width, height = self:getDims()
    self.label:setLoc(width/2, height/2 + LABEL_OFFSET_Y)
    wait(AHScaleTo.new(self, FADE_DURATION, AHEaseSineInOut.new(), 1):start())
  end)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MatryoshkaQuestionLabel.new()
  local fontSize = MatryoshkaCommon.FONT_SIZE
  local cornerRadius = math.floor(Screen.getWidthPart(0.01))

  local width = Screen.safeWidth
  local height = fontSize + 4

  local self = FilledRoundedRectangle.new(width, height, cornerRadius, COLOR_BG)
  Mixin.mix(self, MatryoshkaQuestionLabel)

  self.label = Label.new("", Font.new(MatryoshkaCommon.FONT_NORMAL, fontSize))
  self.label:setColor(unpack(COLOR_TEXT))
  self.label:setLoc(width/2, height/2 + LABEL_OFFSET_Y)
  self:insert(self.label)

  self:setScl(1, 0)

  return self
end
