--- MoonsetBackground
-- @author created by Jay Austria on November 7, 2013

class("MoonsetBackground")
require("MoonsetCommon")
require("MoonsetStarOverlay")
require("MoonsetSailboat")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TOP_TOP_START = ColorUtils.colorHex24ToStd(0x60D5C3)
local COLOR_TOP_TOP_END = ColorUtils.colorHex24ToStd(0xA4E3D7)
local COLOR_TOP_BOTTOM_START = ColorUtils.colorHex24ToStd(0x4DBBBE)
local COLOR_TOP_BOTTOM_END = ColorUtils.colorHex24ToStd(0x7FCFDA)

local COLOR_BOTTOM_TOP_START = ColorUtils.colorHex24ToStd(0x6BC6C9)
local COLOR_BOTTOM_TOP_END = ColorUtils.colorHex24ToStd(0x95D7E0)
local COLOR_BOTTOM_BOTTOM_START = ColorUtils.colorHex24ToStd(0x237BA2)
local COLOR_BOTTOM_BOTTOM_END = ColorUtils.colorHex24ToStd(0x4A9ABA)

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MoonsetBackground:startColorAnimation(duration)
  self.colorShiftAction = AHSpawn.new(
    self.topGradient:newGradientToAction(duration, nil, COLOR_TOP_TOP_END, COLOR_TOP_BOTTOM_END),
    self.bottomGradient:newGradientToAction(duration, nil, COLOR_BOTTOM_TOP_END, COLOR_BOTTOM_BOTTOM_END)
  )
end

function MoonsetBackground:setPaused(isPaused)
  self.colorShiftAction:pause(isPaused)
end

function MoonsetBackground:showEndAnimation()
  self.starsOverlay:animate()
  AHOpacityTo.new(self.starsOverlay, 1, nil, 1):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newStarfield(height)
  local container = Group.new(Screen.fullWidth, height)
  local stars = Sprite.newWithName("stars.png")
  stars:setEffectiveDims(Screen.getWidthPart(0.75))
  container:insertAtCenter(stars)
  return container
end

function MoonsetBackground.new(horizonY)
  local island = Sprite.newWithName("island.png")
  local islandW, islandH = island:getDims()

  local selfW = Screen.fullWidth + islandW
  local selfH = Screen.fullHeight
  local self = Group.new(selfW, selfH)
  Mixin.mix(self, MoonsetBackground)

  self.topGradient = RectangleGradient.new(selfW, selfH - horizonY, COLOR_TOP_TOP_START, COLOR_TOP_BOTTOM_START)
  self.topGradient:setAnchorPointTop()
  self.topGradient:setLoc(self:getWidthPart(0.5), self:getHeight())
  self:insert(self.topGradient)

  self.bottomGradient = RectangleGradient.new(selfW, horizonY, COLOR_BOTTOM_TOP_START, COLOR_BOTTOM_BOTTOM_START)
  self.bottomGradient:setAnchorPointBottom()
  self.bottomGradient:setLoc(self:getWidthPart(0.5), 0)
  self:insert(self.bottomGradient)

  local stars = newStarfield((self:getHeight() - horizonY))
  stars:setAnchorPointTopLeft()
  stars:setLoc(0, self:getHeight())
  self:insert(stars)

  local starsOverlay = MoonsetStarOverlay.new(selfH, horizonY)
  starsOverlay:setAnchorPointBottomLeft()
  starsOverlay:setOpacity(0)
  self:insert(starsOverlay)
  self.starsOverlay = starsOverlay

  local islandX = selfW
  local islandY = horizonY - math.floor(islandH * 0.1)
  island:setAnchorPointBottomRight()
  island:setLoc(islandX, islandY)
  self:insert(island)

  local boatDescriptors =
  {
    {
      boatStyleIndex = 2,
      x = islandX - math.floor(islandW * 0.6),
      y = islandY - math.floor(islandH * 0.2),
      scale = 0.8,
      animationDelay = 0.0
    },
    {
      boatStyleIndex = 3,
      x = islandX - math.floor(islandW * 0.3),
      y = islandY - math.floor(islandH * 0.45),
      scale = 1.0,
      animationDelay = 0.7
    }
  }

  for _, boatDescriptor in ipairs(boatDescriptors) do
    local sailboat = MoonsetSailboat.new(boatDescriptor.boatStyleIndex)
    sailboat:setLoc(boatDescriptor.x, boatDescriptor.y)
    sailboat:setScl(boatDescriptor.scale)

    runInCoroutine(function()
      waitFor(boatDescriptor.animationDelay)
      sailboat:startStationaryAnimation()
    end)

    self:insert(sailboat)
  end

  return self
end
