--- MoonsetConfiguration

class("MoonsetConfiguration")

require("data.keyframes.KeyframeSet")
require("data.generators.LearningFocusedRoundGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CORRECT_ANSWER_POINTS = 1000
local INCORRECT_ANSWER_POINTS = -200
local TIME_BONUS_PER_SECOND = 100
local MAX_STREAK_BONUS_POINTS = 50
local ROUND_TIME = 64
local EXTRA_SECONDS_RANK_TWO = 24
local EXTRA_SECONDS_RANK_THREE = 38
local TUTORIAL_DELAY = 8
local TIME_REMAINING_TO_START_RESURFACING_MISTAKES =
  ROUND_TIME *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING

local CORRECT_TO_WIN_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {correctToWin = 6, mistakesAllowedBeforeIncrease = 2}),
  Keyframe.new(0.3, {correctToWin = 6, mistakesAllowedBeforeIncrease = 1}),
  Keyframe.new(0.4, {correctToWin = 6, mistakesAllowedBeforeIncrease = 0}),
  Keyframe.new(0.7, {correctToWin = 7, mistakesAllowedBeforeIncrease = 4}),
  Keyframe.new(0.8, {correctToWin = 7, mistakesAllowedBeforeIncrease = 3}),
  Keyframe.new(0.9, {correctToWin = 7, mistakesAllowedBeforeIncrease = 2})
)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getCorrectToWinFrame()
  local difficulty = GameSession.getDifficulty()
  return CORRECT_TO_WIN_THRESHOLDS:getFrameAt(difficulty)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MoonsetConfiguration.getTimeRemainingToStartResurfacingMistakes()
  return TIME_REMAINING_TO_START_RESURFACING_MISTAKES
end

function MoonsetConfiguration.getTutorialDelay()
  return TUTORIAL_DELAY
end

function MoonsetConfiguration.getCorrectAnswerPoints()
  return CORRECT_ANSWER_POINTS
end

function MoonsetConfiguration.getIncorrectAnswerPoints()
  return INCORRECT_ANSWER_POINTS
end

function MoonsetConfiguration.getTimeBonusPerSecond()
  return TIME_BONUS_PER_SECOND
end

function MoonsetConfiguration.getMaxStreakBonusPoints()
  return MAX_STREAK_BONUS_POINTS
end

function MoonsetConfiguration.getRoundTime()
  return ROUND_TIME
end

function MoonsetConfiguration.getNumberOfConcepts()
  local correctToWinFrame = getCorrectToWinFrame()
  return correctToWinFrame.correctToWin
end

function MoonsetConfiguration.getMaxStreakBonus()
  local maxCorrect = MoonsetConfiguration.getNumberOfConcepts()
  local maxStreakBonus = maxCorrect * MAX_STREAK_BONUS_POINTS
  return maxStreakBonus
end

function MoonsetConfiguration.getPassScore()
  local correctToWinFrame = getCorrectToWinFrame()
  local passScore = correctToWinFrame.correctToWin * CORRECT_ANSWER_POINTS
  local mistakesAllowedModifier = correctToWinFrame.mistakesAllowedBeforeIncrease * INCORRECT_ANSWER_POINTS
  return passScore + mistakesAllowedModifier
end

function MoonsetConfiguration.getExtraSecondsRankTwo()
  return EXTRA_SECONDS_RANK_TWO
end

function MoonsetConfiguration.getExtraSecondsRankThree()
  return EXTRA_SECONDS_RANK_THREE
end

function MoonsetConfiguration.getRankTwoScore()
  local passScore = MoonsetConfiguration.getPassScore()
  local extraSecondsNeeded = MoonsetConfiguration.getExtraSecondsRankTwo()
  local rankTwoScore = passScore + extraSecondsNeeded * TIME_BONUS_PER_SECOND
  return rankTwoScore
end

function MoonsetConfiguration.getRankThreeScore()
  local passScore = MoonsetConfiguration.getPassScore()
  local extraSecondsNeeded = MoonsetConfiguration.getExtraSecondsRankThree()
  local rankThreeScore = passScore + extraSecondsNeeded * TIME_BONUS_PER_SECOND
  return rankThreeScore
end


function MoonsetConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 1.00}},

    {key = 0.075, valueProbabilities = {Basic = 0.90,
                                        Easy = 0.10}},

    {key = 0.15, valueProbabilities = {Basic = 0.30,
                                       Easy = 0.70}},

    {key = 0.225, valueProbabilities = {Easy = 0.75,
                                        Medium = 0.25}},

    {key = 0.3, valueProbabilities = {Easy = 0.35,
                                      Medium = 0.50,
                                      Hard = 0.15}},

    {key = 0.375, valueProbabilities = {Easy = 0.30,
                                        Medium = 0.40,
                                        Hard = 0.30}},

    {key = 0.45, valueProbabilities = {Easy = 0.15,
                                       Medium = 0.50,
                                       Hard = 0.35}},

    {key = 0.525, valueProbabilities = {Easy = 0.10,
                                        Medium = 0.40,
                                        Hard = 0.50}},

    {key = 0.6, valueProbabilities = {Easy = 0.05,
                                      Medium = 0.3,
                                      Hard = 0.65}},

    {key = 0.675, valueProbabilities = {Easy = 0.05,
                                        Medium = 0.25,
                                        Hard = 0.70}},

    {key = 0.75, valueProbabilities = {Easy = 0.05,
                                       Medium = 0.10,
                                       Hard = 0.85}},

    {key = 0.8, valueProbabilities = {Medium = 0.1,
                                      Hard = 0.8,
                                      Advanced = 0.1}},

    {key = 0.9, valueProbabilities = {Hard = 0.8,
                                      Advanced = 0.2}},

    {key = 1.0, valueProbabilities = {Hard = 0.7,
                                      Advanced = 0.3}}
  }
end
