-- @author Created by Jay Austria on Nov. 6, 2013
class("MoonsetMoon")

require("utils.NumericMapping")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MOON_COLOR = ColorUtils.colorHex24ToStd(0xC6F1F4)
local MOON_REFLECTION_COLOR = ColorUtils.colorWithOpacity(MOON_COLOR, MoonsetCommon.REFLECTION_OPACITY)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setScissorRect(prop)
  local blX, blY = prop:getWorldLoc()
  local width, height = prop:getEffectiveDims()
  local tlX, tlY = 0, blY + height
  local brX, brY = Screen.fullWidth, blY

  local scissorRect = MOAIScissorRect.new()
  scissorRect:setRect(tlX, tlY, brX, brY)
  prop:setScissorRect(scissorRect)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MoonsetMoon:setProgress(progress)
 self.super.setProgress(self, progress)
 progress = self:getProgress()
 self.topMoon:setY(self.topMapping:getInverseMappedValue(progress))
 self.bottomMoon:setY(self.bottomMapping:getInverseMappedValue(progress))
end

function MoonsetMoon:markReferencePosition()
  setScissorRect(self.topMoon)
  setScissorRect(self.bottomMoon)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function newMoon(diameter, color)
  local moon = Sprite.newWithName("moon.png")
  local heightToKeepMoonRound = diameter * 0.9
  moon:setEffectiveDims(diameter, heightToKeepMoonRound)
  moon:setColor(unpack(color))
  return moon
end

function MoonsetMoon.new(radius)
  local diameter = radius * 2
  local spacing = 0
  local self = Group.new(diameter, diameter * 2)
  ProgressIndicator.makeProgressIndicator(self)
  ProgressTimer.makeTimer(self)
  Mixin.mix(self, MoonsetMoon)

  local halfX, halfY = self:getWidthPart(0.5), self:getHeightPart(0.5)

  local topMoon = newMoon(diameter, MOON_COLOR)
  topMoon:setAnchorPointBottom()
  topMoon:setLoc(halfX, halfY)
  self:insert(topMoon)

  -- Note: Need to use anchor point 0 instead of 1 because of the negative scale later on.
  local bottomMoon = newMoon(diameter, MOON_REFLECTION_COLOR)
  bottomMoon:setAnchorPointBottom()
  bottomMoon:setLoc(halfX, halfY)
  self:insert(bottomMoon)

  local scaleX, scaleY = bottomMoon:getScl()
  bottomMoon:setScl(scaleX, -scaleY * 0.7)

  self.topMoon = topMoon
  self.bottomMoon = bottomMoon

  self.topMapping = NormalizedRangeNumericMapping.new(topMoon:getY() - topMoon:getEffectiveHeight(), topMoon:getY())
  -- Note: Sign is inverted due to negative scale.
  self.bottomMapping = NormalizedRangeNumericMapping.new(bottomMoon:getY() - bottomMoon:getEffectiveHeight(), bottomMoon:getY())

  return self
end
