--- MoonsetOverlayBlend
-- @author Created by Joe Poka on August 9, 2016.

class("MoonsetOverlayBlend")

require("graphics.OffscreenRenderedTexture")
require("utils.ShaderUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SHADER_UNIFORM_TIME_INDEX = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getShader()
  local shader = MOAIShader.new()
  shader:reserveUniforms(2)
  shader:declareUniformSampler(1, "sampler", 1)
  shader:declareUniformFloat(SHADER_UNIFORM_TIME_INDEX, "gameTime", 0.0)
  shader:setVertexAttribute(1, "position")
  shader:setVertexAttribute(2, "uv")

  local vsh = ShaderUtils.loadShader("MoonsetOverlayShader.vsh")
  local fsh = ShaderUtils.loadShader("MoonsetOverlayColorShader.fsh")
  shader:load(vsh, fsh)

  return shader
end

local function createTexture(group)
  local groupWidth = group:getWidth()
  local groupHeight = group:getHeight()
  local contentScale = Host.contentScale
  local textureWidth = MathUtils.incrementUntilPowerOf2(groupWidth * contentScale)
  local textureHeight = MathUtils.incrementUntilPowerOf2(groupHeight * contentScale)
  local texture = OffscreenRenderedTexture.new(group, textureWidth, textureHeight)
  return texture
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MoonsetOverlayBlend:updateTime(time)
  self.shader:setAttr(SHADER_UNIFORM_TIME_INDEX, time)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MoonsetOverlayBlend.new(group)
  local texture = createTexture(group)
  local self = Sprite.newWithTexture(texture)
  Mixin.mix(self, MoonsetOverlayBlend)

  self.shader = getShader();
  self:setShader(self.shader)

  return self
end
