--- MoonsetPromptTextModel
-- @author Created by Jay Austria on June 16, 2015

class("MoonsetPromptTextModel")

require("text.dynamic.TextModel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------
MoonsetPromptTextModel.HIGHLIGHTED_ATTRIBUTE = "isHighlighted"


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getHighlightedItemsFromArray(items)
  local highlightedItems = {}

  for _, item in ipairs(items) do
    if item:getAttribute(MoonsetPromptTextModel.HIGHLIGHTED_ATTRIBUTE) then
      table.insert(highlightedItems, item)
    end
  end

  return highlightedItems
end

local function setHighlightedLineBreakAttributes(items)
  local highlightedItems = getHighlightedItemsFromArray(items)

  local firstItem = highlightedItems[1]
  local lastItem = highlightedItems[#highlightedItems]

  for _, item in ipairs(highlightedItems) do
    local isFirstItem = (item == firstItem)
    local isLastItem = (item == lastItem)

    if (not isFirstItem) then
      item:setAttribute("allowBreakLineBefore", false)
    end

    if (not isLastItem) then
      item:setAttribute("allowBreakLineAfter", false)
    end
  end
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MoonsetPromptTextModel:resetItems(items)
  setHighlightedLineBreakAttributes(items)
  self.super.resetItems(self, items)
end

function MoonsetPromptTextModel:getHighlightedItems()
  local highlightedItems = getHighlightedItemsFromArray(self.items)
  return highlightedItems
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function MoonsetPromptTextModel.new(items)
  local self = TextModel.new(items)
  Mixin.mix(self, MoonsetPromptTextModel)

  return self
end
