--- MoonsetTip
-- @author Created by karl on March 02, 2015.

class("MoonsetTip")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_BG = {1.0, 1.0, 1.0}
local COLOR_TIP = {0.3, 0.29, 0.3, 0.6}

local TIP_SPACING = 10

local BASE_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 20)

local BASE_ATTRIBUTES = {
   font = BASE_FONT
}

local ATTRIBUTES_FOR_LABELS = {
  b = { font = Font.new(FontFaces.DIN_OT_BOLD, 20) },
  --offset needed because the medium italic font file oddly has a higher baseline y
  i = { font = Font.new(FontFaces.DIN_OT_MEDIUM_ITALIC, 20), yOffset = -1 }
}

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MoonsetTip.new(text)
  local self = Group.new(Screen.fullWidth, Screen.getSafeY(0.5))
  Mixin.mix(self, MoonsetTip)

  local bg = FilledRectangle.new(self:getWidth(), self:getHeight(), COLOR_BG)
  self:insertAtCenter(bg)

  local workArea = Group.new(Screen.getSafeWidthPart(0.8), self:getHeightPart(0.9))
  workArea:setLoc(Screen.getSafeX(0.5), self:getHeightPart(0.5))
  self:insert(workArea)

  local tipLabel = Label.new(Localizer.get("Misc.TIP"), Font.new(FontFaces.DIN_OT_BOLD, 16))
  tipLabel:setColor(unpack(COLOR_TIP))
  tipLabel:setAnchorPointTopLeft()
  tipLabel:setLoc(0, workArea:getHeight())
  workArea:insert(tipLabel)

  local continueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 13))
  continueLabel:setColor(unpack(COLOR_TIP))
  continueLabel:setAnchorPointBottom()
  continueLabel:setLoc(workArea:getWidthPart(0.5), Screen.safeBottom)
  workArea:insert(continueLabel)

  local textWidth = workArea:getWidth()

  local textArea = DynamicTextArea.new(text, textWidth, BASE_ATTRIBUTES, ATTRIBUTES_FOR_LABELS, Alignment.LEFT, Alignment.TOP, lineSpacing)
  textArea:setColor(unpack(COLOR_TIP))
  textArea:setAnchorPointTop()
  textArea:setLoc(workArea:getWidthPart(0.5), workArea:getHeight() - tipLabel:getHeight() - TIP_SPACING)
  workArea:insert(textArea)

  return self
end
