--- MoonsetValidator
-- @author Created by Anton Morant on April 17, 2014.
-- ConceptValidator for Moonset.

class("MoonsetValidator")

require("validators.ConceptValidator")
require("validators.TagValidator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'sentence',
  'correct',
  'incorrect',
  'tip'
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local SINGLE_TAGS = { "blank" }

-- TODO
local function testForUnicodeCharacters(sentence)
end

local function validateSentence(self, sentence)
  --testForUnicodeCharacters(sentence)
  self:testForEndingPunctuation(sentence)
end

local function validateWordsSurroundingBlank(self, concept)
  local sentence = concept.sentence
  local correct = concept.correct
  local incorrect = concept.incorrect

  local blankTag = string.format("<%s/>", SINGLE_TAGS[1])

  local wordsInSentence = StringUtils.split(sentence, " ")
  for index, word in ipairs(wordsInSentence) do
    if word == blankTag then
      if index > 1 then
        local wordBeforeBlank = wordsInSentence[index - 1]
        self:expectNotEqual(wordBeforeBlank, correct, "word before blank is the same as correct")
        self:expectNotEqual(wordBeforeBlank, incorrect, "word before blank is the same as incorrect")
      end
      if index < #wordsInSentence then
        local wordAfterBlank = wordsInSentence[index + 1]
        self:expectNotEqual(wordAfterBlank, correct, "word after blank is the same as correct")
        self:expectNotEqual(wordAfterBlank, incorrect, "word after blank is the same as incorrect")
      end
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------


function MoonsetValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)

  validateSentence(self, concept.sentence)
  self:validateTags(concept.sentence, SINGLE_TAGS)
  self:validateTags(concept.tip, nil, nil, nil, {"i"})
  validateWordsSurroundingBlank(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MoonsetValidator.new()
  local self = ConceptValidator.new()
  TagValidator.make(self)
  Mixin.mix(self, MoonsetValidator)

  return self
end
