--- NominaConfiguration
-- @author Created by Aaron Barrett on August 7, 2014.

class("NominaConfiguration")

require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

NominaConfiguration.POINTS_PER_CORRECT_RESPONSE = 500
NominaConfiguration.POINTS_PER_FULL_SPEED_BONUS = 200
NominaConfiguration.HUMANS_PER_GAME = 4
NominaConfiguration.MAX_DIFFICULTY_TO_SHOW_NAMES = 0.5
NominaConfiguration.MAX_DIFFICULTY_TO_SHOW_FACTS = 0.45
NominaConfiguration.SPEED_BONUS_DURATION_PROPORTION = 0.35
NominaConfiguration.SPEED_BONUS_DECAY_WINDOW_START_PROPORTION = 0.5

local TOTAL_EXTRA_LIVES_BONUS = 4000
local MISTAKES_ALLOWED_RANK_TWO = 2
local MAX_CORRECT_PER_GAME = 12

local INITIAL_LIVES_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {livesCount = 8}),
  Keyframe.new(0.2, {livesCount = 7}),
  Keyframe.new(0.3, {livesCount = 6})
)

local TIMER_DURATION_KEYFRAMES = KeyframeSet.newInterpolated(
  Keyframe.new(0.0, {25, 34, 43}),
  Keyframe.new(0.125, {24, 33, 42}),
  Keyframe.new(0.25, {21, 28, 37}),
  Keyframe.new(0.45, {16, 22, 31}),
  Keyframe.new(0.5, {13, 20, 28}),
  Keyframe.new(1.0, {9, 14, 21})
)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getMaximumNumberOfExtraLives()
  return NominaConfiguration.getInitialLivesCount() - 1
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function NominaConfiguration.getInitialLivesCount()
  local difficulty = GameSession.getDifficulty()
  local initialLivesFrame = INITIAL_LIVES_THRESHOLDS:getFrameAt(difficulty)
  return initialLivesFrame.livesCount
end

function NominaConfiguration.getExtraLifeBonusPoints()
  return TOTAL_EXTRA_LIVES_BONUS / getMaximumNumberOfExtraLives()
end

function NominaConfiguration.getQuestionTime(roundNumber)
  local difficulty = GameSession.getDifficulty()
  local timerDurationFrame = TIMER_DURATION_KEYFRAMES:getFrameAt(difficulty)
  return timerDurationFrame[roundNumber]
end

function NominaConfiguration.getPassScore()
  local minimumAnswersToPass = MAX_CORRECT_PER_GAME - getMaximumNumberOfExtraLives()
  local passScore = minimumAnswersToPass * NominaConfiguration.POINTS_PER_CORRECT_RESPONSE
  return passScore
end

function NominaConfiguration.getRankTwoScore()
  local passScore = NominaConfiguration.getPassScore()
  local additionalCorrectAnswersRankTwo = (getMaximumNumberOfExtraLives() - MISTAKES_ALLOWED_RANK_TWO)
  local additionalCorrectAnswerPoints = additionalCorrectAnswersRankTwo * (NominaConfiguration.POINTS_PER_CORRECT_RESPONSE + NominaConfiguration.getExtraLifeBonusPoints())

  return passScore + additionalCorrectAnswerPoints
end

function NominaConfiguration.getRankThreeScore()
  local passScore = NominaConfiguration.getPassScore()
  local additionalCorrectAnswersRankThree = getMaximumNumberOfExtraLives()
  local additionalCorrectAnswerPoints = additionalCorrectAnswersRankThree * (NominaConfiguration.POINTS_PER_CORRECT_RESPONSE + NominaConfiguration.getExtraLifeBonusPoints())

  return passScore + additionalCorrectAnswerPoints
end
