--- NominaContainer
-- @author Created by Aaron Barrett on July 29, 2014.

class("NominaContainer")
require("NominaNameWithFactsPanel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Accessors --------------------------

function NominaContainer:setNameTagDroppedCallback(nameTagDroppedCallback)
  self.nameTagDroppedCallback = nameTagDroppedCallback
end

function NominaContainer:getHumanCircleYOffset()
  return self.humanCircleYOffset
end

function NominaContainer:getHumanCircle()
  return self.humanCircle
end

-- Prepare for Actions ----------------

function NominaContainer:prepareForCorrect()
  local nameWithFactsPanel = self.nameWithFactsPanel
  nameWithFactsPanel:prepareForCorrect()
end

function NominaContainer:prepareForMnemonic()
  local nameWithFactsPanel = self.nameWithFactsPanel
  nameWithFactsPanel:prepareForMnemonic()
end

-- Create Actions ---------------------

function NominaContainer:createMatchingSessionPresentationAction()
  local nameWithFactsPanel = self.nameWithFactsPanel
  local nameWithFactsPanelAppearAction = nameWithFactsPanel:createAppearAction()

  local appearDelay = AHDelay.new(0.2)
  local appearAction = AHSequence.new(
                                      appearDelay,
                                      nameWithFactsPanelAppearAction
                                      )

  return appearAction
end

function NominaContainer:createDisappearAction()
  local nameWithFactsPanel = self.nameWithFactsPanel
  local nameWithFactsPanelAction = nameWithFactsPanel:createDisappearAction()

  local disappearAction = AHSequence.new(
                                         nameWithFactsPanelAction,
                                         function()
                                           self:removeSelf()
                                         end
                                         )

  return disappearAction
end

function NominaContainer:createCorrectAction()
  local nameWithFactsPanel = self.nameWithFactsPanel
  local correctAction = nameWithFactsPanel:createCorrectAction()
  return correctAction
end

function NominaContainer:createRevealNameAction()
  local nameWithFactsPanel = self.nameWithFactsPanel
  local revealNameAction = nameWithFactsPanel:createRevealNameAction()
  return revealNameAction
end

-- DragAndDropHandler Container Protocol -

-- canAcceptDNDObject(object): Returns true if the object is acceptable to the container.
function NominaContainer:canAcceptDNDObject(nameTag)
  return true
end

-- notifyDNDEnter(object): Notify that an object is hovering over the container.
function NominaContainer:notifyDNDEnter(nameTag)

end

-- notifyDNDLeave(object): Notify that an object is no longer hovering over the container.
function NominaContainer:notifyDNDLeave(nameTag)

end

-- notifyDNDMove(object):Notify that an object has moved while hovering over the container.
function NominaContainer:notifyDNDMove(nameTag)

end

-- notifyDNDDrop(object): Notify that an object has been dropped on the container.
function NominaContainer:notifyDNDDrop(nameTag)
  self.nameTagDroppedCallback(nameTag)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function NominaContainer.new(width, humanCircle, humanCount)
  local human = humanCircle:getHuman()
  local nameWithFactsPanel = NominaNameWithFactsPanel.new(width, human, humanCount)

  local height = width + nameWithFactsPanel:getHeightPart(0.5)
  local self = Group.new(width, height)
  Mixin.mix(self, NominaContainer)

  -- The droppable area of the container should extend to the vertical center of the nameWithFactsPanel.
  nameWithFactsPanel:setLoc(width * 0.5, 0)
  self.humanCircleYOffset = nameWithFactsPanel:getHeightPart(0.25)

  self:insert(nameWithFactsPanel)
  self.nameWithFactsPanel = nameWithFactsPanel
  self.humanCircle = humanCircle
  self.previouslyMatchedIncorrectly = false

  return self
end
