--- NominaHuman
-- @author Created by Aaron Barrett on July 17, 2014.

class("NominaHuman")
require("NominaFact")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Genders ----------------------------

NominaHuman.GENDER_MALE = "NominaHuman.GENDER_MALE"
NominaHuman.GENDER_FEMALE = "NominaHuman.GENDER_FEMALE"

-- Content Keys -----------------------

local PREFIX_MALE = "nomina_boy"
local PREFIX_FEMALE = "nomina_girl"

local HUMANS_KEY = "humans"

local NAME_KEY = "nm"
local AVATAR_FILENAME_KEY = "art_link"
local APPEAR_TIME_KEY = "nm_app"

local DISTRACTOR_NAME_KEY = "dist"

local MNEMONIC_KEY = "mnemonic"

-- Facts ------------------------------

local FACTS_COUNT = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function extractNumberedArray(humanFromConcept, keyBase)
  local array = {}

  local index = 1
  local foundItem = nil
  repeat
    local key = keyBase .. "_" .. index
    foundItem = humanFromConcept[key]
    if (foundItem ~= nil) then
      table.insert(array, foundItem)
    end
    index = index + 1
  until (foundItem == nil)

  return array
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Getters ----------------------------

function NominaHuman:getName()
  return self.name
end

function NominaHuman:getAppearTime()
  return self.appearTime
end

function NominaHuman:getFacts()
  return self.facts
end

function NominaHuman:getDistractorNames()
  return self.distractorNames
end

function NominaHuman:getMnemonic()
  return self.mnemonic
end

function NominaHuman:getGender()
  return self.gender
end

function NominaHuman:getAvatarTexture()
  return self.avatarTexture
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function assignGender(self)
  local avatarFilename = self.avatarFilename

  if (StringUtils.startsWith(avatarFilename, PREFIX_MALE)) then
    self.gender = NominaHuman.GENDER_MALE
  elseif (StringUtils.startsWith(avatarFilename, PREFIX_FEMALE)) then
    self.gender = NominaHuman.GENDER_FEMALE
  else
    error("Image filename `" .. tostring(avatarFilename) .. "` does not start with a known gender prefix.")
  end
end

local function loadAvatarTexture(self, concept)
  local avatarFilename = self.avatarFilename
  local avatarTexture = AssetHelper.createPhotoTextureForConcept(concept, avatarFilename)
  self.avatarTexture = avatarTexture
end

local function buildFacts(self, humanFromConcept)
  local facts = {}

  for factIndex = 1, FACTS_COUNT do
    local fact = NominaFact.new(humanFromConcept, factIndex)
    table.insert(facts, fact)
  end

  self.facts = facts
end

-- Constructor ------------------------

function NominaHuman.new(concept, humanIndex)
  local self = {}
  Mixin.mix(self, NominaHuman)

  local humansArrayFromConcept = concept[HUMANS_KEY]
  local humanFromConcept = humansArrayFromConcept[humanIndex]

  self.name = humanFromConcept[NAME_KEY]
  self.avatarFilename = humanFromConcept[AVATAR_FILENAME_KEY]
  self.appearTime = tonumber(humanFromConcept[APPEAR_TIME_KEY])
  self.mnemonic = humanFromConcept[MNEMONIC_KEY]

  self.distractorNames = extractNumberedArray(humanFromConcept, DISTRACTOR_NAME_KEY)

  assignGender(self)

  loadAvatarTexture(self, concept)

  buildFacts(self, humanFromConcept)

  return self
end
