--- NominaMnemonicPanel
-- @author Created by aab29 on July 24, 2014.

class("NominaMnemonicPanel")

require("text.SizeConstrainedTextArea")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.height

local BACKGROUND_WIDTH = WIDTH
local BACKGROUND_HEIGHT = HEIGHT * 0.288

local MARGIN_X = math.max(BACKGROUND_WIDTH * 0.06, Screen.safeLeft)

local CENTER_X = WIDTH * 0.5

local TAP_TO_CONTINUE_LABEL_Y = BACKGROUND_HEIGHT * 0.13

local BACKGROUND_INITIAL_Y = -BACKGROUND_HEIGHT * 0.5
local BACKGROUND_DESTINATION_Y = BACKGROUND_HEIGHT * 0.5

-- Labels and Fonts -------------------

local HEADER_LABEL_MESSAGE = "TIP"
local HEADER_LABEL_FONT = FontFaces.DIN_OT_MEDIUM
local HEADER_LABEL_FONT_SIZE = 18

local MNEMONIC_LABEL_FONT = FontFaces.DIN_OT_REGULAR
local MNEMONIC_LABEL_FONT_HIGHLIGHT = FontFaces.DIN_OT_BOLD
local MNEMONIC_LABEL_WIDTH = BACKGROUND_WIDTH - (2 * MARGIN_X)
local MNEMONIC_LABEL_FONT_SIZE = 20
local MNEMONIC_LABEL_FONT_SIZE_MINIMUM = 10
local MNEMONIC_LABEL_HORIZONTAL_ALIGNMENT = Alignment.LEFT
local MNEMONIC_LABEL_VERTICAL_ALIGNMENT = Alignment.TOP

local TAP_TO_CONTINUE_LABEL_MESSAGE = Localizer.get("Misc.TAP_TO_CONTINUE")
local TAP_TO_CONTINUE_LABEL_FONT = FontFaces.DIN_OT_MEDIUM
local TAP_TO_CONTINUE_LABEL_FONT_SIZE = 14

-- Colors -----------------------------

local HEADER_LABEL_COLOR = ColorUtils.colorHex24ToStd(0x049ecd)
local BACKGROUND_COLOR = {1.0, 1.0, 1.0}
local MNEMONIC_LABEL_COLOR = {0.28, 0.28, 0.28}
local TAP_TO_CONTINUE_LABEL_COLOR = ColorUtils.colorHex24ToStd(0x949494)

-- Z-Indices --------------------------

local Z_BACKGROUND = 10
local Z_HEADER_LABEL = 20
local Z_TAP_TO_CONTINUE_LABEL = 30
local Z_MNEMONIC_LABEL = 40

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createOnTouchDownCallback(self)
  local function onTouchDownCallback()
    if (self.dismissable) then
      self.dismissable = false
      self.finished = true
    end
  end

  return onTouchDownCallback
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function NominaMnemonicPanel:isFinished()
  return self.finished
end

function NominaMnemonicPanel:createPresentAction()
  local background = self.background

  local presentAction = AHSequence.new(
                                       AHLocationYTo.new(background, 0.3, AHEaseSimpleOut.new(2.0), BACKGROUND_DESTINATION_Y),
                                       function()
                                         self.dismissable = true
                                       end
                                       )

  return presentAction
end

function NominaMnemonicPanel:createDismissAction()
  local background = self.background

  local dismissAction = AHSequence.new(
                                       AHLocationYTo.new(background, 0.25, AHEaseSimpleIn.new(2.0), BACKGROUND_INITIAL_Y),
                                       function()
                                         self:removeSelf()
                                         self.finished = true
                                       end
                                       )

  return dismissAction
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, mnemonicText)
  local background = FilledRectangle.new(BACKGROUND_WIDTH,
                                         BACKGROUND_HEIGHT,
                                         BACKGROUND_COLOR)
  background:setLoc(CENTER_X, BACKGROUND_INITIAL_Y)
  self:insert(background, Z_BACKGROUND)
  self.background = background

  local headerLabel = Label.new(HEADER_LABEL_MESSAGE, Font.new(HEADER_LABEL_FONT, HEADER_LABEL_FONT_SIZE))
  headerLabel:setColor(unpack(HEADER_LABEL_COLOR))
  headerLabel:setAnchorPointTopLeft()
  local headerLabelX = MARGIN_X
  local marginY = headerLabel:getHeight()
  local headerLabelY = BACKGROUND_HEIGHT - marginY
  background:insertAtLocation(headerLabel, headerLabelX, headerLabelY, Z_HEADER_LABEL)

  local tapToContinueLabel = Label.new(TAP_TO_CONTINUE_LABEL_MESSAGE, Font.new(TAP_TO_CONTINUE_LABEL_FONT, TAP_TO_CONTINUE_LABEL_FONT_SIZE))
  tapToContinueLabel:setColor(unpack(TAP_TO_CONTINUE_LABEL_COLOR))

  local tapToContinueLabelY = math.max(TAP_TO_CONTINUE_LABEL_Y, Screen.safeBottom + tapToContinueLabel:getHeight())
  background:insertAtLocation(tapToContinueLabel, CENTER_X, tapToContinueLabelY, Z_TAP_TO_CONTINUE_LABEL)
  self.tapToContinueLabel = tapToContinueLabel

  local baseAttributes = {
   font = Font.new(MNEMONIC_LABEL_FONT, MNEMONIC_LABEL_FONT_SIZE),
   color = MNEMONIC_LABEL_COLOR
  }

  local attributesForLabels = {
    i = { font = Font.new(MNEMONIC_LABEL_FONT_HIGHLIGHT, MNEMONIC_LABEL_FONT_SIZE) }
  }

  local maxHeight = headerLabel:getYBottom() - tapToContinueLabel:getYTop() - marginY
  local mnemonicLabel = SizeConstrainedTextArea.new(mnemonicText,
                                                    MNEMONIC_LABEL_WIDTH,
                                                    maxHeight,
                                                    baseAttributes,
                                                    attributesForLabels,
                                                    MNEMONIC_LABEL_FONT_SIZE_MINIMUM,
                                                    MNEMONIC_LABEL_HORIZONTAL_ALIGNMENT,
                                                    MNEMONIC_LABEL_VERTICAL_ALIGNMENT)

  mnemonicLabel:setAnchorPointTop()
  local mnemonicLabelY = headerLabel:getYBottom() - (marginY / 2)
  background:insertAtLocation(mnemonicLabel, CENTER_X, mnemonicLabelY, Z_MNEMONIC_LABEL)
  self.mnemonicLabel = mnemonicLabel

  local onTouchDownCallback = createOnTouchDownCallback(self)
  Button.makeButton(self, onTouchDownCallback)
end

-- Constructor ------------------------

function NominaMnemonicPanel.new(mnemonicText)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, NominaMnemonicPanel)

  createLayout(self, mnemonicText)

  self.finished = false
  self.isDismissable = false

  return self
end
