--- ObliviateAudioPanel
-- @author Created by amorant on November 11, 2014.

class("ObliviateAudioPanel")
require("ObliviateColors")
require("data.visualization.AudioVisualizer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

local VISUALIZER_WIDTH = Screen.getWidthPart(1)
local VISUALIZER_HEIGHT = Screen.getHeightPart(0.13)
local VISUALIZER_X = Screen.getWidthPart(0.5)
local VISUALIZER_Y = Screen.getHeightPart(0.5)

local ROUND_WIDTH = Screen.getWidthPart(0.8)
local ROUND_X = Screen.getWidthPart(0.5)
local ROUND_Y = Screen.getHeightPart(0.7)
local ROUND_FONT_SIZE = 22

local CONTEXT_WIDTH = ROUND_WIDTH
local CONTEXT_X = ROUND_X
local CONTEXT_Y = ROUND_Y
local CONTEXT_FONT_SIZE = 20

local INSTRUCTIONS_WIDTH = Screen.getWidthPart(0.8)
local INSTRUCTIONS_X = Screen.getWidthPart(0.5)
local INSTRUCTIONS_Y = Screen.getHeightPart(0.35)
local INSTRUCTIONS_FONT_SIZE = 14

-- Colors -----------------------------

local VISUALIZER_COLOR = ObliviateColors.PINK
local TEXT_COLOR = ObliviateColors.LIGHT_PURPLE
local INSTRUCTIONS_COLOR = ObliviateColors.PINK

-- Z-Indices --------------------------

local Z_VISUALIZER = 10
local Z_TEXT = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function startVisualizer(self, sound)
  self.visualizer:visualizeSound(sound)
end

local function show(self, context, currentRound)
  local roundLabel = self.roundLabel
  local contextLabel = self.contextLabel
  local readyLabel = self.readyLabel
  local listenLabel = self.listenLabel
  contextLabel:setText(context)
  roundLabel:setText(string.format("ROUND %s OF 4", currentRound))

  local showAnimation = AHSequence.new(
    AHSpawn.new(
      AHOpacityTo.new(self, 0.5, nil, 1),
      AHOpacityTo.new(roundLabel, 1, nil, 1),
      AHOpacityTo.new(readyLabel, 1, nil, 1)
    ),
    AHDelay.new(0.25),
    AHSpawn.new(
      AHOpacityTo.new(roundLabel, 0.5, nil, 0),
      AHOpacityTo.new(readyLabel, 0.5, nil, 0)
    ),
    AHOpacityTo.new(contextLabel, 0.5, nil, 1),
    AHDelay.new(0.5),
    AHOpacityTo.new(listenLabel, 1.5, nil, 1)
  )
  wait(showAnimation:start())
end

local function hide(self)
  local contextLabel = self.contextLabel
  local listenLabel = self.listenLabel
  local hideAnimation = AHSpawn.new(
    AHOpacityTo.new(self, 0.5, nil, 0),
    AHOpacityTo.new(contextLabel, 0.5, nil, 0),
    AHOpacityTo.new(listenLabel, 0.5, nil, 0)
  )
  wait(hideAnimation:start())
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ObliviateAudioPanel:playAudioAndWait(concept)
  local soundFile = concept.clip
  local context = concept.context:upper()
  local currentRound = self.currentRound

  show(self, context, currentRound)

  local sound = AssetHelper.playSoundForConcept(concept, soundFile)
  startVisualizer(self, sound)

  waitUntil(function() return sound:isFinished() end)

  self.currentRound = currentRound + 1

  hide(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self)
  local visualizer = AudioVisualizer.new(VISUALIZER_WIDTH, VISUALIZER_HEIGHT, VISUALIZER_COLOR)
  visualizer:inheritColor(self)
  visualizer:setLoc(VISUALIZER_X, VISUALIZER_Y)
  self:insert(visualizer, Z_VISUALIZER)
  self.visualizer = visualizer

  local roundLabel = Label.new("", Font.new(FontFaces.DIN_OT_MEDIUM, ROUND_FONT_SIZE), ROUND_WIDTH, Alignment.CENTER)
  roundLabel:setColor(unpack(TEXT_COLOR))
  roundLabel:setOpacity(0)
  roundLabel:setLoc(ROUND_X, ROUND_Y)
  self:insert(roundLabel, Z_TEXT)
  self.roundLabel = roundLabel

  local contextLabel = Label.new("", Font.new(FontFaces.DIN_OT_MEDIUM, CONTEXT_FONT_SIZE), CONTEXT_WIDTH, Alignment.CENTER)
  contextLabel:setColor(unpack(TEXT_COLOR))
  contextLabel:setOpacity(0)
  contextLabel:setLoc(CONTEXT_X, CONTEXT_Y)
  self:insert(contextLabel, Z_TEXT)
  self.contextLabel = contextLabel

  local readyLabel = Label.new("GET READY", Font.new(FontFaces.DIN_OT_MEDIUM, INSTRUCTIONS_FONT_SIZE), INSTRUCTIONS_WIDTH, Alignment.CENTER)
  readyLabel:setColor(unpack(INSTRUCTIONS_COLOR))
  readyLabel:setOpacity(0)
  readyLabel:setLoc(INSTRUCTIONS_X, INSTRUCTIONS_Y)
  self:insert(readyLabel, Z_TEXT)
  self.readyLabel = readyLabel

  local listenLabel = Label.new("LISTEN CAREFULLY", Font.new(FontFaces.DIN_OT_MEDIUM, INSTRUCTIONS_FONT_SIZE), INSTRUCTIONS_WIDTH, Alignment.CENTER)
  listenLabel:setColor(unpack(INSTRUCTIONS_COLOR))
  listenLabel:setOpacity(0)
  listenLabel:setLoc(INSTRUCTIONS_X, INSTRUCTIONS_Y)
  self:insert(listenLabel, Z_TEXT)
  self.listenLabel = listenLabel
end

-- Constructor ------------------------

function ObliviateAudioPanel.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ObliviateAudioPanel)

  createElements(self)
  self:setOpacity(0)

  self.currentRound = 1

  return self
end
