--- ObliviateConfiguration
-- @author Created by amorant on November 11, 2014.

class("ObliviateConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ObliviateConfiguration.CORRECT_ANSWERS_TO_SHOW_PER_GAME = 13
ObliviateConfiguration.TOTAL_ANSWERS_TO_SHOW_PER_ROUND = { 4, 5, 6, 6 }
ObliviateConfiguration.MINIMUM_CORRECT_ANSWERS_TO_SHOW_PER_ROUND = { 2, 2, 3, 3 }
ObliviateConfiguration.MAXIMUM_CORRECT_ANSWERS_TO_SHOW_PER_ROUND = { 3, 3, 4, 4 }
ObliviateConfiguration.ROUNDS_COUNT = 4
ObliviateConfiguration.POINTS_PER_CORRECT_RESPONSE = 400
ObliviateConfiguration.POINTS_PER_LIVES_BONUS = 1200
ObliviateConfiguration.INITIAL_LIVES_COUNT = 4
ObliviateConfiguration.EXTRA_LIVES_NEEDED_RANK_TWO = 2
ObliviateConfiguration.EXTRA_LIVES_NEEDED_RANK_THREE = 3
ObliviateConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 5.0
ObliviateConfiguration.SPEED_BONUS_DECAY_START = 2.0
ObliviateConfiguration.POINTS_PER_FULL_SPEED_BONUS = 200

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Score calculations -----------------

local function getPassScore()
  local correctAnswerPoints = ObliviateConfiguration.POINTS_PER_CORRECT_RESPONSE
  local totalAnswerPoints = correctAnswerPoints * ObliviateConfiguration.CORRECT_ANSWERS_TO_SHOW_PER_GAME
  return totalAnswerPoints
end

local function getScoreForLivesAndSpeedBonuses(extraLivesCount, speedBonusCount)
  local passScore = getPassScore()
  local livesBonus = extraLivesCount * ObliviateConfiguration.POINTS_PER_LIVES_BONUS
  local speedBonus = speedBonusCount * ObliviateConfiguration.POINTS_PER_FULL_SPEED_BONUS
  local score = passScore + livesBonus + speedBonus
  return score
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Ranks ------------------------------

function ObliviateConfiguration.getRankTwoScore()
  local extraLivesCount = ObliviateConfiguration.EXTRA_LIVES_NEEDED_RANK_TWO
  local speedBonusCount = 0
  local rankTwoScore = getScoreForLivesAndSpeedBonuses(extraLivesCount, speedBonusCount)
  return rankTwoScore
end

function ObliviateConfiguration.getRankThreeScore()
  local extraLivesCount = ObliviateConfiguration.EXTRA_LIVES_NEEDED_RANK_THREE
  local speedBonusCount = 0
  local rankThreeScore = getScoreForLivesAndSpeedBonuses(extraLivesCount, speedBonusCount)
  return rankThreeScore
end

function ObliviateConfiguration.getDifficultyKeyFrames()
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Easy = 1,
      }
    },
    {
      key = 0.075,
      valueProbabilities = {
        Easy = 0.95,
        Medium = 0.05
      }
    },
    {
      key = 0.15,
      valueProbabilities = {
        Easy = 0.85,
        Medium = 0.15
      }
    },
    {
      key = 0.225,
      valueProbabilities = {
        Easy = 0.65,
        Medium = 0.35
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.35,
        Medium = 0.65
      }
    },
    {
      key = 0.375,
      valueProbabilities = {
        Easy = 0.15,
        Medium = 0.85
      }
    },
    {
      key = 0.45,
      valueProbabilities = {
        Medium = 0.95,
        Hard = 0.05
      }
    },
    {
      key = 0.5,
      valueProbabilities = {
        Medium = 0.85,
        Hard = 0.15
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        Medium = 0.6,
        Hard = 0.4
      }
    },
    {
      key = 0.675,
      valueProbabilities = {
        Medium = 0.4,
        Hard = 0.6
      }
    },
    {
      key = 0.75,
      valueProbabilities = {
        Medium = 0.15,
        Hard = 0.85
      }
    },
    {
      key = 0.8,
      valueProbabilities = {
        Hard = 1.0
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 1.0
      }
    }
  }
end
