--- ObliviateGlow
-- @author Created by amorant on November 25, 2014.

class("ObliviateGlow")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DISTANCE_FOR_ZERO_OPACITY = Screen.getWidthPart(0.4)
local FADE_OUT_DURATION = 0.5

local PARTICLES_COUNT = 20
local PARTICLES_DURATION = 1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function doParticles(self, lastButton)
  if not self.particlesEnabled then
    return
  end

  local glow = self.glow
  local particles = Particles.new("answer_particles.pex")
  particles:setLoc(lastButton:getX(), glow:getY())
  self:insert(particles, Z_PARTICLES)

  local particlesAction = AHCallLogic.new(function()
    particles:start()
    particles:surge(PARTICLES_COUNT)
    waitFor(PARTICLES_DURATION)
    particles:stop()
    particles:removeSelf()
  end)

  return particlesAction:start()
end

local function startFadeOut(self)
  local fadeOutAnimation = self.fadeOutAnimation
  if fadeOutAnimation then
    fadeOutAnimation:stop()
  end

  local glow = self.glow
  fadeOutAnimation = AHOpacityTo.new(glow, FADE_OUT_DURATION, AHEaseSineIn.new(), 0)
  self.fadeOutAnimation = fadeOutAnimation
  fadeOutAnimation:start()
end

local function stopUpdateCoroutine(self)
  local updateCoroutine = self.updateCoroutine
  updateCoroutine:stop()
  self.updateCoroutine = nil
end

local function update(self)
  local trackedButtons = self.trackedButtons
  local glow = self.glow

  local opacity = glow:getOpacity()
  local glowY = glow:getY()
  local buttonsToStopTracking = {}

  local remainingTrackedButtons = TableUtils.shallowCopy(trackedButtons)
  for _, button in ipairs(trackedButtons) do
    local _, buttonY = self:transformObjectLoc(button)
    local distance = glowY - buttonY
    distance = math.min(distance, DISTANCE_FOR_ZERO_OPACITY)

    if distance > 0 then
      local opacityForThisButton = MathUtils.interpolateLinear(distance, 0, DISTANCE_FOR_ZERO_OPACITY, 1, 0)
      opacity = math.max(opacity, opacityForThisButton)
    else
      TableUtils.removeValueFromArray(remainingTrackedButtons, button)
      opacity = 1
      doParticles(self, button)
    end
  end

  if #remainingTrackedButtons < #trackedButtons then
    startFadeOut(self)
  end

  if #remainingTrackedButtons == 0 then
    stopUpdateCoroutine(self)
  end

  glow:setOpacity(opacity)
  self.trackedButtons = remainingTrackedButtons
end

local function startUpdateCoroutine(self)
  local updateCoroutine = self.updateCoroutine
  if not updateCoroutine then
    updateCoroutine = runInCoroutine(function()
      while true do
        update(self)
        coroutine.yield()
      end
    end)
    self.updateCoroutine = updateCoroutine
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ObliviateGlow:trackButton(button)
  local trackedButtons = self.trackedButtons
  table.insert(trackedButtons, button)

  startUpdateCoroutine(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self, filename)
  local glow = Sprite.newWithName(filename)
  glow:setOpacity(0)
  glow:setEffectiveDims(self:getWidth())
  glow:setAnchorPointTop()
  glow:setLoc(self:getWidthPart(0.5), self:getHeight())
  self:insert(glow)
  self.glow = glow
end

-- Constructor ------------------------

function ObliviateGlow.new(filename, particlesEnabled)
  local self = Group.newFullScreen()
  Mixin.mix(self, ObliviateGlow)

  createElements(self, filename)

  self.trackedButtons = {}
  self.fadeOutAnimation = nil
  self.particlesEnabled = particlesEnabled

  return self
end
