--- ObliviateOutroPanel
-- @author Created by amorant on November 21, 2014.

class("ObliviateOutroPanel")
require("ObliviateColors")
require("Particles")
require("graphics.primitives.Checkmark")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

local CHECKMARK_WIDTH = Screen.getWidthPart(0.5)
local CHECKMARK_SHORT_PROPORTION = 0.39
local CHECKMARK_THICKNESS = 0.22

-- Colors -----------------------------

local TOP_COLOR = ObliviateColors.DARK_CYAN
local BOTTOM_COLOR = ObliviateColors.LIGHT_CYAN
local CHECKMARK_COLOR = ObliviateColors.BLACK
local TOP_COLOR_INVISIBLE = ColorUtils.colorWithOpacityMultiplier(TOP_COLOR, 0)
local BOTTOM_COLOR_INVISIBLE = ColorUtils.colorWithOpacityMultiplier(BOTTOM_COLOR, 0)

-- Animation params -------------------

local IMPACT_TIME = 0.5
local DELAY_BEFORE_PARTICLES = IMPACT_TIME - 0.05
local PARTICLES_DURATION = 1
local PARTICLES_COUNT = 35
local SOUND_DURATION = 3

-- Z-Indices --------------------------

local Z_BACKGROUND = 10
local Z_PARTICLES = 20
local Z_CHECKMARK = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createParticlesAction(self)
  local particles = Particles.new("outro_particles.pex")

  local particlesAction = AHCallLogic.new(function()
    self:insertAtCenter(particles, Z_PARTICLES)
    particles:start()
    particles:surge(PARTICLES_COUNT)
    waitFor(PARTICLES_DURATION)
    particles:stop()
    particles:removeSelf()
  end)

  return particlesAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ObliviateOutroPanel:show()
  local background = self.background
  local checkmark = self.checkmark

  checkmark:setScl(5)
  checkmark:setRot(-90)
  local dropCheckmark = AHSpawn.new(
    AHScaleTo.new(checkmark, IMPACT_TIME, nil, 1),
    AHRotationTo.new(checkmark, IMPACT_TIME, nil, 0),
    AHOpacityTo.new(checkmark, IMPACT_TIME, nil, 1)
  )

  local bringBackground = background:newGradientToAction(IMPACT_TIME, nil, TOP_COLOR, BOTTOM_COLOR)

  local particles = createParticlesAction(self)

  local playSound = function() Sound.playSoundEffect("outro") end

  return AHSpawn.new(
    dropCheckmark,
    bringBackground,
    AHDelayed.new(DELAY_BEFORE_PARTICLES, particles),
    playSound,
    AHDelay.new(SOUND_DURATION)
  ):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self)
  local background = RectangleGradient.new(self:getWidth(), self:getHeight(), TOP_COLOR_INVISIBLE, BOTTOM_COLOR_INVISIBLE)
  self:insertAtCenter(background, Z_BACKGROUND)
  self.background = background

  local checkmark = FilledCheckmark.new(CHECKMARK_WIDTH, CHECKMARK_SHORT_PROPORTION, CHECKMARK_THICKNESS, CHECKMARK_COLOR)
  checkmark:setOpacity(0)
  self:insertAtCenter(checkmark, Z_CHECKMARK)
  self.checkmark = checkmark
end

-- Constructor ------------------------

function ObliviateOutroPanel.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ObliviateOutroPanel)

  createElements(self)

  return self
end
