--- ObliviateTimer
-- @author Created by amorant on November 18, 2014.

class("ObliviateTimer")
require("ObliviateColors")
require("ObliviateShadowOverlay")
require("graphics.MultiGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

local GRADIENT_WIDTH = Screen.width
local GRADIENT_HEIGHT = 2 * Screen.height
local Y_FOR_PROGRESS_ONE = 0
local Y_FOR_PROGRESS_ZERO = Screen.height - GRADIENT_HEIGHT

-- Colors -----------------------------

local COLORS = {
  ObliviateColors.BLACK,
  ObliviateColors.BLACK,
  ObliviateColors.PURPLE,
  ObliviateColors.BRIGHT_PINK
}

local THICKNESSES = {
  1,
  0.07,
  0.93
}

local COLOR_RED_BLINK = ObliviateColors.RED

-- Z-Indices --------------------------

local Z_BLINK = 30
local Z_GRADIENT = 20

-- Misc -------------------------------

local WARNING_COUNT = 3
local WARNING_INTERVAL = 1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWarningHandler(self)
  return function()
    local blink = self.blink
    local blinkAnimation = AHSequence.new(
      AHOpacityTo.new(blink, 0.25, nil, 1),
      AHOpacityTo.new(blink, 0.25, nil, 0)
    )

    blinkAnimation:start()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ObliviateTimer:setProgress(progress)
  self.super.setProgress(self, progress)

  local progress = self:getProgress()
  local gradient = self.gradient
  local y = MathUtils.interpolateLinearFromNormalized(progress, Y_FOR_PROGRESS_ZERO, Y_FOR_PROGRESS_ONE)
  gradient:setY(y)
end

function ObliviateTimer:refill()
  return AHTimeTo.new(self, 0.5, AHEaseSineInOut.new(), 0):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self)
  local isHorizontal = false
  local gradient = MultiGradient.new(self:getWidth(), GRADIENT_HEIGHT, false, COLORS, THICKNESSES)
  gradient:setAnchorPointBottomLeft()
  gradient:setLoc(0, 0)
  self:insert(gradient, Z_GRADIENT)
  self.gradient = gradient

  local blink = ObliviateShadowOverlay.new(self:getDims())
  blink:setColor(unpack(COLOR_RED_BLINK))
  blink:setOpacity(0)
  local blinkFiller = FilledRectangle.new(self:getWidth(), self:getHeight())
  blinkFiller:inheritColor(blink)
  blinkFiller:setOpacity(0.2)
  blink:insertAtCenter(blinkFiller)
  self:insertAtCenter(blink, Z_BLINK)
  self.blink = blink
end

-- Constructor ------------------------

function ObliviateTimer.new(timeoutListener)
  local self = Group.newFullScreen()
  ProgressIndicator.makeProgressIndicator(self)
  ProgressTimer.makeTimer(self, nil, timeoutListener)
  Mixin.mix(self, ObliviateTimer)

  self:setWarningCount(WARNING_COUNT)
  self:setWarningInterval(WARNING_INTERVAL)
  createElements(self)
  self:addStandardAudibleWarningSequence()
  self:addWarningHandler(getWarningHandler(self))

  return self
end
