--- ObliviateTipPanel
-- @author Created by amorant on November 13, 2014.

class("ObliviateTipPanel")
require("ObliviateColors")
require("ObliviateShadowOverlay")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TIPS = {
  "To help yourself remember information about people, places, or things, create a mental map and associate the items to different areas of it.",
  "Listen without judging or jumping to conclusions to better understand and remember what you hear.",
  "After listening to new information, reflect and paraphrase what you heard to aid understanding and retention.",
  "Take a deep breath before listening to something to reduce stress, increase focus, and boost retention.",
  "Focus on keywords and cues as you listen to help you recall the most important information.",
  "To help yourself remember a large amount of information, chunk it into smaller, more memorable pieces.",
  "Sitting up straight and maintaining good posture while listening can help improve your retention.",
  "If you formulate questions as you listen to new information, you have a better chance of remembering it.",
  "As you listen to information, mentally repeat what you hear to improve your retention.",
  "Use the first letter of each word or phrase in the information you hear to create a memorable acronym or mnemonic.",
  "Improve your ability to recall words by creating a mental image as you hear them.",
  "Limit external distractions to listen more effectively and increase the chances that you'll remember what you hear.",
  "To help yourself recall new information, try to link it to a personal experience or something you already know.",
  "Repeat important information back to yourself in order to remember it more effectively."
}

-- Dimensions -------------------------

local COLUMN_SPACING = Screen.getHeightPart(0.05)

-- Text -------------------------------

local FONT_TITLE = FontFaces.DIN_OT_BOLD
local FONT_TIP = FontFaces.DIN_OT_MEDIUM
local FONT_TAP = FontFaces.DIN_OT_BOLD

local FONT_SIZE_TITLE = 18
local FONT_SIZE_TIP = 16
local FONT_SIZE_TAP = 14

-- Colors -----------------------------

local COLOR_TITLE = ObliviateColors.LIGHT_PURPLE
local COLOR_TIP = ObliviateColors.PURE_WHITE
local COLOR_TAP = ObliviateColors.PINK
local COLOR_SHADOW = ObliviateColors.DARK_PURPLE

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ObliviateTipPanel:show()
  return AHOpacityTo.new(self, 0.5, nil, 1):start()
end

function ObliviateTipPanel:hide()
  return AHOpacityTo.new(self, 0.5, nil, 0):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self)
  local column = Column.new(COLUMN_SPACING)
  local labelWidth = self:getWidthPart(0.8)

  local titleText = "MEMORY TIP:"
  local titleLabel = Label.new(titleText, Font.new(FONT_TITLE, FONT_SIZE_TITLE), labelWidth, Alignment.CENTER)
  titleLabel:setColor(unpack(COLOR_TITLE))
  titleLabel:inheritOpacity(self)
  column:insert(titleLabel)
  self.titleLabel = titleLabel

  local tipText = TIPS[math.random(#TIPS)]
  local tipLabel = TextArea.new(tipText, Font.new(FONT_TIP, FONT_SIZE_TIP), labelWidth, Alignment.CENTER)
  tipLabel:setColor(unpack(COLOR_TIP))
  tipLabel:inheritOpacity(self)
  column:insert(tipLabel)
  self.tipLabel = tipLabel

  local tapText = Localizer.get("Misc.TAP_TO_CONTINUE")
  local tapLabel = Label.new(tapText, Font.new(FONT_TAP, FONT_SIZE_TAP), labelWidth, Alignment.CENTER)
  tapLabel:setColor(unpack(COLOR_TAP))
  tapLabel:inheritOpacity(self)
  column:insert(tapLabel)
  self.tapLabel = tapLabel

  column:alignChildren()
  self:insertAtCenter(column)

  local shadow = ObliviateShadowOverlay.new(self:getDims())
  shadow:setColor(unpack(COLOR_SHADOW))
  shadow:inheritOpacity(self)
  self:insertAtCenter(shadow)
end

-- Constructor ------------------------

function ObliviateTipPanel.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, ObliviateTipPanel)

  createElements(self)

  self:setOpacity(0)

  return self
end
