--- ObliviateValidator
-- @author Created by amorant on November 11, 2014.
-- ConceptValidator for Obliviate.

class("ObliviateValidator")

require("validators.ConceptValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  "question",
  "answers",
  "clip",
  "category",
  "context"
}

local REQUIRED_ANSWER_FIELDS = {
  "type",
  "text",
  "icon_link"
}

local CORRECT_ANSWERS_COUNT = 4
local WRONG_ANSWERS_COUNT = 3
local ANSWER_TYPE_CORRECT = "correct"
local ANSWER_TYPE_WRONG = "wrong"

local CATEGORIES = {
  "food",
  "household_items",
  "sports",
  "transportation",
  "clothing",
  "hobbies",
  "office_supplies",
  "times",
  "animals",
  "buildings",
  "weather",
  "geographical_features",
  "professions",
  "calendar_days",
  "travel"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validateAnswerTypes(self, answers)
  for _, answer in ipairs(answers) do
    self:assertHasKeys(answer, REQUIRED_ANSWER_FIELDS)
    self:expectIsContained(answer.type, { ANSWER_TYPE_CORRECT, ANSWER_TYPE_WRONG }, "answer type")
  end
end

local function validateAnswerCounts(self, answers)
  local correctAnswers = TableUtils.getFilteredArray(answers, function(answer) return answer.type == ANSWER_TYPE_CORRECT end)
  local wrongAnswers = TableUtils.getFilteredArray(answers, function(answer) return answer.type == ANSWER_TYPE_WRONG end)

  self:expectEqual(#correctAnswers, CORRECT_ANSWERS_COUNT, "Wrong number of correct answers")
  self:expectEqual(#wrongAnswers, WRONG_ANSWERS_COUNT, "Wrong number of wrong answers")
end

local function validateAnswerUniqueness(self, answers)
  local seenAnswerTexts = {}

  for _, answer in ipairs(answers) do
    local answerText = answer.text
    self:expectFalse(seenAnswerTexts[answerText], string.format("Duplicated answer '%s'", answerText))
    seenAnswerTexts[answerText] = true
  end
end

local function validateCategory(self, category)
  self:expectIsContained(category, CATEGORIES, "category")
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ObliviateValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)

  local answers = concept.answers
  validateAnswerTypes(self, answers)
  validateAnswerCounts(self, answers)
  validateAnswerUniqueness(self, answers)

  local category = concept.category
  validateCategory(self, category)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ObliviateValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, ObliviateValidator)

  return self
end
