--- OnboardioAnalytics
-- @author Created by anton on November 24, 2015.

class("OnboardioAnalytics")

require("Analytics")
require("ExperimentsConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local EVENT_KEY_GET_STARTED_OR_LOGIN = "OnboardingGetStartedOrLogInSeen"
local EVENT_KEY_GET_STARTED_OR_LOGIN_GET_STARTED_TAPPED = "OnboardingGetStartedOrLogInGetStartedTapped"
local EVENT_KEY_GET_STARTED_OR_LOGIN_LOGIN_TAPPED = "OnboardingGetStartedOrLogInLogInTapped"

local EVENT_KEY_ADD_ANOTHER_ACCOUNT_LOG_IN_ACTION = "OnboardingSavedAccountsAddAccountLogInTapped"
local EVENT_KEY_ADD_ANOTHER_ACCOUNT_SIGN_UP_ACTION = "OnboardingSavedAccountsAddAccountGetStartedTapped"

local EVENT_KEY_PITCH_CARD_SWIPED = "OnboardingPitchCardSwiped"
local EVENT_KEY_PITCH_SKIPPED = "OnboardingPitchSkipTapped"

local EVENT_KEY_GOAL_SELECTION_STARTED = "OnboardingGoalSelectionStarted"
local EVENT_KEY_GOAL_SELECTION_ARTICULATE_THOUGHTS_ANSWERED = "OnboardingGoalSelectionArticulateThoughtsAnswered"
local EVENT_KEY_GOAL_SELECTION_STRENGTHEN_FOCUS_ANSWERED = "OnboardingGoalSelectionStrengthenFocusAnswered"
local EVENT_KEY_GOAL_SELECTION_IMPROVE_WRITING_ANSWERED = "OnboardingGoalSelectionImproveWritingAnswered"
local EVENT_KEY_GOAL_SELECTION_PROCESS_FASTER_ANSWERED = "OnboardingGoalSelectionProcessFasterAnswered"
local EVENT_KEY_GOAL_SELECTION_ENHANCE_MEMORY_ANSWERED = "OnboardingGoalSelectionEnhanceMemoryAnswered"
local EVENT_KEY_GOAL_SELECTION_CALCULATE_QUICKLY_ANSWERED = "OnboardingGoalSelectionCalculateQuicklyAnswered"
local EVENT_KEY_GOAL_SELECTION_IMPROVE_VOCABULARY_ANSWERED = "OnboardingGoalSelectionImproveVocabularyAnswered"
local EVENT_KEY_GOAL_SELECTION_SKIPPED = "OnboardingGoalSelectionSkipTapped"

local EVENT_KEY_SOCIAL_PROOF_SCREEN = "OnboardingSocialProofScreenSeen"
local EVENT_KEY_SOCIAL_PROOF_CONTINUE_TAPPED = "OnboardingSocialProofContinueTapped"
local EVENT_KEY_SOCIAL_PROOF_SKIPPED = "OnboardingSocialProofSkipped"

local EVENT_KEY_MOTIVATION_SCREEN = "OnboardingMotivationScreenSeen"
local EVENT_KEY_MOTIVATION_SKIPPED = "OnboardingMotivationScreenSkipped"
local EVENT_KEY_MOTIVATION_ANSWERED = "OnboardingMotivationSelected"

local EVENT_KEY_MOTIVATION_REINFORCEMENT_SCREEN = "OnboardingMotivationReinforcementScreenSeen"

local EVENT_KEY_PERSONALIZATION_ARTICULATE_THOUGHTS_ANSWERED = "OnboardingChallengeSectionArticulateThoughtsAnswered"
local EVENT_KEY_PERSONALIZATION_EASILY_DISTRACTED_ANSWERED = "OnboardingChallengeSectionEasilyDistractedReadingAnswered"
local EVENT_KEY_PERSONALIZATION_THOUGHTS_ON_PAPER_ANSWERED = "OnboardingChallengeSectionThoughtsOnPaperAnswered"
local EVENT_KEY_PERSONALIZATION_FORGET_NAMES_ANSWERED = "OnboardingChallengeSectionForgetNamesDatesDetailsAnswered"
local EVENT_KEY_PERSONALIZATION_USE_CALCULATOR_ANSWERED = "OnboardingChallengeSectionUseCalculatorAnswered"

local EVENT_KEY_TIME_COMMITMENT_ANSWERED = "OnboardingTimeCommitAnswered"
local EVENT_KEY_TIME_COMMITMENT_SKIPPED = "OnboardingTimeCommitSkipped"

local EVENT_KEY_LEARNING_PACE_ANSWERED = "OnboardingLearningPaceAnswered"
local EVENT_KEY_LEARNING_PACE_SKIPPED = "OnboardingLearningPaceSkipped"

local EVENT_KEY_ADHD_QUESTION_SCREEN = "OnboardingADHDQuestionScreenSeen"
local EVENT_KEY_ADHD_QUESTION_ANSWERED = "OnboardingADHDQuestionAnswered"
local EVENT_KEY_ADHD_QUESTION_SKIPPED = "OnboardingADHDQuestionSkipped"

local EVENT_KEY_ESL_QUESTION_ANSWERED = "OnboardingESLAnswered"
local EVENT_KEY_ESL_QUESTION_SKIPPED = "OnboardingESLSkipped"

local EVENT_KEY_OTHER_BRAIN_ACTIVITIES_QUESTION_ANSWERED = "OnboardingOtherBrainActivitiesAnswered"
local EVENT_KEY_OTHER_BRAIN_ACTIVITIES_QUESTION_SKIPPED = "OnboardingOtherBrainActivitiesSkipped"

local EVENT_KEY_FIRST_GOAL_QUESTION_ANSWERED = "OnboardingAccomplishmentAnswered"
local EVENT_KEY_FIRST_GOAL_QUESTION_SKIPPED = "OnboardingAccomplishmentSkipped"

local EVENT_KEY_HOW_DID_YOU_HEAR_ABOUT_US_SCREEN = "OnboardingHowDidYouHearAboutUsScreen"
local EVENT_KEY_HOW_DID_YOU_HEAR_ABOUT_US_ANSWERED = "OnboardingHowDidYouHearAboutUsAnswered"
local EVENT_KEY_HOW_DID_YOU_HEAR_ABOUT_US_SKIPPED = "OnboardingHowDidYouHearAboutUsSkipped"

local EVENT_KEY_ENGLISH_TYPE_SCREEN = "OnboardingEnglishTypeScreen"
local EVENT_KEY_ENGLISH_TYPE_SELECTED = "OnboardingEnglishTypeSelected"

local EVENT_KEY_TEST_START_SCREEN = "OnboardingTestStartScreen"
local EVENT_KEY_TEST_STARTED = "OnboardingTestStarted"
local EVENT_KEY_TEST_SKIPPED = "OnboardingTestSkipped"
local EVENT_KEY_TEST_SECTION_COMPLETED = "OnboardingTestSectionCompleted"
local EVENT_KEY_TEST_QUESTION_ANSWERED = "OnboardingPretestQuestionAnswered"

local EVENT_KEY_EPQ_INTRO_SCREEN = "OnboardingEPQIntroScreen"
local EVENT_KEY_EPQ_REVEAL_SCREEN = "OnboardingEPQRevealScreen"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function reportEvent(eventKey, propertiesMapping)
  Analytics.reportEvent(eventKey, propertiesMapping)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioAnalytics.reportGetStartedOrLoginSeen(isLoginButtonFirst)
  local properties = {
    ["login_button_first"] = isLoginButtonFirst
  }
  reportEvent(EVENT_KEY_GET_STARTED_OR_LOGIN, properties)
end

function OnboardioAnalytics.reportGetStartedTapped(isLoginButtonFirst)
  local properties = {
    ["login_button_first"] = isLoginButtonFirst
  }
  reportEvent(EVENT_KEY_GET_STARTED_OR_LOGIN_GET_STARTED_TAPPED, properties)
end

function OnboardioAnalytics.reportLogInTapped(isLoginButtonFirst)
  local properties = {
    ["login_button_first"] = isLoginButtonFirst
  }
  reportEvent(EVENT_KEY_GET_STARTED_OR_LOGIN_LOGIN_TAPPED, properties)
end

function OnboardioAnalytics.reportPitchScreenSwiped(index, isForward)
  local properties = {
    ["screen_number"] = index,
    ["swiped_forward"] = isForward
  }
  reportEvent(EVENT_KEY_PITCH_CARD_SWIPED, properties)
end

function OnboardioAnalytics.reportPitchSkipped(index)
  local properties = {
    ["screen_number"] = index
  }
  reportEvent(EVENT_KEY_PITCH_SKIPPED, properties)
end

function OnboardioAnalytics.reportGoalSelectionStarted()
  reportEvent(EVENT_KEY_GOAL_SELECTION_STARTED)
end

function OnboardioAnalytics.reportGoalSelectionCardAnswered(cardId, selected)
  local goalSelectionCardIdToEvent = {
    ["articulate_thoughts_more_clearly"] = EVENT_KEY_GOAL_SELECTION_ARTICULATE_THOUGHTS_ANSWERED,
    ["strengthen_focus_while_reading"] = EVENT_KEY_GOAL_SELECTION_STRENGTHEN_FOCUS_ANSWERED,
    ["improve_writing"] = EVENT_KEY_GOAL_SELECTION_IMPROVE_WRITING_ANSWERED,
    ["process_written_information_faster"] = EVENT_KEY_GOAL_SELECTION_PROCESS_FASTER_ANSWERED,
    ["enchance_memory"] = EVENT_KEY_GOAL_SELECTION_ENHANCE_MEMORY_ANSWERED,
    ["caluclate_more_quickly_and_precisely"] = EVENT_KEY_GOAL_SELECTION_CALCULATE_QUICKLY_ANSWERED,
    ["improve_vocabulary"] = EVENT_KEY_GOAL_SELECTION_IMPROVE_VOCABULARY_ANSWERED
  }
  local eventKey = goalSelectionCardIdToEvent[cardId]
  if (eventKey == nil) then
    Log.error("Tried to report GoalSelectionAnswered event but cardId was not found: " .. tostring(cardId))
  else
    local properties = {
      ["is_selected"] = selected
    }
    reportEvent(eventKey, properties)
  end
end

function OnboardioAnalytics.reportGoalSelectionSkipTapped(sourceCard)
  local properties = {
    ["source"] = sourceCard
  }
  reportEvent(EVENT_KEY_GOAL_SELECTION_SKIPPED, properties)
end

function OnboardioAnalytics.reportTestStartScreenSeen()
  reportEvent(EVENT_KEY_TEST_START_SCREEN)
end

function OnboardioAnalytics.reportTestStarted()
  reportEvent(EVENT_KEY_TEST_STARTED)
end

function OnboardioAnalytics.reportTestSkipped()
  reportEvent(EVENT_KEY_TEST_SKIPPED)
end

function OnboardioAnalytics.reportTestSectionCompleted(sectionIdentifier, sectionIsSelfRate, questionsAnsweredCount, questionsCorrectCount, questionsPossibleCount, scores)
  local properties = {
    ["section_identifier"] = sectionIdentifier,
    ["section_is_self_rate"] = sectionIsSelfRate,
    ["questions_correct_count"] = questionsCorrectCount,
    ["questions_answered_count"] = questionsAnsweredCount,
    ["questions_possible_count"] = questionsPossibleCount,
    ["onboarding_pretest_score_math"] = scores[OnboardioContent.SKILL_GROUP_IDENTIFIER_MATH].correct,
    ["onboarding_pretest_score_reading"] = scores[OnboardioContent.SKILL_GROUP_IDENTIFIER_READING].correct,
    ["onboarding_pretest_score_writing"] = scores[OnboardioContent.SKILL_GROUP_IDENTIFIER_WRITING].correct,
    ["onboarding_pretest_score_speaking"] = scores[OnboardioContent.SKILL_GROUP_IDENTIFIER_SPEAKING].correct
  }

  reportEvent(EVENT_KEY_TEST_SECTION_COMPLETED, properties)
end

function OnboardioAnalytics.reportQuestionAnswered(answerId, isCorrect, answerTime, questionId, earnedPoints)
  local properties = {
    ["onboarding_pretest_answer_id"] = string.lower(answerId),
    ["onboarding_pretest_answer_is_correct"] = isCorrect,
    ["onboarding_pretest_answer_time_in_ms"] = answerTime,
    ["onboarding_pretest_question_id"] = string.lower(questionId),
    ["onboarding_pretest_answer_earned_points"] = earnedPoints
  }
  reportEvent(EVENT_KEY_TEST_QUESTION_ANSWERED, properties)
end

function OnboardioAnalytics.reportEPQIntroSeen()
  reportEvent(EVENT_KEY_EPQ_INTRO_SCREEN)
end

function OnboardioAnalytics.reportEPQRevealSeen()
  reportEvent(EVENT_KEY_EPQ_REVEAL_SCREEN)
end

function OnboardioAnalytics.reportHowDidYouHearAboutUsSeen()
  reportEvent(EVENT_KEY_HOW_DID_YOU_HEAR_ABOUT_US_SCREEN)
end

function OnboardioAnalytics.reportHowDidYouHearAboutUsAnswered(response)
  local properties = {
    ["hdyhau_response"] = response
  }
  reportEvent(EVENT_KEY_HOW_DID_YOU_HEAR_ABOUT_US_ANSWERED, properties)
end

function OnboardioAnalytics.reportHowDidYouHearAboutUsSkipped()
  reportEvent(EVENT_KEY_HOW_DID_YOU_HEAR_ABOUT_US_SKIPPED)
end

function OnboardioAnalytics.reportAddAnotherAccountLogInAction()
  reportEvent(EVENT_KEY_ADD_ANOTHER_ACCOUNT_LOG_IN_ACTION)
end

function OnboardioAnalytics.reportAddAnotherAccountSignUpAction()
  reportEvent(EVENT_KEY_ADD_ANOTHER_ACCOUNT_SIGN_UP_ACTION)
end

function OnboardioAnalytics.reportEnglishTypeScreenSeen()
  reportEvent(EVENT_KEY_ENGLISH_TYPE_SCREEN)
end

function OnboardioAnalytics.reportEnglishTypeSelected(locale)
  local properties = {
    ["type"] = locale
  }
  reportEvent(EVENT_KEY_ENGLISH_TYPE_SELECTED, properties)
end

function OnboardioAnalytics.reportSocialProofScreenSeen()
  reportEvent(EVENT_KEY_SOCIAL_PROOF_SCREEN)
end

function OnboardioAnalytics.reportSocialProofContinueTapped()
  reportEvent(EVENT_KEY_SOCIAL_PROOF_CONTINUE_TAPPED)
end

function OnboardioAnalytics.reportSocialProofSkipped()
  reportEvent(EVENT_KEY_SOCIAL_PROOF_SKIPPED)
end

function OnboardioAnalytics.reportMotivationScreenSeen()
  reportEvent(EVENT_KEY_MOTIVATION_SCREEN)
end

function OnboardioAnalytics.reportMotivationScreenSkipped()
  reportEvent(EVENT_KEY_MOTIVATION_SKIPPED)
end

function OnboardioAnalytics.reportMotivationScreenAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_MOTIVATION_ANSWERED, properties)
end

function OnboardioAnalytics.reportMotivationReinforcementScreenSeen()
  reportEvent(EVENT_KEY_MOTIVATION_REINFORCEMENT_SCREEN)
end

function OnboardioAnalytics.reportADHDScreenSeen()
  reportEvent(EVENT_KEY_ADHD_QUESTION_SCREEN)
end

function OnboardioAnalytics.reportADHDScreenAnswered(answer)
  local properties = {
    ["answer"] = answer
  }
  reportEvent(EVENT_KEY_ADHD_QUESTION_ANSWERED, properties)
end

function OnboardioAnalytics.reportADHDScreenSkipped()
  reportEvent(EVENT_KEY_ADHD_QUESTION_SKIPPED)
end

function OnboardioAnalytics.reportPersonalizationArticulateThoughtsAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_PERSONALIZATION_ARTICULATE_THOUGHTS_ANSWERED, properties)
end

function OnboardioAnalytics.reportPersonalizationEasilyDistractedAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_PERSONALIZATION_EASILY_DISTRACTED_ANSWERED, properties)
end

function OnboardioAnalytics.reportPersonalizationThoughtsOnPaperAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_PERSONALIZATION_THOUGHTS_ON_PAPER_ANSWERED, properties)
end

function OnboardioAnalytics.reportPersonalizationForgetNamesAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_PERSONALIZATION_FORGET_NAMES_ANSWERED, properties)
end

function OnboardioAnalytics.reportPersonalizationUseCalculatorAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_PERSONALIZATION_USE_CALCULATOR_ANSWERED, properties)
end

function OnboardioAnalytics.reportTimeCommitmentScreenAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_TIME_COMMITMENT_ANSWERED, properties)
end

function OnboardioAnalytics.reportTimeCommitmentScreenSkipped()
  reportEvent(EVENT_KEY_TIME_COMMITMENT_SKIPPED)
end

function OnboardioAnalytics.reportLearningPaceScreenAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_LEARNING_PACE_ANSWERED, properties)
end

function OnboardioAnalytics.reportLearningPaceScreenSkipped()
  reportEvent(EVENT_KEY_LEARNING_PACE_SKIPPED)
end

function OnboardioAnalytics.reportESLScreenAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_ESL_QUESTION_ANSWERED, properties)
end

function OnboardioAnalytics.reportESLScreenSkipped()
  reportEvent(EVENT_KEY_ESL_QUESTION_SKIPPED)
end

function OnboardioAnalytics.reportOtherBrainActivitiesScreenAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_OTHER_BRAIN_ACTIVITIES_QUESTION_ANSWERED, properties)
end

function OnboardioAnalytics.reportOtherBrainActivitiesScreenSkipped()
  reportEvent(EVENT_KEY_OTHER_BRAIN_ACTIVITIES_QUESTION_SKIPPED)
end

function OnboardioAnalytics.reportFirstGoalScreenAnswered(response)
  local properties = {
    ["response"] = response
  }
  reportEvent(EVENT_KEY_FIRST_GOAL_QUESTION_ANSWERED, properties)
end

function OnboardioAnalytics.reportFirstGoalScreenSkipped()
  reportEvent(EVENT_KEY_FIRST_GOAL_QUESTION_SKIPPED)
end
