--- OnboardioCheckboxSelectionPanel
-- @author Created by andrewcousins on September 03, 2024.

class("OnboardioCheckboxSelectionPanel")

require("OnboardioSelectionCheckboxRow")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_PROMPT = OnboardioCommon.COLOR_TABLE.ONBOARDIO_TEXT
local COLOR_SUBTITLE = OnboardioCommon.COLOR_TABLE.GRAY500
local COLOR_LINE = OnboardioCommon.COLOR_TABLE.GRAY100


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getAreIdentifiersSelectedList(self)
  local identifiersSelectedTable = {}
  for _, checkBoxRow in ipairs(self.checkBoxRows) do
    identifiersSelectedTable[checkBoxRow.identifier] = checkBoxRow:getIsSelected()
  end
  return identifiersSelectedTable
end

local function getOnContinueButtonPressed(self, onComplete)
  return function()
    local identifiersSelectedTable = getAreIdentifiersSelectedList(self)

    onComplete(identifiersSelectedTable)
    self:setDismissed(true)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioCheckboxSelectionPanel:playAppear()
  local animation = AHSpawn.new()
  for _, checkBoxRow in ipairs(self.checkBoxRows) do
    animation:addAction(checkBoxRow:getAppearAnimation())
  end
  AHDelayed.new(self:getAppearanceDelayDuration(), animation):start()
  self.super.playAppear(self)
end

function OnboardioCheckboxSelectionPanel:playDisappear()
  local animation = AHSpawn.new()
  for _, checkBoxRow in ipairs(self.checkBoxRows) do
    animation:addAction(checkBoxRow:getDisappearAnimation())
  end
  animation:start()
  self.super.playDisappear(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLine(lineWidth)
  local lineHeight = 1
  local lineColor = COLOR_LINE
  local line = FilledRectangle.new(lineWidth, lineHeight, lineColor)
  return line
end

local function createLayout(self, promptText, subtitleText, buttonDescriptions, onComplete)
  local column = Column.new()
  self:insertAtCenter(column)
  local promptFont = Font.new(FontFaces.DIN_OT_MEDIUM, 21)
  local promptLabel = self:createTextArea(promptText, promptFont, COLOR_PROMPT)
  table.insert(self.fadeableElements, promptLabel)
  column:insert(promptLabel)
  column:insertSpacer(8)

  local subtitleFont = Font.new(FontFaces.DIN_OT_REGULAR, 17)
  local subtitleLabel = self:createTextArea(subtitleText, subtitleFont, COLOR_SUBTITLE)
  table.insert(self.fadeableElements, subtitleLabel)
  column:insert(subtitleLabel)
  column:insertSpacer(24)

  local rowWidth = Screen.getWidthPart(0.914)
  local rowHeight = Screen.getHeightPart(0.076)

  self.checkBoxRows = {}
  for i, buttonDescription in ipairs(buttonDescriptions) do
    local checkBoxRow = OnboardioSelectionCheckboxRow.new(rowWidth, rowHeight, buttonDescription)
    table.insert(self.checkBoxRows, checkBoxRow)
    table.insert(self.buttons, checkBoxRow)
    column:insert(checkBoxRow)
    if (i < #buttonDescriptions) then
      local line = createLine(rowWidth)
      column:insert(line)
      line:inheritOpacity(column)
    end
  end

  column:alignChildren()
  column:setOpacity(0)
  table.insert(self.fadeableElements, column)

  self:addContinueButton(getOnContinueButtonPressed(self, onComplete))
end

-- Constructor ------------------------

function OnboardioCheckboxSelectionPanel.new(promptText, subtitleText, buttonDescriptions, onComplete)
  local self = OnboardioModernPanel.new()
  Mixin.mix(self, OnboardioCheckboxSelectionPanel)

  createLayout(self, promptText, subtitleText, buttonDescriptions, onComplete)

  return self
end
