--- OnboardioCommon
-- @author Created by karl on October 08, 2014.

class("OnboardioCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local isDarkMode = GameSession.getIsDarkMode()

OnboardioCommon.COLOR_TABLE = Colors.getColorTableForMode(isDarkMode)

OnboardioCommon.COLOR_ROUND_TEXT = OnboardioCommon.COLOR_TABLE.ONBOARDING_TEXT
OnboardioCommon.COLOR_WHITE_TEXT = Colors.WHITE
OnboardioCommon.COLOR_LABEL_INVERTED = Colors.COLOR_TABLES[Colors.LIGHT_THEME].TEAL500
OnboardioCommon.COLOR_FADE_OUT = OnboardioCommon.COLOR_TABLE.SURFACE100
OnboardioCommon.COLOR_BLUE = OnboardioCommon.COLOR_TABLE.PRIMARY400
OnboardioCommon.COLOR_BLUE_DOWNSTATE = OnboardioCommon.COLOR_TABLE.PRIMARY500
OnboardioCommon.COLOR_PROGRESS = Colors.COLOR_TABLES[Colors.LIGHT_THEME].GRAY200
OnboardioCommon.COLOR_SKILL = OnboardioCommon.COLOR_TABLE.GRAY500
OnboardioCommon.COLOR_VALUE = OnboardioCommon.COLOR_TABLE.GRAY500
OnboardioCommon.COLOR_OUTRO_TEXT = OnboardioCommon.COLOR_TABLE.ONBOARDING_TEXT_OUTRO
OnboardioCommon.COLOR_WHITE = Colors.WHITE
OnboardioCommon.COLOR_BLACK = Colors.BLACK
OnboardioCommon.COLOR_GOLD = Colors.GOLD
OnboardioCommon.COLOR_GREEN = Colors.GREEN
OnboardioCommon.COLOR_ORANGE = Colors.ORANGE
OnboardioCommon.COLOR_RED = Colors.RED
OnboardioCommon.COLOR_PURPLE = Colors.PURPLE
OnboardioCommon.COLOR_YELLOW = Colors.YELLOW
OnboardioCommon.COLOR_PROMPT_TEXT = OnboardioCommon.COLOR_TABLE.ONBOARDING_TEXT_DARK
OnboardioCommon.COLOR_PROMPT_ADDITIONAL_TEXT = OnboardioCommon.COLOR_TABLE.ONBOARDING_TEXT_ADDITIONAL
OnboardioCommon.COLOR_BACKGROUND = OnboardioCommon.COLOR_TABLE.SURFACE100

OnboardioCommon.PRETEST_BUTTON_BACKGROUND = Colors.WHITE
OnboardioCommon.PRETEST_GRADIANT_COLOR_TOP = ColorUtils.colorHex24StringToStd('#070638')
OnboardioCommon.PRETEST_GRADIANT_COLOR_BOTTOM = ColorUtils.colorHex24StringToStd('#3F3F65')


OnboardioCommon.FONT_FACE_LIGHT = FontFaces.DIN_OT_LIGHT
OnboardioCommon.FONT_FACE_MEDIUM = FontFaces.DIN_OT_MEDIUM
OnboardioCommon.FONT_FACE_BOLD = FontFaces.DIN_OT_BOLD

OnboardioCommon.TEXT_WIDTH = Screen.getSafeWidthPart(0.8)

OnboardioCommon.FONT_NORMAL = Font.new(FontFaces.DIN_OT_MEDIUM, 20)
OnboardioCommon.FONT_BOLD = Font.new(FontFaces.DIN_OT_BOLD, 20)
OnboardioCommon.FONT_EXAMPLE = Font.new(FontFaces.DIN_OT_MEDIUM, 26)
OnboardioCommon.FONT_BIG = Font.new(FontFaces.DIN_OT_LIGHT, 38)
OnboardioCommon.FONT_BIG_BOLD = Font.new(FontFaces.DIN_OT_MEDIUM, 38)
OnboardioCommon.FONT_SMALL = Font.new(FontFaces.DIN_OT_MEDIUM, 14)

OnboardioCommon.SKILL_WRITING = "writing"
OnboardioCommon.SKILL_SPEAKING = "speaking"
OnboardioCommon.SKILL_READING = "reading"
OnboardioCommon.SKILL_MATH = "math"
OnboardioCommon.SKILL_MEMORY = "listening"

OnboardioCommon.SKILLS = {
  OnboardioCommon.SKILL_WRITING,
  OnboardioCommon.SKILL_MEMORY,
  OnboardioCommon.SKILL_SPEAKING,
  OnboardioCommon.SKILL_READING,
  OnboardioCommon.SKILL_MATH
}

OnboardioCommon.SKILL_COLORS = {
  [OnboardioCommon.SKILL_WRITING] = OnboardioCommon.COLOR_GREEN,
  [OnboardioCommon.SKILL_SPEAKING] = OnboardioCommon.COLOR_ORANGE,
  [OnboardioCommon.SKILL_READING] = OnboardioCommon.COLOR_RED,
  [OnboardioCommon.SKILL_MATH] = OnboardioCommon.COLOR_PURPLE,
  [OnboardioCommon.SKILL_MEMORY] = OnboardioCommon.COLOR_YELLOW
}

OnboardioCommon.ENGLISH_LANGUAGE = "English"
OnboardioCommon.SPANISH_LANGUAGE = "Spanish"
OnboardioCommon.ENGLISH_LOCALE = "en_US"
OnboardioCommon.SPANISH_LOCALE = "es_ES"
OnboardioCommon.SHOW_LANGUAGE_PROMPT = (GameSession.getCurrentLocale() == OnboardioCommon.SPANISH_LOCALE)

OnboardioCommon.STARTING_POSITION_DEFAULT = "StartingPositionDefault"
OnboardioCommon.STARTING_POSITION_DEFAULT_WEB = "StartingPositionDefaultFromWeb"
OnboardioCommon.STARTING_POSITION_SKILL_SELECTION = "StartingPositionOnboardingSkillSelection"
OnboardioCommon.STARTING_POSITION_SKILL_SELECTION_STREAMLINED = "StartingPositionOnboardingSkillSelectionStreamlined"
OnboardioCommon.STARTING_POSITION_ADD_ANOTHER_ACCOUNT = "StartingPositionAddAnotherAccount"
OnboardioCommon.STARTING_POSITION_PRETEST = "StartingPositionPretest"
OnboardioCommon.STARTING_POSITION_INTRO_PANEL = "StartingPositionIntroPanel"

OnboardioCommon.CAROUSEL_ANIMATION_RELATIVE_Y_POSITION = 0.64
OnboardioCommon.CAROUSEL_DESCRIPTION_RELATIVE_WIDTH = 0.85

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

function OnboardioCommon.getIsDarkMode()
  return isDarkMode
end

function OnboardioCommon.shouldStartAtDefaultPosition()
  return ((GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_DEFAULT) or
          (GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_DEFAULT_WEB))
end

function OnboardioCommon.shouldStartAtAddAnotherAccountPosition()
  return (GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_ADD_ANOTHER_ACCOUNT)
end

-- This replaces shouldStartAtAddAnotherAccountPosition() for the Returning User Flow Improvement test (Feb 2024)
function OnboardioCommon.shouldStartAtIntroPanel()
  return (GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_INTRO_PANEL)
end

function OnboardioCommon.shouldStartAtAddSkillSelection()
  return ((GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_SKILL_SELECTION) or
          (GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_SKILL_SELECTION_STREAMLINED))
end

function OnboardioCommon.shouldStartAtPretest()
  return (GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_PRETEST)
end

function OnboardioCommon.isStreamlinedOnboarding()
  return (GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_SKILL_SELECTION_STREAMLINED)
end

function OnboardioCommon.isContinuingFromWeb()
  return (GameSession.getStartingPositionIdentifier() == OnboardioCommon.STARTING_POSITION_DEFAULT_WEB)
end
