--- OnboardioInformationalPanel
-- @author Created by andrewcousins on December 05, 2023.

class("OnboardioInformationalPanel")

require("OnboardioModernPanel")
require("OnboardioButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createContinueButton(self)
  local callback = function()
    self:setDismissed()
  end
  local continueText = Localizer.get("CONTINUE")
  local button = OnboardioButton.newSmall(continueText, callback)
  button:setOpacity(0)
  return button
end

local function createTextArea(text, font, color, lineHeight)
  local textWidth = Screen.getSafeWidthPart(0.95)
  local label = DynamicTextArea.newSimple(text, textWidth, font, Alignment.CENTER, Alignment.CENTER)
  if (lineHeight ~= nil) then
    label:setFixedLineHeight(lineHeight)
  end
  label:setColor(unpack(color))
  label:setOpacity(0)
  return label
end

local function createLayout(self, informationalText, headerText)
  local columnSpacer = Screen.getHeightPart(0.01)
  local column = Column.new(columnSpacer)

  local textArea = createTextArea(informationalText, Font.new(FontFaces.DIN_OT_MEDIUM, 20), OnboardioCommon.COLOR_PROMPT_TEXT, 36)
  table.insert(self.fadeableElements, textArea)
  column:insert(textArea)
  column:insertSpacer(columnSpacer * 5)

  local button = createContinueButton(self)
  table.insert(self.fadeableElements, button)
  table.insert(self.buttons, button)
  column:insert(button)

  column:alignChildren()
  self:insertAtCenter(column)

  if (headerText ~= nil) then
    local headerTextArea = createTextArea(headerText, Font.new(FontFaces.DIN_OT_MEDIUM, 18), OnboardioCommon.COLOR_PROMPT_ADDITIONAL_TEXT)
    table.insert(self.fadeableElements, headerTextArea)
    self:insertAtCenter(headerTextArea)
    headerTextArea:setY(column:getYTop() + (columnSpacer * 3) + headerTextArea:getHeightPart(0.5))
  end
end

-- Constructor ------------------------

-- @param informationalText (String) - The main text to display
-- @param headerText (String, Optional) - The header text, shown above the informational text
function OnboardioInformationalPanel.new(informationalText, headerText)
  local self = OnboardioModernPanel.new()
  Mixin.mix(self, OnboardioInformationalPanel)

  createLayout(self, informationalText, headerText)

  return self
end
