--- OnboardioLoginButtons
-- @author Created by aaroncantor on April 23, 2020.

class("OnboardioLoginButtons")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GET_STARTED_KEY = "GET_STARTED"
local ALREADY_MEMBER_KEY = "ALREADY_MEMBER"

local LOG_IN_KEY = "LOG_IN_TO_ACCOUNT"
local NEW_TO_ELEVATE_KEY = "NEW_TO_ELEVATE_SIGN_UP"

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createTopButton(self, text, onPressed)
  local button = OnboardioButton.newSkinny(text, onPressed)
  self:insert(button)
  return button
end

local function createBottomButton(self, text, onPressed)
  local button = OnboardioButton.newHollow(text, onPressed)
  self:insert(button)
  return button
end

local function getButtonData(onContinue, onLogIn, swapOrder)
  local buttonData = {
    {
      text = Localizer.get(GET_STARTED_KEY),
      onPressed = onContinue
    },
    {
      text = Localizer.get(ALREADY_MEMBER_KEY),
      onPressed = onLogIn
    }
  }

  if swapOrder then
    buttonData = {
      {
        text = Localizer.get(LOG_IN_KEY),
        onPressed = onLogIn
      },
      {
        text = Localizer.get(NEW_TO_ELEVATE_KEY),
        onPressed = onContinue
      }
    }
  end
  return buttonData
end

-- Constructor ------------------------

function OnboardioLoginButtons.new(onContinue, onLogIn, swapOrder)
  swapOrder = swapOrder or false
  local self = Column.new()
  Mixin.mix(self, OnboardioLoginButtons)

  local buttonData = getButtonData(onContinue, onLogIn, swapOrder)

  local topButton = createTopButton(self, buttonData[1].text, buttonData[1].onPressed)
  self:insertSpacer(Screen.getHeightPart(0.025))
  local bottomButton = createBottomButton(self, buttonData[2].text, buttonData[2].onPressed)

  self:setAlignment(Alignment.CENTER)
  self:alignChildren()

  topButton:inheritOpacity(self)
  bottomButton:inheritOpacity(self)

  self.buttons = {topButton, bottomButton}

  return self
end
