--- OnboardioModernPanel
-- This is an updated implementation of OnboardioPanel with utility functions
-- that are useful for how we currently implement onboarding screens and transitions.
-- OnboardioPanel is left as is for backwards compatibility with OnboardioTransitioner.
-- @author Created by andrewcousins on August 28, 2024.

class("OnboardioModernPanel")

require("OnboardioPanel")
require("OnboardioButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioModernPanel:playAppear()
  local animation = AHSpawn.new()
  local duration = 0.4
  local ease = AHEaseSimpleInOut.new()
  for _, element in ipairs(self.fadeableElements) do
    animation:addAction(AHOpacityTo.new(element, duration, ease, 1))
  end
  for _, button in ipairs(self.buttons) do
    animation:addAction(function() button:setTouchEnabled(true) end)
  end

  return AHDelayed.new(self:getAppearanceDelayDuration(), animation):start()
end

function OnboardioModernPanel:playDisappear()
  local animation = AHSpawn.new()
  local duration = 0.3
  local ease = AHEaseSimpleInOut.new()
  for _, element in ipairs(self.fadeableElements) do
    animation:addAction(AHOpacityTo.new(element, duration, ease, 0))
  end
  for _, button in ipairs(self.buttons) do
    button:setTouchEnabled(false)
  end

  return animation:start()
end

function OnboardioModernPanel:getAppearanceDelayDuration()
  return 0.25
end

-- Class Methods for creating elements
function OnboardioModernPanel:createTextArea(text, font, color)
  local textWidth = self:getWidthPart(0.95)
  font = font or Font.new(FontFaces.DIN_OT_MEDIUM, 20)
  color = color or OnboardioCommon.COLOR_PROMPT_TEXT
  local textArea = DynamicTextArea.newSimple(text, textWidth, font, Alignment.CENTER)
  textArea:setColor(unpack(color))
  table.insert(self.fadeableElements, textArea)
  textArea:setOpacity(0)
  return textArea
end

function OnboardioModernPanel:createButton(text, callback)
  local button = OnboardioButton.newSkinny(text, callback)
  table.insert(self.buttons, button)
  table.insert(self.fadeableElements, button)
  button:setOpacity(0)
  return button
end

function OnboardioModernPanel:addContinueButton(callback)
  local usePretestStyle = false
  local buttonGroup, continueButton = OnboardioOutroCommon.newButtonGroup(
    Localizer.get("CONTINUE"),
    function()
      if (callback ~= nil) then
        callback()
      end
      self:setDismissed()
    end,
    usePretestStyle
  )

  local buttonGroupY = math.max(self:getHeightPart(0.01), Screen.safeBottom)
  buttonGroup:setAnchorPointBottom()
  self:insertAtLocation(buttonGroup, self:getWidthPart(0.5), buttonGroupY)
  table.insert(self.fadeableElements, continueButton)
  table.insert(self.buttons, continueButton)
  continueButton:setOpacity(0)

  return buttonGroup
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioModernPanel.new()
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioModernPanel)

  self.fadeableElements = {}

  return self
end
