--- OnboardioNavigationOverlay
-- @author Created by karl on November 09, 2015.

class("OnboardioNavigationOverlay")

require("OnboardioCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_COLOR = OnboardioCommon.COLOR_TABLE.GRAY400

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function defaultPhysicalBackButtonCallback()
  GameSession.notifyGameEndScore()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioNavigationOverlay:setBackButtonVisible(isVisible)
  local opacity = 0
  if isVisible then
    opacity = 1
  end
  self.backButtonAnimationInterrupt:insert(AHOpacityTo.new(self.backButton, 0.2, nil, opacity))
  self.backButton:setTouchEnabled(isVisible)

  if not isVisible then
    OnboardioBackButtonHandler.setBackButtonCallback(defaultPhysicalBackButtonCallback)
  end
end

function OnboardioNavigationOverlay:setSkipButtonVisible(isVisible)
  local opacity = 0
  if isVisible then
    opacity = 1
  end

  self.skipButtonAnimationInterrupt:insert(AHOpacityTo.new(self.skipButton, 0.2, nil, opacity))
  self.skipButton:setTouchEnabled(isVisible)
end

function OnboardioNavigationOverlay:setBackButtonCallback(callback)
  self.backButtonCallback = callback or function() end
  OnboardioBackButtonHandler.setBackButtonCallback(callback or defaultPhysicalBackButtonCallback)
end

function OnboardioNavigationOverlay:setSkipButtonCallback(callback)
  self.skipButtonCallback = callback or function() end
end

function OnboardioNavigationOverlay:getBackButtonYBottom()
  return self.backButton:getYBottom()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioNavigationOverlay.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, OnboardioNavigationOverlay)

  local fontSize = 16
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, fontSize)
  local topSafeAreaSize = Screen.height - Screen.safeTop
  local topMargin = math.max(16, topSafeAreaSize)
  local rightSafeAreaSize = Screen.width - Screen.safeRight
  local rightMargin = math.max(16, rightSafeAreaSize)
  local leftMargin = math.max(8, Screen.safeLeft)

  local skipText = Localizer.get("SKIP")
  local skipButton = Label.new(skipText, font)
  skipButton:setColor(unpack(BUTTON_COLOR))
  skipButton:setAnchorPointTopRight()
  skipButton:setLoc(self:getWidth() - rightMargin, self:getHeight() - topMargin)
  self:insert(skipButton)
  local skipTappableWidth = skipButton:getWidthPart(2)
  local skipTappableHeight = skipButton:getHeightPart(2)
  Button.addVirtualButton(skipButton, skipTappableWidth, skipTappableHeight, function()
    self.skipButtonCallback()
  end)
  skipButton:setTouchEnabled(false)

  local backButton = Group.new(skipButton:getDims())
  local backSprite = Sprite.newWithName("back_arrow.png")
  backButton:insertAtCenter(backSprite)
  backSprite:inheritColor(backButton)
  backButton:setAnchorPointTopLeft()
  backButton:setLoc(leftMargin, self:getHeight() - topMargin)
  self:insert(backButton)
  local backTappableWidth = backButton:getWidthPart(2)
  local backTappableHeight = backButton:getHeightPart(2)
  Button.addVirtualButton(backButton, backTappableWidth, backTappableHeight, function()
    self.backButtonCallback()
  end)

  skipButton:setOpacity(0)
  backButton:setOpacity(0)

  self.backButton = backButton
  self.skipButton = skipButton

  self.backButtonAnimationInterrupt = ActionInterrupt.new()
  self.skipButtonAnimationInterrupt = ActionInterrupt.new()

  self:setBackButtonCallback(nil)
  self:setSkipButtonCallback(nil)

  return self
end
