--- OnboardioPanel
-- @author Created by Aaron Barrett on October 01, 2014.

class("OnboardioPanel")
require("OnboardioCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.height

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanel:start()
  -- Override this if needed
end

function OnboardioPanel:playHorizontalTransition(distance)
  local slideDistance = distance or (-self:getWidth())
  local duration = 0.5
  local ease = AHEaseSineInOut.new()
  local horizontalTransitionAction = AHLocationXBy.new(self, duration, ease, slideDistance)
  return horizontalTransitionAction:start()
end

function OnboardioPanel:setButtonsEnabled(enabled)
  local buttons = self.buttons
  for _, button in ipairs(buttons) do
    button:setTouchEnabled(enabled)
  end
end

function OnboardioPanel:setDismissed()
  self.dismissed = true
end

function OnboardioPanel:isDismissed()
  return self.dismissed
end

function OnboardioPanel:getSelectedAnswer()
  return self.selectedAnswer
end

function OnboardioPanel:setSelectedAnswer(selectedAnswer)
  self.selectedAnswer = selectedAnswer
end

function OnboardioPanel:setPressedButton(pressedButton)
  self.pressedButton = pressedButton
end

function OnboardioPanel:getPressedButton()
  return self.pressedButton
end

function OnboardioPanel:getFadableLabels()
  return self.fadableLabels
end

function OnboardioPanel:getButtons()
  return self.buttons
end

function OnboardioPanel:getStationaryButtons()
  return self.stationaryButtons
end

function OnboardioPanel:getSlidingButtons()
  return self.slidingButtons
end

function OnboardioPanel:usesHorizontalTransitionIn()
  return false
end

function OnboardioPanel:usesHorizontalTransitionOut()
  return false
end

function OnboardioPanel:getAppearanceDelayDuration()
  return 0.0
end

function OnboardioPanel:getTransitionDelayDuration()
  return 0.0
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanel.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, OnboardioPanel)

  self.dismissed = false
  self.selectedAnswer = nil
  self.pressedButton = nil

  self.fadableLabels = {}

  self.buttons = {}
  self.stationaryButtons = {}
  self.slidingButtons = {}

  return self
end
