--- OnboardioPanelADHDQuestion
-- @author Created by andrewcousins on December 05, 2023.

class("OnboardioPanelADHDQuestion")

require("OnboardioQuestionPanel")
require("OnboardioPanelMotivationQuestion")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Used in OnboardioPanelADHDResponse
OnboardioPanelADHDQuestion.ANSWER_NO_ID = "no"

local BUTTON_DESCRIPTORS = {
  {
    text = Localizer.get("ADHD_QUESTION_YES"),
    id = "yes"
  },
  {
    text = Localizer.get("ADHD_QUESTION_I_THINK_I_DO"),
    id = "i_think_i_do"
  },
  {
    text = Localizer.get("ADHD_QUESTION_NO"),
    id = OnboardioPanelADHDQuestion.ANSWER_NO_ID
  },
  {
    text = Localizer.get("ADHD_QUESTION_I_PREFER_NOT_TO_SHARE"),
    id = "prefer_not_to_share"
  },
  {
    text = Localizer.get("ADHD_QUESTION_NOT_SURE"),
    id = "not_sure",
    isInverted = true
  }
}

local SUBTITLE_KEYS_BY_MOTIVATION_BUTTON_ID = {
  [OnboardioPanelMotivationQuestion.BUTTON_ID.WORK_SCHOOL] = "ADHD_QUESTION_SUBTITLE_WORK_SCHOOL",
  [OnboardioPanelMotivationQuestion.BUTTON_ID.COMMUNICATION] = "ADHD_QUESTION_SUBTITLE_COMMUNICATION",
  [OnboardioPanelMotivationQuestion.BUTTON_ID.COGNITIVE_HEALTH] = "ADHD_QUESTION_SUBTITLE_COGNITIVE_HEALTH"
}

local DEFAULT_SUBTITLE_KEY = "ADHD_QUESTION_SUBTITLE_DEFAULT"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getContinueCallback()
  return function(self, answerID)
    OnboardioAnalytics.reportADHDScreenAnswered(answerID)
    OnboardioPersonalizationManager.saveADHD(answerID)
    self:setSelectedAnswer(answerID)
    self:setDismissed()
  end
end

local function getSubtitleText(motivationAnswerID)
  local subtitleKey = DEFAULT_SUBTITLE_KEY
  if (motivationAnswerID ~= nil) then
    if (SUBTITLE_KEYS_BY_MOTIVATION_BUTTON_ID[motivationAnswerID] ~= nil) then
      subtitleKey = SUBTITLE_KEYS_BY_MOTIVATION_BUTTON_ID[motivationAnswerID]
    end
  end
  return Localizer.get(subtitleKey)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelADHDQuestion:getAppearanceDelayDuration()
  return 0.25
end

function OnboardioPanelADHDQuestion:playAppear()
  OnboardioAnalytics.reportADHDScreenSeen()
  self.super.playAppear(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanelADHDQuestion.new(motivationAnswerID)
  local promptText = Localizer.get("ADHD_QUESTION_PROMPT")
  local continueCallback = getContinueCallback()

  local subtitleText = getSubtitleText(motivationAnswerID)

  local self = OnboardioQuestionPanel.new(promptText, BUTTON_DESCRIPTORS, continueCallback, subtitleText)
  Mixin.mix(self, OnboardioPanelADHDQuestion)

  return self
end
