--- OnboardioPanelADHDResponse
-- @author Created by andrewcousins on December 05, 2023.

class("OnboardioPanelADHDResponse")

require("OnboardioPanelADHDQuestion")
require("OnboardioInformationalPanel")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PREVIOUS_ANSWER_NO_ID = OnboardioPanelADHDQuestion.ANSWER_NO_ID

local PROMPT_COLOR = OnboardioCommon.COLOR_PROMPT_TEXT
local TEXT_COLOR = OnboardioCommon.COLOR_TABLE.GRAY500

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createSprite(self, spriteName, scale)
  local sprite = Sprite.newWithName(spriteName)
  sprite:setOpacity(0)
  sprite:setScl(scale)
  table.insert(self.fadeableElements, sprite)
  return sprite
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createExperimentalLayout(self)
  local headerText = Localizer.get("ADHD_RESPONSE_HEADER_EXPERIMENT")
  local informationalText = Localizer.get("ADHD_RESPONSE_TEXT_EXPERIMENT")
  local spriteName = "adhd_response.png"
  local backgroundTextureName = "personalization_background.png"
  local columnSpacer = Screen.getHeightPart(0.01)
  local column = Column.new(columnSpacer)

  local backgroundTexture = createSprite(self, backgroundTextureName, 2.5)
  column:insert(backgroundTexture)
  column:insertSpacer(backgroundTexture:getHeightPart(-1.3))

  local sprite = createSprite(self, spriteName, 2)
  column:insert(sprite)
  column:insertSpacer(columnSpacer * 3)

  local headerTextArea = self:createTextArea(headerText, Font.new(FontFaces.DIN_OT_MEDIUM, 22), PROMPT_COLOR)
  column:insert(headerTextArea)

  local textColor = OnboardioCommon.COLOR_TABLE.GRAY500
  local textArea = self:createTextArea(informationalText, Font.new(FontFaces.DIN_OT_REGULAR, 18), TEXT_COLOR)
  textArea:setLineSpacing(7)
  column:insert(textArea)

  column:alignChildren()
  self:insertAtLocationFractional(column, 0.5, 0.66)

  self:addContinueButton()
end

-- Constructor ------------------------

function OnboardioPanelADHDResponse.new(answer)
  if ((answer == OnboardioGame.ANSWER_SKIP) or (answer == PREVIOUS_ANSWER_NO_ID)) then
    -- Do not show this panel if the user skipped or responded "no" to the previous question
    return nil
  end

  local self = nil
  if ExperimentsConfiguration.ENHANCED_ONBOARDING_EXPERIMENT:isActive() then
    self = OnboardioModernPanel.new()
    Mixin.mix(self, OnboardioPanelADHDResponse)
    createExperimentalLayout(self)
  else
    local headerText = Localizer.get("ADHD_RESPONSE_HEADER")
    local informationalText = Localizer.get("ADHD_RESPONSE_TEXT")
    self = OnboardioInformationalPanel.new(informationalText, headerText)
    Mixin.mix(self, OnboardioPanelADHDResponse)
  end

  return self
end
