--- OnboardioPanelAddAnotherAccount
-- @author Created by andrewc on September 22, 2021.

class("OnboardioPanelAddAnotherAccount")

require("OnboardioPanel")
require("OnboardioAnalytics")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HAVE_ACCOUNT_KEY = "ALREADY_HAVE_AN_ACCOUNT"
local LOG_IN_KEY = "LOG_IN"
local NEW_TO_ELEVATE_KEY = "NEW_TO_ELEVATE"
local GET_STARTED_KEY = "GET_STARTED"

local PROMPT_BUTTON_SPACE_SIZE = Screen.getHeightPart(0.02)

local CENTER_LINE_COLOR = OnboardioCommon.COLOR_TABLE.GRAY200
local TEXT_COLOR = OnboardioCommon.COLOR_PROMPT_TEXT
local CENTER_BUFFER = 32

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createTextArea(text)
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 24)
  local label = TextArea.new(text, font, OnboardioCommon.TEXT_WIDTH, Alignment.CENTER)
  local color = TEXT_COLOR
  label:setColor(unpack(color))
  return label
end

local function createButton(buttonText, buttonOnTapped)
  local button = OnboardioButton.newSmall(buttonText, buttonOnTapped)
  button:setTouchEnabled(false)
  return button
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelAddAnotherAccount:usesHorizontalTransitionOut()
  return true
end

function OnboardioPanelAddAnotherAccount:start()
  local duration = 0.4
  self:setButtonsEnabled(true)
  local ease = AHEaseSimpleInOut.new()
  return AHOpacityTo.new(self, duration, ease, 1):start()
end

function OnboardioPanelAddAnotherAccount:playAppear(isNavigatingBackward)
  if isNavigatingBackward then
    local distance = self:getWidth() * 1.5
    self:offsetX(-distance)
    self:setOpacity(1)
    return self:playHorizontalTransition(distance)
  end
end

function OnboardioPanelAddAnotherAccount:playDisappear()
  local delay = 0.4
  self:setButtonsEnabled(false)
  return AHDelayed.new(delay, AHOpacityTo.new(self, 0, nil, 0)):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLogInPromptAndButton(self)
  local logInPromptAndButton = Column.new(PROMPT_BUTTON_SPACE_SIZE)

  local logInPromptText = Localizer.get(HAVE_ACCOUNT_KEY)
  local logInPrompt = createTextArea(logInPromptText)
  logInPromptAndButton:insert(logInPrompt)
  table.insert(self.fadableLabels, logInPrompt)

  local loginButtonText = Localizer.get(LOG_IN_KEY)
  local loginButtonOnTapped = function()
    OnboardioAnalytics.reportAddAnotherAccountLogInAction()
    Host.requestLogin()
  end
  local loginButton = createButton(loginButtonText, loginButtonOnTapped)
  logInPromptAndButton:insert(loginButton)
  table.insert(self.buttons, loginButton)

  logInPromptAndButton:alignChildren()
  return logInPromptAndButton
end

local function createGetStartedPromptAndButton(self)
  local getStartedPromptAndButton = Column.new(PROMPT_BUTTON_SPACE_SIZE)

  local getStartedPromptText = Localizer.get(NEW_TO_ELEVATE_KEY)
  local getStartedPrompt = createTextArea(getStartedPromptText)
  getStartedPromptAndButton:insert(getStartedPrompt)
  table.insert(self.fadableLabels, getStartedPrompt)

  local getStartedButtonText = Localizer.get(GET_STARTED_KEY)
  local getStartedButtonOnTapped = function()
    OnboardioAnalytics.reportAddAnotherAccountSignUpAction()
    self:setDismissed()
  end
  local getStartedButton = createButton(getStartedButtonText, getStartedButtonOnTapped)
  getStartedPromptAndButton:insert(getStartedButton)
  table.insert(self.buttons, getStartedButton)

  getStartedPromptAndButton:alignChildren()
  return getStartedPromptAndButton
end

-- Constructor ------------------------

function OnboardioPanelAddAnotherAccount.new(overlay)
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioPanelAddAnotherAccount)

  overlay:setBackButtonCallback(function()
    Host.requestReturnToLogBackIn()
  end)

  local backButtonYBottom = overlay:getBackButtonYBottom()
  local bottomOfSafeArea = Screen.safeBottom

  local centerLine = FilledRectangle.new(Screen.width, 1, CENTER_LINE_COLOR)
  self:insertAtCenter(centerLine)
  local centerLineY = (backButtonYBottom + bottomOfSafeArea) / 2
  centerLine:setY(centerLineY)

  local logInGroup = createLogInPromptAndButton(self)
  self:insertAtCenter(logInGroup)
  local logInGroupY = centerLineY + CENTER_BUFFER + logInGroup:getHeightPart(0.5)
  logInGroup:setY(logInGroupY)

  local getStartedGroup = createGetStartedPromptAndButton(self)
  self:insertAtCenter(getStartedGroup)
  local getStartedGroupY = centerLineY - CENTER_BUFFER - getStartedGroup:getHeightPart(0.5)
  getStartedGroup:setY(getStartedGroupY)

  self:linkOpacityAttrToChildren(true)
  self:setOpacity(0)

  return self
end
