--- OnboardioPanelHowDidYouHearAboutUs
-- @author Created by andrewc on August 19, 2021.

class("OnboardioPanelHowDidYouHearAboutUs")

require("OnboardioQuestionPanel")
require("OnboardioAnalytics")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_OPTION_DESCRIPTORS = {
  {
    text = Localizer.get("HOW_DID_YOU_HEAR_ABOUT_US_ANSWER_SOCIAL_MEDIA_ADS"),
    id = "social_media_or_online_ads"
  },
  {
    text = Localizer.get("HOW_DID_YOU_HEAR_ABOUT_US_ANSWER_APP_STORE"),
    id = "app_store_or_play_store"
  },
  {
    text = Localizer.get("HOW_DID_YOU_HEAR_ABOUT_US_ANSWER_FRIEND_FAMILY"),
    id = "friend_or_family_member"
  },
  {
    text = Localizer.get("HOW_DID_YOU_HEAR_ABOUT_US_ANSWER_NEWS"),
    id = "news_article_or_blog"
  }
}

local OTHER_BUTTON_DESCRIPTOR = {
  text = Localizer.get("HOW_DID_YOU_HEAR_ABOUT_US_ANSWER_OTHER"),
  id = "other"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getShuffledListOfButtonDescriptors()
  local buttonDescriptors = TableUtils.createShuffledArray(BUTTON_OPTION_DESCRIPTORS)
  return buttonDescriptors
end

local function addOtherButtonDescriptorToEndOfList(descriptorList)
  table.insert(descriptorList, OTHER_BUTTON_DESCRIPTOR)
  return descriptorList
end

local function getButtonDescriptors()
  local buttonDescriptors = getShuffledListOfButtonDescriptors()
  buttonDescriptors = addOtherButtonDescriptorToEndOfList(buttonDescriptors)
  return buttonDescriptors
end

local function getContinueCallback()
  return function(self, answerID)
    OnboardioAnalytics.reportHowDidYouHearAboutUsAnswered(answerID)
    OnboardioPersonalizationManager.saveHowDidYouHearAboutUs(answerID)
    self:setDismissed()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelHowDidYouHearAboutUs:playAppear()
  OnboardioAnalytics.reportHowDidYouHearAboutUsSeen()
  self.super.playAppear(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanelHowDidYouHearAboutUs.new()
  local promptText = Localizer.get("HOW_DID_YOU_HEAR_ABOUT_US_PROMPT")
  local buttonDescriptors = getButtonDescriptors()
  local continueCallback = getContinueCallback()

  local self = OnboardioQuestionPanel.new(promptText, buttonDescriptors, continueCallback)
  Mixin.mix(self, OnboardioPanelHowDidYouHearAboutUs)

  return self
end
