--- OnboardioPanelMotivationQuestion
-- @author Created by andrewcousins on November 16, 2023.

class("OnboardioPanelMotivationQuestion")

require("OnboardioQuestionPanel")
require("OnboardioAnalytics")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- These are used both by analytics and OnboardioPanelMotivationReinforcement
OnboardioPanelMotivationQuestion.BUTTON_ID = {
  WORK_SCHOOL = "work_or_school_performance",
  COMMUNICATION = "better_communication",
  COGNITIVE_HEALTH = "cognitive_health",
  FUN_GAMES = "fun_games",
  OTHER = "other",
  VOCABULARY = "expand_vocabulary"
}

local BUTTON_DESCRIPTORS = {
  {
    text = Localizer.get("MOTIVATION_QUESTION_WORK_SCHOOL"),
    id = OnboardioPanelMotivationQuestion.BUTTON_ID.WORK_SCHOOL
  },
  {
    text = Localizer.get("MOTIVATION_QUESTION_COMMUNICATION"),
    id = OnboardioPanelMotivationQuestion.BUTTON_ID.COMMUNICATION
  },
  {
    text = Localizer.get("MOTIVATION_QUESTION_COGNITIVE_HEALTH"),
    id = OnboardioPanelMotivationQuestion.BUTTON_ID.COGNITIVE_HEALTH
  },
  {
    text = Localizer.get("MOTIVATION_QUESTION_VOCABULARY"),
    id = OnboardioPanelMotivationQuestion.BUTTON_ID.VOCABULARY
  },
  {
    text = Localizer.get("MOTIVATION_QUESTION_FUN_GAMES"),
    id = OnboardioPanelMotivationQuestion.BUTTON_ID.FUN_GAMES
  }
}

local OTHER_BUTTON_DESCRIPTOR = {
  text = Localizer.get("MOTIVATION_QUESTION_OTHER"),
  id = OnboardioPanelMotivationQuestion.BUTTON_ID.OTHER,
  isInverted = true
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getShuffledListOfButtonDescriptors()
  return TableUtils.createShuffledArray(BUTTON_DESCRIPTORS)
end

local function addOtherButtonDescriptorToEndOfList(descriptorList)
  table.insert(descriptorList, OTHER_BUTTON_DESCRIPTOR)
  return descriptorList
end

local function getButtonDescriptors()
  local buttonDescriptors = getShuffledListOfButtonDescriptors()
  buttonDescriptors = addOtherButtonDescriptorToEndOfList(buttonDescriptors)
  return buttonDescriptors
end

local function getContinueCallback(questionAnsweredCallback)
  return function(self, answerID)
    OnboardioAnalytics.reportMotivationScreenAnswered(answerID)
    OnboardioPersonalizationManager.saveMotivation(answerID)
    self:setSelectedAnswer(answerID)
    if (questionAnsweredCallback ~= nil) then
      questionAnsweredCallback(answerID)
    end
    self:setDismissed()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelMotivationQuestion:getAppearanceDelayDuration()
  return 0.25
end

function OnboardioPanelMotivationQuestion:playAppear()
  OnboardioAnalytics.reportMotivationScreenSeen()
  self.super.playAppear(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanelMotivationQuestion.new(questionAnsweredCallback)
  local promptText = Localizer.get("MOTIVATION_QUESTION_PROMPT")
  local continueCallback = getContinueCallback(questionAnsweredCallback)
  local buttonDescriptors = getButtonDescriptors()

  local self = OnboardioQuestionPanel.new(promptText, buttonDescriptors, continueCallback)
  Mixin.mix(self, OnboardioPanelMotivationQuestion)

  return self
end
