--- OnboardioPanelTitle
-- @author Created by aaroncantor on April 23, 2020.

class("OnboardioPanelTitle")

require("OnboardioTitleContent")
require("OnboardioLoginButtons")
require("OnboardioPanel")
require("OnboardioAnalytics")
require("graphics.primitives.Rectangle")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getOnContinueTapped(self)
  return function()
    OnboardioAnalytics.reportGetStartedTapped(self.isLoginButtonFirst)
    self:setDismissed()
  end
end

local function getOnLogInTapped(self)
  return function()
    OnboardioAnalytics.reportLogInTapped(self.isLoginButtonFirst)
    Host.requestLogin()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelTitle:start()
  OnboardioAnalytics.reportGetStartedOrLoginSeen(self.isLoginButtonFirst)

  local contentAnimation = self.titleContent:getAnimation()
  local panelAnimation = AHSpawn.new(
    contentAnimation,
    AHDelayed.new(1.5, AHOpacityTo.new(self.loginButtons, 1, nil, 1))
  )

  return panelAnimation:start()
end

function OnboardioPanelTitle:usesHorizontalTransitionOut()
  return true
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanelTitle.new()
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioPanelTitle)

  local titleContent = OnboardioTitleContent.new()
  titleContent:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.55))
  self:insert(titleContent)

  self.titleContent = titleContent

  self.isLoginButtonFirst = OnboardioCommon.isContinuingFromWeb()
  local loginButtons = OnboardioLoginButtons.new(getOnContinueTapped(self), getOnLogInTapped(self), self.isLoginButtonFirst)
  loginButtons:setAnchorPointBottom()
  loginButtons:setLoc(self:getWidthPart(0.5), Screen.safeBottom + 34)
  loginButtons:setOpacity(0)
  self:insert(loginButtons)
  self.loginButtons = loginButtons
  self.buttons = loginButtons.buttons

  return self
end
