--- OnboardioProgress
-- @author Created by karl on October 08, 2014.

class("OnboardioProgress")
require("OnboardioCommon")
require("widgets.progress.SpriteRevealProgressIndicator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

OnboardioProgress.ANIMATE_DURATION = 1.5

local SPRITE_NAME = "progress_bar.webp"
local MAX_VALUE = 5000

local SKILL_FONT = Font.new(OnboardioCommon.FONT_FACE_MEDIUM, 15.0)

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioProgress:animate()
  return runInCoroutine(function()
      wait(self:newProgressTo(OnboardioProgress.ANIMATE_DURATION, AHEaseExponentialOut.new(), self.endProgress):start())
      self:setProgress(self.endProgress)
    end)
end

function OnboardioProgress:setProgress(progress)
  self.meter:setProgress(progress)
  self.valueProgress:setText(string.format("%04d", progress * MAX_VALUE))
end

function OnboardioProgress:getFinalProgress()
  return self.endProgress
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newProgressIndicator(width, color)
  local height = width * 0.0288
  local fgSprite = Sprite.newWithFilename(SPRITE_NAME)
  fgSprite:setEffectiveDims(width, height)
  fgSprite:setColor(unpack(color))
  local bgSprite = Sprite.newWithFilename(SPRITE_NAME)
  bgSprite:setEffectiveDims(width, height)
  bgSprite:setColor(unpack(OnboardioCommon.COLOR_PROGRESS))

  return SpriteRevealProgressIndicator.new(fgSprite, bgSprite, SpriteRevealProgressIndicator.GROW_RIGHT)
end

-- Constructor ------------------------

function OnboardioProgress.new(width, color, skill, level, value)
  local columnSpacing = -width * 0.013 -- Must be negative to compensate for extra label height.
  local self = Column.new(columnSpacing, Alignment.LEFT)
  ProgressIndicator.makeProgressIndicator(self)
  Mixin.mix(self, OnboardioProgress)

  local meter = newProgressIndicator(width, color)

  local skillLabel = Label.new(skill .. ":", SKILL_FONT)
  skillLabel:setColor(unpack(OnboardioCommon.COLOR_WHITE))

  local valueProgress = Label.new("0000", PrerenderedFont.new(FontFaces.DIN_OT_MEDIUM, 16))
  valueProgress:setColor(unpack(OnboardioCommon.COLOR_WHITE))

  local skillRow = Row.new(nil, Alignment.LEFT)
  skillRow:insert(skillLabel)
  skillRow:insertSpacer(6)
  skillRow:insert(valueProgress)
  skillRow:alignChildren()

  local level_font = PrerenderedFont.new(OnboardioCommon.FONT_FACE_MEDIUM, 13.0)
  local levelLabel = Label.new(level, level_font, nil, nil, nil, 0.6)
  levelLabel:setColor(unpack(OnboardioCommon.COLOR_WHITE))

  local levelLabelYAdjust = width * 0.01

  local scoreRow = Row.new(nil, Alignment.BOTTOM)
  scoreRow:setWidth(meter:getEffectiveWidth())
  scoreRow:insert(skillRow)
  scoreRow:insert(levelLabel)
  scoreRow:alignChildren()
  levelLabel:setY(levelLabel:getY() + levelLabelYAdjust)

  self:insert(scoreRow)
  self:insertSpacer(20)
  self:insert(meter)

  self:alignChildren()

  meter:setProgress(0)

  self.valueProgress = valueProgress
  self.meter = meter
  self.endProgress = value / MAX_VALUE

  self:linkOpacityAttrToChildren(true)

  return self
end
