--- OnboardioSelectionCheckbox
-- @author Created by andrewcousins on September 03, 2024.

class("OnboardioSelectionCheckbox")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_BOX_EMPTY = OnboardioCommon.COLOR_TABLE.GRAY300
local COLOR_BOX_SELECTED = OnboardioCommon.COLOR_BLUE
local COLOR_BOX_CHECKMARK = OnboardioCommon.COLOR_WHITE_TEXT

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioSelectionCheckbox:setSelected(isSelected)
  local opacity = 0
  if isSelected then
    opacity = 1
  end
  self.selectedOverlay:setOpacity(opacity)
end

function OnboardioSelectionCheckbox:getDisappearAnimation(duration)
  return AHSpawn.new(
    AHOpacityTo.new(self, duration, nil, 0),
    AHOpacityTo.new(self.selectedOverlay, duration, nil, 0)
  )
end

function OnboardioSelectionCheckbox:getAppearAnimation(duration)
  return AHOpacityTo.new(self, duration, nil, 1)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioSelectionCheckbox.new(size)
  local color = COLOR_BOX_EMPTY
  local lineWidth = 2
  local cornerRadius = 4
  size = size - lineWidth

  local self = HollowRoundedRectangle.new(size, size, cornerRadius, color, lineWidth)
  Mixin.mix(self, OnboardioSelectionCheckbox)

  local selectedColor = COLOR_BOX_SELECTED
  local selectedOverlay = FilledRoundedRectangle.new(size, size, cornerRadius, selectedColor)
  self:insertAtCenter(selectedOverlay)
  self.selectedOverlay = selectedOverlay

  local checkmarkSize = size * 0.75
  local checkmarkColor = COLOR_BOX_CHECKMARK
  local selectedCheckmark = FilledCheckmark.new(checkmarkSize, nil, nil, checkmarkColor)
  selectedOverlay:insertAtCenter(selectedCheckmark)
  selectedCheckmark:inheritOpacity(selectedOverlay)
  self.selectedCheckmark = selectedCheckmark

  self:setSelected(false)
  self:setOpacity(0)

  return self
end
