--- OnboardioSelectionCheckboxRow
-- @author Created by andrewcousins on September 03, 2024.

class("OnboardioSelectionCheckboxRow")

require("OnboardioSelectionCheckbox")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_LABEL = OnboardioCommon.COLOR_TABLE.GRAY500

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioSelectionCheckboxRow:getDisappearAnimation()
  local duration = 0.3
  return AHSpawn.new(
    AHOpacityTo.new(self.label, duration, nil, 0),
    self.checkbox:getDisappearAnimation(duration)
  )
end

function OnboardioSelectionCheckboxRow:getAppearAnimation()
  local duration = 0.4
  return AHSpawn.new(
    AHOpacityTo.new(self.label, duration, nil, 1),
    self.checkbox:getAppearAnimation(duration)
  )
end

function OnboardioSelectionCheckboxRow:getIsSelected()
  return self.isSelected
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLabel(labelText)
  local fontSize = 17
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, fontSize)
  local fontColor = COLOR_LABEL

  local label = Label.new(labelText, font)
  label:setColor(unpack(fontColor))
  label:setOpacity(0)

  return label
end

-- Constructor ------------------------

function OnboardioSelectionCheckboxRow.new(width, height, buttonDataDescription)
  local self = Group.new(width, height)
  Mixin.mix(self, OnboardioSelectionCheckboxRow)

  local labelText = Localizer.get(buttonDataDescription.copyKey)
  local label = createLabel(labelText)
  label:setAnchorPointLeft()
  self:insertAtCenter(label)
  local labelLeftBuffer = 12
  label:setX(labelLeftBuffer)

  local checkBoxSize = height * 0.39
  local checkbox = OnboardioSelectionCheckbox.new(checkBoxSize)
  self:insertAtCenter(checkbox)
  local checkBoxRightBuffer = 16
  checkbox:setX(width - checkbox:getWidthPart(0.5) - checkBoxRightBuffer)

  self.isSelected = false
  local onTouchUp = function()
    self.isSelected = not self.isSelected
    checkbox:setSelected(self.isSelected)
  end
  Button.makeButton(self, nil, onTouchUp)

  self.icon = icon
  self.label = label
  self.checkbox = checkbox

  self.identifier = buttonDataDescription.identifier

  return self
end
