--- OnboardioTextFactory
-- @author Created by Aaron Barrett on October 16, 2014.

class("OnboardioTextFactory")

require("OnboardioTextInterpreter")
require("OnboardioTextView")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_WIDTH = Screen.getWidthPart(0.90)
local TEXT_HORIZONTAL_ALIGNMENT = Alignment.CENTER

-- Plain Text -------------------------

local PLAIN_TEXT_COLOR = OnboardioCommon.COLOR_BLUE

-- Fancy Text -------------------------

local FANCY_TEXT_COLOR = OnboardioCommon.COLOR_WHITE
local BOLD_TEXT_COLOR = OnboardioCommon.COLOR_TABLE.GRAY500

OnboardioTextFactory.ITEM_TYPE_BLANK = "blank"
OnboardioTextFactory.ITEM_TYPE_PROMPT_PICTURE = "prompt_picture"
OnboardioTextFactory.ITEM_TYPE_BOX = "box"
OnboardioTextFactory.ITEM_TYPE_DIVIDE = "divide"
OnboardioTextFactory.ITEM_TYPE_TIMES = "times"

local ATTRIBUTES_FOR_LABELS = {
  gray = {font = Font.new(FontFaces.DIN_OT_MEDIUM, 20), spacing = FontScaling.getScaledSize(8.0)},
  b = {font = Font.new(FontFaces.DIN_OT_BOLD, 20), color = BOLD_TEXT_COLOR, spacing = FontScaling.getScaledSize(8.0)},
  example = {font = Font.new(FontFaces.DIN_OT_REGULAR, 24), spacing = FontScaling.getScaledSize(9.0)},
  examplel = {font = Font.new(FontFaces.DIN_OT_LIGHT, 26), spacing = FontScaling.getScaledSize(9.0)},
  exampleb = {font = Font.new(FontFaces.DIN_OT_BOLD, 26), spacing = FontScaling.getScaledSize(9.0)},
  big = {font = Font.new(FontFaces.DIN_OT_LIGHT, 36), spacing = FontScaling.getScaledSize(9.0)},
  bigb = {font = Font.new(FontFaces.DIN_OT_MEDIUM, 36), spacing = FontScaling.getScaledSize(9.0)},
  math = {font = Font.new(FontFaces.DIN_OT_MEDIUM, 38), spacing = FontScaling.getScaledSize(14.0), yOffset = FontScaling.getScaledSize(20.0)},

  blank = {itemType = OnboardioTextFactory.ITEM_TYPE_BLANK},
  pic = {itemType = OnboardioTextFactory.ITEM_TYPE_PROMPT_PICTURE},
  box = {itemType = OnboardioTextFactory.ITEM_TYPE_BOX, yOffset = 0.0},
  divide = {itemType = OnboardioTextFactory.ITEM_TYPE_DIVIDE, yOffset = FontScaling.getScaledSize(31.0)},
  times = {itemType = OnboardioTextFactory.ITEM_TYPE_TIMES, yOffset = FontScaling.getScaledSize(33.0)}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newOnboardioTextView(text, fontSize, width)
  fontSize = fontSize or 20
  width = width or TEXT_WIDTH
  local baseAttributes = {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, fontSize),
    spacing = FontScaling.getScaledSize(7.0)
  }
  local attributesForLabels = ATTRIBUTES_FOR_LABELS
  local horizontalAlignment = TEXT_HORIZONTAL_ALIGNMENT
  local verticalAlignment = nil

  local textInterpreter = OnboardioTextInterpreter.new(baseAttributes, attributesForLabels)
  local items = textInterpreter:interpretXmlText(text)

  local textModel = TextModel.new(items)

  local textView = OnboardioTextView.new(width, horizontalAlignment, verticalAlignment)
  textView:setModel(textModel)

  return textView
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Plain text views are used for all panels except for round panels.  They do not allow special formatting.
-- They're just multi-line text views.
function OnboardioTextFactory.newPlainTextView(text, color, fontSize, width)
  color = color or PLAIN_TEXT_COLOR
  local textView = newOnboardioTextView(text, fontSize, width)
  textView:setColor(unpack(color))
  return textView
end

-- Fancy text views are used for round panels.  They allow special formatting, such as
-- varying fonts, boxes for math questions, and images for image questions.
function OnboardioTextFactory.newFancyTextView(text)
  local fontSize = nil
  local textView = newOnboardioTextView(text, fontSize, nil)
  textView:setColor(unpack(FANCY_TEXT_COLOR))

  return textView
end
