--- OnboardioTextInterpreter
-- @author Created by Joe Poka on May 31, 2016.

class("OnboardioTextInterpreter")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

OnboardioTextInterpreter.PUNCTUATION_SET = TableUtils.createSetFromString('.,?!:;%)"')

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioTextInterpreter:interpretXmlText(xmlText)
  local items = self.super.interpretXmlText(self, xmlText)

  -- prevent punctuation immediately following blanks from dropping to next line
  for i = 2, #items do
    local textItem = items[i]
    local text = textItem:getText()
    if (text ~= nil) then
      local characterSet = TableUtils.createSetFromString(text)
      local differenceSet = TableUtils.differenceSet(characterSet, OnboardioTextInterpreter.PUNCTUATION_SET)
      local justPunctuation = TableUtils.isEmpty(differenceSet)

      local previousTextItem = items[i - 1]
      local previousItemType = previousTextItem:getItemType()
      local previousItemWasBlank = (previousItemType == "blank");

      if (justPunctuation and previousItemWasBlank) then
        previousTextItem:setAllowBreakLineAfter(false)
        textItem:setAllowBreakLineBefore(false)

        previousTextItem:setSkipNextSpacing(true)
        textItem:setSkipPreviousSpacing(true)
      end
    end
  end

  return items
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioTextInterpreter.new(baseAttributes, attributesForLabels)
  local self = TextInterpreter.new(baseAttributes, attributesForLabels)
  Mixin.mix(self, OnboardioTextInterpreter)

  return self
end
