--- OnboardioTextView
-- @author Created by Aaron Barrett on October 16, 2014.

class("OnboardioTextView")
require("OnboardioCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BLANK_WIDTH = 60.0
local BLANK_HEIGHT = 2.0

local BLANK_COLOR = ColorUtils.colorHex24ToStd(0x164160)

local DIVIDE_COLOR = OnboardioCommon.COLOR_ROUND_TEXT
local TIMES_COLOR = OnboardioCommon.COLOR_ROUND_TEXT

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createBlankView(self)
  local coverHeight = BLANK_WIDTH * 0.53
  local blankGroup = Group.new(BLANK_WIDTH, coverHeight)
  local blankBacking = FilledRectangle.new(BLANK_WIDTH + 2, coverHeight, BLANK_COLOR)
  blankGroup:insertAtCenter(blankBacking)
  blankBacking:setOpacity(0.67)
  local blankBar = FilledRectangle.new(BLANK_WIDTH, BLANK_HEIGHT, OnboardioCommon.COLOR_BLUE)
  blankGroup:insertAtLocationFractional(blankBar, 0.5, 0.1)

  blankGroup:linkOpacityAttrToChildren(true)
  blankGroup:inheritOpacity(self)
  return blankGroup
end

local function createViewForPromptPictureItem(self, item)
  local filename = item:getAttribute("src")
  local picture = Sprite.newWithName(filename)
  picture:inheritOpacity(self)

  return picture
end

local function createViewForBoxItem(self)
  local box = Sprite.newWithName("box.png")
  local width, height = box:getDims()
  local blankGroup = Group.new(width, height)
  local blankBacking = FilledRectangle.new(width, height, BLANK_COLOR)
  blankGroup:insertAtCenter(blankBacking)
  blankBacking:setOpacity(0.67)
  local blankBar = FilledRectangle.new(width, BLANK_HEIGHT * 2, OnboardioCommon.COLOR_BLUE)
  blankBar:setAnchorPointBottom()
  blankGroup:insertAtBottom(blankBar)

  local yOffset = height * 0.15
  blankBacking:offsetY(yOffset)
  blankBar:offsetY(yOffset)

  blankGroup:linkOpacityAttrToChildren(true)
  blankGroup:inheritOpacity(self)
  return blankGroup
end

local function createViewForDivideItem(self)
  local divideSign = Sprite.newWithName("divide.png")
  divideSign:setColor(unpack(DIVIDE_COLOR))

  divideSign:inheritOpacity(self)

  return divideSign
end

local function createViewForTimesItem(self)
  local timesSign = Sprite.newWithName("times.png")
  timesSign:setColor(unpack(TIMES_COLOR))

  timesSign:inheritOpacity(self)

  return timesSign
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioTextView:getBoxView()
  return self.boxView
end

function OnboardioTextView:createItemView(item)
  local itemType = item:getItemType()

  if (itemType == OnboardioTextFactory.ITEM_TYPE_BLANK) then
    local blankView = createBlankView(self)
    return blankView
  elseif (itemType == OnboardioTextFactory.ITEM_TYPE_PROMPT_PICTURE) then
    local promptPictureView = createViewForPromptPictureItem(self, item)
    return promptPictureView
  elseif (itemType == OnboardioTextFactory.ITEM_TYPE_BOX) then
    local boxView = createViewForBoxItem(self)
    self.boxView = boxView
    return boxView
  elseif (itemType == OnboardioTextFactory.ITEM_TYPE_DIVIDE) then
    local divideView = createViewForDivideItem(self)
    return divideView
  elseif (itemType == OnboardioTextFactory.ITEM_TYPE_TIMES) then
    local timesView = createViewForTimesItem(self)
    return timesView
  else
    return self.super.createItemView(self, item)
  end
end

function OnboardioTextView:createViewForTextItem(item)
  local label = self.super.createViewForTextItem(self, item)

  local color = item:getAttribute("color")
  if (type(color) == "table") then
    label:clearAttrLink(MOAIColor.INHERIT_COLOR)
    label:setColor(unpack(color))
    label:inheritOpacity(self)
  end

  return label
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioTextView.new(textWidth, horizontalAlignment, verticalAlignment)
  local self = TextView.new(textWidth, horizontalAlignment, verticalAlignment)
  Mixin.mix(self, OnboardioTextView)

  return self
end
