--- OnboardioTitleContent
-- @author Created by aaroncantor on April 23, 2020.

class("OnboardioTitleContent")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TITLE = OnboardioCommon.COLOR_PROMPT_TEXT
local COLOR_SUBTITLE = OnboardioCommon.COLOR_TABLE.ONBOARDING_TEXT_LIGHT

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioTitleContent:getAnimation()
  local logoAnimation = self.logo:getAnimation()
  local distance = Screen.getHeightPart(0.1)
  local duration = 1.3
  local delay = 0.1
  local textAnimation = AHSequence.new(
    function()
      self.topLabel:setY(self.topLabel:getY() + distance)
      self.bottomLabel:setY(self.bottomLabel:getY() + distance)
    end,
    AHSpawn.new(
      AHLocationYBy.new(self.topLabel, duration, AHEaseExponentialOut.new(), -distance),
      AHLocationYBy.new(self.bottomLabel, duration, AHEaseExponentialOut.new(), -distance),
      AHSequence.new(
        AHDelay.new(delay),
        AHSpawn.new(
          AHOpacityTo.new(self.topLabel, duration - delay, nil, 1),
          AHOpacityTo.new(self.bottomLabel, duration - delay, nil, 1)
        )
      )
    )
  )

  local contentAnimation = AHStaggeredSpawn.new(
      1,
      logoAnimation,
      textAnimation
    )
  return contentAnimation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLogo()
  local colors = {
    OnboardioCommon.COLOR_RED,
    OnboardioCommon.COLOR_PURPLE,
    OnboardioCommon.COLOR_ORANGE,
    OnboardioCommon.COLOR_GOLD
  }

  local logoHexagon = Sprite.newWithName("intro_brain_logo.png")
  local radius = logoHexagon:getWidthPart(0.55)
  local sideCount = 6
  local startAngle = math.pi * 0.5
  local roundingRadius = radius * 0.3

  local logo = Group.new(radius * 2, radius * 2)

  for _, color in ipairs(colors) do
    local hexagon = FilledRoundedRegularPolygon.new(radius, sideCount, startAngle, roundingRadius, color)
    logo:insertAtCenter(hexagon)
    hexagon:setScl(0)
  end

  logo:insertAtCenter(logoHexagon)
  logoHexagon:setScl(0)

  logo.getAnimation = function(self)
    local scaleDuration = 2
    local stagger = 0.15
    local scaleEase = AHEaseExponentialInOut.new()
    local action = AHStaggeredSpawn.new(stagger)
    for _, childHexagon in ipairs(self.children) do
      action:addAction(AHScaleTo.new(childHexagon, scaleDuration, scaleEase, 1))
    end
    return action
  end

  return logo
end

-- Constructor ------------------------

function OnboardioTitleContent.new()
  local self = Column.new(15)
  Mixin.mix(self, OnboardioTitleContent)

  local logo = createLogo()
  self:insert(logo)

  local topLabel = Label.new("E L E V A T E", Font.new(FontFaces.DIN_OT_LIGHT, 33))
  topLabel:setColor(unpack(COLOR_TITLE))
  self:insert(topLabel)

  local bottomLabel = Label.new(Localizer.get("PERSONALIZED"), Font.new(FontFaces.DIN_OT_BOLD, 16))
  bottomLabel:setColor(unpack(COLOR_SUBTITLE))
  self:insert(bottomLabel)

  topLabel:setOpacity(0)
  bottomLabel:setOpacity(0)

  self.topLabel = topLabel
  self.bottomLabel = bottomLabel
  self.logo = logo

  self:alignChildren()

  return self
end
