--- OnboardioTransitioner
-- @author Created by Aaron Barrett on November 04, 2014.

class("OnboardioTransitioner")
require("OnboardioStationaryTransition")
require("OnboardioSlideTransition")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Add/Remove Panels ------------------

local function playRemovalOnPanel(oldPanel)
  if (oldPanel == nil) then
    return nil
  end

  local removalAction = AHDelayed.new(
                                      0.5,
                                      AHRemove.new(oldPanel)
                                      )
  return removalAction:start()
end

local function playEnableButtonsOnPanel(newPanel)
  if (newPanel == nil) then
    return AHDelay.new(0.35):start()
  end

  local baseDelayDuration = 0.35
  local appearanceDelayDuration = newPanel:getAppearanceDelayDuration()
  local delayDuration = baseDelayDuration + appearanceDelayDuration

  local enableButtonsAction = AHDelayed.new(
                                            delayDuration,
                                            function()
                                               newPanel:setButtonsEnabled(true)
                                            end
                                            )
  return enableButtonsAction:start()
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Play Transitions -------------------

function OnboardioTransitioner.playSimpleTransitionBetweenPanels(oldPanel, newPanel, isNavigatingBackward)
  if (newPanel ~= nil) then
    newPanel:setButtonsEnabled(false)

    if (newPanel.playAppear ~= nil) then
      newPanel:playAppear(isNavigatingBackward)
    end

    if (newPanel:usesHorizontalTransitionIn()) then
      newPanel:playHorizontalTransition()
    end
  end

  if (oldPanel ~= nil) then
    oldPanel:setButtonsEnabled(false)

    if (oldPanel.playDisappear ~= nil) then
      oldPanel:playDisappear(isNavigatingBackward)
    end

    if (oldPanel:usesHorizontalTransitionOut()) then
      oldPanel:playHorizontalTransition()
    end
  end

  playRemovalOnPanel(oldPanel)
  return playEnableButtonsOnPanel(newPanel)
end

function OnboardioTransitioner.playTransitionBetweenPanels(oldPanel, newPanel)
  if (oldPanel ~= nil) then
    oldPanel:setButtonsEnabled(false)
  end
  if (newPanel ~= nil) then
    newPanel:setButtonsEnabled(false)
  end

  local stationaryTransition = OnboardioStationaryTransition.new(oldPanel, newPanel)
  local slideTransition = OnboardioSlideTransition.new(oldPanel, newPanel)

  stationaryTransition:buildDescriptors()
  slideTransition:buildDescriptors()

  stationaryTransition:prepareToTransition()
  slideTransition:prepareToTransition()

  stationaryTransition:playTransition()
  slideTransition:playTransition()

  if (newPanel ~= nil) then
    if (newPanel.playAppear ~= nil) then
      newPanel:playAppear()
    end

    if (newPanel:usesHorizontalTransitionIn()) then
      newPanel:playHorizontalTransition()
    end
  end

  if (oldPanel ~= nil) then
    if (oldPanel.playDisappear ~= nil) then
      oldPanel:playDisappear()
    end

    if (oldPanel:usesHorizontalTransitionOut()) then
      oldPanel:playHorizontalTransition()
    end
  end

  playRemovalOnPanel(oldPanel)
  return playEnableButtonsOnPanel(newPanel)
end

function OnboardioTransitioner.playTransitionIn(panelToShow)
  return OnboardioTransitioner.playTransitionBetweenPanels(nil, panelToShow)
end

function OnboardioTransitioner.playTransitionOut(panelToHide)
  return OnboardioTransitioner.playTransitionBetweenPanels(panelToHide, nil)
end
