--- PretestSession
-- @author Created by Jacob Eiting on October 08, 2014.

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

--- Give the epq index values (0, 1) for the pretest values (0, 1) of each skill group in the map
-- @param table { string : number } of the skill group identifier mapped to a (0.0, 1.0) value
-- @return table { string : number } skill group identifier mapped to epq value out of the max (0.0, 1.0)
function PretestSession.getPerformanceIndices(skillGroupToPretestScore)
  local result = {}
  for skillGroupIdentifier, pretestScore in pairs(skillGroupToPretestScore) do
    result[skillGroupIdentifier] = PretestSession.getPerformanceIndex(pretestScore)
  end
  return result
end

--- Give the epq display values (0, 5000) for the epq index values obtained from getPerformanceIndices
-- @param table { string : number } of the skill group identifier mapped to a (0.0, 1.0) value
-- @return table { string : number } skill group identifier mapped to epq display value
function PretestSession.getPerformanceDisplayValues(skillGroupToPerformanceIndices)
  local result = {}
  for skillGroupIdentifier, performanceIndex in pairs(skillGroupToPerformanceIndices) do
    result[skillGroupIdentifier] = PretestSession.getPerformanceIndexDisplayValue(performanceIndex)
  end
  return result
end

--- Helper function for returning the index and the display value
-- @param table { string : number } of the skill group identifier mapped to a (0.0, 1.0) pretest value
-- @return table { string : { 'index' : number, 'display' : number} } skill group identifier mapped to epq display value
function PretestSession.getEPQFromPretestResult(pretestResult)
  local indices = PretestSession.getPerformanceIndices(pretestResult)
  local displayValues = PretestSession.getPerformanceDisplayValues(indices)

  local result = {}
  for skillGroup, index in pairs(indices) do
    result[skillGroup] = {index = indices[skillGroup], display = displayValues[skillGroup]}
  end
  return result
end

--- Stores the pretest results for extraction by the app
-- @param table { string = number } map of pretest results for each skill group identifier
function PretestSession.setPretestResults(results)
  GameSession.pretestResult = results
end

--- Store the selections made by onboardio
-- @param (table) selections - map of interest ids to bool
function PretestSession.updateInterests(interests)
  if (GameSession.interestsMap == nil) then
    GameSession.interestsMap = {}
  end

  for interest, isInterested in pairs(interests) do
    GameSession.interestsMap[interest] = isInterested
  end
end

-- GameSession.topInterest is currently unused.
function PretestSession.setTopInterest(interest)
  GameSession.topInterest = interest
end
