--- OnboardioContent
-- @author Created by Aaron Barrett on October 06, 2014.

class("OnboardioContent")
require("OnboardioDifficulty")
require("OnboardioRound")
require("OnboardioRoundSelfRate")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

OnboardioContent.SKILL_GROUP_IDENTIFIER_READING = "reading"
OnboardioContent.SKILL_GROUP_IDENTIFIER_WRITING = "writing"
OnboardioContent.SKILL_GROUP_IDENTIFIER_SPEAKING = "speaking"
OnboardioContent.SKILL_GROUP_IDENTIFIER_MATH = "math"
OnboardioContent.SKILL_GROUP_IDENTIFIER_MEMORY = "listening"

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioContent.getNormalSections()
  return OnboardioContent.NORMAL_SECTIONS
end

function OnboardioContent.getSelfRateRounds()
  return OnboardioContent.SELF_RATE_ROUNDS
end

function OnboardioContent.getSelfRateAudioRounds()
  return OnboardioContent.SELF_RATE_AUDIO_ROUNDS
end

function OnboardioContent.load()
  local READING = Localizer.get("READING")
  local WRITING = Localizer.get("WRITING")
  local SPEAKING = Localizer.get("SPEAKING")
  local MATH = Localizer.get("MATH")
  local LISTENING = Localizer.get("MEMORY")

  OnboardioContent.BUTTON_TEXT = "text"
  OnboardioContent.BUTTON_SQUARE = "square"

  OnboardioContent.DISPLAY_NAMES_BY_SKILL_GROUP_IDENTIFIER = {
    [OnboardioContent.SKILL_GROUP_IDENTIFIER_READING] = READING,
    [OnboardioContent.SKILL_GROUP_IDENTIFIER_WRITING] = WRITING,
    [OnboardioContent.SKILL_GROUP_IDENTIFIER_SPEAKING] = SPEAKING,
    [OnboardioContent.SKILL_GROUP_IDENTIFIER_MATH] = MATH,
    [OnboardioContent.SKILL_GROUP_IDENTIFIER_MEMORY] = LISTENING
  }

  OnboardioContent.SKILL_GROUP_IDENTIFIERS_IN_DISPLAY_ORDER = {
    OnboardioContent.SKILL_GROUP_IDENTIFIER_WRITING,
    OnboardioContent.SKILL_GROUP_IDENTIFIER_SPEAKING,
    OnboardioContent.SKILL_GROUP_IDENTIFIER_READING,
    OnboardioContent.SKILL_GROUP_IDENTIFIER_MATH,
    OnboardioContent.SKILL_GROUP_IDENTIFIER_MEMORY
  }

  OnboardioContent.SELF_RATE_AUDIO_ROUNDS = {
    OnboardioRoundSelfRate.new("audio_self_rate_0001", "AUDIO_SELF_RATE_1", OnboardioContent.SKILL_GROUP_IDENTIFIER_MEMORY),
    OnboardioRoundSelfRate.new("audio_self_rate_0002", "AUDIO_SELF_RATE_2", OnboardioContent.SKILL_GROUP_IDENTIFIER_MEMORY)
  }

  OnboardioContent.SELF_RATE_ROUNDS = {
    OnboardioRoundSelfRate.new("self_rate_0001", "SELF_RATE_WRITING", OnboardioContent.SKILL_GROUP_IDENTIFIER_WRITING),
    OnboardioRoundSelfRate.new("self_rate_0002", "SELF_RATE_SPEAKING", OnboardioContent.SKILL_GROUP_IDENTIFIER_SPEAKING),
    OnboardioRoundSelfRate.new("self_rate_0004", "SELF_RATE_READING", OnboardioContent.SKILL_GROUP_IDENTIFIER_READING),
    OnboardioRoundSelfRate.new("self_rate_0005", "SELF_RATE_MATH", OnboardioContent.SKILL_GROUP_IDENTIFIER_MATH),
    OnboardioRoundSelfRate.new("self_rate_0003", "SELF_RATE_MEMORY", OnboardioContent.SKILL_GROUP_IDENTIFIER_MEMORY)
  }

  --- NOTE: Section numbers are 1, 2, 4, and 7 because the intermediate sections were
  -- removed as part of a shortened onboardio and to keep analytics consistent.
  -- The skillGroup passed into each OnboardioRound is for an experiment where we use this value to replace the multiple contributions skillGroups.
  OnboardioContent.NORMAL_SECTIONS = {
    {
      identifier = "section_001",
      -- Swap/Syntax (WRITING/SPEAKING)
      rounds = {
        OnboardioRound.new("onboardio_0001", OnboardioDifficulty.DIFFICULTY_BASIC, OnboardioContent.BUTTON_TEXT, "TEST_1_1_PROMPT", nil, "TEST_1_1_CORRECT", {"TEST_1_1_INCORRECT"}),
        OnboardioRound.new("onboardio_0002", OnboardioDifficulty.DIFFICULTY_EASY, OnboardioContent.BUTTON_TEXT, "TEST_1_2_PROMPT", nil, "TEST_1_2_CORRECT", {"TEST_1_2_INCORRECT"}),
        OnboardioRound.new("onboardio_0003", OnboardioDifficulty.DIFFICULTY_MEDIUM, OnboardioContent.BUTTON_TEXT, "TEST_1_3_PROMPT", nil, "TEST_1_3_CORRECT", {"TEST_1_3_INCORRECT"}),
        OnboardioRound.new("onboardio_0004", OnboardioDifficulty.DIFFICULTY_HARD, OnboardioContent.BUTTON_TEXT, "TEST_1_4_PROMPT", nil, "TEST_1_4_CORRECT", {"TEST_1_4_INCORRECT"})
      },
      answerOptionsCount = 2,
      contributions = {
        [OnboardioContent.SKILL_GROUP_IDENTIFIER_WRITING] = 1.0,
        [OnboardioContent.SKILL_GROUP_IDENTIFIER_SPEAKING] = 1.0
      }
    },
    {
      identifier = "section_002",
      -- add visualization (READING)
      rounds = {
        OnboardioRound.new("onboardio_0013", OnboardioDifficulty.DIFFICULTY_BASIC, OnboardioContent.BUTTON_TEXT, "TEST_2_1_IMAGE", "TEST_2_PROMPT", "TEST_2_1_CORRECT", {"TEST_2_1_INCORRECT"}),
        OnboardioRound.new("onboardio_0014", OnboardioDifficulty.DIFFICULTY_EASY, OnboardioContent.BUTTON_TEXT, "TEST_2_2_IMAGE", "TEST_2_PROMPT", "TEST_2_2_CORRECT", {"TEST_2_2_INCORRECT"}),
        OnboardioRound.new("onboardio_0015", OnboardioDifficulty.DIFFICULTY_MEDIUM, OnboardioContent.BUTTON_TEXT, "TEST_2_3_IMAGE", "TEST_2_PROMPT", "TEST_2_3_CORRECT", {"TEST_2_3_INCORRECT"}),
        OnboardioRound.new("onboardio_0016", OnboardioDifficulty.DIFFICULTY_HARD, OnboardioContent.BUTTON_TEXT, "TEST_2_4_IMAGE", "TEST_2_PROMPT", "TEST_2_4_CORRECT", {"TEST_2_4_INCORRECT"})
      },
      answerOptionsCount = 2,
      contributions = {
        [OnboardioContent.SKILL_GROUP_IDENTIFIER_READING] = 1.0
      }
    },
    {
      identifier = "section_004",
      -- Which does not belong (READING/WRITING)
      rounds = {
        OnboardioRound.new("onboardio_0009", OnboardioDifficulty.DIFFICULTY_BASIC, OnboardioContent.BUTTON_TEXT, "TEST_3_PROMPT", nil, "TEST_3_1_CORRECT", {"TEST_3_1_INCORRECT", "TEST_3_1_INCORRECT_B"}),
        OnboardioRound.new("onboardio_0010", OnboardioDifficulty.DIFFICULTY_EASY, OnboardioContent.BUTTON_TEXT, "TEST_3_PROMPT", nil, "TEST_3_2_CORRECT", {"TEST_3_2_INCORRECT", "TEST_3_2_INCORRECT_B"}),
        OnboardioRound.new("onboardio_0011", OnboardioDifficulty.DIFFICULTY_MEDIUM, OnboardioContent.BUTTON_TEXT, "TEST_3_PROMPT", nil, "TEST_3_3_CORRECT", {"TEST_3_3_INCORRECT", "TEST_3_3_INCORRECT_B"}),
        OnboardioRound.new("onboardio_0012", OnboardioDifficulty.DIFFICULTY_HARD, OnboardioContent.BUTTON_TEXT, "TEST_3_PROMPT", nil, "TEST_3_4_CORRECT", {"TEST_3_4_INCORRECT", "TEST_3_4_INCORRECT_B"})
      },
      answerOptionsCount = 3,
      contributions = {
        [OnboardioContent.SKILL_GROUP_IDENTIFIER_READING] = 1.0,
        [OnboardioContent.SKILL_GROUP_IDENTIFIER_WRITING] = 1.0
      }
    },
    {
      identifier = "section_007",
      -- math for launch / addition/substraction/multiplication/division
      rounds = {
        OnboardioRound.new("onboardio_math_0001", OnboardioDifficulty.DIFFICULTY_BASIC, OnboardioContent.BUTTON_SQUARE, "TEST_4_1_PROMPT", nil, "TEST_4_1_CORRECT", {"TEST_4_1_INCORRECT"}),
        OnboardioRound.new("onboardio_math_0002", OnboardioDifficulty.DIFFICULTY_EASY, OnboardioContent.BUTTON_SQUARE, "TEST_4_2_PROMPT", nil, "TEST_4_2_CORRECT", {"TEST_4_2_INCORRECT"}),
        OnboardioRound.new("onboardio_math_0003", OnboardioDifficulty.DIFFICULTY_MEDIUM, OnboardioContent.BUTTON_SQUARE, "TEST_4_3_PROMPT", nil, "TEST_4_3_CORRECT", {"TEST_4_3_INCORRECT"}),
        OnboardioRound.new("onboardio_math_0004", OnboardioDifficulty.DIFFICULTY_MEDIUM, OnboardioContent.BUTTON_SQUARE, "TEST_4_4_PROMPT", nil, "TEST_4_4_CORRECT", {"TEST_4_4_INCORRECT"}),
        OnboardioRound.new("onboardio_math_0005", OnboardioDifficulty.DIFFICULTY_HARD, OnboardioContent.BUTTON_SQUARE, "TEST_4_5_PROMPT", nil, "TEST_4_5_CORRECT", {"TEST_4_5_INCORRECT"})
      },
      answerOptionsCount = 2,
      contributions = {
        [OnboardioContent.SKILL_GROUP_IDENTIFIER_MATH] = 1.0
      }
    }
  }
end
