--- OnboardioRound
-- @author Created by Aaron Barrett on October 01, 2014.

class("OnboardioRound")
require("OnboardioDifficulty")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioRound:getCorrectDifficultyWeight()
  local difficulty = self.difficulty
  local weight = OnboardioDifficulty.CORRECT_WEIGHTS_BY_DIFFICULTY[difficulty]
  Asserts.assertType(weight, "weight", "number")
  return weight
end

function OnboardioRound:getWrongDifficultyWeight()
  local difficulty = self.difficulty
  local weight = OnboardioDifficulty.WRONG_WEIGHTS_BY_DIFFICULTY[difficulty]
  Asserts.assertType(weight, "weight", "number")
  return weight
end

function OnboardioRound:getDivisionCount()
  return 1
end

function OnboardioRound:getIdentifier()
  return self.identifier
end

function OnboardioRound:getPrompt()
  return self.prompt
end

function OnboardioRound:getHeaderPrompt()
  return self.headerPrompt
end

function OnboardioRound:getCorrectAnswerId()
  return self.correctAnswerId
end

function OnboardioRound:getDistractorAnswerIds()
  return self.distractorAnswerIds
end

function OnboardioRound:getCorrectAnswer()
  return self.correctAnswer
end

function OnboardioRound:getDistractorAnswers()
  return self.distractorAnswers
end

function OnboardioRound:getDifficulty()
  return self.difficulty
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------


-- Constructors -----------------------

function OnboardioRound.new(identifier, difficulty, buttonType, promptId, headerPromptId, correctAnswerId, distractorAnswerIds)
  local self = {}
  Mixin.mix(self, OnboardioRound)

  Asserts.assertType(identifier, "identifier", "string")
  Asserts.assertType(difficulty, "difficulty", "string")
  Asserts.assertType(buttonType, "buttonType", "string")
  Asserts.assertType(promptId, "promptId", "string")
  Asserts.assertType(correctAnswerId, "correctAnswerId", "string")
  Asserts.assertType(distractorAnswerIds, "distractorAnswerIds", "table")

  self.identifier = identifier
  self.difficulty = difficulty
  self.buttonType = buttonType
  self.promptId = promptId
  self.correctAnswerId = correctAnswerId
  self.distractorAnswerIds = distractorAnswerIds

  self.prompt = Localizer.get(promptId)
  if headerPromptId then
    self.headerPrompt = Localizer.get(headerPromptId)
  end
  self.correctAnswer = Localizer.get(correctAnswerId)
  self.distractorAnswers = {}
  for _, answerId in ipairs(distractorAnswerIds) do
    table.insert(self.distractorAnswers, Localizer.get(answerId))
  end

  return self
end
