--- OnboardioActivityDisplay
-- @author Created by karl on October 09, 2014.

class("OnboardioActivityDisplay")
require("graphics.primitives.Checkmark")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ANGLE_START = 0
local ANGLE_END = 720
local ANIM_DURATION = 4
local DEGREES_PER_SECOND = (ANGLE_END - ANGLE_START) / ANIM_DURATION

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioActivityDisplay:animate()
  local fadeDuration = 0.5
  local finalRotateDuration = 0.5
  local animationDuration = ANIM_DURATION
  local initialRotateDuration = animationDuration - fadeDuration
  local initialRotationAmount = initialRotateDuration * DEGREES_PER_SECOND
  local finalRotationAmount = fadeDuration * DEGREES_PER_SECOND

  return AHSpawn.new(
    function() Sound.playSoundEffect("thank_you") end,
    AHDelayed.new(0.15, AHOpacityTo.new(self.arc, 0.4, nil, 1.0)),
    AHSequence.new(
      AHRotationBy.new(self.rotator, initialRotateDuration, nil, initialRotationAmount),
      AHRotationBy.new(self.rotator, finalRotateDuration, AHEaseSimpleOut.new(4), finalRotationAmount)
    ),
    AHDelayed.new(animationDuration - fadeDuration * 2, AHSequence.new(
      AHOpacityTo.new(self.arc, fadeDuration, AHEaseSimpleIn.new(), 0),
      function() Sound.playSoundEffect("check_mark") end,
      AHOpacityTo.new(self.check, fadeDuration, AHEaseSimpleIn.new(), 1)
    ))
  ):start()
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioActivityDisplay.new(diameter, thickness, color)
  local self = Group.new(diameter, diameter)
  Mixin.mix(self, OnboardioActivityDisplay)

  self.rotator = Group.new(self:getDims())
  self:insertAtCenter(self.rotator)

  self.arc = HollowArc.new(diameter/2, math.pi * 0.55, math.pi * 2.45, color, thickness, 0, false)
  self.arc:setOpacity(0.0)
  self.rotator:insertAtCenter(self.arc)

  self.check = HollowCheckmark.new(diameter, nil, nil, color, thickness, nil)
  self.check:setOpacity(0)
  self.check:setLoc(self:getWidthPart(0.48), self:getHeightPart(0.5))
  self.rotator:insert(self.check)

  self.rotator:setRot(ANGLE_START)

  return self
end
