--- OnboardioPanelEpqExplanation
-- @author Created by Aaron Barrett on February 10, 2015.

class("OnboardioPanelEpqExplanation")
require("OnboardioPanel")
require("OnboardioOutroCommon")
require("OnboardioDemoProgress")
require("OnboardioTransitioner")
require("OnboardioTextFactory")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MESSAGE_KEY = "EPQ_EXPLANATION"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelEpqExplanation:usesHorizontalTransitionIn()
  return true
end

function OnboardioPanelEpqExplanation:usesHorizontalTransitionOut()
  return true
end

function OnboardioPanelEpqExplanation:getAppearanceDelayDuration()
  return 0.6
end

function OnboardioPanelEpqExplanation:playAppear()
  return self.demoProgressBar:animate()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanelEpqExplanation.new(demoProgressBar)
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioPanelEpqExplanation)

  local buttons = self.buttons
  local stationaryButtons = self.stationaryButtons

  local function onContinueButtonPressed()
    Sound.playSoundEffect("button_press")
    self:setDismissed()
  end

  local buttonGroup, continueButton = OnboardioOutroCommon.newButtonGroup(Localizer.get("GET_RESULTS"), onContinueButtonPressed)
  buttonGroup:setAnchorPointBottom()

  local buttonGroupY = math.max(self:getHeightPart(0.01), Screen.safeBottom)

  buttonGroup:setLoc(self:getWidthPart(0.5), buttonGroupY)
  self:insert(buttonGroup)

  continueButton:setOpacity(0.0)
  continueButton:setTouchEnabled(false)

  table.insert(buttons, continueButton)
  table.insert(stationaryButtons, continueButton)

  local topGroup = Group.new(self:getWidth(), self:getHeight() - buttonGroup:getHeight())
  topGroup:setAnchorPointBottom()
  topGroup:setLoc(self:getWidthPart(0.5), buttonGroup:getHeight())
  self:insert(topGroup)

  local topColumn = Column.new()
  local separation = self:getWidthPart(0.1)

  local introductionLabel = OnboardioOutroCommon.newTitle(Localizer.get("INTRODUCING_EPQ"))
  local headerLabel = OnboardioOutroCommon.newHeader(Localizer.get("EPQ"))

  topColumn:insert(introductionLabel)
  topColumn:insertSpacer(2.0)
  topColumn:insert(headerLabel)
  topColumn:insertSpacer(separation)
  topColumn:insert(demoProgressBar)
  topColumn:insertSpacer(separation)

  local messageText = Localizer.get(MESSAGE_KEY)
  local messageLabel = OnboardioTextFactory.newPlainTextView(messageText, OnboardioCommon.COLOR_WHITE)
  topColumn:insert(messageLabel)
  topColumn:alignChildren()

  topGroup:insertAtCenter(topColumn)

  self.demoProgressBar = demoProgressBar

  return self
end
