--- OnboardioPanelEpqIntroduction
-- @author Created by andrewc on September 09, 2021.

class("OnboardioPanelEpqIntroduction")

require("OnboardioDemoProgress")
require("OnboardioPanelEpqExplanation")
require("OnboardioPanelStartingEpq")
require("OnboardioPanelThankYou")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_GRADIANT_TOP = OnboardioCommon.PRETEST_GRADIANT_COLOR_TOP
local BACKGROUND_GRADIANT_BOTTOM = OnboardioCommon.PRETEST_GRADIANT_COLOR_BOTTOM

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function waitForSelectedAnswerFromPanel(self, panel)
  local previousPanel = self.currentPanel
  self.currentPanel = panel
  local answer = nil

  if (panel ~= nil) then

    self:insertAtCenter(panel)
    if (panel:usesHorizontalTransitionIn()) then
       panel:setX(self:getWidthPart(1.5))
    end

    local frameToImprovePerformanceOnOldDevices = 1
    waitForFrames(frameToImprovePerformanceOnOldDevices)

    wait(OnboardioTransitioner.playTransitionBetweenPanels(previousPanel, panel))

    waitUntil(function() return panel:isDismissed() end)

    answer = panel:getSelectedAnswer()
  end

  return answer
end

local function waitForThankYouScreen(self)
  local panel = OnboardioPanelThankYou.new()
  waitForSelectedAnswerFromPanel(self, panel)
end

local function waitForDisplayResults(self, resultsToDisplay)
  OnboardioAnalytics.reportEPQIntroSeen()
  local demoProgressBar = OnboardioDemoProgress.new(self:getWidthPart(0.65))
  local epqExplanationPanel = OnboardioPanelEpqExplanation.new(demoProgressBar)
  waitForSelectedAnswerFromPanel(self, epqExplanationPanel)

  self:takeChild(demoProgressBar)
  local startingEpqPanel = OnboardioPanelStartingEpq.new(resultsToDisplay, self.useAlreadyCreatedAccountCopy)
  local replacementProgressBar = startingEpqPanel:getReplacementProgressBar()
  self:takeChild(replacementProgressBar)
  local startY = demoProgressBar:getY() - demoProgressBar:getHeightPart(0.2)
  replacementProgressBar:setY(startY)
  replacementProgressBar:setScl(0.85, 1)
  demoProgressBar:stop()
  local fadeDuration = 0.2
  AHOpacityTo.new(replacementProgressBar, fadeDuration, nil, 1):start()
  wait(demoProgressBar:fadeOut(fadeDuration))
  waitForSelectedAnswerFromPanel(self, startingEpqPanel)
  startingEpqPanel:takeBackReplacementProgressBar()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelEpqIntroduction:start()
  waitForThankYouScreen(self)
  waitForDisplayResults(self, self.resultsToDisplay)
  self:setDismissed()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanelEpqIntroduction.new(resultsToDisplay, useAlreadyCreatedAccountCopy)
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioPanelEpqIntroduction)

  local width, height = self:getDims()
  local background = RectangleGradient.new(width, height, BACKGROUND_GRADIANT_TOP, BACKGROUND_GRADIANT_BOTTOM)
  self:insertAtCenter(background)

  self.resultsToDisplay = resultsToDisplay
  self.useAlreadyCreatedAccountCopy = useAlreadyCreatedAccountCopy

  return self
end
