--- OnboardioPanelStartingEpq
-- @author Created by Aaron Barrett on February 10, 2015.

class("OnboardioPanelStartingEpq")

require("OnboardioPanel")
require("OnboardioOutroCommon")
require("OnboardioProgress")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_BUTTON_GROUP = 10
local Z_TOP_GROUP = 20
local Z_REPLACEMENT_PROGRESS_BAR = 30

local FINISH_SETUP_KEY = "FINISH_SETUP"
local NEXT_KEY = "NEXT"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelStartingEpq:usesHorizontalTransitionIn()
  return true
end

function OnboardioPanelStartingEpq:usesHorizontalTransitionOut()
  return true
end

function OnboardioPanelStartingEpq:getAppearanceDelayDuration()
  return 1.2
end

function OnboardioPanelStartingEpq:playAppear()
  local progressBarAnims = function()
      for _, progressBar in ipairs(self.progressBars) do
        progressBar:animate()
      end

      local replacementProgressBar = self.replacementProgressBar

      local destinationY = self.progressBarDestinationY

      local fadeOutDuration = 0.2
      AHOpacityTo.new(replacementProgressBar, fadeOutDuration, AHEaseSimpleOut.new(2.0), 1.0):start()

      local shiftDuration = 0.5
      local shiftEase = AHEaseSineInOut.new(2.0)
      AHLocationYTo.new(replacementProgressBar, shiftDuration, shiftEase, destinationY):start()
      AHScaleTo.new(replacementProgressBar, shiftDuration, nil, 1, 1):start()
    end

    local appearSpawn = AHSpawn.new(progressBarAnims)
    appearSpawn:addAction(function() Sound.playSoundEffect("count_up") end)
    return appearSpawn:start()
end

function OnboardioPanelStartingEpq:getReplacementProgressBar()
  return self.replacementProgressBar
end

function OnboardioPanelStartingEpq:takeBackReplacementProgressBar()
  local replacementProgressBar = self.replacementProgressBar
  self:takeChild(replacementProgressBar, Z_REPLACEMENT_PROGRESS_BAR)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createProgressBar(self, index, data)
  local progressWidth = self:getWidthPart(0.75)
  local color = OnboardioOutroCommon.getSkillColor(data.skillIdentifier)
  local skill = string.upper(data.displayName)
  local progress = data.filledProportion
  local level = string.upper(OnboardioOutroCommon.inferSkillLevel(progress))
  local value = data.epqDisplayValue

  return OnboardioProgress.new(progressWidth, color, skill, level, value)
end

-- Constructor ------------------------

function OnboardioPanelStartingEpq.new(epqData, useAlreadyCreatedAccountCopy)
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioPanelStartingEpq)

  local buttons = self.buttons
  local stationaryButtons = self.stationaryButtons

  local separation = 28

  local function onPressedFinish()
    Sound.playSoundEffect("button_press")
    self:setDismissed()
  end

  local buttonCopyKey = FINISH_SETUP_KEY
  if (useAlreadyCreatedAccountCopy) then
    buttonCopyKey = NEXT_KEY
  end

  local buttonGroup, finishButton = OnboardioOutroCommon.newButtonGroup(Localizer.get(buttonCopyKey), onPressedFinish)
  buttonGroup:setAnchorPointBottom()

  local buttonGroupY = math.max(self:getHeightPart(0.01), Screen.safeBottom)

  buttonGroup:setLoc(self:getWidthPart(0.5), buttonGroupY)
  self:insert(buttonGroup, Z_BUTTON_GROUP)

  finishButton:setOpacity(0.0)
  finishButton:setTouchEnabled(false)

  table.insert(buttons, finishButton)
  table.insert(stationaryButtons, finishButton)

  local topGroup = Group.new(self:getWidth(), self:getHeight() - buttonGroup:getHeight())
  topGroup:setAnchorPointBottom()
  topGroup:setLoc(self:getWidthPart(0.5), buttonGroup:getHeight())
  self:insert(topGroup, Z_TOP_GROUP)

  self.progressBars = {}
  local topColumn = Column.new(separation)
  local titleLabel = OnboardioOutroCommon.newTitle(Localizer.get("STARTING_EPQ"))
  topColumn:insert(titleLabel)
  topColumn:insertSpacer(18)
  for index, data in ipairs(epqData) do
    local progressBar = createProgressBar(self, index, data)
    topColumn:insert(progressBar)
    table.insert(self.progressBars, progressBar)
    if index == 1 then
      self.replacementProgressBar = progressBar
      progressBar:setOpacity(0.0)
    end
  end

  OnboardioAnalytics.reportEPQRevealSeen()

  topColumn:alignChildren()
  topGroup:insertAtCenter(topColumn)

  self:takeChild(self.replacementProgressBar, Z_REPLACEMENT_PROGRESS_BAR)

  self.progressBarDestinationY = self.replacementProgressBar:getY()

  return self
end
