--- OnboardioIntroPanel2
-- @author Created by karl on November 21, 2015.

class("OnboardioIntroPanel2")

require("OnboardioIntroDescription")
require("OnboardioSkillIcon")
require("OnboardioSkillHexagon")
require("OnboardioGraphAnimationListening")
require("OnboardioGraphAnimationMath")
require("OnboardioGraphAnimationReading")
require("OnboardioGraphAnimationSpeaking")
require("OnboardioGraphAnimationWriting")
require("utils.ShaderUtils")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TITLE_TEXT_KEY = "TITLE_IMPROVE"
local DESCRIPTION_TEXT_KEY = "DESCRIPTION_TRAIN"

local COLOR_SWIPE_LABEL = OnboardioCommon.COLOR_TABLE.ONBOARDING_TEXT_LIGHT
local BACKGROUND_COLOR = OnboardioCommon.COLOR_BACKGROUND

local SIZE_SMALL = 6
local SIZE_MEDIUM = 9
local SIZE_LARGE = 12
local SIZE_HUGE = 17
local SIZE_GIGANTIC = 30

local LAYER_SLOW = "LAYER_SLOW"
local LAYER_MEDIUM = "LAYER_MEDIUM"
local LAYER_FAST = "LAYER_FAST"

local CATEGORY_SLOW_HEXAGONS = "CATEGORY_SLOW_HEXAGONS"
local CATEGORY_MEDIUM_HEXAGONS = "CATEGORY_MEDIUM_HEXAGONS"
local CATEGORY_FAST_HEXAGONS = "CATEGORY_FAST_HEXAGONS"

local CATEGORY_CIRCLES = "CATEGORY_CIRCLES"

local SKILL_CIRCLES = {
  {
    skill = OnboardioCommon.SKILL_WRITING,
    graphAnimationClass = OnboardioGraphAnimationWriting,
    layer = LAYER_FAST,
    x = 0.25,
    y = 0.84,
    radius = 30
  },
  {
    skill = OnboardioCommon.SKILL_MEMORY,
    graphAnimationClass = OnboardioGraphAnimationListening,
    layer = LAYER_SLOW,
    x = 0.18,
    y = 0.38,
    radius = 37
  },
  {
    skill = OnboardioCommon.SKILL_SPEAKING,
    graphAnimationClass = OnboardioGraphAnimationSpeaking,
    layer = LAYER_SLOW,
    x = 0.84,
    y = 0.2,
    radius = 30
  },
  {
    skill = OnboardioCommon.SKILL_READING,
    graphAnimationClass = OnboardioGraphAnimationReading,
    layer = LAYER_MEDIUM,
    x = 0.89,
    y = 0.5,
    radius = 29
  },
  {
    skill = OnboardioCommon.SKILL_MATH,
    graphAnimationClass = OnboardioGraphAnimationMath,
    layer = LAYER_FAST,
    x = 0.65,
    y = 0.8,
    radius = 45
  }
}

local SLOW_HEXAGONS = {
  {
    x = 0.44,
    y = 0.87,
    radius = SIZE_MEDIUM,
    skill = OnboardioCommon.SKILL_MATH
  },
  {
    x = 0.9,
    y = 0.86,
    radius = SIZE_HUGE,
    skill = OnboardioCommon.SKILL_MATH
  },
  {
    x = 0.29,
    y = 0.65,
    radius = SIZE_LARGE,
    skill = OnboardioCommon.SKILL_WRITING
  },
  {
    x = 0.22,
    y = 0.55,
    radius = SIZE_SMALL,
    skill = OnboardioCommon.SKILL_WRITING
  },
  {
    x = 0.92,
    y = 0.33,
    radius = SIZE_SMALL,
    skill = OnboardioCommon.SKILL_READING
  },
  {
    x = 0.15,
    y = 0.17,
    radius = SIZE_LARGE,
    skill = OnboardioCommon.SKILL_MEMORY
  },
  {
    x = 0.73,
    y = 0.1,
    radius = SIZE_SMALL,
    skill = OnboardioCommon.SKILL_SPEAKING
  }
}

local MEDIUM_HEXAGONS = {
  {
    x = 0.76,
    y = 0.38,
    radius = SIZE_MEDIUM,
    skill = OnboardioCommon.SKILL_READING
  },
  {
    x = 0.18,
    y = 0.38,
    radius = SIZE_GIGANTIC,
    skill = OnboardioCommon.SKILL_MEMORY
  }
}

local FAST_HEXAGONS = {
  {
    x = 0.13,
    y = 0.73,
    radius = SIZE_SMALL,
    skill = OnboardioCommon.SKILL_WRITING
  },
  {
    x = 0.84,
    y = 0.67,
    radius = SIZE_LARGE,
    skill = OnboardioCommon.SKILL_MATH
  },
  {
    x = 0.1,
    y = 0.58,
    radius = SIZE_LARGE,
    skill = OnboardioCommon.SKILL_MEMORY
  },
  {
    x = 0.6,
    y = 0.15,
    radius = SIZE_MEDIUM,
    skill = OnboardioCommon.SKILL_SPEAKING
  },
  {
    x = 0.3,
    y = 0.1,
    radius = SIZE_SMALL,
    skill = OnboardioCommon.SKILL_MEMORY
  }
}

local CONNECTOR_LINKS = {
  {
    linkStart = {category = CATEGORY_CIRCLES, index = 5},
    linkEnd = {category = CATEGORY_SLOW_HEXAGONS, index = 1}
  },
  {
    linkStart = {category = CATEGORY_FAST_HEXAGONS, index = 2},
    linkEnd = {category = CATEGORY_SLOW_HEXAGONS, index = 2}
  },
  {
    linkStart = {category = CATEGORY_CIRCLES, index = 5},
    linkEnd = {category = CATEGORY_FAST_HEXAGONS, index = 2}
  },
  {
    linkStart = {category = CATEGORY_SLOW_HEXAGONS, index = 3},
    linkEnd = {category = CATEGORY_CIRCLES, index = 1}
  },
  {
    linkStart = {category = CATEGORY_SLOW_HEXAGONS, index = 3},
    linkEnd = {category = CATEGORY_SLOW_HEXAGONS, index = 4}
  },
  {
    linkStart = {category = CATEGORY_CIRCLES, index = 1},
    linkEnd = {category = CATEGORY_FAST_HEXAGONS, index = 1}
  },
  {
    linkStart = {category = CATEGORY_CIRCLES, index = 4},
    linkEnd = {category = CATEGORY_SLOW_HEXAGONS, index = 5}
  },
  {
    linkStart = {category = CATEGORY_SLOW_HEXAGONS, index = 5},
    linkEnd = {category = CATEGORY_MEDIUM_HEXAGONS, index = 1}
  },
  {
    linkStart = {category = CATEGORY_MEDIUM_HEXAGONS, index = 2},
    linkEnd = {category = CATEGORY_SLOW_HEXAGONS, index = 6}
  },
  {
    linkStart = {category = CATEGORY_MEDIUM_HEXAGONS, index = 2},
    linkEnd = {category = CATEGORY_FAST_HEXAGONS, index = 3}
  },
  {
    linkStart = {category = CATEGORY_SLOW_HEXAGONS, index = 6},
    linkEnd = {category = CATEGORY_FAST_HEXAGONS, index = 5}
  },
  {
    linkStart = {category = CATEGORY_CIRCLES, index = 3},
    linkEnd = {category = CATEGORY_SLOW_HEXAGONS, index = 7}
  },
  {
    linkStart = {category = CATEGORY_CIRCLES, index = 3},
    linkEnd = {category = CATEGORY_FAST_HEXAGONS, index = 4}
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function addObjectArray(allObjects, addedObjects, category)
  if not allObjects[category] then
    allObjects[category] = {}
  end

  local objects = allObjects[category]
  TableUtils.concatenateArrays(objects, addedObjects)
end

local function addObjectToSkillGroup(self, skill, object)
  if self.objectsBySkill[skill] then
    table.insert(self.objectsBySkill[skill], object)
  end
end

local function createIdleAnimation(self)
  local phoneFaces = self.phone.phones
  local fadeDuration = 0.3
  local holdDuration = 0.5
  local fadeEase = nil
  local lastIndex = #OnboardioCommon.SKILLS
  local lastSkill = OnboardioCommon.SKILLS[lastIndex]
  local animate = AHSequence.new()
  for index, skill in ipairs(OnboardioCommon.SKILLS) do
    local fadeOutAction = AHSpawn.new()
    fadeOutAction:addAction(AHOpacityTo.new(phoneFaces[lastIndex], fadeDuration, nil, 0))
    for _, object in ipairs(self.objectsBySkill[lastSkill]) do
      fadeOutAction:addAction(object:createFadeIconColorAction(fadeDuration, fadeEase, 0))
    end
    animate:addAction(fadeOutAction)

    local fadeInAction = AHSpawn.new()
    for _, object in ipairs(self.objectsBySkill[skill]) do
      fadeInAction:addAction(object:createFadeIconColorAction(fadeDuration, fadeEase, 1))
    end
    fadeInAction:addAction(AHOpacityTo.new(phoneFaces[index], fadeDuration, nil, 1))
    animate:addAction(fadeInAction)
    animate:addAction(AHDelay.new(holdDuration))
    lastIndex = index
    lastSkill = skill
  end
  return AHLoop.new(animate)
end

local function createSkillIconShaders(self)
  local iconShaders = {}
  for index, _ in ipairs(SKILL_CIRCLES) do
    local shader = MOAIShader.new()
    local uniformCount = TableUtils.getSize(OnboardioSkillIcon.SHADER_UNIFORM_INDEX)
    shader:reserveUniforms(uniformCount)
    shader:declareUniformSampler(OnboardioSkillIcon.SHADER_UNIFORM_INDEX.SAMPLER, "sampler", 1)
    shader:declareUniformFloat(OnboardioSkillIcon.SHADER_UNIFORM_INDEX.TINT_COLOR_WEIGHT,
                               "tintColorWeight",
                               OnboardioSkillIcon.MINIMUM_TINT_WEIGHT)
    shader:setVertexAttribute(1, "position")
    shader:setVertexAttribute(2, "uv")

    local vsh = ShaderUtils.loadShader("OnboardioSkillIcon.vsh")
    local fsh = ShaderUtils.loadShader("OnboardioSkillIcon.fsh")
    shader:load(vsh, fsh)

    table.insert(iconShaders, shader)

    MetalPrebinder.prebindShader(shader)
  end

  self.iconShaders = iconShaders
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioIntroPanel2:getOverlayedGroup()
  return self.overlay
end

function OnboardioIntroPanel2:getSlideInAction()
  return self.overlay.slideInAction
end

function OnboardioIntroPanel2:getSlideOutAction()
  return self.overlay.slideOutAction
end

function OnboardioIntroPanel2:startAnimation()
  for _, objectArray in pairs(self.allObjects) do
    for _, object in ipairs(objectArray) do
      object:reset()
    end
  end
  if not self.idleAnimation then
    self.idleAnimation = createIdleAnimation(self)
    self.idleAnimation:start()
  else
    self.idleAnimation:pause(false)
  end
end

function OnboardioIntroPanel2:animateOnce()
  local animation = AHSequence.new(
    self:getSlideInAction(),
    function() self:startAnimation() end,
    AHDelayed.new(2, AHOpacityTo.new(self.swipeToBeginLabel, 1, nil, 1))
  )
  return animation:start()
end

function OnboardioIntroPanel2:pauseAnimation()
  if self.idleAnimation then
    self.idleAnimation:pause(true)
  end
end

function OnboardioIntroPanel2:getIcons()
  return self.icons
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createPhone(canvasSize)
  local phones = {}
  for _, name in ipairs(OnboardioCommon.SKILLS) do
    table.insert(phones, Sprite.newWithName("phone_" .. name .. ".png"))
  end
  local group = Group.new(phones[1]:getDims())
  group:insertAtCenter(Sprite.newWithName("phone.png"))
  for _, phone in ipairs(phones) do
    group:insertAtCenter(phone)
    phone:setOpacity(0)
  end
  phones[1]:setOpacity(1)
  group.phones = phones
  group:setEffectiveDims(canvasSize * 0.37)

  return group
end

local function insertHexagon(group, radius, color, x, y)
  local hexagon = OnboardioSkillHexagon.new(radius, color)
  hexagon:setLoc(x, y)
  group:insert(hexagon)
  return hexagon
end

local function insertHexagons(self, group, descriptors)
  local hexagons = {}
  for _, descriptor in ipairs(descriptors) do
    local x = group:getWidthPart(descriptor.x)
    local y = group:getHeightPart(descriptor.y)
    local color = OnboardioCommon.SKILL_COLORS[descriptor.skill]
    local hexagon = insertHexagon(group, descriptor.radius, color, x, y)
    table.insert(hexagons, hexagon)
    addObjectToSkillGroup(self, descriptor.skill, hexagon)
  end
  return hexagons
end

local function createIcon(self, skill, radius, graphAnimationClass, index)
  local shader = self.iconShaders[index]
  local icon = OnboardioSkillIcon.new(skill, graphAnimationClass, shader)
  icon:setEffectiveDims(radius * 2)
  return icon
end

local function insertIcon(self, group, skill, radius, xProportion, yProportion, graphAnimationClass, index)
  local icon = createIcon(self, skill, radius, graphAnimationClass, index)
  local x = group:getWidthPart(xProportion)
  local y = group:getHeightPart(yProportion)
  icon:setLoc(x, y)
  group:insert(icon)
  return icon
end

local function insertIcons(self, layers)
  local icons = {}
  for index, descriptor in ipairs(SKILL_CIRCLES) do
    local layer = layers[descriptor.layer]
    local icon = insertIcon(self, layer, descriptor.skill, descriptor.radius, descriptor.x, descriptor.y, descriptor.graphAnimationClass, index)
    table.insert(icons, icon)
    addObjectToSkillGroup(self, descriptor.skill, icon)
  end
  return icons
end

local function createOverlayedGroups(self, locX, locY, canvasSize)
  local overlay = Group.new(self:getWidth(), self:getHeight())

  local allObjects = {}
  local objects = nil

  local slowHexLayer = Group.new(canvasSize, canvasSize)
  slowHexLayer:setLoc(locX + Screen.getWidthPart(2.0), locY)
  objects = insertHexagons(self, slowHexLayer, SLOW_HEXAGONS)
  addObjectArray(allObjects, objects, CATEGORY_SLOW_HEXAGONS)
  overlay:insert(slowHexLayer)

  local mediumHexLayer = Group.new(canvasSize, canvasSize)
  mediumHexLayer:setLoc(locX + Screen.getWidthPart(4.0), locY)
  objects = insertHexagons(self, mediumHexLayer, MEDIUM_HEXAGONS)
  addObjectArray(allObjects, objects, CATEGORY_MEDIUM_HEXAGONS)
  overlay:insert(mediumHexLayer)

  local fastHexLayer = Group.new(canvasSize, canvasSize)
  fastHexLayer:setLoc(locX + Screen.getWidthPart(6.0), locY)
  objects = insertHexagons(self, fastHexLayer, FAST_HEXAGONS)
  addObjectArray(allObjects, objects, CATEGORY_FAST_HEXAGONS)
  overlay:insert(fastHexLayer)

  local slowIconLayer = Group.new(canvasSize, canvasSize)
  slowIconLayer:setLoc(locX + Screen.getWidthPart(2.0), locY)
  overlay:insert(slowIconLayer)

  local mediumIconLayer = Group.new(canvasSize, canvasSize)
  mediumIconLayer:setLoc(locX + Screen.getWidthPart(4.0), locY)
  overlay:insert(mediumIconLayer)

  local fastIconLayer = Group.new(canvasSize, canvasSize)
  fastIconLayer:setLoc(locX + Screen.getWidthPart(6.0), locY)
  overlay:insert(fastIconLayer)

  local layers = {
    LAYER_SLOW = slowIconLayer,
    LAYER_MEDIUM = mediumIconLayer,
    LAYER_FAST = fastIconLayer
  }
  objects = insertIcons(self, layers)
  addObjectArray(allObjects, objects, CATEGORY_CIRCLES)

  self.icons = objects

  local slideInDuration = 0.5
  local slideEase = AHEaseSimpleInOut.new(2.5)

  overlay.slideInAction = AHSpawn.new(
    AHLocationXTo.new(slowHexLayer, slideInDuration, slideEase, locX),
    AHLocationXTo.new(mediumHexLayer, slideInDuration, slideEase, locX),
    AHLocationXTo.new(fastHexLayer, slideInDuration, slideEase, locX),
    AHLocationXTo.new(slowIconLayer, slideInDuration, slideEase, locX),
    AHLocationXTo.new(mediumIconLayer, slideInDuration, slideEase, locX),
    AHLocationXTo.new(fastIconLayer, slideInDuration, slideEase, locX)
  )

  overlay.slideOutAction = AHSpawn.new(
    AHLocationXTo.new(slowHexLayer, 1, nil, locX - Screen.getWidthPart(2.0)),
    AHLocationXTo.new(mediumHexLayer, 1, nil, locX - Screen.getWidthPart(2.5)),
    AHLocationXTo.new(fastHexLayer, 1, nil, locX - Screen.getWidthPart(3.0))
  )

  self.overlay = overlay
  self.allObjects = allObjects
  self.slowHexLayer = slowHexLayer
end

local function insertLink(self, group, startObject, endObject)
  local startObject = self.allObjects[startObject.category][startObject.index]
  local endObject = self.allObjects[endObject.category][endObject.index]
  local startRadius = startObject:getEffectiveWidthPart(0.5)
  local endRadius = endObject:getEffectiveWidthPart(0.5)
  local line = Sprite.newWithName("connector_line.png")
  local startX, startY = startObject:getLoc()
  local endX, endY = endObject:getLoc()
  local deltaX, deltaY = endX - startX, endY - startY
  local distance = math.sqrt(deltaX * deltaX + deltaY * deltaY)
  local angle = math.atan2(deltaX, deltaY)
  local circleRadii = startRadius + endRadius
  local remainderDistance = distance - circleRadii
  local lineMidDistance = startRadius + remainderDistance / 2
  local locationX = startX + lineMidDistance * math.sin(angle)
  local locationY = startY + lineMidDistance * math.cos(angle)
  local spacer = group:getWidthPart(0.01)
  local lineLength = remainderDistance
  line:setEffectiveHeight(lineLength - spacer)
  line:setLoc(locationX, locationY)
  line:setRot(-math.deg(angle))
  group:insert(line)
end

local function createConnectors(self)
  local canvas = self.slowHexLayer

  for _, descriptor in ipairs(CONNECTOR_LINKS) do
    insertLink(self, canvas, descriptor.linkStart, descriptor.linkEnd)
  end
end

-- Constructor ------------------------

function OnboardioIntroPanel2.new(width, height, canvasSize)
  local self = FilledRectangle.new(width, height, BACKGROUND_COLOR)
  Mixin.mix(self, OnboardioIntroPanel2)

  local phoneX = self:getWidthPart(0.5)
  local phoneY = self:getHeightPart(OnboardioCommon.CAROUSEL_ANIMATION_RELATIVE_Y_POSITION)

  self.objectsBySkill = {
    [OnboardioCommon.SKILL_WRITING] = {},
    [OnboardioCommon.SKILL_SPEAKING] = {},
    [OnboardioCommon.SKILL_READING] = {},
    [OnboardioCommon.SKILL_MATH] = {},
    [OnboardioCommon.SKILL_MEMORY] = {}
  }

  createSkillIconShaders(self)
  createOverlayedGroups(self, phoneX, phoneY, canvasSize)

  createConnectors(self)

  self.phone = createPhone(canvasSize)
  self.phone:setLoc(phoneX, phoneY)
  self:insert(self.phone)

  local titleText = Localizer.get(TITLE_TEXT_KEY)
  local descriptionText = Localizer.get(DESCRIPTION_TEXT_KEY)

  local descriptionWidth = self:getWidthPart(OnboardioCommon.CAROUSEL_DESCRIPTION_RELATIVE_WIDTH)
  local introDescription = OnboardioIntroDescription.new(titleText, descriptionText, descriptionWidth)
  self:insertAtCenter(introDescription)
  introDescription:setY(introDescription:getYLocationForDescription(self:getHeight(), canvasSize))

  local swipeToBeginLabel = Label.new(Localizer.get("SWIPE_BEGIN"), Font.new(FontFaces.DIN_OT_MEDIUM, 14))
  swipeToBeginLabel:setColor(unpack(COLOR_SWIPE_LABEL))
  swipeToBeginLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.10))
  self:insert(swipeToBeginLabel)

  swipeToBeginLabel:setOpacity(0)
  self.swipeToBeginLabel = swipeToBeginLabel

  return self
end
