--- OnboardioIntroPanel3
-- @author Created by karl on November 23, 2015.

class("OnboardioIntroPanel3")
require("OnboardioIntroDescription")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TITLE_TEXT_KEY = "TITLE_TRACK"
local DESCRIPTION_TEXT_KEY = "DESCRIPTION_CUSTOMIZE"
local BACKGROUND_COLOR = OnboardioCommon.COLOR_BACKGROUND

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioIntroPanel3:getSlideInAction()
  local canvasSize = self.canvasSize

  local iconSize = canvasSize * 0.2
  local xPositions = {
    canvasSize * 0.19,
    canvasSize * 0.51,
    canvasSize * 0.82,
    canvasSize * 0.34,
    canvasSize * 0.66
  }
  local yPositions = {
    canvasSize * 0.85,
    canvasSize * 0.85,
    canvasSize * 0.85,
    canvasSize * 0.4,
    canvasSize * 0.4
  }

  local action = AHSpawn.new()
  for index, icon in ipairs(self.icons) do
    action:addAction(AHLocationTo.new(icon, 1, nil, xPositions[index], yPositions[index]))
    action:addAction(AHScaleTo.new(icon, 1, nil, icon:scaleForEffectiveDims(iconSize)))
    action:addAction(icon:createFadeColorAction(1, nil, 1))
  end

  return action
end

function OnboardioIntroPanel3:startAnimation()
  local progress = {0.7, 0.6, 0.8, 0.75, 0.65}
  for index, icon in ipairs(self.icons) do
    icon:createProgressAction(0.3, nil, progress[index]):start()
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioIntroPanel3.new(icons, width, height, canvasSize)
  local self = FilledRectangle.new(width, height, BACKGROUND_COLOR)
  Mixin.mix(self, OnboardioIntroPanel3)

  self.icons = icons
  self.canvasSize = canvasSize

  local titleText = Localizer.get(TITLE_TEXT_KEY)
  local descriptionText = Localizer.get(DESCRIPTION_TEXT_KEY)

  local descriptionWidth = self:getWidthPart(OnboardioCommon.CAROUSEL_DESCRIPTION_RELATIVE_WIDTH)
  local introDescription = OnboardioIntroDescription.new(titleText, descriptionText, descriptionWidth)
  self:insertAtCenter(introDescription)
  introDescription:setY(introDescription:getYLocationForDescription(self:getHeight(), canvasSize))

  return self
end
