--- OnboardioIntroPanel4
-- @author Created by karl on November 23, 2015.

class("OnboardioIntroPanel4")

require("OnboardioIntroDescription")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TITLE_TEXT_KEY = "TITLE_RESEARCH"
local DESCRIPTION_TEXT_KEY = "DESCRIPTION_DEVELOPED"

local BACKGROUND_COLOR = OnboardioCommon.COLOR_BACKGROUND

local SCIENTIST_PARALLAX_AMOUNT = 5

local BOARD_Z_INDICES = {
  BORDER = 0,
  MAIN_SCREEN = 5,
  BOTTOM_BAR = 10,
  TOP_BAR = 15
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getBezelTriangles(width, height, location)
  local variableY

  if (location == "top") then
    variableY = 0
  elseif (location == "bottom") then
    variableY = height
  else
    error("invalid bezel location")
  end

  local leftPoints = {
    { x = 0, y = variableY },
    { x = width, y = height },
    { x = width, y = 0 }
  }

  local rightPoints = {
    { x = 0, y = 0 },
    { x = 0, y = height },
    { x = width, y = variableY }
  }

  local leftBezel = FilledTriangle.newWithTable(leftPoints)
  local rightBezel = FilledTriangle.newWithTable(rightPoints)

  return leftBezel, rightBezel
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioIntroPanel4:getSlideInAction()
  local canvasSize = self.canvasSize

  local iconSize = canvasSize * 0.12
  local xPositions = {
    canvasSize * 0.22,
    canvasSize * 0.22,
    canvasSize * 0.22,
    canvasSize * 0.59,
    canvasSize * 0.59
  }
  local yPositions = {
    canvasSize * 0.72,
    canvasSize * 0.555,
    canvasSize * 0.39,
    canvasSize * 0.72,
    canvasSize * 0.555
  }

  local action = AHSpawn.new()
  for index, icon in ipairs(self.icons) do
    action:addAction(AHLocationTo.new(icon, 1, nil, xPositions[index], yPositions[index]))
    action:addAction(AHScaleTo.new(icon, 1, nil, icon:scaleForEffectiveDims(iconSize)))
    action:addAction(icon:createFadeColorAction(1, nil, 0))
    action:addAction(icon:createGraphShiftAction())
  end

  action:addAction(AHLocationXBy.new(self.scientist, 1, nil, self:getWidthPart(-SCIENTIST_PARALLAX_AMOUNT)))

  return action
end

function OnboardioIntroPanel4:getSlideOutAction()
  return AHLocationXBy.new(self.scientist, 1, nil, self:getWidthPart(-1))
end

function OnboardioIntroPanel4:getOverlayedGroup()
  return self.overlay
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createBoard()
  local width = Screen.getWidthPart(0.85)
  local height = width * 0.7
  local boardGroup = Group.new(width, height)

  local leftColor = {0.85, 0.85, 0.9}
  local rightColor = {0.89, 0.85, 0.87}
  local isHorizontal = true
  local bottomBar = RectangleGradient.new(width * 0.95, height * 0.02, leftColor, rightColor, isHorizontal)
  bottomBar:setAnchorPointBottom()
  boardGroup:insertAtLocationFractional(bottomBar, 0.5, 0.05, BOARD_Z_INDICES.BOTTOM_BAR)

  local centerX = bottomBar:getX()

  local handle = Sprite.newWithName("board_handle.png")
  handle:setAnchorPointTop()
  handle:setLoc(centerX, bottomBar:getYBottom())
  boardGroup:insert(handle, BOARD_Z_INDICES.BOTTOM_BAR)

  local borderColor = {0.38, 0.39, 0.49}
  local border = FilledRectangle.new(width * 0.93, height * 0.86, borderColor)
  border:setAnchorPointBottom()
  border:setLoc(centerX, bottomBar:getYTop())
  boardGroup:insert(border, BOARD_Z_INDICES.BORDER)

  local mainScreenColor = {0.98, 0.97, 0.98}
  local borderThickness = height * 0.02
  local mainScreenWidth = width * 0.93 - 2 * borderThickness
  local mainScreenHeight = height * 0.86 - borderThickness
  local mainScreen = FilledRectangle.new(mainScreenWidth, mainScreenHeight, mainScreenColor)
  mainScreen:setAnchorPointTop()
  mainScreen:setLoc(centerX, border:getYTop())
  boardGroup:insert(mainScreen, BOARD_Z_INDICES.MAIN_SCREEN)

  local topBarWidth = 0.96
  local topBarInfo = {
    { leftColor = {0.82, 0.83, 0.87}, rightColor = {0.92, 0.92, 0.94}, width = 0.93, height = 0.016, bezelLocation = "top" },
    { leftColor = {0.89, 0.9, 0.94}, rightColor = {0.96, 0.95, 0.97}, width = topBarWidth, height = 0.031 },
    { leftColor = {0.84, 0.83, 0.89}, rightColor = {0.94, 0.93, 0.95}, width = 0.94, height = 0.025, bezelLocation = "bottom" }
  }
  local topBar = Column.new()
  for _, info in ipairs(topBarInfo) do
    local barWidth = info.width * width
    local barHeight = info.height * height
    local barSegment = RectangleGradient.new(barWidth, barHeight, info.leftColor, info.rightColor, isHorizontal)

    if (info.bezelLocation ~= nil) then
      local bezelWidth = (topBarWidth - info.width) / 2 * width
      local leftBezel, rightBezel = getBezelTriangles(bezelWidth, barHeight, info.bezelLocation)
      leftBezel:setColor(unpack(info.leftColor))
      rightBezel:setColor(unpack(info.rightColor))
      leftBezel:setAnchorPointRight()
      rightBezel:setAnchorPointLeft()
      barSegment:insertAtLeft(leftBezel)
      barSegment:insertAtRight(rightBezel)
    end
    topBar:insert(barSegment)
  end
  topBar:alignChildren()
  topBar:setAnchorPointBottom()
  topBar:setLoc(centerX, border:getYTop())
  boardGroup:insert(topBar, BOARD_Z_INDICES.TOP_BAR)

  local shadowTopColor = {0.0, 0.0, 0.0, 0.25}
  local shadowBottomColor = {1.0, 1.0, 1.0, 0.0}
  local topBarShadow = RectangleGradient.new(width * 0.93, height * 0.015, shadowTopColor, shadowBottomColor)
  topBarShadow:setAnchorPointTop()
  topBarShadow:setLoc(centerX, topBar:getYBottom())
  boardGroup:insert(topBarShadow, BOARD_Z_INDICES.TOP_BAR)

  return boardGroup
end

-- Constructor ------------------------

function OnboardioIntroPanel4.new(icons, width, height, canvasSize)
  local self = FilledRectangle.new(width, height, BACKGROUND_COLOR)
  Mixin.mix(self, OnboardioIntroPanel4)

  self.overlay = Group.new(width, height)

  local phoneX = self:getWidthPart(0.5)
  local phoneY = self:getHeightPart(OnboardioCommon.CAROUSEL_ANIMATION_RELATIVE_Y_POSITION)
  local canvas = Group.new(canvasSize, canvasSize)
  canvas:setLoc(phoneX, phoneY)
  self:insert(canvas)

  local board = createBoard()
  board:setLoc(canvas:getWidthPart(0.5), canvas:getHeightPart(0.55))
  canvas:insert(board)

  local scientist = Sprite.newWithName("scientist.png")
  scientist:setAnchorPointBottomRight()
  scientist:setLoc(canvas:getLocationWithAnchorPoint(1, 0))
  scientist:setX(scientist:getX() + self:getWidthPart(SCIENTIST_PARALLAX_AMOUNT))
  self.overlay:insert(scientist)

  self.scientist = scientist
  self.icons = icons
  self.canvasSize = canvasSize

  local titleText = Localizer.get(TITLE_TEXT_KEY)
  local descriptionText = Localizer.get(DESCRIPTION_TEXT_KEY)

  local descriptionWidth = self:getWidthPart(OnboardioCommon.CAROUSEL_DESCRIPTION_RELATIVE_WIDTH)
  local introDescription = OnboardioIntroDescription.new(titleText, descriptionText, descriptionWidth)
  self:insertAtCenter(introDescription)
  introDescription:setY(introDescription:getYLocationForDescription(self:getHeight(), canvasSize))

  return self
end
