--- OnboardioPanelIntro
-- @author Created by karl on November 09, 2015.

class("OnboardioPanelIntro")

require("OnboardioNavigationOverlay")
require("OnboardioCommon")
require("OnboardioCarousel")
require("OnboardioSkillSelector")
require("OnboardioAnalytics")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_CONTENTS = 0
local Z_OVERLAY = 100

local CAROUSEL_HEIGHT_PROPORTION = 1.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function shouldSkipPitchCards()
  return OnboardioCommon.shouldStartAtAddSkillSelection()
end

local function insertContents(self, contents)
  table.insert(self.contents, contents)
  self:insert(contents, Z_CONTENTS)
end

local function hideContents(contents)
  contents:setLoc(Screen.getWidthPart(0.5), Screen.getHeightPart(2))
end

local function showContents(self, contents, yPos)
  yPos = yPos or 0.5
  contents:setLoc(self:getWidthPart(0.5), self:getHeightPart(yPos))
end

local function replaceContents(self, replacementContents, yPos)
  yPos = yPos or 0.5
  for _, contents in ipairs(self.contents) do
    if contents ~= replacementContents then
      hideContents(contents)
    end
  end
  showContents(self, replacementContents, yPos)
end

local function completeIntro(self, shouldDoPretest)
  self.overlay:setSkipButtonVisible(false)
  self.overlay:setBackButtonVisible(false)
  waitFor(0.2)

  self:setDismissed()
  self.selectedAnswer = shouldDoPretest
end

local function skipCarousel(self)
  self.overlay:setSkipButtonVisible(false)
  local fadeIn = AHOpacityTo.new(self.skipOverlay, 0.5, nil, 1.0)
  wait(fadeIn:start())
  wait(self.carousel:scrollToLastCard())
  AHOpacityTo.new(self.skipOverlay, 0.5, nil, 0.0):start()
end

local function returnFromSkillSelection(self, pretestButton)
  pretestButton:setTouchEnabled(true)

  self.overlay:setBackButtonVisible(false)
  self.overlay:setBackButtonCallback(nil)
  self.overlay:setSkipButtonVisible(false)

  local callback = function() skipCarousel(self) end

  self.overlay:setSkipButtonCallback(callback)

  self.skillSelector:playFadeAction()
end

local function clearSkillSelection(self)
  hideContents(self.skillSelector)
  self.skillSelector:setOpacity(1)

  self.skillSelector:reset()
end

local function returnToCarousel(self, pretestButton)
  returnFromSkillSelection(self, pretestButton)

  local lastCard = self.carousel.cards[#self.carousel.cards]
  local fadeOut = AHOpacityTo.new(self.skillSelector, 0.3, nil, 0)
  local fadeIn = AHSpawn.new(
    AHOpacityTo.new(self.carousel, 0.3, nil, 1),
    AHOpacityTo.new(lastCard, 0.3, nil, 1)
  )
  local fadeAction = AHSequence.new(
    fadeOut,
    fadeIn
  )
  showContents(self, self.carousel)
  wait(fadeAction:start())

  clearSkillSelection(self)
end

local function returnToSkillSelectionCard(self, pretestButton)
  returnFromSkillSelection(self, pretestButton)

  local fadeOut = AHOpacityTo.new(self.skillSelector, 0.3, nil, 0)
  local fadeIn = AHOpacityTo.new(self.skillSelectionCard, 0.3, nil, 1)
  local fadeAction = AHSequence.new(
    fadeOut,
    fadeIn
  )
  showContents(self, self.skillSelectionCard)
  wait(fadeAction:start())

  clearSkillSelection(self)
end

local function showSkillSelection(self, pretestButton, returnToPreviousCard)
  pretestButton:setTouchEnabled(false)

  self.skillSelector:reset()
  showContents(self, self.skillSelector)
  coroutine.yield() -- Takes 1 frame to update new positions so takeChild calls work

  self.overlay:setBackButtonVisible(true)
  self.overlay:setSkipButtonVisible(true)
  self.overlay:setSkipButtonCallback(function()
    local currentCardAnalyticsIdentifier = self.skillSelector:getCurrentInterestAnalyticsIdentifier()
    OnboardioAnalytics.reportGoalSelectionSkipTapped(currentCardAnalyticsIdentifier)
    hideContents(self.skillSelector)
    self:setDismissed()
  end)
  self.overlay:setBackButtonCallback(function()
    returnToPreviousCard(self, pretestButton)
  end)

  local lastCard = self.carousel.cards[#self.carousel.cards]
  if (shouldSkipPitchCards()) then
    lastCard = self.skillSelectionCard
  end
  wait(self.skillSelector:playShowAnimation(self.carousel, lastCard, pretestButton))
  hideContents(self.carousel)
  hideContents(self.skillSelectionCard)
end

local function createSkipOverlay(self)
  self.skipOverlay = FilledRectangle.newFullScreen(OnboardioCommon.COLOR_BACKGROUND)
  self.skipOverlay:setOpacity(0)
  self:insertAtCenter(self.skipOverlay)
end

local function getCarouselBackButtonCallback(self, returnToLogBackInScreen)
  return function()
    self.carousel:setTouchEnabled(false)
    if returnToLogBackInScreen then
      Host.requestReturnToAddAccount()
    else
      completeIntro(self, OnboardioGame.ANSWER_BACK)
    end
  end
end


local function showCarousel(self, returnToLogBackInScreen)
  local carouselYPos = 0.5
  replaceContents(self, self.carousel, carouselYPos)

  local showBackButton = true
  self.overlay:setBackButtonVisible(showBackButton)
  self.overlay:setBackButtonCallback(getCarouselBackButtonCallback(self, returnToLogBackInScreen))

  local showSkipButton = true
  if showSkipButton then
    AHDelayed.new(0.4, function()
      local isFirstShow = true
      self.overlay:setSkipButtonVisible(true, isFirstShow)
    end):start()
  end

  local skipCallback = function() skipCarousel(self) end

  self.overlay:setSkipButtonCallback(function()
    local index = self.carousel:getCurrentProgressStep()
    OnboardioAnalytics.reportPitchSkipped(index)
    skipCallback()
  end)
end

local function showSkillSelectionCard(self)
  local skillSelectionCard = self.skillSelectionCard
  replaceContents(self, skillSelectionCard)
  self.overlay:setBackButtonVisible(false)
  self.overlay:setSkipButtonVisible(false)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelIntro:playAppear()
  local startAction = AHSequence.new(
    function() self.carousel:animate() end,
    AHDelay.new(0.4),
    AHOpacityTo.new(self.carousel, 0.2, nil, 1),
    function() self.carousel:setTouchEnabled(true) end
  )

  startAction:start()
end

function OnboardioPanelIntro:playDisappear(isNavigatingBackward)
  if isNavigatingBackward then
    local distance = self:getWidth() * 1.5
    return self:playHorizontalTransition(distance)
  end
end

function OnboardioPanelIntro:usesHorizontalTransitionIn()
  return true
end

function OnboardioPanelIntro:getTransitionDelayDuration()
  return 0.1
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanelIntro.new(returnToLogBackInScreen)
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioPanelIntro)

  self.overlay = OnboardioNavigationOverlay.new()
  self:insertAtCenter(self.overlay, Z_OVERLAY)

  self.contents = {}

  local callbacks = {
    onComplete = function(pretestButton)
      OnboardioAnalytics.reportGoalSelectionStarted()
      local shouldStartAtSkillSelection = shouldSkipPitchCards()

      local returnToPreviousCard = returnToCarousel
      if shouldStartAtSkillSelection then
        returnToPreviousCard = returnToSkillSelectionCard
      end

      showSkillSelection(self, pretestButton, returnToPreviousCard)
    end,
    onEnterLastCard = function()
      self.overlay:setSkipButtonVisible(false)
    end,
    onLeaveLastCard = function()
      self.overlay:setSkipButtonVisible(true)
    end,
    onEnterFirstCard = function()
      local showBackButton = true
      self.overlay:setBackButtonVisible(showBackButton)
      self.overlay:setBackButtonCallback(getCarouselBackButtonCallback(self, returnToLogBackInScreen))
    end,
    onLeaveFirstCard = function()
      self.overlay:setBackButtonVisible(false)
    end
  }

  local carouselWidth = self:getWidth()
  local carouselHeight = self:getHeightPart(CAROUSEL_HEIGHT_PROPORTION)
  self.carousel = OnboardioCarousel.new(callbacks, carouselWidth, carouselHeight)

  insertContents(self, self.carousel)

  self.skillSelector = OnboardioSkillSelector.new(function(answers)
    PretestSession.updateInterests(answers)
    hideContents(self.skillSelector)
    self:setDismissed()
  end)
  insertContents(self, self.skillSelector)

  self.skillSelectionCard = OnboardioIntroPanel5.new(self.carousel:getIcons(), callbacks.onComplete, Screen.fullWidth, Screen.fullHeight)
  self:insertAtCenter(self.skillSelectionCard)
  hideContents(self.skillSelectionCard)

  createSkipOverlay(self)

  if (shouldSkipPitchCards()) then
    showSkillSelectionCard(self)
  else
    showCarousel(self, returnToLogBackInScreen)
  end

  return self
end
