--- OnboardioSkillButton
-- @author Created by karl on November 11, 2015.

class("OnboardioSkillButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TEXT = OnboardioCommon.COLOR_PROMPT_ADDITIONAL_TEXT
local COLOR_BORDER = OnboardioCommon.COLOR_TABLE.GRAY300
local COLOR_HIGHLIGHT = OnboardioCommon.COLOR_TABLE.PRIMARY400

local WIDTH = Screen.getWidthPart(0.37)
local HEIGHT = WIDTH * 0.4
local CORNER_RADIUS = HEIGHT * 0.06
local LINE_WIDTH = 1.5

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioSkillButton:showAsSelected()
  self:setColor(unpack(COLOR_HIGHLIGHT))
  self.label:setOpacity(0)
  self.image:setOpacity(1)

  local duration = 0.375
  local fadeOutImage = AHOpacityTo.new(self.image, duration, nil, 0)
  fadeOutImage:start()

  local colorToBorder = AHColorTo.new(self, duration, nil, COLOR_BORDER)
  colorToBorder:start()
end

function OnboardioSkillButton:fadeInLabel()
  local duration = 0.5
  local fadeInLabel = AHOpacityTo.new(self.label, duration, nil, 1)
  return fadeInLabel:start()
end

function OnboardioSkillButton:fadeIn()
  self.label:setOpacity(0)
  self:setOpacity(0)
  local duration = 0.2
  local fadeIn = AHSpawn.new(
    AHOpacityTo.new(self.label, duration, nil, 1),
    AHOpacityTo.new(self, duration, nil, 1)
  )
  fadeIn:start()
end

function OnboardioSkillButton:fadeOut()
  local duration = 0.2
  local fadeOut = AHSpawn.new(
    AHOpacityTo.new(self.label, duration, nil, 0),
    AHOpacityTo.new(self.image, duration, nil, 0),
    AHOpacityTo.new(self, duration, nil, 0)
  )
  fadeOut:start()
end

function OnboardioSkillButton:setVisible(isVisible)
  if isVisible then
    self.label:setOpacity(1)
    self:setOpacity(1)
  else
    self.label:setOpacity(0)
    self:setOpacity(0)
    self.image:setOpacity(0)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newOnboardioSkillButton(text, onPressed, imageName)
  local self = HollowRoundedRectangle.new(WIDTH, HEIGHT, CORNER_RADIUS, COLOR_BORDER, LINE_WIDTH)
  Mixin.mix(self, OnboardioSkillButton)

  local label = Label.new(text, Font.new(FontFaces.DIN_OT_MEDIUM, 18))
  label:setColor(unpack(COLOR_TEXT))
  self:insertAtCenter(label)

  local image = Sprite.newWithName(imageName)
  image:setColor(unpack(COLOR_HIGHLIGHT))
  self:insertAtCenter(image)
  image:setOpacity(0)

  local scaleDownAmount = 0.825
  image:setScl(scaleDownAmount)

  Button.makeButton(self, onPressed)

  self.label = label
  self.image = image

  return self
end

-- Constructor ------------------------

function OnboardioSkillButton.newButtonWithCheckmark(text, onPressed)
  return newOnboardioSkillButton(text, onPressed, "button_checkmark.png")
end

function OnboardioSkillButton.newButtonWithCross(text, onPressed)
  return newOnboardioSkillButton(text, onPressed, "button_cross.png")
end
