--- OnboardioSkillIcon
-- @author Created by karl on November 23, 2015.

class("OnboardioSkillIcon")

require("OnboardioCommon")
require("widgets.progress.BarProgressIndicator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_MONOCHROME = ColorUtils.colorHex24ToStd(0xA6A5C1)
local COLOR_PROGRESS_BG = ColorUtils.colorHex24ToStd(0xE6E4ED)
local COLOR_PROGRESS_FG = ColorUtils.colorHex24ToStd(0xBCBED1)

OnboardioSkillIcon.SHADER_UNIFORM_INDEX = {
  SAMPLER = 1,
  TINT_COLOR_WEIGHT = 2
}

OnboardioSkillIcon.MINIMUM_TINT_WEIGHT = 0.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioSkillIcon:reset()
  if self.shader then
    self.shader:setAttr(OnboardioSkillIcon.SHADER_UNIFORM_INDEX.TINT_COLOR_WEIGHT,
                        OnboardioSkillIcon.MINIMUM_TINT_WEIGHT)
  end
end

function OnboardioSkillIcon:createFadeColorAction(duration, ease, opacity)
  local toColorGraph = self.baseColor
  local toColorProgress = self.baseColor
  if opacity == 0 then
    toColorGraph = COLOR_MONOCHROME
    toColorProgress = COLOR_PROGRESS_FG
  end

  return AHSpawn.new(
    self:createFadeIconColorAction(duration, ease, opacity),
    AHColorTo.new(self.graph, duration, ease, toColorGraph),
    AHOpacityTo.new(self.progressBar, duration, ease, opacity),
    AHColorTo.new(self.progressBar.progressBar, duration, ease, toColorProgress)
  )
end

function OnboardioSkillIcon:createFadeIconColorAction(duration, ease, destinationTintWeight)
  assert(destinationTintWeight == 1 or destinationTintWeight == 0, "destinationTintWeight should be one or zero.")

  local incrementTint = function(t)
    -- note assumption that destinationTintWeight is either one or zero
    local tintWeight = t
    if (destinationTintWeight < 1.0) then
      tintWeight = 1.0 - t
    end
    if self.shader then
      self.shader:setAttr(OnboardioSkillIcon.SHADER_UNIFORM_INDEX.TINT_COLOR_WEIGHT, tintWeight)
    end
  end

  local animation = AHCustomAction.new(duration, ease, incrementTint)
  return animation
end

function OnboardioSkillIcon:createShowGraphAction()
  return AHSpawn.new(
    AHSequence.new(
      AHOpacityTo.new(self.graph, 0.25, nil, 1),
      AHOpacityTo.new(self.progressBar, 0.25, nil, 1)
    ),
    AHLocationYTo.new(self.graphSection, 0.5, nil, self.graphFinalY)
  )
end

function OnboardioSkillIcon:createHideGraphAction()
  return AHSpawn.new(
    AHOpacityTo.new(self.graph, 0.25, nil, 0),
    AHOpacityTo.new(self.progressBar, 0.25, nil, 0)
  )
end

function OnboardioSkillIcon:createProgressAction(duration, ease, progress)
  return self.progressBar:newProgressTo(duration, ease, progress)
end

function OnboardioSkillIcon:createGraphShiftAction()
  local locationX = self:getWidthPart(1.1) + self.graphSection:getWidthPart(0.8)
  local locationY = self:getHeightPart(0.5)
  return AHLocationTo.new(self.graphSection, 1, nil, locationX, locationY)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createProgressBar(self, fgColor)
  local bgColor = COLOR_PROGRESS_BG
  return BarProgressIndicator.new(self:getWidth(), self:getWidthPart(0.12), fgColor, bgColor, BarProgressIndicator.GrowRight)
end

-- Constructor ------------------------

function OnboardioSkillIcon.new(skill, graphAnimationClass, shader)
  local spriteName = "icon_" .. skill .. ".png"

  local self = Sprite.newWithName(spriteName)
  Mixin.mix(self, OnboardioSkillIcon)

  self.shader = shader
  self:setShader(self.shader)

  self.baseColor = OnboardioCommon.SKILL_COLORS[skill]
  self.progressBar = createProgressBar(self, self.baseColor)
  self.graph = graphAnimationClass.new()

  self.graphSection = Column.new(self:getWidthPart(0.02))
  self.graphSection:insert(self.progressBar)
  self.graphSection:insert(self.graph)
  self.graphSection:alignChildren()
  self.graphSection:setLoc(self:getWidthPart(0.5), self:getHeightPart(-0.5))
  self:insert(self.graphSection)
  self.progressBar:setOpacity(0)
  self.graph:setOpacity(0)

  self.graphFinalY = self:getHeightPart(-0.5)

  self.skill = skill

  self.progressBar:setProgress(0)

  return self
end
