--- OnboardioGraphAnimationListening
-- @author Created by Joe Poka on July 22, 2016.

class("OnboardioGraphAnimationListening")

require("OnboardioGraphAnimationScrolling")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioGraphAnimationListening:getScrollDuration()
  return 1.6
end

function OnboardioGraphAnimationListening:getScrollingBlock(textureWidth, textureHeight)
  local theta = 0.0
  local deltaTheta = 0.08
  local maximumTheta = 4.0 * math.pi
  local thetaOffset = math.pi * 0.5

  local minimumAmplitude = textureHeight * 0.2
  local maximumAmplitude = textureHeight * 0.41
  local amplitude = minimumAmplitude
  local verticalOffset = textureHeight * 0.5
  local radius = textureWidth * 0.015

  local image = MOAIImage.new()
  local columnCount = textureWidth
  local rowCount = textureHeight
  image:init(columnCount, rowCount)

  repeat
    if (theta <= maximumTheta * 0.5) then
      amplitude = amplitude * 1.005
      amplitude = math.min(amplitude, maximumAmplitude)
    else
      amplitude = amplitude * 0.995
      amplitude = math.max(amplitude, minimumAmplitude)
    end

    theta = theta + deltaTheta
    local x = math.floor(textureWidth * (theta / maximumTheta))
    local y = math.floor(verticalOffset + math.sin(theta + thetaOffset) * amplitude)
    image:fillCircle(x, y, radius, 1, 1, 1, 1)
  until (theta > maximumTheta)

  local texture = MOAITexture.new()
  texture:load(image)
  MetalPrebinder.prebindTexture(texture)
  
  return texture
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioGraphAnimationListening.new()
  local self = OnboardioGraphAnimationScrolling.new()
  Mixin.mix(self, OnboardioGraphAnimationListening)

  self:startScrolling()

  return self
end
