--- OnboardioGraphAnimationMath
-- @author Created by Joe Poka on July 20, 2016.

class("OnboardioGraphAnimationMath")

require("OnboardioGraphAnimationScrolling")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioGraphAnimationMath:getScrollDuration()
  return 2.5
end

function OnboardioGraphAnimationMath:getScrollingBlock(textureWidth, textureHeight)
  local points = {
    {x = 0, y = 0.32},
    {x = 0.05, y = 0.75},
    {x = 0.1, y = 0.82},
    {x = 0.175, y = 0.625},
    {x = 0.21, y = 0.92},
    {x = 0.3, y = 0.84},
    {x = 0.33, y = 0.93},
    {x = 0.4, y = 0.625},
    {x = 0.5, y = 0.77},
    {x = 0.6, y = 0.52},
    {x = 0.77, y = 0.84},
    {x = 0.81, y = 0.61},
    {x = 0.88, y = 0.62},
    {x = 0.91, y = 0.59},
    {x = 1, y = 0.32}
  }

  for _, point in ipairs(points) do
    point.row = math.floor(point.y * textureHeight)
    point.column = math.floor(point.x * textureWidth)
  end

  local image = MOAIImage.new()
  local columnCount = textureWidth
  local rowCount = textureHeight
  image:init(columnCount, rowCount)

  -- to avoid little artifacts above the graphs on certain devices
  local bottomMargin = 3

  local previousPoint = points[1]
  for i = 2, #points do
    local currentPoint = points[i]
    local startColumn = previousPoint.column
    local endColumn = currentPoint.column
    local startRow = previousPoint.row
    local endRow = currentPoint.row
    for column = startColumn, endColumn do
      local row = math.floor(MathUtils.interpolateLinear(column, startColumn, endColumn, startRow, endRow))
      image:fillRect(column, rowCount - row, column + 1, rowCount - bottomMargin, 1, 1, 1, 1)
    end
    previousPoint = currentPoint
  end

  local texture = MOAITexture.new()
  texture:load(image)
  MetalPrebinder.prebindTexture(texture)

  return texture
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioGraphAnimationMath.new()
  local self = OnboardioGraphAnimationScrolling.new()
  Mixin.mix(self, OnboardioGraphAnimationMath)

  self:startScrolling()

  return self
end
