--- OnboardioGraphAnimationReading
-- @author Created by Jay Austria on Jan 22, 2016.

class("OnboardioGraphAnimationReading")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INITIAL_WIDTH = 95
local LIGHTER_HUE = {0.89, 0.89, 0.89, 0.4}
local DARKER_HUE = {0.91, 0.91, 0.91}
local BAR_HUE = {0.5, 0.5, 0.5}
local BAR_DESCRIPTORS = {
  {
    color = DARKER_HUE,
    startScale = 0.2,
    endScale = 0.5
  },
  {
    color = LIGHTER_HUE,
    startScale = 0.3,
    endScale = 0.1
  },
  {
    color = DARKER_HUE,
    startScale = 0.7,
    endScale = 0.2
  },
  {
    color = LIGHTER_HUE,
    startScale = 0.5,
    endScale = 0.8
  },
  {
    color = DARKER_HUE,
    startScale = 1.0,
    endScale = 0.6
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function startLoopingAction(self)
  if self.loopingAction then
    self.loopingAction:start()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local barMarginX = 4
  local barMarginY = 6
  local barHeight = 30
  local barWidth = (INITIAL_WIDTH - (barMarginX * #BAR_DESCRIPTORS)) / #BAR_DESCRIPTORS

  local barsRow = Row.new(barMarginX, Alignment.BOTTOM)
  local bottomRectangle = FilledRectangle.new(INITIAL_WIDTH, 2, BAR_HUE)

  local barAnimations = AHSpawn.new()
  for _, barDescriptor in ipairs(BAR_DESCRIPTORS) do
    local bar = FilledRectangle.new(barWidth, barHeight, barDescriptor.color)

    bar.endScale = barDescriptor.endScale
    bar.startScale = barDescriptor.startScale
    bar:setScl(1.0, barDescriptor.startScale)
    barsRow:insert(bar)
  end

  barsRow:alignChildren()

  local barsRowContainer = Group.new(INITIAL_WIDTH, barHeight + barMarginY)
  barsRowContainer:insertAtCenter(barsRow)

  self:insert(barsRowContainer)
  self:insert(bottomRectangle)
  self:alignChildren()

  local isRecursive = true
  self:linkColorAttrToChildren(isRecursive)
  self:setColor(unpack(OnboardioCommon.COLOR_RED))

  for _, bar in ipairs(barsRow.children) do
    bar:transformAnchorPoint(0.5, 0.0)
    local loopDuration = 1.67
    local barAction = AHSequence.new(
      AHScaleYTo.new(bar, loopDuration/2, AHEaseSimpleInOut.new(), bar.endScale),
      AHScaleYTo.new(bar, loopDuration/2, AHEaseSimpleInOut.new(), bar.startScale)
    )
    barAnimations:addAction(barAction)
  end

  self.loopingAction = AHLoop.new(barAnimations)
end

function OnboardioGraphAnimationReading.new()
  local self = Column.new()
  Mixin.mix(self, OnboardioGraphAnimationReading)

  createLayout(self)
  startLoopingAction(self)

  return self
end
