--- OnboardioGraphAnimationScrolling
-- @author Created by Joe Poka on July 19, 2016.

class("OnboardioGraphAnimationScrolling")

require("graphics.FlintstonesImage")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DIMENSIONS = {
  width = 95.0,
  graphHeight = 34.0,
  verticalSpacing = 2.0,
  rectangleHeight = 2.0
}

local HUE = {
  bar = {0.5, 0.5, 0.5}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioGraphAnimationScrolling:getScrollDuration()
  -- derived classes override this function and provide subject-specific durations
  local defaultScrollDuration = 3.0
  return defaultScrollDuration
end


function OnboardioGraphAnimationScrolling:getScrollingBlock(textureWidth, textureHeight)
  -- derived classes override this function and provide subject-specific blocks
  return nil
end

function OnboardioGraphAnimationScrolling:startScrolling()
  local textureWidth = 256
  local textureHeight = 128
  local textureScale = 1.0 / Host.contentScale
  local texture = self:getScrollingBlock(textureWidth, textureHeight)

  local scrollingImage = FlintstonesImage.newWithTexture(texture, textureWidth, textureHeight, textureScale)
  self:insertAtCenter(scrollingImage)

  local scaleX = DIMENSIONS.width / textureWidth
  local scaleY = DIMENSIONS.graphHeight / textureHeight
  scrollingImage:setScl(scaleX, scaleY)

  local scrollingImageY = DIMENSIONS.graphHeight * 0.5 + DIMENSIONS.rectangleHeight + DIMENSIONS.verticalSpacing
  scrollingImage:setY(scrollingImageY)

  local duration = self:getScrollDuration()
  scrollingImage:playCycle(duration)

  local isRecursive = true
  self:linkColorAttrToChildren(isRecursive)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  -- bottom rectangle
  local rectangle = FilledRectangle.new(DIMENSIONS.width, DIMENSIONS.rectangleHeight, HUE.bar)
  self:insertAtCenter(rectangle)

  local rectangleY = DIMENSIONS.rectangleHeight * 0.5
  rectangle:setY(rectangleY)
end

-- Constructor ------------------------

function OnboardioGraphAnimationScrolling.new()
  local width = DIMENSIONS.width
  local height = DIMENSIONS.graphHeight + DIMENSIONS.verticalSpacing + DIMENSIONS.rectangleHeight

  local self = Group.new(width, height)
  Mixin.mix(self, OnboardioGraphAnimationScrolling)

  createLayout(self)

  return self
end
