--- OnboardioGraphAnimationSpeaking
-- @author Created by Joe Poka on July 22, 2016.

class("OnboardioGraphAnimationSpeaking")

require("OnboardioGraphAnimationScrolling")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioGraphAnimationSpeaking:getScrollDuration()
  return 1.5
end

function OnboardioGraphAnimationSpeaking:getScrollingBlock(textureWidth, textureHeight)
  local barCount = 9
  local barWidth = textureWidth / barCount / 2
  local maximumBarHeight = math.floor(0.9 * textureHeight)

  local barHeightFactors = {
    0.9, 0.8, 0.6,
    0.2, 0.3, 0.7,
    0.9, 0.4, 0.6
  }

  local image = MOAIImage.new()
  local columnCount = textureWidth
  local rowCount = textureHeight
  image:init(columnCount, rowCount)

  -- to avoid little artifacts above the graphs on certain devices
  local bottomMargin = 3

  local column = barWidth / 2
  for i = 1, barCount do
    local barHeight = math.floor(maximumBarHeight * barHeightFactors[i])
    image:fillRect(column, textureHeight - barHeight, column + barWidth, textureHeight - bottomMargin, 1, 1, 1, 1)
    column = column + 2 * barWidth
  end

  local texture = MOAITexture.new()
  texture:load(image)
  MetalPrebinder.prebindTexture(texture)
  
  return texture
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioGraphAnimationSpeaking.new()
  local self = OnboardioGraphAnimationScrolling.new()
  Mixin.mix(self, OnboardioGraphAnimationSpeaking)

  self:startScrolling()

  return self
end
