--- OnboardioCountdown
-- @author Created by andrewcousins on March 13, 2024.

class("OnboardioCountdown")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.height

local BAR_COLOR = OnboardioCommon.COLOR_WHITE
local BAR_OPACITY = 0.1

local COUNTDOWN_DURATION = 23.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioCountdown:refill()
  self:stop()
  self.expired = false

  local refillDuration = 0.30
  local refillEase = AHEaseSineOut.new()
  local refillAction = self:newProgressTo(refillDuration, refillEase, 1)
  return refillAction:start()
end

function OnboardioCountdown:setProgress(progress)
  self.super.setProgress(self, progress)
  progress = self:getProgress()

  self.bar:setScl(1.0, 1 - progress)
end


function OnboardioCountdown:getTimeWeight()
  local progress = self:getProgress()
  local timeWeight = math.max(0.5, progress)
  return timeWeight
end

function OnboardioCountdown:hasExpired()
  return self.expired
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioCountdown.new()
  local self = Group.new(WIDTH, HEIGHT)
  ProgressIndicator.makeProgressIndicator(self)
  ProgressTimer.makeTimer(self, COUNTDOWN_DURATION, function() self.expired = true end)
  Mixin.mix(self, OnboardioCountdown)


  local bar = FilledRectangle.new(WIDTH, HEIGHT, BAR_COLOR)
  bar:setOpacity(BAR_OPACITY)
  bar:setAnchorPointTop()
  bar:setLoc(WIDTH * 0.5, HEIGHT)
  self:insert(bar)
  bar:setScl(1.0, 0.0)
  self.bar = bar

  self.countdownAction = nil
  self.expired = false

  return self
end