--- OnboardioPretest
-- @author Created by andrewc on September 08, 2021.

class("OnboardioPanelPretest")

require("OnboardioCountdown")
require("OnboardioPanelAudioSection")
require("OnboardioPanelChooseYourLanguage")
require("OnboardioPanelPretestConfirmation")
require("OnboardioPanelRound")
require("OnboardioPanelSelfRate")
require("OnboardioProgressIndicator")
require("OnboardioPretestCircleButtonManager")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_GRADIANT_TOP = ColorUtils.colorHex24ToStd(0x070638)
local BACKGROUND_GRADIANT_BOTTOM = ColorUtils.colorHex24ToStd(0x3F3F65)
local BACKGROUND_COVER_COLOR = ColorUtils.colorHex24ToStd(0x25224A)

local Z_BACKGROUND = 10
local Z_TRANSITION = 15
local Z_COUNTDOWN = 20
local Z_PANEL = 30
local Z_BUTTONS = 40
local Z_PROGRESS_INDICATOR = 70
local Z_OVERLAY = 80

local WRONG_ANSWERS_TO_BYPASS_SECTION = 1
local ANSWER_COUNTDOWN_EXPIRED = "COUNTDOWN_EXPIRED"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getDivisionCountForSectionGroupings(...)
  local divisionCount = 0
  local sectionGroupings = arg
  for _, sectionGrouping in ipairs(sectionGroupings) do
    for _, section in ipairs(sectionGrouping) do
      local rounds = section.rounds
      for _, round in ipairs(rounds) do
        divisionCount = divisionCount + round:getDivisionCount()
      end
    end
  end
  return divisionCount
end

local function processAnswer(self, section, round, answerId)
  local wasAnswerCorrect = (answerId == round:getCorrectAnswerId())

  local contributions = section.contributions
  local scoresBySkillGroupIdentifier = self.scoresBySkillGroupIdentifier
  local scoreEarned = 0

  if (wasAnswerCorrect) then
    local countdown = self.countdown
    local timeWeight = countdown:getTimeWeight()
    local difficultyWeight = round:getCorrectDifficultyWeight()
    scoreEarned = timeWeight * difficultyWeight

    for skillGroupIdentifier, contribution in pairs(contributions) do
      local scores = scoresBySkillGroupIdentifier[skillGroupIdentifier]

      scores.correct = scores.correct + contribution * timeWeight * difficultyWeight
      scores.possible = scores.possible + contribution * difficultyWeight
    end
  else
    local difficultyWeight = round:getWrongDifficultyWeight()

    for skillGroupIdentifier, contribution in pairs(contributions) do
      local scores = scoresBySkillGroupIdentifier[skillGroupIdentifier]
      scores.possible = scores.possible + contribution * difficultyWeight
    end
  end

  return wasAnswerCorrect, scoreEarned
end

local function waitForSelectedAnswerFromPanel(self, panel, shouldStartCountdown)
  local previousPanel = self.currentPanel
  self.currentPanel = panel
  local answer = nil

  if (panel ~= nil) then
    local countdown = self.countdown

    self:insertAtCenter(panel, Z_PANEL)
    if (panel:usesHorizontalTransitionIn()) then
       panel:setX(self:getWidthPart(1.5))
    end

    local frameToImprovePerformanceOnOldDevices = 1
    waitForFrames(frameToImprovePerformanceOnOldDevices)

    wait(OnboardioTransitioner.playTransitionBetweenPanels(previousPanel, panel))

    if (shouldStartCountdown) then
      countdown:start()
    end

    waitUntil(function() return panel:isDismissed() or countdown:hasExpired() end)

    answer = panel:getSelectedAnswer()
    if (answer == nil) then
      answer = ANSWER_COUNTDOWN_EXPIRED
      panel:handleCountdownExpired()
    end
  end

  return answer
end

local function waitForShowSelfRate(self, selfRateRounds, sectionIdentifier)
  local scoresBySkillGroupIdentifier = {}

  for skillGroupIdentifier, _ in pairs(self.scoresBySkillGroupIdentifier) do
    scoresBySkillGroupIdentifier[skillGroupIdentifier] = {
      correct = 0.0,
      possible = 0.0
    }
  end
  for index, selfRateRound in ipairs(selfRateRounds) do
    local isFirstSelfRate = (index == 1)
    local isLastSelfRate = (index == #selfRateRounds)
    local roundPanel = OnboardioPanelSelfRate.new(selfRateRound, isFirstSelfRate, isLastSelfRate)

    local ratingLevel = waitForSelectedAnswerFromPanel(self, roundPanel)
    local ratingAmount = ratingLevel.amount
    local skillGroupIdentifier = selfRateRound:getContributionSkillGroupIdentifier()

    local record = scoresBySkillGroupIdentifier[skillGroupIdentifier]
    record.correct = record.correct + ratingAmount
    record.possible = record.possible + 1.0

    local divisionsToFill = 1
    self.progressIndicator:fillDivisions(divisionsToFill)
  end

  local sectionIsSelfRate = true
  local questionsAnsweredCount = #selfRateRounds
  local questionsCorrectCount = questionsAnsweredCount
  local questionsPossibleCount = questionsAnsweredCount
  OnboardioAnalytics.reportTestSectionCompleted(sectionIdentifier, sectionIsSelfRate, questionsAnsweredCount, questionsCorrectCount, questionsPossibleCount, scoresBySkillGroupIdentifier)

  for skillGroupIdentifier, score in pairs(self.scoresBySkillGroupIdentifier) do
    local newScore = scoresBySkillGroupIdentifier[skillGroupIdentifier]
    score.correct = score.correct + newScore.correct
    score.possible = score.possible + newScore.possible
  end

end

local function waitForShowSelfRateNormal(self)
  local sectionIdentifier = "self_rate_normal"
  local selfRateRounds = OnboardioContent.getSelfRateRounds()

  local remainingDivisionCount = #selfRateRounds
  self.progressIndicator:setRemainingDivisionCount(remainingDivisionCount)

  self.progressIndicator:fadeIn()

  waitForShowSelfRate(self, selfRateRounds, sectionIdentifier)
end

local function waitForShowNormalSection(self, section, isFirstSection)
  local rounds = section.rounds
  local sectionIdentifier = section.identifier
  if self.buttonManager then
    local answerOptions = section.answerOptionsCount
    self.buttonManager:setNextSectionButtonCount(answerOptions)
  end

  local wrongAnswerCount = 0
  local questionsCorrectCount = 0
  local questionsAnsweredCount = 0
  local oldScoresBySkillGroupIdentifier = {}

  for skillGroupIdentifier, scores in pairs(self.scoresBySkillGroupIdentifier) do
    local oldScores = {correct = scores.correct, possible = scores.possible}
    oldScoresBySkillGroupIdentifier[skillGroupIdentifier] = oldScores
  end

  local roundCount = #rounds
  for roundIndex, round in ipairs(rounds) do
    local isFirstQuestionOfSection = (roundIndex == 1)
    local isFirstQuestion = (isFirstSection and isFirstQuestionOfSection)
    local roundPanel = OnboardioPanelRound.new(round, self.buttonManager, self.countdown, self.progressIndicator, isFirstQuestion, isFirstQuestionOfSection)

    local shouldStartCountdown = true

    if ((not isFirstSection) or (roundIndex > 1)) then
      self.countdown:refill()
    end
    questionsAnsweredCount = questionsAnsweredCount + 1
    local answerId = waitForSelectedAnswerFromPanel(self, roundPanel, shouldStartCountdown)
    local timeElapsed = self.countdown:getTime() * 1000
    local wasAnswerCorrect, scoreEarned = processAnswer(self, section, round, answerId)

    OnboardioAnalytics.reportQuestionAnswered(answerId, wasAnswerCorrect, timeElapsed, round.identifier, scoreEarned)

    if (wasAnswerCorrect) then
      questionsCorrectCount = questionsCorrectCount + 1
    else
      wrongAnswerCount = wrongAnswerCount + 1
    end

    if ((wrongAnswerCount >= WRONG_ANSWERS_TO_BYPASS_SECTION) and (roundIndex < roundCount)) then
      local remainingRoundCount = roundCount - roundIndex

      local contributions = section.contributions

      for remainingRoundIndex = roundIndex + 1, roundCount do
        local remainingRound = rounds[remainingRoundIndex]
        local remainingRoundDifficultyWeight = remainingRound:getWrongDifficultyWeight()

        for skillGroupIdentifier, contribution in pairs(contributions) do
          local scores = self.scoresBySkillGroupIdentifier[skillGroupIdentifier]
          scores.possible = scores.possible + contribution * remainingRoundDifficultyWeight
        end
      end

      local divisionsToFill = remainingRoundCount + 1
      self.progressIndicator:fillDivisions(divisionsToFill)

      break
    else
      local divisionsToFill = 1
      self.progressIndicator:fillDivisions(divisionsToFill)
    end
  end

  local sectionIsSelfRate = false
  local questionsPossibleCount = roundCount

  local sectionScores = {}
  for skillGroupIdentifier, score in pairs(self.scoresBySkillGroupIdentifier) do
    local oldScore = oldScoresBySkillGroupIdentifier[skillGroupIdentifier]
    sectionScores[skillGroupIdentifier] = {
      correct = score.correct - oldScore.correct,
      possible = score.possible - oldScore.possible
    }
  end

  OnboardioAnalytics.reportTestSectionCompleted(sectionIdentifier, sectionIsSelfRate, questionsAnsweredCount, questionsCorrectCount, questionsPossibleCount, sectionScores)
end

local function waitForShowNormalSections(self)
  local sections = OnboardioContent.getNormalSections()

  local remainingDivisionCount = getDivisionCountForSectionGroupings(sections)
  local selfRateAudioRounds = OnboardioContent.getSelfRateAudioRounds()
  remainingDivisionCount = remainingDivisionCount + #selfRateAudioRounds
  self.progressIndicator:setRemainingDivisionCount(remainingDivisionCount)
  self.progressIndicator:fadeIn()

  self.buttonManager = OnboardioPretestCircleButtonManager.new(self.countdown)
  self:insertAtCenter(self.buttonManager, Z_BUTTONS)

  for i, section in ipairs(sections) do
    local isFirstSection = (i == 1)
    waitForShowNormalSection(self, section, isFirstSection)
  end

  self.buttonManager:fadeOutInNextTransition()

  self.countdown:refill()
end

local function waitForShowSelfRateAudio(self)
  local sectionIdentifier = "self_rate_audio"
  local selfRateRounds = OnboardioContent.getSelfRateAudioRounds()

  local remainingDivisionCount = #selfRateRounds
  self.progressIndicator:setRemainingDivisionCount(remainingDivisionCount)

  waitForShowSelfRate(self, selfRateRounds, sectionIdentifier)
end

local function waitForShowPretestPanel(self)
  self.overlay:setSkipButtonCallback(function()
    self.pretestConfirmationPanel:setSelectedAnswer(OnboardioGame.ANSWER_SKIP)
    self.pretestConfirmationPanel:setDismissed()
  end)

  self.pretestConfirmationPanel = OnboardioPanelPretestConfirmation.new(self.useConciseTitleForPretestConfirmation)
  self:insertAtCenter(self.pretestConfirmationPanel, Z_OVERLAY)
  self.pretestConfirmationPanel:playAppear()
  self.overlay:setSkipButtonVisible(true)
  waitUntil(function() return self.pretestConfirmationPanel:isDismissed() end)
  self.pretestType = self.pretestConfirmationPanel:getSelectedAnswer()
  OnboardioTransitioner.playTransitionOut(self.pretestConfirmationPanel)
end

local function createCountdown(self)
  local width = self:getWidth()
  local height = self:getHeight()

  local background = RectangleGradient.new(width, height, BACKGROUND_GRADIANT_TOP, BACKGROUND_GRADIANT_BOTTOM)
  self:insertAtCenter(background, Z_BACKGROUND)

  local countdown = OnboardioCountdown.new()

  self:insertAtCenter(countdown, Z_COUNTDOWN)
  self.countdown = countdown
end

local function createProgressIndicator(self)
  local progressIndicator = OnboardioProgressIndicator.new()
  progressIndicator:hide()
  local progressIndicatorY = math.min(self:getHeightPart(0.945), Screen.safeTop - progressIndicator:getHeightPart(0.5))
  progressIndicator:setLoc(self:getWidthPart(0.5), progressIndicatorY)
  self:insert(progressIndicator, Z_PROGRESS_INDICATOR)
  self.progressIndicator = progressIndicator
end

local function waitForShowLanguageSelection(self)
  if (OnboardioPanelChooseYourLanguage.shouldShowChooseYourLanguagePanel()) then
    local chooseYourLanguagePanel = OnboardioPanelChooseYourLanguage.new()
    self.overlay:setSkipButtonCallback(function()
      chooseYourLanguagePanel:setDismissed()
    end)

    self:insertAtCenter(chooseYourLanguagePanel, Z_PANEL)
    wait(OnboardioTransitioner.playTransitionIn(chooseYourLanguagePanel))
    self.overlay:setSkipButtonVisible(true)

    waitUntil(function() return chooseYourLanguagePanel:isDismissed() end)
    wait(OnboardioTransitioner.playTransitionOut(chooseYourLanguagePanel))
  end
end

local function prepareForTransitionAnimation(self)
  local transitionOverlay = FilledRectangle.newFullScreen(OnboardioCommon.COLOR_BACKGROUND)
  self:insertAtCenter(transitionOverlay, Z_TRANSITION)
  self.transitionOverlay = transitionOverlay

  local circleSize = math.max(Screen.width, Screen.height)
  local expandingCircleOverlay = FilledCircle.new(circleSize, BACKGROUND_COVER_COLOR)
  expandingCircleOverlay:setScl(0)
  self:insertAtCenter(expandingCircleOverlay, Z_TRANSITION)
  self.expandingCircleOverlay = expandingCircleOverlay
end

local function getTransitionAnimation(self)
  local animation = AHSequence.new(
    function() Sound.playSoundEffect("intro_whoosh") end,
    AHScaleTo.new(self.expandingCircleOverlay, 0.6, AHEaseBezier.newFlat({0.8, 0.0, 0.4, 1.0}), 1.0),
    function() self.transitionOverlay:setOpacity(0) end,
    AHOpacityTo.new(self.expandingCircleOverlay, 0.2, nil, 0),
    AHRemove.new(self.transitionOverlay, self.expandingCircleOverlay)
  )
  AHDelayed.new(0.1, animation):start()
  -- Start with the first intro slightly before the transition is complete
  return AHDelayed.new(0.6, AHDoNothing.new()):start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelPretest:start()
  waitForShowLanguageSelection(self)
  waitForShowPretestPanel(self)
  self.overlay:setSkipButtonVisible(false)

  createCountdown(self)
  createProgressIndicator(self)

  prepareForTransitionAnimation(self)
  wait(getTransitionAnimation(self))

  if (self.pretestType == OnboardioGame.ANSWER_SKIP) then
    OnboardioAnalytics.reportTestSkipped()
    waitForShowSelfRateNormal(self)
  else
    OnboardioAnalytics.reportTestStarted()
    waitForShowNormalSections(self)
    waitForShowSelfRateAudio(self)
  end

  self.progressIndicator:fadeOut()
  wait(OnboardioTransitioner.playTransitionOut(self.currentPanel))
  self:setDismissed()
end

function OnboardioPanelPretest:playDisappear()
  self.progressIndicator:fadeOut()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioPanelPretest.new(overlay, scoresBySkillGroupIdentifier, onboardingStartedAtPretest)
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioPanelPretest)

  self.overlay = overlay
  self.scoresBySkillGroupIdentifier = scoresBySkillGroupIdentifier

  self.currentPanel = nil
  self.pretestType = nil
  self.useConciseTitleForPretestConfirmation = onboardingStartedAtPretest

  return self
end
