--- OnboardioPanelPretestConfirmation
-- @author Created by Aaron Barrett on October 01, 2014.

class("OnboardioPanelPretestConfirmation")
require("OnboardioPanel")
require("OnboardioButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TITLE_TEXT_KEY = "TITLE_TEST"
local TITLE_CONCISE_TEXT_KEY = "TITLE_TEST_CONCISE"
local PRETEST_BUTTON_TEXT_KEY = "GET_STARTED"
local TEXT_COLOR = OnboardioCommon.COLOR_PROMPT_TEXT

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPanelPretestConfirmation:playAppear()
  OnboardioAnalytics.reportTestStartScreenSeen()
  local animation = AHSpawn.new()
  local duration = 0.4
  local ease = AHEaseSimpleInOut.new()
  for _, label in ipairs(self.fadableLabels) do
    animation:addAction(AHOpacityTo.new(label, duration, ease, 1))
  end
  for _, button in ipairs(self.buttons) do
    animation:addAction(AHOpacityTo.new(button, duration, ease, 1))
    button:setTouchEnabled(true)
  end
  return animation:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createTextArea(self, text, font, color)
  local label = TextArea.new(text, font, OnboardioCommon.TEXT_WIDTH, Alignment.CENTER)
  label:setColor(unpack(color))
  table.insert(self.fadableLabels, label)
  label:setOpacity(0)
  return label
end

local function createStationaryButton(self, text, callback)
  local button = OnboardioButton.newSkinny(text, callback)
  table.insert(self.buttons, button)
  table.insert(self.stationaryButtons, button)
  button:setOpacity(0)
  return button
end

-- Constructor ------------------------

function OnboardioPanelPretestConfirmation.new(useConciseTitle)
  local self = OnboardioPanel.new()
  Mixin.mix(self, OnboardioPanelPretestConfirmation)

  local titleTextKey = TITLE_TEXT_KEY
  if useConciseTitle then
    titleTextKey = TITLE_CONCISE_TEXT_KEY
  end
  local buttonTextKey = PRETEST_BUTTON_TEXT_KEY

  local column = Column.new(Screen.getHeightPart(0.04))
  local titleText = Localizer.get(titleTextKey)
  local pretestButtonText = Localizer.get(buttonTextKey)
  column:insert(createTextArea(self, titleText, Font.new(FontFaces.DIN_OT_MEDIUM, 22), TEXT_COLOR))
  column:insert(createStationaryButton(self, pretestButtonText, function()
    self:setDismissed()
  end))

  column:alignChildren()
  self:insertAtCenter(column)

  return self
end
