--- OnboardioPretestCircleButton
-- @author Created by andrewcousins on March 13, 2024.

class("OnboardioPretestCircleButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIGGLE_SIZE = 2
local WIGGLE_DURATION = 1.25

local SPRITE_SIZE = math.min(Screen.getWidthPart(0.4), Screen.getHeightPart(0.2))

local CORRECT_BLIP_COLOR = ColorUtils.colorHex24ToStd(0x24FF47)
local CORRECT_BLIP_OPACITY = 0.075

local SPARKLE_SCALE = 2.0

local TOP_FIREWORK_COLOR = ColorUtils.colorHex24ToStd(0x9863DC)
local BOTTOM_FIREWORK_COLOR = ColorUtils.colorHex24ToStd(0x18FFFF)

local Z_INDEX = {
  BACKGROUND = 10,
  SPRITE = 20,
  LABEL = 30,
  EFFECT = 40
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Currently unused
-- local function runDefaultTappedAnimation(self)
--   self.baseSprite:setOpacity(1)
--   local scaleAnimation = AHSequence.new(
--     AHScaleTo.new(self, 0.167, AHEaseBezier.newFlat({0.60, 0.00, 0.28, 0.78}), self.baseScale * 1.1),
--     AHScaleTo.new(self, 0.167, AHEaseBezier.newFlat({0.22, -0.34, 0.53, 1.00}), self.baseScale)
--   )
--   Sound.playSoundEffect("button_press")
--   return scaleAnimation:start()
-- end

local function startLargeSparkleAnimation(self)
  local sparkle = self.largeSparkle
  sparkle:setRot(-180)
  sparkle:setScl(0.2 * SPARKLE_SCALE)
  sparkle:setOpacity(1)

  local duration = 0.8
  local fadeDuration = duration * 0.75
  local scaleAnimation = AHScaleTo.new(sparkle, duration, AHEaseBezier.newFlat({0, 0, 0, 0.7}), SPARKLE_SCALE)
  local fadeAnimation = AHOpacityTo.new(sparkle, fadeDuration, nil, 0)
  local spinAnimation = AHRotationBy.new(sparkle, duration, AHEaseBezier.newFlat({0, 0, 0, 0.7}), 180)

  return AHSpawn.new(scaleAnimation, fadeAnimation, spinAnimation):start()
end

local function startSmallSparkleAnimation(self)
  local sparkle = self.smallSparkle
  sparkle:setRot(0)
  sparkle:setScl(0.2 * SPARKLE_SCALE)

  local duration = 0.6
  local fadeDuration = duration * 0.75
  local setUp = function() sparkle:setOpacity(1) end
  local scaleAnimation = AHScaleTo.new(sparkle, duration, AHEaseBezier.newFlat({0, 0, 0, 0.7}), SPARKLE_SCALE)
  local fadeAnimation = AHOpacityTo.new(sparkle, fadeDuration, nil, 0)
  local spinAnimation = AHRotationBy.new(sparkle, duration, AHEaseBezier.newFlat({0, 0, 0, 0.7}), -80)

  return AHDelayed.new(0.05, AHSequence.new(setUp, AHSpawn.new(scaleAnimation, fadeAnimation, spinAnimation))):start()
end

local function getPillOffset(pill, distance)
  local rotation = math.rad(pill:getRot())
  local offsetX = distance * math.cos(rotation)
  local offsetY = distance * math.sin(rotation)
  return offsetX, offsetY
end

local function startFireWorkAnimation(firework, delay)
  local fadeDelay = 0.2
  local fadeDuration = 0.3
  local animation = AHSpawn.new(
    function() firework:setOpacity(1) end,
    AHDelayed.new(fadeDelay, AHOpacityTo.new(firework, fadeDuration, nil, 0))
  )
  local startDuration = 0.7
  local startEase = AHEaseBezier.newFlat({0, 0.5, 0, 1})
  local endDelay = 0.1
  local endDuration = 0.7
  local endEase = AHEaseBezier.newFlat({0, 0.7, 0, 1})
  for _, pill in ipairs(firework.pills) do
    local offsetX, offsetY = getPillOffset(pill, pill:getWidthPart(1))
    local move = AHLocationBy.new(pill, startDuration, startEase, offsetX, offsetY)
    local scale = AHDelayed.new(endDelay, AHScaleXTo.new(pill, endDuration, endEase, 0.75))
    local reset = AHDelayed.new(0.1, function()
      pill:setScl(1)
      pill:offsetX(-offsetX)
      pill:offsetY(-offsetY)
    end)
    animation:addAction(AHSequence.new(AHSpawn.new(move, scale), reset))
  end

  return AHDelayed.new(delay, animation):start()
end

local function startCorrectBlipAnimation(self, scale, startingScale)
  local blip = FilledCircle.new(self:getWidthPart(0.5), CORRECT_BLIP_COLOR)
  blip:setOpacity(CORRECT_BLIP_OPACITY)
  blip:setScl(startingScale)

  local delay = 0.15
  local duration = 0.467
  local insertBlip = function() self:insertAtCenter(blip, Z_INDEX.BACKGROUND) end
  local scaleUp = AHScaleTo.new(blip, duration, AHEaseBezier.newFlat({0, 0, 0, 1}), scale)
  local fadeOut = AHOpacityTo.new(blip, duration, nil, 0)
  return AHDelayed.new(delay, AHSpawn.new(insertBlip, scaleUp, fadeOut)):start()
end

local function positionEffects(self)
  local minDistanceMod = 0.9
  local maxDistanceMod = 1.1
  local offsetDegrees = {10, 130, 230, 315}
  local offsetVariance = 30
  local randomRotation = math.random() * 360
  for i, effect in ipairs({self.largeSparkle, self.whiteFirework, self.smallSparkle, self.blueFirework}) do
    local distanceMod = minDistanceMod + (math.random() * (maxDistanceMod - minDistanceMod))
    local distance = self:getWidthPart(0.5 * distanceMod)
    local angleOffset = (math.random() * offsetVariance) - (offsetVariance / 2)
    local angleInRadians = math.rad(randomRotation + offsetDegrees[i] + angleOffset)
    local locationX = self:getWidthPart(0.5) + (distance * math.cos(angleInRadians))
    local locationY = self:getHeightPart(0.5) + (distance * math.sin(angleInRadians))
    effect:setLoc(locationX, locationY)
  end
end

local function startEffectsAnimation(self)
  positionEffects(self)

  startCorrectBlipAnimation(self, 1.52, 1.18)
  startCorrectBlipAnimation(self, 1.21, 1.15)

  startLargeSparkleAnimation(self)
  startSmallSparkleAnimation(self)

  startFireWorkAnimation(self.blueFirework, 0.1)
  startFireWorkAnimation(self.whiteFirework, 0.15)
end

local function getScaleAnimationForCorrectElement(element, endScale)
  return AHSequence.new(
    AHScaleTo.new(element, 0.167, AHEaseBezier.newFlat({0.60, 0.00, 0.28, 0.78}), endScale * 1.1),
    AHScaleTo.new(element, 0.167, AHEaseBezier.newFlat({0.22, -0.34, 0.53, 1.00}), endScale)
  )
end

local function runCorrectAnimation(self)
  self.correctSprite:setOpacity(1)
  self.baseSprite:setOpacity(0)
  self.correctSprite:setScl(self.correctSprite.baseScale * 0.6)
  self.label:setScl(self.label.baseScale * 0.6)
  local scaleAnimation = AHSpawn.new(
    getScaleAnimationForCorrectElement(self.correctSprite, self.correctSprite.baseScale),
    getScaleAnimationForCorrectElement(self.label, self.label.baseScale)
  )

  scaleAnimation:start()
  startEffectsAnimation(self)
  Sound.playSoundEffect("correct_answer")
  self.correctAnimationCallback()

  local delayBeforeStartingTransition = 0.275
  return AHDelay.new(delayBeforeStartingTransition):start()
end

local function runIncorrectAnimation(self)
  self.incorrectSprite:setOpacity(1)
  self.baseSprite:setOpacity(0)

  local shakeDuration = 0.05
  local shakeEase = AHEaseLinear.new()
  local shakeDistance = self:getWidthPart(0.025) * self.baseScale
  local shakeAnimation = AHSequence.new(
    AHLocationXBy.new(self, shakeDuration, shakeEase, -shakeDistance),
    AHLocationXBy.new(self, shakeDuration*2, shakeEase, shakeDistance*2),
    AHLocationXBy.new(self, shakeDuration*1.5, shakeEase, -shakeDistance*1.5),
    AHLocationXBy.new(self, shakeDuration, shakeEase, shakeDistance),
    AHLocationXBy.new(self, shakeDuration*0.5, shakeEase, -shakeDistance*0.5)
  )

  Sound.playSoundEffect("incorrect_answer")
  return shakeAnimation:start()
end

local function handleButtonTapped(self)
  -- Make sure any overlapping animations have this button in the foreground
  self:bringToFront()

  if self.isCorrect then
    return runCorrectAnimation(self)
  else
    return runIncorrectAnimation(self)
  end
end

local function addButtonFunctionality(self)
  local onTouchDown = function()
    self.countdown:stop()
    self:stopWiggle()
    wait(handleButtonTapped(self))
    self.onTapCallback()
  end
  Button.makeButton(self, onTouchDown)
end

local function getNextWiggle(self, wiggleDoneCallback)
  local wiggleSize = self.baseScale * WIGGLE_SIZE * ((math.random() * 0.25) + 0.75)
  local wiggleDuration = WIGGLE_DURATION * ((math.random() * 0.15) + 0.85)

  local angle = math.random() * 2 * math.pi
  local targetX = self.homeX + wiggleSize * math.cos(angle)
  local targetY = self.homeY + wiggleSize * math.sin(angle)
  local wiggleAnimation = AHLocationTo.new(self, wiggleDuration, AHEaseSineInOut.new(), targetX, targetY)
  return AHSequence.new(wiggleAnimation, wiggleDoneCallback)
end

local function recursiveWiggle(self)
  self.wiggleAnimation:stop()
  if self.canWiggle then
    self.wiggleAnimation = getNextWiggle(self, function() recursiveWiggle(self) end)
    self.wiggleAnimation:start()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPretestCircleButton:setAnswer(text, isCorrect)
  self.label:setText(text)
  if (self.baseScale < 1) then
    self.label.baseScale = 2
  else
    self.label.baseScale = 1
  end
  self.label:setScl(self.label.baseScale)
  self.isCorrect = isCorrect
end

function OnboardioPretestCircleButton:getIsCorrect()
  return self.isCorrect
end

function OnboardioPretestCircleButton:setOnPressedCallback(callback)
  self.onTapCallback = callback
end

function OnboardioPretestCircleButton:setCorrectAnimationCallback(callback)
  self.correctAnimationCallback = callback
end

function OnboardioPretestCircleButton:hide()
  for _, sprite in ipairs(self.sprites) do
    sprite:setOpacity(0)
  end
  self.label:setOpacity(0)
end

function OnboardioPretestCircleButton:setBaseScale(baseScale, duration, ease)
  self.baseScale = baseScale
end

function OnboardioPretestCircleButton:getScaleToBaseAnimation(duration, ease)
  return AHScaleTo.new(self, duration, ease, self.baseScale)
end

function OnboardioPretestCircleButton:getRevealAnswerAnimation()
  local setupFunction = function()
    self.correctSprite:setOpacity(1)
    self.baseSprite:setOpacity(0)
    self.correctSprite:setScl(self.correctSprite.baseScale * 0.6)
    self.label:setScl(self.label.baseScale * 0.6)
  end
  local scaleAnimation = AHSpawn.new(
    getScaleAnimationForCorrectElement(self.correctSprite, self.correctSprite.baseScale),
    getScaleAnimationForCorrectElement(self.label, self.label.baseScale)
  )

  return AHSpawn.new(setupFunction, scaleAnimation)
end

function OnboardioPretestCircleButton:getEffectiveWidth()
  return self:getWidth() * self.baseScale
end

function OnboardioPretestCircleButton:getEffectiveHeight()
  return self:getHeight() * self.baseScale
end

function OnboardioPretestCircleButton:prepareForAppearAnimation(scaleAmount)
  local scaleAmount = scaleAmount or 0.9
  self:hide()
  self:setScl(scaleAmount * self.baseScale)
end

function OnboardioPretestCircleButton:getAppearAnimation()
  return AHSpawn.new(
    AHOpacityTo.new(self.baseSprite, 0.2, nil, 1),
    AHOpacityTo.new(self.label, 0.2, nil, 1),
    AHSequence.new(
      AHScaleTo.new(self, 0.2, AHEaseBezier.newFlat({0.6, 0.0, 0.4, 1.0}), 1.05 * self.baseScale),
      AHScaleTo.new(self, 0.2, AHEaseBezier.newFlat({0.2, 0.0, 0.5, 1.0}), self.baseScale),
      function() self:startWiggle() end
  ))
end

function OnboardioPretestCircleButton:getRoundOverAnimation(includeFadeOut)
  local animation = AHSpawn.new(
    function() self:stopWiggle() end,
    AHOpacityTo.new(self.label, 0.2, nil, 0),
    AHScaleTo.new(self, 0.2, AHEaseBezier.newFlat({0.5, 0.0, 0.8, 1.0}), 0.8 * self.baseScale)
  )
  if includeFadeOut then
    for _, sprite in ipairs(self.sprites) do
      animation:addAction(AHOpacityTo.new(sprite, 0.2, nil, 0))
    end
  else
    animation:addAction(function() self.baseSprite:setOpacity(1) end)
    animation:addAction(AHOpacityTo.new(self.correctSprite, 0.2, nil, 0))
    animation:addAction(AHOpacityTo.new(self.incorrectSprite, 0.2, nil, 0))
  end
  return animation
end

function OnboardioPretestCircleButton:getRoundOverAndHideAnimation()
  return AHSequence.new(
    AHSpawn.new(
      function()
        self:stopWiggle()
        self.baseSprite:setOpacity(1)
      end,
      AHOpacityTo.new(self.label, 0.2, nil, 0),
      AHScaleTo.new(self, 0.2, AHEaseBezier.newFlat({0.5, 0.0, 0.8, 1.0}), 0.8 * self.baseScale),
      AHOpacityTo.new(self.correctSprite, 0.2, nil, 0),
      AHOpacityTo.new(self.incorrectSprite, 0.2, nil, 0),
      AHDelayed.new(0.0, AHSpawn.new(
        AHOpacityTo.new(self.baseSprite, 0.3, nil, 0)
      ))
    ),
    function() self:hide() end)
end

function OnboardioPretestCircleButton:getMoveToNewHome(newX, newY, duration, ease)
  self.homeX = newX
  self.homeY = newY
  self.moveToHomeAnimation:stop()
  self.moveToHomeAnimation = AHSequence.new(
    function() self.canWiggle = false end,
    AHLocationTo.new(self, duration, ease, newX, newY),
    function() self.canWiggle = true end
  )
  return self.moveToHomeAnimation
end

function OnboardioPretestCircleButton:startWiggle()
  recursiveWiggle(self)
end

function OnboardioPretestCircleButton:stopWiggle()
  self.wiggleAnimation:stop()
  self.canWiggle = false
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createSprite(spriteName)
  local sprite = Sprite.newWithName(spriteName)
  sprite:setEffectiveDims(SPRITE_SIZE, SPRITE_SIZE)
  sprite.baseScale = sprite:getScl()
  return sprite
end

local function addHiddenSprite(self, spriteName)
  local sprite = createSprite(spriteName)
  sprite:setOpacity(0)
  self:insertAtCenter(sprite, Z_INDEX.SPRITE)
  table.insert(self.sprites, sprite)
  return sprite
end

local function addSparkles(self)
  local largeSparkle = Sprite.newWithName("sparkle_large.png")
  largeSparkle:setOpacity(0)
  self:insertAtLocationFractional(largeSparkle, 0.63, 1.1, Z_INDEX.EFFECT)
  self.largeSparkle = largeSparkle

  local smallSparkle = Sprite.newWithName("sparkle_small.png")
  smallSparkle:setOpacity(0)
  self:insertAtLocationFractional(smallSparkle, 0.13, 0.3, Z_INDEX.EFFECT)
  self.smallSparkle = smallSparkle
end

local function buildFirework(self, color)
  local pillHeight = self:getWidthPart(0.025)
  local pillWidth = pillHeight * 3
  local cornerRadius = pillHeight / 2
  local fireworkGroup = Group.new(pillWidth * 2.5, pillWidth * 2.5)
  local pills = {}
  for i=1, 8 do
    local pill = FilledRoundedRectangle.new(pillWidth, pillHeight, cornerRadius, color)
    fireworkGroup:insertAtCenter(pill)
    pill:setRot(45 * i)
    local offsetX, offsetY = getPillOffset(pill, pillWidth * 1.5)
    pill:offsetX(offsetX)
    pill:offsetY(offsetY)
    table.insert(pills, pill)
  end
  fireworkGroup:linkOpacityAttrToChildren()
  fireworkGroup.pills = pills
  return fireworkGroup
end

local function addFireworks(self)
  local blueFirework = buildFirework(self, TOP_FIREWORK_COLOR)
  blueFirework:setOpacity(0)
  self:insertAtLocationFractional(blueFirework, 0.075, 0.925, Z_INDEX.EFFECT)
  self.blueFirework = blueFirework

  local whiteFirework = buildFirework(self, BOTTOM_FIREWORK_COLOR)
  whiteFirework:setOpacity(0)
  self:insertAtLocationFractional(whiteFirework, 0.8, 0.2, Z_INDEX.EFFECT)
  self.whiteFirework = whiteFirework
end

local function createLayout(self, baseSprite)
  self:insertAtCenter(baseSprite, Z_INDEX.SPRITE)
  self.sprites = {baseSprite}
  self.baseSprite = baseSprite

  self.correctSprite = addHiddenSprite(self, "circle_button_correct.png")
  self.incorrectSprite = addHiddenSprite(self, "circle_button_incorrect.png")

  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 20)
  local textLabel = Label.new("", font)
  self:insertAtCenter(textLabel, Z_INDEX.LABEL)
  self.label = textLabel
  self.label.baseScale = 1

  addSparkles(self)
  addFireworks(self)
end

-- Constructor ------------------------

function OnboardioPretestCircleButton.new(countdown)
  local baseSprite = createSprite("circle_button_standard.png")
  local buttonSize = baseSprite:getEffectiveWidth()
  local self = Group.new(buttonSize, buttonSize)
  Mixin.mix(self, OnboardioPretestCircleButton)

  self.countdown = countdown

  self.isCorrect = false
  self.baseScale = 1

  self.homeX = 0
  self.homeY = 0
  self.moveToHomeAnimation = AHDoNothing.new()
  self.wiggleAnimation = AHDoNothing.new()
  self.canWiggle = false

  createLayout(self, baseSprite)

  self.onTapCallback = function() end
  self.correctAnimationCallback = function() end
  addButtonFunctionality(self)

  return self
end
