--- OnboardioPretestSelfRateButton
-- @author Created by andrewcousins on March 15, 2024.

class("OnboardioPretestSelfRateButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local NORMAL_SPRITE_NAME = "rectangle_button_standard.png"
local PRESSED_SPRITE_NAME = "rectangle_button_pressed.png"

local LABEL_COLOR = OnboardioCommon.COLOR_WHITE
local FONT = OnboardioCommon.FONT_NORMAL

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getOnTappedAnimation(self)
  return AHSequence.new(
    function() self:setScl(0.95) end,
    AHScaleTo.new(self, 0.167, AHEaseBezier.newFlat({0.60, 0.00, 0.28, 0.78}), 1.05),
    AHScaleTo.new(self, 0.167, AHEaseBezier.newFlat({0.22, -0.34, 0.53, 1.00}), 1)
  )
end

local function getShowAnimation(self)
  if self.shouldFadeIn then
    return AHOpacityTo.new(self, 0.2, nil, 1)
  else
    return function() self:setOpacity(1) end
  end
end

local function getAnimateInAnimation(self)
  return AHSpawn.new(
    function() self:setScl(0.95) end,
    getShowAnimation(self),
    AHOpacityTo.new(self.label, 0.2, nil, 1),
    AHSequence.new(
      AHScaleTo.new(self, 0.2, AHEaseBezier.newFlat({0.6, 0.0, 0.4, 1.0}), 1.05),
      AHScaleTo.new(self, 0.2, AHEaseBezier.newFlat({0.2, 0.0, 0.5, 1.0}), 1)
  ))
end

local function getAnimateOutAnimation(self, shouldFadeOut)
  local animation = AHSpawn.new(
    AHOpacityTo.new(self.label, 0.2, nil, 0),
    AHScaleTo.new(self, 0.2, AHEaseBezier.newFlat({0.5, 0.0, 0.8, 1.0}), 0.95),
    AHDelayed.new(0.2, AHOpacityTo.new(self, 0.167, nil, 0))
  )
  if shouldFadeOut then
    animation:addAction(AHOpacityTo.new(self, 0.2, nil, 0))
    animation:addAction(AHOpacityTo.new(self.pressedSprite, 0.2, nil, 0))
  end
  return animation
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioPretestSelfRateButton:getAppearAnimation()
  return getAnimateInAnimation(self)
end

function OnboardioPretestSelfRateButton:getDisappearAnimation(shouldFadeOut)
  return getAnimateOutAnimation(self, shouldFadeOut)
end

-- Methods to support OnboardioTransitioner
function OnboardioPretestSelfRateButton:getWasPressed()
  return self.wasPressed
end

function OnboardioPretestSelfRateButton:getConceptView()
  return self.label
end

function OnboardioPretestSelfRateButton:playPressedColor()
  -- do nothing
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function setupButtonFunctionality(self, callback)
  local onTouchDown = function()
    self.pressedSprite:setOpacity(1)
    self.wasPressed = true
    Sound.playSoundEffect("button_press")
    wait(getOnTappedAnimation(self):start())
    callback()
  end
  Button.makeButton(self, onTouchDown)
end

local function addLabel(self, text)
  local label = Label.new(text, FONT)
  label:setColor(unpack(LABEL_COLOR))
  self:insertAtCenter(label)
  label:offsetY(4) -- visually center on surface of button
  return label
end

local function loadSprite(spriteName)
  local sprite = Sprite.newWithName(spriteName)
  return sprite
end

local function setupSprites(self)
  self:insertAtCenter(self.baseSprite)

  local pressedSprite = loadSprite(PRESSED_SPRITE_NAME)
  pressedSprite:setOpacity(0)
  self:insertAtCenter(pressedSprite)
  self.pressedSprite = pressedSprite

  self.baseSprite:inheritOpacity(self)
end

-- Constructor ------------------------

function OnboardioPretestSelfRateButton.new(text, callback, shouldFadeIn)
  local baseSprite = loadSprite(NORMAL_SPRITE_NAME)
  local self = Group.new(baseSprite:getEffectiveDims())
  Mixin.mix(self, OnboardioPretestSelfRateButton)

  self.shouldFadeIn = shouldFadeIn

  self.baseSprite = baseSprite
  self.wasPressed = false

  setupSprites(self)
  self.label = addLabel(self, text)
  setupButtonFunctionality(self, callback)

  self:setOpacity(0)
  self.label:setOpacity(0)

  return self
end
