--- OnboardioSkillAnimationArticulation
-- @author Created by Jay Austria  on April 6, 2017.

-- ============================================================================
class("OnboardioSkillAnimationArticulation")
-- ============================================================================

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = OnboardioCommon.COLOR_BACKGROUND

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function addFrontBubbleElement(self, element, fractionalX, fractionalY)
   self.bubbleFront:insertAtLocationFractional(element, fractionalX, fractionalY)
  table.insert(self.frontBubbleElements, element)
end

local function createMockText(width)
  local colorWhite = {1, 1, 1}
  local height = 8
  local cornerRadius = 2
  return FilledRoundedRectangle.new(width, height, cornerRadius, colorWhite)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioSkillAnimationArticulation:reset()
  local bubbleFrontScaleX, _ = self.bubbleFront:getScl()
  local expandedWidthFractional = bubbleFrontScaleX + (bubbleFrontScaleX * self.widthShiftPercentage)

  self.bubbleBack:setScl(0, self.scaleToFit)
  self.bubbleFront:setScl(expandedWidthFractional, self.scaleToFit)
  self.bubbleTail:setScl(0)
  self.bubbleCombined:setScl(0)
  self.overlay:setOpacity(1.0)

  for _, element in ipairs(self.frontBubbleElements) do
    element:setScl(0)
  end
end

function OnboardioSkillAnimationArticulation:playFadeAction(duration)
  local fadeAction = AHSequence.new(
    AHOpacityTo.new(self.overlay, duration, nil, 1.0),
    AHRemove.new(self)
  )
  return fadeAction:start()
end

function OnboardioSkillAnimationArticulation:getAnimationAction()
  local ease = AHEaseElasticOut.new(0.7)

  local staggeredSpawn = AHStaggeredSpawn.new(0.01,
    AHSpawn.new(
      AHScaleTo.new(self.bubbleCombined, 0.56, ease, 1.0),
      AHScaleTo.new(self.bubbleFront, 0.21, AHEaseSimpleInOut.new(), self.scaleToFit),
      AHScaleTo.new(self.bubbleBack, 0.21, AHEaseSimpleInOut.new(), self.scaleToFit),
      AHDelayed.new(0.39, AHScaleTo.new(self.bubbleTail, 0.56, ease, self.scaleToFit)),
      AHOpacityTo.new(self.overlay, 0.35, nil, 0.0)
    )
  )

  for _, element in ipairs(self.frontBubbleElements) do
    staggeredSpawn:addAction(AHDelayed.new(0.14, AHScaleTo.new(element, 0.56, ease, 1)))
  end

  return staggeredSpawn
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------



-- Constructor ------------------------

function OnboardioSkillAnimationArticulation.new(width)
  local widthShiftPercentage = 0.3

  local bubbleFront = Sprite.newWithName("articulation_front.png")
  bubbleFront:setAnchorPointBottomLeft()

  local fullWidth = bubbleFront:getWidth() * (1.0 + widthShiftPercentage)
  local scaleToFit = width / fullWidth
  bubbleFront:setScl(scaleToFit)

  local bubbleBack = Sprite.newWithName("articulation_back.png")
  bubbleBack:setAnchorPointBottomRight()
  bubbleBack:setScl(scaleToFit)

  local bubbleTail = Sprite.newWithName("articulation_tail.png")
  bubbleTail:setAnchorPointTop()
  bubbleTail:setScl(scaleToFit)

  local height = bubbleBack:getEffectiveHeight() + bubbleTail:getEffectiveHeight()
  local tailLocationYFractional = 1.0 - (bubbleBack:getEffectiveHeight() / height)

  local self = Group.new(width, height)
  Mixin.mix(self, OnboardioSkillAnimationArticulation)

  self.widthShiftPercentage = widthShiftPercentage
  self.scaleToFit = scaleToFit
  self.bubbleBack = bubbleBack
  self.bubbleFront = bubbleFront
  self.bubbleTail = bubbleTail

  self.frontBubbleElements = {}

  addFrontBubbleElement(self, Sprite.newWithName("articulation_quote.png"), 0.2, 0.6)
  addFrontBubbleElement(self, Sprite.newWithName("articulation_quote.png"), 0.38, 0.6)
  addFrontBubbleElement(self, createMockText(12), 0.56, 0.78)
  addFrontBubbleElement(self, createMockText(20), 0.75, 0.78)
  addFrontBubbleElement(self, createMockText(38), 0.67, 0.59)
  addFrontBubbleElement(self, createMockText(16), 0.58, 0.41)
  addFrontBubbleElement(self, createMockText(16), 0.76, 0.41)
  addFrontBubbleElement(self, createMockText(38), 0.67, 0.22)

  local bubbleCombined = Group.new(width, height)
  self.bubbleCombined = bubbleCombined
  self:insertAtCenter(bubbleCombined)

  self.bubbleCombined:insertAtLocationFractional(bubbleBack, 1.0, tailLocationYFractional)
  self.bubbleCombined:insertAtLocationFractional(bubbleTail, 0.4, tailLocationYFractional)
  self.bubbleCombined:insertAtLocationFractional(bubbleFront, 0.0, tailLocationYFractional)

  local paddedHeight = height * 1.2
  local overlay = FilledRectangle.new(Screen.width, paddedHeight, BACKGROUND_COLOR)
  self:insertAtCenter(overlay)
  self.overlay = overlay

  self:reset()

  return self
end
