--- OnboardioSkillAnimationListening
-- @author Created by Steven on April 11, 2017

class("OnboardioSkillAnimationListening")

require("graphics.CircleGradient")
require("graphics.MultiGradient")
require("actions.advanced.AHAxisOffsetShake")
require("OnboardioCircleGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEAL = {0.43, 0.76, 0.97}
local NAVY = {0.16, 0.28, 0.72}
local INVISIBLE_BLACK = {0.0, 0.0, 0.0, 0.0}
local SHADOW_COLOR = {0.38, 0.39, 0.49}
local WHITE = {1.0, 1.0, 1.0}
local GRAY = {0.77, 0.78, 0.84}
local RING_COLORS = {
  {1.0, 1.0, 1.0},
  {0.9, 0.9,	0.94}
}

local RING_COUNT = 4 -- NOTE: Excludes bullseye

local Z_INDICES = {
  SHADOW = 0,
  RINGS = 5,
  GLOW = 10,
  BULLSEYE = 15,
  ARROW = 20,
  OVERLAY = 25
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getArrowShakeAction(self)
  local arrow = self.arrow
  local angle = 45
  local amplitudeHigh = angle * 0.06
  local amplitudeMedium = angle * 0.03
  local amplitudeLow = angle * 0.014
  local amplitudeTiny = angle * 0.007
  local amplitudeSmallest = angle * 0.0035

  local speedMultiplier = 0.3
  local easeFactor = 2.0
  local fullRotationShake = AHSequence.new(
    AHDelay.new(0.05),
    AHRotationBy.new(arrow, 0.065 * speedMultiplier, AHEaseSimpleInOut.new(easeFactor), -amplitudeHigh),
    AHRotationBy.new(arrow, 0.060 * speedMultiplier, AHEaseSimpleIn.new(easeFactor), amplitudeHigh),
    AHRotationBy.new(arrow, 0.055 * speedMultiplier, AHEaseSimpleOut.new(easeFactor), amplitudeHigh),
    AHRotationBy.new(arrow, 0.050 * speedMultiplier, AHEaseSimpleIn.new(easeFactor), -amplitudeHigh),
    AHRotationBy.new(arrow, 0.045 * speedMultiplier, AHEaseSimpleOut.new(easeFactor), -amplitudeMedium),
    AHRotationBy.new(arrow, 0.040 * speedMultiplier, AHEaseSimpleIn.new(easeFactor), amplitudeMedium),
    AHRotationBy.new(arrow, 0.035 * speedMultiplier, AHEaseSimpleOut.new(easeFactor), amplitudeMedium),
    AHRotationBy.new(arrow, 0.030 * speedMultiplier, AHEaseSimpleIn.new(easeFactor), -amplitudeMedium),
    AHRotationBy.new(arrow, 0.080 * speedMultiplier, AHEaseSimpleOut.new(easeFactor), -amplitudeLow),
    AHRotationBy.new(arrow, 0.055 * speedMultiplier, AHEaseSimpleInOut.new(easeFactor), amplitudeLow),
    AHRotationBy.new(arrow, 0.080 * speedMultiplier, AHEaseSimpleOut.new(easeFactor), -amplitudeLow),
    AHRotationBy.new(arrow, 0.055 * speedMultiplier, AHEaseSimpleInOut.new(easeFactor), amplitudeLow),
    AHRotationBy.new(arrow, 0.080 * speedMultiplier, AHEaseSimpleOut.new(easeFactor), -amplitudeTiny),
    AHRotationBy.new(arrow, 0.055 * speedMultiplier, AHEaseSimpleInOut.new(easeFactor), amplitudeTiny),
    AHRotationBy.new(arrow, 0.080 * speedMultiplier, AHEaseSimpleOut.new(easeFactor), -amplitudeSmallest),
    AHRotationBy.new(arrow, 0.055 * speedMultiplier, AHEaseSimpleInOut.new(easeFactor), amplitudeSmallest)
  )

  return fullRotationShake
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioSkillAnimationListening:playFadeAction(duration)
  return AHSequence.new(
    AHOpacityTo.new(self.overlay, duration, nil, 1.0),
    AHRemove.new(self)
  ):start()
end

function OnboardioSkillAnimationListening:reset()
  self.arrow:setX(self.arrowOffscreenX)
  self.arrowShadow:setEndColor(unpack(INVISIBLE_BLACK))
end

function OnboardioSkillAnimationListening:getAnimationAction()
  local arrow = self.arrow
  local xLocation = self:getWidthPart(0.5)
  local ease = nil
  local arrowDuration = 0.1
  local arrowScale = 0.7
  local featherScale = 1.11
  local arrowMoveAction = AHSequence.new(
    AHLocationXTo.new(arrow, arrowDuration, ease, xLocation),
    AHSpawn.new(
      AHScaleXTo.new(arrow, arrowDuration, ease, arrowScale),
      AHScaleXTo.new(arrow.topFeather, arrowDuration, ease, featherScale),
      AHScaleXTo.new(arrow.bottomFeather, arrowDuration, ease, featherScale)
    )
  )

  local shadowDuration = 0.3
  local shadowAction = AHSpawn.new(
    self.arrowShadow:newOpacityToAction(shadowDuration, ease, 0.0, 0.3),
    AHLocationTo.new(self.arrowShadow, arrowDuration, ease, self:getWidthPart(0.5), self:getHeightPart(0.5))
  )

  local speedMultiplier = 0.3
  local shakeAction = AHAxisOffsetShake.new(self, -Screen.getWidthPart(0.2), AHAxisOffsetShake.SHAKE_AXIS.X, speedMultiplier)
  local arrowShakeAction = getArrowShakeAction(self)

  local fullAction = AHSpawn.new(
    arrowMoveAction,
    shadowAction,
    AHDelayed.new(
      arrowDuration,
      shakeAction,
      arrowShakeAction
    )
  )

  return fullAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function makeArrow(width, height)
  local isHorizontal = true
  local colors = { TEAL, TEAL, NAVY }
  local thicknesses = { 0.5, 0.5 }
  local arrowShaft = MultiGradient.new(width, height, isHorizontal, colors, thicknesses)
  local leftCircle = FilledCircle.new(height / 2, NAVY)
  local rightCircle = FilledCircle.new(height / 2, {0.17, 0.31, 0.73})
  leftCircle:setScl(0.8, 1.0)
  rightCircle:setScl(0.8, 1.0)
  arrowShaft:insertAtLeft(leftCircle)
  arrowShaft:insertAtRight(rightCircle)

  local topFeather = Sprite.newWithName("listening_arrow_feather.png")
  local bottomFeather = Sprite.newWithName("listening_arrow_feather.png")
  bottomFeather:setScl(1.0, -1.0)
  topFeather:setAnchorPointBottomLeft()
  bottomFeather:setAnchorPointBottomLeft()

  arrowShaft:insertAtLocationFractional(topFeather, 0.03, 1.0)
  arrowShaft:insertAtLocationFractional(bottomFeather, 0.03, 0.0)

  arrowShaft.topFeather = topFeather
  arrowShaft.bottomFeather = bottomFeather

  return arrowShaft
end

local function createLayout(self)
  local width = self:getWidth()

  local outerRadius = width / 2
  local segmentWidth = outerRadius / (RING_COUNT + 0.5) -- NOTE: 0.5 accounts for bullseye
  local bullseyeRadius = segmentWidth / 2

  local radius = outerRadius
  local radiusFactor = 1
  local radiusDecrement = segmentWidth

  local outlineColors = {
    WHITE,
    WHITE,
    GRAY,
    GRAY
  }
  local gradientOutline = OnboardioCircleGradient.new(outerRadius, outlineColors)
  gradientOutline:setRot(-45)
  self:insertAtCenter(gradientOutline, Z_INDICES.RINGS)

  for i = 1, RING_COUNT do
    local colorIndex = (i % 2) + 1
    local color = RING_COLORS[colorIndex]
    if (i == 1) then
      radiusFactor = 0.96
    else
      radiusFactor = 1.0
    end
    local ring = FilledCircle.new(radius * radiusFactor, color)
    self:insertAtCenter(ring, Z_INDICES.RINGS)
    radius = radius - radiusDecrement
  end

  local bullseyeOutline = FilledCircle.new(bullseyeRadius, NAVY)
  self:insertAtCenter(bullseyeOutline, Z_INDICES.BULLSEYE)
  local bullseye = FilledCircle.new(bullseyeRadius * 0.8, TEAL)
  bullseyeOutline:insertAtCenter(bullseye)

  local overlay = FilledRectangle.new(Screen.width, width)
  overlay:setOpacity(0.0)
  self:insertAtCenter(overlay, Z_INDICES.OVERLAY)
  self.overlay = overlay

  local shadowDepth = segmentWidth * 0.75
  local shadow = FilledCircle.new(outerRadius, SHADOW_COLOR)
  shadow:setLoc(width / 2 + shadowDepth, width / 2)
  local connector = FilledRectangle.new(shadowDepth, width, SHADOW_COLOR)
  connector:setLoc((width + shadowDepth) / 2, width / 2)
  self:insert(shadow, Z_INDICES.SHADOW)
  self:insert(connector, Z_INDICES.SHADOW)

  if (not OnboardioCommon.getIsDarkMode()) then
    local glowRadius = outerRadius
    local blurMargin = glowRadius
    local glow = CircleGradient.new(glowRadius, WHITE, {1.0, 1.0, 1.0, 0.8}, blurMargin)
    self:insertAtLocationFractional(glow, 1.1, 1.1, Z_INDICES.GLOW)
  end

  local isHorizontal = true
  local arrowShadow = RectangleGradient.new(outerRadius, bullseyeRadius / 2.5, INVISIBLE_BLACK, INVISIBLE_BLACK, isHorizontal)
  arrowShadow:setAnchorPointRight()
  arrowShadow:setRot(45)
  self:insert(arrowShadow, Z_INDICES.ARROW)
  self.arrowShadow = arrowShadow

  local arrowHeight = bullseyeRadius / 2
  local arrow = makeArrow(width, arrowHeight)
  arrow:setAnchorPointRight()
  local arrowOffscreenX = -(Screen.getWidthPart(0.75) - width / 2)
  arrow:setLoc(arrowOffscreenX, self:getHeightPart(0.5))
  self:insert(arrow, Z_INDICES.ARROW)
  self.arrow = arrow
  self.arrowOffscreenX = arrowOffscreenX
end

-- Constructor ------------------------

function OnboardioSkillAnimationListening.new(width)
  local self = Group.new(width, width)
  Mixin.mix(self, OnboardioSkillAnimationListening)

  createLayout(self)

  self:reset()

  return self
end
