--- OnboardioSkillAnimationWriting
-- @author Created by Joe Poka on April 5, 2017.

class("OnboardioSkillAnimationWriting")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = OnboardioCommon.COLOR_BACKGROUND

local Z_INDEX = {
  PEN = 10,
  INK_WELL = 20,
  OVERLAY = 30
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getAnimationPartOne(self, duration)
  local actions = {}
  local ease = AHEaseElasticOut.new(2.0)

  local inkWellDeltaAngleDegrees = 4.0
  local scaleInkWell = AHScaleTo.new(self.inkWell, duration, ease, 1.05)
  local rotateInkWell = AHRotationBy.new(self.inkWell, duration, ease, inkWellDeltaAngleDegrees)
  table.insert(actions, scaleInkWell)
  table.insert(actions, rotateInkWell)

  local inkWellCapDeltaY = self.inkWellCap:getHeightPart(0.5)
  local translateInkWellCap = AHLocationYBy.new(self.inkWellCap, duration, ease, inkWellCapDeltaY)
  local rotateInkWellCap = AHRotationTo.new(self.inkWellCap, duration, ease, 10.0)
  table.insert(actions, translateInkWellCap)
  table.insert(actions, rotateInkWellCap)

  local penDeltaY = self.inkWell:getHeightPart(0.5)
  local scalePenLeft = AHScaleTo.new(self.pen, duration, ease, 1.05)
  local delayedScalePenLeft = AHDelayed.new(duration * 0.2, scalePenLeft)
  local translatePenLeft = AHLocationYBy.new(self.pen, duration, ease, penDeltaY)
  table.insert(actions, delayedScalePenLeft)
  table.insert(actions, translatePenLeft)

  local scaleInkLeft = AHScaleYTo.new(self.inkLeft, duration, ease, 0.85)
  table.insert(actions, scaleInkLeft)

  local animation = AHSpawn.new(actions)
  return animation
end

local function getAnimationPartTwo(self, duration)
  local actions = {}
  local ease = AHEaseElasticOut.new(0.6)

  local inkWellDeltaAngleDegrees = -4.0
  local rotateInkWell = AHRotationBy.new(self.inkWell, duration, ease, inkWellDeltaAngleDegrees)
  local scaleInkWell = AHScaleTo.new(self.inkWell, duration, ease, 1.0)
  table.insert(actions, scaleInkWell)
  table.insert(actions, rotateInkWell)

  local inkWellCapDeltaY = -self.inkWellCap:getHeightPart(0.5)
  local translateInkWellCap = AHLocationYBy.new(self.inkWellCap, duration, ease, inkWellCapDeltaY)
  local rotateInkWellCap = AHRotationBy.new(self.inkWellCap, duration, AHEaseBounceOut.new(), -10.0)
  table.insert(actions, translateInkWellCap)
  table.insert(actions, rotateInkWellCap)

  local penDeltaY = -self.inkWell:getHeightPart(0.5)
  local scalePenLeft = AHScaleTo.new(self.pen, duration, ease, 1.0)
  local translatePenLeft = AHLocationYBy.new(self.pen, duration, AHEaseBounceOut.new(), penDeltaY)
  table.insert(actions, scalePenLeft)
  table.insert(actions, translatePenLeft)

  local scaleDownInkLeft = AHScaleYTo.new(self.inkLeft, duration, ease, 0.0)
  local scaleUpInkLeft = AHScaleYTo.new(self.inkLeft, duration * 1.0, ease, 0.15)
  local delayedScaleUpInkLeft = AHDelayed.new(duration * 0.5, scaleUpInkLeft)
  local scaleDownInkLeftAgain = AHScaleYTo.new(self.inkLeft, duration * 1.0, ease, 0.0)
  local delayedScaleDownInkLeftAgain = AHDelayed.new(duration * 0.675, scaleDownInkLeftAgain)
  local scaleInkLeft = AHSpawn.new(scaleDownInkLeft, delayedScaleUpInkLeft, delayedScaleDownInkLeftAgain)
  table.insert(actions, scaleInkLeft)

  local scaleUpInkRight = AHScaleYTo.new(self.inkRight, duration * 0.25, nil, 0.35)
  local scaleDownInkRight = AHScaleYTo.new(self.inkRight, duration * 0.25, nil, 0.0)
  local scaleInkRight = AHSequence.new(scaleUpInkRight, scaleDownInkRight)
  local delayedScaleInkRight = AHDelayed.new(duration * 0.0625, scaleInkRight)
  table.insert(actions, delayedScaleInkRight)

  local animation = AHSpawn.new(actions)
  return animation
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioSkillAnimationWriting:playFadeAction()
  local removeAnimation = AHSequence.new(
    AHOpacityTo.new(self.overlay, 0.2, nil, 1.0),
    AHRemove.new(self)
  )
  return removeAnimation:start()
end

function OnboardioSkillAnimationWriting:getAnimationAction()
  local durationPartOne = 0.325
  local durationPartTwo = 0.55

  local animationPartOne = getAnimationPartOne(self, durationPartOne)
  local animationPartTwo = getAnimationPartTwo(self, durationPartTwo)

  local partTwoDelay = durationPartOne * 0.525
  local delayedPartTwo = AHDelayed.new(partTwoDelay, animationPartTwo)

  local animation = AHSpawn.new(animationPartOne, delayedPartTwo)
  return animation
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  self.inkWell = Sprite.newWithName("writing_bottle_no_lid.png")
  self.inkWell:setScl(0.0)
  local x = self:getWidthPart(0.5)
  local y = self:getHeightPart(0.5)
  self.inkWell:setLoc(x, y)
  self:insertAtCenter(self.inkWell, Z_INDEX.INK_WELL)

  local inkColor = {0.176, 0.53, 0.85}
  local inkWidth = self.inkWell:getWidthPart(0.85)
  local inkHeight = self.inkWell:getHeightPart(0.4)
  local inkCornerRadius = inkHeight * 0.1
  local inkSharpCornersList = {
    FilledRoundedRectangle.CORNERS.TOP_LEFT,
    FilledRoundedRectangle.CORNERS.TOP_RIGHT
  }
  self.ink = FilledRoundedRectangle.new(inkWidth, inkHeight, inkCornerRadius, inkColor, inkSharpCornersList)
  x = self.inkWell:getWidthPart(0.5)
  y = self.ink:getHeightPart(0.675)
  self.ink:setLoc(x, y)
  self.inkWell:insert(self.ink)

  local inkSloshHeight = self.ink:getHeight()
  local p1x = 0.0
  local p1y = 0.0
  local p2x = inkWidth
  local p2y = 0.0
  local p3x = 0.0
  local p3y = inkSloshHeight
  self.inkLeft = FilledTriangle.new(p1x, p1y, p2x, p2y, p3x, p3y, inkColor)
  self.inkLeft:setAnchorPointBottom()
  y = self.ink:getY() + self.ink:getHeight() - inkSloshHeight * 0.5
  self.inkLeft:setLoc(x, y)
  self.inkLeft:setScl(1.0, 0.0)
  self.inkWell:insert(self.inkLeft)

  p2x = inkWidth
  p2y = 0.0
  p3x = inkWidth
  p3y = inkSloshHeight
  self.inkRight = FilledTriangle.new(p1x, p1y, p2x, p2y, p3x, p3y, inkColor)
  self.inkRight:setAnchorPointBottom()
  self.inkRight:setLoc(x, y)
  self.inkRight:setScl(1.0, 0.0)
  self.inkWell:insert(self.inkRight)

  self.ink:setBlendMode(MOAIProp2D.BLEND_MULTIPLY)
  self.inkLeft:setBlendMode(MOAIProp2D.BLEND_MULTIPLY)
  self.inkRight:setBlendMode(MOAIProp2D.BLEND_MULTIPLY)

  self.inkWellCap = Sprite.newWithName("writing_lid.png")
  x = self.inkWell:getWidthPart(0.5)
  y = self.inkWell:getHeight()
  self.inkWellCap:setLoc(x, y)
  self.inkWell:insert(self.inkWellCap)

  self.pen = Sprite.newWithName("writing_pen.png")
  self.pen:setScl(0.0)
  x = self.inkWell:getX() - self.pen:getWidthPart(0.065)
  y = self.inkWell:getY() - self.inkWell:getHeightPart(0.5) + self.pen:getHeightPart(0.53)
  self.pen:setLoc(x, y)
  self:insert(self.pen, Z_INDEX.PEN)

  self.overlay = FilledRectangle.new(self:getWidthPart(4.0), self:getHeightPart(1.75), BACKGROUND_COLOR)
  self.overlay:setOpacity(0.0)
  self:insertAtCenter(self.overlay, Z_INDEX.OVERLAY)
end

-- Constructor ------------------------

function OnboardioSkillAnimationWriting.new(width)
  local self = FilledRectangle.new(width, width * 0.5, BACKGROUND_COLOR)
  Mixin.mix(self, OnboardioSkillAnimationWriting)

  createLayout(self)

  local referenceWidth = 150.0
  local scale = width / referenceWidth
  self:setScl(scale)

  return self
end
