--- OnboardioStationaryTransition
-- @author Created by Aaron Barrett on November 10, 2014.

class("OnboardioStationaryTransition")
require("OnboardioViewTransitioner")
require("OnboardioPerspective")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Build Descriptors ------------------

local function createDescriptorForView(view)
  local y = view:getY()
  local perspectiveDistance = OnboardioPerspective.getPerspectiveDistanceForY(y)

  local wasPressed = ((view.getWasPressed ~= nil) and (view:getWasPressed()))

  local descriptor = {
    view = view,
    y = y,
    perspectiveDistance = perspectiveDistance,
    wasPressed = wasPressed
  }

  return descriptor
end

local function createDescriptorsForPanel(panel)
  if (panel == nil) then
    return {}
  end

  local descriptors = {}

  local fadableLabels = panel:getFadableLabels()
  for _, label in ipairs(fadableLabels) do
    local descriptor = createDescriptorForView(label)
    table.insert(descriptors, descriptor)
  end

  local stationaryButtons = panel:getStationaryButtons()
  for _, button in ipairs(stationaryButtons) do
    local descriptor = createDescriptorForView(button)
    table.insert(descriptors, descriptor)
  end

  return descriptors
end

-- Play Transition --------------------

local function playTransitionsIn(self)
  runInCoroutine(function()
    local newDescriptors = self.newDescriptors
    local newPanel = self.newPanel

    local baseDelayDuration = 0.15
    local delayDuration = baseDelayDuration
    if (newPanel ~= nil) then
      delayDuration = delayDuration + newPanel:getAppearanceDelayDuration()
    end

    waitFor(delayDuration)

    for _, descriptor in ipairs(newDescriptors) do
      local view = descriptor.view
      local perspectiveDistance = descriptor.perspectiveDistance

      OnboardioViewTransitioner.playStationaryTransitionInOnView(view, perspectiveDistance)
    end

  end)
end

local function playTransitionsOut(self)
  local oldDescriptors = self.oldDescriptors

  for _, descriptor in ipairs(oldDescriptors) do
    local view = descriptor.view
    local perspectiveDistance = descriptor.perspectiveDistance
    local wasPressed = descriptor.wasPressed

    OnboardioViewTransitioner.playStationaryTransitionOutOnView(view, perspectiveDistance, wasPressed)
  end
end



-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OnboardioStationaryTransition:buildDescriptors()
  local oldPanel = self.oldPanel
  local newPanel = self.newPanel

  local oldDescriptors = createDescriptorsForPanel(oldPanel)
  local newDescriptors = createDescriptorsForPanel(newPanel)

  self.oldDescriptors = oldDescriptors
  self.newDescriptors = newDescriptors
end

function OnboardioStationaryTransition:prepareToTransition()
  local newDescriptors = self.newDescriptors

  for _, descriptor in ipairs(newDescriptors) do
    local view = descriptor.view

    view:setOpacity(0.0)
    view:setScl(0.8)

    local y = descriptor.y
    local perspectiveDistance = descriptor.perspectiveDistance
    view:setY(y - perspectiveDistance)
  end
end

function OnboardioStationaryTransition:playTransition()
  playTransitionsOut(self)
  playTransitionsIn(self)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function OnboardioStationaryTransition.new(oldPanel, newPanel)
  local self = {}
  Mixin.mix(self, OnboardioStationaryTransition)

  self.oldPanel = oldPanel
  self.newPanel = newPanel

  return self
end
