--- OrbitConfiguration
-- @author Created by Alexey Chernikov

class("OrbitConfiguration")

require("data.keyframes.KeyframeSet")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

OrbitConfiguration.POINTS_PER_BONUS_ANSWER = 1000
OrbitConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 3.0
OrbitConfiguration.SPEED_BONUS_DECAY_START = 1.0

local INITIAL_LIVES_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {lives = 7}),
  Keyframe.new(0.2, {lives = 6})
)

local TOTAL_POINTS_FOR_BONUS_LIVES = 4000

local TOTAL_BASE_POINTS = 8000
local MAX_LIVES_LOST_RANK_TWO = 2
local MAX_LIVES_LOST_RANK_THREE = 0
local COMBINED_MAX_SPEED_BONUS_POINTS = 1000

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getExtraLivesBonusForLivesLost(numberOfLivesLost)
  local initialLivesCount = OrbitConfiguration.getInitialLivesCount()
  local bonusPointsPerLife = OrbitConfiguration.getPointsPerLifeBonus()
  local extraLivesRemaining = initialLivesCount - numberOfLivesLost - 1
  return bonusPointsPerLife * extraLivesRemaining
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OrbitConfiguration.getInitialLivesCount()
  local difficulty = GameSession.getDifficulty()
  local frameForDifficulty = INITIAL_LIVES_THRESHOLDS:getFrameAt(difficulty)
  return frameForDifficulty.lives
end

function OrbitConfiguration.getPointsPerLifeBonus()
  local maximumBonusLives = (OrbitConfiguration.getInitialLivesCount() - 1)
  return TOTAL_POINTS_FOR_BONUS_LIVES / maximumBonusLives
end

function OrbitConfiguration.getPointsPerFullSpeedBonus(round)
  local totalQuestions = #round.phrases + #round.bonus.phrases
  local pointsPerFullSpeedBonus = COMBINED_MAX_SPEED_BONUS_POINTS / totalQuestions

  return pointsPerFullSpeedBonus
end

function OrbitConfiguration.getScorePerAnswer(round)
  local totalQuestions = #round.phrases
  local bonusQuestions = #round.bonus.phrases

  local scoreForQuestions = TOTAL_BASE_POINTS
  local scorePerBonus = OrbitConfiguration.POINTS_PER_BONUS_ANSWER
  local scorePerAnswer = (scoreForQuestions - bonusQuestions * scorePerBonus)/totalQuestions
  return scorePerAnswer
end

function OrbitConfiguration.getRankTwoScore()
  local scoreForQuestions = TOTAL_BASE_POINTS
  local extraLivesBonus = getExtraLivesBonusForLivesLost(MAX_LIVES_LOST_RANK_TWO)
  return scoreForQuestions + extraLivesBonus
end

function OrbitConfiguration.getRankThreeScore()
  local scoreForQuestions = TOTAL_BASE_POINTS
  local extraLivesBonus = getExtraLivesBonusForLivesLost(MAX_LIVES_LOST_RANK_THREE)
  return scoreForQuestions + extraLivesBonus
end

function OrbitConfiguration.getDifficultyKeyFrames()
  local cutoffInfo = {
    {endDifficulty = 0.275, contentDifficulty = "Easy"},
    {endDifficulty = 0.5, contentDifficulty = "Medium"},
    {endDifficulty = 1.0, contentDifficulty = "Hard"}
  }
  local difficultyKeyFrames = DifficultyCurves.getHardCutoffKeyframes(cutoffInfo)
  return difficultyKeyFrames
end
