--- OrbitGame
-- @author Created by Alexey Chernikov

class("OrbitGame")

require("StandardUIBuilder")
require("OrbitColors")
require("OrbitConfiguration")
require("game.OrbitGameManager")
require("scene.OrbitScene")

------------------------------------------------------------------------------
-- API
------------------------------------------------------------------------------

-- gameFlow ---------------------------

function OrbitGame:gameFlow()
  self.gameManager:start()
end

------------------------------------------------------------------------------
-- Object Creation
------------------------------------------------------------------------------

local function createGameElements(self)
  self.gameManager = OrbitGameManager.new(self)
  local callbacks = self.gameManager:getCallbacks()

  local scene = OrbitScene.new(self:getUIOverlay(), callbacks)
  self:insertAtCenter(scene)
  self.scene = scene

  self.gameManager:setScene(self.scene)
end

local function createUI(self)
  local builder = StandardUIBuilder.new(self:getUIOverlay(), UI_COMPONENTS_COLOR)

  builder:addPauseButton()
  self.scoreCounter = builder:addScoreLabel()
  self.livesCounter = builder:addLivesDisplay(OrbitConfiguration.getInitialLivesCount())
  self.bonusTray = builder:addBonusTray()
end

-- Constructor ------------------------

function OrbitGame.new()
  local self = BaseGame.new()
  Mixin.mix(self, OrbitGame)

  createUI(self)
  createGameElements(self)

  return self
end
