--- RadioWaveEffect
-- @author Created by Alexey Chernikov

class("RadioWaveEffect")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WAVE_COLOR = {1, 1, 1, 1}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RadioWaveEffect:start(time, startRadius, endRadius, startThickness, endThickness)
  local startScale = startRadius / self.baseRadius
  local endScale = endRadius / self.baseRadius
  local duration = time / #self.waves
  local delayTime = 0.0
  local deltaDelayTime = 0.4

  for _, wave in ipairs(self.waves) do
    local updatedThickness = startThickness
    wave:setVisible(true)
    wave:setScl(startScale)
    wave:setLineWidth(updatedThickness)
    wave:setColor(unpack({1.0, 1.0, 1.0, 0.0}))

    local scaleDown = AHScaleTo.new(wave, duration, nil, endScale)
    local fadeIn = AHOpacityTo.new(wave, duration, nil, 1.0)

    local incrementThicknessCount = 60
    local deltaThickness = (endThickness - startThickness) / incrementThicknessCount
    local incrementThickness = function()
      updatedThickness = updatedThickness + deltaThickness
      wave:setLineWidth(updatedThickness)
    end
    local increaseThickness = AHLoop.new(incrementThickness, incrementThicknessCount)

    local showWaveAnimation = AHSpawn.new(scaleDown, fadeIn, increaseThickness)
    local cleanUp = function() wave:setVisible(false) end
    local showThenCleanUp = AHSequence.new(showWaveAnimation, cleanUp)

    local delayedShowWaveAnimation = AHDelayed.new(delayTime, showThenCleanUp)
    delayedShowWaveAnimation:start()

    delayTime = delayTime + deltaDelayTime
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, numberOfWaves, radius)
  local color = WAVE_COLOR
  local waves = {}
  for i = 1, numberOfWaves do
    local wave = HollowCircle.new(radius, color)
    self:insertAtCenter(wave)
    table.insert(waves, wave)
    wave:setVisible(false)
    wave:inheritColor(self)
  end

  self.waves = waves
end

-- Constructor ------------------------

function RadioWaveEffect.new(numberOfWaves)
  numberOfWaves = numberOfWaves or 3
  local radius = 32.0
  local diameter = radius * 2.0
  local self = Group.new(diameter, diameter)
  Mixin.mix(self, RadioWaveEffect)

  self.baseRadius = radius
  createLayout(self, numberOfWaves, radius)

  return self
end
