--- OrbitRoundGenerator
-- @author Created by Alexey Chernikov

class("OrbitRoundGenerator")

require("data.generators.UtilityGenerators")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getPhrasesByTopic(round)
  local phrasesByTopic = {}
  for _, topic in pairs(round.topics) do
    phrasesByTopic[topic] = {}
  end
  for _, phraseInfo in ipairs(round.phrases) do
    phrasesByTopic[phraseInfo.topic][phraseInfo.phrase] = true
  end

  return phrasesByTopic
end

local function shuffleTopicOrder(round)
  TableUtils.shuffleArray(round.topics)
end

local function makePhraseFromDescription(phraseDescription)
  local phrase = {}
  phrase.phrase = phraseDescription.text
  phrase.topic = phraseDescription.category

  if (phraseDescription.round == "phrases") then
    phrase.spawnTime = tonumber(phraseDescription.spawn_time)
  else
    phrase.spawnTime = 0
  end

  return phrase
end

local function getGameAndBonusPhrases(concept)
  local gamePhrases = {}
  local bonusPhrases = {}
  for _, phraseDescription in ipairs(concept.phrases) do
    local phrase = makePhraseFromDescription(phraseDescription)
    if (phraseDescription.round == "phrases") then
      table.insert(gamePhrases, phrase)
    elseif (phraseDescription.round == "bonus") then
      table.insert(bonusPhrases, phrase)
    end
  end
  return gamePhrases, bonusPhrases
end

local function getTopics(concept)
  local topics = {concept.topic_1, concept.topic_2, concept.topic_3}
  return topics
end

local function packageConceptIntoRound(concept)
  local round = {}

  round.bonus = {}
  round.phrases, round.bonus.phrases = getGameAndBonusPhrases(concept)
  round.topics = getTopics(concept)
  round.sound = concept.sound
  round.concept = concept
  round.id = concept.identifier
  round.phrasesByTopic = getPhrasesByTopic(round)

  return round
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function OrbitRoundGenerator.new()
  local conceptGenerator = GameSession.conceptGenerator
  local difficultyKeyFrames = OrbitConfiguration.getDifficultyKeyFrames()
  local difficultyNameChooser = KeyFrameInterpolator.new(difficultyKeyFrames, GameSession.getDifficulty())
  conceptGenerator:setDifficultyNameChooser(difficultyNameChooser)

  return TransformationGenerator.new(conceptGenerator, function(concept)
    local round = packageConceptIntoRound(concept)
    shuffleTopicOrder(round)
    return round
  end)
end
