--- LayoutInfo
-- @author Created by Alexey Chernikov

class("LayoutInfo")

require("OrbitColors")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PLANET_POINTS_6 = {
  -------- TOP ROW ------
  -- Left
  {
    location = {Screen.width/4 - 16, 3 * Screen.height/4 + 20},
    color = LAYER_COLORS_1[1],
    layerColors = LAYER_COLORS_1
  },
  -- Middle
  {
    location = {Screen.width/2, 3 * Screen.height/4 + 20},
    color = LAYER_COLORS_2[1],
    layerColors = LAYER_COLORS_2
  },
  -- Right
  {
    location = {3 * Screen.width/4 + 16, 3 * Screen.height/4 + 20},
    color = LAYER_COLORS_3[1],
    layerColors = LAYER_COLORS_3
  },
  --------- BOTTOM ROW -----
  -- Left
  {
    location = {Screen.width/4 - 16, Screen.height/4 - 20},
    color = LAYER_COLORS_3[1],
    layerColors = LAYER_COLORS_3
  },
  -- Middle
  {
    location = {Screen.width/2, Screen.height/4 - 20},
    color = LAYER_COLORS_2[1],
    layerColors = LAYER_COLORS_2
  },
  -- Right
  {
    location = {3 * Screen.width/4 + 16, Screen.height/4 - 20},
    color = LAYER_COLORS_1[1],
    layerColors = LAYER_COLORS_1
  },
}

local Y_OFFSET_FOR_3_PLANETS = -20
local X_OFFSET_FOR_3_PLANETS = 22
local PLANET_POINTS_3 = {
  -- Lower left
  {
    location = {Screen.width/4 + X_OFFSET_FOR_3_PLANETS, Screen.height/4 - 20 + Y_OFFSET_FOR_3_PLANETS},
    color = LAYER_COLORS_3[2],
    layerColors = LAYER_COLORS_3
  },
  -- Right
  {
    location = {Screen.width/4 * 3 - X_OFFSET_FOR_3_PLANETS, Screen.height/4 * 2 + Y_OFFSET_FOR_3_PLANETS + 4},
    color = LAYER_COLORS_2[3],
    layerColors = LAYER_COLORS_2
  },
  -- Upper left
  {
    location =  {Screen.width/4 + X_OFFSET_FOR_3_PLANETS, Screen.height / 4 * 3 + 28 + Y_OFFSET_FOR_3_PLANETS},
    color = LAYER_COLORS_1[3],
    layerColors = LAYER_COLORS_1
  }
}

local PLANET_POINTS_2 = {
  -- Bottom
  {
    location = {Screen.width/2 + 8, Screen.height/4 - 22},
    color = LAYER_COLORS_1[1],
    layerColors = LAYER_COLORS_1
  },
  -- Top
  {
    location =  {Screen.width/2 - 8, Screen.height / 4 * 3 + 22},
    color = LAYER_COLORS_2[1],
    layerColors = LAYER_COLORS_2
  }
}

local PLANET_POINTS = {
  {},
  PLANET_POINTS_2,
  PLANET_POINTS_3,
  {},
  {},
  PLANET_POINTS_6
}

local PLANET_RADIUS = 45
local DROP_RADIUS = 40
local FLOATING_PHRASE_RADIUS = 49

local SPAWN_RADIUS = FLOATING_PHRASE_RADIUS * 0.1
-- dx/dy between 2 spawn points in a cluster
local SPAWN_DISTANCE = FLOATING_PHRASE_RADIUS * math.sqrt(2) * 3/4

local SPAWN_POINTS_6 = {
  -- MIDDLE LEFT
  {
    {center = {Screen.width/4 - SPAWN_DISTANCE + 10, Screen.height/2 - SPAWN_DISTANCE + 25}, radius = SPAWN_RADIUS},
    {center = {Screen.width/4 - SPAWN_DISTANCE + 10, Screen.height/2 + SPAWN_DISTANCE + 26}, radius = SPAWN_RADIUS},
    {center = {Screen.width/4 + SPAWN_DISTANCE - 30, Screen.height/2 - SPAWN_DISTANCE - 44}, radius = SPAWN_RADIUS},
    {center = {Screen.width/4 + SPAWN_DISTANCE, Screen.height/2 + SPAWN_DISTANCE + 4}, radius = SPAWN_RADIUS},
  },
    -- MIDDLE RIGHT
  {
    {center = {3 * Screen.width/4 - SPAWN_DISTANCE - 30, Screen.height/2 - SPAWN_DISTANCE + 3}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - SPAWN_DISTANCE, Screen.height/2 + SPAWN_DISTANCE + 28}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 + SPAWN_DISTANCE, Screen.height/2 - SPAWN_DISTANCE - 20}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 + SPAWN_DISTANCE - 30, Screen.height/2 + SPAWN_DISTANCE - 20}, radius = SPAWN_RADIUS},
  },
}

local SPAWN_POINTS_3 = {
  -- TOP RIGHT CLUSTER
  {
    {center = {3 * Screen.width/4 - 20, 3 * Screen.height/4}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - 20 - SPAWN_DISTANCE, 3 * Screen.height/4 - SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - 20 - SPAWN_DISTANCE, 3 * Screen.height/4 + SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - 20 + SPAWN_DISTANCE, 3 * Screen.height/4 - SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - 20 + SPAWN_DISTANCE, 3 * Screen.height/4 + SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
  },

  -- MIDDLE LEFT CLUSTER
  {
    {center = {Screen.width/4, Screen.height/2}, radius = SPAWN_RADIUS},
    {center = {Screen.width/4 - SPAWN_DISTANCE, Screen.height/2 - SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {Screen.width/4 - SPAWN_DISTANCE, Screen.height/2 + SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {Screen.width/4 + SPAWN_DISTANCE, Screen.height/2 - SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {Screen.width/4 + SPAWN_DISTANCE, Screen.height/2 + SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
  },

  -- BOTTOM RIGHT CLUSTER
  {
    {center = {3 * Screen.width/4 - 10, Screen.height/4 - 30}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - 10 - SPAWN_DISTANCE, Screen.height/4 - 30 - SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - 10 - SPAWN_DISTANCE, Screen.height/4 - 30 + SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - 10 + SPAWN_DISTANCE, Screen.height/4 - 30 - SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
    {center = {3 * Screen.width/4 - 10 + SPAWN_DISTANCE, Screen.height/4 - 30 + SPAWN_DISTANCE}, radius = SPAWN_RADIUS},
  }
}

local SPAWN_POINTS_2 = SPAWN_POINTS_3

local SPAWN_POINTS = {
  {},
  SPAWN_POINTS_2,
  SPAWN_POINTS_3,
  {},
  {},
  SPAWN_POINTS_6
}

local BONUS_SPAWN_POINTS_3 = {
  -- TOP RIGHT CLUSTER
  {
    {center = {3 * Screen.width/4 + 20, 3 * Screen.height/4 + 20}, radius = SPAWN_RADIUS},
  },
  -- MIDDLE LEFT CLUSTER
  {
    {center = {Screen.width/4 - 20, Screen.height/2 - 20}, radius = SPAWN_RADIUS},
  },
  -- BOTTOM RIGHT CLUSTER
  {
    {center = {3 * Screen.width/4 + 20, Screen.height/4 - 50}, radius = SPAWN_RADIUS},
  }
}

local BONUS_SPAWN_POINTS = {
  {},
  SPAWN_POINTS_2,
  BONUS_SPAWN_POINTS_3,
  {},
  {},
  SPAWN_POINTS_6
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LayoutInfo.getLayoutForPlanetCount(numberOfPlanets)
  return PLANET_POINTS[numberOfPlanets]
end

function LayoutInfo.getSpawnPointsForPlanetCount(numberOfPlanets)
  return SPAWN_POINTS[numberOfPlanets]
end

function LayoutInfo.getBonusSpawnPointsForPlanetCount(numberOfPlanets)
  return BONUS_SPAWN_POINTS[numberOfPlanets]
end

function LayoutInfo.getFloaterRadius()
  return FLOATING_PHRASE_RADIUS + 2
end

function LayoutInfo.getDropRadius()
  return DROP_RADIUS
end

function LayoutInfo.getPlanetRadius()
  return PLANET_RADIUS
end
