--- FloatingObject
-- @author Created by Alexey Chernikov

class("FloatingObject")

require("scene.floaters.TextInColoredCircle")
require("actions.advanced.AHSmoothShake")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function cancelScale(self)
  if (self.scaleAnim) then
    self.scaleAnim:stop()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function FloatingObject:shake()
  local xShakeAmp, yShakeAmp = 10, 10
  local shakesPerSecond = 10
  local time = 0.5

  local shake = AHSmoothShake.new(self.offsetGroup, xShakeAmp, yShakeAmp, shakesPerSecond, time)
  shake:start()
end

function FloatingObject:select()
  cancelScale(self)
  self.selected = true

  self.scaleAnim = AHScaleTo.new(self, 0.2, nil, 1, 1)
  self.scaleAnim:start()
end

function FloatingObject:deselect()
  cancelScale(self)
  self.selected = false
  self.scaleAnim = AHScaleTo.new(self, 0.2, nil, 1, 1)
  self.scaleAnim:start()
end

function FloatingObject:setSelected(isSelected)
  if (isSelected) then
    self:select()
  else
    self:deselect()
  end
end

function FloatingObject:wobble()
  local wobbleTimer = MOAITimer.new()
  local xFrequency = math.random() * 1 + 0.5
  local yFrequency = math.random() * 1 + 0.5
  local xAmplitude = math.random() * 1 + 2
  local yAmplitude = math.random() * 1 + 2
  wobbleTimer:setSpan(math.huge)
  wobbleTimer:start()
  runInCoroutine(function()
    while(true) do
      if (self.dead) then
        return
      end

      local t = wobbleTimer:getTime()
      local dx = math.cos(xFrequency * t) * xAmplitude
      local dy = math.sin(yFrequency * t) * yAmplitude
      self.offsetGroup:setLoc(self.radius + dx, self.radius + dy)

      coroutine.yield()
    end
  end)
end

function FloatingObject:appear()
  self:setScl(0)
  AHScaleTo.new(self, 0.5, MOAIEaseType.EASE_IN, 1, 1):start()
end

function FloatingObject:removeSelf()
  self.super.removeSelf(self)
  self.dead = true
end

function FloatingObject:setMovementBounds(xMin, yMin, xMax, yMax)
  self.hasMovementBounds = true
  self.xMin, self.yMin, self.xMax, self.yMax = xMin, yMin, xMax, yMax
end

function FloatingObject:setLoc(x, y)
  if (self.hasMovementBounds) then
    x = math.max(x, self.xMin)
    x = math.min(x, self.xMax)
    y = math.max(y, self.yMin)
    y = math.min(y, self.yMax)
  end
  self.super.setLoc(self, x, y)
  self.view:updateGradient(x, y)
end

function FloatingObject:getRadius()
  return self.radius
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function FloatingObject.new(text, radius, colorReferencePoints, thickness)
  local diameter = radius * 2.0
  local self = Group.new(diameter, diameter)
  Mixin.mix(self, FloatingObject)

  local view = TextInColoredCircle.new(text, radius, colorReferencePoints, thickness)
  local offsetGroup = Group.new(diameter, diameter)

  offsetGroup:insertAtCenter(view)
  self:insertAtCenter(offsetGroup)

  self.view = view
  self.offsetGroup = offsetGroup

  self.radius = radius

  return self
end
