--- CircularText
-- @author Created by Alexey Chernikov

class("CircularText")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CircularText:getAngularSizeInRadians()
  return self.totalAngle
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, text, font, radius)
  local angle = 0.0

  for character in text:gmatch(".") do
    local characterView = Label.new(character, font)
    characterView:setAnchorPointLeft()
    local characterWidth = characterView:getWidth()
    self:insert(characterView)

    local x = radius + radius * math.cos(angle)
    local y = radius + radius * math.sin(angle)
    characterView:setRot(math.deg(angle) - 90.0)
    characterView:setLoc(x, y)

    if (character == " ") then
      characterWidth = font:getActualSize() / 2.0
    end

    local deltaAngle = math.pi / 10.0
    deltaAngle = (characterWidth + 2.0) / radius

    angle = angle - deltaAngle
    characterView:inheritColor(self)
  end

  self.totalAngle = math.abs(angle)
end

-- Constructor ------------------------

function CircularText.new(text, font, radius)
  local diameter = radius * 2.0
  local self = Group.new(diameter, diameter)
  Mixin.mix(self, CircularText)
  createLayout(self, text, font, radius)
  return self
end
