--- ParagraphsAnswerButton
-- @author created by Steven Mah on April 5, 2017

class("ParagraphsAnswerButton")

require("text.SizeConstrainedTextArea")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_BACKGROUND_TINT = {0.0, 0.0, 0.0, 0.24}
local CORRECT_BACKGROUND_COLOR = {0.08, 1.0, 0.61, 1.0}
local CORRECT_TEXT_COLOR = {0.05, 0.4, 0.24, 1.0}
local INCORRECT_BACKGROUND_COLOR = {0.96, 0.22, 0.46, 1.0}
local INCORRECT_TEXT_COLOR = {0.52, 0.05, 0.14, 1.0}

local ANSWER_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 17)
local CORNER_RADIUS = 3

local BUTTON_ACTION_DURATION = 0.01

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ParagraphsAnswerButton:createCorrectResponseAction(wasSelected)
  local fullAction = AHSequence.new(
    AHSpawn.new(
      AHColorTo.new(self, BUTTON_ACTION_DURATION, nil, CORRECT_BACKGROUND_COLOR),
      AHColorTo.new(self.answerTextArea, BUTTON_ACTION_DURATION, nil, CORRECT_TEXT_COLOR)
    )
  )

  if (wasSelected) then
    fullAction:addAction(
      AHSequence.new(
        AHScaleBy.new(self, BUTTON_ACTION_DURATION, AHEaseSineIn.new(), -0.05),
        AHScaleTo.new(self, BUTTON_ACTION_DURATION, AHEaseSineInOut.new(), 1.05),
        AHScaleTo.new(self, BUTTON_ACTION_DURATION, AHEaseSineInOut.new(), 1.0)
      )
    )
  end

  return fullAction
end

function ParagraphsAnswerButton:createIncorrectResponseAction()
  local colorAction = AHSpawn.new(
    AHColorTo.new(self, BUTTON_ACTION_DURATION, nil, INCORRECT_BACKGROUND_COLOR),
    AHColorTo.new(self.answerTextArea, BUTTON_ACTION_DURATION, nil, INCORRECT_TEXT_COLOR)
  )

  local bounceAction = AHSequence.new(
    AHScaleBy.new(self, BUTTON_ACTION_DURATION, AHEaseSineIn.new(), -0.05),
    AHScaleTo.new(self, BUTTON_ACTION_DURATION, AHEaseSineInOut.new(), 1.0)
  )

  local width = self:getWidth()

  local shakeAction = AHSequence.new(
    AHLocationXBy.new(self, BUTTON_ACTION_DURATION, nil, -width*0.01),
    AHLocationXBy.new(self, BUTTON_ACTION_DURATION, nil, width*0.02),
    AHLocationXBy.new(self, BUTTON_ACTION_DURATION, nil, -width*0.02),
    AHLocationXBy.new(self, BUTTON_ACTION_DURATION, nil, width*0.01)
  )

  return AHSequence.new(colorAction, bounceAction, shakeAction)
end

function ParagraphsAnswerButton:createFadeOutAction(duration)
  duration = duration or 0.1

  local fadeOutAction = AHSpawn.new(
    AHOpacityTo.new(self, duration, nil, 0.0),
    AHOpacityTo.new(self.answerTextArea, duration, nil, 0.0)
  )

  return fadeOutAction
end

function ParagraphsAnswerButton:createFadeInAction()
  local duration = 0.1

  local fadeInAction = AHSpawn.new(
    AHOpacityTo.new(self, duration, nil, 0.24),
    AHOpacityTo.new(self.answerTextArea, duration, nil, 1.0)
  )

  return fadeInAction
end

function ParagraphsAnswerButton:getText()
  return self.answerTextArea:getText()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ParagraphsAnswerButton.new(answer, width, height, font)
  local self = FilledRoundedRectangle.new(width, height, CORNER_RADIUS, BUTTON_BACKGROUND_TINT)
  Mixin.mix(self, ParagraphsAnswerButton)
  self:setOpacity(0.0)

  font = font or ANSWER_FONT
  local maxTextHeight = height
  local textWidth = width * 0.9
  local baseAttributes = { font = font }
  local answerText = SizeConstrainedTextArea.new(answer.text, textWidth, maxTextHeight, baseAttributes, nil, nil, Alignment.CENTER)
  answerText:setOpacity(0.0)
  self:insertAtCenter(answerText)
  self.answerTextArea = answerText

  self.isCorrect = answer.isCorrect

  return self
end
