--- ParagraphsBonusLayout
-- @author Created by Steven Mah on April 20, 2017

class("ParagraphsBonusLayout")

require("ParagraphsAnswerButton")
require("widgets.progress.TimeTimer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.getWidthPart(0.9)
local HEIGHT = Screen.height - ParagraphsCommon.QUESTION_MENU_HEIGHT

local UI_COLOR = {0.19, 0.76, 0.86}
local BLACK = {0.0, 0.0, 0.0}

local FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 18)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, bonusData)
  local centerX = self:getWidthPart(0.5)
  local padding = Screen.getHeightPart(0.01)
  local header = Label.new("BONUS ROUND", FONT)
  header:setAnchorPointLeft()
  header:setColor(unpack(BLACK))
  self:insertAtLocationFractional(header, 0, 0.85)
  self.header = header

  local lineSeparator = FilledRectangle.new(WIDTH, 1, UI_COLOR)
  lineSeparator:setLoc(centerX, header:getYBottom() - padding)
  self:insert(lineSeparator)

  local questionView = DynamicTextArea.newSimple(bonusData.question, WIDTH, FONT, Alignment.LEFT)
  questionView:setLoc(centerX, lineSeparator:getYBottom() - padding * 5)
  questionView:setColor(unpack(BLACK))
  self:insert(questionView)

  local buttonColumn = Column.new(padding)
  local answerButtonHeight = Screen.getHeightPart(0.15)
  local answers = {}
  for _, answerData in ipairs(bonusData.answers) do
    local answerButton = ParagraphsAnswerButton.new(answerData, WIDTH, answerButtonHeight, FONT)
    answerButton.answerTextArea:setOpacity(1.0)
    answerButton:setColor(unpack(UI_COLOR))
    buttonColumn:insert(answerButton)

    if (answerData.isCorrect) then
      self.correctAnswerButton = answerButton
    end

    local onResponseWrapper = function()
      for _, button in ipairs(self.answers) do
        button:setTouchEnabled(false)
      end
      self.onResponseCallback(answerButton, self.correctAnswerButton)
    end
    Button.makeButton(answerButton, onResponseWrapper)
    table.insert(answers, answerButton)
  end
  self.answers = answers

  local buttonColumnY = ((questionView:getYBottom() - padding + Screen.safeBottom) / 2)

  buttonColumn:alignChildren()
  buttonColumn:setLoc(centerX, buttonColumnY)
  self:insert(buttonColumn)
end

-- Constructor ------------------------

function ParagraphsBonusLayout.new(bonusData, onResponseCallback)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, ParagraphsBonusLayout)

  self.onResponseCallback = onResponseCallback

  createLayout(self, bonusData)

  return self
end
