--- ParagraphsConceptGenerator
-- Created by Steven on March 30, 2017

class("ParagraphsConceptGenerator")

require("data.generators.UtilityGenerators")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local QUESTION_TYPES = ParagraphsCommon.QUESTION_TYPES
local QUESTION_ORDER = ParagraphsCommon.QUESTION_ORDER

local LOCATION_A_TAG = "m1"
local LOCATION_B_TAG = "m2"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setLabelForDestination(answerText)
  if (answerText == LOCATION_A_TAG) then
    answerText = "Location A"
  elseif (answerText == LOCATION_B_TAG) then
    answerText = "Location B"
  else
    error("Invalid location tag.")
  end

  return answerText
end

local function setLabelForKeepAsIs(answerText)
  if ((answerText == "") or (answerText == nil)) then
    answerText = "KEEP AS IS"
  end

  return answerText
end

local function forceLocationAFirst(answers)
  -- Location A and Location B are the two labels specified for insertion type answers
  if (answers[2].text == LOCATION_A_TAG) then
    TableUtils.swap(answers)
  end
end

local function forKeepAsIsFirst(answers)
  if (answers[2].text == "KEEP AS IS") then
    TableUtils.swap(answers)
  end
end

local function packageQuestionType(questionType, concept)
  local correctAnswer = concept[questionType.."_correct"]
  local incorrectAnswer = concept[questionType.."_incorrect"]

  local round = {}
  round.type = questionType

  local answers = {}
  answers[1] = {text = correctAnswer, isCorrect = true}
  answers[2] = {text = incorrectAnswer, isCorrect = false}

  local isMoveType = (questionType == QUESTION_TYPES.MOVE)
  local hasEmptyAnswer = (correctAnswer == "") or (incorrectAnswer == "") or (correctAnswer == nil) or (incorrectAnswer == nil)
  if (isMoveType) then
    forceLocationAFirst(answers)
    answers[1].text = setLabelForDestination(answers[1].text)
    answers[2].text = setLabelForDestination(answers[2].text)
  elseif (hasEmptyAnswer) then
    answers[1].text = setLabelForKeepAsIs(answers[1].text)
    answers[2].text = setLabelForKeepAsIs(answers[2].text)
    forKeepAsIsFirst(answers)
  else
    TableUtils.shuffleArray(answers)
  end

  round.answers = answers

  return round
end

local function packageRound(concept)
  local rounds = {}
  for _, questionType in ipairs(QUESTION_ORDER) do
    local round = packageQuestionType(questionType, concept)
    if (round) then
      rounds[questionType] = round
    end
  end
  return rounds
end

local function packageParagraph(concept)
  local original = concept["paragraph"]
  -- TODO: Talk with content about changing these tags in the content sheets
  local parsedParagraph = string.gsub(original, "<$g$>", "<gi/>")

  return parsedParagraph
end

local function packageBonusQuestion(concept)
  local bonus = {}
  local answers = {}
  answers[1] = {text = concept["bonus_correct"], isCorrect = true}
  answers[2] = {text = concept["bonus_incorrect1"], isCorrect = false}
  answers[3] = {text = concept["bonus_incorrect2"], isCorrect = false}

  bonus.question = concept["bonus_prompt"]
  bonus.answers = answers

  TableUtils.shuffleArray(bonus.answers)

  return bonus
end

local function formatConcept(concept)
  local packagedConcept = {}
  packagedConcept.paragraph = packageParagraph(concept)
  packagedConcept.rounds = packageRound(concept)
  packagedConcept.bonus = packageBonusQuestion(concept)
  packagedConcept.id = concept.identifier

  return packagedConcept
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ParagraphsConceptGenerator.new()
  return TransformationGenerator.new(GameSession.conceptGenerator, function(concept)
    local packagedConcept = formatConcept(concept)
    return packagedConcept
  end)
end
