--- ParagraphsConfiguration
-- @author Created by Steven on March 24, 2017

class("ParagraphsConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ParagraphsConfiguration.QUESTIONS_COUNT = 3
ParagraphsConfiguration.POINTS_FOR_FULL_SPEED_BONUS = 100
ParagraphsConfiguration.NUMBER_OF_LIVES = 3

local DEFAULT_SECONDS_FOR_SPEED_BONUS = 8
local DEFAULT_DECAY_START_SECONDS = 0
local EXTRA_QUESTIONS_COUNT = 1

local MISTAKES_FOR_RANK_THREE = 0
local MISTAKES_FOR_RANK_TWO = 1

local CORRECT_ANSWER_SCORE_FOR_CURRENT_CORRECT_ANSWER_COUNT = {
  4000,
  3000,
  2000,
  1000
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getScoreWithoutSpeedBonusesForMistakeMade(mistakesMade)
  local totalQuestions = ParagraphsConfiguration.QUESTIONS_COUNT + EXTRA_QUESTIONS_COUNT
  local correctQuestionCount = totalQuestions - mistakesMade
  local totalScore = 0

  for i = 1, correctQuestionCount do
    totalScore = totalScore + ParagraphsConfiguration.getCorrectAnswerScore(i)
  end

  return totalScore
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ParagraphsConfiguration.getCorrectAnswerScore(correctAnswerCount)
  correctAnswerCount = MathUtils.clamp(correctAnswerCount, 1, #CORRECT_ANSWER_SCORE_FOR_CURRENT_CORRECT_ANSWER_COUNT)
  return CORRECT_ANSWER_SCORE_FOR_CURRENT_CORRECT_ANSWER_COUNT[correctAnswerCount]
end

function ParagraphsConfiguration.getSecondsForSpeedBonus(markupType)
  return DEFAULT_SECONDS_FOR_SPEED_BONUS
end

function ParagraphsConfiguration.getSpeedBonusDecayStart(markupType)
  return DEFAULT_DECAY_START_SECONDS
end

function ParagraphsConfiguration.getRankTwoScore()
  local pointsRankTwo = getScoreWithoutSpeedBonusesForMistakeMade(MISTAKES_FOR_RANK_TWO)

  return pointsRankTwo
end

function ParagraphsConfiguration.getRankThreeScore()
  local pointsRankThree = getScoreWithoutSpeedBonusesForMistakeMade(MISTAKES_FOR_RANK_THREE)

  return pointsRankThree
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
