--- ParagraphsQuestionLayout
-- @author created by Steven Mah on April 5, 2017

class("ParagraphsQuestionLayout")

require("ParagraphsAnswerButton")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HEADER_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 14)

local HEADER_TEXT = {
  [ParagraphsCommon.QUESTION_TYPES.REPLACE] = "REPLACE THE SENTENCE",
  [ParagraphsCommon.QUESTION_TYPES.COMBINE] = "COMBINE THE SENTENCES",
  [ParagraphsCommon.QUESTION_TYPES.MOVE] = "MOVE THE SENTENCE"
}

local HEADER_COLOR = {0.0, 0.0, 0.0}
local ACTIVE_HEADER_OPACITY = 0.62

local WIDTH = Screen.width

local HEADER_PADDING = 5
local ANSWER_SPACING = 8

local FADE_DURATION = 0.1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ParagraphsQuestionLayout:setTouchEnabled(isEnabled)
  for _, answerButton in ipairs(self.answers) do
    answerButton:setTouchEnabled(isEnabled)
  end
end

function ParagraphsQuestionLayout:createFadeOutAction()
  local fadeOutAction = AHSpawn.new(
    AHOpacityTo.new(self.header, FADE_DURATION, nil, 0.0)
  )

  for _, answerButton in ipairs(self.answers) do
    fadeOutAction:addAction(
      answerButton:createFadeOutAction(FADE_DURATION)
    )
  end

  return fadeOutAction
end

function ParagraphsQuestionLayout:createFadeInAction()
  local fadeOutAction = AHSpawn.new(
    AHOpacityTo.new(self.header, FADE_DURATION, nil, ACTIVE_HEADER_OPACITY)
  )

  for _, answerButton in ipairs(self.answers) do
    fadeOutAction:addAction(
      answerButton:createFadeInAction(FADE_DURATION)
    )
  end

  return fadeOutAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createHeader(self, question)
  local questionType = question.type
  local headerView = Label.new(HEADER_TEXT[questionType], HEADER_FONT)
  local headerIcon = Sprite.newWithName(questionType .. "_icon.png")

  local verticalPadding = Screen.getHeightPart(0.005)
  local iconPaddingX = 5

  local header = Row.new()
  header:insert(headerIcon)
  header:insert(headerView)
  header:alignChildren()

  header:setAnchorPointLeft()
  header:setLoc(HEADER_PADDING, self:getHeight() - header:getHeightPart(0.5) - HEADER_PADDING)
  local isRecursive = true
  header:linkColorAttrToChildren(isRecursive)
  header:setColor(unpack(HEADER_COLOR))
  header:setOpacity(0.0)
  self:insert(header)
  self.header = header
end

local function createAnswers(self, question, isMoveType)
  local answerGroup
  local answerWidth
  local answerHeight

  if (isMoveType) then
    answerWidth = Screen.getSafeWidthPart(0.47)
    answerHeight = Screen.getSafeHeightPart(0.07)
    answerGroup = Row.new(ANSWER_SPACING)
  else
    answerWidth = Screen.getSafeWidthPart(0.96)
    answerHeight = Screen.getSafeHeightPart(0.16)
    answerGroup = Column.new(ANSWER_SPACING / 2)
  end

  local answers = {}
  for _, answer in ipairs(question.answers) do
    local answerButton = ParagraphsAnswerButton.new(answer, answerWidth, answerHeight)
    answerGroup:insert(answerButton)

    if (answerButton.isCorrect) then
      self.correctAnswer = answerButton
    end

    local onResponseWrapper = function()
      for _, button in ipairs(self.answers) do
        button:setTouchEnabled(false)
      end
      self.onResponseCallback(answerButton)
    end

    Button.makeButton(answerButton, onResponseWrapper)
    answerButton:setTouchEnabled(false)

    table.insert(answers, answerButton)
  end

  answerGroup:alignChildren()

  local questionMenuTopY = (ParagraphsCommon.QUESTION_MENU_HEIGHT * 0.1) + Screen.safeBottom
  local headerBottomY = self.header:getYBottom()
  local centerY = (questionMenuTopY + headerBottomY) / 2
  answerGroup:setLoc(Screen.getWidthPart(0.5), centerY)

  self:insert(answerGroup)
  self.answerGroup = answerGroup
  self.answers = answers
end

-- Constructor ------------------------

function ParagraphsQuestionLayout.new(question, onResponseCallback)
  local isMoveType = (question.type == ParagraphsCommon.QUESTION_TYPES.MOVE)
  local height = ParagraphsCommon.ACTIVE_QUESTION_HEIGHT
  if (isMoveType) then
    height = ParagraphsCommon.ACTIVE_MOVE_QUESTION_HEIGHT
  end

  local self = Group.new(WIDTH, height)
  Mixin.mix(self, ParagraphsQuestionLayout)

  self.onResponseCallback = onResponseCallback

  createHeader(self, question)
  createAnswers(self, question, isMoveType)

  return self
end
