--- improving_paragraphs
-- @author Created by Anton Morant on April 16, 2014.
-- ConceptValidator for Paragraphs.

class("ParagraphsValidator")

require("validators.ConceptValidator")
require("validators.TagValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'paragraph',
  'replace_correct',
  'replace_incorrect',
  'combine_correct',
  'combine_incorrect',
  'move_correct',
  'move_incorrect',
  'bonus_prompt',
  'bonus_correct',
  'bonus_incorrect1',
  'bonus_incorrect2'
}

local REQUIRED_QUESTIONS = {
  'replace',
  'combine',
  'move'
}

local SINGLE_TAGS = { "m1", "m2" }
local PAIRED_TAGS = { "r", "m" }
local TERTIARY_TAGS = { "g" }

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function questionSupportsKeepOrApply(questionType)
  return (questionType == "replace") or (questionType == "combine")
end

local function validateAnswers(self, concept)
  for _, questionType in ipairs(REQUIRED_QUESTIONS) do
    local fieldCorrect = questionType .. "_correct"
    local fieldIncorrect = questionType .. "_incorrect"
    local textCorrect = concept[fieldCorrect]
    local textIncorrect = concept[fieldIncorrect]

    if questionSupportsKeepOrApply(questionType) then
      self:expectTrue(textCorrect ~= "" or textIncorrect ~= "", "Only one answer for " .. questionType .. " may be empty")
    else
      self:expectNotEmpty(textCorrect, fieldCorrect)
      self:expectNotEmpty(textIncorrect, fieldIncorrect)
    end

    self:expectNotEqual(textCorrect, textIncorrect, "Correct and incorrect answers cannot be the same.")
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ParagraphsValidator:validateConcept(concept)
  local allowEmpty = true
  self:validateRequiredKeys(concept, REQUIRED_KEYS, allowEmpty)

  self:validateTags(concept.paragraph, SINGLE_TAGS, PAIRED_TAGS, TERTIARY_TAGS)
  validateAnswers(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ParagraphsValidator.new()
  local self = ConceptValidator.new()
  TagValidator.make(self)
  Mixin.mix(self, ParagraphsValidator)

  return self
end
