--- PassagesAnswerButton
-- @author Created by Steven Mah on May 20, 2016.

class("PassagesAnswerButton")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_BACKGROUND_TINT = {0.0, 0.0, 0.0, 0.24}
local CORRECT_BACKGROUND_COLOR = {0.08, 1.0, 0.61, 1.0}
local CORRECT_TEXT_COLOR = {0.05, 0.4, 0.24, 1.0}
local INCORRECT_BACKGROUND_COLOR = {0.96, 0.22, 0.46, 1.0}
local INCORRECT_TEXT_COLOR = {0.52, 0.05, 0.14, 1.0}

local ANSWER_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 17)
local CORNER_RADIUS = 3

local BUTTON_ACTION_DURATION = 0.01

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PassagesAnswerButton:createCorrectResponseAction(wasSelected)
  local fullAction = AHSequence.new(
    AHSpawn.new(
      AHColorTo.new(self, BUTTON_ACTION_DURATION, nil, CORRECT_BACKGROUND_COLOR),
      AHColorTo.new(self.answerTextArea, BUTTON_ACTION_DURATION, nil, CORRECT_TEXT_COLOR)
    )
  )

  if (wasSelected) then
    fullAction:addAction(
      AHSequence.new(
        AHScaleBy.new(self, BUTTON_ACTION_DURATION, AHEaseSineIn.new(), -0.05),
        AHScaleTo.new(self, BUTTON_ACTION_DURATION, AHEaseSineInOut.new(), 1.05),
        AHScaleTo.new(self, BUTTON_ACTION_DURATION, AHEaseSineInOut.new(), 1.0)
      )
    )
  end

  return fullAction
end

function PassagesAnswerButton:createIncorrectResponseAction()
  local colorAction = AHSpawn.new(
    AHColorTo.new(self, BUTTON_ACTION_DURATION, nil, INCORRECT_BACKGROUND_COLOR),
    AHColorTo.new(self.answerTextArea, BUTTON_ACTION_DURATION, nil, INCORRECT_TEXT_COLOR)
  )

  local bounceAction = AHSequence.new(
    AHScaleBy.new(self, BUTTON_ACTION_DURATION, AHEaseSineIn.new(), -0.05),
    AHScaleTo.new(self, BUTTON_ACTION_DURATION, AHEaseSineInOut.new(), 1.0)
  )

  local width = self:getWidth()

  local shakeAction = AHSequence.new(
    AHLocationXBy.new(self, BUTTON_ACTION_DURATION, nil, -width*0.01),
    AHLocationXBy.new(self, BUTTON_ACTION_DURATION, nil, width*0.02),
    AHLocationXBy.new(self, BUTTON_ACTION_DURATION, nil, -width*0.02),
    AHLocationXBy.new(self, BUTTON_ACTION_DURATION, nil, width*0.01)
  )

  return AHSequence.new(colorAction, bounceAction, shakeAction)
end

function PassagesAnswerButton:createFadeOutAction(duration)
  duration = duration or 0.1

  local fadeOutAction = AHSpawn.new(
    AHOpacityTo.new(self, duration, nil, 0.0),
    AHOpacityTo.new(self.answerTextArea, duration, nil, 0.0)
  )

  return fadeOutAction
end

function PassagesAnswerButton:createFadeInAction()
  local duration = 0.1

  local fadeInAction = AHSpawn.new(
    AHOpacityTo.new(self, duration, nil, 0.24),
    AHOpacityTo.new(self.answerTextArea, duration, nil, 1.0)
  )

  return fadeInAction
end

function PassagesAnswerButton:getText()
  return self.answerTextArea:getText()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function PassagesAnswerButton.new(answer, width, height)
  local self = FilledRoundedRectangle.new(width, height, CORNER_RADIUS, BUTTON_BACKGROUND_TINT)
  Mixin.mix(self, PassagesAnswerButton)
  self:setOpacity(0.0)

  -- Needed because some answers are numbers
  local answerAsString = tostring(answer.text)
  local answerText = DynamicTextArea.newSimple(answerAsString, width * 0.8, ANSWER_FONT, Alignment.CENTER)
  answerText:setOpacity(0.0)
  self:insertAtCenter(answerText)
  self.answerTextArea = answerText

  self.isCorrect = answer.isCorrect

  return self
end
