--- PassagesQuestionsMenu
-- @author Created by Steven Mah on May 26, 2015.

class("PassagesConfiguration")

require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

PassagesConfiguration.NUMBER_OF_QUESTIONS = 3
PassagesConfiguration.SPEED_BONUS_DECAY_START = 0.0
PassagesConfiguration.BASE_SCORE_FOR_CORRECT_ANSWER = 2600
PassagesConfiguration.POINTS_FOR_FULL_SPEED_BONUS = 1000

local NUMBER_OF_LIVES_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {lives = 3}),
  Keyframe.new(0.4, {lives = 2})
)
local FULL_EXTRA_LIVES_POINT_BONUS = 1000

local LOST_LIVES_ALLOWED_FOR_RANK_TWO = 1
local LOST_LIVES_ALLOWED_FOR_RANK_THREE = 0

local MINIMUM_QUESTION_TIME = 11
local MIDDLE_QUESTION_TIME = 24
local MAXIMUM_QUESTION_TIME = 27

local SPEED_BONUS_DURATION_FRACTION = 0.5

local TIMER_DIFFICULTY_THRESHOLD = 0.3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getBaseScore()
  local totalQuestions = PassagesConfiguration.NUMBER_OF_QUESTIONS
  local baseScorePerQuestion = PassagesConfiguration.BASE_SCORE_FOR_CORRECT_ANSWER
  local baseScore = totalQuestions * baseScorePerQuestion

  return baseScore
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PassagesConfiguration.getNumberOfLives()
  local difficulty = GameSession.getDifficulty()
  local thresholdFrame = NUMBER_OF_LIVES_THRESHOLDS:getFrameAt(difficulty)
  return thresholdFrame.lives
end

function PassagesConfiguration.getPointsPerLifeBonus()
  local maxNumberOfBonusLives = PassagesConfiguration.getNumberOfLives() - 1
  return FULL_EXTRA_LIVES_POINT_BONUS / maxNumberOfBonusLives
end

-- This is public for TestPassagesConfiguration.lua
function PassagesConfiguration.getTimePerQuestion(difficulty)
  -- invert the difficulty, cause we want the max to be at the easiest level
  local timeLowerBound, timeUpperBound
  local difficultyLowerBound, difficultyUpperBound

  -- We're not using a straight interpolation/mapping here because of high failure rates
  -- at the lower difficulties before this change
  if (difficulty <= TIMER_DIFFICULTY_THRESHOLD) then
    difficultyLowerBound = 0
    difficultyUpperBound = TIMER_DIFFICULTY_THRESHOLD
    timeLowerBound = MIDDLE_QUESTION_TIME
    timeUpperBound = MAXIMUM_QUESTION_TIME
  else
    difficultyLowerBound = TIMER_DIFFICULTY_THRESHOLD
    difficultyUpperBound = 0.75
    timeLowerBound = MINIMUM_QUESTION_TIME
    timeUpperBound = MIDDLE_QUESTION_TIME
  end
  local questionTime = MathUtils.interpolateLinearWithClamp(difficulty,
                                                            difficultyLowerBound,
                                                            difficultyUpperBound,
                                                            timeUpperBound,
                                                            timeLowerBound)

  return questionTime
end

function PassagesConfiguration.getSpeedBonusWindow()
  local difficulty = GameSession.getDifficulty()
  local timePerQuestion = PassagesConfiguration.getTimePerQuestion(difficulty)
  return timePerQuestion * SPEED_BONUS_DURATION_FRACTION
end


function PassagesConfiguration.getRankTwoScore()
  local correctAnswersCount = PassagesConfiguration.NUMBER_OF_QUESTIONS - LOST_LIVES_ALLOWED_FOR_RANK_TWO
  local scoreFromCorrectAnswers = correctAnswersCount * PassagesConfiguration.BASE_SCORE_FOR_CORRECT_ANSWER
  local scoreFromExtraLives = FULL_EXTRA_LIVES_POINT_BONUS - (PassagesConfiguration.getPointsPerLifeBonus() * LOST_LIVES_ALLOWED_FOR_RANK_TWO)
  return scoreFromExtraLives + scoreFromCorrectAnswers
end

function PassagesConfiguration.getRankThreeScore()
  local correctAnswersCount = PassagesConfiguration.NUMBER_OF_QUESTIONS - LOST_LIVES_ALLOWED_FOR_RANK_THREE
  local scoreFromCorrectAnswers = correctAnswersCount * PassagesConfiguration.BASE_SCORE_FOR_CORRECT_ANSWER
  local scoreFromExtraLives = FULL_EXTRA_LIVES_POINT_BONUS - (PassagesConfiguration.getPointsPerLifeBonus() * LOST_LIVES_ALLOWED_FOR_RANK_THREE)
  return scoreFromExtraLives + scoreFromCorrectAnswers
end
