--- PassagesIntroInterstitial
-- @author Created by Jay Austria on December 23, 2015.

class("PassagesIntroInterstitial")

require("PassagesConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {0.13, 0.14, 0.16}
local INTRO_IMAGE_FRAME_COLOR = {0.26, 0.27, 0.33}
local FRAME_HEIGHT = 12
local IMAGE_MARGIN_TOP = Screen.getHeightPart(0.1)
local DESCRIPTION_WIDTH = Screen.getWidthPart(0.8)
local MARGIN_Y_BOTTOM_AREA_PROPORTION = 0.11

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function isScreenThin()
  return not Screen.isTabletAspectRatio()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, introDescriptor)
  local previewImage = AssetHelper.createPhotoSpriteForConcept(introDescriptor.concept, introDescriptor.concept.illustration)
  local stretchFitScale = Screen.fullWidth / previewImage:getWidth()
  previewImage:setScl(stretchFitScale)

  local frame = FilledRectangle.new(Screen.fullWidth, previewImage:getEffectiveHeight() + FRAME_HEIGHT, INTRO_IMAGE_FRAME_COLOR)

  local marginY = 0
  if isScreenThin() then
    marginY = IMAGE_MARGIN_TOP
  end
  local centerX = Screen.getWidthPart(0.5)
  local frameY = Screen.fullHeight - previewImage:getEffectiveHeightPart(0.5) - marginY
  self:insertAtLocation(frame, centerX, frameY)

  frame:insertAtCenter(previewImage)

  local bottomAreaHeight = frame:getYBottom()
  local marginY = bottomAreaHeight * MARGIN_Y_BOTTOM_AREA_PROPORTION

  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  tapToContinueLabel:setOpacity(0.6)
  tapToContinueLabel:setAnchorPointBottom()
  self:insertAtLocation(tapToContinueLabel, centerX, math.max(marginY, Screen.safeBottom))

  local descriptionYTop = frame:getYBottom() - marginY
  local descriptionYBottom = tapToContinueLabel:getYTop() + marginY
  local descriptionMaxHeight = descriptionYTop - descriptionYBottom

  local baseAttributes = { font = Font.new(FontFaces.DIN_OT_LIGHT, 22)}
  local descriptionLabel = SizeConstrainedTextArea.new(introDescriptor.summary,
                                                       DESCRIPTION_WIDTH,
                                                       descriptionMaxHeight,
                                                       baseAttributes)
  descriptionLabel:setAnchorPointTop()
  self:insertAtLocation(descriptionLabel, centerX, descriptionYTop)
end

function PassagesIntroInterstitial.new(introDescriptor)
  local self = FilledRectangle.newFullScreen(BACKGROUND_COLOR)
  Mixin.mix(self, PassagesIntroInterstitial)

  createLayout(self, introDescriptor)

  return self
end
