--- PassagesQuestionTab
-- @author Created by Steven Mah on May 20, 2015.

class("PassagesQuestionTab")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INACTIVE_HEIGHT = PassagesCommon.QUESTION_MENU_HEIGHT

-- Fudge this, rather than moving to align with asymmetric safe area
local ACTIVE_WIDTH = Screen.fullWidth * 1.5
local ACTIVE_HEIGHT = PassagesCommon.ACTIVE_QUESTION_HEIGHT

local MENU_BUTTON_STYLES_BY_QUESTION_TYPE = PassagesCommon.MENU_BUTTON_STYLES_BY_QUESTION_TYPE

local INCORRECT_BACKGROUND_COLOR = {0.96, 0.22, 0.46}
local INCORRECT_ICON_COLOR = {0.52, 0.05, 0.14, 1.0}
local CORRECT_BACKGROUND_COLOR = {0.17, 0.88, 0.55}
local CORRECT_ICON_COLOR = {0.05, 0.4, 0.24, 1.0}

local TAB_ACTION_COMPONENT_DURATION = 0.1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createMoveAndScaleDownAction(self)
  local xAnchorPoint, _ = self:getAnchorPoint()

  local fullAction = AHSequence.new(
    AHSpawn.new(
      AHLocationYBy.new(self, TAB_ACTION_COMPONENT_DURATION * 2, AHEaseSineOut.new(), self:getHeightPart(0.5)),
      AHScaleTo.new(self, TAB_ACTION_COMPONENT_DURATION * 2, nil, 1.0, 1.0)
    ),
    function() self:transformAnchorPoint(xAnchorPoint, 0.0) end,
    AHLocationYTo.new(self, TAB_ACTION_COMPONENT_DURATION, AHEaseSineInOut.new(), self:getHeightPart(-0.25)),
    AHLocationYTo.new(self, TAB_ACTION_COMPONENT_DURATION, AHEaseSineIn.new(), 0.0)
  )

  return fullAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PassagesQuestionTab:createMoveAndScaleUpAction()
  local xAnchorPoint, _ = self:getAnchorPoint()
  self:transformAnchorPoint(xAnchorPoint, 1.0)

  local destinationHeight = ACTIVE_HEIGHT

  if (self.isTwoAnswers) then
    destinationHeight = PassagesCommon.ACTIVE_TRUE_FALSE_HEIGHT
  end

  local width, height = self:getDims()
  local xScale = ACTIVE_WIDTH / width
  local yScale = destinationHeight / height

  local fullAction = AHSequence.new(
    AHLocationYBy.new(self, TAB_ACTION_COMPONENT_DURATION, AHEaseSineOut.new(), self:getHeightPart(-0.25)),
    AHSpawn.new(
      AHOpacityTo.new(self.icon, TAB_ACTION_COMPONENT_DURATION * 2, nil, 0.0),
      AHLocationYTo.new(self, TAB_ACTION_COMPONENT_DURATION * 2, AHEaseSineInOut.new(), destinationHeight * 1.1)
    ),
    AHSpawn.new(
      AHLocationYTo.new(self, TAB_ACTION_COMPONENT_DURATION * 2, AHEaseSineInOut.new(), destinationHeight),
      AHScaleTo.new(self, TAB_ACTION_COMPONENT_DURATION * 2, nil, xScale, yScale)
    )
  )

  return fullAction
end

function PassagesQuestionTab:createCorrectResponseAction()
  local correctResponseAction = AHSequence.new(
    createMoveAndScaleDownAction(self),
    function() self.icon = self.checkIcon end,
    AHSpawn.new(
      AHColorTo.new(self, TAB_ACTION_COMPONENT_DURATION, nil, CORRECT_BACKGROUND_COLOR),
      AHOpacityTo.new(self.checkIcon, TAB_ACTION_COMPONENT_DURATION, nil, 1.0)
    )
  )

  return correctResponseAction
end

function PassagesQuestionTab:createIncorrectResponseAction()
  local incorrectResponseAction = AHSequence.new(
    createMoveAndScaleDownAction(self),
    function() self.icon = self.crossIcon end,
    AHSpawn.new(
      AHColorTo.new(self, TAB_ACTION_COMPONENT_DURATION, nil, INCORRECT_BACKGROUND_COLOR),
      AHOpacityTo.new(self.crossIcon, TAB_ACTION_COMPONENT_DURATION, nil, 1.0)
    )
  )

  return incorrectResponseAction
end

function PassagesQuestionTab:createOutroAction()
  local styleData = MENU_BUTTON_STYLES_BY_QUESTION_TYPE[self.questionType]
  local iconColor = {0.0, 0.0, 0.0, 0.2}

  local fullOutroAction = AHSequence.new(
    AHSpawn.new(
      AHColorTo.new(self, TAB_ACTION_COMPONENT_DURATION, nil, styleData.backgroundColor),
      AHColorTo.new(self.icon, TAB_ACTION_COMPONENT_DURATION, nil, iconColor),
      AHSequence.new(
        AHLocationYBy.new(self, 0.2, AHEaseQuadraticOut.new(), self:getHeightPart(0.2)),
        AHLocationYBy.new(self, 0.2, AHEaseQuadraticIn.new(), -self:getHeightPart(0.2))
      )
    )
  )

  return fullOutroAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, styleData)
  local icon = Sprite.newWithName(styleData.iconName)
  icon:setLoc(self.iconX, self.iconY)
  self:insert(icon)
  self.icon = icon

  local crossIcon = Sprite.newWithName("cross.png")
  crossIcon:setColor(unpack(INCORRECT_ICON_COLOR))
  crossIcon:setOpacity(0.0)
  crossIcon:setLoc(self.iconX, self.iconY)
  self:insert(crossIcon)
  self.crossIcon = crossIcon

  local checkIcon = Sprite.newWithName("checkmark.png")
  checkIcon:setColor(unpack(CORRECT_ICON_COLOR))
  checkIcon:setOpacity(0.0)
  checkIcon:setLoc(self.iconX, self.iconY)
  self:insert(checkIcon)
  self.checkIcon = checkIcon
end

-- Constructor ------------------------

function PassagesQuestionTab.new(question, width, iconX, iconY)
  local styleData = MENU_BUTTON_STYLES_BY_QUESTION_TYPE[question.type]
  assert(styleData, "Style Data is undefined for question type")

  local self = FilledRectangle.new(width, INACTIVE_HEIGHT, styleData.fillColor)
  Mixin.mix(self, PassagesQuestionTab)

  self.hasBeenActivated = false
  self.questionType = question.type
  self.isTwoAnswers = (#question.answers == 2)
  self.iconX = iconX
  self.iconY = iconY

  createLayout(self, styleData)

  return self
end
