class("PassagesRoundGenerator")

require("data.generators.UtilityGenerators")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local QUESTION_TYPES =
{
  "overview",
  "inference",
  "technique",
  "details",
  "vocab"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function isValidString(text)
  return ((text ~= nil) and (text ~= ""))
end

local function isAnswerTrue(answer)
  return (answer.text:lower() == "true")
end

local function isAnswerFalse(answer)
  return (answer.text:lower() == "false")
end

local function answersAreTrueFalse(answers)
  return (#answers == 2) and ((isAnswerTrue(answers[1]) and isAnswerFalse(answers[2]))
                              or (isAnswerFalse(answers[1]) and isAnswerTrue(answers[2])))
end

local function sortAnswers(answers)
  if answersAreTrueFalse(answers) then
    if isAnswerFalse(answers[1]) then
      TableUtils.swap(answers)
    end
  else
    TableUtils.shuffleArray(answers)
  end
end

local function packageQuestionType(type, concept)
  local questionText = concept[type.."_question"]
  if not isValidString(questionText) then return nil end

  local question = {}
  question.type = type
  question.text = questionText

  local answers = {}
  answers[1] = {text = concept[type.."_correct"], isCorrect = true}
  answers[2] = {text = concept[type.."_incorrect_1"], isCorrect = false}
  if isValidString(concept[type.."_incorrect_2"]) then
    answers[3] = {text = concept[type.."_incorrect_2"], isCorrect = false}
  end

  sortAnswers(answers)

  question.answers = answers

  question.rationale = concept[type.."_rationale"]

  return question
end

local function packageIntro(concept)
  local intro = {}
  intro.concept = concept
  intro.summary = concept["excerpt"]

  return intro
end

local function packagePassage(concept)
  return concept["passage"]
end

local function packageQuestions(concept, numberOfQuestions)
  local questions = {}
  for _, questionType in pairs(QUESTION_TYPES) do
    local question = packageQuestionType(questionType, concept)
    if (question) then
      table.insert(questions, question)
    end
  end

  while #questions > numberOfQuestions do
    table.remove(questions, math.random(#questions))
  end

  return questions
end

local function packageConceptIntoRound(concept, numberOfQuestions)
  local round = {}

  round.intro = packageIntro(concept)
  round.passage = packagePassage(concept)
  round.questions = packageQuestions(concept, numberOfQuestions)
  round.id = concept.identifier

  return round
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function PassagesRoundGenerator.new(numberOfQuestions)
  return TransformationGenerator.new(GameSession.conceptGenerator, function(concept)
    return packageConceptIntoRound(concept, numberOfQuestions)
  end)
end
