--- PercentagesChatRationaleHelper.lua

class("PercentagesChatRationaleHelper")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local OPTION_IDS = {
  ANOTHER_EXAMPLE = "id_another_example",
  ALTERNATIVE_EXPLANTION = "id_alternative_explanation",
}

local INTRO_MESSAGE_OPENER_FORMATTED = "Here's how to calculate <b>%d%% of %d</b> in more detail"
local INTRO_MESSAGE_ENDER = "Is there anything else you’d like to know?"

local ANOTHER_EXAMPLE_USER_MESSAGE_FORMATTED = "Show me another example of <b>%d%%</b>."
local ANOTHER_EXAMPLE_MESSAGE_OPENER_FORMATTED = "Here's how to calculate <b>%d%% of %d</b>"

local ALTERNATIVE_EXPLANTION_USER_MESSAGE_FORMATTED = "What's another way to calculate <b>%d%%</b>?"

PercentagesChatRationaleHelper.messageCreatorsByTipType = {}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Intro steps messages -------------------------------------------------------

local function createStepsMessageTipAdd(value, roundIndex, tipKey, percentages, expectedAnswers)
  assert(roundIndex == 3, "Tried to create a tip for adding before round three.")

  local percentageRound1 = percentages[1]
  local percentageRound2 = percentages[2]
  local percentageRound3 = percentages[3]
  local answer1 = expectedAnswers[1]
  local answer2 = expectedAnswers[2]
  local answer3 = expectedAnswers[3]

  return string.format("First, find %d%% of %d:<br/><b>%d → %d</b>", percentageRound1, value, value, answer1) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Next, find %d%% of %d:<br/><b>%d → %d</b>", percentageRound2, value, value, answer2) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finally, add both numbers together to find %d%%:<br/><b>%d + %d → %d</b>", percentageRound3, answer1, answer2, answer3)
end

local function createStepsMessageTipSubtract(value, roundIndex, tipKey, percentages, expectedAnswers)
  assert(roundIndex == 3, "Tried to create a tip for subtracting before round three.")

  local percentageMin = math.min(percentages[1], percentages[2])
  local percentageMax = math.max(percentages[1], percentages[2])
  local percentageRound3 = percentages[3]
  local answerMin = math.min(expectedAnswers[1], expectedAnswers[2])
  local answerMax = math.max(expectedAnswers[1], expectedAnswers[2])
  local expectedAnswer = expectedAnswers[3]

  return string.format("First, find %d%% of %d:<br/><b>%d → %d</b>", percentageMax, value, value, answerMax) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Next, find %d%% of %d:<br/><b>%d → %d</b>", percentageMin, value, value, answerMin) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finally, find the difference to calculate %d%%:<br/><b>%d - %d → %d</b>", percentageRound3, answerMax, answerMin, expectedAnswer)
end

local function createStepsMessageTipMultiply4(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value1Percent = MathUtils.round(value/100)
  local value4Percent = value1Percent * 4
  local value10Percent = MathUtils.round(value/10)
  local percentage = percentages[roundIndex]
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 10%% of %d:<br/><b>%d → %d</b>", value, value, value10Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Next, find 1%% of %d:<br/><b>%d → %d</b>", value, value, value1Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply this number by four to find 4%%:<br/><b>%d × 4 → %d</b>", value1Percent, value4Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finally, add the results of 10%% and 4%% together to find %d%%:<br/><b>%d + %d → %d</b>", percentage, value10Percent, value4Percent, expectedAnswer)
end

local function createStepsMessageTipMultiply3(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value1Percent = MathUtils.round(value/100)
  local value3Percent = value1Percent * 3
  local value10Percent = MathUtils.round(value/10)
  local percentage = percentages[roundIndex]
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 10%% of %d:<br/><b>%d → %d</b>", value, value, value10Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Next, find 1%% of %d:<br/><b>%d → %d</b>", value, value, value1Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply this number by three to find 3%%:<br/><b>%d × 3 → %d</b>", value1Percent, value3Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finally, add the results of 10%% and 3%% together to find %d%%:<br/><b>%d + %d → %d</b>", percentage, value10Percent, value3Percent, expectedAnswer)
end

local function createStepsMessageTipFind1Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local valueAfterMovingDecimal = MathUtils.round(value/100)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, rewrite the number with a decimal point to the right of the ones place:<br/><b>%d → %d.</b>", value, value) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Next, move the decimal point two digits to the left:<br/><b>%.d. → %.2f</b>", value, valueAfterMovingDecimal) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finally, drop the decimal point and any zeros after it:<br/><b>%.2f → %d</b>", valueAfterMovingDecimal, expectedAnswer)
end

local function createStepsMessageTipFind2Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value1PercentUnrounded = value/100
  local value1Percent = string.format("%d", MathUtils.round(value1PercentUnrounded))

  -- A couple of our concepts go fractional when finding 1%. Handle them as a special case to avoid rounding
  -- to an integer.
  if (not MathUtils.isInteger(value1PercentUnrounded)) then
    value1Percent = string.format("%.1f", value1PercentUnrounded)
  end

  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 1%% of %d:<br/><b>%d → %s</b>", value, value, value1Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply by two to find 2%%:<br/><b>%s × 2 → %d</b>", value1Percent, expectedAnswer)
end

local function createStepsMessageTipFind2PercentAlt(value, roundIndex, tipKey, percentages, expectedAnswers)
  -- Core rationale seems to be practically the same so just use the same step breakdown
  return createStepsMessageTipFind2Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
end

local function createStepsMessageTipFind3Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value1Percent = MathUtils.round(value/100)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 1%% of %d:<br/><b>%d → %d</b>", value, value, value1Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply by three to find 3%%:<br/><b>%d × 3 → %d</b>", value1Percent, expectedAnswer)
end

local function createStepsMessageTipFind3PercentAlt(value, roundIndex, tipKey, percentages, expectedAnswers)
  -- Core rationale seems to be practically the same so just use the same step breakdown
  return createStepsMessageTipFind3Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
end

local function createStepsMessageTipFind4Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value1Percent = MathUtils.round(value/100)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 1%% of %d:<br/><b>%d → %d</b>", value, value, value1Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply by four to find 4%%:<br/><b>%d × 4 → %d</b>", value1Percent, expectedAnswer)
end

local function createStepsMessageTipFind4PercentAlt(value, roundIndex, tipKey, percentages, expectedAnswers)
  -- Core rationale seems to be practically the same so just use the same step breakdown
  return createStepsMessageTipFind4Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
end

local function createStepsMessageTipFind5Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value10Percent = MathUtils.round(value/10)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 10%% of %d:<br/><b>%d → %d</b>", value, value, value10Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Divide this number in half to find 5%%:<br/><b>%d ÷ 2 → %d</b>", value10Percent, expectedAnswer)
end

local function createStepsMessageTipFind10Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local valueAfterMovingDecimal = MathUtils.round(value/10)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, rewrite the number with a decimal point to the right of the ones place:<br/><b>%d → %d.</b>", value, value) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Next, move the decimal point one digit to the left:<br/><b>%.d. → %.1f</b>", value, valueAfterMovingDecimal) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finally, drop the decimal point and any zeros after it:<br/><b>%.1f → %d</b>", valueAfterMovingDecimal, expectedAnswer)
end

local function createStepsMessageTipFind20Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value10Percent = MathUtils.round(value/10)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 10%% of %d:<br/><b>%d → %d</b>", value, value, value10Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply by two to find 20%%:<br/><b>%d × 2 → %d</b>", value10Percent, expectedAnswer)
end

local function createStepsMessageTipFind20PercentAlt(value, roundIndex, tipKey, percentages, expectedAnswers)
  -- Core rationale seems to be practically the same so just use the same step breakdown
  return createStepsMessageTipFind20Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
end

local function createStepsMessageTipFind25Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Divide this number by four:<br/><b>%d ÷ 4 → %d</b>", value, expectedAnswer)
end

local function createStepsMessageTipFind25PercentAlt(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value50Percent = MathUtils.round(value/2)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 50%% of %d:<br/><b>%d → %d</b>", value, value, value50Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Then, find 50%% of %d to find your answer:<br/><b>%d → %d</b>", value50Percent, value50Percent, expectedAnswer)
end

local function createStepsMessageTipFind40Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value10Percent = MathUtils.round(value/10)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 10%% of %d:<br/><b>%d → %d</b>", value, value, value10Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply by four to find 40%%:<br/><b>%d × 4 → %d</b>", value10Percent, expectedAnswer)
end

local function createStepsMessageTipFind50Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Divide this number by two:<br/><b>%d ÷ 2 → %d</b>", value, expectedAnswer)
end

local function createStepsMessageTipFind60Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value10Percent = MathUtils.round(value/10)
  local value50Percent = MathUtils.round(value/2)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 10%% of %d:<br/><b>%d → %d</b>", value, value, value10Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Then, find 50%% of %d:<br/><b>%d → %d</b>", value, value, value50Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finally, add both numbers together to find 60%%:<br/><b>%d + %d → %d</b>", value10Percent, value50Percent, expectedAnswer)
end

local function createStepsMessageTipFind90Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value10Percent = MathUtils.round(value/10)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("First, find 10%% of %d:<br/><b>%d → %d</b>", value, value, value10Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Then, subtract this from %d to find 90%%:<br/><b>%d - %d → %d</b>", value, value, value10Percent, expectedAnswer)
end

local function createIntro(value, roundIndex, tipKey, percentages, expectedAnswers)
  local stepsMessageFunction = PercentagesChatRationaleHelper.messageCreatorsByTipType[tipKey]
  assert(stepsMessageFunction ~= nil, string.format("No step message creator function found for tip key %s", tipKey))

  local percentage = percentages[roundIndex]
  local message = string.format(INTRO_MESSAGE_OPENER_FORMATTED, percentage, value) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
                  stepsMessageFunction(value, roundIndex, tipKey, percentages, expectedAnswers) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
                  INTRO_MESSAGE_ENDER

  return message
end

-- Show another example messages ----------------------------------------------

local function createAnotherExampleOption(anotherValue, roundIndex, tipKey, percentages, anotherExpectedAnswers)
  local stepsMessageFunction = PercentagesChatRationaleHelper.messageCreatorsByTipType[tipKey]
  assert(stepsMessageFunction ~= nil, string.format("No step message creator function found for tip key %s", tipKey))

  local percentage = percentages[roundIndex]

  local userMessage = string.format(ANOTHER_EXAMPLE_USER_MESSAGE_FORMATTED, percentage)

  local systemMessage = string.format(ANOTHER_EXAMPLE_MESSAGE_OPENER_FORMATTED, percentage, anotherValue) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
                        stepsMessageFunction(anotherValue, roundIndex, tipKey, percentages, anotherExpectedAnswers)
  return {
    id = OPTION_IDS.ANOTHER_EXAMPLE,
    userMessage = userMessage,
    systemMessage = systemMessage
  }
end

-- Alternative explanation messages -------------------------------------------
local function createAlternativeExplanationMessageTipFind1Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate 1%% of any number that ends with two or more zeros is to drop the last two zeros.") .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Drop the last two zeros to find your answer:<br/><b>%d → %d</b>", value, expectedAnswer)
end

local function createAlternativeExplanationMessageTipFind5Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value1Percent = MathUtils.round(value/100)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate 5%% of a number is to find 1%% and multiply that by five.") .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("First, find 1%% of %d:<br/><b>%d → %d</b>", value, value, value1Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply by five to find 5%%:<br/><b>%d × 5 → %d</b>", value1Percent, expectedAnswer)
end

local function createAlternativeExplanationMessageTipFind10Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate 10%% of a number that ends with one or more zeros is to drop the last zero.") .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Drop the last zero to find your answer:<br/><b>%d → %d</b>", value, expectedAnswer)
end

local function createAlternativeExplanationMessageTipFind30Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value10Percent = MathUtils.round(value/10)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate 30%% is to find 10%% and triple it.") .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("First, find 10%% of %d:<br/><b>%d → %d</b>", value, value, value10Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply by three to find 30%%:<br/><b>%d × 3 → %d</b>", value10Percent, expectedAnswer)
end

local function createAlternativeExplanationMessageTipFind75Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value25Percent = MathUtils.round(value/4)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate 75%% is to find 25%% and subtract it from the original value.") .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("First, find 25%% of %d:<br/><b>%d → %d</b>", value, value, value25Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Then, subtract this from %d to find 75%%:<br/><b>%d - %d → %d</b>", value, value, value25Percent, expectedAnswer)
end

local function createAlternativeExplanationMessageTipFind95Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local value1Percent = MathUtils.round(value/100)
  local value5Percent = value1Percent * 5
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate 95%% is to find 5%% and subtract it from the original value.") .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("First, find 1%% of %d:<br/><b>%d → %d</b>", value, value, value1Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Multiply by five to find 5%%:<br/><b>%d × 5 → %d</b>", value1Percent, value5Percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Then, subtract this from %d to find 95%%:<br/><b>%d - %d → %d</b>", value, value, value5Percent, expectedAnswer)
end

local function createAlternativeExplanationMessageTipFindPercentOf100(value, roundIndex, tipKey, percentages, expectedAnswers)
  local percent = percentages[roundIndex]
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate %d%% of 100 is to remember that percentages represent proportions out of 100.", percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("When finding any percent out of 100, the percentage and the answer will always be the same:<br/><b>%d%% → %d</b>", percent, expectedAnswer)
end

local function createAlternativeExplanationMessageTipYOfX10Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local percent = percentages[roundIndex]
  local percentAfterMovingDecimal = MathUtils.round(percent/10)
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate %d%% of 10 is to flip the values.", percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finding %d%% of 10 is equivalent to finding 10%% of %d", percent, percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("To find 10%% of %d, rewrite the number with a decimal point to the right of the ones place:<br/><b>%d → %d.</b>", percent, percent, percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Then, move the decimal point one digit to the left:<br/><b>%d. → %.1f</b>", percent, percentAfterMovingDecimal) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finally, drop the decimal point and any zeros after it:<br/><b>%.1f → %d</b>", percentAfterMovingDecimal, expectedAnswer)
end

local function createAlternativeExplanationMessageTipYOfX50Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local percent = percentages[roundIndex]
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate %d%% of 50 is to flip the values.", percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finding %d%% of 50 is equivalent to finding 50%% of %d", percent, percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("To find 50%% of %d, divide it by two:<br/><b>%d ÷ 2 → %d</b>", percent, percent, expectedAnswer)
end

local function createAlternativeExplanationMessageTipYOfX200Percent(value, roundIndex, tipKey, percentages, expectedAnswers)
  local percent = percentages[roundIndex]
  local expectedAnswer = expectedAnswers[roundIndex]

  return string.format("Another way to calculate %d%% of 200 is to flip the values.", percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("Finding %d%% of 200 is equivalent to finding 200%% of %d", percent, percent) .. ChatConversationUtils.MESSAGE_SPLIT_DELIMITER ..
         string.format("To find 200%% of %d, multiply it by two:<br/><b>%d × 2 → %d</b>", percent, percent, expectedAnswer)
end

local function createAlternativeExplanationOption(value, roundIndex, tipKey, percentages, expectedAnswers)
  local percentage = percentages[roundIndex]

  local userMessage = string.format(ALTERNATIVE_EXPLANTION_USER_MESSAGE_FORMATTED, percentage)

  local alternativeExplanationFunction = PercentagesChatRationaleHelper.messageCreatorsByTipType[tipKey]
  assert(alternativeExplanationFunction ~= nil, string.format("No step message creator function found for tip key %s", tipKey))

  local systemMessage = alternativeExplanationFunction(value, roundIndex, tipKey, percentages, expectedAnswers)

  return {
    id = OPTION_IDS.ALTERNATIVE_EXPLANTION,
    userMessage = userMessage,
    systemMessage = systemMessage,
  }
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PercentagesChatRationaleHelper.createConversation(value, anotherValue, roundIndex, tipKey, alternativeTipKeys, percentages, expectedAnswers, anotherExpectedAnswers)
  assert(PercentagesChatRationaleHelper.messageCreatorsByTipType ~= nil,
        "PercentagesChatRationaleHelper.initialize() must be called prior to using")

  local conversation = {}

  conversation.intro = createIntro(value, roundIndex, tipKey, percentages, expectedAnswers)

  conversation.options = {}
  table.insert(conversation.options, createAnotherExampleOption(anotherValue, roundIndex, tipKey, percentages, anotherExpectedAnswers))

  local alternativeTipKey = alternativeTipKeys[roundIndex]
  local hasAlternativeExplanation = ((alternativeTipKey ~= nil) and (string.len(alternativeTipKey) > 0))
  if (hasAlternativeExplanation) then
    table.insert(conversation.options, createAlternativeExplanationOption(value, roundIndex, alternativeTipKey, percentages, expectedAnswers))
  end

  return conversation
end

function PercentagesChatRationaleHelper.initialize()
  ChatRationaleAnalytics.setSkill("percentages")

  PercentagesChatRationaleHelper.messageCreatorsByTipType = {

    -- Intro steps message creator functions
    ["TIP_ADD"] = createStepsMessageTipAdd,
    ["TIP_SUBTRACT"] = createStepsMessageTipSubtract,
    ["TIP_MULTIPLY_4"] = createStepsMessageTipMultiply4,
    ["TIP_MULTIPLY_3"] = createStepsMessageTipMultiply3,
    ["TIP_FIND_1_PERCENT"] = createStepsMessageTipFind1Percent,
    ["TIP_FIND_2_PERCENT"] = createStepsMessageTipFind2Percent,
    ["TIP_FIND_2_PERCENT_ALT"] = createStepsMessageTipFind2PercentAlt,
    ["TIP_FIND_3_PERCENT"] = createStepsMessageTipFind3Percent,
    ["TIP_FIND_3_PERCENT_ALT"] = createStepsMessageTipFind3PercentAlt,
    ["TIP_FIND_4_PERCENT"] = createStepsMessageTipFind4Percent,
    ["TIP_FIND_4_PERCENT_ALT"] = createStepsMessageTipFind4PercentAlt,
    ["TIP_FIND_5_PERCENT"] = createStepsMessageTipFind5Percent,
    ["TIP_FIND_10_PERCENT"] = createStepsMessageTipFind10Percent,
    ["TIP_FIND_20_PERCENT"] = createStepsMessageTipFind20Percent,
    ["TIP_FIND_20_PERCENT_ALT"] = createStepsMessageTipFind20PercentAlt,
    ["TIP_FIND_25_PERCENT"] = createStepsMessageTipFind25Percent,
    ["TIP_FIND_25_PERCENT_ALT"] = createStepsMessageTipFind25PercentAlt,
    ["TIP_FIND_40_PERCENT"] = createStepsMessageTipFind40Percent,
    ["TIP_FIND_50_PERCENT"] = createStepsMessageTipFind50Percent,
    ["TIP_FIND_60_PERCENT"] = createStepsMessageTipFind60Percent,
    ["TIP_FIND_90_PERCENT"] = createStepsMessageTipFind90Percent,

    -- alternative explanation message creator functions
    ["EXPANDED_TIP_FIND_1_PERCENT"] = createAlternativeExplanationMessageTipFind1Percent,
    ["EXPANDED_TIP_FIND_5_PERCENT"] = createAlternativeExplanationMessageTipFind5Percent,
    ["EXPANDED_TIP_FIND_10_PERCENT"] = createAlternativeExplanationMessageTipFind10Percent,
    ["EXPANDED_TIP_FIND_30_PERCENT"] = createAlternativeExplanationMessageTipFind30Percent,
    ["EXPANDED_TIP_FIND_75_PERCENT"] = createAlternativeExplanationMessageTipFind75Percent,
    ["EXPANDED_TIP_FIND_95_PERCENT"] = createAlternativeExplanationMessageTipFind95Percent,
    ["EXPANDED_TIP_FIND_PERCENT_OF_100"] = createAlternativeExplanationMessageTipFindPercentOf100,
    ["EXPANDED_TIP_Y_OF_X_10_PERCENT"] = createAlternativeExplanationMessageTipYOfX10Percent,
    ["EXPANDED_TIP_Y_OF_X_50_PERCENT"] = createAlternativeExplanationMessageTipYOfX50Percent,
    ["EXPANDED_TIP_Y_OF_X_200_PERCENT"] = createAlternativeExplanationMessageTipYOfX200Percent
  }
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
