--- PercentagesConceptGenerator
-- @author Created by Jay Austria on May 5, 2015.

class("PercentagesConceptGenerator")

require("data.generators.UtilityGenerators")
require("PercentagesChatRationaleHelper")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local KEYS_TO_FORMAT_WHITELIST = {
  TIP_ADD = true,
  TIP_SUBTRACT = true
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function formatTipText(text, percentages)
  local firstNum = percentages[3]
  -- Fill in the smaller number first, so the order works with addition and subtraction
  local secondNum = math.min(percentages[1], percentages[2])
  local thirdNum = math.max(percentages[1], percentages[2])
  return string.format(text, firstNum, secondNum, thirdNum)
end

local function tipKeysToText(keys, percentages)
  local tips = {}

  for _, key in ipairs(keys) do
    local text = Localizer.get(key)
    if KEYS_TO_FORMAT_WHITELIST[key] then
      text = formatTipText(text, percentages)
    end
    table.insert(tips, text)
  end

  return tips
end

local function tipKeysToConversations(value, anotherValue, tipKeys, alternativeTipKeys, percentages, expectedAnswers, anotherExpectedAnswers)
  local conversations = {}
  for roundIndex, tipKey in ipairs(tipKeys) do
    local conversation = PercentagesChatRationaleHelper.createConversation(value, anotherValue, roundIndex, tipKey, alternativeTipKeys, percentages, expectedAnswers, anotherExpectedAnswers)
    table.insert(conversations, conversation)
  end

  return conversations
end

local function packageConceptIntoRound(concept)
  local value = concept.value
  local anotherValue = concept.anotherValue
  local expectedAnswers = {}
  local anotherExpectedAnswers = {}
  local percentages = {}
  local tipKeys = {}
  local alternativeTipKeys = {}

  local questionsCount = PercentagesConfiguration.QUESTIONS_COUNT_PER_SET
  for i = 1, questionsCount do
     --gotta round due to float imprecision
    local expectedAnswer = MathUtils.round(value * (concept["percent" .. i] / 100.0))
    local anotherExpectedAnswer =  MathUtils.round(anotherValue * (concept["percent" .. i] / 100.0))

    local percentage = concept["percent" .. i]
    local tipKey = concept["tip" .. i]
    local alternativeTipKey = concept["alternativeTip" .. i] or ""

    table.insert(expectedAnswers, expectedAnswer)
    table.insert(anotherExpectedAnswers, anotherExpectedAnswer)
    table.insert(percentages, percentage)
    table.insert(tipKeys, tipKey)
    table.insert(alternativeTipKeys, alternativeTipKey)
  end

  local tips = tipKeysToText(tipKeys, percentages)
  local conversations = tipKeysToConversations(value, anotherValue, tipKeys, alternativeTipKeys, percentages, expectedAnswers, anotherExpectedAnswers)

  local round =
  {
    identifier = concept.identifier,
    expectedAnswers = expectedAnswers,
    percentages = percentages,
    tips = tips,
    conversations = conversations,
    value = value,
  }

  return round
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PercentagesConceptGenerator.new()
  local conceptGenerator = GameSession.conceptGenerator

  return TransformationGenerator.new(conceptGenerator, function(concept)
    return packageConceptIntoRound(concept)
  end)
end
