--- PercentagesConfiguration
-- @author Created by Jay Austria on April 15, 2015.

class("PercentagesConfiguration")

require("ExperimentsConfiguration")
require("difficulty.DifficultyUtils")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

PercentagesConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 4.0
PercentagesConfiguration.SPEED_BONUS_DECAY_START = 0.5
PercentagesConfiguration.POINTS_PER_FULL_SPEED_BONUS = 50
PercentagesConfiguration.POINTS_PER_CORRECT_RESPONSE = 500
PercentagesConfiguration.QUESTIONS_COUNT_PER_SET = 3
PercentagesConfiguration.SETS_TO_COMPLETE_COUNT = 4

local INITIAL_LIVES_COUNT = 4
local MAXIMUM_BONUS_LIVES = (INITIAL_LIVES_COUNT - 1)
local MAXIMUM_EXTRA_LIVES_BONUS = 4900
local POINTS_PER_LIVES_BONUS = (MAXIMUM_EXTRA_LIVES_BONUS / MAXIMUM_BONUS_LIVES)

local EXTRA_LIVES_RANK_TWO = MAXIMUM_BONUS_LIVES - 1
local EXTRA_LIVES_RANK_THREE = MAXIMUM_BONUS_LIVES

-- Value used in CoreMS's HighDifficultyReachedHighlightProducer, update if modified
local HIGH_DIFFICULTY_THRESHOLD = 0.6

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PercentagesConfiguration.getNumberOfLives()
  return INITIAL_LIVES_COUNT
end

function PercentagesConfiguration.getExtraLivesBonus()
  return POINTS_PER_LIVES_BONUS
end

function PercentagesConfiguration.getIsHighLevel()
  return (GameSession.getDifficulty() >= HIGH_DIFFICULTY_THRESHOLD)
end

function PercentagesConfiguration.shouldShowHighDifficultyInterstitial()
  return DifficultyUtils.shouldShowHighDifficultyInterstitial(HIGH_DIFFICULTY_THRESHOLD, GameSession.getDifficulty())
end

function PercentagesConfiguration.getPassScore()
  local questionCount = PercentagesConfiguration.QUESTIONS_COUNT_PER_SET * PercentagesConfiguration.SETS_TO_COMPLETE_COUNT
  local maxPossibleBaseScore = PercentagesConfiguration.POINTS_PER_CORRECT_RESPONSE * questionCount

  local maxIncorrectCount = PercentagesConfiguration.getNumberOfLives() - 1
  local maxSubtractedScore = PercentagesConfiguration.POINTS_PER_CORRECT_RESPONSE * maxIncorrectCount

  local minPossibleBaseScore = maxPossibleBaseScore - maxSubtractedScore

  return minPossibleBaseScore
end

function PercentagesConfiguration.getRankTwoScore()
  local rankTwoBonusPoints = EXTRA_LIVES_RANK_TWO * POINTS_PER_LIVES_BONUS
  return PercentagesConfiguration.getPassScore() + rankTwoBonusPoints
end

function PercentagesConfiguration.getRankThreeScore()
  local rankThreeBonusPoints = EXTRA_LIVES_RANK_THREE * POINTS_PER_LIVES_BONUS
  return PercentagesConfiguration.getPassScore() + rankThreeBonusPoints
end

function PercentagesConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 1.0}},

    {key = 0.15, valueProbabilities = {Basic = 0.3,
                                       Easy = 0.7}},

    {key = 0.225, valueProbabilities = {Easy = 0.6,
                                        Medium = 0.4}},

    {key = 0.3, valueProbabilities = {Easy = 0.4,
                                      Medium = 0.6}},

    {key = 0.375, valueProbabilities = {Medium = 0.7,
                                        Hard = 0.3}},

    {key = 0.45, valueProbabilities = {Medium = 0.3,
                                       Hard = 0.5,
                                       Advanced = 0.2}},

    {key = 0.525, valueProbabilities = {Medium = 0.1,
                                        Hard = 0.6,
                                        Advanced = 0.3}},

    {key = 0.6, valueProbabilities = {Hard = 0.65,
                                      Advanced = 0.35}},

    {key = 0.8, valueProbabilities = {Hard = 0.4,
                                      Advanced = 0.6}},

    {key = 1.0, valueProbabilities = {Hard = 0.2,
                                      Advanced = 0.8}}
  }
end
