--- PercentagesCorrectAnswerBurst
-- @author Created by Jay Austria on May 11, 2015.

class("PercentagesCorrectAnswerBurst")

require("PercentagesCommon")
require("PercentagesInputField")
require("PercentagesHelpPanel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MINIMUM_WIDTH = 40
local RADIUS_PADDING = 78
local INNER_RIM_PADDING = 71.5
local INNER_RIM_THICKNESS = 1.5
local OUTER_RIM_PADDING = 84.5
local OUTER_RIM_THICKNESS = 0.5

local INNER_ORBITER_PADDING = 13
local INNER_ORBITER_THICKNESS = 2

local INNER_ORBITER_PADDING = 35
local INNER_ORBITER_THICKNESS = 0.5

local FADED_WHITE_1 = ColorUtils.colorRGBToStd(255, 255, 255, 128)
local FADED_WHITE_2 = ColorUtils.colorRGBToStd(255, 255, 255, 64)
local BURST_CORRECT_COLOR = ColorUtils.colorRGBToStd(207, 254, 232)

local ORBITER_DESCRIPTORS =
{
  {
    padding = 10,
    thickness = 2,
    arcLength = 2 * math.pi / 6,
    arcCount = 3,
    gapLength = 2 * math.pi / 3,
    rotationPeriod = 2.5,
    rotationDirectionMultiplier = 1,
    color = PercentagesCommon.COLOR.WHITE
  },
  {
    padding = 30,
    thickness = 0.5,
    arcLength = 2 * math.pi / 6,
    arcCount = 2,
    gapLength = 2 * math.pi / 3,
    rotationPeriod = 6,
    rotationDirectionMultiplier = -1,
    color = FADED_WHITE_1
  },
  {
    padding = 34,
    thickness = 0.5,
    arcLength = 2 * math.pi / 6,
    arcCount = 1,
    gapLength = 0,
    rotationPeriod = 6,
    rotationDirectionMultiplier = 1,
    color = FADED_WHITE_2
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------
local function createArcGroup(radius, thickness, length, count, rotationGap, color)
  local diameter = radius * 2
  local arcGroup = Group.new(diameter, diameter)

  local startRadians = 0
  for arcIndex = 1, count do

    local endRadians = startRadians + length
    local arc = HollowArc.new(radius, startRadians, endRadians, color, thickness, 0, false)
    arcGroup:insertAtCenter(arc)

    startRadians = startRadians + rotationGap
  end

  arcGroup:linkOpacityAttrToChildren()
  arcGroup:setOpacity(0.0)

  return arcGroup
end

local function createLayout(self, view)
  local innerCircle = FilledCircle.new(self:getWidthPart(0.5), BURST_CORRECT_COLOR)
  self:insertAtCenter(innerCircle)
  innerCircle:setScl(0.0)
  self.innerCircle = innerCircle

  local innerRimRadius = self:getWidthPart(0.5) + INNER_RIM_PADDING
  local innerRim = HollowCircle.new(innerRimRadius, BURST_CORRECT_COLOR, INNER_RIM_THICKNESS)
  self:insertAtCenter(innerRim)
  innerRim:setScl(0.0)
  self.innerRim = innerRim

  local outerRimRadius = self:getWidthPart(0.5) + OUTER_RIM_PADDING
  local outerRim = HollowCircle.new(outerRimRadius, BURST_CORRECT_COLOR, OUTER_RIM_THICKNESS)
  self:insertAtCenter(outerRim)
  outerRim:setScl(0.0)
  self.outerRim = outerRim

  innerCircle:insertAtCenter(view)
  view:setVisible(false)
  self.view = view

  local arcGroups = {}
  for _, descriptor in ipairs(ORBITER_DESCRIPTORS) do
    local arcsRadius = self:getWidthPart(0.5) + descriptor.padding
    local arcs = createArcGroup(arcsRadius, descriptor.thickness, descriptor.arcLength, descriptor.arcCount, descriptor.gapLength, descriptor.color)
    self:insertAtCenter(arcs)

    local loopedRotation = AHLoop.new(AHRotationBy.new(arcs, descriptor.rotationPeriod, nil, 360 * descriptor.rotationDirectionMultiplier))
    loopedRotation:start()
    arcs.loopedRotation = loopedRotation

    table.insert(arcGroups, arcs)
  end

  self.arcGroups = arcGroups
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PercentagesCorrectAnswerBurst:getBurstAnimation()
  local showInnerCircle = AHSpawn.new(
                            AHScaleTo.new(self.innerCircle, 0.5, AHEaseBackOut.new(2.0), 1.0),
                            AHColorTo.new(self.innerCircle, 1.0, AHEaseExponentialIn.new(), PercentagesCommon.COLOR.WHITE)
                          )

  local showInnerRim = AHScaleTo.new(self.innerRim, 1.7, AHEaseExponentialOut.new(), 1.0)
  local showOuterRim = AHScaleTo.new(self.outerRim, 1.7, AHEaseExponentialOut.new(), 1.0)

  local fadeInnerRim = AHDelayed.new(0.8, AHOpacityTo.new(self.innerRim, 1.0, nil, PercentagesCommon.INVISIBLE_OPACITY))
  local fadeOuterRim = AHDelayed.new(0.8, AHOpacityTo.new(self.outerRim, 1.0, nil, PercentagesCommon.INVISIBLE_OPACITY))

  local hideInnerCircle = AHScaleTo.new(self.innerCircle, 0.5, AHEaseExponentialInOut.new(), 0.0)

  local innerCircleSequence = AHSequence.new(showInnerCircle, AHDelay.new(0.3), hideInnerCircle)
  local innerRimSpawn = AHSpawn.new(showInnerRim, fadeInnerRim)
  local outerRimSpawn = AHSpawn.new(showOuterRim, fadeOuterRim)

  local arcsSpawn = AHSpawn.new()
  for _, arcGroup in ipairs(self.arcGroups) do
    local arcSequence = AHSequence.new(
                          AHDelay.new(0.3),
                          AHOpacityTo.new(arcGroup, 0.2, nil, PercentagesCommon.VISIBLE_OPACITY),
                          AHDelay.new(0.6),
                          AHOpacityTo.new(arcGroup, 0.5, nil, PercentagesCommon.INVISIBLE_OPACITY),
                          function() arcGroup.loopedRotation:stop() end
                        )

    arcsSpawn:addAction(arcSequence)
  end

  local delayedShowLabel = AHDelayed.new(0.05, function() self.view:setVisible(true) end)

  return AHSpawn.new(delayedShowLabel, innerCircleSequence, innerRimSpawn, outerRimSpawn, arcsSpawn)
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function newInternal(view)
  local radius = math.max(view:getWidth(), MINIMUM_WIDTH) + RADIUS_PADDING

  local self = Group.new(radius, radius)
  Mixin.mix(self, PercentagesCorrectAnswerBurst)

  createLayout(self, view)

  return self
end

function PercentagesCorrectAnswerBurst.new(value)
  local valueLabel = Label.new(tostring(value), Font.new(FontFaces.DIN_OT_LIGHT, PercentagesCommon.INPUT_FIELD_FONT_SIZE))
  valueLabel:setColor(unpack({0.4, 0.1, 0.4}))
  return newInternal(valueLabel)
end

function PercentagesCorrectAnswerBurst.newWithImage(filename)
  local image = Sprite.newWithName(filename)
  image:setScl(0.6)
  return newInternal(image)
end
