--- PercentagesGradientRectangle
-- @author Created by Jay Austria on May 18, 2015.

class("PercentagesGradientRectangle")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PercentagesGradientRectangle:getColorChangeAction(stayGreen)
  local colorChange = AHSequence.new()

  colorChange:addAction(self:newGradientToAction(0.5, nil, self.topColorGreen, self.bottomColorGreen))

  if not stayGreen then
    local top = self.originalTopColor
    local bottom = self.originalBottomColor
    colorChange:addAction(self:newGradientToAction(0.5, nil, top, bottom))
  end

  return colorChange
end

function PercentagesGradientRectangle:playColorChange()
  local colorAction = self:getColorChangeAction()

  return colorAction:start()
end

function PercentagesGradientRectangle:getOpacityAndColorFadeAction()
  local toTopColor = TableUtils.shallowCopy(self.topColorGreen)
  local toBottomColor = TableUtils.shallowCopy(self.bottomColorGreen)

  local toAction = self:newGradientToAction(0.5, nil, toTopColor, toBottomColor)

  local resetColor = function()
    local resetTopColor = TableUtils.shallowCopy(self.originalTopColor)
    local resetBottomColor = TableUtils.shallowCopy(self.originalBottomColor)
    resetTopColor[4] = PercentagesCommon.INVISIBLE_OPACITY
    resetBottomColor[4] = PercentagesCommon.INVISIBLE_OPACITY
    self:setBeginColor(unpack(resetTopColor))
    self:setEndColor(unpack(resetBottomColor))
  end

  local backTopColor = TableUtils.shallowCopy(self.originalTopColor)
  local backBottomColor = TableUtils.shallowCopy(self.originalBottomColor)
  backTopColor[4] = PercentagesCommon.INVISIBLE_OPACITY
  backBottomColor[4] = PercentagesCommon.VISIBLE_OPACITY
  local fadeIn = self:newGradientToAction(0.5, nil, backTopColor, backBottomColor)

  local fadeAction = AHSequence.new(toAction, AHDelay.new(1.0), resetColor, fadeIn)

  return fadeAction
end

function PercentagesGradientRectangle:fadeForOutro()
  local topColor = TableUtils.shallowCopy(self.topColorGreen)
  local bottomColor = TableUtils.shallowCopy(self.bottomColorGreen)
  topColor[4] = PercentagesCommon.INVISIBLE_OPACITY
  bottomColor[4] = PercentagesCommon.VISIBLE_OPACITY

  return self:newGradientToAction(0.5, nil, topColor, bottomColor)
end

function PercentagesGradientRectangle:setGreenColors(top, bottom)
  self.topColorGreen = top
  self.bottomColorGreen = bottom
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PercentagesGradientRectangle.new(width, height, topColor, bottomColor)
  local self = RectangleGradient.new(width, height, topColor, bottomColor)
  Mixin.mix(self, PercentagesGradientRectangle)

  self.originalTopColor = topColor
  self.originalBottomColor = bottomColor

  self:setGreenColors(PercentagesCommon.BACKGROUND_COLOR_TOP_GREEN, PercentagesCommon.BACKGROUND_COLOR_BOTTOM_GREEN)

  return self
end
