--- PercentagesHelpPanel
-- @author Created by Jay Austria on May 11, 2015.

class("PercentagesHelpPanel")

require("widgets.rationales.ChatRationale")
require("widgets.rationales.ChatRationaleLearnMoreButton")
require("widgets.rationales.ChatRationaleAnalytics")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_COLOR = {0.39, 0.39, 0.39}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getLearnMorePressedCallback(self)
  return function()
    -- Call these immediately! Avoids a bug where the learnMoreButton and callback
    -- get called a frame too late if the logic is included as the first function
    -- call in an AHSequence
    self.learnMoreButton:setTouchEnabled(false)
    if (self.learnMorePressedCallback ~= nil) then
      self.learnMorePressedCallback()
    end

    local openAction = AHLocationYTo.new(self.panel, 0.7, AHEaseExponentialInOut.new(), Screen.getHeightPart(0.5))

    self.chatRationale:openWithAction(openAction)
  end
end

local function getChatRationaleClosedCallback(self)
  return function()
    local closeAction = AHSequence.new(
      AHLocationYTo.new(self.panel, 0.5, AHEaseSineInOut.new(), 0),
      function()
        self.learnMoreButton:setTouchEnabled(true)

        if (self.learnMoreClosedCallback ~= nil) then
          self.learnMoreClosedCallback()
        end
      end
    )

    self.chatRationale:closeWithAction(closeAction)
  end
end

local function createLayout(self)
  local panel = FilledRectangle.new(Screen.fullWidth, Screen.getSafeY(0.5), {1, 1, 1})
  self.expandedY = panel:getHeightPart(0.5)
  self.hiddenY = panel:getHeightPart(-0.5)

  panel:setLoc(self:getWidthPart(0.5), self.hiddenY)
  self:insert(panel)
  self.panel = panel

  local tipText = Localizer.get("Misc.TIP")

  local tipHeader = Label.new(tipText, Font.new(FontFaces.DIN_OT_MEDIUM, 20))
  tipHeader:setColor(unpack(PercentagesCommon.COLOR.BLACK))
  tipHeader:setOpacity(0.44)
  tipHeader:setAnchorPointLeft()
  tipHeader:setLoc(Screen.getSafeX(0.09), panel:getHeightPart(0.9))
  panel:insert(tipHeader)

  local tipLabel = DynamicTextArea.newSimple("", Screen.getSafeWidthPart(0.82), Font.new(FontFaces.DIN_OT_MEDIUM, 28))
  tipLabel:setAnchorPointTopLeft()
  tipLabel:setColor(unpack(TEXT_COLOR))
  tipLabel:setLoc(Screen.getSafeX(0.09), panel:getHeightPart(0.82))
  panel:insert(tipLabel)
  self.tipLabel = tipLabel

  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  tapToContinueLabel:setColor(unpack(PercentagesCommon.COLOR.BLACK))
  tapToContinueLabel:setOpacity(0.44)
  tapToContinueLabel:setLoc(Screen.getSafeX(0.5), panel:getHeightPart(0.1))
  panel:insert(tapToContinueLabel)
end

local function createLayoutExpanded(self)
  local backgroundColor = ColorUtils.colorHex24ToStd(0xeCe4e7)

  local panel = FilledRectangle.new(Screen.width, Screen.height, backgroundColor)
  self.expandedY = 0
  self.hiddenY = panel:getHeightPart(-0.5)

  panel:setLoc(self:getWidthPart(0.5), self.hiddenY)
  self:insert(panel)
  self.panel = panel

  local tipLabel = DynamicTextArea.newSimple("", Screen.getSafeWidthPart(0.82), Font.new(FontFaces.DIN_OT_MEDIUM, 24))
  tipLabel:setAnchorPointTopLeft()
  tipLabel:setColor(unpack(ColorUtils.colorHex24ToStd(0x1a1a1a)))
  panel:insertAtLocation(tipLabel, Screen.getSafeX(0.09), panel:getHeightPart(0.95))
  self.tipLabel = tipLabel

  local learnMorePressedCallback = getLearnMorePressedCallback(self)
  local learnMoreButton = ChatRationaleLearnMoreButton.new(backgroundColor, learnMorePressedCallback)
  learnMoreButton:setAnchorPointTopLeft()
  panel:insertAtLocation(learnMoreButton, tipLabel:getXLeft(), panel:getHeightPart(0.8))
  learnMoreButton:setTouchEnabled(false)
  self.learnMoreButton = learnMoreButton

  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  tapToContinueLabel:setColor(unpack(ColorUtils.colorHex24ToStd(0x747474)))
  panel:insertAtLocation(tapToContinueLabel, Screen.getSafeX(0.5), panel:getHeightPart(0.55))

  local chatRationaleClosedCallback = getChatRationaleClosedCallback(self)
  local chatRationale = ChatRationale.new(backgroundColor, chatRationaleClosedCallback)
  Screen.rootGroup:insertAtLocation(chatRationale, Screen.getWidthPart(0.5), 0)
  chatRationale:bringToFront()
  self.chatRationale = chatRationale
end

local function getSlideAction(self, targetY)
  local panel = self.panel
  local slideDuration = 0.5
  local slideAction = AHLocationYTo.new(self.panel, slideDuration, AHEaseExponentialInOut.new(), targetY)
  local playSound = function() Sound.playSoundEffect("whoosh") end
  local actions = {slideAction, playSound}

  -- make sure to set panelIsOffScreen flag when done sliding off-screen
  if (targetY == self.hiddenY) then
    local setOffscreenFlag = AHDelayed.new(slideDuration, function() self.panelIsOffScreen = true end)
    table.insert(actions, setOffscreenFlag)
  end

  local combinedAction = AHSpawn.new(actions)

  return combinedAction
end

local function reportOnPresentationAnalytics(self)
  ChatRationaleAnalytics.reportExperimentExposed()

  local hasLearnMoreAvailable = (self.learnMoreButton ~= nil)
  ChatRationaleAnalytics.reportMainRationaleSeen(hasLearnMoreAvailable)
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PercentagesHelpPanel:animateShow()
  self.panelIsOffScreen = false
  local slideAction = AHSequence.new(
    getSlideAction(self, self.expandedY),
    function()
      reportOnPresentationAnalytics(self)

      if (self.learnMoreButton ~= nil) then
        self.learnMoreButton:setTouchEnabled(true)
      end
    end
  )

  self.slideInterrupt:insert(slideAction)

  return slideAction
end

function PercentagesHelpPanel:prepareForHide()
  if (self.learnMoreButton ~= nil) then
    self.learnMoreButton:setTouchEnabled(false)
  end
end

function PercentagesHelpPanel:animateHide()
  local slideAction = getSlideAction(self, self.hiddenY)

  self.slideInterrupt:insert(slideAction)

  return slideAction
end

function PercentagesHelpPanel:setHelpText(text, conversationDescriptor, conceptId)
  self.conceptId = conceptId

  local labelText = text or ""

  runInCoroutine(function()
    waitUntil(function() return self.panelIsOffScreen end)
    self.tipLabel:setText(labelText)

    if (ExperimentsConfiguration.EXPANDED_RATIONALES_EXPERIMENT:isActive()) then
      if (self.learnMoreButton ~= nil) then
        local marginY = self.learnMoreButton:getHeightPart(0.5)
        local learnMoreButtonY = self.tipLabel:getYBottom() - marginY
        self.learnMoreButton:setY(learnMoreButtonY)
      end

      if ((self.chatRationale ~= nil) and (conversationDescriptor ~= nil)) then
        self.chatRationale:setConversationDescriptor(conversationDescriptor)
      end
    end
  end)
end

function PercentagesHelpPanel:setLearnMorePressedCallback(callback)
  self.learnMorePressedCallback = callback
end

function PercentagesHelpPanel:setLearnMoreClosedCallback(callback)
  self.learnMoreClosedCallback = callback
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PercentagesHelpPanel.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, PercentagesHelpPanel)

  self.slideInterrupt = ActionInterrupt.new()

  self.panelIsOffScreen = true
  self.learnMorePressedCallback = nil
  self.learnMoreClosedCallback = nil
  self.conceptId = nil

  if (ExperimentsConfiguration.EXPANDED_RATIONALES_EXPERIMENT:isActive()) then
    createLayoutExpanded(self)
  else
    createLayout(self)
  end

  return self
end
