--- PercentagesHighLevelInterstitial
-- @author Created by Jay Austria on July 13, 2015.

class("PercentagesHighLevelInterstitial")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BODY_TEXT = Localizer.get("HIGH_DIFFICULTY")
local CONTINUE_TEXT = Localizer.get("Misc.CONTINUE")
local BODY_COLOR = {0.96, 0.56, 0.52}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local font = FontFaces.DIN_OT_MEDIUM

  local bodyLabel = DynamicTextArea.newSimple(BODY_TEXT, Screen.getSafeWidthPart(0.8), Font.new(font, 18))
  bodyLabel:setColor(unpack(BODY_COLOR))
  local continueLabel = Label.new(CONTINUE_TEXT, Font.new(font, 16))

  local layoutColumn = Column.new()

  layoutColumn:insert(bodyLabel)
  layoutColumn:insertSpacer(Screen.getSafeY(0.35))
  layoutColumn:insert(continueLabel)

  layoutColumn:alignChildren()

  layoutColumn:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.35))
  self:insert(layoutColumn)

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PercentagesHighLevelInterstitial:prepareForAnimateIn()
  self:setOpacity(PercentagesCommon.INVISIBLE_OPACITY)
end

function PercentagesHighLevelInterstitial:doAnimateIn()
  local fadeDuration = 0.5
  local fadeIn = AHOpacityTo.new(self, fadeDuration, nil, PercentagesCommon.VISIBLE_OPACITY)

  return fadeIn:start()
end

function PercentagesHighLevelInterstitial:doAnimateOut()
  local fadeDuration = 0.5
  local fadeOut = AHOpacityTo.new(self, fadeDuration, nil, PercentagesCommon.INVISIBLE_OPACITY)
  local removeFromScene = AHRemove.new(self)

  local combinedFadeOut = AHSequence.new(fadeOut, removeFromScene)

  return combinedFadeOut:start()
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PercentagesHighLevelInterstitial.new()
  local self = PercentagesGradientRectangle.new(Screen.fullWidth,
                                                Screen.fullHeight,
                                                PercentagesCommon.BACKGROUND_COLOR_TOP,
                                                PercentagesCommon.BACKGROUND_COLOR_BOTTOM)

  Mixin.mix(self, PercentagesHighLevelInterstitial)

  createLayout(self)

  return self
end
