--- PercentagesInputField
-- @author Created by Jay Austria on May 6, 2015.

class("PercentagesInputField")

require("actions.advanced.Shake")
require("widgets.number_pad.NumberPadInputField")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local VISIBLE_OPACITY = 1.0
local INVISIBLE_OPACITY = 0.0

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PercentagesInputField:showCorrection(correctValue)
  local viewsRow = self.viewsRow
  local originX = viewsRow:getX()
  local animationDuration = 0.5
  local correctionSlideX = 60

  local isRecursive = true
  viewsRow:linkColorAttrToChildren(isRecursive)

  local incorrectMoveAction = AHLocationXBy.new(viewsRow, animationDuration, AHEaseSimpleInOut.new(), correctionSlideX)
  local incorrectColorAction = AHColorTo.new(viewsRow, animationDuration, nil, self.incorrectColor)

  local correctLabel = Label.new(tostring(correctValue), self.font)
  correctLabel:setOpacity(INVISIBLE_OPACITY)
  correctLabel:setLoc(viewsRow:getLoc())
  self:insert(correctLabel)
  local correctMoveAction = AHLocationXBy.new(correctLabel, animationDuration, AHEaseSimpleInOut.new(), -correctionSlideX)
  local correctFadeAction = AHOpacityTo.new(correctLabel, animationDuration, AHEaseSimpleInOut.new(), VISIBLE_OPACITY)

  local correctionAnimation = AHSpawn.new(incorrectMoveAction, incorrectColorAction, correctMoveAction, correctFadeAction)
  correctionAnimation:start()

  local extraLength = 5
  local strikethroughLength = viewsRow:getWidth() + extraLength
  local strikethroughThickness = 2
  local strikethrough = FilledRectangle.new(strikethroughLength, strikethroughThickness, self.defaultColor)
  viewsRow:insertAtCenter(strikethrough)

  self.correctLabel = correctLabel
  self.originX = originX
  self.strikethrough = strikethrough
end

function PercentagesInputField:clearCorrection()
  local viewsRow = self.viewsRow
  local correctLabel = self.correctLabel
  local strikethrough = self.strikethrough
  local fadeDuration = 0.3

  local fadeOut = AHSpawn.new(
                    AHOpacityTo.new(viewsRow, fadeDuration, AHEaseSimpleInOut.new(), INVISIBLE_OPACITY),
                    AHOpacityTo.new(correctLabel, fadeDuration, AHEaseSimpleInOut.new(), INVISIBLE_OPACITY),
                    AHOpacityTo.new(strikethrough, fadeDuration, AHEaseSimpleInOut.new(), INVISIBLE_OPACITY)
                  )

  local resetViewsRow = function()
    viewsRow:setX(self.originX)
    viewsRow:setColor(unpack(self.defaultColor))

    correctLabel:removeSelf()
    strikethrough:removeSelf()
    self:clear()
  end

  local clearSequence = AHSequence.new(fadeOut, resetViewsRow)
  clearSequence:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PercentagesInputField.new(height, fontFace, fontSize, defaultColor, incorrectColor)
  local self = NumberPadInputField.new(height, fontFace, fontSize, defaultColor, incorrectColor)
  Mixin.mix(self, PercentagesInputField)

  return self
end
