--- PercentagesPrompt
-- @author Created by Jay Austria on May 14, 2015.

class("PercentagesPrompt")

require("PercentagesShapeDeck")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local lastPickedType = nil
local function getRandomShapeType()
  local typesArray = TableUtils.createArrayFromSet(PercentagesShape.Types)
  repeat
    randomType = TableUtils.getRandomElement(typesArray)
  until (randomType ~= lastPickedType)

  lastPickedType = randomType

  return randomType
end

local function createNewDeck(self)
  local shapeType = getRandomShapeType()
  local percentagesDeck = PercentagesShapeDeck.new(shapeType)
  local deckX = self.ofLabel.originX - 65
  percentagesDeck:setLoc(deckX, baselineY)
  self:insert(percentagesDeck)
  self.percentagesDeck = percentagesDeck

  return percentagesDeck
end

local function createLayout(self)
  local yNormalized = 0.75
  local baselineY = Screen.getSafeY(yNormalized)
  local wordSpacingX = 18

  --  Polish. Used to increase readability of any text in front of it.
  local heightYNormalized = 0.16
  local backdropHeight = Screen.getSafeHeightPart(heightYNormalized)
  local locationYNormalized = yNormalized + 0.02
  local backdropLocationY = Screen.getSafeY(locationYNormalized)

  local topColor = ColorUtils.makeColorBetweenTwoColors(PercentagesCommon.BACKGROUND_COLOR_BOTTOM,
                                                        PercentagesCommon.BACKGROUND_COLOR_TOP,
                                                        locationYNormalized + heightYNormalized * 0.5)
  local bottomColor = ColorUtils.makeColorBetweenTwoColors(PercentagesCommon.BACKGROUND_COLOR_BOTTOM,
                                                           PercentagesCommon.BACKGROUND_COLOR_TOP,
                                                           locationYNormalized - heightYNormalized * 0.5)
  local topColorGreen = ColorUtils.makeColorBetweenTwoColors(PercentagesCommon.BACKGROUND_COLOR_BOTTOM_GREEN,
                                                             PercentagesCommon.BACKGROUND_COLOR_TOP_GREEN,
                                                             locationYNormalized + heightYNormalized * 0.5)
  local bottomColorGreen = ColorUtils.makeColorBetweenTwoColors(PercentagesCommon.BACKGROUND_COLOR_BOTTOM_GREEN,
                                                                PercentagesCommon.BACKGROUND_COLOR_TOP_GREEN,
                                                                locationYNormalized - heightYNormalized * 0.5)
  topColor[4] = PercentagesCommon.INVISIBLE_OPACITY
  bottomColor[4] = PercentagesCommon.VISIBLE_OPACITY
  topColorGreen[4] = PercentagesCommon.INVISIBLE_OPACITY
  bottomColorGreen[4] = PercentagesCommon.INVISIBLE_OPACITY
  local gradientBackdrop = PercentagesGradientRectangle.new(self:getWidth(), backdropHeight, topColor, bottomColor)

  gradientBackdrop:setGreenColors(topColorGreen, bottomColorGreen)
  gradientBackdrop:setLoc(Screen.getSafeX(0.5), backdropLocationY)
  self:insert(gradientBackdrop)
  self.gradientBackdrop = gradientBackdrop

  local ofLabel = Label.new(Localizer.get("OF"), Font.new(FontFaces.DIN_OT_MEDIUM, 30))
  local originX = Screen.getSafeX(0.45)
  ofLabel:setAnchorPointBottom()
  ofLabel:setLoc(originX, baselineY)
  ofLabel.originX = originX
  self:insert(ofLabel)
  self.ofLabel = ofLabel

  local newDeck = createNewDeck(self)
  newDeck:bringToBack()

  local ofLabelMaxX = ofLabel:getX() + ofLabel:getWidthPart(0.55)

  local valueLabel = Label.new("UNSET", Font.new(FontFaces.DIN_OT_LIGHT, 58))
  local alignmentOffset = -8
  originX = ofLabelMaxX + wordSpacingX
  valueLabel:setAnchorPointBottomLeft()
  valueLabel:setLoc(originX, baselineY + alignmentOffset)
  valueLabel.originX = originX
  self:insert(valueLabel)
  self.valueLabel = valueLabel

  local percentLabel = Label.new("0%", Font.new(FontFaces.DIN_OT_MEDIUM, 32))
  originX = self.percentagesDeck:getX()
  local offsetX = 4
  percentLabel:setLoc(originX + offsetX, baselineY + percentLabel:getHeightPart(0.5))
  self:insert(percentLabel)
  self.percentLabel = percentLabel

  self.baselineY = baselineY
end

local function initializeDeckForPlayInFromSwitch(self, deck, percentage)
  local deckY = self.baselineY + deck:getHeightPart(0.28)
  deck:setY(deckY)

  deck:setShapesScale(0.0)
  deck:setShapesColor(PercentagesCommon.COLOR.CORRECT_GREEN)
end

local function setPercentLabelValue(self, value)
  local percentLabel = self.percentLabel
  local labelString = string.format("%d%%", value)
  percentLabel:setText(labelString)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PercentagesPrompt:prepareForIntro()
  local percentagesDeck = self.percentagesDeck
  percentagesDeck:setY(self:getHeightPart(-0.6))

  local ofLabel = self.ofLabel
  ofLabel:setX(Screen.getWidthPart(0.0))
  ofLabel:setOpacity(PercentagesCommon.INVISIBLE_OPACITY)

  local valueLabel = self.valueLabel
  valueLabel:setX(percentagesDeck:getX() - valueLabel:getWidthPart(0.5))
  valueLabel:setOpacity(PercentagesCommon.INVISIBLE_OPACITY)

  local gradientBackdrop = self.gradientBackdrop
  gradientBackdrop.endColor:setOpacity(PercentagesCommon.INVISIBLE_OPACITY)

  local percentLabel = self.percentLabel
  percentLabel:setOpacity(PercentagesCommon.INVISIBLE_OPACITY)
  percentLabel:setRot(-180)
end

function PercentagesPrompt:getIntroAnimation(doFadeInBackdrop, digitCount)
  local labelAnimationDuration = 1.0

  local ofLabel = self.ofLabel
  local fadeInOfLabel = AHOpacityTo.new(ofLabel, labelAnimationDuration, nil, PercentagesCommon.VISIBLE_OPACITY)
  local delayedFadeInOfLabel = AHDelayed.new(0.0, fadeInOfLabel)
  local slideInOfLabel = AHLocationXTo.new(ofLabel, labelAnimationDuration, AHEaseExponentialOut.new(), ofLabel.originX)
  local ofLabelAnimation = AHDelayed.new(0.5, AHSpawn.new(delayedFadeInOfLabel, slideInOfLabel))

  local valueLabel = self.valueLabel
  local fadeInValueLabel = AHOpacityTo.new(valueLabel, labelAnimationDuration, nil, PercentagesCommon.VISIBLE_OPACITY)
  local delayedFadeInValue = AHDelayed.new(0.0, fadeInValueLabel)
  slideInValueLabel = AHLocationXTo.new(valueLabel, labelAnimationDuration, AHEaseExponentialOut.new(), valueLabel.originX)
  local valueLabelAnimation = AHDelayed.new(0.5, AHSpawn.new(delayedFadeInValue, slideInValueLabel))

  local percentagesDeck = self.percentagesDeck
  local percentagesDeckY = self.baselineY + percentagesDeck:getHeightPart(0.28)
  local slideInDeck = AHLocationYTo.new(percentagesDeck, 0.6, AHEaseSimpleInOut.new(), percentagesDeckY)
  local deckIntroAnimation = percentagesDeck:getIntroAnimation()

  local percentLabel = self.percentLabel
  local fadeInPercent = AHSpawn.new(
                          AHDelayed.new(0.2, AHOpacityTo.new(percentLabel, labelAnimationDuration * 0.9, nil, PercentagesCommon.VISIBLE_OPACITY)),
                          AHRotationTo.new(percentLabel, labelAnimationDuration, AHEaseExponentialOut.new(), 0)
                        )
  local delayedFadeInPercent = AHDelayed.new(0.75, fadeInPercent)

  local expectedDigits = 3
  local offsetX = Screen.getWidthPart(0.025) * (expectedDigits - digitCount)
  local targetX = Screen.getSafeX(0.5) + offsetX
  local slideDuration = 0.5
  local alignAction = AHDelayed.new(0.5, AHLocationXTo.new(self, slideDuration, AHEaseSimpleInOut.new(), targetX))

  local introSpawn = AHSpawn.new(ofLabelAnimation, valueLabelAnimation, slideInDeck, deckIntroAnimation, delayedFadeInPercent, alignAction)

  if doFadeInBackdrop then
    local fadeBackdrop = self.gradientBackdrop:newOpacityToAction(0.2, nil, PercentagesCommon.INVISIBLE_OPACITY, PercentagesCommon.VISIBLE_OPACITY)
    local delayedFadeBackdrop = AHDelayed.new(1.0, fadeBackdrop)
    introSpawn:addAction(delayedFadeBackdrop)
  end

  return introSpawn
end

function PercentagesPrompt:presentFirstQuestion(percentage, value)
  local valueLabel = self.valueLabel
  valueLabel:setText(value)

  setPercentLabelValue(self, percentage)
end

function PercentagesPrompt:presentNextShape(percentage, wasPreviousCorrect)
  local percentagesDeck = self.percentagesDeck
  local percentLabel = self.percentLabel

  local setPercent = function()
    setPercentLabelValue(self, percentage)
  end

  local popFrontAnimation = percentagesDeck:getPopFrontAnimation(percentage, wasPreviousCorrect)

  local fadeOut = AHOpacityTo.new(percentLabel, 0.4, nil, PercentagesCommon.INVISIBLE_OPACITY)
  local fadeIn = AHOpacityTo.new(percentLabel, 0.4, nil, PercentagesCommon.VISIBLE_OPACITY)
  local fadeLabelSequence = AHSequence.new(fadeOut, AHDelay.new(1.4))

  if percentage then
    fadeLabelSequence:addAction(setPercent)
    fadeLabelSequence:addAction(fadeIn)
  end

  local presentAnimation = AHSpawn.new(popFrontAnimation, fadeLabelSequence)

  return presentAnimation:start()
end

function PercentagesPrompt:playSetSwitchAnimation(percentage, value)
  local percentagesDeck = self.percentagesDeck
  percentagesDeck:animateCollapse()

  local newDeck = createNewDeck(self)
  newDeck:bringToBack()

  initializeDeckForPlayInFromSwitch(self, newDeck, percentage)
  local doFadeBackdrop = false
  local numberOfDigits = math.log10(value)
  local newDeckAnimation = self:getIntroAnimation(doFadeBackdrop, numberOfDigits)
  newDeckAnimation:start()

  local ofLabel = self.ofLabel
  local valueLabel = self.valueLabel
  local fadeDuration = 0.5

  local fadeLabels = AHSpawn.new(
                       AHLocationXTo.new(ofLabel, fadeDuration, AHEaseSimpleInOut.new(), newDeck:getX()),
                       AHOpacityTo.new(ofLabel, fadeDuration, nil, PercentagesCommon.INVISIBLE_OPACITY),
                       AHLocationXTo.new(valueLabel, fadeDuration, AHEaseSimpleInOut.new(), newDeck:getX()),
                       AHOpacityTo.new(valueLabel, fadeDuration, nil, PercentagesCommon.INVISIBLE_OPACITY)
                     )

  local updateLabels = function()
    local valueLabel = self.valueLabel
    valueLabel:setText(value)

    setPercentLabelValue(self, percentage)
  end

  local showLabels = AHSpawn.new(
                       AHOpacityTo.new(ofLabel, fadeDuration, nil, PercentagesCommon.VISIBLE_OPACITY),
                       AHOpacityTo.new(valueLabel, fadeDuration, nil, PercentagesCommon.VISIBLE_OPACITY)
                     )
  local switchLabelsSequence = AHSequence.new(fadeLabels, updateLabels, showLabels)

  local backdropFadeSequence = self.gradientBackdrop:getOpacityAndColorFadeAction()
  local combinedSpawn = AHSpawn.new(backdropFadeSequence, switchLabelsSequence)

  return combinedSpawn:start()
end

function PercentagesPrompt:getOutroAnimation()
  local ofLabel = self.ofLabel
  local percentagesDeck = self.percentagesDeck
  local outroDuration = 0.8

  local ofDuration = outroDuration * 0.4
  local ofLabelAction = AHSpawn.new(
                          AHLocationXTo.new(ofLabel, ofDuration, AHEaseSimpleInOut.new(), percentagesDeck:getX()),
                          AHOpacityTo.new(ofLabel, ofDuration, nil, PercentagesCommon.INVISIBLE_OPACITY),
                          AHScaleTo.new(ofLabel, ofDuration, nil, 0.5)
                        )


  local fadeGradientBackdrop = self.gradientBackdrop:fadeForOutro()

  local outroAnimation = AHSpawn.new(ofLabelAction, fadeGradientBackdrop)
  return outroAnimation
end

function PercentagesPrompt:getShapes()
  return self.percentagesDeck:getChildren()
end

function PercentagesPrompt:getValueLabel()
  return self.valueLabel
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PercentagesPrompt.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, PercentagesPrompt)

  createLayout(self)

  self:prepareForIntro()

  return self
end
