class("PounceConfiguration")

require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

PounceConfiguration.POINTS_PER_CORRECT_RESPONSE = 800
PounceConfiguration.POINTS_PER_FULL_SPEED_BONUS = 100
PounceConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 2.5
PounceConfiguration.CORRECT_NEEDED_TO_WIN = 5

local NUMBER_OF_LIVES_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {lives = 5}),
  Keyframe.new(0.2, {lives = 4})
)

local MAXIMUM_EXTRA_LIVES_BONUS = 3000
local MISTAKES_ALLOWED_RANK_TWO = 1
local MISTAKES_ALLOWED_RANK_THREE = 0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getMaximumBonusLives()
  return PounceConfiguration.getNumberOfLives() - 1
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PounceConfiguration.getNumberOfLives()
  local difficulty = GameSession.getDifficulty()
  local frameForDifficulty = NUMBER_OF_LIVES_THRESHOLDS:getFrameAt(difficulty)
  return frameForDifficulty.lives
end

function PounceConfiguration.getExtraLivesBonus()
  local maximumBonusLives = getMaximumBonusLives()
  return MAXIMUM_EXTRA_LIVES_BONUS / maximumBonusLives
end


function PounceConfiguration.getPassScore()
  local correctNeeded = PounceConfiguration.CORRECT_NEEDED_TO_WIN
  local passScore = PounceConfiguration.POINTS_PER_CORRECT_RESPONSE * correctNeeded
  return passScore
end

function PounceConfiguration.getRankTwoScore()
  local passScore = PounceConfiguration.getPassScore()
  local extraLivesRequired = getMaximumBonusLives() - MISTAKES_ALLOWED_RANK_TWO
  local extraLivesBonus = extraLivesRequired * PounceConfiguration.getExtraLivesBonus()
  local rankTwoScore = passScore + extraLivesBonus
  return rankTwoScore
end

function PounceConfiguration.getRankThreeScore()
  local passScore = PounceConfiguration.getPassScore()
  local extraLivesRequired = getMaximumBonusLives() - MISTAKES_ALLOWED_RANK_THREE
  local extraLivesBonus = extraLivesRequired * PounceConfiguration.getExtraLivesBonus()
  local rankThreeScore = passScore + extraLivesBonus
  return rankThreeScore
end

function PounceConfiguration.getDifficultyKeyFrames()
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Basic = 0.6,
        Easy = 0.4
      }
    },
    {
      key = 0.075,
      valueProbabilities = {
        Basic = 0.4,
        Easy = 0.6
      }
    },
    {
      key = 0.2,
      valueProbabilities = {
        Easy = 0.8,
        Medium = 0.2
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.5,
        Medium = 0.5
      }
    },
    {
      key = 0.375,
      valueProbabilities = {
        Easy = 0.45,
        Medium = 0.55
      }
    },
    {
      key = 0.45,
      valueProbabilities = {
        Easy = 0.4,
        Medium = 0.45,
        Hard = 0.15
      }
    },
    {
      key = 0.525,
      valueProbabilities = {
        Easy = 0.25,
        Medium = 0.45,
        Hard = 0.3
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        Easy = 0.15,
        Medium = 0.35,
        Hard = 0.5
      }
    },
    {
      key = 0.675,
      valueProbabilities = {
        Medium = 0.2,
        Hard = 0.7,
        Advanced = 0.1
      }
    },
    {
      key = 0.75,
      valueProbabilities = {
        Hard = 0.7,
        Advanced = 0.3
      }
    },
    {
      key = 0.9,
      valueProbabilities = {
        Hard = 0.5,
        Advanced = 0.5
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 0.3,
        Advanced = 0.7
      }
    }
  }
end
