class("PounceRoundGenerator")

require("GameSession")
require("data.generators.UtilityGenerators")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_UNIQUE_CONCEPT_GENERATION_ATTEMPTS = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function conceptToRound(concept)
  local round = concept

  round.answers = {}

  local i = 1
  while round["incorrect_"..i] do
    table.insert(round.answers, round["incorrect_"..i])
    i = i + 1
  end

  table.insert(round.answers, round.correct)

  return round
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PounceRoundGenerator:generateNextValue()
  local concept = nil
  local attemptIndex = 0
  local isDistinctFromPrevious = nil
  repeat
    concept = self.generator:next()
    isDistinctFromPrevious = (concept.correct ~= self.previouslySeenCorrectAnswer)
    attemptIndex = attemptIndex + 1
  until (isDistinctFromPrevious or (attemptIndex >= MAX_UNIQUE_CONCEPT_GENERATION_ATTEMPTS))

  self.previouslySeenCorrectAnswer = concept.correct

  return self.transformFunction(concept)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PounceRoundGenerator.new()
  local self = TransformationGenerator.new(GameSession.conceptGenerator, conceptToRound)
  Mixin.mix(self, PounceRoundGenerator)

  self.previouslySeenCorrectAnswer = nil

  return self
end
