class("PounceInterface")
require("StandardUIBuilder")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_UI_COMPONENTS = {1.0, 1.0, 1.0, 0.8}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PounceInterface:decrementLives()
  self.livesDisplay:decrement()
  return self.livesDisplay:getLives()
end

function PounceInterface:getLivesCount()
  return self.livesDisplay:getLives()
end

function PounceInterface:setPoints(points)
  self.points:setScore(points)
end

function PounceInterface:setMaxCorrectAnswers(maxCorrect)
  self.progressIndicator:setStepCount(maxCorrect)
end

function PounceInterface:notifyCorrectAnswer()
  self.progressIndicator:incrementProgress()
end

function PounceInterface:isProgressComplete()
  return self.progressIndicator:getProgress() >= 1.0
end

function PounceInterface:getProgress()
  return self.progressIndicator:getProgress()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, uiGroup)
  local builder = StandardUIBuilder.new(uiGroup, COLOR_UI_COMPONENTS)

  self.livesDisplay = builder:addLivesDisplay(PounceConfiguration.getNumberOfLives())

  self.pause = builder:addPauseButton()
  self.points = builder:addScoreLabel()
  self.bonusTray = builder:addBonusTray()
end

-- Constructor ------------------------

function PounceInterface.new(uiGroup, progressIndicator)
  local self = {}
  Mixin.mix(self, PounceInterface)
  createLayout(self, uiGroup)

  self.progressIndicator = progressIndicator

  return self
end
