class("PounceScene")

require("scene.question.QuestionView")
require("scene.answers.AnswersView")
require("widgets.HexagonBackground")
require("scene.answers.StalkForeshadowBackground")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SECONDS_BEFORE_AUTO_ROUND_START = 1.5

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function dismissAnswerOverlay(self)
  self.awaitingAnswer = true
  self.questionView:minimize()
  self:hideSpotlightBackground()
end

local function fadeContinueLabelOpacityWithDurationAndDelay(self, opacity, duration, delay)
  local fadeAction = AHSequence.new()

  if (delay > 0) then
    fadeAction:addAction(AHDelay.new(delay))
  end

  fadeAction:addAction(AHOpacityTo.new(self.continueLabel, duration, nil, opacity))

  fadeAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PounceScene:showSpotlightBackground(shouldShowTapMessage, isFirstRound)
  self.blurryStalks:springIn(shouldShowTapMessage, isFirstRound)
end

function PounceScene:hideSpotlightBackground()
  self.blurryStalks:springOut()
end

function PounceScene:setRound(roundInfo)
  self:waitForProgressAnimation()

  self.roundInfo = roundInfo
  self.questionView:setQuestion(roundInfo)
  self.answersView:setAnswers(roundInfo.answers)
  self.questionView:maximize()
  self.awaitingAnswer = false
  local shouldShowTapMessage = true
  self:showSpotlightBackground(shouldShowTapMessage, self.isFirstRound)

  self.isFirstRound = false
end

function PounceScene:readyForAction()
  return self.awaitingAnswer
end

function PounceScene:changeShownAnswerSet()
  Sound.playSoundEffect("switch_terms")
  self.answersView:changeShownAnswerSet()
end

function PounceScene:revealAnswer(isCorrect, submittedAnswer)
  if (not isCorrect) then
    local opacity = 1.0
    local duration = 0.06
    local delay = 1.2
    fadeContinueLabelOpacityWithDurationAndDelay(self, opacity, duration, delay)
  end

  return self.questionView:revealAnswer(self, isCorrect, submittedAnswer)
end

function PounceScene:playCorrectAnimation(answerID)
  Sound.playSoundEffect("correct")
  return self.answersView:playCorrectAnimation(answerID)
end

function PounceScene:playIncorrectAnimation(answerID)
  Sound.playSoundEffect("incorrect")
  return self.answersView:playIncorrectAnimation(answerID)
end

function PounceScene:playWinAnimation()
  Sound.playSoundEffect("goal_reached")
  self.outro:showDefaultGameWinOutro()
  return self.background:doShineAnimation()
end

function PounceScene:playLoseAnimation()
  self.outro:showDefaultGameLossOutro()
  return self.background:doDrainAnimation()
end

function PounceScene:slideOutQuestion(wasCorrect)
  if (not wasCorrect) then
    local opacity = 0
    local duration = 0.04
    local delay = 0
    fadeContinueLabelOpacityWithDurationAndDelay(self, opacity, duration, delay)
  end

  return self.questionView:slideOut()
end

function PounceScene:waitForProgressAnimation()
  self.progressIndicator:waitForHighlightAnimation()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, callbacks)
  local background = HexagonBackground.new(Screen.width, Screen.height)

  local onBackgroundTouched = function()
    if (self.awaitingAnswer) then
      callbacks.screenTappedDuringGame()
    else
      dismissAnswerOverlay(self)
    end
    return true
  end
  Button.makeButton(background, onBackgroundTouched)

  self:insertAtCenter(background)
  self.background = background

  local blurryStalks = StalkForeshadowBackground.new()
  self:insertAtCenter(blurryStalks)
  self.blurryStalks = blurryStalks

  local questionOverlayGroup = Group.newFullScreen() -- Some stalks must go on top of the question

  local answers = AnswersView.new(questionOverlayGroup, callbacks)
  self:insertAtCenter(answers)
  self.answersView = answers

  local question = QuestionView.new()
  self:insertAtLocation(question, Screen.width/2, -100)
  self.questionView = question

  self:insertAtCenter(questionOverlayGroup)

  local continueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 14))
  continueLabel:setColor(unpack({0.41, 0.45, 0.46, 1.0}))
  continueLabel:setOpacity(0.0)
  local minimumContinueLabelY = Screen.safeBottom + continueLabel:getHeight()
  local continueLabelY = self:getHeightPart(0.5) - (question:getRationaleHeight() * 1.5)
  self:insertAtLocation(continueLabel, self:getWidthPart(0.5), math.max(minimumContinueLabelY, continueLabelY))
  self.continueLabel = continueLabel
end

-- Constructor ------------------------

function PounceScene.new(outro, callbacks)
  local self = Group.newFullScreen()
  Mixin.mix(self, PounceScene)

  createLayout(self, callbacks)

  self.progressIndicator = self.background
  self.outro = outro
  self.isFirstRound = true

  return self
end
