class("BeanStalkDroopAnimation")
require("scene.animations.BeanStalkAnimation")
require("scene.animations.PounceAnimation")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getTwerkAmplitude(self)
  local height = self.targetPoints[3][2]
  local heightPercent = math.min(self.beanStalk:getHeightPercent(height) * 3, 1)
  local twerkAmplutide = -3 - 8 * (1 - heightPercent)
  return twerkAmplutide
end

local function getDroopCurvature(self)
  local droopDirection = MathUtils.sign(self.targetPoints[2][1])
  local droopAmplitude = self.targetPoints[2][1]

  local height = self.targetPoints[3][2]
  local heightPercent = self.beanStalk:getHeightPercent(height)
  --local droopMagnitude = math.min(math.abs(droopAmplitude), 40)
  local droopMagnitude = math.abs(droopAmplitude) - (1 - heightPercent) * math.abs(droopAmplitude)

  local droopCurvature = -droopAmplitude + droopDirection * droopMagnitude

  if (math.abs(self.targetPoints[3][1] - self.targetPoints[1][1]) > 5) then
    droopCurvature = self.targetPoints[1][1] - self.targetPoints[3][1]
  end
  return droopCurvature
end

local function getAnimationFunction(self)
  local ease = function(t)
      local overshoot = 2.0
      t = t - 1
      return t * t * ((overshoot + 1) * t + overshoot) + 1
    end

  local springAmplitude = -self.targetPoints[2][1]
  local twerkAmplutide = getTwerkAmplitude(self)
  local minScale = 0.1
  local fallStartTime = 0.12
  local attachmentScaleOutTimeStart = 0.7
  local droopCurvature = getDroopCurvature(self)

  local targetAngle = self.beanStalk:getLastSegmentAngle()

  local function getRotation()
    local angle = self.beanStalk:getLastSegmentAngle()
    local rotation = (angle - targetAngle)/7
    local rotationDirection = MathUtils.sign(rotation)
    rotation = rotationDirection * math.min(math.abs(rotation), 0.5 * math.pi/8)
    rotation = math.deg(rotation)
    return rotation
  end

  local function twerk(t)
    self.targetPoints[3][2] = self.shape[3][2] + twerkAmplutide * t
    self.beanStalk:setPointLocations(self.targetPoints)
    local rotation = getRotation()
    self.beanStalk:setAttachmentRotation(rotation)
  end


  local function fall(t)
    local compressionTime = 1 - t
    local compression = ease(compressionTime)
    compression = compression * (1 - minScale) + minScale
    compression = compression * 1.25 - 0.25
    for index, point in pairs(self.targetPoints) do
      self.currentPoints[index][2] = point[2] * compression
    end

    local heightPercent = self.beanStalk:getHeightPercent(self.currentPoints[#self.currentPoints][2])
    local curvature = self.beanStalk:getCurvatureAtHeightPercent(heightPercent)

    self.currentPoints[2][1] = curvature * (1 - t) + (-droopCurvature) * t
    self.currentPoints[3][1] = - heightPercent * springAmplitude * t + self.targetPoints[3][1]
    self.currentPoints[1][1] = - t * 50
    self.currentPoints[1][2] = self.currentPoints[1][2] - (t) * 75
    self.currentPoints[2][2] = self.currentPoints[2][2] - (t) * 75
    self.currentPoints[3][2] = self.currentPoints[3][2] - (t) * 75
    self.beanStalk:setPointLocations(self.currentPoints)

    local rotation = getRotation()
    self.beanStalk:setAttachmentRotation(rotation)

    if (t > attachmentScaleOutTimeStart) then
      local normalizedTime = (t - attachmentScaleOutTimeStart)/(1 - attachmentScaleOutTimeStart)
      normalizedTime = normalizedTime * normalizedTime
      self.beanStalk:setAttachmentScale(1 - normalizedTime * 0.9)
    end
  end

  return function(t)
    if (t < fallStartTime) then
      local normalizedTime = t/fallStartTime
      twerk(normalizedTime)
    else
      local normalizedTime = (t - fallStartTime)/(1 - fallStartTime)
      fall(normalizedTime)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BeanStalkDroopAnimation:start(animationTime, completeHandler)
  animationTime = animationTime or 0.5
  local animation = PounceAnimation.run(animationTime, self.animationFunction, completeHandler)
  self.animation = animation
  return animation
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function BeanStalkDroopAnimation.new(beanStalk)
  local self = BeanStalkAnimation.new()
  Mixin.mix(self, BeanStalkDroopAnimation)

  self.shape = beanStalk:getPoints()
  self.targetPoints = beanStalk:getPoints()
  self.currentPoints = TableUtils.deepCopy(self.targetPoints)
  self.beanStalk = beanStalk
  self.animationFunction = getAnimationFunction(self)

  return self
end
