class("BeanStalkSpringInAnimation")
require("scene.animations.BeanStalkAnimation")
require("scene.animations.PounceAnimation")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getSpringFunction(self)
  self.beanStalk:setAttachmentScale(1)
  local translateAmplitude = 30
  local minScale = 0.2

  local bounceRandomize = math.random() * 0.3
  local ease = function(t)
    local decay = 1 - math.pow(t, 1)
    local bounce = math.cos(5 * (math.pi - 2.3 - bounceRandomize) * t * (t - 0.1))
    return 1 - decay * bounce
  end

  local function getStemOffset(heightPercent)
    return (1 - heightPercent) * translateAmplitude
  end

  local function getCompressionAtTime(t)
    local compression = t * (1 - minScale) + minScale
    return compression
  end

  local function getCurvatureAtTime(t)
    local compression = getCompressionAtTime(t)
    local height = self.targetPoints[#self.targetPoints][2] * compression
    local heightPercent = self.beanStalk:getHeightPercent(height)
    return self.beanStalk:getCurvatureAtHeightPercent(heightPercent)
  end

  local function getAngleAtTime(t)
    local lastPoint = self.targetPoints[#self.targetPoints]
    local prevPoint = self.targetPoints[#self.targetPoints - 1]
    local lastX, lastY = lastPoint[1], lastPoint[2]
    local prevY = prevPoint[2]
    local curvature = getCurvatureAtTime(t)
    local compression = getCompressionAtTime(t)

    local curLastX, curLastY = lastX, lastY * compression
    local curPrevX, curPrevY = curvature, prevY * compression
    return MathUtils.angleBetweenVectors(curPrevX, curPrevY, 0, Screen.height, curLastX, curLastY)
  end

  local targetAngle = getAngleAtTime(1)
  local curvatureEccentricity = math.min(math.abs(self.targetPoints[2][1])/70, 1)
  local curvatureStartMultiplier = 1 + (1 - math.pow(curvatureEccentricity, 0.4)) * 4
  local curveMidPointEase = 1 - curvatureEccentricity * 0.3
  local curveEndPointEase = 1 + curvatureEccentricity * 0.3

  return function(t)
    local t = ease(t)
    local compression = getCompressionAtTime(t)

    for index, point in pairs(self.targetPoints) do
      local indexCompression = compression
      if (index == 2) then indexCompression = math.pow(compression, curveMidPointEase) end
      if (index == 3) then indexCompression = math.pow(compression, curveEndPointEase) end
      self.currentPoints[index][2] = point[2] * indexCompression - Screen.width/4 * (1 - indexCompression)
    end

    local height = self.currentPoints[#self.currentPoints][2]
    local heightPercent = self.beanStalk:getHeightPercent(height)
    local dx = getStemOffset(heightPercent)

    local dy = 0
    local curvatureAtHeight = self.beanStalk:getCurvatureAtHeightPercent(heightPercent)
    local curvature =  curvatureAtHeight * t + curvatureStartMultiplier * curvatureAtHeight * (1 - t)
    self.currentPoints[2][1] = curvature
    self.beanStalk:translateStem(dx, dy)
    self.beanStalk:setPointLocations(self.currentPoints)


    local angle = self.beanStalk:getLastSegmentAngle()
    local rotation = math.deg(angle - targetAngle)
    local maxAngle = 30
    rotation = MathUtils.sign(rotation) * math.min(math.abs(rotation), maxAngle)
    rotation = maxAngle * (1 - t) + rotation * t
    self.beanStalk:setAttachmentRotation(rotation)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BeanStalkSpringInAnimation:start(animationTime, completeHandler)
  animationTime = animationTime or self.showTime or 0.4

  local animation = PounceAnimation.run(animationTime, self.animationFunction, completeHandler)
  self.animation = animation
  return animation
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function BeanStalkSpringInAnimation.new(beanStalk, showTime)
  local self = BeanStalkAnimation.new()
  Mixin.mix(self, BeanStalkSpringInAnimation)

  self.targetPoints = beanStalk:getShape()
  self.currentPoints = TableUtils.deepCopy(self.targetPoints)
  self.beanStalk = beanStalk
  self.animationFunction = getSpringFunction(self)
  self.showTime = showTime

  return self
end
