class("BeanStalkSpringOutAnimation")
require("scene.animations.BeanStalkAnimation")
require("scene.animations.PounceAnimation")
require("scene.animations.BeanStalkAnimationUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getSpringFunction(self)
  local scaledPoints = TableUtils.deepCopy(self.currentPoints)
  local straightenTime = 0.3
  local compressionStartTime = 0
  local disappearStartTime = 0.9
  local disappearEndTime = 1

  local function straightenOut(t)
    BeanStalkAnimationUtils.approachTargetPoints(t, self.startPoints, self.straightenedPoints, self.currentPoints)
  end

  local function compress(t)
    local compression = 1 - t

    compression = compression * 0.9 + 0.1
    compression = math.pow(compression, 3)

    for index, point in pairs(self.currentPoints) do
      scaledPoints[index][1] = point[1]
      scaledPoints[index][2] = (point[2] * compression) - t * 100
    end
  end

  local startAngle = self.beanStalk:getLastSegmentAngle()

  return function(t)
    if (t <= straightenTime) then
      local adjustedStraightenTime = t / straightenTime
      straightenOut(adjustedStraightenTime)
    end
    if (t > compressionStartTime) then

      local compressionTime = (t - compressionStartTime)/(1 - compressionStartTime)
      compressionTime = math.pow(compressionTime, 1.3) * (compressionTime - straightenTime) * 1.25
      compress(compressionTime)
      self.beanStalk:setPointLocations(scaledPoints)
    else
      self.beanStalk:setPointLocations(self.currentPoints)
    end

    local angle = self.beanStalk:getLastSegmentAngle()
    self.beanStalk:setAttachmentRotation(math.deg((angle - startAngle)/5))

    if (t > disappearStartTime) then
      local disappearTime = (t - disappearStartTime)/(disappearEndTime - disappearStartTime)
      disappearTime = math.min(disappearTime, 1)
      self.beanStalk:setAttachmentScale(1 - disappearTime)
      if (disappearTime >= 1) then
        self.beanStalk:hide()
      end
    end
  end
end



--- Returns the closest point on a segment defined by two points, to a point outside the segment
-- @param (x1, y1) - first point of segment
-- @param (x2, y2) - second point of segment
-- @param (x0, y0) - point outside the segment
local function projectPointToSegment(x1, y1, x2, y2, x0, y0)
  local dx, dy    = x2 - x1, y2 - y1    -- direction vector
  local drx, dry  = x0 - x1, y0 - y1    -- vector from point on line to point outside of line
  local directingLengthSquared = dx * dx + dy * dy -- simplify calculation by not taking square root

  -- Start by calculating the projection of (drx, dry) onto the line.
  local pl = (drx * dx + dry * dy)/directingLengthSquared -- this is a normalized value which will later be divided by an sqrt of the length, so optimize by calculating immediately

  if (pl < 0) then
    return x1, y1
  elseif (pl > 1) then
    return x2, y2
  end

  local px, py = pl * dx, pl * dy -- calculate (drx, dry) projected onto the line
  -- Then we need to calculate the head point of the projection
  local x, y = x1 + px, y1 + py

  return x, y
end

local function getStraightenedOutAngledPoints(startPoints)
  local result = {{}, {}, {}}
  result[1][1] = startPoints[1][1] - 10
  result[1][2] = startPoints[1][2]
  result[3][1] = startPoints[3][1]
  result[3][2] = startPoints[3][2]
  result[2][1], result[2][2] = projectPointToSegment(result[1][1], result[1][2], result[3][1], result[3][2], startPoints[2][1], startPoints[2][2])
  return result
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BeanStalkSpringOutAnimation:start(animationTime, completeHandler)
  animationTime = animationTime or 0.55
  local animation = PounceAnimation.runWithDelay(animationTime, self.delayTime, self.animationFunction, completeHandler)
  self.animation = animation
  return animation
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function BeanStalkSpringOutAnimation.new(beanStalk, delayTime)
  local self = BeanStalkAnimation.new()
  Mixin.mix(self, BeanStalkSpringOutAnimation)

  self.startPoints = beanStalk:getPoints()
  self.currentPoints = TableUtils.deepCopy(self.startPoints)
  self.straightenedPoints = getStraightenedOutAngledPoints(self.startPoints)
  self.beanStalk = beanStalk
  self.animationFunction = getSpringFunction(self)
  self.delayTime = delayTime or 0.3

  return self
end
