class("BeanStalkStretchAnimation")
require("scene.animations.BeanStalkAnimation")
require("scene.animations.PounceAnimation")
require("scene.animations.BeanStalkAnimationUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getAnimationFunction(self)
  local ease = function(t)
      local overshoot = 0.2
      t = t - 1
      return t * t * ((overshoot + 1) * t + overshoot) + 1
    end

  local startPoints = TableUtils.deepCopy(self.currentPoints)
  local springAmplitude = -self.targetPoints[2][1]
  local translateAmplitude = 30
  local straightenUpTime = 0.4
  local slideOutStartTime = 0.6
  local targetAttachmentScale = 1.2
  local attachmentScaleOutTimeStart = 0.7

  local startAngle = self.beanStalk:getLastSegmentAngle()
  local e = 2.718281828459
  local function straightenUp(t)
    t = 2 * math.pow(e, (1 - t + -0.1)) * math.sin(t * (t - 0.4))
    if (t > 0) then
      t = 0.73 * (math.log10(t + 0.05) + 1.35)
    end
    if (t < 0) then
      local rotation = self.beanStalk:getLastSegmentAngle() - startAngle
      self.beanStalk:setAttachmentRotation(math.deg(rotation))
    end
    BeanStalkAnimationUtils.approachTargetPoints(t, startPoints, self.straightenedPoints, self.currentPoints)
    local heightPercent = self.beanStalk:getHeightPercent(self.currentPoints[#self.currentPoints][2])
    local curvature = self.beanStalk:getCurvatureAtHeightPercent(heightPercent)
    self.currentPoints[2][1] = curvature * (1 - t) + t * self.currentPoints[3][1]

    local currentScale = 1 + (targetAttachmentScale - 1) * t
    self.beanStalk:setAttachmentScale(currentScale)
    self.beanStalk:setPointLocations(self.currentPoints)
  end
  local function slideOut(t)
    local compression = ease(1 - t) * 1.5 - 0.5

    for index, point in pairs(self.straightenedPoints) do
      self.currentPoints[index][2] = point[2] * compression - 10 * t
    end

    local heightPercent = self.currentPoints[#self.currentPoints][2]/(self.targetPoints[#self.targetPoints][2]/0.8)
    local dx = (1 - heightPercent) * translateAmplitude

    self.currentPoints[2][1] = (heightPercent * springAmplitude + self.targetPoints[2][1] + dx) * t + self.currentPoints[3][1]
    self.beanStalk:setPointLocations(self.currentPoints)

    if (t > attachmentScaleOutTimeStart) then
      local normalizedTime = (t - attachmentScaleOutTimeStart)/(1 - attachmentScaleOutTimeStart)
      normalizedTime = normalizedTime * normalizedTime
      self.beanStalk:setAttachmentScale(targetAttachmentScale - targetAttachmentScale * normalizedTime * 0.9)
    end
  end

  return function(t)
    if (t < straightenUpTime) then
      local normalizedTime = t/straightenUpTime
      straightenUp(normalizedTime)
    elseif (t > slideOutStartTime) then
      local normalizedTime = (t - slideOutStartTime)/(1 - slideOutStartTime)
      slideOut(normalizedTime)
    end

  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BeanStalkStretchAnimation:start(animationTime, completeHandler)
  animationTime = animationTime or 0.7
  local animation = PounceAnimation.run(animationTime, self.animationFunction, completeHandler)
  self.animation = animation
  return animation
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function BeanStalkStretchAnimation.new(beanStalk)
  local self = BeanStalkAnimation.new()
  Mixin.mix(self, BeanStalkStretchAnimation)


  self.currentPoints = beanStalk:getPoints()
  self.straightenedPoints = TableUtils.deepCopy(self.currentPoints)
  self.targetPoints = TableUtils.deepCopy(self.currentPoints)
  self.straightenedPoints[2][2] = self.straightenedPoints[2][2] * 1.25
  self.straightenedPoints[3][2] = self.straightenedPoints[3][2] * 1.2


  self.beanStalk = beanStalk
  self.animationFunction = getAnimationFunction(self)

  return self
end
