class("PounceAnimation")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PounceAnimation.run(animationTime, animatedFunction, completeHandler)
  local timer = MOAITimer.new()
  timer:setSpan(animationTime)
  MOAICoroutine.new():run(function()
    timer:start()
    while(timer:isActive()) do
      local normalizedTime = timer:getTime()/animationTime
      animatedFunction(normalizedTime)
      coroutine.yield()
    end
    if (completeHandler) then completeHandler() end
  end)

  return timer
end

function PounceAnimation.runWithDelay(animationTime, delay, animatedFunction, completeHandler)
  local timer = MOAITimer.new()
  timer:setSpan(animationTime + delay)
  MOAICoroutine.new():run(function()
    timer:start()
    while(timer:isActive()) do
      local curTime = timer:getTime()
      if (curTime > delay) then
        local normalizedTime = (curTime - delay)/animationTime
        animatedFunction(normalizedTime)
      end
      coroutine.yield()
    end
    if (completeHandler) then completeHandler() end
  end)

  return timer
end

function PounceAnimation.loop(animationTime, animatedFunction)
  local timer = MOAITimer.new()
  timer:setSpan(animationTime)

  local animation = MOAICoroutine.new()
  animation:run(function()
    while(true) do
      if (not timer:isActive()) then
        timer:start()
      end
      local normalizedTime = timer:getTime()/animationTime
      animatedFunction(normalizedTime)
      coroutine.yield()
    end
  end)
  return animation
end

function PounceAnimation.cycle(animationTime, animatedFunction)
  return PounceAnimation.loop(animationTime, function(normalizedTime)
    local cycleTime = math.sin(normalizedTime * 2 * math.pi)/2 + 0.5
    animatedFunction(cycleTime)
  end)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PounceAnimation.new()
  local self = {}
  Mixin.mix(self, PounceAnimation)
  return self
end
