class("AnswerButton")

require("text.SizeConstrainedTextArea")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PARTICLES_OFFSET_BY_BOX_TYPE =
{
  {0.25, 0.13},
  {0.14, 0.25},
  {0.25, 0.17}
}

local MAX_LABEL_WIDTH = 120
local MIN_LABEL_SIZE = 14
local MAX_LABEL_SIZE = 18

local Z_PARTICLES = 100

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getTapQuadrant(self)
  local buttonX, buttonY = self:getWorldBounds()
  local buttonW, buttonH = self:getDims()
  buttonX = buttonX + buttonW/2
  buttonY = buttonY + buttonH/2

  if buttonX > Screen.width/2 then
    if buttonY > Screen.height/2 then
      return 1
    else
      return 4
    end
  else
    if buttonY > Screen.height/2 then
      return 2
    else
      return 3
    end
  end
end

local function showFeedbackParticles(self, quadrant, color)
  local feedback = TapFeedbackParticles.new(color)
  local buttonW, buttonH = self:getDims()
  local particlesOffset = PARTICLES_OFFSET_BY_BOX_TYPE[self.boxType] or {0, 0}
  local offsetX = particlesOffset[1] * buttonW
  local offsetY = particlesOffset[2] * buttonH

  local leftX = offsetX
  local rightX = buttonW - offsetX
  local bottomY = offsetY
  local topY = buttonH - offsetY

  if (quadrant == 1) then
    feedback:setLoc(leftX, bottomY)
    feedback:setScl(-1, -1)
  elseif (quadrant == 2) then
    feedback:setLoc(rightX, bottomY)
    feedback:setScl(1, -1)
  elseif (quadrant == 3) then
    feedback:setLoc(rightX, topY)
    feedback:setScl(1, 1)
  else
    feedback:setLoc(leftX, topY)
    feedback:setScl(-1, 1)
  end

  feedback:burstOut()
  -- Don't insert into current group - insert into screen root group, so it overlays alll the things
  Screen.rootGroup:insert(feedback, Z_PARTICLES)
  -- This linking is important for both setting the right position and getting the particles to move
  -- with the button during stretch animation:
  feedback:setAttrLink(MOAIProp2D.INHERIT_TRANSFORM, self, MOAIProp2D.TRANSFORM_TRAIT)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function AnswerButton:hide()
  self.background:setVisible(false)
  self.text:setColor(0, 0, 0, 0)
  self:setColor(0, 0, 0, 0)
end

function AnswerButton:show()
  self.background:setVisible(true)
  self.text:setColor(1, 1, 1, 1)
  self:setColor(1, 1, 1, 1)
end

function AnswerButton:showFeedbackParticles(color)
  showFeedbackParticles(self, self.lastFeedbackParticleQuadrant, color)
end

function AnswerButton:setHandler(handler)
  self.handler = handler
end

function AnswerButton:getAnswer()
  return self.answer
end

function AnswerButton:getID()
  return self.id
end

function AnswerButton:setID(id)
  self.id = id
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createTextView(answer)
  local height = 24

  local baseAttributes = {
    font = Font.new(FontFaces.DEFAULT_FONT, MAX_LABEL_SIZE)
  }

  local labelAttributes = nil

  local answerView = SizeConstrainedTextArea.new(answer,
                                                 MAX_LABEL_WIDTH,
                                                 height,
                                                 baseAttributes,
                                                 labelAttributes,
                                                 MIN_LABEL_SIZE,
                                                 Alignment.CENTER)
  return answerView
end

local function createLayout(answer, answerBoxTypeIndex)
  assert(answerBoxTypeIndex >= 1 and answerBoxTypeIndex <= 3,
         string.format("Invalid answer box type index provided: %d", answerBoxTypeIndex))

  local background = Sprite.newWithName("pounce_answer_box_0"..answerBoxTypeIndex..".png")
  local backGroundScale = 0.8
  background:setScl(backGroundScale)

  local self = Group.new(background:getEffectiveWidth(), background:getEffectiveHeight())

  self:insertAtCenter(background)
  self.background = background

  local text = createTextView(answer)
  self:insertAtCenter(text)
  self.text = text

  background:setAttrLink(MOAIColor.INHERIT_COLOR, self, MOAIColor.COLOR_TRAIT)

  self.boxType = answerBoxType

  return self
end

function AnswerButton.new(answer, answerBoxTypeIndex)
  if (answerBoxTypeIndex == nil) then
    answerBoxTypeIndex = math.random(3)
  end

  local self = createLayout(answer, answerBoxTypeIndex)
  Mixin.mix(self, AnswerButton)

  local onTouchDown = function()
    self.lastFeedbackParticleQuadrant = getTapQuadrant(self)
    if (self.handler) then
      self.handler(self)
    end
  end
  local onTouchUp = nil
  local onTouchCancel = nil
  Button.makeButton(self, onTouchDown, onTouchUp, onTouchCancel)

  self.answer = answer

  return self
end
