class("BeanStalkAnswer")
require("scene.answers.AnswerButton")
require("scene.answers.TapFeedbackParticles")
require("scene.answers.BeanStalk")
require("scene.animations.BeanStalkAnimation")
require("scene.animations.BeanStalkSpringInAnimation")
require("scene.animations.BeanStalkSpringOutAnimation")
require("scene.animations.BeanStalkDroopAnimation")
require("scene.animations.BeanStalkStretchAnimation")
require("scene.animations.BeanStalkWobbleAnimation")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local STALK_COLOR = {0.1, 0.6, 0.4}

local CORRECT_COLOR = {17/255, 224/255, 70/255, 1}
local INCORRECT_COLOR = {172/255, 61/255, 40/255, 1}
local CORRECT_TEXT_COLOR = {0/255, 78/255, 48/255, 1}
local INCORRECT_TEXT_COLOR = {76/255, 0/255, 29/255, 1}

local BOX_COLORS =
{
  {42/255, 140/255, 152/255},
  {75/255, 132/255, 114/255},
  {27/255, 130/255, 163/255},
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function cancelAnimation(self)
  if (self.animation) then
    self.animation:stop()
    self.animation = nil
  end
end

local function hide(self)
  self:setColor(0, 0, 0, 0)
  self.beanStalk:hide()
  self.answerButton:hide()

end

local function show(self)
  self:setColor(self.color[1], self.color[2], self.color[3], self.color[4])
  self.beanStalk:show()
  self.answerButton:show()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BeanStalkAnswer:activate()
  if (self.activated) then return end
  cancelAnimation(self)
  self.answerButton:setTouchEnabled(true)
  show(self)
  self.activated = true
end

function BeanStalkAnswer:show()
  show(self)
end

function BeanStalkAnswer:deactivate()
  if (not self.activated) then return end
  self.answerButton:setTouchEnabled(false)
  -- Don't hide on deactivation - possible animations running, use self:hide()
  self.activated = false
end

function BeanStalkAnswer:deactivateImmediately()
  cancelAnimation(self)
  self.answerButton:setTouchEnabled(false)
  hide(self)
  self.activated = false
end

function BeanStalkAnswer:hide()
  cancelAnimation(self)
  hide(self)
end

function BeanStalkAnswer:getCurrentAnimation()
  return self.animation
end

function BeanStalkAnswer:springIn(showTime, completeHandler)
  if (self.onScreen) then return end
  self.onScreen = true
  self.animation = self.beanStalk:animate(BeanStalkSpringInAnimation, showTime, completeHandler)
  return self.animation
end

function BeanStalkAnswer:springOut(delay, completeHandler)
  --if (not self.onScreen) then return end
  self.onScreen = false

  self.animation = self.beanStalk:animate(BeanStalkSpringOutAnimation, delay, completeHandler)
  return self.animation
end

function BeanStalkAnswer:springOutImmediately()
  if (not self.onScreen) then return end
  self.onScreen = false

  local completeHandler = nil
  self.animation = self.beanStalk:animate(BeanStalkSpringOutAnimation, 0, completeHandler)
  return self.animation
end

local function animateOut(self, animationType, textColor, nonTextColor, completeHandler, showParticles)
  if (not self.onScreen) then return end
  self.onScreen = false

  if showParticles then
    self.answerButton:showFeedbackParticles(nonTextColor)
  end

  local animTime = 0.1
  AHColorTo.new(self, animTime, nil, nonTextColor):start()
  AHColorTo.new(self.answerButton.text, animTime, nil, textColor):start()
  self.animation = self.beanStalk:animate(animationType, nil, completeHandler)
  return self.animation
end

function BeanStalkAnswer:droopOut(completeHandler)
  local nonTextColor = INCORRECT_COLOR
  local textColor = INCORRECT_TEXT_COLOR
  return animateOut(self, BeanStalkDroopAnimation, textColor, nonTextColor, completeHandler, false)
end

function BeanStalkAnswer:stretchOut(completeHandler)
  local nonTextColor = CORRECT_COLOR
  local textColor = CORRECT_TEXT_COLOR
  return animateOut(self, BeanStalkStretchAnimation, textColor, nonTextColor, completeHandler, true)
end

function BeanStalkAnswer:wobble()
  self.beanStalk:animate(BeanStalkWobbleAnimation)
end

function BeanStalkAnswer:setID(id)
  self.answerButton:setID(id)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createBeanStalk(self, stalkParams)
  local beanStalk = BeanStalk.new(stalkParams)

  local rotation = stalkParams.rotation or 0
  rotation = math.rad(rotation)

  local yOffset = (math.sin(rotation) - math.cos(rotation)) * (Screen.height/2 + 10)
  local stalkX = self:getWidthPart(0.5)
  local stalkY = self:getHeightPart(0.5) + yOffset
  beanStalk:setLoc(stalkX, stalkY)

  return beanStalk
end


local function createLayout(answer, stalkParams)
  local answerButton = AnswerButton.new(answer, stalkParams.boxType)

  local width, height = answerButton:getDims()
  local self = Group.new(width, height)

  local stalk = createBeanStalk(self, stalkParams)

  local colorIndex = stalkParams.colorIndex or math.random(#BOX_COLORS)
  colorIndex = math.min(colorIndex, #BOX_COLORS)

  self.beanStalk = stalk
  self.answerButton = answerButton

  answerButton:setAttrLink(MOAIColor.INHERIT_COLOR, self, MOAIColor.COLOR_TRAIT)
  stalk:setAttrLink(MOAIColor.INHERIT_COLOR, self, MOAIColor.COLOR_TRAIT)

  self.color = stalkParams.color or BOX_COLORS[colorIndex] or STALK_COLOR
  self:setColor(self.color[1], self.color[2], self.color[3], self.color[4])

  self:insert(stalk)
  stalk:attachObject(answerButton)
  self.colorIndex = colorIndex

  return self
end

function BeanStalkAnswer.new(answer, handler, stalkParams)
  local self = createLayout(answer, stalkParams)
  Mixin.mix(self, BeanStalkAnswer)

  self.answer = answerv
  self.activated = true
  self:deactivateImmediately()
  self.answerButton:setHandler(handler)

  return self
end
