class("TapFeedbackParticles")
require("scene.shapes.DashedArc")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TapFeedbackParticles:burstOut(animationTime)
  local timer = MOAITimer.new()
  animationTime = animationTime or 0.5
  timer:setSpan(animationTime)
  local minR, maxR = self.radius/2, self.radius * 2
  local minW, maxW = self.dashWidth/8, self.dashWidth
  local minH, maxH = self.dashHeight/8, self.dashHeight

  local arc = self.arc

  MOAICoroutine.new():run(function()
    timer:start()
    while(timer:isActive()) do
      local time = timer:getTime()/animationTime
      local sizeTime = 1 - time
      local r = minR + (maxR - minR) * time
      local w = minW + (maxW - minW) * sizeTime
      local h = minH + (maxH - minH) * sizeTime

      arc:setRadius(r)
      arc:setDashSize(w, h)

      coroutine.yield()
    end
    self:removeSelf()
  end)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, color)
  local radius = 50
  local dashWidth = 20
  local dashHeight = 6
  local startAngle = 0
  local endAngle = 90
  local dashCount = 4

  local arc = DashedArc.new(radius, dashWidth, dashHeight, dashCount, startAngle, endAngle)
  arc:setColor(color[1], color[2], color[3], color[4])

  self.radius = radius
  self.dashWidth, self.dashHeight = dashWidth, dashHeight
  self.arc = arc

  self:insertAtCenter(arc)
end

function TapFeedbackParticles.new(color)
  local self = Group.newFullScreen()
  Mixin.mix(self, TapFeedbackParticles)

  createLayout(self, color)

  return self
end
