class("CollectionOfWordsWithBlanks")
require("scene.question.WordWithBlank")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function indicesOfBlankInLine(line)
  local text = line.renderedCharacters
  if (text == nil) then return end
  local first = string.find(text, "_")
  local last = string.find(text, "_[^_]*$") or first
  return first, last
end

local function getBoundingBoxForCharacterRangeInLine(line, first, last)
  local firstCharacter = line[first]
  local lastCharacter = line[last]
  local xMin = firstCharacter.xMin
  local yMin = firstCharacter.yMin
  local xMax = lastCharacter.xMax
  local yMax = lastCharacter.yMax

  xMin = xMin/Host.contentScale
  yMin = yMin/Host.contentScale
  xMax = xMax/Host.contentScale
  yMax = yMax/Host.contentScale
  return {xMin = xMin, yMin = yMin, xMax = xMax, yMax = yMax}
end

local function findBlankBoundingBoxes(glyphs)
  local boundingBoxes = {}
  for lineIndex, line in ipairs(glyphs) do
    local blankStart, blankEnd = indicesOfBlankInLine(line)
    if (blankStart and blankEnd) then
      boundingBoxes[#boundingBoxes + 1] = getBoundingBoxForCharacterRangeInLine(line, blankStart, blankEnd)
    end
  end
  return boundingBoxes
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function CollectionOfWordsWithBlanks:getRevealAnswerAnimation(submittedAnswer)
  local submittedAnswerComponents = StringUtils.split(submittedAnswer, ",")
  local animations = {}
  local delaySeconds = 0
  local delayPerQuestion = 0.05

  for index, correctComponent in ipairs(self.answerComponents) do
    local submittedComponent = submittedAnswerComponents[index]
    local isCorrect = (submittedComponent == correctComponent)
    local curAnimation = nil
    if (isCorrect) then
      curAnimation = self.answers[index]:showAsCorrect(delaySeconds)
    else
      local wordWithBlank = self.answers[index]
      wordWithBlank:setText(submittedAnswer)
      curAnimation = wordWithBlank:showAsIncorrect(delaySeconds)
    end
    animations[#animations + 1] = curAnimation
    delaySeconds = delaySeconds + delayPerQuestion
  end

  local animation = AHSpawn.new(unpack(animations))

  return animation
end

function CollectionOfWordsWithBlanks:getSettleAnswersDownAnimation(time)
  local animations = {}
  for index, answer in ipairs(self.answers) do
    animations[#animations + 1] = answer:getSettleDownAnimation(time)
  end
  local animation = AHSpawn.new(unpack(animations))
  return animation
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createAnswers(self, answer, glyphTable, fontSize, boxHeight, colorIndex)
  local answers = {}
  local blankBoundingBoxes = findBlankBoundingBoxes(glyphTable)
  local answerComponents = StringUtils.split(answer, ",")
  assert(#blankBoundingBoxes == #answerComponents,
         "The concept question should have as many blanks as components in the correct answer. "
          .. "bounding boxes: " .. #blankBoundingBoxes
          .. ", answer components: " .. #answerComponents
          .. ", answer: \"" .. answer .. "\"")

  local minFontSize = fontSize
  local index = 1
  while(index <= #blankBoundingBoxes) do
    local boundingBox = blankBoundingBoxes[index]
    local correspondingAnswer = answerComponents[index]
    local wordWithBlank = WordWithBlank.new(boundingBox, correspondingAnswer, fontSize, minFontSize, colorIndex)
    if (wordWithBlank:getFontSize() < minFontSize) then
      index = 1
      minFontSize = wordWithBlank:getFontSize()
      for _, answerValue in pairs(answers) do
        answerValue:removeSelf()
      end
      answers = {}
    else
      local x = boundingBox.xMin + self:getWidthPart(0.5)
      local y = boxHeight - (boundingBox.yMin)  + self:getHeightPart(0.5)
      self:insertAtLocation(wordWithBlank, x, y)
      answers[#answers + 1] = wordWithBlank
      index = index + 1
    end
  end

  self.answers = answers
  self.answerComponents = answerComponents
end

function CollectionOfWordsWithBlanks.new(answer, glyphs, fontSize, boxHeight, colorIndex)
  local self = Group.newFullScreen()
  Mixin.mix(self, CollectionOfWordsWithBlanks)

  createAnswers(self, answer, glyphs, fontSize, boxHeight, colorIndex)

  return self
end
