--- PounceRationaleView
-- @author Created by Joe Poka on June 8, 2016.

class("PounceRationaleView")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getCornerRadius(length)
  local cornerRadius = length * 0.025
  return cornerRadius
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PounceRationaleView:show()
  local correctText = self.answer
  self.correctLabel:setText(correctText)

  local correctDefinition = self.definitions[self.answer]
  self.correctDefinitionLabel:setText(correctDefinition)

  -- self.submittedAnswer is nil when question times out
  local someAnswerWasSubmitted = (self.submittedAnswer ~= nil)
  if (someAnswerWasSubmitted) then
    local incorrectText = self.submittedAnswer
    self.incorrectLabel:setText(incorrectText)
    self.incorrectLabel:setVisible(true)

    local incorrectDefinition = self.definitions[self.submittedAnswer]
    self.incorrectDefinitionLabel:setText(incorrectDefinition)
    self.incorrectDefinitionLabel:setVisible(true)
  else
    self.incorrectLabel:setVisible(false)
    self.incorrectDefinitionLabel:setVisible(false)
  end

  -- position correct word and definition
  local height = self:getHeight()
  local y = height * (someAnswerWasSubmitted and 0.76 or 0.57)
  self.correctLabel:setY(y)

  local verticalSpacing = height * 0.03
  local halfCorrectHeight = self.correctLabel:getHeightPart(0.5)
  local halfCorrectDefinitionHeight = self.correctDefinitionLabel:getHeightPart(0.5)
  y = y - halfCorrectHeight - verticalSpacing - halfCorrectDefinitionHeight
  self.correctDefinitionLabel:setY(y)

  -- position incorrect word and definition
  verticalSpacing = height * 0.1
  local halfIncorrectHeight = self.incorrectLabel:getHeightPart(0.5)
  y = y - halfCorrectDefinitionHeight - verticalSpacing - halfIncorrectHeight
  self.incorrectLabel:setY(y)

  verticalSpacing = height * 0.03
  local halfIncorrectDefinitionHeight = self.incorrectDefinitionLabel:getHeightPart(0.5)
  y = y - halfIncorrectHeight - verticalSpacing - halfIncorrectDefinitionHeight
  self.incorrectDefinitionLabel:setY(y)

  local duration = 0.3
  local slideDown = AHLocationYBy.new(self, duration, AHEaseSimpleIn.new(), -self.slideAmountY)
  slideDown:start()

  return slideDown
end

function PounceRationaleView:hide()
  self.answer = nil
  self.submittedAnswer = nil

  local duration = 0.2
  local slideUp = AHLocationYBy.new(self, duration, AHEaseSimpleIn.new(), self.slideAmountY)
  local hideIncorrectItems = function()
    self.incorrectLabel:setVisible(false)
  end

  local sequence = AHSequence.new(slideUp, hideIncorrectItems)
  sequence:start()
end

function PounceRationaleView:setAnswer(answer)
  self.answer = answer
end

function PounceRationaleView:setSubmittedAnswer(submittedAnswer)
  self.submittedAnswer = submittedAnswer
end

function PounceRationaleView:setDefinitions(definitions)
  self.definitions = definitions
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()
  local cornerRadius = getCornerRadius(width)

  local borderColor = {1.0, 1.0, 1.0, 1.0}
  local lineWidth = width * 0.01
  local blurMargin = 0.0
  local border = HollowRoundedRectangle.new(width, height, cornerRadius, borderColor, lineWidth, blurMargin)
  self:insertAtCenter(border)

  local correctColor = {0.09, 0.75, 0.3, 1.0}
  local incorrectColor = {0.94, 0.2, 0.24, 1.0}
  local definitionColor = {1.0, 1.0, 1.0, 1.0}

  local wordFont = Font.new(FontFaces.DIN_OT_MEDIUM, 14.0)
  local definitionFont = Font.new(FontFaces.DIN_OT_MEDIUM, 12.0)

  local innerWidth = width * 0.85
  self.correctLabel = TextArea.new("", wordFont, innerWidth)
  self.correctLabel:setColor(unpack(correctColor))
  self:insertAtCenter(self.correctLabel)

  self.correctDefinitionLabel = TextArea.new("", definitionFont, innerWidth)
  self.correctDefinitionLabel:setColor(unpack(definitionColor))
  self:insertAtCenter(self.correctDefinitionLabel)

  self.incorrectLabel = TextArea.new("", wordFont, innerWidth)
  self.incorrectLabel:setColor(unpack(incorrectColor))
  self:insertAtCenter(self.incorrectLabel)

  self.incorrectDefinitionLabel = TextArea.new("", definitionFont, innerWidth)
  self.incorrectDefinitionLabel:setColor(unpack(definitionColor))
  self:insertAtCenter(self.incorrectDefinitionLabel)
end

-- Constructor ------------------------

function PounceRationaleView.new(width, height)
  local cornerRadius = getCornerRadius(width)
  local backgroundColor = {0.05, 0.1, 0.05, 1.0}

  local self = FilledRoundedRectangle.new(width, height, cornerRadius, backgroundColor)
  Mixin.mix(self, PounceRationaleView)

  createLayout(self)

  self.slideAmountY = self:getHeightPart(0.95)

  return self
end
