class("WordWithBlank")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CORRECT_ANSWER_COLORS =
{
  {180/255, 255/255, 130/255},
  {139/255, 252/255, 166/255}
}

local WRONG_ANSWER_COLOR = {0.9, 0.3, 0.3}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createAnswerAnimation(self, answerColor, delaySeconds)
  local answerView = self.answerView
  local showTime = 0.07

  answerView:setScl(1, 0)

  local fadeUnderscoreColor = AHColorTo.new(self.underscore, showTime, nil, answerColor)
  fadeUnderscoreColor:start()

  answerView:setColor(unpack(answerColor))

  local delayBeforeScaling = AHDelay.new(delaySeconds or 0)
  local scaleAnswer = AHScaleTo.new(answerView, showTime, nil, 1, 1)

  local animation = AHSequence.new(
    delayBeforeScaling,
    scaleAnswer
  )

  return animation
end

local function createPopinAnimation(self, color, delaySeconds)
  local animation = createAnswerAnimation(self, color, delaySeconds)
  return animation
end

local function createSettleDownAnimation(self, time, delaySeconds)
  local waitBeforeMove = AHDelay.new(delaySeconds)
  local move = AHLocationYBy.new(self.answerView, time, nil, -2)
  return AHSequence.new(
                        waitBeforeMove,
                        move
                        )
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WordWithBlank:setText(text)
  self.answerView:setText(text)
end

function WordWithBlank:showAsCorrect(delaySeconds)
  return createPopinAnimation(self, CORRECT_ANSWER_COLORS[self.colorIndex], delaySeconds)
end

function WordWithBlank:showAsIncorrect(delaySeconds)
  return createPopinAnimation(self, WRONG_ANSWER_COLOR, delaySeconds)
end

function WordWithBlank:getSettleDownAnimation(time, delaySeconds)
  delaySeconds = delaySeconds or 0
  time = time or 0.2
  return createSettleDownAnimation(self, time, delaySeconds)
end

function WordWithBlank:getFontSize()
  return self.fontSize
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, boundingBox, word, fontSize, maxFontSizeOverride)
  local w, h = boundingBox.xMax - boundingBox.xMin, boundingBox.yMax - boundingBox.yMin
  h = 26
  local x = w/2
  local y = 0
  local blankRectangleWidth, blankRectangleHeight = w, 1.5
  local maxFontSize = maxFontSizeOverride or fontSize
  local minFontSize = fontSize - 6

  local blankRectangle = FilledRectangle.new(blankRectangleWidth, blankRectangleHeight)
  local answerView = DeprecatedLabel.newSingleLineFitted(word.."", w, h, FontFaces.DEFAULT_FONT, maxFontSize, minFontSize, Alignment.CENTER, Alignment.CENTER)
  fontSize = answerView.fontRenderEngine:getFontSize()
  answerView:setColor(1, 1, 1)
  blankRectangle:setColor(1, 1, 1)

  answerView:setAnchorPointBottom()
  blankRectangle:setLoc(x, y - blankRectangleHeight/2)
  answerView:setLoc(x, y + 0)
  answerView:setScl(1, 0)

  self:insert(answerView)
  self:insert(blankRectangle)

  self.answerView = answerView
  self.underscore = blankRectangle
  self.fontSize = fontSize
end

function WordWithBlank.new(boundingBox, word, fontSize, maxFontSizeOverride, colorIndex)
  local self = Group.newDeprecated()
  Mixin.mix(self, WordWithBlank)
  self.colorIndex = colorIndex or 1
  self.word = word
  createLayout(self, boundingBox, word, fontSize, maxFontSizeOverride)
  return self
end
