class("DashedArc")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DASH_COLOR = {1, 1, 1, 1}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DashedArc:setRadius(radius)
  local startAngle = self.startAngle
  local angleAmplitude = self.endAngle - startAngle
  local dAngle = angleAmplitude/(self.dashCount - 1)
  for i, dash in ipairs(self.dashes) do
    local angleDegrees = startAngle + (i - 1) * dAngle
    local angleRadians = math.rad(angleDegrees)

    local x = math.cos(angleRadians) * radius + self:getWidthPart(0.5)
    local y = math.sin(angleRadians) * radius + self:getHeightPart(0.5)

    dash:setRot(angleDegrees)
    dash:setLoc(x, y)
  end
end

function DashedArc:setDashSize(width, height)
  for i, dash in ipairs(self.dashes) do
    dash:setScl(width, height)
  end
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, dashWidth, dashHeight, dashCount)
  local dashes = {}

  for i = 1, dashCount do
    local dash = FilledRectangle.new(1, 1, DASH_COLOR)
    dash:setScl(dashWidth, dashHeight)
    dash:setAttrLink(MOAIColor.INHERIT_COLOR, self, MOAIColor.COLOR_TRAIT)
    self:insertAtCenter(dash)
    dashes[#dashes + 1] = dash
  end

  self.dashes = dashes
end


function DashedArc.new(radius, dashWidth, dashHeight, dashCount, startAngle, endAngle)
  radius = radius or 100
  dashWidth = dashWidth or 10
  dashHeight = dashHeight or 10
  startAngle = startAngle or 0
  endAngle = endAngle or 360
  dashCount = dashCount or 12

  local self = Group.new(dashWidth, dashHeight)
  Mixin.mix(self, DashedArc)

  self.startAngle, self.endAngle = startAngle, endAngle
  self.dashCount = dashCount
  createLayout(self, dashWidth, dashHeight, dashCount)
  self:setRadius(radius)

  return self
end
