class("DynamicSpline")

require("graphics.ShapeUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DynamicSpline:setPoints(...)
  local skeleton = arg

  if (type(skeleton[1]) == "table") then
    skeleton = ShapeUtils.flattenCoordPairs(skeleton[1])
  end

  self.skeleton = skeleton
  local points = MOAIMath.pointsForCardinalSpline(skeleton, self.pointsPerSegment, self.tension)
  self.points = points
end

function DynamicSpline:setThickness(thickness)
  self.thickness = thickness or 1
end

function DynamicSpline:setTension(tension)
  self.tension = tension
  local points = MOAIMath.pointsForCardinalSpline(self.skeleton, self.pointsPerSegment, self.tension)
  self.points = points
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function DynamicSpline.new(pointsPerSegment, tension, thickness, ...)
  local self = Group.newFullScreen()
  Mixin.mix(self, DynamicSpline)

  self.pointsPerSegment = pointsPerSegment or 24
  self.tension = tension or -1
  self:setThickness(thickness)
  self:setPoints(unpack(arg))

  local function drawBeveledLine()
    local blurMargin = 0
    MOAIDraw.drawRoundBeveledLine(self.points, self.thickness, blurMargin)
  end

  local width = Screen.width
  local height = Screen.height

  local deck = MOAIScriptDeck.new()
  deck:setDrawCallback(drawBeveledLine)

  self:setDeck(deck)
  self:setAnchorPointCenter()

  return self
end
