--- PounceValidator
-- @author Created by Anton Morant on April 16, 2014.
-- ConceptValidator for Pounce.

class("PounceValidator")

require("validators.ConceptValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'question',
  'correct',
  'incorrect_1',
  'incorrect_2',
  'incorrect_3',
  'incorrect_4',
  'correct_definition',
  'incorrect_definition_1',
  'incorrect_definition_2',
  'incorrect_definition_3',
  'incorrect_definition_4'
}

local BLANK_CHAR = "_"
local ANSWER_SEPARATOR_CHAR = ","

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validateQuestion(self, text)
  local _, count = text:gsub(BLANK_CHAR, "")
  self:expectTrue(count == 1, "Question should have exactly one blank: " .. text)

  self:testForEndingPunctuation(text)
end

local function validateAnswer(self, text)
  local answerComponents = StringUtils.split(text, ANSWER_SEPARATOR_CHAR)
  local count = #answerComponents

  self:expectTrue(count == 1, "Answer should have exactly one component: " .. text)

  self:expectStringIsLettersOnly(text, "answer", ANSWER_SEPARATOR_CHAR)
end

local function validateNoRedundancies(self, concept)
  local tableA = {concept.correct, concept.incorrect_1, concept.incorrect_2, concept.incorrect_3, concept.incorrect_4}
  local tableB = TableUtils.createSetFromArray(tableA)
  local sizeOfA = TableUtils.getSize(tableA)
  local sizeOfB = TableUtils.getSize(tableB)
  local duplicateFound = (sizeOfA ~= sizeOfB)

  self:expectTrue(duplicateFound == false, "One of the answer options is the same as another. Must all be unique.")
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PounceValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)

  validateQuestion(self, concept.question)
  self:validateCharactersAdjacentToBlanks(concept.question, BLANK_CHAR)
  validateAnswer(self, concept.correct)
  validateAnswer(self, concept.incorrect_1)
  validateAnswer(self, concept.incorrect_2)
  validateAnswer(self, concept.incorrect_3)
  validateAnswer(self, concept.incorrect_4)
  validateNoRedundancies(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function PounceValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, PounceValidator)

  return self
end
