--- PronunciationCardSurfaceButtons
-- @author Created by andrewc on August 24, 2021.

class("PronunciationCardSurfaceButtons")

require("PronunciationCommon")
require("widgets.ValidityButtonBar")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH_TO_HEIGHT_RATIO = 0.2667

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWillAnimateAffirmativeButton(sideIndex)
  return (sideIndex == PronunciationCommon.SIDE_INDEX.RIGHT)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PronunciationCardSurfaceButtons:markButtonCorrect(sideIndex)
  return self:playCorrectAnimation(getWillAnimateAffirmativeButton(sideIndex))
end

function PronunciationCardSurfaceButtons:markButtonIncorrect(sideIndex)
  return self:playIncorrectAnimation(getWillAnimateAffirmativeButton(sideIndex))
end

function PronunciationCardSurfaceButtons:runTapAnimation(sideIndex, isCorrect)
  if isCorrect then
    return self:markButtonCorrect(sideIndex)
  else
    return self:markButtonIncorrect(sideIndex)
  end
end

function PronunciationCardSurfaceButtons:runFadeButtonsAnimation()
  return self:createFadeOutAction():start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function PronunciationCardSurfaceButtons.new(width, buttonCallback)
  local styleDescriptor = {
    defaultBackingColor = PronunciationCommon.COLOR_BUTTON,
    correctBackingColor = PronunciationCommon.COLOR_OUTLINE,
    centerDividerColor = PronunciationCommon.COLOR_OUTLINE,
    incorrectBackingColor = PronunciationCommon.COLOR_OUTLINE,
    defaultTextColor = PronunciationCommon.COLOR_OUTLINE,
    correctTextColor = PronunciationCommon.COLOR_BLACK,
    incorrectTextColor = PronunciationCommon.COLOR_BLACK,
    outlineColor = PronunciationCommon.COLOR_OUTLINE,
    outlineWidth = PronunciationCommon.LINE_WIDTH,
    outlineBlur = PronunciationCommon.BLUR_MARGIN,
    width = width,
    height = width * WIDTH_TO_HEIGHT_RATIO
  }
  local self = ValidityButtonBar.newWithStyleDescriptor(styleDescriptor)
  Mixin.mix(self, PronunciationCardSurfaceButtons)

  self:setNegativeButtonCallback(function() buttonCallback(PronunciationCommon.SIDE_INDEX.LEFT) end)
  self:setAffirmativeButtonCallback(function() buttonCallback(PronunciationCommon.SIDE_INDEX.RIGHT) end)

  self:prepareForShowAction()

  return self
end
