class("PronunciationConfiguration")

require("data.generators.LearningFocusedRoundGenerator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

PronunciationConfiguration.POINTS_PER_CORRECT_RESPONSE = 1000
PronunciationConfiguration.POINTS_PER_LIVES_BONUS = 500
PronunciationConfiguration.POINTS_PER_FULL_SPEED_BONUS = 200
PronunciationConfiguration.INITIAL_LIVES_COUNT = 4
PronunciationConfiguration.CORRECT_RESPONSES_NEEDED_TO_WIN = 8
PronunciationConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 2.0
PronunciationConfiguration.SPEED_BONUS_DECAY_START = 0.5

PronunciationConfiguration.CONCEPTS_REMAINING_BEFORE_RESURFACING_MISTAKES = math.floor(
  PronunciationConfiguration.CORRECT_RESPONSES_NEEDED_TO_WIN *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING
)

local EXTRA_LIVES_NEEDED_RANK_TWO = 2
local EXTRA_LIVES_NEEDED_RANK_THREE = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function minPointsWithExtraLives(extraLives)
  local correctNeeded = PronunciationConfiguration.CORRECT_RESPONSES_NEEDED_TO_WIN
  local livesCount = PronunciationConfiguration.INITIAL_LIVES_COUNT
  local minCorrectAnswers = correctNeeded - livesCount + extraLives + 1
  local answerPoints = minCorrectAnswers * PronunciationConfiguration.POINTS_PER_CORRECT_RESPONSE
  local livesPoints = extraLives * PronunciationConfiguration.POINTS_PER_LIVES_BONUS

  return answerPoints + livesPoints
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PronunciationConfiguration.getPassScore()
  return minPointsWithExtraLives(0)
end

function PronunciationConfiguration.getRankTwoScore()
  return minPointsWithExtraLives(EXTRA_LIVES_NEEDED_RANK_TWO)
end

function PronunciationConfiguration.getRankThreeScore()
  return minPointsWithExtraLives(EXTRA_LIVES_NEEDED_RANK_THREE)
end

function PronunciationConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Easy = 0.95,
                                      Medium = 0.05}},

    {key = 0.075, valueProbabilities = {Easy = 0.85,
                                        Medium = 0.15}},

    {key = 0.15, valueProbabilities = {Easy = 0.40,
                                       Medium = 0.60}},

    {key = 0.225, valueProbabilities = {Easy = 0.2,
                                        Medium = 0.7,
                                        Hard = 0.10}},

    {key = 0.3, valueProbabilities = {Easy = 0.05,
                                      Medium = 0.80,
                                      Hard = 0.15}},

    {key = 0.375, valueProbabilities = {Medium = 0.68,
                                        Hard = 0.32}},

    {key = 0.45, valueProbabilities = {Medium = 0.50,
                                       Hard = 0.50}},

    {key = 0.525, valueProbabilities = {Medium = 0.40,
                                        Hard = 0.60}},

    {key = 0.6, valueProbabilities = {Medium = 0.30,
                                      Hard = 0.70}},

    {key = 0.675, valueProbabilities = {Medium = 0.20,
                                        Hard = 0.80}},

    {key = 0.75, valueProbabilities = {Medium = 0.10,
                                       Hard = 0.90}},

    {key = 0.8, valueProbabilities = {Medium = 0.05,
                                      Hard = 0.95}},

    {key = 1.0, valueProbabilities = {Hard = 1.0}}
  }
end
