--- PronunciationPlayIcon
-- @author Created by karl on August 25, 2014.

class("PronunciationPlayIcon")

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createOpacityToAction(group, opacity)
  local fadeDuration = 0.1
  return AHOpacityTo.new(group, fadeDuration, nil, opacity)
end

local function createOpacityPulseAction(group)
  local pulseDuration = 0.5
  local opacityHalf = 0.5
  local opacityFull = 1.0
  return AHLoop.new(AHSequence.new(AHOpacityTo.new(group, pulseDuration, nil, opacityHalf),
                                   AHOpacityTo.new(group, pulseDuration, nil, opacityFull)))
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PronunciationPlayIcon:showPressed(isPressed)

  if (isPressed) then
    local downAction = AHSpawn.new(
      AHSequence.new(
        AHSpawn.new(createOpacityToAction(self.labelUpState, 0.0),
                    createOpacityToAction(self.triangleUpState, 0.0)
        ),
        createOpacityToAction(self.labelDownState, 1.0)
      ),
      createOpacityPulseAction(self.outline)
    )
    self.pressedStateFadeInterrupt:insert(downAction)
  else
    local upAction = AHSpawn.new(
      AHSequence.new(
        createOpacityToAction(self.labelDownState, 0.0),
        AHSpawn.new(createOpacityToAction(self.labelUpState, 1.0),
                    createOpacityToAction(self.triangleUpState, 1.0)
        )
      ),
      createOpacityToAction(self.outline, 1.0)
    )
    self.pressedStateFadeInterrupt:insert(upAction)
  end
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createTriangle(sideLength, labelColor)
  local bx, by = 0, 0
  local mx, my = sideLength * math.cos(math.pi * 0.2), math.floor(sideLength * 0.5)
  local tx, ty = 0, sideLength
  return FilledTriangle.new(bx, by, mx, my, tx, ty, labelColor)
end

local function createLabel(text, fontSize, labelColor)
  local label = Label.new(text, Font.new(FontFaces.DIN_OT_MEDIUM, fontSize))
  label:setColor(unpack(labelColor))
  return label
end

local function createOutline(width, height, outlineColor)
  local cornerRadius = (height / 2) - 1 -- Fudge the corner radius to avoid a graphical glitch
  local lineWidth = height * 0.04
  local blurMargin = 0.5
  return HollowRoundedRectangle.new(width, height, cornerRadius, outlineColor, lineWidth, blurMargin)
end

local function createLayout(self, height, color)
  local sideLength = math.floor(height * 0.3)
  local triangleUpState = createTriangle(sideLength, color)
  self.triangleUpState = triangleUpState

  local fontSize = sideLength
  local labelUpState = createLabel("PLAY", fontSize, color)
  self.labelUpState = labelUpState

  local spacerSize = sideLength * 0.2
  local extraPadding = triangleUpState:getWidth() * 3.5
  local width = extraPadding + triangleUpState:getWidth() + spacerSize + labelUpState:getWidth() + extraPadding

  local elementSpacing = 4
  local rowUpState = Row.new(elementSpacing)
  rowUpState:insert(triangleUpState)
  rowUpState:insertSpacer(spacerSize)
  rowUpState:insert(labelUpState)
  rowUpState:alignChildren()
  self:insertAtCenter(rowUpState)

  local labelDownState = createLabel("PLAYING", fontSize, color)
  labelDownState:setOpacity(0)
  self:insertAtCenter(labelDownState)
  self.labelDownState = labelDownState

  local outline = createOutline(width, height, color)
  self:insertAtCenter(outline)
  self.outline = outline
end

-- Constructor ------------------------

function PronunciationPlayIcon.new(height, color)
  local self = Group.new(height, height)
  Mixin.mix(self, PronunciationPlayIcon)

  createLayout(self, height, color)

  self.pressedStateFadeInterrupt = ActionInterrupt.new()

  return self
end
