--- PronunciationSquaresEmitter
-- @author Created by karl on July 25, 2014.

class("PronunciationSquaresEmitter")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SPEED = 250

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function makeSquare(color)
  local sprite = Sprite.newWithName("particle.png")
  sprite:setColor(unpack(color))
  return sprite
end

local function emitSquare(self, angle, distance, duration)
  return runInCoroutine(function()
    local square = makeSquare(self.color)
    self:insert(square)
    local x = math.cos(angle) * distance
    local y = math.sin(angle) * distance
    AHLocationTo.new(square, duration, AHEaseSineOut.new(), x, y):start()
    waitFor(duration * 0.1)
    AHRotationTo.new(square, duration, AHEaseSineOut.new(), 90):start()
    waitFor(duration * 1.0)
    wait(AHOpacityTo.new(square, duration*0.8, nil, 0):start())
    square:removeSelf()
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PronunciationSquaresEmitter:emit()
  return runInCoroutine(function()
    local baseDuration = self.distance / SPEED
    for _, angle in ipairs(self.angles) do
      emitSquare(self, angle, self.distance, baseDuration)
      waitFor(0.1)
    end
  end)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function PronunciationSquaresEmitter.new(color, distance, angles)
  local self = Group.new(0, 0)
  Mixin.mix(self, PronunciationSquaresEmitter)

  self.color = color
  self.distance = distance
  self.angles = angles

  return self
end
