class("PunctuationAnswerButton")

require("PunctuationAnswerReactionPositive")
require("PunctuationColors")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ANSWER_BUTTON_COLOR = PunctuationColors.getMenuBackgroundColor()

local ICON_DATA_BY_ANSWER = {
  ["comma"] = {
    spriteName = "punctuation_menu_comma.png"
  },
  ["semicolon"] = {
    spriteName = "punctuation_menu_semicolon.png"
  },
  ["colon"] = {
    spriteName = "punctuation_menu_colon.png"
  },
  ["hyphen"] = {
    spriteName = "punctuation_menu_hyphen.png",
    xOffset = 3
  },
  ["apostrophe"] = {
    spriteName = "punctuation_menu_apostrophe.png"
  },
  ["question_mark"] = {
    spriteName = "punctuation_menu_questionMark.png",
    xOffset = 3
  },
  ["period"] = {
    spriteName = "punctuation_menu_period.png"
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getIconForAnswer(answer, answerIsPunctuation)
  if (answerIsPunctuation) then
    local iconData = ICON_DATA_BY_ANSWER[answer]
    assert(iconData ~= nil, "Invalid answer type: " .. tostring(answer))
    return Sprite.newWithName(iconData.spriteName)
  else
    return Label.new("no mark", Font.new(FontFaces.DIN_OT_MEDIUM, 14))
  end
end

local function createCircleOutline(radius)
  local color = PunctuationColors.BUTTON_OUTLINE
  local lineWidth = 1
  local outline = HollowCircle.new(radius, color, lineWidth)

  local gradient = Sprite.newWithName("punctuation_button_gradient.png")
  gradient:setEffectiveDims(radius * 2, nil)

  outline:insertAtCenter(gradient)

  return outline
end

local function createCharacterLabel(character)
  local fontHeight = 24
  local characterLabel = Label.new(character, Font.new(FontFaces.DIN_OT_LIGHT, fontHeight))
  characterLabel:setColor(unpack(PunctuationColors.BUTTON_CHARACTER))
  return characterLabel
end

local function positionIconAndCharacterLabel(answer, icon, characterLabel)
  characterLabel:offsetX(-icon:getWidthPart(0.15))
  characterLabel:offsetY(characterLabel:getHeightPart(0.1))

  icon:offsetX(characterLabel:getWidthPart(0.7))

  local iconData = ICON_DATA_BY_ANSWER[answer]
  if (iconData ~= nil) then
    if (iconData.xOffset ~= nil) then
      icon:offsetX(iconData.xOffset)
    end
    if (iconData.yOffset ~= nil) then
      icon:offsetY(iconData.yOffset)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationAnswerButton:getAnswer()
  return self.answer
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PunctuationAnswerButton.new(width, height, size, answer, character, handler)
  local self = FilledRectangle.new(width, height, ANSWER_BUTTON_COLOR)
  Mixin.mix(self, PunctuationAnswerButton)

  local buttonArea = size * 0.8
  local answerIsPunctuation = (answer ~= "nothing")

  local icon = getIconForAnswer(answer, answerIsPunctuation)
  self:insertAtCenter(icon)

  if (answerIsPunctuation) then
    local circleRadius = (buttonArea / 2)
    local circleOutline = createCircleOutline(circleRadius)
    self:insertAtCenter(circleOutline)

    local characterLabel = createCharacterLabel(character)
    self:insertAtCenter(characterLabel)

    positionIconAndCharacterLabel(answer, icon, characterLabel)
  end

  local tapHandler = function(event)
    local reaction = PunctuationAnswerReactionPositive.new(buttonArea, buttonArea)
    self:insertAtCenter(reaction)

    local animation = reaction:createReactionAnimation()
    animation:start()

    if (self.handler) then
      self.handler(event)
    end

    self:bringToFront()
  end

  Button.makeButton(self, tapHandler)

  self.answer = answer
  self.handler = handler

  return self
end
