class("PunctuationAnswerReaction")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local preloaded = {}

local function getGradientFillCirclePair(radius)
  local smallerCircle, largerCircle = nil, nil

  if (radius < Screen.getWidthPart(0.04)) then
    smallerCircle = Sprite.newWithName("punctuation_gradient_circle_small.png")
    largerCircle = Sprite.newWithName("punctuation_gradient_circle_small.png")
  elseif (radius < Screen.getWidthPart(0.075)) then
    smallerCircle = Sprite.newWithName("punctuation_gradient_circle_small.png")
    largerCircle = Sprite.newWithName("punctuation_gradient_circle_medium.png")
  else
    smallerCircle = Sprite.newWithName("punctuation_gradient_circle_big.png")
    largerCircle = Sprite.newWithName("punctuation_gradient_circle_big.png")
  end

  return smallerCircle, largerCircle
end

local function createLayout(self, radius, color)
  local innerSolidFill = FilledCircle.new(radius, color)
  local innerGradientFill, outerGradientFill = getGradientFillCirclePair(radius)
  innerGradientFill:setColor(unpack(color))
  outerGradientFill:setColor(unpack(color))

  local innerScale, outerScale = self:getInnerAndOuterCircleScale(innerGradientFill, outerGradientFill)
  innerGradientFill:setScl(innerScale)
  outerGradientFill:setScl(outerScale)

  local diameter = radius * 2
  local innerCircle = Group.new(diameter, diameter)
  local outerCircle = Group.new(diameter, diameter)

  innerCircle:insertAtCenter(innerGradientFill)
  innerCircle:insertAtCenter(innerSolidFill)
  outerCircle:insertAtCenter(outerGradientFill)
  self:insertAtCenter(innerCircle)
  self:insertAtCenter(outerCircle)

  self.innerCircle = innerCircle
  self.outerCircle = outerCircle
  self.innerGradientFill = innerGradientFill
  self.innerSolidFill = innerSolidFill

  self:linkColorAttrToChildren(true)
  self.radius = radius
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationAnswerReaction:getInnerAndOuterCircleScale(innerGradientFill, outerGradientFill)
  local innerScale = (2 * self.radius) / innerGradientFill:getWidth()
  local outerScale = (2 * self.radius) / outerGradientFill:getWidth()

  return innerScale, outerScale
end

function PunctuationAnswerReaction:createReactionAnimation(seconds)
  error("override me")
end

function PunctuationAnswerReaction.getPreloadedInstance(name, class, width, height)
  if (preloaded[name]) then
    return preloaded[name].instance
  end
  preloaded[name] = {instance = class.new(width, height)}
  return preloaded[name].instance
end

function PunctuationAnswerReaction.getAnimationForName(name, seconds)
  local instance = preloaded[name].instance
  local animation = preloaded[name].animation
  if (animation == nil) then
    animation = instance:createReactionAnimation(seconds)
    preloaded[name].animation = animation
  end
  instance:reset()
  return animation
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
function PunctuationAnswerReaction.new(width, height, color)
  local diameter = math.min(width, height)

  local self = Group.new(diameter, diameter)
  self.radius = diameter / 2

  Mixin.mix(self, PunctuationAnswerReaction)
  createLayout(self, self.radius, color)

  return self
end
