class("PunctuationBackground")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local RECTANGLES =
{
  {
    x = Screen.getWidthPart(0.4),
    y = Screen.getHeightPart(1),
    width = Screen.getWidthPart(2),
    height = Screen.width * 1,
    rotation = 20,
    opacity = 0.1
  },
  {
    x = Screen.getWidthPart(0.5),
    y = Screen.getHeightPart(0.5),
    width = Screen.getWidthPart(1),
    height = Screen.getHeightPart(1.4),
    rotation = -18,
    opacity = 0.1
  },
 {
    x = Screen.getWidthPart(0.5),
    y = Screen.getHeightPart(0.2),
    width = Screen.getWidthPart(1.5),
    height = Screen.getHeightPart(2.2),
    rotation = 60,
    opacity = 0.1
  },
  {
    x = Screen.getWidthPart(0.5),
    y = Screen.getHeightPart(0.1),
    width = Screen.getWidthPart(2),
    height = Screen.getHeightPart(0.2),
    rotation = -20,
    opacity = 0.1
  },

}

local GRADIENT_FILLING_COLORS = {
  {0.29, 0.0, 0.23, 1.0},
  {0.29, 0.0, 0.23, 1.0},
  {0.63, 0.15, 0.72, 1.0},
  {0.84, 0.18, 0.74, 1.0},
  {1.0, 0.14, 0.14, 1.0}
}

local GRADIENT_THICKNESSES = {
  0.1,
  0.3,
  0.3,
  0.3
}

local CORRECT_COLORS = {
  {0.02, 0.05, 0.29, 1.0},
  {0, 1.0, 0.78, 1.0}
}

local NEUTRAL_COLORS = {
  {0.12, 0.02, 0.11, 1.0},
  {0.43, 0.13, 0.38, 1.0}
}

local WRONG_BACKGROUND_COLORS = {
  {0.16, 0.0, 0.22, 1.0},
  {1.0, 0.20, 0.20, 1.0}
}

local WRONG_GREY_BACKGROUND_COLORS = {
  {0.24, 0.24, 0.24, 1.0},
  {0.5, 0.5, 0.5, 1.0}
}

local TRANSPARENT_BLACK_COLOR = {0, 0, 0, 0}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createTransitionColorsAction(self, backingColors, gradientOpacity, duration)
  local beginColor = backingColors[1]
  local endColor = backingColors[2]
  local backingAction = self.backing:newGradientToAction(duration, nil, beginColor, endColor)
  local gradientAction = self.gradient:createOpacityTo(duration, nil, gradientOpacity)
  local combinedAction = AHSpawn.new(backingAction, gradientAction)

  return combinedAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationBackground:transitionToCorrectColors()
  local gradientOpacity = 0.0
  local duration = 0.2
  local transitionAction = createTransitionColorsAction(self, CORRECT_COLORS, gradientOpacity, duration)
  transitionAction:start()
end

function PunctuationBackground:restoreColor()
  local gradientOpacity = 1.0
  local duration = 1.0
  local transitionAction = createTransitionColorsAction(self, NEUTRAL_COLORS, gradientOpacity, duration)
  transitionAction:start()
end

function PunctuationBackground:transitionToIncorrectAndRationaleColors()
  local gradientOpacity = 0.0
  local duration = 0.2
  local transitionIncorrectAction = createTransitionColorsAction(self, WRONG_BACKGROUND_COLORS, gradientOpacity, duration)
  local transitionRationaleAction = createTransitionColorsAction(self, WRONG_GREY_BACKGROUND_COLORS, gradientOpacity, duration)
  local transitionAction = AHSequence.new(transitionIncorrectAction, transitionRationaleAction)
  transitionAction:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()
  local backing = RectangleGradient.new(width, height, TRANSPARENT_BLACK_COLOR, TRANSPARENT_BLACK_COLOR)
  self:insertAtCenter(backing)
  self.backing = backing

  local gradient = MultiGradient.new(width, height, false, GRADIENT_FILLING_COLORS, GRADIENT_THICKNESSES)
  self:insertAtCenter(gradient)
  self.gradient = gradient

  local colorRectangleCollection = {}
  for _, layoutRectangle in pairs(RECTANGLES) do
    local colorRectangle = FilledRectangle.new(layoutRectangle.width, layoutRectangle.height, TRANSPARENT_BLACK_COLOR)
    colorRectangle:setLoc(layoutRectangle.x, layoutRectangle.y)
    colorRectangle:setRot(layoutRectangle.rotation)
    colorRectangle:setOpacity(layoutRectangle.opacity)
    self:insert(colorRectangle)

    table.insert(colorRectangleCollection, colorRectangle)
  end
  self.colorRectangleCollection = colorRectangleCollection
end

function PunctuationBackground.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, PunctuationBackground)

  createLayout(self)

  return self
end
