class("PunctuationBlankGlyph")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MARGIN_X = 2 / Host.contentScale
local ICON_FILENAMES = {
  comma = "punctuation_comma.png",
  semicolon = "punctuation_semicolon.png",
  colon = "punctuation_colon.png",
  nothing = "punctuation_no_mark.png",
  hyphen = "punctuation_hyphen.png",
  apostrophe = "punctuation_apostrophe.png",
  question_mark = "punctuation_questionMark.png",
  period = "punctuation_period.png"
}

local REVEALED_ICON_COLOR = PunctuationColors.getNeutralIconColor()
local HINT_SCREEN_ICON_COLOR = PunctuationColors.getCorrectColor()
local INACTIVE_ICON_COLOR = {1, 1, 1, 0.25}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationBlankGlyph:getLeftMargin()
  return MARGIN_X
end

function PunctuationBlankGlyph:getRightMargin()
  return MARGIN_X
end

function PunctuationBlankGlyph:setVisualState(punctuationType, isCorrect)
  self.icon:removeSelf()

  local iconFilename = ICON_FILENAMES[punctuationType]
  local icon = Sprite.newWithName(iconFilename)
  self:insertAtCenter(icon)
  self.icon = icon

  local isRecursive = true
  self:linkColorAttrToChildren(isRecursive)
  self:setColor(unpack(INACTIVE_ICON_COLOR))

  self.icon:setScl(0.0)

  local targetColor = nil
  if (isCorrect == true) then
    targetColor = REVEALED_ICON_COLOR
  else
    targetColor = HINT_SCREEN_ICON_COLOR
  end

  local duration = 0.4
  local colorAction = AHColorTo.new(self, duration, nil, targetColor)
  local scaleAction = AHScaleTo.new(self.icon, duration, nil, 1.0)
  local combinedAction = AHSpawn.new(colorAction, scaleAction)
  combinedAction:start()
end

function PunctuationBlankGlyph:restoreVisualState()
  local duration = 0.4
  local colorAction = AHColorTo.new(self, duration, nil, REVEALED_ICON_COLOR)
  colorAction:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, blankSprite)
  self:insertAtCenter(blankSprite)
  self.icon = blankSprite

  local isRecursive = true
  self:linkColorAttrToChildren(isRecursive)
  self:setColor(unpack(INACTIVE_ICON_COLOR))
end

function PunctuationBlankGlyph.new()
  local blankSprite = Sprite.newWithName("punctuation_unknown.png")
  local width, height = blankSprite:getDims()

  local self = Group.new(width, height)
  Mixin.mix(self, PunctuationBlankGlyph)

  createLayout(self, blankSprite)

  return self
end
