class("PunctuationConfiguration")

require("data.generators.LearningFocusedRoundGenerator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

PunctuationConfiguration.GAME_DURATION = 65
PunctuationConfiguration.INCORRECT_TIME_PENALTY = 5
PunctuationConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 3.5
PunctuationConfiguration.POINTS_PER_FULL_SPEED_BONUS = 50
PunctuationConfiguration.SECONDS_BEFORE_HELPER = 10

local POINTS_PER_CORRECT_ANSWER_BY_CONTENT_DIFFICULTY = {
  Easy = 1000,
  Medium = 1000,
  Hard = 1010,
  Advanced = 1015
}

local CORRECT_NEEDED_RANK_1 = 1
local CORRECT_NEEDED_RANK_2 = 6
local CORRECT_NEEDED_RANK_3 = 8

PunctuationConfiguration.TIME_REMAINING_TO_START_RESURFACING_MISTAKES =
  PunctuationConfiguration.GAME_DURATION *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getPointsForCorrectAnswerForRankThreshold()
  return PunctuationConfiguration.getPointsForCorrectAnswer("Easy")
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationConfiguration.getPointsForCorrectAnswer(contentDifficulty)
  local points = POINTS_PER_CORRECT_ANSWER_BY_CONTENT_DIFFICULTY[contentDifficulty]
  assert(points ~= nil, "ContentDifficulty is not associated with a point value: " .. tostring(contentDifficulty))
  return points
end

function PunctuationConfiguration.getPassScore()
  return getPointsForCorrectAnswerForRankThreshold() * CORRECT_NEEDED_RANK_1
end

function PunctuationConfiguration.getRankTwoScore()
  return getPointsForCorrectAnswerForRankThreshold() * CORRECT_NEEDED_RANK_2
end

function PunctuationConfiguration.getRankThreeScore()
  return getPointsForCorrectAnswerForRankThreshold() * CORRECT_NEEDED_RANK_3
end

function PunctuationConfiguration.getDifficultyKeyframes()
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Easy = 1.0
      }
    },
    {
      key = 0.075,
      valueProbabilities = {
        Easy = 0.95,
        Medium = 0.05
      }
    },
    {
      key = 0.15,
      valueProbabilities = {
        Easy = 0.85,
        Medium = 0.15
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.5,
        Medium = 0.5
      }
    },
    {
      key = 0.35,
      valueProbabilities = {
        Easy = 0.3,
        Medium = 0.6,
        Hard = 0.1
      }
    },
    {
      key = 0.4,
      valueProbabilities = {
        Easy = 0.2,
        Medium = 0.6,
        Hard = 0.2
      }
    },
    {
      key = 0.5,
      valueProbabilities = {
        Medium = 0.6,
        Hard = 0.4
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        Medium = 0.4,
        Hard = 0.55,
        Advanced = 0.05
      }
    },
    {
      key = 0.675,
      valueProbabilities = {
        Medium = 0.1,
        Hard = 0.7,
        Advanced = 0.2
      }
    },
    {
      key = 0.75,
      valueProbabilities = {
        Hard = 0.55,
        Advanced = 0.45
      }
    },
    {
      key = 0.8,
      valueProbabilities = {
        Hard = 0.5,
        Advanced = 0.5
      }
    },
    {
      key = 0.9,
      valueProbabilities = {
        Hard = 0.4,
        Advanced = 0.6
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 0.3,
        Advanced = 0.7
      }
    }
  }
end
