class("PunctuationFairyDust")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createParticle()
  local radius = math.random() * 0.5 + 0.5
  local color = {1, 1, 1}
  return FilledCircle.new(radius, color)
end

local function initParticle(self, particle, width, height)
  local x0 = math.random() * math.cos(math.random() * 2 * math.pi) * self.burstRadius
  local y0 = math.random() * math.sin(math.random() * 2 * math.pi) * self.burstRadius

  particle:setLoc(x0, y0)

  particle.x0, particle.y0 = x0, y0
  particle.height = height * math.random()
  particle.width = width * math.random()
  particle:setOpacity(0)
  particle:inheritColor(self)
end

local function spawn(self, numberToSpawn, width, height)
  local particles = {}
  local particleGroup = Group.newDeprecated()
  for i = 1, numberToSpawn do
    local particle = createParticle(self)
    initParticle(self, particle, width, height)
    particleGroup:insert(particle)
    particles[#particles + 1] = particle
  end
  return particles, particleGroup
end

local function getTime(self)
  return self.t
end

local function setTime(self, t)
  self.t = t
  for _, particle in ipairs(self.particles) do
    local x = particle.x0 + particle.width * t
    local y = particle.y0 - particle.height * t
    particle:setLoc(x, y)
    particle:setOpacity(1 - t)
  end
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationFairyDust:createSprinkleAction(duration)
  self.t = 0
  local sprinkle = AHNonAttributeTo.new(self, duration, getTime, setTime, nil, 1)
  return sprinkle
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PunctuationFairyDust.new(sprinkleCount, width, height)
  local self = Group.newDeprecated()
  Mixin.mix(self, PunctuationFairyDust)

  self.sprinkleCount = sprinkleCount
  self.burstRadius = 10
  self.t = 0
  self.heightVariation = 0.5

  height = height or width
  local particles, particleGroup = spawn(self, self.sprinkleCount, width, height)
  self:insert(particleGroup)
  self.particles = particles

  return self
end
