class("PunctuationOutroEffect")
require("graphics.LinePathTracer")
require("PunctuationAnswerReactionPositive")
require("PunctuationColors")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local END_POINTS_BY_INDEX =
{
  {Screen.width/4, Screen.width},
  {3 * Screen.width/4, 0},
  {0, Screen.width},
  {Screen.width, 0},
  {Screen.width/4, 0},
  {3 * Screen.width/4, Screen.width},
  {Screen.width/8, Screen.width/4},
  {7 * Screen.width/8, 3 * Screen.width/4},
  {Screen.width/2, 0},
  {Screen.width/2, Screen.width},
  {Screen.width, Screen.width/2},
  {0, Screen.width/2},
}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createLineAndOrb(self)
  local radius = Screen.width * 0.05
  local lineParams =
  {
    lineWidth = 3,
    maxVisibleViews = 4,
    maxTailLength = 40
  }
  local orb = PunctuationAnswerReactionPositive.new(radius, radius)
  local line = LinePathTracer.new(lineParams)
  line:setColor(unpack(PunctuationColors.getCorrectColor()))

  self:insert(orb)
  self:insert(line)

  orb:setX(-Screen.width)

  return line, orb
end

local function getX0X1ForIndex(index)
  local endPoints = END_POINTS_BY_INDEX[index % #END_POINTS_BY_INDEX + 1]
  return endPoints[1], endPoints[2]
end

local function createLineAndOrbAnimation(line, orb, seconds, index)
  local x0, x1 = getX0X1ForIndex(index)
  local y0 = -Screen.height * 0.03
  local y1 = Screen.height

  line:setTipAdvanceHandler(function(trajectory, trajectoryIndex)
    local position = trajectory[trajectoryIndex]
    if (position == nil) then
      return
    end
    local x = position[1]
    local y = position[2]
    orb:setLoc(x, y)
  end)
  local orbAnimation = orb:createReactionAnimation(seconds / 2)
  local lineAnimation = line:createTracePathAnimation(x0, y0, x1, y1, seconds)
  local animation = AHSpawn.new(orbAnimation, lineAnimation)
  return animation
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationOutroEffect:createOutroAnimation(seconds)
  local totalStalks = 10
  local staggerDelay = (seconds/totalStalks) * 0.45
  local durationPerStalk = seconds - totalStalks * staggerDelay
  local result = AHSpawn.new()

  for i = 1, totalStalks do
    local line, orb = createLineAndOrb(self)
    local animation = createLineAndOrbAnimation(line, orb, durationPerStalk, i)
    local curDelay = staggerDelay * (i - 1)
    local waitAndLaunch = AHDelayed.new(curDelay, animation)
    result:addAction(waitAndLaunch)
  end
  result:addAction(function()Sound.playSoundEffect("punctuation_goal_reached") end)
  return result
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PunctuationOutroEffect.new()
  local self = Group.newDeprecated()
  Mixin.mix(self, PunctuationOutroEffect)
  return self
end
