class("PunctuationProgressMeterStalk")
require("PunctuationAnswerReactionPositive")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local STALK_WIDTH = 2
local STALK_COLOR = PunctuationColors.getCorrectColor()

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createHeightAnimation(self, seconds, targetScale, ease)
  local x = self.stalk:getX()
  local startYScale = nil
  local ds = nil
  local updateScale = function(t)
    if (startYScale == nil) then
      local _, stalkScaleY = self.stalk:getScl()
      startYScale = stalkScaleY
      ds = targetScale - startYScale
    end
    local scale = startYScale + ds * t

    local orbRadius = self:getWidthPart(0.5)
    local height = self.stalk:getHeight() * scale
    local y = height + orbRadius
    self.stalk:setScl(1, scale)
    self.orb:setLoc(x, y)
  end

  local animation = AHCustomAction.new(seconds, ease, updateScale)
  return animation
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationProgressMeterStalk:setCorrectState()
  self.orb:setOpacity(0)
  self.stalk:setOpacity(0)

  local duration = 0.2
  local orbDiameter = self:getWidth()
  local particles = PunctuationAnswerReaction.getPreloadedInstance("stalk", PunctuationAnswerReactionPositive, orbDiameter, orbDiameter)
  local particlesAnimation = PunctuationAnswerReaction.getAnimationForName("stalk")
  particles:reset()

  local showOrb = AHOpacityTo.new(self.orb, duration, nil, 1)
  local showStalk =  AHOpacityTo.new(self.stalk, duration, nil, 1)
  local colorStalk = AHColorTo.new(self.stalk, duration, nil, STALK_COLOR)

  local correctAnimation = AHSpawn.new(showOrb, showStalk, colorStalk, particlesAnimation)

  local correctSequence = AHSequence.new(correctAnimation)
  correctSequence:start()

  self:takeChild(particles)
  local orbRadius = orbDiameter * 0.5
  particles:setLoc(orbRadius, self:getHeight() + orbRadius)
end

function PunctuationProgressMeterStalk:createHeightChangeAnimation(seconds, changePercentage, ease)
  seconds = seconds or 0.5
  changePercentage = changePercentage or 1.5
  return createHeightAnimation(self, seconds, changePercentage, ease)
end

function PunctuationProgressMeterStalk:createHeightRestoreAnimation(seconds, ease)
  return createHeightAnimation(self, seconds, 1, ease)
end

function PunctuationProgressMeterStalk:createUpDownRelativeAnimation(seconds, changePercentage, upEase, downEase)
  local up = self:createHeightChangeAnimation(seconds / 4, changePercentage, upEase)
  local down = self:createHeightRestoreAnimation((3 * seconds) / 4, downEase)
  return AHSequence.new(up, down)
end

function PunctuationProgressMeterStalk:createUpDownAbsoluteAnimation(seconds, dy, upEase, downEase)
  local height = self:getHeight()
  local percentage = (height + dy) / height
  return self:createUpDownRelativeAnimation(seconds, percentage, upEase, downEase)
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()

  local stalk = FilledRectangle.new(STALK_WIDTH, height, STALK_COLOR)
  stalk:setX(self:getWidthPart(0.5))
  stalk:setAnchorPointBottom()
  self:insert(stalk)
  self.stalk = stalk

  local lineWidth = 1.0
  local outlineBlurMargin = 0.5
  local orbRadius = width * 0.5
  local orb = HollowCircle.new(orbRadius, STALK_COLOR, lineWidth, outlineBlurMargin)
  local orbY = height + orbRadius
  orb:setLoc(self:getWidthPart(0.5), orbY)
  self:insert(orb)
  self.orb = orb
end

function PunctuationProgressMeterStalk.newHidden(width, height)
  local self = Group.new(width, height)
  Mixin.mix(self, PunctuationProgressMeterStalk)

  createLayout(self)

  self.orb:setOpacity(0)
  self.stalk:setOpacity(0)

  return self
end
