--- PunctuationPromptTextView
-- @author Created by Jay Austria on August 12, 2016.

class("PunctuationPromptTextView")

require("PunctuationTextInterpreter")
require("PunctuationBlankGlyph")
require("actions.advanced.Shake")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getBaseAttributes()
  return {
    font = Font.new(FontFaces.DIN_OT_LIGHT, 23),
    color = {1,1,1}
  }
end

local function getAttributesForLabels()
  return {
    blank = {
      itemType = "blank"
    },
    i = {
      font = Font.new(FontFaces.DIN_OT_LIGHT_ITALIC, 23)
    }
  }
end

local function getBlankItemView(self)
  for _, itemView in ipairs(self.itemViews) do
    local item = self.itemsForViews[itemView]
    if (item:getAttribute("itemType") == "blank") then
      return itemView
    end
  end

  error("PunctuationPromptTextView did not contain a blank item")
  return nil
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationPromptTextView:createItemView(item)
  local itemType = item:getItemType()

  local prop = nil
  if (itemType == "blank") then
    prop = PunctuationBlankGlyph.new()
  else
    prop = self.super.createItemView(self, item)
  end

  return prop
end

function PunctuationPromptTextView:setText(text)
  if text ~= self:getText() then
    local textInterpreter = self.textInterpreter
    local textModel = self.textModel

    local items = textInterpreter:interpretXmlText(text)
    textModel:resetItems(items)

    self.text = text
  end
end

function PunctuationPromptTextView:getText()
  return self.text
end

function PunctuationPromptTextView:getBlankWorldLocation()
  local blankView = getBlankItemView(self)

  return blankView:getCenterWorldLoc()
end

function PunctuationPromptTextView:createSetCorrectAction(punctuationType)
  local particles = PunctuationAnswerReaction.getPreloadedInstance("sentence_correct")
  local x, y = self:getBlankWorldLocation()
  particles:setLoc(x, y)

  local blankGlyph = getBlankItemView(self)
  local isCorrect = true
  blankGlyph:setVisualState(punctuationType, isCorrect)

  local particlesAnimation = PunctuationAnswerReaction.getAnimationForName("sentence_correct")

  return particlesAnimation
end

function PunctuationPromptTextView:createSetIncorrectAction(punctuationType)
  local particles = PunctuationAnswerReaction.getPreloadedInstance("sentence_wrong")
  local x, y = self:getBlankWorldLocation()
  particles:setLoc(x, y)

  local blankGlyph = getBlankItemView(self)
  local isCorrect = false
  blankGlyph:setVisualState(punctuationType, isCorrect)

  local xShakeAmp = 10
  local yShakeAmp = 1
  local shakesPerSecond = 4
  local durationSeconds = 0.8
  local shake = AHSmoothShake.new(self, xShakeAmp, yShakeAmp, shakesPerSecond, durationSeconds)
  shake:start()

  local particlesAnimation = PunctuationAnswerReaction.getAnimationForName("sentence_wrong")
  return particlesAnimation
end

function PunctuationPromptTextView:restoreVisualState()
  local blankGlyph = getBlankItemView(self)
  blankGlyph:restoreVisualState()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createElements(self)
  local baseAttributes = getBaseAttributes()
  local attributesForLabels = getAttributesForLabels()
  local textInterpreter = PunctuationTextInterpreter.new(baseAttributes, attributesForLabels)
  self.textInterpreter = textInterpreter

  local textModel = TextModel.new(textInterpreter:interpretXmlText(""))
  self:setModel(textModel)

  self.animationBuilder = TextAnimationBuilder.new()
end

function PunctuationPromptTextView.new(textWidth)
  local lineSpacing = 2.5
  local self = TextView.new(textWidth, nil, nil, lineSpacing)
  Mixin.mix(self, PunctuationPromptTextView)

  createElements(self)

  self.startingYsForGlyphs = {}

  return self
end
