class("PunctuationRoundGenerator")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_CONCEPTS_SKIPPED = 3

PunctuationRoundGenerator.CONCEPT_IDENTIFIER_KEY = "conceptID"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWordBeforeBlank(sentence)
  local pattern = "([^%s]+)<blank/>"
  return string.match(sentence, pattern)
end

local function removeTagsFromWord(word)
  if (word ~= nil) then
    local pattern = "%b<>"
    return string.gsub(word, pattern, "")
  end
  return nil
end

local function getlastCharacterInWord(word)
  if (word ~= nil) then
    return word[#word]
  end
  return nil
end

local function getCharacterBeforeBlank(sentence)
  local pattern = "([^%s]+)<blank/>"
  local wordBeforeBlank = string.match(sentence, pattern)
  local wordWithNoTags = removeTagsFromWord(wordBeforeBlank)
  local lastCharacterInWord = getlastCharacterInWord(wordWithNoTags)
  return lastCharacterInWord
end

local function getRuleForRound(round)
  return round.rules[1].ruleName
end

local function doesRoundRepeatRule(self, round)
  local rule = getRuleForRound(round)
  return (rule == self.lastRuleUsed)
end

local function saveLastRuleUsed(self, round)
  local rule = getRuleForRound(round)
  self.lastRuleUsed = rule
end

local function sortAnswerOptions(options)
  table.sort(options, function(o1, o2) return o1 > o2 end)
end

local function packageAnswer(correctAnswerTable)
  if (TableUtils.isEmpty(correctAnswerTable)) then
    return nil
  end

  local result = {}
  result.hint =
  {
    body = correctAnswerTable.body,
    grammar_note = correctAnswerTable.grammar_note
  }
  result.correct = correctAnswerTable.correct_answer
  result.options = TableUtils.csvToArray(correctAnswerTable.incorrect_answers)
  result.options[#result.options + 1] = result.correct
  result.ruleName = correctAnswerTable.rule_name

  sortAnswerOptions(result.options)

  return result
end

-- Exposed for testing
function PunctuationRoundGenerator.conceptToRound(concept)
  local round = {}
  local rules = {}
  round.sentence = concept.sentence
  round.rules = rules
  round.questionCount = concept.errors
  round[PunctuationRoundGenerator.CONCEPT_IDENTIFIER_KEY] = concept.identifier
  round.difficulty = concept.difficulty
  round.characterBeforeBlank = getCharacterBeforeBlank(round.sentence)

  rules[1] = packageAnswer(concept.correct_1)

  return round
end

local function createNewRound(self)
  local concept = self.conceptGenerator:next()
  return PunctuationRoundGenerator.conceptToRound(concept)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationRoundGenerator:next()
  local round = createNewRound(self)
  local conceptsSkippedCount = 0
  while (doesRoundRepeatRule(self, round) and not (conceptsSkippedCount >= MAX_CONCEPTS_SKIPPED)) do
    round = createNewRound(self)
    conceptsSkippedCount = conceptsSkippedCount + 1
  end
  saveLastRuleUsed(self, round)
  return round
end

function PunctuationRoundGenerator:getRampIndex()
  return self.roundNumber
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function PunctuationRoundGenerator.new()
  local self = {}
  Mixin.mix(self, PunctuationRoundGenerator)
  self.conceptGenerator = GameSession.conceptGenerator

  local difficultyKeyFrames = PunctuationConfiguration.getDifficultyKeyframes()
  local difficultyNameChooser = KeyFrameInterpolator.new(difficultyKeyFrames, GameSession.getDifficulty())
  self.conceptGenerator:setDifficultyNameChooser(difficultyNameChooser)
  self.roundNumber = 0
  self.lastRuleUsed = ""

  return self
end
