--- PunctuationTextInterpreter
-- @author created by Jay Austria on Augsut 17, 2016

class("PunctuationTextInterpreter")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BLANK_ITEM_TYPE = "blank"
local BLANK_TAG = "<blank/>"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function isCharacterWhitespace(character)
  return ((character ~= nil) and (character == " "))
end

local function isWhitespaceBeforeTag(text)
  local isWhitespace = nil

  local tagStartIndex, _ = text:find(BLANK_TAG)
  if (tagStartIndex ~= nil) then
    local characterBeforeTagIndex = tagStartIndex - 1
    local characterBeforeTag = text:sub(characterBeforeTagIndex, characterBeforeTagIndex)

    isWhitespace = isCharacterWhitespace(characterBeforeTag)
  end

  return isWhitespace
end

local function isWhitespaceAfterTag(text)
  local isWhitespace = nil

  local _, tagEndIndex = text:find(BLANK_TAG)
  if (tagEndIndex ~= nil) then
    local characterAfterTagIndex = tagEndIndex + 1
    local characterAfterTag = text:sub(characterAfterTagIndex, characterAfterTagIndex)

    isWhitespace = isCharacterWhitespace(characterAfterTag)
  end

  return isWhitespace
end

local function overrideSpacingPropertiesForBlanks(self, items)
  for _, item in ipairs(items) do
    if (item:getAttribute("itemType") == BLANK_ITEM_TYPE) then
      if (self.hasWhitespaceBeforeTag == true) then
        item:setAttribute("skipPreviousSpacing", false)
        item:setAttribute("allowBreakLineBefore", true)
      else
        item:setAttribute("skipPreviousSpacing", true)
        item:setAttribute("allowBreakLineBefore", false)
      end

      if (self.hasWhitespaceAfterTag == true) then
        item:setAttribute("skipNextSpacing", false)
        item:setAttribute("allowBreakLineAfter", true)
      else
        item:setAttribute("skipNextSpacing", true)
        item:setAttribute("allowBreakLineAfter", false)
      end
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function PunctuationTextInterpreter:parseXmlTagElement(element, attributesForElement)
  local items = self.super.parseXmlTagElement(self, element, attributesForElement)

  overrideSpacingPropertiesForBlanks(self, items)

  return items
end

function PunctuationTextInterpreter:interpretXmlText(text)
  self.hasWhitespaceBeforeTag = isWhitespaceBeforeTag(text)
  self.hasWhitespaceAfterTag = isWhitespaceAfterTag(text)

  return self.super.interpretXmlText(self, text)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function PunctuationTextInterpreter.new(baseAttributes, attributesForLabels)
  local self = TextInterpreter.new(baseAttributes, attributesForLabels)
  Mixin.mix(self, PunctuationTextInterpreter)

  self.hasWhitespaceBeforeTag = nil
  self.hasWhitespaceAfterTag = nil

  return self
end
