--- RainBill
-- @author Created by Aaron Barrett on July 06, 2015.

class("RainBill")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.getWidthPart(0.14)
local HEIGHT = Screen.getHeightPart(0.04)

local CORNER_RADIUS = HEIGHT * 0.12
local BORDER_THICKNESS = HEIGHT * 0.14

local OUTER_RECTANGLE_COLOR = {1.0, 1.0, 1.0}

local Z_OUTER_RECTANGLE = 10
local Z_INNER_RECTANGLE = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainBill:playAppear(destinationY)
  local ease = AHEaseExponentialOut.new()
  local appearAction = AHSpawn.new(
                                   AHLocationYTo.new(self, 0.4, ease, destinationY),
                                   AHOpacityTo.new(self, 0.3, ease, 1.0)
                                   )
  self.actionInterrupt:insert(appearAction)
end

function RainBill:playDisappear()
  local disappearAction = AHSequence.new(
                                         AHSpawn.new(
                                                     AHLocationYBy.new(self, 0.4, AHEaseSimpleIn.new(4.0), Screen.fullHeight),
                                                     AHDelayed.new(0.1525, AHOpacityTo.new(self, 0.075, nil, 0.0))
                                                     ),
                                         AHRemove.new(self)
                                         )
  self.actionInterrupt:insert(disappearAction)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RainBill.new(roundColor)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, RainBill)

  local outerRectangle = FilledRoundedRectangle.new(WIDTH, HEIGHT, CORNER_RADIUS, OUTER_RECTANGLE_COLOR)
  outerRectangle:inheritOpacity(self)
  self:insertAtCenter(outerRectangle, Z_OUTER_RECTANGLE)

  local innerRectangle = FilledRoundedRectangle.new(WIDTH - BORDER_THICKNESS, HEIGHT - BORDER_THICKNESS, CORNER_RADIUS, roundColor)
  innerRectangle:inheritOpacity(self)
  self:insertAtCenter(innerRectangle, Z_INNER_RECTANGLE)

  self.actionInterrupt = ActionInterrupt.new()

  return self
end
