--- RainConfiguration
-- @author Created by Aaron Barrett on July 06, 2015.

class("RainConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

RainConfiguration.POINTS_PER_CORRECT_RESPONSE = 800
RainConfiguration.INITIAL_LIVES_COUNT = 3
RainConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 5.0
RainConfiguration.SPEED_BONUS_DECAY_START = 0.5
RainConfiguration.POINTS_PER_FULL_SPEED_BONUS = 75
RainConfiguration.ROUNDS_COUNT = 7
RainConfiguration.POINTS_PER_LIVES_BONUS = 2200

local BILLS_EXCEEDING_TOTAL_COUNT_TO_SHOW_REDUCE_HELPER = 2
local BILLS_EXCEEDING_TOTAL_COUNT_TO_SHOW_REDUCE_HELPER_FOR_FIRST_PLAY = 1
local DIFFICULTY_TO_STOP_SHOWING_REDUCE_HELPER = 0.4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getPassScore()
  local correctResponsesNeededToWin = RainConfiguration.ROUNDS_COUNT - (RainConfiguration.INITIAL_LIVES_COUNT - 1)
  return RainConfiguration.POINTS_PER_CORRECT_RESPONSE * correctResponsesNeededToWin
end

local function getExtraLivesNeededRankTwo()
  return RainConfiguration.INITIAL_LIVES_COUNT - 2
end

local function getExtraLivesNeededRankThree()
  return RainConfiguration.INITIAL_LIVES_COUNT - 1
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainConfiguration.shouldShowReduceHelper(billsExceedingTotalCount)
  local difficulty = GameSession.getDifficulty()
  local difficultyIsLowEnough = (difficulty <= DIFFICULTY_TO_STOP_SHOWING_REDUCE_HELPER)
  local isFirstPlay = GameSession.hasNeverWon()
  local excessBills = (billsExceedingTotalCount >= BILLS_EXCEEDING_TOTAL_COUNT_TO_SHOW_REDUCE_HELPER)
  local excessBillsForFirstPlay = (billsExceedingTotalCount >= BILLS_EXCEEDING_TOTAL_COUNT_TO_SHOW_REDUCE_HELPER_FOR_FIRST_PLAY)

  return (difficultyIsLowEnough and (excessBills or (isFirstPlay and excessBillsForFirstPlay)))
end

function RainConfiguration.getRankTwoScore()
  local rankTwoBonusPoints = getExtraLivesNeededRankTwo() * RainConfiguration.POINTS_PER_LIVES_BONUS
  return getPassScore() + rankTwoBonusPoints
end

function RainConfiguration.getRankThreeScore()
  local rankThreeBonusPoints = getExtraLivesNeededRankThree() * RainConfiguration.POINTS_PER_LIVES_BONUS
  return getPassScore() + rankThreeBonusPoints
end

function RainConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 1.0}},

    {key = 0.075, valueProbabilities = {Basic = 0.70,
                                        Easy = 0.30}},

    {key = 0.15, valueProbabilities = {Basic = 0.30,
                                       Easy = 0.70}},

    {key = 0.3, valueProbabilities = {Easy = 0.4,
                                      Medium = 0.6}},

    {key = 0.45, valueProbabilities = {Medium = 0.15,
                                       Hard = 0.85}},

    {key = 0.6, valueProbabilities = {Hard = 0.50,
                                      Advanced = 0.50}},

    {key = 0.75, valueProbabilities = {Hard = 0.10,
                                      Advanced = 0.90}},

    {key = 0.8, valueProbabilities = {Hard = 0.05,
                                      Advanced = 0.95}},

    {key = 1.0, valueProbabilities = {Advanced = 1.0}}
  }
end
