--- RainDot
-- @author Created by Aaron Barrett on July 07, 2015.

class("RainDot")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

RainDot.RADIUS = (Screen.getWidthPart(0.4) + Screen.getHeightPart(0.2)) * 0.5

local FULL_BACK_OPACITY = 0.40
local WRONG_OPACITY = 0.20
local DIMMED_OPACITY = 0.05

local GRAYED_COLOR = {0.78, 0.78, 0.78}

local WANDER_RADIUS = Screen.getHeightPart(0.056)
local WANDER_DISTANCE_PER_SECOND = Screen.getHeightPart(0.034)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function stopWandering(self)
  local wanderCoroutine = self.wanderCoroutine
  if (wanderCoroutine ~= nil) then
    wanderCoroutine:stop()
  end
end

local function getFullOpacity(self)
  if (self.isMainDot) then
    return 1.0
  else
    return FULL_BACK_OPACITY
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainDot:playWrong()
  local fullOpacity = getFullOpacity(self)
  local wrongAction = AHSequence.new(
                                     AHOpacityTo.new(self, 0.2, nil, WRONG_OPACITY),
                                     AHDelay.new(0.7),
                                     AHSpawn.new(
                                                 AHColorTo.new(self, 0.4, nil, GRAYED_COLOR),
                                                 AHOpacityTo.new(self, 0.4, nil, fullOpacity)
                                                 ),
                                     AHDelay.new(1.2),
                                     AHOpacityTo.new(self, 0.4, nil, 0.0),
                                     function()
                                       stopWandering(self)
                                     end
                                     )
  self.colorInterrupt:insert(wrongAction)
end

function RainDot:playCorrect()
  stopWandering(self)
  local correctOpacityAction = AHOpacityTo.new(self, 0.3, nil, 1.0)
  self.colorInterrupt:insert(correctOpacityAction)

  local correctScaleAction = AHScaleTo.new(self, 0.3, AHEaseExponentialIn.new(2.0), 4.0)
  self.scaleInterrupt:insert(correctScaleAction)
end

function RainDot:playPostCorrectFade()
  local opacityAction = AHOpacityTo.new(self, 0.5, nil, 0.0)
  self.colorInterrupt:insert(opacityAction)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RainDot.new(roundColor, originX, originY, isMainDot)
  local self = FilledCircle.new(RainDot.RADIUS, roundColor)
  Mixin.mix(self, RainDot)

  self.isMainDot = isMainDot

  if (not isMainDot) then
    self.wanderCoroutine = runInCoroutine(function()
      local ease = AHEaseSimpleInOut.new(2.0)
      while (true) do
        local currentX, currentY = self:getLoc()
        local destinationX = originX - WANDER_RADIUS + math.random() * 2.0 * WANDER_RADIUS
        local destinationY = originY - WANDER_RADIUS + math.random() * 2.0 * WANDER_RADIUS
        local distance = MathUtils.distance(currentX, currentY, destinationX, destinationY)
        local duration = distance / WANDER_DISTANCE_PER_SECOND

        local wanderAction = AHLocationTo.new(self, duration, ease, destinationX, destinationY)
        wait(wanderAction:start())
      end
    end)
  end

  local initialOpacity = getFullOpacity(self)
  self:setOpacity(initialOpacity)

  self.scaleInterrupt = ActionInterrupt.new()
  self.colorInterrupt = ActionInterrupt.new()

  return self
end
