--- RainDotCluster
-- @author Created by Aaron Barrett on July 07, 2015.

class("RainDotCluster")

require("RainDot")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.getHeightPart(1.5)
local HEIGHT = WIDTH

-- Shift to safe center
local CENTER_X = (WIDTH + Screen.safeLeft - (Screen.fullWidth - Screen.safeRight)) * 0.5
local CENTER_Y = (HEIGHT + Screen.safeBottom - (Screen.fullHeight - Screen.safeTop)) * 0.5

local DOT_START_RADIUS = Screen.getHeightPart(0.08)

local BACK_DOTS_COUNT = 3

local Z_DOT_SPINNER = 10
local Z_MAIN_DOT = 20
local Z_COLUMN = 30

local SELECTION_FONT_SIZE = 140
local SUBMIT_FONT_SIZE = 21

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainDotCluster:playAppear()
  local submitLabel = self.submitLabel

  local appearAction = AHSpawn.new(
                                   AHScaleTo.new(self, 0.6, AHEaseExponentialOut.new(), 1.0),
                                   AHOpacityTo.new(submitLabel, 0.15, nil, 1.0)
                                   )
  self.actionInterrupt:insert(appearAction)
end

function RainDotCluster:playDisappear()
  local submitLabel = self.submitLabel

  local disappearAction = AHSpawn.new(
                                      AHScaleTo.new(self, 0.4, AHEaseSimpleIn.new(4.0), 0.0),
                                      AHOpacityTo.new(submitLabel, 0.15, nil, 0.0)
                                      )
  self.actionInterrupt:insert(disappearAction)
end

function RainDotCluster:playCorrect()
  FunctionUtils.callMethodOnTargets("playCorrect", self.allDots)

  local selectionLabel = self.selectionLabel
  local submitLabel = self.submitLabel

  local submitLabelAction = AHOpacityTo.new(submitLabel, 0.2, nil, 0.0)
  submitLabelAction:start()

  local correctAction = AHSequence.new(
                                       AHScaleTo.new(selectionLabel, 0.3, AHEaseSineOut.new(), 1.15),
                                       AHScaleTo.new(selectionLabel, 0.2, AHEaseSineIn.new(), 1.10)
                                       )
  return correctAction:start()
end

function RainDotCluster:playPostCorrectFade()
  FunctionUtils.callMethodOnTargets("playPostCorrectFade", self.allDots)
end

function RainDotCluster:playFadeSelectionLabel()
  local selectionLabel = self.selectionLabel
  local fadeAction = AHOpacityTo.new(selectionLabel, 0.5, nil, 0.0)
  return fadeAction:start()
end

function RainDotCluster:playWrong(correctAnswer)
  FunctionUtils.callMethodOnTargets("playWrong", self.allDots)

  local submitLabel = self.submitLabel
  local selectionLabel = self.selectionLabel

  local submitLabelAction = AHOpacityTo.new(submitLabel, 0.1, nil, 0.0)

  local selectionLabelAction = AHSequence.new(
                                              AHDelay.new(0.73),
                                              AHOpacityTo.new(selectionLabel, 0.45, nil, 0.0),
                                              function()
                                                selectionLabel:setText(correctAnswer)
                                              end,
                                              AHOpacityTo.new(selectionLabel, 0.45, nil, 1.0)
                                              )

  local shakeEase = AHEaseSineInOut.new()
  local shakeXAction = AHSequence.new(
                                      AHLocationXBy.new(self, 0.08, shakeEase, -36.0),
                                      AHLocationXBy.new(self, 0.09, shakeEase, 57.0),
                                      AHLocationXBy.new(self, 0.08, shakeEase, -46.0),
                                      AHLocationXBy.new(self, 0.08, shakeEase, 29.0),
                                      AHLocationXBy.new(self, 0.10, shakeEase, -8.0),
                                      AHLocationXBy.new(self, 0.06, shakeEase, 4.0)
                                      )
  local shakeYAction = AHSequence.new(
                                      AHLocationYBy.new(self, 0.08, shakeEase, -1.0),
                                      AHLocationYBy.new(self, 0.09, shakeEase, 1.0),
                                      AHLocationYBy.new(self, 0.08, shakeEase, -5.0),
                                      AHLocationYBy.new(self, 0.08, shakeEase, 1.0),
                                      AHLocationYBy.new(self, 0.10, shakeEase, -3.0),
                                      AHLocationYBy.new(self, 0.06, shakeEase, 1.0)
                                      )
  local shakeScaleAction = AHSequence.new(
                                          AHScaleTo.new(self, 0.08, shakeEase, 0.95),
                                          AHScaleTo.new(self, 0.09, shakeEase, 0.96),
                                          AHScaleTo.new(self, 0.08, shakeEase, 0.95),
                                          AHScaleTo.new(self, 0.08, shakeEase, 0.98),
                                          AHScaleTo.new(self, 0.10, shakeEase, 1.01),
                                          AHScaleTo.new(self, 0.06, shakeEase, 1.00),
                                          function()
                                            self.spinAction:stop()
                                          end
                                          )

  local shakeAction = AHSpawn.new(
                                  shakeXAction,
                                  shakeYAction,
                                  shakeScaleAction
                                  )

  local wrongAction = AHSpawn.new(
                                  submitLabelAction,
                                  selectionLabelAction,
                                  shakeAction
                                  )

  return wrongAction:start()
end

function RainDotCluster:setSelectedAmount(selectedAmount)
  local selectionLabel = self.selectionLabel

  local selectedAmountText = tostring(selectedAmount)
  local digitsCount = #selectedAmountText
  local scale = nil
  if (digitsCount <= 2) then
    scale = 1.0
  elseif (digitsCount <= 3) then
    scale = 0.9
  else
    scale = 0.8
  end

  selectionLabel:setText(selectedAmountText)
  selectionLabel:setScl(scale)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RainDotCluster.new(roundColor)
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, RainDotCluster)

  local spinAction = AHSpawn.new()

  local backDots = {}
  local allDots = {}
  for dotIndex = 1, BACK_DOTS_COUNT do
    local dotSpinner = Group.new(WIDTH, HEIGHT)

    local isMainDot = false
    local dot = RainDot.new(roundColor, CENTER_X, CENTER_Y, isMainDot)

    local startX = CENTER_X + math.random() * 2.0 * DOT_START_RADIUS - DOT_START_RADIUS
    local startY = CENTER_Y + math.random() * 2.0 * DOT_START_RADIUS - DOT_START_RADIUS
    dot:setLoc(startX, startY)
    dotSpinner:insert(dot)
    table.insert(backDots, dot)
    table.insert(allDots, dot)

    dotSpinner:setLoc(CENTER_X, CENTER_Y)
    self:insert(dotSpinner, Z_DOT_SPINNER)
    local spinDirection = nil
    if (MathUtils.isEven(dotIndex)) then
      spinDirection = 1.0
    else
      spinDirection = -1.0
    end

    local dotSpinAction = AHRotationBy.new(dotSpinner, 60.0, nil, 1800.0 * spinDirection)
    spinAction:addAction(dotSpinAction)
  end

  local isMainDot = true
  local mainDot = RainDot.new(roundColor, CENTER_X, CENTER_Y, isMainDot)
  mainDot:setLoc(CENTER_X, CENTER_Y)
  self:insert(mainDot, Z_MAIN_DOT)
  self.mainDot = mainDot
  table.insert(allDots, mainDot)

  self.backDots = backDots
  self.allDots = allDots

  local columnSpacing = -Screen.getHeightPart(0.017)
  local column = Column.new(columnSpacing)

  local selectionFont = Font.new(FontFaces.DIN_OT_LIGHT, SELECTION_FONT_SIZE)
  local selectionLabel = Label.new("0", selectionFont)
  column:insert(selectionLabel)
  self.selectionLabel = selectionLabel

  local submitFont = Font.new(FontFaces.DIN_OT_MEDIUM, SUBMIT_FONT_SIZE)
  local submitLabel = Label.new(Localizer.get("SUBMIT"), submitFont)
  submitLabel:setOpacity(0.0)
  column:insert(submitLabel)
  self.submitLabel = submitLabel

  column:alignChildren()
  column:setLoc(CENTER_X, CENTER_Y * 1.04)
  self:insert(column, Z_COLUMN)

  self.spinAction = spinAction:start()

  self.actionInterrupt = ActionInterrupt.new()

  self:setScl(0.0)

  return self
end
