--- RainEquation
-- @author Created by Aaron Barrett on July 08, 2015.

class("RainEquation")

require("RainMathGlyph")
require("RainMathLayoutBuilder")
require("RainMathActionBuilder")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LUA_PATTERN_TO_ITERATE_EACH_CHARACTER = "."

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getCenterStartX(self, glyphs)
  local glyphsWidth = RainMathLayoutBuilder.getWidthOfGlyphs(glyphs)
  return self:getWidthPart(0.5) - 0.5 * glyphsWidth
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainEquation:resetStrikethroughs()
  local glyphs = self.glyphs
  for _, glyph in ipairs(glyphs) do
    glyph:resetStrikethrough()
  end
end

function RainEquation:getInitialGlyphs()
  return self.initialGlyphs
end

function RainEquation:getGlyphs()
  return self.glyphs
end

function RainEquation:createAppearAction()
  local initialGlyphs = self.initialGlyphs

  local appearAction = AHSpawn.new()
  for _, glyph in ipairs(initialGlyphs) do
    local glyphAction = AHOpacityTo.new(glyph, 0.5, nil, 1.0)
    appearAction:addAction(glyphAction)
  end

  return appearAction
end

function RainEquation:createDisappearAction()
  local glyphs = self.glyphs

  local disappearAction = AHSpawn.new()
  for _, glyph in ipairs(glyphs) do
    local glyphAction = glyph:createDisappearAction()
    disappearAction:addAction(glyphAction)
  end

  return disappearAction
end

function RainEquation:addGlyphsForText(text)
  local glyphs = self.glyphs
  local roundColor = self.roundColor

  local addedGlyphs = {}
  for character in string.gmatch(text, LUA_PATTERN_TO_ITERATE_EACH_CHARACTER) do
    local glyph = RainMathGlyph.new(character, roundColor)
    self:insert(glyph)
    table.insert(glyphs, glyph)
    table.insert(addedGlyphs, glyph)
  end

  return addedGlyphs
end

function RainEquation:centerGlyphs(glyphs)
  glyphs = glyphs or self.initialGlyphs
  local startX = getCenterStartX(self, glyphs)
  RainMathLayoutBuilder.alignGlyphs(glyphs, startX)
end

function RainEquation:createCenterAction(glyphs)
  glyphs = glyphs or self.initialGlyphs
  local startX = getCenterStartX(self, glyphs)
  return RainMathActionBuilder.createAlignAction(glyphs, startX)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RainEquation.new(sourceText, roundColor)
  local self = Group.new(2.0, 2.0)
  Mixin.mix(self, RainEquation)

  self.roundColor = roundColor
  self.glyphs = {}

  local initialGlyphs = self:addGlyphsForText(sourceText)
  local width, height = RainMathLayoutBuilder.getDimensionsOfGlyphs(initialGlyphs)
  self:setDims(width, height)
  self.initialGlyphs = initialGlyphs

  self:centerGlyphs()

  return self
end
