--- RainHelper
-- @author Created by Aaron Barrett on July 14, 2015.

class("RainHelper")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

RainHelper.KIND_TAP = "RainHelper.KIND_TAP"
RainHelper.KIND_REDUCE = "RainHelper.KIND_REDUCE"

local TEXT_BY_HELPER_KIND = {
  [RainHelper.KIND_TAP] = Localizer.get("HELPER_TAP"),
  [RainHelper.KIND_REDUCE] = Localizer.get("HELPER_REDUCE")
}

local NEEDS_ARROW_BY_HELPER_KIND = {
  [RainHelper.KIND_TAP] = true,
  [RainHelper.KIND_REDUCE] = false
}

local LABEL_WIDTH = Screen.getSafeWidthPart(0.7)
local BUBBLE_WIDTH = Screen.getSafeWidthPart(0.95)

local BUBBLE_VERTICAL_MARGIN = Screen.getHeightPart(0.02)

local BUBBLE_CORNER_RADIUS = Screen.getHeightPart(0.01)

local BUBBLE_COLOR = {0.0, 0.0, 0.0}

local COLOR_WHITE = {1.0, 1.0, 1.0}
local LABEL_COLOR = COLOR_WHITE

local BUBBLE_Y = Screen.getSafeY(0.15)
local ARROW_FUDGE_Y = 1.0

local LABEL_FONT_SIZE = 21


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainHelper:playAppear()
  local actionInterrupt = self.actionInterrupt

  local appearAction = AHDelayed.new(0.4, AHOpacityTo.new(self, 0.2, nil, 1.0))
  actionInterrupt:insert(appearAction)

  self.showing = true
end

function RainHelper:playDisappear()
  local actionInterrupt = self.actionInterrupt

  local disappearAction = AHOpacityTo.new(self, 0.2, nil, 0.0)
  actionInterrupt:insert(disappearAction)

  self.showing = false
end

function RainHelper:isShowing()
  return self.showing
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------


-- Constructor ------------------------

function RainHelper.new(helperKind, roundColor)
  local self = Group.newFullScreen()
  Mixin.mix(self, RainHelper)

  local text = TEXT_BY_HELPER_KIND[helperKind]
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, LABEL_FONT_SIZE)
  local label = DynamicTextArea.newSimple(text, LABEL_WIDTH, font, Alignment.CENTER, Alignment.CENTER)

  label:setColor(unpack(LABEL_COLOR))
  label:inheritOpacity(self)

  local bubbleHeight = label:getHeight() + BUBBLE_VERTICAL_MARGIN * 2.0
  local bubble = FilledRoundedRectangle.new(BUBBLE_WIDTH, bubbleHeight, BUBBLE_CORNER_RADIUS, BUBBLE_COLOR)
  bubble:insertAtCenter(label)
  bubble:inheritOpacity(self)
  bubble:setLoc(Screen.getSafeX(0.5), BUBBLE_Y + bubbleHeight)
  self:insert(bubble)

  local needsArrow = NEEDS_ARROW_BY_HELPER_KIND[helperKind]
  if (needsArrow) then
    local arrow = Sprite.newWithName("shared_arrow.png")
    arrow:setColor(unpack(BUBBLE_COLOR))
    arrow:inheritOpacity(self)
    arrow:setScl(1.0, -1.0)
    arrow:setLoc(Screen.getSafeX(0.5), bubble:getY() - bubble:getHeightPart(0.5) - arrow:getHeightPart(0.5) + ARROW_FUDGE_Y)
    self:insert(arrow)
  end

  self.actionInterrupt = ActionInterrupt.new()

  self:setOpacity(0.0)
  self.showing = false

  return self
end
