--- RainMathActionBuilder
-- @author Created by Aaron Barrett on July 29, 2015.

class("RainMathActionBuilder")

require("RainMathLayoutBuilder")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainMathActionBuilder.createAlignAction(glyphs, startX)
  local xCoordinates = RainMathLayoutBuilder.getXCoordinatesForGlyphs(glyphs, startX)

  local ease = AHEaseSimpleInOut.new(2.0)

  local alignAction = AHSpawn.new()
  for glyphIndex, glyph in ipairs(glyphs) do
    local destinationX = xCoordinates[glyphIndex]
    local glyphAction = AHLocationXTo.new(glyph, 0.7, ease, destinationX)

    alignAction:addAction(glyphAction)
  end

  return alignAction
end

function RainMathActionBuilder.createGlyphSpawn(glyphs, createActionCallback)
  local spawnAction = AHSpawn.new()
  for _, glyph in ipairs(glyphs) do
    local glyphAction = createActionCallback(glyph)
    spawnAction:addAction(glyphAction)
  end
  return spawnAction
end

function RainMathActionBuilder.addGlyphSpawnToAction(parentAction, glyphs, createActionCallback)
  local glyphSpawn = RainMathActionBuilder.createGlyphSpawn(glyphs, createActionCallback)
  parentAction:addAction(glyphSpawn)
  return glyphSpawn
end

function RainMathActionBuilder.addGlyphActionsToAction(parentAction, glyphs, createActionCallback)
  for _, glyph in ipairs(glyphs) do
    local glyphAction = createActionCallback(glyph)
    parentAction:addAction(glyphAction)
  end
end
