--- RainMathGlyph
-- @author Created by Aaron Barrett on July 08, 2015.

-- NOTE FOR MONDAY, JULY 20, 2015:
-- This class has been cleaned up, but it still has this `k` value factored into
-- its action code.  I'll take that out after this release.

class("RainMathGlyph")

require("RainMathGlyphLabel")
require("RainMathGlyphSprite")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_NUMBER = {0.47, 0.47, 0.47}

local COLOR_STRIKETHROUGH = {0.31, 0.31, 0.31}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function needsNumberColor(character)
  local characterIsNumeric = string.find(character, "%d")
  local characterIsQuestionMark = (character == "?")
  return ((characterIsNumeric) or (characterIsQuestionMark))
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainMathGlyph:addStrikethrough()
  local strikethroughWidth = 2.0
  local height = self:getHeight()
  local strikethrough = FilledRectangle.new(strikethroughWidth, height, COLOR_STRIKETHROUGH)
  strikethrough:setAnchorPointTop()
  strikethrough:setLoc(self:getWidthPart(0.5), height)
  strikethrough:setScl(1.0, 0.0)
  self:insert(strikethrough)
  self.strikethrough = strikethrough
  return strikethrough
end

function RainMathGlyph:createStrikethroughAction()
  local strikethrough = self.strikethrough
  local strikeAction = AHSpawn.new(
                                   AHOpacityTo.new(self, 0.4, nil, 0.4),
                                   AHScaleYTo.new(strikethrough, 0.4, AHEaseSimpleInOut.new(2.0), 1.0)
                                   )
  return strikeAction
end

function RainMathGlyph:resetStrikethrough()
  local strikethrough = self.strikethrough
  if (strikethrough ~= nil) then
    strikethrough:setScl(1.0, 0.0)
    strikethrough:setOpacity(1.0)
  end
end

function RainMathGlyph:createDisappearAction()
  local strikethrough = self.strikethrough

  local fadeAction = AHOpacityTo.new(self, 0.2, nil, 0.0)

  if (strikethrough ~= nil) then
    return AHSpawn.new(
                       fadeAction,
                       AHOpacityTo.new(strikethrough, 0.2, nil, 0.0)
                       )
  end

  return fadeAction
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createGlyphView(character)
  if (RainMathGlyphSprite.filenameSpecifiedForCharacter(character)) then
    return RainMathGlyphSprite.new(character)
  else
    return RainMathGlyphLabel.new(character)
  end
end

-- Constructor ------------------------

function RainMathGlyph.new(character, roundColor)
  local self = createGlyphView(character)
  Mixin.mix(self, RainMathGlyph)

  local color = nil
  if (needsNumberColor(character)) then
    color = COLOR_NUMBER
  else
    color = roundColor
  end
  self:setColor(unpack(color))
  self:setOpacity(0.0)

  self.strikethrough = nil

  return self
end
