--- RainMathLayoutBuilder
-- @author Created by Aaron Barrett on July 08, 2015.

class("RainMathLayoutBuilder")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainMathLayoutBuilder.getDimensionsOfGlyphs(glyphs)
  local penX = 0.0
  local maxHeight = 0.0

  for glyphIndex, glyph in ipairs(glyphs) do
    penX = penX + glyph:getLeftMargin() + glyph:getWidth() + glyph:getRightMargin()
    maxHeight = math.max(maxHeight, glyph:getHeight())
  end

  local width = penX
  local height = maxHeight

  return width, height
end

function RainMathLayoutBuilder.getWidthOfGlyphs(glyphs)
  local width, _ = RainMathLayoutBuilder.getDimensionsOfGlyphs(glyphs)
  return width
end

function RainMathLayoutBuilder.getHeightOfGlyphs(glyphs)
  local _, height = RainMathLayoutBuilder.getDimensionsOfGlyphs(glyphs)
  return height
end

function RainMathLayoutBuilder.getXCoordinatesForGlyphs(glyphs, startX)
  local xCoordinates = {}

  local penX = startX or 0.0

  for glyphIndex, glyph in ipairs(glyphs) do
    local glyphWidth = glyph:getWidth()
    local glyphLeftMargin = glyph:getLeftMargin()

    glyphX = penX + glyphLeftMargin + glyphWidth * 0.5
    xCoordinates[glyphIndex] = glyphX
    penX = penX + glyphLeftMargin + glyphWidth + glyph:getRightMargin()
  end

  return xCoordinates
end

function RainMathLayoutBuilder.alignGlyphs(glyphs, startX, glyphsY)
  startX = startX or 0.0
  glyphsY = glyphsY or 0.5 * RainMathLayoutBuilder.getHeightOfGlyphs(glyphs)

  local xCoordinates = RainMathLayoutBuilder.getXCoordinatesForGlyphs(glyphs, startX)
  for glyphIndex, glyph in ipairs(glyphs) do
    local glyphX = xCoordinates[glyphIndex]
    glyph:setLoc(glyphX, glyphsY)
  end
end
