--- RainMoneyBar
-- @author Created by Aaron Barrett on July 06, 2015.

class("RainMoneyBar")

require("RainMoneyBarColumn")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLUMN_SPACING = Screen.getWidthPart(0.01)

local DEMO_COLUMN_INDEX = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainMoneyBar:getSelectedAmount()
  local columns = self.columns

  local totalAmount = 0
  for _, column in ipairs(columns) do
    totalAmount = totalAmount + column:getSelectedAmount()
  end

  return totalAmount
end

function RainMoneyBar:setTouchEnabled(enabled)
  local columns = self.columns
  for _, column in ipairs(columns) do
    column:setTouchEnabled(enabled)
  end
end

function RainMoneyBar:createAppearAction()
  local columns = self.columns
  local demoColumn = self.demoColumn

  local columnsAction = AHSpawn.new()
  local ease = AHEaseBackOut.new(1.3)

  for _, column in ipairs(columns) do
    local delayDuration = math.random() * 0.13
    local columnAction = AHDelayed.new(delayDuration, AHLocationYTo.new(column, 0.5, ease, self:getHeightPart(0.5)))
    columnsAction:addAction(columnAction)
  end

  local appearAction = AHSequence.new(
                                      columnsAction,
                                      function()
                                        self:setTouchEnabled(true)
                                      end
                                      )
  return appearAction
end

function RainMoneyBar:playDisappear()
  self:setTouchEnabled(false)

  local disappearAction = AHLocationYTo.new(self, 0.2, AHEaseExponentialIn.new(), -self:getHeightPart(0.5))
  return disappearAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RainMoneyBar.new(denominations, onBillAddedCallback, onBillRemovedCallback, roundColor)
  local self = Row.new(COLUMN_SPACING)
  Mixin.mix(self, RainMoneyBar)

  local columns = {}

  for _, denomination in ipairs(denominations) do
    local column = RainMoneyBarColumn.new(denomination, onBillAddedCallback, onBillRemovedCallback, roundColor)
    self:insert(column)
    table.insert(columns, column)
  end

  self:alignChildren()

  for _, column in ipairs(columns) do
    column:setY(-self:getHeightPart(0.5))
  end

  self.columns = columns
  self.demoColumn = columns[DEMO_COLUMN_INDEX]

  return self
end
