--- RainOutro
-- @author Created by Aaron Barrett on July 07, 2015.

class("RainOutro")

require("graphics.primitives.Checkmark")
require("RainZipper")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CHECKMARKS_COUNT = 5

local CHECKMARK_WIDTH = Screen.getWidthPart(0.3)

RainOutro.OUTRO_COLORS_COUNT = 3

local Z_WHITE_BACKGROUND = 10
local Z_BACK_ZIPPER = 20
local Z_FRONT_ZIPPER = 30
local Z_SLIDER = 40

local COLOR_WHITE = {1.0, 1.0, 1.0}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainOutro:playOutro()
  local checkmarks = self.checkmarks
  local slider = self.slider

  local sliderAction = AHSequence.new(
                                      AHLocationYTo.new(slider, 0.8, AHEaseElasticOut.new(0.8), self:getHeightPart(0.48)),

                                      AHLocationYTo.new(slider, 0.8, AHEaseExponentialInOut.new(4.0), self:getHeightPart(0.5)),
                                      AHLocationYTo.new(slider, 0.8, AHEaseExponentialInOut.new(4.0), self:getHeightPart(0.48))
                                      )

  local staggeredCheckmarksAction = AHStaggeredSpawn.new(0.026)

  for checkmarkIndex, checkmark in rpairs(checkmarks) do
    local secondDestinationY = (checkmarkIndex / CHECKMARKS_COUNT) * self:getHeightPart(0.23)
    local thirdDestinationY = slider:getHeightPart(0.34) + (checkmarkIndex / CHECKMARKS_COUNT) * self:getHeightPart(0.15)
    local fourthDestinationY = secondDestinationY

    local checkmarkAction = AHSequence.new(
                                           AHLocationYTo.new(checkmark, 0.8, AHEaseElasticOut.new(0.8), secondDestinationY),
                                           AHLocationYTo.new(checkmark, 0.8, AHEaseElasticOut.new(0.6), thirdDestinationY),
                                           AHLocationYTo.new(checkmark, 0.8, AHEaseElasticOut.new(0.5), fourthDestinationY)
                                           )
    staggeredCheckmarksAction:addAction(checkmarkAction)
  end

  local checkmarksAction = AHSpawn.new(
                                       sliderAction,
                                       AHDelayed.new(0.3, staggeredCheckmarksAction)
                                       )

  local roundColors = self.roundColors
  local frontZipper = self.frontZipper
  local frontZipperRay = frontZipper:getRayTransform()
  local backZipper = self.backZipper
  local backZipperRay = backZipper:getRayTransform()
  local whiteBackground = self.whiteBackground

  local zippersAction = AHSequence.new(
                                       AHSpawn.new(
                                                   AHLocationXTo.new(frontZipperRay, 0.3, AHEaseSimpleInOut.new(2.0), self:getHeightPart(0.5)),
                                                   AHRotationBy.new(frontZipperRay, 0.8, AHEaseSimpleInOut.new(4.0), 180.0)
                                                   ),
                                       function()
                                         backZipperRay:setRot(180.0)
                                       end,
                                       AHRotationTo.new(frontZipperRay, 0.8, AHEaseSimpleInOut.new(4.0), 0.0),
                                       function()
                                         whiteBackground:setVisible(true)

                                         frontZipper:setScl(-1.0, 1.0)
                                         frontZipper:setColor(unpack(roundColors[2]))
                                         frontZipperRay:setRot(180.0)


                                         backZipper:setScl(-1.0, 1.0)
                                         backZipper:setColor(unpack(roundColors[3]))
                                         backZipperRay:setRot(180.0)
                                       end,
                                       AHSpawn.new(
                                                   AHRotationBy.new(frontZipperRay, 0.8, AHEaseSimpleInOut.new(4.0), -180.0),
                                                   AHRotationBy.new(backZipperRay, 1.5, AHEaseSimpleInOut.new(8.0), -180.0)
                                                   )
                                       )

  local outroAction = AHSpawn.new(
                                  checkmarksAction,
                                  zippersAction
                                  )

  return outroAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createZipper(self, initialColor)
  -- Because of the way that math.atan2 works, it's easiest to orient zippers pointing to the left
  -- Since we want them pointing at the top of the screen, just swap their width and height and
  -- rotate them 90 degrees.
  local height = self:getWidth()
  local width = self:getHeight()
  local zipper = RainZipper.new(width, height, initialColor)
  zipper:setRot(90.0)
  return zipper
end

-- Constructor ------------------------

function RainOutro.new(roundColors)
  local self = Group.newFullScreen()
  Mixin.mix(self, RainOutro)

  local whiteBackground = FilledRectangle.new(self:getWidth(), self:getHeight(), COLOR_WHITE)
  whiteBackground:setVisible(false)
  self:insertAtCenter(whiteBackground, Z_WHITE_BACKGROUND)
  self.whiteBackground = whiteBackground

  local backZipperInitialColor = roundColors[2]
  local backZipper = createZipper(self, backZipperInitialColor)
  self:insertAtCenter(backZipper, Z_BACK_ZIPPER)
  self.backZipper = backZipper

  local frontZipperInitialColor = roundColors[1]
  local frontZipper = createZipper(self, frontZipperInitialColor)
  self:insertAtCenter(frontZipper, Z_FRONT_ZIPPER)
  self.frontZipper = frontZipper

  local frontZipperRay = frontZipper:getRayTransform()
  frontZipperRay:setLoc(self:getHeight(), self:getWidthPart(0.5))

  local slider = Group.new(CHECKMARK_WIDTH, self:getHeightPart(0.32))

  local checkmarks = {}
  for checkmarkIndex = 1, CHECKMARKS_COUNT do
    local checkmark = FilledCheckmark.new(CHECKMARK_WIDTH)

    local opacity = checkmarkIndex / CHECKMARKS_COUNT
    checkmark:setOpacity(opacity)

    slider:insertAtCenter(checkmark)

    table.insert(checkmarks, checkmark)
  end
  self.checkmarks = checkmarks

  local topCheckmark = checkmarks[CHECKMARKS_COUNT]
  slider:setLoc(self:getWidthPart(0.5), self:getHeight() + topCheckmark:getHeightPart(0.5))
  self:insert(slider, Z_SLIDER)
  self.slider = slider

  self.roundColors = roundColors

  return self
end
