--- RainRound
-- @author Created by Aaron Barrett on July 06, 2015.

class("RainRound")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LOW_ANSWER_DENOMINATIONS = {20, 10, 5, 1}
local MEDIUM_ANSWER_DENOMINATIONS = {50, 10, 5, 1}
local HIGH_ANSWER_DENOMINATIONS = {100, 10, 5, 1}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainRound:getConceptIdentifier()
  return self.conceptIdentifier
end

function RainRound:getTipIdentifier()
  return self.tipIdentifier
end

function RainRound:getDividend()
  return self.dividend
end

function RainRound:getDivisor()
  return self.divisor
end

function RainRound:getTipText()
  return self.tipText
end

function RainRound:getCorrectAnswer()
  return MathUtils.round(self.dividend / self.divisor)
end

function RainRound:getDenominations()
  return self.denominations
end

function RainRound:getTipPartition()
  return self.tipPartition
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function chooseDenominations(self)
  local correctAnswer = self:getCorrectAnswer()

  if (correctAnswer < 90) then
    return LOW_ANSWER_DENOMINATIONS
  elseif (correctAnswer < 140) then
    return MEDIUM_ANSWER_DENOMINATIONS
  else
    return HIGH_ANSWER_DENOMINATIONS
  end
end

-- Constructor ------------------------

function RainRound.new(concept)
  local self = {}
  Mixin.mix(self, RainRound)

  self.conceptIdentifier = concept["identifier"]
  self.dividend = concept["dividend"]
  self.divisor = concept["divisor"]
  self.tipPartition = concept["part_number"]
  self.tipIdentifier = concept["tip_identifier"]
  self.tipText = Localizer.get("TIP_" .. self.tipIdentifier)

  self.denominations = chooseDenominations(self)

  return self
end
