--- RainTipDropZeroes
-- @author Created by Aaron Barrett on July 08, 2015.

class("RainTipDropZeroes")
require("RainEquation")
require("RainMathLayoutBuilder")
require("RainMathActionBuilder")
require("RainZeroHelpers")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainTipDropZeroes:createAppearAction()
  local equationLine = self.equationLine
  return equationLine:createAppearAction()
end

function RainTipDropZeroes:createDisappearAction()
  local equationLine = self.equationLine
  return equationLine:createDisappearAction()
end

function RainTipDropZeroes:prepareToAppear()
  local equationLine = self.equationLine
  equationLine:centerGlyphs()
  equationLine:resetStrikethroughs()
end

function RainTipDropZeroes:createTipAction()
  local glyphsToDrop = self.glyphsToDrop
  local glyphsToCenter = self.glyphsToCenter
  local equationLine = self.equationLine
  local questionMarkGlyph = self.questionMarkGlyph
  local droppedSolutionGlyphs = self.droppedSolutionGlyphs
  local initialGlyphs = equationLine:getInitialGlyphs()

  local tipAction = AHSequence.new()

  RainMathActionBuilder.addGlyphSpawnToAction(tipAction, glyphsToDrop, function(glyph)
    return glyph:createStrikethroughAction()
  end)

  tipAction:addAction(AHDelay.new(1.4))

  local alignPhaseAction = AHSpawn.new()
  local centerAction = equationLine:createCenterAction(glyphsToCenter)
  alignPhaseAction:addAction(centerAction)
  local questionMarkFadeAction = AHOpacityTo.new(questionMarkGlyph, 0.5, nil, 0.0)
  alignPhaseAction:addAction(questionMarkFadeAction)
  RainMathActionBuilder.addGlyphSpawnToAction(alignPhaseAction, glyphsToDrop, function(glyph)
    return glyph:createDisappearAction()
  end)
  tipAction:addAction(alignPhaseAction)

  RainMathActionBuilder.addGlyphSpawnToAction(tipAction, droppedSolutionGlyphs, function(glyph)
    return AHOpacityTo.new(glyph, 0.5, nil, 1.0)
  end)

  return tipAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RainTipDropZeroes.new(round, roundColor)
  local dividend = round:getDividend()
  local divisor = round:getDivisor()

  local dividendText = tostring(dividend)
  local divisorText = tostring(divisor)

  local dividendZeroesToDrop, divisorZeroesToDrop = RainZeroHelpers.getZeroesToDrop(dividendText, divisorText)

  local dividendTextLength = #dividendText
  local dividendDroppedText = string.sub(dividendText, 1, dividendTextLength - dividendZeroesToDrop)
  local dividendDropped = tonumber(dividendDroppedText)

  local divisorTextLength = #divisorText
  local divisorDroppedText = string.sub(divisorText, 1, divisorTextLength - divisorZeroesToDrop)
  local divisorDropped = tonumber(divisorDroppedText)

  local droppedSolution = MathUtils.round(dividendDropped / divisorDropped)
  local droppedSolutionText = tostring(droppedSolution)

  local divisionSignText = " / "
  local divisionSignTextLength = #divisionSignText
  local divisorStartIndex = dividendTextLength + divisionSignTextLength + divisorTextLength - divisorZeroesToDrop + 1

  local equationText = dividendText .. divisionSignText .. divisorText .. " = ?"
  local equationLine = RainEquation.new(equationText, roundColor)

  local width, height = equationLine:getDims()

  local self = Group.new(width, height)
  Mixin.mix(self, RainTipDropZeroes)

  local initialGlyphs = equationLine:getInitialGlyphs()
  local initialGlyphsCount = #initialGlyphs

  local questionMarkGlyph = initialGlyphs[initialGlyphsCount]

  local droppedSolutionGlyphs = equationLine:addGlyphsForText(droppedSolutionText)

  local dividendGlyphsToDrop = TableUtils.subarray(initialGlyphs, dividendTextLength - dividendZeroesToDrop + 1, dividendTextLength)
  local divisorGlyphsToDrop = TableUtils.subarray(initialGlyphs, divisorStartIndex, divisorStartIndex + divisorZeroesToDrop - 1)
  local glyphsToDrop = TableUtils.createConcatenatedArray(dividendGlyphsToDrop, divisorGlyphsToDrop)

  local remainingLeftGlyphs = TableUtils.subarray(initialGlyphs, 1, dividendTextLength - dividendZeroesToDrop)
  local remainingMiddleGlyphs = TableUtils.subarray(initialGlyphs, dividendTextLength + 1, divisorStartIndex - 1)
  local remainingRightGlyphs = TableUtils.subarray(initialGlyphs, divisorStartIndex + divisorZeroesToDrop, initialGlyphsCount - 1)
  local glyphsToCenter = TableUtils.createConcatenatedArray(remainingLeftGlyphs, remainingMiddleGlyphs, remainingRightGlyphs, droppedSolutionGlyphs)

  RainMathLayoutBuilder.alignGlyphs(droppedSolutionGlyphs)

  FunctionUtils.callMethodOnTargets("addStrikethrough", glyphsToDrop)

  self:insertAtCenter(equationLine)
  self.equationLine = equationLine

  self.glyphsToDrop = glyphsToDrop
  self.glyphsToCenter = glyphsToCenter
  self.questionMarkGlyph = questionMarkGlyph
  self.droppedSolutionGlyphs = droppedSolutionGlyphs

  return self
end
