--- RainTipSplitDividend
-- @author Created by Aaron Barrett on July 10, 2015.

class("RainTipSplitDividend")
require("RainEquation")
require("RainMathLayoutBuilder")
require("RainMathActionBuilder")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainTipSplitDividend:createAppearAction()
  local topEquationLine = self.topEquationLine
  local appearAction = topEquationLine:createAppearAction()
  return appearAction
end

function RainTipSplitDividend:createDisappearAction()
  local topEquationLine = self.topEquationLine
  local bottomEquationLine = self.bottomEquationLine

  local disappearAction = AHSpawn.new(
                                      topEquationLine:createDisappearAction(),
                                      bottomEquationLine:createDisappearAction()
                                      )
  return disappearAction
end

function RainTipSplitDividend:prepareToAppear()
  local bottomDividendGlyphs = self.bottomDividendGlyphs
  local topEquationLineY = self.topEquationLineY

  for _, glyph in ipairs(bottomDividendGlyphs) do
    glyph:setY(topEquationLineY)
  end
end

function RainTipSplitDividend:createTipAction()
  local topEquationLine = self.topEquationLine
  local bottomEquationLine = self.bottomEquationLine
  local originalDividendGlyphs = self.originalDividendGlyphs
  local topDividendGlyphs = self.topDividendGlyphs
  local bottomDividendGlyphs = self.bottomDividendGlyphs

  local bottomEquationInitialGlyphs = bottomEquationLine:getInitialGlyphs()

  local bottomEquationSignGlyphs = self.bottomEquationSignGlyphs

  local topQuestionMarkGlyph = self.topQuestionMarkGlyph
  local bottomQuestionMarkGlyph = self.bottomQuestionMarkGlyph

  local topQuotientGlyphs = self.topQuotientGlyphs
  local bottomQuotientGlyphs = self.bottomQuotientGlyphs

  local tipAction = AHSequence.new()

  RainMathActionBuilder.addGlyphSpawnToAction(tipAction, originalDividendGlyphs, function(glyph)
    return AHOpacityTo.new(glyph, 0.4, nil, 0.0)
  end)

  local bottomEquationAppearPhaseAction = AHSpawn.new()
  RainMathActionBuilder.addGlyphActionsToAction(bottomEquationAppearPhaseAction, bottomEquationInitialGlyphs, function(glyph)
    return AHOpacityTo.new(glyph, 0.4, nil, 1.0)
  end)
  RainMathActionBuilder.addGlyphActionsToAction(bottomEquationAppearPhaseAction, bottomDividendGlyphs, function(glyph)
    return AHLocationYTo.new(glyph, 0.4, slideEase, bottomEquationLine:getHeightPart(0.5))
  end)
  RainMathActionBuilder.addGlyphActionsToAction(bottomEquationAppearPhaseAction, topDividendGlyphs, function(glyph)
    return AHOpacityTo.new(glyph, 0.4, nil, 1.0)
  end)
  RainMathActionBuilder.addGlyphActionsToAction(bottomEquationAppearPhaseAction, bottomEquationSignGlyphs, function(glyph)
    return AHOpacityTo.new(glyph, 0.4, nil, 1.0)
  end)
  tipAction:addAction(bottomEquationAppearPhaseAction)

  tipAction:addAction(AHDelay.new(1.4))

  local questionMarkDisappearPhaseAction = AHSpawn.new(
                                                       AHOpacityTo.new(topQuestionMarkGlyph, 0.4, nil, 0.0),
                                                       AHOpacityTo.new(bottomQuestionMarkGlyph, 0.4, nil, 0.0)
                                                       )
  tipAction:addAction(questionMarkDisappearPhaseAction)

  local quotientAppearPhaseAction = AHSpawn.new()
  RainMathActionBuilder.addGlyphActionsToAction(quotientAppearPhaseAction, topQuotientGlyphs, function(glyph)
    return AHOpacityTo.new(glyph, 0.2, nil, 1.0)
  end)
  RainMathActionBuilder.addGlyphActionsToAction(quotientAppearPhaseAction, bottomQuotientGlyphs, function(glyph)
    return AHOpacityTo.new(glyph, 0.2, nil, 1.0)
  end)
  tipAction:addAction(quotientAppearPhaseAction)

  return tipAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RainTipSplitDividend.new(round, roundColor)
  local dividend = round:getDividend()
  local divisor = round:getDivisor()
  local partition = round:getTipPartition()

  local topDividend = math.max(partition, dividend - partition)
  local bottomDividend = math.abs(dividend - topDividend)
  local bottomEquationSign = MathUtils.sign(dividend - topDividend)

  local originalDividendText = tostring(dividend)
  local topDividendText = tostring(topDividend)
  local bottomDividendText = tostring(bottomDividend)

  local divisorText = tostring(divisor)

  local divisionSignText = " / "
  local topEquationText = originalDividendText .. divisionSignText .. divisorText .. " = ?"
  local bottomEquationText = bottomDividendText .. divisionSignText .. divisorText .. " = ?"

  local topEquationLine = RainEquation.new(topEquationText, roundColor)
  local bottomEquationLine = RainEquation.new(bottomEquationText, roundColor)

  local width = topEquationLine:getWidth()
  local height = topEquationLine:getHeight() + bottomEquationLine:getHeight()

  local topEquationLineY = topEquationLine:getHeightPart(1.5)
  local bottomEquationLineY = topEquationLine:getHeightPart(0.5)

  local self = Group.new(width, height)
  Mixin.mix(self, RainTipSplitDividend)

  self.topEquationLineY = topEquationLineY

  local topEquationGlyphs = topEquationLine:getInitialGlyphs()
  local bottomEquationGlyphs = bottomEquationLine:getInitialGlyphs()

  local originalDividendTextLength = #originalDividendText
  local originalDividendFinalDigitGlyph = topEquationGlyphs[originalDividendTextLength]

  local topDividendGlyphs = topEquationLine:addGlyphsForText(topDividendText)
  self.topDividendGlyphs = topDividendGlyphs

  local originalDividendGlyphs = TableUtils.subarray(topEquationGlyphs, 1, originalDividendTextLength)
  self.originalDividendGlyphs = originalDividendGlyphs

  local postDividendSpaceGlyph = topEquationGlyphs[originalDividendTextLength + 1]
  local topDividendWidth = RainMathLayoutBuilder.getWidthOfGlyphs(topDividendGlyphs)
  local topDividendStartX = postDividendSpaceGlyph:getX() - postDividendSpaceGlyph:getLeftMargin() - topDividendWidth
  RainMathLayoutBuilder.alignGlyphs(topDividendGlyphs, topDividendStartX)

  local bottomDividendTextLength = #bottomDividendText
  local bottomDividendFinalDigitGlyph = bottomEquationGlyphs[bottomDividendTextLength]

  local bottomDividendGlyphs = TableUtils.subarray(bottomEquationGlyphs, 1, bottomDividendTextLength)
  self.bottomDividendGlyphs = bottomDividendGlyphs

  local topEquationTextLength = #topEquationText
  self.topQuestionMarkGlyph = topEquationGlyphs[topEquationTextLength]

  local bottomEquationTextLength = #bottomEquationText
  self.bottomQuestionMarkGlyph = bottomEquationGlyphs[bottomEquationTextLength]

  local bottomEquationX = 0.5 * (width + originalDividendFinalDigitGlyph:getX() - bottomDividendFinalDigitGlyph:getX())

  topEquationLine:setLoc(self:getWidthPart(0.5), topEquationLineY)
  self:insert(topEquationLine)
  self.topEquationLine = topEquationLine

  bottomEquationLine:setLoc(bottomEquationX, bottomEquationLineY)
  self:insert(bottomEquationLine)
  self.bottomEquationLine = bottomEquationLine

  local topQuotient = MathUtils.round(topDividend / divisor)
  local topQuotientText = tostring(topQuotient)
  local topQuotientGlyphs = topEquationLine:addGlyphsForText(topQuotientText)
  local topEquationGlyphsMinusQuestionMark = TableUtils.subarray(topEquationGlyphs, 1, topEquationTextLength - 1)
  local topQuotientStartX = RainMathLayoutBuilder.getWidthOfGlyphs(topEquationGlyphsMinusQuestionMark)
  RainMathLayoutBuilder.alignGlyphs(topQuotientGlyphs, topQuotientStartX)
  self.topQuotientGlyphs = topQuotientGlyphs

  local bottomQuotient = MathUtils.round(bottomDividend / divisor)
  local bottomQuotientText = tostring(bottomQuotient)
  local bottomQuotientGlyphs = bottomEquationLine:addGlyphsForText(bottomQuotientText)
  local bottomEquationGlyphsMinusQuestionMark = TableUtils.subarray(bottomEquationGlyphs, 1, bottomEquationTextLength - 1)
  local bottomQuotientStartX = RainMathLayoutBuilder.getWidthOfGlyphs(bottomEquationGlyphsMinusQuestionMark)
  RainMathLayoutBuilder.alignGlyphs(bottomQuotientGlyphs, bottomQuotientStartX)
  self.bottomQuotientGlyphs = bottomQuotientGlyphs

  local bottomEquationSignText = nil
  if (bottomEquationSign < 0) then
    bottomEquationSignText = "- "
  else
    bottomEquationSignText = "+ "
  end
  local bottomEquationSignGlyphs = bottomEquationLine:addGlyphsForText(bottomEquationSignText)
  local bottomEquationSignWidth = RainMathLayoutBuilder.getWidthOfGlyphs(bottomEquationSignGlyphs)
  RainMathLayoutBuilder.alignGlyphs(bottomEquationSignGlyphs, -bottomEquationSignWidth)
  self.bottomEquationSignGlyphs = bottomEquationSignGlyphs

  return self
end
