--- RainZipper
-- @author Created by Aaron Barrett on July 15, 2015.

class("RainZipper")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RainZipper:getRayTransform()
  return self.rayTransform
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RainZipper.new(width, height, color)
  local self = Group.newCustomDraw(width, height, color)
  Mixin.mix(self, RainZipper)

  local rayTransform = MOAITransform2D.new()
  rayTransform:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
  rayTransform:setRot(0.0)
  self.rayTransform = rayTransform

  local function drawCallback()
    local rayX, rayY = self.rayTransform:getLoc()

    local rayAngle = math.rad(self.rayTransform:getRot())
    if (rayAngle > math.pi) then
      rayAngle = rayAngle - math.pi * 2.0
    end

    local angleToTopLeft = math.atan2(height - rayY, -rayX)
    local angleToBottomLeft = math.atan2(-rayY, -rayX)
    local angleToTopRight = math.atan2(height - rayY, width - rayX)
    local angleToBottomRight = math.atan2(-rayY, width - rayX)

    local coordinates = {
      rayX, rayY,
    }

    if (rayAngle < angleToTopRight) then
      table.insert(coordinates, width)
      table.insert(coordinates, rayY + math.tan(rayAngle) * (width - rayX))

    elseif (rayAngle < angleToTopLeft) then
      table.insert(coordinates, rayX + math.tan(rayAngle - math.pi * 0.5) * (rayY - height))
      table.insert(coordinates, height)

      table.insert(coordinates, width)
      table.insert(coordinates, height)

    else
      table.insert(coordinates, 0.0)
      table.insert(coordinates, rayY - math.tan(rayAngle) * rayX)

      table.insert(coordinates, 0.0)
      table.insert(coordinates, height)

      table.insert(coordinates, width)
      table.insert(coordinates, height)
    end

    local mirrorRayAngle = -rayAngle

    if (mirrorRayAngle > angleToBottomRight) then
      table.insert(coordinates, width)
      table.insert(coordinates, rayY + math.tan(mirrorRayAngle) * (width - rayX))

    elseif (mirrorRayAngle > angleToBottomLeft) then
      table.insert(coordinates, width)
      table.insert(coordinates, 0.0)

      table.insert(coordinates, rayX + math.tan(mirrorRayAngle - math.pi * 0.5) * rayY)
      table.insert(coordinates, 0.0)

    else
      table.insert(coordinates, width)
      table.insert(coordinates, 0.0)

      table.insert(coordinates, 0.0)
      table.insert(coordinates, 0.0)

      table.insert(coordinates, 0.0)
      table.insert(coordinates, rayY - math.tan(mirrorRayAngle) * rayX)
    end

    MOAIDraw.fillFan(coordinates)
  end
  self.deck:setDrawCallback(drawCallback)

  return self
end
