--- RiffleButton
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleButton")

require("RiffleCommon")
require("actions.advanced.Shake")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------
local DEFAULT_LABEL_TEXT = "-unset-"
local LABEL_FONT = FontFaces.DIN_OT_MEDIUM
local DEFAULT_FONT_SIZE = 20
local NO_TINT = RiffleCommon.COLORS.WHITE
local DEFAULT_TEXT_COLOR = RiffleCommon.COLORS.WHITE
local FRAME_RATE = 60
local DEFAULT_WIDTH = Screen.getWidthPart(0.875)

RiffleButton.BUTTON_TYPE =
{
  ORANGE = "RiffleButton.BUTTON_TYPE.ORANGE",
  BLUE = "RiffleButton.BUTTON_TYPE.BLUE",
  BLACK = "RiffleButton.BUTTON_TYPE.BLACK"
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createLayout(self, textColor, fontSize)
  local baseAttributes = {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, fontSize)
  }

  local attributesForLabels = {
    b = { font = Font.new(FontFaces.DIN_OT_BOLD, fontSize) },
    i = { font = Font.new(FontFaces.DIN_OT_MEDIUM_ITALIC, fontSize) }
  }

  local width = self:getWidth()
  local label = DynamicTextArea.new(DEFAULT_LABEL_TEXT, width, baseAttributes, attributesForLabels, Alignment.CENTER)
  label:setColor(unpack(textColor))
  self:insertAtCenter(label)

  self.label = label
end

local function createButtonByType(buttonType, fontSize)
  fontSize = fontSize or DEFAULT_FONT_SIZE
  local spriteName = nil
  local tintColor = NO_TINT
  local textColor = DEFAULT_TEXT_COLOR
  local pressedAnimation = nil

  if (buttonType == RiffleButton.BUTTON_TYPE.ORANGE) then
    spriteName = "buttonOrange_000.png"
    pressedAnimation = Animation.newWithPrefix("buttonOrange_")
  elseif (buttonType == RiffleButton.BUTTON_TYPE.BLUE) then
    spriteName = "buttonBlue_000.png"
    pressedAnimation = Animation.newWithPrefix("buttonBlue_")
  elseif (buttonType == RiffleButton.BUTTON_TYPE.BLACK) then
    -- Use the blue assets and just tint them dark
    spriteName = "buttonBlue_000.png"
    pressedAnimation = Animation.newWithPrefix("buttonBlue_")
    tintColor = RiffleCommon.COLORS.BLACK
    textColor = RiffleCommon.COLORS.BLACK
  else
    error("No course of action defined for button type: " .. tostring(buttonType))
  end

  local buttonSprite = Sprite.newWithName(spriteName)
  local targetScale = DEFAULT_WIDTH / buttonSprite:getWidth()
  buttonSprite:setScl(targetScale)

  local self = Group.new(buttonSprite:getEffectiveDims())
  buttonSprite:setColor(unpack(tintColor))
  self:insertAtCenter(buttonSprite)

  createLayout(self, textColor, fontSize)

  self.pressedAnimateAction = AHAnimate.new(buttonSprite, pressedAnimation, FRAME_RATE)
  self.pressedAnimation = pressedAnimation
  self.buttonType = buttonType
  self.buttonSprite = buttonSprite

  return self
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------
function RiffleButton:setText(text)
  self.label:setText(text)
end

function RiffleButton:doPressedAnimation()
  self.pressedAnimateAction:start()

  if (self.buttonType == RiffleButton.BUTTON_TYPE.BLACK) then
    self.label:setColor(unpack(RiffleCommon.COLORS.WHITE))
  end
end

function RiffleButton:doShakeAnimation()
  local groupShakeWidth = self:getWidthPart(1.5)
  local shakeAction = AHAxisOffsetShake.new(self, groupShakeWidth)
  shakeAction:start()
end

function RiffleButton:fadeOut()
  local fadeOut = AHOpacityTo.new(self, 1.2, nil, RiffleCommon.INVISIBLE_OPACITY)
  return fadeOut:start()
end

function RiffleButton:showAsCorrect()
  return AHDelay.new(2):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------
local function newInternal(buttonType, fontSize)
  local self = createButtonByType(buttonType, fontSize)
  Mixin.mix(self, RiffleButton)

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)

  return self
end

function RiffleButton.newNoTouchHandling(color, fontSize)
  local self = newInternal(color, fontSize)

  return self
end

function RiffleButton.new(buttonType, pressedCallback, fontSize)
  local self = newInternal(buttonType, fontSize)
  Button.makeButton(self, pressedCallback)

  return self
end
