--- RiffleConceptGenerator
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleConceptGenerator")

require("data.generators.UtilityGenerators")
require("RiffleCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function chooseLayoutStyle(round)
  local hasSubheader = (round.subheader ~= nil) and (#round.subheader > 0)
  local hasImage = (round.image ~= nil) and (#round.image > 0)

  if hasSubheader and hasImage then
    return RiffleCommon.LayoutStyle.IMAGE_AND_SUBHEADER
  elseif hasSubheader and not hasImage then
    return RiffleCommon.LayoutStyle.SUBHEADER_ONLY
  elseif not hasSubheader and hasImage then
    return RiffleCommon.LayoutStyle.IMAGE_ONLY
  else
    return RiffleCommon.LayoutStyle.NO_ORNAMENTS
  end
end

local function categorizeQuestionsInDifficultyBuckets(questions)
  local bucketedQuestions = {}
  for _, question in ipairs(questions) do
    local difficulty = question.filter_difficulty

    if not bucketedQuestions[difficulty] then
      bucketedQuestions[difficulty] = {}
    end

    local bucket = bucketedQuestions[difficulty]
    table.insert(bucket, question)
  end

  return bucketedQuestions
end

local function selectQuestionsForRound(questions, difficulties)
  local bucketedQuestions = categorizeQuestionsInDifficultyBuckets(questions)

  local selectedQuestions = {}
  local doRemove = true
  for _, difficulty in ipairs(difficulties) do
    local questionBucket = bucketedQuestions[difficulty]
    local selectedQuestion = TableUtils.getRandomElement(questionBucket, doRemove)

    table.insert(selectedQuestions, selectedQuestion)
  end

  return selectedQuestions
end

local function packageConceptIntoRound(concept)
  local round = {}
  round.identifier = concept.identifier
  round.passage = concept.passage
  round.image = concept.image
  round.imageCredit = concept.imageCredit
  round.assets_path = concept.assets_path

  if concept.title then
    round.title = string.upper(concept.title)
  end

  if concept.subheader then
    round.subheader = string.upper(concept.subheader)
  end

  local questions = concept.content
  local allSkimmingQuestions = TableUtils.getFilteredArray(questions, function(value) return (value.round == "skim") end)
  local allScanningQuestions = TableUtils.getFilteredArray(questions, function(value) return (value.round == "scan") end)

  local skimmingDifficulties = RiffleConfiguration.getSkimmingDifficulties()
  local selectedSkimmingQuestions = selectQuestionsForRound(allSkimmingQuestions, skimmingDifficulties)

  local scanningDifficulties = RiffleConfiguration.getScanningDifficulties()
  local selectedScanningQuestions = selectQuestionsForRound(allScanningQuestions, scanningDifficulties)

  round.skimmingQuestions = selectedSkimmingQuestions
  round.scanningQuestions = selectedScanningQuestions

  round.layoutStyle = chooseLayoutStyle(round)

  return round
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function RiffleConceptGenerator.new(concept)
  local conceptGenerator = GameSession.conceptGenerator
  local self = TransformationGenerator.new(conceptGenerator, function(concept) return packageConceptIntoRound(concept) end)
  Mixin.mix(self, RiffleConceptGenerator)

  return self
end
