--- RiffleOutro
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleOutro")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------
local PANEL_HEIGHT = Screen.height
local PANEL_SHEER_AMOUNT = Screen.getHeightPart(0.05)
local LEFT_X = -Screen.getWidthPart(0.5)
local RIGHT_X = Screen.getWidthPart(0.5)
local BOTTOM_Y = PANEL_HEIGHT * -0.5
local TOP_Y = PANEL_HEIGHT * 0.5

local START_VERTICIES =
{
  {x = LEFT_X, y = BOTTOM_Y},
  {x = RIGHT_X, y = BOTTOM_Y},
  {x = RIGHT_X, y = TOP_Y},
  {x = LEFT_X, y = TOP_Y}
}

local END_VERTICIES =
{
  {x = LEFT_X, y = BOTTOM_Y - PANEL_SHEER_AMOUNT},
  {x = RIGHT_X, y = BOTTOM_Y + PANEL_SHEER_AMOUNT},
  {x = RIGHT_X, y = TOP_Y + PANEL_SHEER_AMOUNT},
  {x = LEFT_X, y = TOP_Y - PANEL_SHEER_AMOUNT}
}

local PANEL_COLOR = RiffleCommon.COLORS.BLACK

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local whiteBacking = FilledRectangle.new(Screen.width, Screen.height, RiffleCommon.COLORS.WHITE)
  whiteBacking:setLoc(self:getWidthPart(0.5), self:getHeightPart(-0.5))
  self:insert(whiteBacking)

  local fakePanel = SimpleMorphingPolygon.new(Screen.width, PANEL_HEIGHT, START_VERTICIES, END_VERTICIES)
  fakePanel:setColor(unpack(PANEL_COLOR))
  fakePanel:setLoc(self:getWidthPart(0.5), -self:getHeightPart(0.277))
  self:insert(fakePanel)

  local circles =
  {
    {
      object = FilledCircle.new(Screen.getWidthPart(0.2), RiffleCommon.COLORS.WHITE),
      appearDelay = 0.7
    },
    {
      object = HollowCircle.new(Screen.getWidthPart(0.3), RiffleCommon.COLORS.AQUA, 2),
      appearDelay = 0.8
    },
    {
      object = HollowCircle.new(Screen.getWidthPart(0.44), RiffleCommon.COLORS.BLACK, 1),
      appearDelay = 0.9
    },
    {
      object = HollowCircle.new(Screen.getWidthPart(0.46), RiffleCommon.COLORS.WHITE, 1),
      appearDelay = 1.0
    },
    {
      object = HollowCircle.new(Screen.getWidthPart(0.27), RiffleCommon.COLORS.WHITE, 1),
      appearDelay = 1.1
    },
    {
      object = HollowCircle.new(Screen.getWidthPart(0.4), RiffleCommon.COLORS.ORANGE, 3),
      appearDelay = 1.2
    },
    {
      object = HollowCircle.new(Screen.getWidthPart(0.2), RiffleCommon.COLORS.BLACK, 8),
      appearDelay = 1.3
    }
  }

  local finalCircleScale = math.max(6, Screen.height / Screen.getWidthPart(0.2))

  local circleAnimations = AHSpawn.new()
  for _, circle in ipairs(circles) do
    self:insertAtCenter(circle.object)
    circle.object:setScl(0.0)

    local circleAnimation = AHSequence.new(
      AHDelay.new(circle.appearDelay),
      AHScaleTo.new(circle.object, 2.0, AHEaseExponentialInOut.new(), 1.0),
      AHDelay.new(0.4),
      AHScaleTo.new(circle.object, 2.0, AHEaseSineInOut.new(), finalCircleScale)
    )

    circleAnimations:addAction(circleAnimation)
  end

  self.whiteBacking = whiteBacking
  self.fakePanel = fakePanel
  self.circleAnimations = circleAnimations
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiffleOutro:getAnimation()
  local backgroundAnimateDuration = 1.5
  local backgroundAnimation = AHSpawn.new(
    AHLocationYTo.new(self.fakePanel, backgroundAnimateDuration, AHEaseExponentialInOut.new(), self:getHeightPart(1.0)),
    AHLocationYTo.new(self.whiteBacking, backgroundAnimateDuration, AHEaseExponentialInOut.new(), self:getHeightPart(0.5)),
    self.fakePanel:createMorphToAction(backgroundAnimateDuration, AHEaseExponentialInOut.new(), 1.0),
    self.circleAnimations
  )

  local tick = Sprite.newWithName("goaltick.png")
  self:insertAtCenter(tick)
  tick:setScl(0.0)

  local tickAnimation = AHSequence.new(
    function() Sound.playSoundEffect("skimming_goalReach") end,
    AHDelay.new(1.5),
    AHScaleTo.new(tick, 1.0, AHEaseSimpleInOut.new(), 1.0),
    AHDelay.new(1.5),
    AHScaleTo.new(tick, 1.0, AHEaseSimpleInOut.new(), 0.0)
  )

  local combinedAnimation = AHSpawn.new(backgroundAnimation, tickAnimation)

  return combinedAnimation
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function RiffleOutro.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, RiffleOutro)

  createLayout(self)

  return self
end
