--- RiffleOvertimePenaltyPopup
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleOvertimePenaltyPopup")

require("RiffleCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HEADER_TEXT = "TIME'S UP"
local HEADER_LOCATION_Y_MULTIPLIER = 0.68
local HEADER_COLOR = RiffleCommon.COLORS.ORANGE
local OPTION_BUTTON_OFFSET_Y = Screen.getHeightPart(0.02)
local OPTION_BUTTON_DISABLED_COLOR = RiffleCommon.COLORS.DARK_GREY
local TARGET_OPTION_BUTTON_WIDTH = Screen.getWidthPart(0.875)
local TEXT_COLOR = RiffleCommon.COLORS.WHITE
local TEXT_COLOR_TAPPED = RiffleCommon.COLORS.BLACK
local DETAILS_COLOR = RiffleCommon.COLORS.GREY
local FRAME_RATE = 60

local BUTTON_DESCRIPTORS =
{
  CONTINUE = {
    TEXT = "Keep skimming",
    DETAIL = "use one life to get 15 more seconds"
  },
  STOP = {
    TEXT = "I'm done",
    DETAIL = "no penalty incurred"
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createOptionButton(self, buttonText, buttonDetails, onTap)
  local buttonSprite = Sprite.newWithName("timeUp_button_000.png")
  local targetScale = TARGET_OPTION_BUTTON_WIDTH / buttonSprite:getWidth()
  buttonSprite:setScl(targetScale)

  local optionButton = Group.new(buttonSprite:getEffectiveDims())
  optionButton:insertAtCenter(buttonSprite)

  local heightThird = optionButton:getEffectiveHeightPart(0.33)

  local textLabel = Label.new(buttonText, Font.new(FontFaces.DIN_OT_MEDIUM, 18))
  textLabel:setLoc(optionButton:getEffectiveWidthPart(0.5), 2 * heightThird)
  textLabel:setColor(unpack(TEXT_COLOR))
  optionButton:insert(textLabel)

  local detailsLabel = Label.new(buttonDetails, Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  detailsLabel:setLoc(optionButton:getEffectiveWidthPart(0.5), heightThird)
  detailsLabel:setColor(unpack(DETAILS_COLOR))
  optionButton:insert(detailsLabel)

  local pressedAnimation = Animation.newWithPrefix("timeUp_button_")
  optionButton.pressedAnimateAction = AHAnimate.new(buttonSprite, pressedAnimation, FRAME_RATE)

  optionButton.textLabel = textLabel
  optionButton.detailsLabel = detailsLabel

  local tapWrapper = function()
    textLabel:setColor(unpack(TEXT_COLOR_TAPPED))
    wait(optionButton.pressedAnimateAction:start())

    if onTap then
      onTap()
    end
  end

  Button.makeButton(optionButton, tapWrapper)

  return optionButton
end

local function createLayout(self, canUserContinue)
  local centerX = self:getWidthPart(0.5)

  local header = Label.new(HEADER_TEXT, Font.new(FontFaces.DIN_OT_MEDIUM, 36))
  header:setLoc(centerX, self:getHeightPart(HEADER_LOCATION_Y_MULTIPLIER))
  header:setColor(unpack(HEADER_COLOR))
  self:insert(header)

  local registerContinueTapped = function()
    if self.continueTappedCallback then
      self.continueTappedCallback()
    end
  end

  local continueButtonDescriptor = BUTTON_DESCRIPTORS.CONTINUE
  local continueButton = createOptionButton(self, continueButtonDescriptor.TEXT, continueButtonDescriptor.DETAIL, registerContinueTapped)
  local continueButtonY = header:getY() - header:getHeightPart(0.5) - continueButton:getHeightPart(0.5) - OPTION_BUTTON_OFFSET_Y
  continueButton:setLoc(centerX, continueButtonY)
  self:insert(continueButton)

  local registerStopTapped = function()
    if self.stopTappedCallback then
      self.stopTappedCallback()
    end
  end

  local stopButtonDescriptor = BUTTON_DESCRIPTORS.STOP
  local stopButton = createOptionButton(self, stopButtonDescriptor.TEXT, stopButtonDescriptor.DETAIL, registerStopTapped)
  local stopButtonY = continueButtonY - continueButton:getHeightPart(0.5) - stopButton:getHeightPart(0.5) - OPTION_BUTTON_OFFSET_Y
  stopButton:setLoc(centerX, stopButtonY)
  self:insert(stopButton)

  if not canUserContinue then
    continueButton:setTouchEnabled(false)
    continueButton:linkColorAttrToChildren()
    continueButton:setColor(unpack(OPTION_BUTTON_DISABLED_COLOR))
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiffleOvertimePenaltyPopup:prepareForAnimateIn()
  self:setLoc(Screen.getWidthPart(0.5), self:getHeightPart(-0.5))
end

function RiffleOvertimePenaltyPopup:animateIn()
  local animateAction = AHLocationYTo.new(self, 0.5, AHEaseSimpleInOut.new(), self:getHeightPart(0.5))

  return animateAction:start()
end

function RiffleOvertimePenaltyPopup:animateOutAndRemove()
  local animateAction = AHSequence.new(
    AHLocationYTo.new(self, 0.5, AHEaseSimpleInOut.new(), self:getHeightPart(-0.5)),
    AHRemove.new(self)
  )

  return animateAction:start()
end

function RiffleOvertimePenaltyPopup:setContinueTappedCallback(callback)
  self.continueTappedCallback = callback
end

function RiffleOvertimePenaltyPopup:setStopTappedCallback(callback)
  self.stopTappedCallback = callback
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function RiffleOvertimePenaltyPopup.new(uiBarHeight, canUserContinue)
  local height = Screen.height - uiBarHeight
  local self = FilledRectangle.new(Screen.width, height, RiffleCommon.COLORS.BLACK)
  Mixin.mix(self, RiffleOvertimePenaltyPopup)

  createLayout(self, canUserContinue)

  return self
end
