--- RifflePerCharacterAnimatingLabel
-- @author Created by Jay Austria on February 25, 2015.

class("RifflePerCharacterAnimatingLabel")

require("RiffleCommon")
require("text.ComposedGlyphLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------
function RifflePerCharacterAnimatingLabel:animateIn(dY)
  local baseY = self.children[1]:getY()

  local minRotation = -90
  local maxRotation = 90
  local maxDelay = 0.5

  local duration = 1
  local ease = AHEaseExponentialOut.new()

  local action = AHSpawn.new()
  for _, child in ipairs(self.children) do
    local delay = math.random() * maxDelay
    local rotation = math.random(minRotation, maxRotation)

    child:setY(baseY + dY)
    child:setRot(rotation)
    action:addAction(AHDelayed.new(delay, AHSpawn.new(
      AHLocationYTo.new(child, duration, ease, baseY),
      AHRotationTo.new(child, duration, ease, 0)
    )))
  end

  return action:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function RifflePerCharacterAnimatingLabel.new(text, fontFace, fontSize, color)
  local font = Font.new(fontFace, fontSize)
  local default = nil
  local willBuildGlyphDescriptors = true
  local label = Label.new(text, font, default, default, willBuildGlyphDescriptors)
  local self = ComposedGlyphLabel.new(label, label:getGlyphTable())
  Mixin.mix(self, RifflePerCharacterAnimatingLabel)

  self.text = text
  self:setColor(unpack(color))

  return self
end
