--- RiffleQuestionLayerInterstitialGroup
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleQuestionLayerInterstitialGroup")

require("RiffleQuestionLayerScanningQuestionGroup")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CONGRATULATORY_TEXT =
{
  PERFECT = {
    message = "GREAT",
    fontSize = 80
  },
  NEAR_PERFECT =
  {
    message = "GOOD",
    fontSize = 80
  },
  POOR =
  {
    message = "NICE TRY",
    fontSize = 64
  }
}

local CONGRATULATORY_FONT = FontFaces.DIN_OT_MEDIUM
local CONGRATULATORY_LABEL_COLOR = RiffleCommon.COLORS.ORANGE
local CONGRATULATORY_LABEL_ANIMATE_DY = Screen.getHeightPart(-0.6)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getCongratulatoryTextForNumberIncorrect(incorrectCount)
  if incorrectCount <= 0 then
    return CONGRATULATORY_TEXT.PERFECT
  elseif incorrectCount <= 1 then
    return CONGRATULATORY_TEXT.NEAR_PERFECT
  else
    return CONGRATULATORY_TEXT.POOR
  end
end

local function createLayout(self, width, height, registerButtonPress, question, questionIndex, numberIncorrect)
  local congratulatoryTextDescriptor = getCongratulatoryTextForNumberIncorrect(numberIncorrect)
  local message = congratulatoryTextDescriptor.message
  local fontSize = congratulatoryTextDescriptor.fontSize
  local congratulatoryLabel = RifflePerCharacterAnimatingLabel.new(message, CONGRATULATORY_FONT, fontSize, CONGRATULATORY_LABEL_COLOR)
  congratulatoryLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.8))
  self:insert(congratulatoryLabel)

  local upNextLabel = Label.new("UP NEXT", Font.new(FontFaces.DIN_OT_MEDIUM, 20))
  upNextLabel:setRot(-10)
  upNextLabel:setAnchorPointTopLeft()
  local upNextLabelX = (self:getWidth() - upNextLabel:getWidth()) * 0.5
  upNextLabel:setLoc(upNextLabelX, congratulatoryLabel:getY() - 60)
  upNextLabel:setColor(unpack(RiffleCommon.COLORS.WHITE))
  upNextLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self:insert(upNextLabel)

  local questionGroup = RiffleQuestionLayerScanningQuestionGroup.new(width, height, registerButtonPress, question, questionIndex)
  local headerLabel = questionGroup:getHeaderLabel()
  questionGroup:hide()

  headerLabel:setRot(10)
  headerLabel:setAnchorPointTopRight()
  local headerLabelX = width / 2 + headerLabel:getEffectiveWidthPart(0.5)
  headerLabel:setLoc(headerLabelX, upNextLabel:getY() - 20)
  headerLabel:clearAttrLink(MOAIColor.ATTR_OPACITY)

  questionGroup:setLoc(self:getWidthPart(0.5), self:getHeight())
  questionGroup:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self:insert(questionGroup)

  self.congratulatoryLabel = congratulatoryLabel
  self.upNextLabel = upNextLabel
  self.headerLabel = headerLabel
  self.questionGroup = questionGroup

  self:setAnchorPointTop()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiffleQuestionLayerInterstitialGroup:animateIn()
  local congratulatoryLabel = self.congratulatoryLabel
  local upNextLabel = self.upNextLabel
  local headerLabel = self.headerLabel
  local questionGroup = self.questionGroup

  local animateCongratulatoryLabel = function() congratulatoryLabel:animateIn(CONGRATULATORY_LABEL_ANIMATE_DY) end
  local presentDuration = 1.0
  local presentAnimation =
    AHDelayed.new(0.7,
      AHSpawn.new(
        AHRotationTo.new(upNextLabel, presentDuration, AHEaseSimpleOut.new(), 0),
        AHOpacityTo.new(upNextLabel, presentDuration, nil, RiffleCommon.VISIBLE_OPACITY),
        AHRotationTo.new(headerLabel, presentDuration, AHEaseSimpleOut.new(), 0),
        AHOpacityTo.new(headerLabel, presentDuration, nil, RiffleCommon.VISIBLE_OPACITY)
      )
    )

  local showInterstitial = AHSpawn.new(animateCongratulatoryLabel, presentAnimation)

  local transitionDuration = 0.7
  local fadeDuration = 0.5
  local transitionToQuestion = AHSpawn.new(
    AHOpacityTo.new(congratulatoryLabel, fadeDuration, nil, RiffleCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(upNextLabel, fadeDuration, AHEaseSimpleOut.new(), RiffleCommon.INVISIBLE_OPACITY),
    AHLocationYTo.new(upNextLabel, transitionDuration, AHEaseSimpleInOut.new(), self:getHeightPart(1.1)),
    AHLocationYTo.new(self, transitionDuration, AHEaseSimpleInOut.new(), self:getHeight()),
    function() questionGroup:animateIn() end
  )

  local removeGroups = function()
    congratulatoryLabel:removeSelf()
    upNextLabel:removeSelf()
  end

  local combinedAnimation = AHSequence.new(showInterstitial, transitionToQuestion, removeGroups)

  return combinedAnimation:start()
end

function RiffleQuestionLayerInterstitialGroup:collapseForScanningRound()
  self.questionGroup:collapseForScanningRound()
end

function RiffleQuestionLayerInterstitialGroup:getFadeOutAction()
  return self.questionGroup:getFadeOutAction()
end

function RiffleQuestionLayerInterstitialGroup:registerBeginScanningPressed()
  return self.questionGroup:registerBeginScanningPressed()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function RiffleQuestionLayerInterstitialGroup.new(width, height, registerButtonPress, question, questionIndex, numberIncorrect)
  local self = Group.new(width, height)
  Mixin.mix(self, RiffleQuestionLayerInterstitialGroup)

  createLayout(self, width, height, registerButtonPress, question, questionIndex, numberIncorrect)

  return self
end
