--- RiffleQuestionLayerScanningQuestionGroup
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleQuestionLayerScanningQuestionGroup")

require("RiffleCommon")
require("RiffleConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BEGIN_BUTTON_TEXT = "Begin"
local HEADER_TEXT = "SCAN THE ARTICLE"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function scaleLabelIfNeeded(label, maxWidth)
  if (label:getWidth() > maxWidth) then
    label:setEffectiveDims(maxWidth)
  end
end

local function getFadeAction(self, opacity)
  local fadeDuration = 0.5
  local fadeAction = AHSpawn.new(
    AHOpacityTo.new(self.headerLabel, fadeDuration, nil, opacity),
    AHOpacityTo.new(self.countLabel, fadeDuration, nil, opacity),
    AHOpacityTo.new(self.questionLabel, fadeDuration, nil, opacity),
    AHOpacityTo.new(self.beginButton, fadeDuration, nil, opacity),
    AHOpacityTo.new(self.tipLabel, fadeDuration, nil, opacity),
    AHOpacityTo.new(self.scanInstructions, fadeDuration, nil, opacity)
  )

  return fadeAction
end

local function createLayout(self, width, height, buttonPressCallback, question, questionIndex)
  local centerX = self:getWidthPart(0.5)
  local headerLabel = Label.new(HEADER_TEXT, Font.new(FontFaces.DIN_OT_BOLD, 48))
  scaleLabelIfNeeded(headerLabel, width)
  headerLabel:setLoc(centerX, self:getHeight() - headerLabel:getEffectiveHeightPart(0.4))
  headerLabel:setColor(unpack(RiffleCommon.COLORS.AQUA))
  headerLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  headerLabel:setAnchorPointTop()
  self:insert(headerLabel)

  local countText = RiffleCommon.getCountText(questionIndex, RiffleConfiguration.getScanningQuestionCount())
  local countLabel = Label.new(countText, Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  countLabel:setAnchorPointLeft()
  countLabel:setLoc(0, headerLabel:getYBottom() - countLabel:getHeight())
  self:insert(countLabel)

  local baseAttributes = {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, 20)
  }

  local attributesForLabels = {
    b = { font = Font.new(FontFaces.DIN_OT_BOLD, 20) },
    i = { font = Font.new(FontFaces.DIN_OT_MEDIUM_ITALIC, 20) }
  }

  local questionLabel = DynamicTextArea.new(question.prompt, width, baseAttributes, attributesForLabels)
  questionLabel:setAnchorPointTop()
  self:insert(questionLabel)

  local beginButton = RiffleButton.new(RiffleButton.BUTTON_TYPE.BLUE, buttonPressCallback)
  beginButton:setLoc(centerX, self:getHeightPart(0.28))
  beginButton:setText(BEGIN_BUTTON_TEXT)
  beginButton:setTouchEnabled(false)
  self:insert(beginButton)

  local topY = countLabel:getY() - countLabel:getHeightPart(0.5)
  local bottomY = beginButton:getY() + beginButton:getHeightPart(0.5)
  local questionY = (topY - bottomY) * 0.5 + bottomY + questionLabel:getHeightPart(0.5)
  questionLabel:setLoc(centerX, questionY)

  local isBeginner = RiffleConfiguration.isBeginner()
  local tipLabel =  RiffleTipLabel.new(isBeginner, RiffleTipLabel.TYPE.SCAN)
  local tipLabelY = (beginButton:getY() - beginButton:getHeightPart(0.5)) * 0.5
  tipLabel:setLoc(centerX, tipLabelY)
  self:insert(tipLabel)

  local scanInstructions = Label.new("TAP THE TEXT WITH THE ANSWER", Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  scanInstructions:setColor(unpack(RiffleCommon.COLORS.AQUA))
  scanInstructions:setVisible(false)
  scanInstructions:setLoc(centerX, self:getHeightPart(0.95))
  self:insert(scanInstructions)

  self:setAnchorPointTop()

  self.headerLabel = headerLabel
  self.countLabel = countLabel
  self.questionLabel = questionLabel
  self.beginButton = beginButton
  self.tipLabel = tipLabel
  self.scanInstructions = scanInstructions
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiffleQuestionLayerScanningQuestionGroup:getHeaderLabel()
  return self.headerLabel
end

function RiffleQuestionLayerScanningQuestionGroup:registerBeginScanningPressed()
  self.beginButton:doPressedAnimation()
  self.beginButton:setTouchEnabled(false)
end

function RiffleQuestionLayerScanningQuestionGroup:hide()
  self.headerLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self.countLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self.questionLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self.beginButton:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self.tipLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self.scanInstructions:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
end

function RiffleQuestionLayerScanningQuestionGroup:getFadeOutAction()
    local fadeOutAction = getFadeAction(self, RiffleCommon.INVISIBLE_OPACITY)

    return fadeOutAction
end

function RiffleQuestionLayerScanningQuestionGroup:collapseForScanningRound()
  self.scanInstructions:setVisible(true)
  self.scanInstructions:setOpacity(RiffleCommon.INVISIBLE_OPACITY)

  local fadeDuration = 0.5
  local collapseDuration = 1.0
  local collapseAnimation = AHSpawn.new(
    AHLocationYTo.new(self.questionLabel, collapseDuration, AHEaseExponentialInOut.new(), self:getHeightPart(0.9)),
    AHOpacityTo.new(self.headerLabel, fadeDuration, nil, RiffleCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(self.countLabel, fadeDuration, nil, RiffleCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(self.beginButton, fadeDuration, nil, RiffleCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(self.tipLabel, fadeDuration, nil, RiffleCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(self.scanInstructions, fadeDuration, nil, RiffleCommon.VISIBLE_OPACITY)
  )

  local removeGroups = AHSpawn.new(
    AHRemove.new(self.headerLabel),
    AHRemove.new(self.countLabel),
    AHRemove.new(self.beginButton),
    AHRemove.new(self.tipLabel)
  )

  local collapseSequence = AHSequence.new(collapseAnimation, removeGroups)
  collapseSequence:start()
end

function RiffleQuestionLayerScanningQuestionGroup:animateIn()
  local animateDuration = 0.6
  local animateAction = AHSpawn.new(
    AHLocationYTo.new(self, animateDuration, AHEaseSimpleInOut.new(), self:getHeight()),
    AHLocationYTo.new(self.headerLabel, animateDuration, AHEaseSimpleInOut.new(), self:getHeight() - self.headerLabel:getHeightPart(0.4)),
    AHOpacityTo.new(self.headerLabel, animateDuration, nil, RiffleCommon.VISIBLE_OPACITY),
    AHOpacityTo.new(self.countLabel, animateDuration, nil, RiffleCommon.VISIBLE_OPACITY),
    AHOpacityTo.new(self.beginButton, animateDuration, nil, RiffleCommon.VISIBLE_OPACITY),
    AHOpacityTo.new(self.tipLabel, animateDuration, nil, RiffleCommon.VISIBLE_OPACITY),
    AHOpacityTo.new(self.scanInstructions, animateDuration, nil, RiffleCommon.VISIBLE_OPACITY),
    AHOpacityTo.new(self.questionLabel, animateDuration, nil, RiffleCommon.VISIBLE_OPACITY)
  )

  local animateSequence = AHSequence.new(
    animateAction,
    function() self.beginButton:setTouchEnabled(true) end
  )

  return animateSequence:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function RiffleQuestionLayerScanningQuestionGroup.new(width, height, buttonPressCallback, question, questionIndex)
  local self = Group.new(width, height)
  Mixin.mix(self, RiffleQuestionLayerScanningQuestionGroup)

  createLayout(self, width, height, buttonPressCallback, question, questionIndex)

  return self, self.headerLabel
end
