--- RiffleQuestionLayerSkimmingQuestionGroup
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleQuestionLayerSkimmingQuestionGroup")

require("RiffleCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HEADER_TEXT = "SKIMMING REVIEW"
local HEADER_COLOR = RiffleCommon.COLORS.ORANGE
local QUESTION_COLOR = RiffleCommon.COLORS.WHITE
local QUESTION_LOCATION_Y = Screen.getHeightPart(0.53)
local OPTION_BUTTON_SPACING_Y = Screen.getHeightPart(0.015)
local QUESTION_BUTTONS_SPACING_Y = Screen.getHeightPart(0.04)
local OPTION_FONT_SIZE = 15
local COUNT_LABEL_OFFSET_Y = -10

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function scaleLabelIfNeeded(label, maxWidth)
  if (label:getWidth() > maxWidth) then
    label:setEffectiveDims(maxWidth)
  end
end

local function getFadeAction(self, opacity)
  local fadeDuration = 0.5
  local fadeAction = AHSpawn.new(
    AHOpacityTo.new(self.headerLabel, fadeDuration, nil, opacity),
    AHOpacityTo.new(self.countLabel, fadeDuration, nil, opacity),
    AHOpacityTo.new(self.questionLabel, fadeDuration, nil, opacity)
  )

  for _, button in ipairs(self.buttons) do
    local buttonFade = AHOpacityTo.new(button, fadeDuration, nil, opacity)
    fadeAction:addAction(buttonFade)
  end

  return fadeAction
end

local function createLayout(self, question, options, optionTapCallback, questionIndex)
  local centerX = self:getWidthPart(0.5)
  local width = self:getWidth()
  local height = self:getHeight()

  local headerLabel = Label.new(HEADER_TEXT, Font.new(FontFaces.DIN_OT_BOLD, 48))
  scaleLabelIfNeeded(headerLabel, width)
  headerLabel:setColor(unpack(HEADER_COLOR))
  headerLabel:setLoc(centerX, self:getEffectiveHeightPart(0.9))
  self:insert(headerLabel)
  self.headerLabel = headerLabel

  local totalQuestions = RiffleConfiguration.getSkimmingQuestionCount()
  local countText = RiffleCommon.getCountText(questionIndex, totalQuestions)
  local countLabel = Label.new(countText, Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  countLabel:setAnchorPointLeft()
  countLabel:setLoc(0, headerLabel:getY() - headerLabel:getEffectiveHeightPart(0.5) + COUNT_LABEL_OFFSET_Y)
  self:insert(countLabel)
  self.countLabel = countLabel

  local baseAttributes = {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, 20)
  }

  local attributesForLabels = {
    b = { font = Font.new(FontFaces.DIN_OT_BOLD, 20) },
    i = { font = Font.new(FontFaces.DIN_OT_MEDIUM_ITALIC, 20) }
  }

  local questionLabel = DynamicTextArea.new(question, width, baseAttributes, attributesForLabels)
  questionLabel:setAnchorPointTop()
  questionLabel:setLoc(centerX, QUESTION_LOCATION_Y)
  questionLabel:setColor(unpack(QUESTION_COLOR))
  self:insert(questionLabel)
  self.questionLabel = questionLabel

  local buttonsColumn = Column.new()
  self:insert(buttonsColumn)

  local buttons = {}
  for optionIndex, option in ipairs(options) do
    local onOptionTapWrapper = function()
      optionTapCallback(option)
    end

    local optionButton = RiffleButton.new(RiffleButton.BUTTON_TYPE.ORANGE, onOptionTapWrapper, OPTION_FONT_SIZE)
    optionButton:setText(option)

    buttonsColumn:insert(optionButton)

    local islastOption = (optionIndex == #options)
    if not islastOption then
      buttonsColumn:insertSpacer(OPTION_BUTTON_SPACING_Y)
    end

    table.insert(buttons, optionButton)
  end

  buttonsColumn:alignChildren()

  local buttonsColumnY = questionLabel:getY() - questionLabel:getHeight() - QUESTION_BUTTONS_SPACING_Y
  buttonsColumn:setAnchorPointTop()
  buttonsColumn:setLoc(centerX, buttonsColumnY)

  self.buttons = buttons
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RiffleQuestionLayerSkimmingQuestionGroup:getFadeInAction()
  local fadeInAction = getFadeAction(self, RiffleCommon.VISIBLE_OPACITY)

  return fadeInAction
end

function RiffleQuestionLayerSkimmingQuestionGroup:getFadeOutAction()
  local fadeOutAction = getFadeAction(self, RiffleCommon.INVISIBLE_OPACITY)

  return fadeOutAction
end

function RiffleQuestionLayerSkimmingQuestionGroup:hide()
  self.headerLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self.countLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  self.questionLabel:setOpacity(RiffleCommon.INVISIBLE_OPACITY)

  for _, button in ipairs(self.buttons) do
    button:setOpacity(RiffleCommon.INVISIBLE_OPACITY)
  end
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function RiffleQuestionLayerSkimmingQuestionGroup.new(width, height, question, options, optionTapCallback, questionIndex)
  local self = Group.new(width, height)
  Mixin.mix(self, RiffleQuestionLayerSkimmingQuestionGroup)

  createLayout(self, question, options, optionTapCallback, questionIndex)

  return self, self.buttons
end
