--- RiffleTimer
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleTimer")

require("widgets.progress.TimeTimer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------
local TIMER_FONT = FontFaces.DIN_OT_MEDIUM
local TIMER_FONT_SIZE = 16
local DEFAULT_LABEL_TEXT = "MODE UNSET"
local LABEL_PADDING_X = 8

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createLayout(duration, onTimeout)
  local timerLabel = TimeTimer.new(TIMER_FONT, TIMER_FONT_SIZE, duration, onTimeout)
  timerLabel:addStandardWarningSequence()
  local modeLabel = Label.new(DEFAULT_LABEL_TEXT, Font.newWithActualSize(TIMER_FONT, TIMER_FONT_SIZE))

  local groupWidth = timerLabel:getWidth() + LABEL_PADDING_X + modeLabel:getWidth()
  local groupHeight = timerLabel:getHeight()

  local self = Group.new(groupWidth, groupHeight)

  timerLabel:setAnchorPointRight()
  timerLabel:setLoc(groupWidth, self:getHeightPart(0.5))
  self:insert(timerLabel)

  local modeLabelX = groupWidth - timerLabel:getWidth() - LABEL_PADDING_X
  modeLabel:setAnchorPointRight()
  modeLabel:setLoc(modeLabelX, self:getHeightPart(0.5))
  self:insert(modeLabel)

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)
  self:linkColorAttrToChildren(isRecursive)

  self.timerLabel = timerLabel
  self.modeLabel = modeLabel

  return self
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------
function RiffleTimer:setLabelText(text)
  self.modeLabel:setText(text)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function RiffleTimer.new(duration, onTimeout)
  local self = createLayout(duration, onTimeout)
  Mixin.mix(self, RiffleTimer)

  FunctionUtils.forwardMethodCalls(self, self.timerLabel, "start", "pause", "stop", "setTimeoutHandler", "setDuration", "addWarningHandler", "getTimeRemaining", "getDuration")

  return self
end
