--- RiffleUIBar
-- @author Created by Jay Austria on February 23, 2015.

class("RiffleUIBar")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TIMER_OFFSET_Y = -20
local OVERLAY_HEIGHT = RiffleCommon.UI_BAR_HEIGHT + 10

local EAST_WEST_OFFSET = 10
local SLIDE_IN_DURATION = 0.8

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function arrangeElements(self, pauseButton, scoreLabel, bonusTray, livesCounter, timer)
  local widgetY = self:getHeight() - (Screen.height - UIOverlay.NORTHWEST.y)

  pauseButton:setAnchorPointLeft()
  pauseButton:setLoc(UIOverlay.NORTHWEST.x - EAST_WEST_OFFSET, widgetY)
  pauseButton:setColor(unpack(self.uiColor))
  self:insert(pauseButton)

  scoreLabel:setAnchorPointLeft()
  scoreLabel:setLoc(UIOverlay.NORTHWEST.x + StandardUIBuilder.STANDARD_SCORE_MARGIN_X, widgetY)
  scoreLabel:setColor(unpack(self.uiColor))
  self:insert(scoreLabel)

  bonusTray:setAnchorPointTopLeft()
  bonusTray:setLoc(UIOverlay.NORTHWEST.x - EAST_WEST_OFFSET, widgetY - pauseButton:getHeight())
  bonusTray:setColor(unpack(self.uiColor))
  self:insert(bonusTray)

  livesCounter:setAnchorPointRight()
  livesCounter:setLoc(UIOverlay.NORTHEAST.x + EAST_WEST_OFFSET * 0.6, widgetY)
  livesCounter:setColor(unpack(self.uiColor))
  self:insert(livesCounter)

  timer:setAnchorPointRight()
  timer:setLoc(UIOverlay.NORTHEAST.x + EAST_WEST_OFFSET, widgetY + TIMER_OFFSET_Y)
  timer:setColor(unpack(self.uiColor))
  self:insert(timer)
end

local function createLayoutImageAndSubheader()
  local barColor = RiffleCommon.COLORS.BLACK
  local uiColor = RiffleCommon.COLORS.WHITE
  local self = FilledRectangle.new(Screen.width, RiffleCommon.UI_BAR_HEIGHT, barColor)

  self.uiColor = uiColor
  self.warningFlashGroup = self
  self.warningColor = RiffleCommon.COLORS.WHITE
  self.originalColor = barColor

  return self
end

local function createLayoutNoImage()
  local barColor = RiffleCommon.COLORS.GREY
  local barOverlayColor = RiffleCommon.COLORS.WHITE
  local uiColor = RiffleCommon.COLORS.GREY
  local self = FilledRectangle.new(Screen.width, RiffleCommon.UI_BAR_HEIGHT, barColor)

  local barOverlay = FilledRectangle.new(Screen.width, OVERLAY_HEIGHT, barOverlayColor)
  local overlayOffset = RiffleCommon.UI_BAR_HEIGHT - OVERLAY_HEIGHT
  barOverlay:setAnchorPointBottom()
  barOverlay:setLoc(self:getWidthPart(0.5), overlayOffset)
  self:insert(barOverlay)

  self.uiColor = uiColor
  self.warningFlashGroup = barOverlay
  self.warningColor = RiffleCommon.COLORS.BLACK
  self.originalColor = barOverlayColor

  return self
end

local function createLayoutNoSubheader()
  local barColor = RiffleCommon.COLORS.TRANSLUCENT_BLACK
  local uiColor = RiffleCommon.COLORS.WHITE
  local self = FilledRectangle.new(Screen.width, RiffleCommon.UI_BAR_HEIGHT, barColor)

  self.uiColor = uiColor
  self.warningFlashGroup = self
  self.warningColor = RiffleCommon.COLORS.BLACK
  self.originalColor = barColor

  return self
end

local function createLayoutNoOrnaments()
  return createLayoutNoImage()
end

local function chooseLayoutConstructor(layoutStyle)
  if (layoutStyle == RiffleCommon.LayoutStyle.IMAGE_AND_SUBHEADER) then
    return createLayoutImageAndSubheader
  elseif (layoutStyle == RiffleCommon.LayoutStyle.SUBHEADER_ONLY) then
    return createLayoutNoImage
  elseif (layoutStyle == RiffleCommon.LayoutStyle.IMAGE_ONLY) then
    return createLayoutNoSubheader
  else
    return createLayoutNoOrnaments
  end
end

local function runWarningBlink(self)
  local warningFlashGroup = self.warningFlashGroup
  local originalColor = { warningFlashGroup:getColor() }

  local warningBlink = AHSequence.new(
    AHColorTo.new(warningFlashGroup, 0.1, nil, self.warningColor),
    AHColorTo.new(warningFlashGroup, 0.8, nil, originalColor)
  )

  warningBlink:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------
function RiffleUIBar:hide()
  local x, y = self:getLoc()
  self.originalY = y

  self:setLoc(x, Screen.height + self:getHeight())
end

function RiffleUIBar:animateIn()
  local ease = AHEaseExponentialOut.new()
  local slideIn = AHLocationYTo.new(self, SLIDE_IN_DURATION, ease, self.originalY)

  slideIn:start()
end

function RiffleUIBar:setDisplayForTimeOut(doDisplay)
  local warningFlashGroup = self.warningFlashGroup
  local destinationColor = nil
  if doDisplay then
    destinationColor = RiffleCommon.COLORS.BLACK
  else
    destinationColor = self.originalColor
  end

  local colorAction = AHColorTo.new(warningFlashGroup, 0.5, nil, destinationColor)
  colorAction:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function RiffleUIBar.new(pauseButton, scoreLabel, bonusTray, livesCounter, timer, layoutStyle)
  local selectedLayoutConstructor = chooseLayoutConstructor(layoutStyle)
  local self = selectedLayoutConstructor()
  Mixin.mix(self, RiffleUIBar)

  arrangeElements(self, pauseButton, scoreLabel, bonusTray, livesCounter, timer)

  local warningEffectWrapper = function()
    runWarningBlink(self)
  end
  timer:addWarningHandler(warningEffectWrapper)

  return self
end
