--- Rocket Background
-- @author Created by Karl Stenerud on September 3, 2013.

class("Background")
require("widgets.Scroller")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Colors -----------------------------

local COLOR_BG = 0x23bddf

-- Z Ordering -------------------------

local Z_BACKGROUND = 0
local Z_MOUNTAINS = 10
local Z_CLOUDS = 20

-- Other ------------------------------

local BOOST_ON_DURATION = 0.2
local BOOST_OFF_DURATION = 0.5

local STREAK_OPACITY = 1

-- Number of background pieces to load.
local BG_PIECE_COUNT = 2
local STARFIELD_PIECE_COUNT = 2

local function startScrollAction(self, scrollAction)
  if self.scrollAction then
    self.scrollAction:stop()
    self.scrollAction = nil
  end
  self.scrollAction = scrollAction
  return self.scrollAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function Background:rampScrollSpeed(rampupDuration, scrollSpeed)
  self.scrollSpeed = -scrollSpeed

  return startScrollAction(self, Scroller.scrollSpeedTo(self.scroller, rampupDuration, AHEaseSineInOut.new(), self.scrollSpeed))
end

function Background:boostScrollSpeed(boostDuration, boostSpeed)
  boostSpeed = -boostSpeed
  return startScrollAction(self, AHSequence.new(
                                                AHSpawn.new(
                                                            Scroller.scrollSpeedTo(self.scroller, BOOST_ON_DURATION, AHEaseSineOut.new(), boostSpeed),
                                                            AHOpacityTo.new(self.streakScroller, BOOST_ON_DURATION, nil, STREAK_OPACITY)
                                                            ),
                                                AHDelay.new(boostDuration),
                                                AHSpawn.new(
                                                            Scroller.scrollSpeedTo(self.scroller, BOOST_OFF_DURATION, AHEaseSineOut.new(), self.scrollSpeed),
                                                            AHOpacityTo.new(self.streakScroller, BOOST_OFF_DURATION, nil, 0)
                                                            )
                                                )
                           )
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function getBGSpriteFilename(index)
  return string.format("rocket_bg_%03d.png", index)
end

local function makeBGScroller(self)
  local scrollerAssets = {}
  for i=1, BG_PIECE_COUNT do
    local sprite = Sprite.newWithName(getBGSpriteFilename(i))
    sprite:setEffectiveDims(self:getDims())
    scrollerAssets[i] = sprite
  end

  return FixedScroller.new(scrollerAssets)
end

local function makeStarScroller(self, filename)
  local scrollerAssets = {}
  for i=1, STARFIELD_PIECE_COUNT do
    local sprite = Sprite.newWithName(filename)
    sprite:setEffectiveDims(self:getDims())
    scrollerAssets[i] = sprite
  end

  return FixedScroller.new(scrollerAssets)
end

local function loadSprite(name, scale)
  scale = scale or 1
  local sprite = Sprite.newWithName(name)
  sprite:setScl(scale)
  return sprite
end

local function makeItemScroller1()
  local wrapY = Screen.fullHeight * 8

  local assets = {}

  assets[loadSprite("rocket_layer1_planet_001.png", 1)] = {x=math.floor(Screen.fullWidth * 0.35), y=math.floor(wrapY * 0.95)}
  assets[loadSprite("rocket_layer1_streak_001.png", 4)] = {x=math.floor(Screen.fullWidth * 0.50), y=math.floor(wrapY * 0.15)}
  assets[loadSprite("rocket_layer1_planet_002.png", 1)] = {x=math.floor(Screen.fullWidth * 0.75), y=math.floor(wrapY * 0.40)}
  assets[loadSprite("rocket_layer1_planet_003.png", 1)] = {x=math.floor(Screen.fullWidth * 0.20), y=math.floor(wrapY * 0.45)}
  assets[loadSprite("rocket_layer1_streak_002.png", 4)] = {x=math.floor(Screen.fullWidth * 0.50), y=math.floor(wrapY * 0.60)}

  return ItemScroller.new(assets, wrapY, Screen.fullWidth, Screen.fullHeight*2)
end

local function makeItemScroller2()
  local wrapY = Screen.fullHeight * 8

  local assets = {}
  assets[loadSprite("rocket_layer2_streak_001.png", 4)] = {x=math.floor(Screen.fullWidth * 0.50), y=math.floor(wrapY * 0.20)}
  assets[loadSprite("rocket_layer2_planet_001.png", 1)] = {x=math.floor(Screen.fullWidth * 0.10), y=math.floor(wrapY * 0.30)}
  assets[loadSprite("rocket_layer2_streak_002.png", 4)] = {x=math.floor(Screen.fullWidth * 0.50), y=math.floor(wrapY * 0.40)}
  assets[loadSprite("rocket_layer2_planet_002.png", 1)] = {x=math.floor(Screen.fullWidth * 0.90), y=math.floor(wrapY * 0.60)}
  assets[loadSprite("rocket_layer2_streak_003.png", 4)] = {x=math.floor(Screen.fullWidth * 0.50), y=math.floor(wrapY * 0.80)}

  return ItemScroller.new(assets, wrapY, Screen.fullWidth, Screen.fullHeight*2)
end

-- Constructor ------------------------

function Background.new()
  local self = Group.new(Screen.fullWidth, Screen.fullHeight)
  Mixin.mix(self, Background)

  self.scrollSpeed = 0

  self.streakScroller = makeStarScroller(self, "rocket_bg_starstreaks.png")
  self.streakScroller:linkAttrToChildren(MOAIColor.INHERIT_COLOR, MOAIColor.COLOR_TRAIT)
  self.streakScroller:setOpacity(0)

  local scrollers = {
    makeBGScroller(self),
    self.streakScroller,
    makeStarScroller(self, "rocket_bg_stars.png"),
    makeItemScroller1(),
    makeItemScroller2()
  }

  local scrollerProportions = {
    0.50,
    0.65,
    0.65,
    1.20,
    1.40
  }

  self.scroller = ParallaxScroller.new(scrollers, scrollerProportions)
  self:insertAtCenter(self.scroller)

  return self
end
