--- Rocket Launch Platform
-- @author Created by Karl Stenerud on Oct 14, 2013.

class("LaunchPlatform")
require("RocketCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUILDING_Y_OFFSET = 36

local BUILDING_X_OFFSET = -32

local ANIM_FRAMERATE = 24.0

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function LaunchPlatform:getRocketLandingPosition(rocket, parent)
  local rocketW, rocketH = rocket:getDims()
  local rocketX, rocketY = unpack(self.rocketLandingPosition)
  local worldX, worldY = self:modelToWorld(rocketX, rocketY + rocketH)
  return parent:worldToModel(worldX, worldY)
end

function LaunchPlatform:startSmoke()
  self.smokeSprite:setVisible(true)
  getAnimateAndVanishAction(self.smokeSprite, self.smokeAnim, ANIM_FRAMERATE):start()
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function LaunchPlatform.new()
  local groundSprite = Sprite.newWithName("rocket_launch_platform1.png")
  groundSprite:setEffectiveDims(Screen.width)
  local launchPlatformScale = groundSprite:getScl()
  local buildingSprite = Sprite.newWithName("rocket_launch_building1.png")
  buildingSprite:setScl(launchPlatformScale)

  -- Additional part of the sprite that can be moved on screen to help with the safe area
  local bufferGroundHeight = groundSprite:getEffectiveHeightPart(0.37)
  local safeAreaAdjust = math.min(bufferGroundHeight, Screen.safeBottom)

  local buildingLocationY = (BUILDING_Y_OFFSET * launchPlatformScale) + safeAreaAdjust

  local self = Group.new(Screen.fullWidth, buildingLocationY + buildingSprite:getHeight() + safeAreaAdjust)
  Mixin.mix(self, LaunchPlatform)

  groundSprite:setAnchorPointBottom()
  groundSprite:setLoc(Screen.fullWidth/2, safeAreaAdjust - bufferGroundHeight)
  self:insert(groundSprite)

  buildingSprite:setAnchorPointBottom()
  buildingSprite:setLoc(Screen.fullWidth/2 + BUILDING_X_OFFSET, buildingLocationY)
  self:insert(buildingSprite)

  self.rocketLandingPosition = {Screen.fullWidth/2, safeAreaAdjust}

  self.smokeAnim = Animation.newWithPrefix("rocket_smoke/rocket_smoke_")
  self.smokeSprite = Sprite.newWithName("rocket_smoke/rocket_smoke_000.png")
  self.smokeSprite:setScl(2 * launchPlatformScale)
  self.smokeSprite:setAnchorPointCenter()
  self.smokeSprite:setLoc(Screen.fullWidth/2, buildingLocationY)
  self.smokeSprite:setVisible(false)
  self:insert(self.smokeSprite)

  return self
end
