--- Rocket
-- @author Created by Karl Stenerud on Oct 9, 2013.

class("Rocket")
require("Slipstream")
require("RocketEngine")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Z Ordering -------------------------

local Z_THRUST = 10
local Z_SLIPSTREAM = 20
local Z_ROCKET = 200

-- Other ------------------------------

local VARIANCE_X = 4
local VARIANCE_ANGLE = 2
local ROCK_DURATION = 0.2

local ANIM_FRAMERATE = 24.0


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function Rocket:runBoost(duration)
  return runInCoroutine(function()
    self.slipstream:startBoost()
    self.engine:runBoost(duration)
    waitFor(duration)
    self.slipstream:endBoost()
  end)
end

function Rocket:startNormalThrust()
  self.engine:startNormalThrust()
end

function Rocket:startIgnition()
  self.engine:startIgnition()
end

function Rocket:reset()
  self.slipstream:reset()
  self.engine:stop()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function Rocket.new()
  local rocketSprite = Sprite.newWithName("rocket_rocket.png")
  local rocketW, rocketH = rocketSprite:getDims()

  local self = Group.new(rocketW, rocketH)
  Mixin.mix(self, Rocket)

  self.rocketSprite = rocketSprite
  self:insertAtCenter(self.rocketSprite, Z_ROCKET)

  self.slipstream = Slipstream.new()
  self.slipstream:setAnchorPointTop()
  self.slipstream:setLoc(rocketW*0.48, rocketH * 1.02)
  self:insert(self.slipstream, Z_SLIPSTREAM)
  self.slipstream:reset()

  self.engine = RocketEngine.new()
  self.engine:setAnchorPointTop()
  self.engine:setLoc(rocketW/2, 3)
  self:insert(self.engine)
  self.engine:stop()

  self:setAnchorPointCenter()

  return self
end
