--- Rocket Button
-- @author Created by Karl Stenerud on Oct 9, 2013.

class("RocketButton")
require("RocketCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_BUTTON_DEFAULT = {0.44, 0.69, 0.65}
local COLOR_BUTTON_CORRECT = {0.25, 0.87, 0.46}
local COLOR_BUTTON_INCORRECT = {0.82, 0.32, 0.29}
local COLOR_BUTTON_TEXT = {1.0, 1.0, 1.0}

local FONT = Font.new(RocketCommon.FONT_NORMAL, 24)

local NORMAL_OPACITY = 0.2
local HIGHLIGHT_OPACITY = 0.8

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function onTouchDown(self)
  self.onTouched(self)
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RocketButton:setVisible(isVisible)
  self.isVisible = isVisible
  if self.label then
    self.label:setVisible(isVisible)
  end
  self.outerRect:setVisible(isVisible)
  self.innerRect:setVisible(isVisible)
end

function RocketButton:setText(text)
  return self.label:setText(text)
end

function RocketButton:unmark()
  self.outerRect:setColor(unpack(COLOR_BUTTON_DEFAULT))
  self.outerRect:setOpacity(NORMAL_OPACITY)
  self.innerRect:setColor(unpack(COLOR_BUTTON_DEFAULT))
  self.innerRect:setOpacity(NORMAL_OPACITY)
  self:setScl(1.0)
end

function RocketButton:markCorrect()
  self.outerRect:setColor(unpack(COLOR_BUTTON_CORRECT))
  self.outerRect:setOpacity(HIGHLIGHT_OPACITY)
  self.innerRect:setColor(unpack(COLOR_BUTTON_CORRECT))
  self.innerRect:setOpacity(HIGHLIGHT_OPACITY)
  AHSequence.new(
                 AHScaleTo.new(self, 0.05, AHEaseSineOut.new(), 0.97),
                 AHScaleTo.new(self, 0.1, AHEaseSineIn.new(), 1)
                 ):start()
end

function RocketButton:markIncorrect()
  self.outerRect:setColor(unpack(COLOR_BUTTON_INCORRECT))
  self.outerRect:setOpacity(HIGHLIGHT_OPACITY)
  self.innerRect:setColor(unpack(COLOR_BUTTON_INCORRECT))
  self.innerRect:setOpacity(HIGHLIGHT_OPACITY)
  AHScaleTo.new(self, 0.15, AHEaseSineIn.new(), 1.03):start()
end

function RocketButton:createFadeAction(duration, ease, destinationOpacity)
  local fadeOutAction = AHSpawn.new(
                                    AHOpacityTo.new(self.innerRect, duration, ease, destinationOpacity),
                                    AHOpacityTo.new(self.outerRect, duration, ease, destinationOpacity),
                                    AHOpacityTo.new(self.label, duration, ease, destinationOpacity)
                                    )
  return fadeOutAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RocketButton.new(onTouched)
  local self = Group.new(math.floor(Screen.safeWidth*0.75), math.floor(Screen.safeHeight*0.15))
  Mixin.mix(self, RocketButton)
  self.isVisible = true

  local borderWidth = math.floor(Screen.safeWidth * 0.03)
  local cornerRadius = math.floor(Screen.safeWidth * 0.005)
  local selfW, selfH = self:getDims()

  self.outerRect = FilledRoundedRectangle.new(selfW, selfH, cornerRadius, {1,1,1})

  self:insertAtCenter(self.outerRect)

  local innerW, innerH = selfW-borderWidth, selfH-borderWidth
  self.innerRect = FilledRectangle.new(innerW, innerH, {1,1,1})
  self:insertAtCenter(self.innerRect)

  self.label = Label.new("", FONT)
  self.label:setColor(unpack(COLOR_BUTTON_TEXT))
  self:insertAtCenter(self.label)

  self:unmark()
  self.isCorrect = false

  self.onTouched = onTouched
  Button.makeButton(self, onTouchDown, nil, nil)

  return self
end
