--- RocketConceptGenerator
-- @author Created by andrewcousins on April 03, 2023.

class("RocketConceptGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_RETRY_ATTEMPTS = 5

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RocketConceptGenerator:next()
  local conceptRetryCount = 0
  local concept = self.conceptGenerator:next()
  -- In rare cases the concept generator will select the same concept difficulty over and over
  -- and run out after the 14th concept.
  -- Instead of resetting and potentially getting a repeat concept, we try a few more times
  -- to see if we pull a concept of a different difficulty.
  while (concept == nil) do
    conceptRetryCount = conceptRetryCount + 1
    if (conceptRetryCount > MAX_RETRY_ATTEMPTS) then
      self.conceptGenerator:reset()
    end
    concept = self.conceptGenerator:next()
  end
  
  return concept
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RocketConceptGenerator.new()
  local self = {}
  Mixin.mix(self, RocketConceptGenerator)

  self.conceptGenerator = GameSession.conceptGenerator
  self.conceptGenerator:setAllowFewerResults(true)


  return self
end
