class("RocketConfiguration")

require("data.keyframes.KeyframeSet")
require("data.generators.LearningFocusedRoundGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

RocketConfiguration.POINTS_PER_CORRECT_RESPONSE = 400
RocketConfiguration.POINTS_PER_LIVES_BONUS = 1000
RocketConfiguration.POINTS_PER_FULL_SPEED_BONUS = 100
RocketConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 2.0
RocketConfiguration.SPEED_BONUS_DECAY_START = 0.0
RocketConfiguration.CONCEPTS_TO_WIN_COUNT = 14
RocketConfiguration.TUTORIAL_DELAY = 8

RocketConfiguration.ANSWER_DURATION_MINIMUM = 0.6

RocketConfiguration.NUMBER_OF_CONCEPTS_BEFORE_RESURFACING_MISTAKES = math.floor(
  RocketConfiguration.CONCEPTS_TO_WIN_COUNT *
  LearningFocusedRoundGenerator.DEFAULT_PROGRESS_BEFORE_RESURFACING
)

local ANSWER_DURATION_AND_MULTIPLER_KEYFRAMES = KeyframeSet.newInterpolated(
  Keyframe.new(0.0, {start = 11.1, multiplier = 0.915}),
  Keyframe.new(0.3, {start = 11.1, multiplier = 0.915}),
  Keyframe.new(0.4, {start = 9.6, multiplier = 0.9091}),
  Keyframe.new(1.0, {start = 9.6, multiplier = 0.9091})
)

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RocketConfiguration.getAnswerDurationStart()
  local difficulty = GameSession.getDifficulty()
  local answerTimingKeyframe = ANSWER_DURATION_AND_MULTIPLER_KEYFRAMES:getFrameAt(difficulty)
  return answerTimingKeyframe.start
end

function RocketConfiguration.getAnswerDurationMultiplier()
  local difficulty = GameSession.getDifficulty()
  local answerTimingKeyframe = ANSWER_DURATION_AND_MULTIPLER_KEYFRAMES:getFrameAt(difficulty)
  return answerTimingKeyframe.multiplier
end

function RocketConfiguration.getMaxSpeedBonuses()
  local correctNeeded = RocketConfiguration.CONCEPTS_TO_WIN_COUNT
  local maxBonuses = correctNeeded * RocketConfiguration.POINTS_PER_FULL_SPEED_BONUS
  return maxBonuses
end

function RocketConfiguration.getPassScore()
  local correctNeeded = RocketConfiguration.CONCEPTS_TO_WIN_COUNT
  local passScore = correctNeeded * RocketConfiguration.POINTS_PER_CORRECT_RESPONSE
  return passScore
end

function RocketConfiguration.getRankTwoScore()
  local passScore = RocketConfiguration.getPassScore()
  local rankTwoScore = passScore + RocketConfiguration.POINTS_PER_LIVES_BONUS
  return rankTwoScore
end

function RocketConfiguration.getRankThreeScore()
  local passScore = RocketConfiguration.getPassScore()
  local rankThreeScore = passScore + (2 * RocketConfiguration.POINTS_PER_LIVES_BONUS)
  return rankThreeScore
end

function RocketConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 1.00}},

    {key = 0.075, valueProbabilities = {Basic = 0.90,
                                        Easy = 0.10}},

    {key = 0.15, valueProbabilities = {Basic = 0.45,
                                       Easy = 0.50,
                                       Medium = 0.05}},

    {key = 0.225, valueProbabilities = {Basic = 0.05,
                                        Easy = 0.50,
                                        Medium = 0.45}},

    {key = 0.3, valueProbabilities = {Easy = 0.35,
                                      Medium = 0.50,
                                      Hard = 0.15}},

    {key = 0.375, valueProbabilities = {Easy = 0.30,
                                        Medium = 0.40,
                                        Hard = 0.30}},

    {key = 0.45, valueProbabilities = {Easy = 0.15,
                                       Medium = 0.50,
                                       Hard = 0.35}},

    {key = 0.525, valueProbabilities = {Easy = 0.10,
                                        Medium = 0.40,
                                        Hard = 0.50}},

    {key = 0.6, valueProbabilities = {Easy = 0.08,
                                      Medium = 0.32,
                                      Hard = 0.60}},

    {key = 0.675, valueProbabilities = {Easy = 0.06,
                                        Medium = 0.22,
                                        Hard = 0.72}},

    {key = 0.75, valueProbabilities = {Easy = 0.05,
                                       Medium = 0.12,
                                       Hard = 0.83}},

    {key = 0.8, valueProbabilities = {Medium = 0.05,
                                      Hard = 0.85,
                                      Advanced = 0.1}},

    {key = 0.9, valueProbabilities = {Hard = 0.8,
                                      Advanced = 0.2}},

    {key = 1.0, valueProbabilities = {Hard = 0.7,
                                      Advanced = 0.3}},
  }
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
