--- Rocket Engine
-- @author Created by Karl Stenerud on Oct 9, 2013.

class("RocketEngine")
require("RocketCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Z Ordering -------------------------

local Z_FLAME = 10
local Z_ACCEL = 20
local Z_WINGSTREAK = 30

-- Other ------------------------------

local ANIM_FRAMERATE = 24


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function startEngineAnim(self)
  if not self.engineAnim then
    self.engineAnim = AHLoop.new(
                                 AHAnimate.new(self.engineFlameSprite, self.engineFlameAnim, ANIM_FRAMERATE)
                                 ):start()
  end
end

local function stopEngineAnim(self)
  if self.engineAnim then
    self.engineAnim:stop()
    self.engineAnim = nil
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RocketEngine:runBoost(duration)
  startEngineAnim(self)
  if self.boostAction then
    self.boostAction:stop()
  end
  self.boostAction = AHSpawn.new(
                                 getAnimateAndVanishAction(self.engineAccelSprite, self.engineAccelAnim, ANIM_FRAMERATE),
                                 getAnimateAndVanishAction(self.wingStreakLeftSprite, self.wingStreakLeftAnim, ANIM_FRAMERATE),
                                 getAnimateAndVanishAction(self.wingStreakRightSprite, self.wingStreakRightAnim, ANIM_FRAMERATE)
                                 )
  self.boostAction:start()
  self.engineAccelSprite:setVisible(true)
  self.wingStreakLeftSprite:setVisible(true)
  self.wingStreakRightSprite:setVisible(true)
  self.engineFlameSprite:setVisible(false)
end

function RocketEngine:startIgnition()
  startEngineAnim(self)
  self.engineFlameSprite:setVisible(true)
end

function RocketEngine:startNormalThrust()
  startEngineAnim(self)
  self.engineFlameSprite:setVisible(true)
end

function RocketEngine:stop()
  self.engineAccelSprite:setVisible(false)
  self.wingStreakLeftSprite:setVisible(false)
  self.wingStreakRightSprite:setVisible(false)
  self.engineFlameSprite:setVisible(false)
  stopEngineAnim(self)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RocketEngine.new()
  local engineAccelSprite = Sprite.newWithName("rocket_accelerate/rocket_accelerate_000.png")

  local self = Group.new(engineAccelSprite:getDims())
  Mixin.mix(self, RocketEngine)

  self.engineAccelSprite = engineAccelSprite
  local selfW, selfH = self:getDims()

  self.engineFlameAnim = Animation.newWithPrefix("rocket_flame/rocket_flame_")
  self.engineFlameSprite = Sprite.newWithName("rocket_flame/rocket_flame_000.png")
  self.engineFlameSprite:setAnchorPointTop()
  self.engineFlameSprite:setLoc(selfW/2, selfH)
  self.engineFlameSprite:setVisible(false)
  self:insert(self.engineFlameSprite, Z_FLAME)

  self.engineAccelAnim = Animation.newWithPrefix("rocket_accelerate/rocket_accelerate_")
  self.engineAccelSprite:setAnchorPointTop()
  self.engineAccelSprite:setLoc(selfW/2, selfH)
  self.engineAccelSprite:setVisible(false)
  self:insert(self.engineAccelSprite, Z_ACCEL)

  self.wingStreakLeftAnim = Animation.newWithPrefix("rocket_wingstreak/rocket_wingstreak_")
  self.wingStreakLeftSprite = Sprite.newWithName("rocket_wingstreak/rocket_wingstreak_000.png")
  self.wingStreakLeftSprite:setAnchorPointTop()
  self.wingStreakLeftSprite:setLoc(selfW/2 - 32, selfH + 20)
  self.wingStreakLeftSprite:setVisible(false)
  self:insert(self.wingStreakLeftSprite, Z_WINGSTREAK)

  self.wingStreakRightAnim = Animation.newWithPrefix("rocket_wingstreak/rocket_wingstreak_")
  self.wingStreakRightSprite = Sprite.newWithName("rocket_wingstreak/rocket_wingstreak_000.png")
  self.wingStreakRightSprite:setAnchorPointTop()
  self.wingStreakRightSprite:setLoc(selfW/2 + 36, selfH + 20)
  self.wingStreakRightSprite:setVisible(false)
  self:insert(self.wingStreakRightSprite, Z_WINGSTREAK)

  return self
end
