--- The little side bar that shows up in a bunch of games.  It's the "continue" button in Swell, for instance.
-- @author Created by Aaron Barrett on Wednesday, July 17, 2013.

-- ===========================================================================
class("SidebarButton")
-- ===========================================================================

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_LABEL = 10

local DEFAULT_Y = math.floor(Screen.getSafeY(0.25))

local WIGGLE_START_X = Screen.safeRight - 70.0
local WIGGLE_DESTINATION_X = WIGGLE_START_X - 10.0

local FONT = FontFaces.DIN_OT_BOLD

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SidebarButton:getSlideInAction()
  local slideInAction = AHLocationXTo.new(self, 0.5, nil, WIGGLE_START_X)
  return slideInAction
end

function SidebarButton:getSlideOutAction()
  local width = self:getDims()
  local destinationX = Screen.fullWidth + width * 0.5

  local slideOutAction = AHLocationXTo.new(self, 0.25, nil, destinationX)
  return slideOutAction
end

function SidebarButton:getWiggleAction()
  local wiggleAction = AHLoop.new(
                                  AHSequence.new(
                                                 AHLocationXTo.new(self, 1.15, AHEaseSimpleOut.new(2.0), WIGGLE_DESTINATION_X),
                                                 AHLocationXTo.new(self, 0.95, AHEaseExponentialOut.new(), WIGGLE_START_X)
                                                 )
                                  )

  return wiggleAction
end

function SidebarButton:show()
  if (self.movementAction) then
    self.movementAction:stop()
  end

  local movementCoroutine = MOAICoroutine.new()
  self.movementAction = movementCoroutine
  movementCoroutine.tag = "sidebarButtonMovement"

  local slideInAction = self:getSlideInAction()
  local wiggleAction = self:getWiggleAction()

  movementCoroutine:run(function()
      wait(slideInAction:start())
      wait(wiggleAction:start())
    end)
end

function SidebarButton:hide()
  if (self.movementAction) then
    self.movementAction:stop()
  end

  local slideOutAction = self:getSlideOutAction()
  self.movementAction = slideOutAction
  slideOutAction.tag = "sidebarButtonMovement"

  slideOutAction:start()
end

function SidebarButton:positionAtStart(y)
  y = y or DEFAULT_Y

  local width = self:getDims()
  self:setLoc(Screen.fullWidth + width * 0.5, y)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SidebarButton.newWithImageAndText(filename, text, color)
  if (not color) then
    color = {0.14, 0.40, 0.48}
  end
  local self = Sprite.newWithName(filename)
  Mixin.mix(self, SidebarButton)

  local label = Label.new(text, Font.new(FONT, 26))
  label:setColor(unpack(color))
  self:insertAtCenter(label, Z_LABEL)
  self.label = label

  return self
end
