-- Rocket Slipstream

class("Slipstream")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Z Ordering -------------------------

local Z_SLIPSTREAM_OUTER = 10
local Z_SLIPSTREAM_INNER = 20

-- Other Stuff ------------------------

local INNER_SCALE_X = 0.6

local SCALE_ACTION_DURATION = 0.5

local OPACITY = 0.18

local BOOST_START_DURATION = 0.2
local BOOST_END_DURATION = 0.5


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function Slipstream:startBoost()
  return AHSpawn.new(
                     AHOpacityTo.new(self, BOOST_START_DURATION, AHEaseSineOut.new(), 1),
                     AHScaleTo.new(self, BOOST_START_DURATION, nil, 1.5)
                     ):start()
end

function Slipstream:endBoost()
  return AHSpawn.new(
                     AHOpacityTo.new(self, BOOST_END_DURATION, AHEaseSineOut.new(), 0),
                     AHScaleTo.new(self, BOOST_END_DURATION, AHEaseSineIn.new(), 1)
                     ):start()
end

function Slipstream:reset()
  self:setScl(1)
  self:setOpacity(0)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function Slipstream.new()
  local streamOuter = Sprite.newWithName("rocket_blur.png")

  local self = Group.new(streamOuter:getWidth(), Screen.fullHeight)
  Mixin.mix(self, Slipstream)

  self.streamOuter = streamOuter
  local streamW, streamH = self:getDims()
  self.streamOuter:setEffectiveDims(streamW, streamH)
  self.streamOuter:setAnchorPointTop()
  self.streamOuter:setLoc(streamW/2, streamH)
  self.streamOuter:setOpacity(OPACITY)
  self:insert(self.streamOuter, Z_SLIPSTREAM_OUTER)

  self.streamInner = Sprite.newWithName("rocket_blur.png")
  self.streamInner:setEffectiveDims(streamW * INNER_SCALE_X, streamH)
  self.streamInner:setAnchorPointTop()
  self.streamInner:setLoc(streamW/2, streamH)
  self.streamInner:setOpacity(OPACITY)
  self:insert(self.streamInner, Z_SLIPSTREAM_INNER)

  self:linkAttrToChildren(MOAIColor.INHERIT_COLOR, MOAIColor.COLOR_TRAIT)

  self:setBounds(0.0, 0.0, 0, streamW, streamH, 0)
  self:setAnchorPointCenter()

  return self
end
