--- RocketValidator
-- @author Created by Anton Morant on April 17, 2014.
-- ConceptValidator for Rocket.

class("RocketValidator")

require("validators.ConceptValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'question',
  'answer',
  'distractor1',
  'distractor2',
  'distractor3',
  'gameReportDefinition',
  'distractorDefinition1',
  'distractorDefinition2',
  'distractorDefinition3'
}

local MAXIMUM_QUESTION_LENGTH = 18

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------
local function validateWordsAreUnique(self, concept)
  local question = concept.question
  local answer = concept.answer
  local distractor1 = concept.distractor1
  local distractor2 = concept.distractor2
  local distractor3 = concept.distractor3

  local words = {question, answer, distractor1, distractor2, distractor3}
  local fieldName = nil
  local message = "One or more of the following was identical to another: vocab word, definition, distractors."
  self:expectNoDuplicates(words, fieldName, message)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RocketValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)

  local question = concept.question
  local answer = concept.answer
  local distractor1 = concept.distractor1
  local distractor2 = concept.distractor2
  local distractor3 = concept.distractor3
  local gameReportDefinition = concept.gameReportDefinition

  self:expectLessOrEqual(#question, MAXIMUM_QUESTION_LENGTH, "length of question")
  self:expectStringIsLettersOnly(question, "question", " -")
  self:expectStringIsLettersOnly(answer, "answer", " -")
  self:expectStringIsLettersOnly(distractor1, "distractor1", " -")
  self:expectStringIsLettersOnly(distractor2, "distractor2", " -")
  self:expectStringIsLettersOnly(distractor3, "distractor3", " -")

  validateWordsAreUnique(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function RocketValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, RocketValidator)

  return self
end
