--- RutabagaDefinitionsList
-- @author Created by Jay Austria on May 25, 2016.

class("RutabagaDefinitionsList")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_COLOR = {1.0, 0.22, 0.22}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RutabagaDefinitionsList:createPresentAction()
  local presentAction = AHSpawn.new(
    AHOpacityTo.new(self, 0.4, nil, 1.0),
    AHScaleTo.new(self, 0.4, AHEaseSineIn.new(), 1.0)
  )

  return presentAction
end

function RutabagaDefinitionsList:createDismissAction()
  local dismissAction = AHSpawn.new(
    AHOpacityTo.new(self, 0.4, nil, 0.0),
    AHScaleTo.new(self, 0.4, AHEaseSineIn.new(), 1.2)
  )

  return dismissAction
end

function RutabagaDefinitionsList:prepareToPresent()
  self:setOpacity(0.0)
  self:setScl(1.2)
end

function RutabagaDefinitionsList:initializeWithPartsAndDefinitions(partsAndDefinitions)
  self:removeAll()

  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 20)
  for _, partAndDefinition in ipairs(partsAndDefinitions) do
    local part = partAndDefinition.part
    local definition = partAndDefinition.definition

    local entryText = string.format("%s - %s", part, definition)
    local entry = Label.new(entryText, font)
    entry:setColor(unpack(TEXT_COLOR))

    self:insert(entry)
  end

  self:alignChildren()

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------


-- Constructor ------------------------

function RutabagaDefinitionsList.new()
  local self = Column.new(5)
  Mixin.mix(self, RutabagaDefinitionsList)

  return self
end
