--- RutabagaDirectiveLabel
-- @author Created by Jay Austria on June 01, 2016.

class("RutabagaDirectiveLabel")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LABEL_COLOR = {1.0, 0.76, 0.30}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RutabagaDirectiveLabel:setText(left, middle, right)
  self.leftLabel:setText(left)
  self.middleLabel:setText(middle)
  self.rightLabel:setText(right)

  self:alignChildren()
end

function RutabagaDirectiveLabel:prepareForAnimateIn()
  local magnifiedScale = 15.0
  self.leftLabel:setOpacity(0.0)
  self.leftLabel:setScl(magnifiedScale)

  self.middleLabel:setOpacity(0.0)
  self.middleLabel:setScl(magnifiedScale)

  self.rightLabel:setOpacity(0.0)
  self.rightLabel:setScl(magnifiedScale)
end

function RutabagaDirectiveLabel:createAnimateInAction()
  local duration = 0.6
  local exitDuration = 0.3
  local exitDelayDuration = 0.5
  local fadeInDelayDuration = 0.15
  local staggerDelayDuration = 0.15
  local ease = AHEaseExponentialOut.new(0.7)
  local animateAction = AHSequence.new(
    AHSpawn.new(
      AHSpawn.new(
        AHDelayed.new(fadeInDelayDuration, AHOpacityTo.new(self.leftLabel, duration, nil, 1.0)),
        AHScaleTo.new(self.leftLabel, duration, ease, 1.0)
      ),
      AHDelayed.new(staggerDelayDuration,
        AHSpawn.new(
          AHDelayed.new(fadeInDelayDuration, AHOpacityTo.new(self.middleLabel, duration, nil, 1.0)),
          AHScaleTo.new(self.middleLabel, duration, ease, 1.0)
        )
      ),
      AHDelayed.new(staggerDelayDuration * 2.0,
        AHSpawn.new(
          AHDelayed.new(fadeInDelayDuration, AHOpacityTo.new(self.rightLabel, duration, nil, 1.0)),
          AHScaleTo.new(self.rightLabel, duration, ease, 1.0)
        )
      )
    ),
    AHDelay.new(exitDelayDuration),
    AHSpawn.new(
      AHOpacityTo.new(self.leftLabel, exitDuration, nil, 0.0),
      AHOpacityTo.new(self.middleLabel, exitDuration, nil, 0.0),
      AHOpacityTo.new(self.rightLabel, exitDuration, nil, 0.0)
    )
  )

  return animateAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, 16)
  local unsetString = ""

  local leftLabel = Label.new(unsetString, font)
  leftLabel:setColor(unpack(LABEL_COLOR))
  self:insert(leftLabel)
  self.leftLabel = leftLabel

  local middleLabel = Label.new(unsetString, font)
  middleLabel:setColor(unpack(LABEL_COLOR))
  self:insert(middleLabel)
  self.middleLabel = middleLabel

  local rightLabel = Label.new(unsetString, font)
  rightLabel:setColor(unpack(LABEL_COLOR))
  self:insert(rightLabel)
  self.rightLabel = rightLabel
end

-- Constructor ------------------------

function RutabagaDirectiveLabel.new()
  local itemSpacing = 6
  local self = Row.new(itemSpacing)
  Mixin.mix(self, RutabagaDirectiveLabel)

  createLayout(self)

  return self
end
