--- RutabagaHighLevelInterstitial
-- @author Created by Jay Austria on May 31, 2016.

class("RutabagaHighLevelInterstitial")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local YELLOWISH_COLOR = {1.0, 0.76, 0.30}
local FADED_YELLOWISH_COLOR = {0.8, 0.56, 0.10}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createFadeAction(self, targetOpacity)
  return AHOpacityTo.new(self, 0.3, nil, targetOpacity)
end

local function createAndStartLoopingScaleAnimations(self, innerRing, outerRing)
  local loopDuration = 1.5

  local outerRingLoopingAction = AHLoop.new(
    AHSequence.new(
      AHScaleBy.new(outerRing, loopDuration / 2,  AHEaseSineInOut.new(), 0.1),
      AHScaleBy.new(outerRing, loopDuration / 2, AHEaseSineInOut.new(), -0.1)
    )
  )

  local innerRingLoopingAction = AHDelayed.new(0.1, AHLoop.new(
    AHSequence.new(
      AHScaleBy.new(innerRing, loopDuration / 2, AHEaseSineInOut.new(), 0.1),
      AHScaleBy.new(innerRing, loopDuration / 2, AHEaseSineInOut.new(), -0.1)
    )
  ))

  self.innerRingLoopingAction = innerRingLoopingAction:start()
  self.outerRingLoopingAction = outerRingLoopingAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RutabagaHighLevelInterstitial:prepareForAnimateIn()
    self:setOpacity(0.0)
end

function RutabagaHighLevelInterstitial:doAnimateIn()
  local action = createFadeAction(self, 1.0)

  return action:start()
end

function RutabagaHighLevelInterstitial:doAnimateOutAndRemove()
  local stopLoopingActions = function()
    self.innerRingLoopingAction:stop()
    self.outerRingLoopingAction:stop()
  end

  local action = AHSequence.new(
    createFadeAction(self, 0.0),
    stopLoopingActions,
    AHRemove.new(self)
  )

  return action:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local outerRingLineWidth = 0.4
  local outerRing = HollowCircle.new(self:getWidthPart(0.24), FADED_YELLOWISH_COLOR, outerRingLineWidth)
  outerRing:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.7))
  self:insert(outerRing)

  local innerRingLineWidth = 1
  local innerRing = HollowCircle.new(self:getWidthPart(0.22), YELLOWISH_COLOR, innerRingLineWidth)
  innerRing:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.7))
  self:insert(innerRing)

  local icon = Sprite.newWithName("columnUnlock.png")
  icon:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.7))
  self:insert(icon)

  local headerText = "FOURTH COLUMN UNLOCKED"
  local headerFont = Font.new(FontFaces.DIN_OT_MEDIUM, 24)
  local headerWidth = Screen.getSafeWidthPart(0.8)
  local headerLabel = TextArea.new(headerText, headerFont, headerWidth, Alignment.CENTER)
  headerLabel:setColor(unpack(YELLOWISH_COLOR))
  headerLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.46))
  self:insert(headerLabel)

  local detailText = "You’re playing Word Parts at a high difficulty level. You'll now see longer words made up of 4 parts."
  local detailWidth = Screen.getSafeWidthPart(0.7)
  local detailFont = Font.new(FontFaces.DIN_OT_MEDIUM, 16)
  local detailLabelLineSpacing = 5
  local detailLabel = DynamicTextArea.newSimple(detailText, detailWidth, detailFont, Alignment.CENTER, nil, detailLabelLineSpacing)
  detailLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.29))
  self:insert(detailLabel)

  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  tapToContinueLabel:setColor(unpack(YELLOWISH_COLOR))
  tapToContinueLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.12))
  self:insert(tapToContinueLabel)

  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)

  createAndStartLoopingScaleAnimations(self, innerRing, outerRing)
end

-- Constructor ------------------------

function RutabagaHighLevelInterstitial.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, RutabagaHighLevelInterstitial)

  createLayout(self)

  return self
end
