--- RutabagaPartPillar
-- @author Created by Jay Austria on May 18, 2015.

class("RutabagaPartPillar")

require("RutabagaCommon")
require("RutabagaVisualStateGroup")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PART_TEXT_COLOR = ColorUtils.colorHex24ToStd(0x390B04)
local PART_TEXT_COLOR_INACTIVE = {0.42, 0.01, 0.0}
local ACTIVATE_OFFSET_Y = -Screen.getHeightPart(0.04)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getTextColorsForStates()
  return {
    [RutabagaCommon.STATES.UNACTIVATED] = PART_TEXT_COLOR,
    [RutabagaCommon.STATES.ACTIVATED] = PART_TEXT_COLOR,
    [RutabagaCommon.STATES.PAUSED_INCORRECT] = PART_TEXT_COLOR,
    [RutabagaCommon.STATES.PAUSED_UNACTIVATED] = PART_TEXT_COLOR_INACTIVE
  }
end

local function createButtonColorsForStates(buttonColor)
  return {
    [RutabagaCommon.STATES.UNACTIVATED] = buttonColor,
    [RutabagaCommon.STATES.ACTIVATED] = RutabagaCommon.COLORS.ACTIVATED_COLUMN_COLORS.BUTTON_COLOR,
    [RutabagaCommon.STATES.PAUSED_INCORRECT] = RutabagaCommon.COLORS.PAUSED_INCORRECT_COLUMN_COLORS.BUTTON_COLOR,
    [RutabagaCommon.STATES.PAUSED_UNACTIVATED] = RutabagaCommon.COLORS.PAUSED_UNACTIVATED_COLUMN_COLORS.BUTTON_COLOR
  }
end

local function createPillarColorsForStates(pillarColors)
  return {
    [RutabagaCommon.STATES.UNACTIVATED] = {
      beginColor = pillarColors.BEGIN_COLOR,
      endColor = pillarColors.END_COLOR
    },
    [RutabagaCommon.STATES.ACTIVATED] = {
      beginColor = RutabagaCommon.COLORS.ACTIVATED_COLUMN_COLORS.BEGIN_COLOR,
      endColor = RutabagaCommon.COLORS.ACTIVATED_COLUMN_COLORS.END_COLOR
    },
    [RutabagaCommon.STATES.PAUSED_INCORRECT] = {
      beginColor = RutabagaCommon.COLORS.PAUSED_INCORRECT_COLUMN_COLORS.BEGIN_COLOR,
      endColor = RutabagaCommon.COLORS.PAUSED_INCORRECT_COLUMN_COLORS.END_COLOR
    },
    [RutabagaCommon.STATES.PAUSED_UNACTIVATED] = {
      beginColor = RutabagaCommon.COLORS.PAUSED_UNACTIVATED_COLUMN_COLORS.BEGIN_COLOR,
      endColor = RutabagaCommon.COLORS.PAUSED_UNACTIVATED_COLUMN_COLORS.END_COLOR
    }
  }
end

local function registerButtonPress(self)
  if (self.onPressCallback ~= nil) then
    self.onPressCallback(self)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RutabagaPartPillar:setTouchEnabled(isEnabled)
  self.partButton:setTouchEnabled(isEnabled)
end

function RutabagaPartPillar:getPartLabel()
  return self.partLabel
end

function RutabagaPartPillar:flagForActivatation()
  self.willActivate = true
end

function RutabagaPartPillar:flagForDeactivation()
  self.willDeactivate = true
end

function RutabagaPartPillar:resetActivateFlags()
  self.willActivate = false
  self.willDeactivate = false
end

function RutabagaPartPillar:getWillActivate()
  return self.willActivate
end

function RutabagaPartPillar:getWillDeactivate()
  return self.willDeactivate
end

function RutabagaPartPillar:getActivated()
  return self.isActivated
end

function RutabagaPartPillar:setIsActivated(activated)
  self.isActivated = activated
end

function RutabagaPartPillar:setWasPreviouslyActivated(activated)
  self.wasPreviouslyActivated = activated
end

function RutabagaPartPillar:getWasPreviouslyActivated()
  return self.wasPreviouslyActivated
end

function RutabagaPartPillar:setOnPressCallback(callback)
  self.onPressCallback = callback
end

function RutabagaPartPillar:playRepositionAction(endY)
  local repositionAction = AHDelayed.new(0.05, AHLocationYTo.new(self, 0.4, AHEaseSimpleInOut.new(5.0), endY))
  self.animationInterrupt:insert(repositionAction)
end

function RutabagaPartPillar:establishHomeY()
  self.homeY = self:getY()
  self.offsetY = self:getY() + ACTIVATE_OFFSET_Y
end

function RutabagaPartPillar:playActivateAction(endY, isLastTap, onActivateAction)
  if (endY == nil) then
    endY = self.homeY
  end

  assert(endY ~= nil, "Pillar's initial position has not yet been established. May need to call establishHomeY")

  local returnDuration = nil
  local returnEase = nil
  if (isLastTap) then
    returnDuration = 0.35
    returnEase = AHEaseSineInOut.new()
  else
    returnDuration = 0.34
    returnEase = AHEaseBackOut.new(1.0)
  end

  local activateAction = AHSpawn.new(
    function()
      if (onActivateAction ~= nil) then
        onActivateAction()
      end
    end,
    AHSequence.new(
      AHLocationYTo.new(self, 0.1, AHEaseBackOut.new(1.0), self.offsetY),
      AHLocationYTo.new(self, returnDuration, returnEase, endY)
    )
  )

  self.animationInterrupt:insert(activateAction)
end

function RutabagaPartPillar:setVisualState(state)
  self.columnFill:setVisualState(state)
  self.partButton:setVisualState(state)
  self.partLabel:setVisualState(state)
end

function RutabagaPartPillar:getPartDefinition()
  return self.partDefinition
end

function RutabagaPartPillar:getPartText()
  return self.partText
end

function RutabagaPartPillar:displayForPromotedPresent()
  self.flashLayer:setOpacity(1.0)
end

function RutabagaPartPillar:createPromotedPresentAction()
  local promotedPresentAction = AHOpacityTo.new(self.flashLayer, 0.5, nil, 0.0)

  return promotedPresentAction
end

function RutabagaPartPillar:getIsTouchAllowed()
  return self.isTouchAllowed
end

function RutabagaPartPillar:setIsTouchAllowed(allowed)
  self.isTouchAllowed = allowed
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function initializeTouch(self)
  local touchBeganWrapper = function()
    registerButtonPress(self)
  end

  Button.makeButton(self.partButton, touchBeganWrapper)
end

local function createLayout(self, partText, pillarColors, buttonColor)
  local width, height = self:getDims()
  local isHorizontal = false
  local colorsForPillarStates = createPillarColorsForStates(pillarColors)

  local columnColorSetter = function(column, colorPair)
    column:setBeginColor(unpack(colorPair.beginColor))
    column:setEndColor(unpack(colorPair.endColor))
  end

  local initialColumnBeginColor = colorsForPillarStates[RutabagaCommon.STATES.UNACTIVATED].beginColor
  local initialColumnEndColor = colorsForPillarStates[RutabagaCommon.STATES.UNACTIVATED].endColor

  local columnFill = RectangleGradient.new(width, height, initialColumnBeginColor, initialColumnEndColor, isHorizontal)
  RutabagaVisualStateGroup.makeVisualStateGroup(columnFill, colorsForPillarStates, columnColorSetter)

  local columnFillYFudgedToShowRoundedCorners = self:getHeight() - RutabagaCommon.STANDARD_CORNER_RADIUS
  columnFill:setAnchorPointTop()
  columnFill:setLoc(self:getWidthPart(0.5), columnFillYFudgedToShowRoundedCorners)
  self:insert(columnFill)
  self.columnFill = columnFill

  local buttonHeight = RutabagaCommon.COLUMN_BUTTON_HEIGHT
  local colorsForButtonStates = createButtonColorsForStates(buttonColor)
  local initialButtonColor = colorsForButtonStates[RutabagaCommon.STATES.UNACTIVATED]

  local buttonColorSetter = function(button, color)
    button:setColor(unpack(color))
  end

  local partButton = FilledRoundedRectangle.new(width, buttonHeight, RutabagaCommon.STANDARD_CORNER_RADIUS, initialButtonColor)
  RutabagaVisualStateGroup.makeVisualStateGroup(partButton, colorsForButtonStates, buttonColorSetter)
  partButton:setAnchorPointTop()
  partButton:setLoc(self:getWidthPart(0.5), self:getHeight())
  self:insert(partButton)
  self.partButton = partButton

  local partTextUpper = string.upper(partText)
  local fontSize = 19
  local colorsForTextStates = getTextColorsForStates()
  local textColorSetter = function(text, color)
    text:setColor(unpack(color))
  end
  local partLabel = Label.new(partTextUpper, Font.new(FontFaces.DIN_OT_MEDIUM, fontSize))
  RutabagaVisualStateGroup.makeVisualStateGroup(partLabel, colorsForTextStates, textColorSetter)
  partLabel:setColor(unpack(PART_TEXT_COLOR))
  partButton:insertAtCenter(partLabel)
  self.partLabel = partLabel

  local flashLayer = FilledRoundedRectangle.new(width, height, RutabagaCommon.STANDARD_CORNER_RADIUS, RutabagaCommon.COLORS.FLASH_COLOR)
  flashLayer:setOpacity(0.0)
  self:insertAtCenter(flashLayer)
  self.flashLayer = flashLayer
end

function RutabagaPartPillar.new(width, partText, partDefinition, pillarColors, buttonColor)
  local height = RutabagaCommon.COLUMN_HEIGHT / RutabagaCommon.SHRUNKEN_SCALE
  local self = Group.new(width, height)

  Mixin.mix(self, RutabagaPartPillar)

  createLayout(self, partText, pillarColors, buttonColor)
  initializeTouch(self)

  self.isActivated = false
  self.animationInterrupt = ActionInterrupt.new()
  self.willActivate = false
  self.willDeactivate = false
  self.wasPreviouslyActivated = false
  self.partDefinition = partDefinition
  self.partText = partText
  self.homeY = nil
  self.offsetY = nil
  self.isTouchAllowed = true

  return self
end
