--- RutabagaPartsStackCorrectOverlay
-- @author Created by Jay Austria on May 22, 2015.

class("RutabagaPartsStackCorrectOverlay")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playCombineLabelsAction(self)
  if (#self.textLabels > 1) then
    local resultWidth = 0
    for _, textLabel in ipairs(self.textLabels) do
      local scaledLeftMargin = (textLabel:getLeftMargin() * textLabel:getScl())
      local scaledRightMargin = (textLabel:getRightMargin() * textLabel:getScl())

      resultWidth = resultWidth + textLabel:getEffectiveWidth() + scaledLeftMargin + scaledRightMargin
    end

    local combineLabelsAction = AHSpawn.new()
    local currentScanX = self:getEffectiveWidthPart(0.5) - (resultWidth / 2)
    local targetX = nil
    local targetY = self.textLabels[1]:getY()
    local labelWidth = nil
    for _, textLabel in ipairs(self.textLabels) do
      local scaledLeftMargin = (textLabel:getLeftMargin() * textLabel:getScl())
      local scaledRightMargin = (textLabel:getRightMargin() * textLabel:getScl())

      labelWidth = textLabel:getEffectiveWidth()
      targetX = currentScanX + (labelWidth / 2) + scaledLeftMargin
      combineLabelsAction:addAction(AHLocationTo.new(textLabel, 0.25, AHEaseSineInOut.new(), targetX, targetY))

      currentScanX = currentScanX + labelWidth + scaledLeftMargin + scaledRightMargin
    end

    combineLabelsAction:start()
  end
end

local function transformAndTakeTextLabels(self)
  for _, textLabel in ipairs(self.textLabels) do
    if (self.isShrunken) then
      textLabel:setScl(RutabagaCommon.SHRUNKEN_SCALE)
    end

    -- Transform textLabel's coordinates into stack space by going from world
    -- space (screen) -> stack space
    local textWorldX, textWorldY = textLabel:getWorldLoc()
    local textX, textY = self:transformLoc(Screen.rootGroup, textWorldX, textWorldY)

    local scaleCompensatedTextX = textX + textLabel:getEffectiveWidthPart(0.5)
    local scaleCompensatedTextY = textY + textLabel:getEffectiveHeightPart(0.5)

    self:takeChild(textLabel)
    textLabel:setLoc(scaleCompensatedTextX, scaleCompensatedTextY)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RutabagaPartsStackCorrectOverlay:createCorrectAction(additionalDelay)
  additionalDelay = additionalDelay or 0

  return AHSpawn.new(
    function()
      transformAndTakeTextLabels(self)
      playCombineLabelsAction(self)
    end,
    AHSequence.new(
      AHOpacityTo.new(self.flashOverlay, 0.05, nil, 1.0),
      AHOpacityTo.new(self.filledLayer, 0.1, nil, 1.0),
      function()
        if self.shouldShowCascadeTexture then
          self.noisyCascadeLayer:setVisibleRecursive(true)
        end
      end,
      AHDelay.new(0.05),
      AHOpacityTo.new(self.flashOverlay, 0.5, nil, 0.0)
    ),
    AHSequence.new(
      AHDelayed.new(additionalDelay, AHLocationYBy.new(self, 1.0, AHEaseSimpleIn.new(5.0), -self:getHeight())),
      function()
        self.noisyCascadeLayer:release()
      end,
      AHRemove.new(self)
    )
  )
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createBorder(self)
  local borderThickness = 6
  local border = FilledRectangle.new(borderThickness, self:getHeight(), RutabagaCommon.COLORS.FLASH_COLOR)

  return border
end

local function createLayout(self, textLabels, isShrunken, shouldShowCascadeTexture)
  local width = self:getWidth()
  local fillHeight = nil

  if (not shouldShowCascadeTexture) then
    fillHeight = self:getHeight()
  elseif (isShrunken) then
    fillHeight = RutabagaCommon.COLUMN_BUTTON_HEIGHT * RutabagaCommon.SHRUNKEN_SCALE
  else
    fillHeight = RutabagaCommon.COLUMN_BUTTON_HEIGHT
  end

  local noisyCascadeLayer = RutabagaNoisyCascadeLayer.new(self:getWidth(), self:getHeight() - fillHeight)
  noisyCascadeLayer:setAnchorPointBottom()
  noisyCascadeLayer:setX(self:getWidthPart(0.5))
  noisyCascadeLayer:setVisibleRecursive(false)
  self:insert(noisyCascadeLayer)
  self.noisyCascadeLayer = noisyCascadeLayer

  local filledLayer = Group.new(self:getWidth(), self:getHeight())
  self:insertAtCenter(filledLayer)
  self.filledLayer = filledLayer

  local filledBackground = FilledRoundedRectangle.new(width, fillHeight, RutabagaCommon.STANDARD_CORNER_RADIUS, RutabagaCommon.COLORS.FLASH_COLOR)
  filledBackground:setAnchorPointTop()
  filledBackground:setLoc(self:getWidthPart(0.5), self:getHeight())
  filledLayer:insert(filledBackground)

  local leftBorder = createBorder(self)
  leftBorder:setAnchorPointLeft()
  leftBorder:setY(self:getHeightPart(0.5) - RutabagaCommon.STANDARD_CORNER_RADIUS)
  filledLayer:insert(leftBorder)

  local rightBorder = createBorder(self)
  rightBorder:setAnchorPointRight()
  rightBorder:setLoc(self:getWidth(), self:getHeightPart(0.5) - RutabagaCommon.STANDARD_CORNER_RADIUS)
  filledLayer:insert(rightBorder)

  local flashOverlay = FilledRoundedRectangle.new(self:getWidth(), self:getHeight(), RutabagaCommon.STANDARD_CORNER_RADIUS, RutabagaCommon.COLORS.FLASH_COLOR)
  self:insertAtCenter(flashOverlay)
  self.flashOverlay = flashOverlay

  local isRecursive = true
  filledLayer:linkOpacityAttrToChildren(isRecursive)
  filledLayer:setOpacity(0.0)
  flashOverlay:setOpacity(0.0)

  self.textLabels = textLabels
end

function RutabagaPartsStackCorrectOverlay.new(width, height, textLabels, isShrunken, shouldShowCascadeTexture)
  local self = Group.new(width, height)
  Mixin.mix(self, RutabagaPartsStackCorrectOverlay)

  createLayout(self, textLabels, isShrunken, shouldShowCascadeTexture)

  self.isShrunken = isShrunken
  self.shouldShowCascadeTexture = shouldShowCascadeTexture

  return self
end
