--- RutabagaPrompt
-- @author Created by Jay Austria on May 24, 2016.

class("RutabagaPrompt")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local INSTRUCTIONS_OPACITY_FULL = 0.5
local INSTRUCTIONS_OPACITY_FADED_OUT = 0.43
local INSTRUCTIONS_COLOR = {0.98, 0.91, 0.7, INSTRUCTIONS_OPACITY_FULL}

local PROMPT_OPACITY_FULL = 1.0
local PROMPT_OPACITY_FADED_OUT = 0.42
local PROMPT_COLOR = {0.98, 0.91, 0.7}

local INSTRUCTIONS_LABEL_Y_OFFSET = Screen.getHeightPart(0.04)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function prepareForPresent(self)
  self.instructionsLabel:setOpacity(0.0)
  self.promptLabel:setOpacity(0.0)
end

local function createFadeAction(self, targetInstructionsOpacity, targetPromptOpacity)
  local duration = 0.2
  local fadeAction = AHSpawn.new(
    AHOpacityTo.new(self.instructionsLabel, duration, nil, targetInstructionsOpacity),
    AHOpacityTo.new(self.promptLabel, duration, nil, targetPromptOpacity)
  )

  return fadeAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RutabagaPrompt:createRationaleFadeOutAction()
  return createFadeAction(self, INSTRUCTIONS_OPACITY_FADED_OUT, PROMPT_OPACITY_FADED_OUT)
end

function RutabagaPrompt:createRationaleFadeInAction()
  return createFadeAction(self, INSTRUCTIONS_OPACITY_FULL, PROMPT_OPACITY_FULL)
end

function RutabagaPrompt:createPresentAction()
  local duration = 0.5
  local presentAction = AHSpawn.new(
    AHOpacityTo.new(self.instructionsLabel, duration, nil, INSTRUCTIONS_COLOR[4]),
    AHOpacityTo.new(self.promptLabel, duration, nil, 1.0)
  )

  return presentAction
end

function RutabagaPrompt:createExitAction()
  local duration = 0.7
  local exitAction = AHSequence.new(
    AHSpawn.new(
      AHOpacityTo.new(self.instructionsLabel, duration, nil, 0.0),
      AHOpacityTo.new(self.promptLabel, duration, nil, 0.0)
    ),
    AHRemove.new(self)
  )

  return exitAction
end

function RutabagaPrompt:createExitForOutroAction()
  local duration = 0.5
  local exitAction = AHSequence.new(
    AHLocationYBy.new(self, duration, AHEaseSimpleIn.new(5.0), Screen.fullHeight),
    AHRemove.new(self)
  )

  return exitAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, instructionsText, promptText)
  local promptWidth = Screen.getSafeWidthPart(0.7)
  local promptLabel = TextArea.new(promptText, Font.new(FontFaces.DIN_OT_MEDIUM, 24), promptWidth, Alignment.CENTER)
  promptLabel:setColor(unpack(PROMPT_COLOR))
  promptLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.81))
  self:insert(promptLabel)
  self.promptLabel = promptLabel

  local instructionsLabel = Label.new(instructionsText, Font.new(FontFaces.DIN_OT_BOLD, 13))
  instructionsLabel:setColor(unpack(INSTRUCTIONS_COLOR))
  local instructionsLabelY = promptLabel:getYTop() + INSTRUCTIONS_LABEL_Y_OFFSET
  instructionsLabel:setLoc(self:getWidthPart(0.5), instructionsLabelY)
  self:insert(instructionsLabel)
  self.instructionsLabel = instructionsLabel
end

-- Constructor ------------------------

function RutabagaPrompt.newHidden(instructionsText, promptText)
  local self = Group.newFullScreen()
  Mixin.mix(self, RutabagaPrompt)

  createLayout(self, instructionsText, promptText)
  prepareForPresent(self)

  return self
end
