--- RutabagaRoundProgressionTracker
-- @author Created by Jay Austria on May 25, 2016.

class("RutabagaRoundProgressionTracker")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAX_PART_COUNTS = 3
local MAX_PART_COUNTS_EXPERT = 4
local CORRECT_NEEDED_TO_FORCE_PROMOTION = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RutabagaRoundProgressionTracker:notifyCorrectResponse()
    self.previousPartCount = self.currentPartCount

    self.correctCounts[self.currentPartCount] = self.correctCounts[self.currentPartCount] + 1

    local correctCountForPartCount = self.correctCounts[self.currentPartCount]
    local didHitPromotionThreshold = (correctCountForPartCount >= CORRECT_NEEDED_TO_FORCE_PROMOTION)
    if (didHitPromotionThreshold) then
      self.currentPartCount = math.min(self.maxPartCount, self.currentPartCount + 1)
    end
end

function RutabagaRoundProgressionTracker:getPartCount()
  return self.currentPartCount
end

function RutabagaRoundProgressionTracker:getPreviousPartCount()
  return self.previousPartCount
end

function RutabagaRoundProgressionTracker:getDidPromote()
  local didPromote = ((self.previousPartCount ~= nil) and (self.currentPartCount ~= self.previousPartCount))

  return didPromote
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function initializeCorrectCountMap(self)
  self.correctCounts = {}
  for partCount = 1, self.maxPartCount do
    self.correctCounts[partCount] = 0
  end
end

-- Constructor ------------------------

function RutabagaRoundProgressionTracker.new()
  local self = {}
  Mixin.mix(self, RutabagaRoundProgressionTracker)

  self.previousPartCount = nil
  self.currentPartCount = 1

  if (RutabagaConfiguration.getIsHighLevel()) then
    self.maxPartCount = MAX_PART_COUNTS_EXPERT
  else
    self.maxPartCount = MAX_PART_COUNTS
  end

  initializeCorrectCountMap(self)

  return self
end
