--- RutabagaScene
-- @author Created by Jay Austria on May 18, 2015.

class("RutabagaScene")

require("RutabagaPartsStackLayer")
require("RutabagaPartsStackCorrectOverlay")
require("RutabagaPrompt")
require("RutabagaDefinitionsList")
require("RutabagaHighLevelInterstitial")
require("RutabagaPausedOverlay")
require("RutabagaDirectiveLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PAUSED_BACKING_COLOR = {0.36, 0.08, 0.1}
local GLOW_TINT_COLOR = {0.53, 0.24, 0.06}
local RATIONALE_SCALE_BY = 0.08

local Z_PAUSED_BACKING = 10
local Z_UI = 20
local Z_PARTS_STACKS = 30
local Z_CORRECT_OVERLAY = 40
local Z_GLOW_OVERLAY = 50
local Z_PAUSED_OVERLAY = 60
local Z_PAUSED_UI = 70

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function registerResponse(self, isComplete, isCorrect)
  if (self.responseCallback ~= nil) then
    local assembledResponse = nil

    self.responseCallback(isComplete, isCorrect)
  end
end

local function createFirstRoundDirectionsAction(self)
  local directiveLabel = self.directiveLabel
  directiveLabel:setText("TAP THE", "CORRECT WORD", "PART")
  directiveLabel:prepareForAnimateIn()

  return directiveLabel:createAnimateInAction()
end

local function createPromotedDirectionsAction(self)
  local directiveLabel = self.directiveLabel
  directiveLabel:setText("ASSEMBLE", "THE", "WORD")
  directiveLabel:prepareForAnimateIn()

  directiveLabel:setY(Screen.getSafeY(0.8))

  return directiveLabel:createAnimateInAction()
end

local function createEndOfRoundResponseAction(self, additionalDelayForReadingResult, isTimeOut)
  local currentPartsStackLayer = self.currentPartsStackLayer
  local width, height = currentPartsStackLayer:getColumnsDimensions()
  local textLabels = currentPartsStackLayer:getCorrectTextLabels()

  if (isTimeOut) then
    currentPartsStackLayer:highlightCorrectText()
  else
    width = width * currentPartsStackLayer:getScl()
  end

  local isShrunken = (currentPartsStackLayer:getColumnsCount() >= RutabagaCommon.COLUMNS_COUNT_SHRUNKEN)
  local shouldShowCascadeTexture = (not isTimeOut)
  local correctOverlay = RutabagaPartsStackCorrectOverlay.new(width, height, textLabels, isShrunken, shouldShowCascadeTexture)
  correctOverlay:setAnchorPointBottom()
  correctOverlay:setX(Screen.getSafeX(0.5))
  self:insert(correctOverlay, Z_CORRECT_OVERLAY)

  local correctResponseAction = AHDelayed.new(0.44, AHSpawn.new(
    correctOverlay:createCorrectAction(additionalDelayForReadingResult),
    AHDelayed.new(0.3, AHRemove.new(currentPartsStackLayer)),
    AHDelayed.new(0.2 + additionalDelayForReadingResult, self.prompt:createExitAction())
  ))

  return correctResponseAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function RutabagaScene:createShowAnswerForTimeOutAction()
  local additionalDelayForReadingResult = 1
  local isTimeOut = true
  return createEndOfRoundResponseAction(self, additionalDelayForReadingResult, isTimeOut)
end

function RutabagaScene:getAssembledResponse()
  local assembledResponse = nil
  if (self.currentPartsStackLayer ~= nil) then
    assembledResponse = self.currentPartsStackLayer:getAssembledResponse()
  end

  return assembledResponse
end

function RutabagaScene:createIncorrectResponseAction()
  local currentPartsStackLayer = self.currentPartsStackLayer

  local incorrectResponseAction = AHSpawn.new(
    AHScaleBy.new(currentPartsStackLayer, 0.4, AHEaseSimpleInOut.new(5.0), -RATIONALE_SCALE_BY),
    self.pausedOverlay:createFadeInAction(),
    AHDelayed.new(0.1, AHSpawn.new(
      AHOpacityTo.new(self.tapToContinueLabel, 0.1, nil, 0.55),
      AHOpacityTo.new(self.pausedBacking, 0.1, nil, 1.0),
      AHOpacityTo.new(self.glowOverlay, 0.1, nil, 0.0),
      self.prompt:createRationaleFadeOutAction()
    )),
    AHSequence.new(
      function()
        local partsAndDefinitions = currentPartsStackLayer:getIncorrectPartsAndDefinitions()
        self.definitionsList:initializeWithPartsAndDefinitions(partsAndDefinitions)
        self.definitionsList:prepareToPresent()
      end,
      self.definitionsList:createPresentAction()
    )
  )

  return incorrectResponseAction
end

function RutabagaScene:createCorrectResponseAction(additionalDelayForReadingResult)
  local isTimeOut = false
  return createEndOfRoundResponseAction(self, additionalDelayForReadingResult, isTimeOut)
end

function RutabagaScene:createRationaleDismissAction()
  local currentPartsStackLayer = self.currentPartsStackLayer
  local rationaleDismissAction = AHSpawn.new(
    self.definitionsList:createDismissAction(),
    AHOpacityTo.new(self.tapToContinueLabel, 0.1, nil, 0.0),
    AHOpacityTo.new(self.pausedBacking, 0.1, nil, 0.0),
    self.prompt:createRationaleFadeInAction(),
    self.pausedOverlay:createFadeOutAction(),
    currentPartsStackLayer:createRationaleDismissAction(),
    AHScaleBy.new(currentPartsStackLayer, 0.4, AHEaseSimpleInOut.new(5.0), RATIONALE_SCALE_BY),
    AHOpacityTo.new(self.glowOverlay, 0.1, nil, 0.6)
  )
  return rationaleDismissAction
end

function RutabagaScene:presentRound(round, isFirstRound, didRoundProgressionPromote, isFirstPromotion, onStartCallback, doesRequireShrink)
  local formattedInstructions = string.upper(round.instructionsText)
  local roundPrompt = RutabagaPrompt.newHidden(formattedInstructions, round.wordDefinition)
  self:insertAtCenter(roundPrompt, Z_UI)
  self.prompt = roundPrompt

  local partsStackLayerScale = nil
  if (doesRequireShrink and not didRoundProgressionPromote) then
    partsStackLayerScale = RutabagaCommon.SHRUNKEN_SCALE
  else
    partsStackLayerScale = 1.0
  end

  local newPartsStackLayer = RutabagaPartsStackLayer.newWithPartsDescriptors(round.parts)
  local stackLayerTopY = RutabagaCommon.COLUMN_HEIGHT
  newPartsStackLayer:setAnchorPointTop()
  newPartsStackLayer:setScl(partsStackLayerScale)
  newPartsStackLayer:setLoc(self:getWidthPart(0.5), stackLayerTopY)
  self:insert(newPartsStackLayer, Z_PARTS_STACKS)
  self.currentPartsStackLayer = newPartsStackLayer
  newPartsStackLayer:initializeColumnsHidden(didRoundProgressionPromote)

  local promptDelayDuration = 0.5
  local stackDelayDuration = 0.0

  if (isFirstRound) then
    stackDelayDuration = 1.7
    promptDelayDuration = 2.5
  elseif (isFirstPromotion) then
    stackDelayDuration = 0.2
    promptDelayDuration = 2.0
  end

  local presentAction = AHSpawn.new(
    AHDelayed.new(promptDelayDuration, roundPrompt:createPresentAction(didRoundProgressionPromote)),
    AHDelayed.new(stackDelayDuration, newPartsStackLayer:createPresentAction(didRoundProgressionPromote, onStartCallback))
  )

  local willAnimateShrink = (doesRequireShrink and didRoundProgressionPromote)

  if (isFirstRound) then
    local firstRoundDirectionsAction = createFirstRoundDirectionsAction(self)
    presentAction:addAction(firstRoundDirectionsAction)
  elseif (isFirstPromotion) then
    local promotedDirectionsAction = createPromotedDirectionsAction(self)
    presentAction:addAction(promotedDirectionsAction)
  elseif (willAnimateShrink) then
    presentAction:addAction(
      AHDelayed.new(1.52, AHScaleTo.new(newPartsStackLayer, 0.8, AHEaseElasticOut.new(1.0), RutabagaCommon.SHRUNKEN_SCALE))
    )
  end

  local responseWrapper = function(isComplete, isCorrect)
    registerResponse(self, isComplete, isCorrect)
  end

  newPartsStackLayer:setResponseCallback(responseWrapper)

  return presentAction:start()
end

function RutabagaScene:setResponseCallback(callback)
  self.responseCallback = callback
end

function RutabagaScene:prepareForAndCreateOutro()
  self.currentPartsStackLayer:disallowAllInput(self)
  self.currentPartsStackLayer:setColumnsTouchEnabled(false)

  local hideAction = AHSpawn.new(
    AHLocationYBy.new(self.currentPartsStackLayer, 0.5, AHEaseExponentialIn.new(), -Screen.fullHeight),
    self.prompt:createExitForOutroAction()
  )

  return hideAction
end

function RutabagaScene:showHighLevelInterstitial()
  local interstitial = RutabagaHighLevelInterstitial.new()
  interstitial:prepareForAnimateIn()

  local tapToContinue = TapToContinue.new(interstitial)
  wait(interstitial:doAnimateIn())
  tapToContinue:waitForTrigger()
  wait(interstitial:doAnimateOutAndRemove())
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local pausedBacking = FilledRectangle.newFullScreen(PAUSED_BACKING_COLOR)
  self:insertAtCenter(pausedBacking, Z_PAUSED_BACKING)
  pausedBacking:setOpacity(0.0)
  self.pausedBacking = pausedBacking

  local glowOverlay = Group.newFullScreen()
  self:insertAtCenter(glowOverlay, Z_GLOW_OVERLAY)
  self.glowOverlay = glowOverlay

  local glow1 = Sprite.newWithName("glow.png")
  glow1:setColor(unpack(GLOW_TINT_COLOR))
  glow1:setBlendMode(MOAIProp2D.BLEND_ADD)
  glow1:setScl(6.0)
  glow1:setLoc(glowOverlay:getWidthPart(0.65), glowOverlay:getHeightPart(0.55))
  self.glowOverlay:insert(glow1)

  local glow2 = Sprite.newWithName("glow.png")
  glow2:setColor(unpack(GLOW_TINT_COLOR))
  glow2:setBlendMode(MOAIProp2D.BLEND_ADD)
  glow2:setScl(6.0)
  glow2:setLoc(glowOverlay:getWidthPart(0.3), glowOverlay:getHeightPart(0.4))
  self.glowOverlay:insert(glow2)

  local isHorizontal = false
  local beginColor = {0.36, 0.01, 0.0, 0.0}
  local endColor = {0.36, 0.01, 0.0, 0.75}
  local overlayGradient = RectangleGradient.new(Screen.fullWidth, Screen.getSafeY(0.18), beginColor, endColor, isHorizontal)
  overlayGradient:setAnchorPointBottom()
  overlayGradient:setX(Screen.getWidthPart(0.5))
  self.glowOverlay:insert(overlayGradient)

  local isRecursive = true
  self.glowOverlay:linkOpacityAttrToChildren(isRecursive)
  self.glowOverlay:setOpacity(0.6)

  local pausedOverlay = RutabagaPausedOverlay.new()
  self:insertAtCenter(pausedOverlay, Z_PAUSED_OVERLAY)
  self.pausedOverlay = pausedOverlay

  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_BOLD, 15))
  tapToContinueLabel:setColor(unpack({1.0, 0.64, 0.47}))
  tapToContinueLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.07))
  tapToContinueLabel:setOpacity(0.0)
  self:insert(tapToContinueLabel, Z_PAUSED_UI)
  self.tapToContinueLabel = tapToContinueLabel

  local definitionsList = RutabagaDefinitionsList.new()
  definitionsList:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.22))
  self:insert(definitionsList, Z_PAUSED_UI)
  self.definitionsList = definitionsList

  self.currentPartsStackLayer = nil
  self.nextPartsStackLayer = nil

  local directiveLabel = RutabagaDirectiveLabel.new()
  self:insert(directiveLabel, Z_UI)
  directiveLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.6))
  self.directiveLabel = directiveLabel
end

function RutabagaScene.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, RutabagaScene)

  createLayout(self)

  local directiveLabel = RutabagaDirectiveLabel.new()
  self:insert(directiveLabel, Z_UI)
  directiveLabel:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.6))
  self.directiveLabel = directiveLabel

  return self
end
