--- SimonConfiguration
-- @author Created by Aaron Barrett on February 23, 2015.

class("SimonConfiguration")

require("difficulty.DifficultyCurves")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SimonConfiguration.DIFFICULTY_BUCKETS = {"Easy", "Medium", "Hard"}
SimonConfiguration.POINTS_PER_PERFECT_SET_BONUS = 200
SimonConfiguration.POINTS_PER_FULL_SPEED_BONUS = 200
SimonConfiguration.INITIAL_LIVES_COUNT = 5
SimonConfiguration.POINTS_PER_CORRECT_RESPONSE = 1000
SimonConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS_TYPE_HALF = 10.0
SimonConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS_TYPE_COMBINED = 20.0
SimonConfiguration.SPEED_BONUS_DECAY_START_PROPORTION = 0.25

local FULL_POINTS_PER_LIVES_BONUS = 5000

local SETS_TO_COMPLETE_COUNT = 5
local EXTRA_LIVES_NEEDED_RANK_TWO = 2
local EXTRA_LIVES_NEEDED_RANK_THREE = 4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getPassScore()
  return SETS_TO_COMPLETE_COUNT * SimonConfiguration.POINTS_PER_CORRECT_RESPONSE
end

local function getExtraLivesPointsRankTwo()
  return EXTRA_LIVES_NEEDED_RANK_TWO * SimonConfiguration.getPointsPerLivesBonus()
end

local function getExtraLivesPointsRankThree()
  return EXTRA_LIVES_NEEDED_RANK_THREE * SimonConfiguration.getPointsPerLivesBonus()
end

local function getPerfectSetBonusRankTwo()
  local perfectSetCount = EXTRA_LIVES_NEEDED_RANK_TWO + 1
  return (perfectSetCount * SimonConfiguration.POINTS_PER_PERFECT_SET_BONUS)
end

local function getPerfectSetBonusRankThree()
  local perfectSetCount = EXTRA_LIVES_NEEDED_RANK_THREE + 1
  return (perfectSetCount * SimonConfiguration.POINTS_PER_PERFECT_SET_BONUS)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonConfiguration.getPointsPerLivesBonus()
  local maxBonusLives = SimonConfiguration.INITIAL_LIVES_COUNT - 1
  return (FULL_POINTS_PER_LIVES_BONUS / maxBonusLives)
end

function SimonConfiguration.getRankTwoScore()
  local passScore = getPassScore()
  local extraLivesBonus = getExtraLivesPointsRankTwo()
  local perfectSetBonus = getPerfectSetBonusRankTwo()
  return passScore + extraLivesBonus + perfectSetBonus
end

function SimonConfiguration.getRankThreeScore()
  local passScore = getPassScore()
  local extraLivesBonus = getExtraLivesPointsRankThree()
  local perfectSetBonus = getPerfectSetBonusRankThree()
  return passScore + extraLivesBonus + perfectSetBonus
end

function SimonConfiguration.getDifficultyKeyFrames()
  local keyFrameCutoffs = {
    {endDifficulty = 0.225, contentDifficulty = SimonConfiguration.DIFFICULTY_BUCKETS[1]},
    {endDifficulty = 0.6, contentDifficulty = SimonConfiguration.DIFFICULTY_BUCKETS[2]},
    {endDifficulty = 1.0, contentDifficulty = SimonConfiguration.DIFFICULTY_BUCKETS[3]}
  }

  local keyFrames = DifficultyCurves.getHardCutoffKeyframes(keyFrameCutoffs)

  return keyFrames
end
