--- SimonContentManager
-- @author Created by Aaron Barrett on April 10, 2015.

class("SimonContentManager")

require("SimonConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonContentManager:getConcept()
  return self.concept
end

function SimonContentManager:getAudioRounds()
  return self.audioRounds
end

function SimonContentManager:getAllSteps()
  return self.allSteps
end

function SimonContentManager:getDescription()
  return self.description
end

function SimonContentManager:getCategory()
  return self.category
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildConcept(self)
  local conceptGenerator = GameSession.conceptGenerator
  local contentDifficulties = SimonConfiguration.DIFFICULTY_BUCKETS
  local difficultyKeyFrames = SimonConfiguration.getDifficultyKeyFrames()
  local difficultyNameChooser = KeyFrameInterpolator.new(difficultyKeyFrames, GameSession.getDifficulty())

  conceptGenerator:setDifficultyNameChooser(difficultyNameChooser)

  self.concept = conceptGenerator:next()
end

local function buildAllSteps(self)
  local concept = self.concept

  local rawSteps = concept["steps"]
  local allSteps = {}

  for _, rawStep in ipairs(rawSteps) do
    local step = rawStep["item"]
    table.insert(allSteps, step)
  end

  self.allSteps = allSteps
end

local function buildAudioRounds(self)
  local concept = self.concept
  local allSteps = self.allSteps

  local rounds = {}

  local stepsStartIndex = 1

  local clipsCount = 2

  for clipIndex = 1, clipsCount do
    local audioFilenameKey = "audio_filename_" .. clipIndex
    local stepsCountKey = "steps_count_" .. clipIndex

    local audioFilename = concept[audioFilenameKey]
    local roundStepsCount = concept[stepsCountKey]

    local stepsEndIndex = stepsStartIndex + roundStepsCount - 1
    local roundSteps = TableUtils.subarray(allSteps, stepsStartIndex, stepsEndIndex)

    local round = SimonRound.new(concept, roundSteps, audioFilename)
    table.insert(rounds, round)

    stepsStartIndex = stepsEndIndex + 1
  end

  self.audioRounds = rounds
end

local function buildDescription(self)
  local concept = self.concept
  self.description = concept["description"]
end

local function buildCategory(self)
  local concept = self.concept
  self.category = concept["category"]
end

-- Constructor ------------------------

function SimonContentManager.new()
  local self = {}
  Mixin.mix(self, SimonContentManager)

  buildConcept(self)
  buildAllSteps(self)
  buildAudioRounds(self)
  buildDescription(self)
  buildCategory(self)

  return self
end
