--- SimonCorrection
-- @author Created by Aaron Barrett on March 03, 2015.

class("SimonCorrection")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getDestinationIndex(items, itemsToMove)
  local firstItemToMove = itemsToMove[1]
  local soughtCorrectIndex = firstItemToMove:getCorrectIndex() - 1
  if (soughtCorrectIndex < 1) then
    return 1
  end

  for itemIndex, item in ipairs(items) do
    if (item:getCorrectIndex() == soughtCorrectIndex) then
      return itemIndex + 1
    end
  end

  error("Unable to find destination index.")
end

local function getItemsToMove(self, items)
  local startIndex = self.startIndex
  local length = self.length

  local itemsToMove = {}
  for itemIndex = startIndex, startIndex + length - 1 do
    local item = items[itemIndex]
    table.insert(itemsToMove, item)
  end

  return itemsToMove
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonCorrection:applyToItems(items)
  local itemsToMove = getItemsToMove(self, items)

  local startIndex = self.startIndex
  local length = self.length
  for i = 1, length do
    table.remove(items, startIndex)
  end

  local destinationIndex = getDestinationIndex(items, itemsToMove)

  for itemIndex = #itemsToMove, 1, -1 do
    local itemToMove = itemsToMove[itemIndex]
    table.insert(items, destinationIndex, itemToMove)
  end
end

-- Accessors --------------------------

function SimonCorrection:getStartIndex()
  return self.startIndex
end

function SimonCorrection:getLength()
  return self.length
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SimonCorrection.new(startIndex, length)
  local self = {}
  Mixin.mix(self, SimonCorrection)

  self.startIndex = startIndex
  self.length = length

  return self
end
