--- SimonCorrectionBuilder
-- @author Created by Aaron Barrett on March 09, 2015.

class("SimonCorrectionBuilder")

require("SimonCorrection")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getIndexOfItemWithCorrectIndex(items, correctIndex)
  for itemIndex, item in ipairs(items) do
    if (item:getCorrectIndex() == correctIndex) then
      return itemIndex
    end
  end
  error("Unable to find item with correctIndex `" .. tostring(correctIndex) .. "`")
end

local function getContiguousSubarrays(items)
  local itemsCount = #items

  local subarrays = {}
  local currentSubarray = {items[1]}

  for itemIndex = 2, itemsCount do
    local item = items[itemIndex]
    local previousItem = items[itemIndex - 1]

    local expected = previousItem:getCorrectIndex() + 1
    local actual = item:getCorrectIndex()

    local isContiguous = (expected == actual)
    if (isContiguous) then
      table.insert(currentSubarray, item)
    else
      table.insert(subarrays, currentSubarray)
      currentSubarray = {item}
    end
  end

  table.insert(subarrays, currentSubarray)

  return subarrays
end

local function getSubarrayToMove(subarrays, items)
  local selectedSubarray = nil
  local currentMinLength = math.huge

  for _, subarray in ipairs(subarrays) do
    local firstItem = subarray[1]
    local firstCorrectIndex = firstItem:getCorrectIndex()
    local actualIndex = getIndexOfItemWithCorrectIndex(items, firstCorrectIndex)
    if (actualIndex ~= firstCorrectIndex) then
      local length = #subarray
      if (length < currentMinLength) then
        selectedSubarray = subarray
        currentMinLength = length
      end
    end
  end

  return selectedSubarray
end

local function makeSureSubarrayMarksItemsWrong(subarray)
  for _, item in ipairs(subarray) do
    item:setShouldMarkWrong(true)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonCorrectionBuilder.createCorrections(itemsToCorrect)
  local items = TableUtils.shallowCopy(itemsToCorrect)

  local corrections = {}

  local done = false
  repeat
    local subarrays = getContiguousSubarrays(items)
    local subarraysCount = #subarrays
    if (subarraysCount == 1) then
      done = true
    else
      local subarrayToMove = getSubarrayToMove(subarrays, items)

      makeSureSubarrayMarksItemsWrong(subarrayToMove)

      local firstCorrectIndex = subarrayToMove[1]:getCorrectIndex()

      local correctionStartIndex = getIndexOfItemWithCorrectIndex(items, firstCorrectIndex)
      local correctionLength = #subarrayToMove
      local correction = SimonCorrection.new(correctionStartIndex, correctionLength)
      table.insert(corrections, correction)

      correction:applyToItems(items)
    end
  until (done)

  return corrections
end
