--- SimonItem
-- @author Created by Aaron Barrett on February 24, 2015.

class("SimonItem")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Dimensions -------------------------

SimonItem.WIDTH = Screen.getWidthPart(0.88)

-- Colors -----------------------------

local COLOR_BACKGROUND_SELECTED = {1.0, 1.0, 1.0}
local COLOR_LABEL_NORMAL = {1.0, 1.0, 1.0}

-- Z-Indices --------------------------

local Z_BACKGROUND = 10
local Z_LABEL = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playSelection(self, destinationScale, destinationBackgroundColor, destinationLabelColor, zIndex, scaleEase)
  self:setZIndex(zIndex)

  local background = self.background
  local label = self.label
  local selectionInterrupt = self.selectionInterrupt

  local selectionAction = AHSpawn.new(
                                      AHScaleTo.new(self, 0.2, scaleEase, destinationScale),
                                      AHColorTo.new(label, 0.1, nil, destinationLabelColor)
                                      )
  if (destinationBackgroundColor ~= nil) then
    local colorAction = AHColorTo.new(background, 0.1, nil, destinationBackgroundColor)
    selectionAction:addAction(colorAction)
  end

  selectionInterrupt:insert(selectionAction)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonItem:createFloppyFlyInAction(middleScale, overshootY, destinationY, destinationIndex)
  local background = self.background
  local label = self.label

  local destinationBackgroundColor = self.palette:getNormalColorForIndex(destinationIndex)

  local duration = 1.1

  local flyInAction = AHSpawn.new(
                                  AHColorTo.new(background, duration, nil, destinationBackgroundColor),
                                  AHOpacityTo.new(background, duration, nil, 1.0),
                                  AHOpacityTo.new(label, duration, AHEaseSimpleIn.new(3.0), 1.0),
                                  AHSequence.new(
                                                 AHSpawn.new(
                                                             AHScaleTo.new(self, duration * 0.3, AHEaseSimpleOut.new(4.0), 1.0),
                                                             AHLocationYTo.new(self, duration * 0.3, AHEaseSimpleOut.new(2.0), overshootY)
                                                             ),
                                                 AHSpawn.new(
                                                             AHSequence.new(
                                                                            AHScaleTo.new(self,duration * 0.2, AHEaseSimpleIn.new(4.0), middleScale),
                                                                            AHScaleTo.new(self, duration * 0.2, AHEaseSimpleOut.new(4.0), 1.0)
                                                                            ),
                                                             AHLocationYTo.new(self, duration * 0.4, AHEaseSimpleInOut.new(2.0), destinationY)
                                                             )
                                                 )
                                  )

  return flyInAction
end

function SimonItem:createSteadyFlyInAction(duration, destinationY, destinationIndex)
  local background = self.background
  local label = self.label

  local destinationBackgroundColor = self.palette:getNormalColorForIndex(destinationIndex)

  local flyInAction = AHSpawn.new(
                                  AHColorTo.new(background, duration, nil, destinationBackgroundColor),
                                  AHOpacityTo.new(background, duration, nil, 1.0),
                                  AHOpacityTo.new(label, duration, AHEaseSimpleIn.new(3.0), 1.0),
                                  AHLocationYTo.new(self, duration, AHEaseExponentialOut.new(), destinationY),
                                  AHScaleTo.new(self, duration, AHEaseSimpleOut.new(1.5 + math.random() * 2.0), 1.0)
                                  )

  return flyInAction
end

function SimonItem:createArrangeAction(destinationX, destinationY, stepIndex)
  local background = self.background

  local destinationColor = self.palette:getNormalColorForIndex(stepIndex)

  local duration = 0.2
  local locationEase = AHEaseSimpleOut.new(2.0)

  local arrangeAction = AHSpawn.new(
                                    AHLocationXTo.new(self, duration, locationEase, destinationX),
                                    AHLocationYTo.new(self, duration, locationEase, destinationY),
                                    AHColorTo.new(self.background, duration, nil, destinationColor)
                                    )

  return arrangeAction
end

function SimonItem:applyColorForIndex(stepIndex)
  local background = self.background
  local color = self.palette:getNormalColorForIndex(stepIndex)
  background:setColor(unpack(color))
end

function SimonItem:getText()
  return self.text
end

function SimonItem:getCorrectIndex()
  return self.correctIndex
end

function SimonItem:setShouldMarkWrong(shouldMarkWrong)
  self.shouldMarkWrong = shouldMarkWrong
end

function SimonItem:createMarkWrongAction()
  local background = self.background
  local label = self.label

  local markAction = AHSpawn.new(
                                 AHOpacityTo.new(background, 0.4, nil, 0.6),
                                 AHOpacityTo.new(label, 0.4, nil, 0.25),
                                 AHScaleTo.new(self, 0.4, nil, 0.9),
                                 AHSequence.new(
                                                AHDelay.new(0.3),
                                                AHLocationXBy.new(self, 0.15, AHEaseSimpleInOut.new(2.0), self:getWidthPart(0.075)),
                                                AHLocationXBy.new(self, 0.15, AHEaseSimpleInOut.new(2.0), -self:getWidthPart(0.15)),
                                                AHLocationXBy.new(self, 0.15, AHEaseSimpleInOut.new(2.0), self:getWidthPart(0.081)),
                                                AHLocationXBy.new(self, 0.15, AHEaseSimpleInOut.new(2.0), -self:getWidthPart(0.006))
                                                )
                                 )
  return markAction
end

function SimonItem:createMarkCorrectAction()
  local background = self.background
  local label = self.label
  local palette = self.palette
  local correctIndex = self.correctIndex

  local backgroundCorrectColor = palette:getCorrectColorForIndex(correctIndex)
  local labelCorrectColor = palette:getCorrectTextColor()

  local markAction = AHSpawn.new(
                                 AHColorTo.new(background, 0.4, nil, backgroundCorrectColor),
                                 AHColorTo.new(label, 0.4, nil, labelCorrectColor),
                                 AHSequence.new(
                                                AHScaleTo.new(self, 0.2, AHEaseSimpleIn.new(2.0), 1.1),
                                                AHScaleTo.new(self, 0.2, AHEaseSimpleOut.new(2.0), 1.0)
                                                ),
                                 AHSequence.new(
                                                AHLocationYBy.new(self, 0.2, AHEaseBackOut.new(1.1), -self:getHeightPart(0.15)),
                                                AHLocationYBy.new(self, 0.2, AHEaseBackOut.new(1.1), self:getHeightPart(0.15))
                                                )
                                 )
  return markAction
end

function SimonItem:createCorrectionAction(destinationY)
  local correctionAction = AHSpawn.new(
                                       AHLocationYTo.new(self, 0.6, AHEaseSimpleInOut.new(2.0), destinationY)
                                       )

  if (self.shouldMarkWrong) then
    local background = self.background
    local label = self.label

    local palette = self.palette
    local correctIndex = self.correctIndex

    local backgroundCorrectColor = palette:getCorrectColorForIndex(correctIndex)
    local labelCorrectColor = palette:getCorrectTextColor()

    local scaleAction = AHSequence.new(
                                       AHScaleTo.new(self, 0.36, AHEaseSineIn.new(), 1.08),
                                       AHScaleTo.new(self, 0.24, AHEaseSineOut.new(), 1.0)
                                       )
    correctionAction:addAction(scaleAction)

    local backgroundColorAction = AHColorTo.new(background, 0.6, nil, backgroundCorrectColor)
    correctionAction:addAction(backgroundColorAction)

    local labelColorAction = AHColorTo.new(label, 0.6, nil, labelCorrectColor)
    correctionAction:addAction(labelColorAction)

    local labelOpacityAction = AHOpacityTo.new(label, 0.6, nil, 1.0)
    correctionAction:addAction(labelOpacityAction)
  end

  return correctionAction
end

function SimonItem:createMarkAction()
  if (self.shouldMarkWrong) then
    return self:createMarkWrongAction()
  else
    return self:createMarkCorrectAction()
  end
end

function SimonItem:getShouldMarkWrong()
  return self.shouldMarkWrong
end

function SimonItem:createQuickFadeOutAction()
  local background = self.background
  local label = self.label
  local quickFadeOutAction = AHSpawn.new(
                                         AHOpacityTo.new(background, 0.2, nil, 0.0),
                                         AHOpacityTo.new(label, 0.2, nil, 0.0)
                                         )
  return quickFadeOutAction
end

function SimonItem:getLeftEdgeX()
  local zoomer = self.zoomer
  local zoomerScale = zoomer:getScl()
  local scale = self:getScl()
  local x = self:getX()
  return x - self:getWidthPart(0.5) * scale * zoomerScale
end

--- Drag And Drop Object --------------

-- Called when the object is picked up.
function SimonItem:notifyDNDPickUp()
  Sound.playSoundEffect("pick_up")

  local destinationScale = 1.04
  local destinationBackgroundColor = COLOR_BACKGROUND_SELECTED
  local destinationLabelColor = self.palette:getCorrectTextColor()
  local zIndex = SimonPanelOrdering.Z_ITEM_SELECTED
  local scaleEase = AHEaseBackOut.new(1.8)
  playSelection(self, destinationScale, destinationBackgroundColor, destinationLabelColor, zIndex, scaleEase)

  self.onPickedUpCallback(self)
end

-- Called when the object is dropped. container will only be present if it was dropped inside.
function SimonItem:notifyDNDDrop(container)
  Sound.playSoundEffect("drop")

  local destinationScale = 1.0
  local destinationBackgroundColor = nil
  local destinationLabelColor = COLOR_LABEL_NORMAL
  local zIndex = SimonPanelOrdering.Z_ITEM
  local scaleEase = AHEaseBackIn.new(1.8)
  playSelection(self, destinationScale, destinationBackgroundColor, destinationLabelColor, zIndex, scaleEase)

  self.onDroppedCallback(self, container)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors -----------------------

function SimonItem.new(height, text, correctIndex, onPickedUpCallback, onDroppedCallback, palette, zoomer)
  local self = Group.new(SimonItem.WIDTH, height)
  Mixin.mix(self, SimonItem)

  local backgroundColor = palette:getDefaultItemColor()
  local background = FilledRectangle.new(SimonItem.WIDTH, height, backgroundColor)
  background:setOpacity(0.0)
  self:insertAtCenter(background, Z_BACKGROUND)
  self.background = background

  local label = Label.new(text, Font.new(FontFaces.DIN_OT_MEDIUM, 16.0))
  label:setOpacity(0.0)
  self:insertAtCenter(label, Z_LABEL)
  self.label = label

  self.correctIndex = correctIndex
  self.onPickedUpCallback = onPickedUpCallback
  self.onDroppedCallback = onDroppedCallback
  self.palette = palette
  self.selectionInterrupt = ActionInterrupt.new()
  self.text = text
  self.indexToInsert = nil
  self.shouldMarkWrong = false
  self.zoomer = zoomer

  return self
end
