--- SimonItemPalette
-- @author Created by Aaron Barrett on March 31, 2015.

class("SimonItemPalette")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonItemPalette:getDefaultItemColor()
  return self.normalColors[1]
end

function SimonItemPalette:getNormalColorForIndex(index)
  return self.normalColors[index]
end

function SimonItemPalette:getCorrectColorForIndex(index)
  return self.correctColors[index]
end

function SimonItemPalette:getCorrectTextColor()
  return self.correctTextColor
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SimonItemPalette.new(theme, itemsCount)
  local self = {}
  Mixin.mix(self, SimonItemPalette)

  local normalColors = {}
  local correctColors = {}

  local normalTopColor = theme:getItemTopColor()
  local normalBottomColor = theme:getItemBottomColor()
  local correctTopColor = theme:getItemCorrectTopColor()
  local correctBottomColor = theme:getItemCorrectBottomColor()

  for itemIndex = 1, itemsCount do
    local blendAmount = (itemIndex - 1) / (itemsCount - 1)

    local normalColor = ColorUtils.makeColorBetweenTwoColors(normalTopColor, normalBottomColor, blendAmount)
    table.insert(normalColors, normalColor)

    local correctColor = ColorUtils.makeColorBetweenTwoColors(correctTopColor, correctBottomColor, blendAmount)
    table.insert(correctColors, correctColor)
  end

  self.normalColors = normalColors
  self.correctColors = correctColors

  self.correctTextColor = theme:getItemCorrectTextColor()

  return self
end
