--- SimonOutro
-- @author Created by Aaron Barrett on March 27, 2015.

class("SimonOutro")

require("Particles")
require("SimonOutroCheckmark")
require("SimonRectangleSpewer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_REFLECTION_CHECKMARK = 10
local Z_SPEWER = 20
local Z_PARTICLES = 30
local Z_MAIN_CHECKMARK = 40

local SPEWER_MARGIN = 26.0

local CHECKMARK_SIDES_COUNT = 4

local PARTICLES_SOURCE_DISTANCE = Screen.getWidthPart(0.06)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function startParticles(self)
  local particleSystems = self.particleSystems
  for _, particles in ipairs(particleSystems) do
    particles:surge(32)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonOutro:playOutro()
  return runInCoroutine(function()
      local mainCheckmark = self.mainCheckmark
      local reflectionCheckmark = self.reflectionCheckmark
      local spewer = self.spewer

      mainCheckmark:playSort()
      wait(reflectionCheckmark:playSort())
      spewer:playSpew()
      startParticles(self)

      waitFor(0.9)

      reflectionCheckmark:playFadeOut()

      mainCheckmark:playFall()
      wait(reflectionCheckmark:playFall())
    end)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createParticles(width, height, distance, index)
  local filename = "outro_particles.pex"
  local particles = Particles.new(filename)

  local angle = math.pi * 0.5 * index
  local sine = math.sin(angle)
  local cosine = math.cos(angle)
  local sourceX = sine * distance + cosine * height
  local sourceY = cosine * distance + sine * height
  local sourceVarianceX = cosine * width + sine * height
  local sourceVarianceY = sine * width + cosine * height

  particles:setAngle(math.deg(-angle + math.pi * 0.5))
  particles:setSourcePosition(sourceX, sourceY)
  particles:setSourcePositionVariance(sourceVarianceX, sourceVarianceY)

  particles:start()
  return particles
end

-- Constructor ------------------------

function SimonOutro.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, SimonOutro)

  local mainCheckmark = SimonOutroCheckmark.new()
  self:insertAtCenter(mainCheckmark, Z_MAIN_CHECKMARK)
  self.mainCheckmark = mainCheckmark

  local reflectionCheckmark = SimonOutroCheckmark.new()
  reflectionCheckmark:setScl(1.0, -1.0)
  reflectionCheckmark:setOpacity(0.1)
  reflectionCheckmark:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.1))
  self:insert(reflectionCheckmark, Z_REFLECTION_CHECKMARK)
  self.reflectionCheckmark = reflectionCheckmark

  local checkmarkWidth, checkmarkHeight = mainCheckmark:getDims()
  local spewer = SimonRectangleSpewer.new(checkmarkWidth + SPEWER_MARGIN, checkmarkHeight + SPEWER_MARGIN, SimonRectangleSpewer.KIND_OUTRO)
  self:insertAtCenter(spewer, Z_SPEWER)
  self.spewer = spewer

  local particleSystems = {}
  for sideIndex = 1, CHECKMARK_SIDES_COUNT do
    local particles = createParticles(checkmarkWidth * 0.5, PARTICLES_SOURCE_DISTANCE, 0.5 * checkmarkHeight + PARTICLES_SOURCE_DISTANCE, sideIndex)
    self:insertAtCenter(particles, Z_PARTICLES)
    table.insert(particleSystems, particles)
  end
  self.particleSystems = particleSystems

  return self
end
