--- SimonOutroCheckmark
-- @author Created by Aaron Barrett on March 27, 2015.

class("SimonOutroCheckmark")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PIECES_COUNT = 4

local SPACING = 6.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonOutroCheckmark:playSort()
  local pieces = self.pieces

  local staggerDuration = 0.07
  local sortAction = AHStaggeredSpawn.new(staggerDuration)

  for _, piece in ipairs(pieces) do
    local pieceAction = AHSequence.new(
                                       AHLocationYTo.new(piece, 1.3, AHEaseSineInOut.new(), piece.firstStopY),
                                       AHLocationYTo.new(piece, 0.8, AHEaseSineInOut.new(), piece.destinationY)
                                       )
    sortAction:addAction(pieceAction)
  end

  return sortAction:start()
end

function SimonOutroCheckmark:playFadeOut()
  local fadeOutAction = AHOpacityTo.new(self, 0.3, nil, 0.0)
  return fadeOutAction:start()
end

function SimonOutroCheckmark:playFall()
  local pieces = self.pieces

  local fallDistance = -Screen.height

  local staggerDuration = 0.03

  local fallAction = AHStaggeredSpawn.new(staggerDuration,
                                          AHLocationYBy.new(pieces[3], 0.8, AHEaseSineIn.new(), fallDistance),
                                          AHLocationYBy.new(pieces[1], 0.8, AHEaseSineIn.new(), fallDistance),
                                          AHLocationYBy.new(pieces[2], 0.8, AHEaseSineIn.new(), fallDistance),
                                          AHSequence.new(
                                                         AHLocationYBy.new(pieces[4], 0.3, AHEaseSineInOut.new(), Screen.getHeightPart(0.06)),
                                                         AHLocationYBy.new(pieces[4], 0.8, AHEaseSineIn.new(), fallDistance)
                                                         )
                                          )
  return fallAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SimonOutroCheckmark.new()
  local self = Column.new(SPACING)
  Mixin.mix(self, SimonOutroCheckmark)

  local pieces = {}

  for pieceIndex = 1, PIECES_COUNT do
    local filename = "checkmark_" .. pieceIndex .. ".png"
    local piece = Sprite.newWithName(filename)
    piece:inheritOpacity(self)
    self:insert(piece)
    table.insert(pieces, piece)
  end

  self:alignChildren()

  for _, piece in ipairs(pieces) do
    piece.destinationY = piece:getY()
  end

  pieces[1].firstStopY = pieces[4].destinationY
  pieces[2].firstStopY = pieces[3].destinationY
  pieces[3].firstStopY = pieces[1].destinationY
  pieces[4].firstStopY = pieces[2].destinationY

  for _, piece in ipairs(pieces) do
    local startingY = piece.firstStopY + Screen.height
    piece:setY(startingY)
  end

  self.pieces = pieces

  return self
end
