--- SimonPanelBegin
-- @author Created by Aaron Barrett on March 26, 2015.

class("SimonPanelBegin")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SLIDE_IN_DISTANCE = Screen.getWidthPart(0.25)
local SLIDE_OUT_DISTANCE = -Screen.getHeightPart(0.05)

local Z_COLUMN = 10

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function enableDismissal(self)
  local submitButton = self.submitButton
  local onDismissCallback = self.onDismissCallback
  local moreInfoButton = self.moreInfoButton
  submitButton:setCallback(onDismissCallback)
  self:setTouchEnabled(true)
  moreInfoButton:setTouchEnabled(true)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonPanelBegin:playAppear()
  local column = self.column

  column:setLoc(self:getWidthPart(0.5) - SLIDE_IN_DISTANCE, self:getHeightPart(0.55))
  self.panelChoice = nil
  self.dismissed = false

  local appearAction = AHSequence.new(
                                      AHSpawn.new(
                                                  AHLocationXBy.new(column, 0.5, AHEaseExponentialOut.new(), SLIDE_IN_DISTANCE),
                                                  AHOpacityTo.new(column, 0.5, nil, 1.0)
                                                  ),
                                      function()
                                        enableDismissal(self)
                                      end
                                      )
  return appearAction:start()
end

function SimonPanelBegin:playDisappear()
  local column = self.column

  local disappearAction = AHSpawn.new(
                                      AHLocationYBy.new(column, 0.5, AHEaseSimpleIn.new(4.0), SLIDE_OUT_DISTANCE),
                                      AHOpacityTo.new(column, 0.5, nil, 0.0)
                                      )
  return disappearAction:start()
end

function SimonPanelBegin:isDismissed()
  return self.dismissed
end

function SimonPanelBegin:getPanelChoice()
  return self.panelChoice
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function configureDismissCallback(self)
  local submitButton = self.submitButton
  local function onDismissCallback()
    submitButton:playPressed()
    self.panelChoice = SimonGame.PANEL_CHOICE_START_GAME
    self:setTouchEnabled(false)
    self.moreInfoButton:setTouchEnabled(false)
    self.dismissed = true
  end
  self.onDismissCallback = onDismissCallback
  Button.makeButton(self, onDismissCallback)
  self:setTouchEnabled(false)
end

local function createParagraphLabel(column)
  local text = "Listen to the audio and then correctly order the items by putting the first stated item at the top."
  local width = Screen.getWidthPart(0.87)
  local fontFace = FontFaces.DIN_OT_MEDIUM
  local fontSize = 18
  local attributesForLabels = nil
  local horizontalAlignment = Alignment.CENTER
  local label = SimonTextLabel.new(text, width, fontFace, fontSize, attributesForLabels, horizontalAlignment)
  label:inheritOpacity(column)
  return label
end

local function createMoreInfoLabel(self, column, theme)
  local color = theme:getGrayedTextColor()

  local moreInfoLabel = Label.new("MORE INFO", Font.new(FontFaces.DIN_OT_MEDIUM, 13.0))
  moreInfoLabel:setColor(unpack(color))
  local underline = FilledRectangle.new(moreInfoLabel:getWidth(), 1.5, color)
  underline:setLoc(moreInfoLabel:getWidthPart(0.5), 0.75)
  moreInfoLabel:insert(underline)
  moreInfoLabel:inheritOpacity(column)
  underline:inheritOpacity(column)

  return moreInfoLabel
end

local function createMoreInfoButton(self, moreInfoLabel)
  local function onMoreInfoCallback()
    self.panelChoice = SimonGame.PANEL_CHOICE_MORE_INFO
    self:setTouchEnabled(false)
    self.moreInfoButton:setTouchEnabled(false)
    self.dismissed = true
  end

  local expandedWidth = moreInfoLabel:getWidth() + 20
  local expandedHeight = moreInfoLabel:getHeight() + 50
  local moreInfoButton = Button.addVirtualButton(moreInfoLabel, expandedWidth, expandedHeight, onMoreInfoCallback)

  return moreInfoButton
end

-- Constructor ------------------------

function SimonPanelBegin.new(theme, submitButton)
  local self = Group.newFullScreen()
  Mixin.mix(self, SimonPanelBegin)

  self.submitButton = submitButton

  configureDismissCallback(self)

  local spacing = 2.0
  local column = Column.new(spacing)
  local paragraphLabel = createParagraphLabel(column)
  column:insert(paragraphLabel)
  local moreInfoLabel = createMoreInfoLabel(self, column, theme)
  self.moreInfoLabel = moreInfoLabel
  local moreInfoButton = createMoreInfoButton(self, moreInfoLabel)
  self.moreInfoButton = moreInfoButton
  column:insert(moreInfoLabel)
  column:alignChildren()
  column:setOpacity(0.0)
  self:insert(column, Z_COLUMN)
  self.column = column

  self.panelChoice = nil
  self.dismissed = false

  return self
end
