--- SimonPanelIntermission
-- @author Created by Aaron Barrett on March 09, 2015.

class("SimonPanelIntermission")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SLIDE_IN_DISTANCE = Screen.getWidthPart(0.25)
local SLIDE_OUT_DISTANCE = -Screen.getHeightPart(0.05)

local Z_LABEL = 10

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonPanelIntermission:playAppear()
  local label = self.label

  local appearAction = AHSequence.new(
                                      AHSpawn.new(
                                                  AHLocationXBy.new(label, 0.5, AHEaseExponentialOut.new(), SLIDE_IN_DISTANCE),
                                                  AHOpacityTo.new(label, 0.5, nil, 1.0)
                                                  ),
                                      function()
                                        self:setTouchEnabled(true)
                                      end
                                      )
  return appearAction:start()
end

function SimonPanelIntermission:playDisappear()
  local label = self.label

  local disappearAction = AHSpawn.new(
                                      AHLocationYBy.new(label, 0.5, AHEaseSimpleIn.new(4.0), SLIDE_OUT_DISTANCE),
                                      AHOpacityTo.new(label, 0.5, nil, 0.0)
                                      )
  return disappearAction:start()
end

function SimonPanelIntermission:isDismissed()
  return self.dismissed
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function configureSubmitButton(self, submitButton)
  local function onDismissCallback()
    submitButton:playPressed()
    self:setTouchEnabled(false)
    self.dismissed = true
  end
  submitButton:setCallback(onDismissCallback)
  Button.makeButton(self, onDismissCallback)
  self:setTouchEnabled(false)
end

local function buildParagraphLabel(self, text)
  local width = Screen.getWidthPart(0.8)
  local fontFace = FontFaces.DIN_OT_MEDIUM
  local fontSize = 18
  local attributesForLabels = nil
  local horizontalAlignment = Alignment.LEFT
  local label = SimonTextLabel.new(text, width, fontFace, fontSize, attributesForLabels, horizontalAlignment)
  label:setLoc(self:getWidthPart(0.5) - SLIDE_IN_DISTANCE, self:getHeightPart(0.55))
  label:setOpacity(0.0)
  self:insert(label, Z_LABEL)
  self.label = label
end

-- Constructor ------------------------

function SimonPanelIntermission.new(text, submitButton)
  local self = Group.newFullScreen()
  Mixin.mix(self, SimonPanelIntermission)

  configureSubmitButton(self, submitButton)

  buildParagraphLabel(self, text)

  self.dismissed = false

  return self
end
