--- SimonRectangleSpewer
-- @author Created by Aaron Barrett on March 24, 2015.

class("SimonRectangleSpewer")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SimonRectangleSpewer.KIND_PERFECT_ANSWER = "SimonRectangleSpewer.KIND_PERFECT_ANSWER"
SimonRectangleSpewer.KIND_OUTRO = "SimonRectangleSpewer.KIND_OUTRO"

local RECTANGLE_DESCRIPTORS_PERFECT_ANSWER = {
  {thickness = 20.0, expansionAmountPart = 0.10, staggerDuration = 0.0},
  {thickness = 1.0, expansionAmountPart = 0.10, staggerDuration = 0.31},
  {thickness = 1.0, expansionAmountPart = 0.10, staggerDuration = 0.31}
}

local RECTANGLE_DESCRIPTORS_OUTRO = {
  {thickness = 14.0, expansionAmountPart = 0.12, staggerDuration = 0.0},
  {thickness = 0.5, expansionAmountPart = 0.30, staggerDuration = 0.0},
  {thickness = 0.25, expansionAmountPart = 0.50, staggerDuration = 0.0},
  {thickness = 0.5, expansionAmountPart = 0.25, staggerDuration = 0.1},
  {thickness = 0.5, expansionAmountPart = 0.45, staggerDuration = 0.2},
  {thickness = 0.25, expansionAmountPart = 0.25, staggerDuration = 0.2}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonRectangleSpewer:playSpew()
  local rectangles = self.rectangles
  local descriptors = self.descriptors
  local staggerDuration = self.staggerDuration

  local spewAction = AHSpawn.new()

  local width, height = self:getDims()

  local accumulatedStaggerDuration = 0.0

  for rectangleIndex, rectangle in ipairs(rectangles) do
    local descriptor = descriptors[rectangleIndex]

    local expansionAmountPart = descriptor.expansionAmountPart
    local expansionAmount = Screen.getWidthPart(expansionAmountPart)

    local destinationXScale = (width + expansionAmount) / width
    local destinationYScale = (height + expansionAmount) / height

    local staggerDuration = descriptor.staggerDuration
    accumulatedStaggerDuration = accumulatedStaggerDuration + staggerDuration

    local rectangleAction = AHSequence.new(
                                           AHDelay.new(accumulatedStaggerDuration),
                                           AHSpawn.new(
                                                       AHSequence.new(
                                                                      AHOpacityTo.new(rectangle, 0.1, nil, 1.0),
                                                                      AHOpacityTo.new(rectangle, 0.9, nil, 0.0)
                                                                      ),
                                                       AHScaleXTo.new(rectangle, 1.0, nil, destinationXScale),
                                                       AHScaleYTo.new(rectangle, 1.0, nil, destinationYScale)
                                                       )
                                           )
    spewAction:addAction(rectangleAction)

    rectangle:setVisible(true)
  end

  return spewAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SimonRectangleSpewer.new(width, height, kind)
  local self = Group.new(width, height)
  Mixin.mix(self, SimonRectangleSpewer)

  local rectangles = {}

  local blurMargin = 0.0

  local descriptors = nil
  if (kind == SimonRectangleSpewer.KIND_PERFECT_ANSWER) then
    descriptors = RECTANGLE_DESCRIPTORS_PERFECT_ANSWER
  elseif (kind == SimonRectangleSpewer.KIND_OUTRO) then
    descriptors = RECTANGLE_DESCRIPTORS_OUTRO
  else
    error("Undefined rectangle spewer kind: `" .. tostring(kind) .. "`")
  end
  self.descriptors = descriptors

  for _, descriptor in ipairs(descriptors) do
    local thickness = descriptor.thickness
    local rectangle = HollowRectangle.new(width, height, thickness, blurMargin)
    rectangle:setVisible(false)
    rectangle:setOpacity(0.0)
    self:insertAtCenter(rectangle)
    table.insert(rectangles, rectangle)
  end

  self.rectangles = rectangles

  return self
end
