--- SimonSparks
-- @author Created by Aaron Barrett on March 24, 2015.

class("SimonSparks")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.getWidthPart(0.08)
local HEIGHT = Screen.getHeightPart(0.08)

local SPARK_THICKNESS = Screen.getWidthPart(0.005)
local SPARK_LENGTH = Screen.getWidthPart(0.05)
local HOLDER_LENGTH = HEIGHT

local SPARKS_COUNT = 3
local SPARK_SPACING_ANGLE = 35.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonSparks:playExplode()
  local sparks = self.sparks

  local explodeAction = AHSpawn.new()

  local locationEase = AHEaseSimpleInOut.new(2.0)
  local scaleUpEase = AHEaseSimpleIn.new(2.0)
  local scaleDownEase = AHEaseSimpleOut.new(2.0)

  for _, spark in ipairs(sparks) do
    local locationAction = AHLocationYTo.new(spark, 0.3, locationEase, HOLDER_LENGTH)
    local scaleAction = AHSequence.new(
                                       AHScaleYTo.new(spark, 0.15, scaleUpEase, 1.0),
                                       AHScaleYTo.new(spark, 0.15, scaleDownEase, 0.0)
                                       )
    explodeAction:addAction(locationAction)
    explodeAction:addAction(scaleAction)
  end

  local explodeAndRemoveAction = AHSequence.new(
                                                explodeAction,
                                                AHRemove.new(self)
                                                )

  return explodeAndRemoveAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SimonSparks.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, SimonSparks)

  local sparks = {}

  local startAngle = -0.5 * (SPARK_SPACING_ANGLE * (SPARKS_COUNT - 1))
  local endAngle = -startAngle

  for angle = startAngle, endAngle, SPARK_SPACING_ANGLE do
    local holder = Group.new(SPARK_THICKNESS, HOLDER_LENGTH)
    holder:setAnchorPointBottom()

    local spark = FilledRectangle.new(SPARK_THICKNESS, SPARK_LENGTH)
    spark:setScl(1.0, 0.0)
    spark:setLoc(holder:getWidthPart(0.5), spark:getHeight())
    holder:insert(spark)

    holder:setRot(angle)

    self:insertAtCenter(holder)
    table.insert(sparks, spark)
  end

  self.sparks = sparks

  return self
end
