--- SimonStarField
-- @author Created by Aaron Barrett on April 01, 2015.

class("SimonStarField")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local STAR_WIDTH = 3.0
local STAR_HEIGHT = STAR_WIDTH

local MIN_TWINKLE_DELAY_DURATION = 0.2
local MAX_TWINKLE_DELAY_DURATION = 5.0

local STAR_DESCRIPTORS = {
  {xProportion = 0.24, yProportion = 0.99},
  {xProportion = 0.44, yProportion = 0.99},
  {xProportion = 0.64, yProportion = 0.98},
  {xProportion = 0.28, yProportion = 0.94},
  {xProportion = 0.38, yProportion = 0.92},
  {xProportion = 0.77, yProportion = 0.91},
  {xProportion = 0.94, yProportion = 0.90},
  {xProportion = 0.07, yProportion = 0.89},
  {xProportion = 0.34, yProportion = 0.88},
  {xProportion = 0.66, yProportion = 0.87},
  {xProportion = 0.04, yProportion = 0.85},
  {xProportion = 0.57, yProportion = 0.84},
  {xProportion = 0.37, yProportion = 0.83},
  {xProportion = 0.89, yProportion = 0.82},
  {xProportion = 0.16, yProportion = 0.78}
}

local TWINKLE_KIND_ROTATE = 1
local TWINKLE_KIND_OPACITY = 2
local TWINKLE_KINDS_COUNT = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createRandomTwinkleAction(star)
  local randomTwinkleKind = math.random(TWINKLE_KINDS_COUNT)
  if (randomTwinkleKind == TWINKLE_KIND_ROTATE) then
    return AHSequence.new(
                          AHOpacityTo.new(star, 1.5, nil, 0.0),
                          AHOpacityTo.new(star, 1.5, nil, 1.0)
                          )
  elseif (randomTwinkleKind == TWINKLE_KIND_OPACITY) then
    return AHRotationBy.new(star, 3.0, nil, 1080.0)
  end
  error("Undefined twinkle kind: `" .. tostring(randomTwinkleKind) .. "`")
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonStarField:playTwinkle()
  return runInCoroutine(function()
      local stars = self.stars
      local starsCount = #stars
      while (true) do
        local delayDuration = MIN_TWINKLE_DELAY_DURATION + math.random() * (MAX_TWINKLE_DELAY_DURATION - MIN_TWINKLE_DELAY_DURATION)
        waitFor(delayDuration)
        local randomlySelectedStar = stars[math.random(starsCount)]
        local twinkleAction = createRandomTwinkleAction(randomlySelectedStar)
        twinkleAction:start()
      end
    end)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildStar(self, descriptor)
  local star = FilledRectangle.new(STAR_WIDTH, STAR_HEIGHT)
  star:setRot(45.0)
  local x = self:getWidthPart(descriptor.xProportion)
  local y = self:getHeightPart(descriptor.yProportion)
  star:setLoc(x, y)
  self:insert(star)
  return star
end

-- Constructor ------------------------

function SimonStarField.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, SimonStarField)

  local stars = {}
  for _, descriptor in ipairs(STAR_DESCRIPTORS) do
    local star = buildStar(self, descriptor)
    table.insert(stars, star)
  end
  self.stars = stars

  return self
end
