--- SimonSubmitButton
-- @author Created by Aaron Barrett on February 24, 2015.

class("SimonSubmitButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.width
SimonSubmitButton.HEIGHT = Screen.getHeightPart(0.12) + Screen.safeBottom
local SUBMERGED_PEEK_AMOUNT = Screen.getHeightPart(0.03) + Screen.safeBottom

local BACKGROUND_PADDING = Screen.getHeightPart(0.5)
local BACKGROUND_HEIGHT = SimonSubmitButton.HEIGHT + BACKGROUND_PADDING

local EMERGED_Y = 0.5 * (SimonSubmitButton.HEIGHT - BACKGROUND_PADDING)
local SUBMERGED_Y = SUBMERGED_PEEK_AMOUNT - 0.5 * BACKGROUND_HEIGHT

local Z_MOUNTAINS = 10
local Z_IMAGERY = 20
local Z_BACKGROUND = 30
local Z_LABEL = 40


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonSubmitButton:setCallback(callback)
  self.callback = callback
end

function SimonSubmitButton:playPressed()
  local background = self.background
  local backgroundColor = self.backgroundColor
  local backgroundPressedColor = self.backgroundPressedColor
  background:setColor(unpack(backgroundPressedColor))
  local pressedAction = AHColorTo.new(background, 0.3, nil, backgroundColor)
  return pressedAction:start()
end

function SimonSubmitButton:playSubmerge()
  local background = self.background
  local label = self.label

  self:setTouchEnabled(false)

  local submergeAction = AHSpawn.new(
    AHLocationYTo.new(background, 0.5, AHEaseBackIn.new(1.2), SUBMERGED_Y),
    AHOpacityTo.new(label, 0.5, nil, 0)
  )
  return submergeAction:start()
end

function SimonSubmitButton:playEmergeWithText(text)
  local background = self.background
  local label = self.label

  label:setText(text)
  local emergeAction = AHSequence.new(
                                      AHSpawn.new(
                                        AHLocationYTo.new(background, 0.5, AHEaseBackOut.new(1.2), EMERGED_Y),
                                        AHOpacityTo.new(label, 0.5, nil, 1)
                                      ),
                                      function()
                                        self:setTouchEnabled(true)
                                      end
                                      )
  return emergeAction:start()
end

function SimonSubmitButton:playFadeOut()
  local label = self.label
  local mountains = self.mountains
  local background = self.background
  local imagery = self.imagery

  local fadeAction = AHSpawn.new(
                                 AHOpacityTo.new(label, 0.5, nil, 0.0),
                                 AHOpacityTo.new(mountains, 0.5, nil, 0.0),
                                 AHOpacityTo.new(background, 0.5, nil, 0.0),
                                 AHOpacityTo.new(imagery, 0.5, nil, 0.0)
                                 )
  return fadeAction:start()
end

function SimonSubmitButton:prepareToFadeIn()
  local label = self.label
  local mountains = self.mountains
  local background = self.background
  local imagery = self.imagery
  background:setY(SUBMERGED_Y)
  background:setOpacity(0.0)
  mountains:setOpacity(0.0)
  imagery:setOpacity(0.0)
  label:setOpacity(0.0)
end

function SimonSubmitButton:playFadeIn()
  local label = self.label
  local mountains = self.mountains
  local background = self.background
  local imagery = self.imagery
  local fadeInAction = AHSpawn.new(
                                   AHOpacityTo.new(label, 0.5, nil, 1.0),
                                   AHOpacityTo.new(mountains, 0.5, nil, 1.0),
                                   AHOpacityTo.new(background, 0.5, nil, 1.0),
                                   AHOpacityTo.new(imagery, 0.5, nil, 1.0)
                                   )
  return fadeInAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SimonSubmitButton.new(theme)
  local self = Group.new(WIDTH, SimonSubmitButton.HEIGHT)
  Mixin.mix(self, SimonSubmitButton)

  local mountainsFilename = theme:getSubmitButtonMountainsFilename()
  local mountains = Sprite.newWithName(mountainsFilename)
  mountains:setLoc(self:getWidthPart(0.5), SUBMERGED_PEEK_AMOUNT + mountains:getHeightPart(0.5))
  self:insert(mountains, Z_MOUNTAINS)
  self.mountains = mountains

  local backgroundColor = theme:getSubmitButtonColor()
  local background = FilledRectangle.new(WIDTH, BACKGROUND_HEIGHT, backgroundColor)
  background:setLoc(self:getWidthPart(0.5), SUBMERGED_Y)
  self:insert(background, Z_BACKGROUND)
  self.background = background

  local imageryFilename = theme:getSubmitButtonImageryFilename()
  local imagery = Sprite.newWithName(imageryFilename)
  imagery:setLoc(self:getWidthPart(0.5), BACKGROUND_HEIGHT + imagery:getHeightPart(0.5))
  background:insert(imagery, Z_IMAGERY)
  self.imagery = imagery

  local label = Label.new("A", Font.new(FontFaces.DIN_OT_MEDIUM, 16.0))
  local labelY = BACKGROUND_HEIGHT - ((self:getHeight() - Screen.safeBottom) / 2)
  label:setLoc(self:getWidthPart(0.5), labelY)
  background:insert(label, Z_LABEL)
  self.label = label

  self.backgroundColor = backgroundColor
  self.backgroundPressedColor = theme:getSubmitButtonPressedColor()

  self.callback = function() end
  Button.makeButton(self, function() self.callback() end)

  return self
end
