--- SimonTriangle
-- @author Created by Aaron Barrett on March 23, 2015.

class("SimonTriangle")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.getHeightPart(0.06)
local HEIGHT = Screen.getHeightPart(0.04)

local WIND_UP_DISTANCE = HEIGHT

local COLOR = {1.0, 1.0, 1.0}
local LINE_WIDTH = 0.6
local BLUR_MARGIN = 0.25

local OUTER_TRIANGLE_OPACITY = 0.25

local INNER_TRIANGLE_SCALE = 0.67

local Z_OUTER_TRIANGLE = 10
local Z_INNER_TRIANGLE = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SimonTriangle:playFlyIntoPlace(destinationY)
  local flyIntoPlaceAction = AHDelayed.new(0.1, AHLocationYTo.new(self, 1.2, AHEaseBounceOut.new(), destinationY))
  return flyIntoPlaceAction:start()
end

function SimonTriangle:playDrop(destinationY)
  local dropAction = AHSequence.new(
                                    AHLocationYBy.new(self, 0.3, AHEaseSimpleInOut.new(4.0), WIND_UP_DISTANCE),
                                    AHLocationYTo.new(self, 0.1, AHEaseSimpleOut.new(2.0), destinationY)
                                    )
  return dropAction:start()
end

function SimonTriangle:createQuickFadeOutAction()
  local quickFadeOutAction = AHOpacityTo.new(self, 0.2, nil, 0.0)
  return quickFadeOutAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SimonTriangle.new()
  local leftX = 0.0
  local leftY = HEIGHT

  local rightX = WIDTH
  local rightY = HEIGHT

  local middleX = WIDTH * 0.5
  local middleY = 0.0

  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, SimonTriangle)

  local outerTriangle = HollowTriangle.new(leftX, leftY, rightX, rightY, middleX, middleY, COLOR, LINE_WIDTH, BLUR_MARGIN)
  outerTriangle:setOpacity(OUTER_TRIANGLE_OPACITY)
  outerTriangle:inheritColor(self)
  self:insertAtCenter(outerTriangle, Z_OUTER_TRIANGLE)

  local innerTriangle = FilledTriangle.new(leftX, leftY, rightX, rightY, middleX, middleY, COLOR)
  innerTriangle:setScl(INNER_TRIANGLE_SCALE)
  innerTriangle:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.54))
  innerTriangle:inheritOpacity(self)
  self:insert(innerTriangle, Z_INNER_TRIANGLE)

  return self
end
