--- SquaresBackground
-- @author Created by Aaron Barrett on December 16, 2013.

class("SquaresBackground")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- Z-indices --------------------------

local Z_GRADIENT = 10
local Z_SMUDGE = 20
local Z_STAR = 30
local Z_CLOUD = 40

-- Dimensions -------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.height

local CENTER_X = WIDTH * 0.5
local CENTER_Y = HEIGHT * 0.5

-- Colors -----------------------------

local GRADIENT_TOP_COLOR_INDEX = 1
local GRADIENT_BOTTOM_COLOR_INDEX = 2
local CLOUD_COLOR_INDEX = 3

local COLOR_LEVELS = {
                      {filledProportion = 0.00, colors = {{0.94, 0.80, 0.73}, {0.94, 0.74, 0.54}, {0.84, 0.24, 0.14}}},
                      {filledProportion = 0.40, colors = {{0.94, 0.82, 0.73}, {0.89, 0.61, 0.51}, {0.73, 0.35, 0.30}}},
                      {filledProportion = 0.45, colors = {{0.93, 0.80, 0.71}, {0.83, 0.55, 0.52}, {0.62, 0.31, 0.33}}},
                      {filledProportion = 0.50, colors = {{0.93, 0.82, 0.72}, {0.78, 0.46, 0.49}, {0.58, 0.26, 0.33}}},
                      {filledProportion = 0.55, colors = {{0.92, 0.79, 0.71}, {0.63, 0.37, 0.47}, {0.37, 0.24, 0.34}}},
                      {filledProportion = 0.60, colors = {{0.91, 0.79, 0.72}, {0.52, 0.37, 0.50}, {0.25, 0.19, 0.33}}},
                      {filledProportion = 0.65, colors = {{0.84, 0.69, 0.67}, {0.43, 0.38, 0.49}, {0.12, 0.15, 0.29}}},
                      {filledProportion = 0.70, colors = {{0.81, 0.65, 0.67}, {0.40, 0.40, 0.51}, {0.12, 0.15, 0.29}}},
                      {filledProportion = 0.75, colors = {{0.78, 0.66, 0.72}, {0.31, 0.35, 0.37}, {0.12, 0.15, 0.29}}},
                      {filledProportion = 0.80, colors = {{0.64, 0.62, 0.67}, {0.31, 0.35, 0.47}, {0.12, 0.15, 0.29}}},
                      {filledProportion = 0.85, colors = {{0.41, 0.56, 0.61}, {0.32, 0.37, 0.48}, {0.12, 0.15, 0.29}}},
                      {filledProportion = 1.00, colors = {{0.31, 0.45, 0.47}, {0.25, 0.32, 0.47}, {0.18, 0.21, 0.36}}}
                      }

local COLOR_LEVEL_COUNT = #COLOR_LEVELS

local STARTING_TOP_COLOR = COLOR_LEVELS[1].colors[GRADIENT_TOP_COLOR_INDEX]
local STARTING_BOTTOM_COLOR = COLOR_LEVELS[1].colors[GRADIENT_BOTTOM_COLOR_INDEX]
local STARTING_CLOUD_COLOR = COLOR_LEVELS[1].colors[CLOUD_COLOR_INDEX]

local TRANSITION_DURATION = 3.21

local COLOR_CHANNEL_ATTRIBUTE_IDS = {
                                     MOAIColor.ATTR_R_COL,
                                     MOAIColor.ATTR_G_COL,
                                     MOAIColor.ATTR_B_COL
                                     }

-- Attribute IDs ----------------------

local ATTRIBUTE_CURVE_INPUT = MOAIAnimCurve.ATTR_TIME
local ATTRIBUTE_CURVE_OUTPUT = MOAIAnimCurve.ATTR_VALUE

-- Clouds -----------------------------

local CLOUD_POSITION_PROPORTIONS = {
                                    {x = 0.50, y = 0.18},
                                    {x = 0.25, y = 0.45},
                                    {x = 0.38, y = 0.62},
                                    {x = 0.22, y = 0.72}
                                    }

local CLOUD_OPACITY = 0.15

-- Stars ------------------------------

local STAR_DESCRIPTORS = {
                          {x = 0.12, y = 0.72, radius = 1.15},
                          {x = 0.65, y = 0.81, radius = 1.87},
                          {x = 0.42, y = 0.54, radius = 0.26},
                          {x = 0.84, y = 0.65, radius = 0.45},
                          {x = 0.27, y = 0.86, radius = 0.56},
                          {x = 0.09, y = 0.97, radius = 1.44},
                          {x = 0.01, y = 0.88, radius = 0.46},
                          {x = 0.92, y = 0.79, radius = 1.54},
                          {x = 0.58, y = 0.99, radius = 1.26},
                          {x = 0.78, y = 0.76, radius = 0.85},
                          {x = 0.86, y = 0.95, radius = 1.27},
                          {x = 0.37, y = 0.57, radius = 1.38},
                          {x = 0.93, y = 0.83, radius = 0.36},
                          {x = 0.18, y = 0.72, radius = 1.96},
                          {x = 0.36, y = 0.90, radius = 1.53},
                          {x = 0.45, y = 0.81, radius = 0.28},
                          {x = 0.28, y = 0.84, radius = 1.08},
                          {x = 0.49, y = 0.63, radius = 1.18},
                          {x = 0.88, y = 0.57, radius = 1.03},
                          {x = 0.47, y = 0.78, radius = 1.42},
                          {x = 0.72, y = 0.71, radius = 0.71},
                          {x = 0.87, y = 0.86, radius = 1.62},
                          {x = 0.75, y = 0.77, radius = 1.36},
                          {x = 0.84, y = 0.64, radius = 1.38},
                          {x = 0.11, y = 0.91, radius = 1.11},
                          {x = 0.91, y = 0.87, radius = 1.04},
                          {x = 0.14, y = 0.52, radius = 0.69},
                          {x = 0.46, y = 0.53, radius = 0.72},
                          {x = 0.53, y = 0.76, radius = 0.63},
                          {x = 0.78, y = 0.84, radius = 0.57},
                          {x = 0.25, y = 0.64, radius = 0.48},
                          {x = 0.32, y = 0.69, radius = 0.67},
                          {x = 0.91, y = 0.90, radius = 0.65},
                          {x = 0.54, y = 0.42, radius = 0.68},
                          {x = 0.76, y = 0.38, radius = 0.76},
                          {x = 0.23, y = 0.56, radius = 0.63},
                          {x = 0.38, y = 0.34, radius = 0.50},
                          {x = 0.05, y = 0.29, radius = 0.48},
                          {x = 0.22, y = 0.49, radius = 0.69},
                          {x = 0.41, y = 0.50, radius = 0.67}
                          }

local STAR_COLOR = {0.98, 0.98, 0.98}

local STAR_STEPS = 4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresBackground:transitionToFilledProportion(proportion)
  if (self.transitionAction) then
    self.transitionAction:stop()
  end

  local curves = self.curves
  local channelEase = nil
  local channelActions = {}
  for i, curve in ipairs(curves) do
    local channelAction = AHAttributeTo.new(curve, TRANSITION_DURATION, ATTRIBUTE_CURVE_INPUT, channelEase, proportion)
    table.insert(channelActions, channelAction)
  end

  local transitionAction = AHSpawn.new(unpack(channelActions))

  self.transitionAction = transitionAction
  transitionAction:start()
end

function SquaresBackground:addTouchHandler(onBackgroundTouched)
  TouchManager.addTouchEnabledMethods(self, onBackgroundTouched, TouchHandler.BEHAVIOR_ALWAYS)
  self:setTouchEnabled(true)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function addGradient(self)
  local gradient = RectangleGradient.new(WIDTH, HEIGHT, STARTING_TOP_COLOR, STARTING_BOTTOM_COLOR)
  gradient:setLoc(CENTER_X, CENTER_Y)
  self:insert(gradient, Z_GRADIENT)

  self.gradient = gradient

  local gradientTopColor = gradient:getBeginColor()
  local gradientBottomColor = gradient:getEndColor()

  local trackedColors = self.trackedColors
  table.insert(trackedColors, gradientTopColor)
  table.insert(trackedColors, gradientBottomColor)
end

local function addClouds(self)
  local trackedColors = self.trackedColors

  local cloudColor = ColorUtils.tableToMOAIColor(STARTING_CLOUD_COLOR)
  table.insert(trackedColors, cloudColor)

  for i, cloudPositionProportion in ipairs(CLOUD_POSITION_PROPORTIONS) do
    local filename = "clouds_" .. i .. ".png"
    local cloud = Sprite.newWithName(filename)

    local x = cloudPositionProportion.x * WIDTH
    local y = cloudPositionProportion.y * HEIGHT

    cloud:setLoc(x, y)
    cloud:setOpacity(CLOUD_OPACITY)

    cloud:setAttrLink(MOAIColor.INHERIT_COLOR, cloudColor, MOAIColor.COLOR_TRAIT)

    self:insert(cloud, Z_CLOUD)
  end

  local smudge = Sprite.newWithName("smudge.png")
  local smudgeWidth, smudgeHeight = smudge:getDims()

  local smudgeScaleX = WIDTH / smudgeWidth
  local smudgeScaleY = (HEIGHT * 0.6) / smudgeHeight

  smudge:setAnchorPointBottom()
  smudge:setScl(smudgeScaleX, smudgeScaleY)
  smudge:setLoc(CENTER_X, 0.0)

  smudge:setColor(0.75, 0.75, 0.75)

  smudge:setAttrLink(MOAIColor.INHERIT_COLOR, cloudColor, MOAIColor.COLOR_TRAIT)

  self:insert(smudge, Z_SMUDGE)
end

local function addStars(self)
  for i, descriptor in ipairs(STAR_DESCRIPTORS) do
    local radius = descriptor.radius
    local star = FilledCircle.new(radius, STAR_COLOR)
    star:setPointCount(STAR_STEPS)

    local x = descriptor.x * WIDTH
    local y = descriptor.y * HEIGHT

    star:setLoc(x, y)

    self:insert(star, Z_STAR)
  end
end

local function addCurves(self)
  local trackedColors = self.trackedColors

  local curves = {}

  for channelIndex, channelAttributeID in ipairs(COLOR_CHANNEL_ATTRIBUTE_IDS) do

    for trackedColorIndex, trackedColor in ipairs(trackedColors) do
      local curve = MOAIAnimCurve.new()
      curve:setWrapMode(MOAIAnimCurve.APPEND)
      curve:reserveKeys(COLOR_LEVEL_COUNT)

      for levelIndex, level in ipairs(COLOR_LEVELS) do
        local filledProportion = level.filledProportion
        local levelColor = level.colors[trackedColorIndex]
        local channelValue = levelColor[channelIndex]

        curve:setKey(levelIndex, filledProportion, channelValue, MOAIEaseType.LINEAR)
      end

      trackedColor:setAttrLink(channelAttributeID, curve, ATTRIBUTE_CURVE_OUTPUT)
      table.insert(curves, curve)

      curve:scheduleUpdate()
    end

  end

  self.curves = curves
end

-- Constructor ------------------------

function SquaresBackground.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, SquaresBackground)

  self.trackedColors = {}

  addGradient(self)
  addClouds(self)
  addStars(self)
  addCurves(self)

  self.transitionAction = nil

  return self
end
