--- SquaresBalloon
-- @author Created by Aaron Barrett on December 16, 2013.

class("SquaresBalloon")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SquaresBalloon.FOCUS_STATE_NORMAL = "focus_state_normal"
SquaresBalloon.FOCUS_STATE_GRAYED = "focus_state_grayed"
SquaresBalloon.FOCUS_STATE_FOCUSED = "focus_state_focused"

-- Z-indices --------------------------

local Z_TAIL = 10
local Z_ROPE = 20
local Z_CIRCLE = 30
local Z_ICON = 40
local Z_CHARGE_UP_CIRCLE = 50
local Z_LABEL = 60
local Z_HALO = 70

-- Font -------------------------------

local LABEL_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 24.0)

local LABEL_WIDTH_FACTOR = 1.8
local MAX_LABEL_WIDTH = LABEL_WIDTH_FACTOR * SquaresConstants.BASE_RADIUS

-- Rope (because string is confusing) -

local ROPE_Y_OFFSET = 2.0
local TAIL_Y_OFFSET = 2.5

-- Circle -----------------------------

SquaresBalloon.OVERLAP_AMOUNT = 3.0

-- Opacities --------------------------

local BALLOON_OPACITY_NORMAL = 0.75
local BALLOON_OPACITY_GRAYED = 0.2

local ICON_OPACITY_NORMAL = 0.2
local ICON_OPACITY_GRAYED = 0.1

local LABEL_OPACITY_NORMAL = 1.0
local LABEL_OPACITY_GRAYED = 0.15

local ROPE_OPACITY_NORMAL = 1.0
local ROPE_OPACITY_GRAYED = 0.1

local BALLOON_OPACITY_FOR_FOCUS_STATE = {}
BALLOON_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_FOCUSED] = BALLOON_OPACITY_NORMAL
BALLOON_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_NORMAL] = BALLOON_OPACITY_NORMAL
BALLOON_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_GRAYED] = BALLOON_OPACITY_GRAYED

local ICON_OPACITY_FOR_FOCUS_STATE = {}
ICON_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_FOCUSED] = ICON_OPACITY_NORMAL
ICON_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_NORMAL] = ICON_OPACITY_NORMAL
ICON_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_GRAYED] = ICON_OPACITY_GRAYED

local LABEL_OPACITY_FOR_FOCUS_STATE = {}
LABEL_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_FOCUSED] = LABEL_OPACITY_NORMAL
LABEL_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_NORMAL] = LABEL_OPACITY_NORMAL
LABEL_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_GRAYED] = LABEL_OPACITY_GRAYED

local ROPE_OPACITY_FOR_FOCUS_STATE = {}
ROPE_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_FOCUSED] = ROPE_OPACITY_NORMAL
ROPE_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_NORMAL] = ROPE_OPACITY_NORMAL
ROPE_OPACITY_FOR_FOCUS_STATE[SquaresBalloon.FOCUS_STATE_GRAYED] = ROPE_OPACITY_GRAYED

-- Charge Up --------------------------

local CHARGE_UP_CIRCLE_COLOR = {1.0, 1.0, 1.0}

-- Adjuster Scales --------------------

local RADIUS_FOR_SMALL_ADJUSTER = SquaresConstants.BASE_RADIUS * 0.87
local SMALL_ADJUSTER_SCALE = 0.75

-- Durations --------------------------

SquaresBalloon.CHARGE_UP_DURATION = 0.285

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getAdjusterScaleForDestinationRadius(destinationRadius)
  if (destinationRadius < RADIUS_FOR_SMALL_ADJUSTER) then
    return SMALL_ADJUSTER_SCALE
  end
  return 1.0
end

local function scaleLabelIfNeeded(label)
  local labelEffectiveWidth = math.min(label:getWidth(), MAX_LABEL_WIDTH)
  label:setEffectiveDims(labelEffectiveWidth)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresBalloon:getIconRotation()
  local icon = self.icon
  local rotation = icon:getRot()
  return rotation
end

function SquaresBalloon:getPosition()
  local body = self.body
  return body:getPosition()
end

function SquaresBalloon:getPowerupFlavor()
  return self.powerupFlavor
end

function SquaresBalloon:getRadius()
  return self.radius
end

function SquaresBalloon:getKind()
  return self.kind
end

function SquaresBalloon:getIdentifier()
  return self.identifier
end

function SquaresBalloon:getConcept()
  return self.concept
end

function SquaresBalloon:getBody()
  return self.body
end

function SquaresBalloon:getFixture()
  return self.fixture
end

function SquaresBalloon:getFocusState()
  return self.focusState
end

function SquaresBalloon:setBody(body)
  self.body = body
end

function SquaresBalloon:getIsOnScreen()
  local _, y = self:getPosition()
  local radius = self.radius
  local amountThatMustBeVisible = radius * 0.5
  if (y < amountThatMustBeVisible) then
    return false
  end

  return true
end

function SquaresBalloon:doesMatch(otherBalloon)
  return (self.identifier == otherBalloon:getIdentifier())
end

function SquaresBalloon:createEffect(balloons, popParticles)
  local powerupFlavor = self.powerupFlavor

  if (powerupFlavor == SquaresPowerup.FLAVOR_FIREWORK) then
    return SquaresEffectFirework.new(self, balloons, popParticles)
  elseif (powerupFlavor == SquaresPowerup.FLAVOR_BOW) then
    return SquaresEffectBow.new(self, balloons, popParticles)
  elseif (powerupFlavor == SquaresPowerup.FLAVOR_SHRINKER) then
    return SquaresEffectShrinker.new(balloons)
  end

  error("Unknown powerup flavor.")
  return nil
end

function SquaresBalloon:createChargeUpAction()
  local chargeUpCircle = self.chargeUpCircle

  local pulseAction = AHSequence.new(
                                     AHDelay.new(0.06),
                                     AHOpacityTo.new(chargeUpCircle, 0.12, AHEaseExponentialIn.new(), 0.45),
                                     AHOpacityTo.new(chargeUpCircle, 0.005, nil, 0.0),
                                     AHOpacityTo.new(chargeUpCircle, 0.06, AHEaseExponentialIn.new(), 0.50),
                                     AHOpacityTo.new(chargeUpCircle, 0.005, nil, 0.0),
                                     AHOpacityTo.new(chargeUpCircle, 0.03, AHEaseExponentialIn.new(), 0.55),
                                     AHOpacityTo.new(chargeUpCircle, 0.005, nil, 0.0)
                                     )

  local shakeXAction = AHSequence.new(
                                      AHLocationXBy.new(self, 0.030, nil, 2.5),
                                      AHLocationXBy.new(self, 0.025, nil, -2.1),
                                      AHLocationXBy.new(self, 0.025, nil, 2.2),
                                      AHLocationXBy.new(self, 0.020, nil, -1.8),
                                      AHLocationXBy.new(self, 0.025, nil, 2.2),
                                      AHLocationXBy.new(self, 0.020, nil, -1.8),
                                      AHLocationXBy.new(self, 0.025, nil, 1.9),
                                      AHLocationXBy.new(self, 0.035, nil, -2.2),
                                      AHLocationXBy.new(self, 0.025, nil, 2.1),
                                      AHLocationXBy.new(self, 0.020, nil, -2.1),
                                      AHLocationXBy.new(self, 0.010, nil, 1.2),
                                      AHLocationXBy.new(self, 0.010, nil, -1.8)
                                      )

  local shakeYAction = AHSequence.new(
                                      AHLocationYBy.new(self, 0.025, nil, -2.3),
                                      AHLocationYBy.new(self, 0.030, nil, 2.6),
                                      AHLocationYBy.new(self, 0.020, nil, -2.3),
                                      AHLocationYBy.new(self, 0.020, nil, 1.9),
                                      AHLocationYBy.new(self, 0.030, nil, -2.1),
                                      AHLocationYBy.new(self, 0.020, nil, 2.0),
                                      AHLocationYBy.new(self, 0.030, nil, -2.2),
                                      AHLocationYBy.new(self, 0.025, nil, 2.1),
                                      AHLocationYBy.new(self, 0.030, nil, -2.0),
                                      AHLocationYBy.new(self, 0.015, nil, 2.0),
                                      AHLocationYBy.new(self, 0.015, nil, -2.3),
                                      AHLocationYBy.new(self, 0.010, nil, 1.9)
                                      )

  local chargeUpAction = AHSpawn.new(
                                     AHCallLogic.new(function()
                                                         local circleRadius = self.radius + SquaresBalloon.OVERLAP_AMOUNT
                                                         chargeUpCircle:setScl(circleRadius)
                                                       end),
                                     pulseAction,
                                     shakeXAction,
                                     shakeYAction
                                     )

  return chargeUpAction
end

function SquaresBalloon:setFocusState(focusState)
  if (self.focusState ~= focusState) then

    if (self.focusAction) then
      self.focusAction:stop()
    end

    self.focusState = focusState
    local circle = self.circle
    local label = self.label
    local rope = self.rope

    local focusDuration = 0.12

    local balloonOpacity = BALLOON_OPACITY_FOR_FOCUS_STATE[focusState]
    local circleOpacityAction = AHOpacityTo.new(circle, focusDuration, nil, balloonOpacity)

    local labelOpacity = LABEL_OPACITY_FOR_FOCUS_STATE[focusState]
    local labelOpacityAction = AHOpacityTo.new(label, focusDuration, nil, labelOpacity)

    local ropeOpacity = ROPE_OPACITY_FOR_FOCUS_STATE[focusState]
    local ropeOpacityAction = AHOpacityTo.new(rope, focusDuration, nil, ropeOpacity)

    local iconOpacityAction = nil

    local icon = self.icon
    if (icon) then
      local iconOpacity = ICON_OPACITY_FOR_FOCUS_STATE[focusState]
      iconOpacityAction = AHOpacityTo.new(icon, focusDuration, nil, iconOpacity)
    end

    local focusAction = AHSpawn.new(
                                    circleOpacityAction,
                                    labelOpacityAction,
                                    ropeOpacityAction,
                                    iconOpacityAction
                                    )

    self.focusAction = focusAction
    focusAction:start()

    if (focusState == SquaresBalloon.FOCUS_STATE_FOCUSED) then
      local radius = self.radius
      local baseRadius = self.baseRadius
      local halo = SquaresHalo.new(radius)
      halo:setLoc(baseRadius, baseRadius)
      self:insert(halo, Z_HALO)
      halo:startAnimating()
      self.halo = halo
    else
      local halo = self.halo
      if (halo and halo:getIsShowing()) then
        halo:disappear()
      end
    end

  end
end

function SquaresBalloon:startRopeSway()
  local rope = self.rope
  local ropeSwayAction = AHLoop.new(
                                    AHSequence.new(
                                                   AHRotationTo.new(rope, 1.2, AHEaseSineInOut.new(2.0), 4.0),
                                                   AHRotationTo.new(rope, 1.2, AHEaseSineInOut.new(2.0), -4.0)
                                                   )
                                    )
  self.ropeSwayAction = ropeSwayAction
  ropeSwayAction:start()
end

function SquaresBalloon:stopBowIcon()
  local powerupFlavor = self.powerupFlavor
  if (powerupFlavor == SquaresPowerup.FLAVOR_BOW) then
    local icon = self.icon
    if (icon) then
      icon:stopAnimating()
    end
  end
end

function SquaresBalloon:startIconAnimating()
  local icon = self.icon
  if (icon) then
    icon:startAnimating()
  end
end

function SquaresBalloon:startAnimating()
  self:startRopeSway()
  self:startIconAnimating()
end

function SquaresBalloon:resetFixtureWithRadius(radius)
  local body = self.body

  if (self.fixture) then
    self.fixture:destroy()
  end

  local fixture = body:addCircle(0.0, 0.0, radius)
  fixture:setDensity(1.0)
  fixture:setFriction(0.0)
  fixture:setRestitution(0.3)
  body:resetMassData()

  self.fixture = fixture

  return fixture
end

function SquaresBalloon:createResizeAction(destinationRadius, delayDuration, shouldPlayShrinkSound)
  delayDuration = delayDuration or 0.0

  local circle = self.circle
  local rope = self.rope
  local tail = self.tail
  local label = self.label

  local existingResizeAction = self.resizeAction

  local baseRadius = self.baseRadius

  local circleDestinationRadius = destinationRadius + SquaresBalloon.OVERLAP_AMOUNT

  local scaleDuration = 1.1
  local elasticEase = AHEaseElasticOut.new(0.2)
  local circleScaleAction = AHScaleTo.new(circle, scaleDuration, elasticEase, circleDestinationRadius)

  local bottomY = baseRadius - destinationRadius

  local ropeDestinationY = bottomY - ROPE_Y_OFFSET
  local ropeMoveAction = AHLocationYTo.new(rope, scaleDuration, elasticEase, ropeDestinationY)

  local tailDestinationY = bottomY - TAIL_Y_OFFSET
  local tailMoveAction = AHLocationYTo.new(tail, scaleDuration, elasticEase, tailDestinationY)

  local adjusterScale = getAdjusterScaleForDestinationRadius(destinationRadius)
  local labelScale = adjusterScale
  if (label:getScl() < 1.0) then
    labelScale = MathUtils.clamp(adjusterScale * label:getScl(), 0.5, 1.0)
  end
  local labelAction = AHScaleTo.new(label, scaleDuration, elasticEase, labelScale)

  local adjusterAction = nil
  local iconAdjuster = self.iconAdjuster
  if (iconAdjuster) then
    adjusterAction = AHSpawn.new(
                                 labelAction,
                                 AHScaleTo.new(iconAdjuster, scaleDuration, elasticEase, adjusterScale)
                                 )
  else
    adjusterAction = labelAction
  end

  local haloScaleAction = nil
  local halo = self.halo
  if (halo) then
    local haloDiameter = halo:getDims()
    local haloRadius = haloDiameter * 0.5
    local haloDestinationRadius = destinationRadius / haloRadius
    haloScaleAction = AHScaleTo.new(halo, scaleDuration, elasticEase, haloDestinationRadius)
  end

  local resizeAction = AHSequence.new(
                                      AHDelay.new(delayDuration),
                                      AHCallLogic.new(function()
                                                          if (existingResizeAction) then
                                                            existingResizeAction:stop()
                                                          end
                                                          if (shouldPlayShrinkSound) then
                                                            Sound.playSoundEffect("shrink")
                                                          end
                                                        end),
                                      AHSpawn.new(
                                                  circleScaleAction,
                                                  ropeMoveAction,
                                                  tailMoveAction,
                                                  adjusterAction,
                                                  haloScaleAction
                                                  ),
                                      AHCallLogic.new(function()
                                                          self.resizeAction = nil
                                                        end)
                                      )

  return resizeAction
end

function SquaresBalloon:resize(amount, delayDuration, shouldPlayShrinkSound)
  local originalRadius = self.radius

  local resizedRadius = originalRadius + amount
  if (resizedRadius < SquaresConstants.MIN_RADIUS) then
    resizedRadius = SquaresConstants.MIN_RADIUS
  elseif (resizedRadius > SquaresConstants.MAX_RADIUS) then
    resizedRadius = SquaresConstants.MAX_RADIUS
  end

  self:resetFixtureWithRadius(resizedRadius)
  self.radius = resizedRadius

  local resizeAction = self:createResizeAction(resizedRadius, delayDuration, shouldPlayShrinkSound)
  self.resizeAction = resizeAction

  return resizeAction
end

function SquaresBalloon:pop(allowPowerups, popParticles)
  if (self.focusAction) then
    self.focusAction:stop()
  end
  if (self.ropeSwayAction) then
    self.ropeSwayAction:stop()
  end
  if (self.resizeAction) then
    self.resizeAction:stop()
  end

  local circle = self.circle
  local rope = self.rope
  local label = self.label
  local icon = self.icon
  local circleRadius = self.radius + SquaresBalloon.OVERLAP_AMOUNT

  local destinationScaleFactor = 3.6
  local destinationCircleRadius = circleRadius * destinationScaleFactor

  local popDuration = 0.145
  local scaleEase = AHEaseSimpleIn.new(3.8)

  if (self.halo) then
    self.halo:removeSelf()
  end

  if (icon) then
    if (allowPowerups) then
      icon:activate()
    else
      icon:disappear()
    end
  end

  if (popParticles) then
    popParticles:showParticlesForBalloon(self)
  end

  local popAction = AHSequence.new(
                                   AHSpawn.new(
                                               AHScaleTo.new(circle, popDuration, scaleEase, destinationCircleRadius),
                                               AHOpacityTo.new(circle, popDuration, nil, 0.0),
                                               AHOpacityTo.new(rope, popDuration, nil, 0.0),
                                               AHOpacityTo.new(label, 0.6, nil, 0.0)
                                               ),
                                   AHCallLogic.new(function()
                                                       self:removeSelf()
                                                     end)
                                   )

  self.popAction = popAction
  popAction:start()

  self.halo = nil

  self.focusAction = nil
  self.ropeSwayAction = nil
  self.resizeAction = nil
end

function SquaresBalloon:addTouchHandlerToCircle(onBalloonTouched)
  local circle = self.circle
  TouchManager.addTouchHandler(circle, onBalloonTouched, TouchHandler.BEHAVIOR_ALWAYS)
end

function SquaresBalloon:removeTouchHandler()
  TouchManager.removeTouchHandler(self.circle)
end

function SquaresBalloon:fadeLabel()
  local fadeDuration = 0.25
  local fadeLabelAction = AHOpacityTo.new(self.label, fadeDuration, nil, 0.0)
  fadeLabelAction:start()
end

function SquaresBalloon:getCircle()
  local circleRadius = self.radius + SquaresBalloon.OVERLAP_AMOUNT
  local circle = FilledCircle.new(1.0, self.kind.balloonColor)

  circle:setScl(circleRadius)
  circle:setLoc(self.radius, self.radius)

  return circle
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresBalloon.new(kind, concept, radius, powerupFlavor, isGrayed)
  local diameter = radius * 2.0

  local self = Group.new(diameter, diameter)
  Mixin.mix(self, SquaresBalloon)

  self.kind = kind
  self.radius = radius
  self.concept = concept
  self.identifier = concept.identifier
  self.radius = radius
  self.baseRadius = radius
  self.powerupFlavor = powerupFlavor

  local focusState = nil
  if (isGrayed) then
    focusState = SquaresBalloon.FOCUS_STATE_GRAYED
  else
    focusState = SquaresBalloon.FOCUS_STATE_NORMAL
  end

  self.focusState = focusState

  local balloonColor = kind.balloonColor
  local balloonOpacity = BALLOON_OPACITY_FOR_FOCUS_STATE[focusState]

  local circle = self:getCircle()
  circle:setOpacity(balloonOpacity)
  self:insert(circle, Z_CIRCLE)
  self.circle = circle
  circle.balloon = self

  local rope = Sprite.newWithName("rope.png")
  local ropeOpacity = ROPE_OPACITY_FOR_FOCUS_STATE[focusState]
  rope:setOpacity(ropeOpacity)
  rope:setAnchorPointTop()
  rope:setLoc(radius, -ROPE_Y_OFFSET)
  self:insert(rope, Z_ROPE)
  self.rope = rope

  local tail = Sprite.newWithName("tail.png")
  tail:setColor(unpack(balloonColor))
  tail:setAttrLink(MOAIColor.ATTR_OPACITY, circle)
  tail:setAnchorPointTop()
  tail:setLoc(radius, -TAIL_Y_OFFSET)
  self:insert(tail, Z_TAIL)
  self.tail = tail

  local text = concept[kind.name]
  local label = Label.new(text, LABEL_FONT)
  scaleLabelIfNeeded(label)
  label:setLoc(radius, radius)
  local labelOpacity = LABEL_OPACITY_FOR_FOCUS_STATE[focusState]
  label:setOpacity(labelOpacity)
  self:insert(label, Z_LABEL)
  self.label = label

  local shouldAddPowerupIcon = (powerupFlavor ~= nil)
  if (shouldAddPowerupIcon) then
    local icon = SquaresIcon.new(powerupFlavor)
    local iconOpacity = ICON_OPACITY_FOR_FOCUS_STATE[focusState]
    icon:setOpacity(iconOpacity)
    icon:setLoc(radius, radius)

    local iconAdjuster = Group.new(diameter, diameter)
    iconAdjuster:setLoc(radius, radius)

    iconAdjuster:insert(icon)
    self:insert(iconAdjuster, Z_ICON)

    self.icon = icon
    self.iconAdjuster = iconAdjuster

    local chargeUpCircle = FilledCircle.new(1.0, CHARGE_UP_CIRCLE_COLOR)
    chargeUpCircle:setLoc(radius, radius)
    chargeUpCircle:setOpacity(0.0)
    self:insert(chargeUpCircle, Z_CHARGE_UP_CIRCLE)
    self.chargeUpCircle = chargeUpCircle
  end

  self.body = nil
  self.fixture = nil

  self.halo = nil

  self.focusAction = nil
  self.ropeSwayAction = nil
  self.resizeAction = nil

  return self
end
