class("SquaresConfiguration")

require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SquaresConfiguration.POINTS_PER_CORRECT_RESPONSE = 200
SquaresConfiguration.CORRECT_NEEDED_TO_PASS = 30
SquaresConfiguration.MAX_POINTS_FOR_ACCURACY_BONUS = 3000
SquaresConfiguration.BONUS_TYPE_ERRORS = "ERRORS"
SquaresConfiguration.TUTORIAL_DELAY = 13
SquaresConfiguration.PENALTY_RESIZE_FACTOR = 0.2

local MAX_ALLOWED_INCORRECT_RANK_TWO = 1
local MAX_ALLOWED_INCORRECT_RANK_THREE = 0

local BALLOON_GENERATION_DIFFICULTY_FACTOR_KEYFRAMES = KeyframeSet.newInterpolated(
  Keyframe.new(0.0, {difficultyFactor = 1.2}),
  Keyframe.new(0.3, {difficultyFactor = 1.0}),
  Keyframe.new(1.0, {difficultyFactor = 0.9})
)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresConfiguration.getBalloonGenerationDifficultyFactor()
  local difficulty = GameSession.getDifficulty()
  local difficultyFactorFrame = BALLOON_GENERATION_DIFFICULTY_FACTOR_KEYFRAMES:getFrameAt(difficulty)
  return difficultyFactorFrame.difficultyFactor
end

function SquaresConfiguration.getPassScore()
  local correctToPass = SquaresConfiguration.CORRECT_NEEDED_TO_PASS
  local passScore = correctToPass * SquaresConfiguration.POINTS_PER_CORRECT_RESPONSE
  return passScore
end

function SquaresConfiguration.getRankTwoScore()
  local passScore = SquaresConfiguration.getPassScore()
  local accuracyBonus = SquaresConfiguration.getAccuracyBonusScoreForIncorrectCount(MAX_ALLOWED_INCORRECT_RANK_TWO)
  local rankTwoScore = passScore + accuracyBonus
  return rankTwoScore
end

function SquaresConfiguration.getRankThreeScore()
  local passScore = SquaresConfiguration.getPassScore()
  local accuracyBonus = SquaresConfiguration.getAccuracyBonusScoreForIncorrectCount(MAX_ALLOWED_INCORRECT_RANK_THREE)
  local rankThreeScore = passScore + accuracyBonus
  return rankThreeScore
end

function SquaresConfiguration.getMaxScore()
  local passScore = SquaresConfiguration.getPassScore()
  local accuracyBonus = SquaresConfiguration.getAccuracyBonusScoreForIncorrectCount(0)
  local maxScore = passScore + accuracyBonus
  return maxScore
end

function SquaresConfiguration.getAccuracyBonusScoreForIncorrectCount(incorrectAnswersCount)
  local minAccuracyBonus = 0
  local maxAccuracyBonus = SquaresConfiguration.MAX_POINTS_FOR_ACCURACY_BONUS
  local incorrectAnswersForNoBonus = MAX_ALLOWED_INCORRECT_RANK_TWO + 1
  local incorrectAnswersForMaxBonus = 0
  incorrectAnswersCount = MathUtils.clamp(incorrectAnswersCount, incorrectAnswersForMaxBonus, incorrectAnswersForNoBonus)

  local accuracyBonus = MathUtils.mapLinear(incorrectAnswersCount,
                                            incorrectAnswersForNoBonus, incorrectAnswersForMaxBonus,
                                            minAccuracyBonus, maxAccuracyBonus)
  accuracyBonus = math.floor(accuracyBonus)

  return accuracyBonus
end

function SquaresConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 0.90,
                                      Easy = 0.10}},

    {key = 0.15, valueProbabilities = {Basic = 0.50,
                                       Easy = 0.50}},

    {key = 0.225, valueProbabilities = {Easy = 0.65,
                                        Medium = 0.35}},

    {key = 0.375, valueProbabilities = {Easy = 0.30,
                                        Medium = 0.55,
                                        Hard = 0.15}},

    {key = 0.45, valueProbabilities = {Easy = 0.15,
                                       Medium = 0.55,
                                       Hard = 0.30}},

    {key = 0.6, valueProbabilities = {Easy = 0.10,
                                      Medium = 0.45,
                                      Hard = 0.45}},

    {key = 0.675, valueProbabilities = {Easy = 0.08,
                                        Medium = 0.27,
                                        Hard = 0.65}},

    {key = 0.75, valueProbabilities = {Easy = 0.05,
                                       Medium = 0.15,
                                       Hard = 0.80}},

    {key = 0.8, valueProbabilities = {Medium = 0.1,
                                      Hard = 0.80,
                                      Advanced = 0.1}},

    {key = 0.9, valueProbabilities = {Hard = 0.80,
                                      Advanced = 0.2}},

    {key = 1.0, valueProbabilities = {Hard = 0.7,
                                      Advanced = 0.3}}
  }
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
