--- SquaresCounter

class("SquaresCounter")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ROW_SPACING = 6

local DEFAULT_COLOR = {1.0, 1.0, 1.0, 1.0}
local DEFAULT_OPACITY = 1.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function updateLabel(self)
  local text = self.currentValue .. "/" .. self.maxValue
  self.label:setText(text)
  self:alignChildren()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresCounter:setMaxValue(maxValue)
  self.maxValue = maxValue
  updateLabel(self)
end

function SquaresCounter:setValue(value)
  self.currentValue = math.min(value, self.maxValue)
  updateLabel(self)
end

-- UIOverlay Widget -------------------

function SquaresCounter:getKeyColor()
  return self.keyColor
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresCounter.new(maxValue, color)
  maxValue = maxValue or 0

  color = color or DEFAULT_COLOR
  color[4] = color[4] or DEFAULT_OPACITY

  local self = Row.new(ROW_SPACING)
  Mixin.mix(self, SquaresCounter)

  local font = StandardUIBuilder.STANDARD_FONT
  local fontSize = StandardUIBuilder.STANDARD_FONT_SIZE

  self.currentValue = 0

  self.label = Label.new("0/0", Font.new(font, fontSize))
  self.icon = Sprite.newWithName("icon_counter.png")

  self:insert(self.icon)
  self:insert(self.label)

  self:linkAttrToChildren(MOAIColor.INHERIT_COLOR, MOAIColor.COLOR_TRAIT)

  self:setColor(unpack(color))
  self.keyColor = color

  self:setMaxValue(maxValue)

  return self
end
