--- SquaresEffectBow
-- @author Created by Aaron Barrett on January 9, 2014.

class("SquaresEffectBow")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_BOW = 10
local Z_ARROW = 20

local ARROW_FLIGHT_DISTANCE = math.sqrt(Screen.width * Screen.width + Screen.height * Screen.height)
local ARROW_FLIGHT_DURATION = 0.45

local BOW_FRAMERATE = 48.0

local BOW_X_OFFSET = 4.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresEffectBow:getGroup()
  local group = self.group
  return group
end

function SquaresEffectBow:getAction()
  local action = self.action
  return action
end

function SquaresEffectBow:getHitBalloons()
  local hitBalloons = self.hitBalloons
  return hitBalloons
end

function SquaresEffectBow:getDelayDuration()
  return 0.65
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresEffectBow.new(sourceBalloon, otherBalloons, popParticles)
  local self = {}
  Mixin.mix(self, SquaresEffectBow)

  local sourceX, sourceY = sourceBalloon:getPosition()
  local angleInDegrees = sourceBalloon:getIconRotation()

  local angleInRadians = math.rad(angleInDegrees + 90.0)

  local hitBalloons = {}
  local hitBalloonActions = {}

  local destinationX = sourceX + math.cos(angleInRadians) * ARROW_FLIGHT_DISTANCE
  local destinationY = sourceY + math.sin(angleInRadians) * ARROW_FLIGHT_DISTANCE
  local directionVectorX = destinationX - sourceX
  local directionVectorY = destinationY - sourceY

  for balloon in pairs(otherBalloons) do
    local balloonX, balloonY = balloon:getPosition()
    local balloonRadius = balloon:getRadius() + SquaresBalloon.OVERLAP_AMOUNT

    local balloonVectorX = sourceX - balloonX
    local balloonVectorY = sourceY - balloonY

    local a = directionVectorX * directionVectorX + directionVectorY * directionVectorY
    local b = 2.0 * (balloonVectorX * directionVectorX + balloonVectorY * directionVectorY)
    local c = balloonVectorX * balloonVectorX + balloonVectorY * balloonVectorY - balloonRadius * balloonRadius
    local discriminant = b * b - 4.0 * a * c

    local didHitBalloon = (discriminant >= 0.0 and b < 0.0)
    if (didHitBalloon) then
      table.insert(hitBalloons, balloon)

      local distance = math.sqrt(balloonVectorX * balloonVectorX + balloonVectorY * balloonVectorY)
      local delayDuration = (distance / ARROW_FLIGHT_DISTANCE) * ARROW_FLIGHT_DURATION

      local hitBalloonAction = AHSequence.new(
                                              AHDelay.new(delayDuration),
                                              function()
                                                popParticles:show(balloonX, balloonY)
                                                local allowPowerups = false
                                                balloon:pop(allowPowerups)
                                                Sound.playSoundEffect("pop")
                                              end
                                              )

      table.insert(hitBalloonActions, hitBalloonAction)
    end
  end

  local bow = Sprite.newWithName("bow_back_000.png")
  bow:setOpacity(0.0)

  local bowBackAnimation = Animation.newWithPrefix("bow_back_")
  local bowReleaseAnimation = Animation.newWithPrefix("bow_release_")

  local bowWidth, bowHeight = bow:getDims()
  local centerX = bowWidth * 0.5
  local centerY = bowHeight * 0.5

  local arrow = Sprite.newWithName("arrow.png")
  arrow:setVisible(false)
  arrow:setLoc(centerX, centerY)

  local group = Group.new(bowWidth, bowHeight)
  bow:setLoc(centerX + BOW_X_OFFSET, centerY)
  group:insert(bow, Z_BOW)
  group:insert(arrow, Z_ARROW)

  group:setLoc(sourceX, sourceY)
  group:setRot(angleInDegrees)

  local action = AHSequence.new(
                                function() Sound.playSoundEffect("shoot_arrow") end,
                                AHOpacityTo.new(bow, 0.12, nil, 1.0),
                                AHAnimate.new(bow, bowBackAnimation, BOW_FRAMERATE),
                                function() arrow:setVisible(true) end,
                                AHSpawn.new(
                                            AHSequence.new(
                                                           AHAnimate.new(bow, bowReleaseAnimation, BOW_FRAMERATE),
                                                           AHOpacityTo.new(bow, 0.35, nil, 0.0)
                                                           ),
                                            AHLocationYTo.new(arrow, ARROW_FLIGHT_DURATION, nil, ARROW_FLIGHT_DISTANCE),
                                            unpack(hitBalloonActions)
                                            ),
                                function() group:removeSelf() end
                                )

  self.hitBalloons = hitBalloons
  self.group = group
  self.action = action

  return self
end
