--- SquaresEffectFirework
-- @author Created by Aaron Barrett on January 9, 2014.

class("SquaresEffectFirework")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_BLAST_CIRCLE = 10
local Z_PARTICLES = 20

local BLAST_RADIUS = SquaresConstants.BASE_RADIUS * 3.2
local BLAST_DIAMETER = BLAST_RADIUS * 2.0

local BLAST_COLOR = {1.0, 1.0, 1.0, 0.24}

local BLAST_DURATION = 0.6

local PARTICLE_COUNT = 1024

local REMOVAL_DELAY_DURATION = 5.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function startParticles(self, group)
  local particles = self.particles
  particles:start()
  particles:surge(PARTICLE_COUNT)

  local removalAction = AHSequence.new(
                                       AHDelay.new(REMOVAL_DELAY_DURATION),
                                       AHCallLogic.new(function()
                                                           particles:stop()
                                                           group:removeSelf()
                                                         end)
                                       )
  removalAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresEffectFirework:getGroup()
  local group = self.group
  return group
end

function SquaresEffectFirework:getAction()
  local action = self.action
  return action
end

function SquaresEffectFirework:getHitBalloons()
  local hitBalloons = self.hitBalloons
  return hitBalloons
end

function SquaresEffectFirework:getDelayDuration()
  return 0.35
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createParticles(self, group)
  local particles = Particles.new("firework.pex")
  particles:setLoc(BLAST_RADIUS, BLAST_RADIUS)
  group:insert(particles, Z_PARTICLES)
  self.particles = particles
end

-- Constructor ------------------------

function SquaresEffectFirework.new(sourceBalloon, otherBalloons, popParticles)
  local self = {}
  Mixin.mix(self, SquaresEffectFirework)

  local hitBalloons = {}
  local hitBalloonActions = {}

  local sourceX, sourceY = sourceBalloon:getPosition()

  for balloon in pairs(otherBalloons) do
    local balloonX, balloonY = balloon:getPosition()
    
    local distance = MathUtils.distance(sourceX, sourceY, balloonX, balloonY)
    local distanceFromBorder = distance - balloon:getRadius()

    local didHitBalloon = (distanceFromBorder < BLAST_RADIUS)
    if (didHitBalloon) then
      table.insert(hitBalloons, balloon)

      local distanceRatio = distanceFromBorder / BLAST_RADIUS
      local delayDuration = math.pow(distanceRatio, 2.0) * BLAST_DURATION

      local hitBalloonAction = AHSequence.new(
                                              AHDelay.new(delayDuration),
                                              AHCallLogic.new(function()
                                                                  popParticles:show(balloonX, balloonY)
                                                                  local allowPowerups = false
                                                                  balloon:pop(allowPowerups)

                                                                  local popVolume = 0.5
                                                                  Sound.playSoundEffect("pop", popVolume)
                                                                end)
                                              )

      table.insert(hitBalloonActions, hitBalloonAction)
    end
  end

  local group = Group.new(BLAST_DIAMETER, BLAST_DIAMETER)
  group:setLoc(sourceX, sourceY)

  local blastCircle = FilledCircle.new(BLAST_RADIUS, BLAST_COLOR)
  blastCircle:setLoc(BLAST_RADIUS, BLAST_RADIUS)
  blastCircle:setScl(0.0)
  group:insert(blastCircle, Z_BLAST_CIRCLE)

  createParticles(self, group)

  local blastScaleAction = AHScaleTo.new(blastCircle, BLAST_DURATION, AHEaseSimpleOut.new(5.0), 1.0)
  local blastFadeAction = AHOpacityTo.new(blastCircle, BLAST_DURATION, AHEaseSimpleIn.new(2.0), 0.0)

  local action = AHSpawn.new(
                             AHCallLogic.new(function()
                                                 startParticles(self, group)
                                                 Sound.playSoundEffect("firework")
                                               end),
                             blastScaleAction,
                             blastFadeAction,
                             unpack(hitBalloonActions)
                             )

  self.hitBalloons = hitBalloons
  self.group = group
  self.action = action

  return self
end
