--- SquaresHalo
-- @author Created by Aaron Barrett on January 7, 2014.

class("SquaresHalo")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_HANDLE = 10
local Z_DASH = 20

local DASHES_COUNT = 15
local DEGREES_PER_DASH = 360.0 / DASHES_COUNT

local NORMAL_RADIUS = 50.0

local BASE_DASH_WIDTH = 18.0
local DASH_HEIGHT = 3.0

local MARGIN = 4.0

local DASH_COLOR = {1.0, 1.0, 1.0}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresHalo:getIsShowing()
  return self.isShowing
end

function SquaresHalo:createAppearAction()
  local dashes = self.dashes
  local handle = self.handle

  local dashAppearActions = {}
  local appearEase = AHEaseSimpleIn.new(1.5)
  for i, dash in ipairs(dashes) do
    local dashAppearAction = AHOpacityTo.new(dash, 0.32, appearEase, 1.0)
    table.insert(dashAppearActions, dashAppearAction)
  end

  local appearAction = AHSpawn.new(
                                   AHScaleTo.new(handle, 0.3, AHEaseBackOut.new(1.8), 1.0),
                                   unpack(dashAppearActions)
                                   )

  return appearAction
end

function SquaresHalo:createRotateAction()
  local rotateAction = AHLoop.new(
                                  AHRotationBy.new(self, 30.0, nil, 1080.0)
                                  )
  return rotateAction
end

function SquaresHalo:startAnimating()
  local appearAction = self:createAppearAction()
  self.appearAction = appearAction

  local rotateAction = self:createRotateAction()
  self.rotateAction = rotateAction

  appearAction:start()
  rotateAction:start()
end

function SquaresHalo:disappear()
  if (self.appearAction) then
    self.appearAction:stop()
  end

  local dashes = self.dashes
  local handle = self.handle

  local dashDisappearActions = {}
  local disappearEase = AHEaseSimpleOut.new(1.65)
  for i, dash in ipairs(dashes) do
    local dashFadeOutAction = AHOpacityTo.new(dash, 0.15, disappearEase, 0.0)
    local dashScaleDownAction = AHScaleYTo.new(dash, 0.15, disappearEase, 2.5)
    table.insert(dashDisappearActions, dashFadeOutAction)
    table.insert(dashDisappearActions, dashScaleDownAction)
  end

  local disappearAction = AHSequence.new(
                                         AHSpawn.new(
                                                     AHScaleTo.new(handle, 0.15, AHEaseSimpleIn.new(2.0), 1.05),
                                                     unpack(dashDisappearActions)
                                                     ),
                                         AHCallLogic.new(function()
                                                            self.rotateAction:stop()
                                                            self.rotateAction = nil
                                                            self.appearAction = nil
                                                            self.isShowing = false
                                                            self:removeSelf()
                                                          end)
                                         )
  self.appearAction = disappearAction

  return disappearAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresHalo.new(radius)
  local diameter = radius * 2.0

  local self = Group.new(diameter, diameter)
  Mixin.mix(self, SquaresHalo)

  local handle = Group.new(diameter, diameter)
  handle:setLoc(radius, radius)
  handle:setScl(0.0)

  local destinationDashRadius = radius + MARGIN + DASH_HEIGHT
  local dashWidth = BASE_DASH_WIDTH * (radius / NORMAL_RADIUS)

  local dashes = {}
  for i = 1, DASHES_COUNT do
    local dash = FilledRectangle.new(dashWidth, DASH_HEIGHT, DASH_COLOR)

    local angle = i * DEGREES_PER_DASH
    local dashAngle = angle + 90.0
    local angleInRadians = math.rad(angle)

    dash:setRot(dashAngle)

    local dashX = radius + math.cos(angleInRadians) * destinationDashRadius
    local dashY = radius + math.sin(angleInRadians) * destinationDashRadius

    dash:setLoc(dashX, dashY)

    dash:setOpacity(0.0)

    handle:insert(dash, Z_DASH)
    table.insert(dashes, dash)
  end
  self.dashes = dashes

  self:insert(handle, Z_HANDLE)

  self.handle = handle
  self.isShowing = true

  return self
end
