--- SquaresIcon
-- @author Created by Aaron Barrett on January 8, 2014.

class("SquaresIcon")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function stopActions(self)
  if (self.rotateAction) then
    self.rotateAction:stop()
    self.rotateAction = nil
  end

  if (self.pulseAction) then
    self.pulseAction:stop()
    self.pulseAction = nil
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresIcon:createPulseAction()
  local pulseAction = AHLoop.new(
                                 AHSequence.new(
                                                AHScaleTo.new(self, 0.40, AHEaseSimpleInOut.new(2.0), 0.75),
                                                AHScaleTo.new(self, 0.40, AHEaseSimpleInOut.new(2.0), 1.0)
                                                )
                                 )
  return pulseAction
end

function SquaresIcon:createRotateAction()
  local rotateAction = AHLoop.new(
                                  AHRotationBy.new(self, 20.0, nil, -1080.0)
                                  )
  return rotateAction
end

function SquaresIcon:stopAnimating()
  stopActions(self)
end

function SquaresIcon:startAnimating()
  local rotateAction = self:createRotateAction()
  self.rotateAction = rotateAction
  rotateAction:start()

  local powerupFlavor = self.powerupFlavor
  local doesFlavorUsePulse = (powerupFlavor == SquaresPowerup.FLAVOR_SHRINKER)
  if (doesFlavorUsePulse) then
    local pulseAction = self:createPulseAction()
    self.pulseAction = pulseAction
    pulseAction:start()
  end
end

function SquaresIcon:activate()
  stopActions(self)

  local activateAction = nil
  local fadeOutAction = AHOpacityTo.new(self, 0.75, AHEaseExponentialOut.new(1.6), 0.0)

  local powerupFlavor = self.powerupFlavor
  local shouldJustFadeOut = (powerupFlavor == SquaresPowerup.FLAVOR_FIREWORK)
  if (shouldJustFadeOut) then
    activateAction = fadeOutAction
  else
    local scaleUpAction = AHScaleTo.new(self, 0.75, AHEaseBackOut.new(0.26), 5.0)

    activateAction = AHSpawn.new(
                                 fadeOutAction,
                                 scaleUpAction
                                 )

    local shouldSpin = (powerupFlavor ~= SquaresPowerup.FLAVOR_BOW)
    if (shouldSpin) then
      local spinAction = AHRotationBy.new(self, 0.75, AHEaseSimpleIn.new(2.2), 210.0)
      activateAction:addAction(spinAction)
    end
  end

  activateAction:start()
end

function SquaresIcon:disappear()
  stopActions(self)

  local fadeOutAction = AHOpacityTo.new(self, 0.15, nil, 0.0)
  fadeOutAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresIcon.new(powerupFlavor)
  local frameName = "icon_" .. powerupFlavor .. ".png"

  local self = Sprite.newWithName(frameName)
  Mixin.mix(self, SquaresIcon)

  self.powerupFlavor = powerupFlavor

  return self
end
