--- SquaresMatchTracker
-- @author Created by Aaron Barrett on November 03, 2021.

class("SquaresMatchTracker")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function convertToSortedConceptsArray(recordsByIdentifier)
  local records = TableUtils.getValues(recordsByIdentifier)

  table.sort(records, function(recordA, recordB)
    return (recordA:getEventOrderIndex() > recordB:getEventOrderIndex())
  end)

  return TableUtils.map(records, function(record)
    return record:getConcept()
  end)
end

local function incrementEventOrderIndex(self)
  self.eventOrderIndex = self.eventOrderIndex + 1
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresMatchTracker:notifyConceptPresented(concept)
  local identifier = concept.identifier

  local existingRecord = self.unmatchedRecordsByIdentifier[identifier]

  if (existingRecord == nil) then
    existingRecord = self.matchedRecordsByIdentifier[identifier]
  end

  if (existingRecord == nil) then
    local record = SquaresMatchRecord.new(concept, self.eventOrderIndex)
    self.unmatchedRecordsByIdentifier[identifier] = record

    incrementEventOrderIndex(self)
  end
end

function SquaresMatchTracker:notifyCorrectAnswer(concept)
  local identifier = concept.identifier

  local existingRecord = self.unmatchedRecordsByIdentifier[identifier]
  if (existingRecord ~= nil) then
    self.matchedRecordsByIdentifier[identifier] = existingRecord
    self.unmatchedRecordsByIdentifier[identifier] = nil

    incrementEventOrderIndex(self)
  end
end

function SquaresMatchTracker:getMatchedConcepts()
  return convertToSortedConceptsArray(self.matchedRecordsByIdentifier)
end

function SquaresMatchTracker:getUnmatchedConcepts()
  return convertToSortedConceptsArray(self.unmatchedRecordsByIdentifier)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresMatchTracker.new()
  local self = {}
  Mixin.mix(self, SquaresMatchTracker)

  self.matchedRecordsByIdentifier = {}
  self.unmatchedRecordsByIdentifier = {}

  self.eventOrderIndex = 0

  return self
end
