--- SquaresPauseButtonTouchGuard
-- @author Created by Aaron Barrett on November 18, 2021.

-- The purpose of this class is to intercept touch events that fall within the
-- pause button's touch area so that they don't interact with the touch handlers
-- for the background or for any balloons that might be positioned behind the
-- pause button.  This is needed because the background and the balloons
-- are all using `TouchHandler.BEHAVIOR_ALWAYS`, so the pause button (which is
-- using `TouchHandler.BEHAVIOR_FOCUS`) will not block touch events from hitting
-- the touch handlers behind it.  The touch guard simply calls a method on
-- `SquaresFocusTracker` to set its `hasTouchBeenIntercepted` flag to `true`
-- so that the focus tracker knows not to process the touch event.

class("SquaresPauseButtonTouchGuard")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresPauseButtonTouchGuard.new(pauseButtonTouchAreaWidth, pauseButtonTouchAreaHeight, onTouchGuardTouched)
  local self = Group.new(pauseButtonTouchAreaWidth, pauseButtonTouchAreaHeight)
  Mixin.mix(self, SquaresPauseButtonTouchGuard)

  TouchManager.addTouchEnabledMethods(self, onTouchGuardTouched, TouchHandler.BEHAVIOR_ALWAYS)
  self:setTouchEnabled(true)

  return self
end
