--- SquaresPhysics
-- @author Created by Aaron Barrett on December 16, 2013.

class("SquaresPhysics")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GRAVITY = 1.4
local UNITS_TO_METERS_RATIO = 1.0 / 64.0

local VELOCITY_ITERATIONS = 1
local POSITION_ITERATIONS = 1

local INITIAL_BALLOON_VELOCITY = 265.6
local BALLOON_LINEAR_DAMPING = 0.1

local WALL_THICKNESS = Screen.height * 0.075 + SquaresBalloon.OVERLAP_AMOUNT
local WALL_TOP = Screen.safeTop - SquaresBalloon.OVERLAP_AMOUNT

local WALL_RIGHT = Screen.width - SquaresBalloon.OVERLAP_AMOUNT
local WALL_LEFT = SquaresBalloon.OVERLAP_AMOUNT
local WALL_BOTTOM = -Screen.height

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresPhysics:destroy()
  local world = self.world
  world:stop()
end

function SquaresPhysics:setPaused(isPaused)
  local world = self.world
  world:pause(isPaused)
end

function SquaresPhysics:addBalloon(balloon)
  local world = self.world

  local radius = balloon:getRadius()
  local effectiveRadius = radius + SquaresBalloon.OVERLAP_AMOUNT * 2.0

  local body = world:addBody(MOAIBox2DBody.DYNAMIC)
  body:setFixedRotation(true)

  balloon:setBody(body)
  balloon:resetFixtureWithRadius(radius)

  local x = effectiveRadius * 1.5 + math.random() * (Screen.width - effectiveRadius * 3.0)
  local y = -effectiveRadius
  body:setTransform(x, y)
  body:setLinearVelocity(0.0, INITIAL_BALLOON_VELOCITY)
  body:setLinearDamping(BALLOON_LINEAR_DAMPING)

  balloon:setAttrLink(MOAITransform.INHERIT_TRANSFORM, body, MOAITransform.TRANSFORM_TRAIT)
end

function SquaresPhysics:addScanZone(xLeft, xRight, yBottom, yTop)
  local wallsBody = self.wallsBody

  local scanZone = wallsBody:addRect(xLeft, yTop, xRight, yBottom)
  scanZone:setSensor(true)

  return scanZone
end

function SquaresPhysics:removeBalloon(balloon)
  local body = balloon:getBody()
  body:destroy()

  balloon:setBody(nil)
end

function SquaresPhysics:start()
  local world = self.world
  world:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createWorld(self)
  local world = MOAIBox2DWorld.new()
  world:setIterations(VELOCITY_ITERATIONS, POSITION_ITERATIONS)
  world:setGravity(0, GRAVITY)
  world:setUnitsToMeters(UNITS_TO_METERS_RATIO)

  local wallsBody = world:addBody(MOAIBox2DBody.STATIC)
  wallsBody:addRect(WALL_LEFT, WALL_TOP, WALL_RIGHT, WALL_TOP + WALL_THICKNESS)
  wallsBody:addRect(WALL_LEFT - WALL_THICKNESS, WALL_BOTTOM, WALL_LEFT, WALL_TOP)
  wallsBody:addRect(WALL_RIGHT, WALL_BOTTOM, WALL_RIGHT + WALL_THICKNESS, WALL_TOP)

  self.world = world
  self.wallsBody = wallsBody
end

-- Constructor ------------------------

function SquaresPhysics.new()
  local self = {}
  Mixin.mix(self, SquaresPhysics)

  createWorld(self)

  return self
end
