--- SquaresScanner
-- @author Created by Aaron Barrett on January 17, 2014.

class("SquaresScanner")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.width
local HEIGHT = Screen.safeTop
local HEIGHT_TO_SCAN = HEIGHT * 1.38

local BASE_Y = HEIGHT - HEIGHT_TO_SCAN

local SCAN_ROW_COUNT = 6
local SCAN_COLUMN_COUNT = 4

local SCAN_ZONE_COUNT = SCAN_ROW_COUNT * SCAN_COLUMN_COUNT

local SCAN_ZONE_HEIGHT = HEIGHT_TO_SCAN / SCAN_ROW_COUNT
local SCAN_ZONE_WIDTH = WIDTH / SCAN_COLUMN_COUNT

local BALLOONS_NEEDED_PER_SCAN_ZONE = 1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresScanner:destroy()
  self.onGameEndCallback = function() end
end

function SquaresScanner:handleScanZoneCollision(scanZone, phase)
  local originalBalloonCount = scanZone.balloonCount
  local originalFilledZoneCount = self.filledZoneCount

  local updatedBalloonCount = nil
  local updatedFilledZoneCount = originalFilledZoneCount
  if (phase == MOAIBox2DArbiter.BEGIN) then
    updatedBalloonCount = originalBalloonCount + 1

    if ((originalBalloonCount < BALLOONS_NEEDED_PER_SCAN_ZONE) and (updatedBalloonCount >= BALLOONS_NEEDED_PER_SCAN_ZONE)) then
      updatedFilledZoneCount = updatedFilledZoneCount + 1
    end
  elseif (phase == MOAIBox2DArbiter.END) then
    updatedBalloonCount = originalBalloonCount - 1

    if ((originalBalloonCount >= BALLOONS_NEEDED_PER_SCAN_ZONE) and (updatedBalloonCount < BALLOONS_NEEDED_PER_SCAN_ZONE)) then
      updatedFilledZoneCount = updatedFilledZoneCount - 1
    end
  else
    error("Unhandled collision phase")
  end

  if (updatedFilledZoneCount ~= originalFilledZoneCount) then
    local filledProportion = updatedFilledZoneCount / SCAN_ZONE_COUNT
    self.background:transitionToFilledProportion(filledProportion)

    if (updatedFilledZoneCount >= SCAN_ZONE_COUNT) then
      self.onGameEndCallback()
    end
  end

  scanZone.balloonCount = updatedBalloonCount
  self.filledZoneCount = updatedFilledZoneCount
end

function SquaresScanner:startScanning()
  local phasesToHandle = MOAIBox2DArbiter.BEGIN + MOAIBox2DArbiter.END

  local scanZones = {}
  for _, descriptor in ipairs(self.scanZoneDescriptors) do
    local scanZone = self.physics:addScanZone(unpack(descriptor))
    scanZone.balloonCount = 0

    local function onSensorCollision(phase)
      runInCoroutine(function()
        self:handleScanZoneCollision(scanZone, phase)
      end)
    end

    scanZone:setCollisionHandler(onSensorCollision, phasesToHandle)
    table.insert(scanZones, scanZone)
  end
  self.scanZones = scanZones
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresScanner.new(physics, background, onGameEndCallback)
  local self = {}
  Mixin.mix(self, SquaresScanner)

  self.physics = physics
  self.background = background
  self.onGameEndCallback = onGameEndCallback

  local scanZoneDescriptors = {}
  for row = 1, SCAN_ROW_COUNT do
    local yBottom = BASE_Y + (row - 1) * SCAN_ZONE_HEIGHT
    local yTop = yBottom + SCAN_ZONE_HEIGHT

    for column = 1, SCAN_COLUMN_COUNT do
      local xLeft = (column - 1) * SCAN_ZONE_WIDTH
      local xRight = xLeft + SCAN_ZONE_WIDTH

      table.insert(scanZoneDescriptors, {xLeft, xRight, yBottom, yTop})
    end
  end
  self.scanZoneDescriptors = scanZoneDescriptors

  self.filledZoneCount = 0

  return self
end
