--- SquaresScoreCard
-- @author Created by Aaron Barrett on November 04, 2021.

class("SquaresScoreCard")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_SCROLLER = 10
local Z_FLOATING_HEADER_OVERLAY = 20
local Z_FOOTER = 30

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playFadeIn(self)
  return AHOpacityTo.new(self, 0.333, nil, 1.0):start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresScoreCard:playShow()
  return runInCoroutine(function()
    wait(playFadeIn(self))
    self.floatingHeaderOverlay:addTopMarginCover()
    wait(self.scroller:playSlideIn())
    self.scroller:setTouchEnabled(true)
    waitFor(0.17)
    wait(self.footer:playSlideIn())
  end)
end

function SquaresScoreCard:isDismissed()
  return self.hasContinueButtonBeenPressed
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildFooter(self)
  local function onContinueButtonPressedCallback()
    self.hasContinueButtonBeenPressed = true
  end

  local footer = SquaresScoreFooter.new(onContinueButtonPressedCallback)
  footer:setX(self:getWidthPart(0.5))
  local offScreenY = -footer:getHeightPart(0.5)
  footer:setY(offScreenY)

  self:insert(footer, Z_FOOTER)
  self.footer = footer
end

local function buildScroller(self, matchedConcepts, unmatchedConcepts)
  local scrollerVerticalOffset = self.footer:getHeight()
  local remainingY = self:getHeight() - scrollerVerticalOffset
  local scroller = SquaresScoreScroller.new(self:getWidth(), remainingY, matchedConcepts, unmatchedConcepts)
  scroller:setAnchorPointBottomLeft()
  scroller:setLoc(xOffset, scrollerVerticalOffset)
  self:insert(scroller, Z_SCROLLER)
  scroller:setTouchEnabled(false)

  self.scroller = scroller
end

local function configureFloatingHeaders(self)
  local function onScrollUpdatedCallback(scrollY)
    self.floatingHeaderOverlay:updateScrollY(scrollY)
  end
  self.scroller:setOnScrollUpdatedCallback(onScrollUpdatedCallback)

  local contentsColumn = self.scroller:getContentsColumn()

  local headers = contentsColumn:getHeaders()
  local headerOffsetY = contentsColumn:getY() * 0.5 + self.footer:getHeight()

  self.floatingHeaderOverlay:configureHeaders(headers, headerOffsetY)
end

local function buildFloatingHeaderOverlay(self)
  local overlayHeight = self:getHeight() - self.footer:getHeight() + (Screen.fullHeight - Screen.safeTop)
  local overlay = SquaresFloatingHeaderOverlay.new(self:getWidth(), overlayHeight)
  overlay:setX(self:getWidthPart(0.5))
  overlay:setY(self:getHeight() - overlayHeight * 0.5)

  self:insert(overlay, Z_FLOATING_HEADER_OVERLAY)
  self.floatingHeaderOverlay = overlay

  configureFloatingHeaders(self)
end

-- Constructor ------------------------

function SquaresScoreCard.new(matchedConcepts, unmatchedConcepts)
  local self = FilledRectangle.newFullScreen()
  Mixin.mix(self, SquaresScoreCard)

  self:setColor(unpack(SquaresColors.SCORE_CARD_BACKGROUND_COLOR))
  self:setOpacity(0.0)

  buildFooter(self)
  buildScroller(self, matchedConcepts, unmatchedConcepts)
  buildFloatingHeaderOverlay(self)

  self.hasContinueButtonBeenPressed = false

  return self
end
