--- SquaresScoreColumn
-- @author Created by Aaron Barrett on November 08, 2021.

class("SquaresScoreColumn")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CELL_SPACING = Screen.getHeightPart(0.03)
local SECTION_SPACING = CELL_SPACING * 0.25

local MATCHED_ANTONYMS_TEXT = "MATCHED ANTONYMS"
local UNMATCHED_ANTONYMS_TEXT = "UNMATCHED ANTONYMS"


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function prepareElementsToSlideIn(self)
  for _, element in ipairs(self.slidingElements) do
    element:prepareToSlideIn()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresScoreColumn:getSlidingElements()
  return self.slidingElements
end

function SquaresScoreColumn:getHeaders()
  return self.headers
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function addToSelfAndSlidingElements(self, element)
  self:insert(element)
  table.insert(self.slidingElements, element)
end

local function buildHeader(self, elementWidth, headerText)
  local header = SquaresScoreHeader.new(elementWidth, headerText)
  addToSelfAndSlidingElements(self, header)
  table.insert(self.headers, header)
end

local function buildCellSection(self, elementWidth, concepts, isMatched)
  local conceptsCount = #concepts
  for conceptIndex, concept in ipairs(concepts) do
    local cell = SquaresScoreCell.new(elementWidth, concept, isMatched)
    addToSelfAndSlidingElements(self, cell)

    local isLastCell = (conceptIndex == conceptsCount)
    if (not isLastCell) then
      self:insertSpacer(CELL_SPACING)
    end
  end
end

-- Constructor ------------------------

function SquaresScoreColumn.new(width, matchedConcepts, unmatchedConcepts)
  local self = Column.new(0.0, Alignment.CENTER)
  Mixin.mix(self, SquaresScoreColumn)

  local elementWidth = width * 0.87

  self.slidingElements = {}
  self.headers = {}

  self:insertSpacer(CELL_SPACING)

  buildHeader(self, elementWidth, MATCHED_ANTONYMS_TEXT)
  local isMatched = true
  buildCellSection(self, elementWidth, matchedConcepts, isMatched)

  local unmatchedConceptsCount = #unmatchedConcepts
  if (unmatchedConceptsCount > 0) then
    self:insertSpacer(SECTION_SPACING)
    buildHeader(self, elementWidth, UNMATCHED_ANTONYMS_TEXT)
    isMatched = false
    buildCellSection(self, elementWidth, unmatchedConcepts, isMatched)
  end

  self:insertSpacer(CELL_SPACING)

  self:alignChildren()

  prepareElementsToSlideIn(self)

  return self
end
