--- SquaresScoreContinueButton
-- @author Created by Aaron Barrett on November 04, 2021.

class("SquaresScoreContinueButton")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LABEL_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 20.0)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresScoreContinueButton.new(onPressedCallback)
  local width = Screen.getWidthPart(0.85)
  local height = width * 0.15
  local cornerRadius = height * 0.1

  local self = FilledRoundedRectangle.new(width, height, cornerRadius)
  Mixin.mix(self, SquaresScoreContinueButton)

  self:setColor(unpack(SquaresColors.SCORE_CARD_YELLOW_COLOR))

  local label = Label.new("End Game", LABEL_FONT)
  label:setColor(unpack(SquaresColors.WHITE_TEXT_COLOR))

  self:insertAtCenter(label)

  self.onPressedCallback = onPressedCallback

  Button.makeButton(self, function()
    self:setTouchEnabled(false)
    self:setColor(unpack(SquaresColors.WHITE_TEXT_COLOR))
    AHSequence.new(
      -- The label is already white, so this fade out does nothing.
      -- This is a fix for the button flickering as the game ends.
      -- When MOAI stops updating, it continues to alternate between the two frames in
      -- it's frame buffer.
      -- By having a short delay where an animation is happening, we're
      -- filling the frame buffer with the same screen, instead of having
      -- half the buffer have the button in the upstate, causing the flicker.
      AHOpacityTo.new(label, 0.1, nil, 0),
      self.onPressedCallback
    ):start()
  end)

  return self
end
