--- SquaresScoreDefinition
-- @author Created by Aaron Barrett on November 06, 2021.

class("SquaresScoreDefinition")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TERM_FONT = Font.new(FontFaces.DIN_OT_BOLD, 20.0)
local TERM_OPACITY = 0.9

local MEANING_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 14.0)
local MEANING_OPACITY = 0.6

local LEFT_TERM_KEY = "kind_a"
local RIGHT_TERM_KEY = "kind_b"
local LEFT_MEANING_KEY = "definition_a"
local RIGHT_MEANING_KEY = "definition_b"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function getTermKey(isForRightColumn)
  if (isForRightColumn) then
    return RIGHT_TERM_KEY
  else
    return LEFT_TERM_KEY
  end
end

local function getMeaningKey(isForRightColumn)
  if (isForRightColumn) then
    return RIGHT_MEANING_KEY
  else
    return LEFT_MEANING_KEY
  end
end

local function getTermText(concept, isForRightColumn)
  local key = getTermKey(isForRightColumn)
  local text = concept[key]
  assert(text ~= nil, "Missing word field for key '" .. tostring(key) .. "'")
  return text
end

local function getMeaningText(concept, isForRightColumn)
  local key = getMeaningKey(isForRightColumn)
  local text = concept[key]
  assert(text ~= nil, "Missing definition field for key '" .. tostring(key) .. "'")
  return StringUtils.capitalizeFirstLetter(text)
end

local function getAlignment(isForRightColumn)
  if (isForRightColumn) then
    return Alignment.LEFT
  else
    return Alignment.RIGHT
  end
end

local function buildTermLabel(self, termText)
  local termLabel = Label.new(termText, TERM_FONT)
  termLabel:setColor(unpack(SquaresColors.WHITE_TEXT_COLOR))
  termLabel:setOpacity(TERM_OPACITY)
  self:insert(termLabel)
end

local function buildMeaningLabel(self, width, meaningText, alignment)
  local meaningLabel = DynamicTextArea.newSimple(meaningText, width, MEANING_FONT, alignment)
  meaningLabel:setColor(unpack(SquaresColors.WHITE_TEXT_COLOR))
  meaningLabel:setOpacity(MEANING_OPACITY)
  self:insert(meaningLabel)
end

-- Constructor ------------------------

function SquaresScoreDefinition.new(width, concept, isForRightColumn)
  local alignment = getAlignment(isForRightColumn)

  local self = Column.new(nil, alignment)
  Mixin.mix(self, SquaresScoreDefinition)

  local termText = getTermText(concept, isForRightColumn)
  buildTermLabel(self, termText)

  local meaningText = getMeaningText(concept, isForRightColumn)
  buildMeaningLabel(self, width, meaningText, alignment)

  self:alignChildren()

  return self
end
