--- SquaresScoreFooter
-- @author Created by Aaron Barrett on November 06, 2021.

class("SquaresScoreFooter")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.width

local HEIGHT = Screen.getWidthPart(0.4) + Screen.safeBottom

local TITLE_LABEL_TOP_MARGIN = HEIGHT * 0.1
local CONTINUE_BUTTON_BOTTOM_MARGIN = TITLE_LABEL_TOP_MARGIN

local TITLE_TEXT = "GAME WON!"
local TITLE_LABEL_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 32.0)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresScoreFooter:playSlideIn()
  return AHLocationYBy.new(self, 0.5, AHEaseBezier.newFlat({0.0, 0.0, 0.2, 1.0}), HEIGHT):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildTitleLabel(self)
  local titleLabel = Label.new(TITLE_TEXT, TITLE_LABEL_FONT)
  titleLabel:setColor(unpack(SquaresColors.SCORE_CARD_YELLOW_COLOR))
  titleLabel:setX(self:getWidthPart(0.5))
  titleLabel:setY(self:getHeight() - TITLE_LABEL_TOP_MARGIN - titleLabel:getHeightPart(0.5))
  self:insert(titleLabel)
  self.titleLabel = titleLabel
end

local function buildContinueButton(self, onContinueButtonPressedCallback)
  local button = SquaresScoreContinueButton.new(onContinueButtonPressedCallback)
  button:setX(self:getWidthPart(0.5))
  button:setY(0.5 * (self.titleLabel:getYBottom() + Screen.safeBottom + CONTINUE_BUTTON_BOTTOM_MARGIN))
  self:insert(button)
end

-- Constructor ------------------------

function SquaresScoreFooter.new(onContinueButtonPressedCallback)
  local self = FilledRectangle.new(WIDTH, HEIGHT)
  Mixin.mix(self, SquaresScoreFooter)

  self:setColor(unpack(SquaresColors.SCORE_CARD_FOOTER_BACKGROUND_COLOR))

  buildTitleLabel(self)
  buildContinueButton(self, onContinueButtonPressedCallback)
  
  return self
end
