--- SquaresScoreHeader
-- @author Created by Aaron Barrett on November 08, 2021.

class("SquaresScoreHeader")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HEIGHT = 60.0 + (Screen.fullHeight - Screen.safeTop)

local UNDERLINE_THICKNESS = 0.5

local LABEL_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 18.0)

local TEXT_OPACITY = 0.9

local Z_BACKGROUND = 10
local Z_LABEL = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playSlideYAction(self, destinationY)
  self.interrupt:insert(AHLocationYTo.new(self, SquaresScoreSlidingElement.SLIDE_DURATION, AHEaseBezier.newFlat({0.0, 0.0, 0.2, 1.0}), destinationY))
end

local function playHide(self)
  playSlideYAction(self, self.offScreenY)
end

local function playShow(self)
  playSlideYAction(self, self.topY)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresScoreHeader:setHomeY(homeY)
  self.homeY = homeY
  self:setY(homeY)
end

function SquaresScoreHeader:setTopY(topY)
  self.topY = topY
end

function SquaresScoreHeader:setOffScreenY(offScreenY)
  self.offScreenY = offScreenY
end

function SquaresScoreHeader:getHomeY()
  return self.homeY
end

function SquaresScoreHeader:getLabelHeight()
  return self.label:getHeight()
end

function SquaresScoreHeader:getState()
  return self.state
end

function SquaresScoreHeader:initializeState(isTopHeader)
  if (isTopHeader) then
    self.state = SquaresHeaderStates.ACTIVE
  else
    self.state = SquaresHeaderStates.VISIBLE
  end
end

function SquaresScoreHeader:setState(state)
  if (self.state ~= state) then
    if (state == SquaresHeaderStates.HIDDEN) then
      playHide(self)
    elseif ((state == SquaresHeaderStates.ACTIVE) or (state == SquaresHeaderStates.VISIBLE)) then
      playShow(self)
    else
      error("Unhandled header state: '" .. tostring(state) .. "'")
    end
    self.state = state
  end
end

function SquaresScoreHeader:stopHideOrShowAction()
  self.interrupt:stop()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresScoreHeader.new(width, text)
  local self = Group.new(width, HEIGHT)
  Mixin.mix(self, SquaresScoreSlidingElement)
  Mixin.mix(self, SquaresScoreHeader)

  local label = Label.new(text, LABEL_FONT)
  label:setColor(unpack(SquaresColors.WHITE_TEXT_COLOR))
  label:setOpacity(TEXT_OPACITY)
  self:insertAtCenter(label, Z_LABEL)
  self.label = label

  local underline = FilledRectangle.new(label:getWidth(), UNDERLINE_THICKNESS)
  underline:setColor(unpack(SquaresColors.WHITE_TEXT_COLOR))
  underline:setOpacity(TEXT_OPACITY)
  underline:setX(label:getX())
  underline:setY(label:getYBottom())
  self:insert(underline, Z_LABEL)

  local background = FilledRectangle.new(Screen.width, label:getHeight())
  background:setColor(unpack(SquaresColors.SCORE_CARD_BACKGROUND_COLOR))
  self:insertAtCenter(background, Z_BACKGROUND)

  local isRecursive = true
  self:linkColorAttrToChildren(isRecursive)
  self:setOpacity(0.0)

  self.interrupt = ActionInterrupt.new()

  self.homeY = nil
  self.topY = nil
  self.offScreenY = nil
  self.state = nil

  return self
end
