--- SquaresScoreScroller
-- @author Created by Aaron Barrett on November 04, 2021.

class("SquaresScoreScroller")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CELL_SPACING = Screen.getHeightPart(0.03)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playScrollToBottomOfScroller(self)
  local maxScroll = math.max(0.0, self.scroller:getContents():getHeight() - self.scroller:getHeight())
  self.scroller:newScrollYTo(SquaresScoreSlidingElement.SLIDE_DURATION, AHEaseBezier.newFlat({0.0, 0.0, 0.2, 1.0}), maxScroll):start()
end

local function playScrollDownToShowElement(self, element)
  local elementBottom = element:getY() - element:getHeightPart(0.5)
  if (self.contentsColumn:getHeight() - elementBottom > self.scroller:getHeight()) then
    self.scroller:newScrollYBy(SquaresScoreSlidingElement.SLIDE_DURATION, AHEaseBezier.newFlat({0.0, 0.0, 0.2, 1.0}), element:getHeight() + CELL_SPACING):start()
  end
end

local function slideInNextElement(self)
  return runInCoroutine(function()
    Sound.playSoundEffect("score_card_slide")
    self.slidingElementIndex = self.slidingElementIndex + 1
    local slidingElementsCount = #(self.slidingElements)
    if (self.slidingElementIndex <= slidingElementsCount) then
      local element = self.slidingElements[self.slidingElementIndex]

      local isLastElement = (self.slidingElementIndex == slidingElementsCount)
      local isIndexPastMinimumToPreventOverlap = (self.slidingElementIndex > 2)
      if (isLastElement) then
        playScrollToBottomOfScroller(self)
      elseif (isIndexPastMinimumToPreventOverlap) then
        playScrollDownToShowElement(self, element)
      end

      wait(element:playSlideIn())
    end
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresScoreScroller:playSlideIn()
  return runInCoroutine(function()
    for _, element in ipairs(self.slidingElements) do
      wait(slideInNextElement(self))
    end
  end)
end

function SquaresScoreScroller:setTouchEnabled(enabled)
  self.scroller:setTouchEnabled(enabled)
end

function SquaresScoreScroller:setOnScrollUpdatedCallback(onScrollUpdatedCallback)
  self.scroller:setOnScrollUpdatedCallback(onScrollUpdatedCallback)
end

function SquaresScoreScroller:getContentsColumn()
  return self.contentsColumn
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function buildScroller(self)
  self.scroller = TouchScroller.new(self:getWidth(), self:getHeight(), self.contentsColumn, onScrollUpdatedCallback)
  self.scroller:setInertialScrollingEnabled(true)
  self.scroller:setWillResetInitialTouchCoordinateUponScrollStart(true)
  self:insertAtCenter(self.scroller)
end

-- Constructor ------------------------

function SquaresScoreScroller.new(width, height, matchedConcepts, unmatchedConcepts)
  local self = Group.new(width, height)
  Mixin.mix(self, SquaresScoreScroller)

  self.contentsColumn = SquaresScoreColumn.new(width, matchedConcepts, unmatchedConcepts)
  self.slidingElements = self.contentsColumn:getSlidingElements()
  buildScroller(self)

  self.slidingElementIndex = 0

  return self
end
