--- SquaresScrollMotionMeasurer
-- @author Created by Aaron Barrett on November 18, 2021.

-- Takes a collection of snapshots of the `deltaScrollY` over the course of
-- many frames and averages them to measure the overall direction that a
-- touch scroller is scrolling in, rather than just reading the most recent
-- `deltaScrollY`, which fluctuates a lot.

class("SquaresScrollMotionMeasurer")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SNAPSHOTS_COUNT = 25

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresScrollMotionMeasurer:takeSnapshot(deltaScrollY)
  table.insert(self.snapshots, deltaScrollY)
  table.remove(self.snapshots, 1)
end

function SquaresScrollMotionMeasurer:getCurrentModeratedScrollSpeed()
  local total = 0.0
  for _, snapshot in ipairs(self.snapshots) do
    total = total + snapshot
  end
  return total / SNAPSHOTS_COUNT
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresScrollMotionMeasurer.new()
  local self = {}
  Mixin.mix(self, SquaresScrollMotionMeasurer)

  self.snapshots = {}
  for snapshotIndex = 1, SNAPSHOTS_COUNT do
    table.insert(self.snapshots, 0.0)
  end

  return self
end
