--- SquaresSnowMachine
-- @author Created by Aaron Barrett on January 9, 2014.

class("SquaresSnowMachine")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FREEZE_DURATION = 5.0

local UNFREEZE_DURATION = 0.2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function startSnowing(self)
  self:startSystem()
  self.isSnowing = true
end

local function stopSnowing(self)
  self:stopSystem()
  self.isSnowing = false
  Sound.playSoundEffect("unfreeze")
end

local function getRemainingDuration(self)
  local timerAction = self.timerAction
  if (timerAction) then
    local remainingDuration = timerAction.duration - timerAction:getTime()
    return remainingDuration
  end

  return 0.0
end

local function startTimerAndResumer(self, duration)
  local timerAction = AHDelay.new(duration)
  timerAction.duration = duration
  local resumerAction = AHSequence.new(
                                       timerAction,
                                       AHCallLogic.new(function()
                                           self.timerAction = nil
                                           stopSnowing(self)
                                           self.onResumeCallback()
                                         end)
                                       )

  self.timerAction = timerAction
  self.resumerAction = resumerAction
  resumerAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresSnowMachine:destroy()
  self:stop()
  if (self.resumerAction) then
    self.resumerAction:stop()
  end
end

function SquaresSnowMachine:getIsSnowing()
  local isSnowing = self.isSnowing
  return isSnowing
end

function SquaresSnowMachine:remainFrozenForExtraDuration(durationToExtend)
  local remainingDuration = getRemainingDuration(self)

  if (remainingDuration > 0.0 and durationToExtend > 0.0) then
    self.resumerAction:stop()

    local duration = remainingDuration + durationToExtend
    startTimerAndResumer(self, duration)
  end

end

function SquaresSnowMachine:unfreeze()
  self.resumerAction:stop()

  local duration = UNFREEZE_DURATION
  startTimerAndResumer(self, duration)
end

function SquaresSnowMachine:freeze()
  local duration = FREEZE_DURATION

  local remainingDuration = getRemainingDuration(self)
  local wasResumerAlreadyRunning = (remainingDuration > 0.0)
  duration = duration + remainingDuration

  if (self.resumerAction) then
    self.resumerAction:stop()
  end

  if (not(wasResumerAlreadyRunning)) then
    startSnowing(self)
    self.onFreezeCallback()
  end

  startTimerAndResumer(self, duration)

  Sound.playSoundEffect("freeze")
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function SquaresSnowMachine.new(onFreezeCallback, onResumeCallback)
  local self = Particles.new("snow.pex")
  Mixin.mix(self, SquaresSnowMachine)

  self:start()

  self.onFreezeCallback = onFreezeCallback
  self.onResumeCallback = onResumeCallback

  self.timerAction = nil
  self.resumerAction = nil
  self.isSnowing = false

  return self
end
