--- SquaresValidator
-- @author Created by Anton Morant on April 17, 2014.
-- ConceptValidator for Squares.

class("SquaresValidator")

require("validators.ConceptValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  "kind_a",
  "kind_b",
  "definition_a",
  "definition_b"
}

local MAXIMUM_WORD_LENGTH = 13

local ACCEPTABLE_CHARACTER_SET = TableUtils.createSetFromString("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ -")

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validateFieldCanBeCapitalized(self, field)
  self:expectNoError(function()
    StringUtils.capitalizeFirstLetter(field)
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SquaresValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)

  local word = concept.kind_a
  local antonym = concept.kind_b
  local definitionA = concept.definition_a
  local definitionB = concept.definition_b

  self:expectStringCharactersBelongToSet(word, ACCEPTABLE_CHARACTER_SET, "word")
  self:expectStringCharactersBelongToSet(antonym, ACCEPTABLE_CHARACTER_SET, "antonym")
  self:expectLessOrEqual(#word, MAXIMUM_WORD_LENGTH, string.format("length of word (%s)", word))
  self:expectLessOrEqual(#antonym, MAXIMUM_WORD_LENGTH, string.format("length of antonym (%s)", antonym))
  self:expectNotEmpty(definitionA, "definitionA", "definition_a is empty")
  self:expectNotEmpty(definitionB, "definitionB", "definition_b is empty")

  validateFieldCanBeCapitalized(self, definitionA)
  validateFieldCanBeCapitalized(self, definitionB)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SquaresValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, SquaresValidator)

  return self
end
