--- SummationAnswerPuck
-- @author Created by Jay Austria on August 2, 2014.

class("SummationAnswerPuck")
require("SummationColorUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SummationAnswerPuck.PUCK_OPACITY_START = 0.10
SummationAnswerPuck.PUCK_OPACITY_END = 0.39
SummationAnswerPuck.PUCK_INITIAL_COLOR = SummationCommon.PARTIALLY_CORRECT_COLOR
SummationAnswerPuck.PUCK_HEIGHT = 8
SummationAnswerPuck.PUCK_WIDTH = 8

local FLAG_OFFSET_X = Screen.getWidthPart(0.1)
local GLOW_RADIUS = 16
local GLOW_OPACITY = 0.2
local FADE_OUT_DURATION = 0.4


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationAnswerPuck:getLaunchAction(actionDuration, destinationY, value)
  local launchAction = AHSpawn.new(
    function() self:setOpacity(SummationAnswerPuck.PUCK_OPACITY_END) end,
    self.flag:createValueToAction(actionDuration, AHEaseSineInOut.new(), value),
    AHLocationYTo.new(self, actionDuration, AHEaseSineInOut.new(), destinationY)
  )

  return launchAction
end

function SummationAnswerPuck:showAsIncorrect()
  self.puckRectangle:setColor(unpack(SummationCommon.INCORRECT_COLOR))
  self.outerGlow:setColor(unpack(SummationCommon.INCORRECT_COLOR))
  self.outerGlow:setOpacity(GLOW_OPACITY)
  self.flag:showAsIncorrect()
end

function SummationAnswerPuck:showAsCorrect()
  self.puckRectangle:setColor(unpack(SummationCommon.WHITE_COLOR))
  self.outerGlow:setColor(unpack(SummationCommon.WHITE_COLOR))
  self.outerGlow:setOpacity(GLOW_OPACITY)
end

function SummationAnswerPuck:showAsPartiallyCorrect()
  self.puckRectangle:setColor(unpack(SummationCommon.PARTIALLY_CORRECT_COLOR))
  self.outerGlow:setColor(unpack(SummationCommon.PARTIALLY_CORRECT_COLOR))
  self.outerGlow:setOpacity(GLOW_OPACITY)
end

function SummationAnswerPuck:reset()
  local fadeOut = AHSpawn.new(
    AHOpacityTo.new(self.puckRectangle, FADE_OUT_DURATION, nil, SummationCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(self.outerGlow, FADE_OUT_DURATION, nil, SummationCommon.INVISIBLE_OPACITY),
    AHOpacityTo.new(self.flag, FADE_OUT_DURATION, nil, SummationCommon.INVISIBLE_OPACITY)
  )

  local postFadeWrapper = function()
    self.puckRectangle:setColor(unpack(SummationCommon.PARTIALLY_CORRECT_COLOR))
    self.puckRectangle:setOpacity(SummationCommon.VISIBLE_OPACITY)

    self.outerGlow:setColor(unpack(SummationCommon.PARTIALLY_CORRECT_COLOR))
    self.outerGlow:setOpacity(GLOW_OPACITY)

    self:setLoc(self.homeX, self.homeY)

    self.flag:reset()
  end

  local resetAction = AHSequence.new(fadeOut, postFadeWrapper)
  resetAction:start()
end

function SummationAnswerPuck:setHomeLocation(x, y)
  self.homeX = x
  self.homeY = y

  self:setLoc(x, y)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, flag)
  self.flag = flag
  flag:setAnchorPointLeft()
  flag:setLoc(self:getWidthPart(0.5) + FLAG_OFFSET_X, self:getHeightPart(0.5))
  self:insert(flag)

  local outerGlow = FilledRectangle.new(SummationAnswerPuck.PUCK_WIDTH + GLOW_RADIUS, SummationAnswerPuck.PUCK_HEIGHT + GLOW_RADIUS, SummationAnswerPuck.PUCK_INITIAL_COLOR)
  self:insertAtCenter(outerGlow)
  outerGlow:setOpacity(GLOW_OPACITY)
  self.outerGlow = outerGlow

  local puckRectangle = FilledRectangle.new(SummationAnswerPuck.PUCK_WIDTH, SummationAnswerPuck.PUCK_HEIGHT, SummationAnswerPuck.PUCK_INITIAL_COLOR)
  self:insertAtCenter(puckRectangle)
  self.puckRectangle = puckRectangle
end

function SummationAnswerPuck.newWithFlag(flag)
  local self = Group.new(SummationAnswerPuck.PUCK_WIDTH, SummationAnswerPuck.PUCK_WIDTH)
  Mixin.mix(self, SummationAnswerPuck)

  createLayout(self, flag)

  self:setOpacity(SummationAnswerPuck.PUCK_OPACITY_START)

  return self
end
