--- SummationAnswerPuckTrail
-- @author Created by Jay Austria on September 28, 2015

class("SummationAnswerPuckTrail")

require("SummationAnswerPuck")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TRAIL_COPIES_COUNT = 14
local COPY_WIDTH = SummationAnswerPuck.PUCK_WIDTH
local COPY_HEIGHT = SummationAnswerPuck.PUCK_HEIGHT

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function startTrailCoroutine(self)
  local updateTrailPositions = function()
    while true do
      local x, y = self.targetPuck:getLoc()

      for trailCopyIndex, trailCopy in ipairs(self.trailCopies) do
        local previousX, previousY = trailCopy:getLoc()
        trailCopy:setLoc(x, y)

        x = previousX
        y = previousY
      end

      coroutine.yield()
    end
  end

  self.trailCoroutine = runInCoroutine(updateTrailPositions)
end

local function setTrailColor(self, color)
  for _, trailCopy in ipairs(self.trailCopies) do
    trailCopy:setColor(unpack(color))
    trailCopy:setOpacity(trailCopy.originalOpacity)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationAnswerPuckTrail:hide()
  self:setVisibleRecursive(false)
end

function SummationAnswerPuckTrail:show()
  self:setVisibleRecursive(true)
end

function SummationAnswerPuckTrail:reset()
  local fadeAction = AHSpawn.new()
  for _, trailCopy in ipairs(self.trailCopies) do
    fadeAction:addAction(AHOpacityTo.new(trailCopy, SummationCommon.ELEMENT_FADE_OUT_DURATION, nil, SummationCommon.INVISIBLE_OPACITY))
  end

  local postFadeWrapper = function()
    setTrailColor(self, SummationAnswerPuck.PUCK_INITIAL_COLOR)
    self:hide()
  end

  local resetAction = AHSequence.new(fadeAction, postFadeWrapper)
  resetAction:start()
end

function SummationAnswerPuckTrail:showAsIncorrect()
  setTrailColor(self, SummationCommon.INCORRECT_COLOR)
end

function SummationAnswerPuckTrail:showAsCorrect()
  setTrailColor(self, SummationCommon.WHITE_COLOR)
end

function SummationAnswerPuckTrail:showAsPartiallyCorrect()
  setTrailColor(self, SummationAnswerPuck.PUCK_INITIAL_COLOR)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
local function createTrailCopy(self, trailCopyIndex)
  local minOpacity = 0.1
  local maxOpacity = 0.2

  local interpolationValue = (trailCopyIndex - 1) / (TRAIL_COPIES_COUNT - 1)
  local opacity = MathUtils.interpolateLinearFromNormalized(interpolationValue, maxOpacity, minOpacity)

  local trailCopy = FilledRectangle.new(COPY_WIDTH, COPY_HEIGHT, SummationAnswerPuck.PUCK_INITIAL_COLOR)
  trailCopy:setOpacity(opacity)

  trailCopy.originalOpacity = opacity

  return trailCopy
end

local function createLayout(self)
  local trailCopies = {}
  for trailCopyIndex = 1, TRAIL_COPIES_COUNT do
    local trailCopy = createTrailCopy(self, trailCopyIndex)
    self:insert(trailCopy)

    table.insert(trailCopies, trailCopy)
  end

  self.trailCopies = trailCopies
end

function SummationAnswerPuckTrail.new(containingObject, targetPuck)
  local width, height = containingObject:getDims()
  local self = Group.new(width, height)
  Mixin.mix(self, SummationAnswerPuckTrail)

  self.targetPuck = targetPuck

  createLayout(self)
  startTrailCoroutine(self)

  return self
end
