--- SummationConfiguration
-- @author Created by Jay Austria on July 17, 2014.

class("SummationConfiguration")

require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SummationConfiguration.GAME_DURATION = 75
SummationConfiguration.TUTORIAL_DELAY = 8
SummationConfiguration.POINTS_PER_ACCURACY_BONUS = 50
SummationConfiguration.FULLY_CORRECT_DELTA_THRESHOLD = 1.0
SummationConfiguration.CORRECT_ANSWERS_TO_PASS = 1

local CORRECT_ANSWER_POINTS = 1500
local CORRECT_ANSWER_POINTS_DECAY_RATE = 0.825
local MINIMUM_CORRECT_ANSWER_POINTS = 25
local PARTICIPATION_BONUS = 3750
local PARTICIPATION_BONUS_COUNT = 1

local CORRECT_ANSWERS_RANK_TWO = 6
local CORRECT_ANSWERS_RANK_THREE = 8

local PARTIAL_CORRECT_DELTA_THRESHOLD_KEYFRAMES = KeyframeSet.newInterpolated(
  Keyframe.new(0.0, {partiallyCorrectDeltaThreshold = 2.5}),
  Keyframe.new(0.375, {partiallyCorrectDeltaThreshold = 2.0}),
  Keyframe.new(1.0, {partiallyCorrectDeltaThreshold = 2.0})
)

-- Content Difficulty Configuration is in SummationRound.lua

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getDecayingCorrectAnswerPoints(totalCorrectAnswers)
  local numberOfDecayInstances = totalCorrectAnswers - 1
  local totalDecayFactor = CORRECT_ANSWER_POINTS_DECAY_RATE ^ numberOfDecayInstances
  local decayedCorrectAnswerPoints = CORRECT_ANSWER_POINTS * totalDecayFactor
  decayedCorrectAnswerPoints = math.max(decayedCorrectAnswerPoints, MINIMUM_CORRECT_ANSWER_POINTS)

  return decayedCorrectAnswerPoints
end

local function getTotalPointsForNumberOfCorrectAnswers(numberOfCorrectAnswers)
  local totalPoints = 0
  for i = 1, numberOfCorrectAnswers do
    totalPoints = totalPoints + getDecayingCorrectAnswerPoints(i)
  end
  return totalPoints
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationConfiguration.getCorrectAnswerPoints(totalCorrectAnswers)
  local correctAnswerPoints = getDecayingCorrectAnswerPoints(totalCorrectAnswers)
  if (totalCorrectAnswers <= PARTICIPATION_BONUS_COUNT) then
    correctAnswerPoints = correctAnswerPoints + PARTICIPATION_BONUS
  end
  return correctAnswerPoints
end

function SummationConfiguration.getPassScore()
  local correctNeeded = SummationConfiguration.CORRECT_ANSWERS_TO_PASS
  local totalPointsFromCorrectAnswers = getTotalPointsForNumberOfCorrectAnswers(correctNeeded)
  local passScore = PARTICIPATION_BONUS + totalPointsFromCorrectAnswers

  return passScore
end

function SummationConfiguration.getRankTwoScore()
  local correctNeeded = CORRECT_ANSWERS_RANK_TWO
  local totalPointsFromCorrectAnswers = getTotalPointsForNumberOfCorrectAnswers(correctNeeded)
  local pointsRankTwo = PARTICIPATION_BONUS + totalPointsFromCorrectAnswers

  return pointsRankTwo
end

function SummationConfiguration.getRankThreeScore()
  local correctNeeded = CORRECT_ANSWERS_RANK_THREE
  local totalPointsFromCorrectAnswers = getTotalPointsForNumberOfCorrectAnswers(correctNeeded)
  local pointsRankThree = PARTICIPATION_BONUS + totalPointsFromCorrectAnswers

  return pointsRankThree
end

function SummationConfiguration.getDeltaForPartiallyCorrectAnswer()
  local difficulty = GameSession.getDifficulty()
  local frameForDifficulty = PARTIAL_CORRECT_DELTA_THRESHOLD_KEYFRAMES:getFrameAt(difficulty)
  return frameForDifficulty.partiallyCorrectDeltaThreshold
end
