--- SummationCursor
-- @author Created by Jay Austria on August 1, 2014.

class("SummationCursor")
require("actions.advanced.Shake")
require("SummationColorUtils")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MOVE_DURATION = 0.5
local CURSOR_COLOR = SummationCommon.PARTIALLY_CORRECT_COLOR
local CURSOR_OPACITY_START = 0.39
local CURSOR_OPACITY_END = 0.10
local CURSOR_OPACITY_RESPONDED = 0.8
local BULGE_DURATION = 0.1
local FADE_DURATION = 0.4
local BULGE_SCALE_TO = 1.1
local BULGE_SCALE_BACK = 1.0
local CURSOR_OPACITY_PERFECT = 0.5
local CURSOR_INCORRECT_COLOR = { SummationCommon.INCORRECT_COLOR[1], SummationCommon.INCORRECT_COLOR[2], SummationCommon.INCORRECT_COLOR[3], CURSOR_OPACITY_START }
local CURSOR_PERFECT_COLOR =   { SummationCommon.WHITE_COLOR[1], SummationCommon.WHITE_COLOR[2], SummationCommon.WHITE_COLOR[3], CURSOR_OPACITY_PERFECT }
local CURSOR_COLOR_FADED = { SummationCommon.WHITE_COLOR[1], SummationCommon.WHITE_COLOR[2], SummationCommon.WHITE_COLOR[3], SummationCommon.INVISIBLE_OPACITY }


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationCursor:expandToLocationX(locationX)
  local endLocationX = locationX

  local expandAction = AHSpawn.new(
                         AHLocationXTo.new(self, MOVE_DURATION, AHEaseSimpleInOut.new(), endLocationX),
                         AHOpacityTo.new(self, MOVE_DURATION, nil, CURSOR_OPACITY_END)
                       )

  expandAction:start()
end

function SummationCursor:doBulgeAnimation()
  local scaleUpAction = AHScaleTo.new(self, BULGE_DURATION, AHEaseSimpleInOut.new(), BULGE_SCALE_TO)
  local scaleDownAction = AHScaleTo.new(self, BULGE_DURATION, AHEaseSimpleInOut.new(), BULGE_SCALE_BACK)

  local bulgeAnimation = AHSequence.new(scaleUpAction, scaleDownAction)
  bulgeAnimation:start()
end

function SummationCursor:doShakeAnimation()
  local shakeAction = AHRotationShake.new(self)
  shakeAction:start()
end

function SummationCursor:createFadeInAction()
  local fadeAction = AHOpacityTo.new(self, 0.1, nil, CURSOR_OPACITY_RESPONDED)

  return fadeAction
end

function SummationCursor:showAsIncorrect()
  self:setColor(unpack(CURSOR_INCORRECT_COLOR))
  self:setOpacity(CURSOR_OPACITY_RESPONDED)
end

function SummationCursor:morphToPerfectState()
  SummationColorUtils.flipElementColor(self,
                                       CURSOR_PERFECT_COLOR,
                                       CURSOR_COLOR_FADED,
                                       SummationCommon.FLIP_TO_DURATION,
                                       SummationCommon.FLIP_FROM_DURATION,
                                       SummationCommon.FLIP_DELAY)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function SummationCursor.new(width, height)
  local self = FilledRectangle.new(width, height, CURSOR_COLOR)
  Mixin.mix(self, SummationCursor)

  self:setOpacity(CURSOR_OPACITY_START)

  return self
end
