--- SummationFlag
-- @author Created by Jay Austria on August 3, 2014.

class("SummationFlag")
require("SummationCommon")
require("SummationPrice")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FLAG_WIDTH = Screen.getSafeWidthPart(0.53)
local FLAG_HEIGHT = 110

local SHOW_DURATION = 0.1
local TAIL_HEIGHT = 30
local TAIL_WIDTH = 20
local PRICE_FONT_SIZE = 60
local PRICE_LABEL_LOCATION_Y_MULTIPLIER = 0.4
local CORRECT_ANSWER_LABEL_FONT_SIZE = ReferenceDevice.getScaledWidth(18)
local CORRECT_ANSWER_LABEL_LOCATION_Y_MULTIPLIER = 0.8
local RESULT_LABEL_OFFSET_Y = 25
local RESULT_LABEL_FONT_SIZE = ReferenceDevice.getScaledWidth(32)
local INITIAL_VALUE = 0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationFlag:reset()
  self:setColor(unpack(SummationCommon.PARTIALLY_CORRECT_COLOR))
  self.priceLabel:setColor(unpack(SummationCommon.WHITE_COLOR))
  self.correctAnswerLabel:setColor(unpack(SummationCommon.WHITE_COLOR))
  self.resultLabel:setColor(unpack(SummationCommon.WHITE_COLOR))
  self:setOpacity(SummationCommon.VISIBLE_OPACITY)
  self.resultLabel:setOpacity(SummationCommon.INVISIBLE_OPACITY)

  self:setValue(INITIAL_VALUE)
end

function SummationFlag:showAsIncorrect()
  self:setColor(unpack(SummationCommon.INCORRECT_COLOR))
end


function SummationFlag:showAsPartiallyCorrect()
  self.resultLabel:setText(Localizer.get("NICE"))
  self.resultLabel:setOpacity(SummationCommon.VISIBLE_OPACITY)
end

function SummationFlag:showAsFullyCorrect()
  self:setColor(unpack(SummationCommon.WHITE_COLOR))

  self.priceLabel:setColor(unpack(SummationCommon.FULLY_CORRECT_COLOR))

  self.correctAnswerLabel:setColor(unpack(SummationCommon.FULLY_CORRECT_COLOR))

  self.resultLabel:setText(Localizer.get("BULLSEYE"))
  self.resultLabel:setColor(unpack(SummationCommon.WHITE_COLOR))
  self.resultLabel:setOpacity(SummationCommon.VISIBLE_OPACITY)
end

function SummationFlag:createHideAction(duration)
  local hideAction = AHOpacityTo.new(self, duration, nil, SummationCommon.INVISIBLE_OPACITY)

  return hideAction
end

function SummationFlag:setValue(value)
  self.value = value

  self.priceLabel:setText(string.format("%.2f", self.value))
end

function SummationFlag:getValue(value)
  return self.value
end

function SummationFlag:createValueToAction(duration, ease, value)
  return AHNonAttributeTo.new(self, duration, self.getValue, self.setValue, ease, value)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local tail = FilledTriangle.new(0, 0, TAIL_WIDTH, TAIL_HEIGHT * -0.5, TAIL_WIDTH, TAIL_HEIGHT * 0.5)
  tail:setAnchorPointBottom()
  tail:inheritColor(self)
  tail:inheritOpacity(parent)
  tail:setLoc(-tail:getWidthPart(0.5), self:getHeightPart(0.5))
  self:insert(tail)

  local correctAnswerLabel = DeprecatedLabel.newSingleLine(Localizer.get("ACTUAL_SUM"), FontFaces.DEFAULT_FONT, CORRECT_ANSWER_LABEL_FONT_SIZE)
  correctAnswerLabel:setColor(unpack(SummationCommon.WHITE_COLOR))
  correctAnswerLabel:inheritOpacity(self)
  correctAnswerLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(CORRECT_ANSWER_LABEL_LOCATION_Y_MULTIPLIER))
  self:insert(correctAnswerLabel)
  self.correctAnswerLabel = correctAnswerLabel

  local priceString = string.format("%.2f", self.value)
  local priceCharacters = "0123456789."
  local prerenderedFont = PrerenderedFont.newWithActualSize(FontFaces.LIGHT_FONT, PRICE_FONT_SIZE, priceCharacters)
  local priceLabel = Label.new(priceString, prerenderedFont)
  priceLabel:inheritOpacity(self)
  priceLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(PRICE_LABEL_LOCATION_Y_MULTIPLIER))
  priceLabel:setColor(unpack(SummationCommon.WHITE_COLOR))
  self:insert(priceLabel)
  self.priceLabel = priceLabel

  local resultLabel = DeprecatedLabel.newSingleLine("-unset-", FontFaces.LIGHT_FONT, RESULT_LABEL_FONT_SIZE)
  resultLabel:inheritColor(self)
  resultLabel:setOpacity(SummationCommon.INVISIBLE_OPACITY)
  resultLabel:setLoc(self:getWidthPart(0.5), self:getHeight() + RESULT_LABEL_OFFSET_Y)
  self:insert(resultLabel)
  self.resultLabel = resultLabel
end

function SummationFlag.new()
  local self = FilledRectangle.new(FLAG_WIDTH, FLAG_HEIGHT, SummationCommon.PARTIALLY_CORRECT_COLOR)
  Mixin.mix(self, SummationFlag)

  self.value = INITIAL_VALUE

  createLayout(self)

  return self
end
