--- SummationGameNumberLine
-- @author Created by Jay Austria on August 2, 2014.

class("SummationGameNumberLine")
require("SummationCommon")
require("SummationAnswerPuck")
require("SummationAnswerPuckTrail")
require("SummationNumberLine")
require("SummationFlag")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PUCK_LAUNCH_DURATION = 0.7
local FLAG_LOCATION_X = Screen.getSafeX(0.45)
local NUMBER_LINE_FADE_DURATION = 0.4
local BAR_HEIGHT_PADDING_Y = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createHideAndRemoveElementAction(element)
  local hideAndRemove = AHSequence.new()

  if element then
    hideAndRemove:addAction(AHOpacityTo.new(element, SummationCommon.ELEMENT_FADE_OUT_DURATION, nil, SummationCommon.INVISIBLE_OPACITY))
    hideAndRemove:addAction(AHRemove.new(element))
  end

  return hideAndRemove
end

local function hideAndRemoveElement(element)
  local hideAndRemoveElementAction = createHideAndRemoveElementAction(element)

  hideAndRemoveElementAction:start()
end

local function getYFromValue(self, value)
  local clampedValue = MathUtils.clamp(value, 0, self.maximumValue)
  local y = MathUtils.interpolateLinear(clampedValue, 0, self.maximumValue, 0, self:getHeight())

  return y
end

local function getLaunchPuckToValueInternal(self, value, response, puckStopCallback, puckCollideCallback, resumeGameDelay, hideDelay)
  local destinationY = getYFromValue(self, value)
  local collisionY = getYFromValue(self, response - (self.highlightBarValueSpan / 2))

  local showTrail = function()
    self.answerPuckTrail:show()
  end

  local startCheckingCollision = function()
    if self.checkCollisionCoroutine then
      self.checkCollisionCoroutine:stop()
      self.checkCollisionCoroutine = nil
    end

    local checkCollision = function()
      while true do
        local _, puckY = self.answerPuck:getLoc()
        if puckY >= collisionY then
          break
        end
        coroutine.yield()
      end
      puckCollideCallback()
    end
    self.checkCollisionCoroutine = runInCoroutine(checkCollision)
  end

  local launchAction = self.answerPuck:getLaunchAction(PUCK_LAUNCH_DURATION, destinationY, value)

  local puckAction = AHSpawn.new(
                       self.highlightCursor:createFadeInAction(),
                       showTrail,
                       startCheckingCollision,
                       launchAction
                     )

  local puckStopActionWrapper = function()
    if puckStopCallback then
      puckStopCallback()
    end
  end

  local hideAction = AHSpawn.new(
    createHideAndRemoveElementAction(self.highlightBar),
    createHideAndRemoveElementAction(self.highlightCursor),
    function() self.answerPuck:reset() end,
    function() self.answerPuckTrail:reset() end
  )

  local launchAction = AHSpawn.new(
                         AHDelayed.new(resumeGameDelay, self.resumeGameCallback),
                         AHSequence.new(
                           puckAction,
                           puckStopActionWrapper,
                           AHDelayed.new(hideDelay, hideAction)
                         )
                       )

  self.launchAction = launchAction

  return launchAction
end

local function createNewHighlights(self, y)
  local barHeight = self:getMarkYOffset() * self.highlightBarValueSpan - SummationAnswerPuck.PUCK_HEIGHT - BAR_HEIGHT_PADDING_Y
  local barWidth = Screen.fullWidth - SummationCommon.NUMBER_LINE_LOCATION_X
  local highlightBar = SummationHighlightBar.new(barWidth, barHeight, SummationCommon.CURSOR_WIDTH)
  highlightBar:setLoc(self.barLocationX, y)
  self:insert(highlightBar)
  highlightBar:expandToFullSize()

  self.highlightBar = highlightBar

  local cursorHeight = barHeight
  local cursorLocationX = barWidth
  local highlightCursor = SummationCursor.new(SummationCommon.CURSOR_WIDTH, cursorHeight)
  highlightCursor:setLoc(Screen.fullWidth, y)
  self:insert(highlightCursor)
  highlightCursor:expandToLocationX(self:getWidthPart(0.5))

  self.highlightCursor = highlightCursor
end

local function showElementsAsIncorrect(self)
  self.highlightBar:showAsIncorrect()
  self.answerPuck:showAsIncorrect()
  self.answerPuckTrail:showAsIncorrect()
  self.highlightCursor:showAsIncorrect()
  self.answerPuckTrail:showAsIncorrect()
end

local function showElementsAsCorrect(self, isPerfect)
  if isPerfect then
    self.flag:showAsFullyCorrect()
    self.answerPuck:showAsCorrect()
    self.answerPuckTrail:showAsCorrect()
    self.highlightBar:removeSelf()
  else
    self.flag:showAsPartiallyCorrect()
    self.answerPuck:showAsPartiallyCorrect()
    self.answerPuckTrail:showAsPartiallyCorrect()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationGameNumberLine:setResumeGameCallback(callback)
  self.resumeGameCallback = callback
end

function SummationGameNumberLine:getMarkYOffset()
  return self.numberLine:getMarkYOffset()
end

function SummationGameNumberLine:getLaunchPuckToCorrectValueAction(value, response, puckArriveCallback, isPerfect)
  local overridenPuckStopCallback = function()
    puckArriveCallback()
    showElementsAsCorrect(self, isPerfect)
  end

  local puckCollideCallback = function()
    self.highlightCursor:doBulgeAnimation()
  end

  local resumeGameDelay = nil
  local hideDelay = nil
  if isPerfect then
    resumeGameDelay = SummationCommon.FULLY_CORRECT_RESUME_GAME_DELAY
    hideDelay = SummationCommon.FULLY_CORRECT_FLAG_HIDE_DELAY
  else
    resumeGameDelay = SummationCommon.PARTIALLY_CORRECT_RESUME_GAME_DELAY
    hideDelay = SummationCommon.PARTIALLY_CORRECT_FLAG_HIDE_DELAY
  end

  local launchAction = getLaunchPuckToValueInternal(self, value, response, overridenPuckStopCallback, puckCollideCallback, resumeGameDelay, hideDelay)

  return launchAction
end

function SummationGameNumberLine:getLaunchPuckToValueIncorrectAction(value, response, puckStopCallback)
  local overriddenPuckStopCallback = function()
    puckStopCallback()
    showElementsAsIncorrect(self)
  end

  local overriddenPuckCollideCallback = function()
    self.highlightCursor:doShakeAnimation()
    showElementsAsIncorrect(self)
  end

  return getLaunchPuckToValueInternal(self, value, response, overriddenPuckStopCallback, overriddenPuckCollideCallback, SummationCommon.INCORRECT_RESUME_GAME_DELAY, SummationCommon.INCORRECT_FLAG_HIDE_DELAY)
end

function SummationGameNumberLine:resetMarkColors()
  self.numberLine:resetMarkColors()
end

function SummationGameNumberLine:clear()
  hideAndRemoveElement(self.highlightCursor)
  hideAndRemoveElement(self.highlightBar)

  if self.checkCollisionCoroutine then
    self.checkCollisionCoroutine:stop()
    self.checkCollisionCoroutine = nil
  end
end

function SummationGameNumberLine:morphToPerfectState()
  self.numberLine:morphToPerfectState()
  self.highlightCursor:morphToPerfectState()
end

function SummationGameNumberLine:getPuckY()
  local _, puckY = self.answerPuck:getLoc()

  return puckY
end

function SummationGameNumberLine:doLossAnimation()
  self.numberLine:doLossAnimation()
end

function SummationGameNumberLine:doIntro()
  local fadeAction = AHOpacityTo.new(self.numberLine, NUMBER_LINE_FADE_DURATION, nil, SummationCommon.VISIBLE_OPACITY)
  fadeAction:start()
end

function SummationGameNumberLine:enterRespondingState()
  self.numberLine:enterRespondingState()
end

function SummationGameNumberLine:leaveRespondingState()
  self.numberLine:leaveRespondingState()
end

function SummationGameNumberLine:notifyTouchBegan(y)
  createNewHighlights(self, y)
end

function SummationGameNumberLine:notifyUpdateHighlightLocationY(y)
  self.highlightBar:setY(y)
  self.highlightCursor:setY(y)
end

function SummationGameNumberLine:isDoneAnimating()
  return not self.highlightBar:isAnimating()
end

function SummationGameNumberLine:getBarLocationX()
  return self.barLocationX
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()
  local numberLine = SummationNumberLine.new(width,
                                             height,
                                             self.minimumValue,
                                             self.maximumValue)
  numberLine:setOpacity(SummationCommon.INVISIBLE_OPACITY)
  self:insertAtCenter(numberLine)
  self.numberLine = numberLine

  local flag = SummationFlag.new()
  self.flag = flag

  local answerPuck = SummationAnswerPuck.newWithFlag(flag)
  answerPuck:setHomeLocation(width * 0.5, Screen.getHeightPart(-0.3))
  self:insert(answerPuck)
  self.answerPuck = answerPuck

  self.answerPuck:reset()

  local answerPuckTrail = SummationAnswerPuckTrail.new(self, answerPuck)
  self:insertAtCenter(answerPuckTrail)
  self.answerPuckTrail = answerPuckTrail
end

function SummationGameNumberLine.new(width, height, minimumValue, maximumValue, highlightBarValueSpan)
  local self = Group.new(width, height)
  Mixin.mix(self, SummationGameNumberLine)

  self.minimumValue = minimumValue
  self.maximumValue = maximumValue
  self.barLocationX = Screen.fullWidth - SummationCommon.NUMBER_LINE_LOCATION_X + self:getWidthPart(0.5)
  self.highlightBarValueSpan = highlightBarValueSpan

  createLayout(self)

  return self
end
