--- SummationHighlightBar
-- @author Created by Jay Austria on August 1, 2014.

class("SummationHighlightBar")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SCALE_DURATION = 0.2
local FADE_IN_DURATION = 0.55
local BAR_COLOR = ColorUtils.colorRGBToStd(3, 226, 266)
local BAR_OPACITY_START = 0.39
local BAR_OPACITY_END = 0.10
local BEAD_SIZE = 2
local BEAD_COLOR = ColorUtils.colorRGBToStd(6, 255, 248)
local INCORRECT_COLOR = SummationCommon.INCORRECT_COLOR


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationHighlightBar:isAnimating()
  return self.expandAction and self.expandAction:isActive()
end

function SummationHighlightBar:expandToFullSize()
  local scaleAction = AHScaleXTo.new(self, SCALE_DURATION, AHEaseSimpleInOut.new(), 1.0)
  local fadeAction = AHOpacityTo.new(self, FADE_IN_DURATION, nil, BAR_OPACITY_END)

  local expandAction = AHSpawn.new(scaleAction, fadeAction)

  self.expandAction = expandAction

  expandAction:start()
end

function SummationHighlightBar:showAsIncorrect()
  self:setColor(unpack(INCORRECT_COLOR))
  self:setOpacity(BAR_OPACITY_END)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()

  local responseBead = FilledRectangle.new(BEAD_SIZE, BEAD_SIZE, BEAD_COLOR)
  responseBead:setLoc(SummationCommon.CURSOR_WIDTH * 0.5, height * 0.5)
  self:insert(responseBead)
end

function SummationHighlightBar.new(width, height, cursorWidth)
  local totalWidth = width + cursorWidth * 0.5

  local self = FilledRectangle.new(totalWidth, height, BAR_COLOR)
  Mixin.mix(self, SummationHighlightBar)

  createLayout(self)

  self:setOpacity(BAR_OPACITY_START)
  self:setAnchorPointRight()
  self:setScl(0.0, 1.0)

  return self
end
