--- SummationPrice
-- @author Created by Jay Austria on August 3, 2014.

class("SummationPrice")
require("utils.GlyphTableHelpers")
require("SummationCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LABEL_HEIGHT = 80
local LABEL_WIDTH = 160
local LABEL_FONT_SIZE = ReferenceDevice.getScaledWidth(60)


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getDecimalOffsetX(glyphs, text)
  local decimalIndex = string.find(text, ".", 1, true)

  local decimalGlyph = GlyphTableHelpers.getGlyphInRowByIndex(glyphs, decimalIndex)
  local x, _ = GlyphTableHelpers.getGlyphCoordinatesByGlyph(decimalGlyph) / Host.contentScale

  local offset = -x * 0.5 + LABEL_WIDTH * 0.2

  return offset
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationPrice:morphToRespondingState()
  SummationColorUtils.morphElementColors(self, SummationCommon.PRICE_LABEL_COLOR_RESPONDING, SummationCommon.COLOR_MORPH_RESPONDING_DURATION)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, value)
  local width, height = self:getDims()
  local priceValueString = string.format("%.2f", value)
  local shouldBuildGlyphTable = true
  local priceLabel = DeprecatedLabel.newSingleLine(priceValueString, FontFaces.LIGHT_FONT, LABEL_FONT_SIZE, shouldBuildGlyphTable)
  local glyphs = priceLabel:getGlyphTable()

  local decimalOffsetX = getDecimalOffsetX(glyphs, priceValueString)

  priceLabel:setLoc(width * 0.5 + decimalOffsetX, height * 0.5)
  self:insert(priceLabel)

  local isRecursive = true
  self:linkColorAttrToChildren(isRecursive)
  self:linkOpacityAttrToChildren(isRecursive)

  self:setColor(unpack(SummationCommon.PRICE_LABEL_COLOR))

  local startOpacity = SummationCommon.INVISIBLE_OPACITY
  self:setOpacity(startOpacity)
end

function SummationPrice.new(value)
  local self = Group.new(LABEL_WIDTH, LABEL_HEIGHT)
  Mixin.mix(self, SummationPrice)

  createLayout(self, value)

  return self
end
