--- SummationWinAnimation
-- @author Created by Jay Austria on August 5, 2014.

class("SummationWinAnimation")
require("SummationCommon")
require("SummationFireworks")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FADE_IN_DURATION = 1.0
local END_FIREWORKS_RADIUS = Screen.getWidthPart(0.4)
local OUTER_SUBDIVISIONS = 8
local INNER_SUBDIVISIONS = 6
local OUTER_SHAPE_SIZE = 16
local INNER_SHAPE_SIZE = 12
local CHECK_COLOR = {0.27, 0.88, 0.64}
local CHECK_WIDTH = Screen.getWidthPart(0.5)
local CHECK_HEIGHT = Screen.getHeightPart(0.25)
local CHECK_STROKE_THICKNESS = 30
local CHECK_STROKE_LENGTH = 100
local CHECK_ANGLE = 45
local LEFT_STROKE_SCALE = 0.3
local SCALE_CHECK_DURATION = 1.0
local CHECK_FADE_DURATION = 0.5
local FADE_DELAY = 1.2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SummationWinAnimation:runAndWaitForOutro()
  local fadeIn = AHOpacityTo.new(self, FADE_IN_DURATION, nil, SummationCommon.VISIBLE_OPACITY)

  local runFireworks = function()
    self.fireworks:runAnimation()
  end

  local scaleCheckAction = AHScaleTo.new(self.checkmark,
                                         SCALE_CHECK_DURATION,
                                         AHEaseBackOut.new(),
                                         1.0)
  local fadeCheckDelay = AHDelay.new(FADE_DELAY)
  local fadeCheckAction = AHOpacityTo.new(self.checkmark,
                                          CHECK_FADE_DURATION,
                                          nil,
                                          SummationCommon.INVISIBLE_OPACITY)

  local checkSequence = AHSequence.new(scaleCheckAction, fadeCheckDelay, fadeCheckAction)


  local combinedAction = AHSpawn.new(fadeIn, runFireworks, checkSequence)

  Sound.playSoundEffect("estimation_goal_reach")
  wait(combinedAction:start())
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createCheckmark(self)
  local check = Group.new(CHECK_WIDTH, CHECK_HEIGHT)

  local leftStroke = FilledRectangle.new(CHECK_STROKE_THICKNESS, CHECK_STROKE_LENGTH * LEFT_STROKE_SCALE, CHECK_COLOR)
  leftStroke:setAnchorPointBottomLeft()
  local angleRad = math.rad(CHECK_ANGLE)
  local xOffset = -CHECK_STROKE_THICKNESS * math.cos(angleRad)
  local yOffset = CHECK_STROKE_THICKNESS * math.sin(angleRad)
  leftStroke:setLoc(CHECK_WIDTH * 0.5 + xOffset, CHECK_HEIGHT * 0.25 + yOffset)
  leftStroke:setRot(CHECK_ANGLE)
  check:insert(leftStroke)

  local rightStroke = FilledRectangle.new(CHECK_STROKE_THICKNESS, CHECK_STROKE_LENGTH, CHECK_COLOR)
  rightStroke:setAnchorPointBottomRight()
  rightStroke:setLoc(CHECK_WIDTH * 0.5, CHECK_HEIGHT * 0.25)
  rightStroke:setRot(-CHECK_ANGLE)
  check:insert(rightStroke)

  check:linkAttrToChildren(MOAIColor.ATTR_OPACITY, MOAIColor_ATTR_OPACITY)

  return check
end

local function createLayout(self)
  local fireworks = SummationFireworks.new(END_FIREWORKS_RADIUS,
                                           SummationCommon.FULLY_CORRECT_COLOR,
                                           OUTER_SUBDIVISIONS,
                                           INNER_SUBDIVISIONS,
                                           OUTER_SHAPE_SIZE,
                                           INNER_SHAPE_SIZE)
  fireworks:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.5))
  self:insert(fireworks)
  self.fireworks = fireworks

  local checkmark = createCheckmark(self)
  checkmark:setScl(0)
  checkmark:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.5))
  self:insert(checkmark)
  self.checkmark = checkmark
end

function SummationWinAnimation.new()
  local self = FilledRectangle.newFullScreen(SummationCommon.WHITE_COLOR)
  Mixin.mix(self, SummationWinAnimation)

  createLayout(self)

  self:setOpacity(0.0)

  return self
end
