-- @author Created by Jay Austria on Nov. 12, 2013
class("SunsetCloudScroller")
require("deprecated.ColorTracker")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local OPACITY_CLOUDS = 0.3
local OPACITY_REFLECTIONS = 0.22
local REGION_WIDTH = Screen.fullWidth
local REGION_HEIGHT = Screen.fullHeight * 0.25
local GROUP_WIDTH = REGION_WIDTH
local GROUP_HEIGHT = REGION_HEIGHT * 2.0
local SCROLL_MULTIPLIER = 2.0

local COLORING_LEVELS_CLOUDS = {
  {t = 0.0, keyColor = {0.97, 0.88, 0.74, OPACITY_CLOUDS}},
  {t = 0.5, keyColor = {0.94, 0.75, 0.64, OPACITY_CLOUDS}},
  {t = 0.9, keyColor = {0.94, 0.52, 0.50, OPACITY_CLOUDS}},
  {t = 1.0, keyColor = {0.84, 0.42, 0.40, OPACITY_CLOUDS}}
}

local COLORING_LEVELS_REFLECTIONS = {
  {t = 0.0, keyColor = {0.92, 0.83, 0.69, OPACITY_REFLECTIONS}},
  {t = 0.5, keyColor = {0.89, 0.70, 0.59, OPACITY_REFLECTIONS}},
  {t = 0.9, keyColor = {0.89, 0.47, 0.45, OPACITY_REFLECTIONS}},
  {t = 1.0, keyColor = {0.79, 0.37, 0.35, OPACITY_REFLECTIONS}}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createCloud(self, x, y, typeIndex)
  local cloudSpriteName = string.format("cloud0%d.png", typeIndex)
  local newCloud = Sprite.newWithName(cloudSpriteName)
  self.airGroup:insert(newCloud)
  newCloud:setLoc(x, y)
  self.cloudColorTracker:linkColorNodeToSelf(newCloud)

  table.insert(self.clouds, newCloud)

  local newReflection = Sprite.newWithName(cloudSpriteName)
  newReflection:setScl(1, -1)
  self.reflectionGroup:insert(newReflection)
  newReflection:setLoc(x, REGION_HEIGHT - y)
  self.reflectionColorTracker:linkColorNodeToSelf(newReflection)

  table.insert(self.clouds, newReflection)
end

local function createClouds(self)
  self.clouds = {}
  createCloud(self, REGION_WIDTH * 0.8, REGION_HEIGHT * 0.8, 1)
  createCloud(self, REGION_WIDTH * 0.4, REGION_HEIGHT * 0.65, 2)
  createCloud(self, REGION_WIDTH * 0.3, REGION_HEIGHT * 0.4, 3)
  createCloud(self, REGION_WIDTH * -0.2, REGION_HEIGHT * 0.2, 4)
  createCloud(self, REGION_WIDTH * 0.6 - REGION_WIDTH, REGION_HEIGHT * 0.8, 1)
  createCloud(self, REGION_WIDTH * 0.15 - REGION_WIDTH, REGION_HEIGHT * 0.65, 2)
  createCloud(self, REGION_WIDTH * 0.0 - REGION_WIDTH, REGION_HEIGHT * 0.4, 3)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SunsetCloudScroller:shiftColorWithDuration(value, duration)
  self.cloudColorTracker:shiftColorWithDuration(value, duration)
  self.reflectionColorTracker:shiftColorWithDuration(value, duration)
end

function SunsetCloudScroller:setPaused(isPaused)
  for _, cloud in pairs(self.clouds) do
    cloud.scrollAction:pause(isPaused)
  end
  self.cloudColorTracker:setPaused(isPaused)
  self.reflectionColorTracker:setPaused(isPaused)
end

function SunsetCloudScroller:start(roundTime)
  for _, cloud in pairs(self.clouds) do
    local x, _ = cloud:getLoc()
    local scrollCloudAction = AHLocationXBy.new(cloud, roundTime * SCROLL_MULTIPLIER, nil, x + REGION_WIDTH * 2.0)
    cloud.scrollAction = scrollCloudAction
    cloud.scrollAction:start()
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function SunsetCloudScroller.new()
  local self = Group.new(GROUP_WIDTH, GROUP_HEIGHT)
  Mixin.mix(self, SunsetCloudScroller)

  local airGroup = Group.new(GROUP_WIDTH, REGION_HEIGHT)
  airGroup:setLoc(GROUP_WIDTH * 0.5, GROUP_HEIGHT * 0.75)
  self:insert(airGroup)
  self.airGroup = airGroup

  local reflectionGroup = Group.new(GROUP_WIDTH, REGION_HEIGHT)
  reflectionGroup:setLoc(GROUP_WIDTH * 0.5, GROUP_HEIGHT * 0.25)
  self:insert(reflectionGroup)
  self.reflectionGroup = reflectionGroup

  local cloudColorTracker = ColorTracker.newWithTable(COLORING_LEVELS_CLOUDS)
  self.cloudColorTracker = cloudColorTracker

  local reflectionColorTracker = ColorTracker.newWithTable(COLORING_LEVELS_REFLECTIONS)
  self.reflectionColorTracker = reflectionColorTracker

  self.cloudActions = {}
  createClouds(self)

  return self
end
