--- SunsetConfiguration

class("SunsetConfiguration")

require("GameSession")
require("data.generators.LearningFocusedRoundGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SunsetConfiguration.DIFFICULTY_ESCALATION_AFTER_CORRECT_ANSWER_COUNT = 3
SunsetConfiguration.DIFFICULTY_ESCALATION_AMOUNT = 0.2

local CORRECT_ANSWER_POINTS = 1000
local INCORRECT_ANSWER_POINTS = -200
local TIME_BONUS_PER_SECOND = 100
local MAX_STREAK_BONUS_POINTS = 50
local ROUND_TIME = 70
local CORRECT_TO_WIN = 6
local EXTRA_SECONDS_RANK_TWO = 24
local EXTRA_SECONDS_RANK_THREE = 38
local TUTORIAL_DELAY = 12
local TIME_REMAINING_TO_START_RESURFACING_MISTAKES =
  ROUND_TIME *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SunsetConfiguration.getTimeRemainingToStartResurfacingMistakes()
  return TIME_REMAINING_TO_START_RESURFACING_MISTAKES
end

function SunsetConfiguration.getTutorialDelay()
  return TUTORIAL_DELAY
end

function SunsetConfiguration.getCorrectAnswerPoints()
  return CORRECT_ANSWER_POINTS
end

function SunsetConfiguration.getIncorrectAnswerPoints()
  return INCORRECT_ANSWER_POINTS
end

function SunsetConfiguration.getTimeBonusPerSecond()
  return TIME_BONUS_PER_SECOND
end

function SunsetConfiguration.getMaxStreakBonusPoints()
  return MAX_STREAK_BONUS_POINTS
end

function SunsetConfiguration.getRoundTime()
  return ROUND_TIME
end

function SunsetConfiguration.getNumberOfConcepts()
  return CORRECT_TO_WIN
end

function SunsetConfiguration.getMaxStreakBonus()
  local maxCorrect = SunsetConfiguration.getNumberOfConcepts()
  local maxStreakBonus = maxCorrect * MAX_STREAK_BONUS_POINTS
  return maxStreakBonus
end

function SunsetConfiguration.getPassScore()
  local correctNeeded = SunsetConfiguration.getNumberOfConcepts()
  local passScore = correctNeeded * CORRECT_ANSWER_POINTS
  return passScore
end

function SunsetConfiguration.getExtraSecondsRankTwo()
  return EXTRA_SECONDS_RANK_TWO
end

function SunsetConfiguration.getExtraSecondsRankThree()
  return EXTRA_SECONDS_RANK_THREE
end

function SunsetConfiguration.getRankTwoScore()
  local passScore = SunsetConfiguration.getPassScore()
  local extraSecondsNeeded = SunsetConfiguration.getExtraSecondsRankTwo()
  local rankTwoScore = passScore + extraSecondsNeeded * TIME_BONUS_PER_SECOND
  return rankTwoScore
end

function SunsetConfiguration.getRankThreeScore()
  local passScore = SunsetConfiguration.getPassScore()
  local extraSecondsNeeded = SunsetConfiguration.getExtraSecondsRankThree()
  local rankThreeScore = passScore + extraSecondsNeeded * TIME_BONUS_PER_SECOND
  return rankThreeScore
end

function SunsetConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 1.0}},

    {key = 0.075, valueProbabilities = {Basic = 0.75,
                                        Easy = 0.25}},

    {key = 0.15, valueProbabilities = {Basic = 0.30,
                                       Easy = 0.70}},

    {key = 0.225, valueProbabilities = {Easy = 0.75,
                                        Medium = 0.25}},

    {key = 0.3, valueProbabilities = {Easy = 0.5,
                                      Medium = 0.50}},

    {key = 0.375, valueProbabilities = {Easy = 0.30,
                                        Medium = 0.50,
                                        Hard = 0.20}},

    {key = 0.45, valueProbabilities = {Easy = 0.15,
                                       Medium = 0.50,
                                       Hard = 0.35}},

    {key = 0.525, valueProbabilities = {Easy = 0.10,
                                        Medium = 0.40,
                                        Hard = 0.50}},

    {key = 0.675, valueProbabilities = {Medium = 0.30,
                                        Hard = 0.70}},

    {key = 0.75, valueProbabilities = {Medium = 0.15,
                                       Hard = 0.85}},

    {key = 0.8, valueProbabilities = {Medium = 0.05,
                                      Hard = 0.8,
                                      Advanced = 0.15}},

    {key = 0.9, valueProbabilities = {Hard = 0.75,
                                      Advanced = 0.25}},

    {key = 1.0, valueProbabilities = {Hard = 0.65,
                                      Advanced = 0.35}}
  }
end
