--- SunsetRoundGenerator
-- @author created by Jay Austria on Octomom 16, 2013

class("SunsetRoundGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BLANK_TAG = "<blank/>"
local HIGHLIGHTED_OPEN = "<highlighted>"
local HIGHLIGHTED_CLOSED = "</highlighted>"
local LEFT_MARGIN_SINGLE_TAG = "<leftMargin/>"
local RIGHT_MARGIN_SINGLE_TAG = "<rightMargin/>"

local MAX_RETRY_ATTEMPTS = 3

SunsetRoundGenerator.CONCEPT_IDENTIFIER_KEY = "conceptIdentifier"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function attemptToGenerateUnseenConceptPair(self, retryAttempts)
  retryAttempts = retryAttempts or 0
  local concept = self.conceptGenerator:next()
  local conceptPair = string.lower(concept.pair)
  if (retryAttempts < MAX_RETRY_ATTEMPTS) and TableUtils.contains(self.seenConceptPairs, conceptPair) then
    return attemptToGenerateUnseenConceptPair(self, retryAttempts + 1)
  else
    table.insert(self.seenConceptPairs, conceptPair)
    return concept
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SunsetRoundGenerator:getFormattedPromptForConcept(concept, isCorrect)
  local optionText = nil

  if (isCorrect) then
    optionText = concept.optionTextCorrect
  else
    optionText = concept.optionTextIncorrect
  end

  return concept.sentence:gsub(BLANK_TAG, HIGHLIGHTED_OPEN .. LEFT_MARGIN_SINGLE_TAG .. optionText .. RIGHT_MARGIN_SINGLE_TAG .. HIGHLIGHTED_CLOSED)
end

function SunsetRoundGenerator:getHighlightTextForConcept(concept, isCorrect)
  if (isCorrect) then
    return concept.optionTextCorrect
  else
    return concept.optionTextIncorrect
  end
end

function SunsetRoundGenerator:getCorrectionTextForConcept(concept)
  return concept.optionTextCorrect
end

function SunsetRoundGenerator:reset()
  self.conceptGenerator:reset()
end

function SunsetRoundGenerator:generateNextValue()
  local concept = attemptToGenerateUnseenConceptPair(self)

  local round =
  {
    sentence = concept.sentence,
    optionTextCorrect = concept.correct,
    optionTextIncorrect = concept.incorrect,
    tip = concept.tip,
    [SunsetRoundGenerator.CONCEPT_IDENTIFIER_KEY] = concept.identifier
  }

  self.conceptGenerator:markConceptIdentifierAsSeen(round[SunsetRoundGenerator.CONCEPT_IDENTIFIER_KEY])

  return round
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SunsetRoundGenerator.new()
  local self = Generator.new()
  Mixin.mix(self, SunsetRoundGenerator)

  self.conceptGenerator = GameSession.conceptGenerator
  self.conceptGenerator:setMarkGeneratedConceptsAsSeen(false)

  self.seenConceptPairs = {}

  return self
end
