--- Sunset Star Overlay
class("SunsetStarOverlay")

require("SunsetStarSparkle")


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SunsetStarOverlay:animate()
  for _, star in pairs(self.stars) do
    star:animate()
  end
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function setProportionalLoc(self, object, locXProp, locYProp)
  local selfW, selfH = self:getDims()
  object:setLoc(math.floor(selfW * locXProp), math.floor(selfH * locYProp))
end

local function opacityForYProp(locYProp)
  return MathUtils.interpolateLinearFromNormalized(locYProp, 0.2, 0.9)
end

local function starSizeForYProp(locYProp)
  return MathUtils.interpolateLinearFromNormalized(1-locYProp, 2, 7)
end

local function addTwinkleStar(self, locXProp, locYProp, sizeProp, starType, cycleDuration)
  local runAtDoubleSpeed = 0.5
  local star = SunsetStarSparkle.new(starType, cycleDuration * runAtDoubleSpeed)
  star:setEffectiveDims(starSizeForYProp(locYProp) * sizeProp)
  setProportionalLoc(self, star, locXProp, locYProp)
  star:setOpacity(opacityForYProp(locYProp))
  self:insert(star)
  table.insert(self.stars, star)
end

local function addInertStar(self, locXProp, locYProp, sizeProp)
  local size = 2 * sizeProp
  local star = FilledCircle.new(size/2, {1, 1, 1, opacityForYProp(locYProp)})
  setProportionalLoc(self, star, locXProp, locYProp)
  self:insert(star)
end

function SunsetStarOverlay.new(width, height)
  local self = Group.new(width, height)
  Mixin.mix(self, SunsetStarOverlay)

  self.stars = {}

  addTwinkleStar(self, 0.11, 0.09, 1.0, 2, 1.5)
  addTwinkleStar(self, 0.45, 0.20, 1.0, 4, 2.0)
  addInertStar  (self, 0.52, 0.23, 1.0)
  addTwinkleStar(self, 0.28, 0.27, 1.0, 6, 1.9)
  addInertStar  (self, 0.28, 0.35, 1.0)
  addTwinkleStar(self, 0.35, 0.32, 1.0, 1, 1.5)
  addTwinkleStar(self, 0.41, 0.40, 1.0, 2, 2.0)
  addInertStar  (self, 0.45, 0.37, 1.0)
  addTwinkleStar(self, 0.71, 0.43, 1.0, 5, 1.9)
  addTwinkleStar(self, 0.47, 0.44, 1.0, 3, 1.7)
  addTwinkleStar(self, 0.79, 0.48, 1.0, 1, 2.0)
  addInertStar  (self, 0.62, 0.50, 1.0)
  addTwinkleStar(self, 0.60, 0.52, 1.0, 4, 1.6)
  addInertStar  (self, 0.34, 0.59, 1.0)
  addTwinkleStar(self, 0.70, 0.63, 1.0, 2, 1.5)
  addTwinkleStar(self, 0.33, 0.68, 1.0, 5, 1.9)
  addTwinkleStar(self, 0.55, 0.65, 1.0, 3, 1.7)
  addInertStar  (self, 0.90, 0.68, 1.0)
  addInertStar  (self, 0.61, 0.68, 1.0)
  addTwinkleStar(self, 0.66, 0.71, 1.0, 1, 1.6)
  addTwinkleStar(self, 0.17, 0.72, 1.0, 6, 1.8)
  addInertStar  (self, 0.19, 0.79, 1.0)
  addTwinkleStar(self, 0.30, 0.76, 1.0, 5, 2.1)
  addTwinkleStar(self, 0.44, 0.84, 1.0, 3, 1.4)
  addInertStar  (self, 0.67, 0.81, 1.0)
  addTwinkleStar(self, 0.77, 0.80, 1.0, 5, 2.5)
  addInertStar  (self, 0.36, 0.88, 1.0)
  addTwinkleStar(self, 0.17, 0.91, 1.0, 4, 2.0)
  addInertStar  (self, 0.60, 0.89, 1.0)
  addTwinkleStar(self, 0.86, 0.94, 1.0, 3, 1.8)
  addInertStar  (self, 0.31, 0.98, 1.0, 3, 1.8)
  addTwinkleStar(self, 0.21, 0.99, 1.0, 2, 2.0)
  addTwinkleStar(self, 0.63, 0.96, 1.0, 1, 1.5)

  self:linkAttrToChildren(MOAIColor.INHERIT_COLOR, MOAIColor.COLOR_TRAIT)

  return self
end
