--- Sunset Star Sparkle

class("SunsetStarSparkle")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ROTATE_1_8 = 45

local SCALE_FACTORS = {
  {
    {0.2, 0.2, 2.0, 2.0},
    {0.2, 0.2, 2.0, 2.0}
  },
  {
    {0.2, 0.2, 2.0, 3.0},
    {0.2, 0.2, 3.0, 2.0}
  }
}

local STAR_TYPES = {
  {
    phaseOffset = 0.4,
    scaleFactors = SCALE_FACTORS[2],
  },
  {
    phaseOffset = 0.15,
    scaleFactors = SCALE_FACTORS[1],
  },
  {
    phaseOffset = 0.33,
    scaleFactors = SCALE_FACTORS[1],
  },
  {
    phaseOffset = 0.3,
    scaleFactors = SCALE_FACTORS[1],
  },
  {
    phaseOffset = 0.2,
    scaleFactors = SCALE_FACTORS[2],
  },
  {
    phaseOffset = 0.2,
    scaleFactors = SCALE_FACTORS[1],
  }
}

SunsetStarSparkle.MAX_STAR_TYPE = #STAR_TYPES


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function scaleAction(star, duration, scaleX1, scaleY1, scaleX2, scaleY2)
  return AHSequence.new(
    AHScaleTo.new(star, duration/2, AHEaseSineInOut.new(), scaleX1, scaleY1),
    AHScaleTo.new(star, duration/2, AHEaseSineInOut.new(), scaleX2, scaleY2)
  )
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SunsetStarSparkle:animate()
  local plusScaleFactors = self.scaleFactors[1]
  local crossScaleFactors = self.scaleFactors[2]
  local duration = self.cycleDuration
  local phaseOffset = self.phaseOffset

  runInCoroutine(function()
    AHLoop.new(
      scaleAction(self.starPlus, duration, unpack(plusScaleFactors))
    ):start()

    waitFor(duration * phaseOffset)

    AHLoop.new(
      scaleAction(self.starCross, duration, unpack(crossScaleFactors))
    ):start()
  end)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function SunsetStarSparkle.new(starType, cycleDuration)
  local starPlus = Sprite.newWithName("star.png")
  local starCross = Sprite.newWithName("star.png")
  starCross:setRot(ROTATE_1_8)

  local self = Group.new(starPlus:getDims())
  Mixin.mix(self, SunsetStarSparkle)

  starPlus:setScl(0)
  starCross:setScl(0)

  self:insertAtCenter(starPlus)
  self:insertAtCenter(starCross)

  self:linkAttrToChildren(MOAIColor.INHERIT_COLOR, MOAIColor.COLOR_TRAIT)

  self.starPlus = starPlus
  self.starCross = starCross

  local starData = STAR_TYPES[starType]
  self.scaleFactors = starData.scaleFactors
  self.cycleDuration = cycleDuration
  self.phaseOffset = starData.phaseOffset

  return self
end
