-- @author Created by Jay Austria on Nov. 6, 2013
class("SunsetSun")
-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SUN_COLOR = {0.96, 0.89, 0.78}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function updatePointsToDraw(self)
  local radius = self.radius
  local pointsToDraw = {0, 0}
  local startAngle = 0
  local angle = startAngle
  local totalPoints = 180
  local dAngle = math.pi/totalPoints

  for i = 1, totalPoints + 1 do
    local currentPercentage = (i - 1) / totalPoints
    if (currentPercentage <= 1.0) then
      local x = radius * math.cos(angle)
      local y = math.max(radius * math.sin(angle) - radius * self.setAmount, 0)

      table.insert(pointsToDraw, x)
      table.insert(pointsToDraw, y)
      angle = startAngle + i * dAngle
    else
      break
    end
  end
  self.pointsToDraw = pointsToDraw
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SunsetSun:setSetAmount(amount)
  self.setAmount = amount
  updatePointsToDraw(self)
end

function SunsetSun:stopFade()
  self.origin:stop()
  self.script = nil
  self.origin = nil
end

function SunsetSun:fadeSetAmount(startValue, endValue, duration)
  if (self.origin or self.script) then
    self:stopFade()
  end

  local setDistance = endValue - startValue
  local script = MOAIScriptNode.new()
  local origin = MOAITimer.new()

  origin:setSpan(duration)
  script:reserveAttrs(1)
  script:setAttrLink(0, origin, MOAITimer.ATTR_TIME)
  script:setCallback(function()
    local t = script:getAttr(0) / duration
    local adjustedSetAmount = startValue + t * setDistance
    self:setSetAmount(adjustedSetAmount)
  end)
  origin:start()

  self.script = script
  self.origin = origin
end

function SunsetSun:setPaused(isPaused)
  self.origin:pause(isPaused)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
local function createLayout(self, width, height, radius)
  local function drawCallbackWrapper()
    MOAIDraw.fillFan(self.pointsToDraw)
  end

  local view = MOAIScriptDeck.new()
  view:setDrawCallback(drawCallbackWrapper)
  view:setRect(0, 0, width, height)

  local sunGroup = Group.new(width, height * 0.5)
  sunGroup:setAnchorPointBottomLeft()
  sunGroup:setDeck(view)
  sunGroup:setColor(unpack(SUN_COLOR))
  self:insertAtCenter(sunGroup)

  local reflectionGroup = Group.new(width, height * 0.5)
  reflectionGroup:setAnchorPointBottomLeft()
  reflectionGroup:setRot(180)
  reflectionGroup:setDeck(view)
  reflectionGroup:setColor(unpack(SUN_COLOR))
  reflectionGroup:setOpacity(SunsetCommon.REFLECTION_OPACITY)
  self:insertAtCenter(reflectionGroup)

  self.radius = radius
  self.setAmount = 0

  updatePointsToDraw(self)
end

function SunsetSun.new(radius)
  local width = radius * 2.0
  local height = radius * 2.0
  local self = Group.new(width, height)
  Mixin.mix(self, SunsetSun)

  createLayout(self, width, height, radius)

  return self
end
