--- GapFontRendererEngine
-- @author Created by Anton Morant on February 13, 2014.

class("GapFontRenderEngine")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

function calculateGapWidth(self)
  local engine = FontRenderEngine.new()
  engine:setFont(self.font)
  engine:setFontSize(self.fontSize)

  local maxWidth = 0
  for _, word in ipairs(self.words) do
    local label = DeprecatedLabel.new(engine)
    label:setText(word)
    local labelW = label:getDims()
    maxWidth = math.max(labelW, maxWidth)
  end

  local singleBlankLabel = DeprecatedLabel.new(engine)
  -- Note that one of the blanks seems to be discarded:
  singleBlankLabel:setText("  ")
  local blankWidth = singleBlankLabel:getDims()
  -- And again, add one extra blank:
  local numSpaces = math.ceil(maxWidth / blankWidth) + 1
  self.gapText = string.rep(" ", numSpaces)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function GapFontRenderEngine:renderTexture(text)
  if not self.gapText then
    calculateGapWidth(self)
  end
  return self.super.renderTexture(self, self.gapText)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function GapFontRenderEngine.new(words)
  local self = FontRenderEngine.new()
  Mixin.mix(self, GapFontRenderEngine)

  self.words = words

  return self
end
