--- SwapButton
-- @author Created by Anton Morant on February 12, 2014.

class("SwapButton")
require("SwapCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_COLOR = { 212/255, 198/255, 171/255 }
local BUTTON_CONTENTS_COLOR = ColorUtils.colorHex24ToStd(0x786d59)
local BUTTON_CONTENTS_CORRECT_COLOR = ColorUtils.colorHex24ToStd(0x2e7047)
local BUTTON_CONTENTS_INCORRECT_COLOR = ColorUtils.colorHex24ToStd(0x73392b)

local BUTTON_WIDTH = Screen.safeWidth * 0.28
local BUTTON_HEIGHT = Screen.safeWidth * 0.28
local BUTTON_SCALE = 1.00
local ICON_SIZE = BUTTON_WIDTH * 0.34

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function buttonDownWrapper(self)
  return function()
    self.onPressed()
  end
end

local function setButtonAndContentsColor(self, buttonColor, contentsColor)
  self:setColor(unpack(buttonColor))
  self.contentsColumn:setColor(unpack(contentsColor))
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SwapButton:showAsCorrect()
  setButtonAndContentsColor(self, SwapCommon.CORRECT_COLOR, BUTTON_CONTENTS_CORRECT_COLOR)
end

function SwapButton:showAsIncorrect()
  setButtonAndContentsColor(self, SwapCommon.INCORRECT_COLOR, BUTTON_CONTENTS_INCORRECT_COLOR)
end

function SwapButton:createFadeOutAction()
  local fadeOutAction = AHOpacityTo.new(self, SwapCommon.FADE_OUT_DURATION, SwapCommon.FADE_OUT_EASE, 0.0)
  return fadeOutAction
end

function SwapButton:revert()
  setButtonAndContentsColor(self, BUTTON_COLOR, BUTTON_CONTENTS_COLOR)
  self:setScl(BUTTON_SCALE)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SwapButton.new(iconFile, buttonText, onPressed)
  local self = FilledRoundedRectangle.new(BUTTON_WIDTH,
                                          BUTTON_HEIGHT,
                                          SwapCommon.STANDARD_CORNER_RADIUS,
                                          BUTTON_COLOR)
  Mixin.mix(self, SwapButton)

  local contentSpacing = 4
  local contentsColumn = Column.new(contentSpacing)

  local icon = Sprite.newWithName(iconFile)
  icon:setEffectiveDims(ICON_SIZE)
  contentsColumn:insert(icon)

  local label = DeprecatedLabel.newSingleLine(buttonText, FontFaces.DIN_OT_MEDIUM, 16)
  contentsColumn:insert(label)

  contentsColumn:alignChildren()
  self:insertAtCenter(contentsColumn)
  local isRecursive = true
  contentsColumn:linkColorAttrToChildren(isRecursive)
  contentsColumn:setColor(unpack(BUTTON_CONTENTS_COLOR))
  self.contentsColumn = contentsColumn

  contentsColumn:inheritOpacity(self)

  self.onPressed = onPressed

  Button.makeButton(self, buttonDownWrapper(self))
  return self
end
