class("SwapConfiguration")

require("GameSession")
require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SwapConfiguration.SLIDE_DURATION_IN_SECONDS = 17
SwapConfiguration.POINTS_PER_CORRECT_RESPONSE = 800
SwapConfiguration.POINTS_PER_FULL_SPEED_BONUS = 100
SwapConfiguration.POINTS_PER_ACCURACY_BONUS = 1000
SwapConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 2.5
SwapConfiguration.SPEED_BONUS_DECAY_START = 0.0
SwapConfiguration.CORRECT_NEEDED_TO_WIN = 7
SwapConfiguration.INCORRECT_PROGRESS_PENALTY_FRACTION = 0.5
SwapConfiguration.INCORRECT_RESPONSES_ALLOWED_RANK_TWO = 1
SwapConfiguration.INCORRECT_RESPONSES_ALLOWED_RANK_THREE = 0
SwapConfiguration.TUTORIAL_DELAY = 8

local MAXIMUM_PROGRESS_FOR_SCORE_PENALTY_PROTECTION = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SwapConfiguration.getPassScore()
  local pointsPerCorrect = SwapConfiguration.POINTS_PER_CORRECT_RESPONSE
  local correctNeeded = SwapConfiguration.CORRECT_NEEDED_TO_WIN
  local passScore =  pointsPerCorrect * correctNeeded

  return passScore
end

function SwapConfiguration.getRankTwoScore()
  local passScore = SwapConfiguration.getPassScore()
  local rankTwoScore = passScore + SwapConfiguration.POINTS_PER_ACCURACY_BONUS

  return rankTwoScore
end

function SwapConfiguration.getRankThreeScore()
  local passScore = SwapConfiguration.getPassScore()
  local accuracyBonusPoints = SwapConfiguration.POINTS_PER_ACCURACY_BONUS * 2
  local rankThreeScore = passScore + accuracyBonusPoints

  return rankThreeScore
end

function SwapConfiguration.scoreProtectionIsActiveForProgress(progress)
  return (progress <= MAXIMUM_PROGRESS_FOR_SCORE_PENALTY_PROTECTION)
end

function SwapConfiguration.getDifficultyKeyFrames()
  ExperimentsConfiguration.DIFFICULTY_KEYFRAMES_EXPERIMENT:reportExperimentExposed()
  if ExperimentsConfiguration.DIFFICULTY_KEYFRAMES_EXPERIMENT:isActive() then
    return {
      {key = 0.0, valueProbabilities = {Basic = 0.75,
                                        Easy = 0.25}},

      {key = 0.25, valueProbabilities = {Basic = 0.10,
                                         Easy = 0.60,
                                         Medium = 0.3}},

      {key = 0.5, valueProbabilities = {Easy = 0.25,
                                        Medium = 0.5,
                                        Hard = 0.25}},

      {key = 0.75, valueProbabilities = {Medium = 0.4,
                                         Hard = 0.5,
                                         Advanced = 0.1}},

      {key = 1.0, valueProbabilities = {Hard = 0.5,
                                        Advanced = 0.5}}
    }
  end
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Basic = 0.75,
        Easy = 0.25
      }
    },
    {
      key = 0.15,
      valueProbabilities = {
        Easy = 1.0
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.5,
        Medium = 0.5
      }
    },
    {
      key = 0.45,
      valueProbabilities = {
        Easy = 0.3,
        Medium = 0.7
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        Medium = 0.6,
        Hard = 0.4
      }
    },
    {
      key = 0.7,
      valueProbabilities = {
        Medium = 0.3,
        Hard = 0.7
      }
    },
    {
      key = 0.8,
      valueProbabilities = {
        Hard = 0.6,
        Advanced = 0.4
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 0.3,
        Advanced = 0.7
      }
    }
  }
end
