--- SwapRationalePanel
-- @author created by Jay Austria on June 17, 2021

class("SwapRationalePanel")
require("SwapCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HEIGHT_PROPORTION = 0.4
local BACKGROUND_COLOR = {0.28, 0.19, 0.14}
local TAP_TO_CONTINUE_FONT_SIZE = 15
local TAP_TO_CONTINUE_COLOR = {1.0, 1.0, 1.0, 0.5}
local RATIONALE_MARGIN_WIDTH_PROPORTION = 0.09
local RATIONALE_TEXT_COLOR_REGULAR = {1.0, 1.0, 1.0}
local RATIONALE_TEXT_COLOR_HIGHLIGHTED = {0.88, 0.48, 0.40}

local BASE_ATTRIBUTES = {
  font = Font.new(FontFaces.DIN_OT_MEDIUM, 23),
  color = RATIONALE_TEXT_COLOR_REGULAR
}

local ATTRIBUTES_FOR_LABELS = {
  i = {
    font = Font.new(FontFaces.DIN_OT_BOLD, 23),
    color = RATIONALE_TEXT_COLOR_HIGHLIGHTED
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SwapRationalePanel:setText(text)
  self.rationaleTextArea:setText(text)
  self.rationaleTextArea:rerender()
end

function SwapRationalePanel:setScreenYLocations(onscreenY, offscreenY)
  self.offscreenY = offscreenY
  self.onscreenY = onscreenY
end

function SwapRationalePanel:playSlideOnscreenAnimation(duration, ease)
  local ease = AHEaseBezier.newFlat({0.5, 0.4, 0.0, 1.0})
  local slideAction = AHLocationYTo.new(self, 0.5, ease, self.onscreenY)

  self.slideActionInterrupt:insert(slideAction:start())

  return slideAction
end

function SwapRationalePanel:playSlideOffscreenAnimation()
  local ease = AHEaseBezier.newFlat({0.6, 0.0, 0.4, 1.0})
  local slideAction = AHLocationYTo.new(self, 0.33, ease, self.offscreenY)

  self.slideActionInterrupt:insert(slideAction:start())

  return slideAction
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
local function createLayout(self)
  local marginSize = self:getWidthPart(RATIONALE_MARGIN_WIDTH_PROPORTION)

  local tapToContinueFont = Font.new(FontFaces.DIN_OT_MEDIUM, TAP_TO_CONTINUE_FONT_SIZE)
  local tapToContinueText = Localizer.get("Misc.TAP_TO_CONTINUE")
  local tapToContinueLabel = Label.new(tapToContinueText, tapToContinueFont)
  tapToContinueLabel:setColor(unpack(TAP_TO_CONTINUE_COLOR))
  local tapToContinueX = Screen.width * 0.5
  local tapToContinueY = marginSize + tapToContinueLabel:getHeightPart(0.5)
  tapToContinueLabel:setLoc(tapToContinueX, tapToContinueY)
  self:insert(tapToContinueLabel)

  local unsetString = ""
  local rationaleTextWidth = self:getWidth() - (2 * marginSize)
  local tapToContinueLabelMarginY = tapToContinueLabel:getHeight() + (2 * marginSize)
  local rationaleTextHeight = self:getHeight() - tapToContinueLabelMarginY - marginSize
  local rationaleTextArea = SizeConstrainedTextArea.new(unsetString,
                                                        rationaleTextWidth,
                                                        rationaleTextHeight,
                                                        BASE_ATTRIBUTES,
                                                        ATTRIBUTES_FOR_LABELS)
  rationaleTextArea:setAnchorPointTop()
  local rationaleTextAreaX = self:getWidthPart(0.5)
  local rationaleTextAreaY = self:getHeight() - marginSize
  rationaleTextArea:setLoc(rationaleTextAreaX, rationaleTextAreaY)
  self:insert(rationaleTextArea)
  self.rationaleTextArea = rationaleTextArea
end

function SwapRationalePanel.new()
  local width = Screen.width
  local height = Screen.height * HEIGHT_PROPORTION
  local self = FilledRectangle.new(width, height, BACKGROUND_COLOR)
  Mixin.mix(self, SwapRationalePanel)

  createLayout(self)

  self.slideActionInterrupt = ActionInterrupt.new()

  self:setAnchorPointBottom()

  return self
end
