--- SwapVictoryShine
-- @author Created by Aaron Barrett on April 23, 2014.

class("SwapVictoryShine")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.fullWidth
local HEIGHT = Screen.fullHeight * 3.0

local CENTER_X = WIDTH * 0.5
local CENTER_Y = HEIGHT * 0.5

local STRIPE_HEIGHT = Screen.fullWidth + Screen.fullHeight
local TOP_STRIPE_WIDTH = Screen.fullHeight * 0.02
local BOTTOM_STRIPE_WIDTH = Screen.fullHeight * 0.11
local STRIPE_SPACING = Screen.fullHeight * 0.14

local TOP_STRIPE_INITIAL_Y = Screen.fullHeight * 0.25
local BOTTOM_STRIPE_INITIAL_Y = TOP_STRIPE_INITIAL_Y - STRIPE_SPACING

local STRIPE_TRAVEL_DISTANCE = Screen.fullHeight * 2.5

local SLANT_ANGLE = 45.0

local STRIPE_COLOR = {1.0, 1.0, 1.0}
local STRIPE_OPACITY = 1.0

local Z_STRIPE = 10

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SwapVictoryShine:createShineAction()
  local topStripe = self.topStripe
  local bottomStripe = self.bottomStripe

  local locationDuration = 1.2
  local locationEase = AHEaseSimpleInOut.new(2.5)
  local opacityDelayDuration = 0.4
  local opacityDuration = 0.4
  local opacityEase = AHEaseSimpleOut.new(1.5)

  local topStripeLocationAction = AHLocationYBy.new(topStripe, locationDuration, locationEase, STRIPE_TRAVEL_DISTANCE)
  local bottomStripeLocationAction = AHLocationYBy.new(bottomStripe, locationDuration, locationEase, STRIPE_TRAVEL_DISTANCE)

  local topStripeOpacityAction = AHOpacityTo.new(topStripe, opacityDuration, opacityEase, 0.0)
  local bottomStripeOpacityAction = AHOpacityTo.new(bottomStripe, opacityDuration, opacityEase, 0.0)
  local opacityAction = AHSequence.new(
                                    AHDelay.new(opacityDelayDuration),
                                    AHSpawn.new(
                                                topStripeOpacityAction,
                                                bottomStripeOpacityAction
                                                )
                                    )

  local shineAction = AHSpawn.new(
                                  topStripeLocationAction,
                                  bottomStripeLocationAction,
                                  opacityAction
                                  )
  return shineAction
end

function SwapVictoryShine:shine()
  local shineAction = self:createShineAction()
  shineAction:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SwapVictoryShine.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, SwapVictoryShine)

  local topStripe = FilledRectangle.new(TOP_STRIPE_WIDTH, STRIPE_HEIGHT, STRIPE_COLOR)
  topStripe:setRot(SLANT_ANGLE)
  topStripe:setOpacity(STRIPE_OPACITY)
  topStripe:setLoc(CENTER_X, TOP_STRIPE_INITIAL_Y)
  self:insert(topStripe, Z_STRIPE)

  local bottomStripe = FilledRectangle.new(BOTTOM_STRIPE_WIDTH, STRIPE_HEIGHT, STRIPE_COLOR)
  bottomStripe:setRot(SLANT_ANGLE)
  bottomStripe:setOpacity(STRIPE_OPACITY)
  bottomStripe:setLoc(CENTER_X, BOTTOM_STRIPE_INITIAL_Y)
  self:insert(bottomStripe, Z_STRIPE)

  self.topStripe = topStripe
  self.bottomStripe = bottomStripe

  return self
end
