--- ShapeEmitter
-- @author Created by karl on January 30, 2015.

class("ShapeEmitter")

require("SynonymsCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

ShapeEmitter.directionLeft = "left"
ShapeEmitter.directionRight = "right"
ShapeEmitter.directionDown = "down"
ShapeEmitter.directionUp = "up"

local SPEED = 100
local RADIUS = Screen.getWidthPart(0.02)
local ROUNDING_RADIUS = RADIUS * 0.3
local LINE_WIDTH = 1.5
local EMIT_DISTANCE = 70

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newSquare(color)
  return HollowRoundedRegularPolygon.new(RADIUS, 4, 0, ROUNDING_RADIUS, color, LINE_WIDTH)
end

local function newTriangle(color)
  return HollowRoundedRegularPolygon.new(RADIUS, 3, 0, ROUNDING_RADIUS, color, LINE_WIDTH)
end

local function newCircle(color)
  return HollowCircle.new(RADIUS, color, LINE_WIDTH)
end

local function getAngles(direction)
  if direction == ShapeEmitter.directionLeft then
    return {math.pi * 1.15, math.pi, math.pi * 0.85}
  elseif direction == ShapeEmitter.directionRight then
    return {math.pi * 0.15, 0, math.pi * -0.15}
  elseif direction == ShapeEmitter.directionUp then
    return {math.pi * 0.3, math.pi * 0.6, math.pi * 1.5}
  else
    return {math.pi * 1.3, math.pi * 1.6, math.pi * 0.5}
  end
end

local function emitShape(self, shape, angle, distance, duration)
  return runInCoroutine(function()
    self:insert(shape)
    local x = math.cos(angle) * distance
    local y = math.sin(angle) * distance
    AHLocationTo.new(shape, duration * 0.7, AHEaseExponentialOut.new(), x, y):start()
    AHRotationTo.new(shape, duration * 5, nil, 180):start()
    waitFor(duration * 0.5)
    wait(AHOpacityTo.new(shape, duration, nil, 0):start())
    shape:removeSelf()
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function ShapeEmitter:emit()
  local distance = EMIT_DISTANCE
  local angles = getAngles(self.direction)
  local shapes = {
    newTriangle(self.colors[1]),
    newCircle(self.colors[2]),
    newSquare(self.colors[3])
  }
  return runInCoroutine(function()
    local baseDuration = distance / SPEED
    local lastAction
    for i=1, #angles do
      lastAction = emitShape(self, shapes[i], angles[i], distance, baseDuration)
    end
    wait(lastAction)
  end)
end

function ShapeEmitter.emitFrom(group, colors, isVertical, xScale)
  xScale = xScale or 1
  return runInCoroutine(function()
    local x, y = group:getLocationWithAnchorPoint(0.5, 0.5)
    local width = group:getEffectiveWidth() * xScale
    local xOffset = EMIT_DISTANCE * 0.6
    local xRight = x + width / 2 - xOffset
    local xLeft = x - width / 2 + xOffset
    local directions = {ShapeEmitter.directionLeft, ShapeEmitter.directionRight}
    if isVertical then
      directions = {ShapeEmitter.directionDown, ShapeEmitter.directionUp}
      TableUtils.shuffleArray(directions)
    end

    local emitterLeft = ShapeEmitter.new(colors, directions[1])
    emitterLeft:setLoc(xLeft, y)
    group.parent:insert(emitterLeft)
    local emitterRight = ShapeEmitter.new(colors, directions[2])
    emitterRight:setLoc(xRight, y)
    group.parent:insert(emitterRight)

    wait(emitterRight:emit(), emitterLeft:emit())
    emitterRight:removeSelf()
    emitterLeft:removeSelf()
  end)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function ShapeEmitter.new(colors, direction)
  local self = Group.new(0, 0)
  Mixin.mix(self, ShapeEmitter)

  self.colors = colors
  self.direction = direction or ShapeEmitter.directionRight

  return self
end
