--- SynonymsChoiceButton
-- @author Created by karl on January 30, 2015.

class("SynonymsChoiceButton")

require("SynonymsCommon")
require("ShapeEmitter")

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function scaleLabelIfNeeded(label, maxWidth)
  if (label:getWidth() > maxWidth) then
    label:setEffectiveDims(maxWidth)
  end
end

local function createColorChangeAnimation(self, labelColor)
  local duration = 0.1
  return AHSpawn.new(
      AHColorTo.new(self.label, duration, nil, labelColor),
      AHOpacityTo.new(self.fill, duration, nil, 1)
    )
end

local function createCorrectAnimation(self)
  local durationDown = 0.05
  local durationUp = 0.1
  return AHSequence.new(
    AHScaleTo.new(self, durationDown, AHEaseSineInOut.new(), 0.9),
    function() ShapeEmitter.emitFrom(self, {SynonymsCommon.COLORS.WHITE, SynonymsCommon.COLORS.WHITE, SynonymsCommon.COLORS.WHITE}, false, 1.1) end,
    AHSpawn.new(
      createColorChangeAnimation(self, self.selectedColor),
      AHScaleTo.new(self, durationUp, AHEaseSineInOut.new(), 1.0)
    )
  )
end

local function createIncorrectFillAnimation(self)
  local duration = 0.1
  local ease = AHEaseLinear.new()
  return AHOpacityTo.new(self.fill, duration, ease, 1.0)
end

local function createIncorrectLabelAnimation(self)
  local delay = 0.02
  local fadeDuration = 0.08
  local hold = 0.1
  local endColor = SynonymsCommon.COLORS.BLACK_60

  return AHSequence.new(
    AHDelay.new(delay),
    AHColorTo.new(self.label, fadeDuration, ease, SynonymsCommon.COLORS.PENALTY_RED),
    AHDelay.new(hold),
    AHColorTo.new(self.label, fadeDuration, ease, endColor)
  )
end

local function createStrikethroughAnimation(self)
  local delay = 0.333
  local strikethroughAnimation = SynonymsCommon.createStrikethroughAnimation(self.label)
  return AHDelayed.new(delay, strikethroughAnimation)
end

local function createIncorrectAnimation(self)
  local incorrectWiggleAnimation = SynonymsCommon.createWiggleAnimation(self)
  local incorrectFillAnimation = createIncorrectFillAnimation(self)
  local incorrectLabelAnimation = createIncorrectLabelAnimation(self)
  local strikethroughAnimation = createStrikethroughAnimation(self)

  return AHSpawn.new(
    incorrectWiggleAnimation,
    incorrectFillAnimation,
    incorrectLabelAnimation,
    strikethroughAnimation
  )
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsChoiceButton:animateTouch(isCorrect)
  if isCorrect then
    return createCorrectAnimation(self):start()
  else
    return createIncorrectAnimation(self):start()
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SynonymsChoiceButton.new(text, height, fontFace, normalColor, selectedColor, onPress)
  local width = Screen.getWidthPart(0.77)
  local self = Group.new(width, height)
  Mixin.mix(self, SynonymsChoiceButton)

  local maxWidth = self:getWidthPart(0.8)
  local fontSize = 37.5
  local lineWidth = 2
  local cornerRadius = 2
  local color = normalColor
  local horizontalAlignment = Alignment.CENTER

  self.edge = HollowRoundedRectangle.new(width, height, cornerRadius, color, lineWidth)
  self:insertAtCenter(self.edge)
  self.fill = FilledRoundedRectangle.new(width, height, cornerRadius, SynonymsCommon.COLORS.WHITE)
  self.fill:setOpacity(0)
  self:insertAtCenter(self.fill)

  self.label = Label.new(text, Font.new(fontFace, fontSize), nil, horizontalAlignment)
  scaleLabelIfNeeded(self.label, maxWidth)
  self.label:setColor(unpack(color))
  self:insertAtCenter(self.label)

  self.selectedColor = selectedColor

  Button.makeButton(self, onPress)
  self:linkColorAttrToChildren(true)

  return self
end
