--- SynonymsCommon
-- @author Created by karl on January 22, 2015.

class("SynonymsCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SynonymsCommon.COLORS = {
  WHITE = {1, 1, 1},
  GREY = ColorUtils.colorHex24ToStd(0xcccccc),
  DARK_GREY = ColorUtils.colorHex24ToStd(0x848484),
  DARK_GREY_2 = ColorUtils.colorHex24ToStd(0x777777),
  DARK_GREY_3 = ColorUtils.colorHex24ToStd(0x555555),
  DARK_GREY_4 = {0.14, 0.12, 0.13, 0.6},
  DARK_GREY_5 = {0.36, 0.36, 0.36},
  LIGHT_GREY = ColorUtils.colorHex24ToStd(0xdddddd),
  LIGHT_GREY_2 = {0.65, 0.65, 0.65},
  BLUE = ColorUtils.colorHex24ToStd(0x01b0f0),
  PURPLE = ColorUtils.colorHex24ToStd(0x8631f4),
  PINK = ColorUtils.colorHex24ToStd(0xff1993),
  YELLOW = ColorUtils.colorHex24ToStd(0xffd524),
  RED = ColorUtils.colorHex24ToStd(0xff6029),
  PENALTY_RED = {1, 0, 0, 1},
  BLACK = {0, 0, 0},
  BLACK_30 = {0, 0, 0, 0.3},
  BLACK_50 = {0, 0, 0, 0.5},
  BLACK_60 = {0, 0, 0, 0.6}
}

SynonymsCommon.COLOR_PROGRESSION = {
  SynonymsCommon.COLORS.BLUE,
  SynonymsCommon.COLORS.PURPLE,
  SynonymsCommon.COLORS.PINK,
}

SynonymsCommon.FONTS = {
  LIGHT = FontFaces.DIN_OT_LIGHT,
  MEDIUM = FontFaces.DIN_OT_MEDIUM,
  BOLD = FontFaces.DIN_OT_BOLD,
}

SynonymsCommon.TAGS = {
  SCORE = "score",
  TO_PASS = "toPass"
}

SynonymsCommon.MODES = {
  NORMAL = "NORMAL",
  MULTI = "MULTI",
  PICK = "50/50",
  HINT = "HINT"
}

SynonymsCommon.TIMINGS = {
  INCORRECT_FADE_DELAY = 0.333,
  INCORRECT_FADE_DURATION = 0.333,
  INCORRECT_SCALE_DURATION = 0.5
}

SynonymsCommon.COMMON_EASE = AHEaseBezier.newFlat({0.3, 1.0, 0.6, 1.0})
SynonymsCommon.INCORRECT_MOVE_DISTANCE = Screen.getHeightPart(0.01)

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsCommon.checkPartOfSpeech(concept, answer)
  local isNoun = concept.nouns[answer]
  local isVerb = concept.verbs[answer]

  return isNoun, isVerb
end

function SynonymsCommon.createStrikethroughAnimation(label)
  local ease = SynonymsCommon.COMMON_EASE
  local duration = 0.5
  local strikethroughWidth = label:getWidth() + label:getHeightPart(0.4)
  local strikethrough = FilledRectangle.new(strikethroughWidth, 4, SynonymsCommon.COLORS.BLACK)
  strikethrough:setAnchorPointLeft()
  strikethrough:setLoc(label:getWidthPart(0.5) - (strikethroughWidth / 2), label:getHeightPart(0.4))
  strikethrough:setScl(0, 1)
  label:insert(strikethrough)
  return AHScaleXTo.new(strikethrough, duration, ease, 1)
end

function SynonymsCommon.createWiggleAnimation(label)
  local wiggleDuration = 0.05
  local wiggleEase = AHEaseLinear.new()
  local wiggleDistance = Screen.width * 0.024
  local wiggleAnimation = AHSequence.new(
    AHLocationXBy.new(label, wiggleDuration, wiggleEase, -wiggleDistance),
    AHLocationXBy.new(label, wiggleDuration*2, wiggleEase, wiggleDistance*2),
    AHLocationXBy.new(label, wiggleDuration*2, wiggleEase, -wiggleDistance*2),
    AHLocationXBy.new(label, wiggleDuration, wiggleEase, wiggleDistance)
  )
  return wiggleAnimation
end
