--- SynonymsConfiguration
-- @author Created by karl on January 05, 2015.

class("SynonymsConfiguration")

require("data.keyframes.KeyframeSet")
require("data.generators.LearningFocusedRoundGenerator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

SynonymsConfiguration.GAME_DURATION_IN_SECONDS = 70
SynonymsConfiguration.PASS_SCORE = 6000
SynonymsConfiguration.LOST_SECONDS_ON_SKIP = 1

SynonymsConfiguration.TIME_REMAINING_TO_START_RESURFACING_MISTAKES =
  SynonymsConfiguration.GAME_DURATION_IN_SECONDS *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING

SynonymsConfiguration.MAXIMUM_CONTENT_REPETITION_TIMES = 1

local BONUS_BASE_POINTS = 1000
local MIN_BONUS_POINTS = 10
local BONUS_DECAY_RATE = 0.7

local BLOCKS_NEEDED_BY_DIFFICULTY_KEYFRAMES = KeyframeSet.newInterpolated(
  -- difficulty, {Pass, Rank Two, Rank Three}
  Keyframe.new(0.0, {1, 6, 11}),
  Keyframe.new(0.25, {1, 9, 13}),
  Keyframe.new(0.5, {1, 9, 14}),
  Keyframe.new(0.75, {1, 10, 16}),
  Keyframe.new(0.95, {1, 10, 18}),
  Keyframe.new(1.0, {1, 11, 19})
)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getRankRequirements()
  local difficulty = GameSession.getDifficulty()
  return BLOCKS_NEEDED_BY_DIFFICULTY_KEYFRAMES:getFrameAt(difficulty)
end

local function getBlocksRequiredForRank(rankIndex)
  local rankRequirements = getRankRequirements()
  return MathUtils.round(rankRequirements[rankIndex])
end

local function getPointsNeededForRank(rankIndex)
  local extraBlocksRequired = getBlocksRequiredForRank(rankIndex) - getBlocksRequiredForRank(1)
  local extraPointsNeeded = SynonymsConfiguration.getDecayingBonusPoints(extraBlocksRequired)
  return SynonymsConfiguration.PASS_SCORE + extraPointsNeeded
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Cuts the bonus given in half with each one earned, to prevent people from getting craaazy high scores
function SynonymsConfiguration.getDecayingBonusPoints(bonusCount)
  local totalBonusPoints = 0
  local pointsIncrement = BONUS_BASE_POINTS

  for i = 1, bonusCount do
    totalBonusPoints = totalBonusPoints + pointsIncrement

    pointsIncrement = math.max(MIN_BONUS_POINTS, MathUtils.round(pointsIncrement * BONUS_DECAY_RATE))
  end

  return totalBonusPoints
end

function SynonymsConfiguration.getQuestionsCountToPass()
  return getBlocksRequiredForRank(1)
end

function SynonymsConfiguration.getReportedScoreForAnswerScore(answerScore)
  return DIFFICULTY_SCORES[answerScore]
end

function SynonymsConfiguration.getRankTwoScore()
  local rankTwoScore = getPointsNeededForRank(2)
  return rankTwoScore
end

function SynonymsConfiguration.getRankThreeScore()
  local rankThreeScore = getPointsNeededForRank(3)
  return rankThreeScore
end

function SynonymsConfiguration.getDifficultyKeyFrames()
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Medium = 1.0
      }
    },
    {
      key = 0.5625,
      valueProbabilities = {
        Medium = 0.25,
        Hard = 0.75
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Medium = 0.25,
        Hard = 0.75
      }
    }
  }
end
