--- SynonymsHelperBank
-- @author Created by karl on January 28, 2015.

class("SynonymsHelperBank")

require("SynonymsHelperButton")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTONS = {
  {
    text = Localizer.get("MULTI"),
    count = 1
  },
  {
    text = Localizer.get("HINT"),
    count = 1
  },
  {
    text = Localizer.get("50_50"),
    count = 1
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setButtonsEnabled(self, isEnabled)
  for _, button in ipairs(self.children) do
    if button.setTouchEnabled then
      button:setTouchEnabled(isEnabled)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsHelperBank:reset()
  setButtonsEnabled(self, true)
  if self.circle then
    self.circle:removeSelf()
    self.circle = nil
  end
end

function SynonymsHelperBank:relinquishSkipButton(newParent, zIndex)
  newParent:takeChild(self.skipButton, zIndex)
  self.skipButton:setTouchEnabled(true)
end

function SynonymsHelperBank:retakeSkipButton()
  self:takeChild(self.skipButton)
end

function SynonymsHelperBank:getFadeButtonsAction(duration, opacity)
  local hideAction = AHRecursive.new(AHOpacityTo.new, self, duration, nil, opacity)

  return hideAction
end

function SynonymsHelperBank:setTouchEnabled(isEnabled)
  for _, button in ipairs(self.children) do
    if button.setTouchWithoutVisibility then
      button:setTouchWithoutVisibility(isEnabled)
    end
  end

  self.skipButton:setTouchEnabled(isEnabled)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newButton(self, text, color, count, callback)
  local button
  button = SynonymsHelperButton.new(text, color, count, function()
    setButtonsEnabled(self, false)
    callback(button, color)
    Sound.playSoundEffect("adjectives_powerup")
  end)
  return button
end

local function createSkipButton(self, skipCallback)
  local skipButton = Group.new(Screen.safeWidth - self:getWidth(), self:getHeight())

  local skipBackground = Sprite.newWithName("skip_button.png")
  skipBackground:setAnchorPointBottomLeft()
  skipBackground:setLoc(0, self:getHeightPart(0.1))
  skipButton:insert(skipBackground)

  local skipLabel = Label.new(Localizer.get("SKIP"), Font.new(SynonymsCommon.FONTS.MEDIUM, skipBackground:getHeightPart(0.6)))
  skipLabel:setColor(unpack(SynonymsCommon.COLORS.DARK_GREY_3))
  skipLabel:setLoc(skipButton:getWidthPart(0.55), skipBackground:getHeightPart(0.5))
  skipBackground:insert(skipLabel)

  Button.makeButton(skipButton, function()
    runInCoroutine(function()
      setButtonsEnabled(self, false)
      skipCallback()
    end)
  end)

  return skipButton
end

-- Constructor ------------------------

function SynonymsHelperBank.new(onMulti, onHint, onPick, onSkip)
  local self = Row.new(3, Alignment.BOTTOM)
  Mixin.mix(self, SynonymsHelperBank)

  local callbacks = {onMulti, onHint, onPick}

  for i = 1, #BUTTONS do
    local buttonData = BUTTONS[i]
    local color = SynonymsCommon.COLOR_PROGRESSION[i]
    local callback = callbacks[i]

    self:insert(newButton(self, buttonData.text, color, buttonData.count, callback))
  end

  self:alignChildren()
  self.skipButton = createSkipButton(self, onSkip)
  self:insert(self.skipButton)

  self:alignChildren()
  self:linkColorAttrToChildren()

  return self
end
