--- SynonymsHelperButton
-- @author Created by karl on January 22, 2015.

class("SynonymsHelperButton")

require("SynonymsCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BUTTON_WIDTH = Screen.getSafeWidthPart(0.26)
local BUTTON_HEIGHT = BUTTON_WIDTH / 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function onTouchDown(self)
  self.hitCount = self.hitCount + 1
  self:setTouchEnabled(false)
  self.background:setOpacity(1)
  self.onPress()
end

local function hasBeenHitMaxTimes(self)
  return self.hitCount >= self.maxHits
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsHelperButton:setTouchEnabled(isEnabled)
  if (isEnabled and (not hasBeenHitMaxTimes(self))) then
    self:setVisibleRecursive(true)
    self.label:setOpacity(1)
    self.background:setOpacity(1)
    self.super.setTouchEnabled(self, isEnabled)
  elseif not isEnabled then
    self:setVisibleRecursive(false)
    self.label:setOpacity(0)
    self.background:setOpacity(0)
    self.super.setTouchEnabled(self, isEnabled)
  end
end

function SynonymsHelperButton:setTouchWithoutVisibility(isEnabled)
  if hasBeenHitMaxTimes(self) then
    isEnabled = false
  end
  self.super.setTouchEnabled(self, isEnabled)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SynonymsHelperButton.new(text, color, maxHits, onPress)
  local self = Group.new(BUTTON_WIDTH, BUTTON_HEIGHT)
  Button.makeButton(self, function() onTouchDown(self) end)
  Mixin.mix(self, SynonymsHelperButton)

  local margin = self:getHeightPart(0.1)
  local rectW, rectH = self:getWidth(), self:getHeight() - margin * 2
  local cornerRadius = rectH * 0.5

  local background = FilledRoundedRectangle.new(rectW, rectH, cornerRadius, color)
  self:insertAtCenter(background)

  local label = Label.new(text, Font.new(SynonymsCommon.FONTS.MEDIUM, self:getHeightPart(0.37)))
  label:setColor(unpack(SynonymsCommon.COLORS.WHITE))
  self:insertAtCenter(label)

  self:setColor(unpack(color))

  self.onPress = onPress
  self.maxHits = maxHits
  self.hitCount = 0

  self.background = background
  self.label = label

  self:setTouchEnabled(true)

  return self
end
