--- SynonymsHintAnswerMode
-- @author Created by karl on February 02, 2015.

class("SynonymsHintAnswerMode")

require("SynonymsAnswerMode")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsHintAnswerMode:animateIn()
  self.isKeyboardEnabled = true
  DeviceKeyboard.clear()
end

function SynonymsHintAnswerMode:animateAutocorrect(answer)
  local hintLength = #self.hintString
  local remainingAnswerStartIndex = hintLength + 1
  local remainingAnswerEndIndex = #answer
  local remainingAnswerString = answer:sub(remainingAnswerStartIndex, remainingAnswerEndIndex)

  self.super.animateAutocorrect(self, remainingAnswerString)
end

function SynonymsHintAnswerMode:getSubmittedText(answerText)
  return string.lower(self.hintString .. answerText)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SynonymsHintAnswerMode.new(concept, onAnswer)
  local self = SynonymsAnswerMode.new(concept, onAnswer)
  Mixin.mix(self, SynonymsHintAnswerMode)
  self.mode = SynonymsCommon.MODES.HINT

  self.inputField:setIsColorful(true)
  self.inputField:setCorrectForcedColor(SynonymsCommon.COLORS.BLACK)
  self.inputField:setIncorrectBoldColor(SynonymsCommon.COLORS.DARK_GREY_4)
  self.promptLabel:setIncorrectColor(SynonymsCommon.COLORS.BLACK_30)
  self.suggestionLabel:setIncorrectColor(SynonymsCommon.COLORS.WHITE)

  local hint = TableUtils.getRandomElement(concept.hints)

  local length = math.min(math.ceil(#hint * 0.5), 3)
  local hintString = string.sub(hint, 1, length)
  self.inputField:addHint(hintString)
  self.hintString = hintString

  self.suggestionLabel:setCorrectAnswer(hint)

  local label = Label.new(Localizer.get("INSTRUCTIONS_FINISH"), Font.new(SynonymsCommon.FONTS.BOLD, 17))
  label:setColor(unpack(SynonymsCommon.COLORS.BLACK_50))
  label:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.9))
  self:insert(label)

  self.hint = hint
  self.concept = concept

  return self
end
