--- SynonymsIntro
-- @author Created by karl on March 17, 2015.

class("SynonymsIntro")

require("SynonymsCommon")
require("SynonymsScoreRectangle")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function newLabel(text, font, size, color)
  local label = Label.new(text, Font.new(font, size))
  if color then
    label:setColor(unpack(color))
  end
  return label
end

local function newTextLink(text, font, size, onPressed)
  local color = {0.71, 0.71, 0.71}
  local buttonGroup = Row.new(Screen.getWidthPart(0.015))
  local buttonLabel = newLabel(text, font, size, color)
  buttonGroup:insert(buttonLabel)
  local arrow = Sprite.newWithName("arrow.png")
  arrow:setColor(unpack(color))
  buttonGroup:insert(arrow)
  buttonGroup:alignChildren()
  local buttonWidth = buttonGroup:getWidthPart(1.5)

  Button.addVirtualButton(buttonGroup, buttonWidth, buttonWidth, onPressed)
  return buttonGroup
end

local function newButton(text, onPressed)
  local button = Group.new(Screen.getWidthPart(0.8), Screen.getWidthPart(0.15))
  local fontSize = 19.5
  local upColor = SynonymsCommon.COLORS.WHITE
  local textColor = SynonymsCommon.COLORS.DARK_GREY_3
  button:insertAtCenter(FilledRoundedRectangle.new(button:getWidth(), button:getHeight(), button:getHeightPart(0.1), upColor))
  button:insertAtCenter(newLabel(text, SynonymsCommon.FONTS.BOLD, fontSize, textColor))
  button:linkColorAttrToChildren(true)
  Button.makeButton(button, function()
    onPressed()
    button:setTouchEnabled(false)
  end)
  return button
end

local function newSubheaderLabel(self)
  local font = SynonymsCommon.FONTS.MEDIUM
  local fontSize = 20
  local color = {0.8, 0.8, 0.8, 1}
  local row = Row.new(fontSize * 0.4)

  row:insert(newLabel(Localizer.get("HOW_TO_PLAY_SUBHEADER") .. " ", font, fontSize, color))
  row:insert(SynonymsScoreRectangle.new(fontSize * 0.85))
  row:alignChildren()
  local maxRowWidth = self:getWidthPart(0.95)
  row:setEffectiveWidth(math.min(row:getEffectiveWidth(), maxRowWidth))
  return row
end

local function newMultilineLabel(text, font, fontSize, color)
  local label = DynamicTextArea.newSimple(text, Screen.getWidthPart(0.8), Font.new(font, fontSize), Alignment.CENTER)
  if color then
    label:setColor(unpack(color))
  end
  return label
end

local function insertBottomButton(target, text, onPressed)
  local button = newButton(text, onPressed)
  button:setAnchorPointBottom()
  local buttonY = math.max(Screen.getHeightPart(0.05), Screen.getSafeY(0.02))
  button:setLoc(target:getWidthPart(0.5), buttonY)
  target:insert(button)
  target.button = button
end

local function hidePages(self)
  return runInCoroutine(function()
    self:setTouchEnabled(false)
    local lastAction
    for _, page in ipairs(self.pages) do
      lastAction = AHOpacityTo.new(page, 0.3, nil, 0):start()
    end
    wait(lastAction)
  end)
end

local function showPage(self, pageNumber)
  return runInCoroutine(function()
    local page = self.pages[pageNumber]
    wait(AHOpacityTo.new(page, 0.3, nil, 1):start())
    page:setTouchEnabled(true)
  end)
end

local function changePage(self, pageNumber)
  return runInCoroutine(function()
    wait(hidePages(self))
    wait(showPage(self, pageNumber))
  end)
end

local function endIntro(self)
  return runInCoroutine(function()
    AHColorTo.new(self.bg, 0.3, nil, {1, 1, 1}):start()
    wait(hidePages(self))
    self.isEnded = true
  end)
end

local function newPage()
  local page = Group.newFullScreen()
  FunctionUtils.forwardMethodCallsToChildren(page, true, "setTouchEnabled")
  return page
end

local function newPage1(self)
  local page = newPage()
  local boldFont = SynonymsCommon.FONTS.BOLD
  local mediumFont = SynonymsCommon.FONTS.MEDIUM
  local largeFontSize = 38
  local smallFontSize = 16
  local white = SynonymsCommon.COLORS.white

  local col = Column.new(0)
  col:insert(newLabel(Localizer.get("HOW_TO_PLAY"), boldFont, largeFontSize, white))
  col:insert(newSubheaderLabel(self))

  col:insertSpacer(Screen.getWidthPart(0.075))

  local bodyText = TextArea.new(
    Localizer.get("HOW_TO_PLAY_BODY_TEXT"),
    Font.new(mediumFont, smallFontSize),
    Screen.getWidthPart(0.9),
    Alignment.CENTER
  )
  col:insert(bodyText)

  col:insertSpacer(24)

  col:insert(newTextLink(Localizer.get("LEARN_MORE"), mediumFont, 14, function() changePage(self, 2) end))
  col:alignChildren()
  page:insertAtLocationFractional(col, 0.5, 0.51)
  insertBottomButton(page, Localizer.get("BEGIN"), function() endIntro(self) end)
  page:linkColorAttrToChildren(true)

  return page
end

local function newScoreDescription(score, name, examples)
  local lightFont = SynonymsCommon.FONTS.LIGHT
  local boldFont = SynonymsCommon.FONTS.BOLD
  local largeFontSize = 22.5
  local smallFontSize = 17
  local white = SynonymsCommon.COLORS.white
  local examplesString = string.format("%s %s", Localizer.get("EXAMPLES_PREFIX"), examples)

  local col = Column.new()
  col:insert(SynonymsScoreRectangleSet.new(largeFontSize * 3, score))
  col:insertSpacer(2)
  col:insert(newLabel(name, boldFont, largeFontSize, white))
  col:insert(newLabel(examplesString, lightFont, smallFontSize, SynonymsCommon.COLORS.WHITE))
  col:alignChildren()
  return col
end

local function newPage2(self)
  local page = newPage()
  local boldFont = SynonymsCommon.FONTS.BOLD
  local mediumFont = SynonymsCommon.FONTS.MEDIUM
  local largeFontSize = 30
  local smallFontSize = 19
  local white = SynonymsCommon.COLORS.white
  local grey = {0.9, 0.9, 0.9}
  local col = Column.new(0)
  col:insert(newLabel(Localizer.get("HOW_SCORING_WORKS"), boldFont, largeFontSize, white))
  col:insert(newMultilineLabel(Localizer.get("SCORING_EXPLAINER"), mediumFont, smallFontSize, grey))
  col:insertSpacer(Screen.getWidthPart(0.05))
  col:insert(newScoreDescription(1, Localizer.get("COMMON"), Localizer.get("EXAMPLES_COMMON")))
  col:insertSpacer(Screen.getWidthPart(0.07))
  col:insert(newScoreDescription(2, Localizer.get("UNCOMMON"), Localizer.get("EXAMPLES_UNCOMMON")))
  col:insertSpacer(Screen.getWidthPart(0.07))
  col:insert(newScoreDescription(3, Localizer.get("EXPERT"), Localizer.get("EXAMPLES_EXPERT")))
  col:insertSpacer(Screen.getWidthPart(0.2))
  col:alignChildren()
  page:insertAtCenter(col)
  insertBottomButton(page, Localizer.get("OKAY"), function() changePage(self, 1) end)
  page:linkColorAttrToChildren(true)

  local maxColumnHeight = Screen.safeHeight - page.button:getYTop()
  if (col:getHeight() > maxColumnHeight) then
    col:setEffectiveDims(nil, maxColumnHeight)
  end

  return page
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsIntro:run()
  return runInCoroutine(function()
    local firstPageNumber = 1
    if self.shouldShowPage2First then
      firstPageNumber = 2
    end
    showPage(self, firstPageNumber)
    waitUntil(function() return self.isEnded end)
  end)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function insertPage(self, page)
  table.insert(self.pages, page)
  self:insertAtCenter(page)
  page:setOpacity(0)
end

-- Constructor ------------------------

function SynonymsIntro.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, SynonymsIntro)

  self.bg = FilledRectangle.new(self:getWidth(), self:getHeight(), SynonymsCommon.COLORS.DARK_GREY)
  self:insertAtCenter(self.bg)

  self.pages = {}
  insertPage(self, newPage1(self))
  insertPage(self, newPage2(self))

  self:linkColorAttrToChildren(true)

  FunctionUtils.forwardMethodCallsToChildren(self, false, "setTouchEnabled")
  self:setTouchEnabled(false)

  self.shouldShowPage2First = GameSession.hasNeverWon()

  return self
end
