--- SynonymsPickAnswerMode
-- @author Created by karl on February 02, 2015.

class("SynonymsPickAnswerMode")

require("SynonymsCommon")
require("SynonymsChoiceButton")
require("SynonymsPromptLabel")
require("SynonymsAnswerMode")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function hideOtherButtons(self, exceptButton)
  for _, child in ipairs(self.buttons) do
    if child ~= exceptButton then
      child:setVisibleRecursive(false)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsPickAnswerMode:animateIn()
  for _, button in ipairs(self.buttons) do
    AHOpacityTo.new(button, 0.3, nil, 1):start()
  end
end

function SynonymsPickAnswerMode:animateSkip()
  -- Do nothing
end

function SynonymsPickAnswerMode:animateAnswer(roundResponse)
  local isCorrect = roundResponse:getIsCorrect()
  return runInCoroutine(function()
    if isCorrect then
      hideOtherButtons(self, self.selectedButton)
    end
    wait(self.selectedButton:animateTouch(isCorrect))
    waitFor(0.5)
  end)
end

function SynonymsPickAnswerMode:animateAutocorrect(answer)
  -- Do nothing
end

function SynonymsPickAnswerMode:setTouchEnabled(enabled)
  for _, button in ipairs(self.buttons) do
    button:setTouchEnabled(enabled)
  end
end

function SynonymsPickAnswerMode:prepareForRemoval()
  -- Do nothing
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newButton(self, answer, onAnswer)
  local height = Screen.getWidthPart(0.2)
  local font = SynonymsCommon.FONTS.LIGHT
  local normalColor = SynonymsCommon.COLORS.BLACK_60
  local hightlightColor = SynonymsCommon.COLORS.PINK
  local button = nil
  button = SynonymsChoiceButton.new(answer, height, font, normalColor, hightlightColor, function()
    self.selectedButton = button
    onAnswer(answer)
  end)
  button:setOpacity(0)
  return button
end

-- Constructor ------------------------

function SynonymsPickAnswerMode.new(concept, onAnswer)
  local self = SynonymsAnswerMode.new(concept, onAnswer)
  Mixin.mix(self, SynonymsPickAnswerMode)
  self.mode = SynonymsCommon.MODES.PICK

  self.inputField:removeSelf()
  self.suggestionLabel:removeSelf()

  local answers = {
    TableUtils.getRandomElement(concept.correct),
    TableUtils.getRandomElement(concept.distractors)
  }
  SUGGESTION_GENERATOR:cacheSuggestionForAnswer(concept, answers[1], answers[2])
  TableUtils.shuffleArray(answers)

  self.buttons = {}
  self.elementColumn:insertSpacer(20)
  for i=1,2 do
    local answer = answers[i]
    local button = newButton(self, answer, onAnswer)
    self.elementColumn:insert(button)
    self.elementColumn:insertSpacer(20)
    table.insert(self.buttons, button)
  end
  self.elementColumn:alignChildren()

  local label = Label.new(Localizer.get("INSTRUCTIONS_PICK"), Font.new(SynonymsCommon.FONTS.BOLD, 17))
  label:setColor(unpack(SynonymsCommon.COLORS.BLACK_50))
  label:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.9))
  self:insert(label)

  return self
end
