--- SynonymsRationale
-- @author Created by Steven Mah on June 12, 2017

class("SynonymsRationale")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {0.11, 0.11, 0.11}
local GRAY = {0.47, 0.47, 0.47}

local ANSWER_TEMPLATE = Localizer.get("ANSWER_TEMPLATE")
local QUESTION_TEXT = Localizer.get("QUESTION_TEXT")
local NOUN_TEXT = Localizer.get("NOUN_TEXT")
local ADJECTIVE_TEXT = Localizer.get("ADJECTIVE_TEXT")
local VERB_TEXT = Localizer.get("VERB_TEXT")

local NOUN_EXAMPLES = Localizer.get("NOUN_EXAMPLES")
local VERB_EXAMPLES = Localizer.get("VERB_EXAMPLES")
local ADJECTIVE_EXAMPLES = Localizer.get("ADJECTIVE_EXAMPLES")

local TEXT_WIDTH = Screen.getWidthPart(0.9)

local BIG_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 20)
local SMALL_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 18)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, submittedWord, isNoun, isVerb)
  local background = FilledRectangle.newFullScreen(BACKGROUND_COLOR)
  self:insertAtCenter(background)
  self.background = background

  local columnSpacing = Screen.getHeightPart(0.01)
  local contentColumn = Column.new(columnSpacing, Alignment.LEFT)

  local partOfSpeech
  if (isNoun) then
    partOfSpeech = "noun"
  elseif (isVerb) then
    partOfSpeech = "verb"
  else
    error("Rationale triggered, but submitted word is not a noun or a verb.")
  end

  local capitalizedWord = StringUtils.capitalizeFirstLetter(submittedWord)
  local explanationText = string.format(ANSWER_TEMPLATE, capitalizedWord, partOfSpeech)
  local baseAttributes = {
    font = BIG_FONT,
    color = GRAY
  }
  local attributesForLabels = {
    strong = {
      color = {1.0, 1.0, 1.0}
    }
  }
  local lineSpacing = Screen.getHeightPart(0.015)
  local explanationView = DynamicTextArea.new(explanationText, TEXT_WIDTH, baseAttributes, attributesForLabels, nil, nil, lineSpacing)
  contentColumn:insert(explanationView)
  contentColumn:insertSpacer(Screen.getHeightPart(0.065))

  local questionLabel = Label.new(QUESTION_TEXT, BIG_FONT)
  contentColumn:insert(questionLabel)
  contentColumn:insertSpacer(Screen.getHeightPart(0.02))

  local explanationLabel
  local examplesLabel
  if (isNoun) then
    explanationLabel = Label.new(NOUN_TEXT, SMALL_FONT)
    examplesLabel = Label.new(NOUN_EXAMPLES, SMALL_FONT)
  elseif (isVerb) then
    explanationLabel = Label.new(VERB_TEXT, SMALL_FONT)
    examplesLabel = Label.new(VERB_EXAMPLES, SMALL_FONT)
  else
    error("Rationale triggered, but submitted word is not a noun or a verb.")
  end
  examplesLabel:setColor(unpack(GRAY))
  contentColumn:insert(explanationLabel)
  contentColumn:insert(examplesLabel)
  contentColumn:insertSpacer(Screen.getHeightPart(0.01))

  local adjectivesExplanationLabel = Label.new(ADJECTIVE_TEXT, SMALL_FONT)
  contentColumn:insert(adjectivesExplanationLabel)
  local adjectivesExamplesLabel = Label.new(ADJECTIVE_EXAMPLES, SMALL_FONT)
  adjectivesExamplesLabel:setColor(unpack(GRAY))
  contentColumn:insert(adjectivesExamplesLabel)

  contentColumn:alignChildren()
  contentColumn:setAnchorPointTop()
  self:insertAtLocationFractional(contentColumn, 0.5, 0.69)

  local warningIcon = Sprite.newWithName("warning.png")
  warningIcon:setLoc(contentColumn:getX(), contentColumn:getYTop() + warningIcon:getHeight())
  self:insert(warningIcon)

  local returnLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  returnLabel:setColor(unpack(GRAY))
  self:insertAtLocationFractional(returnLabel, 0.5, 0.075)
end

-- Constructor ------------------------

function SynonymsRationale.new(submittedWord, isNoun, isVerb)
  local self = Group.newFullScreen()
  Mixin.mix(self, SynonymsRationale)

  createLayout(self, submittedWord, isNoun, isVerb)

  self:linkOpacityAttrToChildren(true)
  self:setOpacity(0.0)

  return self
end
