--- SynonymsScoreScroller
-- @author Created by karl on February 16, 2015.

class("SynonymsScoreScroller")

require("widgets.TouchScroller")
require("SynonymsScoreCell")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CELL_SPACING = Screen.getHeightPart(0.03)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function slideInNextContents(self)
  return runInCoroutine(function()
    Sound.playSoundEffect("adjectives_results_wordfliesin")
    self.contentsIndex = self.contentsIndex + 1
    if self.contentsIndex <= #self.scoreCells then
      local child = self.scoreCells[self.contentsIndex]
      local isLastScoreCell = (self.contentsIndex == #self.scoreCells)
      if isLastScoreCell then
        local maxScroll = math.max(0, self.scroller:getContents():getHeight() - self.scroller:getHeight())
        self.scroller:newScrollYTo(0.3, AHEaseSineInOut.new(), maxScroll):start()
      else
        local childBottom = child:getY() - child:getHeightPart(0.5)
        if self.answerContents:getHeight() - childBottom > self.scroller:getHeight() then
          self.scroller:newScrollYBy(0.3, AHEaseSineInOut.new(), child:getHeight() + CELL_SPACING):start()
        end
      end
      wait(AHLocationXBy.new(child, 0.3, AHEaseSineOut.new(), -Screen.width):start())
    end
  end)
end

local function slideInStreakNotificationIfNeeded(self)
  local text = self.streakMarkers[self.contentsIndex]
  if text then
    Sound.playSoundEffect("adjectives_results_hotstreak")
    local child = self.scoreCells[self.contentsIndex]
    local band = FilledRectangle.new(self:getWidthPart(1.5), child:getHeight(), SynonymsCommon.COLORS.YELLOW)
    local label = Label.new(text, Font.new(SynonymsCommon.FONTS.BOLD, 37))
    band:insertAtCenter(label)
    band:setLocUsingAnchorPoint(0, child:getY() + child:getHeight(), nil, 1, 1)
    self.answerContents:insert(band)
    AHSequence.new(
      AHDelay.new(0.1),
      AHLocationXTo.new(band, 0.5, AHEaseSineOut.new(), self:getWidthPart(0.5)),
      AHDelay.new(0.75),
      AHLocationXTo.new(band, 0.5, AHEaseSineIn.new(), band:getWidthPart(1.5))
    ):start()
  end
end

local function newStreakMarkers(answers)
  local notificationText = {
    [3] = Localizer.get("STREAK_THREE"),
    [6] = Localizer.get("STREAK_SIX"),
    [9] = Localizer.get("STREAK_NINE"),
    [12] = Localizer.get("STREAK_TWELVE")
  }
  local streakCount = 0
  local streakMarkers = {}
  for index, answer in ipairs(answers) do
    if (answer.score > 0) then
      streakCount = streakCount + 1
      streakMarkers[index] = notificationText[streakCount]
    else
      streakCount = 0
    end
  end
  return streakMarkers
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsScoreScroller:animateFirstEntry()
  return slideInNextContents(self)
end

function SynonymsScoreScroller:animateIn()
  return runInCoroutine(function()
    for i = 1, #self.scoreCells do
      wait(slideInNextContents(self))
      slideInStreakNotificationIfNeeded(self)
    end
  end)
end

function SynonymsScoreScroller:fadeInDisclaimer()
  local action = AHSpawn.new()
  for _, cell in ipairs(self.answerContents.disclaimerCells) do
    action:addAction(AHOpacityTo.new(cell, 0.3, nil, 0.5))
  end
  return action:start()
end

function SynonymsScoreScroller:setTouchEnabled(enabled)
  self.scroller:setTouchEnabled(enabled)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newBottomLabel(text)
  local label = Label.new(text, Font.new(SynonymsCommon.FONTS.LIGHT, 11))
  label:setColor(0, 0, 0, 0.5)
  return label
end

local function newDisclaimerCells(...)
  local disclaimerCells = {}
  for _, text in ipairs(arg) do
    table.insert(disclaimerCells, newBottomLabel(text))
  end
  return disclaimerCells
end

local function newAnswerContents(width, answers)
  local col = Column.new(0, Alignment.LEFT)
  local isTagged = true

  col.cells = {}
  col:insertSpacer(CELL_SPACING)

  for _, answer in ipairs(answers) do
    local cell = SynonymsScoreCell.new(width, answer.concept, answer.answer, answer.score, isTagged)
    col:insert(cell)
    table.insert(col.cells, cell)
    isTagged = false
    col:insertSpacer(CELL_SPACING)
  end

  col.disclaimerCells = newDisclaimerCells(
    Localizer.get("SCORED_INCORRECTLY"),
    Localizer.get("EMAIL")
  )

  for _, cell in ipairs(col.disclaimerCells) do
    cell:setOpacity(0)
    col:insert(cell)
  end

  col:insertSpacer(20)

  col:alignChildren()

  for _, child in ipairs(col.cells) do
    child:setX(child:getX() + Screen.width)
  end

  return col
end

-- Constructor ------------------------

function SynonymsScoreScroller.new(width, height, answers)
  local self = Group.new(width, height)
  Mixin.mix(self, SynonymsScoreScroller)

  self.answerContents = newAnswerContents(width, answers)
  self.scoreCells = self.answerContents.cells
  self.scroller = TouchScroller.new(width, height, self.answerContents)
  self.scroller:setInertialScrollingEnabled(true)
  self.scroller:setWillResetInitialTouchCoordinateUponScrollStart(true)
  self:insertAtCenter(self.scroller)

  self.streakMarkers = newStreakMarkers(answers)
  self.contentsIndex = 0

  return self
end
