--- SynonymsSuggestionGenerator
-- @author Created by andrewc on April 30, 2021.

class("SynonymsSuggestionGenerator")

require("SynonymsSuggestionRecord")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validateConcept(concept)
  assert(concept.prompt ~= nil, "Tried to get a suggestion for a concept with a nil prompt.")
  assert(concept.suggestions ~= nil, "Tried to get a suggestion for a concept with no suggestions.")
  Asserts.assertType(concept.suggestions, "concept.suggestions", "table")
end

local function createSuggestionRecordIfNeeded(self, concept)
  local prompt = concept.prompt
  if (self.suggestionRecordsByPrompt[prompt] == nil) then
    local suggestions = concept.suggestions
    self.suggestionRecordsByPrompt[prompt] = SynonymsSuggestionRecord.new(prompt, suggestions)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsSuggestionGenerator:getMultipleSuggestions(concept, answer, suggestionCount)
  validateConcept(concept)
  createSuggestionRecordIfNeeded(self, concept)

  return self.suggestionRecordsByPrompt[concept.prompt]:getMultipleSuggestions(answer, suggestionCount)
end

function SynonymsSuggestionGenerator:getSuggestion(concept, answer)
  validateConcept(concept)
  createSuggestionRecordIfNeeded(self, concept)

  return self.suggestionRecordsByPrompt[concept.prompt]:getSuggestion(answer)
end

function SynonymsSuggestionGenerator:markAnswersAsUsed(concept, answers)
  validateConcept(concept)
  createSuggestionRecordIfNeeded(self, concept)

  for _, answer in ipairs(answers) do
    self.suggestionRecordsByPrompt[concept.prompt]:markAnswerAsUsed(answer)
  end
end

function SynonymsSuggestionGenerator:cacheSuggestionForAnswer(concept, suggestion, answer)
  validateConcept(concept)
  createSuggestionRecordIfNeeded(self, concept)

  self.suggestionRecordsByPrompt[concept.prompt]:recordSuggestion(answer, suggestion)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function SynonymsSuggestionGenerator.new()
  local self = {}
  Mixin.mix(self, SynonymsSuggestionGenerator)

  self.suggestionRecordsByPrompt = {}

  return self
end
