--- SynonymsSuggestionRecord
-- @author Created by andrewc on June 22, 2021.

class("SynonymsSuggestionRecord")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SKIPPED_ANSWER = "answer_was_skipped"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function sanitizeAnswer(answer)
  return answer or SKIPPED_ANSWER
end

local function addNewSuggestion(self, suggestionTable)
  local doRemove = true
  local suggestion = TableUtils.getRandomElement(self.unusedSuggestions, doRemove)
  table.insert(suggestionTable, suggestion)
end

local function createSuggestionTableForAnswerIfNeeded(self, suggestion, answer)
  if (self.suggestionsByAnswer[answer] == nil) then
    self.suggestionsByAnswer[answer] = {}
  end
end

local function markSuggestionAsUsedIfNeeded(self, suggestion)
  TableUtils.removeValueFromArray(self.unusedSuggestions, suggestion)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsSuggestionRecord:recordSuggestion(answer, suggestion)
  answer = sanitizeAnswer(answer)
  createSuggestionTableForAnswerIfNeeded(self, suggestion, answer)

  markSuggestionAsUsedIfNeeded(self, answer)
  markSuggestionAsUsedIfNeeded(self, suggestion)
  table.insert(self.suggestionsByAnswer[answer], suggestion)
end

function SynonymsSuggestionRecord:markAnswerAsUsed(answer)
  answer = sanitizeAnswer(answer)
  markSuggestionAsUsedIfNeeded(self, answer)
end

function SynonymsSuggestionRecord:getSuggestion(answer)
  answer = sanitizeAnswer(answer)
  createSuggestionTableForAnswerIfNeeded(self, suggestion, answer)
  markSuggestionAsUsedIfNeeded(self, answer)

  if ((#self.suggestionsByAnswer[answer] < 1) and (#self.unusedSuggestions > 0)) then
    addNewSuggestion(self, self.suggestionsByAnswer[answer])
  end

  if (#self.suggestionsByAnswer[answer] > 0) then
    return self.suggestionsByAnswer[answer][1]
  else
    return nil
  end
end

function SynonymsSuggestionRecord:getMultipleSuggestions(answer, maxSuggestionCount)
  answer = sanitizeAnswer(answer)
  createSuggestionTableForAnswerIfNeeded(self, suggestion, answer)

  if (#self.suggestionsByAnswer[answer] < maxSuggestionCount) then
    while ((#self.suggestionsByAnswer[answer] < maxSuggestionCount) and (#self.unusedSuggestions > 0)) do
      addNewSuggestion(self, self.suggestionsByAnswer[answer])
    end
  end

  local suggestionCount = math.min(#self.suggestionsByAnswer[answer], maxSuggestionCount)
  return TableUtils.subarray(self.suggestionsByAnswer[answer], 1, suggestionCount)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SynonymsSuggestionRecord.new(prompt, possibleSuggestions)
  local self = {}
  Mixin.mix(self, SynonymsSuggestionRecord)

  self.prompt = prompt
  self.unusedSuggestions = possibleSuggestions

  self.suggestionsByAnswer = {}

  return self
end
