--- SynonymsTimeUpSequence
-- @author Created by karl on February 06, 2015.

class("SynonymsTimeUpSequence")

require("SynonymsCommon")
require("text.ComposedGlyphLabel")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PERFORMANCE_LABEL_OPACITY = 0.5
local Y_DISTANCE = Screen.getHeightPart(1.8)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsTimeUpSequence:start()
  local children = TableUtils.shallowCopy(self.label.children)
  table.insert(children, self.hand)
  local randomizedOrder = TableUtils.shuffleArray(children)

  local fallAction = AHStaggeredSpawn.new(0.05)
  local duration = 1
  local ease = AHEaseExponentialOut.new(4)
  local angle = 90

  for _, group in ipairs(randomizedOrder) do
    group:setRot(angle)
    fallAction:addAction(AHSpawn.new(
      AHLocationYTo.new(group, duration, ease, group.originalY),
      AHRotationTo.new(group, duration, ease, 0)
    ))
    angle = -angle
  end

  local fadeAction = AHOpacityTo.new(self.performanceLabel, duration / 2, nil, PERFORMANCE_LABEL_OPACITY)

  return AHStaggeredSpawn.new(duration, fallAction, fadeAction):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SynonymsTimeUpSequence.new()
  local self = FilledRectangle.new(Screen.width, Screen.height, SynonymsCommon.COLORS.YELLOW)
  Mixin.mix(self, SynonymsTimeUpSequence)

  local default = nil
  local buildGlyphs = true
  local label = Label.new(Localizer.get("TIME_ENDED"), Font.new(SynonymsCommon.FONTS.BOLD, 56), default, default, buildGlyphs)
  self.label = ComposedGlyphLabel.new(label)
  self.label:setColor(0, 0, 0, 0.7)

  if (self.label:getWidth() >= Screen.getWidthPart(0.8)) then
    self.label:setEffectiveDims(Screen.getWidthPart(0.75))
  end

  self.performanceLabel = Label.new(Localizer.get("HOW_YOU_DID"), Font.new(SynonymsCommon.FONTS.LIGHT, 27))
  self.performanceLabel:setColor(0, 0, 0, 0)

  self.hand = Sprite.newWithName("thats_it.png")

  local col = Column.new(-6, Alignment.CENTER)
  col:insert(self.hand)
  col:insertSpacer(20)
  col:insert(self.label)
  col:insert(self.performanceLabel)
  col:alignChildren()

  for _, character in ipairs(self.label.children) do
    character.originalY = character:getY()
    character:setY(character.originalY + Y_DISTANCE)
  end

  self.hand.originalY = self.hand:getY()
  self.hand:setY(self.hand:getY() + Y_DISTANCE)

  self:insertAtCenter(col)

  return self
end
