--- SynonymsTimer
-- @author Created by karl on February 05, 2015.

class("SynonymsTimer")

require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local STANDARD_WARNING_SOUND = "shared_countdown_warning"


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function onWarning(self)
  return AHSequence.new(
    AHOpacityTo.new(self.timeoutNotice, self.warningDuration / 2, AHEaseSineOut.new(), 1),
    function() Sound.playSoundEffect(STANDARD_WARNING_SOUND) end,
    AHOpacityTo.new(self.timeoutNotice, self.warningDuration / 2, AHEaseSineIn.new(), 0)
  ):start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function SynonymsTimer:pauseAndShowIcon()
  self.timer:pauseAndShowIcon()
end

function SynonymsTimer:pauseForSkip()
  self.timer:setKeyColor(SynonymsCommon.COLORS.WHITE)
  self.timer:setPausedOpacity(1)
  self.timer:getPausedLabel():setColor(unpack(SynonymsCommon.COLORS.WHITE))
  self.timer:getPausedLabel():setOpacity(0)
  self.timer:pauseAndShowIcon()
end

function SynonymsTimer:resumeFromSkip()
  self.timer:setKeyColor(self.timerColor)
  self.timer:setPausedOpacity(0.5)
  self.timer:resume()
  local colorWithoutOpacity = ColorUtils.colorWithOpacity(self.timerColor, nil)
  self.timer:getPausedLabel():setColor(unpack(colorWithoutOpacity))
  AHColorTo.new(self.timer, 0.167, nil, colorWithoutOpacity):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function SynonymsTimer.new(duration, warningDuration, color, onTimeout)
  local self = Group.newFullScreen()
  Mixin.mix(self, SynonymsTimer)

  self.timer = DigitTimer.new(duration, onTimeout, nil, nil, color)
  self.timer:setAnchorPointRight()
  local xOffset = 10
  self.timer:setLoc(UIOverlay.NORTHEAST.x + xOffset, UIOverlay.NORTHEAST.y)
  self:insert(self.timer)

  self.timeoutNotice = Label.new(Localizer.get("TIME_ENDING"), Font.new(FontFaces.DIN_OT_BOLD, 22.5))
  self.timeoutNotice:setColor(unpack(SynonymsCommon.COLORS.BLACK_60))
  self.timeoutNotice:setLoc(UIOverlay.NORTH.x, UIOverlay.NORTH.y)
  self.timeoutNotice:setOpacity(0)
  self:insert(self.timeoutNotice)

  FunctionUtils.forwardMethodCallsInClass(self, self.timer, Timer)

  self.warningDuration = warningDuration

  self.timer:setWarningInterval(1)
  self.timer:setWarningCount(5)
  self.timer:setWarningTimeOffset(warningDuration / 2)
  self.timer:addWarningHandler(function() onWarning(self) end)

  self.timer:setTimeLostOpacityOverride(1.0)

  self.timerColor = color

  return self
end
