--- Totem Background

class("Background")

require("widgets.Scroller")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local Z_INDEX = {
  SKY = 10,
  CLOUDS = 20
}

local SCROLL_SPEED = 10

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getMinimumScaling(self, sprite)
  local widthScale, heightScale = sprite:scaleForEffectiveDims(self:getDims())
  return math.max(widthScale, heightScale)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function Background:animate()
  self.clouds:setScrollSpeed(SCROLL_SPEED)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function makeCloudScroller(self, type, x, y)
  local cloudSprite = Sprite.newWithName(string.format("totem_cloud_%d.png", type))
  local location = {x=math.floor(x), y=math.floor(y)}
  local assets = {[cloudSprite] = location}

  local scroller = ItemScroller.new(assets, Screen.width*2, Screen.width*2, Screen.height, true)
  scroller:linkColorAttrToChildren()
  return scroller
end

local function insertClouds(self)
  local width = Screen.width * 4
  local height = Screen.height
  local scrollers = {
    makeCloudScroller(self, 1, width * 0.00, height * 0.75),
    makeCloudScroller(self, 2, width * 0.47, height * 0.50),
    makeCloudScroller(self, 3, width * 0.80, height * 0.37),
  }

  local scrollerProportions = {
    1.0,
    1.2,
    1.4,
  }

  self.clouds = ParallaxScroller.new(scrollers, scrollerProportions)
  self.clouds:linkColorAttrToChildren()
  self:insertAtCenter(self.clouds, Z_INDEX.CLOUDS)
end

-- Constructor ------------------------

function Background.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, Background)

  local skySprite = Sprite.newWithName("totem_sky.png")
  local scale = getMinimumScaling(self, skySprite)
  skySprite:setScl(scale)
  self:insertAtCenter(skySprite, Z_INDEX.SKY)

  insertClouds(self)

  return self
end
