--- MoneyFraction
-- @author Created by karl on May 28, 2014.

class("MoneyFraction")
require("TotemCommon")


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function MoneyFraction:setTextColor(color)
  for _, label in ipairs(self.labels) do
    label:setColor(unpack(color))
  end
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function MoneyFraction.new(selfW, selfH, numerator, denominator)
  local self = Group.new(selfW, selfH)
  Mixin.mix(self, MoneyFraction)

  local topFont = TotemCommon.FONT_TOTEM_FACE
  local bottomFont = TotemCommon.FONT_TOTEM_FACE_SUBDUED
  local topFontSize = math.floor(selfW * 0.35)
  local bottomFontSize = math.floor(topFontSize * 0.6)
  local topColor = TotemCommon.TOTEM_FACE_TEXT_COLOR
  local bottomColor = TotemCommon.TOTEM_FACE_SUBDUED_COLOR
  local currencyString = Localizer.get("CURRENCY")

  numerator = string.format("%s%.02f", currencyString, numerator)
  denominator = string.format("of %s%.02f", currencyString, denominator)

  local numeratorLabel = DeprecatedLabel.newSingleLine(tostring(numerator), topFont, topFontSize)
  numeratorLabel:setColor(unpack(topColor))
  local denominatorLabel = DeprecatedLabel.newSingleLine(tostring(denominator), bottomFont, bottomFontSize)
  denominatorLabel:setColor(unpack(bottomColor))

  local col = Column.new(-2)
  col:insert(numeratorLabel)
  col:insert(denominatorLabel)
  col:alignChildren()

  self:insertAtCenter(col)

  self.labels = {numeratorLabel, denominatorLabel}

  return self
end
