--- Totem3D
-- @author Created by a Adam Villalobos on Wednesday, August 21, 2013.

class("Totem3D")
require("utils.ShaderUtils")


function Totem3D.makeQuadMesh( xMin, yMin, xMax, yMax, color )

  local function writeTri ( vbo, p1, p2, p3, uv1, uv2, uv3 )
    vbo:writeFloat( p1.x, p1.y, p1.z )
    vbo:writeFloat( uv1.x, uv1.y )
    vbo:writeColor32(unpack(color))

    vbo:writeFloat( p2.x, p2.y, p2.z )
    vbo:writeFloat( uv2.x, uv2.y )
    vbo:writeColor32(unpack(color))

    vbo:writeFloat( p3.x, p3.y, p3.z )
    vbo:writeFloat( uv3.x, uv3.y )
    vbo:writeColor32(unpack(color))
  end

  local function writeFace ( vbo, p1, p2, p3, p4, uv1, uv2, uv3, uv4 )
    writeTri( vbo, p1, p2, p4, uv1, uv2, uv4 )
    writeTri( vbo, p2, p3, p4, uv2, uv3, uv4 )

  end

  local p = {}
  Vector.addPoint ( p, xMin, yMax, 0 ) -- p1
  Vector.addPoint ( p, xMin, yMin, 0 ) -- p2
  Vector.addPoint ( p, xMax, yMin, 0 ) -- p3
  Vector.addPoint ( p, xMax, yMax, 0 ) -- p4

  local uv = {}
  Vector.addPoint ( uv, 1, 1, 0 )
  Vector.addPoint ( uv, 1, 0, 0 )
  Vector.addPoint ( uv, 0, 0, 0 )
  Vector.addPoint ( uv, 0, 1, 0 )

  local vertexFormat = MOAIVertexFormat.new ()
  vertexFormat:declareCoord ( 1, MOAIVertexFormat.GL_FLOAT, 3 )
  vertexFormat:declareUV ( 2, MOAIVertexFormat.GL_FLOAT, 2 )
  vertexFormat:declareColor ( 3, MOAIVertexFormat.GL_UNSIGNED_BYTE )

  local vbo = MOAIVertexBuffer.new()
  vbo:setFormat( vertexFormat )
  vbo:reserveVerts( 6 )

  writeFace( vbo, p [ 1 ], p [ 2 ], p [ 3 ], p [ 4 ], uv [ 1 ], uv [ 2 ], uv [ 3 ], uv [ 4 ])

  vbo:bless()

  local mesh = MOAIMesh.new()
  mesh:setVertexBuffer( vbo )
  mesh:setPrimType( MOAIMesh.GL_TRIANGLES )

  return mesh
end

function Totem3D.makeBoxMesh( xMin, yMin, zMin, xMax, yMax, zMax, color )

  local function writeTri ( vbo, p1, p2, p3, uv1, uv2, uv3 )

    local normal = Vector.cross(Vector.add(p2,Vector.scale(-1,p1)),Vector.add(p3,Vector.scale(-1,p1)))
    normal = Vector.normalize(normal)

    vbo:writeFloat(p1.x, p1.y, p1.z)
    vbo:writeFloat(normal.x,normal.y,normal.z)
    vbo:writeFloat(uv1.x, uv1.y)
    vbo:writeColor32(unpack(color))

    vbo:writeFloat(p2.x, p2.y, p2.z)
    vbo:writeFloat(normal.x,normal.y,normal.z)
    vbo:writeFloat(uv2.x, uv2.y)
    vbo:writeColor32(unpack(color))

    vbo:writeFloat(p3.x, p3.y, p3.z)
    vbo:writeFloat(normal.x,normal.y,normal.z)
    vbo:writeFloat(uv3.x, uv3.y)
    vbo:writeColor32(unpack(color))
  end

  local function writeFace ( vbo, p1, p2, p3, p4, uv1, uv2, uv3, uv4 )
    writeTri( vbo, p1, p2, p4, uv1, uv2, uv4 )
    writeTri( vbo, p2, p3, p4, uv2, uv3, uv4 )
  end

  local p = {}

  --box points
  Vector.addPoint ( p, xMin, yMax, zMax ) -- p1
  Vector.addPoint ( p, xMin, yMin, zMax ) -- p2
  Vector.addPoint ( p, xMax, yMin, zMax ) -- p3
  Vector.addPoint ( p, xMax, yMax, zMax ) -- p4

  Vector.addPoint ( p, xMin, yMax, zMin ) -- p5
  Vector.addPoint ( p, xMin, yMin, zMin  ) -- p6
  Vector.addPoint ( p, xMax, yMin, zMin  ) -- p7
  Vector.addPoint ( p, xMax, yMax, zMin  ) -- p8

  local uv = {}
  Vector.addPoint ( uv, 0, 0, 0 )
  Vector.addPoint ( uv, 0, 1, 0 )
  Vector.addPoint ( uv, 1, 1, 0 )
  Vector.addPoint ( uv, 1, 0, 0 )

  local vertexFormat = MOAIVertexFormat.new ()
  vertexFormat:declareCoord ( 1, MOAIVertexFormat.GL_FLOAT, 3 )
  vertexFormat:declareCoord ( 2, MOAIVertexFormat.GL_FLOAT, 3 )
  vertexFormat:declareUV ( 3, MOAIVertexFormat.GL_FLOAT, 2 )
  vertexFormat:declareColor ( 4, MOAIVertexFormat.GL_UNSIGNED_BYTE )

  local vbo = MOAIVertexBuffer.new ()
  vbo:setFormat ( vertexFormat )
  vbo:reserveVerts ( 36 )

  writeFace ( vbo, p [ 1 ], p [ 2 ], p [ 3 ], p [ 4 ], uv [ 1 ], uv [ 2 ], uv [ 3 ], uv [ 4 ])
  writeFace ( vbo, p [ 4 ], p [ 3 ], p [ 7 ], p [ 8 ], uv [ 1 ], uv [ 2 ], uv [ 3 ], uv [ 4 ])
  writeFace ( vbo, p [ 8 ], p [ 7 ], p [ 6 ], p [ 5 ], uv [ 1 ], uv [ 2 ], uv [ 3 ], uv [ 4 ])
  writeFace ( vbo, p [ 5 ], p [ 6 ], p [ 2 ], p [ 1 ], uv [ 1 ], uv [ 2 ], uv [ 3 ], uv [ 4 ])
  writeFace ( vbo, p [ 5 ], p [ 1 ], p [ 4 ], p [ 8 ], uv [ 1 ], uv [ 2 ], uv [ 3 ], uv [ 4 ])
  writeFace ( vbo, p [ 2 ], p [ 6 ], p [ 7 ], p [ 3 ], uv [ 1 ], uv [ 2 ], uv [ 3 ], uv [ 4 ])

  vbo:bless ()

  local vsh = ShaderUtils.loadShader('shader.vsh')
  local fsh = ShaderUtils.loadShader('shader.fsh')

  local shader = MOAIShader.new ()

  shader:reserveUniforms ( 2 )
  shader:declareUniform ( 1, 'transform', MOAIShader.UNIFORM_WORLD_VIEW_PROJ )
  shader:declareUniform ( 2, 'normalTransform', MOAIShader.UNIFORM_NORMAL )

  shader:setVertexAttribute( 1, 'position' )
  shader:setVertexAttribute( 2, 'normal' )
  shader:setVertexAttribute( 3, 'uv' )
  shader:setVertexAttribute( 4, 'color' )
  shader:load ( vsh, fsh )

  local mesh = MOAIMesh.new ()
  mesh:setShader(shader)
  mesh:setVertexBuffer ( vbo )
  mesh:setPrimType ( MOAIMesh.GL_TRIANGLES )

  return mesh
end

function Totem3D.makeCylinderMesh(radius, height, verticeCount, color)
  local vertexFormat = MOAIVertexFormat.new ()
  vertexFormat:declareCoord ( 1, MOAIVertexFormat.GL_FLOAT, 3 )
  vertexFormat:declareUV ( 2, MOAIVertexFormat.GL_FLOAT, 2 )
  vertexFormat:declareColor ( 3, MOAIVertexFormat.GL_UNSIGNED_BYTE )

  local vbo = MOAIVertexBuffer.new()
  vbo:setFormat(vertexFormat)
  vbo:reserveVerts(verticeCount)

  local angleDelta = (math.pi*2)/verticeCount

  for i = verticeCount,0,-1 do
    local theta = i * angleDelta
    local locX = math.cos(theta) * radius
    local locY = height/2
    local locZ = math.sin(theta) * radius
    local texU = i / verticeCount
    local texV = 1

    if i % 2 == 1 then
      locY = -height/2
      texV = 0
    end

    vbo:writeFloat(locX, locY, locZ)
    vbo:writeFloat(texU, texV)
    vbo:writeColor32(unpack(color))
  end

  vbo:bless ()

  local indexCount = verticeCount + 2
  local ibo = MOAIIndexBuffer.new()
  ibo:reserve(indexCount)

  for i = 0,indexCount do
    ibo:setIndex(i, i % verticeCount)
  end

  local vsh = ShaderUtils.loadShader('shader.vsh')
  local fsh = ShaderUtils.loadShader('shader.fsh')

  local shader = MOAIShader.new ()

  shader:reserveUniforms ( 1 )
  shader:declareUniform ( 1, 'transform', MOAIShader.UNIFORM_WORLD_VIEW_PROJ )

  shader:setVertexAttribute( 1, 'position' )
  shader:setVertexAttribute( 2, 'uv' )
  shader:setVertexAttribute( 3, 'color' )
  shader:load ( vsh, fsh )

  local mesh = MOAIMesh.new ()
  mesh:setShader(shader)
  mesh:setVertexBuffer(vbo)
  mesh:setIndexBuffer(ibo)
  mesh:setPrimType(MOAIMesh.GL_TRIANGLE_STRIP)

  return mesh
end
