class("TotemConfiguration")


require("GameSession")
require("Localizer")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

TotemConfiguration.GAME_DURATION_IN_SECONDS = 105
TotemConfiguration.POINTS_PER_CORRECT_RESPONSE = 1000
TotemConfiguration.TOTEMS_TO_COMPLETE_COUNT = 2
TotemConfiguration.POINTS_PER_EXTRA_SECOND_REMAINING = 100

local EXTRA_TIME_NEEDED_RANK_TWO = 17
local EXTRA_TIME_NEEDED_RANK_THREE = 34

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TotemConfiguration.getAnswerCount()
  return TotemConfiguration.TOTEMS_TO_COMPLETE_COUNT * 3
end

function TotemConfiguration.getTotemContentTemplate()
  return {
    {
      type = "percent",
      help = Localizer.get("PERCENT"),
      fields = {"numerator", "denominator"}
    },
    {
      type = "fraction",
      help = Localizer.get("FRACTION"),
      fields = {"numerator", "denominator"}
    },
    {
      type = "ratio",
      help = Localizer.get("RATIO"),
      fields = {"numerator", "denominator"}
    },
    {
      type = "money",
      help = Localizer.get("MONEY"),
      fields = {"numerator", "denominator", "money_denominator"}
    }
  }
end

function TotemConfiguration.getPassScore()
  local totemsNeeded = TotemConfiguration.getAnswerCount()
  local passScore = totemsNeeded * TotemConfiguration.POINTS_PER_CORRECT_RESPONSE
  return passScore
end

function TotemConfiguration.getRankTwoScore()
  local passScore = TotemConfiguration.getPassScore()
  local extraSecondsNeeded = EXTRA_TIME_NEEDED_RANK_TWO
  local bonusPerSecond = TotemConfiguration.POINTS_PER_EXTRA_SECOND_REMAINING
  local rankTwoScore = passScore + extraSecondsNeeded * bonusPerSecond
  return rankTwoScore
end

function TotemConfiguration.getRankThreeScore()
  local passScore = TotemConfiguration.getPassScore()
  local extraSecondsNeeded = EXTRA_TIME_NEEDED_RANK_THREE
  local bonusPerSecond = TotemConfiguration.POINTS_PER_EXTRA_SECOND_REMAINING
  local rankThreeScore = passScore + extraSecondsNeeded * bonusPerSecond
  return rankThreeScore
end

function TotemConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 1.0}},

    {key = 0.0375, valueProbabilities = {Basic = 0.60,
                                        Easy = 0.40}},

    {key = 0.075, valueProbabilities = {Basic = 0.50,
                                        Easy = 0.50}},

    {key = 0.15, valueProbabilities = {Basic = 0.30,
                                       Easy = 0.70}},

    {key = 0.1875, valueProbabilities = {Basic = 0.10,
                                         Easy = 0.90}},

    {key = 0.225, valueProbabilities = {Easy = 0.65,
                                        Medium = 0.35}},

    {key = 0.3, valueProbabilities = {Easy = 0.35,
                                      Medium = 0.65}},

    {key = 0.375, valueProbabilities = {Easy = 0.20,
                                        Medium = 0.80}},

    {key = 0.45, valueProbabilities = {Medium = 0.60,
                                       Hard = 0.40}},

    {key = 0.525, valueProbabilities = {Medium = 0.35,
                                        Hard = 0.65}},

    {key = 0.6, valueProbabilities = {Medium = 0.15,
                                      Hard = 0.85}},

    {key = 0.675, valueProbabilities = {Medium = 0.05,
                                        Hard = 0.95}},

    {key = 0.75, valueProbabilities = {Hard = 1.00}},

    {key = 1.0, valueProbabilities = {Hard = 1.00}}
  }
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
