--- TotemFraction
-- @author Created by karl on May 27, 2014.

class("TotemFraction")
require("TotemCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local SPACING_ADJUST_NUMERATOR = -3
local SPACING_ADJUST_DENOMINATOR = 6


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TotemFraction:setTextColor(color)
  for _, label in ipairs(self.labels) do
    label:setColor(unpack(color))
  end
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TotemFraction.new(selfW, selfH, numerator, denominator)
  local self = Group.new(selfW, selfH)
  Mixin.mix(self, TotemFraction)

  local font = TotemCommon.FONT_TOTEM_FACE
  local fontSize = math.floor(selfW * 0.4)
  local color = TotemCommon.TOTEM_FACE_TEXT_COLOR

  local numeratorLabel = DeprecatedLabel.newSingleLine(tostring(numerator), font, fontSize)
  numeratorLabel:setColor(unpack(color))
  local denominatorLabel = DeprecatedLabel.newSingleLine(tostring(denominator), font, fontSize)
  denominatorLabel:setColor(unpack(color))

  local maxW = math.max(numeratorLabel:getWidth(), denominatorLabel:getWidth())
  local spacing = math.floor(fontSize * 0.05)

  local separator = FilledRectangle.new(math.floor(maxW * 1.2), spacing, color)

  local col = Column.new(0)
  col:insert(numeratorLabel)
  col:insert(separator)
  col:insert(denominatorLabel)
  col:alignChildren()

  local numX, numY = numeratorLabel:getLoc()
  numeratorLabel:setLoc(numX, numY + SPACING_ADJUST_NUMERATOR)
  local denX, denY = denominatorLabel:getLoc()
  denominatorLabel:setLoc(denX, denY + SPACING_ADJUST_DENOMINATOR)

  self:insertAtCenter(col)

  self.labels = {numeratorLabel, denominatorLabel, separator}

  return self
end
