--- TotemValidator
-- @author Created by Anton Morant on April 17, 2014.
-- ConceptValidator for Totem.

class("TotemValidator")

require("validators.ConceptValidator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  "num_1",
  "denom_1",
  "money_1",
  "num_2",
  "denom_2",
  "money_2",
  "num_3",
  "denom_3",
  "money_3",
  "num_4",
  "denom_4",
  "money_4",
  "difficulty"
}

local ITEMS_COUNT = 4
local ALLOWED_PERCENTAGES = TableUtils.createSetFromArray({
  10, 12, 13, 15, 17, 20, 23, 25, 30, 34, 38, 40, 45, 50, 56, 60, 63, 67, 70, 75, 78, 80, 84, 88, 89, 90
})
local MINIMUM_MONEY = 2
local MAXIMUM_MONEY = 100
local MAXIMUM_DENOMINATOR = 100

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validatePercentage(self, numerator, denominator)
  local percent = math.ceil((numerator / denominator * 100))
  self:expectLessOrEqual(denominator, MAXIMUM_DENOMINATOR, "The denominator is too big")
  self:expectTrue(ALLOWED_PERCENTAGES[percent], string.format("%d (from %d/%d) is not a valid percentage", percent, numerator, denominator))
end

local function validatePercentages(self, concept)
  for i = 1, ITEMS_COUNT do
    local numerator = concept["num_" .. i]
    local denominator = concept["denom_" .. i]
    validatePercentage(self, numerator, denominator)
  end
end

local function validateMoneys(self, concept)
  for i = 1, ITEMS_COUNT do
    local fieldName = "money_" .. i
    local money = concept[fieldName]
    local isNumber = self:expectType(money, "number", fieldName)

    if isNumber then
      local isInteger = MathUtils.isInteger(money)
      self:expectTrue(isInteger, string.format("Value for %s (%s) should be an integer value", fieldName, money))
      self:expectNumberInRange(money, MINIMUM_MONEY, MAXIMUM_MONEY, fieldName)
    end
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TotemValidator:validateConcept(concept)
  self:validateRequiredKeys(concept, REQUIRED_KEYS)

  validatePercentages(self, concept)
  validateMoneys(self, concept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TotemValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, TotemValidator)

  return self
end
