varying LOWP vec2 uvVarying;

uniform LOWP sampler2D sampler;
uniform LOWP float normalizationValueHorizontal;
uniform LOWP float normalizationValueVertical;

uniform LOWP float doColor;
uniform LOWP vec4 lowerLeftColor;
uniform LOWP vec4 lowerRightColor;
uniform LOWP vec4 upperRightColor;
uniform LOWP vec4 upperLeftColor;
uniform LOWP float doThreeOptionColor;
uniform LOWP vec4 threeOptionLowerLeftColor;
uniform LOWP vec4 threeOptionLowerRightColor;
uniform LOWP vec4 threeOptionUpperRightColor;
uniform LOWP vec4 threeOptionUpperLeftColor;

// Wanted to split the different blends into their own functions, but the shader
// failed on a device while working on desktop for some reason. Different OpenGL versions?
void main()
{
  // normalizationValues help gradients start/end at screen bounds (instead of larger texture bounds)
  LOWP float horizontalDistance = clamp(uvVarying.x / normalizationValueHorizontal, 0.0, 1.0);
  LOWP float verticalDistance = clamp((uvVarying.y - normalizationValueVertical) / (1.0 - normalizationValueVertical), 0.0, 1.0);

  LOWP vec4 backDropColor = texture2D(sampler, uvVarying);
  LOWP vec3 runningResultColor = vec3(backDropColor.rgb);

  if (doThreeOptionColor > 0.5)
  {
    // The following code implements color blending
    const LOWP vec3 lumVec = vec3(0.3, 0.59, 0.11);
    LOWP vec3 colorTintColor = mix(mix(threeOptionUpperLeftColor.rgb, threeOptionUpperRightColor.rgb, horizontalDistance), mix(threeOptionLowerLeftColor.rgb, threeOptionLowerRightColor.rgb, horizontalDistance), verticalDistance);

    LOWP vec3 a1 = colorTintColor + vec3(dot(runningResultColor, lumVec) - dot(colorTintColor, lumVec));

    LOWP float minComponent = min(a1.r, min(a1.g, a1.b));
    LOWP float maxComponent = max(a1.r, max(a1.g, a1.b));

    LOWP vec3 L = vec3(dot(a1, lumVec));
    LOWP vec3 A = (L + (((a1 - L) * (vec3(1.0) - L)) / (vec3(maxComponent) - L)));
    LOWP vec3 B = (L + (((a1 - L) * L) / (L - vec3(minComponent))));
    // Use alpha of one of the color quadrants to interpolate between the blend and the original color
    runningResultColor = mix(runningResultColor, (maxComponent > 1.0 ? A : (minComponent < 0.0 ? B : a1)), threeOptionUpperLeftColor.a);
  }

  if (doColor > 0.5)
  {
    // The following code implements color blending
    const LOWP vec3 lumVec = vec3(0.3, 0.59, 0.11);
    LOWP vec3 colorTintColor = mix(mix(upperLeftColor.rgb, upperRightColor.rgb, horizontalDistance), mix(lowerLeftColor.rgb, lowerRightColor.rgb, horizontalDistance), verticalDistance);

    LOWP vec3 a1 = colorTintColor + vec3(dot(runningResultColor, lumVec) - dot(colorTintColor, lumVec));

    LOWP float minComponent = min(a1.r, min(a1.g, a1.b));
    LOWP float maxComponent = max(a1.r, max(a1.g, a1.b));

    LOWP vec3 L = vec3(dot(a1, lumVec));
    LOWP vec3 A = (L + (((a1 - L) * (vec3(1.0) - L)) / (vec3(maxComponent) - L)));
    LOWP vec3 B = (L + (((a1 - L) * L) / (L - vec3(minComponent))));
    // Use alpha of one of the color quadrants to interpolate between the blend and the original color
    runningResultColor = mix(runningResultColor, (maxComponent > 1.0 ? A : (minComponent < 0.0 ? B : a1)), upperLeftColor.a);
  }

  LOWP vec4 finalOutputColor = vec4(runningResultColor, 1.0);
  gl_FragColor = finalOutputColor;
}
