--- TransitionsConfiguration
-- @author Created by joepoka on October 20, 2016.

class("TransitionsConfiguration")

require("difficulty.DifficultyUtils")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local POINTS_PER_CORRECT_ANSWER = 1000
local CORRECT_NEEDED_COUNT_RANK_TWO = 7
local CORRECT_NEEDED_COUNT_RANK_THREE = 9

-- Value used in CoreMS's HighDifficultyReachedHighlightProducer, update if modified
local HIGH_DIFFICULTY_THRESHOLD = 0.4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TransitionsConfiguration.getIsHighLevel()
  return (GameSession.getDifficulty() >= HIGH_DIFFICULTY_THRESHOLD)
end

function TransitionsConfiguration.shouldShowHighDifficultyInterstitial()
  return DifficultyUtils.shouldShowHighDifficultyInterstitial(HIGH_DIFFICULTY_THRESHOLD, GameSession.getDifficulty())
end

function TransitionsConfiguration.getRankTwoScore()
  return POINTS_PER_CORRECT_ANSWER * CORRECT_NEEDED_COUNT_RANK_TWO
end

function TransitionsConfiguration.getRankThreeScore()
  return POINTS_PER_CORRECT_ANSWER * CORRECT_NEEDED_COUNT_RANK_THREE
end

function TransitionsConfiguration.getGameDuration()
  return 75
end

function TransitionsConfiguration.getTimePenalty()
  return 6
end

function TransitionsConfiguration.getPointsPerAnswer()
  return POINTS_PER_CORRECT_ANSWER
end

function TransitionsConfiguration.getPointsPerFullSpeedBonus()
  return 100
end

function TransitionsConfiguration.getSpeedBonusWindow()
  return 5
end

function TransitionsConfiguration.getSpeedBonusDecayStart()
  return 0
end

function TransitionsConfiguration.getCorrectConceptsBeforeThreeOption()
  return 3
end

function TransitionsConfiguration.getDifficultyKeyFrames()
  return {
    {
      key = 0.0,
      valueProbabilities = {
        Basic = 0.5,
        Easy = 0.5
      }
    },
    {
      key = 0.075,
      valueProbabilities = {
        Basic = 0.4,
        Easy = 0.55,
        Medium = 0.05,
      }
    },
    {
      key = 0.15,
      valueProbabilities = {
        Basic = 0.3,
        Easy = 0.55,
        Medium = 0.15,
      }
    },
    {
      key = 0.225,
      valueProbabilities = {
        Basic = 0.1,
        Easy = 0.65,
        Medium = 0.25,
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.5,
        Medium = 0.5,
      }
    },
    {
      key = 0.375,
      valueProbabilities = {
        Easy = 0.45,
        Medium = 0.55,
      }
    },
    {
      key = 0.45,
      valueProbabilities = {
        Easy = 0.4,
        Medium = 0.45,
        Hard = 0.15
      }
    },
    {
      key = 0.525,
      valueProbabilities = {
        Easy = 0.25,
        Medium = 0.45,
        Hard = 0.3
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        Easy = 0.15,
        Medium = 0.4,
        Hard = 0.45
      }
    },
    {
      key = 0.675,
      valueProbabilities = {
        Easy = 0.05,
        Medium = 0.15,
        Hard = 0.8
      }
    },
    {
      key = 0.75,
      valueProbabilities = {
        Medium = 0.05,
        Hard = 0.95
      }
    },
    {
      key = 0.8,
      valueProbabilities = {
        Hard = 0.95,
        Advanced = 0.05
      }
    },
    {
      key = 0.9,
      valueProbabilities = {
        Hard = 0.8,
        Advanced = 0.2
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 0.7,
        Advanced = 0.3
      }
    }
  }
end
