--- TransitionsCross
-- @author Created by Steven Mah on May 19, 2017

class("TransitionsCross")

require("actions.advanced.AHSmoothShake")
require("graphics.ShapeUtils")

-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

local CHECKMARK_DEFAULT_SEGMENT_PROPORTION = 0.35
local CHECKMARK_DEFAULT_THICKNESS_PROPORTION = 0.3

local CROSS_DESCRIPTORS = {
  MAIN = {
    scale = 1.0,
    maxOpacity = 1.0,
    zIndex = 20
  },
  FIRST_GHOST = {
    scale = 1.0,
    maxOpacity = 0.5,
    zIndex = 10
  },
  SECOND_GHOST = {
    scale = 1.0,
    maxOpacity = 0.25,
    zIndex = 0
  }
}

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

local function getCrossPoints(width, segmentProportion, thicknessProportion)
  local segmentLength = segmentProportion * width
  local segmentWidth = thicknessProportion * width

  local points = {
    { x = segmentLength, y = (width + segmentWidth) / 2 },
    { x = segmentLength, y = width },
    { x = width - segmentLength, y = width },
    { x = width - segmentLength, y = (width + segmentWidth) / 2 },
    { x = width, y = (width + segmentWidth) / 2 },
    { x = width, y = (width - segmentWidth) / 2 },
    { x = width - segmentLength, y = (width - segmentWidth) / 2 },
    { x = width - segmentLength, y = 0},
    { x = segmentLength, y = 0 },
    { x = segmentLength, y = (width - segmentWidth) / 2 },
    { x = 0, y = (width - segmentWidth) / 2 },
    { x = 0, y = (width + segmentWidth) / 2 },
  }

  for _, point in ipairs(points) do
    point.x = point.x / width
    point.y = point.y / width
  end

  return points
end

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function TransitionsCross:getAppearAnimation(duration)
  local shakeAmplitude = self:getWidthPart(0.25)

  local fullAppearAction = AHStaggeredSpawn.new(
    duration * 0.1
  )

  for crossKey, cross in pairs(self.crosses) do
    local crossDesriptors = CROSS_DESCRIPTORS[crossKey]

    fullAppearAction:addAction(
      AHSpawn.new(
        AHOpacityTo.new(cross, duration / 4, nil, crossDesriptors.maxOpacity),
        AHSmoothShake.new(cross, shakeAmplitude, shakeAmplitude, 1, duration)
      )
    )
  end

  return AHSpawn.new(
    AHScaleTo.new(self, duration, nil, 1.0),
    fullAppearAction
  )
end

function TransitionsCross:getHideAnimation(duration)
  local fullHideAction = AHSpawn.new()

  for _, cross in pairs(self.crosses) do
    fullHideAction:addAction(
      AHOpacityTo.new(cross, duration/ 4, nil, 0.0)
    )
  end

  return fullHideAction
end

-- ---------------------------------------------------------------------------
-- Object Creation
-- ---------------------------------------------------------------------------

local function createLayout(self, width, segmentProportion, thicknessProportion, color)
  segmentProportion = segmentProportion or CHECKMARK_DEFAULT_SEGMENT_PROPORTION
  thicknessProportion = thicknessProportion or CHECKMARK_DEFAULT_THICKNESS_PROPORTION
  assert((segmentProportion * 2 + thicknessProportion == 1), "Segment proportions and thickness proportions must sum to 1.")
  local points = getCrossPoints(width, segmentProportion, thicknessProportion)

  local crosses = {}
  for crossKey, descriptor in pairs(CROSS_DESCRIPTORS) do
    local cross = FilledGraphicsPrimitive.new(width, width, color)
    cross._normalizedPointLists = {points}
    cross:updateGraphicsPrimitive()
    cross:setOpacity(0.0)
    self:insertAtCenter(cross, descriptor.zIndex)

    crosses[crossKey] = cross
  end

  self.crosses = crosses
end

-- Constructor ------------------------

function TransitionsCross.new(width, color, segmentProportion, thicknessProportion)
  local self = Group.new(width, width)
  Mixin.mix(self, TransitionsCross)

  createLayout(self, width, segmentProportion, thicknessProportion, color)

  return self
end
