--- TransitionsCube
-- @author Created by Steven Mah on March 2, 2017

class("TransitionsCube")

require("TransitionsMorphingRectangle")
require("TransitionsCubeVertices")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH, HEIGHT = TransitionsCommon.CUBE_WIDTH, TransitionsCommon.CUBE_HEIGHT
local VERTICES = TransitionsCubeVertices.getPrecalculatedFrameVertices()

local Z_INDICES = {
  LEFT = 0,
  BACK = 5,
  TOP = 10,
  BOTTOM = 15,
  RIGHT = 20,
  FRONT = 25,
  BANNER = 30
}

local DEFAULT_SPIN_DURATION = 1.0
local DEFAULT_ROTATIONS_PER_SECOND = 2
local FPS = 60

local TRANSPARENT_WHITE = {1.0, 1.0, 1.0, 0.1}
local TRANSPARENT_BLACK = {0.03, 0.07, 0.08, 0.6}
local MAX_OUTLINE_OPACITY = 0.25

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TransitionsCube:getMorphAction(duration, rotationsPerSecond, ease)
  duration = duration or DEFAULT_SPIN_DURATION
  rotationsPerSecond = rotationsPerSecond or DEFAULT_ROTATIONS_PER_SECOND
  local amount = duration * FPS

  local cubeSpinAction = AHSpawn.new(
    self.banner:createMorphToAction(duration, rotationsPerSecond, ease, amount)
  )
  for cubeFaceName, cubeFace in pairs(self.cubeFaces) do
    cubeSpinAction:addAction(
      cubeFace:createMorphToAction(duration, rotationsPerSecond, ease, amount)
    )
  end

  return cubeSpinAction
end

function TransitionsCube:getFadeCubeAction(duration, ease, opacity)
  local cubeOpacity = MathUtils.clamp(opacity, 0.0, 0.1)
  local outlineOpacity = MathUtils.clamp(opacity, 0.0, MAX_OUTLINE_OPACITY)
  local fadeAction = AHSpawn.new()
  for cubeFaceName, cubeFace in pairs(self.cubeFaces) do
    fadeAction:addAction(
      cubeFace:getFadeAction(duration, ease, cubeOpacity, outlineOpacity)
    )
  end

  return fadeAction
end

function TransitionsCube:getFadeBannerAction(duration, ease, opacity)
  local bannerOpacity = MathUtils.clamp(opacity, 0.0, 0.6)
  local outlineOpacity = MathUtils.clamp(opacity, 0.0, MAX_OUTLINE_OPACITY)

  return self.banner:getFadeAction(duration, ease, bannerOpacity, outlineOpacity)
end

function TransitionsCube:getFadeFrontFaceAction(duration, ease, opacity)
  local frontFace = self.cubeFaces.FRONT
  local cubeOpacity = MathUtils.clamp(opacity, 0.0, 0.1)
  local outlineOpacity = MathUtils.clamp(opacity, 0.0, MAX_OUTLINE_OPACITY)

  return frontFace:getFadeAction(duration, ease, cubeOpacity, outlineOpacity)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()
  self.cubeFaces = {}
  for cubeFaceName, vertexFrames in pairs(VERTICES) do
    local cubeFace = TransitionsMorphingRectangle.new(width, height, vertexFrames)
    cubeFace:setColor(unpack(TRANSPARENT_WHITE))
    cubeFace:setBlendMode(MOAIProp2D.BLEND_ADD)
    cubeFace:setOpacity(0.0)
    self:insertAtCenter(cubeFace, Z_INDICES[cubeFaceName])
    self.cubeFaces[cubeFaceName] = cubeFace
  end

  -- NOTE: Make one more face to use as the banner layer
  local banner = TransitionsMorphingRectangle.new(width, height, VERTICES.FRONT, nil, true)
  banner:setColor(unpack(TRANSPARENT_BLACK))
  banner:setOpacity(0.0)
  self:insertAtCenter(banner, Z_INDICES.BANNER)
  self.banner = banner
end

-- Constructor ------------------------

function TransitionsCube.new()
  local self = Group.new(WIDTH, HEIGHT)
  Mixin.mix(self, TransitionsCube)

  createLayout(self)

  self:resizeToFitChildren()

  return self
end
