--- TransitionsHighLevelScreen
-- @author Created by Steven Mah on May 12, 2016

class("TransitionsHighLevelScreen")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local HEADER_FONT = Font.new(FontFaces.DIN_OT_BOLD, 30)
local BODY_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 16)
local BUTTON_FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 15)

local HEADER_TEXT = "THIRD OPTION UNLOCKED"
local MAIN_TEXT = "You’re playing Transitions at a high difficulty level. You’ll now see sentences with up to 3 transition choices."

local WIDTH = Screen.getWidthPart(0.9)
local SPACING = Screen.getHeightPart(0.04)

local COLORS = {
  HEADER = {1.0, 0.98, 0.93},
  BODY = {1.0, 1.0, 1.0},
  BUTTON = {0.77, 0.91, 0.96, 0.0}
}

local BUTTON_VISIBLY_OPACITY = 0.4

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TransitionsHighLevelScreen:getRevealAction(duration, ease)
  return AHSpawn.new(
    AHOpacityTo.new(self, duration, ease, 1.0),
    AHOpacityTo.new(self.tapToContinueButton, duration, ease, BUTTON_VISIBLY_OPACITY)
  )
end

function TransitionsHighLevelScreen:getHideAndRemoveAction(duration, ease)
  local hideAction = AHSpawn.new(
    AHOpacityTo.new(self, duration, ease, 0.0),
    AHOpacityTo.new(self.tapToContinueButton, duration, ease, 0.0)
  )

  local removeAction = AHSpawn.new(
    AHRemove.new(self),
    AHRemove.new(self.contentColumn),
    AHRemove.new(self.tapToContinueButton)
  )

  return AHSequence.new(
    hideAction, removeAction
  )
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, tapToContinueCallback)
  local alignment = Alignment.CENTER

  local icon = Sprite.newWithName("three_icon.png")
  icon:setEffectiveDims(Screen.getWidthPart(0.3))
  local header = DynamicTextArea.newSimple(HEADER_TEXT, WIDTH, HEADER_FONT, alignment)
  header:setColor(unpack(COLORS.HEADER))
  local mainText = DynamicTextArea.newSimple(MAIN_TEXT, WIDTH * 0.9, BODY_FONT, alignment)
  mainText:setColor(unpack(COLORS.BODY))

  local contentColumn = Column.new()
  contentColumn:insert(icon)
  contentColumn:insertSpacer(SPACING)
  contentColumn:insert(header)
  contentColumn:insertSpacer(SPACING)
  contentColumn:insert(mainText)
  contentColumn:alignChildren()
  contentColumn:linkOpacityAttrToChildren(true)
  contentColumn:inheritOpacity(self)
  self.textLayer:insertAtLocationFractional(contentColumn, 0.5, 0.5)
  self.contentColumn = contentColumn

  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), BUTTON_FONT)
  tapToContinueLabel:setColor(unpack(COLORS.BUTTON))
  local width, height = tapToContinueLabel:getDims()
  Button.addVirtualButton(tapToContinueLabel, Screen.width, Screen.getHeightPart(0.2), tapToContinueCallback)
  self.textLayer:insertAtLocationFractional(tapToContinueLabel, 0.5, 0.1)
  self.tapToContinueButton = tapToContinueLabel
end

-- Constructor ------------------------

function TransitionsHighLevelScreen.new(tapToContinueCallback, textLayer)
  local self = Group.newFullScreen()
  Mixin.mix(self, TransitionsHighLevelScreen)

  self.actionInterrupt = ActionInterrupt.new()
  self.textLayer = textLayer

  createLayout(self, tapToContinueCallback)
  local isRecursive = true
  self:linkOpacityAttrToChildren(isRecursive)

  return self
end
