--- TransitionsLensFlare
-- @author Created by Steven Mah on May 11, 2017

class("TransitionsLensFlare")

require("graphics.CircleGradient")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LARGE_RADIUS = Screen.getHeightPart(0.08)
local MEDIUM_RADIUS = LARGE_RADIUS * 0.8
local SMALL_RADIUS = LARGE_RADIUS * 0.35
local X_OFFSET = Screen.getWidthPart(0.035)
local Y_OFFSET = Screen.getHeightPart(0.03)

-- local COLOR = {1.0, 0.85, 0.8, 0.0}
local COLOR = {1.0, 0.98, 0.76, 0.0}
local VISIBLE_OPACITY = 0.4

local FLARE_DESCRIPTORS = {
  {
    radius = LARGE_RADIUS,
    startPoint = { x = Screen.getWidthPart(0.3), y = Screen.getHeightPart(0.55) },
    targetOffset = { x = -X_OFFSET, y = Y_OFFSET }
  },
  {
    radius = SMALL_RADIUS,
    startPoint = { x = Screen.getWidthPart(0.35), y = Screen.getHeightPart(0.53) },
    targetOffset = { x = -0.9 * X_OFFSET, y = 0.9 * Y_OFFSET }
  },
  {
    radius = MEDIUM_RADIUS,
    startPoint = { x = Screen.getWidthPart(0.4), y = Screen.getHeightPart(0.5) },
    targetOffset = { x = -1.1 * X_OFFSET, y = 1.1 * Y_OFFSET }
  },
  {
    radius = MEDIUM_RADIUS,
    startPoint = { x = Screen.getWidthPart(0.6), y = Screen.getHeightPart(0.5) },
    targetOffset = { x = X_OFFSET, y = -Y_OFFSET }
  },
  {
    radius = LARGE_RADIUS,
    startPoint = { x = Screen.getWidthPart(0.75), y = Screen.getHeightPart(0.25) },
    targetOffset = { x = X_OFFSET, y = -Y_OFFSET }
  },
  {
    radius = SMALL_RADIUS,
    startPoint = { x = Screen.getWidthPart(0.5), y = Screen.getHeightPart(0.8) },
    targetOffset = { x = 1.2 * X_OFFSET, y = 0 }
  },
  {
    radius = MEDIUM_RADIUS,
    startPoint = { x = Screen.getWidthPart(0.6), y = Screen.getHeightPart(0.7) },
    targetOffset = { x = X_OFFSET, y = 0 }
  },
  {
    radius = SMALL_RADIUS,
    startPoint = { x = Screen.getWidthPart(0.65), y = Screen.getHeightPart(0.65) },
    targetOffset = { x = X_OFFSET, y = 0 }
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function resetFlareLocations(self)
  for flareIndex, flare in ipairs(self.flares) do
    flare:setLoc(flare.origin.x, flare.origin.y)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TransitionsLensFlare:getAnimation(fadeOutDuration)
  fadeOutDuration = fadeOutDuration or 1.5
  local fullAction = AHSpawn.new()
  for _, flare in ipairs(self.flares) do
    local targetOffset = flare.targetOffset
    fullAction:addAction(
      AHSpawn.new(
        AHLocationBy.new(flare, 10.0, AHEaseExponentialOut.new(), targetOffset.x, targetOffset.y),
        AHSequence.new(
          flare:newOpacityToAction(0.25, nil, VISIBLE_OPACITY, VISIBLE_OPACITY),
          AHDelay.new(0.5),
          flare:newOpacityToAction(fadeOutDuration, nil, 0.0, 0.0),
          function() resetFlareLocations(self) end
        )
      )
    )
  end

  return fullAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local flares = {}
  for _, descriptor in ipairs(FLARE_DESCRIPTORS) do
    local flare = CircleGradient.new(descriptor.radius, COLOR, COLOR)
    flare:setLoc(descriptor.startPoint.x, descriptor.startPoint.y)
    flare.origin = descriptor.startPoint
    flare.targetOffset = descriptor.targetOffset

    flare:setBlendMode(MOAIProp2D.BLEND_ADD)
    self:insert(flare)
    table.insert(flares, flare)
  end

  self.flares = flares
end

-- Constructor ------------------------

function TransitionsLensFlare.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, TransitionsLensFlare)

  createLayout(self)

  return self
end
