--- TransitionsMorphingRectangle
-- @author Created by Steven Mah on March 3, 2017

class("TransitionsMorphingRectangle")

-------------------------------------------------------------------------------
-- Configuration
-------------------------------------------------------------------------------

local WIDTH, HEIGHT = Screen.width, Screen.height
local DEFAULT_COLOR = {0.5, 0.5, 0.5}
local DEFAULT_OUTLINE_COLOR = {1.0, 0.85, 0.8}

-------------------------------------------------------------------------------
-- Utility
-------------------------------------------------------------------------------

-- NOTE: Same functinoality as ShapeUtils.getMaxEdges, but just with flattenedPoints points
local function getMaxEdges(flattenedPoints)
  local maxX, maxY = -math.huge, -math.huge
  for coordinateIndex = 1, #flattenedPoints - 1 do
    local x = flattenedPoints[coordinateIndex]
    local y = flattenedPoints[coordinateIndex]

    maxX = math.max(maxX, x)
    maxY = math.max(maxY, y)
  end
  return maxX, maxY
end

-------------------------------------------------------------------------------
-- API
-------------------------------------------------------------------------------

function TransitionsMorphingRectangle:getMorphAmount()
  return self.frame
end

function TransitionsMorphingRectangle:setMorphAmount(amount)
  local newFrame = (MathUtils.round(amount * self.speedFactor)) % 60 + 1
  --- TODO: Frame hangs at start of morph due to easing. This is a hacky fix to
  -- get around that
  if (amount <= 1.12) then
    newFrame = MathUtils.round(MathUtils.interpolateLinear(amount, 1, 1.12, 1, 5))
  end
  self.frame = newFrame
end

function TransitionsMorphingRectangle:createMorphToAction(duration, rotationsPerSecond, ease, amount)
  self.speedFactor = rotationsPerSecond
  return AHNonAttributeTo.new(self, duration, self.getMorphAmount, self.setMorphAmount, ease, amount)
end

function TransitionsMorphingRectangle:getFadeAction(duration, ease, faceOpacity, outlineOpacity)
  return AHSpawn.new(
    AHOpacityTo.new(self, duration, ease, faceOpacity),
    AHOpacityTo.new(self.outline, duration, ease, outlineOpacity)
  )
end

-------------------------------------------------------------------------------
-- Object Creation
-------------------------------------------------------------------------------

local function createLayout(self, isBanner)
  local width, height = self:getDims()
  local drawCallback = function()
    local frame = MathUtils.round(self.frame)
    local vertices = self.vertexFrames[frame]
    self.currentVertices = vertices
    MOAIDraw.fillFan(vertices)
  end
  local outlineDrawCallback = function()
    MOAIDraw.drawJoinedLineLoop(self.currentVertices, self.lineWidth)
  end

  local deck = MOAIScriptDeck.new()
  deck:setRect(0.0, 0.0, WIDTH, HEIGHT)
  deck:setDrawCallback(drawCallback)
  self:setDeck(deck)
  self.deck = deck

  if (isBanner) then
    local linePadding = 2 * self.lineWidth
    self.outline = HollowRectangle.new(width + linePadding, height + linePadding, self.lineWidth, self.lineWidth, DEFAULT_OUTLINE_COLOR)
    self.outline:setOpacity(0.0)
    self:insertAtCenter(self.outline)
  else
    self.outline = HollowGraphicsPrimitive.new(width, height, DEFAULT_OUTLINE_COLOR, self.lineWidth)
    self.outline:setOpacity(0.0)
    self.outline.deck:setDrawCallback(outlineDrawCallback)
    self:insertAtCenter(self.outline)
  end
end

-- Constructor ------------------------

function TransitionsMorphingRectangle.new(width, height, vertexFrames, color, isBanner)
  local self = Group.new(width, height)
  Mixin.mix(self, TransitionsMorphingRectangle)

  self.frame = 1
  self.lineWidth = 1.0 / Host.contentScale
  if (isBanner) then
    self.lineWidth = 1.0
  end
  self.vertexFrames = vertexFrames
  createLayout(self, isBanner)

  color = color or DEFAULT_COLOR
  self:setColor(unpack(color))
  self.outline:setBlendMode(MOAIProp2D.BLEND_ADD)

  return self
end
