--- TransitionsRoundGenerator
-- @author Created by Steven Mah on February 8, 2016

class("TransitionsRoundGenerator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local NUMBER_OF_TWO_OPTIONS = TransitionsConfiguration.getCorrectConceptsBeforeThreeOption()

local FILTER_KEYS = {
  DIFFICULTY = "difficulty",
  WORD_COUNT = "part"
}

local MAX_RETRY_ATTEMPTS = 3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function packageConcept(concept)
  local replacements = {}
  table.insert(replacements, concept.correct)
  table.insert(replacements, concept.incorrect1)

  if ((concept.incorrect2 ~= nil) and (concept.incorrect2 ~= "")) then
    table.insert(replacements, concept.incorrect2)
  end

  concept.replacements = replacements
end

local function hasSameWords(self, concept)
  for _, replacement in ipairs(concept.replacements) do
    if (not self.previousTransitions[replacement]) then
      return false
    end
  end
  return true
end

local function setPreviousTransitions(self, concept)
  self.previousTransitions = {}
  self.previousTransitions[concept.correct] = true
  self.previousTransitions[concept.incorrect1] = true

  if ((concept.incorrect2 ~= nil) and (concept.incorrect2 ~= "")) then
    self.previousTransitions[concept.incorrect2] = true
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TransitionsRoundGenerator:switchToThreeOptionsIfNeeded(correctResponses)
  local shouldChange = (correctResponses == NUMBER_OF_TWO_OPTIONS) and (self.numberOfOptions == 2) and self.isHighDifficulty
  if (shouldChange) then
    self.numberOfOptions = 3

    return true
  end
end

function TransitionsRoundGenerator:getNextRound()
  -- NOTE: There are no three option Basic or Easy concepts
  local nextDifficulty = self.difficultyNameChooser:nextValue()
  local chosenDifficultyTooLow = (nextDifficulty == "Basic") or (nextDifficulty == "Easy")
  if chosenDifficultyTooLow and (self.numberOfOptions == 3) then
    nextDifficulty = "Medium"
  end

  local filters = {
    [FILTER_KEYS.DIFFICULTY] = nextDifficulty,
    [FILTER_KEYS.WORD_COUNT] = tostring(self.numberOfOptions)
  }
  self.conceptGenerator:setFilters(filters)

  for i = 1, MAX_RETRY_ATTEMPTS do
    nextConcept = self.conceptGenerator:next()
    packageConcept(nextConcept)
    local sameAsPreviousTransitions = hasSameWords(self, nextConcept)
    if (not sameAsPreviousTransitions) then
      break
    end
  end

  self.conceptGenerator:markConceptIdentifierAsSeen(nextConcept.identifier)
  setPreviousTransitions(self, nextConcept)
  return nextConcept
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function TransitionsRoundGenerator.new(conceptGenerator, difficultyNameChooser)
  local self = {}
  Mixin.mix(self, TransitionsRoundGenerator)

  self.conceptGenerator = conceptGenerator
  self.difficultyNameChooser = difficultyNameChooser
  self.numberOfOptions = 2
  self.isHighDifficulty = TransitionsConfiguration.getIsHighLevel()
  self.previousTransitions = {}

  self.conceptGenerator:setAllowFewerResults(false)
  self.conceptGenerator:setMarkGeneratedConceptsAsSeen(false)

  return self
end
