--- TransitionsScene
-- @author Created by Steven Mah on October 20, 2016.

class("TransitionsScene")

require("TransitionsRationale")
require("TransitionsPromptTextView")
require("TransitionsMenu")
require("widgets.TapToContinue")
require("TransitionsBlendCanvas")
require("TransitionsCube")
require("TransitionsLensFlare")
require("TransitionsAboutScreen")
require("TransitionsHighLevelScreen")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GLOW_COLOR = {1.0, 0.98, 0.76, 0.0}

local PROMPT_Y_PROPORTION = 0.615

local Z_INDICES = {
  BACKGROUND = 0,
  BLUE_LAYER = 5,
  CUBE = 10,
  MENU = 15,
  LENS_FLARE = 20,
  TEXT = 25,
  GLOW = 30,
  OVERLAY = 35,
  INTERSTITIAL = 40
}

local PROMPT_BACKING_PADDING = Screen.getHeightPart(0.1)
local CUBE_X_SCALE = Screen.width * 1.1 / TransitionsCommon.CUBE_WIDTH
local BACKGROUND_OVERSCALE_FACTOR = 1.05

local CUBE_SPIN_DURATION = 1.0

local INCORRECT_BLEND_COLORS = {
  LOWER_LEFT_COLOR = {0.93, 0.28, 0.39, 0.7},
  LOWER_RIGHT_COLOR = {0.93, 0.28, 0.39, 0.7},
  UPPER_RIGHT_COLOR = {0.65, 0.24, 0.55, 0.7},
  UPPER_LEFT_COLOR = {0.65, 0.24, 0.55, 0.7}
}

local BLUE_BLEND_COLORS = {
  LOWER_LEFT_COLOR = {0.0, 0.5, 0.8, 0.2},
  LOWER_RIGHT_COLOR = {0.0, 0.5, 0.8, 0.2},
  UPPER_RIGHT_COLOR = {0.0, 0.5, 0.8, 0.2},
  UPPER_LEFT_COLOR = {0.0, 0.5, 0.8, 0.2}
}

local TAP_BLEND_COLORS = {
  LOWER_LEFT_COLOR = {0.54, 1.0, 0.0, 0.45},
  LOWER_RIGHT_COLOR = {1.0, 0.38, 0.0, 0.45},
  UPPER_RIGHT_COLOR = {0.54, 1.0, 0.0, 0.45},
  UPPER_LEFT_COLOR = {1.0, 0.38, 0.0, 0.45}
}

local BLUE_LAYER_MAX_OPACITY = 0.7

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getTransitionsFromCorrectResponseAction(self, cubeMorphDuration, cubeYScale, isTransitioningToOutro)
  local cube = self.cube
  local prompt = self.promptTextView

  local animationSegmentDuration = cubeMorphDuration / 2
  local rotationsPerSecond = 4.0
  local easeFactor = 4.0
  local morphEase = AHEaseSimpleInOut.new(easeFactor)
  local firstScaleEase = AHEaseExponentialIn.new()
  local secondScaleEase = AHEaseQuadraticOut.new()
  local _, currentYScale = self.cube:getScl()
  cubeYScale = cubeYScale or currentYScale

  local cubeScaleAction = AHSequence.new(
    AHScaleTo.new(cube, animationSegmentDuration, firstScaleEase, 1.0, cubeYScale),
    AHScaleTo.new(cube, animationSegmentDuration, secondScaleEase, CUBE_X_SCALE, cubeYScale)
  )

  local cubeMorph = cube:getMorphAction(cubeMorphDuration, rotationsPerSecond, morphEase)

  local cubeAction = AHSpawn.new(
    cubeScaleAction,
    cubeMorph,
    cube:getFadeCubeAction(animationSegmentDuration, nil, 0.5),
    AHDelayed.new(
      cubeMorphDuration * 0.25,
      function()
        Sound.playSoundEffect("return_to_game")
      end
    )
  )

  if (isTransitioningToOutro) then
    cubeAction:addAction(
      AHDelayed.new(
        cubeMorphDuration * 0.75,
        cube:getFadeBannerAction(animationSegmentDuration / 2, secondScaleEase, 0.0),
        cube:getFadeCubeAction(animationSegmentDuration / 2, secondScaleEase, 0.0)
      )
    )
  else
    cubeAction:addAction(
      AHDelayed.new(
        cubeMorphDuration * 0.75,
        cube:getFadeBannerAction(animationSegmentDuration, secondScaleEase, 1.0),
        cube:getFadeCubeAction(animationSegmentDuration, secondScaleEase, 0.0)
      )
    )
  end

  local transitionAction = AHSpawn.new(
    cubeAction,
    prompt:getHideAndRemoveAction(animationSegmentDuration / 2),
    AHScaleTo.new(prompt, animationSegmentDuration / 2, AHEaseSimpleIn.new(easeFactor), 0.75),
    AHLocationXTo.new(prompt, animationSegmentDuration / 2, AHEaseSimpleIn.new(easeFactor), Screen.getWidthPart(0.35))
  )

  return transitionAction
end

local function getRoundIntroAfterCorrect(self, nextPrompt, cubeYScale, switchedToThree)
  nextPrompt:setScl(0.9)
  self.textLayer:insertAtLocationFractional(nextPrompt, 0.525, PROMPT_Y_PROPORTION, Z_INDICES.PROMPT)

  local cubeMorphDuration = CUBE_SPIN_DURATION
  if (switchedToThree) then
    cubeMorphDuration = cubeMorphDuration * 2.0
  end

  local animationSegmentDuration = cubeMorphDuration / 2
  local isTransitioningToOutro = false
  local transitionAction = getTransitionsFromCorrectResponseAction(self, cubeMorphDuration, cubeYScale, isTransitioningToOutro)

  local revealThirdOption = AHDoNothing.new()

  if (switchedToThree) then
    local radius = self.menu:getHeight()
    local glow = CircleGradient.new(radius, GLOW_COLOR, GLOW_COLOR, radius)
    glow:setBlendMode(MOAIProp2D.BLEND_ADD)
    glow:setLoc(self.menu:getLoc())
    self.textLayer:insert(glow, Z_INDICES.GLOW)

    local glowAction = AHSequence.new(
      glow:newOpacityToAction(animationSegmentDuration, nil, 0.75, 0.5),
      glow:newOpacityToAction(animationSegmentDuration, nil, 0.0, 0.0),
      AHRemove.new(glow)
    )
    transitionAction:addAction(
      glowAction
    )
    local revealEase = AHEaseExponentialOut.new()
    revealThirdOption = self.menu:getRevealThirdButtonAction(animationSegmentDuration, revealEase)
  end

  local newPromptDelay = 0.1
  local newPromptAnimationDuration = animationSegmentDuration
  if (switchedToThree) then
    newPromptDelay = 0.3
    newPromptAnimationDuration = animationSegmentDuration / 2
  end

  local ease = nil
  local newPromptAction = AHDelayed.new(
    newPromptDelay,
    AHScaleTo.new(nextPrompt, newPromptAnimationDuration, ease, 1.0),
    AHLocationXTo.new(nextPrompt, newPromptAnimationDuration, ease, Screen.getWidthPart(0.5)),
    AHSequence.new(
      AHOpacityTo.new(nextPrompt, newPromptAnimationDuration, ease, 1.0),
      function()
        nextPrompt:clearOpacityAttrLinkToChildren(true)
        self.promptTextView = nextPrompt
      end
    )
  )

  local menuAction = AHSpawn.new(
    newPromptAction,
    AHSequence.new(
      function() self.menu:swapWords(self.round) end,
      AHSpawn.new(
        revealThirdOption,
        self.menu:getFadeButtonLabelsAction(1.0, animationSegmentDuration / 2, nil)
      )
    )
  )

  local fullAction = AHStaggeredSpawn.new(
    animationSegmentDuration,
    AHSequence.new(
      transitionAction,
      AHRemove.new(self.promptTextView)
    ),
    menuAction
  )

  return fullAction
end

local function getRoundIntroAfterIncorrect(self, nextPrompt, cubeYScale)
  local cube = self.cube
  local prompt = self.promptTextView
  local rationale = self.rationale
  local menu = self.menu

  local animationSegmentDuration = CUBE_SPIN_DURATION / 2
  nextPrompt:setLoc(Screen.getWidthPart(0.5), cube:getY())
  self.textLayer:insert(nextPrompt, Z_INDICES.PROMPT)

  local slideDuration = 0.75
  local slideAction = AHSpawn.new(
    AHLocationYTo.new(cube, slideDuration, nil, Screen.getHeightPart(PROMPT_Y_PROPORTION)),
    AHLocationYTo.new(prompt, slideDuration, nil, Screen.getHeightPart(PROMPT_Y_PROPORTION)),
    AHLocationYTo.new(nextPrompt, slideDuration, nil, Screen.getHeightPart(PROMPT_Y_PROPORTION)),
    AHScaleTo.new(cube, slideDuration, nil, CUBE_X_SCALE, cubeYScale),
    cube:getFadeBannerAction(slideDuration, nil, 1.0)
  )
  local hideElementsAction = AHSpawn.new(
    AHScaleTo.new(rationale, animationSegmentDuration, nil, 0.95),
    rationale:getFadeAction(0.0, animationSegmentDuration, nil),
    self:getResetShaderColorsAction(animationSegmentDuration, ease),
    prompt:getHideAndRemoveAction(animationSegmentDuration)
  )
  local showElementsAction = AHSpawn.new(
    menu:getResetAction(animationSegmentDuration),
    AHOpacityTo.new(nextPrompt, animationSegmentDuration, nil, 1.0),
    menu:getFadeButtonLabelsAction(1.0, animationSegmentDuration)
  )
  local hideRationale = AHSpawn.new(
    slideAction,
    AHSequence.new(
      hideElementsAction,
      function() menu:swapWords(self.round) end,
      showElementsAction
    )
  )

  local fullAction = AHSequence.new(
    hideRationale,
    function()
      nextPrompt:clearOpacityAttrLinkToChildren(true)
      self.promptTextView = nextPrompt
    end
  )

  return fullAction
end

local function createAndHideNewPrompt(self, newRound)
  local nextPrompt = TransitionsPromptTextView.new(newRound.prompt)
  local cubeYScale = (nextPrompt:getHeight() + PROMPT_BACKING_PADDING) / self.cube:getHeight()
  nextPrompt:setOpacity(0.0)
  nextPrompt:linkOpacityAttrToChildren(true)

  self.round = newRound
  self.rationale:setRoundData(newRound)

  return nextPrompt, cubeYScale
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TransitionsScene:setTouchEnabled(isEnabled)
  self.menu:setTouchEnabled(isEnabled)
end

function TransitionsScene:getIntroAction(newRound)
  local nextPrompt, cubeYScale = createAndHideNewPrompt(self, newRound)
  self.promptTextView = nextPrompt
  nextPrompt:linkOpacityAttrToChildren(true)
  nextPrompt:setOpacity(1.0)
  nextPrompt:clearOpacityAttrLinkToChildren(true)
  local cube = self.cube
  local menu = self.menu
  menu:swapWords(newRound)
  self.rationale:setRoundData(newRound)
  cube:setScl(CUBE_X_SCALE, cubeYScale)

  wait(menu:getFullFadeAction(1.0, 0.0):start())
  wait(cube:getFadeBannerAction(0.0, nil, 1.0):start())


  local promptX, promptY = cube:getLoc()
  self.textLayer:insertAtLocationFractional(nextPrompt, 0.5, 1.5, Z_INDICES.PROMPT)

  local slideDuration = 1.5
  local slideEase = AHEaseExponentialOut.new()
  local restingY = Screen.getHeightPart(PROMPT_Y_PROPORTION)
  local slidePromptDownAction = AHSpawn.new(
    AHLocationYTo.new(nextPrompt, slideDuration, slideEase, restingY),
    AHLocationYTo.new(cube, slideDuration, slideEase, restingY)
  )

  local menuRestingY = Screen.getHeightPart(0.29)
  local minimumY = Screen.safeBottom + menu:getHeightPart(0.5)

  local slideMenuAction = AHLocationYTo.new(self.menu, slideDuration, slideEase, math.max(menuRestingY, minimumY))

  local staggerDuration = 0.1
  local introAction = AHStaggeredSpawn.new(
    staggerDuration,
    slidePromptDownAction,
    slideMenuAction
  )

  local backgroundScaleDuration = slideDuration + staggerDuration
  local scaleX, scaleY = self.background:scaleForEffectiveDims(Screen.width * BACKGROUND_OVERSCALE_FACTOR, Screen.height * BACKGROUND_OVERSCALE_FACTOR)
  local backgroundScaleAction = AHScaleTo.new(self.background, backgroundScaleDuration, AHEaseExponentialOut.new(), scaleX, scaleY)
  return AHSpawn.new(
    introAction,
    backgroundScaleAction
  )
end

function TransitionsScene:getRevealRoundAfterCorrectResponse(newRound, switchedToThree)
  local nextPrompt, cubeYScale = createAndHideNewPrompt(self, newRound)

  return getRoundIntroAfterCorrect(self, nextPrompt, cubeYScale, switchedToThree)
end

function TransitionsScene:getRevealRoundAfterIncorrectResponse(newRound)
  local nextPrompt, cubeYScale = createAndHideNewPrompt(self, newRound)

  return getRoundIntroAfterIncorrect(self, nextPrompt, cubeYScale)
end

function TransitionsScene:getCorrectResponseAction(submittedAnswer, willSwitchToThree, isOutOfTime)
  local prompt = self.promptTextView
  local cube = self.cube
  local triggerDuration = 1.2
  local revertDuration = 1.5
  local flareFadeOutDuration = 1.5
  if (willSwitchToThree and not isOutOfTime) then
    triggerDuration = triggerDuration + 0.5
    flareFadeOutDuration = flareFadeOutDuration * 2.0
    self.menu:switchToThreeOptionLabelColor()
  end
  local buttonAnimation = AHStaggeredSpawn.new(
    triggerDuration,
    self.menu:getCorrectAction(triggerDuration),
    self.menu:getFadeButtonLabelsAction(0.0, 0.5)
  )
  -- NOTE: Interfering actions cause outro transition to be weird; hacky fix so we can
  -- stop said actions for a cleaner outro; find a less hacky fix
  self.menu.correctResponseAction = buttonAnimation

  local fadeBannerDuration = 1.5
  local fadeOutBannerAction = AHSpawn.new(
    cube:getFadeBannerAction(fadeBannerDuration, ease, 0.0),
    cube:getFadeFrontFaceAction(fadeBannerDuration, ease, 1.0)
  )

  local screenWidth, screenHeight = Screen.width, Screen.height
  local backgroundWidth, backgroundHeight = self.background:getDims()
  local scaleX, scaleY = self.background:scaleForEffectiveDims(screenWidth * BACKGROUND_OVERSCALE_FACTOR, screenHeight * BACKGROUND_OVERSCALE_FACTOR)
  local screenBounceAction = AHSequence.new(
    AHScaleTo.new(self.background, triggerDuration * 0.75, AHEaseExponentialOut.new(), screenWidth / backgroundWidth, screenHeight / backgroundHeight),
    AHScaleTo.new(self.background, fadeBannerDuration, nil, scaleX, scaleY)
  )

  local glowsAction = AHSequence.new(
    AHDelay.new(0.25),
    self.fullScreenGlow:newOpacityToAction(triggerDuration + 0.5, nil, 0.75, 0.5),
    self.fullScreenGlow:newOpacityToAction(revertDuration, nil, 0.0, 0.0)
  )

  local isCorrect = true
  local holdingAction = AHSequence.new(
    prompt:getReplaceBlankAction(submittedAnswer, isCorrect),
    AHDelay.new(0.5)
  )

  local blendAction
  if (willSwitchToThree and not isOutOfTime) then
    blendAction = AHSequence.new(
      function()
        self.blendedGroup:getShaderColorsToAction(TAP_BLEND_COLORS, 0.1):start()
      end,
      AHDelay.new(2.5),
      self.blendedGroup:getThreeOptionShaderAction(0.1),
      AHDelay.new(0.5),
      self.blendedGroup:getResetShaderColorsAction(2.4)
    )
  else
    blendAction = AHSequence.new(
      self.blendedGroup:getShaderColorsToAction(TAP_BLEND_COLORS, 0.1),
      AHDelay.new(0.5),
      self.blendedGroup:getResetShaderColorsAction(2.4)
    )
  end

  local freeActions = AHSpawn.new(
    screenBounceAction,
    buttonAnimation,
    fadeOutBannerAction,
    glowsAction,
    blendAction,
    self.flareLayer:getAnimation(flareFadeOutDuration)
  )
  local correctAction = AHSpawn.new(
    holdingAction,
    function()
      freeActions:start()
    end
  )

  return correctAction
end

function TransitionsScene:getIncorrectReponseAction(correctAnswer, submittedAnswer)
  local prompt = self.promptTextView
  local cube = self.cube
  local rationale = self.rationale

  local cubeBottomY = cube:getYBottom()
  local promptShift = cube:getHeightPart(0.05)
  local newCubeBottomY = cubeBottomY + promptShift

  local duration = 0.6
  rationale:setHeight(newCubeBottomY)
  rationale:setY(newCubeBottomY)
  rationale:setText(submittedAnswer)
  rationale:setScl(1.1)

  local shakeAmplitude = MathUtils.randomSign(Screen.getWidthPart((1 - BACKGROUND_OVERSCALE_FACTOR) / 2))
  local centerX = Screen.getWidthPart(0.5)
  local shakeEase = AHEaseSineInOut.new()
  local shakeSegmentDuration = duration / 4
  local screenShake = AHSequence.new(
    AHLocationXTo.new(self.background, shakeSegmentDuration, shakeEase, centerX + shakeAmplitude),
    AHLocationXTo.new(self.background, shakeSegmentDuration, shakeEase, centerX - shakeAmplitude),
    AHLocationXTo.new(self.background, shakeSegmentDuration, shakeEase, centerX + shakeAmplitude),
    AHLocationXTo.new(self.background, shakeSegmentDuration, shakeEase, centerX - shakeAmplitude / 2),
    AHLocationXTo.new(self.background, shakeSegmentDuration, shakeEase, centerX + shakeAmplitude / 4),
    AHLocationXTo.new(self.background, shakeSegmentDuration, shakeEase, centerX)
  )

  local buttonAnimation = AHSpawn.new(
    function()
      screenShake:start()
    end,
    self.menu:getIncorrectAction(duration),
    self.blendedGroup:getShaderColorsToAction(INCORRECT_BLEND_COLORS, duration / 2)
  )
  local isCorrect = false
  local replaceWordAction = prompt:getReplaceBlankAction(correctAnswer, isCorrect)

  local translateDuration = 1.5
  local shiftEase = AHEaseExponentialInOut.new()
  local cubeRationaleY = cube:getY() + promptShift
  local shiftAction = AHSpawn.new(
    AHLocationYTo.new(cube, translateDuration, shiftEase, cubeRationaleY),
    AHLocationYTo.new(prompt, translateDuration, shiftEase, cubeRationaleY)
  )

  local ease = AHEaseExponentialOut.new()
  local textAnimationDelay = 0.65
  local textAnimationDuration = translateDuration - textAnimationDelay
  local revealAction = AHSpawn.new(
    rationale:getFadeAction(1.0, textAnimationDuration, ease),
    AHScaleTo.new(rationale, textAnimationDuration, ease, 1.0),
    replaceWordAction
  )

  local blendDuration = 0.75
  local showRationaleAction = AHSpawn.new(
    shiftAction,
    AHDelayed.new(
      textAnimationDelay,
      revealAction
    ),
    self.menu:getFullFadeAction(0.0, textAnimationDelay),
    self.menu:getFadeOutAndResetAction(textAnimationDelay),
    self:getShowBlueTintAction(blendDuration)
  )

  local fullResponseAction = AHSequence.new(
    buttonAnimation,
    showRationaleAction,
    function()
      rationale:waitForDismissal()
    end
  )

  return fullResponseAction
end

function TransitionsScene:getOutroAnimation(previousRoundCorrect, submittedAnswer)
  local duration = 0.6
  local opacity = 0.0
  local prompt = self.promptTextView
  local cube = self.cube
  local fadeOutDuration = 0.25
  local textAction = AHSpawn.new(
    AHSequence.new(
      function()
        if (self.menu.correctResponseAction) then
          self.menu.correctResponseAction:stop()
        end
      end,
      self.menu:getFullFadeAction(opacity, fadeOutDuration)
    ),
    AHOpacityTo.new(prompt, fadeOutDuration, ease, 0.0),
    cube:getFadeCubeAction(fadeOutDuration, ease, opacity),
    cube:getFadeBannerAction(fadeOutDuration, ease, opacity),
    AHScaleTo.new(self.rationale, duration, nil, 0.95),
    self.rationale:getFadeAction(0.0, fadeOutDuration, nil)
  )

  local checkmarkDelay = 0.1
  local cleanupResponse = AHDoNothing.new()
  if (previousRoundCorrect and submittedAnswer ~= nil) then
    local cubeMorphDuration = CUBE_SPIN_DURATION
    local isTransitioningToOutro = true
    local animationSegmentDuration = cubeMorphDuration / 2
    local transitionAction = getTransitionsFromCorrectResponseAction(self, cubeMorphDuration, nil, isTransitioningToOutro)

    cleanupResponse = AHStaggeredSpawn.new(
      animationSegmentDuration * 1.1,
      transitionAction,
      AHSpawn.new(
        self.menu:getFullFadeAction(0.0, animationSegmentDuration),
        self.fullScreenGlow:newOpacityToAction(cubeMorphDuration, nil, 0.0, 0.0)
      ),
      AHRemove.new(self.menu)
    )
    checkmarkDelay = 1.0
  end

  local shakeAmplitude = MathUtils.randomSign(Screen.getWidthPart((1 - BACKGROUND_OVERSCALE_FACTOR) / 2))
  local centerX = Screen.getWidthPart(0.5)
  local shakeEase = AHEaseSineInOut.new()
  local shakeDuration = 0.4
  local screenShake = AHSequence.new(
    AHLocationXTo.new(self.background, shakeDuration, shakeEase, centerX + shakeAmplitude),
    AHLocationXTo.new(self.background, shakeDuration, shakeEase, centerX - shakeAmplitude),
    AHLocationXTo.new(self.background, shakeDuration / 2, shakeEase, centerX + shakeAmplitude),
    AHLocationXTo.new(self.background, shakeDuration / 2, shakeEase, centerX - shakeAmplitude),
    AHLocationXTo.new(self.background, shakeDuration / 4, shakeEase, centerX + shakeAmplitude),
    AHLocationXTo.new(self.background, shakeDuration / 4, shakeEase, centerX - shakeAmplitude),
    AHLoop.new(
      AHSequence.new(
        AHLocationXTo.new(self.background, shakeDuration / 8, shakeEase, centerX + shakeAmplitude),
        AHLocationXTo.new(self.background, shakeDuration / 8, shakeEase, centerX - shakeAmplitude)
      )
    )
  )

  local glowDelay = 1.2 + checkmarkDelay
  local glowAction = AHSequence.new(
    AHSpawn.new(
      AHOpacityTo.new(self.darkLayer, glowDelay, nil, 0.35),
      self:getShowBlueTintAction(glowDelay),
      function()
        screenShake:start()
      end
    ),
    AHSpawn.new(
      AHSequence.new(
        self.fullScreenGlow:newOpacityToAction(0.1, nil, 0.75, 0.5),
        self.fullScreenGlow:newOpacityToAction(3.0, nil, 0.0, 0.0)
      ),
      function()
        self.darkLayer:setOpacity(0.25)
        screenShake:stop()
      end,
      self.flareLayer:getAnimation(3.0)
    )
  )

  local outroAnimation = AHSequence.new(
    function()
      prompt:linkOpacityAttrToChildren(true)
    end,
    AHSpawn.new(
      textAction,
      cleanupResponse,
      glowAction,
      AHSequence.new(
        AHDelay.new(checkmarkDelay),
        function()
          Sound.playSoundEffect("outro")
        end,
        self.outroGroup:getAnimation()
      )
    )
  )

  return outroAnimation
end

function TransitionsScene:getResetShaderColorsAction(duration, ease)
  return AHSpawn.new(
    self.blueLayer:newOpacityToAction(duration, ease, 0.0, 0.0),
    AHOpacityTo.new(self.darkLayer, duration, ease, 0.0),
    self.blendedGroup:getResetShaderColorsAction(duration, ease)
  )
end

function TransitionsScene:getShowBlueTintAction(duration, ease)
  return AHSpawn.new(
    self.blueLayer:newOpacityToAction(duration, ease, BLUE_LAYER_MAX_OPACITY, BLUE_LAYER_MAX_OPACITY),
    AHOpacityTo.new(self.darkLayer, duration, ease, 0.25),
    self.blendedGroup:getShaderColorsToAction(BLUE_BLEND_COLORS, duration)
  )
end

function TransitionsScene:showAndWaitForInterstitial(class, waitCallback, tapToContinueCallback)
  local interstitial = class.new(tapToContinueCallback, self.textLayer)
  interstitial:setOpacity(0.0)
  self:insertAtCenter(interstitial, Z_INDICES.INTERSTITIAL)

  local duration = 0.5
  local ease = nil
  return AHSequence.new(
    interstitial:getRevealAction(duration, ease),
    function()
      waitUntil(waitCallback)
    end,
    interstitial:getHideAndRemoveAction(duration, ease)
  )
end

function TransitionsScene:getFadeOutOverlayAction(duration, ease)
  return AHOpacityTo.new(self.fadeFromBlackOverylay, duration, ease, 0.0)
end

function TransitionsScene:getClearSceneAnimation(duration)
  local opacity = 0.0
  local hideAllElements = AHSpawn.new(
    self.menu:getFullFadeAction(opacity, duration),
    self.cube:getFadeCubeAction(duration, nil, opacity),
    self.cube:getFadeBannerAction(duration, nil, opacity),
    self.rationale:getFadeAction(opacity, duration, nil),
    self.promptTextView:getHideAndRemoveAction(duration, nil)
  )

  return hideAllElements
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, onResponseCallback)
  self.blendCanvas = Group.newFullScreen()
  self.textLayer = Group.newFullScreen()
  self.fadeFromBlackOverylay = FilledRectangle.newFullScreen({0.0, 0.0, 0.0})

  self:insertAtCenter(self.fadeFromBlackOverylay, Z_INDICES.OVERLAY)
  self:insertAtCenter(self.textLayer, Z_INDICES.TEXT)

  local startingExtraOverscale = 0.025 + BACKGROUND_OVERSCALE_FACTOR
  self.background = Sprite.newWithName("background.png")
  self.background:setEffectiveDims(Screen.width * startingExtraOverscale, Screen.height * startingExtraOverscale)
  self.blendCanvas:insertAtCenter(self.background, Z_INDICES.BACKGROUND)

  self.cube = TransitionsCube.new()
  self.cube:setEffectiveWidth(Screen.width)
  self.blendCanvas:insertAtLocationFractional(self.cube, 0.5, 1.5, Z_INDICES.CUBE)

  local onResponseWrapper = function(word)
    onResponseCallback(word)
  end

  self.menu = TransitionsMenu.new(onResponseWrapper, self.textLayer)
  self.blendCanvas:insertAtLocationFractional(self.menu, 0.5, -0.5, Z_INDICES.MENU)

  local rationaleHeight = Screen.getHeightPart(PROMPT_Y_PROPORTION) - self.cube:getHeightPart(0.5)
  self.rationale = TransitionsRationale.new(rationaleHeight)
  self.rationale:setAnchorPointTop()
  self.rationale:setOpacity(0.0)
  self.textLayer:insertAtCenter(self.rationale, Z_INDICES.RATIONALE)

  self.flareLayer = TransitionsLensFlare.new()
  self.blendCanvas:insertAtCenter(self.flareLayer, Z_INDICES.LENS_FLARE)

  local topColor = {0.0, 0.29, 0.32, 0.0}
  local bottomColor = {0.0, 0.03, 0.12, 0.0}
  self.blueLayer = RectangleGradient.new(Screen.width, Screen.height, topColor, bottomColor)
  self.darkLayer = FilledRectangle.newFullScreen({0.0, 0.0, 0.0, 0.0})
  self.blendCanvas:insertAtCenter(self.blueLayer, Z_INDICES.BLUE_LAYER)
  self.blendCanvas:insertAtCenter(self.darkLayer, Z_INDICES.BLUE_LAYER)

  self.blendedGroup = TransitionsBlendCanvas.new(self.blendCanvas)
  self.blendedGroup:setLoc(self.blendedGroup:getWidthPart(0.5), self.blendedGroup:getHeightPart(0.5))
  self:insert(self.blendedGroup)

  self.promptTextView = TransitionsPromptTextView.new("--unset--")
  self.promptTextView:setOpacity(0.0)
  self.promptTextView:setScl(0.0)
  self.textLayer:insertAtLocationFractional(self.promptTextView, 0.5, PROMPT_Y_PROPORTION)

  local glowRadius = Screen.height
  self.fullScreenGlow = CircleGradient.new(glowRadius, GLOW_COLOR, GLOW_COLOR, glowRadius)
  self.fullScreenGlow:setBlendMode(MOAIProp2D.BLEND_ADD)
  self:insertAtLocationFractional(self.fullScreenGlow, 0.5, 0.55, Z_INDICES.GLOW)

  self.outroGroup = TransitionsPreloadedOutro.preloadedOutro
  self:insertAtCenter(self.outroGroup)
end

-- Constructor ------------------------

function TransitionsScene.new(onResponseCallback)
  local self = Group.newFullScreen()
  Mixin.mix(self, TransitionsScene)

  createLayout(self, onResponseCallback)

  self.activeTopBlend = nil

  return self
end
