--- TransitionsTextAnimationBuilder
-- @author Created by Steven Mah on May 15, 2017

class("TransitionsTextAnimationBuilder")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TransitionsTextAnimationBuilder:_createShowItemAnimation(itemView, itemLayout, textView)
  local originalOpacity = itemView:getOpacity()
  return AHSequence.new(
    function()
      textView:insert(itemView)
      itemView:setOpacity(0)
      itemView:setScl(1.2)
      itemView:setLoc(itemLayout:getX(), itemLayout:getY())
      itemView:setAnchorPoint(itemLayout:getAnchorX(), itemLayout:getAnchorY())
    end,
    AHSpawn.new(
      AHOpacityTo.new(itemView, self.showDuration, nil, originalOpacity),
      AHScaleTo.new(itemView, self.showDuration, AHEaseExponentialOut.new(), 1.0)
    )
  )
end

function TransitionsTextAnimationBuilder:createItemsReplacedAnimation(textView, oldItemLayouts, newItemLayouts, newHeight)
  local hideAnimations = self:_createAllHideItemAnimations(oldItemLayouts, newItemLayouts)
  local moveAnimations = self:_createAllMoveItemAnimations(oldItemLayouts, newItemLayouts)
  local showAnimations = self:_createAllShowItemAnimations(textView, oldItemLayouts, newItemLayouts)
  local adjustHeightAnimation = self:_createAdjustHeightAnimationIfNeeded(textView, newHeight)
  local animation = AHSpawn.new(hideAnimations, moveAnimations, adjustHeightAnimation, showAnimations)
  return animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TransitionsTextAnimationBuilder.new()
  local self = TextAnimationBuilder.new()
  Mixin.mix(self, TransitionsTextAnimationBuilder)

  self:setMoveDuration(0.35)
  self:setShowDuration(0.7)
  self:setHideDuration(0.35)
  self:setAdjustHeightDuration(0.35)

  return self
end
