--- TypoBonusLabel
-- @author Created by anton on August 20, 2015.

class("TypoBonusLabel")

require("TypoCommon")
require("text.ComposedGlyphLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DISTANCE_TO_TRAVEL = Screen.getHeightPart(0.7)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getBonusColors()
  return ColorUtils.makeGradient(TypoCommon.COLOR_ORANGE, TypoCommon.COLOR_PURPLE, 5)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoBonusLabel:prepareToShow()
  local minRotation = -90
  local maxRotation = 90

  for _, letter in ipairs(self.children) do
    local currentY = letter:getY()
    local offscreenY = currentY - DISTANCE_TO_TRAVEL
    local rotation = MathUtils.getRandomFloat(minRotation, maxRotation)

    letter:setY(offscreenY)
    letter:setRot(rotation)
  end
end

function TypoBonusLabel:getShowAnimation(maxDuration)
  local maxDelay = 0.5
  local travelDuration = maxDuration - maxDelay

  local animation = AHSpawn.new()

  for _, letter in ipairs(self.children) do
    local delay = math.random() * maxDelay
    local distanceToTravel = DISTANCE_TO_TRAVEL
    local finalRotation = 0

    animation:addAction(AHDelayed.new(delay, AHSpawn.new(
      AHLocationYBy.new(letter, travelDuration, AHEaseExponentialOut.new(), distanceToTravel),
      AHRotationTo.new(letter, travelDuration, AHEaseExponentialOut.new(), finalRotation)
    )))
  end

  return animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TypoBonusLabel.new(text, font, fontSize)
  assert(#text == 5, string.format("Unexpected text: %s", text))
  local adjustedFontSize = FontScaling.getScaledSize(fontSize)
  local label = DeprecatedLabel.newSingleLine(text, font, adjustedFontSize)

  local self = ComposedGlyphLabel.new(label)
  Mixin.mix(self, TypoBonusLabel)

  local colors = getBonusColors()
  for i, child in ipairs(self.children) do
    child:setColor(unpack(colors[i]))
  end

  return self
end
