--- TypoCommon
-- @author Created by anton on July 24, 2015.

class("TypoCommon")

require("utils.ColorUtils")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- XML tags ---------------------------

-- Error that the user has to find, not corrected yet
TypoCommon.ERROR_TAG_LABEL = "error"
-- Error that the user had to find and has found, after correction
TypoCommon.CORRECT_TAG_LABEL = "correct"
-- Error that the user does not have to find, shown in its correct form
TypoCommon.DISABLED_TAG_LABEL = "disabled"

TypoCommon.DROPCAP_TAG_LABEL = "dropcap"

-- TextItem attributes ----------------

TypoCommon.IS_INCORRECT_ATTRIBUTE = "isIncorrect"
TypoCommon.IS_CORRECT_ATTRIBUTE = "isCorrect"
TypoCommon.ERROR_ID_ATTRIBUTE = "errorId"

-- Error types ------------------------

TypoCommon.ERROR_SPELLING = "spell"
TypoCommon.ERROR_PUNCTUATION_CAPITALIZATION = "cap_punct"
TypoCommon.ERROR_EXTRA_MISSING = "missing_extra"
TypoCommon.ERROR_BONUS = "bonus"

-- Colors -----------------------------

-- Rounds & bubbles

TypoCommon.COLOR_PURPLE = ColorUtils.colorRGBToStd(223, 60, 169)
TypoCommon.COLOR_PURPLE_LIGHT = ColorUtils.colorRGBToStd(221, 101, 185)
TypoCommon.COLOR_PURPLE_DARK = ColorUtils.colorRGBToStd(192, 0, 120)

TypoCommon.COLOR_PINK = ColorUtils.colorRGBToStd(251, 46, 121)
TypoCommon.COLOR_PINK_LIGHT = ColorUtils.colorRGBToStd(252, 74, 138)
TypoCommon.COLOR_PINK_DARK = ColorUtils.colorRGBToStd(200, 26, 89)

TypoCommon.COLOR_ORANGE = ColorUtils.colorRGBToStd(246, 160, 50)
TypoCommon.COLOR_ORANGE_LIGHT = ColorUtils.colorRGBToStd(248, 190, 109)
TypoCommon.COLOR_ORANGE_DARK = ColorUtils.colorRGBToStd(241, 137, 99)

TypoCommon.COLOR_GRAY = { 0.85, 0.85, 0.85 }
TypoCommon.COLOR_GRAY_LIGHT = { 0.90, 0.90, 0.90 }
TypoCommon.COLOR_GRAY_DARK = { 0.75, 0.75, 0.75 }

-- Text

TypoCommon.COLOR_TEXT_GRAY_LIGHT = ColorUtils.colorHex24ToStd(0x949494)
TypoCommon.COLOR_TEXT_GRAY_DARK = ColorUtils.colorRGBToStd(77, 77, 77)

TypoCommon.COLOR_WRONG_ANSWER = ColorUtils.colorRGBToStd(255, 89, 71)
TypoCommon.COLOR_DIM_GRAY_LIGHT = ColorUtils.colorRGBToStd(207, 207, 207)
TypoCommon.COLOR_DIM_GRAY_DARK = ColorUtils.colorRGBToStd(153, 153, 153)
TypoCommon.COLOR_TEXT_ERROR_RED = {1.0, 0.0, 0.0}

TypoCommon.COLOR_TEXT_CONTINUE = {0.79, 0.79, 0.79}

TypoCommon.UNDERLINE_THICKNESS = 1 / Host.contentScale

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createBonusColorRotationAnimation(object)
  local durationPerColor = 0.5
  return AHLoop.new(
    AHSequence.new(
      AHColorTo.new(object, durationPerColor, AHEaseSineInOut.new(), TypoCommon.COLOR_PURPLE),
      AHColorTo.new(object, durationPerColor, AHEaseSineInOut.new(), TypoCommon.COLOR_PINK),
      AHColorTo.new(object, durationPerColor, AHEaseSineInOut.new(), TypoCommon.COLOR_ORANGE)
    )
  )
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoCommon.getLightColor(color)
  if color == TypoCommon.COLOR_PURPLE then
    return TypoCommon.COLOR_PURPLE_LIGHT
  elseif color == TypoCommon.COLOR_PINK then
    return TypoCommon.COLOR_PINK_LIGHT
  elseif color == TypoCommon.COLOR_ORANGE then
    return TypoCommon.COLOR_ORANGE_LIGHT
  elseif color == TypoCommon.COLOR_GRAY then
    return TypoCommon.COLOR_GRAY_LIGHT
  end
end

function TypoCommon.getDarkColor(color)
  if color == TypoCommon.COLOR_PURPLE then
    return TypoCommon.COLOR_PURPLE_DARK
  elseif color == TypoCommon.COLOR_PINK then
    return TypoCommon.COLOR_PINK_DARK
  elseif color == TypoCommon.COLOR_ORANGE then
    return TypoCommon.COLOR_ORANGE_DARK
  elseif color == TypoCommon.COLOR_GRAY then
    return TypoCommon.COLOR_GRAY_DARK
  end
end

function TypoCommon.startBonusColorAnimation(object)
  if TypoCommon.bonusColorRotationMaster == nil then
    local bonusColorRotationMaster = MOAIColor.new()
    local animation = createBonusColorRotationAnimation(bonusColorRotationMaster)
    animation:start()
    TypoCommon.bonusColorRotationMaster = bonusColorRotationMaster
  end

  object:overrideColor(TypoCommon.bonusColorRotationMaster)
end

function TypoCommon.stopBonusColorAnimation(object)
  object:clearAttrLink(MOAIProp2D.INHERIT_COLOR_RAW)
end
