--- TypoConceptGenerator
-- @author Created by anton on July 24, 2015.

class("TypoConceptGenerator")

require("TypoCorrection")
require("TypoCommon")
require("data.generators.UtilityGenerators")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function extractCorrectTextsByErrorNumber(text)
  local correctTextsByErrorNumber = {}
  text:gsub("<([0-9]+)>(.-)</[0-9]+>", function(errorNumber, correctText)
    correctTextsByErrorNumber[errorNumber] = correctText
  end)
  return correctTextsByErrorNumber
end

local function getCorrectTextsByErrorNumber(passagesByErrorType, bonusText)
  local correctTextsByErrorNumber = {}
  for _, passageText in pairs(passagesByErrorType) do
    local newCorrectTextsByErrorNumber = extractCorrectTextsByErrorNumber(passageText)
    correctTextsByErrorNumber = TableUtils.unionSet(correctTextsByErrorNumber, newCorrectTextsByErrorNumber)
  end

  local bonusCorrectTextsByErrorNumber = extractCorrectTextsByErrorNumber(bonusText)
  correctTextsByErrorNumber = TableUtils.unionSet(correctTextsByErrorNumber, bonusCorrectTextsByErrorNumber)

  return correctTextsByErrorNumber
end

local function getCorrectionsByErrorId(errorsData, passagesByErrorType, bonusText)
  local correctTextsByErrorNumber = getCorrectTextsByErrorNumber(passagesByErrorType, bonusText)
  local correctionsByErrorId = {}
  for _, errorData in ipairs(errorsData) do
    local errorNumber = errorData.error_number
    local correction = TypoCorrection.new(errorData, correctTextsByErrorNumber[errorNumber])
    correctionsByErrorId[correction:getErrorId()] = correction
  end
  return correctionsByErrorId
end

local function getPassagesByErrorType(rawConcept)
  return {
    [TypoCommon.ERROR_SPELLING] = rawConcept.passageSpelling,
    [TypoCommon.ERROR_PUNCTUATION_CAPITALIZATION] = rawConcept.passagePunctuationCapitalization,
    [TypoCommon.ERROR_EXTRA_MISSING] = rawConcept.passageMissingExtra
  }
end

local function processConcept(rawConcept)
  local passagesByErrorType = getPassagesByErrorType(rawConcept)
  local correctionsByErrorId = getCorrectionsByErrorId(rawConcept.errors, passagesByErrorType, rawConcept.bonusText)

  local concept = {
    identifier = rawConcept.identifier,
    difficulty = rawConcept.difficulty,
    passagesByErrorType = passagesByErrorType,
    bonusText = rawConcept.bonusText,
    correctionsByErrorId = correctionsByErrorId
  }

  return concept
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- This method is exposed for the Validator
function TypoConceptGenerator.processConcept(rawConcept)
  return processConcept(rawConcept)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TypoConceptGenerator.new()
  return TransformationGenerator.new(GameSession.conceptGenerator, processConcept)
end
