--- TypoCorrection
-- @author Created by amorant on July 02, 2015.

class("TypoCorrection")

require("TypoQuoteReplacer")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoCorrection:getErrorId()
  return self.errorId
end

function TypoCorrection:getCorrectText()
  return self.correctText
end

function TypoCorrection:getIncorrectText()
  return self.incorrectText
end

function TypoCorrection:getErrorType()
  return self.errorType
end

function TypoCorrection:getDifficulty()
  return self.difficulty
end

function TypoCorrection:hasBeenApplied()
  return self.applied
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newCorrection(errorNumber, correctText, incorrectText, round, difficulty)
  local self = {}
  Mixin.mix(self, TypoCorrection)

  correctText = TypoQuoteReplacer.replaceStraightQuotesWithCurlyQuotes(correctText)
  incorrectText = TypoQuoteReplacer.replaceStraightQuotesWithCurlyQuotes(incorrectText)

  self.errorId = "error_" .. errorNumber
  self.correctText = correctText
  self.incorrectText = incorrectText
  self.errorType = round
  self.difficulty = difficulty

  return self
end

-- Constructor ------------------------

function TypoCorrection.new(correctionData, correctText)
  Asserts.assertType(correctionData.error_number, "error_number", "string")
  Asserts.assertType(correctionData.incorrect, "incorrect", "string")
  Asserts.assertType(correctionData.round, "round", "string")
  Asserts.assertType(correctionData.difficulty, "difficulty", "string")
  Asserts.assertType(correctText, "correctText for " .. correctionData.error_number, "string")

  assert(correctionData.incorrect ~= correctText, "Correct and incorrect text must differ for " .. correctionData.error_number)

  return newCorrection(correctionData.error_number, correctText, correctionData.incorrect, correctionData.round, correctionData.difficulty)
end

function TypoCorrection.newExampleCorrection(errorNumber, correctText, incorrectText)
  errorNumber = tostring(errorNumber)

  Asserts.assertType(errorNumber, "error_number", "string")
  Asserts.assertType(incorrectText, "incorrect", "string")
  Asserts.assertType(correctText, "correctText", "string")

  assert(incorrectText ~= correctText, "Correct and incorrect text must differ")

  return newCorrection(errorNumber, correctText, incorrectText, nil, nil)
end
