--- TypoExampleTextView
-- @author Created by anton on August 24, 2015.

class("TypoExampleTextView")

require("TypoTextView")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoExampleTextView:createViewForTextItem(item)
  local itemView = self.TypoTextView.createViewForTextItem(self, item)

  -- As well as disabling touching globally on the ExampleTextView, we do this to remove undesired
  -- touch handlers from the screen.
  if itemView.setTouchEnabled then
    itemView:setTouchEnabled(false)
  end

  return itemView
end

function TypoExampleTextView:getRevertCorrectAnimation(errorId)
  local itemViewsToAnimate = TableUtils.getFilteredTable(self.viewsForItems, function(itemView)
    local item = self.itemsForViews[itemView]
    local itemErrorId = item:getAttribute(TypoCommon.ERROR_ID_ATTRIBUTE)
    return itemErrorId == errorId
  end)

  local animation = AHSpawn.new()
  for item, itemView in pairs(itemViewsToAnimate) do
    animation:addAction(AHColorTo.new(itemView, 0.5, nil, self.textColor))
    animation:addAction(function()
      -- Hack: TextItem attributes are supposed to be read-only.
      -- The game was designed to swap all incorrect items, but for animation reasons now we leave
      -- some of them untouched (instead of replacing them with new items with equal text).
      item.attributes[TypoCommon.IS_INCORRECT_ATTRIBUTE] = true
      item.attributes[TypoCommon.IS_CORRECT_ATTRIBUTE] = false
    end)
  end

  return animation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TypoExampleTextView.new(textWidth)
  local self = TypoTextView.new(textWidth)
  Mixin.mix(self, TypoExampleTextView, "TypoTextView")

  -- An example text view should receive no touches
  self:setTouchEnabled(false)

  return self
end
