--- TypoGameIntro
-- @author Created by anton on August 19, 2015.

class("TypoGameIntro")

require("TypoCommon")
require("TypoBubbleSwarm")
require("TypoBubbleLayouts")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TITLE_TEXT_COLOR = TypoCommon.COLOR_PURPLE
local SUBTITLE_TEXT_COLOR = TypoCommon.COLOR_TEXT_GRAY_LIGHT
local PARAGRAPH_TEXT_COLOR = TypoCommon.COLOR_TEXT_GRAY_DARK

local TITLE_TEXT_SIZE = 33
local SUBTITLE_TEXT_SIZE = 16
local PARAGRAPH_TEXT_SIZE = 17
local CONTINUE_TEXT_SIZE = 14

local INTRO_TEXT = "In this game, you'll review a passage in three rounds.<br/><br/>Each round focuses on a single type of error. This is proven to be the most effective way to proofread; you'll read a passage multiple times, but you'll identify many more errors."

local SLIDE_DISTANCE = Screen.height / 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoGameIntro:prepareToShow()
  local slidingGroup = self.slidingGroup
  local offscreenY = slidingGroup:getY() - SLIDE_DISTANCE
  slidingGroup:setY(offscreenY)
  slidingGroup:setOpacity(0)
end

function TypoGameIntro:getShowAnimation()
  local slidingGroup = self.slidingGroup
  local duration = 0.7

  return AHSpawn.new(
    AHLocationYBy.new(slidingGroup, duration, AHEaseQuadraticOut.new(), SLIDE_DISTANCE),
    AHOpacityTo.new(slidingGroup, duration, AHEaseQuadraticOut.new(), 1)
  )
end

function TypoGameIntro:getHideAnimation()
  local slidingGroup = self.slidingGroup
  local duration = 0.7

  local animation = AHSpawn.new(
    AHLocationYBy.new(slidingGroup, duration, AHEaseQuadraticOut.new(), SLIDE_DISTANCE),
    AHOpacityTo.new(slidingGroup, duration, AHEaseQuadraticOut.new(), 0)
  )

  local removeSelf = AHRemove.new(self)

  return AHSequence.new(animation, removeSelf)
end

function TypoGameIntro:waitForTap()
  local tapToContinue = self.tapToContinue
  waitFor(1)
  tapToContinue:rearm()
  tapToContinue:waitForTrigger()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self)
  local background = FilledRectangle.newFullScreen(COLOR_BACKGROUND)
  self:insertAtCenter(background)

  local column = Column.new(0, Alignment.LEFT)
  local textWidth = self:getWidthPart(0.86)
  local lineSpacing = 1.2
  local spacingBeforeParagraph = self:getHeightPart(0.02)
  local spacingBetweenSections = self:getHeightPart(0.03)

  local titleLabel = Label.new("ABOUT THE GAME", Font.new(FontFaces.DIN_OT_BOLD, TITLE_TEXT_SIZE))
  titleLabel:setColor(unpack(TITLE_TEXT_COLOR))
  column:insert(titleLabel)
  column:insertSpacer(spacingBeforeParagraph)

  local introParagraph = DynamicTextArea.newSimple(INTRO_TEXT, textWidth, Font.new(FontFaces.DIN_OT_MEDIUM, PARAGRAPH_TEXT_SIZE))
  introParagraph:setLineSpacing(lineSpacing)
  introParagraph:setColor(unpack(PARAGRAPH_TEXT_COLOR))
  column:insert(introParagraph)
  column:insertSpacer(spacingBetweenSections)

  column:alignChildren()
  column:setAnchorPointTop()
  column:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.65))
  self.column = column

  local continueLabel = Label.new("CONTINUE", Font.new(FontFaces.DIN_OT_MEDIUM, CONTINUE_TEXT_SIZE))
  continueLabel:setColor(unpack(SUBTITLE_TEXT_COLOR))
  continueLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.07))
  self.continueLabel = continueLabel

  local underlineThickness = TypoCommon.UNDERLINE_THICKNESS
  local continueUnderline = FilledRectangle.new(continueLabel:getWidth(), underlineThickness)
  local underlineYOffset = continueLabel:getHeightPart(-0.03)
  continueUnderline:setLoc(continueLabel:getWidthPart(0.5), underlineYOffset)
  continueLabel:insert(continueUnderline)
  continueLabel:linkColorAttrToChildren()

  local bubbleSwarm = TypoBubbleSwarm.new()
  bubbleSwarm:setBubbleLayouts(TypoBubbleLayouts.LAYOUTS_INTRO, TITLE_TEXT_COLOR)
  self.bubbleSwarm = bubbleSwarm

  local slidingGroup = Group.newFullScreen()
  slidingGroup:insert(column)
  slidingGroup:insert(continueLabel)
  slidingGroup:insertAtCenter(bubbleSwarm)
  slidingGroup:linkOpacityAttrToChildren(true)
  self:insertAtCenter(slidingGroup)
  self.slidingGroup = slidingGroup

  local tapToContinue = TapToContinue.newInGroup(continueLabel)
  tapToContinue:setDims(Screen.width, Screen.getHeightPart(0.2))
  tapToContinue:disarm()
  self.tapToContinue = tapToContinue
end

-- Constructor ------------------------

function TypoGameIntro.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, TypoGameIntro)

  createElements(self)

  return self
end
