--- TypoHowToPlayPanel
-- @author Created by anton on August 25, 2015.

class("TypoHowToPlayPanel")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = { 0, 0, 0 }
local INFO_TEXT_COLOR = { 1, 1, 1 }
local TITLE_TEXT_COLOR = TypoCommon.COLOR_TEXT_GRAY_LIGHT
local BACKGROUND_OPACITY = 0.96

local INFO_TEXT_SIZE = 20
local TITLE_TEXT_SIZE = 14

local ANIMATION_DURATION = 0.25

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function playShowAnimation(self)
  local animation = AHSpawn.new(
    AHOpacityTo.new(self.background, ANIMATION_DURATION, nil, BACKGROUND_OPACITY),
    AHOpacityTo.new(self, ANIMATION_DURATION, nil, 1)
  )

  wait(animation:start())
end

local function playHideAnimation(self)
  local animation = AHSpawn.new(
    AHOpacityTo.new(self.background, ANIMATION_DURATION, nil, 0),
    AHOpacityTo.new(self, ANIMATION_DURATION, nil, 0)
  )

  wait(animation:start())
end

local function prepareToShow(self)
  self.background:setOpacity(0)
  self:setOpacity(0)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoHowToPlayPanel:showAndWaitForTap()
  local tapToContinue = self.tapToContinue

  playShowAnimation(self)
  waitFor(0.5)
  tapToContinue:rearm()
  tapToContinue:waitForTrigger()
  playHideAnimation(self)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self)
  local background = FilledRectangle.newFullScreen(BACKGROUND_COLOR)
  self:insertAtCenter(background)
  self.background = background

  local column = Column.new(Screen.getHeightPart(0.03), Alignment.CENTER)

  local titleLabel = Label.new("HOW TO PLAY", Font.new(FontFaces.DIN_OT_MEDIUM, TITLE_TEXT_SIZE))
  titleLabel:setColor(unpack(TITLE_TEXT_COLOR))
  titleLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
  column:insert(titleLabel)
  self.titleLabel = titleLabel

  local infoText = "TAP THE TEXT TO CORRECT ANY ERRORS YOU FIND"
  local infoLabel = TextArea.new(infoText, Font.new(FontFaces.DIN_OT_MEDIUM, INFO_TEXT_SIZE), self:getWidthPart(0.96), Alignment.CENTER)
  infoLabel:setColor(unpack(INFO_TEXT_COLOR))
  infoLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))
  column:insert(infoLabel)
  self.infoLabel = infoLabel

  local continueLabel = Label.new("CONTINUE", Font.new(FontFaces.DIN_OT_MEDIUM, TITLE_TEXT_SIZE))
  continueLabel:setColor(unpack(TITLE_TEXT_COLOR))
  continueLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.07))
  continueLabel:inheritOpacity(self)
  self:insert(continueLabel)
  self.continueLabel = continueLabel

  local underlineThickness = TypoCommon.UNDERLINE_THICKNESS
  local continueUnderline = FilledRectangle.new(continueLabel:getWidth(), underlineThickness)
  local underlineYOffset = continueLabel:getHeightPart(-0.03)
  continueUnderline:setLoc(continueLabel:getWidthPart(0.5), underlineYOffset)
  continueLabel:insert(continueUnderline)
  continueLabel:linkColorAttrToChildren()

  column:alignChildren()
  column:inheritOpacity(self)
  column:linkOpacityAttrToChildren(true)
  self:insertAtCenter(column)
  self.column = column

  local tapToContinue = TapToContinue.newInGroup(continueLabel)
  tapToContinue:setDims(Screen.width, Screen.getHeightPart(0.2))
  tapToContinue:disarm()
  self.tapToContinue = tapToContinue
end

-- Constructor ------------------------

function TypoHowToPlayPanel.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, TypoHowToPlayPanel)

  createElements(self)
  prepareToShow(self)

  return self
end
