--- TypoMoreInfoPanel
-- @author Created by anton on August 20, 2015.

class("TypoMoreInfoPanel")

require("TypoCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_COLOR = TypoCommon.COLOR_TEXT_GRAY_DARK
local BACK_TEXT_SIZE = 16
local INFO_TEXT_SIZE = 18

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoMoreInfoPanel:setTouchEnabled(isEnabled)
  self.backButton:setTouchEnabled(isEnabled)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self, round, goBackTouchHandler)
  local roundColor = round:getColor()

  local column = Column.new(self:getHeightPart(0.01), Alignment.LEFT)
  column:insertSpacer(self:getHeightPart(0.2))
  self.column = column

  local backLabel = Label.new("BACK", Font.new(FontFaces.DIN_OT_MEDIUM, BACK_TEXT_SIZE))
  local backArrow = Sprite.newWithName("arrow.png")
  local backLabelAndArrow = Row.new(5)
  backLabel:setColor(unpack(roundColor))
  backArrow:setColor(unpack(roundColor))
  backArrow:setRot(180)
  backLabelAndArrow:insert(backArrow)
  backLabelAndArrow:insert(backLabel)
  backLabelAndArrow:alignChildren()
  local buttonWidth = backLabelAndArrow:getWidthPart(3)
  local buttonHeight = backLabelAndArrow:getHeightPart(4)
  Button.addVirtualButton(backLabelAndArrow, buttonWidth, buttonHeight, goBackTouchHandler)
  column:insert(backLabelAndArrow)
  column:insertSpacer(self:getHeightPart(0.005))
  self.backButton = backLabelAndArrow

  local infoText = round:getInfoText()
  local lineSpacing = 1.2
  local infoLabelHeight = self:getHeightPart(0.4) -- HACK: TextArea gets the height wrong with line spacing
  local infoLabel = TextArea.new(infoText, Font.new(FontFaces.DIN_OT_MEDIUM, INFO_TEXT_SIZE), self:getWidthPart(0.86), nil, infoLabelHeight, nil, lineSpacing)
  infoLabel:setColor(unpack(TEXT_COLOR))
  column:insert(infoLabel)

  column:alignChildren()
  column:linkOpacityAttrToChildren(true)
  column:inheritOpacity(self)
  column:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.48))
  self:insert(column)
end

-- Constructor ------------------------

function TypoMoreInfoPanel.new(round, goBackTouchHandler)
  local self = Group.newFullScreen()
  Mixin.mix(self, TypoMoreInfoPanel)

  createElements(self, round, goBackTouchHandler)

  self:setTouchEnabled(false)

  return self
end
