--- TypoNoErrorLabel
-- @author Created by anton on August 21, 2015.

class("TypoNoErrorLabel")

require("TypoCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.getWidthPart(0.35)
local HEIGHT = Screen.getHeightPart(0.1)
local CORNER_RADIUS = 8
local TEXT_SIZE = 15

local COLOR_TEXT_TOP = ColorUtils.colorRGBToStd(255, 255, 255)
local COLOR_TEXT_BOTTOM = TypoCommon.COLOR_WRONG_ANSWER
local BACKGROUND_COLOR = TypoCommon.COLOR_TEXT_GRAY_DARK
local OPACITY = 0.95

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function positionNextToItemView(self, itemView)
  local itemViewX, itemViewY = itemView:getCenterWorldLoc()
  local itemViewTopY = itemViewY + itemView:getHeightPart(0.5)
  local distanceY = self:getHeightPart(0.1)

  local x = itemViewX
  local y = itemViewTopY + distanceY
  self:setLoc(x, y)

  local leftX = self:getXLeft()
  local rightX = self:getXRight()
  local leftMargin = Screen.getWidthPart(0.07)
  local rightMargin = Screen.width - leftMargin

  if leftX < leftMargin then
    local offsetToMoveRight = leftMargin - leftX
    self:setX(x + offsetToMoveRight)
  elseif rightX > rightMargin then
    local offsetToMoveLeft = rightX - rightMargin
    self:setX(x - offsetToMoveLeft)
  end

  return self
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoNoErrorLabel:getShowAnimation(itemView, duration)
  local travelAmount = self:getHeight() * 2
  return AHSequence.new(
    function()
      positionNextToItemView(self, itemView)
      self:setVisibleRecursive(true)
      self:setOpacity(0)
      self:setY(self:getY() - travelAmount)
    end,
    AHSpawn.new(
      AHLocationYBy.new(self, duration, nil, travelAmount),
      AHOpacityTo.new(self, duration, AHEaseSineIn.new(), OPACITY)
    )
  )
end

function TypoNoErrorLabel:getHideAnimation(duration)
  return AHSequence.new(
    AHOpacityTo.new(self, duration, AHEaseSineIn.new(), 0),
    function()
      self:setVisibleRecursive(false)
    end
  )
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createElements(self)
  local column = Column.new(0, Alignment.CENTER)

  local topLabel = Label.new("NO ERROR", Font.new(FontFaces.DIN_OT_MEDIUM, TEXT_SIZE))
  topLabel:setColor(unpack(COLOR_TEXT_TOP))
  column:insert(topLabel)

  local bottomText = string.format("-%s SECONDS", TypoConfiguration.getSecondsToTakeOnIncorrect())
  local bottomLabel = Label.new(bottomText, Font.new(FontFaces.DIN_OT_MEDIUM, TEXT_SIZE))
  bottomLabel:setColor(unpack(COLOR_TEXT_BOTTOM))
  column:insert(bottomLabel)

  column:alignChildren()
  self:insertAtCenter(column)

  self:setAnchorPointBottom()
  self:setVisibleRecursive(false)
  self:linkOpacityAttrToChildren(true)
end

-- Constructor ------------------------

function TypoNoErrorLabel.new()
  local self = FilledRoundedRectangle.new(WIDTH, HEIGHT, CORNER_RADIUS, BACKGROUND_COLOR)
  Mixin.mix(self, TypoNoErrorLabel)

  createElements(self)

  return self
end
