--- TypoParticles
-- @author Created by anton on August 17, 2015.

class("TypoParticles")

require("TypoCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PARTICLE_COUNT = 18
local MIN_RADIUS = math.floor(Screen.getWidthPart(0.01))
local MAX_RADIUS = math.ceil(Screen.getWidthPart(0.03))
local MIN_DISTANCE = math.floor(Screen.getWidthPart(0.05))
local MAX_DISTANCE = math.ceil(Screen.getWidthPart(0.2))

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getRegularRoundColor(mainColor)
  assert(mainColor, "No main color set")

  local roll = math.random()

  if roll < 0.17 then
    return TypoCommon.COLOR_GRAY
  elseif roll < 0.33 then
    return TypoCommon.COLOR_GRAY_LIGHT
  elseif roll < 0.50 then
    return TypoCommon.COLOR_GRAY_DARK
  elseif roll < 0.67 then
    return mainColor
  elseif roll < 0.83 then
    return TypoCommon.getLightColor(mainColor)
  else
    return TypoCommon.getDarkColor(mainColor)
  end
end

local function getBonusRoundColor()
  local roll = math.random()

  if roll < 0.11 then
    return TypoCommon.COLOR_GRAY
  elseif roll < 0.22 then
    return TypoCommon.COLOR_GRAY_LIGHT
  elseif roll < 0.33 then
    return TypoCommon.COLOR_GRAY_DARK
  elseif roll < 0.40 then
    return TypoCommon.COLOR_PURPLE
  elseif roll < 0.48 then
    return TypoCommon.COLOR_PURPLE_LIGHT
  elseif roll < 0.55 then
    return TypoCommon.COLOR_PURPLE_DARK
  elseif roll < 0.63 then
    return TypoCommon.COLOR_PINK
  elseif roll < 0.70 then
    return TypoCommon.COLOR_PINK_LIGHT
  elseif roll < 0.78 then
    return TypoCommon.COLOR_PINK_DARK
  elseif roll < 0.85 then
    return TypoCommon.COLOR_ORANGE
  elseif roll < 0.93 then
    return TypoCommon.COLOR_ORANGE_LIGHT
  else
    return TypoCommon.COLOR_ORANGE_DARK
  end
end

local function getColor(self)
  if self.isBonus then
    return getBonusRoundColor()
  else
    return getRegularRoundColor(self.mainColor)
  end
end

local function spawnParticle(self, particle)
  local radius = math.random(MIN_RADIUS, MAX_RADIUS)
  local distance = math.random(MIN_DISTANCE, MAX_DISTANCE)
  local direction = MathUtils.getRandomFloat(0, 2 * math.pi)
  local xDistance = distance * math.cos(direction)
  local yDistance = distance * math.sin(direction)
  local color = getColor(self)

  particle:setWidth(2 * radius)
  particle:setHeight(2 * radius)
  particle:setColor(unpack(color))
  particle:setScl(1)
  particle:setVisibleRecursive(true)
  particle:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.5))

  local scatterX = AHLocationXBy.new(particle, 1.2, AHEaseExponentialOut.new(), xDistance)
  local scatterY = AHLocationYBy.new(particle, 1.2, AHEaseExponentialOut.new(), yDistance)
  local scaleDown = AHScaleTo.new(particle, 1.1, AHEaseSineIn.new(), 0)

  runInCoroutine(function()
    scatterX:start()
    scatterY:start()
    waitFor(0.1)
    wait(scaleDown:start())
    particle:setVisibleRecursive(false)
  end)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoParticles:setMainColor(mainColor)
  self.mainColor = mainColor
end

function TypoParticles:setIsBonus(isBonus)
  self.isBonus = true
end

function TypoParticles:show()
  local particlePool = self.particlePool

  for _, particle in ipairs(particlePool) do
    spawnParticle(self, particle)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function createParticlesPool(self)
  -- This is an optimization. We create the particles upfront to avoid inserting/removing during
  -- animations.
  local particlePool = {}
  local defaultRadius = 1
  local defaultColor = { 0, 0, 0 }

  for i = 1, PARTICLE_COUNT do
    local particle = FilledCircle.new(defaultRadius, defaultColor)
    particle:setVisibleRecursive(false)
    self:insert(particle)

    table.insert(particlePool, particle)
  end

  self.particlePool = particlePool
end

-- Constructor ------------------------

function TypoParticles.new()
  local self = Group.newMinimumSized()
  Mixin.mix(self, TypoParticles)

  createParticlesPool(self)

  self.mainColor = nil
  self.isBonus = false

  return self
end
