--- TypoRound
-- @author Created by anton on July 24, 2015.

class("TypoRound")

require("TypoConfiguration")
require("TypoQuoteReplacer")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local ROUND_DESCRIPTIONS = {
  [TypoCommon.ERROR_SPELLING] = Localizer.get("ROUND_DESCRIPTION_SPELLING"),
  [TypoCommon.ERROR_PUNCTUATION_CAPITALIZATION] = Localizer.get("ROUND_DESCRIPTION_PUNCTUATION_CAPITALIZATION"),
  [TypoCommon.ERROR_EXTRA_MISSING] = Localizer.get("ROUND_DESCRIPTION_EXTRA_MISSING"),
  [TypoCommon.ERROR_BONUS] = Localizer.get("ROUND_DESCRIPTION_BONUS")
}

local ROUND_MORE_INFO_TEXTS = {
  [TypoCommon.ERROR_SPELLING] = Localizer.get("ROUND_MORE_INFO_TEXT_SPELLING"),
  [TypoCommon.ERROR_PUNCTUATION_CAPITALIZATION] = Localizer.get("ROUND_MORE_INFO_TEXT_PUNCTUATION_CAPITALIZATION"),
  [TypoCommon.ERROR_EXTRA_MISSING] = Localizer.get("ROUND_MORE_INFO_TEXT_EXTRA_MISSING")
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoRound:getPassageText()
  return self.passageText
end

function TypoRound:getRoundIndex()
  assert(not self.isBonus, "Bonus rounds have no index")
  return self.roundIndex
end

function TypoRound:getErrorType()
  return self.errorType
end

function TypoRound:isBonusRound()
  return self.isBonus
end

function TypoRound:getDescription()
  local description = ROUND_DESCRIPTIONS[self.errorType]
  assert(description, string.format("Description not found for TypoRound %s", tostring(self.errorType)))
  return description
end

function TypoRound:getInfoText()
  local moreInfoText = ROUND_MORE_INFO_TEXTS[self.errorType]
  assert(moreInfoText, string.format("More info text not found for TypoRound %s", tostring(self.errorType)))
  return moreInfoText
end

function TypoRound:getErrorIds()
  return self.errorIds
end

function TypoRound:getDuration()
  if self:isBonusRound() then
    return TypoConfiguration.getBonusRoundDuration()
  else
    return TypoConfiguration.getRoundDuration(self:getRoundIndex())
  end
end

function TypoRound:getColor()
  assert(not self.isBonus, "Bonus rounds have no color")
  return self.color
end

function  TypoRound:getBubbleEnterLayouts()
  return self.bubbleEnterLayouts
end

function  TypoRound:getBubbleExitLayouts()
  return self.bubbleExitLayouts
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function newTypoRound(passageText, errorType, errorIds, bubbleEnterLayouts, bubbleExitLayouts, isBonus)
  local self = {}
  Mixin.mix(self, TypoRound)

  passageText = TypoQuoteReplacer.replaceStraightQuotesWithCurlyQuotes(passageText)

  self.passageText = passageText
  self.errorType = errorType
  self.errorIds = errorIds
  self.bubbleEnterLayouts = bubbleEnterLayouts
  self.bubbleExitLayouts = bubbleExitLayouts
  self.isBonus = isBonus

  return self
end

-- Constructor ------------------------

function TypoRound.new(passageText, roundIndex, errorType, errorIds, color, bubbleEnterLayouts, bubbleExitLayouts)
  local self = newTypoRound(passageText, errorType, errorIds, bubbleEnterLayouts, bubbleExitLayouts, false)

  self.roundIndex = roundIndex
  self.color = color

  return self
end

function TypoRound.newBonusRound(passageText, errorIds, bubbleEnterLayouts, bubbleExitLayouts)
  local self = newTypoRound(passageText, TypoCommon.ERROR_BONUS, errorIds, bubbleEnterLayouts, bubbleExitLayouts, true)

  return self
end
