--- TypoTextAnimationBuilder
-- @author Created by anton on August 17, 2015.

class("TypoTextAnimationBuilder")

require("text.dynamic.TextAnimationBuilder")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MOVE_DURATION = 0.3
local SHOW_DURATION = MOVE_DURATION
local HIDE_DURATION = 0.2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoTextAnimationBuilder:_createShowItemAnimation(itemView, itemLayout, textView)
  local originalScale = itemView:getScl()
  return AHSequence.new(
    function()
      -- This is an optimization. Normally, it's the show animation's job to insert the item view
      -- in the text view. However, TypoTextView allows pre-caching item views to optimize the
      -- replacement animation. So, if the itemView is already parented, we must not insert.
      if not itemView.parent then
        textView:insert(itemView)
      end
      itemView:setLoc(itemLayout:getX(), itemLayout:getY())
      itemView:setAnchorPoint(itemLayout:getAnchorX(), itemLayout:getAnchorY())
      itemView:transformAnchorPoint(0.5, 0.5)
      itemView:setScl(0)
      itemView:setVisibleRecursive(true) -- precached labels are invisible
    end,
    AHScaleTo.new(itemView, SHOW_DURATION, AHEaseExponentialOut.new(), originalScale),
    function()
      itemView:transformAnchorPoint(itemLayout:getAnchorX(), itemLayout:getAnchorY())
    end
  )
end

function TypoTextAnimationBuilder:_createHideItemAnimation(itemView)
  return AHSequence.new(
    function() itemView:transformAnchorPoint(0.5, 0.5) end,
    AHScaleTo.new(itemView, HIDE_DURATION, AHEaseExponentialIn.new(), 0),
    function() itemView:removeSelf() end
  )
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TypoTextAnimationBuilder.new()
  local self = TextAnimationBuilder.new()
  Mixin.mix(self, TypoTextAnimationBuilder)
  self:setMoveDuration(MOVE_DURATION)

  return self
end
