--- TypoTextController
-- @author Created by anton on July 23, 2015.

class("TypoTextController")

require("TypoCommon")
require("text.dynamic.TextInterpreter")
require("text.dynamic.TextModel")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BASE_ATTRIBUTES = {
  font = Font.new(FontFaces.TISA_REGULAR, 22)
}

local ATTRIBUTES_FOR_LABELS = {
  [TypoCommon.ERROR_TAG_LABEL] = { [TypoCommon.IS_INCORRECT_ATTRIBUTE] = true },
  [TypoCommon.CORRECT_TAG_LABEL] = { [TypoCommon.IS_CORRECT_ATTRIBUTE] = true },
  [TypoCommon.DISABLED_TAG_LABEL] = {},
  [TypoCommon.DROPCAP_TAG_LABEL] = { size = 70, yOffset = FontScaling.getScaledSize(-20) },
  i = { font = Font.new(FontFaces.TISA_ITALIC, 22) }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- Text processing --------------------

local function getProcessedText(text)
  local textAfterReplacingOpenTags = text:gsub("<([0-9]+)>", string.format("<%s errorId=\"error_%%1\">", TypoCommon.DISABLED_TAG_LABEL))
  local textAfterReplacingCloseTags = textAfterReplacingOpenTags:gsub("</([0-9]+)>", string.format("</%s>", TypoCommon.DISABLED_TAG_LABEL))
  local textWithDropCap = textAfterReplacingCloseTags:gsub("^(.)", string.format("<%s>%%1</%s>", TypoCommon.DROPCAP_TAG_LABEL, TypoCommon.DROPCAP_TAG_LABEL))
  return textWithDropCap
end

local function getTextWithSplittedPunctuation(text)
  if text[#text] == "." or text[#text] == "?" then
    text = text:sub(1, #text -1) .. "<invisible_separator/>" .. text[#text]
  end
  return text
end

local function getItemsForText(self, errorId, tagLabel, text)
  local formattedText = string.format("<%s errorId=\"%s\">%s</%s>", tagLabel, errorId, text, tagLabel)
  return self.textInterpreter:interpretXmlText(formattedText)
end

-- Item processing --------------------

local function getItemStartIndexAndCountForErrorId(self, errorId)
  local startIndex = nil
  local count = 0

  for index, item in ipairs(self.textModel:getItems()) do
    if item:getAttribute(TypoCommon.ERROR_ID_ATTRIBUTE) == errorId then
      startIndex = startIndex or index
      count = count + 1
    elseif startIndex ~= nil then
      break
    end
  end

  assert(startIndex and count > 0, string.format("errorId %s not found in items", tostring(errorId)))

  return startIndex, count
end

local function getRefinedReplacementParameters(oldItems, newItems, firstItemIndex)
  local firstItemIndexRefined = firstItemIndex
  local oldItemCountRefined = #oldItems
  local newItemsRefined = TableUtils.shallowCopy(newItems)

  for index, oldItem in ipairs(oldItems) do
    if index > #newItems then
      break
    end
    local newItem = newItems[index]
    if oldItem:getText() == newItem:getText() then
      firstItemIndexRefined = firstItemIndexRefined + 1
      oldItemCountRefined = oldItemCountRefined - 1
      TableUtils.removeValueFromArray(newItemsRefined, newItem)
    end
  end

  if #newItemsRefined > 0 and oldItemCountRefined > 0 then
    for index, oldItem in rpairs(oldItems) do
      local newItemIndex = index + #newItems - #oldItems
      if newItemIndex <= 0 then
        break
      end
    local newItem = newItems[newItemIndex]
      if oldItem:getText() == newItem:getText() then
        oldItemCountRefined = oldItemCountRefined - 1
        TableUtils.removeValueFromArray(newItemsRefined, newItem)
      else
        break
      end
    end
  end

  return firstItemIndexRefined, oldItemCountRefined, newItemsRefined
end


-- TODO: Put these tests into automated tests
-- local TEST_PARAMS = {
--   {
--     oldItems = { {getText=function() return "a" end, t = "a"} },
--     newItems = { {getText=function() return "b" end, t = "b"} },
--     firstItemIndex = 10
--   },
--   {
--     oldItems = { {getText=function() return "a" end, t = "a"}, {getText=function() return "b" end, t = "b" } },
--     newItems = { {getText=function() return "a" end, t = "a"} },
--     firstItemIndex = 10
--   },
--   {
--     oldItems = { {getText=function() return "a" end, t = "a"}, {getText=function() return "b" end, t = "b"} },
--     newItems = { {getText=function() return "a" end, t = "a"}, {getText=function() return "x" end, t = "x"}, {getText=function() return "b" end, t = "b"}, },
--     firstItemIndex = 10
--   },
--   {
--     oldItems = { {getText=function() return "a" end, t = "a"}, {getText=function() return "b" end, t = "b"} },
--     newItems = { {getText=function() return "x" end, t = "x"} },
--     firstItemIndex = 10
--   },
--   {
--     oldItems = { {getText=function() return "a" end, t = "a"} },
--     newItems = { },
--     firstItemIndex = 10
--   },
--   {
--     oldItems = { {getText=function() return "a" end, t = "a"}, {getText=function() return "b" end, t = "b"} },
--     newItems = { {getText=function() return "a" end, t = "a"}, {getText=function() return "b" end, t = "b"}, {getText=function() return "x" end, t = "x"} },
--     firstItemIndex = 10
--   }
-- }

-- for _, params in ipairs(TEST_PARAMS) do
--   local firstItemIndexRefined, oldItemCountRefined, newItemsRefined =
--     getRefinedReplacementParameters(params.oldItems, params.newItems, params.firstItemIndex)

--   print(firstItemIndexRefined, StringUtils.tostring(oldItemCountRefined), StringUtils.tostring(newItemsRefined))
--   print()
-- end

-- Error replacement ------------------

local function replaceErrorText(self, errorId, tagLabel, text)
  local textModel = self.textModel
  local firstItemIndex, oldItemCount = getItemStartIndexAndCountForErrorId(self, errorId)
  local newItems = getItemsForText(self, errorId, tagLabel, text)

  textModel:replaceItems(firstItemIndex, oldItemCount, newItems)
end

local function replaceErrorTextSubtle(self, errorId, tagLabel, text, forceReplace)
  forceReplace = forceReplace or false
  text = getTextWithSplittedPunctuation(text)

  local textModel = self.textModel
  local firstItemIndex, oldItemCount = getItemStartIndexAndCountForErrorId(self, errorId)
  local newItems = getItemsForText(self, errorId, tagLabel, text)

  -- For some error types, some items stay unchanged, so we don't animate in/out all of them,
  -- unless a full replacement has been forced using forceReplace.
  if (not forceReplace) then
    local oldItems = TableUtils.subarray(textModel:getItems(), firstItemIndex, firstItemIndex + oldItemCount - 1)
    firstItemIndex, oldItemCount, newItems = getRefinedReplacementParameters(oldItems, newItems, firstItemIndex)
  end
  textModel:replaceItems(firstItemIndex, oldItemCount, newItems)
end

-- Pre-rendering ----------------------

local function prerenderReplacement(self, errorId)
  local correction = self.correctionsByErrorId[errorId]
  assert(correction, string.format("No correction found for errorId %s", tostring(errorId)))

  local textView = self.textView
  local text = getTextWithSplittedPunctuation(correction:getCorrectText())
  local newItems = getItemsForText(self, errorId, TypoCommon.CORRECT_TAG_LABEL, text)

  textView:prerenderViewsForTextItems(errorId, newItems)
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

-- Protectec API -----------------------

function TypoTextController:_replaceErrorWithCorrect(errorId, forceReplace)
  local correction = self.correctionsByErrorId[errorId]
  assert(correction, string.format("No correction found for errorId %s", tostring(errorId)))
  replaceErrorTextSubtle(self, errorId, TypoCommon.CORRECT_TAG_LABEL, correction:getCorrectText(), forceReplace)
end

function TypoTextController:_replaceErrorWithIncorrect(errorId)
  local correction = self.correctionsByErrorId[errorId]
  assert(correction, string.format("No correction found for errorId %s", tostring(errorId)))
  replaceErrorText(self, errorId, TypoCommon.ERROR_TAG_LABEL, correction:getIncorrectText())
end

function TypoTextController:_replaceErrorWithIncorrectSubtle(errorId)
  local correction = self.correctionsByErrorId[errorId]
  assert(correction, string.format("No correction found for errorId %s", tostring(errorId)))
  replaceErrorTextSubtle(self, errorId, TypoCommon.ERROR_TAG_LABEL, correction:getIncorrectText())
end

-- Public API --------------------------

function TypoTextController:getTextModel()
  return self.textModel
end

function TypoTextController:startRound(round)
  local textModel = self.textModel
  local textInterpreter = self.textInterpreter
  local enabledErrorIds = self.enabledErrorIds

  local passageText = getProcessedText(round:getPassageText())
  local textItems = textInterpreter:interpretXmlText(passageText)
  local errorIdsToEnable = round:getErrorIds()

  -- Set the original text
  textModel:resetItems(textItems)

  -- Make replacements to show errors. We do it in a transaction to them all at once.
  textModel:beginTransaction()

  for _, errorId in ipairs(errorIdsToEnable) do
    self:_replaceErrorWithIncorrect(errorId)
    prerenderReplacement(self, errorId)
  end

  -- Finish the transaction with a resetItems() instead of endTransaction() to avoid animations.
  textModel:resetItems()

  self.enabledErrorIds = errorIdsToEnable
end

function TypoTextController:correctError(errorId, forceReplace)
  self:_replaceErrorWithCorrect(errorId, forceReplace)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TypoTextController.new(correctionsByErrorId, textView)
  local self = {}
  Mixin.mix(self, TypoTextController)

  local textInterpreter = TextInterpreter.new(BASE_ATTRIBUTES, ATTRIBUTES_FOR_LABELS)
  local textModel = TextModel.new({})

  self.textInterpreter = textInterpreter
  self.textModel = textModel
  self.correctionsByErrorId = correctionsByErrorId
  self.enabledErrorIds = {}
  self.textView = textView

  return self
end
