--- TypoTextPanel
-- @author Created by amorant on July 01, 2015.

class("TypoTextPanel")

require("TypoTextView")
require("widgets.TouchScroller")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local AUTO_SCROLL_DURATION = 0.7

-- Colors -----------------------------

local TEXT_COLOR = TypoCommon.COLOR_TEXT_GRAY_DARK

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getAverageYValueForViews(views)
  local uniqueViewYCoordinates = {}
  for _, itemView in pairs(views) do
    local scrollY = itemView:getY()
    uniqueViewYCoordinates[scrollY] = true
  end
  local sumOfUniqueYCoordinates = 0
  local numberOfViews = 0
  for yCoordinate, _ in pairs(uniqueViewYCoordinates) do
    sumOfUniqueYCoordinates = sumOfUniqueYCoordinates + yCoordinate
    numberOfViews = numberOfViews + 1
  end
  return (sumOfUniqueYCoordinates / numberOfViews)
end

local function getScrollCoordinateForViews(self, views)
  local textViewYCoordinate = getAverageYValueForViews(views)
  local _, viewWrapperYCoordinate = self.viewWrapper:transformLoc(self.textView, nil, textViewYCoordinate)

  local scrollYCoordinate = (self.viewWrapper:getHeight() - viewWrapperYCoordinate) - self:getHeightPart(0.5)
  return scrollYCoordinate
end

local function getScrollDeltaForNextError(self, errors)
  local scrollValues = {}
  for _, errorId in ipairs(errors) do
    local views = self.textView:getViewsForError(errorId)
    table.insert(scrollValues, getScrollCoordinateForViews(self, views))
  end

  local currentScroll = self:getScrollY()
  local returnIsCloserToCurrentScroll = function(a, b)
    local aDistance = math.abs(a - currentScroll)
    local bDistance = math.abs(b - currentScroll)
    return aDistance < bDistance
  end
  table.sort(scrollValues, returnIsCloserToCurrentScroll)
  local scrollValue = scrollValues[1]

  local minimumScrollDelta = self:getHeightPart(0.35)
  local scrollDelta = scrollValue - currentScroll
  if (math.abs(scrollDelta) < minimumScrollDelta) then
    return 0
  else
    return scrollDelta
  end
end

local function attachToTextViewHeightChanges(textView, viewWrapper)
  local originalSetHeight = textView.setHeight

  textView.setHeight = function(textViewSelf, height)
    originalSetHeight(textViewSelf, height)
    viewWrapper:alignChildren()
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function TypoTextPanel:getTextView()
  return self.textView
end

function TypoTextPanel:getBottomArea()
  return self.bottomArea
end

function TypoTextPanel:scrollToBottom()
  local scrollY = self.viewWrapper:getHeight()
  self:setScrollY(scrollY)
end

function TypoTextPanel:newScrollToTopAction(duration)
  local scrollY = 0
  return self:newScrollYTo(duration, AHEaseQuadraticOut.new(), scrollY)
end

function TypoTextPanel:newScrollToBottomAction(duration)
  local scrollY = self.viewWrapper:getHeight()
  return self:newScrollYTo(duration, AHEaseQuadraticIn.new(), scrollY)
end

function TypoTextPanel:scrollToNextError(errors)
  local scrollDelta = getScrollDeltaForNextError(self, errors)
  local ease = AHEaseBezier.newFlat({0.30, 0.00, 0.10, 1.00})
  return self:newScrollYBy(AUTO_SCROLL_DURATION, ease, scrollDelta):start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function TypoTextPanel.new(width, height, answerHandler)
  local textView = TypoTextView.new(width, answerHandler)
  textView:setLineSpacing(4)
  textView:setTextColor(TEXT_COLOR)

  local viewWrapper = Column.new(0)
  local bottomArea = Group.new(width, height / 3)
  attachToTextViewHeightChanges(textView, viewWrapper)
  viewWrapper:insert(textView)
  viewWrapper:insert(bottomArea)
  viewWrapper:alignChildren()

  local self = TouchScroller.new(width, height, viewWrapper)
  Mixin.mix(self, TypoTextPanel)

  self:setInertialScrollingEnabled(true)
  self:setWillResetInitialTouchCoordinateUponScrollStart(true)

  self.textView = textView
  self.bottomArea = bottomArea
  self.viewWrapper = viewWrapper

  self.remainingErrors = {}

  return self
end
