--- UpperBackground
-- @author Created by steven on March 16, 2016.

class("UpperBackground")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = Screen.getWidthPart(1.0)
local HEIGHT = Screen.getHeightPart(1.0)

local BASE_COLORS = {
  {0.913, 0.658, 0.894},
  {0.988, 0.964, 0.988},
  {1.0, 0.8, 0.796}
}

local QUESTION_OVERLAY_COLOR = {1.0, 0.8, 0.796, 0.0}

local CORRECT_COLORS = {
  {0.0, 0.866, 1.0},
  {0.164, 0.964, 1.0},
  {0.749, 0.933, 1.0},
  {0.956, 0.874, 0.976}
}

local INCORRECT_COLORS = {
  {0.431, 0.078, 0.576},
  {0.431, 0.078, 0.576},
  {0.964, 0.2, 0.670},
  {0.964, 0.2, 0.670}
}

local INCORRECT_OVERLAY_COLORS = {
  {1.0, 0.243, 0.929, 0.0},
  {0.768, 0.101, 0.654, 0.0}
}

local OUTRO_COLORS = {
  {0.164, 0.964, 1.0},
  {0.415, 0.796, 1.0},
  {0.533, 0.533, 0.984},
  {1.0, 0.725, 1.0}
}

local BASE_THICKNESSES = {
  0.5,
  0.5
}

local CORRECT_THICKNESSES = {
  0.3,
  0.2,
  0.6
}

local INCORRECT_THICKNESSES = {
  0.2,
  0.5,
  0.3
}

local OUTRO_THICKNESSES = {
  0.2,
  0.35,
  0.45
}

local NEXT_BACKGROUND_Z = 0
local ACTIVE_BACKGROUND_Z = 1

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getTableFromSingleColor(color, numberOfFields)
  local numberOfColors = numberOfFields
  local newColorTable = {}

  for colorIndex = 1, numberOfColors do
    newColorTable[colorIndex] = color
  end

  return newColorTable
end

local function createBackground(colors, thicknesses)
  local background = MultiGradient.new(WIDTH, HEIGHT, false, colors, thicknesses)
  return background
end

local function getColorAtYLoc(y)
  local normalizedYForScreen = MathUtils.clamp(y / Screen.fullHeight, 0.0, 1.0)
  local color = {}
  local normalizedYForGradient = 0.0

  if (normalizedYForScreen <= 0.45) then
    normalizedYForGradient = MathUtils.normalize(normalizedYForScreen, 0.0, 0.45)
    color = ColorUtils.makeColorBetweenTwoColors(OUTRO_COLORS[4], OUTRO_COLORS[3], normalizedYForGradient)
  elseif (normalizedYForScreen > 0.45) and (normalizedYForScreen <= 0.8) then
    normalizedYForGradient = MathUtils.normalize(normalizedYForScreen, 0.45, 0.8)
    color = ColorUtils.makeColorBetweenTwoColors(OUTRO_COLORS[3], OUTRO_COLORS[2], normalizedYForGradient)
  else
    normalizedYForGradient = MathUtils.normalize(normalizedYForScreen, 0.8, 1.0)
    color = ColorUtils.makeColorBetweenTwoColors(OUTRO_COLORS[2], OUTRO_COLORS[1], normalizedYForGradient)
  end

  return color
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperBackground:createBackgroundAction(duration, newBackground)
  local oldBackground = self.currentBackground
  self:insertAtCenter(newBackground, NEXT_BACKGROUND_Z)

  local swapAction = function()
    if newBackground and newBackground.parent then
      newBackground:setZIndex(ACTIVE_BACKGROUND_Z)
    end

    if oldBackground then
      oldBackground:removeSelf()
    end

    self.currentBackground = newBackground
  end

  local fadeAndSwapAction = AHSequence.new(
    oldBackground:createOpacityTo(duration, nil, 0.0),
    swapAction
  )

  return fadeAndSwapAction
end

function UpperBackground:createToNeutralAction(duration)
  local newBackground = createBackground(BASE_COLORS, BASE_THICKNESSES)

  return self:createBackgroundAction(duration, newBackground)
end

function UpperBackground:createNeutralToQuestionAction(duration)
  local background = self.currentBackground
  local startColor = {1.0, 1.0, 1.0}
  local endColor = {1.0, 0.8, 0.796}
  local midColor = ColorUtils.makeColorBetweenTwoColors(startColor, endColor, 0.0)

  local colorAction = background:newGradientToAction(duration, nil, {startColor, midColor, endColor})
  return colorAction
end

function UpperBackground:createToCorrectAction(duration)
  local newBackground = createBackground(CORRECT_COLORS, CORRECT_THICKNESSES)

  return self:createBackgroundAction(duration, newBackground)
end

function UpperBackground:createToIncorrectAction(duration)
  local newBackground = createBackground(INCORRECT_COLORS, INCORRECT_THICKNESSES)

  return self:createBackgroundAction(duration, newBackground)
end

function UpperBackground:createToOutroAction(duration)
  local newBackground = createBackground(OUTRO_COLORS, OUTRO_THICKNESSES)

  return self:createBackgroundAction(duration, newBackground)
end

function UpperBackground:createColorToAction(duration, color)
  local currentBackground = self.currentBackground
  local newColors = getTableFromSingleColor(color, #currentBackground.colors)

  local toColorAction = currentBackground:newGradientToAction(duration, nil, newColors)
  return toColorAction
end

function UpperBackground:getOutroColorAtYLocation(y)
  return getColorAtYLoc(y)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors ------------------------

function UpperBackground.new()
  local self = Group.newFullScreen()
  Mixin.mix(self, UpperBackground)

  self.currentBackground = createBackground(BASE_COLORS, BASE_THICKNESSES)
  self:createColorToAction(0.0, {1.0, 1.0, 1.0}):start() -- background needs to start as white
  self:insertAtCenter(self.currentBackground, ACTIVE_BACKGROUND_Z)

  return self
end

function UpperBackground.newIncorrectTopLayer()
  local topLayer = RectangleGradient.new(Screen.fullWidth, Screen.getHeightPart(0.5), INCORRECT_OVERLAY_COLORS[1], INCORRECT_COLORS[2])
  topLayer:setAnchorPointBottomLeft()
  return topLayer
end

function UpperBackground.newQuestionTopLayer()
  local topLayer = RectangleGradient.new(Screen.fullWidth, Screen.fullHeight, QUESTION_OVERLAY_COLOR, QUESTION_OVERLAY_COLOR)
  topLayer:setAnchorPointBottomLeft()
  topLayer:setBlendMode(MOAIProp2D.BLEND_ADD)

  return topLayer
end
