--- UpperBrick
-- @author Created by steven on February 01, 2016.

class("UpperBrick")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local EMPTY_BRICK_COLOR = UpperCommon.WHITE

local Z_SHADOW = 0
local Z_BACKGROUND = 10

local SHADOW_OPACITY = 0.3

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createColorToAction(self, duration, ease, color)
  local background = self.background
  local colorAction = AHColorTo.new(background, duration, ease, color)
  return colorAction
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperBrick:createHighlightAction(duration, ease, color)
  local highlightAction = AHSpawn.new(
    createColorToAction(self, duration, ease, color),
    AHOpacityTo.new(self.shadow, duration * 0.1, nil, 0.0)
  )

  return highlightAction
end

function UpperBrick:createOpacityToAction(duration, ease, opacity)
  local background = self.background
  local opacityAction = AHSpawn.new(
    AHOpacityTo.new(background, duration, ease, opacity),
    AHOpacityTo.new(self.shadow, duration * 0.1, nil, 0.0)
  )

  return opacityAction
end

function UpperBrick:createResetColorAction()
  local solidWhite = EMPTY_BRICK_COLOR
  solidWhite[4] = 1.0

  local resetAction = AHSpawn.new(
    createColorToAction(self, 0.5, nil, EMPTY_BRICK_COLOR),
    AHOpacityTo.new(self.shadow, 1.0, nil, SHADOW_OPACITY)
  )

  return resetAction
end

function UpperBrick:showBackground()
  self.background:setOpacity(1.0)
end

function UpperBrick:createFailAction(shouldGoLeft)
  local xMovementRandomnessFactor = MathUtils.getRandomFloat(0.0, 0.2)

  local xMovementAmount = Screen.getWidthPart(0.12 + xMovementRandomnessFactor)
  local yUpAmount = Screen.getHeightPart(0.32)
  local yDownAmount = -yUpAmount * 6.0
  local destinationRotation = MathUtils.getRandomFloat(40.0, 60.0)

  if (shouldGoLeft) then
    xMovementAmount = -xMovementAmount
    destinationRotation = -destinationRotation
  end

  local failAction = AHSpawn.new(
    AHLocationXBy.new(self, 0.6, AHEaseSineIn.new(), xMovementAmount),
    AHRotationTo.new(self, 1.5, AHEaseSimpleOut.new(2.5), destinationRotation),
    AHSequence.new(
      AHLocationYBy.new(self, 0.3, AHEaseSineOut.new(), yUpAmount),
      AHLocationYBy.new(self, 0.5, AHEaseSineIn.new(), yDownAmount)
    )
  )

  return failAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors ------------------------

function UpperBrick.new(width, height, color)
  local self = Group.new(width, height)
  Mixin.mix(self, UpperBrick)

  color = color or EMPTY_BRICK_COLOR
  local background = FilledRectangle.new(width, height, color)
  self:insertAtCenter(background, Z_BACKGROUND)
  self.background = background

  local shadow = Sprite.newWithName("shadow.png")
  shadow:setOpacity(SHADOW_OPACITY)
  shadow:setEffectiveDims(self:getWidthPart(1.3), self:getHeightPart(1.4))
  self:insertAtCenter(shadow, Z_SHADOW)
  self.shadow = shadow

  return self
end
