--- UpperConfiguration
-- @author Created by steven on February 01, 2016.

class("UpperConfiguration")

require("ExperimentsConfiguration")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

UpperConfiguration.BONUS_SCORE_PER_EXTRA_LIFE = 1000

UpperConfiguration.MAX_SPEED_BONUS_PER_RESPONSE = 100
UpperConfiguration.SPEED_BONUS_DECAY_START = 1.5
UpperConfiguration.SPEED_BONUS_DURATION = 4.0

UpperConfiguration.WORDS_PER_MINUTE_INCREMENT = 30

UpperConfiguration.NUMBER_OF_STAGES_TO_WIN = 3
UpperConfiguration.LIVES_COUNT = 3
UpperConfiguration.TUTORIAL_DELAY = 8

local SCORE_FOR_CORRECT_ANSWER = {
  4000,
  1500,
  1000
}

-- Values used in CoreMS's LifeAnalogySpeedHighlightProducer, update if modified
local INITIAL_WPM_BY_DIFFICULTY = {
  {difficulty = 0.00, startingWPM = 120},
  {difficulty = 0.25, startingWPM = 170},
  {difficulty = 0.4, startingWPM = 206},
  {difficulty = 0.5, startingWPM = 260},
  {difficulty = 0.5625, startingWPM = 300},
  {difficulty = 0.75, startingWPM = 415},
  {difficulty = 1.0, startingWPM = 440}
}

local MINIMUM_COUNTDOWN_DURATION = 13.5
local INITIAL_COUNTDOWN_DURATION = 15.5
local TUTORIAL_COUNTDOWN_DURATION = 21

local EXTRA_LIVES_NEEDED_FOR_RANK_TWO = 1
local EXTRA_LIVES_NEEDED_FOR_RANK_THREE = 2

local MAXIMUM_NUMBER_OF_MISTAKES = UpperConfiguration.LIVES_COUNT - 1
local MINIMUM_CORRECT_ANSWERS_TO_PASS = UpperConfiguration.NUMBER_OF_STAGES_TO_WIN - MAXIMUM_NUMBER_OF_MISTAKES

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getScoreForCorrectAnswerCount(answerCount)
  local score = 0
  for i = 1, answerCount do
    score = score + UpperConfiguration.getCorrectAnswerScore(i)
  end
  return score
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperConfiguration.getCorrectAnswerScore(correctAnswerNumber)
  correctAnswerNumber = MathUtils.clamp(correctAnswerNumber, 1, #SCORE_FOR_CORRECT_ANSWER)
  return SCORE_FOR_CORRECT_ANSWER[correctAnswerNumber]
end

function UpperConfiguration.getInitialCountdownDuration()
  return INITIAL_COUNTDOWN_DURATION
end

function UpperConfiguration.getMinimumCountdownDuration()
  return MINIMUM_COUNTDOWN_DURATION
end

function UpperConfiguration.getTutorialCountdownDuration()
  return TUTORIAL_COUNTDOWN_DURATION
end

function UpperConfiguration.getPassScore()
  local minimumCorrectAnswerPoints = getScoreForCorrectAnswerCount(MINIMUM_CORRECT_ANSWERS_TO_PASS)
  return minimumCorrectAnswerPoints
end

function UpperConfiguration.getRankTwoScore()
  local correctAnswers = MINIMUM_CORRECT_ANSWERS_TO_PASS + EXTRA_LIVES_NEEDED_FOR_RANK_TWO
  local correctAnswerPoints = getScoreForCorrectAnswerCount(correctAnswers)
  local livesBonus = UpperConfiguration.BONUS_SCORE_PER_EXTRA_LIFE * EXTRA_LIVES_NEEDED_FOR_RANK_TWO
  local rankTwoScore = correctAnswerPoints + livesBonus

  return rankTwoScore
end

function UpperConfiguration.getRankThreeScore()
  local correctAnswers = MINIMUM_CORRECT_ANSWERS_TO_PASS + EXTRA_LIVES_NEEDED_FOR_RANK_THREE
  local correctAnswerPoints = getScoreForCorrectAnswerCount(correctAnswers)
  local livesBonus = UpperConfiguration.BONUS_SCORE_PER_EXTRA_LIFE * EXTRA_LIVES_NEEDED_FOR_RANK_THREE
  local rankThreeScore = correctAnswerPoints + livesBonus

  return rankThreeScore
end

function UpperConfiguration.getInitialWordsPerMinuteForDifficulty(difficulty)
  local keyFrames = INITIAL_WPM_BY_DIFFICULTY
  local frameCount = #keyFrames
  local previousFrame = keyFrames[1]

  for frameIndex = 2, frameCount do
    local frame = keyFrames[frameIndex]
    local frameDifficulty = frame.difficulty

    if (difficulty <= frameDifficulty) then
      local previousFrameDifficulty = previousFrame.difficulty

      local previousFrameWPM = previousFrame.startingWPM
      local frameWPM = frame.startingWPM

      local interpolatedWPM = MathUtils.interpolateLinear(difficulty,
                                                          previousFrameDifficulty, frameDifficulty,
                                                          previousFrameWPM, frameWPM)

      local decimalPlaceToRoundTo = -1 -- Round to the nearest 10.
      local roundedWPM = MathUtils.round(interpolatedWPM, decimalPlaceToRoundTo)

      return roundedWPM
    end

    previousFrame = frame
  end

  error("No WPM key frame found for difficulty `" .. tostring(difficulty) .. "` in the initial WPM frames.")
end

function UpperConfiguration.getDifficultyKeyFrames()
  local cutoffThresholds = {
    {endDifficulty = 0.0625, contentDifficulty = "Basic"},
    {endDifficulty = 0.2625, contentDifficulty = "Easy"},
    {endDifficulty = 1.0, contentDifficulty = "Medium"}
  }

  return DifficultyCurves.getHardCutoffKeyframes(cutoffThresholds)
end
