--- UpperDynamicTextArea
-- @author Created by Steven on February 12, 2016

class("UpperDynamicTextArea")

-- ---------------------------------------------------------------------------
-- Configuration
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- Utility
-- ---------------------------------------------------------------------------

-- ---------------------------------------------------------------------------
-- API
-- ---------------------------------------------------------------------------

function UpperDynamicTextArea:scrambleAndHideText()
  local destinationsForGlyphs = {}

  for _, view in ipairs(self.itemViews) do
    if view.layout then
      local rows = view.layout:getRows()
      for _, row in ipairs(rows) do
        for _, glyph in ipairs(row) do
          local randomXOffset = MathUtils.getRandomFloat(-Screen.getSafeWidthPart(0.1), Screen.getSafeWidthPart(0.1))
          local randomYOffset = MathUtils.getRandomFloat(-Screen.getSafeWidthPart(0.2), Screen.getSafeWidthPart(0.2))
          local finalX, finalY = glyph:getLoc()
          local destination = {x = finalX, y = finalY}
          destinationsForGlyphs[glyph] = destination
          local initialX = finalX + randomXOffset
          local initialY = finalY + randomYOffset
          glyph:setLoc(initialX, initialY)
          glyph:setOpacity(0.0)
          glyph:setScl(2.0)
        end
      end
    end
  end

  self.destinationsForGlyphs = destinationsForGlyphs
end

function UpperDynamicTextArea:createAnimateInAction()
  local maxDelay = 0.3
  local duration = 2.0
  local animateInAction = AHSpawn.new()

  for glyph, destination in pairs(self.destinationsForGlyphs) do
    local finalX = destination.x
    local finalY = destination.y
    local delayDuration = math.random() * maxDelay
    animateInAction:addAction(
      AHDelayed.new(
        delayDuration,
        AHLocationTo.new(glyph, duration, AHEaseExponentialOut.new(), finalX, finalY),
        AHOpacityTo.new(glyph, duration / 2, nil, 1.0),
        AHScaleTo.new(glyph, duration, AHEaseExponentialOut.new(), 1.0)
      )
    )
  end

  return animateInAction
end

-- ---------------------------------------------------------------------------
-- Object creation
-- ---------------------------------------------------------------------------

function UpperDynamicTextArea.new(text, width, baseAttributes, attributesForLabels, horizontalAlignment, verticalAlignment, lineSpacing)
  assert(baseAttributes, "Must provide baseAttributes")
  attributesForLabels = attributesForLabels or {}

  local self = DynamicTextArea.new(text, width, baseAttributes, attributesForLabels, horizontalAlignment, verticalAlignment, lineSpacing)
  Mixin.mix(self, UpperDynamicTextArea)

  return self
end

function UpperDynamicTextArea.newSimple(text, width, font, horizontalAlignment, color, verticalAlignment, lineSpacing)
  local baseAttributes = { font = font, color = color}
  return UpperDynamicTextArea.new(text, width, baseAttributes, nil, horizontalAlignment, verticalAlignment, lineSpacing)
end
