--- UpperGameBeginInterstitial
-- @author Created by Steven Mah on February 2, 2016.

class("UpperGameBeginInterstitial")

require("UpperCommon")
require("UpperShadowLabel")
require("UpperShimmeringLabel")
require("UpperAboutGameInterstitial")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local PROCESSING_LABEL_FONT = Font.new(FontFaces.DIN_OT_BOLD, 17)
local WPM_TEXT_FONT = Font.new(FontFaces.DIN_OT_BOLD, 30)
local NUMBER_FONT = Font.new(FontFaces.DIN_OT_BOLD, 184)
local BEGIN_BUTTON_FONT = Font.new(FontFaces.DIN_OT_BOLD, 13)

local TEXT_LABEL_COLOR = ColorUtils.colorHex24ToStd(0x0C4F80)
local BACKGROUND_BEGIN_COLOR = {0.39, 0.85, 0.98}
local BACKGROUND_END_COLOR = ColorUtils.colorHex24ToStd(0x00F9FF)
local WPM_COLOR = {1.0, 1.0, 1.0}

local PULSE_CIRCLE_COLOR = {0.43, 0.93, 0.92}
local PULSE_CIRCLE_BASE_RADIUS = Screen.getHeightPart(0.005)

local BEGIN_BUTTON_INITIAL_RADIUS = Screen.fullHeight
local BEGIN_BUTTON_Y = Screen.getSafeY(0.24)
local BEGIN_BUTTON_SCALED_SIZE = 0.06

local LEARN_MORE_Y = Screen.getSafeY(0.05)

local BACKGROUND_Z = 0
local BUTTON_SHADOW_Z = 10
local PULSE_CIRCLE_Z = 20
local LEARN_MORE_Z = 30
local BEGIN_BUTTON_Z = 40
local TEXT_COLUMN_Z = 50
local WHITE_OVERLAY_Z = 60
local ABOUT_SCREEN_Z = 1000

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function removeElements(self)
  self.textColumn:removeSelf()
  self.background:removeSelf()
  self.beginLabel:removeSelf()
  self.beginButton:removeSelf()
  self.pulseCircle:removeSelf()
  self.buttonShadow:removeSelf()
  self.learnMoreButton:removeSelf()
end

local function createLayout(self, wordsPerMinute)
  local background = RectangleGradient.new(Screen.getWidthPart(1.1), Screen.getHeightPart(1.1), BACKGROUND_BEGIN_COLOR, BACKGROUND_END_COLOR)
  self:insertAtCenter(background, BACKGROUND_Z)
  self.background = background

  local textColumn = Column.new(-Screen.getSafeHeightPart(0.04))
  local processingLabel = Label.new(Localizer.get("ROUND_ONE"), PROCESSING_LABEL_FONT)
  processingLabel:setColor(unpack(TEXT_LABEL_COLOR))
  textColumn:insertAtCenter(processingLabel)
  textColumn:insertSpacer(Screen.getSafeHeightPart(0.08))

  local wpmNumberAsString = tostring(wordsPerMinute)
  local wpmNumberLabel = UpperShadowLabel.new(wpmNumberAsString, NUMBER_FONT, nil, {0.25, 0.25, 0.5, 0.3})
  textColumn:insertAtCenter(wpmNumberLabel)
  textColumn:insertSpacer(Screen.getSafeHeightPart(0.04))

  local wpmTextLabel = UpperShadowLabel.new(UpperCommon.WPM_STRING, WPM_TEXT_FONT, nil, {0.25, 0.25, 0.5, 0.3})
  textColumn:insertAtCenter(wpmTextLabel)

  textColumn:alignChildren()
  textColumn:setLoc(Screen.getSafeX(0.5), Screen.getHeightPart(2.0))
  self:insert(textColumn, TEXT_COLUMN_Z)
  self.textColumn = textColumn

  local beginCircle = FilledCircle.new(BEGIN_BUTTON_INITIAL_RADIUS, WPM_COLOR)
  self.beginCircle = beginCircle

  local onPressBeginGame = function()
    Sound.playSoundEffect("round_transition")
    self.beginButton:setTouchEnabled(false)
    self.learnMoreButton:setTouchEnabled(false)

    local startGame = true
    self.gameCanBegin = startGame
  end

  local beginButton = Button.makeButton(beginCircle, onPressBeginGame)
  beginButton:setLoc(Screen.getSafeX(0.5), BEGIN_BUTTON_Y)
  beginButton:setTouchEnabled(false)
  self:insert(beginButton, BEGIN_BUTTON_Z)
  self.beginButton = beginButton

  local beginLabel = Label.new(Localizer.get("BEGIN"), BEGIN_BUTTON_FONT)
  beginLabel:setColor(unpack(TEXT_LABEL_COLOR))
  beginLabel:setOpacity(0.0)
  beginLabel:setLoc(Screen.getSafeX(0.5), BEGIN_BUTTON_Y)
  self:insert(beginLabel, BEGIN_BUTTON_Z)
  self.beginLabel = beginLabel

  local buttonShadow = RectangleGradient.new(Screen.fullHeight, Screen.getSafeHeightPart(0.4), {0.0, 0.0, 0.2, 0.5}, BACKGROUND_END_COLOR)
  buttonShadow:setAnchorPointTop()
  buttonShadow:setRot(50)
  buttonShadow:setLoc(Screen.getSafeX(0.5), BEGIN_BUTTON_Y)
  self:insert(buttonShadow, BUTTON_SHADOW_Z)
  self.buttonShadow = buttonShadow

  local pulseCircle = FilledCircle.new(PULSE_CIRCLE_BASE_RADIUS, PULSE_CIRCLE_COLOR)
  pulseCircle:setLoc(Screen.getSafeX(0.5), BEGIN_BUTTON_Y)
  pulseCircle:setOpacity(0.5)
  pulseCircle:setBlendMode(MOAIProp2D.BLEND_ADD)
  self:insert(pulseCircle, PULSE_CIRCLE_Z)
  self.pulseCircle = pulseCircle

  local aboutScreen = UpperAboutGameInterstitial.new(self)
  self.aboutScreen = aboutScreen

  local learnMoreLabel = Font.new(FontFaces.DIN_OT_BOLD, 12)
  local learnMoreLabel = UpperShimmeringLabel.new(Localizer.get("LEARN_MORE"), FontFaces.DIN_OT_BOLD, 12)
  learnMoreLabel:setOpacity(0.0)

  local onPressLearnMore = function()
    Sound.playSoundEffect("button_click")
    self.learnMoreButton:setTouchEnabled(false)
    self.beginButton:setTouchEnabled(false)

    self.parent:insertAtCenter(aboutScreen, ABOUT_SCREEN_Z)

    local transitionAction = AHSequence.new(
      AHSpawn.new(
        AHScaleTo.new(self, 2.0, AHEaseExponentialOut.new(), 0.75),
        AHDelayed.new(1.0, function() self:setScl(1.0) end),
        aboutScreen:createFadeInAction()
      )
    )
    transitionAction:start()
  end

  local learnMoreButton = Button.addVirtualButton(learnMoreLabel, learnMoreLabel:getWidthPart(1.5), learnMoreLabel:getHeightPart(3.0), onPressLearnMore)
  learnMoreButton:setTouchEnabled(false)
  learnMoreLabel:setLoc(Screen.getSafeX(0.5), LEARN_MORE_Y)
  self:insert(learnMoreLabel, LEARN_MORE_Z)
  self.learnMoreButton = learnMoreLabel
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperGameBeginInterstitial:preloadFirstAboutScreenIfNeeded()
  if (self.shouldStartRoundWithAboutScreen) then
    local loadWithBackgroundVisible = true
    local aboutScreen = UpperAboutGameInterstitial.new(self, loadWithBackgroundVisible)
    self.parent:insertAtCenter(aboutScreen, ABOUT_SCREEN_Z)
    self.aboutScreen = aboutScreen
  end
end

function UpperGameBeginInterstitial:createFadeInAction()
  if (self.shouldStartRoundWithAboutScreen) then
    self.shouldStartRoundWithAboutScreen = false
    return self.aboutScreen:createFadeInAction()
  end

  local textColumn = self.textColumn
  local beginCircle = self.beginCircle
  local beginLabel = self.beginLabel
  local pulseCircle = self.pulseCircle
  local buttonShadow = self.buttonShadow

  local textDropDownAction = AHLocationYTo.new(textColumn, 1.0, AHEaseExponentialOut.new(), Screen.getSafeY(0.63))
  local buttonShrinkAction = AHSpawn.new(
    AHScaleTo.new(beginCircle, 1.0, AHEaseExponentialOut.new(), BEGIN_BUTTON_SCALED_SIZE),
    AHScaleTo.new(buttonShadow, 1.0, AHEaseExponentialOut.new(), 1.9 * BEGIN_BUTTON_SCALED_SIZE, 1.0),
    AHDelayed.new(0.5, function() self.beginButton:setTouchEnabled(true) end)
  )

  local buttonTextFadeInAction = AHDelayed.new(
    0.6,
    AHOpacityTo.new(beginLabel, 1.0, AHEaseExponentialOut.new(), 1.0)
  )
  local opacityPulseAction = AHSequence.new(
    AHOpacityTo.new(pulseCircle, 0.75, AHEaseSimpleInOut.new(), 0.1),
    AHOpacityTo.new(pulseCircle, 0.75, AHEaseSimpleInOut.new(), 0.25)
  )
  local pulseCircleGrowAction = AHSpawn.new(
    AHScaleTo.new(pulseCircle, 1.5, AHEaseSimpleInOut.new(), 20.0),
    opacityPulseAction
  )
  local pulseCircleShrinkAction = AHSpawn.new(
    AHScaleTo.new(pulseCircle, 1.5, AHEaseSimpleInOut.new(), 18.0),
    opacityPulseAction
  )
  local fullPulseAction = AHSequence.new(pulseCircleGrowAction, pulseCircleShrinkAction)
  local pulseLoop = AHLoop.new(fullPulseAction)
  self.pulseLoop = pulseLoop
  pulseLoop:start()

  local fullRevealAction = AHStaggeredSpawn.new(0.1, textDropDownAction, buttonShrinkAction, buttonTextFadeInAction)

  local learnMoreButton = self.learnMoreButton

  local shimmerLoop = AHLoop.new(
    AHDelayed.new(0.5, learnMoreButton:createShimmerAction(1.0))
  )

  local learnMoreFadeInAction = AHSequence.new(
    AHDelay.new(0.75),
    function() self.learnMoreButton:setTouchEnabled(true) end,
    AHOpacityTo.new(learnMoreButton, 0.5, nil, 1.0),
    shimmerLoop
  )

  local fullRevealAction = AHStaggeredSpawn.new(
    0.1,
    textDropDownAction,
    buttonShrinkAction,
    buttonTextFadeInAction,
    learnMoreFadeInAction
  )

  return AHSequence.new(AHDelay.new(0.15), fullRevealAction)
end

function UpperGameBeginInterstitial:createFadeOutToGameAction()
  local beginLabel = self.beginLabel
  local beginCircle = self.beginCircle
  local textColumn = self.textColumn
  local pulseCircle = self.pulseCircle
  local buttonShadow = self.buttonShadow
  local learnMore = self.learnMoreButton

  local whiteOverlay = FilledRectangle.new(self.background:getWidth(), self.background:getHeight(), WPM_COLOR)
  whiteOverlay:setOpacity(0.0)
  self:insertAtCenter(whiteOverlay, WHITE_OVERLAY_Z)

  self.pulseLoop:stop()

  local buttonAction = AHSequence.new(
    AHScaleBy.new(beginCircle, 0.25, AHEaseExponentialOut.new(), -0.5),
    AHScaleTo.new(beginCircle, 0.4, AHEaseSimpleInOut.new(), 0.15)
  )

  local labelAction = AHSpawn.new(
    AHScaleTo.new(beginLabel, 0.25, nil, 0.25),
    AHOpacityTo.new(beginLabel, 0.25, nil, 0.0)
  )

  local shadowMimicAction = AHSequence.new(
    AHScaleXBy.new(buttonShadow, 0.25, AHEaseExponentialOut.new(), -0.5),
    AHScaleXTo.new(buttonShadow, 0.4, AHEaseSimpleInOut.new(), 0.25)
  )
  local shrinkShadowAction = AHScaleYTo.new(buttonShadow, 0.65, nil, 0.0)
  local fullShadowAction = AHSpawn.new(shadowMimicAction, shrinkShadowAction)

  local textShiftAction = AHLocationYBy.new(textColumn, 1.0, nil, Screen.getSafeHeightPart(0.1))

  local growPulseCircleAction = AHScaleBy.new(pulseCircle, 0.5, nil, 0.5)
  local internalTransformActions = AHSpawn.new(buttonAction, labelAction, textShiftAction, fullShadowAction, growPulseCircleAction)

  local fadeInWhiteOverlayAction = AHOpacityTo.new(whiteOverlay, 0.25, nil, 1.0)
  local scaleX, scaleY = self:scaleForEffectiveDims(Screen.getSafeWidthPart(0.296), UpperCommon.STANDARD_BRICK_HEIGHT)
  local scaleToBrickAction = AHScaleTo.new(self, 1.0, AHEaseExponentialInOut.new(), scaleX, scaleY)

  local fullToBrickAction = AHStaggeredSpawn.new(0.15, internalTransformActions, scaleToBrickAction, fadeInWhiteOverlayAction)

  local linkOpacityAndRemoveElements = function()
    local isRecursive = true
    self:linkOpacityAttrToChildren(isRecursive)
    removeElements(self)
  end

  local fadeOutAction = AHSequence.new(linkOpacityAndRemoveElements, AHOpacityTo.new(self, 0.5, nil, 0.0))
  local fullHideAction = AHStaggeredSpawn.new(0.75, fullToBrickAction, fadeOutAction)
  return fullHideAction
end

function UpperGameBeginInterstitial:resetInterstitial()
  removeElements(self)
  createLayout(self, self.wordsPerMinute)
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function UpperGameBeginInterstitial.new(wordsPerMinute)
  local self = Group.newFullScreen()
  Mixin.mix(self, UpperGameBeginInterstitial)
  self.shouldStartRoundWithAboutScreen = GameSession.hasNeverWon()
  self.wordsPerMinute = wordsPerMinute
  createLayout(self, wordsPerMinute)
  self.gameCanBegin = false

  return self
end
