--- UpperGradientBrick
-- @author Created by steven on March 22, 2016.

class("UpperGradientBrick")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local GLOW_Z = 0
local BACKGROUND_Z = 10

local GLOW_WIDTH_PADDING = 20
local GLOW_HEIGHT_PADDING = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperGradientBrick:createGradientToAction(duration, topColor, bottomColor)
  local colorAction = self.background:newGradientToAction(duration, nil, topColor, bottomColor)

  return colorAction
end

function UpperGradientBrick:createScaleAction(width, glowPadding)
  local width = width or UpperCommon.RESPONSE_BRICK_WIDTH
  local glowPadding = glowPadding or GLOW_WIDTH_PADDING

  local background = self.background
  local glow = self.glow

  local backgroundScale = background:scaleForEffectiveDims(width)
  local glowScale = glow:scaleForEffectiveDims(width + glowPadding)

  background:setEffectiveWidth(0)
  glow:setEffectiveWidth(0)

  local makeVisible = function()
    background:setVisible(true)
    glow:setVisible(true)
  end

  local scaleAction = AHSpawn.new(
    makeVisible,
    AHScaleXTo.new(background, 0.05, AHEaseSimpleOut.new(), backgroundScale),
    AHScaleXTo.new(glow, 0.05, AHEaseSimpleOut.new(), glowScale)
  )

  return scaleAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors ------------------------

function UpperGradientBrick.new(width, height, color)
  local self = Group.new(width, height)
  Mixin.mix(self, UpperGradientBrick)

  local background = RectangleGradient.new(width, height, color, color)
  self.background = background
  background:setVisible(false)
  self:insertAtCenter(self.background, BACKGROUND_Z)

  local glow = Sprite.newWithName("glow.png")
  glow:setEffectiveDims(self:getWidth() + GLOW_WIDTH_PADDING, self:getHeight() + GLOW_HEIGHT_PADDING)
  glow:setOpacity(0.25)
  glow:setBlendMode(MOAIProp2D.BLEND_ADD)
  glow:setVisible(false)
  self.glow = glow
  self:insertAtCenter(self.glow, GLOW_Z)

  return self
end
