--- UpperRationale
-- @author Created by steven on February 01, 2016.

class("UpperRationale")

require("UpperDynamicTextArea")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {0.0, 0.0, 0.0}
local BASE_FONT_COLOR = {0.5, 0.5, 0.5}
local HIGHLIGHT_FONT_COLOR = {1.0, 1.0, 1.0}

local TEXT_BASE_ATTRIBUTES = {
  font = Font.new(FontFaces.DIN_OT_MEDIUM, 20),
  color = BASE_FONT_COLOR
}

local TEXT_LABEL_ATTRIBUTES = {
  strong = {
    color = HIGHLIGHT_FONT_COLOR
  }
}

local CONTENTS_COLUMN_SPACING_Y = Screen.getSafeHeightPart(0.025)
local PASSAGE_WIDTH = Screen.getSafeWidthPart(0.84)
local CONTINUE_LABEL_Y = Screen.getSafeY(0.05)

local PASSAGE_LINE_SPACING = 6

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createLayout(self, round)
  local contentColumn = Column.new(CONTENTS_COLUMN_SPACING_Y, Alignment.LEFT)
  local snippetWithTags = round.snippetWithFormatting
  local question = round.question
  local correctResponse = round.correctResponse

  local passageHeader = Label.new(Localizer.get("PASSAGE"), UpperCommon.RATIONALE_HEADER_FONT)
  passageHeader:setColor(unpack(HIGHLIGHT_FONT_COLOR))
  contentColumn:insert(passageHeader)

  local passageTextArea = UpperDynamicTextArea.new(snippetWithTags,
                                                   PASSAGE_WIDTH,
                                                   TEXT_BASE_ATTRIBUTES,
                                                   TEXT_LABEL_ATTRIBUTES,
                                                   Alignment.LEFT, nil,
                                                   PASSAGE_LINE_SPACING)
  contentColumn:insert(passageTextArea)

  contentColumn:insertSpacer(Screen.getSafeHeightPart(0.02))

  local questionHeader = Label.new(Localizer.get("QUESTION"), UpperCommon.RATIONALE_HEADER_FONT)
  contentColumn:insert(questionHeader)

  local questionTextArea = DynamicTextArea.newSimple(question, PASSAGE_WIDTH, UpperCommon.RATIONALE_QUESTION_FONT, Alignment.LEFT, nil, PASSAGE_LINE_SPACING)
  contentColumn:insert(questionTextArea)

  contentColumn:insertSpacer(Screen.getSafeHeightPart(0.01))

  local correctText = string.format(Localizer.get("CORRECT_RESPONSE"), string.upper(correctResponse))
  local correctLabel = Label.new(correctText, UpperCommon.RATIONALE_RESPONSE_FONT)
  contentColumn:insert(correctLabel)
  contentColumn:insertSpacer(-Screen.getSafeHeightPart(0.03))

  local isRecursive = true
  contentColumn:linkOpacityAttrToChildren(isRecursive)
  contentColumn:setOpacity(0.0)
  contentColumn:alignChildren()
  contentColumn:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.52))
  self:insert(contentColumn)
  self.contentColumn = contentColumn

  local continueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), UpperCommon.RATIONALE_CONTINUE_FONT)
  continueLabel:setLoc(Screen.getSafeX(0.5), CONTINUE_LABEL_Y)
  continueLabel:setOpacity(0.0)
  self:insert(continueLabel)
  self.continueLabel = continueLabel
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperRationale:createFadeInAction()
  local duration = 0.4
  local showAction = AHSpawn.new(
    AHOpacityTo.new(self, duration, nil, 1.0),
    AHOpacityTo.new(self.contentColumn, duration, nil, 1.0),
    AHOpacityTo.new(self.continueLabel, duration, nil, 1.0)
  )

  return showAction
end

function UpperRationale:createFadeOutAction()
  local duration = 0.2
  local hideAndRemoveAction = AHSequence.new(
    AHSpawn.new(
      AHOpacityTo.new(self, duration, nil, 0.0),
      AHOpacityTo.new(self.contentColumn, duration, nil, 0.0),
      AHOpacityTo.new(self.continueLabel, duration, nil, 0.0)
    ),
    AHRemove.new(self)
  )

  return hideAndRemoveAction
end

function UpperRationale:addPlayerResponse(incorrectResponse)
  local contentColumn = self.contentColumn

  local incorrectText = string.format(Localizer.get("YOUR_RESPONSE"), string.upper(incorrectResponse))
  local incorrectLabel = Label.new(incorrectText, UpperCommon.RATIONALE_RESPONSE_FONT)
  incorrectLabel:setColor(unpack(BASE_FONT_COLOR))
  contentColumn:insert(incorrectLabel)

  contentColumn:linkOpacityAttrToChildren(isRecursive)
  contentColumn:alignChildren()
  contentColumn:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.52))
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

function UpperRationale.new(round)
  local self = FilledRectangle.newFullScreen(BACKGROUND_COLOR)
  Mixin.mix(self, UpperRationale)

  createLayout(self, round)

  local canContinue = function() self.canContinue = true end
  Button.addVirtualButton(self, nil, nil, canContinue)

  return self
end

function UpperRationale.newHidden(round)
  local self = UpperRationale.new(round)
  self:setOpacity(0.0)

  return self
end
