--- UpperResponseBrick
-- @author Created by steven on March 10, 2016.

class("UpperResponseBrick")

require("UpperBrick")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local CORRECT_ANSWER_COLOR = {0.39, 0.85, 0.98}
local CORRECT_ANSWER_TEXT_COLOR = {0.31, 0.68, 0.82}
local CORRECT_RESPONSE_UNCHOSEN_TEXT_COLOR = {1.0, 1.0, 1.0}
local CORRECT_RESPONSE_UNCHOSEN_BRICK_COLOR = {0.466, 0.839, 0.960}

local INCORRECT_RESPONSE_HIGHLIGHT_COLOR = UpperCommon.INCORRECT_RED
local INCORRECT_RESPONSE_TEXT_COLOR = {1.0, 1.0, 1.0}
local INCORRECT_RESPONSE_UNCHOSEN_TEXT_COLOR = {0.93, 0.24, 0.59}
local INCORRECT_RESPONSE_UNCHOSEN_BRICK_COLOR = {0.729, 0.062, 0.450, 0.7}

local ANSWER_BRICK_COLOR = UpperCommon.WHITE

local Z_GLOW = 0
local Z_LABEL = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getFormattedText(text)
  -- Responses for true/false questions often appear in the content as "TRUE" and "FALSE",
  -- and it is difficult to fix on Content's side.
  -- Automatically convert "TRUE" and "FALSE" to "true" and "false".
  local lowercaseResponse = string.lower(text)
  local isResponseTrueOrFalse = (lowercaseResponse == Localizer.get("TRUE") or lowercaseResponse == Localizer.get("FALSE"))
  if (isResponseTrueOrFalse) then
    text = lowercaseResponse
  end

  return text
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperResponseBrick:getLabel()
  return self.label
end

function UpperResponseBrick:createIncorrectResponseHighlightAction(isWrongBrick, isTimeOut)
  local textColor = INCORRECT_RESPONSE_UNCHOSEN_TEXT_COLOR
  local brickColor = INCORRECT_RESPONSE_UNCHOSEN_BRICK_COLOR
  local scaleAction = AHDoNothing.new()

  if (isWrongBrick) then
    textColor = INCORRECT_RESPONSE_TEXT_COLOR
    brickColor = INCORRECT_RESPONSE_HIGHLIGHT_COLOR

    if (not isTimeOut) then
      scaleAction = AHSequence.new(
        AHScaleTo.new(self, 0.1, AHEaseSineInOut.new(), 0.6),
        AHScaleTo.new(self, 0.2, AHEaseSineInOut.new(), 1.0)
      )
    end
  end

  local highlightAction = AHSpawn.new(
    AHColorTo.new(self.label, 0.1, nil, textColor),
    self:createHighlightAction(0.1, nil, brickColor),
    scaleAction
  )

  return highlightAction
end

function UpperResponseBrick:createFailAction(shouldGoLeft)
  local xMovementRandomnessFactor = math.random() * 0.2

  local xMovementAmount = Screen.getWidthPart(0.05)
  local yUpAmount = Screen.getHeightPart(0.25)
  local yDownAmount = -Screen.getHeightPart(1.25)
  local destinationRotation = 60.0

  if (shouldGoLeft) then
    xMovementAmount = -xMovementAmount
    destinationRotation = -destinationRotation
  end

  local xShiftAction = AHLocationXBy.new(self, 0.5, nil, xMovementAmount)
  local rotationAction = AHRotationTo.new(self, 1.0, AHEaseSimpleOut.new(2.5), destinationRotation)
  local yShiftAction = AHSequence.new(
    AHLocationYBy.new(self, 0.3, AHEaseSineOut.new(), yUpAmount),
    AHLocationYBy.new(self, 0.5, AHEaseSineIn.new(), yDownAmount)
  )

  local failAction = AHSpawn.new(xShiftAction, rotationAction, yShiftAction)

  return failAction
end

function UpperResponseBrick:createCorrectResponseBrickAction()
  local label = self.label

  local glow = Sprite.newWithName("glow.png")
  glow:setEffectiveDims(self:getWidth() + 20, self:getHeight() + 20)
  glow:setOpacity(0.0)
  glow:setBlendMode(MOAIProp2D.BLEND_ADD)
  self.glow = glow
  self:insertAtCenter(self.glow, GLOW_Z)

  local brickAction = AHSequence.new(
    AHSpawn.new(
      AHScaleTo.new(self, 0.05, nil, 0.6),
      AHColorTo.new(label, 0.1, nil, CORRECT_ANSWER_TEXT_COLOR),
      AHOpacityTo.new(glow, 0.1, nil, 0.25)
    ),
    AHScaleTo.new(self, 0.15, nil, 1.2),
    AHScaleTo.new(self, 0.15, nil, 0.9),
    AHScaleTo.new(self, 0.15, nil, 1.1),
    AHScaleTo.new(self, 0.25, nil, 1.0)
  )

  local outlineAction = AHStaggeredSpawn.new(0.1)

  for actionIndex = 1, 3 do
    local outline = HollowRectangle.new(self:getWidth(), self:getHeight(), 1, 1, {1.0, 1.0, 1.0})
    self:insertAtCenter(outline)

    local expandOutline = AHScaleTo.new(outline, 0.5, AHEaseSimpleOut.new(), 1.75)
    local shrinkOutline = AHScaleTo.new(outline, 0.5, AHEaseSimpleInOut.new(), 1.0)
    local outlineScaleAction = AHSequence.new(expandOutline, shrinkOutline)
    local fadeOutline = AHOpacityTo.new(outline, 0.75, nil, 0.0)

    outlineAction:addAction(outlineScaleAction)
    outlineAction:addAction(fadeOutline)
  end

  local fullResponseAction = AHStaggeredSpawn.new(0.05, brickAction, outlineAction)

  return fullResponseAction
end

function UpperResponseBrick:createCorrectResponseUnchosenBrickAction(thisBrickOnBottom)
  local label = self.label
  local yDestination = -Screen.getHeightPart(1.1)
  local rotationDestination = 70 * MathUtils.randomSign()

  local bounceAction = AHDoNothing.new()

  if thisBrickOnBottom then
    rotationDestination = 60 * MathUtils.randomSign() -- needs to rotate slower for animation timing
    bounceAction = AHSequence.new(
      AHLocationYBy.new(self, 0.1, AHEaseSimpleOut.new(), -Screen.getHeightPart(0.05)),
      AHLocationYBy.new(self, 0.25, AHEaseSimpleInOut.new(), Screen.getHeightPart(0.05))
    )
  end

  local brickAction = AHSpawn.new(
    AHSpawn.new(
      bounceAction,
      AHColorTo.new(label, 0.1, nil, CORRECT_RESPONSE_UNCHOSEN_TEXT_COLOR),
      self:createHighlightAction(0.1, nil, CORRECT_RESPONSE_UNCHOSEN_BRICK_COLOR),
      AHDelayed.new(
        0.1,
        AHRotationBy.new(self, 1.0, nil, rotationDestination),
        AHLocationYBy.new(self, 1.0, AHEaseSineInOut.new(), yDestination)
      )
    )
  )

  return AHSequence.new(brickAction, AHRemove.new(self))
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors ------------------------

function UpperResponseBrick.new(text, isCorrect)
  text = getFormattedText(text)

  local width = UpperCommon.RESPONSE_BRICK_WIDTH
  local height = UpperCommon.RESPONSE_BRICK_HEIGHT
  local color = ANSWER_BRICK_COLOR

  local self = UpperBrick.new(width, height, color)
  Mixin.mix(self, UpperResponseBrick)

  self.background:setOpacity(1.0)

  local label = Label.new(text, UpperCommon.RESPONSE_BRICK_FONT, nil, Alignment.CENTER)
  label:setColor(unpack(UpperCommon.BRICK_TEXT_COLOR))
  self:insertAtCenter(label, Z_LABEL)
  self.label = label
  self.isCorrect = isCorrect

  return self
end
