--- UpperRoundGenerator
-- @author Created by steven on February 5, 2016.

class("UpperRoundGenerator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------



-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function removeMarkup(snippetWithFormatting)
  return snippetWithFormatting:gsub("<.->", "")
end

function buildRoundsFromConcept(concept)
  local identifier = concept.identifier
  assert(type(identifier) == "string", "Missing or malformed identifier for concept: " .. tostring(concept))
  local conceptRounds = concept.up_rounds
  assert(type(conceptRounds) == "table", "Missing or malformed 'up_rounds' array for concept: " .. tostring(identifier))

  local rounds = {}
  for roundNumber, conceptRound in ipairs(conceptRounds) do
    local round = {
      snippet = removeMarkup(conceptRound.snippet_tags),
      snippetWithFormatting = conceptRound.snippet_tags,
      question = conceptRound.question,
      correctResponse = conceptRound.correct_answer,
      incorrectResponse = conceptRound.incorrect_answer,
      correctSide = conceptRound.correct_place,
      questionIdentifier = conceptRound.question_identifier
    }

    table.insert(rounds, round)
  end

  return rounds
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperRoundGenerator:generateNextValue()
  local rounds = self.rounds
  local currentRoundIndex = self.currentRoundIndex
  local round = rounds[currentRoundIndex]

  self.currentRoundIndex = currentRoundIndex + 1

  return round
end

function UpperRoundGenerator:getConceptIdentifier()
  return self.conceptIdentifier
end

function UpperRoundGenerator:getConceptDifficulty()
  return self.conceptDifficulty
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function UpperRoundGenerator.new()
  local concept = GameSession.conceptGenerator:next()

  local rounds = buildRoundsFromConcept(concept)

  local self = Generator.new()
  Mixin.mix(self, UpperRoundGenerator)

  self.rounds = rounds
  self.conceptIdentifier = concept.identifier
  self.conceptDifficulty = concept.difficulty
  self.currentRoundIndex = 1
  return self
end
