--- UpperRoundStartInterstitial
-- @author Created by Steven Mah on February 16, 2016.

class("UpperRoundStartInterstitial")

require("UpperCommon")
require("UpperShadowLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local TEXT_LABEL_COLOR = ColorUtils.colorHex24ToStd(0x0C4F80)
local TEXT_LABEL_COLOR_INCORRECT = UpperCommon.WHITE
local BACKGROUND_COLOR = {0.39, 0.85, 0.98}
local BACKGROUND_COLOR_INCORRECT = {0.0, 0.0, 0.0, 0.0}
local WPM_COLOR = {1.0, 1.0, 1.0}

local STRINGS_FOR_NUMBERS = {
  [2] = Localizer.get("TWO"),
  [3] = Localizer.get("THREE"),
  [4] = Localizer.get("FOUR"),
  [5] = Localizer.get("FIVE")
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getStringForNumber(number)
  local stringForNumber = STRINGS_FOR_NUMBERS[number]
  assert((stringForNumber ~= nil), "No string defined for number.")

  return stringForNumber
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperRoundStartInterstitial:createFadeInAction()
  local background = self.background
  local textColumn = self.textColumn

  local fadeInAnimation = AHSpawn.new(

    AHOpacityTo.new(self.roundLabel, 0.5, nil, 1.0),
    self.wpmNumberLabel:createFadeInAction(0.5, 0.3),
    self.wpmTextLabel:createFadeInAction(0.5, 0.3)
  )

  if (self.answeredCorrectly == true) then
    fadeInAnimation:addAction(background:newOpacityToAction(0.5, nil, 0.9, 0.5))
    fadeInAnimation:addAction(AHScaleTo.new(self, 1.0, AHEaseExponentialOut.new(), 1.0))
  else
    fadeInAnimation:addAction(AHOpacityTo.new(background, 0.5, nil, 0.5))
  end

  return fadeInAnimation
end

function UpperRoundStartInterstitial:createFadeOutToGameAction()
  local background = self.background
  local textColumn = self.textColumn

  local fadeOutAction = AHSpawn.new(
    AHOpacityTo.new(self.roundLabel, 0.5, nil, 0.0),
    self.wpmNumberLabel:createFadeOutAction(0.5),
    self.wpmTextLabel:createFadeOutAction(0.5)
  )

  if self.answeredCorrectly == true then
    fadeOutAction:addAction(AHScaleTo.new(self, 1.0, AHEaseExponentialInOut.new(), 2.0))
    fadeOutAction:addAction(background:newOpacityToAction(0.5, nil, 0.0, 0.0))
  else
    fadeOutAction:addAction(AHOpacityTo.new(background, 0.5, nil, 0.0))
  end

  return AHSequence.new(fadeOutAction, AHRemove.new(self))
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self, roundNumber, wordsPerMinute)
  local background = nil

  if (not self.answeredCorrectly) then
    background = FilledRectangle.new(Screen.getWidthPart(2.0), Screen.getHeightPart(2.0), BACKGROUND_COLOR_INCORRECT)
  else
    local backgroundTopColor = ColorUtils.colorHex24ToStd(0x29CAFC)
    backgroundTopColor[4] = 0.0
    local backgroundBottomColor = ColorUtils.colorHex24ToStd(0x4AFDFF)
    backgroundBottomColor[4] = 0.0

    background = RectangleGradient.new(Screen.getWidthPart(2.0), Screen.getHeightPart(2.0), backgroundTopColor, backgroundBottomColor)
  end

  self:insertAtCenter(background, BACKGROUND_Z)
  self.background = background

  local textColumn = Column.new(-Screen.getSafeHeightPart(0.05))
  local roundAstext = getStringForNumber(roundNumber)
  local roundLabel = Label.new(string.format(Localizer.get("ROUND_TEMPLATE"), roundAstext), UpperCommon.ROUND_NUMBER_FONT)

  if (self.answeredCorrectly) then
    roundLabel:setColor(unpack(TEXT_LABEL_COLOR))
  else
    roundLabel:setColor(unpack(TEXT_LABEL_COLOR_INCORRECT))
  end

  roundLabel:setOpacity(0.0)
  self.roundLabel = roundLabel
  textColumn:insertAtCenter(roundLabel)
  textColumn:insertSpacer(Screen.getSafeHeightPart(0.1))

  local wpmNumberAsString = tostring(wordsPerMinute)
  TEXT_LABEL_COLOR[4] = 0.1
  local wpmNumberLabel = UpperShadowLabel.new(wpmNumberAsString, UpperCommon.NUMBER_FONT, nil, TEXT_LABEL_COLOR)
  wpmNumberLabel:setOpacity(0.0)
  self.wpmNumberLabel = wpmNumberLabel
  textColumn:insertAtCenter(wpmNumberLabel)
  textColumn:insertSpacer(Screen.getSafeHeightPart(0.05))

  local wpmTextLabel = UpperShadowLabel.new(UpperCommon.WPM_STRING, UpperCommon.WPM_TEXT_FONT, nil, TEXT_LABEL_COLOR)
  wpmTextLabel:setOpacity(0.0)
  self.wpmTextLabel = wpmTextLabel
  textColumn:insertAtCenter(wpmTextLabel)
  textColumn:alignChildren()

  textColumn:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.65))
  self:insert(textColumn, TEXT_COLUMN_Z)
  self.textColumn = textColumn
end

-- Constructor ------------------------

function UpperRoundStartInterstitial.new(roundNumber, wordsPerMinute, answeredCorrectly)
  local self = Group.newFullScreen()
  Mixin.mix(self, UpperRoundStartInterstitial)

  self.answeredCorrectly = answeredCorrectly

  createLayout(self, roundNumber, wordsPerMinute)

  if self.answeredCorrectly then
    self:setScl(2.0)
  else
    self:setScl(0.75)
  end

  return self
end
