--- UpperShadowLabel
-- @author Created by steven on March 14, 2016.

class("UpperShadowLabel")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local MAIN_TEXT_COLOR = UpperCommon.WHITE
local SHADOW_TEXT_COLOR = ColorUtils.colorHex24ToStd(0x0C4F80)

local SHADOW_Z = 1
local MAIN_Z = 2

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperShadowLabel:createFadeInAction(duration, shadowOpacity)
  local shadowOpacity = shadowOpacity or 0.5
  local fadeInAction = AHStaggeredSpawn.new(
    duration / 2,
    AHOpacityTo.new(self.mainLabel, duration, nil, 1.0),
    AHOpacityTo.new(self.shadowLabel, duration, nil, shadowOpacity)
  )

  return fadeInAction
end

function UpperShadowLabel:createFadeOutAction(duration)
  local fadeInAction = AHStaggeredSpawn.new(
    0.1,
    AHOpacityTo.new(self.shadowLabel, duration, nil, 0.0),
    AHOpacityTo.new(self.mainLabel, duration - 0.1, nil, 0.0)
  )

  return fadeInAction
end

function UpperShadowLabel:setOpacity(opacity)
  self.mainLabel:setOpacity(opacity)
  self.shadowLabel:setOpacity(opacity)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors ------------------------

function UpperShadowLabel.new(text, font, color, shadowColor)
  local color = color or MAIN_TEXT_COLOR
  local mainLabel = Label.new(text, font)
  mainLabel:setColor(unpack(color))

  local shadowColor = shadowColor or SHADOW_TEXT_COLOR
  local shadowLabel = Label.new(text, font)
  shadowLabel:setColor(unpack(shadowColor))

  local self = Group.new(mainLabel:getWidth(), mainLabel:getHeight())
  Mixin.mix(self, UpperShadowLabel)

  self:insertAtCenter(mainLabel, MAIN_Z)
  self.mainLabel = mainLabel

  shadowLabel:setLoc(mainLabel:getX() + 2, mainLabel:getY() - 2)
  self:insert(shadowLabel, SHADOW_Z)
  self.shadowLabel = shadowLabel

  return self
end
