--- UpperShimmeringLabel
-- @author Created by steven on March 14, 2016.

class("UpperShimmeringLabel")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DEFAULT_COLOR = {0.12, 0.29, 0.51}
local SHIMMER_HIGHLIGHT_COLOR = ColorUtils.colorHex24ToStd(0x0890B5)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function setColor(self, ...)
  for _, glyph in ipairs(self.children) do
    glyph:setColor(...)
  end

  self.keyColor = {...}
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperShimmeringLabel:createShimmerAction(duration)
  local glyphs = self.children
  local durationPerCharacter = duration / #glyphs
  local colorChangeDuration = 0.5
  local colorReturnDelayDuration = 0.5

  local shimmerSequence = AHSpawn.new()

  for glyphIndex, glyph in ipairs(glyphs) do
    local delayDurationUntilColorChange = ((glyphIndex - 1) * durationPerCharacter)
    shimmerSequence:addAction(
      AHSequence.new(
        AHDelay.new(delayDurationUntilColorChange),
        AHColorTo.new(glyph, colorChangeDuration, nil, SHIMMER_HIGHLIGHT_COLOR),
        AHDelay.new(colorReturnDelayDuration),
        AHColorTo.new(glyph, colorChangeDuration, nil, self.keyColor)
      )
    )
  end

  return shimmerSequence
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors ------------------------

function UpperShimmeringLabel.new(text, fontFace, fontSize)
  local self = Label.new(text, PrerenderedFont.new(fontFace, fontSize))
  Mixin.mix(self, UpperShimmeringLabel)

  setColor(self, unpack(DEFAULT_COLOR))

  return self
end
