--- UpperTower
-- @author Created by steven on February 01, 2016.

class("UpperTower")
require("UpperBrick")
require("UpperWordBrick")
require("UpperResponseBrick")
require("UpperCommon")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

UpperTower.OUTRO_PAN_DURATION = 3.0

local DELAY_UNTIL_NEXT_BRICK = 0.5
local BRICK_FADE_AMOUNT = -0.9

local CORRECT_RESPONSE_TEXT_COLOR = {0.13, 0.33, 0.52}

local INCORRECT_RESPONSE_TEXT_COLOR = {1.0, 1.0, 1.0}

local NUMBER_OF_STARTING_BRICKS = 10
local NUMBER_OF_WIN_BRICKS = 32

local QUESTION_AREA_Y = Screen.getSafeY(0.79)
local QUESTION_DROP_DURATION = 1.0

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function fillInStartingBricks(self)
  local shiftDistance = UpperCommon.STANDARD_BRICK_HEIGHT + UpperCommon.BRICK_PADDING

  for brickIndex = 1, NUMBER_OF_STARTING_BRICKS do
    self:addEmptyBrick(Screen.getSafeWidthPart(0.296))
    self:shiftBricksDown(shiftDistance)
  end
  self:currentBricksToOldBricks()
end

local function hideAndRemoveQuestionShadow(self)
  hideAndRemoveShadowAction = AHSequence.new(
    AHOpacityTo.new(self.shadow, 0.1, nil, 0.0),
    AHRemove.new(self.shadow)
  )
  hideAndRemoveShadowAction:start()
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperTower:addWinBricks()
  local height = UpperCommon.RESPONSE_BRICK_HEIGHT
  local width = Screen.getSafeWidthPart(0.1)

  for brickIndex = 1, NUMBER_OF_WIN_BRICKS do
    local brick = UpperGradientBrick.new(width, height, {1.0, 1.0, 1.0})
    self:insertAtCenter(brick)
    table.insert(self.victoryBricks, brick)
  end
end

function UpperTower:repositionVictoryBricks()
  local winBricks = self.victoryBricks
  local topBrick = self.allBricks[#self.allBricks]

  local yLoc = topBrick:getY() + topBrick:getHeightPart(0.5) + (UpperCommon.RESPONSE_BRICK_HEIGHT * 0.5) + UpperCommon.BRICK_PADDING

  for _, brick in ipairs(winBricks) do
    brick:setY(yLoc)
    yLoc = yLoc + UpperCommon.RESPONSE_BRICK_HEIGHT + UpperCommon.BRICK_PADDING
  end
end

function UpperTower:createShiftBricksDownAction(shiftDistance, shiftDuration, shiftEase)
  local shiftDuration = shiftDuration or 0.1

  local fullShiftAction = AHSpawn.new()

  for i, brick in ipairs(self.allBricks) do
    local x = brick:getX()
    local y = brick:getY()

    local shiftDownAction = AHLocationYBy.new(brick, shiftDuration, shiftEase, -shiftDistance)
    fullShiftAction:addAction(shiftDownAction)
  end

  return fullShiftAction
end

function UpperTower:shiftBricksDown(shiftDistance, shiftDuration, shiftEase)
  local shiftAction = self:createShiftBricksDownAction(shiftDistance, shiftDuration, shiftEase)

  shiftAction:start()
end

function UpperTower:createResetBrickColorAction()
  local numberOfBricks = #self.allBricks
  local colorResetAction = AHSpawn.new()

  for brickIndex = numberOfBricks, numberOfBricks - 10, -1 do
    colorResetAction:addAction(
      self.allBricks[brickIndex]:createResetColorAction()
    )
  end

  return colorResetAction
end

function UpperTower:setVisibility(isVisible)
  local allBricks = self.allBricks
  for _, brick in ipairs(allBricks) do
    brick:setVisibleRecursive(isVisible)
  end
end

function UpperTower:setAnswerHasBeenSubmitted(hasAnswerBeenSubmitted, isAnswerCorrect)
  self.isAnswerSubmitted = hasAnswerBeenSubmitted
  self.isSelectedBrickCorrect = isAnswerCorrect
end

function UpperTower:currentBricksToOldBricks()
  for _, oldBrick in ipairs(self.bricksInPreviousRound) do
    oldBrick:removeSelf()
  end

  self.bricksInPreviousRound = self.bricksInCurrentRound
  self.bricksInCurrentRound = {}
  self.allBricks = TableUtils.createConcatenatedArray(self.bricksInPreviousRound, self.bricksInCurrentRound)
end

function UpperTower:startCountdown(isFirstRound)
  local wordBrick = self.wordBrick
  local shiftDistance = UpperCommon.STANDARD_BRICK_HEIGHT + UpperCommon.BRICK_PADDING

  -- needed for special case of first round animation
  if not isFirstRound then
    local didAnswerCorrectly = (self.isAnswerSubmitted and self.isSelectedBrickCorrect)
    if didAnswerCorrectly then
      self:currentBricksToOldBricks()
      self:shiftBricksDown(((UpperCommon.RESPONSE_BRICK_HEIGHT + UpperCommon.STANDARD_BRICK_HEIGHT)/2 + UpperCommon.BRICK_PADDING))
    end

    AHOpacityTo.new(wordBrick.shadow, 0.1, nil, 0.3):start()

    if didAnswerCorrectly then
      wordBrick:resizeWordBrickToFitText(Localizer.get("READY"))
      AHDelayed.new(0.05, AHOpacityTo.new(wordBrick.label, 0.00, nil, 1.0)):start()
    else
      wordBrick:fadeInLabel()
      waitFor(0.25)
    end

    Sound.playSoundEffect("countdown")
    waitFor(1.0)

    local hasAnswerBeenSubmitted = false
    local isAnswerCorrect = nil
    self:setAnswerHasBeenSubmitted(hasAnswerBeenSubmitted, isAnswerCorrect)
  end

  self:addEmptyBrick(wordBrick:getAssignedWidth())
  self:shiftBricksDown(shiftDistance)
  wordBrick:resizeWordBrickToFitText(Localizer.get("SET"))
  AHDelayed.new(0.05, AHOpacityTo.new(wordBrick.label, 0.00, nil, 1.0)):start()
  Sound.playSoundEffect("countdown")

  waitFor(1.0)
  self:addEmptyBrick(wordBrick:getAssignedWidth())
  self:shiftBricksDown(shiftDistance)
  wordBrick:resizeWordBrickToFitText(Localizer.get("GO"))
  AHDelayed.new(0.05, AHOpacityTo.new(wordBrick.label, 0.00, nil, 1.0)):start()
  Sound.playSoundEffect("countdown")

  waitFor(1.0)
end

function UpperTower:addEmptyBrick(width, color, height)
  local height = height or UpperCommon.STANDARD_BRICK_HEIGHT

  local brick = UpperBrick.new(width, height, color)
  local x, y = self.wordBrick:getLoc()
  brick:setLoc(x, y)
  self:insert(brick)

  table.insert(self.bricksInCurrentRound, brick)
  table.insert(self.allBricks, brick)
end

function UpperTower:createOutroColorAction(background)
  local fullColorAction = AHSpawn.new()

  for _, brick in ipairs(self.victoryBricks) do
    local _, bottomY = brick:getWorldLoc()
    local topY = bottomY + brick:getHeight()

    local topColor = background:getOutroColorAtYLocation(topY)
    local bottomColor = background:getOutroColorAtYLocation(bottomY)

    fullColorAction:addAction(
      brick:createGradientToAction(0.5, topColor, bottomColor)
    )
  end

  return fullColorAction
end

function UpperTower:createGameWinPanAction()
  local panDistance = -(self.victoryBricks[NUMBER_OF_WIN_BRICKS]:getY() - Screen.getHeightPart(0.5))
  local panDuration = UpperTower.OUTRO_PAN_DURATION
  local panEase = AHEaseSimpleInOut.new()
  local panUpAction = AHSpawn.new(
    AHLocationYBy.new(self, panDuration, panEase, panDistance),
    AHLocationYBy.new(self.correctBrick, panDuration, panEase, panDistance),
    AHLocationYBy.new(self.wordBrick, panDuration, panEase, panDistance)
  )

  local bricksAction = AHSequence.new()
  for brickIndex = 1, NUMBER_OF_WIN_BRICKS - 2 do
    local brick = self.victoryBricks[brickIndex]
    bricksAction:addAction(brick:createScaleAction())
  end

  local brickProportion = 0.8
  local glowPadding = 18
  for brickIndex = NUMBER_OF_WIN_BRICKS - 1, NUMBER_OF_WIN_BRICKS do
    local brick = self.victoryBricks[brickIndex]
    local destinationWidth = UpperCommon.RESPONSE_BRICK_WIDTH * brickProportion

    bricksAction:addAction(brick:createScaleAction(destinationWidth, glowPadding))

    brickProportion = (brickProportion - 0.4)
    glowPadding = (glowPadding * 2/3)
  end

  return AHSpawn.new(panUpAction, bricksAction, function() Sound.playSoundEffect("outro") end)
end

function UpperTower:getCollapseTowerAction()
  local collapseAction = AHSpawn.new()
  local bricksToRemove = self.bricksInCurrentRound
  local questionTextArea = self.questionTextArea
  local shouldGoLeft = false

  local responseAndCurrentBrick = TableUtils.createConcatenatedArray({self.topBrick}, {self.bottomBrick}, bricksToRemove)

  local xMovementAmount = -Screen.getWidthPart(0.05)
  local yUpAmount = Screen.getHeightPart(0.25)
  local yDownAmount = -Screen.getHeightPart(1.25)
  local destinationRotation = -40.0

  local xShiftAction = AHLocationXBy.new(questionTextArea, 0.5, nil, xMovementAmount)
  local rotationAction = AHRotationTo.new(questionTextArea, 0.5, AHEaseSimpleOut.new(2.5), destinationRotation)
  local yShiftAction = AHSequence.new(
    AHLocationYBy.new(questionTextArea, 0.3, AHEaseSineOut.new(), yUpAmount),
    AHLocationYBy.new(questionTextArea, 0.5, AHEaseSineIn.new(), yDownAmount)
  )

  local questionFailAction = AHSequence.new(
    AHSpawn.new(xShiftAction, rotationAction, yShiftAction),
    AHRemove.new(questionTextArea)
  )
  collapseAction:addAction(questionFailAction)

  for index, brick in ipairs(responseAndCurrentBrick) do
    local shouldGoLeft = (index % 2 == 0)
    local brickAction = brick:createFailAction(shouldGoLeft)
    local removeAfterFall = AHRemove.new(brick)
    collapseAction:addAction(AHSequence.new(brickAction, removeAfterFall))
  end

  return collapseAction
end

function UpperTower:createPanUpAction(isFirstRationale)
  local wordBrick = self.wordBrick
  local bricks = self.bricksInPreviousRound
  local topMostBrick = bricks[#bricks]

  local wordBrickY = topMostBrick:getY() + ((UpperCommon.RESPONSE_BRICK_HEIGHT + UpperCommon.STANDARD_BRICK_HEIGHT) / 2) + UpperCommon.BRICK_PADDING

  if (isFirstRationale) then
    wordBrickY = topMostBrick:getY() + UpperCommon.STANDARD_BRICK_HEIGHT + UpperCommon.BRICK_PADDING
  end

  wordBrick:setY(wordBrickY)
  wait(wordBrick:resizeWordBrickToFitText(Localizer.get("READY")))
  wait(wordBrick:hideLabel())

  local towerPanDistance = Screen.getHeightPart(0.5) - wordBrick:getY()
  local panDuration = 2.2
  local shiftEase = AHEaseSineOut.new()

  local shiftBricksUp = AHSpawn.new(
    AHLocationYBy.new(wordBrick, panDuration, shiftEase, towerPanDistance)
  )

  for _, brick in ipairs(bricks) do
    local shiftUpAction = AHLocationYBy.new(brick, panDuration, shiftEase, towerPanDistance)
    shiftBricksUp:addAction(shiftUpAction)
  end

  return shiftBricksUp
end

function UpperTower:addAnswerBricksAndQuestion(bottomBrick, topBrick, questionTextArea, shadow)
  local wordBrick = self.wordBrick
  self.shadow = shadow
  self:addEmptyBrick(wordBrick:getAssignedWidth(), {1.0, 1.0, 1.0})

  self.topBrick = topBrick
  self.bottomBrick = bottomBrick
  if topBrick.isCorrect then
    self.correctBrick = topBrick
    self.incorrectBrick = bottomBrick
  else
    self.correctBrick = bottomBrick
    self.incorrectBrick = topBrick
  end
  self.questionTextArea = questionTextArea

  local towerShiftDistance = (UpperCommon.RESPONSE_BRICK_HEIGHT + UpperCommon.STANDARD_BRICK_HEIGHT) / 2 + UpperCommon.BRICK_PADDING
  local shiftDuration = QUESTION_DROP_DURATION
  local fadeOutWordBrick = wordBrick:createFadeOutWordBrickAction(1.0)

  local towerShiftAction = AHSequence.new(
    AHSpawn.new(
      AHLocationYBy.new(wordBrick, shiftDuration / 4, AHEaseSineInOut.new(), -Screen.getSafeHeightPart(0.25)),
      self:createShiftBricksDownAction(Screen.getSafeHeightPart(0.25), shiftDuration / 4, AHEaseSineInOut.new())
    ),
    AHSpawn.new(
      AHLocationYBy.new(wordBrick, shiftDuration / 2, AHEaseBounceOut.new(), Screen.getSafeHeightPart(0.25) - towerShiftDistance),
      self:createShiftBricksDownAction(-Screen.getSafeHeightPart(0.25) + towerShiftDistance, shiftDuration / 2, AHEaseBounceOut.new())
    )
  )

  local questionAreaDropDistance = Screen.getHeightPart(0.5) - bottomBrick:getY()

  local questionAndResponseDropAction = AHStaggeredSpawn.new(
    0.01,
    AHLocationYBy.new(bottomBrick, shiftDuration, AHEaseBounceOut.new(), questionAreaDropDistance),
    AHLocationYBy.new(topBrick, shiftDuration, AHEaseBounceOut.new(), questionAreaDropDistance),
    AHLocationYBy.new(questionTextArea, shiftDuration, AHEaseBounceOut.new(), questionAreaDropDistance),
    AHLocationYBy.new(shadow, shiftDuration, AHEaseBounceOut.new(), questionAreaDropDistance),
    AHOpacityTo.new(shadow, 1.0, nil, 0.3)
  )

  local completeDropDownAction = AHSpawn.new(
    fadeOutWordBrick,
    questionAndResponseDropAction,
    AHDelayed.new(0.24, towerShiftAction),
    function() Sound.playSoundEffect("question_block_fall") end
  )
  completeDropDownAction:start()
end

function UpperTower:createCorrectAnswerAction()
  local numberOfBricks = #self.allBricks
  local firstBrick = self.allBricks[numberOfBricks]

  local tintTowerBricksAction = AHSpawn.new()

  for brickIndex = numberOfBricks, numberOfBricks - 10, -1 do
    self.allBricks[brickIndex]:setBlendMode(MOAIProp2D.BLEND_ADD)
    tintTowerBricksAction:addAction(self.allBricks[brickIndex]:createHighlightAction(1.0, AHEaseSimpleInOut.new(), {0.698, 0.862, 0.960, 0.5}))
  end

  local unchosenBrickOnBottom = false
  local shiftTowerAction = AHDoNothing.new()
  if self.topBrick.isCorrect then
    unchosenBrickOnBottom = true
    local deltaY = UpperCommon.BRICK_PADDING + UpperCommon.RESPONSE_BRICK_HEIGHT + Screen.getSafeHeightPart(0.05)
    shiftTowerAction = AHSequence.new(
      self:createShiftBricksDownAction(Screen.getSafeHeightPart(0.05), 0.1, AHEaseSimpleOut.new()),
      self:createShiftBricksDownAction(-deltaY, 0.5, AHEaseSimpleInOut.new())
    )
  end

  local correctResponseAction = AHSpawn.new(
    self.correctBrick:createCorrectResponseBrickAction(),
    AHColorTo.new(self.questionTextArea, 0.5, nil, CORRECT_RESPONSE_TEXT_COLOR),
    tintTowerBricksAction,
    shiftTowerAction,
    self.incorrectBrick:createCorrectResponseUnchosenBrickAction(unchosenBrickOnBottom)
  )

  hideAndRemoveQuestionShadow(self)

  return correctResponseAction
end

function UpperTower:createIncorrectAnswerAction(isTimeOut)
  local incorrectBrickLabel = self.incorrectBrick:getLabel()
  local unchosenBrickLabel = self.correctBrick:getLabel()

  local numberOfBricks = #self.allBricks
  local firstBrick = self.allBricks[numberOfBricks]

  local tintTowerBricksAction = AHSpawn.new()

  for brickIndex = numberOfBricks, numberOfBricks - 10, -1 do
    self.allBricks[brickIndex]:setBlendMode(MOAIProp2D.BLEND_ADD)
    tintTowerBricksAction:addAction(self.allBricks[brickIndex]:createHighlightAction(0.1, nil, {0.737, 0.074, 0.439, 0.5}))
  end

  local questionColorAction = AHColorTo.new(self.questionTextArea, 0.1, nil, INCORRECT_RESPONSE_TEXT_COLOR)

  local incorrectColorAction = AHSpawn.new(
    tintTowerBricksAction,
    AHColorTo.new(self.questionTextArea, 0.1, nil, INCORRECT_RESPONSE_TEXT_COLOR),
    self.incorrectBrick:createIncorrectResponseHighlightAction(true, isTimeOut),
    self.correctBrick:createIncorrectResponseHighlightAction(false)
  )

  local collapseAction = self:getCollapseTowerAction()

  local incorrectResponseAction = AHStaggeredSpawn.new(
    0.25,
    incorrectColorAction,
    collapseAction
  )

  hideAndRemoveQuestionShadow(self)

  return incorrectResponseAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors ------------------------

function UpperTower.new(wordBrick)
  local self = Group.newFullScreen()
  Mixin.mix(self, UpperTower)
  self.wordBrick = wordBrick
  self.isAnswerSubmitted = false
  self.isSelectedBrickCorrect = nil
  self.bricksInPreviousRound = {}
  self.bricksInCurrentRound = {}
  self.victoryBricks = {}
  self.allBricks = {}
  fillInStartingBricks(self)

  self:setVisibility(false)

  return self
end
