--- UpperWordBrick
-- @author Created by steven on March 10, 2016.

class("UpperWordBrick")

require("UpperBrick")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BRICK_WIDTH_KEYFRAMES = {
    [5] = Screen.getSafeWidthPart(0.296),
    [10] = Screen.getSafeWidthPart(0.48),
    [11] = Screen.getSafeWidthPart(0.72)
}

local BRICK_RESIZE_DURATION = 0.1
local WORD_BRICK_COLOR = {1.0, 1.0, 1.0, 0.0}

local Z_SHADOW = 0
local Z_LABEL = 20

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getDiscreetBrickWidth(text)
  local length = string.len(text)
  local width = 0

  if length <= 5 then
    width = BRICK_WIDTH_KEYFRAMES[5]
  elseif length > 5 and length <= 10 then
    width = BRICK_WIDTH_KEYFRAMES[10]
  else
    width = BRICK_WIDTH_KEYFRAMES[11]
  end

  return width
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperWordBrick:getAssignedWidth()
  return self.effectiveWidth
end

function UpperWordBrick:hideLabel()
  self.label:setOpacity(0.0)
end

function UpperWordBrick:fadeInLabel(duration)
  local duration = duration or 0.25
  local fadeInAction = AHOpacityTo.new(self.label, 0.5, nil, 1.0)

  fadeInAction:start()
end

function UpperWordBrick:fadeInWordBrick()
  local duration = 0.0
  local fadeInAction = AHSpawn.new(
    AHOpacityTo.new(self.label, duration, nil, 1.0),
    self:createOpacityToAction(duration, nil, 1.0),
    AHOpacityTo.new(self.shadow, duration * 2.0, nil, 0.3)
  )

  return fadeInAction:start()
end

function UpperWordBrick:createFadeOutWordBrickAction(duration)
  local fadeOutAction = AHSpawn.new(
    AHOpacityTo.new(self.label, duration, nil, 0.0),
    self:createOpacityToAction(duration, nil, 0.0),
    AHOpacityTo.new(self.shadow, duration * 0.1, nil, 0.0)
  )

  return fadeOutAction
end

function UpperWordBrick:resizeWordBrickToResponseBrick()
  self:showBackground()
  local xScale = UpperCommon.RESPONSE_BRICK_WIDTH / self:getWidth()
  local yScale = UpperCommon.RESPONSE_BRICK_HEIGHT / UpperCommon.STANDARD_BRICK_HEIGHT
  local brickBackground = self.background

  brickBackground:setScl(xScale, yScale)
end

function UpperWordBrick:setWordLabelsForRound(labels)
  self.wordIndex = 1
  self.wordsInCurrentRound = labels
end

function UpperWordBrick:showNextWord()
  self:showBackground()
  local wordIndex = self.wordIndex
  local background = self.background
  local shadow = self.shadow
  local oldLabel = self.label
  local newLabel = self.wordsInCurrentRound[wordIndex]
  self.wordIndex = wordIndex + 1
  self.label = newLabel

  local newWidth = getDiscreetBrickWidth(newLabel:getText())

  self.effectiveWidth = newWidth

  local scaleX, scaleY = self.background:scaleForEffectiveDims(newWidth, UpperCommon.STANDARD_BRICK_HEIGHT)
  local shadowScaleX, shadowScaleY = self.shadow:scaleForEffectiveDims(newWidth * 1.3, UpperCommon.STANDARD_BRICK_HEIGHT * 1.4)

  local disposeOldLabel = function()
    waitForFrames(1)
  end

  local fullTransformAction = AHStaggeredSpawn.new(
    0.05,
    AHSpawn.new(
      AHScaleTo.new(background, BRICK_RESIZE_DURATION, nil, scaleX, scaleY),
      AHScaleTo.new(shadow, BRICK_RESIZE_DURATION, nil, shadowScaleX, shadowScaleY)
    ),
    AHSpawn.new(
      AHOpacityTo.new(oldLabel, 0.00, nil, 0.0),
      AHOpacityTo.new(newLabel, 0.00, nil, 1.0),
      disposeOldLabel
    )
  )

  fullTransformAction:start()
end

function UpperWordBrick:resizeWordBrickToFitText(text)
  self:showBackground()
  local background = self.background
  local shadow = self.shadow
  local oldLabel = self.label

  local newWidth = getDiscreetBrickWidth(text)
  local newLabel = Label.new(text, UpperCommon.STANDARD_BRICK_FONT)
  newLabel:setColor(unpack(UpperCommon.BRICK_TEXT_COLOR))
  newLabel:setOpacity(0.0)
  self:insertAtCenter(newLabel, Z_LABEL)
  self.label = newLabel

  self.effectiveWidth = newWidth

  local scaleX, scaleY = self.background:scaleForEffectiveDims(newWidth, UpperCommon.STANDARD_BRICK_HEIGHT)
  local shadowScaleX, shadowScaleY = self.shadow:scaleForEffectiveDims(newWidth * 1.3, UpperCommon.STANDARD_BRICK_HEIGHT * 1.4)

  local disposeOldLabel = function()
    waitForFrames(1)
    oldLabel:removeSelf()
  end

  local fullTransformAction = AHStaggeredSpawn.new(
    0.05,
    AHSpawn.new(
      AHScaleTo.new(background, BRICK_RESIZE_DURATION, nil, scaleX, scaleY),
      AHScaleTo.new(shadow, BRICK_RESIZE_DURATION, nil, shadowScaleX, shadowScaleY)
    ),
    AHSpawn.new(
      AHOpacityTo.new(oldLabel, 0.00, nil, 0.0),
      disposeOldLabel
    )
  )

  fullTransformAction:start()
end

function UpperWordBrick:removeAllLabels()
  local wordLabels = self.wordsInCurrentRound

  for _, word in ipairs(wordLabels) do
    word:removeSelf()
  end

  self.wordsInCurrentRound = {}
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructors ------------------------

function UpperWordBrick.new(text)
  local width = getDiscreetBrickWidth(text)
  local height = UpperCommon.STANDARD_BRICK_HEIGHT
  local color = WORD_BRICK_COLOR

  local self = UpperBrick.new(width, height, color)
  Mixin.mix(self, UpperWordBrick)

  local label = Label.new(text, UpperCommon.STANDARD_BRICK_FONT, nil, Alignment.CENTER)
  label:setColor(unpack(UpperCommon.BRICK_TEXT_COLOR))
  label:setOpacity(0.0)
  self:insertAtCenter(label, Z_LABEL)
  self.label = label
  self.effectiveWidth = width

  self.wordsInCurrentRound = {}
  self.wordIndex = 1

  return self
end
