--- UpperValidator
-- @author Created by Steven on March 24, 2016.
-- ConceptValidator for Upper.

class("UpperValidator")

require("validators.ConceptValidator")
require("validators.TagValidator")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local REQUIRED_KEYS = {
  'up_rounds',
}

local REQUIRED_PER_ROUND_PARAMETERS = {
  "snippet_tags",
  "question",
  "correct_answer",
  "incorrect_answer",
  "correct_place",
  "question_identifier"
}

local VALID_ANSWER_SIDES = {
  "left", "right", "random"
}

local SNIPPET_OPTIONAL_PAIRED_TAGS = {
  "strong"
}

local NUMBER_OF_ROUNDS = 3
local MAXIMUM_SNIPPET_WORD_LENGTH = 16
local MAXIMUM_ANSWER_WORD_LENGTH = 14
local PUNCTUATION_REGEX = "[.,;]"
local MARKUP_REGEX = "<.->"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------


local function validateMaximumWordLength(self, text, fieldName, maximumLength)
  local words = StringUtils.split(text, " ")
  for _, word in ipairs(words) do
    local trimmedWord = word:gsub(PUNCTUATION_REGEX, "")
    trimmedWord = trimmedWord:gsub(MARKUP_REGEX, "")
    self:expectLessOrEqual(#trimmedWord, maximumLength, string.format("length of word in %s (%s)", fieldName, word))
  end
end

local function validateRoundResponsesAreDifferent(self, round)
  local correctAnswer = round.correct_answer
  local incorrectAnswer = round.incorrect_answer
  self:expectNotEqual(correctAnswer, incorrectAnswer, "correct_answer must be different from incorrect_answer")
end

local function validateRounds(self, conceptRounds)
  self:expectGreaterOrEqual(#conceptRounds, NUMBER_OF_ROUNDS, 'number of rounds')

  for roundIndex, round in ipairs(conceptRounds) do
    self:validateRequiredKeys(round, REQUIRED_PER_ROUND_PARAMETERS)

    validateMaximumWordLength(self, round.snippet_tags, "snippet_tags", MAXIMUM_SNIPPET_WORD_LENGTH)
    validateMaximumWordLength(self, round.correct_answer, "correct_answer", MAXIMUM_ANSWER_WORD_LENGTH)
    validateMaximumWordLength(self, round.incorrect_answer, "incorrect_answer", MAXIMUM_ANSWER_WORD_LENGTH)

    validateRoundResponsesAreDifferent(self, round)

    self:expectIsContained(round.correct_place, VALID_ANSWER_SIDES, string.format("correct_place (round %s)", roundIndex))

    self:validateTags(round.snippet_tags, nil, nil, nil, SNIPPET_OPTIONAL_PAIRED_TAGS, nil)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function UpperValidator:validateConcept(concept)
  validateRounds(self, concept.up_rounds)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function UpperValidator.new()
  local self = ConceptValidator.new()
  Mixin.mix(self, UpperValidator)

  TagValidator.make(self)

  return self
end
