--- VendettaAnswerBeam
-- @author Created by Joe Poka on September 19, 2016.

class("VendettaAnswerBeam")

require("VendettaTriangleStripLine")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaAnswerBeam:getAppearAction(duration, ease)
  -- for beams, we need to "do more" in the interval t = 0.0 to t = 1.0
  -- specifically, we need to let the line appear, translate, and disappear
  -- recall for lines (like VendettaAnswerLine) we only need the line to appear
  -- fully (and not "disappear off the end")
  local maximumCount = #self.coordinates + 4 * self.visibleSegmentCount

  local coordinatesCount = #self.coordinates
  local updateFunction = function(t)
    -- drawIndex is an index into the collection of all coordinates
    self.endIndex = math.min(1 + math.floor(t * maximumCount), maximumCount)

    -- add coordinates in groups of four => always draw triangles in pairs
    -- this ensures that the line tip is a flat line and not a pointy triangle
    -- from an unpaired triangle
    self.endIndex = 4 * math.floor(self.endIndex / 4)

    self.endDrawIndex = math.min(self.endIndex, coordinatesCount)

    local updatedStartIndex = self.endIndex - (self.visibleSegmentCount * 4 - 1)
    self.startDrawIndex = MathUtils.clamp(updatedStartIndex, 1, coordinatesCount)
  end

  self.deck:setDrawCallback(function()
    local coordinates = TableUtils.subarray(self.coordinates, self.startDrawIndex, self.endDrawIndex)
    MOAIDraw.fillStrip(coordinates)
  end)

  local lineAnimation = AHCustomAction.new(duration, ease, updateFunction)
  return lineAnimation
end

function VendettaAnswerBeam:hide(duration)
  local opacityTo = AHOpacityTo.new(self, duration, nil, 0.0)
  opacityTo:start()
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function VendettaAnswerBeam.new(points, width, color, visibleSegmentCount)
  local self = VendettaTriangleStripLine.new(points, width, color)
  Mixin.mix(self, VendettaAnswerBeam)

  self.visibleSegmentCount = visibleSegmentCount

  self:setColor(unpack(color))
  self:setBlendMode(MOAIProp2D.BLEND_ADD)

  return self
end
