--- VendettaAnswerLine
-- @author Created by Joe Poka on September 19, 2016.

class("VendettaAnswerLine")

require("VendettaTriangleStripLine")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaAnswerLine:setTipOrb(orb)
  self.orb = orb
end

function VendettaAnswerLine:getAppearAction(duration, ease)
  local coordinatesCount = #self.coordinates
  local updateFunction = function(t)
    -- drawIndex is an index into the collection of all coordinates
    self.endDrawIndex = math.min(1 + math.floor(t * coordinatesCount), coordinatesCount)

    -- add coordinates in groups of four => always draw triangles in pairs
    -- this ensures that the line tip is a flat line and not a pointy triangle
    -- from an unpaired triangle
    self.endDrawIndex = 4 * math.floor(self.endDrawIndex / 4)

    if (self.orb and self.endDrawIndex >= 4) then
      local rightVertex = {
        self.coordinates[self.endDrawIndex - 1],
        self.coordinates[self.endDrawIndex]
      }

      local leftVertex = {
        self.coordinates[self.endDrawIndex - 3],
        self.coordinates[self.endDrawIndex - 2]
      }

      local x = math.min(leftVertex[1], rightVertex[1]) + 0.5 * math.abs(rightVertex[1] - leftVertex[1])
      local y = math.min(leftVertex[2], rightVertex[2]) + 0.5 * math.abs(rightVertex[2] - leftVertex[2])
      self.orb:setLoc(x, y)

      if (t >= 1.0) then
        self.orb:setY(self.orb:getY() - self.orb:getHeightPart(0.5))
      end

      local scale = math.min(t, 1.0)
      self.orb:setScl(scale)
    end
  end

  self.deck:setDrawCallback(function()
    local coordinates = TableUtils.subarray(self.coordinates, 1, self.endDrawIndex)
    MOAIDraw.fillStrip(coordinates)
  end)

  local reducedDuration = 0.48 * duration
  local lineAnimation = AHCustomAction.new(duration, ease, updateFunction)
  local appearAction = AHStaggeredSpawn.new(
    reducedDuration,
    lineAnimation,
    AHColorTo.new(self, reducedDuration, ease, self.destinationColor)
  )

  return appearAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function VendettaAnswerLine.new(points, width, color)
  local self = VendettaTriangleStripLine.new(points, width, color)
  Mixin.mix(self, VendettaAnswerLine)

  return self
end
