--- VendettaConfiguration
-- @author Created by steven on July 07, 2016.

class("VendettaConfiguration")

require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

VendettaConfiguration.CONCEPTS_COUNT_PER_GAME = 16

VendettaConfiguration.POINTS_PER_CORRECT_ANSWER = 600

VendettaConfiguration.SYNONYMS_TO_PROMPT_RATIO = 3

VendettaConfiguration.GAME_DURATION = 90
VendettaConfiguration.MINIMUM_TIME_NEEDED_FOR_ROUND = 5
VendettaConfiguration.MINIMUM_TIME_NEEDED_FOR_NEW_WORD = 1
VendettaConfiguration.INCORRECT_RESPONSE_TIME_PENALTY = 5
VendettaConfiguration.DIFFICULTY_BUCKETS = {"Easy", "Medium", "Hard"}
VendettaConfiguration.POINTS_PER_FULL_SPEED_BONUS = 25

VendettaConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 3
VendettaConfiguration.SPEED_BONUS_DECAY_START = 0

VendettaConfiguration.TUTORIAL_DELAY = 8

local MATCHES_REQUIRED_RANK_TWO = 18
local MATCHES_REQUIRED_RANK_THREE = 22

local BUCKET_COUNT_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.0, {initial = 2, max = 3}),
  Keyframe.new(0.2, {initial = 2, max = 4}),
  Keyframe.new(0.4, {initial = 3, max = 4})
)

local BUCKET_COUNT_TO_PROMPT_COUNT_MODIFIER_WEIGHTS = {
  [1] = {values = {0}, weights = {1}},
  [2] = {values = {0, 1}, weights = {1, 1}},
  [3] = {values = {-1, 0, 1, 2}, weights = {3, 3, 3, 1}},
  [4] = {values = {-3, -2, -1, 0, 1}, weights = {1, 4, 6, 6, 1}}
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function validateBucketCountsToExtraPromptWeights()
  for bucketCount, valueWieghtPairs in pairs(BUCKET_COUNT_TO_PROMPT_COUNT_MODIFIER_WEIGHTS) do
    local values = valueWieghtPairs.values
    local weights = valueWieghtPairs.weights
    assert(#values == #weights, "There must be an equal number of values and weights.")
    for _, value in ipairs(values) do
      assert(math.abs(value) <= bucketCount, "Extra prompt modifier should not exceed the number of buckets.")
    end
  end
end
validateBucketCountsToExtraPromptWeights()

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaConfiguration.getInitialBucketCount()
  local difficulty = GameSession.getDifficulty()
  local bucketValues = BUCKET_COUNT_THRESHOLDS:getFrameAt(difficulty)
  return bucketValues.initial
end

function VendettaConfiguration.getMaxBucketCount()
  local difficulty = GameSession.getDifficulty()
  local bucketValues = BUCKET_COUNT_THRESHOLDS:getFrameAt(difficulty)
  return bucketValues.max
end

function VendettaConfiguration.getRoundPromptCountModifierForBucketCount(bucketCount)
  bucketCount = MathUtils.clamp(bucketCount, 1, #BUCKET_COUNT_TO_PROMPT_COUNT_MODIFIER_WEIGHTS)
  local extraPromptValuesAndWeights = BUCKET_COUNT_TO_PROMPT_COUNT_MODIFIER_WEIGHTS[bucketCount]

  return TableUtils.selectRandomValueWithWeights(extraPromptValuesAndWeights.values, extraPromptValuesAndWeights.weights)
end

function VendettaConfiguration.getRankTwoScore()
  local correctResponsePoints = VendettaConfiguration.POINTS_PER_CORRECT_ANSWER * MATCHES_REQUIRED_RANK_TWO

  return correctResponsePoints
end

function VendettaConfiguration.getRankThreeScore()
  local correctResponsePoints = VendettaConfiguration.POINTS_PER_CORRECT_ANSWER * MATCHES_REQUIRED_RANK_THREE

  return correctResponsePoints
end

function VendettaConfiguration.getDifficultyKeyFrames()
  local keyFrames = {
    {
      key = 0.0,
      valueProbabilities = {
        Easy = 1.0,
      }
    },
    {
      key = 0.075,
      valueProbabilities = {
        Easy = 0.9,
        Medium = 0.1,
      }
    },
    {
      key = 0.15,
      valueProbabilities = {
        Easy = 0.8,
        Medium = 0.2,
      }
    },
    {
      key = 0.225,
      valueProbabilities = {
        Easy = 0.5,
        Medium = 0.4,
        Hard = 0.1
      }
    },
    {
      key = 0.3,
      valueProbabilities = {
        Easy = 0.35,
        Medium = 0.5,
        Hard = 0.15
      }
    },
    {
      key = 0.375,
      valueProbabilities = {
        Easy = 0.3,
        Medium = 0.4,
        Hard = 0.3
      }
    },
    {
      key = 0.45,
      valueProbabilities = {
        Easy = 0.15,
        Medium = 0.5,
        Hard = 0.35
      }
    },
    {
      key = 0.525,
      valueProbabilities = {
        Easy = 0.1,
        Medium = 0.4,
        Hard = 0.5
      }
    },
    {
      key = 0.6,
      valueProbabilities = {
        Easy = 0.05,
        Medium = 0.3,
        Hard = 0.65
      }
    },
    {
      key = 0.675,
      valueProbabilities = {
        Medium = 0.22,
        Hard = 0.78
      }
    },
    {
      key = 0.75,
      valueProbabilities = {
        Medium = 0.05,
        Hard = 0.95
      }
    },
    {
      key = 1.0,
      valueProbabilities = {
        Hard = 1.0
      }
    }
  }

  return keyFrames
end
