--- VendettaPrompt
-- @author Created by steven on July 19, 2016.

class("VendettaPrompt")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 22)
local MAX_WIDTH = Screen.getWidthPart(0.237)

local REVIEW_COLOR = {0.65, 0.45, 0.58}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function hasSynonym(self, synonym)
  for _, child in ipairs(self.synonymColumn.children) do
    if (child:getWord() == synonym:getWord()) then
      return true
    end
  end

  return false
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaPrompt:getWord()
  return self.promptWord
end

function VendettaPrompt:takeSynonymIfUnique(synonym)
  synonym:resetGlyphs()
  synonym:setScl(0.4)

  local color = REVIEW_COLOR
  color[4] = 0.0
  synonym:setColor(unpack(color))

  if (hasSynonym(self, synonym)) then
    synonym:removeSelf()
  else
    self.synonymColumn:takeChild(synonym)
    self.synonymColumn:alignChildren()
  end
end

function VendettaPrompt:getSynonymColumnHeight()
  return self.synonymColumn:getHeight()
end

function VendettaPrompt:createRevealSynonymsAction(duration, ease)
  self.synonymColumn:linkOpacityAttrToChildren()

  return AHOpacityTo.new(self.synonymColumn, duration, ease, 0.75)
end

function VendettaPrompt:createHideAction(duration, ease)
  local hideAction = AHSpawn.new(
    AHOpacityTo.new(self, duration, ease, 0.0),
    AHOpacityTo.new(self.synonymColumn, duration, ease, 0.0)
  )

  return hideAction
end

function VendettaPrompt:getIdentifier()
  return self.identifier
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local synonymColumn = Column.new()
  synonymColumn:setAnchorPointTop()
  synonymColumn:setLoc(self:getWidthPart(0.5), -self:getHeightPart(0.5))
  synonymColumn:setOpacity(0.0)
  self:insert(synonymColumn)
  self.synonymColumn = synonymColumn

  self:setColor(unpack(VendettaCommon.DEFAULT_TEXT_COLOR))
  if (self:getWidth() >= MAX_WIDTH) then
    self:setEffectiveDims(MAX_WIDTH)
  end
end

-- Constructor ------------------------

function VendettaPrompt.new(concept)
  local self = Label.new(concept.prompt, FONT)
  Mixin.mix(self, VendettaPrompt)

  createLayout(self)

  self.promptWord = concept.prompt
  self.identifier = concept.identifier

  return self
end
