--- VendettaRationale
-- @author Created by steven on July 12, 2016.

class("VendettaRationale")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_VISIBLE_OPACITY = 0.66
local FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 20)

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function formatRationaleText(self)
  local promptWord = StringUtils.capitalizeFirstLetter(self.synonym:getWord())
  local rationaleText = self.synonym:getRationale()

  local text = promptWord .. " means " .. rationaleText .. "."

  return text
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaRationale:createRevealAction(duration, ease)
  duration = duration or 0.5
  local revealAction = AHSpawn.new(
    AHOpacityTo.new(self.textArea, duration, ease, 1.0),
    AHOpacityTo.new(self.tapLabel, duration, ease, BACKGROUND_VISIBLE_OPACITY)
  )

  return revealAction
end

function VendettaRationale:createHideAction(duration, ease)
  duration = duration or 0.5
  local hideAction = AHSpawn.new(
    AHOpacityTo.new(self.textArea, duration, ease, 0.0),
    AHOpacityTo.new(self.tapLabel, duration, ease, 0.0)
  )

  return hideAction
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local rationaleText = formatRationaleText(self)

  local definitionTextArea = DynamicTextArea.newSimple(rationaleText, Screen.getWidthPart(0.75), FONT, Alignment.CENTER)
  local definitionTextAreaY = math.max(Screen.getSafeY(0.175), Screen.getHeightPart(0.195))
  definitionTextArea:setLoc(Screen.getWidthPart(0.5), definitionTextAreaY)

  -- Highlight the word
  definitionTextArea:clearAttrLinkToChildren(MOAIColor.INHERIT_COLOR, true)
  definitionTextArea:linkOpacityAttrToChildren()
  local views = definitionTextArea:getItemViews()
  local highlightColor = {0.68, 0.35, 0.52}
  views[1]:setColor(unpack(highlightColor))

  self:insert(definitionTextArea)
  self.textArea = definitionTextArea

  local tapLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  tapLabel:setLoc(Screen.getWidthPart(0.5), Screen.getSafeY(0.075))
  self:insert(tapLabel)
  self.tapLabel = tapLabel
end

-- Constructor ------------------------

function VendettaRationale.new(synonym)
  local self = Group.newFullScreen()
  Mixin.mix(self, VendettaRationale)

  self.synonym = synonym
  createLayout(self)

  return self
end
