--- VendettaRoundTransitionLine
-- @author Created by Joe Poka on September 19, 2016.

class("VendettaRoundTransitionLine")

require("VendettaTriangleStripLine")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaRoundTransitionLine:setCircuitLayer(circuitLayer)
  self.circuitLayer = circuitLayer
  self.initalY = self.circuitLayer:getY()
end

function VendettaRoundTransitionLine:getAppearAction(duration, ease)
  self.verticalOffset = 0.0

  local maximumCount = #self.coordinates + 4 * self.visibleSegmentCount

  local coordinatesCount = #self.coordinates
  local updateFunction = function(t)
    -- drawIndex is an index into the collection of all coordinates
    self.endIndex = math.min(1 + math.floor(t * maximumCount), maximumCount)

    -- add coordinates in groups of four => always draw triangles in pairs
    -- this ensures that the line tip is a flat line and not a pointy triangle
    -- from an unpaired triangle
    self.endIndex = 4 * math.floor(self.endIndex / 4)

    self.endDrawIndex = math.min(self.endIndex, coordinatesCount)

    local updatedStartIndex = self.endIndex - (self.visibleSegmentCount * 4 - 1)
    self.startDrawIndex = MathUtils.clamp(updatedStartIndex, 1, coordinatesCount)

    self.verticalOffset = self.circuitLayer:getY() - self.initalY
  end

  self.deck:setDrawCallback(function()
    local coordinates = TableUtils.subarray(self.coordinates, self.startDrawIndex, self.endDrawIndex)

    for i = 2, #coordinates, 2 do
      coordinates[i] = coordinates[i] + self.verticalOffset
    end

    MOAIDraw.fillStrip(coordinates)
  end)

  local lineAnimation = AHCustomAction.new(duration, ease, updateFunction)
  return lineAnimation
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function VendettaRoundTransitionLine.new(points, width, color)
  local self = VendettaTriangleStripLine.new(points, width, color)
  Mixin.mix(self, VendettaRoundTransitionLine)

  self.visibleSegmentCount = 100
  self:setColor(unpack(color))

  return self
end
