--- VendettaSynonym
-- @author Created by steven on July 11, 2016.

class("VendettaSynonym")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local WIDTH = VendettaCommon.CARD_WIDTH

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaSynonym:getWord()
  local synonymData = self.synonymData

  return synonymData.word
end

function VendettaSynonym:getRationale()
  local synonymData = self.synonymData

  return synonymData.rationale
end

function VendettaSynonym:isWordSynonym(promptWord)
  local synonymData = self.synonymData

  return (synonymData.prompt == promptWord)
end

function VendettaSynonym:getPromptIdentifier()
  return self.synonymData.promptIdentifier
end

function VendettaSynonym:getAppearAction(duration, ease)
  local spreadAction = AHSpawn.new()

  for glyph, x in pairs(self.destinationsForGlyphs) do
    spreadAction:addAction(
      AHLocationXTo.new(glyph, duration, ease, x)
    )
  end

  return spreadAction
end

function VendettaSynonym:getHideAction(duration, ease)
  local collectAction = AHStaggeredSpawn.new(0.01)
  local centerX = self:getWidthPart(0.5)
  local deltaY = self:getHeightPart(-1.0)

  for glyph, x in pairs(self.destinationsForGlyphs) do
    local width, _ = glyph:getDims()
    local deltaX = centerX - x - (width / 2)
    collectAction:addAction(
      AHLocationBy.new(glyph, duration, ease, deltaX, deltaY)
    )
  end

  return collectAction
end

function VendettaSynonym:resetGlyphs()
  for glyph, x in pairs(self.destinationsForGlyphs) do
    glyph:setLoc(x, 0)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function VendettaSynonym.new(synonymData, shouldStackLetters)
  local self = Label.new(synonymData.word, VendettaCommon.SYNONYM_FONT)
  Mixin.mix(self, VendettaSynonym)

  self:setColor(unpack(VendettaCommon.DEFAULT_TEXT_COLOR))

  if (self:getWidth() > WIDTH) then
    self:setEffectiveDims(WIDTH)
  end

  local rows = self.layout:getRows()
  local centerX = self:getWidthPart(0.5)
  local destinationsForGlyphs = {}

  for _, row in ipairs(rows) do
    for _, glyph in ipairs(row) do
      local finalX, y = glyph:getLoc()
      destinationsForGlyphs[glyph] = finalX

      if (shouldStackLetters) then
        local width, _ = glyph:getDims()
        glyph:setLoc(centerX - (width / 2), y)
      end
    end
  end

  self.destinationsForGlyphs = destinationsForGlyphs
  self.synonymData = synonymData

  return self
end
