--- VendettaSynonymGenerator
-- @author Created by Jay Austria on September 21, 2023.

class("VendettaSynonymGenerator")

require("data.generators.Generator")
require("ExperimentsConfiguration")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function nextQueuedWordToResurfaceWasJustSeen(self)
  if (self.mostRecentlyGeneratedWord == nil) then
    return false
  end

  if (TableUtils.isEmpty(self.synonymsDataPoolToResurface)) then
    return false
  end

  local nextQueuedSynonymToResurface = self.synonymsDataPoolToResurface[#self.synonymsDataPoolToResurface]

  return (nextQueuedSynonymToResurface.word == self.mostRecentlyGeneratedWord)
end

local function createWordToSynonymDataMap(self, synonymsData)
  for _, data in ipairs(synonymsData) do
    self.wordToSynonymData[data.word] = data
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaSynonymGenerator:reset()
  -- Do nothing
end

function VendettaSynonymGenerator:generateNextValue()
  local thresholdReachedForResurfacingInRound = (#self.synonymsDataPoolToResurface >= self.totalWordsRemaining)

  local synonymsDataPoolToUse = self.synonymsDataPoolUnseen
  if (thresholdReachedForResurfacingInRound and (not nextQueuedWordToResurfaceWasJustSeen(self))) then
    synonymsDataPoolToUse = self.synonymsDataPoolToResurface
  end

  local nextValue = TableUtils.arrayPopBack(synonymsDataPoolToUse)

  if (nextValue ~= nil) then
    self.mostRecentlyGeneratedWord = nextValue.word
    self.totalWordsRemaining = self.totalWordsRemaining - 1
  end

  return nextValue
end

function VendettaSynonymGenerator:notifyMistake(word)
  local synonymData = self.wordToSynonymData[word]
  assert(synonymData ~= nil, string.format("Synonym data was not fount for word '%s'", word))

  if (not TableUtils.contains(self.synonymsDataPoolToResurface, synonymData)) then
    TableUtils.arrayPushFront(self.synonymsDataPoolToResurface, synonymData)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function VendettaSynonymGenerator.new(synonymsData)
  local self = Generator.new(Generator.EXHAUSTION_HANDLER_GIVE_UP)
  Mixin.mix(self, VendettaSynonymGenerator)

  self.synonymsDataPoolUnseen = TableUtils.deepCopy(synonymsData)
  self.synonymsDataPoolToResurface = {}
  self.totalWordsRemaining = #synonymsData
  self.mostRecentlyGeneratedWord = nil

  self.wordToSynonymData = {}
  createWordToSynonymDataMap(self, synonymsData)

  return self
end
