--- VendettaTriangleStripLine
-- @author Created by Joe Poka on September 16, 2016.

class("VendettaTriangleStripLine")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function VendettaTriangleStripLine:changeColorTo(color, opacity, duration, ease)
  color = color or self.destinationColor
  duration = duration or 0.5
  local adjustColor = ColorUtils.colorWithOpacity(color, opacity)

  self.animationInterrupt:insert(AHColorTo.new(self, duration, ease, adjustColor))
end

function VendettaTriangleStripLine:getAppearAction(duration, ease)
  -- subclasses override this function
  return nil
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function VendettaTriangleStripLine.new(points, width, color)
  local defaultColor = {1.0, 1.0, 1.0, 1.0}
  local self = Group.newCustomDraw(Screen.width, Screen.height, defaultColor)
  Mixin.mix(self, VendettaTriangleStripLine)

  self.destinationColor = color
  self.animationInterrupt = ActionInterrupt.new()

  local angles = {}
  for i = 1, #points - 1 do
    local currentPoint = points[i]
    local nextPoint = points[i + 1]
    local deltaX = currentPoint.x - nextPoint.x
    local deltaY = currentPoint.y - nextPoint.y
    local angle1 = ShapeUtils.getVectorDirectionAngle(deltaX, deltaY)
    local angle2 = 0.5 * math.pi - angle1
    table.insert(angles, angle2)
  end
  table.insert(angles, 0.0)

  self.halfWidth = width * 0.5
  local squareVertices = {}
  for i = 1, #points do
    local currentPoint = points[i]
    local angle = angles[i]
    local xAdjust = self.halfWidth * math.cos(angle)
    local yAdjust = self.halfWidth * math.sin(angle)

    local currentLeft = {
      x = currentPoint.x - xAdjust,
      y = currentPoint.y + yAdjust
    }

    local currentRight = {
      x = currentPoint.x + xAdjust,
      y = currentPoint.y - yAdjust
    }

    table.insert(squareVertices, currentLeft)
    table.insert(squareVertices, currentRight)
  end

  self.coordinates = ShapeUtils.flattenCoordPairsNew(squareVertices)
  self.startDrawIndex = 1
  self.endDrawIndex = 1

  return self
end
