--- WitAnalytics
-- @author Created by andrewcousins on October 04, 2023.

class("WitAnalytics")

require("Analytics")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local EVENT_WIT_ANALYTICS = "EloquenceGameEnded"

local PROPERTY_NUMBER_OF_ROUNDS_TIMEDOUT = "rounds_timed_out"
local PROPERTY_NUMBER_OF_ROUNDS_ENDED_EARLY = "rounds_ended_early"
local PROPERTY_WORDS_IN_TIMED_OUT_ROUNDS = "words_in_timed_out_rounds"
local PROPERTY_WORDS_IN_ENDED_EARLY_ROUNDS = "words_in_ended_early_rounds"

local PROPERTY_DID_WIN = "did_win"

-- Must be instantiate using WitAnalytics.instantiate()
local shared_instance = nil

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function sharedInstanceExists()
  if (shared_instance == nil) then
    Log.error("NOT REPORTING ANALYTICS. Use WitAnalytics.instantiate() before calling WitAnalytics methods.")
    return false
  end
  return true
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WitAnalytics.roundTimedOut(numberOfWordsSubmitted)
  if sharedInstanceExists() then
    shared_instance.roundsTimedOut = shared_instance.roundsTimedOut + 1
    shared_instance.timedOutWordsSubmitted = shared_instance.timedOutWordsSubmitted + numberOfWordsSubmitted
  end
end

function WitAnalytics.roundEndedEarly(numberOfWordsSubmitted)
  if sharedInstanceExists() then
    shared_instance.roundsEndedEarly = shared_instance.roundsEndedEarly + 1
    shared_instance.endedEarlyWordsSubmitted = shared_instance.endedEarlyWordsSubmitted + numberOfWordsSubmitted
  end
end

-- Because of how Wit is setup, all rounds cause a "time out".
-- To account for this, we subtract out the ended early stats
function WitAnalytics.gameEnded(didWin)
  if sharedInstanceExists() then
    local eventKey = EVENT_WIT_ANALYTICS
    local propertiesMapping = {}
    propertiesMapping[PROPERTY_NUMBER_OF_ROUNDS_TIMEDOUT] = shared_instance.roundsTimedOut - shared_instance.roundsEndedEarly
    propertiesMapping[PROPERTY_NUMBER_OF_ROUNDS_ENDED_EARLY] = shared_instance.roundsEndedEarly
    propertiesMapping[PROPERTY_WORDS_IN_TIMED_OUT_ROUNDS] = shared_instance.timedOutWordsSubmitted - shared_instance.endedEarlyWordsSubmitted
    propertiesMapping[PROPERTY_WORDS_IN_ENDED_EARLY_ROUNDS] = shared_instance.endedEarlyWordsSubmitted
    propertiesMapping[PROPERTY_DID_WIN] = didWin
    Analytics.reportEvent(eventKey, propertiesMapping)
  end
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function WitAnalytics.instantiate()
  shared_instance = {
    roundsTimedOut = 0,
    roundsEndedEarly = 0,
    timedOutWordsSubmitted = 0,
    endedEarlyWordsSubmitted = 0
  }
end
