--- WitCollectedWordsView

class("WitCollectedWordsView")

require("scene.effects.BlockGrid")
require("scene.effects.BlockPictureFadeInOutAction")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local DEFAULT_TEXT_COLOR = {1, 1, 1}
local INITIAL_FONT_SIZE = 17
local REDUCED_FONT_SIZE = 12

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getY(self)
  return self:getHeight() - self.line * (self.fontSize + self.dy) - self.fontSize/2 - self.yMargin
end

local function increaseLeadingX(self, viewWidth)
  self.leadingX = self.leadingX + viewWidth + self.dx
end

local function getX(self, viewWidth)
  local x = self.leadingX
  if(x + viewWidth > self:getWidth() - 2 * self.xMargin) then
    x = self.xMargin
    self.leadingX = self.xMargin
    self.line = self.line+ 1
  end
  return x
end
local function positionView(self, view)
  local viewWidth = view:getWidth()
  local x = getX(self, viewWidth)
  local y = getY(self)
  view:setLoc(x, y)
  increaseLeadingX(self, viewWidth)
end

local function initAppearEffect(self, view)
  local width, height = view:getDims()
  local additionalWidth = self.fontSize/2
  local additionalHeight = self.fontSize/2
  width = width + additionalWidth
  height = height + additionalHeight
  local x, y = view:getLoc()
  local blockWidth, blockHeight = 12, 12
  local viewHideTime = 0.2
  local viewAppearTime = 0.3
  local blockAnimationTime = 0.5

  local correctColor = WitColors.getCorrectAnswerColor()
  local grid = BlockGrid.new(width, height, blockWidth, blockHeight, correctColor)
  local gridEffect = BlockPictureFadeInOutAction.new(grid, blockAnimationTime, correctColor[4])
  local viewHide = AHDelay.new(viewHideTime)
  local viewShow = AHOpacityTo.new(view, viewAppearTime, nil, 1)
  local viewEffect = AHSequence.new(viewHide, viewShow)
  local cleanup = function() grid:removeSelf() end

  view:setOpacity(0)
  self:insert(grid)
  grid:setLoc(x + width/2 - additionalWidth/2, y)

  local gridAction = AHSequence.new(gridEffect, cleanup)
  local appearEffect = AHSpawn.new(gridEffect, viewEffect)
  appearEffect:start()
end

local function shrinkAndReinsertCurrentWords(self)
  self.fontSize = REDUCED_FONT_SIZE

  local textsToAdd = {}
  for wordViewIndex = 1, #self.wordViews - 1 do
    local wordView = self.wordViews[wordViewIndex]
    local textMinusComma = wordView.text:sub(1, -2)
    table.insert(textsToAdd, textMinusComma)
  end

  self:clear()

  for _, text in ipairs(textsToAdd) do
    self:addWord(text, false)
  end
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WitCollectedWordsView:addWord(text, doAppearEffect)
  if doAppearEffect == nil then
    doAppearEffect = true
  end

  local view = Label.new(text..",", Font.new(FontFaces.DIN_OT_MEDIUM, self.fontSize))
  view:setAnchorPointLeft()

  positionView(self, view)
  self.words:insert(view)
  table.insert(self.wordViews, view)
  view:setColor(unpack(self.wordColor))
  view:inheritColor(self)

  if self.line >= 2 and (self.fontSize ~= REDUCED_FONT_SIZE) then
    shrinkAndReinsertCurrentWords(self)
    self:addWord(text)
  elseif doAppearEffect then
    initAppearEffect(self, view)
  end
end

function WitCollectedWordsView:clear()
  self.words:removeAll()
  self.wordViews = {}
  self.line = 0
  self.leadingX = self.xMargin
end

function WitCollectedWordsView:setWordColor(color)
  self.wordColor = color
end

function WitCollectedWordsView:resetSizing()
  self.fontSize = INITIAL_FONT_SIZE
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local width, height = self:getDims()
  local words = Group.new(width, height)

  self:insertAtCenter(words)

  self.words = words
end

-- Constructor ------------------------

function WitCollectedWordsView.new(width, height)
  local self = Group.new(width, height)
  Mixin.mix(self, WitCollectedWordsView)
  createLayout(self)

  self.xMargin, self.yMargin = 0, 5
  self.dy = 8

  self.line = 0
  self.fontSize = INITIAL_FONT_SIZE
  self.dx = 10
  self.leadingX = self.xMargin
  self.wordColor = DEFAULT_TEXT_COLOR
  self.wordViews = {}

  return self
end
