class("WitColors")
local ROUND_PROGRESSION_COLORS =
{
  {
    gradient =
    {
      ColorUtils.colorHex24ToStd(0x2F4252),
      ColorUtils.colorHex24ToStd(0x17A0D0)
    },
    highlight = ColorUtils.colorHex24ToStd(0x62C8EB),
    progress = ColorUtils.colorHex24ToStd(0x62BDEB)
  },
  {
    gradient =
    {
      ColorUtils.colorHex24ToStd(0x3E4393),
      ColorUtils.colorHex24ToStd(0x4AA7E3)
    },
    highlight = ColorUtils.colorHex24ToStd(0x4AA7E3),
    progress = ColorUtils.colorHex24ToStd(0x5F9FFF)
  },
  {
    gradient =
    {
      ColorUtils.colorHex24ToStd(0x4A37A3),
      ColorUtils.colorHex24ToStd(0xCB5676)
    },
    highlight = ColorUtils.colorHex24ToStd(0xC995FF),
    progress = ColorUtils.colorHex24ToStd(0xC660DF)
  },
  -- WIN OUTRO:
  {
    gradient =
    {
      ColorUtils.colorHex24ToStd(0x44CCA9),
      ColorUtils.colorHex24ToStd(0x1EABA0)
    },
    highlight = ColorUtils.colorHex24ToStd(0x005557)
  },
  -- LOSE OUTRO:
  {
    gradient =
    {
      ColorUtils.colorHex24ToStd(0xEC2031),
      ColorUtils.colorHex24ToStd(0xFFA0A8)
    },
    highlight = ColorUtils.colorHex24ToStd(0x440409)
  }
}

local CORRECT_ANSWER_COLOR = ColorUtils.colorHex24ToStd(0x00FFCE)
local INCORRECT_ANSWER_COLOR = ColorUtils.colorHex24ToStd(0xF75F5F)

function WitColors.getWordSuggestionsColor(roundNumber)
  return ROUND_PROGRESSION_COLORS[roundNumber].highlight
end

function WitColors.getIncorrectAnswerColor()
  return INCORRECT_ANSWER_COLOR
end

function WitColors.getCorrectAnswerColor()
  return CORRECT_ANSWER_COLOR
end

function WitColors.getBackgroundGradientByColorSchemeIndex(n)
  local colorInfo = ROUND_PROGRESSION_COLORS[n]
  local gradient = colorInfo.gradient
  return gradient
end

function WitColors.getHighlightColorForRound(n)
  local colorInfo = ROUND_PROGRESSION_COLORS[n]
  return colorInfo.highlight
end

function WitColors.getHeaderColorForRound(n)
  local colorInfo = ROUND_PROGRESSION_COLORS[n]
  return colorInfo.highlight
end

function WitColors.getCollectedWordsColorForRound(n)
  local colorInfo = ROUND_PROGRESSION_COLORS[n]
  return colorInfo.highlight
end

function WitColors.getHintColorSchemeForRound(n)
  local colorScheme = {}
  colorScheme.highlight = WitColors.getHighlightColorForRound(n)
  colorScheme.wrong = WitColors.getIncorrectAnswerColor(n)
  return colorScheme
end

function WitColors.getProgressBarColorsByRound()
  local progressBarColorsByRound = {}
  local initialColors = {}
  local completeColors = {}
  local failColors = {}
  progressBarColorsByRound.initial = initialColors
  progressBarColorsByRound.complete = completeColors
  progressBarColorsByRound.failed = failColors

  for roundNumber, colorInfo in ipairs(ROUND_PROGRESSION_COLORS) do
    initialColors[roundNumber] = colorInfo.progress
    completeColors[roundNumber] = WitColors.getCorrectAnswerColor()
    failColors[roundNumber] = {0, 0, 0}
  end
  return progressBarColorsByRound
end

function WitColors.getInputTextColor()
  return {0, 0, 0, 0.5}
end

function WitColors.getTimerBackgroundOpacity()
  return 0.2
end

function WitColors.getTimerFillColor()
  return {0, 0, 0, 0.3}
end
