class("WitConfiguration")

require("data.keyframes.KeyframeSet")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

WitConfiguration.POINTS_PER_CORRECT_RESPONSE = 800
WitConfiguration.ROUNDS_PER_GAME_COUNT = 3
WitConfiguration.ROUND_DURATION_IN_SECONDS = 29
WitConfiguration.POINTS_PER_EXTRA_CORRECT_RESPONSE = 800
WitConfiguration.POINTS_PER_ADVANCED_RESPONSE = 200

local ROUND_DIFFICULTIES = {"Easy", "Medium", "Hard"}

local ROUND_DIFFICULTY_THRESHOLDS = KeyframeSet.newThreshold(
  Keyframe.new(0.00, {1, 1, 1}),
  Keyframe.new(0.15, {1, 1, 2}),
  Keyframe.new(0.25, {1, 2, 2}),
  Keyframe.new(0.30, {1, 2, 3}),
  Keyframe.new(0.45, {2, 2, 3}),
  Keyframe.new(0.60, {2, 3, 3}),
  Keyframe.new(0.80, {3, 3, 3})
)

local CORRECT_RESPONSES_NEEDED_TO_PASS = 3
local EXTRA_CORRECT_RESPONSES_NEEDED_RANK_TWO = 4
local EXTRA_CORRECT_RESPONSES_NEEDED_RANK_THREE = 7

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WitConfiguration.getCorrectResponsesNeededToPass()
  return CORRECT_RESPONSES_NEEDED_TO_PASS
end

function WitConfiguration.getPassScore()
  return WitConfiguration.getCorrectResponsesNeededToPass() * WitConfiguration.POINTS_PER_CORRECT_RESPONSE
end

function WitConfiguration.getRankTwoScore()
  return WitConfiguration.getPassScore() + EXTRA_CORRECT_RESPONSES_NEEDED_RANK_TWO * WitConfiguration.POINTS_PER_CORRECT_RESPONSE
end

function WitConfiguration.getRankThreeScore()
  return WitConfiguration.getPassScore() + EXTRA_CORRECT_RESPONSES_NEEDED_RANK_THREE * WitConfiguration.POINTS_PER_CORRECT_RESPONSE
end

function WitConfiguration.getRoundDifficulties()
  local difficulty = GameSession.getDifficulty()
  local roundDifficultyIndexes = ROUND_DIFFICULTY_THRESHOLDS:getFrameAt(difficulty)
  local convertedRoundDifficulties = {}
  for roundNumber, difficultyIndex in ipairs(roundDifficultyIndexes) do
    convertedRoundDifficulties[roundNumber] = ROUND_DIFFICULTIES[difficultyIndex]
  end
  return convertedRoundDifficulties
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
