--- WitInterstitialProgressDisplay

class("WitInterstitialProgressDisplay")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local function createHeaderAndBody(headerText, bodyText)
  local headerFont = FontFaces.DIN_OT_MEDIUM
  local bodyFont = FontFaces.DIN_OT_LIGHT
  local headerFontSize = WitConstants.HUGE_FONT_SIZE
  local bodyFontSize = WitConstants.LARGE_FONT_SIZE
  local dy = 0

  local headerView = Label.new(headerText, Font.new(headerFont, headerFontSize))
  local bodyView = Label.new(bodyText, Font.new(bodyFont, bodyFontSize))

  local headerW, headerH = headerView:getDims()
  local bodyW, bodyH = bodyView:getDims()
  local totalH = headerH + bodyH
  local totalW = math.max(headerW, bodyW)

  headerView:setLoc(0, headerH/2)
  bodyView:setLoc(0, -bodyH/2)

  local view = Group.newDeprecated()
  view:insert(headerView)
  view:insert(bodyView)

  headerView:inheritColor(view)
  bodyView:inheritColor(view)

  view:setBounds(0, 0, 0, totalW, totalH, 0)

  headerView:inheritColor(view)
  bodyView:inheritColor(view)

  return view
end
local function initAsHeaderAndBodyView(self, headerText, bodyText)
  local view = createHeaderAndBody(headerText, bodyText)
  self:insert(view)

  local w, h = view:getDims()
  view:setLoc(w/2, h/2)

  self:setBounds(0, 0, 0, w, h, 0)
  self:setAnchorPointCenter()

  view:inheritColor(self)
end

local function getPluralitySuffix(plurality)
  if(plurality == 1) then
    return ""
  else
    return "s"
  end
end
local function createWordsRemainingDisplay(self, wordsForPassedRound, wordsRemaining)
  local headerText = "You got "..wordsForPassedRound.." word"..getPluralitySuffix(wordsForPassedRound).."!"
  local bodyText = wordsRemaining.." to go"
  initAsHeaderAndBodyView(self, headerText, bodyText)
end

local function createBonusDisplay(self, wordsForAllRounds)
  local headerText = "You've collected "..wordsForAllRounds.." word"..getPluralitySuffix(wordsForPassedRound).."!"
  local bodyText = "get bonus points for extra words"
  initAsHeaderAndBodyView(self, headerText, bodyText)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function createLayout(self, wordsForPassedRound, wordsForAllRounds, goalWordCount)
  if (wordsForAllRounds >= goalWordCount) then
    createBonusDisplay(self, wordsForAllRounds)
  else
    createWordsRemainingDisplay(self, wordsForPassedRound, goalWordCount - wordsForAllRounds)
  end
end

-- Constructor ------------------------

function WitInterstitialProgressDisplay.new(wordsForPassedRound, wordsForAllRounds, goalWordCount)
  local self = Group.newDeprecated()
  Mixin.mix(self, WitInterstitialProgressDisplay)

  createLayout(self, wordsForPassedRound, wordsForAllRounds, goalWordCount)

  return self
end
