--- WitUnusedWordsView

class("WitUnusedWordsView")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {0.7, 0.7, 0.7, 0}


-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getWordsToShow(self, dictionary, wordsList)
  local unusedWordsSet = dictionary:getUnusedWordsFromList(wordsList)
  local unusedWordsList = TableUtils.createArrayFromSet(unusedWordsSet)
  local numberOfElementsToShow = math.min(3, #unusedWordsList)

  local wordsToShow = TableUtils.createRandomSampleOfArray(unusedWordsList, numberOfElementsToShow)

  return wordsToShow
end

local function createLayout(self, dictionary, suggestionsList, totalWidth, totalHeight, wordsColor)
  local wordsToShow = getWordsToShow(self, dictionary, suggestionsList)

  local x = 0
  local dx = Screen.getWidthPart(0.0375)
  local fontSize = WitConstants.SMALL_FONT_SIZE

  local headerText = ""

  if #wordsToShow > 0 then
    headerText = "OTHER WORDS"
  end

  local header = Label.new(headerText, Font.new(FontFaces.DIN_OT_MEDIUM, WitConstants.TINY_FONT_SIZE))
  local background = FilledRectangle.new(totalWidth, totalHeight, BACKGROUND_COLOR)

  local margin = totalWidth * 0.1
  local wordsWidth = totalWidth - 2 * margin
  local wordsLineSpacing = 4

  local wordsAttributes = {
    font = Font.new(FontFaces.DIN_OT_MEDIUM, fontSize),
    color = wordsColor
  }

  local text = TableUtils.reduce(wordsToShow, function(a, b) return a .. ", " .. b end, "")
  local wordsArea = DynamicTextArea.new(text, wordsWidth, wordsAttributes, nil, Alignment.CENTER, Alignment.CENTER, wordsLineSpacing, true)

  wordsArea:setAnchorPointTop()

  local headerWidth, headerHeight = header:getDims()
  header:setLoc(0, totalHeight/2 - headerHeight/2)
  wordsArea:setLoc(0, Screen.getHeightPart(-0.035))

  self:insert(background)
  self:insert(wordsArea)
  self:insert(header)

  self.wordsArea = wordsArea
  self.header = header
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WitUnusedWordsView:createShowAction()
  local animTime = 0.3

  self.header:setOpacity(0)
  self.wordsArea:setOpacity(0)

  local showHeader = AHOpacityTo.new(self.header, animTime, nil, 1)
  local showWords = AHOpacityTo.new(self.wordsArea, animTime, nil, 1)
  local show = AHSpawn.new(showHeader, showWords)

  return show
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function WitUnusedWordsView.new(dictionary, suggestionsList, w, h, wordsColor)
  w, h = w or Screen.fullWidth, h or 40
  local self = Group.newDeprecated()
  Mixin.mix(self, WitUnusedWordsView)
  createLayout(self, dictionary, suggestionsList, w, h, wordsColor)
  return self
end
