class("DictionaryWordsMatcher")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function getMatchDistanceThreshold(word)
  local wordLength = #word
  if(wordLength <= 3) then
    return 0
  elseif(wordLength <= 6) then
    return 1
  elseif(wordLength <= 10) then
    return 2
  else
    return 3
  end
end

local function findWordWithMinLevenshteinDistance(word, setOfPotentialMatches, distanceThreshold)
  local minDistance = math.huge
  local closestWord = nil

  for potentialMatch, _ in pairs(setOfPotentialMatches) do
    local distance = StringUtils.getLevenshteinDistance(word, potentialMatch)
    if(distance < minDistance) then
      minDistance = distance
      closestWord = potentialMatch
    end
  end

  if(minDistance <= distanceThreshold) then
    return closestWord
  end
end

local function getBestMatchFromSet(self, word, setOfPotentialMatches)
  local distanceThreshold = getMatchDistanceThreshold(word)
  local foundWord = findWordWithMinLevenshteinDistance(word, setOfPotentialMatches, distanceThreshold)
  return foundWord
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function DictionaryWordsMatcher:getBestMatch(word)
  local isExactMatch = true
  if(self.dictionary:contains(word)) then return word, isExactMatch end
  isExactMatch = false
  local potentialMatches = self.dictionary:getUnusedAcceptableWords()
  return getBestMatchFromSet(self, word, potentialMatches), isExactMatch
end

function DictionaryWordsMatcher:getBestExclusionMatch(word)
  local isExactMatch = true
  if(self.dictionary:excludes(word)) then return word, isExactMatch end
  isExactMatch = false
  local potentialMatches = self.dictionary:getExcludedWords()
  return getBestMatchFromSet(self, word, potentialMatches), isExactMatch
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function DictionaryWordsMatcher.new(dictionary)
  local self = {}
  Mixin.mix(self, DictionaryWordsMatcher)
  self.dictionary = dictionary
  return self
end
