class("WitDictionary")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function buildAcceptableWords(self, round)
  local acceptableWords = {}
  local excludedWords = {}
  local advancedWords = TableUtils.createSetFromArray(round.advanced)

  TableUtils.joinArrayToSet(round.common, acceptableWords)
  TableUtils.joinArrayToSet(round.advanced, acceptableWords)
  if(round.excludedWords) then
    excludedWords = TableUtils.createSetFromArray(round.excludedWords)
  end

  self.acceptableWords = acceptableWords
  self.advancedWords = advancedWords
  self.excludedWords = excludedWords
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WitDictionary:contains(word)
  return self.acceptableWords[word]
end

function WitDictionary:excludes(word)
  return self.excludedWords[word]
end

function WitDictionary:removeFromListOfAcceptedWords(word)
  self.acceptableWords[word] = nil
end

function WitDictionary:isAdvanced(word)
  return self.advancedWords[word]
end

function WitDictionary:use(word)
  self.usedWords[word] = true
end

function WitDictionary:isUsed(word)
  return self.usedWords[word]
end

function WitDictionary:getUnusedWordsFromList(wordsList)
  local wordsSet = TableUtils.createSetFromArray(wordsList)
  local usedWordsSet = self.usedWords

  local wordsWithoutUsedWords = TableUtils.differenceSet(wordsSet, usedWordsSet)

  return wordsWithoutUsedWords
end

function WitDictionary:getUnusedAcceptableWords()
  local acceptableWordsSet = self.acceptableWords
  local usedWordsSet = self.usedWords

  local unusedWords = TableUtils.differenceSet(acceptableWordsSet, usedWordsSet)

  return unusedWords
end

function WitDictionary:getExcludedWords()
  return self.excludedWords
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function WitDictionary.new(round)
  local self = {}
  Mixin.mix(self, WitDictionary)
  buildAcceptableWords(self, round)
  self.usedWords = {}
  return self
end
