class("WitRoundGenerator")
require("data.generators.UtilityGenerators")


------------------------------------------------------------------------------
-- Configuration
------------------------------------------------------------------------------

local MAX_UNIQUE_TASK_GENERATION_ATTEMPTS = 30

local DIFFICULTY_KEY = "difficulty"

------------------------------------------------------------------------------
-- Utility
------------------------------------------------------------------------------

local function conceptToRound(concept)
  local round = TableUtils.deepCopy(concept)
  round.common = TableUtils.csvToArray(concept.common)
  round.advanced = TableUtils.csvToArray(concept.advanced)
  round.suggestions = TableUtils.csvToArray(concept.suggestList)

  if(concept.noAccept) then
    round.excludedWords = TableUtils.csvToArray(concept.noAccept)
  else
    round.excludedWords = {}
  end

  return round
end

local function getHasSeenSimilar(self, concept)
  local hasSeenTaskType = self.seenTaskTypes[concept.task]
  local hasSeenReplaceWord = self.seenReplaceWords[concept.replaceWord]

  return (hasSeenTaskType or hasSeenReplaceWord)
end

local function getRoundDifficulty(self, roundNumber)
  local roundDifficulties = self.roundDifficulties
  local constrainedRoundNumber = MathUtils.clamp(roundNumber, 1, #roundDifficulties)
  local difficulty = roundDifficulties[constrainedRoundNumber]
  Log.debug(string.format("Selected %s difficulty for round %d (constrained to %d).", difficulty, roundNumber, constrainedRoundNumber))
  return difficulty
end

------------------------------------------------------------------------------
-- API
------------------------------------------------------------------------------

function WitRoundGenerator:generateNextValue(roundNumber)
  local difficulty = getRoundDifficulty(self, roundNumber)
  local difficultyFilter = {
    [DIFFICULTY_KEY] = difficulty
  }
  self.generator:setFilters(difficultyFilter)

  local concept = self.generator:next()
  local attemptIndex = 0

  local hasSeenSimilar = getHasSeenSimilar(self, concept)

  while(hasSeenSimilar and attemptIndex < MAX_UNIQUE_TASK_GENERATION_ATTEMPTS) do
    attemptIndex = attemptIndex + 1
    concept = self.generator:next()

    hasSeenSimilar = getHasSeenSimilar(self, concept)
  end

  self.seenTaskTypes[concept.task] = true
  self.seenReplaceWords[concept.replaceWord] = true

  return self.transformFunction(concept)
end


------------------------------------------------------------------------------
-- Object Creation
------------------------------------------------------------------------------

-- Constructor ------------------------

function WitRoundGenerator.new()
  local conceptGenerator = GameSession.conceptGenerator

  local self = TransformationGenerator.new(conceptGenerator, conceptToRound)
  Mixin.mix(self, WitRoundGenerator)

  self.seenTaskTypes = {}
  self.seenReplaceWords = {}

  self.roundDifficulties = WitConfiguration.getRoundDifficulties()

  return self
end
