class("InterstitialView")
require("actions.advanced.AHBlink")

require("widgets.TapToContinue")
require("WitInterstitialProgressDisplay")
require("scene.widgets.BlinkingSign")
require("WitUnusedWordsView")
require("WitColors")


local FADE_IN_TIME = 0.2
local DELAYED_TAP_TO_CONTINUE_TIME = 0.6
local DEFAULT_SUGGESTION_COLOR = {1, 1, 1}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createTapToContinueSign(self)
  local tapToContinueView = BlinkingSign.newTapToContinue()
  tapToContinueView:setLoc(0, 0)

  self.animations[#self.animations + 1] = AHOpacityTo.new(tapToContinueView, FADE_IN_TIME, nil, 1):start()
  self:insert(tapToContinueView)

  return tapToContinueView
end

local function createWordsCollectedInfo(self)
  local stats = self.roundStats
  local wordsCollectedView = WitInterstitialProgressDisplay.new(stats.wordsForPassedRound, stats.wordsForAllRounds, stats.goalWordCount)
  self:insert(wordsCollectedView)

  local colorSchemeIndex = self.suggestionInfo.colorSchemeIndex
  local collectedWordsInfoColor = WitColors.getWordSuggestionsColor(colorSchemeIndex)
  wordsCollectedView:setColor(unpack(collectedWordsInfoColor))

  wordsCollectedView:setOpacity(0)
  self.animations[#self.animations + 1] = AHOpacityTo.new(wordsCollectedView, FADE_IN_TIME, nil, 1):start()

  return wordsCollectedView
end

local function createSuggestionsView(self)
  local suggestionInfo = self.suggestionInfo
  local unusedWords = WitUnusedWordsView.new(suggestionInfo.dictionary, suggestionInfo.suggestions, nil, nil, DEFAULT_SUGGESTION_COLOR)
  self:insert(unusedWords)

  self.animations[#self.animations + 1] = unusedWords:createShowAction():start()

  return unusedWords
end

local function createLayout(self)
  local tapToContinueSign = createTapToContinueSign(self)
  local wordsCollectedView = createWordsCollectedInfo(self)
  local unusedWords = createSuggestionsView(self)
  wordsCollectedView:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.7888))
  tapToContinueSign:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.3592))
  unusedWords:setLoc(Screen.getSafeX(0.5), Screen.getSafeY(0.6126))
end

local function getInitTapToContinueAction(self)
  local waitForInterstitial = AHDelay.new(math.huge)

  self.animations = {}
  self.animations[#self.animations + 1] = waitForInterstitial

  local function initLayout()
    createLayout(self)
  end

  local tapToContinueArmDelay = AHDelay.new(DELAYED_TAP_TO_CONTINUE_TIME)

  local function initTapToContinue()
    local cleanup = function()
      for _, animation in ipairs(self.animations) do
        animation:stop()
      end
      self:removeSelf()
    end

    TapToContinue.new(nil, nil, cleanup)
  end

  local showInterstitialAction = AHSequence.new(initLayout, tapToContinueArmDelay, initTapToContinue, waitForInterstitial)

  return showInterstitialAction
end


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function InterstitialView:initShowAction()
  local tapToContinue = getInitTapToContinueAction(self)
  return tapToContinue
end


-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

-- roundStats =
-- {
--    passedRoundNumber (index of round that just ended)
--    wordsForPassedRound (how many answers were accepted this round)
--    wordsForAllRounds (how many answers were accepted across all rounds)
--    goalWordCount (how many words needed to pass game)
-- }
-- suggestionInfo =
-- {
--    dictionary (wit dictionary object)
--    suggestions (list of words to suggest)
--    colorSchemeIndex (color scheme index)
-- }
function InterstitialView.new(roundStats, suggestionInfo)
  local self = Group.newDeprecated()
  Mixin.mix(self, InterstitialView)
  self.roundStats = roundStats
  self.suggestionInfo = suggestionInfo
  return self
end
