--- WitTaskView
-- @author Created by Steven Mah on April 11, 2016.

class("WitTaskView")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local FONT = Font.new(FontFaces.DIN_OT_MEDIUM, 16)
local SPACING = 7.5

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WitTaskView:createAnimateInAction(duration)
  local duration = duration or 0.5
  local beLabel = self.beLabel
  local moreLabel = self.moreLabel
  local taskLabel = self.taskLabel

  local y = self:getHeightPart(0.5)

  local ease = AHEaseExponentialOut.new()
  local animateInAction = AHStaggeredSpawn.new(
    0.1,
    AHSpawn.new(
      AHOpacityTo.new(beLabel, duration, nil, 1.0),
      AHLocationTo.new(beLabel, duration, ease, self.beFinalX, y),
      AHScaleTo.new(beLabel, duration, ease, 1.0)
    ),
    AHSpawn.new(
      AHOpacityTo.new(moreLabel, duration, nil, 1.0),
      AHLocationTo.new(moreLabel, duration, ease, self.moreFinalX, y),
      AHScaleTo.new(moreLabel, duration, ease, 1.0)
    ),
    AHSpawn.new(
      AHOpacityTo.new(taskLabel, duration, nil, 1.0),
      AHLocationTo.new(taskLabel, duration, ease, self.taskFinalX, y),
      AHScaleTo.new(taskLabel, duration, ease, 1.0)
    )
  )

  return animateInAction
end

function WitTaskView:createFadeAction(duration, opacity)
  local hideAction = AHSpawn.new(
    AHOpacityTo.new(self.beLabel, duration, nil, opacity),
    AHOpacityTo.new(self.moreLabel, duration, nil, opacity),
    AHOpacityTo.new(self.taskLabel, duration, nil, opacity)
  )

  return hideAction
end

-- ----------------------------------------------------------------------------
-- Object construction
-- ----------------------------------------------------------------------------

function WitTaskView.new(taskText, color)
  local beLabel = Label.new("BE ", FONT)
  beLabel:setColor(unpack(color))
  beLabel:setOpacity(0.0)

  local moreLabel = Label.new("MORE ", FONT)
  moreLabel:setColor(unpack(color))
  moreLabel:setOpacity(0.0)

  local taskLabel = Label.new(string.upper(taskText), FONT)
  taskLabel:setColor(unpack(color))
  taskLabel:setOpacity(0.0)

  local beWidth, height = beLabel:getDims()
  local moreWidth = moreLabel:getWidth()
  local taskWidth = taskLabel:getWidth()
  local width = beWidth + moreWidth + taskWidth

  local self = Group.new(width, height)
  Mixin.mix(self, WitTaskView)

  local beLabelX = beLabel:getWidthPart(0.5)
  beLabel:setLoc(width * -0.5, height * 1.5)
  self:insert(beLabel)
  self.beLabel = beLabel
  self.beFinalX = beLabelX

  local moreLabelX = beLabelX + (beWidth + moreWidth) / 2 + SPACING
  moreLabel:setScl(2.0)
  moreLabel:setLoc(moreLabelX * 2, self:getHeightPart(1.5))
  self:insert(moreLabel)
  self.moreLabel = moreLabel
  self.moreFinalX = moreLabelX

  local taskLabelX = moreLabelX + (taskWidth + moreWidth) / 2 + SPACING
  taskLabel:setScl(2.0)
  taskLabel:setLoc(width * 1.5, height * 0.5)
  self:insert(taskLabel)
  self.taskLabel = taskLabel
  self.taskFinalX = taskLabelX

  self:setAnchorPointLeft()

  return self
end
