module ("BlockPictureFlowUpAction", package.seeall)


function BlockPictureFlowUpAction.new(blockPicture)
  local self = AHSpawn.new()
  Mixin.mix(self, BlockPictureFlowUpAction)
  local animationTime = 0.67

  local w, h = blockPicture:getDims()
  local totalRows = blockPicture:getRowCount()
  local delayPerRow = animationTime/totalRows/2
  local maxYOffset = -h


  local movementEase = AHEaseSimpleOut.new(3)

  local currentDelay
  local currentRowAnimation
  local currentMoveTime
  local normalizedHeight
  local easedHeight
  local function rowIterateCallback(row, gridY)
    currentDelay = delayPerRow * (gridY - 1)
    currentRowAnimation = AHSpawn.new()
    currentMoveTime = animationTime - currentDelay
    normalizedHeight = (gridY - 1)/(totalRows - 1)
    easedHeight = math.pow(normalizedHeight * 0.25 + 0.75, 1.5)
    self:addAction(currentRowAnimation)
  end
  local function viewIterateCallback(view, gridX, gridY)
    local x0, y0 = view:getLoc()
    local initView = function()
      view:setOpacity(0)
      view:setLoc(x0, y0 + maxYOffset * easedHeight)
    end
    local wait = AHDelay.new(currentDelay)
    local move = AHLocationTo.new(view, currentMoveTime, movementEase, x0, y0)
    local appear = AHOpacityTo.new(view, currentMoveTime, nil, 1)
    local show = AHSpawn.new(move, appear)
    local viewAnimation = AHSequence.new(initView, wait, show)
    currentRowAnimation:addAction(viewAnimation)
  end

  blockPicture:iterateViews(viewIterateCallback, rowIterateCallback)

  return self
end
