module ("BlinkingSign", package.seeall)
require("actions.advanced.AHBlink")
require("widgets.AlertPulse")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local LABEL_TEXT = "TAP TO CONTINUE"

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createBlinkingLabel()
  local font = Font.new(FontFaces.DIN_OT_MEDIUM, WitConstants.SMALL_FONT_SIZE)
  local label = Label.new(LABEL_TEXT, font)

  local blink = AHBlink.new(label, 1, nil, {1, 1, 1, 1}, {1, 1, 1, 0.3}, 1)
  local animateTapToContinue = AHLoop.new(blink)
  animateTapToContinue:start()

  return label
end

local function createPulseCircle()
  local pulseRadius = Screen.getHeightPart(0.0105)
  local pulseCircle = AlertPulse.new(pulseRadius, nil, 1.6, 0.4)
  pulseCircle:pulse(math.huge)

  return pulseCircle
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function BlinkingSign:disposeOfSelf()
  self.animation:stop()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

function BlinkingSign.newTapToContinue()
  local label = createBlinkingLabel()
  local pulse = createPulseCircle()

  local labelWidth, labelHeight = label:getDims()
  local pulseHeight = pulse:getHeight()
  local height = labelHeight + pulseHeight

  local self = Group.new(labelWidth, height)

  self:insertAtCenter(label)
  pulse:setLoc(self:getWidthPart(0.5), label:getY() + height)
  self:insert(pulse)

  label:inheritColor(self)
  pulse:inheritColor(self)

  return self
end
