--- Word Birds Answer Label
-- @author Created by Karl Stenerud on September 10, 2013.

class("WordBirdsAnswerLabel")
require("WordBirdsCommon")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local COLOR_TEXT = {1, 1, 1, 1}
local COLOR_BG = {0, 0, 0.1, 0.2}
local COLOR_EDGE = {1, 1, 1, 0.7}
local COLOR_CORRECT_BG = WordBirdsConfiguration.COLOR_CORRECT_BG
local COLOR_CORRECT_EDGE = WordBirdsConfiguration.COLOR_CORRECT_HIGHLIGHT
local COLOR_CORRECT_TEXT = WordBirdsConfiguration.COLOR_CORRECT_HIGHLIGHT

local COLOR_INCORRECT_BG = WordBirdsConfiguration.COLOR_INCORRECT_BG
local COLOR_INCORRECT_EDGE = WordBirdsConfiguration.COLOR_INCORRECT_HIGHLIGHT
local COLOR_INCORRECT_TEXT = WordBirdsConfiguration.COLOR_INCORRECT_HIGHLIGHT

local SELF_WIDTH = Screen.getSafeWidthPart(0.8)
local SELF_HEIGHT = 50
local CORNER_RADIUS = 4.5
local LINE_WIDTH = 2.5

local FONT_NAME = FontFaces.DIN_OT_BOLD
local FONT_SIZE = 24

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createShakeAnimation(self)
  local moveDistance = 6
  local shakeDuration = 0.05
  return AHSequence.new(
    AHLocationXBy.new(self, shakeDuration/2, AHEaseSineOut.new(), -moveDistance*1.0),
    AHLocationXBy.new(self, shakeDuration, AHEaseSineInOut.new(), moveDistance*2.0),
    AHLocationXBy.new(self, shakeDuration, AHEaseSineInOut.new(), -moveDistance*1.5),
    AHLocationXBy.new(self, shakeDuration, AHEaseSineIn.new(), moveDistance*1.0),
    AHLocationXBy.new(self, shakeDuration/2, AHEaseSineIn.new(), -moveDistance*0.5)
  )
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WordBirdsAnswerLabel:setText(text)
  self.label:setText(text)
end

function WordBirdsAnswerLabel:markCorrect(answer)
  local correctEase = AHEaseSineOut.new()
  return AHSequence.new(
    function()
      self:setText(answer)
    end,
    AHSpawn.new(
      AHScaleTo.new(self, 0.13, correctEase, 0.95),
      AHColorTo.new(self.background, 0.13, correctEase, COLOR_CORRECT_BG),
      AHColorTo.new(self.box, 0.13, correctEase, COLOR_CORRECT_EDGE),
      AHColorTo.new(self.label, 0.13, correctEase, COLOR_CORRECT_TEXT)
    ),
    AHScaleTo.new(self, 0.21, AHEaseSineInOut.new(), 1.05)
  ):start()
end

function WordBirdsAnswerLabel:markIncorrect()
  return runInCoroutine(function()
    self:setText(self.fullAnswer)
    wait(AHSpawn.new(
      AHColorTo.new(self.background, 0.13, nil, COLOR_INCORRECT_BG),
      AHColorTo.new(self.box, 0.13, nil, COLOR_INCORRECT_EDGE),
      AHColorTo.new(self.label, 0.13, nil, COLOR_INCORRECT_TEXT),
      createShakeAnimation(self)
    ):start())
  end)
end

function WordBirdsAnswerLabel:markTimeout()
  return runInCoroutine(function()
    self:setText(self.fullAnswer)
    wait(AHSpawn.new(createShakeAnimation(self)):start())
  end)
end

function WordBirdsAnswerLabel:markSkipped()
  self:setText(self.fullAnswer)
end

-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function WordBirdsAnswerLabel.new(fullAnswer)
  local self = Group.new(SELF_WIDTH, SELF_HEIGHT)
  Mixin.mix(self, WordBirdsAnswerLabel)
  local selfW, selfH = self:getDims()

  self.fullAnswer = fullAnswer

  self.cornerRadius = CORNER_RADIUS
  self.background = FilledRoundedRectangle.new(selfW, selfH, CORNER_RADIUS, COLOR_BG)
  self.box = HollowRoundedRectangle.new(selfW, selfH, CORNER_RADIUS, COLOR_EDGE, LINE_WIDTH, nil)

  self:insertAtCenter(self.background)
  self:insertAtCenter(self.box)

  self.label = Label.new("", Font.new(FONT_NAME, FONT_SIZE))
  self.label:setColor(unpack(COLOR_TEXT))
  self.label:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.55))
  self:insert(self.label)

  self:linkColorAttrToChildren()

  return self
end
