-- Word Birds Bird


local ANIM_FRAMERATE = 24

local IDLE_DELAY_MIN = 5
local IDLE_DELAY_MAX = 10


local WordBirdsBirdFunctions = {}

function WordBirdsBirdFunctions:flyIn(duration, locX, locY)
  return runExclusivelyInCoroutine(self, self, true, function()
    wait(self:performFlyIn(duration, locX, locY))
  end)
end

function WordBirdsBirdFunctions:flyOut(duration, locX, locY)
  return runExclusivelyInCoroutine(self, self, true, function()
    self.idling = false
    wait(self:performFlyOut(duration, locX, locY))
  end)
end

function WordBirdsBirdFunctions:idle()
  return runExclusivelyInCoroutine(self, self, true, function()
    wait(self:performIdle())
  end)
end

function WordBirdsBirdFunctions:beginIdling()
  return runInCoroutine(function()
    self.idling = true
    while self.idling do
      local delay = math.random() * (IDLE_DELAY_MAX - IDLE_DELAY_MIN) + IDLE_DELAY_MIN
      waitFor(delay)
      self:idle()
      coroutine.yield()
    end
  end)
end

class("WordBirdsBirdTypePeek")
-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WordBirdsBirdTypePeek:performFlyIn(duration, landingX, landingY)
  return runInCoroutine(function()
      self:setLoc(landingX, landingY)
      wait(AHAnimate.new(self.birdSprite, self.peekAnimation, ANIM_FRAMERATE):start())
    end)
end

function WordBirdsBirdTypePeek:performFlyOut(duration, locX, locY)
  local action = AHSequence.new(
                                AHAnimate.new(self.birdSprite, self.takeOffAnimation, ANIM_FRAMERATE),
                                AHSpawn.new(
                                            AHLoop.new(
                                                       AHAnimate.new(self.birdSprite, self.flyOutAnimation, ANIM_FRAMERATE),
                                                       4
                                                       ),
                                            AHLocationTo.new(self, duration, nil, locX, locY)
                                            )
                                )
  return action:start()
end

function WordBirdsBirdTypePeek:performIdle()
  return runInCoroutine(function()
      wait(AHAnimate.new(self.birdSprite, self.idleAnimation, ANIM_FRAMERATE):start())
    end)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function WordBirdsBirdTypePeek.new(isFacingRight)
  local birdSprite = Sprite.newWithName("bird2/popup/wordbirds_bird2_popup_000.png")

  local self = Group.new(birdSprite:getDims())
  Mixin.mix(self, WordBirdsBirdTypePeek)
  Mixin.mix(self, WordBirdsBirdFunctions)

  self.birdSprite = birdSprite
  local selfW, selfH = self:getDims()
  if not isFacingRight then
    self.birdSprite:setScl(-1, 1)
  end
  self:insertAtCenter(self.birdSprite)

  self.peekAnimation = Animation.newWithPrefix("bird2/popup/wordbirds_bird2_popup_")
  self.takeOffAnimation = Animation.newWithPrefix("bird2/takeoff/wordbirds_bird2_takeoff_")
  self.flyOutAnimation = Animation.newWithPrefix("bird2/fly/wordbirds_bird2_fly_")
  self.idleAnimation = Animation.newWithPrefix("bird2/idle/wordbirds_bird2_idle_")

  self:setBounds(0.0, 0.0, 0, selfW, selfH, 0)
  self:setAnchorPointCenter()

  return self
end




class("WordBirdsBirdTypePerch")
-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WordBirdsBirdTypePerch:performFlyIn(duration, locX, locY)
  local action = AHSpawn.new(
                             AHAnimate.new(self.birdSprite, self.landAnimation, ANIM_FRAMERATE),
                             AHLocationTo.new(self, duration, nil, locX, locY)
                             )
  return action:start()
end

function WordBirdsBirdTypePerch:performFlyOut(duration, locX, locY)
  local action = AHSequence.new(
                                AHAnimate.new(self.birdSprite, self.takeOffAnimation, ANIM_FRAMERATE),
                                AHSpawn.new(
                                            AHAnimate.new(self.birdSprite, self.flyOutAnimation, ANIM_FRAMERATE, nil, duration),
                                            AHLocationTo.new(self, duration, nil, locX, locY)
                                            )
                                )
  return action:start()
end

function WordBirdsBirdTypePerch:performIdle()
  return runInCoroutine(function()
      wait(AHAnimate.new(self.birdSprite, self.idleAnimation, ANIM_FRAMERATE):start())
    end)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function WordBirdsBirdTypePerch.new(isFacingRight)
  local birdSprite = Sprite.newWithName("bird1/flyland/wordbirds_bird1_flyland_000.png")

  local self = Group.new(birdSprite:getDims())
  Mixin.mix(self, WordBirdsBirdTypePerch)
  Mixin.mix(self, WordBirdsBirdFunctions)

  self.birdSprite = birdSprite
  local selfW, selfH = self:getDims()
  if not isFacingRight then
    self.birdSprite:setScl(-1, 1)
  end
  self:insertAtCenter(self.birdSprite)

  self.flyOutAnimation = Animation.newWithPrefix("bird1/flyaway/wordbirds_bird1_flyaway_")
  self.landAnimation = Animation.newWithPrefix("bird1/flyland/wordbirds_bird1_flyland_")
  self.takeOffAnimation = Animation.newWithPrefix("bird1/takeoff/wordbirds_bird1_takeoff_")
  self.idleAnimation = Animation.newWithPrefix("bird1/idle/wordbirds_bird1_idle_")

  self:setBounds(0.0, 0.0, 0, selfW, selfH, 0)
  self:setAnchorPointCenter()

  return self
end




class("WordBirdsBirdTypeHop")
-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

local ANCHOR_POINT_LAND = {0.5, 0}
local ANCHOR_POINT_TAKEOFF = {0.2, 0}
local ANGLE_TAKEOFF = 30
local OFFSET_TAKEOFF = {50, -30}

function WordBirdsBirdTypeHop:performFlyIn(duration, locX, locY)
  return runInCoroutine(function()
    local hopDistance = 8
    if not self.isFacingRight then
      hopDistance = hopDistance * -1
    end

    local landDuration = self.landAnimation.frameCount / ANIM_FRAMERATE

    AHLocationTo.new(self, duration, nil, locX - hopDistance, locY):start()
    wait(AHAnimate.new(self.birdSprite, self.flyAnimation, ANIM_FRAMERATE, nil, duration):start())
    AHLocationTo.new(self, landDuration, AHEaseSineOut.new(), locX, locY):start()
    wait(AHAnimate.new(self.birdSprite, self.landAnimation, ANIM_FRAMERATE):start())
  end)
end

function WordBirdsBirdTypeHop:performFlyOut(duration, locX, locY)
  return runInCoroutine(function()
    local anchorPoint = ANCHOR_POINT_TAKEOFF
    local rotation = ANGLE_TAKEOFF
    local takeoffOffsetX, takeoffOffsetY = unpack(OFFSET_TAKEOFF)

    if not self.isFacingRight then
      rotation = rotation * -1
      takeoffOffsetX = -takeoffOffsetX
    end

    local spriteX, spriteY = self.birdSprite:getLoc()

    wait(AHAnimate.new(self.birdSprite, self.takeOffAnimation, ANIM_FRAMERATE):start())
    self.birdSprite:setAnchorPoint(unpack(anchorPoint))
    self.birdSprite:setLoc(spriteX+takeoffOffsetX, spriteY+takeoffOffsetY)
    self.birdSprite:setRot(rotation)
    wait(AHSpawn.new(
                    AHAnimate.new(self.birdSprite, self.flyAnimation, ANIM_FRAMERATE, nil, duration),
                    AHLocationTo.new(self, duration, nil, locX, locY)
                    ):start())
  end)
end

function WordBirdsBirdTypeHop:performIdle()
  return runInCoroutine(function()
      wait(AHAnimate.new(self.birdSprite, self.idleAnimation, ANIM_FRAMERATE):start())
    end)
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

-- Constructor ------------------------

function WordBirdsBirdTypeHop.new(isFacingRight)
  local birdSprite = Sprite.newWithName("bird3/fly/wordbirds_bird3_fly_000.png")

  local self = Group.new(birdSprite:getDims())
  Mixin.mix(self, WordBirdsBirdTypeHop)
  Mixin.mix(self, WordBirdsBirdFunctions)

  self.isFacingRight = isFacingRight

  self.birdSprite = birdSprite
  self.birdSprite:setAnchorPoint(unpack(ANCHOR_POINT_LAND))
  if not isFacingRight then
    self.birdSprite:setScl(-1, 1)
  end
  self:insertAtCenter(self.birdSprite)

  self.flyAnimation = Animation.newWithPrefix("bird3/fly/wordbirds_bird3_fly_")
  self.landAnimation = Animation.newWithPrefix("bird3/land/wordbirds_bird3_land_")
  self.takeOffAnimation = Animation.newWithPrefix("bird3/takeoff/wordbirds_bird3_takeoff_")
  self.idleAnimation = Animation.newWithPrefix("bird3/idle/wordbirds_bird3_idle_")

  return self
end
