--- WordBirdsBirdData.lua
-- @author created by Jay Austria on April 26, 2016

require("WordBirdsBird")

WordBirdsBirdData = {}

local left = 1
local right = 2

local peek = 3
local perch = 4
local hop = 5


-- Positioning
--
-- X uses center relative positioning, Y uses bottom relative positioning.
--
-- X data is center relative, meaning that:
--   * 100 = 100 px to the RIGHT of the center
--   * -200 = 200 px to the LEFT of the center
--
-- Y data is bottom relative, meaning that:
--   * 100 = 100 px from the BOTTOM of the screen.
--   * -200 = 200 px BELOW the screen (i.e. out of the screen area)
--
--
-- X and Y data can be represented as ABSOLUTE or PROPORTIONAL:
--
-- If the value is between -1 and 1, it is PROPORTIONAL,
-- meaning that it's a proportion of the screen width or height.
-- If it's <= -1 or >= 1, it is an ABSOLUTE value in pixels.
--
-- So for example:
--   * 0.5 on a Y value means half the screen height
--   * -0.2 on an X value means center of screen, then LEFT 20% of the screen width
--   * 0.4 on an X value means center of screen, then RIGHT 40% of the screen width
--   * 2 means 2 pixels.
--   * 150 means 150 pixels.
--
-- Basically, if you're dealing in fractional values, it's a proportion of the screen size.


-- Bird Data
--
-- Birds will be added in the order they appear in BIRD_DATA.
--
-- Fields are as follows:
--
-- bird_type: the type of bird to create. Can be perch, peek, or hop.
--   * perch: Flies in and lands normally.
--   * peek: Does not fly in, but instead peeks up from behind the house
--           at the "fly_in_to" position.
--   * hop: Flies in and hops as it lands.
--
-- facing: The direction the bird will face. Can be left or right.
--
-- scale: How big the bird will be. 1.0 means normal sized. 2.0 means double. 0.5 means half.
--
-- fly_in_from_y: Where the bird flies in from (Y position). Bottom relative.
--                Note: the X position for this is handled automatically.
--
-- fly_in_to_x: Where the bird flies in to (X position). Center relative.
--
-- fly_in_to_y: Where the bird flies in to (Y position). Bottom relative.
--
-- fly_in_duration: Duration of the flight in in seconds.
--
-- fly_out_to_y: Where the bird flies out to (Y position). Bottom relative.
--               Note: the X position for this is handled automatically.
--
-- fly_out_duration: Duration of the flight out in seconds.

-- Rightmost roof
WordBirdsBirdData.BIRD_DATA_ROOF_1 = {
  {
    bird_type = hop, --bird#1
    facing = left,
    scale = 0.85,
    fly_in_from_y = 0.30,
    fly_in_to_x = 24,
    fly_in_to_y = 52,
    fly_in_duration = 0.47,
    fly_out_to_y = 0.3,
    fly_out_duration = 0.68
  },
  {
    bird_type = perch, --bird#4
    facing = left,
    scale = 0.8,
    fly_in_from_y = 0.5,
    fly_in_to_x = 75,
    fly_in_to_y = 70,
    fly_in_duration = 0.7,
    fly_out_to_y = 0.4,
    fly_out_duration = 1.2
  },
  {
    bird_type = hop, --bird#7
    facing = left,
    scale = 0.85,
    fly_in_from_y = 0.12,
    fly_in_to_x = 101,
    fly_in_to_y = 9,
    fly_in_duration = 0.34,
    fly_out_to_y = 0.45,
    fly_out_duration = 0.85
  },
  {
    bird_type = perch, --bird#8
    facing = right,
    scale = 0.85,
    fly_in_from_y = 0.5,
    fly_in_to_x = 52,
    fly_in_to_y = 48,
    fly_in_duration = 0.8,
    fly_out_to_y = 0.13,
    fly_out_duration = 0.48
  },
  {
    bird_type = hop, --bird#11
    facing = left,
    scale = 0.72,
    fly_in_from_y = 0.28,
    fly_in_to_x = 145,
    fly_in_to_y = 89,
    fly_in_duration = 0.34,
    fly_out_to_y = 0.48,
    fly_out_duration = 0.97
  },
  {
    bird_type = hop, --bird#15
    facing = left,
    scale = 0.8,
    fly_in_from_y = 0.2,
    fly_in_to_x = 168,
    fly_in_to_y = 40,
    fly_in_duration = 0.14,
    fly_out_to_y = 0.5,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch, -- new bird #1
    facing = right,
    scale = 0.95,
    fly_in_from_y = 0.5,
    fly_in_to_x = 120,
    fly_in_to_y = 30,
    fly_in_duration = 0.9,
    fly_out_to_y = 0.13,
    fly_out_duration = 0.58
  },
  {
    bird_type = peek, -- new bird #2
    facing = left,
    scale = 0.7,
    fly_in_from_y = 0.5,
    fly_in_to_x = 37,
    fly_in_to_y = 65,
    fly_in_duration = 1.8,
    fly_out_to_y = 0.35,
    fly_out_duration = 1.1
  },
}

-- Foreground left roof
WordBirdsBirdData.BIRD_DATA_ROOF_2 = {
  {
    bird_type = perch, --bird#3
    facing = right,
    scale = 0.9,
    fly_in_from_y = 0.45,
    fly_in_to_x = -55,
    fly_in_to_y = 35,
    fly_in_duration = 0.8,
    fly_out_to_y = 0.2,
    fly_out_duration = 0.65
  },
  {
    bird_type = perch, --bird#9
    facing = right,
    scale = 0.95,
    fly_in_from_y = 0.5,
    fly_in_to_x = -40,
    fly_in_to_y = 20,
    fly_in_duration = 0.75,
    fly_out_to_y = 0.4,
    fly_out_duration = 0.98
  },
  {
    bird_type = hop, --bird#10
    facing = right,
    scale = 0.85,
    fly_in_from_y = 0.25,
    fly_in_to_x = -108,
    fly_in_to_y = 14,
    fly_in_duration = 0.5,
    fly_out_to_y = 0.35,
    fly_out_duration = 0.6
  },
  {
    bird_type = perch, --bird#14
    facing = right,
    scale = 0.8,
    fly_in_from_y = 0.5,
    fly_in_to_x = -140,
    fly_in_to_y = 20,
    fly_in_duration = 0.8,
    fly_out_to_y = 0.35,
    fly_out_duration = 0.97
  },
  {
    bird_type = hop, -- new bird #3
    facing = left,
    scale = 0.85,
    fly_in_from_y = 0.25,
    fly_in_to_x = -87,
    fly_in_to_y = 3,
    fly_in_duration = 0.6,
    fly_out_to_y = 0.35,
    fly_out_duration = 0.6
  },
}
-- Background left roof
WordBirdsBirdData.BIRD_DATA_ROOF_3 = {
  {
    bird_type = hop, --bird#2
    facing = right,
    scale = 0.70,
    fly_in_from_y = 0.225,
    fly_in_to_x = -113,
    fly_in_to_y = 72,
    fly_in_duration = 0.6,
    fly_out_to_y = 0.29,
    fly_out_duration = 0.75
  },
  {
    bird_type = perch, --bird#5
    facing = right,
    scale = 0.72,
    fly_in_from_y = 0.5,
    fly_in_to_x = -111,
    fly_in_to_y = 66,
    fly_in_duration = 0.8,
    fly_out_to_y = 0.28,
    fly_out_duration = 1.0
  },
  {
    bird_type = peek, --bird#6
    facing = right,
    scale = 0.7,
    fly_in_from_y = 0.5,
    fly_in_to_x = -38,
    fly_in_to_y = 75,
    fly_in_duration = 2,
    fly_out_to_y = 0.35,
    fly_out_duration = 1.0
  },
  {
    bird_type = perch, --bird#12
    facing = right,
    scale = 0.70,
    fly_in_from_y = 0.4,
    fly_in_to_x = -135,
    fly_in_to_y = 50,
    fly_in_duration = 0.7,
    fly_out_to_y = 0.26,
    fly_out_duration = 1.0
  },
  {
    bird_type = hop, --bird#13
    facing = right,
    scale = 0.75,
    fly_in_from_y = 0.4,
    fly_in_to_x = -136,
    fly_in_to_y = 64,
    fly_in_duration = 0.4,
    fly_out_to_y = 0.35,
    fly_out_duration = 0.97
  },
}

WordBirdsBirdData.OUTRO_DATA = {
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5,
    fly_in_to_y = Screen.fullHeight * 0.4,
    fly_in_duration = 0,
    fly_out_to_y = 0.5,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 50,
    fly_in_to_y = Screen.fullHeight * 0.4 + 30,
    fly_in_duration = 0,
    fly_out_to_y = 0.55,
    fly_out_duration = 0.9
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 100,
    fly_in_to_y = Screen.fullHeight * 0.4 - 30,
    fly_in_duration = 0,
    fly_out_to_y = 0.52,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 50,
    fly_in_to_y = Screen.fullHeight * 0.4 + 0,
    fly_in_duration = 0,
    fly_out_to_y = 0.55,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 80,
    fly_in_to_y = Screen.fullHeight * 0.4 + 20,
    fly_in_duration = 0,
    fly_out_to_y = 0.55,
    fly_out_duration = 0.9
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 50,
    fly_in_to_y = Screen.fullHeight * 0.4 - 20,
    fly_in_duration = 0,
    fly_out_to_y = 0.51,
    fly_out_duration = 0.9
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 70,
    fly_in_to_y = Screen.fullHeight * 0.4 - 20,
    fly_in_duration = 0,
    fly_out_to_y = 0.53,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 90,
    fly_in_to_y = Screen.fullHeight * 0.4 + 10,
    fly_in_duration = 0,
    fly_out_to_y = 0.52,
    fly_out_duration = 0.9
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 110,
    fly_in_to_y = Screen.fullHeight * 0.4 - 10,
    fly_in_duration = 0,
    fly_out_to_y = 0.5,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 80,
    fly_in_to_y = Screen.fullHeight * 0.4 + 10,
    fly_in_duration = 0,
    fly_out_to_y = 0.50,
    fly_out_duration = 1.1
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5,
    fly_in_to_y = Screen.fullHeight * 0.4,
    fly_in_duration = 0,
    fly_out_to_y = 0.5,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 50,
    fly_in_to_y = Screen.fullHeight * 0.4 + 30,
    fly_in_duration = 0,
    fly_out_to_y = 0.55,
    fly_out_duration = 0.9
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 100,
    fly_in_to_y = Screen.fullHeight * 0.4 - 30,
    fly_in_duration = 0,
    fly_out_to_y = 0.52,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 50,
    fly_in_to_y = Screen.fullHeight * 0.4 + 0,
    fly_in_duration = 0,
    fly_out_to_y = 0.55,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 80,
    fly_in_to_y = Screen.fullHeight * 0.4 + 20,
    fly_in_duration = 0,
    fly_out_to_y = 0.55,
    fly_out_duration = 0.9
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 50,
    fly_in_to_y = Screen.fullHeight * 0.4 - 20,
    fly_in_duration = 0,
    fly_out_to_y = 0.51,
    fly_out_duration = 0.9
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 70,
    fly_in_to_y = Screen.fullHeight * 0.4 - 20,
    fly_in_duration = 0,
    fly_out_to_y = 0.53,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 90,
    fly_in_to_y = Screen.fullHeight * 0.4 + 10,
    fly_in_duration = 0,
    fly_out_to_y = 0.52,
    fly_out_duration = 0.9
  },
  {
    bird_type = hop,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 110,
    fly_in_to_y = Screen.fullHeight * 0.4 - 10,
    fly_in_duration = 0,
    fly_out_to_y = 0.5,
    fly_out_duration = 0.9
  },
  {
    bird_type = perch,
    facing = right,
    scale = 1,
    fly_in_from_y = 0,
    fly_in_to_x = Screen.fullWidth * -0.5 - 80,
    fly_in_to_y = Screen.fullHeight * 0.4 + 10,
    fly_in_duration = 0,
    fly_out_to_y = 0.50,
    fly_out_duration = 1.1
  }
}
