--- WordBirdsConfiguration.lua
-- @author created by Jay Austria on April 26, 2016

class("WordBirdsConfiguration")

require("GameSession")
require("data.generators.LearningFocusedRoundGenerator")


-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

WordBirdsConfiguration.GAME_DURATION_IN_SECONDS = 80

WordBirdsConfiguration.TIME_REMAINING_TO_START_RESURFACING_MISTAKES =
  WordBirdsConfiguration.GAME_DURATION_IN_SECONDS *
  LearningFocusedRoundGenerator.DEFAULT_REMAINING_BEFORE_RESURFACING

WordBirdsConfiguration.POINTS_PER_CORRECT_RESPONSE = 1400
WordBirdsConfiguration.CORRECT_NEEDED_COUNT = 1
WordBirdsConfiguration.POINTS_PER_FULL_SPEED_BONUS = 100
WordBirdsConfiguration.SPEED_BONUS_WINDOW_IN_SECONDS = 3.0
WordBirdsConfiguration.SPEED_BONUS_DECAY_START = 0.0
WordBirdsConfiguration.INITIAL_DIFFICULTY = 0
WordBirdsConfiguration.DIFFICULTY_BUCKETS = {"Basic", "Easy", "Medium", "Hard", "Advanced"}

WordBirdsConfiguration.NORMAL_FONT = FontFaces.DIN_OT_MEDIUM

WordBirdsConfiguration.COLOR_CORRECT_BG = ColorUtils.colorRGBToStd(85, 254, 206)
WordBirdsConfiguration.COLOR_CORRECT_HIGHLIGHT = ColorUtils.colorRGBToStd(39, 138, 128)
WordBirdsConfiguration.COLOR_INCORRECT_BG = ColorUtils.colorRGBToStd(177, 60, 60)
WordBirdsConfiguration.COLOR_INCORRECT_HIGHLIGHT = ColorUtils.colorRGBToStd(85, 36, 36)
WordBirdsConfiguration.UI_COMPONENTS_COLOR = {1.0, 1.0, 1.0, 0.55}

WordBirdsConfiguration.MAX_SQUARE_BUTTONS_PER_ROW = 5

WordBirdsConfiguration.BUTTON_SPACING = Screen.getSafeWidthPart(0.05)
WordBirdsConfiguration.BUTTON_ROW_WIDTH = Screen.getSafeWidthPart(0.9)
WordBirdsConfiguration.SQUARE_BUTTON_SIZE = (WordBirdsConfiguration.BUTTON_ROW_WIDTH - WordBirdsConfiguration.BUTTON_SPACING * (WordBirdsConfiguration.MAX_SQUARE_BUTTONS_PER_ROW-1)) / WordBirdsConfiguration.MAX_SQUARE_BUTTONS_PER_ROW

WordBirdsConfiguration.TUTORIAL_DELAY = 8.0
WordBirdsConfiguration.MINIMUM_TIME_TO_START_NEXT_ROUND = 1.5

local CORRECT_NEEDED_COUNT_RANK_TWO = 5
local CORRECT_NEEDED_COUNT_RANK_THREE = 7

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WordBirdsConfiguration.getPassScore()
  local passScore = WordBirdsConfiguration.CORRECT_NEEDED_COUNT * WordBirdsConfiguration.POINTS_PER_CORRECT_RESPONSE
  return passScore
end

function WordBirdsConfiguration.getRankTwoScore()
  local rankTwoScore = CORRECT_NEEDED_COUNT_RANK_TWO * WordBirdsConfiguration.POINTS_PER_CORRECT_RESPONSE
  return rankTwoScore
end

function WordBirdsConfiguration.getRankThreeScore()
  local rankThreeScore = CORRECT_NEEDED_COUNT_RANK_THREE * WordBirdsConfiguration.POINTS_PER_CORRECT_RESPONSE
  return rankThreeScore
end

function WordBirdsConfiguration.getDifficultyKeyFrames()
  return {
    {key = 0.0, valueProbabilities = {Basic = 1.0}},

    {key = 0.075, valueProbabilities = {Basic = 0.9,
                                        Easy = 0.1}},

    {key = 0.15, valueProbabilities = {Basic = 0.75,
                                       Easy = 0.25}},

    {key = 0.2, valueProbabilities = {Basic = 0.35,
                                      Easy = 0.65}},

    {key = 0.3, valueProbabilities = {Basic = 0.25,
                                      Easy = 0.70,
                                      Medium = 0.05}},

    {key = 0.375, valueProbabilities = {Easy = 0.60,
                                        Medium = 0.40}},

    {key = 0.45, valueProbabilities = {Easy = 0.35,
                                       Medium = 0.65}},

    {key = 0.6, valueProbabilities = {Easy = 0.10,
                                      Medium = 0.65,
                                      Hard = 0.25}},

    {key = 0.75, valueProbabilities = {Medium = 0.45,
                                       Hard = 0.55}},

    {key = 0.8, valueProbabilities = {Medium = 0.3,
                                      Hard = 0.65,
                                      Advanced = 0.05}},

    {key = 0.9, valueProbabilities = {Medium = 0.15,
                                      Hard = 0.65,
                                      Advanced = 0.2}},

    {key = 1.0, valueProbabilities = {Hard = 0.7,
                                      Advanced = 0.3}}
  }
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------
