--- WordBirdsErrorInterstitial

class("WordBirdsErrorInterstitial")

-- ----------------------------------------------------------------------------
-- Configuration
-- ----------------------------------------------------------------------------

local BACKGROUND_COLOR = {0.07, 0.13, 0.15}
local BACKGROUND_OPACITY_FULL = 0.98
local HEADER_TEXT = "<b>Why is my answer incorrect?</b>"

local TEXT_WIDTH = Screen.getSafeWidthPart(0.8)
local BASE_ATTRIBUTES = {
  font = Font.new(FontFaces.DIN_OT_REGULAR, 20)
}
local ATTRIBUTES_FOR_LABELS = {
  b = {
    font = Font.new(FontFaces.DIN_OT_BOLD, 20)
  },
  highlight1 = {
    color = {0.94, 0.33, 0.28}
  },
  highlight2 = {
    color = {0.36, 0.78, 0.36}
  }
}

-- ----------------------------------------------------------------------------
-- Utility
-- ----------------------------------------------------------------------------

local function createOpacityAction(self, opacity)
  local duration = 0.1
  local action = AHOpacityTo.new(self, duration, nil, opacity)
  return action
end

-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WordBirdsErrorInterstitial:playShowAction()
  local action = createOpacityAction(self, BACKGROUND_OPACITY_FULL)
  return action:start()
end

function WordBirdsErrorInterstitial:playHideAction()
  local action = createOpacityAction(self, 0.0)
  return action:start()
end

-- ----------------------------------------------------------------------------
-- Object creation
-- ----------------------------------------------------------------------------

local function createLayout(self)
  local text = string.format("%s<br/><br/>%s", HEADER_TEXT, self.rationaleText)
  local rationaleLabel = DynamicTextArea.new(text, TEXT_WIDTH, BASE_ATTRIBUTES, ATTRIBUTES_FOR_LABELS, Alignment.LEFT, Alignment.TOP)
  rationaleLabel:setAnchorPointTop()
  rationaleLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.78))
  self:insert(rationaleLabel)

  local tapToContinueLabel = Label.new(Localizer.get("Misc.TAP_TO_CONTINUE"), Font.new(FontFaces.DIN_OT_MEDIUM, 15))
  tapToContinueLabel:setLoc(self:getWidthPart(0.5), self:getHeightPart(0.05))
  self:insert(tapToContinueLabel)
end

function WordBirdsErrorInterstitial.newHidden(concept)
  local self = FilledRectangle.newFullScreen(BACKGROUND_COLOR)
  Mixin.mix(self, WordBirdsErrorInterstitial)

  self:setOpacity(0.0)

  self.rationaleText = concept.rationale

  createLayout(self)

  return self
end
