-- Falling star generator

class("WordBirdsFallingStarGenerator")

local STAR_DELAY_MIN = 3
local STAR_DELAY_MAX = 5

local STAR_FALL_DURATION = 5

local ANIM_FRAMERATE = 24

local function getRandomStarLocation(self, assetHeight)
  local selfW, selfH = self:getDims()
  local xArea = selfW
  local xPos = math.random() * xArea

  return xPos, selfH + assetHeight/2
end

local function dropStar(self)
  return runInCoroutine(function()
    local star = Sprite.newWithName("falling star/wordbirds_fallingstar_000.png")
    local starHeight = star:getHeight()
    star:setLoc(getRandomStarLocation(self, starHeight))
    self:insert(star)
    star:setScl(0.5)
    star:setOpacity(0.8)
    local animAction = AHLoop.new(AHAnimate.new(star, self.starAnimation, ANIM_FRAMERATE)):start()
    local move = AHLocationYTo.new(star, STAR_FALL_DURATION, AHEaseSineIn.new(), -starHeight):start()
    waitFor(STAR_FALL_DURATION * 0.5)
    AHOpacityTo.new(star, STAR_FALL_DURATION * 0.3, AHEaseSineOut.new(), 0.0):start()
    wait(move)
    animAction:stop()
    star:removeSelf()
  end)
end

function WordBirdsFallingStarGenerator:start()
  return runInCoroutine(function()
    if not self.running then
      self.running = true
      while self.running do
        dropStar(self)
        local delay = math.random() * (STAR_DELAY_MAX - STAR_DELAY_MIN) + STAR_DELAY_MIN
        waitFor(delay)
      end
    end
  end)
end

function WordBirdsFallingStarGenerator:stop()
  self.running = false
end

function WordBirdsFallingStarGenerator.new()
  local self = Group.new(Screen.fullWidth, Screen.fullHeight)
  Mixin.mix(self, WordBirdsFallingStarGenerator)

  self.starAnimation = Animation.newWithPrefix("falling star/wordbirds_fallingstar_")

  return self
end
