--- Word Birds Interstitial

class("WordBirdsInterstitial")
require("graphics.primitives.RoundedRegularPolygon")


-- ----------------------------------------------------------------------------
-- API
-- ----------------------------------------------------------------------------

function WordBirdsInterstitial:animateIn(shouldDarken)
  return runInCoroutine(function()
    if shouldDarken then
      AHOpacityTo.new(self.background, 0.3, AHEaseSineOut.new(), 0.5):start()
    end
    wait(AHSequence.new(
      AHScaleTo.new(self.badge, 0.25, AHEaseSineOut.new(), 1.1),
      AHScaleTo.new(self.badge, 0.08, AHEaseSineInOut.new(), 1.0)
    ):start())
  end)
end

function WordBirdsInterstitial:animateOut()
  Sound.playSoundEffect("wordbirds_transition_swipe")
  return AHSpawn.new(
    AHScaleTo.new(self.badge, 0.3, AHEaseSineIn.new(), 0),
    AHOpacityTo.new(self.background, 0.3, nil, 0)
  ):start()
end


-- ----------------------------------------------------------------------------
-- Object Creation
-- ----------------------------------------------------------------------------

local function newHexagon(radius, color)
  return FilledRoundedRegularPolygon.new(radius, 6, 0, radius * 0.1, color)
end

-- Constructor ------------------------

function WordBirdsInterstitial.new(message)
  local self = Group.newFullScreen()
  Mixin.mix(self, WordBirdsInterstitial)

  local selfW, selfH = self:getDims()

  local backgroundColor = ColorUtils.colorRGBToStd(40, 51, 99, 128)
  local bigHexColor = ColorUtils.colorRGBToStd(245, 245, 248, 107)
  local smallHexColor = ColorUtils.colorRGBToStd(245, 245, 248, 132)
  local labelColor = ColorUtils.colorRGBToStd(46, 93, 118)

  self.background = FilledRectangle.new(selfW, selfH, backgroundColor)
  self:insertAtCenter(self.background)

  local birdIcon = Sprite.newWithName("wordbirds_birdicon.png")

  local hexRadius = birdIcon:getWidthPart(0.8)
  local bigHex = newHexagon(hexRadius, bigHexColor)
  local hexW, hexH = bigHex:getDims()

  local smallHex = newHexagon(hexRadius * 0.9, smallHexColor)
  bigHex:insertAtCenter(smallHex)

  self.badge = Group.new(hexW, hexH)
  self:insertAtCenter(self.badge)
  self.badge:insertAtCenter(bigHex)

  local birdHeight = birdIcon:getHeight()
  birdIcon:setLoc(hexW/2, hexH/2 + math.floor(birdHeight * 0.3))
  self.badge:insert(birdIcon)

  local label = Label.new(message, Font.new(WordBirdsConfiguration.NORMAL_FONT, 16))
  label:setColor(unpack(labelColor))
  label:setLoc(hexW/2, hexH/2 - math.floor(birdHeight * 0.6))
  self.badge:insert(label)

  local rotateToEdge = 360/12
  bigHex:setRot(rotateToEdge)
  self.badge:setScl(0)
  self.background:setOpacity(0)

  return self
end
